/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ide;

import java.net.URL;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.target.ITargetResource;
import org.eclipse.target.Site;
import org.eclipse.ui.IMemento;

/**
 * This is a site that gives access to the projects in a workspace
 */
public class IdeWorkspaceSite extends Site {

	private static final String WORKSPACE_SITE_LOCATION_ID = "Workspace"; //$NON-NLS-1$
	
	private static Site site;

	public synchronized static Site getInstance() {
		if (site == null) {
			try {
				site = Site.createSite(TargetIdePlugin.SITE_TYPE_ID, WORKSPACE_SITE_LOCATION_ID);
			} catch (CoreException e) {
				TargetIdePlugin.log(e);
				return null;
			}
		}
		return site;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#save(org.eclipse.ui.IMemento)
	 */
	public void save(IMemento memento) {
		// Nothing to save
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#init(org.eclipse.ui.IMemento)
	 */
	protected void init(IMemento memento) {
		// Nothing to load
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#getRootResource()
	 */
	public ITargetResource getRootResource() {
		return IdeTargetResource.createResource(this, ResourcesPlugin.getWorkspace().getRoot());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#canBeReached(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public boolean canBeReached(IProgressMonitor monitor) throws CoreException {
		return true;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#toUrl()
	 */
	public URL toUrl() {
		return null;
	}

}
