/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.registry;

import java.net.URL;

import org.eclipse.core.runtime.*;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.internal.WorkbenchPlugin;
import org.eclipse.ui.model.IWorkbenchAdapter;
import org.osgi.framework.Bundle;

public class SiteWizardDescriptor implements IAdaptable, IWorkbenchAdapter {
	
	protected final static String ATT_NAME = "name"; //$NON-NLS-1$
	protected final static String ATT_CLASS = "class"; //$NON-NLS-1$
	protected final static String ATT_ICON = "icon"; //$NON-NLS-1$
	protected final static String ATT_ID = "id"; //$NON-NLS-1$
	private final IConfigurationElement element;
	private final String descText;
	private String id;
	private String iconPath;
	private String name;
	
	/**
	 * @param element
	 * @param descText
	 */
	public SiteWizardDescriptor(IConfigurationElement element, String descText) {
		this.element = element;
		this.descText = descText;
		loadFromExtension();
	}

	private void loadFromExtension() {
		id = element.getAttribute(ATT_ID);
		name = element.getAttribute(ATT_NAME);
		iconPath = element.getAttribute(ATT_ICON);
	}
	
	public ImageDescriptor getImageDescriptor() {
		Bundle bundle = Platform.getBundle(element.getDeclaringExtension().getNamespace());
		URL fullPathString = bundle.getEntry(iconPath);
		return ImageDescriptor.createFromURL(fullPathString);
	}
	
	public Wizard createWizard() throws CoreException {
		Object obj = WorkbenchPlugin.createExtension(element, ATT_CLASS);
		return (Wizard) obj;
	}

	public String getDescription() {
		return descText;
	}
	public String getName() {
		return name;
	}
	public String getId() {
		return id;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class adapter) {
		if (adapter.equals(IWorkbenchAdapter.class)) {
			return this;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.model.IWorkbenchAdapter#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object o) {
		return new Object[0];
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.model.IWorkbenchAdapter#getImageDescriptor(java.lang.Object)
	 */
	public ImageDescriptor getImageDescriptor(Object object) {
		return getImageDescriptor();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.model.IWorkbenchAdapter#getLabel(java.lang.Object)
	 */
	public String getLabel(Object o) {
		return getName();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.model.IWorkbenchAdapter#getParent(java.lang.Object)
	 */
	public Object getParent(Object o) {
		return null;
	}
}
