/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ui;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.target.*;
import org.eclipse.target.Site;
import org.eclipse.target.Target;
import org.eclipse.target.internal.ITargetImages;
import org.eclipse.target.internal.TargetPlugin;

/**
 * Used to show Site instances in the UI. Sites are really just remote
 * resources but are shown with a different icon and label.
 * 
 * @see TargetResourceElement
 */
public class SiteElement extends TargetResourceElement {
	
	public SiteElement() {
	}

	public Site getSite(Object o) {
		if (o instanceof Site) {
			return (Site)o;
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.target.internal.ui.TargetResourceElement#getRemoteResource(java.lang.Object)
	 */
	public ITargetResource getRemoteResource(Object object) {
		Site site = getSite(object);
		if (site == null) {
			return null;
		}
		return site.getRootResource();
	}
	
	public ImageDescriptor getImageDescriptor(Object object) {
		return TargetPlugin.getDefault().getImageDescriptor(ITargetImages.IMG_SITE_ELEMENT);
	}
	
	public String getLabel(Object o) {
		Site site = getSite(o);
		if (site == null) {
			return super.getLabel(o);
		}
		return site.getDisplayName();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.target.internal.ui.TargetResourceElement#getParent(java.lang.Object)
	 */
	public Object getParent(Object o) {
		Site site = getSite(o);
		if (site == null) {
			return super.getParent(o);
		}
		return Target.getSiteManager();
	}
}
