/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ui.actions;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.*;
import org.eclipse.jface.window.Window;
import org.eclipse.osgi.util.NLS;
import org.eclipse.target.*;
import org.eclipse.target.internal.TargetMessages;
import org.eclipse.target.internal.ui.TargetAction;

/**
 * Discards a remembered Site from Target Management. Before a Site can be
 * discarded all resources using this site must be deconfigured. This action will
 * prompt the user if a Site cannot be discarded because of existing connections.
 */
public class DiscardSiteAction extends TargetAction {

	/**
	 * @see TeamAction#isEnabled()
	 */
	protected boolean isEnabled() throws CoreException {
		return getSelectedSites().length == 1;
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		deletedSelected();
	}
	
	private void deletedSelected() {
		try {
			Site[] sites = getSelectedSites();
			if(sites.length > 0) {
				MessageDialog d = new MessageDialog(getShell(),
							TargetMessages.SiteExplorerView_promptForDeletingSitesTitle, 
							null,
							NLS.bind(TargetMessages.SiteExplorerView_promptForDeletingSites, new Object[] { new Integer(sites.length).toString() }), 
							MessageDialog.QUESTION, 
							new String[] {
								IDialogConstants.YES_LABEL,
								IDialogConstants.CANCEL_LABEL }, 0);
				if( d.open() != Window.OK ) {
					return;
				}
				ISiteManager manager = Target.getSiteManager();
				if (!manager.okToRemoveSites(sites, getShell())) {
					return;
				}
				// sites aren't mapped, just ask then delete them form the view 
				// and from the target manager.
				for (int i = 0; i < sites.length; i++) {
					Target.getSiteManager().removeSite(sites[i]);
					sites[i].dispose();
				}					
			}
		} catch (CoreException e) {
			ErrorDialog.openError(getShell(),
				TargetMessages.Error, 
				TargetMessages.CreateNewFolderAction_errorDeletingSites, 
				e.getStatus());
		}
	}
}
