/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.bus.core;

import java.util.HashSet;
import java.util.Set;

/**
 * This code handles the class Service defined by :
 * <ul>
 * <li>"id"</li>
 * <li>"name"</li>
 * <li>"class"</li>
 * <li>"typeId"</li>
 * <li>"paramInId"</li>
 * <li>"paramOutId"</li>
 * </ul>
 */
public class Service
{
	private String id = "";

	private String name = "";

	private IService classe = null;

	private String typeId = "";

	private Set<String> paramInId = new HashSet<String>();

	private String paramOutId = "";

	/**
	 * Constructor
	 * 
	 * @param serviceId : String of the service id
	 * @param serviceName : String of the service name
	 * @param serviceClass : IService of the class
	 */
	public Service(String serviceId, String serviceName, IService serviceClass)
	{
		id = new String(serviceId);
		name = new String(serviceName);
		classe = serviceClass;
	}

	/**
	 * Constructor
	 * 
	 * @param serviceId : String of the service id
	 * @param serviceName : String of the service name
	 * @param serviceClass : IService of the class
	 * @param idType : String of the type id
	 * @param idParameterIn : set (of String) of parameter input id
	 * @param IdParameterOut : parameter output id
	 */
	public Service(String serviceId, String serviceName, IService serviceClass, String idType, Set<String> idParameterIn,
	        String IdParameterOut)
	{
		id = new String(serviceId);
		name = new String(serviceName);
		classe = serviceClass;
		typeId = new String(idType);
		paramOutId = new String(IdParameterOut);
		paramInId.addAll(idParameterIn);
	}

	/**
	 * Sets type id of this Service
	 * 
	 * @param idType : Type id
	 */
	public void setTypeId(String idType)
	{
		typeId = idType;
	}

	/**
	 * Sets output Parameter id of this Service
	 * 
	 * @param idParamOut : output Parameter id
	 */
	public void setParamOutId(String idParamOut)
	{
		paramOutId = idParamOut;
	}

	/**
	 * Sets input Parameter id of this Service. Add this id in the Set of input Paramter ids
	 * 
	 * @param idParamIn : output Parameter id
	 */
	public void setParamInId(String idParamIn)
	{
		paramInId.add(idParamIn);
	}

	/**
	 * Returns service id of this service
	 * 
	 * @return service id
	 */
	public String getId()
	{
		return id;
	}

	/**
	 * Returns service name of this service
	 * 
	 * @return service name
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * Returns service class of this service
	 * 
	 * @return service class
	 */
	public IService getClass_()
	{
		return classe;
	}

	/**
	 * Returns service type id of this service
	 * 
	 * @return type id of this service
	 */
	public String getIdType()
	{
		return typeId;
	}

	/**
	 * Returns service output parameter id of this service
	 * 
	 * @return id of output parameter of this service
	 */
	public String getIdParameterOut()
	{
		return paramOutId;
	}

	/**
	 * Returns service input parameter ids of this service
	 * 
	 * @return Set of all ids of input parameter of this service
	 */
	public Set<String> getIdParameterIn()
	{
		return paramInId;
	}
}
