/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.bus.core.metamodel;

/**
 * Handles the Service class defined by the fallowing attributes :
 * <ul>
 * <li>uri</li>
 * <li>id</li>
 * <li>name</li>
 * <li>version</li>
 * <li>description</li>
 * <li>pluginPathForMetaModel</li>
 * <li>pluginNameSpace</li>
 * </ul>
 * 
 * @author <a href="mailto:marion.feau@c-s.fr">Marion Feau</a>
 */
public class Metamodel
{
	private String uri = "";

	private String name = "";

	private String version = "";

	private String completName = "";

	private String description = "";

	private String metamodelPath = "";

	private String pluginNameSpace = "";

	private String pluginName = "";

	/**
	 * Constructor
	 * 
	 */
	public Metamodel()
	{
		// nothing
	}

	/**
	 * Constructor
	 * 
	 * @param urlMM
	 * @param nameMM
	 * @param versionMM
	 * @param completNameMM
	 * @param descriptionMM
	 * @param pluginPathForMM
	 * @param pluginNameSpaceMM
	 * @param pluginNameMM
	 *            (/plugin/...._3.0.1)
	 * 
	 */
	public Metamodel(String urlMM, String nameMM, String versionMM,
	        String completNameMM, String descriptionMM, String pluginPathForMM,
	        String pluginNameSpaceMM, String pluginNameMM)
	{
		uri = urlMM;
		name = nameMM;
		version = versionMM;
		completName = completNameMM;
		description = descriptionMM;
		metamodelPath = pluginPathForMM;
		pluginNameSpace = pluginNameSpaceMM;
		pluginName = pluginNameMM;
		String plugString = "/plugin/";
		int decal = plugString.length();
		int index = pluginNameMM.indexOf("/plugin/");
		if (index != -1)
		{
			pluginName = pluginNameMM.substring(index + decal);
		}
	}

	/**
	 * Return metamodel uri of this metamodel
	 * 
	 * @return metamodel uri
	 */
	public String getUri()
	{
		return uri;
	}

	/**
	 * Return metamodel name of this metamodel
	 * 
	 * @return metamodel name
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * Return metamodel version of this metamodel
	 * 
	 * @return metamodel version
	 */
	public String getVersion()
	{
		return version;
	}

	/**
	 * Return metamodel complet name of this metamodel
	 * 
	 * @return metamodel complet name
	 */
	public String getCompleteName()
	{
		return completName;
	}

	/**
	 * Return metamodel description of this metamodel
	 * 
	 * @return metamodel description
	 */
	public String getDescription()
	{
		return description;
	}

	/**
	 * Return plugin path for metamodel
	 * 
	 * @return metamodel metamodelPath
	 */
	public String getMetamodelPath()
	{
		return metamodelPath;
	}

	/**
	 * Returns the plugin namespace
	 * 
	 * @return metamodel pluginNameSpace
	 */
	public String getPluginNameSpace()
	{
		return pluginNameSpace;
	}

	/**
	 * Returns the plugin name
	 * 
	 * @return metamodel pluginNameSpace
	 */
	public String getPluginName()
	{
		return pluginName;
	}
}
