/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.bus.core.parameter;

/**
 * Handles the class Parameter which is parameter of a Service.<br>
 * A parameter is defined by :
 * <ul>
 * <li>id</li>
 * <li>name</li>
 * <li>description</li>
 * <li>kind</li>
 * </ul>
 * <br>
 */
public class Parameter
{
	private String id = null;

	private String name = null;

	private String description = null;

	private String kind = null;

	/**
	 * Constructor
	 * 
	 * @param paramId : parameter id
	 * @param paramName : parameter name
	 * @param paramDescription : parameter description
	 * @param paramKind : parameter kind
	 */
	public Parameter(String paramId, String paramName, String paramDescription, String paramKind)
	{
		id = new String(paramId);
		name = new String(paramName);
		description = new String(paramDescription);
		kind = new String(paramKind);
	}

	/**
	 * Get the parameter id of this parameter
	 * 
	 * @return parameter id
	 */
	public String getIdParam()
	{
		return id;
	}

	/**
	 * Get the parameter name of this parameter
	 * 
	 * @return parameter name
	 */
	public String getNameParam()
	{
		return name;
	}

	/**
	 * Get the parameter description of this parameter
	 * 
	 * @return parameter description
	 */
	public String getDescriptionParam()
	{
		return description;
	}

	/**
	 * Get the parameter kind of this parameter
	 * 
	 * @return parameter kind
	 */
	public String getKindParam()
	{
		return kind;
	}
}
