/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.bus.core.parameter;

/**
 * Handle the class Type which is the type of a Service.<br>
 * It is defined by :
 * <ul>
 * <li>id</li>
 * <li>name</li>
 * <li>parentId</li>
 * <li>category</li>
 * </ul>
 * <br>
 */
public class Type
{

    private String id = null;

    private String name = null;

    private String parentId = null;

    private String category = null;

    /**
     * Constructor
     * 
     * @param typeId :type id
     * @param typeName : type name
     * @param typeParentId : parent id of this type
     * @param typeCategory : category of this type
     */
    public Type(String typeId, String typeName, String typeParentId, String typeCategory)
    {
        id = new String(typeId);
        name = new String(typeName);
        parentId = typeParentId;
        category = typeCategory;
    }

    /**
     * Get the type id of this type
     * 
     * @return type id
     */
    public String getIdType()
    {
        return id;
    }

    /**
     * Get the type name of this type
     * 
     * @return type name
     */
    public String getNameType()
    {
        return name;
    }

    /**
     * Get the parent id of this type
     * 
     * @return parent id
     */
    public String getParentIdType()
    {
        return parentId;
    }

    /**
     * Get the category of this type
     * 
     * @return category
     */
    public String getCategoryType()
    {
        return category;
    }
}
