/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS)
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *********************************************************************************************/
package org.topcased.checkresultmodel.dialog;

import java.util.HashMap;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
import org.eclipse.ui.wizards.newresource.BasicNewFileResourceWizard;
import org.osgi.framework.Bundle;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;
import org.topcased.checkresultmodel.Model;

public class CheckResultDialog extends Dialog
{
	private Model resultModel;

	private boolean globalResult;

	/**
	 * Indicates if the selected elements should be shown in the tree too
	 */
	private boolean showSelected = false;

	/**
	 * Menu item to switch between showing and hiding the selected elements
	 */
	private MenuItem switchItem;

	/**
	 * Tree viewer used to render the model
	 */
	private TreeViewer treeViewer;

	/**
	 * The save menu item icon
	 */
	private static final Image SAVE_IMAGE;

	private static final Image SHOW_HIDE_IMAGE;

	private static IStructuredSelection previousSelection = StructuredSelection.EMPTY;
	
	static
	{
		Image tempImage;
		Bundle bundle = Platform.getBundle("org.topcased.checkresultmodel");
		tempImage = ImageDescriptor.createFromURL(FileLocator.find(bundle, new Path("icons/Save.gif"), null)).createImage();
		SAVE_IMAGE = tempImage;

		tempImage = ImageDescriptor.createFromURL(FileLocator.find(bundle, new Path("icons/ShowHide.gif"), null)).createImage();
		SHOW_HIDE_IMAGE = tempImage;
	}

	public CheckResultDialog(Shell parentShell, Model model, boolean globalResult, boolean defaultShowSelected)
	{
		super(parentShell);
		resultModel = model;
		this.globalResult = globalResult;
		showSelected = defaultShowSelected;
		setShellStyle(SWT.RESIZE | SWT.MAX | SWT.CLOSE | SWT.SYSTEM_MODAL | SWT.TITLE);
	}

	/**
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell shell)
	{
		super.configureShell(shell);
		shell.setText("Result");
		shell.setMinimumSize(600, 400);
		Menu mainMenu = new Menu(shell, SWT.BAR | SWT.LEFT_TO_RIGHT);
		mainMenu.setEnabled(true);
		mainMenu.setVisible(true);

		shell.setMenuBar(mainMenu);

		MenuItem fileMenuItem = new MenuItem(mainMenu, SWT.CASCADE);
		fileMenuItem.setText("File");

		Menu fileMenu = new Menu(fileMenuItem);
		fileMenu.setEnabled(true);
		fileMenu.setVisible(true);

		fileMenuItem.setMenu(fileMenu);

		MenuItem saveItem = new MenuItem(fileMenu, SWT.PUSH);
		saveItem.setText("Save...");
		saveItem.setImage(SAVE_IMAGE);
		saveItem.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				saveResult();
			}

		});

		MenuItem closeItem = new MenuItem(fileMenu, SWT.PUSH);
		closeItem.setText("Close");
		closeItem.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				close();
			}

		});

		switchItem = new MenuItem(mainMenu, SWT.PUSH);
		switchItem.setImage(SHOW_HIDE_IMAGE);
		if (showSelected)
		{
			switchItem.setText("Hide Selected");
		}
		else
		{
			switchItem.setText("Show Selected");
		}
		switchItem.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				switchAction();
			}
		});
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent)
	{
		Composite topComposite = (Composite) super.createDialogArea(parent);
		GridLayout layout = new GridLayout(1, false);
		topComposite.setLayout(layout);

		GridData gridData;

		Label mainResultLabel = new Label(topComposite, SWT.NONE);
		mainResultLabel.setText("The result of the expression is : " + globalResult + ".");
		gridData = new GridData();
		gridData.minimumWidth = 300;
		gridData.widthHint = 300;
		mainResultLabel.setLayoutData(gridData);

		Label subResultLabel = new Label(topComposite, SWT.NONE);
		subResultLabel.setText("Here is the result of the rule : ");
		gridData = new GridData();
		subResultLabel.setLayoutData(gridData);

		treeViewer = CheckresultmodelTreeViewerFactory.createTreeViewer(topComposite, showSelected);
		treeViewer.setInput(resultModel);
		gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		gridData.minimumHeight = 200;
		gridData.heightHint = 300;
		gridData.minimumWidth = 500;
		gridData.widthHint = 700;
		treeViewer.getTree().setLayoutData(gridData);

		return topComposite;
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent)
	{
	}

	private void saveResult()
	{
		BasicNewFileResourceWizard wizard = new BasicNewFileResourceWizard();
		wizard.init(PlatformUI.getWorkbench(), previousSelection);

		WizardDialog dialog = new WizardDialog(getShell(), wizard)
		{
			protected void finishPressed()
			{
				WizardNewFileCreationPage wizardPage = (WizardNewFileCreationPage) getCurrentPage();
				IPath workspace = ResourcesPlugin.getWorkspace().getRoot().getLocation();
				IPath file = wizardPage.getContainerFullPath().append(wizardPage.getFileName());
				IPath fullPath = workspace.append(file).addFileExtension("checkresultmodel");

				IService writeService = ServicesManager.getInstance().getService(IServiceCst.WRITE_EMF_XMI);
				HashMap<String, Object> parameters = new HashMap<String, Object>();
				parameters.put("PathFileString", fullPath.toString());
				parameters.put("Model", resultModel);
				writeService.serviceRun(parameters);
				setReturnCode(OK);
				getWizard().dispose();
				getWizard().setContainer(null);
				super.close();
			}
		};
		dialog.open();
	}

	private void switchAction()
	{
		showSelected = !showSelected;
		treeViewer.setContentProvider(CheckresultmodelTreeViewerFactory.getContentProvider(showSelected));
		if (showSelected)
		{
			switchItem.setText("Hide Selected");
		}
		else
		{
			switchItem.setText("Show Selected");
		}
	}
}
