/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), S�bastien Gabel (CS)
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *********************************************************************************************/
package org.topcased.editors.ocleditor.actions;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ocl.OCL;
import org.eclipse.ocl.Query;
import org.eclipse.ocl.ecore.CallOperationAction;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.ecore.SendSignalAction;
import org.eclipse.ocl.helper.OCLHelper;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;
import org.topcased.checkresultmodel.CheckresultmodelFactory;
import org.topcased.checkresultmodel.Context;
import org.topcased.checkresultmodel.Item;
import org.topcased.checkresultmodel.Model;
import org.topcased.checkresultmodel.ModelInformation;
import org.topcased.checkresultmodel.Package;
import org.topcased.checkresultmodel.Rule;
import org.topcased.checkresultmodel.State;
import org.topcased.checkresultmodel.dialog.CheckResultDialog;
import org.topcased.editors.ocleditor.editor.OCLEditor;
import org.topcased.editors.ocleditor.editor.OCLEditorDocumentPartitioner;

public class LoadModelAction implements IEditorActionDelegate
{

	/**
	 * Current editor
	 */
	private IEditorPart currentEditor = null;

	/**
	 * Factory
	 */
	private static CheckresultmodelFactory resultFactory = CheckresultmodelFactory.eINSTANCE;

	/**
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action)
	{
		if (currentEditor != null)
		{
			Model resultModel = resultFactory.createModel();
			EObject modelInstance = null;
			EPackage ePackage = null;
			OCLEditor editor = (OCLEditor) currentEditor;
			TextSelection currentSelection = (TextSelection) editor.getSelectionProvider().getSelection();
			ITypedRegion partition = null;
			try
			{
				partition = editor.getDocumentProvider().getDocument(editor.getEditorInput()).getPartition(currentSelection.getOffset());
			}
			catch (BadLocationException e1)
			{
				CorePlugin.log(e1);
			}

			if (partition != null && partition.getType().equals(OCLEditorDocumentPartitioner.RULE_CONTENT))
			{
				EClassifier context = editor.getContextClassifierForPartition(partition);

				if (context instanceof EClass)
				{
					Package contextPackage = resultFactory.createPackage();
					contextPackage.setName(context.getEPackage().getName());
					contextPackage.setPackageReference(context.getEPackage());
					resultModel.getContextPackages().add(contextPackage);

					Context contextClass = resultFactory.createContext();
					contextClass.setName(((EClass) context).getName());
					contextClass.setContextClass((EClass) context);
					contextPackage.getContext().add(contextClass);

					try
					{
						String rule = editor.getDocumentProvider().getDocument(editor.getEditorInput()).get(partition.getOffset(),
						        partition.getLength());
						rule = rule.substring(rule.indexOf(':') + 1);

						OCL<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> ocl = OCL
						        .newInstance(EcoreEnvironmentFactory.INSTANCE);
						OCLHelper<EClassifier, EOperation, EStructuralFeature, Constraint> helper = ocl.createOCLHelper();
						helper.setContext(context);
						Query<EClassifier, EClass, EObject> query = ocl.createQuery(helper.createQuery(rule));

						if (query.resultType() == ocl.getEnvironment().getOCLStandardLibrary().getBoolean())
						{
							Rule oclRule = resultFactory.createRule();
							oclRule.setName(query.queryText());
							oclRule.setRuleText(query.queryText());
							contextClass.getRules().add(oclRule);

							LoadModelDialog dialog = new LoadModelDialog(editor.getEditorSite().getShell(), context.getEPackage());
							dialog.open();
							if (dialog.getReturnCode() == Dialog.OK)
							{
								Object result = dialog.getResult();
								if (result != null)
								{
									modelInstance = (EObject) result;
									ModelInformation information = resultFactory.createModelInformation();
									information.setModelContent(modelInstance);
									resultModel.setInformation(information);
									ePackage = modelInstance.eClass().getEPackage();

									if (!EPackage.Registry.INSTANCE.containsKey(ePackage.getNsURI()))
									{
										EPackage.Registry.INSTANCE.put(ePackage.getNsURI(), ePackage);
									}
								}

								// TODO check the context subpackages too ?
								if (!context.getEPackage().getNsURI().equals(ePackage.getNsURI()))
								{
									MessageDialog.openWarning(editor.getEditorSite().getShell(), "Warning",
									        "The loaded model package does not match the rule package.");
								}

								Map<String, Object> parameters = new HashMap<String, Object>();
								parameters.put("Model", modelInstance);
								parameters.put("OCLRule", query.getExpression());

								IService checkService = ServicesManager.getInstance().getService(IServiceCst.OCL_CHECKER);

								Map<?, ?> resultMap = (Map<?, ?>) checkService.serviceRun(parameters);
								
								List<?> listSelected = (List<?>) resultMap.get("selected");
								for (int i = 0; i < listSelected.size(); i++)
								{
									Item item = resultFactory.createItem();
									item.setName(State.SELECTED);
									item.setReference((EObject) listSelected.get(i));
									item.setValue(Boolean.TRUE);
									oclRule.getSelectedItems().add(item);
								}

								List<?> listRejected = (List<?>) resultMap.get("rejected");
								for (int i = 0; i < listRejected.size(); i++)
								{
									Item item = resultFactory.createItem();
									item.setName(State.REJECTED);
									item.setReference((EObject) listRejected.get(i));
									item.setValue(Boolean.FALSE);
									oclRule.getRejectedItems().add(item);
								}

								CheckResultDialog resultDialog = new CheckResultDialog(editor.getEditorSite().getShell(), resultModel,
										listRejected.isEmpty(), true);
								resultDialog.open();
							}
						}
						else
						{
							MessageDialog.openError(editor.getEditorSite().getShell(), "Error", "The rule is not a boolean rule.");
						}
					}
					catch (Exception e)
					{
						if (e.getMessage() != null)
						{
							CorePlugin.log(e.getMessage(), IStatus.ERROR);
						}
						CorePlugin.log(e);
						MessageDialog.openError(editor.getEditorSite().getShell(), "Error",
						        "An error occured, the expression may be incorrect : " + e.getMessage());
					}
				}
				else
				{
					MessageDialog.openInformation(editor.getEditorSite().getShell(), "Result",
					        "The context is not a class. The rule cannot be processed.");
				}
			}
			else
			{
				MessageDialog.openError(editor.getEditorSite().getShell(), "Error", "The current element is not a rule.");
			}
		}
		else
		{
			CorePlugin.log("No editor was set for this action.", IStatus.ERROR);
		}
	}

	/**
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection)
	{
		// nothing to do
	}

	/**
	 * @see org.eclipse.ui.IEditorActionDelegate#setActiveEditor(org.eclipse.jface.action.IAction, org.eclipse.ui.IEditorPart)
	 */
	public void setActiveEditor(IAction action, IEditorPart targetEditor)
	{
		currentEditor = targetEditor;
	}
}
