/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS)
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *********************************************************************************************/
package org.topcased.editors.ocleditor.actions;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;

public class LoadModelDialog extends Dialog
{

	private Text modelText;

	private Object result;

	private EPackage rulePackage;

	/**
	 * Constructor
	 * 
	 * @param parentShell The parent shell
	 * @param thePackage The current package
	 */
	public LoadModelDialog(Shell parentShell, EPackage thePackage)
	{
		super(parentShell);
		this.rulePackage = thePackage;
	}

	/**
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell shell)
	{
		super.configureShell(shell);
		shell.setText("Load model dialog");
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent)
	{
		Composite topComposite = (Composite) super.createDialogArea(parent);
		GridLayout layout = new GridLayout(4, false);
		topComposite.setLayout(layout);

		GridData gData;

		Label modelLabel = new Label(topComposite, SWT.NONE);
		modelLabel.setText("Model instance : ");

		modelText = new Text(topComposite, SWT.SINGLE | SWT.BORDER);
		gData = new GridData(SWT.FILL, SWT.NONE, true, false);
		gData.minimumWidth = 300;
		modelText.setLayoutData(gData);

		Button modelBrowseButton = new Button(topComposite, SWT.PUSH);
		modelBrowseButton.setText("Browse...");
		modelBrowseButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				FileDialog dialog = new FileDialog(getShell());
				String result = dialog.open();
				if (result != null)
				{
					modelText.setText(result);
					IService readService = ServicesManager.getInstance().getService(IServiceCst.READ_XMI_EMF);
					Map<String, Object> parameters = new HashMap<String, Object>();
					parameters.put("XMIFile", modelText.getText());
					EObject resultRead = (EObject) readService.serviceRun(parameters);
					if (resultRead != null)
					{
						if (!resultRead.eClass().getEPackage().getNsURI().equals(rulePackage.getNsURI()))
						{
							MessageDialog.openError(getShell(), "Package Error",
									"The package of the model is different from the package of the rule.");
							getButton(IDialogConstants.OK_ID).setEnabled(false);
						}
						else
						{
							getButton(IDialogConstants.OK_ID).setEnabled(true);
						}
					}
					else
					{
						MessageDialog.openError(getShell(), "Model Error", "The model could not be loaded.");
						getButton(IDialogConstants.OK_ID).setEnabled(false);
					}
				}
			}
		});

		Button modelBrowseWorkspaceButton = new Button(topComposite, SWT.PUSH);
		modelBrowseWorkspaceButton.setText("Browse Workspace...");
		modelBrowseWorkspaceButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				ResourceSelectionDialog dialog = new ResourceSelectionDialog(getShell(), ResourcesPlugin.getWorkspace().getRoot(),
						"Select a model");
				if (dialog.open() == Window.OK)
				{
					Object[] result = dialog.getResult();
					if (result != null)
					{
						if (result.length > 0)
						{
							if (result[0] instanceof IFile)
							{
								modelText.setText(((IFile) result[0]).getFullPath().toString());
								IService readService = ServicesManager.getInstance().getService(IServiceCst.READ_XMI_EMF);
								Map<String, Object> parameters = new HashMap<String, Object>();
								parameters.put("XMIFile", modelText.getText());
								EObject resultRead = (EObject) readService.serviceRun(parameters);
								if (resultRead != null)
								{
									if (!resultRead.eClass().getEPackage().getNsURI().equals(rulePackage.getNsURI()))
									{
										MessageDialog.openError(getShell(), "Package Error",
												"The package of the model is different from the package of the rule.");
										getButton(IDialogConstants.OK_ID).setEnabled(false);
									}
									else
									{
										getButton(IDialogConstants.OK_ID).setEnabled(true);
									}
								}
								else
								{
									MessageDialog.openError(getShell(), "Model Error", "The model could not be loaded.");
									getButton(IDialogConstants.OK_ID).setEnabled(false);
								}

							}
						}
					}
				}
			}
		});

		return topComposite;
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createButton(org.eclipse.swt.widgets.Composite, int, java.lang.String, boolean)
	 */
	protected Button createButton(Composite parent, int id, String label, boolean defaultButton)
	{
		Button button = super.createButton(parent, id, label, defaultButton);
		if (id == IDialogConstants.OK_ID)
		{
			button.setEnabled(false);

		}
		return button;
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed()
	{
		IService readService = ServicesManager.getInstance().getService("org.topcased.service.readxmiemf.read");
		Map<String, Object> parameters = new HashMap<String, Object>();
		parameters.put("XMIFile", modelText.getText());
		try
		{
			result = readService.serviceRun(parameters);
			if (result != null)
			{
				close();
			}
			else
			{
				MessageDialog.openError(this.getShell(), "Error", "No model could be read.");
			}
		}
		catch (Exception e)
		{
			MessageDialog.openError(this.getShell(), "Error", "An error occured while reading the model.");
		}
	}

	/**
	 * Returns the result
	 * 
	 * @return the result
	 */
	public Object getResult()
	{
		return result;
	}
}
