/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.editors.ocleditor.wizard;

import java.io.File;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;

/**
 * Wizard allowing creation of OCL rule files.
 */
public class NewRuleFileWizardPage extends WizardPage
{
	private Text containerText;

	private Text fileText;

	private Combo modelUriCombo;

	private Text modelText;

	private Button modelUriRadioButton;

	private Button modelFileRadioButton;

	private Button modelBrowseWorkspaceButton;

	private Button modelBrowseSystemButton;

	private ISelection selection;

	private ComboViewer modelUriComboViewer;

	/**
	 * Constructor
	 * 
	 * @param selection The selection
	 */
	protected NewRuleFileWizardPage(ISelection selection)
	{
		super("page");
		setTitle("OCL Rule File");
		setDescription("Creates a new ocl rule file based on an user model.");

		this.selection = selection;
	}

	/**
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent)
	{
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;

		GridData gd;

		Label containerLabel = new Label(container, SWT.NONE);
		containerLabel.setText("Location : ");
		gd = new GridData();
		gd.widthHint = containerLabel.getText().length() * containerLabel.getFont().getFontData()[0].getHeight() * 2 / 3;// 80;
		containerLabel.setLayoutData(gd);

		containerText = new Text(container, SWT.BORDER | SWT.SINGLE);
		containerText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		containerText.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				dialogChanged();
			}
		});

		Button button = new Button(container, SWT.PUSH);
		button.setText("Browse...");
		button.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				handleBrowse();
			}
		});

		Label fileLabel = new Label(container, SWT.NONE);
		fileLabel.setText("File name : ");

		fileText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(SWT.FILL, SWT.NONE, true, false);
		gd.horizontalSpan = 2;
		fileText.setLayoutData(gd);
		fileText.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				dialogChanged();
			}
		});

		modelUriRadioButton = new Button(container, SWT.RADIO);
		modelUriRadioButton.setText("Select a model URI : ");
		modelUriRadioButton.setSelection(true);
		gd = new GridData();
		gd.widthHint = modelUriRadioButton.getFont().getFontData()[0].getHeight() * modelUriRadioButton.getText().length() * 2 / 3 + 20;// 100;
		modelUriRadioButton.setLayoutData(gd);
		modelUriRadioButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				modelUriCombo.setEnabled(true);
				modelText.setEnabled(false);
				modelBrowseWorkspaceButton.setEnabled(false);
				modelBrowseSystemButton.setEnabled(false);
				dialogChanged();
			}
		});

		modelUriCombo = new Combo(container, SWT.READ_ONLY | SWT.BORDER);
		gd = new GridData(SWT.FILL, SWT.NONE, true, false);
		gd.horizontalSpan = 2;
		modelUriCombo.setLayoutData(gd);
		//modelUriCombo.setVisibleItemCount(15);
		modelUriCombo.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				dialogChanged();
			}
		});

		modelUriComboViewer = new ComboViewer(modelUriCombo);
		modelUriComboViewer.setSorter(new ViewerSorter());

		fillComboBox();

		modelFileRadioButton = new Button(container, SWT.RADIO);
		modelFileRadioButton.setText("Select a model file :");
		modelFileRadioButton.setSelection(false);
		modelFileRadioButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				modelUriCombo.setEnabled(false);
				modelText.setEnabled(true);
				modelBrowseWorkspaceButton.setEnabled(true);
				modelBrowseSystemButton.setEnabled(true);
				dialogChanged();
			}
		});

		modelText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(SWT.FILL, SWT.NONE, true, false);
		gd.horizontalSpan = 2;
		modelText.setLayoutData(gd);
		modelText.setEnabled(false);
		modelText.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				dialogChanged();
			}
		});

		Composite modelButtonsComposite = new Composite(container, SWT.NONE);
		gd = new GridData(SWT.RIGHT, SWT.NONE, true, false);
		gd.horizontalSpan = 3;
		modelButtonsComposite.setLayoutData(gd);
		modelButtonsComposite.setLayout(new GridLayout(2 , true));

		modelBrowseWorkspaceButton = new Button(modelButtonsComposite, SWT.PUSH);
		modelBrowseWorkspaceButton.setText("Browse workspace...");
		modelBrowseWorkspaceButton.setEnabled(false);
		modelBrowseWorkspaceButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				handleWorkspaceBrowse();
			}
		});

		modelBrowseSystemButton = new Button(modelButtonsComposite, SWT.PUSH);
		modelBrowseSystemButton.setText("Browse file system...");
		modelBrowseSystemButton.setEnabled(false);
		modelBrowseSystemButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				handleSystemBrowse();
			}
		});

		initialize();
		dialogChanged();
		setControl(container);
	}

	/**
	 * Fill the text fields with default information
	 * 
	 */
	private void initialize()
	{
		if (selection != null && selection.isEmpty() == false && selection instanceof IStructuredSelection)
		{
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1)
            {
	            return;
            }
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource)
			{
				IContainer container;
				if (obj instanceof IContainer)
                {
	                container = (IContainer) obj;
                }
                else
                {
	                container = ((IResource) obj).getParent();
                }
				containerText.setText(container.getFullPath().toString());
			}
		}
		fileText.setText("ocl_rule.ocl");
	}

	private void handleBrowse()
	{
		ContainerSelectionDialog dialog = new ContainerSelectionDialog(getShell(), ResourcesPlugin.getWorkspace().getRoot(), false,
				"Select new file container");
		if (dialog.open() == ContainerSelectionDialog.OK)
		{
			Object[] result = dialog.getResult();
			if (result.length == 1)
			{
				containerText.setText(((Path) result[0]).toString());
			}
		}
	}

	private void handleWorkspaceBrowse()
	{
		ResourceSelectionDialog dialog = new ResourceSelectionDialog(getShell(), ResourcesPlugin.getWorkspace().getRoot(),
				"Select new file container");
		if (dialog.open() == ResourceSelectionDialog.OK)
		{
			Object[] result = dialog.getResult();
			if (result.length == 1)
			{
				modelText.setText(((IFile) result[0]).getFullPath().toString());
			}
		}
	}

	private void handleSystemBrowse()
	{
		FileDialog dialog = new FileDialog(getShell());
		String result = dialog.open();
		if (result != null && !result.equals(""))
		{
			modelText.setText(new File(result).getPath());
		}
	}

	/**
	 * Fills in the list according to values register in EMF registry.
	 */
	private void fillComboBox()
	{
		for (String uri : EPackage.Registry.INSTANCE.keySet())
		{
			modelUriComboViewer.add(uri);
		}
		modelUriComboViewer.getCombo().select(0);
	}

	/**
	 * Handles error/warning messages when items from the dialog change
	 */
	private void dialogChanged()
	{
		IResource container = ResourcesPlugin.getWorkspace().getRoot().findMember(new Path(getContainerName()));
		String fileName = getFileName();

		if (getContainerName().length() == 0)
		{
			updateStatus("File location must be specified");
			return;
		}
		if (container == null || (container.getType() & (IResource.PROJECT | IResource.FOLDER)) == 0)
		{
			updateStatus("File location must exist");
			return;
		}
		if (!container.isAccessible())
		{
			updateStatus("Project must be writable");
			return;
		}
		if (fileName.length() == 0)
		{
			updateStatus("File name must be specified");
			return;
		}
		if (fileName.replace('\\', '/').indexOf('/', 1) > 0)
		{
			updateStatus("File name must be valid");
			return;
		}
		int dotLoc = fileName.lastIndexOf('.');
		if (dotLoc != -1)
		{
			String ext = fileName.substring(dotLoc + 1);
			if (ext.equalsIgnoreCase("ocl") == false)
			{
				updateStatus("File extension must be \"ocl\"");
				return;
			}
		}
		if (getMetamodelUri().length() == 0)
		{
			updateStatus("The model must be specified");
			return;
		}
		updateStatus(null);
	}

	private void updateStatus(String message)
	{
		setMessage(message, WizardPage.ERROR);
		setPageComplete(message == null);
	}

	public String getContainerName()
	{
		return containerText.getText();
	}

	public String getFileName()
	{
		return fileText.getText();
	}

	public String getMetamodelUri()
	{
		if (modelUriRadioButton.getSelection())
		{
			return modelUriCombo.getText();
		}
		else
		{
			return modelText.getText();
		}
	}

	public String getMetaModelFromFile()
	{
		if (modelUriRadioButton.getSelection())
		{
			return null;
		}
		else
		{
			return modelText.getText();
		}
	}
}
