/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.externalbus.core;

import java.util.HashSet;
import java.util.Set;

/**
 * Handle the class ExternalService defined by : 
 * "id", "name", "class", "typeId", "paramInId", "paramOutId" 
 * <br>
 */
public class ExternalService
{
    private String id = "";

    private String name = "";

    private IExternalService clazz = null;
    
    private String className = "";

    private String typeId = "";

    private Set<String> paramInId = new HashSet<String>();

    private String paramOutId = "";


    /**
     * Constructor
     * 
     * @param serviceId : String of the service id
     * @param serviceName : String of the service name
     * @param serviceClassName  : String of the service class name
     * @param externalServiceClass : IExternalService of the class
     */
    public ExternalService(String serviceId, String serviceName, String serviceClassName, IExternalService externalServiceClass)
    {
        id = new String(serviceId);
        name = new String(serviceName);
        className = new String(serviceClassName);
        clazz = externalServiceClass;
        
    }

    /**
     * Constructor
     * 
     * @param serviceId : String of the service id
     * @param serviceName : String of the service name
     * @param externalServiceClass : IExternalService of the class
     * @param idType : String of the type id
     * @param idParameterIn : set (of String) of parameter input id
     * @param IdParameterOut : parameter output id
     */
    public ExternalService(String serviceId, String serviceName, IExternalService externalServiceClass, String idType, Set<String> idParameterIn,
            String idParameterOut)
    {
        id = new String(serviceId);
        name = new String(serviceName);
        clazz = externalServiceClass;
        typeId = new String(idType);
        paramOutId = new String(idParameterOut);
        
        for (String element : idParameterIn)
        {
        	paramInId.add(element);
        }
    }
    
    /**
     * Set type id of this Service
     * @param idType : Type id
     */
    public void setTypeId(String idType)
    {
        typeId = idType;
    }
    
    /**
     * Set output ExternalParameter id of this Service
     * @param idParamOut : output ExternalParameter id
     */
    public void setParamOutId(String idParamOut)
    {
        paramOutId = idParamOut;
    }

    /**
     * Set input ExternalParameter id of this Service.
     * Add this id in the Set of input Paramter ids
     * @param idParamIn : output ExternalParameter id
     */
    public void setParamInId(String idParamIn)
    {
        paramInId.add(idParamIn);
    }
    
    
    /**
     * Returns service id of this service
     * 
     * @return service id
     */
    public String getId()
    {
        return id;
    }

    /**
     * Returns service name of this service
     * 
     * @return service name
     */
    public String getName()
    {
        return name;
    }

    /**
     * Returns service class name of this service
     * 
     * @return service class name
     */
    public String getClassName()
    {
        return className;
    }
    
    /**
     * Returns external service class of this service
     * 
     * @return service class
     */
    public IExternalService getClazz()
    {
        return clazz;
    }

    /**
     * Returns service type id of this service
     * 
     * @return type id of this service
     */
    public String getIdType()
    {
        return typeId;
    }

    /**
     * Returns service output parameter id of this service
     * 
     * @return id of output parameter of this service
     */
    public String getIdParameterOut()
    {
        return paramOutId;
    }

    /**
     * Returns service input parameter ids of this service
     * 
     * @return Set of all ids of input parameter of this service
     */
    public Set<String> getIdParameterIn()
    {
        return paramInId;
    }

}
