/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.externalbus.core;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.runtime.IStatus;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Create a serializable Eobject with all informations concerning the ExternalService <br>
 */
public class SerializableExternalService
{
	// Service arguments
	/** TAG : externalService */
	private static final String TAG_EXTERNALSERVICES = "externalServices";

	/** TAG : externalService */
	private static final String TAG_EXTERNALSERVICE = "externalService";

	/** Attribut of externalService : id */
	private static final String ATT_EXTERNALSERVICE_ID = "id";

	/** Attribut of externalService : name */
	private static final String ATT_EXTERNALSERVICE_NAME = "name";

	// ExternalParameter arguments
	/** TAG : parameter */
	private static final String TAG_PARAM = "parameter";

	/** TAG : parameter in */
	private static final String TAG_PARAM_IN = "in";

	/** TAG : parameter out */
	private static final String TAG_PARAM_OUT = "out";

	/** Attribute of parameter : name */
	private static final String ATT_PARAM_NAME = "name";

	/** Attribute of parameter : description */
	private static final String ATT_PARAM_DESCRIPTION = "description";

	/** Attribute of parameter : kind */
	private static final String ATT_PARAM_KIND = "kind";
	
	/** Sets of external services */
	private Set<String> externalServiceIds = new HashSet<String>();

	/**
	 * Constructor
	 */
	public SerializableExternalService()
	{
		externalServiceIds = ExternalServicesManager.getInstance().getExternalServiceIds();
	}

	/**
	 * Constructor
	 * 
	 * @param aExternalServiceId
	 */
	public SerializableExternalService(String aExternalServiceId)
	{
		externalServiceIds.add(aExternalServiceId);
	}

	/**
	 * Constructor
	 * 
	 * @param aExternalServiceIds (set of all ExternalService id)
	 */
	public SerializableExternalService(Set<String> aExternalServiceIds)
	{
		externalServiceIds = aExternalServiceIds;
	}

	/**
	 * Serialize ExternalService informations : externalService, ExternalParameter (in & out), Type.
	 * 
	 * @return serializable document
	 */
	public Document externalServiceSerializated()
	{
		Document document = null;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		String externalServiceId = "";
		try
		{
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.newDocument();

			Element root = document.createElement(TAG_EXTERNALSERVICES);
			document.appendChild(root);

			if (externalServiceIds != null)
			{
				Iterator<String> itExt = externalServiceIds.iterator();
				while (itExt.hasNext())
				{
					externalServiceId = itExt.next();
					ExternalService externalService = ExternalServicesManager.getInstance().getExternalService(externalServiceId);
					if (externalService != null)
					{
						// Tag creation : externalService
						Element extServiceTag = document.createElement(TAG_EXTERNALSERVICE);
						root.appendChild(extServiceTag);
						extServiceTag.setAttribute(ATT_EXTERNALSERVICE_NAME, externalService.getName());
						extServiceTag.setAttribute(ATT_EXTERNALSERVICE_ID, externalService.getId());

						// Tag parameter IN
						Element parameterInTag = document.createElement(TAG_PARAM_IN);
						extServiceTag.appendChild(parameterInTag);
						Set<String> paramInIds = externalService.getIdParameterIn();
						if (paramInIds != null)
						{
							Iterator<String> it = paramInIds.iterator();
							while (it.hasNext())
							{
								String paramInId = it.next();
								ExternalParameter param = ExternalServicesManager.getInstance().getParameter(paramInId);
								if (param != null)
								{
									Element parameterTag = document.createElement(TAG_PARAM);
									parameterInTag.appendChild(parameterTag);
									parameterTag.setAttribute(ATT_PARAM_NAME, param.getNameParam());
									parameterTag.setAttribute(ATT_PARAM_DESCRIPTION, param.getDescriptionParam());
									parameterTag.setAttribute(ATT_PARAM_KIND, param.getKindParam());
								}
							}
						}

						// Tag parameter OUT
						Element parameterOutTag = document.createElement(TAG_PARAM_OUT);
						extServiceTag.appendChild(parameterOutTag);
						String paramOut = externalService.getIdParameterOut();
						if (paramOut != null)
						{
							ExternalParameter param = ExternalServicesManager.getInstance().getParameter(paramOut);
							if (param != null)
							{
								Element parameterTag = document.createElement(TAG_PARAM);
								parameterOutTag.appendChild(parameterTag);
								parameterTag.setAttribute(ATT_PARAM_NAME, param.getNameParam());
								parameterTag.setAttribute(ATT_PARAM_DESCRIPTION, param.getDescriptionParam());
								parameterTag.setAttribute(ATT_PARAM_KIND, param.getKindParam());
							}
						}
					}
				}
			}
		}
		catch (ParserConfigurationException e)
		{
			CorePlugin.log(this.getClass(), "Parse Configuration Exception" + e.getMessage(), IStatus.ERROR);

		}
		return document;
	}
}
