/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.externalbus.core.helper;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Set of convenient methods to invoke external services.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 * 
 */
public class ExternalServiceHelper
{

	/**
	 * Creates a Document instance from an XML String.
	 * 
	 * @param xmlSource - String in the xml format.
	 * @return a Document
	 * @throws SAXException
	 * @throws ParserConfigurationException
	 * @throws IOException
	 */
	public static Document stringToDom(String xmlSource) throws SAXException, ParserConfigurationException, IOException
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		return builder.parse(new InputSource(new StringReader(xmlSource)));
	}

	/**
	 * Converts an XML document into a String.
	 * 
	 * @param node An XML node
	 * @return a String representing the flattened node
	 */
	public static String documentToString(Node node) throws TransformerException
	{
		Source source = new DOMSource(node);
		StringWriter stringWriter = new StringWriter();
		Result result = new StreamResult(stringWriter);
		TransformerFactory factory = TransformerFactory.newInstance();
		Transformer transformer = factory.newTransformer();
		transformer.transform(source, result);
		return stringWriter.getBuffer().toString();
	}

	/**
	 * @return
	 */
	public static Document sendErrorParameterDocument() throws ParserConfigurationException
	{
		Document returnDocument = null;
		returnDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		return returnDocument;
	}

	/**
	 * @param modelAsElement
	 * @throws ParserConfigurationException
	 * @throws TransformerConfigurationException
	 * @throws TransformerFactoryConfigurationError
	 * @throws TransformerException
	 */
	public static void transformModel(Element modelAsElement, String fileName_) throws ParserConfigurationException,
	        TransformerConfigurationException, TransformerFactoryConfigurationError, TransformerException
	{
		Document modelDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		modelDocument.appendChild(modelDocument.importNode(modelAsElement, true));

		DOMSource source = new DOMSource(modelDocument);
		StreamResult result = new StreamResult(new File(fileName_));
		ByteArrayOutputStream bytearray = new ByteArrayOutputStream();
		result = new StreamResult(bytearray);
		Transformer transformer = TransformerFactory.newInstance().newTransformer();
		transformer.transform(source, result);
	}
}
