/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.generator.services.wizard;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.pde.core.plugin.IPluginModelBase;
import org.eclipse.pde.core.plugin.IPluginReference;
import org.eclipse.pde.internal.ui.wizards.templates.PluginReference;
import org.eclipse.pde.ui.IFieldData;
import org.eclipse.pde.ui.IPluginContentWizard;
import org.topcased.bus.core.CorePlugin;
import org.topcased.generator.services.generation.TopcasedBusServicesConstants;
import org.topcased.generator.services.generation.TopcasedBusServicesGenerator;
import org.topcased.generator.services.generation.TopcasedPluginFileGenerator;

/**
 * This class represents the part of the wizard that is specific to the plug-in/service
 * creation.<br>
 * This service can run an external tool and define an external service extension.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class TopcasedBusServicesGeneratorWizard extends Wizard implements IPluginContentWizard
{
	/**
	 * Key for the <i>Project Name</i>.
	 */
	private static final String PROJECT_NAME_KEY = "projectName";

	/**
	 * The Topcased Setting page to add to the existing wizard
	 */
	private TopcasedBusServicesGeneratorWizardPage newTopcasedPage;

	/**
	 * @see org.eclipse.pde.ui.IPluginContentWizard#getNewFiles()
	 */
	public String[] getNewFiles()
	{
		return new String[0];
	}

	/**
	 * @see org.eclipse.pde.ui.IPluginContentWizard#init(org.eclipse.pde.ui.IFieldData)
	 */
	public void init(IFieldData data)
	{
		// Nothing to do
	}

	/**
	 * @see org.eclipse.pde.ui.IPluginContentWizard#getDependencies(java.lang.String)
	 */
	public IPluginReference[] getDependencies(String schemaVersion)
	{
		// The bus core plugins are referenced by the classes
		// So they must be returned here to be added to the list of the plug-in dependencies
		IPluginReference[] runtimeDependency = new IPluginReference[5];
		runtimeDependency[0] = new PluginReference("org.eclipse.emf.ecore", "", 0);
		runtimeDependency[1] = new PluginReference("org.eclipse.emf.ecore.edit", "", 0);
		runtimeDependency[2] = new PluginReference("org.eclipse.core.resources", "", 0);
		runtimeDependency[3] = new PluginReference("org.topcased.bus.core", "", 0);
		runtimeDependency[4] = new PluginReference("org.topcased.generator.services", "", 0);

		return runtimeDependency;
	}

	/**
	 * @see org.eclipse.pde.ui.IPluginContentWizard#performFinish(org.eclipse.core.resources.IProject,
	 *      org.eclipse.pde.core.plugin.IPluginModelBase,
	 *      org.eclipse.core.runtime.IProgressMonitor)
	 */
	public boolean performFinish(IProject project, IPluginModelBase model, IProgressMonitor monitor)
	{
		// This map contains some elements that will be used during the classes generation.
		final Map<String, String> arguments = new HashMap<String, String>();
		arguments.put(PROJECT_NAME_KEY, project.getName().toLowerCase());
		arguments.put(TopcasedBusServicesConstants.LAST_NAME_KEY, newTopcasedPage.getLastName());
		arguments.put(TopcasedBusServicesConstants.FIRST_NAME_KEY, newTopcasedPage.getFirstName());
		try
		{
			TopcasedPluginFileGenerator.generatePluginXMLFile(project, model, newTopcasedPage.getServiceName());
			TopcasedBusServicesGenerator.generateTopcasedPlugin(arguments, project);
		}
		catch (Exception e)
		{
			CorePlugin.log(this.getClass(), "An error occured during the generation : " + e.getMessage(), IStatus.ERROR, e);
		}
		return true;
	}

	/**
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	public void addPages()
	{
		super.addPages();
		newTopcasedPage = new TopcasedBusServicesGeneratorWizardPage();
		addPage(newTopcasedPage);
	}

	/**
	 * @see org.eclipse.jface.wizard.Wizard#getWindowTitle()
	 */
	public String getWindowTitle()
	{
		return newTopcasedPage.getTitle();
	}

	/**
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	public boolean performFinish()
	{
		return true;
	}
}