/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Guillaume Jolly (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.generator.services.wizard;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.topcased.facilities.dialogs.ITopcasedWizardConstants;
import org.topcased.generator.services.TopcasedBusServicesGeneratorPlugin;
import org.topcased.generator.services.generation.TopcasedBusServicesConstants;

/**
 * This wizard gathers information/parameters for the future generated TOPCASED service.<br>
 * The generated plug-in is just a sample structure to show developer how to carry out a
 * Topcased Service and how to call it. It proposes to :
 * <ul>
 * <li>give a virtual Topcased user name (just for test purpose)</li>
 * <li>give a free text area (just for test purpose)</li>
 * </ul>
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class TopcasedBusServicesGeneratorWizardPage extends WizardPage implements ITopcasedWizardConstants
{
	/**
	 * Last name text field
	 */
	private Text lastNameText;

	/**
	 * First name text field
	 */
	private Text firstNameText;

	/**
	 * Service Name
	 */
	private Text serviceNameText;

	/**
	 * Constructor
	 */
	public TopcasedBusServicesGeneratorWizardPage()
	{
		super("");
		setTitle("TOPCASED External Tool Call");
		setDescription("Fill in parameters used to call the future service.");
		setImageDescriptor(TopcasedBusServicesGeneratorPlugin.getImageDescriptor(TOPCASED_LOGO_PATH));
	}

	/**
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent)
	{
		Composite toplevel = new Composite(parent, SWT.NONE);
		GridLayout mainLayout = new GridLayout();
		mainLayout.verticalSpacing = 10;
		toplevel.setLayout(mainLayout);

		// First group
		final Group firstGroup = new Group(toplevel, SWT.SHADOW_OUT);
		firstGroup.setText("About the service : ");
		firstGroup.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		firstGroup.setLayout(new GridLayout(2, false));

		// First field
		final Label serviceNameLabel = new Label(firstGroup, SWT.NONE);
		serviceNameLabel.setText("Service Name : ");

		serviceNameText = new Text(firstGroup, SWT.SINGLE | SWT.BORDER);
		serviceNameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// Second group
		final Group secondGroup = new Group(toplevel, SWT.SHADOW_OUT);
		secondGroup.setText("Register a new Topcased user : ");
		secondGroup.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		secondGroup.setLayout(new GridLayout(2, false));

		// First field
		final Label firstNameLabel = new Label(secondGroup, SWT.NONE);
		firstNameLabel.setText("First Name : ");

		firstNameText = new Text(secondGroup, SWT.SINGLE | SWT.BORDER);
		firstNameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		firstNameText.setText(TopcasedBusServicesConstants.FIRST_NAME_VALUE);

		// Second field
		final Label lastNameLabel = new Label(secondGroup, SWT.NONE);
		lastNameLabel.setText("Last Name : ");

		lastNameText = new Text(secondGroup, SWT.BORDER | SWT.SINGLE);
		lastNameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		lastNameText.setText(TopcasedBusServicesConstants.LAST_NAME_VALUE);
		setControl(toplevel);
	}

	/**
	 * Retrieves the content of the <b>first name</b> field.
	 * 
	 * @return the service name to invoke
	 */
	public String getFirstName()
	{
		return firstNameText.getText();
	}

	/**
	 * Retrieves the content of the <b>last name</b> field.
	 * 
	 * @return the first name of the current topcased user
	 */
	public String getLastName()
	{
		return lastNameText.getText();
	}

	/**
	 * Retrieves the content of the <b>service name</b> field.
	 * 
	 * @return the future service name
	 */
	public String getServiceName()
	{
		return serviceNameText.getText();
	}
}
