/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.commands;

import org.eclipse.gef.commands.Command;
import org.eclipse.jface.resource.StringConverter;
import org.eclipse.swt.graphics.RGB;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.util.DIUtils;

/**
 * The command used to change the background color of a graphElement
 * 
 * Creation : 22 mars 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 * @deprecated Use the {@link ChangeDiagramElementPropertyCommand} instead
 */
public class ChangeBackgroundColorCommand extends Command
{

    private DiagramElement element;

    private String oldRGB;

    private String newRGB;

    /**
     * Constructor
     * 
     * @param element the DiagramElement
     * @param rgb the new color
     */
    public ChangeBackgroundColorCommand(DiagramElement element, RGB rgb)
    {
        super("Change Background Color");
        this.element = element;
        this.newRGB = StringConverter.asString(rgb);
    }

    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        return element != null;
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        oldRGB = DIUtils.getPropertyValue(element, ModelerPropertyConstants.BACKGROUND_COLOR);
        redo();
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        DIUtils.setProperty(element, ModelerPropertyConstants.BACKGROUND_COLOR, newRGB);
    }

    /**
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        if ("".equals(oldRGB))
        {
            DIUtils.setProperty(element, ModelerPropertyConstants.BACKGROUND_COLOR, null);
        }
        else
        {
            DIUtils.setProperty(element, ModelerPropertyConstants.BACKGROUND_COLOR, oldRGB);
        }
    }

}
