/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.commands;

import org.eclipse.gef.commands.Command;
import org.eclipse.jface.resource.StringConverter;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Display;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.util.DIUtils;

/**
 * The command used to change the Font of a graphElement. Before changing the
 * font in the model, check if the Font is not equal to the default font (in
 * this case, remove the property from the DI model)
 * 
 * Creation : 4 oct. 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 * @deprecated Use the {@link ChangeDiagramElementPropertyCommand} instead
 */
public class ChangeFontCommand extends Command
{

    private DiagramElement element;

    private String oldFontData;

    private String newFontData;

    /**
     * Constructor
     * 
     * @param element the DiagramElement
     * @param fontData the new color
     */
    public ChangeFontCommand(DiagramElement element, FontData fontData)
    {
        super("Change Font");
        this.element = element;
        this.newFontData = StringConverter.asString(fontData);
    }

    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        return element != null;
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        oldFontData = DIUtils.getPropertyValue(element, ModelerPropertyConstants.FONT);
        redo();
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        // check if the newFontData is equal to the default Font
        if (newFontData.equals(StringConverter.asString(Display.getDefault().getSystemFont().getFontData()[0])))
        {
            // delete the property
            DIUtils.setProperty(element, ModelerPropertyConstants.FONT, null);
        }
        else
        {
            // add a new Property that identify the new FontData
            DIUtils.setProperty(element, ModelerPropertyConstants.FONT, newFontData);
        }
    }

    /**
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        if ("".equals(oldFontData))
        {
            DIUtils.setProperty(element, ModelerPropertyConstants.FONT, null);
        }
        else
        {
            DIUtils.setProperty(element, ModelerPropertyConstants.FONT, oldFontData);
        }
    }

}
