/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.commands;

import org.eclipse.gef.commands.Command;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;

/**
 * The command used to change the foreground color of a graphElement
 * 
 * Creation : 3 oct. 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 * @deprecated Use the {@link ChangeDiagramElementPropertyCommand} instead
 */
public class ChangeMultiPortStateCommand extends Command
{

    private GraphElement element;

    private String oldState;

    private String newState;

    /**
     * Constructor
     * 
     * @param element the GraphElement
     * @param state the new state
     */
    public ChangeMultiPortStateCommand(GraphElement element, String state)
    {
        super("Change MultiPort State");
        this.element = element;
        this.newState = state;
    }

    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        return element != null;
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        oldState = DIUtils.getPropertyValue(element, ModelerPropertyConstants.MULTIPORT_STATE);
        redo();
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        DIUtils.setProperty(element, ModelerPropertyConstants.MULTIPORT_STATE, newState);
    }

    /**
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        if ("".equals(oldState))
        {
            DIUtils.setProperty(element, ModelerPropertyConstants.MULTIPORT_STATE, null);
        }
        else
        {
            DIUtils.setProperty(element, ModelerPropertyConstants.MULTIPORT_STATE, oldState);
        }
    }

}
