/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.commands;

import org.eclipse.gef.commands.Command;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.util.DIUtils;

/**
 * The command used to change the orientation of a diagram
 * 
 * Creation 27 avr. 2006
 *
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 * @deprecated Use the {@link ChangeDiagramElementPropertyCommand} instead
 */
public class ChangePageOrientationCommand extends Command
{

    private Diagram diagram;

    private String oldOrientation;

    private String newOrientation;

    /**
     * Constructor
     * 
     * @param diag the Diagram
     * @param newOrientation the new orientation of the diagram
     */
    public ChangePageOrientationCommand(Diagram diag, String newOrientation)
    {
        super("Change Diagram Orientation");
        this.diagram = diag;
        this.newOrientation = newOrientation;

    }

    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        return diagram != null;
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        oldOrientation = DIUtils.getPropertyValue(diagram, ModelerPropertyConstants.ORIENTATION);

        redo();
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        DIUtils.setProperty(diagram, ModelerPropertyConstants.ORIENTATION, newOrientation);
    }

    /**
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        DIUtils.setProperty(diagram, ModelerPropertyConstants.ORIENTATION, oldOrientation);
    }

}
