/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies), Nicolas Laleve (Anyware
 *    Technologies)  - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.commands;

import java.util.ArrayList;

import org.eclipse.gef.commands.Command;

/**
 * This command just act like the GEF CompoundCommand.
 * 
 * The difference is that you can add already executed command, so
 * they will not be executed a second time.
 * 
 * @see org.eclipse.gef.commands.CompoundCommand
 */
public class CompoundCommand extends org.eclipse.gef.commands.CompoundCommand
{
    private ArrayList unexecutedCommands;

    public CompoundCommand()
    {
        unexecutedCommands = new ArrayList();
    }
    
    public void add(Command command)
    {
        super.add(command);
        unexecutedCommands.add(command);
    }
    
    /**
     * Add an already executed command to the list of commands
     * 
     * @param command the executed command
     */
    public void addExecuted(Command command)
    {
        super.add(command);
    }
    
    public void execute()
    {
        for (int i = 0; i < unexecutedCommands.size(); i++) {
            Command cmd = (Command) unexecutedCommands.get(i);
            cmd.execute();
        }
    }
}
