/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), 
 *    Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies),
 *    Thomas Friol (Anyware Technologies)
 *    - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.documentation;

import java.util.EventObject;

import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.commands.CommandStackListener;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

/**
 * A class defining an empty composite to use in an IDocPage. <br/> Creation :
 * 10 oct. 2005
 * 
 * @author <a href="mailto:thomas.friol@anyware-tech.com">Thomas FRIOL</a>
 */
public abstract class DocPageComposite extends Composite implements CommandStackListener
{
    /* The edited model element */
    private EModelElement modelElement;

    private CommandStack commandStack;

    /**
     * Construct a new empty DocPageComposite.
     * 
     * @param parent the parent composite
     * @param style the composite style
     * @param stack the command stack to use to execute commands
     */
    public DocPageComposite(Composite parent, int style, CommandStack stack)
    {
        super(parent, style);
        setLayout(new GridLayout());
        this.commandStack = stack;
        this.commandStack.addCommandStackListener(this);

        this.addDisposeListener(new DisposeListener()
        {
            /**
             * @see org.eclipse.swt.events.DisposeListener#widgetDisposed(org.eclipse.swt.events.DisposeEvent)
             */
            public void widgetDisposed(DisposeEvent e)
            {
                handleDispose();
            }
        });
        
        createContents(this);
    }
    
    /**
     * Called when the composite is disposed.
     * <b>Subclasses must release local resources and listeners here.</b>
     */
    protected void handleDispose()
    {
        this.commandStack.removeCommandStackListener(this);
    }

    /**
     * Change the edited object
     * @param modelElement the edited object
     */
    public void setModelElement(EModelElement modelElement)
    {
        if (modelElement == null || this.modelElement != modelElement)
        {
            this.modelElement = modelElement;
            refresh();
        }
    }

    /**
     * Return this composite command stack.
     * 
     * @return a command stack
     */
    protected CommandStack getCommandStack()
    {
        return commandStack;
    }

    /**
     * Get the current edited model element.
     * 
     * @return a model element
     */
    protected EModelElement getModelElement()
    {
        return modelElement;
    }

    /**
     * Create the content of this composite. Clients should override this method
     * to create their own composite.
     * 
     * @param parent the parent composite
     */
    protected abstract void createContents(Composite parent);

    /**
     * Informs this composite that it needs to be refresh.
     */
    protected void refresh()
    {
        // Do nothing
    }
    
    /**
     * @see org.eclipse.gef.commands.CommandStackListener#commandStackChanged(java.util.EventObject)
     */
    public void commandStackChanged(EventObject event)
    {
        refresh();
    }
}
