/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/

package org.topcased.modeler.ecore.commands;

import java.util.Collections;
import java.util.Set;

import org.eclipse.emf.common.command.AbstractCommand;
import org.eclipse.emf.ecore.change.ChangeDescription;
import org.eclipse.emf.ecore.change.util.ChangeRecorder;
import org.eclipse.emf.ecore.resource.Resource;

/**
 * TODO Comment this class
 */
public abstract class EMFRecordingChangeCommand extends AbstractCommand
{
    private Set<Resource> notifiers;

    private ChangeRecorder recorder;

    private ChangeDescription change;

    /**
     * Constructor
     * 
     * @param resource
     */
    public EMFRecordingChangeCommand(Resource resource)
    {
        super("Recording Change Command");
        recorder = new ChangeRecorder();
        notifiers = Collections.singleton(resource);
    }

    /**
     * I run the runnable when I execute the first time.
     */
    public void execute()
    {
        try
        {
            recorder.beginRecording(notifiers);
            doExecute();
        }
        finally
        {
            change = recorder.endRecording();
        }
    }

    /**
     * I am ready to execute if I haven't recorded any changes, yet.
     * 
     * @return boolean
     */
    protected boolean prepare()
    {
        return change == null;
    }

    /**
     * TODO comment this method
     */
    protected abstract void doExecute();

    /**
     * Applies (undoes) changes recorded previously, recording the new changes
     * meanwhile.
     */
    private void applyChanges()
    {
        try
        {
            recorder.beginRecording(notifiers);
            change.apply();
        }
        finally
        {
            change = recorder.endRecording();
        }
    }

    /**
     * I can undo if I have recorded any changes previously.
     * 
     * @return boolean
     */
    public boolean canUndo()
    {
        return change != null;
    }

    /**
     * Undoes by applying recorded changes.
     */
    public void undo()
    {
        applyChanges();
    }

    /**
     * Redoes by applying changes recorded in the last undo.
     */
    public void redo()
    {
        applyChanges();
    }

    /**
     * @see org.eclipse.gef.commands.Command#dispose()
     */
    public void dispose()
    {
        change = null;
        recorder = null;
        notifiers = null;
    }
}