/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/

package org.topcased.modeler.ecore.documentation;

import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.topcased.modeler.documentation.EAnnotationCommentsComposite;
import org.topcased.modeler.documentation.EAnnotationDocPage;

/**
 * This Doc page adds a tab item for the GenModel Documentation
 * 
 * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class EcoreDocPage extends EAnnotationDocPage
{
    private static final String GENMODEL_SOURCE = "http://www.eclipse.org/emf/2002/GenModel";

    private static final String GENMODEL_DOC_KEY = "documentation";

    private EAnnotationCommentsComposite genModelComposite;

    /**
     * Constructor
     * 
     * @param stack basic Command Stack
     */
    public EcoreDocPage(CommandStack stack)
    {
        super(stack);
    }

    /**
     * @see org.topcased.modeler.documentation.EAnnotationDocPage#createTabItems(org.eclipse.swt.widgets.TabFolder)
     */
    protected void createTabItems(TabFolder tabFolder)
    {
        createGenModelTabItem(tabFolder);
        super.createTabItems(tabFolder);
    }

    /**
     * Create the content of the GenModel tab item. It contains a multi lines
     * text field in order to edit the documentation of GenModel annotation.
     * 
     * @param parent the parent tab folder
     */
    private void createGenModelTabItem(TabFolder parent)
    {
        TabItem tabItem = new TabItem(parent, SWT.NONE);
        tabItem.setText("GenModel Documentation");

        Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout());
        container.setLayoutData(new GridData(GridData.FILL_BOTH));

        tabItem.setControl(container);
        genModelComposite = new EAnnotationCommentsComposite(container, SWT.NONE, getCommandStack(), GENMODEL_SOURCE,
                GENMODEL_DOC_KEY);
        genModelComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
    }

    /**
     * @see org.topcased.modeler.documentation.EAnnotationDocPage#setModelElement(org.eclipse.emf.ecore.EModelElement)
     */
    public void setModelElement(EModelElement elt)
    {
        super.setModelElement(elt);

        genModelComposite.setModelElement(elt);
    }
}
