/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.ediagram;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.gef.EditPartFactory;
import org.topcased.modeler.ecore.EcorePlugin;
import org.topcased.modeler.ecore.ediagram.edit.EAnnotationEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EAttributeEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EClassEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EDataTypeEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EEnumEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EEnumLiteralEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EModelElementEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EOperationEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EPackageEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EReferenceEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EStringToStringMapEntryEditPart;
import org.topcased.modeler.editor.EditPart2ModelAdapterFactory;
import org.topcased.modeler.editor.IConfiguration;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.IPaletteManager;
import org.topcased.modeler.graphconf.DiagramGraphConf;

/**
 * A diagram configuration : manages Palette, EditPartFactory for this diagram.
 * <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class EcoreConfiguration implements IConfiguration
{

    private EcorePaletteManager paletteManager;

    private EcoreEditPartFactory editPartFactory;

    private EcoreCreationUtils creationUtils;

    /**
     * The DiagramGraphConf that contains graphical informations on the
     * configuration
     */
    private DiagramGraphConf diagramGraphConf;

    /**
     * Constructor. Initialize Adapter factories.
     * 
     * @generated
     */
    public EcoreConfiguration()
    {
        registerAdapters();
    }

    /**
     * Registers the Adapter Factories for all the EditParts
     * 
     * @generated
     */
    private void registerAdapters()
    {
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(EPackageEditPart.class, org.eclipse.emf.ecore.EPackage.class),
                EPackageEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(EClassEditPart.class, org.eclipse.emf.ecore.EClass.class),
                EClassEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(EOperationEditPart.class, org.eclipse.emf.ecore.EOperation.class),
                EOperationEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(EAttributeEditPart.class, org.eclipse.emf.ecore.EAttribute.class),
                EAttributeEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(EDataTypeEditPart.class, org.eclipse.emf.ecore.EDataType.class),
                EDataTypeEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(EEnumEditPart.class, org.eclipse.emf.ecore.EEnum.class),
                EEnumEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(EEnumLiteralEditPart.class, org.eclipse.emf.ecore.EEnumLiteral.class),
                EEnumLiteralEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(EAnnotationEditPart.class, org.eclipse.emf.ecore.EAnnotation.class),
                EAnnotationEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(EStringToStringMapEntryEditPart.class, java.util.Map.Entry.class),
                EStringToStringMapEntryEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(EModelElementEditPart.class, org.eclipse.emf.ecore.EModelElement.class),
                EModelElementEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(EReferenceEditPart.class, org.eclipse.emf.ecore.EReference.class),
                EReferenceEditPart.class);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getId()
     * @generated
     */
    public String getId()
    {
        return new String("org.topcased.modeler.ecore.ediagram");
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getName()
     * @generated
     */
    public String getName()
    {
        return new String("Ecore Diagram");
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getEditPartFactory()
     * @generated
     */
    public EditPartFactory getEditPartFactory()
    {
        if (editPartFactory == null)
        {
            editPartFactory = new EcoreEditPartFactory();
        }

        return editPartFactory;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getPaletteManager()
     * @generated
     */
    public IPaletteManager getPaletteManager()
    {
        if (paletteManager == null)
        {
            paletteManager = new EcorePaletteManager(getCreationUtils());
        }

        return paletteManager;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getCreationUtils()
     * @generated
     */
    public ICreationUtils getCreationUtils()
    {
        if (creationUtils == null)
        {
            creationUtils = new EcoreCreationUtils(getDiagramGraphConf());
        }

        return creationUtils;
    }

    /**
     * @see org.topcased.modeler.editor.IConfiguration#getDiagramGraphConf()
     */
    public DiagramGraphConf getDiagramGraphConf()
    {
        if (diagramGraphConf == null)
        {
            URI fileURI = URI.createURI(EcorePlugin.getDefault().getBundle().getResource(
                    "org/topcased/modeler/ecore/ediagram/diagram.graphconf").toString());
            ResourceSet resourceSet = new ResourceSetImpl();
            Resource resource = resourceSet.getResource(fileURI, true);
            if (resource != null && resource.getContents().get(0) instanceof DiagramGraphConf)
            {
                diagramGraphConf = (DiagramGraphConf) resource.getContents().get(0);
            }
        }

        return diagramGraphConf;
    }
}
