/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.ediagram;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreSwitch;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.topcased.model.management.IAnnotationConstants;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.ecore.ediagram.edit.AnnotationLinkEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EAnnotationDocumentationEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EAnnotationEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EAttributeEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EClassEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EDataTypeEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EEnumEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EEnumLiteralEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EModelElementEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EOperationEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EPackageEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EReferenceEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EStringToStringMapEntryEditPart;
import org.topcased.modeler.ecore.ediagram.edit.EcoreDiagramEditPart;
import org.topcased.modeler.ecore.ediagram.edit.InheritanceEditPart;
import org.topcased.modeler.edit.EListEditPart;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * Part Factory : associates a model object to its controller. <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class EcoreEditPartFactory implements EditPartFactory
{

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
     * @generated
     */
    public EditPart createEditPart(EditPart context, Object model)
    {
        if (model instanceof Diagram)
        {
            return new EcoreDiagramEditPart((Diagram) model);
        }
        else if (model instanceof GraphNode)
        {
            final GraphNode node = (GraphNode) model;
            EObject element = Utils.getElement(node);
            if (element != null)
            {
                if ("http://www.eclipse.org/emf/2002/Ecore".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new NodeEcoreSwitch(node).doSwitch(element);
                }
                return new EMFGraphNodeEditPart(node);
            }
            return new GraphNodeEditPart(node);
        }
        else if (model instanceof GraphEdge)
        {
            final GraphEdge edge = (GraphEdge) model;
            EObject element = Utils.getElement(edge);
            if (element != null)
            {
                if ("http://www.eclipse.org/emf/2002/Ecore".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new EdgeEcoreSwitch(edge).doSwitch(element);
                }
                return new EMFGraphEdgeEditPart(edge);
            }

            if (edge.getSemanticModel() instanceof SimpleSemanticModelElement)
            {
                if (EcoreSimpleObjectConstants.SIMPLE_OBJECT_INHERITANCE.equals(((SimpleSemanticModelElement) edge.getSemanticModel()).getTypeInfo()))
                {
                    return new InheritanceEditPart(edge);
                }
                if (EcoreSimpleObjectConstants.SIMPLE_OBJECT_ANNOTATIONLINK.equals(((SimpleSemanticModelElement) edge.getSemanticModel()).getTypeInfo()))
                {
                    return new AnnotationLinkEditPart(edge);
                }
            }

            return new GraphEdgeEditPart(edge);
        }

        throw new IllegalStateException("No edit part matches with the '" + model.getClass().getName() + "' model element. Check EcoreEditPartFactory#createEditPart(EditPart,Object) class");
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class NodeEcoreSwitch extends EcoreSwitch<Object>
    {
        /** The graphical node */
        private GraphNode node;

        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param node the graphical node
         * @generated
         */
        public NodeEcoreSwitch(GraphNode node)
        {
            this.node = node;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#caseEPackage(org.eclipse.emf.ecore.EPackage)
         * @generated
         */
        public Object caseEPackage(org.eclipse.emf.ecore.EPackage object)
        {
            return new EPackageEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#caseEClass(org.eclipse.emf.ecore.EClass)
         * @generated NOT
         */
        public Object caseEClass(org.eclipse.emf.ecore.EClass object)
        {
            String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
            if (!"".equals(feature))
            {
                int featureID = Integer.parseInt(feature);
                // REV Hack code to be compliant with previous Ecore models
                if (featureID == 9 || featureID == 19)
                {
                    featureID += 2;
                }
                EListEditPart list = new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
                list.setSynchronized(true);
                return list;
            }
            else
            {
                return new EClassEditPart(node);
            }
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#caseEOperation(org.eclipse.emf.ecore.EOperation)
         * @generated
         */
        public Object caseEOperation(org.eclipse.emf.ecore.EOperation object)
        {
            return new EOperationEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#caseEAttribute(org.eclipse.emf.ecore.EAttribute)
         * @generated
         */
        public Object caseEAttribute(org.eclipse.emf.ecore.EAttribute object)
        {
            return new EAttributeEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#caseEDataType(org.eclipse.emf.ecore.EDataType)
         * @generated
         */
        public Object caseEDataType(org.eclipse.emf.ecore.EDataType object)
        {
            return new EDataTypeEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#caseEEnum(org.eclipse.emf.ecore.EEnum)
         * @generated NOT
         */
        public Object caseEEnum(org.eclipse.emf.ecore.EEnum object)
        {
            String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
            if (!"".equals(feature))
            {
                int featureID = Integer.parseInt(feature);
                // REV Hack code to be compliant with previous Ecore models
                if (featureID == 7)
                {
                    featureID += 2;
                }
                EListEditPart list = new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
                list.setSynchronized(true);
                return list;
            }
            else
            {
                return new EEnumEditPart(node);
            }
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#caseEEnumLiteral(org.eclipse.emf.ecore.EEnumLiteral)
         * @generated
         */
        public Object caseEEnumLiteral(org.eclipse.emf.ecore.EEnumLiteral object)
        {
            return new EEnumLiteralEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#caseEAnnotation(org.eclipse.emf.ecore.EAnnotation)
         * @generated NOT
         */
        public Object caseEAnnotation(org.eclipse.emf.ecore.EAnnotation object)
        {
            if (IAnnotationConstants.DOCUMENTATION_SOURCE.equals(object.getSource()))
            {
                String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
                if (!"".equals(feature))
                {
                    int featureID = Integer.parseInt(feature);
                    EListEditPart list = new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
                    list.setSynchronized(true);
                    return list;
                }
                else
                {
                    return new EAnnotationDocumentationEditPart(node);
                }
            }
            else
            {
                String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
                if (!"".equals(feature))
                {
                    int featureID = Integer.parseInt(feature);
                    EListEditPart list = new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
                    list.setSynchronized(true);
                    return list;
                }
                else
                {
                    return new EAnnotationEditPart(node);
                }
            }
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#caseEStringToStringMapEntry(java.util.Map.Entry)
         * @generated
         */
        public Object caseEStringToStringMapEntry(java.util.Map.Entry<String, String> object)
        {
            return new EStringToStringMapEntryEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#caseEModelElement(org.eclipse.emf.ecore.EModelElement)
         * @generated
         */
        public Object caseEModelElement(org.eclipse.emf.ecore.EModelElement object)
        {
            return new EModelElementEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphNodeEditPart(node);
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class EdgeEcoreSwitch extends EcoreSwitch<Object>
    {
        /** The graphical edge */
        private GraphEdge edge;

        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param edge the graphical edge
         * @generated
         */
        public EdgeEcoreSwitch(GraphEdge edge)
        {
            this.edge = edge;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#caseEReference(org.eclipse.emf.ecore.EReference)
         * @generated
         */
        public Object caseEReference(org.eclipse.emf.ecore.EReference object)
        {
            return new EReferenceEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.emf.ecore.util.EcoreSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphEdgeEditPart(edge);
        }
    }

}
