/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.ediagram;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.palette.PaletteSeparator;
import org.eclipse.gef.requests.CreationFactory;
import org.topcased.modeler.editor.GraphElementCreationFactory;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.palette.ModelerConnectionCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerPaletteManager;

/**
 * Generated Palette Manager <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class EcorePaletteManager extends ModelerPaletteManager
{
    // declare all the palette categories of the diagram
    private PaletteDrawer objectsDrawer;

    private PaletteDrawer connectionsDrawer;

    private ICreationUtils creationUtils;

    /**
     * The Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param utils the creation utils for the tools of the palette
     * @generated
     */
    public EcorePaletteManager(ICreationUtils utils)
    {
        super();
        this.creationUtils = utils;
    }

    /**
     * Creates the main categories of the palette <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createCategories()
    {
        createObjectsDrawer();
        createConnectionsDrawer();
    }

    /**
     * Updates the main categories of the palette <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void updateCategories()
    {
        // deletion of the existing categories and creation of the updated
        // categories

        getRoot().remove(objectsDrawer);
        createObjectsDrawer();

        getRoot().remove(connectionsDrawer);
        createConnectionsDrawer();
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private void createObjectsDrawer()
    {
        objectsDrawer = new PaletteDrawer("Objects", null);
        List<PaletteEntry> entries = new ArrayList<PaletteEntry>();

        CreationFactory factory;

        factory = new GraphElementCreationFactory(creationUtils, EcorePackage.eINSTANCE.getEPackage(), "default");
        entries.add(new ModelerCreationToolEntry("EPackage", "EPackage", factory, EcoreImageRegistry.getImageDescriptor("EPACKAGE"), EcoreImageRegistry.getImageDescriptor("EPACKAGE_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, EcorePackage.eINSTANCE.getEClass(), "default");
        entries.add(new ModelerCreationToolEntry("EClass", "EClass", factory, EcoreImageRegistry.getImageDescriptor("ECLASS"), EcoreImageRegistry.getImageDescriptor("ECLASS_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, EcorePackage.eINSTANCE.getEDataType(), "default");
        entries.add(new ModelerCreationToolEntry("EDataType", "EDataType", factory, EcoreImageRegistry.getImageDescriptor("EDATATYPE"), EcoreImageRegistry.getImageDescriptor("EDATATYPE_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, EcorePackage.eINSTANCE.getEEnum(), "default");
        entries.add(new ModelerCreationToolEntry("EEnum", "EEnum", factory, EcoreImageRegistry.getImageDescriptor("EENUM"), EcoreImageRegistry.getImageDescriptor("EENUM_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, EcorePackage.eINSTANCE.getEAnnotation(), "default");
        entries.add(new ModelerCreationToolEntry("EAnnotation", "EAnnotation", factory, EcoreImageRegistry.getImageDescriptor("EANNOTATION"),
                EcoreImageRegistry.getImageDescriptor("EANNOTATION_LARGE")));

        entries.add(new PaletteSeparator());

        factory = new GraphElementCreationFactory(creationUtils, EcorePackage.eINSTANCE.getEOperation(), "default");
        entries.add(new ModelerCreationToolEntry("EOperation", "EOperation", factory, EcoreImageRegistry.getImageDescriptor("EOPERATION"), EcoreImageRegistry.getImageDescriptor("EOPERATION_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, EcorePackage.eINSTANCE.getEAttribute(), "default");
        entries.add(new ModelerCreationToolEntry("EAttribute", "EAttribute", factory, EcoreImageRegistry.getImageDescriptor("EATTRIBUTE"), EcoreImageRegistry.getImageDescriptor("EATTRIBUTE_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, EcorePackage.eINSTANCE.getEEnumLiteral(), "default");
        entries.add(new ModelerCreationToolEntry("EEnumLiteral", "EEnumLiteral", factory, EcoreImageRegistry.getImageDescriptor("EENUMLITERAL"),
                EcoreImageRegistry.getImageDescriptor("EENUMLITERAL_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, EcorePackage.eINSTANCE.getEStringToStringMapEntry(), "default");
        entries.add(new ModelerCreationToolEntry("Detail Entry", "Detail Entry", factory, EcoreImageRegistry.getImageDescriptor("ESTRINGTOSTRINGMAPENTRY"),
                EcoreImageRegistry.getImageDescriptor("ESTRINGTOSTRINGMAPENTRY_LARGE")));

        objectsDrawer.addAll(entries);
        getRoot().add(objectsDrawer);
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private void createConnectionsDrawer()
    {
        connectionsDrawer = new PaletteDrawer("Connections", null);
        List<PaletteEntry> entries = new ArrayList<PaletteEntry>();

        CreationFactory factory;

        factory = new GraphElementCreationFactory(creationUtils, EcorePackage.eINSTANCE.getEReference(), "default");
        entries.add(new ModelerConnectionCreationToolEntry("EReference", "EReference", factory, EcoreImageRegistry.getImageDescriptor("EREFERENCE"),
                EcoreImageRegistry.getImageDescriptor("EREFERENCE_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, EcoreSimpleObjectConstants.SIMPLE_OBJECT_INHERITANCE, "default", false);
        entries.add(new ModelerConnectionCreationToolEntry("Inheritance", "Inheritance", factory, EcoreImageRegistry.getImageDescriptor("INHERITANCE"),
                EcoreImageRegistry.getImageDescriptor("INHERITANCE_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, EcoreSimpleObjectConstants.SIMPLE_OBJECT_ANNOTATIONLINK, "default", false);
        entries.add(new ModelerConnectionCreationToolEntry("EAnnotation Link", "EAnnotation Link", factory, EcoreImageRegistry.getImageDescriptor("ANNOTATIONLINK"),
                EcoreImageRegistry.getImageDescriptor("ANNOTATIONLINK_LARGE")));

        connectionsDrawer.addAll(entries);
        getRoot().add(connectionsDrawer);
    }

}
