/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.ediagram.commands;

import java.util.Iterator;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.EditPart;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.ecore.ediagram.EcoreSimpleObjectConstants;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;

/**
 * EClass restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class EClassRestoreConnectionCommand extends EModelElementRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public EClassRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated
     */
    protected void initializeCommands()
    {
        super.initializeCommands();

        GraphElement node = getGraphElement();
        EObject nodeObject = Utils.getElement(node);

        if (nodeObject instanceof EClass)
        {
            Iterator<EObject> itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(node);
                    GraphElement node2 = (GraphElement) obj;
                    EObject nodeObject2 = Utils.getElement(node2);

                    if (nodeObject2 instanceof EClass)
                    {
                        if (autoRef)
                        {
                            createEReferenceFromEClassToEClass(node, node);
                        }
                        else
                        {
                            // if the node is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createEReferenceFromEClassToEClass(node, node2);
                            // if node is the target of the edge or if it is the
                            // source and that the SourceTargetCouple is
                            // reversible
                            createEReferenceFromEClassToEClass(node2, node);
                        }
                    }

                    if (nodeObject2 instanceof EClass && !autoRef)
                    {
                        // if the node is the source of the edge or if it is
                        // the target and that the SourceTargetCouple is
                        // reversible
                        createInheritanceFromEClassToEClass(node, node2);
                        // if node is the target of the edge or if it is the
                        // source and that the SourceTargetCouple is
                        // reversible
                        createInheritanceFromEClassToEClass(node2, node);
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated
     */
    private void createEReferenceFromEClassToEClass(GraphElement srcNode, GraphElement targetNode)
    {
        EClass sourceObject = (EClass) Utils.getElement(srcNode);
        EClass targetObject = (EClass) Utils.getElement(targetNode);

        for (EStructuralFeature obj : sourceObject.getEStructuralFeatures())
        {
            if (obj instanceof EReference)
            {
                EReference edgeObject = (EReference) obj;
                if (targetObject.equals(edgeObject.getEType()))
                {
                    // check if the relation does not exists yet
                    if (!isAlreadyPresent(getExistingEdges(srcNode, targetNode, EReference.class), edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            EReferenceEdgeCreationCommand cmd = new EReferenceEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                            cmd.setTarget(targetNode);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated
     */
    private void createInheritanceFromEClassToEClass(GraphElement srcNode, GraphElement targetNode)
    {
        EClass sourceObject = (EClass) Utils.getElement(srcNode);
        EClass targetObject = (EClass) Utils.getElement(targetNode);

        if (sourceObject.getESuperTypes().contains(targetObject))
        {
            // check if the relation does not exists yet
            if (getExistingEdges(srcNode, targetNode, EcoreSimpleObjectConstants.SIMPLE_OBJECT_INHERITANCE).size() == 0)
            {
                GraphEdge edge = Utils.createGraphEdge(EcoreSimpleObjectConstants.SIMPLE_OBJECT_INHERITANCE);
                InheritanceEdgeCreationCommand cmd = new InheritanceEdgeCreationCommand(null, edge, srcNode, false);
                cmd.setTarget(targetNode);
                add(cmd);
            }
        }
    }

}