/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.ediagram.commands;

import java.util.Iterator;

import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.commands.AbstractRestoreConnectionCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.ecore.ediagram.EcoreSimpleObjectConstants;
import org.topcased.modeler.utils.Utils;

/**
 * EModelElement restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class EModelElementRestoreConnectionCommand extends AbstractRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public EModelElementRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated
     */
    protected void initializeCommands()
    {

        GraphElement node = getGraphElement();
        EObject nodeObject = Utils.getElement(node);

        if (nodeObject instanceof EModelElement)
        {
            Iterator<EObject> itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(node);
                    GraphElement node2 = (GraphElement) obj;
                    EObject nodeObject2 = Utils.getElement(node2);
                    if (nodeObject2 instanceof EAnnotation && !autoRef)
                    {
                        // if node is the target of the edge or if it is the
                        // source and that the SourceTargetCouple is
                        // reversible
                        createAnnotationLinkFromEAnnotationToEModelElement(node2, node);
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated
     */
    private void createAnnotationLinkFromEAnnotationToEModelElement(GraphElement srcNode, GraphElement targetNode)
    {
        EAnnotation sourceObject = (EAnnotation) Utils.getElement(srcNode);
        EModelElement targetObject = (EModelElement) Utils.getElement(targetNode);

        if (targetObject.getEAnnotations().contains(sourceObject))
        {
            // check if the relation does not exists yet
            if (getExistingEdges(srcNode, targetNode, EcoreSimpleObjectConstants.SIMPLE_OBJECT_ANNOTATIONLINK).size() == 0)
            {
                GraphEdge edge = Utils.createGraphEdge(EcoreSimpleObjectConstants.SIMPLE_OBJECT_ANNOTATIONLINK);
                AnnotationLinkEdgeCreationCommand cmd = new AnnotationLinkEdgeCreationCommand(null, edge, srcNode, false);
                cmd.setTarget(targetNode);
                add(cmd);
            }
        }
    }

}