/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.ecore.ediagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.impl.EStringToStringMapEntryImpl;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.requests.DirectEditRequest;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Composite;
import org.topcased.draw2d.figures.CommentFigure;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.modeler.ModelerColorConstants;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.ecore.ediagram.EcoreEditPolicyConstants;
import org.topcased.modeler.ecore.ediagram.policies.AnnotationLinkEdgeCreationEditPolicy;
import org.topcased.modeler.ecore.ediagram.policies.EAnnotationDocumentationDirectEditPolicy;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.LabelCellEditorLocator;
import org.topcased.modeler.edit.ModelerLabelDirectEditManager;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.utils.Utils;

/**
 * The EAnnotation object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques Lescot </a>
 * @generated
 */
public class EAnnotationDocumentationEditPart extends EMFGraphNodeEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj
     * @generated
     */
    public EAnnotationDocumentationEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * @see org.eclipse.gef.EditPart#activate()
     */
    public void activate()
    {
        super.activate();
        EAnnotation elem = (EAnnotation) getEObject();
        EStringToStringMapEntryImpl entry = (EStringToStringMapEntryImpl) elem.getDetails().get(0);
        if (entry != null)
        {
            entry.eAdapters().add(getModelListener());
        }
    }

    /**
     * @see org.eclipse.gef.EditPart#deactivate()
     */
    public void deactivate()
    {
        EAnnotation elem = (EAnnotation) getEObject();
        EStringToStringMapEntryImpl entry = (EStringToStringMapEntryImpl) elem.getDetails().get(0);
        if (entry != null)
        {
            entry.eAdapters().remove(getModelListener());
        }
        super.deactivate();
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, null);

        // Provide the Policy to associate the EAnnotation to an Ecore object
        installEditPolicy(EcoreEditPolicyConstants.ANNOTATIONLINK_EDITPOLICY, new AnnotationLinkEdgeCreationEditPolicy());

        // resize only in horizontal way
        ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(PositionConstants.EAST_WEST);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

        // add a custom directEditPolicy
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new EAnnotationDocumentationDirectEditPolicy());
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        CommentFigure fig = (CommentFigure) getFigure();

        if (((EAnnotation) getEObject()).getDetails().get("documentation") != null)
        {
            fig.setText(((EAnnotation) getEObject()).getDetails().get("documentation"));
        }
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#refreshTextAndFont()
     */
    protected void refreshTextAndFont()
    {
        if (getLabel() != null)
        {
            Font font = Utils.getFont(getGraphNode());
            if (font == null)
            {
                font = getDefaultFont();
            }
            getLabel().setFont(font);
        }
    }

    /**
     * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
     */
    public void performRequest(Request request)
    {
        if (request.getType() == RequestConstants.REQ_DIRECT_EDIT)
        {
            if (!(request instanceof DirectEditRequest))
            {
                return;
            }
            performDirectEdit();
        }
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#performDirectEdit()
     */
    protected void performDirectEdit()
    {
        if (getDirectEditManager() == null)
        {
            ILabel label = getLabel();
            setDirectEditManager(new ModelerLabelDirectEditManager(this, TextCellEditor.class, new LabelCellEditorLocator(getFigure()), label)
            {
                /**
                 * @see org.eclipse.gef.tools.DirectEditManager#createCellEditorOn(org.eclipse.swt.widgets.Composite)
                 */
                protected CellEditor createCellEditorOn(Composite composite)
                {
                    return new TextCellEditor(composite, SWT.MULTI | SWT.WRAP);
                }
            });

        }
        getDirectEditManager().show();
    }

    /**
     * Return label contained in the header
     * 
     * @return Label
     */
    public ILabel getLabel()
    {
        return ((CommentFigure) getFigure()).getLabel();
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getHeaderFontData()
     */
    public FontData[] getHeaderFontData()
    {
        if (getLabel() != null)
        {
            return getLabel().getFont().getFontData();
        }
        return null;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {
        return new CommentFigure();
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getDefaultBackgroundColor()
     */
    protected Color getDefaultBackgroundColor()
    {
        return ModelerColorConstants.yellow;
    }

}
