/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.ediagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.swt.graphics.Image;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.ecore.ediagram.EcoreImageRegistry;
import org.topcased.modeler.ecore.ediagram.commands.EAttributeRestoreConnectionCommand;
import org.topcased.modeler.ecore.ediagram.figures.EAttributeFigure;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.NonResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.utils.Utils;

/**
 * The EAttribute object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques Lescot </a>
 * @generated
 */
public class EAttributeEditPart extends EModelElementEditPart
{

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public EAttributeEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Get the EAttribute model object
     * 
     * @return the model object
     */
    protected EAttribute getModelEAttribute()
    {
        return (EAttribute) Utils.getElement(getGraphNode());
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        // installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, null);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new NonResizableEditPolicy());

        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new EAttributeRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(ModelerEditPolicyConstants.CHANGE_BACKGROUND_COLOR_EDITPOLICY, null);
        installEditPolicy(ModelerEditPolicyConstants.CHANGE_FOREGROUND_COLOR_EDITPOLICY, null);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated NOT
     */
    protected IFigure createFigure()
    {
        return new EAttributeFigure();
    }

    /**
     * Update the EditableLabel with the correct icon and name
     * 
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        ComposedLabel lbl = (ComposedLabel) ((ILabelFigure) getFigure()).getLabel();

        lbl.setPrefixIcon(getPrefixIcon());
        lbl.setMain(getModelEAttribute().getName() != null ? getModelEAttribute().getName() : "null");
        lbl.setSuffix(getModelEAttribute().getEType() != null && getModelEAttribute().getEType().getName() != null ? " : " + getModelEAttribute().getEType().getName() : " : null");
    }

    /**
     * Get the Image associated with the bounds of the EAttribute
     * 
     * @return the Image associated
     */
    private Image getPrefixIcon()
    {
        String imageName = "EATTRIBUTE_";
        int minOccurs = getModelEAttribute().getLowerBound();
        int maxOccurs = getModelEAttribute().getUpperBound();
        if (minOccurs >= 0 && (minOccurs <= maxOccurs || maxOccurs == -1))
        {
            switch (minOccurs)
            {
                case 0:
                    imageName += "0";
                    break;
                case 1:
                    imageName += "1";
                    break;
                default:
                    imageName += "N";
                    break;
            }

            if (minOccurs != maxOccurs)
            {
                switch (maxOccurs)
                {
                    case -1:
                        imageName += "_TO_UNBOUNDED";
                        break;
                    case 0:
                        return null;
                    case 1:
                        imageName += "_TO_1";
                        break;
                    default:
                        imageName += minOccurs <= 1 ? "_TO_N" : "_TO_M";
                        break;
                }
            }
        }
        else
        {
            imageName += "N_TO_M";
        }

        return EcoreImageRegistry.getImage(imageName);
    }

}
