/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.ediagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.ecore.ediagram.EcoreEditPolicyConstants;
import org.topcased.modeler.ecore.ediagram.EcoreImageRegistry;
import org.topcased.modeler.ecore.ediagram.commands.EClassRestoreConnectionCommand;
import org.topcased.modeler.ecore.ediagram.figures.EClassFigure;
import org.topcased.modeler.ecore.ediagram.policies.EClassLayoutEditPolicy;
import org.topcased.modeler.ecore.ediagram.policies.EReferenceEdgeCreationEditPolicy;
import org.topcased.modeler.ecore.ediagram.policies.InheritanceEdgeCreationEditPolicy;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.utils.Utils;

/**
 * The EClass object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class EClassEditPart extends EModelElementEditPart
{
    private static boolean italicFontOK = false;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public EClassEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Get the EClass model object
     * 
     * @return the model object
     */
    protected EClass getModelEClass()
    {
        return (EClass) getEObject();
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(EcoreEditPolicyConstants.EREFERENCE_EDITPOLICY, new EReferenceEdgeCreationEditPolicy());

        installEditPolicy(EcoreEditPolicyConstants.INHERITANCE_EDITPOLICY, new InheritanceEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new EClassRestoreConnectionCommand(getHost());
            }
        });

        ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(PositionConstants.EAST_WEST);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new EClassLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * Update the ComposedLabel with the correct Suffix label (if necessary) and the right icon and font for the Main
     * EditableLabel
     * 
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        EClassFigure fig = (EClassFigure) getFigure();
        ComposedLabel lbl = (ComposedLabel) fig.getLabel();
        if (getModelEClass().getName() != null)
        {
            lbl.setMain(getModelEClass().getName());
        }

        if (getModelEClass().isInterface())
        {
            ((EditableLabel) lbl.getMain()).setIcon(EcoreImageRegistry.getImage("INTERFACE"));
        }
        else
        {
            ((EditableLabel) lbl.getMain()).setIcon(EcoreImageRegistry.getImage("ECLASS"));
        }

        // check if the class is not included in the EPackage of the current
        // diagram
        if (getModelEClass().getEPackage() != null && getModelEClass().getEPackage() != Utils.getElement(((GraphElement) getParent().getModel()).getSemanticModel().getGraphElement()))
        {
            if (getModelEClass().getEPackage().getName() != null)
            {
                lbl.setSuffix("<<from " + getModelEClass().getEPackage().getName() + ">>");
            }
        }
        else
        {
            lbl.setSuffix("");
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new EClassFigure();
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getDefaultFont()
     */
    protected Font getDefaultFont()
    {
        if (getModelEClass().isAbstract())
        {
            if (!italicFontOK)
            {
                FontData[] fData = super.getDefaultFont().getFontData();
                fData[0].setStyle(SWT.ITALIC);
                JFaceResources.getFontRegistry().put("EClassItalicFont", fData);
                italicFontOK = true;
            }
            return JFaceResources.getFontRegistry().get("EClassItalicFont");
        }
        else
        {
            return super.getDefaultFont();
        }
    }

}