/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.ediagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.ecore.ediagram.commands.EOperationRestoreConnectionCommand;
import org.topcased.modeler.ecore.ediagram.figures.EOperationFigure;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.NonResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.utils.Utils;

/**
 * The EOperation object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques Lescot </a>
 * @generated
 */
public class EOperationEditPart extends EModelElementEditPart
{

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public EOperationEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Get the EOperation model object
     * 
     * @return the model object
     */
    protected EOperation getModelEOperation()
    {
        return (EOperation) Utils.getElement(getGraphNode());
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
     */
    public void activate()
    {
        super.activate();
        EObject model = getEObject();
        if (model instanceof EOperation)
        {
            listenParameters((EOperation) getEObject());
            listenExceptions((EOperation) getEObject());
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
     */
    public void deactivate()
    {
        EObject model = getEObject();
        if (model instanceof EOperation)
        {
            unlistenExceptions((EOperation) getEObject());
            unlistenParameters((EOperation) getEObject());
        }
        super.deactivate();
    }

    /**
     * Listen to all the parameters of the operation
     * 
     * @param operation the model object
     */
    private void listenParameters(EOperation operation)
    {
        for (EParameter param : operation.getEParameters())
        {
            // Only listen to object that are not yet listened
            if (!param.eAdapters().contains(getModelListener()))
            {
                param.eAdapters().add(getModelListener());
            }
            listenType(param);
        }
    }

    /**
     * Stop listening to all the parameters of the operation
     * 
     * @param operation the model object
     */
    private void unlistenParameters(EOperation operation)
    {
        for (EParameter param : operation.getEParameters())
        {
            unlistenType(param);
            param.eAdapters().remove(getModelListener());
        }
    }

    /**
     * Listen to the type of the parameter
     * 
     * @param param the model object
     */
    private void listenType(EParameter param)
    {
        // Only listen to object that are not yet listened
        if (param.getEType() != null && !param.getEType().eAdapters().contains(getModelListener()))
        {
            param.getEType().eAdapters().add(getModelListener());
        }
    }

    /**
     * Stop listening to the type of the parameter
     * 
     * @param param the model object
     */
    private void unlistenType(EParameter param)
    {
        if (param.getEType() != null)
        {
            param.getEType().eAdapters().remove(getModelListener());
        }
    }

    /**
     * Listen to all the exceptions of the operation
     * 
     * @param operation the model object
     */
    private void listenExceptions(EOperation operation)
    {
        for (EClassifier ex : operation.getEExceptions())
        {
            // Only listen to object that are not yet listened
            if (!ex.eAdapters().contains(getModelListener()))
            {
                ex.eAdapters().add(getModelListener());
            }
        }
    }

    /**
     * Stop listening to all the excpetions of the operation
     * 
     * @param operation the model object
     */
    private void unlistenExceptions(EOperation operation)
    {
        for (EClassifier ex : operation.getEExceptions())
        {
            ex.eAdapters().remove(getModelListener());
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {

        Object notifier = msg.getNotifier();
        Object newObject = msg.getNewValue();
        Object oldObject = msg.getOldValue();

        if (notifier instanceof EOperation)
        {
            if (notifier == getEObject())
            {
                switch (msg.getFeatureID(EOperation.class))
                {
                    case EcorePackage.EOPERATION__EEXCEPTIONS:
                    case EcorePackage.EOPERATION__EPARAMETERS:
                        updateModelListening(oldObject, newObject);
                        break;
                    default:
                        break;
                }
                super.handleModelChanged(msg);
            }
            else
            {
                refreshHeaderLabel();
            }
        }
        else if (notifier instanceof EParameter)
        {
            switch (msg.getFeatureID(EParameter.class))
            {
                case EcorePackage.EPARAMETER__ETYPE:
                    updateModelListening(oldObject, newObject);
                    break;
                default:
                    break;
            }
            refreshHeaderLabel();
        }
        else if (notifier instanceof EClassifier)
        {
            switch (msg.getFeatureID(EClassifier.class))
            {
                case EcorePackage.ECLASSIFIER__NAME:
                    refreshHeaderLabel();
                    break;
                default:
                    break;
            }
        }
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, null);
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new EOperationRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new NonResizableEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.CHANGE_BACKGROUND_COLOR_EDITPOLICY, null);
        installEditPolicy(ModelerEditPolicyConstants.CHANGE_FOREGROUND_COLOR_EDITPOLICY, null);
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        ComposedLabel lbl = (ComposedLabel) ((ILabelFigure) getFigure()).getLabel();

        // lbl.setIcon(EcoreImageRegistry.getImage("EOPERATION"));
        lbl.setMain(getModelEOperation().getName() != null && !"".equals(getModelEOperation().getName()) ? getModelEOperation().getName() : "null");
        lbl.setSuffix(getSuffixText());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated NOT
     */
    protected IFigure createFigure()
    {
        return new EOperationFigure();
    }

    private String getSuffixText()
    {
        String text = "";
        text += "(";
        text += getParametersText();
        text += ")";
        if (getModelEOperation().getEType() != null)
        {
            text += " : " + getModelEOperation().getEType().getName();
        }
        text += getExceptionsText();
        return text;
    }

    private String getParametersText()
    {
        StringBuffer text = new StringBuffer();
        boolean first = true;
        for (EParameter param : getModelEOperation().getEParameters())
        {
            if (first)
            {
                first = false;
            }
            else
            {
                text.append(",");
            }

            EClassifier eType = param.getEType();
            if (eType == null)
            {
                text.append("null");
            }
            else
            {
                text.append(param.getEType().getName());
            }
        }

        return text.toString();
    }

    private String getExceptionsText()
    {
        StringBuffer text = new StringBuffer();
        boolean first = true;
        for (EClassifier ex : getModelEOperation().getEExceptions())
        {
            if (first)
            {
                text.append(" throws ");
                first = false;
            }
            else
            {
                text.append(", ");
            }

            text.append(ex.getName());
        }

        return text.toString();
    }
}
