/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.ediagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.ecore.ediagram.commands.EPackageRestoreConnectionCommand;
import org.topcased.modeler.ecore.ediagram.figures.EPackageFigure;
import org.topcased.modeler.ecore.ediagram.policies.EPackageLayoutEditPolicy;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.utils.Utils;

/**
 * The EPackage object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class EPackageEditPart extends EModelElementEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public EPackageEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Get the EPackage model object
     * 
     * @return the model object
     */
    protected EPackage getModelEPackage()
    {
        return (EPackage) Utils.getElement(getGraphNode());
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new EPackageRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new EPackageLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new EPackageFigure();
    }

    /**
     * Update the ComposedLabel with the correct Suffix label (if necessary) and
     * the right icon and font for the Main EditableLabel
     * 
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        super.refreshHeaderLabel();

        EPackageFigure fig = (EPackageFigure) getFigure();
        ComposedLabel lbl = (ComposedLabel) fig.getLabel();

        // check if the package is not included in the EPackage of the current
        // diagram
        if (getModelEPackage().getESuperPackage() != null
                && getModelEPackage().getESuperPackage() != Utils.getElement(((GraphElement) getParent().getModel()).getSemanticModel().getGraphElement()))
        {
            if (getModelEPackage().getESuperPackage().getName() != null)
            {
                lbl.setSuffix("<<from " + getModelEPackage().getESuperPackage().getName() + ">>");
            }
        }
        else
        {
            lbl.setSuffix("");
        }
    }
}
