/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.ediagram.edit;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.draw2d.ConnectionRouter;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.PolylineDecoration;
import org.eclipse.draw2d.RotatableDecoration;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.requests.DirectEditRequest;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.gmf.runtime.draw2d.ui.internal.figures.ConnectionLayerEx;
import org.eclipse.jface.viewers.TextCellEditor;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.Label;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.ecore.EcorePlugin;
import org.topcased.modeler.ecore.ediagram.EcoreEdgeObjectConstants;
import org.topcased.modeler.ecore.ediagram.EcoreEditPolicyConstants;
import org.topcased.modeler.ecore.ediagram.figures.EReferenceFigure;
import org.topcased.modeler.ecore.ediagram.policies.AnnotationLinkEdgeCreationEditPolicy;
import org.topcased.modeler.ecore.ediagram.policies.EReferenceEdgeObjectUVEditPolicy;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.LabelCellEditorLocator;
import org.topcased.modeler.edit.ModelerLabelDirectEditManager;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.exceptions.BoundsFormatException;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.utils.Utils;

/**
 * EReference controller <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class EReferenceEditPart extends EMFGraphEdgeEditPart
{
    private RotatableDecoration srcDecor;
    
    private RotatableDecoration targetDecor;

    private EReference opposite;

    /** The DirectEditManager */
    private DirectEditManager manager;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param model the graph object
     * @generated
     */
    public EReferenceEditPart(GraphEdge model)
    {
        super(model);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(EcoreEditPolicyConstants.ANNOTATIONLINK_EDITPOLICY, new AnnotationLinkEdgeCreationEditPolicy());
        
        installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_UV_EDITPOLICY, new EReferenceEdgeObjectUVEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
        installEditPolicy(ModelerEditPolicyConstants.CHANGE_BACKGROUND_COLOR_EDITPOLICY, null);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Figure
     * @generated
     */
    protected IFigure createFigure()
    {
        EReferenceFigure connection = new EReferenceFigure();

        createTargetDecoration(connection);

        return connection;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param connection the PolylineConnection
     * @generated
     */
    private void createTargetDecoration(PolylineConnection connection)
    {

        PolylineDecoration decoration = new PolylineDecoration();
        decoration.setScale(10, 5);
        connection.setTargetDecoration(decoration);

    }

    /**
     * Get the EReference model object
     * 
     * @return the model object
     */
    protected EReference getEReference()
    {
        return (EReference) Utils.getElement(getGraphEdge());
    }

    /**
     * The decoration of the containment figure
     * 
     * @return the RotatableDecoration
     */
    private RotatableDecoration createSourceDecoration()
    {
        PolygonDecoration decoration = new PolygonDecoration();
        PointList decorationPointList = new PointList();
        decorationPointList.addPoint(0, 0);
        decorationPointList.addPoint(-1, 1);
        decorationPointList.addPoint(-2, 0);
        decorationPointList.addPoint(-1, -1);
        decoration.setTemplate(decorationPointList);
        decoration.setScale(10, 5);
        return decoration;
    }

    /**
     * The decoration of the reference (if one-way reference)
     * 
     * @return the RotatableDecoration
     */
    private RotatableDecoration createTargetDecoration()
    {
        PolylineDecoration decoration = new PolylineDecoration();
        decoration.setScale(10, 5);
        return decoration;
    }

    /**
     * Handle selection
     * 
     * @param value The value of the selection
     */
    public void setSelected(int value)
    {
        super.setSelected(value);
        ((EditableLabel) getEREferenceFigure().getNameEdgeObjectFigure()).setSelected(value != 0);
    }

    /**
     * Get the PolylineConnection associated with the edge figure
     * 
     * @return the PolylineConnection
     */
    private PolylineConnection getConnection()
    {
        return (PolylineConnection) getFigure();
    }

    /**
     * <!-- begin-user-doc --> The decoration of the reference (if one-way reference) <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#refreshEdgeObjects()
     * @generated NOT
     */
    protected void refreshEdgeObjects()
    {
        super.refreshEdgeObjects();

        updateSourceDecoration();
        updateTargetDecoration();
        updateNameLabel();
        updateMultiplicityLabel();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the name Label
     * 
     * @generated
     */
    private void updateNameLabel()
    {
        ((Label) ((EReferenceFigure) getFigure()).getNameEdgeObjectFigure()).setText(((EReference) getEObject()).getName());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the multiplicity Label
     * 
     * @generated NOT
     */
    private void updateMultiplicityLabel()
    {
        Label targetCount = (Label) getEREferenceFigure().getMultiplicityEdgeObjectFigure();
        try
        {
            targetCount.setText(createCountString(getEReference()));
        }
        catch (BoundsFormatException e)
        {
            targetCount.setText("error");
            IStatus status = new Status(IStatus.ERROR, EcorePlugin.getDefault().getBundle().getSymbolicName(), IStatus.ERROR, e.getMessage(), null);
            ResourcesPlugin.getPlugin().getLog().log(status);
        }
    }

    // Ajout des fonctions pour le DirectEdit

    /**
     * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
     */
    public void performRequest(Request request)
    {
        if (request.getType() == RequestConstants.REQ_DIRECT_EDIT)
        {
            if (request instanceof DirectEditRequest && !(directEditHitTest(((DirectEditRequest) request).getLocation().getCopy())))
            {
                return;
            }
            performDirectEdit();
        }
    }

    private boolean directEditHitTest(Point requestLoc)
    {
        IFigure targetName = getEREferenceFigure().getNameEdgeObjectFigure();
        if (targetName != null)
        {
            targetName.translateToRelative(requestLoc);
            return targetName.containsPoint(requestLoc);
        }
        return false;
    }

    private void performDirectEdit()
    {
        if (manager == null)
        {
            IFigure targetName = getEREferenceFigure().getNameEdgeObjectFigure();
            manager = new ModelerLabelDirectEditManager(this, TextCellEditor.class, new LabelCellEditorLocator(targetName), targetName);
        }
        manager.show();
    }

    /**
     * Calculate the String corresponding to the bounds
     * 
     * @param ref the EReference object
     * @return the String to display at the endPoint of the connection
     */
    private String createCountString(EReference ref)
    {
        int lower = ref.getLowerBound();
        int upper = ref.getUpperBound();

        if (lower < 0)
        {
            throw new BoundsFormatException("LowerBound must be [0..n]");
        }
        if (upper < -1)
        {
            throw new BoundsFormatException("UpperBound must be [-1..n]");
        }
        if (upper != -1 && lower > upper)
        {
            throw new BoundsFormatException("UpperBound must be higher than LowerBound");
        }

        if (lower == upper)
        {
            return "" + lower;
        }
        return lower + ".." + (upper == -1 ? "*" : "" + upper);
    }

    /**
     * 
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {
        switch (msg.getFeatureID(EReference.class))
        {
            case EcorePackage.EREFERENCE__NAME:
                updateNameLabel();
                break;
            case EcorePackage.EREFERENCE__LOWER_BOUND:
            case EcorePackage.EREFERENCE__UPPER_BOUND:
            case EcorePackage.EREFERENCE__MANY:
                updateMultiplicityLabel();
                break;
            case EcorePackage.EREFERENCE__EOPPOSITE:
                updateEOpposite(getEReference().getEOpposite());
                break;
            case EcorePackage.EREFERENCE__CONTAINER:
            case EcorePackage.EREFERENCE__CONTAINMENT:
                refreshVisuals();
                break;
            default:
                break;
        }
        super.handleModelChanged(msg);
    }

    private void updateEOpposite(EReference opp)
    {
        if (opposite == opp)
        {
            return;
        }
        if (opposite != null)
        {
            opposite.eAdapters().remove(getModelListener());
        }
        opposite = opp;
        if (opposite != null)
        {
            opposite.eAdapters().add(getModelListener());
        }
    }

    private void updateSourceDecoration()
    {
        if (getEReference().isContainment())
        {
            if (srcDecor == null)
            {
                srcDecor = createSourceDecoration();
                getConnection().setSourceDecoration(srcDecor);
            }
        }
        else if (srcDecor != null)
        {
            srcDecor = null;
            getConnection().setSourceDecoration(srcDecor);
        }
    }

    private void updateTargetDecoration()
    {
        if (getEReference().getEOpposite() != null)
        {
            targetDecor = null;
            getConnection().setTargetDecoration(targetDecor);
        }
        else if (targetDecor == null)
        {
            targetDecor = createTargetDecoration();
            getConnection().setTargetDecoration(targetDecor);
        }
    }

    /**
     * Return the figure associated with the controller
     * 
     * @return IFigure
     */
    public EReferenceFigure getEREferenceFigure()
    {
        return (EReferenceFigure) getFigure();
    }

    /**
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#getEdgeObjectFigure(org.topcased.modeler.di.model.EdgeObject)
     */
    public IEdgeObjectFigure getEdgeObjectFigure(EdgeObject edgeObject)
    {
        if (EcoreEdgeObjectConstants.MULTIPLICITY_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return getEREferenceFigure().getMultiplicityEdgeObjectFigure();
        }
        else if (EcoreEdgeObjectConstants.NAME_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return getEREferenceFigure().getNameEdgeObjectFigure();
        }

        return null;
    }

    /**
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#getDefaultRouter(org.eclipse.gmf.runtime.draw2d.ui.internal.figures.ConnectionLayerEx)
     */
    protected ConnectionRouter getDefaultRouter(ConnectionLayerEx cLayer)
    {
        // Use the rectilinear router for an autoReference
        if (getSource() == getTarget())
        {
            return cLayer.getRectilinearRouter();
        }

        return super.getDefaultRouter(cLayer);
    }
}
