/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.ediagram.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.topcased.draw2d.figures.EditableLabel;

/**
 * <!-- begin-user-doc -->The figure to display a EAnnotation. A Figure with a
 * bent corner and an embedded TextFlow within a FlowPage that contains text.
 * <!-- end-user-doc -->
 * 
 * @generated
 */
public class EAnnotationFigure extends org.topcased.draw2d.figures.ClassFigure
{
    /**
     * The default amount of pixels subtracted from the figure's height and
     * width to determine the size of the corner.
     */
    private static final int DEFAULT_CORNER_SIZE = 10;

    /**
     * Constructor <!-- begin-user-doc --> Creates a new EAnnotationFigure with
     * a MarginBorder that is the given size and a FlowPage containing a
     * TextFlow with the style WORD_WRAP_SOFT. <!-- end-user-doc -->
     * 
     * @generated
     */
    public EAnnotationFigure()
    {
        super();
    }

    /**
     * Create a custom contentPane Figure
     * 
     * @see org.topcased.draw2d.figures.ClassFigure#drawFigure()
     */
    protected void drawFigure()
    {
        ToolbarLayout layout = new ToolbarLayout();
        layout.setMinorAlignment(ToolbarLayout.ALIGN_CENTER);
        layout.setStretchMinorAxis(true);
        setLayoutManager(layout);
        setOpaque(true);

        setBorder(new MarginBorder(3, 3, 3, 12));
        setLabel(new EditableLabel());
        ((EditableLabel) getLabel()).setLabelAlignment(PositionConstants.LEFT);
        add(getLabel());

        setContentPane(new Figure());
        getContentPane().setLayoutManager(new ToolbarLayout());
        add(getContentPane());
    }

    /**
     * Draw a specific Figure
     * 
     * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
     */
    protected void paintFigure(Graphics graphics)
    {
        Rectangle rect = getBounds().getCopy();

        graphics.translate(getLocation());

        // fill the note
        PointList outline = new PointList();

        outline.addPoint(0, 0);
        outline.addPoint(rect.width - DEFAULT_CORNER_SIZE, 0);
        outline.addPoint(rect.width - 1, DEFAULT_CORNER_SIZE);
        outline.addPoint(rect.width - 1, rect.height - 1);
        outline.addPoint(0, rect.height - 1);

        graphics.fillPolygon(outline);

        // draw the inner outline
        PointList innerLine = new PointList();

        innerLine.addPoint(rect.width - DEFAULT_CORNER_SIZE - 1, 0);
        innerLine.addPoint(rect.width - DEFAULT_CORNER_SIZE - 1, DEFAULT_CORNER_SIZE);
        innerLine.addPoint(rect.width - 1, DEFAULT_CORNER_SIZE);
        innerLine.addPoint(rect.width - DEFAULT_CORNER_SIZE - 1, 0);
        innerLine.addPoint(0, 0);
        innerLine.addPoint(0, rect.height - 1);
        innerLine.addPoint(rect.width - 1, rect.height - 1);
        innerLine.addPoint(rect.width - 1, DEFAULT_CORNER_SIZE);

        graphics.drawPolygon(innerLine);

        graphics.translate(getLocation().getNegated());
    }

}
