/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.ediagram.policies;

import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.ReconnectRequest;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.commands.ReconnectNodeToNodeCommand;
import org.topcased.modeler.commands.ReconnectTargetToSourceCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.ecore.ediagram.EcoreSimpleObjectConstants;
import org.topcased.modeler.ecore.ediagram.commands.AnnotationLinkEdgeCreationCommand;
import org.topcased.modeler.ecore.ediagram.edit.AnnotationLinkEditPart;
import org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy;
import org.topcased.modeler.utils.SourceTargetData;
import org.topcased.modeler.utils.Utils;

/**
 * AnnotationLink edge creation <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class AnnotationLinkEdgeCreationEditPolicy extends AbstractEdgeCreationEditPolicy
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#createCommand(org.eclipse.gef.EditDomain,
     *      org.topcased.modeler.di.model.GraphEdge,
     *      org.topcased.modeler.di.model.GraphElement)
     * @generated
     */
    protected CreateTypedEdgeCommand createCommand(EditDomain domain, GraphEdge edge, GraphElement source)
    {
        return new AnnotationLinkEdgeCreationCommand(domain, edge, source);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkEdge(org.topcased.modeler.di.model.GraphEdge)
     * @generated
     */
    protected boolean checkEdge(GraphEdge edge)
    {
        if (edge.getSemanticModel() instanceof SimpleSemanticModelElement)
        {
            return EcoreSimpleObjectConstants.SIMPLE_OBJECT_ANNOTATIONLINK.equals(((SimpleSemanticModelElement) edge.getSemanticModel()).getTypeInfo());
        }
        return false;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkSource(org.topcased.modeler.di.model.GraphElement)
     * @generated NOT
     */
    protected boolean checkSource(GraphElement source)
    {
        EObject object = Utils.getElement(source);
        // check if the EAnnotation is not already associated to
        // a model object
        if (object instanceof org.eclipse.emf.ecore.EAnnotation && !isEAnnotationLinked(source))
        {
            return true;
        }
        return false;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkTargetForSource(org.topcased.modeler.di.model.GraphElement,
     *      org.topcased.modeler.di.model.GraphElement)
     * @generated
     */
    protected boolean checkTargetForSource(GraphElement source, GraphElement target)
    {
        EObject sourceObject = Utils.getElement(source);
        EObject targetObject = Utils.getElement(target);

        if (sourceObject instanceof org.eclipse.emf.ecore.EAnnotation
                && targetObject instanceof org.eclipse.emf.ecore.EModelElement)
        {
            if (!sourceObject.equals(targetObject))
            {
                return true;
            }
        }
        return false;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkCommand(org.eclipse.gef.commands.Command)
     * @generated
     */
    protected boolean checkCommand(Command command)
    {
        return command instanceof AnnotationLinkEdgeCreationCommand;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#getSourceTargetData(org.topcased.modeler.di.model.GraphElement,
     *      org.topcased.modeler.di.model.GraphElement)
     * @generated
     */
    protected SourceTargetData getSourceTargetData(GraphElement source, GraphElement target)
    {
        EObject sourceObject = Utils.getElement(source);
        EObject targetObject = Utils.getElement(target);

        if (sourceObject instanceof org.eclipse.emf.ecore.EAnnotation
                && targetObject instanceof org.eclipse.emf.ecore.EModelElement)
        {
            return new SourceTargetData(false, false, SourceTargetData.NONE, null, null, null, null, null, null, null,
                    "eAnnotations");
        }
        return null;
    }

    /**
     * Check if the EAnnotation is already associated with a model object
     * 
     * @param src the EAnnotation GraphNode
     * @return true if the EAnnotation is already associated with a model object
     */
    private boolean isEAnnotationLinked(GraphElement src)
    {
        // get the EAnnotation model object
        EAnnotation eAnnotation = (EAnnotation) Utils.getElement(src);

        // get the model Object that is currently associated with the
        // EAnnotation
        EModelElement modelObjectContainer = eAnnotation.getEModelElement();

        // if the model Object is the Package associated with the diagram, that
        // means that the EAnnotation is not currently associated with a model
        // object
        return Utils.getElement(src.getContainer()) != modelObjectContainer;
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
     * @generated
     */
    protected Command getReconnectSourceCommand(ReconnectRequest request) 
    {
    	ConnectionEditPart connection = request.getConnectionEditPart();
        if (connection instanceof AnnotationLinkEditPart)
        {
        	final AnnotationLinkEditPart edit = (AnnotationLinkEditPart) connection;
        	
        	final GraphElement newSource = (GraphElement) getHost().getModel();
            
            if (checkSource(newSource))
            {
                final GraphElement graphElt = (GraphElement) edit.getTarget().getModel();
                ReconnectNodeToNodeCommand command = new ReconnectTargetToSourceCommand(edit, "eAnnotations", ReconnectNodeToNodeCommand.SOURCE)
                {
                    protected EObject getObjectToUpdate()
                    {
                        return Utils.getElement(graphElt);
                    }
                };
                command.setNewElement(newSource);
                return command;
            }
        }

        return null;
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
     * 
     * @generated
     */
    protected Command getReconnectTargetCommand(ReconnectRequest request)
    {
    	ConnectionEditPart connection = request.getConnectionEditPart();
        if (connection instanceof AnnotationLinkEditPart)
        {
        	final AnnotationLinkEditPart edit = (AnnotationLinkEditPart) connection;
        	
        	final GraphElement newTarget = (GraphElement) getHost().getModel();
            final GraphElement source = (GraphElement) edit.getSource().getModel();
            
            if (checkTargetForSource(source, newTarget))
            {
                final GraphElement graphElt = source;
                ReconnectNodeToNodeCommand command = new ReconnectTargetToSourceCommand(edit, "eAnnotations", ReconnectNodeToNodeCommand.TARGET)
                {
                    protected EObject getObjectToUpdate()
                    {
                        return Utils.getElement(graphElt);
                    }
                };
                command.setNewElement(newTarget);
                return command;
            }
        }

        return null;
    }
}