/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.ecore.ediagram.policies;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.EStringToStringMapEntryImpl;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.editpolicies.DirectEditPolicy;
import org.eclipse.gef.requests.DirectEditRequest;
import org.eclipse.jface.viewers.CellEditor;
import org.topcased.modeler.commands.ChangeLabelTextCommand;
import org.topcased.modeler.di.model.EMFSemanticModelBridge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.ecore.EcorePlugin;
import org.topcased.modeler.edit.GraphNodeEditPart;

/**
 * Manage the DirectEditPolicy for a Label in the Editing Window
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class EAnnotationDocumentationDirectEditPolicy extends DirectEditPolicy
{
    private String oldValue;

    /**
     * @see org.eclipse.gef.editpolicies.DirectEditPolicy#getDirectEditCommand(org.eclipse.gef.requests.DirectEditRequest)
     */
    protected Command getDirectEditCommand(DirectEditRequest request)
    {
        EAnnotation eAnnotation = null;

        if (getHost().getModel() instanceof GraphNode)
        {
            GraphNode graphNode = (GraphNode) getHost().getModel();
            EMFSemanticModelBridge e = (EMFSemanticModelBridge) graphNode.getSemanticModel();
            eAnnotation = (EAnnotation) e.getElement();
        }

        if (eAnnotation == null)
        {
            EcorePlugin.log("Current model object is null. Unable to create command.", IStatus.ERROR);
            return UnexecutableCommand.INSTANCE;
        }

        EStringToStringMapEntryImpl entry = (EStringToStringMapEntryImpl) eAnnotation.getDetails().get(0);
        EStructuralFeature feature = entry.eClass().getEStructuralFeature("value");
        if (feature == null)
        {
            EcorePlugin.log("Selected feature (value) is not available for the current model object. Unable to create command.", IStatus.ERROR);
            return UnexecutableCommand.INSTANCE;
        }

        CellEditor cellEditor = request.getCellEditor();
        ChangeLabelTextCommand command = new ChangeLabelTextCommand(entry, feature);
        command.setOldName(oldValue);
        command.setName((String) cellEditor.getValue());
        return command;

    }

    /**
     * @see org.eclipse.gef.editpolicies.DirectEditPolicy#showCurrentEditValue(org.eclipse.gef.requests.DirectEditRequest)
     */
    protected void showCurrentEditValue(DirectEditRequest request)
    {
        String value = (String) request.getCellEditor().getValue();
        if (getHost() instanceof GraphNodeEditPart)
        {
            GraphNodeEditPart graphNodeEditPart = (GraphNodeEditPart) getHost();
            graphNodeEditPart.handleNameChange(value);
        }
    }

    /**
     * Saves the initial text value so that if the user's changes are not
     * committed then
     * 
     * @param request
     */
    protected void storeOldEditValue(DirectEditRequest request)
    {
        oldValue = (String) request.getCellEditor().getValue();
    }

}
