/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.ecore.ediagram.utils;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EParameter;

/**
 * Provide a structure for handling Operation data <br>
 * creation : 1 june 2005
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 */
public class EOperationDataStructure
{

    /**
     * Internal class to handle a parameter
     */
    public class ParameterObject
    {

        /** Parameter name */
        private String theName;

        /** Parameter type */
        private EClassifier theType;

        /**
         * The constructor
         * 
         * @param name parameter name
         * @param type parameter type
         */
        public ParameterObject(String name, EClassifier type)
        {
            theName = name;
            theType = type;
        }

        /**
         * Get property name
         * 
         * @return name of the property
         */
        public String getName()
        {
            return theName;
        }

        /**
         * Get property type
         * 
         * @return type of the property
         */
        public EClassifier getType()
        {
            return theType;
        }

        /**
         * Set new name for the parameter
         * 
         * @param name
         */
        public void setName(String name)
        {
            theName = name;
        }

        /**
         * Set new type for the parameter
         * 
         * @param type
         */
        public void setType(EClassifier type)
        {
            theType = type;
        }

    } // End internal class

    /** A collection for ParameterObject objects */
    private ArrayList<ParameterObject> dataList;

    /**
     * The constructor
     * 
     * @param operation
     */
    public EOperationDataStructure(EOperation operation)
    {
        dataList = new ArrayList<ParameterObject>();
        if (operation != null)
        {
            addAll(operation.getEParameters());
        }
    }

    /**
     * Add a property to the structure
     * 
     * @param parameter
     */
    public void add(EParameter parameter)
    {
        dataList.add(new ParameterObject(parameter.getName(), parameter.getEType()));
    }

    /**
     * 
     * @param name
     * @param type
     */
    public void add(String name, EClassifier type)
    {
        dataList.add(new ParameterObject(name, type));
    }

    /**
     * 
     * @param object
     */
    public void remove(Object object)
    {
        dataList.remove(object);
    }

    /**
     * Add a collection of properties to an instance specification
     * 
     * @param parameters
     */
    public void addAll(Collection<EParameter> parameters)
    {
        for (EParameter parameter : parameters)
        {
            add(parameter);
        }
    }

    /**
     * get the data collection
     * 
     * @return a Collection of InstanceStructureObject objects
     */
    public Collection<ParameterObject> getData()
    {
        return dataList;
    }

    /**
     * Get the name of a given object
     * 
     * @param object OperationDataObject object
     * @return the name
     */
    public String getDisplayName(Object object)
    {
        String result = ((ParameterObject) object).getName();
        if (result == null || result.equals(""))
        {
            result = "not set";
        }
        return result;
    }

    /**
     * Get the type name of a given object
     * 
     * @param object OperationDataObject object
     * @return the name of the type
     */
    public String getDisplayType(Object object)
    {
        EClassifier type = getType(object);
        if (type == null)
        {
            return null;
        }
        return type.getName();
    }

    /**
     * Get the type of a given object
     * 
     * @param object OperationDataObject object
     * @return the type
     */
    public EClassifier getType(Object object)
    {
        return ((ParameterObject) object).getType();
    }

    /**
     * 
     * @param object
     * @param name
     */
    public void setName(Object object, String name)
    {
        ((ParameterObject) object).setName(name);
    }

    /**
     * 
     * @param object
     * @param type
     */
    public void setType(Object object, EClassifier type)
    {
        ((ParameterObject) object).setType(type);
    }
}
