/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.ecore.editor;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.WrappedException;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.topcased.modeler.documentation.IDocPage;
import org.topcased.modeler.ecore.EcoreActionConstants;
import org.topcased.modeler.ecore.EcorePlugin;
import org.topcased.modeler.ecore.actions.DeleteInheritanceLinkAction;
import org.topcased.modeler.ecore.documentation.EcoreDocPage;
import org.topcased.modeler.ecore.editor.outline.EcoreOutlinePage;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.ModelerGraphicalViewer;

/**
 * Generated Model editor <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class EcoreEditor extends Modeler
{
    /**
     * @see org.topcased.modeler.editor.Modeler#getAdapterFactories() <!-- begin-user-doc --> <!-- end-user-doc -->
     * @generated
     */
    protected List<AdapterFactory> getAdapterFactories()
    {
        List<AdapterFactory> factories = new ArrayList<AdapterFactory>();
        factories.add(new org.eclipse.emf.ecore.provider.EcoreItemProviderAdapterFactory());
        factories.add(new org.topcased.modeler.ecore.providers.EcoreModelerProviderAdapterFactory());

        factories.addAll(super.getAdapterFactories());

        return factories;
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#getId()
     * @generated
     */
    public String getId()
    {
        return "org.topcased.modeler.ecore.editor.EcoreEditor";
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#getAdapter(java.lang.Class)
     */
    public Object getAdapter(Class type)
    {
        if (type == IDocPage.class)
        {
            return new EcoreDocPage(getCommandStack());
        }
        return super.getAdapter(type);
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#getPreferenceStore()
     */
    public IPreferenceStore getPreferenceStore()
    {
        return EcorePlugin.getDefault().getPreferenceStore();
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#createOutlinePage()
     */
    protected IContentOutlinePage createOutlinePage()
    {
        return new EcoreOutlinePage(this);
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#openFile(org.eclipse.core.resources.IFile, boolean)
     */
    protected EObject openFile(IFile file, boolean resolve)
    {
        if (resolve)
        {
            // Resolves the EPackages in the Ecore registry
            Set<Object> packagesURI = new HashSet<Object>(EPackage.Registry.INSTANCE.keySet());
            Iterator<Object> it = packagesURI.iterator();
            while (it.hasNext())
            {
                String uri = (String) it.next();
                try
                {
                    EPackage.Registry.INSTANCE.getEPackage(uri);
                }
                catch (WrappedException we)
                {
                    EcorePlugin.log("The EPackage '" + uri + "' cannot be loaded.", IStatus.WARNING);
                }
            }
        }
        return super.openFile(file, resolve);
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#getContextMenuProvider(org.topcased.modeler.editor.ModelerGraphicalViewer)
     */
    protected ContextMenuProvider getContextMenuProvider(ModelerGraphicalViewer viewer)
    {
        return new EcoreContextMenuProvider(viewer, getActionRegistry());
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#configureGraphicalViewer()
     */
    protected void configureGraphicalViewer()
    {
        super.configureGraphicalViewer();

        IAction deleteImplementLinkAction = getActionRegistry().getAction(EcoreActionConstants.DELETE_INHERITANCE_LINK);
        getGraphicalViewer().addSelectionChangedListener((ISelectionChangedListener) deleteImplementLinkAction);
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#createActions()
     */
    protected void createActions()
    {
        super.createActions();

        ActionRegistry registry = getActionRegistry();
        IAction action;

        action = new DeleteInheritanceLinkAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());
    }
}
