/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), 
 *    Mathieu Garcia (Anyware Technologies), 
 *    Jacques Lescot (Anyware Technologies),
 *    Thomas Friol (Anyware Technologies)
 *    - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.ecore.editor.outline;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.outline.ModelContentProvider;

/**
 * This content provider adds a "Registered Packages" node that shows all the packages registered using the package
 * extension point from EMF <br>
 * creation : 19 déc. 2005
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class EcoreContentProvider extends ModelContentProvider
{

    /**
     * Constructor
     * 
     * @param modeler the editor
     * @param delegatedProvider the ecore provider
     */
    public EcoreContentProvider(Modeler modeler, ITreeContentProvider delegatedProvider)
    {
        super(modeler, delegatedProvider);
    }

    /**
     * @see org.topcased.modeler.editor.outline.ModelContentProvider#getChildren(java.lang.Object)
     */
    public Object[] getChildren(Object parentElement)
    {
        // Return the edited root model object as root element
        if (parentElement instanceof Diagrams)
        {
            return new Object[] {EcoreUtil.getRootContainer(((Diagrams) parentElement).getModel()), getAdditionalResources(), EPackage.Registry.INSTANCE};
        }

        if (parentElement instanceof EPackage.Registry)
        {
            List<RegisteredPackage> registeredPackages = new ArrayList<RegisteredPackage>();
            for (String uri : EPackage.Registry.INSTANCE.keySet())
            {
                registeredPackages.add(new RegisteredPackage(uri));
            }
            return registeredPackages.toArray();
        }

        if (parentElement instanceof RegisteredPackage)
        {
            RegisteredPackage pack = (RegisteredPackage) parentElement;
            return new Object[] {pack.getEPackage()};
        }

        return super.getChildren(parentElement);
    }

    /**
     * @see org.topcased.modeler.editor.outline.ModelContentProvider#hasChildren(java.lang.Object)
     */
    public boolean hasChildren(Object element)
    {
        if (element instanceof EPackage.Registry)
        {
            return EPackage.Registry.INSTANCE.size() > 0;
        }
        if (element instanceof RegisteredPackage)
        {
            return true;
        }
        return super.hasChildren(element);
    }
}
