/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), 
 *    Mathieu Garcia (Anyware Technologies), 
 *    Jacques Lescot (Anyware Technologies),
 *    Thomas Friol (Anyware Technologies)
 *    - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.ecore.editor.outline;

import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.IPageSite;
import org.topcased.modeler.ecore.EcorePlugin;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.outline.ModelNavigator;

/**
 * This navigator add an element "Registered Package" in the tree <br>
 * creation : 19 déc. 2005
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class EcoreNavigator extends ModelNavigator
{

    /**
     * Constructor
     * 
     * @param parent the parent composite
     * @param editor the modeler
     * @param pageSite the outline site
     */
    public EcoreNavigator(Composite parent, Modeler editor, IPageSite pageSite)
    {
        super(parent, editor, pageSite);
    }

    /**
     * @see org.topcased.modeler.editor.outline.ModelNavigator#initProviders()
     */
    protected void initProviders()
    {
        AdapterFactoryContentProvider adapterContentProvider = new NavigatorAdapterFactoryContentProvider(
                getModeler().getAdapterFactory());
        adapterContentProvider.inputChanged(getTreeViewer(), null, null);
        getTreeViewer().setContentProvider(new EcoreContentProvider(getModeler(), adapterContentProvider));

        ILabelProvider fullLabelProvider = new DecoratingLabelProvider(new EcoreLabelProvider(
                new AdapterFactoryLabelProvider(getModeler().getAdapterFactory())),
                EcorePlugin.getDefault().getWorkbench().getDecoratorManager().getLabelDecorator());
        getTreeViewer().setLabelProvider(fullLabelProvider);
    }
}
