/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/

package org.topcased.modeler.ecore.internal.properties.metadata;

import java.util.Iterator;

import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.ExtendedMetaData;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.topcased.modeler.ecore.commands.EMFRecordingChangeCommand;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.properties.TextChangeHelper;
import org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection;

/**
 * Section to edit length extended metadata annotation
 * 
 * @see ExtendedMetaData#setLengthFacet(EDataType, int)
 * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class LengthSection extends AbstractTabbedPropertySection
{
    /**
     * A boolean that store if refreshing is happening and no model modifications should be performed
     */
    private boolean isRefreshing = false;

    /**
     * The text for the section.
     */
    private Text lengthTxt;

    /**
     * The section label;
     */
    private CLabel labelTxt;

    // /**
    // * @see org.eclipse.ui.views.properties.tabbed.ISection#createControls(org.eclipse.swt.widgets.Composite,
    // * org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage)
    // */
    // public void createControls(Composite parent, TabbedPropertySheetPage aTabbedPropertySheetPage)
    // {
    // super.createControls(parent, aTabbedPropertySheetPage);
    // Composite composite = getWidgetFactory().createFlatFormComposite(parent);
    // FormData data;
    //
    // String label = getLabelText();
    // CLabel labelTxt = getWidgetFactory().createCLabel(composite, label);
    //
    // lengthTxt = getWidgetFactory().createText(composite, "");
    //
    // data = new FormData();
    // data.left = new FormAttachment(0, 0);
    // data.right = new FormAttachment(lengthTxt, -ITabbedPropertyConstants.HSPACE);
    // data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    // labelTxt.setLayoutData(data);
    //
    // data = new FormData();
    // data.left = new FormAttachment(0, getStandardLabelWidth(composite, new String[] {label}));
    // data.right = new FormAttachment(100, 0);
    // data.top = new FormAttachment(labelTxt, 0, SWT.CENTER);
    // lengthTxt.setLayoutData(data);
    //
    // hookListeners();
    // }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#createWidgets(org.eclipse.swt.widgets.Composite)
     */
    protected void createWidgets(Composite composite)
    {
        labelTxt = getWidgetFactory().createCLabel(composite, getLabelText());

        lengthTxt = getWidgetFactory().createText(composite, "");
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#setSectionData(org.eclipse.swt.widgets.Composite)
     */
    protected void setSectionData(Composite composite)
    {
        FormData data = new FormData();
        data.left = new FormAttachment(0, 0);
        data.right = new FormAttachment(lengthTxt, -ITabbedPropertyConstants.HSPACE);
        data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
        labelTxt.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0, getStandardLabelWidth(composite, new String[] {getLabelText()}));
        data.right = new FormAttachment(100, 0);
        data.top = new FormAttachment(labelTxt, 0, SWT.CENTER);
        lengthTxt.setLayoutData(data);
    }

    /**
     * Adds the listeners on the widgets
     */
    protected void hookListeners()
    {
        TextChangeHelper listener = new TextChangeHelper()
        {
            public void textChanged(Control control)
            {
                handleTextModified();
            }
        };
        listener.startListeningTo(lengthTxt);
        listener.startListeningForEnter(lengthTxt);
    }

    /**
     * Handle the text modified event.
     */
    protected void handleTextModified()
    {
        if (!isRefreshing)
        {
            try
            {
                if (lengthTxt.getText() == null || lengthTxt.getText().length() == 0)
                {
                    lengthTxt.setText("-1");
                }
                final int newLength = Integer.parseInt(lengthTxt.getText());

                EditingDomain editingDomain = ((Modeler) getPart().getAdapter(Modeler.class)).getEditingDomain();
                if (getEObjectList().size() == 1)
                {
                    int oldLength = ExtendedMetaData.INSTANCE.getLengthFacet((EDataType) getEObject());
                    if (oldLength != newLength)
                    {
                        editingDomain.getCommandStack().execute(new EMFRecordingChangeCommand(getEObject().eResource())
                        {
                            protected void doExecute()
                            {
                                ExtendedMetaData.INSTANCE.setLengthFacet((EDataType) getEObject(), newLength);
                            }
                        });
                    }
                }
                else
                {
                    CompoundCommand compoundCommand = new CompoundCommand();
                    /* apply the property change to all selected elements */
                    for (Iterator<EObject> i = getEObjectList().iterator(); i.hasNext();)
                    {
                        final EObject nextObject = i.next();

                        int oldLength = ExtendedMetaData.INSTANCE.getLengthFacet((EDataType) nextObject);
                        if (oldLength != newLength)
                        {
                            editingDomain.getCommandStack().execute(new EMFRecordingChangeCommand(nextObject.eResource())
                            {
                                protected void doExecute()
                                {
                                    ExtendedMetaData.INSTANCE.setLengthFacet((EDataType) nextObject, newLength);
                                }
                            });
                        }
                    }
                    editingDomain.getCommandStack().execute(compoundCommand);
                }
            }
            catch (NumberFormatException e)
            {
                refresh();
            }
        }
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.ISection#refresh()
     */
    public void refresh()
    {
        isRefreshing = true;

        lengthTxt.setText("" + ExtendedMetaData.INSTANCE.getLengthFacet((EDataType) getEObject()));

        isRefreshing = false;
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getFeature()
     */
    protected EStructuralFeature getFeature()
    {
        return null;
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getLabelText()
     */
    protected String getLabelText()
    {
        return "Length";
    }

}
