/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.ecore.internal.properties.metadata;

import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.ExtendedMetaData;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.topcased.modeler.ecore.commands.EMFRecordingChangeCommand;
import org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection;

/**
 * Section to edit qualified extended metadata annotation
 * 
 * @see ExtendedMetaData#setQualified(org.eclipse.emf.ecore.EPackage, boolean)
 * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class QualifiedSection extends AbstractBooleanPropertySection
{
    /**
     * @see org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection#getFeature()
     */
    protected EStructuralFeature getFeature()
    {
        return null;
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection#getFeatureValue()
     */
    protected boolean getFeatureValue()
    {
        return ExtendedMetaData.INSTANCE.isQualified((EPackage) getEObject());
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getLabelText()
     */
    protected String getLabelText()
    {
        return "Qualified";
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection#handleCheckButtonModified()
     */
    protected void handleCheckButtonModified()
    {
        EditingDomain editingDomain = (EditingDomain) getPart().getAdapter(EditingDomain.class);
        final boolean newValue = getCheckButton().getSelection();
        if (getEObjectList().size() == 1)
        {
            editingDomain.getCommandStack().execute(new EMFRecordingChangeCommand(getEObject().eResource())
            {
                protected void doExecute()
                {
                    ExtendedMetaData.INSTANCE.setQualified((EPackage) getEObject(), newValue);
                }
            });
        }
        else
        {
            CompoundCommand compoundCommand = new CompoundCommand();
            /* apply the property change to all selected elements */
            for (final EObject nextObject : getEObjectList())
            {
                editingDomain.getCommandStack().execute(new EMFRecordingChangeCommand(nextObject.eResource())
                {
                    protected void doExecute()
                    {
                        ExtendedMetaData.INSTANCE.setQualified((EPackage) nextObject, newValue);
                    }
                });
            }
            editingDomain.getCommandStack().execute(compoundCommand);
        }
    }
}