/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.edit.policies;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.AbstractEditPolicy;
import org.topcased.modeler.ModelerRequestConstants;
import org.topcased.modeler.commands.DeleteModelCommand;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.utils.Utils;

/**
 * The EditPolicy that handle the Delete Model Object command
 * 
 * Creation : 09 may 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 * 
 */
public class DeleteModelObjectEditPolicy extends AbstractEditPolicy
{

    /**
     * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#getCommand(org.eclipse.gef.Request)
     */
    public Command getCommand(Request request)
    {

        if (ModelerRequestConstants.REQ_DELETE_MODEL_OBJECT.equals(request.getType()))
        {
            EObject model = null;
            if (getHost() instanceof EMFGraphNodeEditPart || getHost() instanceof EMFGraphEdgeEditPart)
            {
                GraphElement element = (GraphElement) getHost().getModel();
                model = Utils.getElement(element);
            }

            if (model != null)
            {

                if (getHost().getViewer().getEditDomain() instanceof MixedEditDomain)
                {
                    MixedEditDomain domain = ((MixedEditDomain) getHost().getViewer().getEditDomain());
                    Diagrams diagrams = ((Modeler) domain.getEditorPart()).getDiagrams();
                    if (domain != null && diagrams != null)
                    {
                        DeleteModelCommand command = new DeleteModelCommand(domain, diagrams, model);
                        return command;
                    }

                }
            }
        }
        return null;
    }

    /**
     * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#understandsRequest(org.eclipse.gef.Request)
     */
    public boolean understandsRequest(Request req)
    {
        if (ModelerRequestConstants.REQ_DELETE_MODEL_OBJECT.equals(req.getType()))
        {
            return true;
        }

        return false;
    }

}
