/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.editor;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.ui.views.properties.IPropertySource;

/**
 * This property source provider returns the associated model object (if it
 * exists) for a given edit part. <br>
 * creation : 30 nov. 2004
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma </a>
 * @deprecated Use the TabbedPropertiesContentProvider instead
 */
public class GEFPropertySourceProvider extends AdapterFactoryContentProvider
{

    /**
     * Constructor
     * 
     * @param adapterFactory
     */
    public GEFPropertySourceProvider(AdapterFactory adapterFactory)
    {
        super(adapterFactory);
    }

    /**
     * @see org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider#getPropertySource(java.lang.Object)
     */
    public IPropertySource getPropertySource(Object object)
    {
        Object propertysource = object;
        if (object instanceof EObject)
        {
            propertysource = object;
        }
        else if (object instanceof IAdaptable)
        {
            // Try IAdaptable
            IAdaptable adapted = (IAdaptable) object;
            Object eObject = adapted.getAdapter(EObject.class);
            if (eObject != null)
            {
                propertysource = eObject;
            }
        }
        else if (object != null)
        {
            // Try registered adapter
            Object adapted = Platform.getAdapterManager().getAdapter(object, EObject.class);
            if (adapted != null)
            {
                propertysource = adapted;
            }
        }
        return super.getPropertySource(propertysource);
    }

}
