/*******************************************************************************
 * Copyright (c) 2005, 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.editor;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EValidator;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.MouseWheelHandler;
import org.eclipse.gef.MouseWheelZoomHandler;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.AlignmentAction;
import org.eclipse.gef.ui.actions.DirectEditAction;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.actions.MatchHeightAction;
import org.eclipse.gef.ui.actions.MatchWidthAction;
import org.eclipse.gef.ui.actions.ToggleGridAction;
import org.eclipse.gef.ui.actions.ToggleSnapToGeometryAction;
import org.eclipse.gef.ui.actions.ZoomInAction;
import org.eclipse.gef.ui.actions.ZoomOutAction;
import org.eclipse.gef.ui.palette.PaletteContextMenuProvider;
import org.eclipse.gef.ui.parts.GraphicalEditorWithPalette;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.ide.IGotoMarker;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.topcased.modeler.ModelerActionConstants;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.diagrams.model.util.DiagramsUtils;
import org.topcased.modeler.dialogs.ChooseGraphElementDialog;
import org.topcased.modeler.edit.DiagramsRootEditPart;
import org.topcased.modeler.editor.outline.DiagramsOutlinePage;
import org.topcased.modeler.editor.properties.ModelerPropertySheetPage;
import org.topcased.modeler.internal.IntegrityChecker;
import org.topcased.modeler.internal.ModelerPlugin;
import org.topcased.modeler.internal.actions.AutoResizeAction;
import org.topcased.modeler.internal.actions.ChangeBackgroundColorAction;
import org.topcased.modeler.internal.actions.ChangeDiagramPropertiesAction;
import org.topcased.modeler.internal.actions.ChangeFontAction;
import org.topcased.modeler.internal.actions.ChangeForegroundColorAction;
import org.topcased.modeler.internal.actions.ChangeRouterAction;
import org.topcased.modeler.internal.actions.DeleteGraphElementAction;
import org.topcased.modeler.internal.actions.DeleteModelObjectAction;
import org.topcased.modeler.internal.actions.GoToRootDiagramAction;
import org.topcased.modeler.internal.actions.ModelerValidateAction;
import org.topcased.modeler.internal.actions.MoveBackwardAction;
import org.topcased.modeler.internal.actions.MoveForwardAction;
import org.topcased.modeler.internal.actions.MoveToBackAction;
import org.topcased.modeler.internal.actions.MoveToFrontAction;
import org.topcased.modeler.internal.actions.NextDiagramAction;
import org.topcased.modeler.internal.actions.OpenParentDiagramAction;
import org.topcased.modeler.internal.actions.PreviousDiagramAction;
import org.topcased.modeler.internal.actions.RestoreConnectionsAction;
import org.topcased.modeler.internal.actions.ShowDocViewAction;
import org.topcased.modeler.internal.actions.ShowPropertiesViewAction;
import org.topcased.modeler.internal.editor.palette.PaletteStateRecorder;
import org.topcased.modeler.internal.exceptions.IntegrityException;
import org.topcased.modeler.preferences.ModelerPreferenceConstants;
import org.topcased.modeler.utils.ResourceUtils;
import org.topcased.modeler.utils.Utils;

/**
 * Basic EMF modeler. <br>
 * creation : 30 nov. 2004
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma </a>
 */
public abstract class Modeler extends GraphicalEditorWithPalette implements ISelectionProvider, IEditingDomainProvider, IGotoMarker, ITabbedPropertySheetPageContributor
{
    /**
     * The global edited diagrams model
     */
    private Resource diResource;

    /**
     * The currently edited diagram
     */
    private Diagram activeDiagram;

    /** The class that manage the navigation throught the differents diagrams */
    private NavigationManager navigation;

    private ModelerConfigurationManager configurationManager;

    private TabbedPropertySheetPage propertySheetPage;

    /**
     * Constructor. Set the edit domain.
     */
    public Modeler()
    {
        super();
        navigation = new NavigationManager(this);
    }

    /**
     * @see org.eclipse.gef.ui.parts.GraphicalEditor#dispose()
     */
    public void dispose()
    {
        super.dispose();
        navigation = null;
    }

    /**
     * @see org.eclipse.gef.commands.CommandStackListener#commandStackChanged(java.util.EventObject)
     */
    public void commandStackChanged(final EventObject event)
    {
        if (Display.getCurrent() != Display.getDefault())
        {
            Display.getDefault().asyncExec(new Runnable()
            {
                public void run()
                {
                    safeCommandStackChanged(event);
                }
            });
        }
        else
        {
            safeCommandStackChanged(event);
        }
    }

    /**
     * Handle the event in the UI thread
     * 
     * @param event the command stack event
     */
    protected void safeCommandStackChanged(EventObject event)
    {
        firePropertyChange(PROP_DIRTY);
        super.commandStackChanged(event);
    }

    /**
     * Creates the GraphicalViewer on the specified <code>Composite</code>.
     * 
     * @param parent the parent composite
     * @see org.eclipse.gef.ui.parts.GraphicalEditor#createGraphicalViewer(org.eclipse.swt.widgets.Composite)
     */
    protected void createGraphicalViewer(Composite parent)
    {
        GraphicalViewer viewer = new ModelerGraphicalViewer();
        viewer.createControl(parent);
        setGraphicalViewer(viewer);
        configureGraphicalViewer();
        hookGraphicalViewer();
        initializeGraphicalViewer();
    }

    /**
     * Subclass should override this method to instantiate their own ContextMenu
     * 
     * @param viewer the GraphicalViewer
     * @return the ContextMenuProvider
     */
    protected ContextMenuProvider getContextMenuProvider(ModelerGraphicalViewer viewer)
    {
        return new ModelerContextMenuProvider(viewer, getActionRegistry());
    }

    /**
     * @see org.eclipse.gef.ui.parts.GraphicalEditor#configureGraphicalViewer()
     */
    protected void configureGraphicalViewer()
    {
        super.configureGraphicalViewer();
        final ModelerGraphicalViewer viewer = (ModelerGraphicalViewer) getGraphicalViewer();

        // configure the EditPartViewer
        ScalableFreeformRootEditPart root = new DiagramsRootEditPart();
        viewer.setModelerEditor(this);
        viewer.setRootEditPart(root);

        // set the ContextMenuProvider
        ContextMenuProvider provider = getContextMenuProvider(viewer);
        viewer.setContextMenu(provider);
        ((IEditorSite) getSite()).registerContextMenu("org.topcased.modeler", provider, viewer, false);

        // Begin : Deal with keys
        KeyHandler keyHandler = new ModelerKeyHandler(this);
        keyHandler.put(KeyStroke.getPressed(SWT.F2, 0), getActionRegistry().getAction(GEFActionConstants.DIRECT_EDIT));
        viewer.setKeyHandler(keyHandler);
        // End : Deal with keys

        // Zoom
        List<String> zoomLevels = new ArrayList<String>();
        zoomLevels.add(ZoomManager.FIT_ALL);
        zoomLevels.add(ZoomManager.FIT_WIDTH);
        zoomLevels.add(ZoomManager.FIT_HEIGHT);
        root.getZoomManager().setZoomLevelContributions(zoomLevels);

        IAction zoomIn = new ZoomInAction(root.getZoomManager());
        IAction zoomOut = new ZoomOutAction(root.getZoomManager());
        getActionRegistry().registerAction(zoomIn);
        getActionRegistry().registerAction(zoomOut);
        getSite().getKeyBindingService().registerAction(zoomIn);
        getSite().getKeyBindingService().registerAction(zoomOut);

        IAction snapAction = new ToggleSnapToGeometryAction(getGraphicalViewer());
        getActionRegistry().registerAction(snapAction);

        IAction showGrid = new ToggleGridAction(getGraphicalViewer());
        getActionRegistry().registerAction(showGrid);

        // Scroll-wheel Zoom
        getGraphicalViewer().setProperty(MouseWheelHandler.KeyGenerator.getKey(SWT.CTRL), MouseWheelZoomHandler.SINGLETON);

        registerSelectionChangedActions(viewer);
    }

    /**
     * @see org.eclipse.gef.ui.parts.GraphicalEditorWithPalette#configurePaletteViewer()
     */
    protected void configurePaletteViewer()
    {
        getPaletteViewer().setContextMenu(new PaletteContextMenuProvider(getPaletteViewer()));

        // TODO also create our own PaletteCustomizer
    }

    /**
     * Register actions that listen to selection
     * 
     * @param viewer the graphical viewer : emitter of the selection changed events
     */
    protected void registerSelectionChangedActions(GraphicalViewer viewer)
    {
        // ---------------------------------------
        // register selection change with actions
        // ---------------------------------------

        // // TODO These actions are not yet implemented. Model refactoring is
        // needed
        // IAction copyAction =
        // getActionRegistry().getAction(ActionFactory.COPY.getId());
        // viewer.addSelectionChangedListener((ISelectionChangedListener)
        // copyAction);
        //
        // IAction cutAction =
        // getActionRegistry().getAction(ActionFactory.CUT.getId());
        // viewer.addSelectionChangedListener((ISelectionChangedListener)
        // cutAction);

        IAction changeBackGroundColorAction = getActionRegistry().getAction(ModelerActionConstants.CHANGE_BACKGROUND_COLOR);
        viewer.addSelectionChangedListener((ISelectionChangedListener) changeBackGroundColorAction);

        IAction changeForeGroundColorAction = getActionRegistry().getAction(ModelerActionConstants.CHANGE_FOREGROUND_COLOR);
        viewer.addSelectionChangedListener((ISelectionChangedListener) changeForeGroundColorAction);

        IAction changeRouterAction = getActionRegistry().getAction(ModelerActionConstants.CHANGE_ROUTER);
        viewer.addSelectionChangedListener((ISelectionChangedListener) changeRouterAction);

        IAction changeFontAction = getActionRegistry().getAction(ModelerActionConstants.CHANGE_FONT);
        viewer.addSelectionChangedListener((ISelectionChangedListener) changeFontAction);

        IAction changeDiagramPropertiesAction = getActionRegistry().getAction(ModelerActionConstants.CHANGE_DIAGRAM_PROPERTIES);
        viewer.addSelectionChangedListener((ISelectionChangedListener) changeDiagramPropertiesAction);

        IAction deleteModelObjectAction = getActionRegistry().getAction(ModelerActionConstants.DELETE_MODEL_OBJECT);
        viewer.addSelectionChangedListener((ISelectionChangedListener) deleteModelObjectAction);

        IAction moveForwardAction = getActionRegistry().getAction(ModelerActionConstants.MOVE_FORWARD);
        viewer.addSelectionChangedListener((ISelectionChangedListener) moveForwardAction);

        IAction moveBackwardAction = getActionRegistry().getAction(ModelerActionConstants.MOVE_BACKWARD);
        viewer.addSelectionChangedListener((ISelectionChangedListener) moveBackwardAction);

        IAction moveToFrontAction = getActionRegistry().getAction(ModelerActionConstants.MOVE_TO_FRONT);
        viewer.addSelectionChangedListener((ISelectionChangedListener) moveToFrontAction);

        IAction moveToBackAction = getActionRegistry().getAction(ModelerActionConstants.MOVE_TO_BACK);
        viewer.addSelectionChangedListener((ISelectionChangedListener) moveToBackAction);

        IAction autoResizeAction = getActionRegistry().getAction(ModelerActionConstants.AUTO_RESIZE);
        viewer.addSelectionChangedListener((ISelectionChangedListener) autoResizeAction);

        IAction restoreConnAction = getActionRegistry().getAction(ModelerActionConstants.RESTORE_CONNECTIONS);
        viewer.addSelectionChangedListener((ISelectionChangedListener) restoreConnAction);
    }

    /**
     * Creates actions for the modeler.
     * 
     * @see org.eclipse.gef.ui.parts.GraphicalEditor#createActions()
     */
    protected void createActions()
    {
        super.createActions();

        ActionRegistry registry = getActionRegistry();
        IAction action;

        action = new DeleteGraphElementAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new DirectEditAction((IWorkbenchPart) this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        // // TODO These actions are not yet implemented. Model refactoring is
        // needed
        // action = new CopyAction(this);
        // registry.registerAction(action);
        // getSelectionActions().add(action.getId());
        //
        // action = new CutAction(this);
        // registry.registerAction(action);
        // getSelectionActions().add(action.getId());
        //
        // action = new PasteAction(this);
        // registry.registerAction(action);
        // getSelectionActions().add(action.getId());

        action = new ChangeRouterAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new ChangeBackgroundColorAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new ChangeForegroundColorAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new ChangeFontAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new ChangeDiagramPropertiesAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new AutoResizeAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new DeleteModelObjectAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new RestoreConnectionsAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        // Alignment Actions
        action = new AlignmentAction((IWorkbenchPart) this, PositionConstants.LEFT);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new AlignmentAction((IWorkbenchPart) this, PositionConstants.RIGHT);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new AlignmentAction((IWorkbenchPart) this, PositionConstants.TOP);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new AlignmentAction((IWorkbenchPart) this, PositionConstants.BOTTOM);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new AlignmentAction((IWorkbenchPart) this, PositionConstants.CENTER);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new AlignmentAction((IWorkbenchPart) this, PositionConstants.MIDDLE);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        // Size Actions
        action = new MatchWidthAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new MatchHeightAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new AutoResizeAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new ShowPropertiesViewAction(this);
        registry.registerAction(action);

        action = new ShowDocViewAction(this);
        registry.registerAction(action);

        // Previous and next navigation Action
        action = new PreviousDiagramAction(this);
        registry.registerAction(action);

        action = new NextDiagramAction(this);
        registry.registerAction(action);

        // Diagram navigation Action
        action = new OpenParentDiagramAction(this);
        registry.registerAction(action);

        action = new GoToRootDiagramAction(this);
        registry.registerAction(action);

        // Change diagram plans actions
        action = new MoveForwardAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new MoveBackwardAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new MoveToFrontAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new MoveToBackAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        // Validation action
        action = new ModelerValidateAction(this);
        registry.registerAction(action);
    }

    /**
     * This method creates the list of factories used by the editor. <br>
     * Resource, DI, Diagrams and Reflexive factories are added here. <br>
     * <b>This method could be overridden to added specific factories before those. </b> <br>
     * 
     * @return the factories' list
     */
    protected List<AdapterFactory> getAdapterFactories()
    {
        List<AdapterFactory> factories = new ArrayList<AdapterFactory>();
        factories.add(new org.topcased.modeler.di.model.provider.DiagramInterchangeItemProviderAdapterFactory());
        factories.add(new org.topcased.modeler.diagrams.model.provider.DiagramsItemProviderAdapterFactory());
        factories.add(new ResourceItemProviderAdapterFactory());
        factories.add(new ReflectiveItemProviderAdapterFactory());

        return factories;
    }

    /**
     * This method creates the ComposedAdapterFactory that groups all the factories used by the editor. <br>
     * <br>
     * 
     * @return the multi-factory
     */
    protected ComposedAdapterFactory createAdapterFactory()
    {
        // Create an adapter factory that yields item providers.

        return new ComposedAdapterFactory(getAdapterFactories());
    }

    /**
     * Returns the global adapter factory used by the editor
     * 
     * @return the composed adapter factory
     */
    public AdapterFactory getAdapterFactory()
    {
        if (getEditDomain() instanceof MixedEditDomain)
        {
            MixedEditDomain editDomain = (MixedEditDomain) getEditDomain();
            if (editDomain.getAdapterFactory() == null)
            {
                editDomain.setAdapterFactory(createAdapterFactory());
            }

            return editDomain.getAdapterFactory();
        }

        return createAdapterFactory();
    }

    /**
     * Returns the active diagram configuration, given by the configuration manager.
     * 
     * @return the active configuration
     */
    public IConfiguration getActiveConfiguration()
    {
        return getConfigurationManager().getActiveConfiguration();
    }

    /**
     * Returns the global edited model.
     * 
     * @return the diagrams model
     */
    public Diagrams getDiagrams()
    {
        return (Diagrams) diResource.getContents().get(0);
    }

    /**
     * Increase the access to the action registry to public
     * 
     * @return the action registry
     */
    public ActionRegistry getPublicActionRegistry()
    {
        // TODO see if the method could not be deleted
        return getActionRegistry();
    }

    /**
     * Returns the main edit part of the editor
     * 
     * @return the root edit part
     */
    public RootEditPart getRootEditPart()
    {
        return getGraphicalViewer().getRootEditPart();
    }

    /**
     * The modeler can be adapted into a IPropertySheetPage or a IContentOutline.
     * 
     * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
     */
    public Object getAdapter(Class type)
    {
        if (type == IPropertySheetPage.class)
        {
            return getPropertySheetPage();
        }
        if (type == IContentOutlinePage.class)
        {
            return createOutlinePage();
        }
        if (type == DefaultEditDomain.class || type == EditDomain.class)
        {
            return getEditDomain();
        }
        if (type == MixedEditDomain.class)
        {
            DefaultEditDomain editDomain = getEditDomain();
            if (editDomain instanceof MixedEditDomain)
            {
                return editDomain;
            }
        }
        if (type == EditingDomain.class)
        {
            DefaultEditDomain editDomain = getEditDomain();
            if (editDomain instanceof MixedEditDomain)
            {
                return ((MixedEditDomain) editDomain).getEMFEditingDomain();
            }
        }
        if (type == ZoomManager.class)
        {
            return getGraphicalViewer().getProperty(ZoomManager.class.toString());
        }

        return super.getAdapter(type);
    }

    private IPropertySheetPage getPropertySheetPage()
    {
        if (propertySheetPage == null)
        {
            propertySheetPage = new ModelerPropertySheetPage(this);
        }
        return propertySheetPage;

        // PropertySheetPage page = new PropertySheetPage()
        // {
        // /**
        // * @see
        // org.eclipse.ui.views.properties.PropertySheetPage#setActionBars(org.eclipse.ui.IActionBars)
        // */
        // public void setActionBars(IActionBars actionBars)
        // {
        // super.setActionBars(actionBars);
        // actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(),
        // getActionRegistry().getAction(ActionFactory.UNDO.getId()));
        // actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(),
        // getActionRegistry().getAction(ActionFactory.REDO.getId()));
        // }
        // };
        // page.setPropertySourceProvider(new
        // GEFPropertySourceProvider(getAdapterFactory()));
        // return page;
    }

    /**
     * Create the outline for this editor.<br>
     * <b>This method can be overridden by subclasses to customize the outline</b>
     * 
     * @return the outline
     */
    protected IContentOutlinePage createOutlinePage()
    {
        return new DiagramsOutlinePage(this);
    }

    /**
     * Returns the current palette root.
     * 
     * @see org.eclipse.gef.ui.parts.GraphicalEditorWithPalette#getPaletteRoot()
     */
    protected PaletteRoot getPaletteRoot()
    {
        if (getActiveConfiguration() != null)
        {
            return getActiveConfiguration().getPaletteManager().getRoot();
        }

        return null;
    }

    /**
     * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
     */
    public void doSave(IProgressMonitor monitor)
    {
        // Do the work within an operation because this is a long running
        // activity that modifies the workbench.
        //
        WorkspaceModifyOperation operation = new WorkspaceModifyOperation()
        {
            public void execute(IProgressMonitor mon) throws CoreException
            {
                try
                {
                    asyncSave(mon);
                }
                catch (IntegrityException ie)
                {
                    throw new CoreException(new Status(IStatus.ERROR, ModelerPlugin.getId(), IStatus.ERROR, ie.getMessage(), ie));
                }
                catch (IOException ioe)
                {
                    throw new CoreException(new Status(IStatus.ERROR, ModelerPlugin.getId(), IStatus.ERROR, ioe.getMessage(), ioe));
                }
            }
        };

        try
        {
            // This runs the options, and shows progress.
            //
            new ProgressMonitorDialog(getSite().getShell()).run(true, false, operation);

            getCommandStack().markSaveLocation();
        }
        catch (InvocationTargetException ite)
        {
            Throwable t = ite.getCause();
            if (t instanceof CoreException)
            {
                CoreException ce = (CoreException) t;
                ErrorDialog.openError(getSite().getShell(), "Unable to save", ce.getStatus().getMessage(), ce.getStatus());
            }
            else
            {
                ModelerPlugin.log(t);
                ModelerPlugin.displayDialog("Unable to save", "An error occurred during save.\nSee error logs for more details.", IStatus.ERROR);
            }
        }
        catch (InterruptedException ie)
        {
            // Ignore cancel
        }
        finally
        {
            monitor.done();
        }
    }

    /**
     * Effective save action : <br>
     * <ol>
     * <li> Save the diagram state
     * <li> Save the palette state
     * <li> Save the active diagram
     * <li> Check the model integrity
     * <li> Save the models in the right files
     * </ol>
     * If one of these tasks fails, the save is cancelled and a message is send to the user.
     * 
     * @param monitor
     * @throws IntegrityException The model is not valid
     * @throws IOException An error occurred during save
     */
    protected void asyncSave(IProgressMonitor monitor) throws IntegrityException, IOException
    {
        monitor.beginTask("Save the diagrams", 5);

        // Step 1 : Save current states
        monitor.subTask("Save current state");
        saveDiagramSettings();
        savePaletteState();
        saveActiveDiagram();
        monitor.worked(1);
        // End Step 1

        // Step 2 : Check integrity
        if (needCheckIntegrity())
        {
            monitor.subTask("Check model integrity");
            IFile file = ((IFileEditorInput) getEditorInput()).getFile();
            try
            {
                IntegrityChecker.checkModelIntegrity(file, getResourceSet(), new SubProgressMonitor(monitor, 1));
            }
            catch (IOException ioe)
            {
                ModelerPlugin.log("The integrity check cannot be processed", IStatus.WARNING);
                ModelerPlugin.log(ioe);
            }
        }
        monitor.worked(1);
        // End Step 2

        // Step 3 : save resources
        monitor.subTask("Save files");
        saveResources(new SubProgressMonitor(monitor, 3));
        monitor.worked(3);
        // End Step 3
    }

    /**
     * Check whether it is necessary to check the model integrity on the ResourceSet.
     * 
     * @return <code>true</code> if the integrity check is necessary
     */
    protected boolean needCheckIntegrity()
    {
        return ModelerPlugin.getDefault().getPreferenceStore().getBoolean(ModelerPreferenceConstants.P_CHECK_INTEGRITY);
    }

    /**
     * Save the resources from the current ResourceSet and add UUIDs if the preference is set
     * 
     * @param monitor The progress monitor
     * 
     * @throws IOException throws if a resource cannot be saved
     */
    protected void saveResources(IProgressMonitor monitor) throws IOException
    {
        monitor.beginTask("Save resources", getResourceSet().getResources().size());

        Map<String, String> options = new HashMap<String, String>();
        options.put(XMLResource.OPTION_PROCESS_DANGLING_HREF, XMLResource.OPTION_PROCESS_DANGLING_HREF_DISCARD);
        options.put(XMLResource.OPTION_ENCODING, "UTF-8");

        // Save resources
        for (Resource resource : getResourceSet().getResources())
        {
            if (!ResourceUtils.isReadOnly(resource))
            {
                if (ModelerPlugin.getDefault().getPreferenceStore().getBoolean(ModelerPreferenceConstants.P_USE_UUID) && ResourceUtils.isModelResource(resource))
                {
                    ResourceUtils.addUUID(resource.getContents().get(0));
                }
                resource.save(options);
            }
            monitor.worked(1);
        }

        monitor.done();
    }

    /**
     * @see org.eclipse.ui.part.EditorPart#doSaveAs()
     */
    public void doSaveAs()
    {
        // Do nothing
    }

    /**
     * @see org.eclipse.gef.ui.parts.GraphicalEditor#initializeGraphicalViewer()
     */
    protected void initializeGraphicalViewer()
    {
        restoreActiveDiagram();
        getGraphicalViewer().addDropTargetListener(new OutlineDropListener(getGraphicalViewer(), this));
        getGraphicalViewer().addDropTargetListener(new PaletteDropListener(getGraphicalViewer()));
    }

    /**
     * Update the palette contents with objects of the model that can be displayed in the current diagram.
     */
    protected void updatePaletteRoot()
    {
        if (getActiveConfiguration() != null)
        {
            PaletteRoot oldRoot = getEditDomain().getPaletteViewer().getPaletteRoot();
            getActiveConfiguration().getPaletteManager().updatePalette();
            PaletteRoot newRoot = getActiveConfiguration().getPaletteManager().getRoot();

            if (oldRoot != newRoot)
            {
                // Deselect previous active tool
                if (oldRoot != null)
                {
                    oldRoot.setDefaultEntry(null);
                    getEditDomain().getPaletteViewer().setActiveTool(null);
                }

                getEditDomain().setPaletteRoot(newRoot);
            }
        }
    }

    /**
     * Update the EditPartFactory used by the viewer to map Model element with edit parts.
     */
    protected void updateEditPartFactory()
    {
        if (getActiveConfiguration() != null)
        {
            EditPartFactory factory = getActiveConfiguration().getEditPartFactory();
            if (factory != null)
            {
                // Change the factory to the new one (specific to the active
                // diagram)
                getGraphicalViewer().setEditPartFactory(factory);
            }
        }
    }

    /**
     * Notify the configuration manager that the active diagram changed.
     */
    protected void updateConfiguration()
    {
        getConfigurationManager().setActiveDiagram(getActiveDiagram());
    }

    /**
     * Set the currently edited diagram
     * 
     * @param diagram the diagram to edit
     */
    public void setActiveDiagram(final Diagram diagram)
    {
        if (Display.getCurrent() != Display.getDefault())
        {
            Display.getDefault().asyncExec(new Runnable()
            {
                public void run()
                {
                    navigation.set(diagram);
                }
            });
        }
        else
        {
            navigation.set(diagram);
        }
    }

    /**
     * Returns the diagram navigation
     * 
     * @return the navigation
     */
    public NavigationManager getNavigationManager()
    {
        return navigation;
    }

    /**
     * Save the active diagram in the Diagrams model
     */
    protected void saveActiveDiagram()
    {
        getDiagrams().setActiveDiagram(activeDiagram);
    }

    /**
     * Restore the active diagram stored the Diagrams model
     */
    protected void restoreActiveDiagram()
    {
        Diagram lastDiagram = getDiagrams().getActiveDiagram();
        if (lastDiagram == null)
        {
            lastDiagram = DiagramsUtils.getRootDiagram(getDiagrams());
        }
        setActiveDiagram(lastDiagram);
    }

    /**
     * Save the settings (viewport, zoom...) for the current diagram
     */
    protected void saveDiagramSettings()
    {
        if (activeDiagram != null)
        {
            // We do not use command because these modifications are
            // non-dirtying and non undoable
            RootEditPart root = getGraphicalViewer().getRootEditPart();
            if (root instanceof ScalableFreeformRootEditPart)
            {
                ScalableFreeformRootEditPart scalableRoot = (ScalableFreeformRootEditPart) root;
                // Save viewport
                Viewport viewPort = (Viewport) scalableRoot.getFigure();
                Point viewLoc = viewPort.getViewLocation();
                activeDiagram.setViewport(viewLoc);

                // Save Zoom
                ZoomManager zoomManager = scalableRoot.getZoomManager();
                double zoomLevel = zoomManager.getZoom();
                activeDiagram.setZoom(zoomLevel);
            }
        }
    }

    /**
     * Save the state of the diagram palette (pinned, expanded...)
     */
    protected void savePaletteState()
    {
        if (activeDiagram != null)
        {
            PaletteStateRecorder recorder = new PaletteStateRecorder();
            recorder.saveState(activeDiagram.getSemanticModel().getPresentation(), getPaletteViewer());
        }
    }

    /**
     * Restore the state of the diagram palette (pinned, expanded...)
     */
    protected void restorePaletteState()
    {
        if (activeDiagram != null)
        {
            PaletteStateRecorder recorder = new PaletteStateRecorder();
            recorder.restoreState(activeDiagram.getSemanticModel().getPresentation(), getPaletteViewer());
        }
    }

    /**
     * Restore the settings (viewport, zoom...) for the current diagram
     */
    protected void restoreDiagramSettings()
    {
        if (activeDiagram != null)
        {
            // We do not use command because these modifications are
            // non-dirtying and non undoable
            RootEditPart root = getGraphicalViewer().getRootEditPart();
            if (root instanceof ScalableFreeformRootEditPart)
            {
                ScalableFreeformRootEditPart scalableRoot = (ScalableFreeformRootEditPart) root;

                // Restore Zoom
                double zoomLevel = activeDiagram.getZoom();
                ZoomManager zoomManager = scalableRoot.getZoomManager();
                zoomManager.setZoom(zoomLevel);

                // Restore viewport
                Point viewLoc = activeDiagram.getViewport();
                if (viewLoc == null)
                {
                    viewLoc = new Point(0, 0);
                }
                Viewport viewPort = (Viewport) scalableRoot.getFigure();
                viewPort.setViewLocation(viewLoc);
            }
        }
    }

    /**
     * Change the current diagram
     * 
     * @param diagram the new current diagram
     */
    void changeActiveDiagram(Diagram diagram)
    {
        saveDiagramSettings();
        savePaletteState();

        activeDiagram = diagram;

        // Update the active diagram configuration
        updateConfiguration();

        // Update the palette
        updatePaletteRoot();
        // Update the edit part factory
        updateEditPartFactory();
        // Update the GEF viewer
        getGraphicalViewer().setContents(diagram);

        restoreDiagramSettings();
        restorePaletteState();

        EditPart diagEP = (EditPart) getGraphicalViewer().getEditPartRegistry().get(diagram);
        if (diagEP != null)
        {
            getGraphicalViewer().select(diagEP);
        }
    }

    /**
     * Returns the currently edited diagram.
     * 
     * @return the edited diagram
     */
    public Diagram getActiveDiagram()
    {
        return activeDiagram;
    }

    /**
     * @see org.eclipse.ui.ISaveablePart#isDirty()
     */
    public boolean isDirty()
    {
        // delegate to the command stack
        return getCommandStack().isDirty();
    }

    /**
     * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
     */
    public boolean isSaveAsAllowed()
    {
        return false;
    }

    /**
     * Get the ResourceSet
     * 
     * @return ResourceSet
     */
    public ResourceSet getResourceSet()
    {
        ResourceSet resourceSet = null;
        if (getEditDomain() instanceof MixedEditDomain)
        {
            resourceSet = ((MixedEditDomain) getEditDomain()).getEMFEditingDomain().getResourceSet();
        }

        return resourceSet;
    }

    /**
     * Return the EObject root of a File
     * 
     * @param file the file to read
     * @return the root EObject the root object
     */
    protected EObject openFile(IFile file)
    {
        // TODO Check if we absolutely need to resolve all the references ?
        return openFile(file, true);
    }

    /**
     * Return the EObject root of a File
     * 
     * @param file the file to read
     * @param resolve indicates that all the references must be resolved
     * @return the root EObject the root object
     */
    protected EObject openFile(IFile file, boolean resolve)
    {
        URI uri = URI.createPlatformResourceURI(file.getFullPath().toString(), false);
        Resource resource = getResourceSet().getResource(uri, true);
        if (resolve)
        {
            EcoreUtil.resolveAll(getResourceSet());
        }
        return resource.getContents().get(0);
    }

    /**
     * Create and initialize the GEF-EMF EditDomain
     * 
     * @return the modeler EditDomain
     */
    protected MixedEditDomain createEditDomain()
    {
        MixedEditDomain mixedEditDomain = new MixedEditDomain(this);
        mixedEditDomain.setAdapterFactory(createAdapterFactory());

        return mixedEditDomain;
    }

    /**
     * @see org.eclipse.ui.part.EditorPart#setInput(org.eclipse.ui.IEditorInput)
     */
    protected void setInput(IEditorInput input)
    {
        try
        {
            MixedEditDomain editDomain = createEditDomain();
            setEditDomain(editDomain);

            IFile file = ((IFileEditorInput) input).getFile();
            EObject modelInput = openFile(file);

            if (modelInput instanceof Diagrams)
            {
                DiagramsUtils.updateDI((Diagrams) modelInput);

                diResource = modelInput.eResource();

                setPartName(file.getName());
                setContentDescription(file.getFullPath().toString());
                doSetInput(input);
            }
            else
            {
                ModelerPlugin.displayDialog("Topcased Editor open error", "The input is not a diagrams model.", IStatus.ERROR);
            }
        }
        catch (Exception e)
        {
            ModelerPlugin.displayDialog("Topcased Editor open error", "Unable to open editor. Please, check your input model.", IStatus.ERROR);
            ModelerPlugin.log(e);
        }
    }

    /**
     * @param input the IEditorInput
     * @see org.eclipse.ui.part.EditorPart#setInput(org.eclipse.ui.IEditorInput)
     */
    protected void doSetInput(IEditorInput input)
    {
        super.setInput(input);
    }

    /**
     * Sets the Diagrams model
     * 
     * @param diag the model object Diagrams
     */
    protected void setDiagrams(Diagrams diag)
    {
        this.diResource = diag.eResource();
    }

    /**
     * Close the editor
     * 
     * @param save
     */
    public void close(final boolean save)
    {
        Display display = getSite().getShell().getDisplay();
        display.asyncExec(new Runnable()
        {
            public void run()
            {
                getSite().getPage().closeEditor(Modeler.this, save);
            }
        });
    }

    /**
     * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
     */
    public void createPartControl(Composite parent)
    {
        try
        {
            super.createPartControl(parent);
        }
        catch (Throwable e)
        {
            ModelerPlugin.log(e);
            close(false);
            ModelerPlugin.displayDialog("Topcased Error", "Unable to open editor. Please check the input model.", IStatus.ERROR);

        }
    }

    /**
     * Delegates the method to the graphical viewer
     * 
     * @see org.eclipse.jface.viewers.ISelectionProvider#addSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
     */
    public void addSelectionChangedListener(ISelectionChangedListener listener)
    {
        getGraphicalViewer().addSelectionChangedListener(listener);
    }

    /**
     * Delegates the method to the graphical viewer
     * 
     * @see org.eclipse.jface.viewers.ISelectionProvider#getSelection()
     */
    public ISelection getSelection()
    {
        return getGraphicalViewer().getSelection();
    }

    /**
     * Delegates the method to the graphical viewer
     * 
     * @see org.eclipse.jface.viewers.ISelectionProvider#removeSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
     */
    public void removeSelectionChangedListener(ISelectionChangedListener listener)
    {
        getGraphicalViewer().removeSelectionChangedListener(listener);
    }

    /**
     * Delegates the method to the graphical viewer
     * 
     * @see org.eclipse.jface.viewers.ISelectionProvider#setSelection(org.eclipse.jface.viewers.ISelection)
     */
    public void setSelection(ISelection selection)
    {
        getGraphicalViewer().setSelection(selection);
    }

    /**
     * Get the preference store to use for this modeler.<br>
     * Clients should overrides this method to provide their specific modeler plugin preference store.<br>
     * The given preference store is used to store commons preferences as the 'Snap to geometry' , the 'Outline
     * sorters', the 'Outline filters', the 'Delete actions' options.<br>
     * Default implementation returns <code>null</code>.<br>
     * Constants to access this preferences can be accessed by the interface
     * {@link org.topcased.modeler.preferences.ModelerPreferenceConstants}.<br>
     * 
     * @return this modeler preference store.
     */
    public IPreferenceStore getPreferenceStore()
    {
        return null;
    }

    /**
     * @see org.eclipse.emf.edit.domain.IEditingDomainProvider#getEditingDomain()
     */
    public EditingDomain getEditingDomain()
    {
        EditingDomain emfEditingDomain = null;

        if (getEditDomain() instanceof MixedEditDomain)
        {
            emfEditingDomain = ((MixedEditDomain) getEditDomain()).getEMFEditingDomain();
        }

        return emfEditingDomain;
    }

    /**
     * @see org.eclipse.ui.ide.IGotoMarker#gotoMarker(org.eclipse.core.resources.IMarker)
     */
    public void gotoMarker(IMarker marker)
    {
        try
        {
            if (marker.getType().equals(EValidator.MARKER))
            {
                String uriAttribute = marker.getAttribute(EValidator.URI_ATTRIBUTE, null);
                if (uriAttribute != null)
                {
                    URI uri = URI.createURI(uriAttribute);
                    EObject eObject = getResourceSet().getEObject(uri, true);
                    if (eObject != null)
                    {
                        gotoEObject(eObject);
                    }
                }
            }
        }
        catch (CoreException exception)
        {
            ModelerPlugin.log(exception);
        }
    }

    /**
     * Sets the selection state for an editor to reveal the position of the given EObject.<br>
     * It searches in the different diagrams the occurrences of the EObject and ask the user to select the destination
     * if the given EObject has several graphical representation.
     * 
     * @param eObject the EObject to select
     */
    public void gotoEObject(EObject eObject)
    {
        if (eObject != null)
        {
            GraphElement[] elts = Utils.getGraphElements(getDiagrams(), eObject);
            if (elts.length > 0)
            {
                GraphElement graphElt = chooseGraphElement(elts);
                if (graphElt != null)
                {
                    setActiveDiagram(Utils.getDiagram(graphElt.getContainer()));
                    EditPart editPart = (EditPart) getGraphicalViewer().getEditPartRegistry().get(graphElt);
                    if (editPart != null)
                    {
                        // BUGFIX #344 Scroll to the selected object
                        ((GraphicalEditPart) getGraphicalViewer().getRootEditPart()).getFigure().validate();

                        getGraphicalViewer().reveal(editPart);
                        getGraphicalViewer().select(editPart);
                    }
                }
            }
        }
    }

    /**
     * Returns the destination GraphElement from a choices list
     * 
     * @param elements the list of available GraphElements
     * @return the selected GraphElement
     */
    private GraphElement chooseGraphElement(GraphElement[] elements)
    {
        GraphElement chosenElt = null;
        if (elements.length == 1)
        {
            chosenElt = elements[0];
        }
        else
        {
            ChooseGraphElementDialog dialog = new ChooseGraphElementDialog(getSite().getShell(), getEditDomain(), elements);

            if (dialog.open() == Window.OK)
            {
                Object[] selection = dialog.getResult();

                if (selection != null && selection.length > 0)
                {
                    chosenElt = (GraphElement) selection[0];
                }
            }
        }

        return chosenElt;
    }

    /**
     * Returns the manager for the different diagram configurations.
     * 
     * @return the manager
     */
    protected ModelerConfigurationManager getConfigurationManager()
    {
        if (configurationManager == null)
        {
            configurationManager = new ModelerConfigurationManager();
        }
        return configurationManager;
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor#getContributorId()
     */
    public String getContributorId()
    {
        return "org.topcased.modeler.editor.Modeler";
    }

    /**
     * Return the editorID associated with this Topcased modeler. This ID is the same given in the editor extension.
     * 
     * @return String an ID identifying the editor
     */
    public abstract String getId();
}