/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.editor.outline;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.FeatureMap;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.provider.IWrapperItemProvider;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.topcased.modeler.di.model.GraphElement;

/**
 * This Adapter listens to dragging operations and fill or clean the clipboard
 * with filtered datas. <br>
 * creation : 6 dec. 2004
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma </a>
 */
public class OutlineDragAdapter extends DragSourceAdapter
{
    private ISelectionProvider selectionProvider;

    private List transferData;

    /**
     * Constructs a new drag adapter.
     * 
     * @param provider the object that provide the selected object
     */
    public OutlineDragAdapter(ISelectionProvider provider)
    {
        selectionProvider = provider;
    }

    /**
     * Erase data from the clipboard object
     * 
     * @see org.eclipse.swt.dnd.DragSourceListener#dragFinished(org.eclipse.swt.dnd.DragSourceEvent)
     */
    public void dragFinished(DragSourceEvent event)
    {
        transferData = null;
        OutlineToDiagramTransfer.getInstance().setObject(null);
    }

    /**
     * @see org.eclipse.swt.dnd.DragSourceAdapter#dragSetData(org.eclipse.swt.dnd.DragSourceEvent)
     */
    public void dragSetData(DragSourceEvent event)
    {
        event.data = getSelection();
    }

    /**
     * Put selection on the clipboard object
     * 
     * @see org.eclipse.swt.dnd.DragSourceListener#dragStart(org.eclipse.swt.dnd.DragSourceEvent)
     */
    public void dragStart(DragSourceEvent event)
    {
        List data = getSelection();
        if (data.isEmpty())
        {
            // cancel drag
            event.doit = false;
        }
        OutlineToDiagramTransfer.getInstance().setObject(data);
    }

    /**
     * Compute the outline selection : filter on selection, only model elements
     * (displayed as GraphNode) can be dragged.
     * 
     * @return the list of selected model elements
     */
    protected List getSelection()
    {
        if (transferData != null)
        {
            return transferData;
        }

        IStructuredSelection selection = (IStructuredSelection) selectionProvider.getSelection();
        transferData = new ArrayList();
        Iterator it = selection.iterator();
        while (it.hasNext())
        {
            Object sel = it.next();

            // Ignore diagram objects
            if ((sel instanceof IWrapperItemProvider || sel instanceof FeatureMap.Entry || sel instanceof EObject)
                    && !(sel instanceof GraphElement))
            {
                transferData.add(AdapterFactoryEditingDomain.unwrap(sel));
            }
        }
        return transferData;
    }

}
