/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Thomas Friol (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.editor.outline.filters;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.ui.dialogs.ListSelectionDialog;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.extensions.OutlineManager;
import org.topcased.modeler.extensions.OutlineManager.FilterConfiguration;
import org.topcased.modeler.internal.ModelerImageRegistry;
import org.topcased.modeler.internal.ModelerPlugin;
import org.topcased.modeler.preferences.ModelerPreferenceConstants;

/**
 * A class defining an action to select the editor outline filters.<br>
 * Creation : 2 dc. 2005
 * 
 * @author <a href="mailto:thomas@anyware-tech.com">Thomas Friol</a>
 */
public class FiltersAction extends Action
{
    private TreeViewer outlineViewer;

    private Modeler modeler;

    public FiltersAction(TreeViewer outlineViewer, Modeler modeler)
    {
        super("Filters...", ModelerImageRegistry.getImageDescriptor("OUTLINE_FILTERS"));
        this.outlineViewer = outlineViewer;
        this.modeler = modeler;
    }

    /**
     * @see org.eclipse.jface.action.Action#run()
     */
    public void run()
    {
        ListSelectionDialog dialog = new ListSelectionDialog(ModelerPlugin.getActiveWorkbenchShell(),
                OutlineManager.getInstance().getFilters(modeler.getSite().getId()), new ArrayContentProvider(),
                new FiltersLabelProvider(), "Please select the filters to use in the outline.");

        IPreferenceStore ps = modeler.getPreferenceStore();
        Collection configs = getFilterConfigurations(ps != null ? ps.getString(ModelerPreferenceConstants.FILTERS_PREF)
                : "");
        dialog.setInitialSelections(configs.toArray());
        dialog.setTitle("Outline filters");
        dialog.open();

        Object[] filters = dialog.getResult();
        if (filters != null)
        {
            // Remove all previous filters
            for (Iterator it = configs.iterator(); it.hasNext();)
            {
                FilterConfiguration config = (FilterConfiguration) it.next();
                outlineViewer.removeFilter(config.getFilter());
            }

            // Add all new filters
            String newConfigs = "";
            for (int i = 0; i < filters.length; i++)
            {
                FilterConfiguration config = (FilterConfiguration) filters[i];

                outlineViewer.addFilter(config.getFilter());
                newConfigs += config.getId();
                if (i != filters.length - 1)
                {
                    newConfigs += ",";
                }
            }

            // Store the filters in the preferences
            if (ps != null)
            {
                ps.setValue(ModelerPreferenceConstants.FILTERS_PREF, newConfigs);
            }
        }
    }

    public static Collection getFilterConfigurations(String filtersList)
    {
        StringTokenizer st = new StringTokenizer(filtersList, ",");
        ArrayList configs = new ArrayList();
        while (st.hasMoreTokens())
        {
            String configId = st.nextToken();
            FilterConfiguration config = OutlineManager.getInstance().getFilterConfiguration(configId);
            if (config != null)
            {
                configs.add(config);
            }
        }

        return configs;
    }
}
