/*******************************************************************************
 * Copyright (c) 2005 Anyware Technologies
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies) - initial API and implementation
 *******************************************************************************/

package org.topcased.modeler.export.internal;

import java.awt.image.BufferedImage;

import org.apache.batik.transcoder.image.ImageTranscoder;
import org.apache.batik.transcoder.image.JPEGTranscoder;

/**
 * Export a diagram into a JPEG file
 * 
 * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class JPEGExporter extends AbstractAWTImageExporter
{

    /**
     * @see org.topcased.modeler.export.internal.AbstractAWTImageExporter#createImageTranscoder()
     */
    protected ImageTranscoder createImageTranscoder()
    {
        JPEGTranscoder imageTranscoder = new JPEGTranscoder();
        imageTranscoder.addTranscodingHint(JPEGTranscoder.KEY_QUALITY, new Float(0.8));
        return imageTranscoder;
    }

    /**
     * @see org.topcased.modeler.export.internal.AbstractAWTImageExporter#preprocessImage(java.awt.image.BufferedImage)
     */
    protected BufferedImage preprocessImage(BufferedImage image)
    {
        BufferedImage newImg = image;
        if (image.getType() != BufferedImage.TYPE_INT_RGB)
        {
            newImg = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.TYPE_INT_RGB);
            java.awt.Graphics g = newImg.getGraphics();
            g.drawImage(image, 0, 0, null);
            g.dispose();
        }
        return newImg;
    }
}
