/*******************************************************************************
 * Copyright (c) 2005 Anyware Technologies
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies) - initial API and implementation
 *******************************************************************************/

package org.topcased.modeler.export.internal;

import java.io.IOException;
import java.io.OutputStream;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gmf.runtime.draw2d.ui.render.awt.internal.svg.export.GraphicsSVG;
import org.topcased.modeler.export.ExportException;
import org.topcased.modeler.export.IImageExporter;

import com.sun.org.apache.xml.internal.serialize.OutputFormat;
import com.sun.org.apache.xml.internal.serialize.XMLSerializer;

/**
 * This class exports a figure into a svg file
 *
 * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class SVGExporter implements IImageExporter
{
    private static final String XML_ENCODING = "ISO-8859-1";

    /**
     * Save the given figure in the SVG format
     *
     * @param figure The figure to save
     * @param os The output stream
     * @throws ExportException
     */
    public void export(IFigure figure, OutputStream os) throws ExportException
    {
        if (figure instanceof Viewport)
        {
            // Reinit the figure
            ((Viewport) figure).setViewLocation(0, 0);
        }

        Dimension size = figure.getPreferredSize();
        GraphicsSVG graphics = GraphicsSVG.getInstance(new Rectangle(0, 0, size.width, size.height));

        figure.paint(graphics);

        // Sets the XML encoding
        OutputFormat format = new OutputFormat();
        format.setEncoding(XML_ENCODING);
        format.setIndenting(true);

        // Serializes the document
        XMLSerializer serializer = new XMLSerializer(os, format);
        try
        {
            serializer.serialize(graphics.getRoot());
        }
        catch (IOException e)
        {
            throw new ExportException("Error during the SVG export", e);
        }
        finally
        {
            graphics.dispose();
        }
    }
}
