/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.internal.actions;

import java.util.Collections;

import org.eclipse.gef.ui.actions.Clipboard;
import org.eclipse.gef.ui.actions.WorkbenchPartAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionFactory;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.internal.ModelerImageRegistry;

/**
 * 
 * Copy into the Clipboard the selected object
 * 
 * @author jako
 */
public class CopyAction extends WorkbenchPartAction implements ISelectionChangedListener
{

    private Object template;

    /**
     * Constructs a new CopyTemplateAction. You must manually add this action to
     * the palette viewer's list of selection listeners. Otherwise, this
     * action's enabled state won't be updated properly.
     * 
     * @param part the workbench part
     */
    public CopyAction(IWorkbenchPart part)
    {
        super(part);
    }

    /**
     * Returns whether the selected EditPart is a GraphNodeEditPart.
     * 
     * @return whether the selected EditPart is a GraphNodeEditPart
     */
    protected boolean calculateEnabled()
    {
        if (!(template instanceof GraphNodeEditPart))
        {
            return false;
        }

        return template != null;
    }

    /**
     * Disposes the Clipboard contents when it is no longer needed
     * 
     * @see org.eclipse.gef.ui.actions.EditorPartAction#dispose()
     */
    public void dispose()
    {
        Clipboard.getDefault().setContents(Collections.EMPTY_LIST);
        template = null;
    }

    /**
     * Initializes the copy action
     * 
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
     */
    protected void init()
    {
        setId(ActionFactory.COPY.getId());
        setText("Copy");
        setAccelerator(SWT.CONTROL | 'C');
        setImageDescriptor(ModelerImageRegistry.getImageDescriptor("COPY"));
    }

    /**
     * Sets the default {@link Clipboard Clipboard's}contents to be the
     * currently selected template.
     */
    public void run()
    {
        Clipboard.getDefault().setContents(((GraphNodeEditPart) template).getModel());
    }

    /**
     * Sets the selected EditPart and refreshes the enabled state of this
     * action.
     * 
     * @see ISelectionChangedListener#selectionChanged(SelectionChangedEvent)
     */
    public void selectionChanged(SelectionChangedEvent event)
    {
        ISelection s = event.getSelection();
        if (!(s instanceof IStructuredSelection))
        {
            return;
        }
        IStructuredSelection selection = (IStructuredSelection) s;
        template = null;

        if (selection != null && selection.size() == 1)
        {
            Object obj = selection.getFirstElement();
            if (obj instanceof GraphNodeEditPart)
            {
                GraphNodeEditPart ed = (GraphNodeEditPart) obj;
                template = ed;

            }
        }

        refresh();

    }

}
