/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.internal.actions;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.topcased.modeler.commands.CreateDiagramCommand;
import org.topcased.modeler.commands.CreateDiagramsCommand;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.extensions.DiagramDescriptor;
import org.topcased.modeler.extensions.ICreationDiagram;

/**
 * Create a new Diagram linked with a model object.<br>
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class CreateDiagramAction extends Action
{
    private DiagramDescriptor diagramDescriptor;

    private EObject object;

    private Modeler editor;

    private boolean initializeContent;

    private Command creationCommand;

    /**
     * Constructor
     * 
     * @param ed the Modeler object
     * @param obj the model object associated with the diagram
     * @param diagram the DiagramDescriptor associated with the diagram to create
     */
    public CreateDiagramAction(Modeler ed, EObject obj, DiagramDescriptor diagram)
    {
        this(ed, obj, diagram, false);
    }

    /**
     * Constructor
     * 
     * @param ed the Modeler object
     * @param obj the model object associated with the diagram
     * @param diagram the DiagramDescriptor associated with the diagram to create
     * @param initialize if the children of the model objects must be imported graphically in the diagram
     */
    public CreateDiagramAction(Modeler ed, EObject obj, DiagramDescriptor diagram, boolean initialize)
    {
        super(diagram.getName());
        this.editor = ed;
        this.object = obj;
        this.diagramDescriptor = diagram;
        this.initializeContent = initialize;
        this.creationCommand = diagramDescriptor.getCreationCommand(object);

        ImageDescriptor icon = this.diagramDescriptor.getIconDescriptor();
        if (icon != null)
        {
            setImageDescriptor(icon);
        }
    }

    /**
     * Execute the Action
     * 
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run()
    {
        if (creationCommand == null)
        {
            // The CreateDiagramsCommand should be first executed before the CreateDiagramCommand : indeed, the Diagrams
            // element should be first created when it does not exist yet.
            ((CommandStack) editor.getAdapter(CommandStack.class)).execute(new CreateDiagramsCommand(editor, object));
            ((CommandStack) editor.getAdapter(CommandStack.class)).execute(new CreateDiagramCommand(editor, object, diagramDescriptor, initializeContent));
        }
        else if (creationCommand instanceof ICreationDiagram)
        {
            EObject realObject = ((ICreationDiagram) creationCommand).getLinkedElementWithDiagram(object);
            ((CommandStack) editor.getAdapter(CommandStack.class)).execute(creationCommand);
            ((CommandStack) editor.getAdapter(CommandStack.class)).execute(new CreateDiagramsCommand(editor, realObject));
            ((CommandStack) editor.getAdapter(CommandStack.class)).execute(new CreateDiagramCommand(editor, realObject, diagramDescriptor, initializeContent));
        }
    }
}
