/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Thomas Friol (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.internal.actions;

import org.eclipse.gef.ui.actions.DeleteAction;
import org.eclipse.ui.IWorkbenchPart;
import org.topcased.modeler.dialogs.ConfirmationDialog;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.internal.ModelerPlugin;
import org.topcased.modeler.preferences.ModelerPreferenceConstants;

/**
 * Redefines the GEF delete action to prompt a confirmation dialog.<br>
 * 
 * Creation : 12 dec. 2005
 * 
 * @author <a href="mailto:thomas@anyware-tech.com">Thomas Friol</a>
 */
public class DeleteGraphElementAction extends DeleteAction
{

    /**
     * Constructor
     * 
     * @param part
     */
    public DeleteGraphElementAction(IWorkbenchPart part)
    {
        super(part);
    }

    /**
     * @see org.eclipse.gef.ui.actions.DeleteAction#run()
     */
    public void run()
    {
        Modeler modeler = (Modeler) getWorkbenchPart();
        ConfirmationDialog dialog = new ConfirmationDialog(ModelerPlugin.getActiveWorkbenchShell(), "Delete From Diagram",
                "Are you sure you want to delete these graphical elements and not the corresponding model elements ?", modeler.getPreferenceStore(), ModelerPreferenceConstants.DELETE_ACTION_CONFIRM);
        int result = dialog.open();
        if (result == 0)
        {
            super.run();
        }
    }

    /**
     * @see org.eclipse.gef.ui.actions.DeleteAction#init()
     */
    protected void init()
    {
        super.init();
        setText("Delete From Diagram");
        setImageDescriptor(ModelerPlugin.getImageDescriptor("icons/deleteFromDiagram.gif"));
    }
}
