/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies)
 *    - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.internal.actions;

import org.eclipse.gef.ui.actions.WorkbenchPartAction;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPart;
import org.topcased.modeler.ModelerActionConstants;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.internal.ModelerImageRegistry;
import org.topcased.validation.ui.actions.ValidateAction;

/**
 * Validate a list of EObjects with the constraints from the metamodel. <br>
 * creation : 11 janv. 2006
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class ModelerValidateAction extends WorkbenchPartAction
{

    /**
     * The Constructor
     * 
     * @param part the IWorkbenchPart
     */
    public ModelerValidateAction(IWorkbenchPart part)
    {
        super(part);
    }

    /**
     * Validate a list of EObjects
     * 
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run()
    {
        IWorkbenchPart part = getWorkbenchPart();
        if (part instanceof Modeler)
        {
            Modeler modeler = (Modeler) part;
            IFileEditorInput input = (IFileEditorInput)modeler.getEditorInput();
            ValidateAction action = new ValidateAction(input.getFile(), modeler.getDiagrams().getModel());
            action.run();
        }
    }

    /**
     * Determine if the action must appear in the context menu
     * 
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled()
    {
        return true;
    }

    /**
     * Initializes the paste action
     * 
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
     */
    protected void init()
    {
        setId(ModelerActionConstants.VALIDATE);
        setText("Model validation");
        setImageDescriptor(ModelerImageRegistry.getImageDescriptor("VALIDATE"));
    }

}
