/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.internal.actions;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.gef.ui.actions.Clipboard;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionFactory;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.internal.ModelerImageRegistry;
import org.topcased.modeler.internal.ModelerPlugin;

/**
 * Create and send paste order with the new object to copy
 * 
 * @author jako
 */
public class PasteAction extends SelectionAction
{
    /**
     * Paste the contents from the Clipboard to current selection
     * 
     * @param part the part associated with this action
     */
    public PasteAction(IWorkbenchPart part)
    {
        super(part);
    }

    /**
     * Returns <code>true</code> if the {@link Clipboard clipboard's}contents
     * are not <code>null</code>.
     * 
     * @return true if the paste action can be executed
     */
    protected boolean calculateEnabled()
    {
        Object contents = Clipboard.getDefault().getContents();
        
        return contents != null && contents instanceof GraphNode;
    }

//    /**
//     * Creates and returns a command (which may be <code>null</code>) to
//     * create a new EditPart based on the template on the clipboard. and
//     * translate the new object
//     * 
//     * @return the paste command
//     */
//    protected Command createPasteCommand()
//    {
//        if (getSelectedObjects() == null || getSelectedObjects().isEmpty())
//            return null;
//
//        // get the parent object
//        Object parentObject = getSelectedObjects().get(0);
//
//        // get the request to create
//        CreateRequest request = new CreateRequest();
//
//        // get the object to paste
//        GraphNode graphNode = (GraphNode) Clipboard.getDefault().getContents();
//
//        // set the size, location and factory to use for the object
//        request.setSize(graphNode.getSize());
//        request.setLocation(graphNode.getPosition().getTranslated(OFFSET_X, OFFSET_Y));
//        request.setFactory(getFactory(graphNode));
//
//        if (parentObject instanceof EditPart)
//        {
//            return ((EditPart) parentObject).getCommand(request);
//        }
//        return null;
//    }
//
//    /**
//     * Returns the appropriate Factory object to be used for the specified
//     * graphElt. This Factory is used on the CreateRequest that is sent to the
//     * target EditPart.
//     * 
//     * @param graphElt the graphElement
//     * @return a Factory
//     */
//    protected CreationFactory getFactory(Object graphElt)
//    {
//        if (graphElt instanceof GraphElement)
//            return new GraphElementCreationFactory((GraphElement) graphElt);
//
//        return null;
//    }

    /**
     * Initializes the paste action
     * 
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
     */
    protected void init()
    {
        setId(ActionFactory.PASTE.getId());
        setText("Paste");
        setAccelerator(SWT.CONTROL | 'V');
        setImageDescriptor(ModelerImageRegistry.getImageDescriptor("PASTE"));
    }

    /**
     * Executes the command.
     */
    public void run()
    {
        ModelerPlugin.displayDialog("Warning", "This action is not yet implemented.", IStatus.WARNING);
//        execute(createPasteCommand());
        // Modeler modeler = (Modeler)
        // ModelerPlugin.getActiveWorkbenchWindow().getActivePage().getActiveEditor();
        // modeler.selectObject(modeler.getLastCreatedObject());

    }

}
