/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.internal.preferences;

import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.core.runtime.preferences.DefaultScope;
import org.osgi.service.prefs.Preferences;
import org.topcased.modeler.internal.ModelerPlugin;
import org.topcased.modeler.preferences.ModelerPreferenceConstants;

/**
 * This class contributes to the
 * <code>org.eclipse.core.runtime.preferences</code> extension point.
 * 
 * Creation : 1 aout 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class ModelerPreferenceInitializer extends AbstractPreferenceInitializer
{
    /**
     * Initializes a preference store with default preference values for the
     * Modeler plug-in.
     * 
     * @see org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer#initializeDefaultPreferences()
     */
    public void initializeDefaultPreferences()
    {
        Preferences store = new DefaultScope().getNode(ModelerPlugin.getId());

        store.putBoolean(ModelerPreferenceConstants.P_USE_ANTIALIAS, false);
        store.putBoolean(ModelerPreferenceConstants.P_USE_UUID, false);
        store.putBoolean(ModelerPreferenceConstants.P_CHECK_INTEGRITY, true);
        store.putBoolean(ModelerPreferenceConstants.P_INITIALIZE_DIAGRAM, false);

        store.put(ModelerPreferenceConstants.P_PAGE_FORMATS, ModelerPreferenceConstants.DEFAULT_PAGE_FORMATS);
        store.put(ModelerPreferenceConstants.P_DEFAULT_PAGE_FORMAT, ModelerPreferenceConstants.DEFAULT_PAGE_FORMAT);
        store.put(ModelerPreferenceConstants.P_PAGE_MARGINS, ModelerPreferenceConstants.DEFAULT_PAGE_MARGINS);
        store.put(ModelerPreferenceConstants.P_DEFAULT_PAGE_MARGIN, ModelerPreferenceConstants.DEFAULT_PAGE_MARGIN);
        store.put(ModelerPreferenceConstants.P_DEFAULT_UNIT, ModelerPreferenceConstants.DEFAULT_UNIT);
        store.put(ModelerPreferenceConstants.P_ORIENTATION, ModelerPreferenceConstants.DEFAULT_ORIENTATION);
    }
}
