/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.sam.actions;

import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.WorkbenchPartAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.ui.IWorkbenchPart;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.sam.SAMActionConstants;
import org.topcased.modeler.sam.SAMRequestConstants;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.SynchronisationGate;

/**
 * created 13 mai 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 * 
 */
public class AddNewPortAction extends WorkbenchPartAction implements ISelectionChangedListener
{
    /**
     * The selected EditPart object
     */
    private EMFGraphNodeEditPart template;
    
    /**
     * @param part
     */
    public AddNewPortAction(IWorkbenchPart part)
    {
        super(part);
    }
    
    /**
     * Add a new diagram
     * 
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run()
    {
        // construct the AddNewPortRequest
        Request request = new Request(SAMRequestConstants.REQ_ADD_NEW_PORT);
        
        // get the Command
        Command actionCommand = template.getCommand(request);
        
        // execute the command
        getCommandStack().execute(actionCommand);
    }
    
    /**
     * Determine if the action must appear in the context menu
     * 
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
     */
    protected boolean calculateEnabled()
    {
        if (template == null)
        {
            return false;
        }
        
        // construct the AddNewPortRequest
        Request request = new Request(SAMRequestConstants.REQ_ADD_NEW_PORT);
        
        // return true if the EditPart can understand the Request
        return template.understandsRequest(request);
    }
    
    /**
     * Initializes the delete model object action
     * 
     * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
     */
    protected void init()
    {
        setId(SAMActionConstants.ADD_NEW_PORT);
        setText("Add New Port");
    }
    
    /**
     * Sets the selected EditPart and refreshes the enabled state of this
     * action.
     * 
     * @param event
     * 
     * @see ISelectionChangedListener#selectionChanged(SelectionChangedEvent)
     */
    public void selectionChanged(SelectionChangedEvent event)
    {
        ISelection s = event.getSelection();
        if (!(s instanceof IStructuredSelection))
            return;
        IStructuredSelection selection = (IStructuredSelection) s;
        template = null;
        
        if (selection != null && selection.size() == 1)
        {
            Object obj = selection.getFirstElement();
            if (obj instanceof EMFGraphNodeEditPart)
            {
                EMFGraphNodeEditPart emfGraphNodeEditPart = (EMFGraphNodeEditPart) obj;
                
                // the action is available only for Composition and
                // Decomposition model objects
                if (Utils.getElement((GraphNode) emfGraphNodeEditPart.getModel()) instanceof SynchronisationGate)
                {
                    template = emfGraphNodeEditPart;
                }
            }
        }
        
        refresh();
    }
    
}
