/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.automatondiagram;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.gef.EditPartFactory;
import org.topcased.modeler.editor.EditPart2ModelAdapterFactory;
import org.topcased.modeler.editor.IConfiguration;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.IPaletteManager;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.sam.SAMPlugin;
import org.topcased.modeler.sam.automatondiagram.edit.AbstractStateEditPart;
import org.topcased.modeler.sam.automatondiagram.edit.InitialStateEditPart;
import org.topcased.modeler.sam.automatondiagram.edit.MacroStateEditPart;
import org.topcased.modeler.sam.automatondiagram.edit.StateEditPart;
import org.topcased.modeler.sam.automatondiagram.edit.TransitionEditPart;
import org.topcased.modeler.sam.edit.InControlPortEditPart;
import org.topcased.modeler.sam.edit.InDataPortEditPart;
import org.topcased.modeler.sam.edit.MultiPortEditPart;
import org.topcased.modeler.sam.edit.OutControlPortEditPart;
import org.topcased.modeler.sam.edit.OutDataPortEditPart;

/**
 * A diagram configuration : manages Palette, EditPartFactory for this diagram.
 * <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class AutomConfiguration implements IConfiguration
{
    
    private AutomPaletteManager paletteManager;
    
    private AutomEditPartFactory editPartFactory;
    
    private AutomCreationUtils creationUtils;
    
    /**
     * The DiagramGraphConf that contains graphical informations on the
     * configuration
     */
    private DiagramGraphConf diagramGraphConf;
    
    /**
     * Constructor. Initialize Adapter factories.
     * 
     * @generated
     */
    public AutomConfiguration()
    {
        registerAdapters();
    }
    
    /**
     * Registers the Adapter Factories for all the EditParts
     * 
     * @generated
     */
    private void registerAdapters()
    {
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(StateEditPart.class, org.topcased.sam.State.class),
                StateEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(InitialStateEditPart.class, org.topcased.sam.InitialState.class),
                InitialStateEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(MacroStateEditPart.class, org.topcased.sam.MacroState.class),
                MacroStateEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(MultiPortEditPart.class, org.topcased.sam.MultiPort.class),
                MultiPortEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(InControlPortEditPart.class, org.topcased.sam.InControlPort.class),
                InControlPortEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(OutControlPortEditPart.class, org.topcased.sam.OutControlPort.class),
                OutControlPortEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(InDataPortEditPart.class, org.topcased.sam.InDataPort.class),
                InDataPortEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(OutDataPortEditPart.class, org.topcased.sam.OutDataPort.class),
                OutDataPortEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(TransitionEditPart.class, org.topcased.sam.Transition.class),
                TransitionEditPart.class);
        Platform.getAdapterManager().registerAdapters(
                new EditPart2ModelAdapterFactory(AbstractStateEditPart.class, org.topcased.sam.AbstractState.class),
                AbstractStateEditPart.class);
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getId()
     * @generated
     */
    public String getId()
    {
        return new String("org.topcased.modeler.sam.automatondiagram");
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getName()
     * @generated
     */
    public String getName()
    {
        return new String("Automaton Diagram");
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getEditPartFactory()
     * @generated
     */
    public EditPartFactory getEditPartFactory()
    {
        if (editPartFactory == null)
            editPartFactory = new AutomEditPartFactory();
        
        return editPartFactory;
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getPaletteManager()
     * @generated
     */
    public IPaletteManager getPaletteManager()
    {
        if (paletteManager == null)
            paletteManager = new AutomPaletteManager(getCreationUtils());
        
        return paletteManager;
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getCreationUtils()
     * @generated
     */
    public ICreationUtils getCreationUtils()
    {
        if (creationUtils == null)
            creationUtils = new AutomCreationUtils(getDiagramGraphConf());
        
        return creationUtils;
    }
    
    /**
     * @see org.topcased.modeler.editor.IConfiguration#getDiagramGraphConf()
     */
    public DiagramGraphConf getDiagramGraphConf()
    {
        if (diagramGraphConf == null)
        {
            URI fileURI = URI.createURI(SAMPlugin.getDefault().getBundle().getResource(
                    "org/topcased/modeler/sam/automatondiagram/diagram.graphconf").toString());
            ResourceSet resourceSet = new ResourceSetImpl();
            Resource resource = resourceSet.getResource(fileURI, true);
            if (resource != null && resource.getContents().get(0) instanceof DiagramGraphConf)
            {
                diagramGraphConf = (DiagramGraphConf) resource.getContents().get(0);
            }
        }
        
        return diagramGraphConf;
    }
    
}
