/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.automatondiagram;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.sam.automatondiagram.edit.AbstractStateEditPart;
import org.topcased.modeler.sam.automatondiagram.edit.AutomDiagramEditPart;
import org.topcased.modeler.sam.automatondiagram.edit.InitialStateEditPart;
import org.topcased.modeler.sam.automatondiagram.edit.MacroStateEditPart;
import org.topcased.modeler.sam.automatondiagram.edit.StateEditPart;
import org.topcased.modeler.sam.automatondiagram.edit.TransitionEditPart;
import org.topcased.modeler.sam.edit.InControlPortEditPart;
import org.topcased.modeler.sam.edit.InDataPortEditPart;
import org.topcased.modeler.sam.edit.MultiPortEditPart;
import org.topcased.modeler.sam.edit.OutControlPortEditPart;
import org.topcased.modeler.sam.edit.OutDataPortEditPart;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.util.SAMSwitch;

/**
 * Part Factory : associates a model object to its controller. <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class AutomEditPartFactory implements EditPartFactory
{
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
     * @generated
     */
    public EditPart createEditPart(EditPart context, Object model)
    {
        if (model instanceof Diagram)
        {
            return new AutomDiagramEditPart((Diagram) model);
        }
        else if (model instanceof GraphNode)
        {
            final GraphNode node = (GraphNode) model;
            EObject element = Utils.getElement(node);
            if (element != null)
            {
                if ("http://www.topcased.org/SAM/1.0".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new NodeSAMSwitch(node).doSwitch(element);
                }
                return new EMFGraphNodeEditPart(node);
            }
            return new GraphNodeEditPart(node);
        }
        else if (model instanceof GraphEdge)
        {
            final GraphEdge edge = (GraphEdge) model;
            EObject element = Utils.getElement(edge);
            if (element != null)
            {
                if ("http://www.topcased.org/SAM/1.0".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new EdgeSAMSwitch(edge).doSwitch(element);
                }
                return new EMFGraphEdgeEditPart(edge);
            }
            return new GraphEdgeEditPart(edge);
        }
        
        throw new IllegalStateException("No edit part matches with the '" + model.getClass().getName()
                + "' model element. Check AutomEditPartFactory#createEditPart(EditPart,Object) class");
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class NodeSAMSwitch extends SAMSwitch<Object>
    {
        /** The graphical node */
        private GraphNode node;
        
        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param node the graphical node
         * @generated
         */
        public NodeSAMSwitch(GraphNode node)
        {
            this.node = node;
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseState(org.topcased.sam.State)
         * @generated
         */
        public Object caseState(org.topcased.sam.State object)
        {
            return new StateEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseInitialState(org.topcased.sam.InitialState)
         * @generated
         */
        public Object caseInitialState(org.topcased.sam.InitialState object)
        {
            return new InitialStateEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseMacroState(org.topcased.sam.MacroState)
         * @generated
         */
        public Object caseMacroState(org.topcased.sam.MacroState object)
        {
            return new MacroStateEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseMultiPort(org.topcased.sam.MultiPort)
         * @generated
         */
        public Object caseMultiPort(org.topcased.sam.MultiPort object)
        {
            return new MultiPortEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseInControlPort(org.topcased.sam.InControlPort)
         * @generated
         */
        public Object caseInControlPort(org.topcased.sam.InControlPort object)
        {
            return new InControlPortEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseOutControlPort(org.topcased.sam.OutControlPort)
         * @generated
         */
        public Object caseOutControlPort(org.topcased.sam.OutControlPort object)
        {
            return new OutControlPortEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseInDataPort(org.topcased.sam.InDataPort)
         * @generated
         */
        public Object caseInDataPort(org.topcased.sam.InDataPort object)
        {
            return new InDataPortEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseOutDataPort(org.topcased.sam.OutDataPort)
         * @generated
         */
        public Object caseOutDataPort(org.topcased.sam.OutDataPort object)
        {
            return new OutDataPortEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseAbstractState(org.topcased.sam.AbstractState)
         * @generated
         */
        public Object caseAbstractState(org.topcased.sam.AbstractState object)
        {
            return new AbstractStateEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphNodeEditPart(node);
        }
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class EdgeSAMSwitch extends SAMSwitch<Object>
    {
        /** The graphical edge */
        private GraphEdge edge;
        
        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param edge the graphical edge
         * @generated
         */
        public EdgeSAMSwitch(GraphEdge edge)
        {
            this.edge = edge;
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseTransition(org.topcased.sam.Transition)
         * @generated
         */
        public Object caseTransition(org.topcased.sam.Transition object)
        {
            return new TransitionEditPart(edge);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphEdgeEditPart(edge);
        }
    }
    
}
