/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.automatondiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.AbstractState;
import org.topcased.sam.State;
import org.topcased.sam.Transition;

/**
 * <br>
 * creation : 30 juin 2005
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class StateRestoreConnectionCommand extends AbstractStateRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public StateRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated NOT
     */
    protected void initializeCommands()
    {
        
        super.initializeCommands();
        
        GraphElement node = getGraphElement();
        EObject nodeObject = Utils.getElement(node);
        
        if (nodeObject instanceof State)
        {
            Iterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement
                        && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(node);
                    GraphElement node2 = (GraphElement) obj;
                    EObject nodeObject2 = Utils.getElement(node2);
                    
                    if (nodeObject2 instanceof AbstractState)
                    {
                        if (autoRef)
                        {
                            createTransitionFromAbstractStateToState(node, node);
                        }
                        else
                        {
                            // if node is the target of the edge or if it is the
                            // source and that the SourceTargetCouple is
                            // reversible
                            createTransitionFromAbstractStateToState(node2, node);
                        }
                    }
                    
                }
            }
        }
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated
     */
    private void createTransitionFromAbstractStateToState(GraphElement srcNode, GraphElement targetNode)
    {
        AbstractState sourceObject = (AbstractState) Utils.getElement(srcNode);
        State targetObject = (State) Utils.getElement(targetNode);
        
        EList edgeObjectList = ((org.topcased.sam.Automaton) Utils.getDiagramModelObject(srcNode)).getListTransitions();
        for (Iterator it = edgeObjectList.iterator(); it.hasNext();)
        {
            Object obj = it.next();
            if (obj instanceof Transition)
            {
                Transition edgeObject = (Transition) obj;
                if (targetObject.equals(edgeObject.getDest()) && sourceObject.equals(edgeObject.getSource())
                        && sourceObject.getOutlink().contains(edgeObject)
                        && targetObject.getInlink().contains(edgeObject))
                {
                    // check if the relation does not exists yet
                    List existing = getExistingEdges(srcNode, targetNode, Transition.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            TransitionEdgeCreationCommand cmd = new TransitionEdgeCreationCommand(getEditDomain(),
                                    (GraphEdge) edge, srcNode, false);
                            cmd.setTarget(targetNode);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }
    
}
