/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.automatondiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.DiagramEditPart;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.sam.automatondiagram.figures.AutomDiagramFigure;
import org.topcased.modeler.sam.edit.policies.SAMDiagramLayoutEditPolicy;
import org.topcased.modeler.sam.systemdiagram.commands.RestorePortsCommand;
import org.topcased.sam.Automaton;
import org.topcased.sam.SAMPackage;

/**
 * created 12 avr. 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 * 
 */
public class AutomDiagramEditPart extends DiagramEditPart
{
    /**
     * The Constructor
     * 
     * @param model
     */
    public AutomDiagramEditPart(Diagram model)
    {
        super(model);
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.DiagramEditPart#getLayoutEditPolicy()
     * @generated NOT
     */
    protected EditPolicy getLayoutEditPolicy()
    {
        return new SAMDiagramLayoutEditPolicy();
    }
    
    /**
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     */
    protected IFigure createFigure()
    {
        IFigure page = super.createFigure();
        
        // REV weird code ....use a command ?
        RestorePortsCommand restorePortsCommand = new RestorePortsCommand(getViewer().getEditDomain(),
                (GraphNode) getModel(),
                ((ModelerGraphicalViewer) getViewer()).getModelerEditor().getActiveConfiguration().getCreationUtils());
        if (restorePortsCommand != null && restorePortsCommand.canExecute())
        {
            restorePortsCommand.execute();
        }
        
        return page;
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.DiagramEditPart#createBodyFigure()
     * @generated
     */
    protected IFigure createBodyFigure()
    {
        return new AutomDiagramFigure();
    }
    
    /**
     * @see org.topcased.modeler.edit.DiagramEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {
        if (msg.getNotifier() instanceof Automaton)
        {
            switch (msg.getFeatureID(Automaton.class))
            {
                case SAMPackage.AUTOMATON__LIST_PORTS:
                case SAMPackage.AUTOMATON__LIST_MULTI_PORT:
                    // update graphically all the ports that may have been added
                    // in the model so that they are synchronized
                    // REV weird code ....use a command ?
                    RestorePortsCommand restorePortsCommand = new RestorePortsCommand(
                            getViewer().getEditDomain(),
                            (GraphNode) getModel(),
                            ((ModelerGraphicalViewer) getViewer()).getModelerEditor().getActiveConfiguration().getCreationUtils());
                    if (restorePortsCommand != null && restorePortsCommand.canExecute())
                    {
                        restorePortsCommand.execute();
                    }
                    break;
            }
        }
        
        super.handleModelChanged(msg);
    }
}
