/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.automatondiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.PolylineDecoration;
import org.topcased.draw2d.figures.Label;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.policies.EdgeObjectOffsetEditPolicy;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.sam.SAMEditPolicyConstants;
import org.topcased.modeler.sam.automatondiagram.AutomEdgeObjectConstants;
import org.topcased.modeler.sam.automatondiagram.figures.TransitionFigure;
import org.topcased.modeler.sam.automatondiagram.policies.TransitionEdgeObjectUVEditPolicy;
import org.topcased.modeler.sam.edit.policies.CustomEditPolicy;
import org.topcased.sam.Transition;

/**
 * Transition controller <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class TransitionEditPart extends EMFGraphEdgeEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param model the graph object
     * @generated
     */
    public TransitionEditPart(GraphEdge model)
    {
        super(model);
    }
    
    private Transition getTransition()
    {
        return (Transition) getEObject();
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();
        
        installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_UV_EDITPOLICY, new TransitionEdgeObjectUVEditPolicy());
        installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_OFFSET_EDITPOLICY, new EdgeObjectOffsetEditPolicy());
        
        installEditPolicy(SAMEditPolicyConstants.CUSTOM_EDITPOLICY, new CustomEditPolicy());
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Figure
     * @generated
     */
    protected IFigure createFigure()
    {
        TransitionFigure connection = new TransitionFigure();
        
        createTargetDecoration(connection);
        
        return connection;
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param connection the PolylineConnection
     * @generated
     */
    private void createTargetDecoration(PolylineConnection connection)
    {
        
        PolylineDecoration decoration = new PolylineDecoration();
        decoration.setScale(10, 5);
        connection.setTargetDecoration(decoration);
        
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#refreshEdgeObjects()
     * @generated
     */
    protected void refreshEdgeObjects()
    {
        super.refreshEdgeObjects();
        updateConditionLabel();
        updatePriorityLabel();
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the condition Label
     * 
     * @generated NOT
     */
    private void updateConditionLabel()
    {
        ((Label) ((TransitionFigure) getFigure()).getConditionEdgeObjectFigure()).setText(computeTransitionText());
    }
    
    /**
     * Compute the String to associate with the transition
     * 
     * @return the condition/emission text
     */
    private String computeTransitionText()
    {
        if (getTransition().getCondition() != null && !"".equals(getTransition().getCondition())
                && (getTransition().getEmission() == null || "".equals(getTransition().getEmission())))
        {
            return "? " + getTransition().getCondition();
        }
        else if ((getTransition().getCondition() == null || "".equals(getTransition().getCondition()))
                && getTransition().getEmission() != null && !"".equals(getTransition().getEmission()))
        {
            return "! " + getTransition().getEmission();
        }
        else if (getTransition().getCondition() != null && !"".equals(getTransition().getCondition())
                && getTransition().getEmission() != null && !"".equals(getTransition().getEmission()))
        {
            return "? " + getTransition().getCondition() + "\n! " + getTransition().getEmission();
        }
        else
        {
            return "";
        }
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the priority Label
     * 
     * @generated NOT
     */
    private void updatePriorityLabel()
    {
        Label srcCount = (Label) ((TransitionFigure) getFigure()).getPriorityEdgeObjectFigure();
        
        // TODO : display it only if several transitions from a same source
        // state
        if (getTransition().getPriority() != null)
        {
            if (getTransition().getPriority().intValue() < 1)
            {
                srcCount.setText("");
            }
            else
            {
                srcCount.setText(getTransition().getPriority().toString());
            }
        }
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#getEdgeObjectFigure(org.topcased.modeler.di.model.EdgeObject)
     * @generated
     */
    public IEdgeObjectFigure getEdgeObjectFigure(EdgeObject edgeObject)
    {
        if (AutomEdgeObjectConstants.CONDITION_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((TransitionFigure) getFigure()).getConditionEdgeObjectFigure();
        }
        if (AutomEdgeObjectConstants.PRIORITY_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((TransitionFigure) getFigure()).getPriorityEdgeObjectFigure();
        }
        return null;
    }
    
    // TODO Remove that code
//    /**
//     * <!-- begin-user-doc --> <!-- end-user-doc -->
//     * 
//     * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
//     * @generated NOT
//     */
//    public void performRequest(Request request)
//    {
//        if (request.getType() == RequestConstants.REQ_OPEN)
//        {
//            TransitionEditDialog transitionDlg = new TransitionEditDialog(getTransition(),
//                    ModelerPlugin.getActiveWorkbenchShell());
//            if (transitionDlg.open() == Window.OK)
//            {
//                UpdateTransitionCommand command = new UpdateTransitionCommand(getTransition(), transitionDlg.getData());
//                getViewer().getEditDomain().getCommandStack().execute(command);
//            }
//        }
//        else
//        {
//            super.performRequest(request);
//        }
//    }
    
}
