/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.sam.automatondiagram.policies;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.sam.automatondiagram.commands.TransitionEdgeCreationCommand;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.AbstractState;
import org.topcased.sam.State;
import org.topcased.sam.Transition;

/**
 * Provides the EditPolicies which make the command when the user make a
 * transition between two States
 * 
 * Created 14 June 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 * @deprecated Use the TransitionEdgeCreationEditPolicy
 */
public class TransitionCreationEditPolicy extends GraphicalNodeEditPolicy
{
    
    /**
     * Return the command associated with the target element
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
     */
    protected Command getConnectionCompleteCommand(CreateConnectionRequest request)
    {
        // check if the command created from the source node is an command to
        // create a Transition
        if (request.getStartCommand() instanceof TransitionEdgeCreationCommand)
        {
            // get the targetNode of the connection
            GraphNode targetNode = (GraphNode) getHost().getModel();
            GraphNode sourceNode = (GraphNode) request.getSourceEditPart().getModel();
            
            if (Utils.getElement(sourceNode) instanceof AbstractState && Utils.getElement(targetNode) instanceof State)
            {
                TransitionEdgeCreationCommand command = (TransitionEdgeCreationCommand) request.getStartCommand();
                if (command != null)
                    // set the target node
                    command.setTarget(targetNode);
                
                return command;
            }
        }
        return null;
    }
    
    /**
     * Changes the request with the source element of the command
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
     */
    protected Command getConnectionCreateCommand(CreateConnectionRequest request)
    {
        Object newObject = request.getNewObject();
        if (newObject instanceof GraphEdge)
        {
            GraphEdge edge = (GraphEdge) newObject;
            GraphNode sourceNode = (GraphNode) getHost().getModel();
            if (Utils.getElement(edge) instanceof Transition && Utils.getElement(sourceNode) instanceof AbstractState)
            {
                // create the command associated with these parameters
                TransitionEdgeCreationCommand command = new TransitionEdgeCreationCommand(
                        getHost().getViewer().getEditDomain(), edge, sourceNode, true);
                request.setStartCommand(command);
                return command;
            }
        }
        return null;
    }
    
    /**
     * not implemented
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
     */
    protected Command getReconnectSourceCommand(ReconnectRequest request)
    {
        return UnexecutableCommand.INSTANCE;
    }
    
    /**
     * not implemented
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
     */
    protected Command getReconnectTargetCommand(ReconnectRequest request)
    {
        return UnexecutableCommand.INSTANCE;
    }
    
}
