/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.sam.commands;

import org.eclipse.gef.commands.Command;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.Composition;
import org.topcased.sam.InDataPort;
import org.topcased.sam.OutDataPort;
import org.topcased.sam.SAMFactory;
import org.topcased.sam.SynchronisationGate;

/**
 * 
 * Creation : 09 may 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class AddNewPortCommand extends Command
{
    private GraphNode syncGateNode;
    
    private GraphNode dataPortNode;
    
    private Modeler modeler;
    
    /**
     * Constructor
     * 
     * @param element the GraphNode that will contain the newly created Port
     */
    public AddNewPortCommand(GraphNode element, Modeler editor)
    {
        super("Add new port");
        this.syncGateNode = element;
        this.modeler = editor;
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        return syncGateNode != null;
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        redo();
    }
    
    /**
     * Add a new DataPort object to the SynchronisationGate model object
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        ICreationUtils factory = modeler.getActiveConfiguration().getCreationUtils();
        SynchronisationGate syncGate = (SynchronisationGate) Utils.getElement(syncGateNode);
        if (syncGate instanceof Composition)
        {
            // create the model object
            InDataPort inDataPort = SAMFactory.eINSTANCE.createInDataPort();
            syncGate.getIn().add(inDataPort);
            inDataPort.setParentSync(syncGate);
            
            // create the graphNode and its position property
            dataPortNode = (GraphNode) factory.createGraphElement(inDataPort);
            syncGateNode.getContained().add(dataPortNode);
        }
        else
        {
            // create the model object
            OutDataPort outDataPort = SAMFactory.eINSTANCE.createOutDataPort();
            syncGate.getOut().add(outDataPort);
            outDataPort.setParentSync(syncGate);
            
            // create the graphNode
            dataPortNode = (GraphNode) factory.createGraphElement(outDataPort);
            syncGateNode.getContained().add(dataPortNode);
        }
    }
    
    /**
     * Remove the last added DataPort
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        SynchronisationGate syncGate = (SynchronisationGate) Utils.getElement(syncGateNode);
        if (syncGate instanceof Composition)
        {
            syncGate.getIn().remove(Utils.getElement(dataPortNode));
            syncGateNode.getContained().remove(dataPortNode);
        }
        else
        {
            syncGate.getOut().remove(Utils.getElement(dataPortNode));
            syncGateNode.getContained().remove(dataPortNode);
        }
        
    }
    
}
