/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jrme Wrd (Anyware Technologies) - initial API and implementation
 ******************************************************************************/
package org.topcased.modeler.sam.commands;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.di.model.EMFSemanticModelBridge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.sam.util.InstancesUtil;
import org.topcased.sam.ModelContent;
import org.topcased.sam.SAMFactory;
import org.topcased.sam.System;

/**
 * Represents the command used for a drag and drop of an existing
 * system/automaton from the outline. The new system is empty and has its
 * instanceOf reference set to the existing system. The corresponding graphnode
 * is also created in this command.
 * 
 * @author jrme
 */
public class CreateInstanceCommand extends Command
{
    private final ModelContent referenced;

    private final System newContainerSystem;

    private final GraphNode newParentNode;

    private final Point location;

    // used for undo/redo.
    private ModelContent referencerModelContent = null;

    // the graph element associated to the refencerModelContent.
    private GraphNode newNode = null;

    public CreateInstanceCommand(EObject newContainerParent, GraphNode newParentNode, GraphNode newNode, Point location)
    {
        this.referenced = (ModelContent) InstancesUtil.getEmfElement(newNode);
        this.newParentNode = newParentNode;
        this.location = location;
        this.newNode = newNode;
        if (newContainerParent instanceof System)
        {
            this.newContainerSystem = (System) newContainerParent;
        }
        else
        {
            this.newContainerSystem = null;
            throw new IllegalArgumentException("the new container for the imported system should be a System");
        }
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    @Override
    public void execute()
    {
        redo();
    }

    @Override
    public boolean canExecute()
    {
        return true;
    }

    @Override
    public void undo()
    {
        assert ((referencerModelContent != null) && (newNode != null)) : "undo condition";

        // remove the empty system
        newContainerSystem.getListElements().remove(referencerModelContent);

        // for undo/redo
        referencerModelContent = null;

        // remove the graphNode
        newParentNode.getContained().remove(newNode);
    }

    @Override
    public boolean canUndo()
    {
        return ((referencerModelContent != null) && (newNode != null));
    }

    @Override
    public void redo()
    {
        Object o = SAMFactory.eINSTANCE.create(referenced.eClass());

        o.toString();
        ModelContent referencer = InstancesUtil.createModelContentCopy(referenced);
        referencerModelContent = referencer;

        referencer.setName(referenced.getName());
        InstancesUtil.setIsInstanceOf(referencer, referenced);
        referencer.setParentSystem(newContainerSystem);

        ((EMFSemanticModelBridge) newNode.getSemanticModel()).setElement(referencer);

        InstancesUtil.updateAllContents(referencer, referencer);

        newParentNode.getContained().add(newNode);
        newNode.setPosition(location);
    }
}
