/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.sam.commands;

import java.util.Map;

import org.eclipse.gef.commands.Command;
import org.topcased.sam.IdentifiedItem;

/**
 * 
 * Creation : 09 may 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class EditRequirementCommand extends Command
{
    private IdentifiedItem identifiedItem;
    
    private String oldRequirement;
    
    private String newRequirement;
    
    /**
     * Constructor
     * 
     * @param item the model Object IdentifiedItem
     * @param params a Map that contains the new requirement String
     */
    public EditRequirementCommand(IdentifiedItem item, Map params)
    {
        super("Edit Requirement");
        this.identifiedItem = item;
        newRequirement = (String) params.get("editRequirement");
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        return identifiedItem != null && newRequirement != null;
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        oldRequirement = identifiedItem.getRequirements();
        redo();
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        identifiedItem.setRequirements(newRequirement);
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        identifiedItem.setRequirements(oldRequirement);
    }
    
}
