/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.sam.commands;

import org.eclipse.gef.commands.Command;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.Property;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.sam.SAMPropertyConstants;

/**
 * 
 * Creation : 09 may 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class InversePortsCommand extends Command
{
    private GraphNode node;
    
    /**
     * Constructor
     * 
     * @param element the GraphElement to delete
     */
    public InversePortsCommand(GraphNode element)
    {
        super("Inverse Ports");
        this.node = element;
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        return node != null;
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        redo();
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        Property property = DIUtils.getProperty(node, SAMPropertyConstants.DATASTORE_TYPE);
        
        if (property != null)
        {
            if (SAMPropertyConstants.DATASTORE_TYPE_LEFT.equals(property.getValue()))
            {
                property.setValue(SAMPropertyConstants.DATASTORE_TYPE_RIGHT);
            }
            else
            {
                property.setValue(SAMPropertyConstants.DATASTORE_TYPE_LEFT);
            }
        }
        else
        {
            // the Property does not exist yet : by default, the DataStore has
            // its type to "left"
            DIUtils.setProperty(node, SAMPropertyConstants.DATASTORE_TYPE, SAMPropertyConstants.DATASTORE_TYPE_RIGHT);
        }
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        redo();
    }
    
}
