/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.sam.commands;

import java.util.List;
import java.util.ListIterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.topcased.modeler.commands.AbstractRestoreConnectionCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.sam.systemdiagram.commands.CreateFlowCommand;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.ControlFlow;
import org.topcased.sam.ControlPort;
import org.topcased.sam.DataFlow;
import org.topcased.sam.DataPort;
import org.topcased.sam.System;

/**
 * <br>
 * creation : 30 juin 2005
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class PortRestoreConnectionCommand extends AbstractRestoreConnectionCommand
{
    /**
     * 
     * @param part
     */
    public PortRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }
    
    /**
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     */
    protected void initializeCommands()
    {
        GraphNode node = (GraphNode) getGraphElement();
        EObject nodeObject = Utils.getElement(node);
        
        if (nodeObject instanceof DataPort)
        {
            TreeIterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                if (obj instanceof GraphNode)
                {
                    boolean autoRef = obj.equals(node);
                    GraphNode node2 = (GraphNode) obj;
                    EObject nodeObject2 = Utils.getElement(node2);
                    
                    if (nodeObject2 instanceof DataPort)
                    {
                        if (!autoRef)
                        {
                            // if node is the target of the edge or if it is the
                            // source and that the SourceTargetCouple is
                            // reversible
                            createDataFlow(node, node2);
                            createDataFlow(node2, node);
                        }
                    }
                    
                }
            }
        }
        
        if (nodeObject instanceof ControlPort)
        {
            TreeIterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                if (obj instanceof GraphNode)
                {
                    boolean autoRef = obj.equals(node);
                    GraphNode node2 = (GraphNode) obj;
                    EObject nodeObject2 = Utils.getElement(node2);
                    
                    if (nodeObject2 instanceof ControlPort)
                    {
                        if (!autoRef)
                        {
                            // if node is the target of the edge or if it is the
                            // source and that the SourceTargetCouple is
                            // reversible
                            createControlFlow(node, node2);
                            createControlFlow(node2, node);
                        }
                    }
                    
                }
            }
        }
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode
     * @param targetNode
     * 
     * @generated
     */
    private void createDataFlow(GraphNode srcNode, GraphNode targetNode)
    {
        DataPort sourceObject = (DataPort) Utils.getElement(srcNode);
        DataPort targetObject = (DataPort) Utils.getElement(targetNode);
        EObject diagramObject = Utils.getDiagramModelObject(srcNode);
        
        // Flows are available only in the Functional Behavior Diagram
        if (diagramObject instanceof System)
        {
            EList edgeObjectList = ((System) diagramObject).getListFlows();
            for (ListIterator it = edgeObjectList.listIterator(); it.hasNext();)
            {
                Object obj = it.next();
                if (obj instanceof DataFlow)
                {
                    DataFlow edgeObject = (DataFlow) obj;
                    if (edgeObject.getDest() != null && edgeObject.getDest().contains(targetObject)
                            && edgeObject.getSource() != null && edgeObject.getSource().equals(sourceObject)
                            && sourceObject.getOutlink() != null && sourceObject.getOutlink().equals(edgeObject)
                            && targetObject.getInlink() != null && targetObject.getInlink().equals(edgeObject))
                    {
                        // check if the relation does not exists yet
                        List existing = getExistingEdges(srcNode, targetNode, DataFlow.class);
                        if (!isAlreadyPresent(existing, edgeObject))
                        {
                            ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                            GraphElement edge = factory.createGraphElement(edgeObject);
                            if (edge instanceof GraphEdge)
                            {
                                CreateFlowCommand cmd = new CreateFlowCommand(getEditDomain(), (GraphEdge) edge,
                                        srcNode, false);
                                cmd.setTarget(targetNode);
                                add(cmd);
                            }
                        }
                    }
                }
            }
        }
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode
     * @param targetNode
     * 
     * @generated
     */
    private void createControlFlow(GraphNode srcNode, GraphNode targetNode)
    {
        ControlPort sourceObject = (ControlPort) Utils.getElement(srcNode);
        ControlPort targetObject = (ControlPort) Utils.getElement(targetNode);
        EObject diagramObject = Utils.getDiagramModelObject(srcNode);
        
        // Flows are available only in the Functional Behavior Diagram
        if (diagramObject instanceof System)
        {
            EList edgeObjectList = ((System) diagramObject).getListFlows();
            for (ListIterator it = edgeObjectList.listIterator(); it.hasNext();)
            {
                Object obj = it.next();
                if (obj instanceof ControlFlow)
                {
                    ControlFlow edgeObject = (ControlFlow) obj;
                    if (edgeObject.getDest() != null && edgeObject.getDest().contains(targetObject)
                            && edgeObject.getSource() != null && edgeObject.getSource().equals(sourceObject)
                            && sourceObject.getOutlink() != null && sourceObject.getOutlink().equals(edgeObject)
                            && targetObject.getInlink() != null && targetObject.getInlink().equals(edgeObject))
                    {
                        // check if the relation does not exists yet
                        List existing = getExistingEdges(srcNode, targetNode, ControlFlow.class);
                        if (!isAlreadyPresent(existing, edgeObject))
                        {
                            ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                            GraphElement edge = factory.createGraphElement(edgeObject);
                            if (edge instanceof GraphEdge)
                            {
                                CreateFlowCommand cmd = new CreateFlowCommand(getEditDomain(), (GraphEdge) edge,
                                        srcNode, false);
                                cmd.setTarget(targetNode);
                                add(cmd);
                            }
                        }
                    }
                }
            }
        }
    }
}
