/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.sam.commands;

import java.util.Map;

import org.eclipse.gef.commands.Command;
import org.topcased.modeler.sam.dialogs.TransitionEditDialog;
import org.topcased.sam.Transition;

/**
 * Class that create a command in order to update transition parameters <br>
 * 
 * Created 5 July 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class UpdateTransitionCommand extends Command
{
    /** Current operation */
    private Transition _transition;
    
    /** Old values */
    private String _oldCondition;
    
    private String _oldEmission;
    
    /** New values */
    private String _condition;
    
    private String _emission;
    
    /**
     * Create a command for updating parameters on a given transition
     * 
     * @param transition
     * @param data
     */
    public UpdateTransitionCommand(Transition transition, Map data)
    {
        _transition = transition;
        
        // Store new data
        _condition = (String) data.get(TransitionEditDialog.TRANSITION_CONDITION);
        _emission = (String) data.get(TransitionEditDialog.TRANSITION_EMISSION);
        
        // Store old data
        _oldCondition = _transition.getCondition();
        _oldEmission = _transition.getEmission();
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        return true;
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        // Perform update for main data
        redo();
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        _transition.setCondition(_oldCondition);
        _transition.setEmission(_oldEmission);
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        _transition.setCondition(_condition);
        _transition.setEmission(_emission);
    }
}
