/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.sam.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;

/**
 * Edit a requirement in a new Window in a multilign textfield
 * 
 * @author jako
 * @deprecated Remove this dialog. Use the corresponding section in the Properties View
 */
public class EditRequirementDialog extends Dialog implements ITopcasedDialogConstants
{
    // Requirement
    private String requirement;
    
    // SWT Objects
    private Composite dialogComposite;
    
    private Text requirementTxt;
    
    /**
     * Create the Dialog window that handles the requirement editing
     * 
     * @param initialRequirement the initial requirement
     * @param parentShell the parent Shell
     */
    public EditRequirementDialog(String initialRequirement, Shell parentShell)
    {
        super(parentShell);
        setBlockOnOpen(true);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        if (initialRequirement == null)
            this.requirement = "";
        else
            this.requirement = initialRequirement;
    }
    
    /**
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("Requirement Editing");
        newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);
        
        super.configureShell(newShell);
    }
    
    /**
     * Create the Dialog area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        // Dialog
        dialogComposite = (Composite) super.createDialogArea(parent);
        GridData gdComp = new GridData(GridData.FILL_BOTH);
        gdComp.widthHint = DEFAULT_DIALOG_WIDTH;
        gdComp.heightHint = DEFAULT_DIALOG_HEIGHT;
        dialogComposite.setLayoutData(gdComp);
        
        // Edit requirement
        Label label = new Label(dialogComposite, SWT.WRAP);
        label.setText("Edit requirement :");
        label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        
        requirementTxt = new Text(dialogComposite, SWT.MULTI | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        GridData gd = new GridData(GridData.FILL_BOTH);
        requirementTxt.setLayoutData(gd);
        requirementTxt.setText(requirement);
        
        return dialogComposite;
    }
    
    /**
     * Store the requirement text
     * 
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        requirement = requirementTxt.getText();
        
        super.okPressed();
    }
    
    /**
     * Returns the new Requirement
     * 
     * @return the string representing the new requirement
     */
    public String getNewRequirement()
    {
        if (requirement != null)
            return requirement;
        
        return "";
    }
}
