/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.sam.dialogs;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;
import org.topcased.sam.Transition;

/**
 * Updating transition parameters<br>
 * 
 * Created 05 July 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 * @deprecated The editing of these properties should be done through the Properties View
 */
public class TransitionEditDialog extends Dialog implements ITopcasedDialogConstants
{
    /** Current edited operation */
    private Transition _transition;
    
    // SWT Objects
    private Composite _dialogComposite;
    
    private Text _conditionTxt;
    
    private Text _emissionTxt;
    
    // The results
    private Map _data;
    
    // Constants
    public static final String TRANSITION_CONDITION = "TransitionCondition";
    
    public static final String TRANSITION_EMISSION = "TransitionEmission";
    
    /**
     * Create the dialog for a given transition
     * 
     * @param transition
     * @param parentShell
     */
    public TransitionEditDialog(Transition transition, Shell parentShell)
    {
        super(parentShell);
        setBlockOnOpen(true);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        _transition = transition;
    }
    
    /**
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("Transition Edit");
        newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);
        super.configureShell(newShell);
    }
    
    /**
     * Create the Dialog area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        // Dialog
        _dialogComposite = (Composite) super.createDialogArea(parent);
        GridData dialogLayoutData = new GridData(GridData.FILL_BOTH);
        dialogLayoutData.widthHint = DEFAULT_DIALOG_WIDTH;
        dialogLayoutData.heightHint = DEFAULT_DIALOG_HEIGHT;
        _dialogComposite.setLayoutData(dialogLayoutData);
        createTransitionGroup(_dialogComposite);
        
        return _dialogComposite;
    }
    
    /**
     * Creates the group
     * 
     * @param parent the parent Composite
     */
    protected void createTransitionGroup(Composite parent)
    {
        Label label = new Label(parent, SWT.WRAP);
        label.setText("Event/Condition on the entries :");
        label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        
        _conditionTxt = new Text(parent, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
        if (_transition.getCondition() != null)
        {
            _conditionTxt.setText(_transition.getCondition());
        }
        GridData gd = new GridData(GridData.FILL_BOTH);
        _conditionTxt.setLayoutData(gd);
        
        label = new Label(parent, SWT.WRAP);
        label.setText("Emission of an Exit/Output :");
        label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        
        _emissionTxt = new Text(parent, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
        if (_transition.getEmission() != null)
        {
            _emissionTxt.setText(_transition.getEmission());
        }
        _emissionTxt.setLayoutData(gd);
    }
    
    /**
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        _data = new HashMap();
        _data.put(TRANSITION_CONDITION, _conditionTxt.getText());
        _data.put(TRANSITION_EMISSION, _emissionTxt.getText());
        super.okPressed();
    }
    
    /**
     * Return a map containing property data that may changed
     * 
     * @return a Map
     */
    public Map getData()
    {
        // TODO change this method to typed accessor
        return _data;
    }
    
}
