/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.sam.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.ToolbarLayout;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.figures.ModelerLabelledPortFigure;
import org.topcased.modeler.figures.ModelerPortFigure;
import org.topcased.modeler.sam.figures.OutDataPortFigure;
import org.topcased.modeler.sam.systemdiagram.edit.CompositionEditPart;
import org.topcased.modeler.sam.systemdiagram.edit.DataStoreEditPart;
import org.topcased.modeler.sam.systemdiagram.edit.DecompositionEditPart;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.Composition;
import org.topcased.sam.DataStore;

/**
 * The OutDataPort object controller <!-- begin-user-doc --> <!-- end-user-doc
 * -->
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques Lescot </a>
 * @generated
 */
public class OutDataPortEditPart extends SAMPortEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj
     * @generated
     */
    public OutDataPortEditPart(GraphNode obj)
    {
        super(obj);
    }
    
    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();
        
        if (getParent().getModel() instanceof GraphNode)
        {
            if (Utils.getElement((GraphNode) getParent().getModel()) instanceof DataStore
                    || Utils.getElement((GraphNode) getParent().getModel()) instanceof Composition)
            {
                // prevent "delete from model" actions if the parent is a
                // DataStore or a Composition
                installEditPolicy(ModelerEditPolicyConstants.DELETE_MODEL_OBJECT_EDITPOLICY, null);
            }
        }
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated NOT
     */
    protected IFigure createFigure()
    {
        if (getParent() instanceof DataStoreEditPart || getParent() instanceof CompositionEditPart
                || getParent() instanceof DecompositionEditPart)
        {
            return new OutDataPortFigure();
        }
        else
        {
            return new ModelerLabelledPortFigure(new OutDataPortFigure(), ToolbarLayout.ALIGN_TOPLEFT);
        }
    }
    
    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#refreshConstraints()
     */
    protected void refreshConstraints()
    {
        if (getParent() instanceof DataStoreEditPart)
        {
            ((ModelerPortFigure) getPortFigure()).setPosition(PositionConstants.TOP);
        }
        else if (getParent() instanceof CompositionEditPart) {
            switch (((CompositionEditPart) getParent()).getDirection())
            {
                case PositionConstants.NORTH:
                    ((ModelerPortFigure) getPortFigure()).setPosition(PositionConstants.TOP);
                    break;
                case PositionConstants.SOUTH:
                    ((ModelerPortFigure) getPortFigure()).setPosition(PositionConstants.BOTTOM);
                    break;
                case PositionConstants.WEST:
                    ((ModelerPortFigure) getPortFigure()).setPosition(PositionConstants.LEFT);
                    break;
                default:
                    ((ModelerPortFigure) getPortFigure()).setPosition(PositionConstants.RIGHT);
                    break;
            }
        }
        else if (getParent() instanceof DecompositionEditPart) {
            switch (((DecompositionEditPart) getParent()).getDirection())
            {
                case PositionConstants.NORTH:
                    ((ModelerPortFigure) getPortFigure()).setPosition(PositionConstants.TOP);
                    break;
                case PositionConstants.SOUTH:
                    ((ModelerPortFigure) getPortFigure()).setPosition(PositionConstants.BOTTOM);
                    break;
                case PositionConstants.WEST:
                    ((ModelerPortFigure) getPortFigure()).setPosition(PositionConstants.LEFT);
                    break;
                default:
                    ((ModelerPortFigure) getPortFigure()).setPosition(PositionConstants.RIGHT);
                    break;
            }
        }
        
        super.refreshConstraints();
    }
    
}
