/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.sam.edit.policies;

import java.util.Map;

import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.AbstractEditPolicy;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.sam.SAMRequestConstants;
import org.topcased.modeler.sam.commands.AddNewPortCommand;
import org.topcased.modeler.sam.commands.EditRequirementCommand;
import org.topcased.modeler.sam.commands.InversePortsCommand;
import org.topcased.modeler.sam.util.InstancesUtil;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.IdentifiedItem;

/**
 * The EditPolicy that handle the Edit Requirement command created 13 mai 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class CustomEditPolicy extends AbstractEditPolicy {

	/**
	 * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#getCommand(org.eclipse.gef.Request)
	 */
	public Command getCommand(Request request) {

//		if (SAMRequestConstants.REQ_EDIT_REQUIREMENT.equals(request.getType())) {
//			GraphElement element = (GraphElement) getHost().getModel();
//			IdentifiedItem item = (IdentifiedItem) Utils.getElement(element);
//			Map params = request.getExtendedData();
//			EditRequirementCommand command = new EditRequirementCommand(item,
//					params);
//			return command;
//		}
		if (SAMRequestConstants.REQ_INVERSE_PORTS.equals(request.getType())) {
			GraphNode node = (GraphNode) getHost().getModel();
			InversePortsCommand command = new InversePortsCommand(node);
			return command;
		}
		if (SAMRequestConstants.REQ_ADD_NEW_PORT.equals(request.getType())) {
			Modeler modeler = ((ModelerGraphicalViewer) getHost().getViewer())
					.getModelerEditor();
			GraphNode node = (GraphNode) getHost().getModel();
			AddNewPortCommand command = new AddNewPortCommand(node, modeler);
			return command;
		}

		return null;
	}

	/**
	 * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#understandsRequest(org.eclipse.gef.Request)
	 */
	public boolean understandsRequest(Request req) {
		
//		if (SAMRequestConstants.REQ_EDIT_REQUIREMENT.equals(req.getType())) {
//
//			if (getHost().getModel() instanceof GraphElement) {
//				GraphElement graphElement = (GraphElement) getHost().getModel();
//				return !(InstancesUtil.isAReference(Utils.getElement(graphElement)));
//			}
//		}

		if (SAMRequestConstants.REQ_INVERSE_PORTS.equals(req.getType())) {
			// used on datastores, so always possible (not instances)
			return true;
		}
		if (SAMRequestConstants.REQ_ADD_NEW_PORT.equals(req.getType())) {
			// used on synchronization gates, so always possible (not instances)
			return true;
		}

		return false;
	}

}
