/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.sam.edit.policies;

import org.eclipse.gef.Request;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.edit.policies.DeleteModelObjectEditPolicy;
import org.topcased.modeler.sam.edit.SAMPortEditPart;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.Composition;
import org.topcased.sam.Decomposition;
import org.topcased.sam.InputPort;
import org.topcased.sam.OutputPort;
import org.topcased.sam.Port;
import org.topcased.sam.SynchronisationGate;

/**
 * A custom Policy for the Ports in SAM. Enablement of the "Delete" command
 * depends on the type of container and the type of Port
 * 
 * Creation 23 août 06
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class DeleteSAMPortEditPolicy extends DeleteModelObjectEditPolicy
{
    /**
     * Do not delete the Port when contained by a SynchonisationGate that the
     * minimum amount is reached
     * 
     * Do not delete instances of a port.
     * 
     * @see org.topcased.modeler.edit.policies.DeleteModelObjectEditPolicy#understandsRequest(org.eclipse.gef.Request)
     */
    @Override
    public boolean understandsRequest(Request req)
    {
        if (!super.understandsRequest(req))
        {
            return false;
        }

        if (getHost() instanceof SAMPortEditPart)
        {
            Port port = (Port) Utils.getElement((GraphElement) ((SAMPortEditPart) getHost()).getModel());
            if (port.eContainer() instanceof SynchronisationGate)
            {
                // A Composition should have at least 2 Input Ports and a
                // Decomposition 2 Output Ports
                if (((port instanceof InputPort) && (port.eContainer() instanceof Composition) && (((Composition) port.eContainer()).getIn().size() < 3))
                        || ((port instanceof OutputPort) && (port.eContainer() instanceof Decomposition) && (((Decomposition) port.eContainer()).getOut().size() < 3)))
                {
                    return false;
                }
            }

            // do not allow deletion of ports that are instances of others.
            if (port.getIsInstanceOf() != null)
            {
                return false;
            }
        }
        return true;
    }
}
