/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/

package org.topcased.modeler.sam.edit.policies;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.jface.resource.StringConverter;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.edit.policies.MultiPortLayoutEditPolicy;
import org.topcased.modeler.sam.commands.MovePortCommand;

/**
 * An EditPolicy that handle the Add and Orphan children commands of the
 * MultiPort object
 * 
 * Creation : 23 nov. 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class SAMMultiPortLayoutEditPolicy extends MultiPortLayoutEditPolicy
{
    
    /**
     * The constructor
     */
    public SAMMultiPortLayoutEditPolicy()
    {
        super();
    }
    
    /**
     * @see org.eclipse.gef.editpolicies.OrderedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart,
     *      org.eclipse.gef.EditPart)
     */
    protected Command createAddCommand(EditPart child, EditPart after)
    {
        if (getHost().getModel() instanceof GraphNode && child.getModel() instanceof GraphNode)
        {
            int attachment = StringConverter.asInt(DIUtils.getPropertyValue((GraphElement) getHost().getModel(),
                    ModelerPropertyConstants.PORT_POSITION));
            return new MovePortCommand((GraphNode) getHost().getModel(), (GraphNode) child.getModel(),
                    new Point(-1, -1), attachment);
        }
        return UnexecutableCommand.INSTANCE;
    }
    
}
