package org.topcased.modeler.sam.edit.policies;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.requests.DirectEditRequest;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.sam.util.InstancesUtil;
import org.topcased.modeler.utils.Utils;

/**
 * Custom edit policy so that elements having a not null isInstanceOf reference
 * cannot be direcly edited.
 * 
 * @author jrme
 */
public class SamLabelDirectEditPolicy extends LabelDirectEditPolicy
{
    private String oldValue;

    /*
     * understandsRequest is not enough to disable direct editing, so we make
     * sure here that no command is created for instances.
     * 
     * (non-Javadoc)
     * 
     * @see org.topcased.modeler.edit.policies.LabelDirectEditPolicy#getDirectEditCommand(org.eclipse.gef.requests.DirectEditRequest)
     */
    @Override
    protected Command getDirectEditCommand(DirectEditRequest request)
    {
        if (!understandsRequest(request))
        {
            return UnexecutableCommand.INSTANCE;
        }
        return super.getDirectEditCommand(request);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.gef.editpolicies.DirectEditPolicy#understandsRequest(org.eclipse.gef.Request)
     */
    @Override
    public boolean understandsRequest(Request request)
    {
        if (!super.understandsRequest(request))
        {
            return false;
        }

        if (getHost().getModel() instanceof GraphElement)
        {
            EObject element = Utils.getElement((GraphElement) getHost().getModel());
            if (InstancesUtil.isAReference(element))
            {
                return false;
            }
        }
        return true;
    }

    /*
     * This resets the cellEditor contents to the original value each time the
     * user types something.
     * 
     * (non-Javadoc)
     * 
     * @see org.topcased.modeler.edit.policies.LabelDirectEditPolicy#showCurrentEditValue(org.eclipse.gef.requests.DirectEditRequest)
     */
    @Override
    protected void showCurrentEditValue(DirectEditRequest request)
    {
        if (!understandsRequest(request))
        {
            request.getCellEditor().setValue(oldValue);
        }
        super.showCurrentEditValue(request);
    }

    /*
     * Necessary in order to reset the cell editor in showCurrentEditValue
     * 
     * (non-Javadoc)
     * 
     * @see org.topcased.modeler.edit.policies.LabelDirectEditPolicy#storeOldEditValue(org.eclipse.gef.requests.DirectEditRequest)
     */
    @Override
    protected void storeOldEditValue(DirectEditRequest request)
    {
        oldValue = (String) request.getCellEditor().getValue();
    }

}
