/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.editor;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.topcased.modeler.documentation.IDocPage;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.sam.SAMActionConstants;
import org.topcased.modeler.sam.SAMPlugin;
import org.topcased.modeler.sam.actions.AddNewPortAction;
import org.topcased.modeler.sam.actions.EditRequirementAction;
import org.topcased.modeler.sam.actions.InversePortsAction;
import org.topcased.modeler.sam.documentation.IdentifiedItemDocPage;
import org.topcased.modeler.sam.editor.outline.SamOutlinePage;
import org.topcased.modeler.sam.providers.SAMModelerProviderAdapterFactory;
import org.topcased.sam.provider.SAMItemProviderAdapterFactory;

/**
 * Generated Model editor <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class SAMEditor extends Modeler
{
    /**
     * @see org.topcased.modeler.editor.Modeler#getAdapterFactories()
     */
    @Override
    protected List<AdapterFactory> getAdapterFactories()
    {
        List<AdapterFactory> factories = new ArrayList<AdapterFactory>();
        factories.add(new SAMItemProviderAdapterFactory());
        factories.add(new SAMModelerProviderAdapterFactory());
        factories.addAll(super.getAdapterFactories());

        return factories;
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#getId()
     * @generated
     */
    public String getId()
    {
        return "org.topcased.modeler.sam.editor.SAMEditor";
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#createActions()
     */
    @Override
    protected void createActions()
    {
        super.createActions();

        ActionRegistry registry = getActionRegistry();
        IAction action;

//        action = new EditRequirementAction(this);
//        registry.registerAction(action);
//        getSelectionActions().add(action.getId());

        action = new AddNewPortAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());

        action = new InversePortsAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#configureGraphicalViewer()
     */
    @Override
    protected void configureGraphicalViewer()
    {
        super.configureGraphicalViewer();

//        IAction editRequirementAction = getActionRegistry().getAction(SAMActionConstants.EDIT_REQUIREMENT);
//        getGraphicalViewer().addSelectionChangedListener((ISelectionChangedListener) editRequirementAction);

        IAction addNewPortAction = getActionRegistry().getAction(SAMActionConstants.ADD_NEW_PORT);
        getGraphicalViewer().addSelectionChangedListener((ISelectionChangedListener) addNewPortAction);

        IAction inversePortsAction = getActionRegistry().getAction(SAMActionConstants.INVERSE_PORTS);
        getGraphicalViewer().addSelectionChangedListener((ISelectionChangedListener) inversePortsAction);
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#getContextMenuProvider(org.topcased.modeler.editor.ModelerGraphicalViewer)
     */
    @Override
    protected ContextMenuProvider getContextMenuProvider(ModelerGraphicalViewer viewer)
    {
        return new SAMContextMenuProvider(viewer, getActionRegistry());
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#getAdapter(java.lang.Class)
     */
    @Override
    public Object getAdapter(Class type)
    {
        if (type == IDocPage.class)
        {
            return new IdentifiedItemDocPage(getCommandStack());
        }
        return super.getAdapter(type);
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#getPreferenceStore()
     */
    @Override
    public IPreferenceStore getPreferenceStore()
    {
        return SAMPlugin.getDefault().getPreferenceStore();
    }

    /**
     * Overriden so as to use our custom outline page ('Imported Libraries'
     * section)
     * 
     * @see org.topcased.modeler.editor.Modeler#createOutlinePage()
     */
    @Override
    protected IContentOutlinePage createOutlinePage()
    {
        return new SamOutlinePage(this);
    }
}
