/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.sam.editor.outline;

import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.provider.IWrapperItemProvider;

import org.topcased.modeler.editor.outline.OutlineDragAdapter;
import org.topcased.sam.Automaton;
import org.topcased.sam.ModelContent;
import org.topcased.sam.System;

/**
 * Class acting as a wrapper around a {@link ModelContent} (ie a {@link System}
 * or an {@link Automaton}) so that it will not be seen as a 'normal' model
 * elsewhere (eg in content/label providers) and so that custom actions on
 * instances of this class may be added.
 * 
 * Implements {@link IWrapperItemProvider} so that a selection containing an
 * {@link InstanciableModel} object can be understood for the drag and drop in
 * {@link OutlineDragAdapter#getSelection} (only really interested in getValue
 * here)
 * 
 * @author jrme
 */
public class InstanciableModel implements IWrapperItemProvider
{
    private ModelContent model;

    /**
     * @param model the wrapped modelcontent that can be instanciated.
     */
    public InstanciableModel(ModelContent model)
    {
        this.model = model;
    }

    /**
     * 
     * @return the system or automaton wrapped by this instance.
     */
    public ModelContent getModelContent()
    {
        return model;
    }

    /**
     * @return an empty array, since only first level automatas and systems can
     *         be instanciated.
     */
    public Object[] getChildren()
    {
        if (model == null)
        {
            throw new NullPointerException();
        }
        return new Object[0];
    }

    /**
     * @return false, only first level modelcontents can be instanciated.
     */
    public boolean hasChildren()
    {
        return false;
    }

    public EStructuralFeature getFeature()
    {
        // not used
        return null;
    }

    public int getIndex()
    {
        // not used
        return 0;
    }

    public Object getOwner()
    {
        // not used
        return null;
    }

    /**
     * Returns the system or automaton wrapped by this instance.
     * 
     * @see org.eclipse.emf.edit.provider.IWrapperItemProvider#getValue()
     */
    public Object getValue()
    {
        return getModelContent();
    }

    public void setIndex(int index)
    {
        // not used
    }

    public void dispose()
    {
        // not used
    }
}