/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jrme Wrd (Anyware Technologies) - initial API and implementation
 ******************************************************************************/
package org.topcased.modeler.sam.editor.outline;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.graphics.Image;
import org.topcased.modeler.editor.outline.ModelLabelProvider;
import org.topcased.modeler.sam.SAMImageRegistry;
import org.topcased.modeler.sam.editor.outline.SamOutlineContentProvider.FilteredAdditionalResources;
import org.topcased.modeler.sam.util.InstancesUtil;
import org.topcased.sam.Automaton;
import org.topcased.sam.ModelContent;
import org.topcased.sam.System;

public class SamOutlineLabelProvider extends ModelLabelProvider
{
    private static final String POSSIBLE_IMPORTS_SECTION_LABEL = "Instanciable Models";

    /**
     * Constructor
     * 
     * @param delegatedProvider the ecore provider
     */
    public SamOutlineLabelProvider(ILabelProvider delegatedProvider)
    {
        super(delegatedProvider);
    }

    /**
     * Adds two cases for the 'imports' section title and elements
     * (InstanciableModel objects).
     * 
     * @see org.topcased.modeler.editor.outline.ModelLabelProvider#getText(java.lang.Object)
     */
    @Override
    public String getText(Object element)
    {
        if (element instanceof InstanciableModel)
        {
            return super.getText(((InstanciableModel) element).getModelContent());
        }

        if (element instanceof FilteredAdditionalResources)
        {
            return POSSIBLE_IMPORTS_SECTION_LABEL;
        }

        return super.getText(element);
    }

    /**
     * Adds images for the 'import' section and its elements.
     * 
     * Also changes the images for instanciated systems and automatas.
     * 
     * @see org.topcased.modeler.editor.outline.ModelLabelProvider#getImage(java.lang.Object)
     */
    @Override
    public Image getImage(Object element)
    {
        if (element instanceof InstanciableModel)
        {
            ModelContent model = ((InstanciableModel) element).getModelContent();

            if (model instanceof System)
            {
                return SAMImageRegistry.getImage("INSTANCIABLE_SYSTEM");
            }
            else if (model instanceof Automaton)
            {
                return SAMImageRegistry.getImage("INSTANCIABLE_AUTOMATON");
            }
            assert (false);
        }

        if (element instanceof FilteredAdditionalResources)
        {
            return SAMImageRegistry.getImage("MODEL_LIBRARY");
        }

        if (element instanceof System && InstancesUtil.isAReference((System) element))
        {
            return SAMImageRegistry.getImage("INSTANCIATED_SYSTEM");
        }
        if (element instanceof Automaton && InstancesUtil.isAReference((Automaton) element))
        {
            return SAMImageRegistry.getImage("INSTANCIATED_AUTOMATON");
        }

        return super.getImage(element);
    }
}
