/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies) - initial API and implementation
 ******************************************************************************/
package org.topcased.modeler.sam.editor.outline;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.IPageSite;

import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.outline.DiagramsOutlinePage;
import org.topcased.modeler.editor.outline.ModelNavigator;

/**
 * This class extends the basic {@link DiagramsOutlinePage} used by default so
 * as to add an 'Possible imports' section. No need to register a custom dnd
 * handler here, it's done with the layout policies.
 * 
 * creation : 18 avr. 07
 * 
 * @author jrme
 */
public class SamOutlinePage extends DiagramsOutlinePage
{

    /**
     * Serves to toggle displaying of the additionnal resources section. Mainly
     * for debugging purposes. Used by
     * {@link SamOutlineContentProvider#getElements()}.
     * 
     * WARNING : if enabled, it will be possible to drag and drop nested
     * elements in a model, even though this has not been taken into account
     * yet.
     */
    static boolean DISPLAY_ADDITIONAL_RESOURCES_SECTION = false;

    /**
     * The navigator part of this outline. Needed since the superclass doesn't
     * allow access to it.
     */
    private ModelNavigator navigator;

    /**
     * Constructor. Nothing special added.
     * 
     * @param modeler the editor
     */
    public SamOutlinePage(Modeler modeler)
    {
        super(modeler);
    }

    /**
     * Creates a custom Navigator for this outline, and keeps a reference to it.
     * 
     * @see org.topcased.modeler.editor.outline.DiagramsOutlinePage#createNavigator(org.eclipse.swt.widgets.Composite,
     *      org.topcased.modeler.editor.Modeler, org.eclipse.ui.part.IPageSite)
     */
    @Override
    protected ModelNavigator createNavigator(Composite parent, Modeler editor, IPageSite pageSite)
    {
        this.navigator = new SamOutlineNavigator(parent, editor, pageSite);
        return navigator;
    }
}
