/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.sam.editor.outline.actions;

import java.util.List;

import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.jface.action.Action;

import org.topcased.modeler.sam.util.InstancesUtil;
import org.topcased.sam.Automaton;
import org.topcased.sam.ModelContent;
import org.topcased.sam.MultiPort;
import org.topcased.sam.Port;
import org.topcased.sam.System;

/**
 * Action that removes the isInstanceOf references of a System/Automaton and its
 * children ports/multiports.
 * 
 * @author jrme
 */
public class RemoveInstanceAction extends Action
{
    private static final String LABEL = "Unwire instance";

    private final EditingDomain domain;

    private final ModelContent objectToDereference;

    /**
     * 
     * @param domain
     * @param modelContent the system/automaton to be 'uninstanciated'
     */
    public RemoveInstanceAction(EditingDomain domain, ModelContent modelContent)
    {
        super(LABEL);
        this.domain = domain;
        objectToDereference = modelContent;
    }

    @Override
    public boolean isEnabled()
    {
        return true;
    }

    private void deReferenceChildren(ModelContent modelContent)
    {
        List<Port> portList = InstancesUtil.getListPorts(modelContent);
        List<MultiPort> multiportList = InstancesUtil.getListMultiPorts(modelContent);

        for (Port port : portList)
        {
            if (InstancesUtil.isAReference(port))
            {
                port.setIsInstanceOf(null);
            }
        }

        for (MultiPort port : multiportList)
        {
            if (InstancesUtil.isAReference(port))
            {
                port.setIsInstanceOf(null);
            }
        }

    }

    @Override
    public void run()
    {
        ModelContent oldInstanceOf = null;

        if (objectToDereference instanceof System)
        {
            oldInstanceOf = ((System) objectToDereference).getIsInstanceOf();
            ((System) objectToDereference).setIsInstanceOf(null);
        }
        else if (objectToDereference instanceof Automaton)
        {
            oldInstanceOf = ((Automaton) objectToDereference).getIsInstanceOf();
            ((Automaton) objectToDereference).setIsInstanceOf(null);
        }
        else
        {
            assert (false);
        }
        deReferenceChildren(objectToDereference);
        // objectToDereference.eNotify(new NotificationImpl(Notification.SET,
        // oldInstanceOf, null));
    }
}
