/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.sam.internal.properties.sections;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.swt.widgets.Composite;
import org.topcased.modeler.sam.util.InstancesUtil;
import org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection;

public class InstanceOfPropertySection extends AbstractBooleanPropertySection
{

    /**
     * Not used for this property since the feature it's related to is not a
     * boolean.
     * 
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getFeature()
     */
    @Override
    protected EStructuralFeature getFeature()
    {
        return null;
    }

    /**
     * Returns <code>true</code> if the current eObject is an instance of
     * another one.
     * 
     * @see org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection#getFeatureValue()
     */
    @Override
    protected boolean getFeatureValue()
    {
        EObject object = getEObject();

        return (InstancesUtil.isAReference(object));
    }

    @Override
    protected String getLabelText()
    {
        return "Is an instance";
    }

    /**
     * Overriden so as to disable editing of this checkbox.
     * 
     * @see org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection#createWidgets(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void createWidgets(Composite composite)
    {
        super.createWidgets(composite);
        getCheckButton().setEnabled(false);
    }
}
