/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.providers;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.topcased.modeler.providers.IDeleteObjectProvider;
import org.topcased.modeler.providers.IDeletePartnerProvider;
import org.topcased.modeler.providers.ILabelFeatureProvider;
import org.topcased.sam.Composition;
import org.topcased.sam.DataPort;
import org.topcased.sam.DataStore;
import org.topcased.sam.Decomposition;
import org.topcased.sam.InDataPort;
import org.topcased.sam.OutDataPort;
import org.topcased.sam.Port;
import org.topcased.sam.SAMPackage;

/**
 * This is the item provider adpater for a {@link org.topcased.sam.Port} object.
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */

public class PortModelerProvider extends NamedItemModelerProvider implements ILabelFeatureProvider,
        IDeletePartnerProvider, IDeleteObjectProvider
{
    /**
     * This constructs an instance from a factory and a notifier. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param adapterFactory
     * 
     * @generated
     */
    public PortModelerProvider(AdapterFactory adapterFactory)
    {
        super(adapterFactory);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.providers.ILabelFeatureProvider#getLabelFeature(java.lang.Object)
     * @generated
     */
    public EAttribute getLabelFeature(Object object)
    {
        return SAMPackage.eINSTANCE.getNamedItem_Name();
    }

    /**
     * @see org.topcased.modeler.providers.IDeletePartnerProvider#getDeletePartners(java.lang.Object)
     */
    public List getDeletePartners(Object object)
    {
        ArrayList deletelist = new ArrayList();

        Port port = (Port) object;

        if (port.getOutlink() != null)
        {
            deletelist.add(port.getOutlink());
        }

        return deletelist;
    }

    /**
     * @see org.topcased.modeler.providers.IDeleteObjectProvider#canBeDeleted(java.lang.Object)
     */
    public boolean canBeDeleted(Object object)
    {
        // Prevent deleting a Port that could invalidate a Composition
        if (object instanceof DataPort && ((EObject) object).eContainer() instanceof Composition)
        {
            Composition composition = (Composition) ((EObject) object).eContainer();
            return (object instanceof InDataPort && composition.getIn().size() > 2 || object instanceof OutDataPort
                    && composition.getOut().size() > 1);
        }

        // Prevent deleting a Port that could invalidate a Decomposition
        if (object instanceof DataPort && ((EObject) object).eContainer() instanceof Decomposition)
        {
            Decomposition decomposition = (Decomposition) ((EObject) object).eContainer();
            return (object instanceof OutDataPort && decomposition.getOut().size() > 2 || object instanceof InDataPort
                    && decomposition.getIn().size() > 1);
        }

        // Prevent deleting a Port that could invalidate a DataStore
        if (object instanceof Port && ((EObject) object).eContainer() instanceof DataStore)
        {
            return false;
        }

        return true;
    }
}
