/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.providers;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.edit.provider.ChangeNotifier;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.IChangeNotifier;
import org.eclipse.emf.edit.provider.IDisposable;
import org.eclipse.emf.edit.provider.INotifyChangedListener;
import org.topcased.modeler.providers.IDeleteObjectProvider;
import org.topcased.modeler.providers.IDeletePartnerProvider;
import org.topcased.modeler.providers.ILabelFeatureProvider;
import org.topcased.sam.util.SAMAdapterFactory;

/**
 * This is the factory that is used to provide the interfaces needed to support
 * Viewers. The adapters generated by this factory convert EMF adapter
 * notifications into calls to {@link #fireNotifyChanged fireNotifyChanged}.
 * The adapters also support Eclipse property sheets. Note that most of the
 * adapters are shared among multiple instances. <!-- begin-user-doc --> <!--
 * end-user-doc -->
 * 
 * @generated
 */
public class SAMModelerProviderAdapterFactory extends SAMAdapterFactory implements ComposeableAdapterFactory,
        IChangeNotifier, IDisposable
{
    /**
     * This keeps track of the root adapter factory that delegates to this
     * adapter factory. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected ComposedAdapterFactory parentAdapterFactory;
    
    /**
     * This is used to implement
     * {@link org.eclipse.emf.edit.provider.IChangeNotifier}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected IChangeNotifier changeNotifier = new ChangeNotifier();
    
    /**
     * This keeps track of all the supported types checked by
     * {@link #isFactoryForType isFactoryForType}. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    protected Collection supportedTypes = new ArrayList();
    
    /**
     * This constructs an instance. <!-- begin-user-doc --> <!-- end-user-doc
     * -->
     * 
     * @generated NOT
     */
    public SAMModelerProviderAdapterFactory()
    {
        supportedTypes.add(ILabelFeatureProvider.class);
        supportedTypes.add(IDeletePartnerProvider.class);
        supportedTypes.add(IDeleteObjectProvider.class);
    }
    
    /**
     * This returns the root adapter factory that contains this factory. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public ComposeableAdapterFactory getRootAdapterFactory()
    {
        return parentAdapterFactory == null ? this : parentAdapterFactory.getRootAdapterFactory();
    }
    
    /**
     * This sets the composed adapter factory that contains this factory. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public void setParentAdapterFactory(ComposedAdapterFactory parentAdapterFactory)
    {
        this.parentAdapterFactory = parentAdapterFactory;
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param type the type to test
     * @return true if supported
     * 
     * @generated
     */
    public boolean isFactoryForType(Object type)
    {
        return supportedTypes.contains(type) || super.isFactoryForType(type);
    }
    
    /**
     * This implementation substitutes the factory itself as the key for the
     * adapter. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param notifier the notifier
     * @param type the object to adapt
     * @return the Adapter the created adatper
     * 
     * @generated
     */
    public Adapter adapt(Notifier notifier, Object type)
    {
        return super.adapt(notifier, this);
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param object the object to adapt
     * @param type the type to adapt
     * @return the adapted Object
     * 
     * @generated
     */
    public Object adapt(Object object, Object type)
    {
        if (isFactoryForType(type))
        {
            Object adapter = super.adapt(object, type);
            if (!(type instanceof Class) || (((Class) type).isInstance(adapter)))
            {
                return adapter;
            }
        }
        
        return null;
    }
    
    /**
     * This adds a listener. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param notifyChangedListener the listener to add
     * 
     * @generated
     */
    public void addListener(INotifyChangedListener notifyChangedListener)
    {
        changeNotifier.addListener(notifyChangedListener);
    }
    
    /**
     * This removes a listener. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param notifyChangedListener the listener to remove
     * 
     * @generated
     */
    public void removeListener(INotifyChangedListener notifyChangedListener)
    {
        changeNotifier.removeListener(notifyChangedListener);
    }
    
    /**
     * This delegates to {@link #changeNotifier} and to
     * {@link #parentAdapterFactory}. <!-- begin-user-doc --> <!-- end-user-doc
     * -->
     * 
     * @generated
     */
    public void fireNotifyChanged(Notification notification)
    {
        changeNotifier.fireNotifyChanged(notification);
        
        if (parentAdapterFactory != null)
        {
            parentAdapterFactory.fireNotifyChanged(notification);
        }
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.AbstractState} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private AbstractStateModelerProvider abstractstateModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.AbstractState}.
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Adapter
     * 
     * @generated
     */
    public Adapter createAbstractStateAdapter()
    {
        if (abstractstateModelerProvider == null)
        {
            abstractstateModelerProvider = new AbstractStateModelerProvider(this);
        }
        
        return abstractstateModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.Automaton} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private AutomatonModelerProvider automatonModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.Automaton}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createAutomatonAdapter()
    {
        if (automatonModelerProvider == null)
        {
            automatonModelerProvider = new AutomatonModelerProvider(this);
        }
        
        return automatonModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.InitialState} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private InitialStateModelerProvider initialstateModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.InitialState}.
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Adapter
     * 
     * @generated
     */
    public Adapter createInitialStateAdapter()
    {
        if (initialstateModelerProvider == null)
        {
            initialstateModelerProvider = new InitialStateModelerProvider(this);
        }
        
        return initialstateModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.MacroState} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private MacroStateModelerProvider macrostateModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.MacroState}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createMacroStateAdapter()
    {
        if (macrostateModelerProvider == null)
        {
            macrostateModelerProvider = new MacroStateModelerProvider(this);
        }
        
        return macrostateModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.State} instances. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private StateModelerProvider stateModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.State}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createStateAdapter()
    {
        if (stateModelerProvider == null)
        {
            stateModelerProvider = new StateModelerProvider(this);
        }
        
        return stateModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.Transition} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private TransitionModelerProvider transitionModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.Transition}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createTransitionAdapter()
    {
        if (transitionModelerProvider == null)
        {
            transitionModelerProvider = new TransitionModelerProvider(this);
        }
        
        return transitionModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.ControlPort} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private ControlPortModelerProvider controlportModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.ControlPort}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createControlPortAdapter()
    {
        if (controlportModelerProvider == null)
        {
            controlportModelerProvider = new ControlPortModelerProvider(this);
        }
        
        return controlportModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.DataPort} instances. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private DataPortModelerProvider dataportModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.DataPort}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createDataPortAdapter()
    {
        if (dataportModelerProvider == null)
        {
            dataportModelerProvider = new DataPortModelerProvider(this);
        }
        
        return dataportModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.InControlPort} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private InControlPortModelerProvider incontrolportModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.InControlPort}.
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Adapter
     * 
     * @generated
     */
    public Adapter createInControlPortAdapter()
    {
        if (incontrolportModelerProvider == null)
        {
            incontrolportModelerProvider = new InControlPortModelerProvider(this);
        }
        
        return incontrolportModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.InDataPort} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private InDataPortModelerProvider indataportModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.InDataPort}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createInDataPortAdapter()
    {
        if (indataportModelerProvider == null)
        {
            indataportModelerProvider = new InDataPortModelerProvider(this);
        }
        
        return indataportModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.InputPort} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private InputPortModelerProvider inputportModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.InputPort}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createInputPortAdapter()
    {
        if (inputportModelerProvider == null)
        {
            inputportModelerProvider = new InputPortModelerProvider(this);
        }
        
        return inputportModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.OutControlPort} instances. <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private OutControlPortModelerProvider outcontrolportModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.OutControlPort}.
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Adapter
     * 
     * @generated
     */
    public Adapter createOutControlPortAdapter()
    {
        if (outcontrolportModelerProvider == null)
        {
            outcontrolportModelerProvider = new OutControlPortModelerProvider(this);
        }
        
        return outcontrolportModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.OutDataPort} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private OutDataPortModelerProvider outdataportModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.OutDataPort}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createOutDataPortAdapter()
    {
        if (outdataportModelerProvider == null)
        {
            outdataportModelerProvider = new OutDataPortModelerProvider(this);
        }
        
        return outdataportModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.OutputPort} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private OutputPortModelerProvider outputportModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.OutputPort}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createOutputPortAdapter()
    {
        if (outputportModelerProvider == null)
        {
            outputportModelerProvider = new OutputPortModelerProvider(this);
        }
        
        return outputportModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.Port} instances. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private PortModelerProvider portModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.Port}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createPortAdapter()
    {
        if (portModelerProvider == null)
        {
            portModelerProvider = new PortModelerProvider(this);
        }
        
        return portModelerProvider;
    }
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.MultiPort}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createMultiPortAdapter()
    {
        if (multiportModelerProvider == null)
        {
            multiportModelerProvider = new MultiPortModelerProvider(this);
        }
        
        return multiportModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.Composition} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private CompositionModelerProvider compositionModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.Composition}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createCompositionAdapter()
    {
        if (compositionModelerProvider == null)
        {
            compositionModelerProvider = new CompositionModelerProvider(this);
        }
        
        return compositionModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.ControlFlow} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private ControlFlowModelerProvider controlflowModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.ControlFlow}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createControlFlowAdapter()
    {
        if (controlflowModelerProvider == null)
        {
            controlflowModelerProvider = new ControlFlowModelerProvider(this);
        }
        
        return controlflowModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.DataFlow} instances. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private DataFlowModelerProvider dataflowModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.DataFlow}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createDataFlowAdapter()
    {
        if (dataflowModelerProvider == null)
        {
            dataflowModelerProvider = new DataFlowModelerProvider(this);
        }
        
        return dataflowModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.DataStore} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private DataStoreModelerProvider datastoreModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.DataStore}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createDataStoreAdapter()
    {
        if (datastoreModelerProvider == null)
        {
            datastoreModelerProvider = new DataStoreModelerProvider(this);
        }
        
        return datastoreModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.Decomposition} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private DecompositionModelerProvider decompositionModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.Decomposition}.
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Adapter
     * 
     * @generated
     */
    public Adapter createDecompositionAdapter()
    {
        if (decompositionModelerProvider == null)
        {
            decompositionModelerProvider = new DecompositionModelerProvider(this);
        }
        
        return decompositionModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.Flow} instances. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private FlowModelerProvider flowModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.Flow}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createFlowAdapter()
    {
        if (flowModelerProvider == null)
        {
            flowModelerProvider = new FlowModelerProvider(this);
        }
        
        return flowModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.SynchronisationGate} instances. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private SynchronisationGateModelerProvider synchronisationgateModelerProvider;
    
    /**
     * This creates an adapter for a
     * {@link org.topcased.sam.SynchronisationGate}. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @return the Adapter
     * 
     * @generated
     */
    public Adapter createSynchronisationGateAdapter()
    {
        if (synchronisationgateModelerProvider == null)
        {
            synchronisationgateModelerProvider = new SynchronisationGateModelerProvider(this);
        }
        
        return synchronisationgateModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.System} instances. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private SystemModelerProvider systemModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.System}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createSystemAdapter()
    {
        if (systemModelerProvider == null)
        {
            systemModelerProvider = new SystemModelerProvider(this);
        }
        
        return systemModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.Model} instances. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private ModelModelerProvider modelModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.Model}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createModelAdapter()
    {
        if (modelModelerProvider == null)
        {
            modelModelerProvider = new ModelModelerProvider(this);
        }
        
        return modelModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.ModelContent} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private ModelContentModelerProvider modelcontentModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.ModelContent}.
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Adapter
     * 
     * @generated
     */
    public Adapter createModelContentAdapter()
    {
        if (modelcontentModelerProvider == null)
        {
            modelcontentModelerProvider = new ModelContentModelerProvider(this);
        }
        
        return modelcontentModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.IdentifiedItem} instances. <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private IdentifiedItemModelerProvider identifieditemModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.IdentifiedItem}.
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Adapter
     * 
     * @generated
     */
    public Adapter createIdentifiedItemAdapter()
    {
        if (identifieditemModelerProvider == null)
        {
            identifieditemModelerProvider = new IdentifiedItemModelerProvider(this);
        }
        
        return identifieditemModelerProvider;
    }
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.NamedItem} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private NamedItemModelerProvider nameditemModelerProvider;
    
    /**
     * This keeps track of the one adapter used for all
     * {@link org.topcased.sam.MultiPort} instances. <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     * 
     * @generated
     */
    private MultiPortModelerProvider multiportModelerProvider;
    
    /**
     * This creates an adapter for a {@link org.topcased.sam.NamedItem}. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public Adapter createNamedItemAdapter()
    {
        if (nameditemModelerProvider == null)
        {
            nameditemModelerProvider = new NamedItemModelerProvider(this);
        }
        
        return nameditemModelerProvider;
    }
    
    /**
     * This disposes all of the item providers created by this factory. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public void dispose()
    {
        if (abstractstateModelerProvider != null)
            abstractstateModelerProvider.dispose();
        if (automatonModelerProvider != null)
            automatonModelerProvider.dispose();
        if (initialstateModelerProvider != null)
            initialstateModelerProvider.dispose();
        if (macrostateModelerProvider != null)
            macrostateModelerProvider.dispose();
        if (stateModelerProvider != null)
            stateModelerProvider.dispose();
        if (transitionModelerProvider != null)
            transitionModelerProvider.dispose();
        if (controlportModelerProvider != null)
            controlportModelerProvider.dispose();
        if (dataportModelerProvider != null)
            dataportModelerProvider.dispose();
        if (incontrolportModelerProvider != null)
            incontrolportModelerProvider.dispose();
        if (indataportModelerProvider != null)
            indataportModelerProvider.dispose();
        if (inputportModelerProvider != null)
            inputportModelerProvider.dispose();
        if (outcontrolportModelerProvider != null)
            outcontrolportModelerProvider.dispose();
        if (outdataportModelerProvider != null)
            outdataportModelerProvider.dispose();
        if (outputportModelerProvider != null)
            outputportModelerProvider.dispose();
        if (portModelerProvider != null)
            portModelerProvider.dispose();
        if (compositionModelerProvider != null)
            compositionModelerProvider.dispose();
        if (controlflowModelerProvider != null)
            controlflowModelerProvider.dispose();
        if (dataflowModelerProvider != null)
            dataflowModelerProvider.dispose();
        if (datastoreModelerProvider != null)
            datastoreModelerProvider.dispose();
        if (decompositionModelerProvider != null)
            decompositionModelerProvider.dispose();
        if (flowModelerProvider != null)
            flowModelerProvider.dispose();
        if (synchronisationgateModelerProvider != null)
            synchronisationgateModelerProvider.dispose();
        if (systemModelerProvider != null)
            systemModelerProvider.dispose();
        if (modelModelerProvider != null)
            modelModelerProvider.dispose();
        if (modelcontentModelerProvider != null)
            modelcontentModelerProvider.dispose();
        if (identifieditemModelerProvider != null)
            identifieditemModelerProvider.dispose();
        if (nameditemModelerProvider != null)
            nameditemModelerProvider.dispose();
        if (multiportModelerProvider != null)
            multiportModelerProvider.dispose();
    }
    
}
