/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.systemdiagram;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.sam.edit.InControlPortEditPart;
import org.topcased.modeler.sam.edit.InDataPortEditPart;
import org.topcased.modeler.sam.edit.MultiPortEditPart;
import org.topcased.modeler.sam.edit.OutControlPortEditPart;
import org.topcased.modeler.sam.edit.OutDataPortEditPart;
import org.topcased.modeler.sam.systemdiagram.edit.AutomatonEditPart;
import org.topcased.modeler.sam.systemdiagram.edit.CompositionEditPart;
import org.topcased.modeler.sam.systemdiagram.edit.ControlFlowEditPart;
import org.topcased.modeler.sam.systemdiagram.edit.DataFlowEditPart;
import org.topcased.modeler.sam.systemdiagram.edit.DataStoreEditPart;
import org.topcased.modeler.sam.systemdiagram.edit.DecompositionEditPart;
import org.topcased.modeler.sam.systemdiagram.edit.SystDiagramEditPart;
import org.topcased.modeler.sam.systemdiagram.edit.SystemEditPart;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.util.SAMSwitch;

/**
 * Part Factory : associates a model object to its controller. <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class SystEditPartFactory implements EditPartFactory
{
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
     * @generated
     */
    public EditPart createEditPart(EditPart context, Object model)
    {
        if (model instanceof Diagram)
        {
            return new SystDiagramEditPart((Diagram) model);
        }
        else if (model instanceof GraphNode)
        {
            final GraphNode node = (GraphNode) model;
            EObject element = Utils.getElement(node);
            if (element != null)
            {
                if ("http://www.topcased.org/SAM/1.0".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new NodeSAMSwitch(node).doSwitch(element);
                }
                return new EMFGraphNodeEditPart(node);
            }
            return new GraphNodeEditPart(node);
        }
        else if (model instanceof GraphEdge)
        {
            final GraphEdge edge = (GraphEdge) model;
            EObject element = Utils.getElement(edge);
            if (element != null)
            {
                if ("http://www.topcased.org/SAM/1.0".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new EdgeSAMSwitch(edge).doSwitch(element);
                }
                return new EMFGraphEdgeEditPart(edge);
            }
            return new GraphEdgeEditPart(edge);
        }
        
        throw new IllegalStateException("No edit part matches with the '" + model.getClass().getName()
                + "' model element. Check SystEditPartFactory#createEditPart(EditPart,Object) class");
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class NodeSAMSwitch extends SAMSwitch<Object>
    {
        /** The graphical node */
        private GraphNode node;
        
        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param node the graphical node
         * @generated
         */
        public NodeSAMSwitch(GraphNode node)
        {
            this.node = node;
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseSystem(org.topcased.sam.System)
         * @generated
         */
        public Object caseSystem(org.topcased.sam.System object)
        {
            return new SystemEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseAutomaton(org.topcased.sam.Automaton)
         * @generated
         */
        public Object caseAutomaton(org.topcased.sam.Automaton object)
        {
            return new AutomatonEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseDataStore(org.topcased.sam.DataStore)
         * @generated
         */
        public Object caseDataStore(org.topcased.sam.DataStore object)
        {
            return new DataStoreEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseComposition(org.topcased.sam.Composition)
         * @generated
         */
        public Object caseComposition(org.topcased.sam.Composition object)
        {
            return new CompositionEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseDecomposition(org.topcased.sam.Decomposition)
         * @generated
         */
        public Object caseDecomposition(org.topcased.sam.Decomposition object)
        {
            return new DecompositionEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseInControlPort(org.topcased.sam.InControlPort)
         * @generated
         */
        public Object caseInControlPort(org.topcased.sam.InControlPort object)
        {
            return new InControlPortEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseOutControlPort(org.topcased.sam.OutControlPort)
         * @generated
         */
        public Object caseOutControlPort(org.topcased.sam.OutControlPort object)
        {
            return new OutControlPortEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseInDataPort(org.topcased.sam.InDataPort)
         * @generated
         */
        public Object caseInDataPort(org.topcased.sam.InDataPort object)
        {
            return new InDataPortEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseOutDataPort(org.topcased.sam.OutDataPort)
         * @generated
         */
        public Object caseOutDataPort(org.topcased.sam.OutDataPort object)
        {
            return new OutDataPortEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseMultiPort(org.topcased.sam.MultiPort)
         * @generated
         */
        public Object caseMultiPort(org.topcased.sam.MultiPort object)
        {
            return new MultiPortEditPart(node);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphNodeEditPart(node);
        }
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class EdgeSAMSwitch extends SAMSwitch<Object>
    {
        /** The graphical edge */
        private GraphEdge edge;
        
        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param edge the graphical edge
         * @generated
         */
        public EdgeSAMSwitch(GraphEdge edge)
        {
            this.edge = edge;
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseDataFlow(org.topcased.sam.DataFlow)
         * @generated
         */
        public Object caseDataFlow(org.topcased.sam.DataFlow object)
        {
            return new DataFlowEditPart(edge);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#caseControlFlow(org.topcased.sam.ControlFlow)
         * @generated
         */
        public Object caseControlFlow(org.topcased.sam.ControlFlow object)
        {
            return new ControlFlowEditPart(edge);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.topcased.sam.util.SAMSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphEdgeEditPart(edge);
        }
    }
    
}
