/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.systemdiagram;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.palette.PaletteSeparator;
import org.eclipse.gef.requests.CreationFactory;
import org.topcased.modeler.editor.GraphElementCreationFactory;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.palette.ModelerConnectionCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerPaletteManager;
import org.topcased.sam.Composition;
import org.topcased.sam.DataStore;
import org.topcased.sam.Decomposition;
import org.topcased.sam.InDataPort;
import org.topcased.sam.OutDataPort;
import org.topcased.sam.SAMFactory;
import org.topcased.sam.SAMPackage;

/**
 * Generated Palette Manager <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class SystPaletteManager extends ModelerPaletteManager
{

    // declare all the palette categories of the diagram
    private PaletteDrawer objectsDrawer;

    private PaletteDrawer connectionsDrawer;

    private ICreationUtils creationUtils;

    /**
     * The Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param utils the creation utils for the tools of the palette
     * @generated
     */
    public SystPaletteManager(ICreationUtils utils)
    {
        super();
        this.creationUtils = utils;
    }

    /**
     * Creates the main categories of the palette <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createCategories()
    {
        createObjectsDrawer();
        createConnectionsDrawer();
    }

    /**
     * Updates the main categories of the palette <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void updateCategories()
    {
        // deletion of the existing categories and creation of the updated
        // categories

        getRoot().remove(objectsDrawer);
        createObjectsDrawer();

        getRoot().remove(connectionsDrawer);
        createConnectionsDrawer();
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated NOT
     */
    private void createObjectsDrawer()
    {
        objectsDrawer = new PaletteDrawer("Objects", null);
        List<PaletteEntry> entries = new ArrayList<PaletteEntry>();

        CreationFactory factory;

        factory = new GraphElementCreationFactory(creationUtils, SAMPackage.eINSTANCE.getSystem(), "default");
        entries.add(new ModelerCreationToolEntry("System", "System", factory, SystImageRegistry.getImageDescriptor("SYSTEM"), SystImageRegistry.getImageDescriptor("SYSTEM_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, SAMPackage.eINSTANCE.getAutomaton(), "default");
        entries.add(new ModelerCreationToolEntry("Automaton", "Automaton", factory, SystImageRegistry.getImageDescriptor("AUTOMATON"), SystImageRegistry.getImageDescriptor("AUTOMATON_LARGE")));

        entries.add(new PaletteSeparator());

        factory = new GraphElementCreationFactory(creationUtils, SAMPackage.eINSTANCE.getDataStore(), "default")
        {
            public EObject getNewModelObject()
            {
                DataStore element = (DataStore) super.getNewModelObject();
                EObject inDataPort = SAMFactory.eINSTANCE.createInDataPort();
                EObject outDataPort = SAMFactory.eINSTANCE.createOutDataPort();
                element.setIn((InDataPort) inDataPort);
                element.setOut((OutDataPort) outDataPort);
                ((InDataPort) inDataPort).setParentDataStore(element);
                ((OutDataPort) outDataPort).setParentDataStore(element);
                return element;
            }
        };
        entries.add(new ModelerCreationToolEntry("DataStore", "DataStore", factory, SystImageRegistry.getImageDescriptor("DATASTORE"), SystImageRegistry.getImageDescriptor("DATASTORE_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, SAMPackage.eINSTANCE.getComposition(), "default")
        {
            public EObject getNewModelObject()
            {
                Composition element = (Composition) super.getNewModelObject();
                InDataPort inDataPort1 = SAMFactory.eINSTANCE.createInDataPort();
                InDataPort inDataPort2 = SAMFactory.eINSTANCE.createInDataPort();
                OutDataPort outDataPort1 = SAMFactory.eINSTANCE.createOutDataPort();
                element.getIn().add(inDataPort1);
                element.getIn().add(inDataPort2);
                element.getOut().add(outDataPort1);
                inDataPort1.setParentSync(element);
                inDataPort2.setParentSync(element);
                outDataPort1.setParentSync(element);
                return element;
            }
        };
        entries.add(new ModelerCreationToolEntry("Composition", "Composition", factory, SystImageRegistry.getImageDescriptor("COMPOSITION"), SystImageRegistry.getImageDescriptor("COMPOSITION_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, SAMPackage.eINSTANCE.getDecomposition(), "default")
        {
            public EObject getNewModelObject()
            {
                Decomposition element = (Decomposition) super.getNewModelObject();

                InDataPort inDataPort3 = SAMFactory.eINSTANCE.createInDataPort();
                OutDataPort outDataPort2 = SAMFactory.eINSTANCE.createOutDataPort();
                OutDataPort outDataPort3 = SAMFactory.eINSTANCE.createOutDataPort();
                element.getIn().add(inDataPort3);
                element.getOut().add(outDataPort2);
                element.getOut().add(outDataPort3);
                inDataPort3.setParentSync(element);
                outDataPort2.setParentSync(element);
                outDataPort3.setParentSync(element);
                return element;
            }
        };
        entries.add(new ModelerCreationToolEntry("Decomposition", "Decomposition", factory, SystImageRegistry.getImageDescriptor("DECOMPOSITION"),
                SystImageRegistry.getImageDescriptor("DECOMPOSITION_LARGE")));

        entries.add(new PaletteSeparator());

        factory = new GraphElementCreationFactory(creationUtils, SAMPackage.eINSTANCE.getMultiPort(), "default");
        entries.add(new ModelerCreationToolEntry("MultiPort", "MultiPort", factory, SystImageRegistry.getImageDescriptor("MULTIPORT"), SystImageRegistry.getImageDescriptor("MULTIPORT_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, SAMPackage.eINSTANCE.getInControlPort(), "default");
        entries.add(new ModelerCreationToolEntry("InControlPort", "InControlPort", factory, SystImageRegistry.getImageDescriptor("INCONTROLPORT"),
                SystImageRegistry.getImageDescriptor("INCONTROLPORT_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, SAMPackage.eINSTANCE.getOutControlPort(), "default");
        entries.add(new ModelerCreationToolEntry("OutControlPort", "OutControlPort", factory, SystImageRegistry.getImageDescriptor("OUTCONTROLPORT"),
                SystImageRegistry.getImageDescriptor("OUTCONTROLPORT_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, SAMPackage.eINSTANCE.getInDataPort(), "default");
        entries.add(new ModelerCreationToolEntry("InDataPort", "InDataPort", factory, SystImageRegistry.getImageDescriptor("INDATAPORT"), SystImageRegistry.getImageDescriptor("INDATAPORT_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, SAMPackage.eINSTANCE.getOutDataPort(), "default");
        entries.add(new ModelerCreationToolEntry("OutDataPort", "OutDataPort", factory, SystImageRegistry.getImageDescriptor("OUTDATAPORT"), SystImageRegistry.getImageDescriptor("OUTDATAPORT_LARGE")));

        objectsDrawer.addAll(entries);
        getRoot().add(objectsDrawer);
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private void createConnectionsDrawer()
    {
        connectionsDrawer = new PaletteDrawer("Connections", null);
        List<PaletteEntry> entries = new ArrayList<PaletteEntry>();

        CreationFactory factory;

        factory = new GraphElementCreationFactory(creationUtils, SAMPackage.eINSTANCE.getDataFlow(), "default");
        entries.add(new ModelerConnectionCreationToolEntry("DataFlow", "DataFlow", factory, SystImageRegistry.getImageDescriptor("DATAFLOW"), SystImageRegistry.getImageDescriptor("DATAFLOW_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, SAMPackage.eINSTANCE.getControlFlow(), "default");
        entries.add(new ModelerConnectionCreationToolEntry("ControlFlow", "ControlFlow", factory, SystImageRegistry.getImageDescriptor("CONTROLFLOW"),
                SystImageRegistry.getImageDescriptor("CONTROLFLOW_LARGE")));

        connectionsDrawer.addAll(entries);
        getRoot().add(connectionsDrawer);
    }

}
