/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.sam.systemdiagram.commands;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.topcased.modeler.commands.CreateGraphEdgeCommand;
import org.topcased.modeler.di.model.EMFSemanticModelBridge;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.ControlFlow;
import org.topcased.sam.ControlPort;
import org.topcased.sam.DataFlow;
import org.topcased.sam.DataPort;
import org.topcased.sam.Flow;
import org.topcased.sam.Port;
import org.topcased.sam.System;

/**
 * <b>Creation command for DataFlow and ControlFlow</b> <br>
 * Creation : 06 June 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class CreateFlowCommand extends CreateGraphEdgeCommand
{
    /**
     * Constructor
     * 
     * @param domain the EditDomain (used to init the name)
     * @param newObj the edge to create
     * @param src the source node
     * @param needModelUpdate
     */
    public CreateFlowCommand(EditDomain domain, GraphEdge newObj, GraphNode src, boolean needModelUpdate)
    {
        super(domain, newObj, src, needModelUpdate);
    }

    /**
     * @see org.topcased.modeler.commands.CreateGraphEdgeCommand#redoModel()
     */
    protected void redoModel()
    {
        // retrieve the model objects
        Port sourcePort = (Port) Utils.getElement(getSource());
        Port targetPort = (Port) Utils.getElement(getTarget());
        Flow flow = (Flow) Utils.getElement(getEdge());

        if (flow instanceof DataFlow)
        {
            // check if the sourcePort has already an outlink
            if (sourcePort.getOutlink() != null)
            {
                // the Flow already exist
                flow = sourcePort.getOutlink();
                ((EMFSemanticModelBridge) getEdge().getSemanticModel()).setElement(flow);
            }
            else
            {
                // add the new Flow to the model
                EObject diagramModelObject = Utils.getDiagramModelObject(getSource());
                ((System) diagramModelObject).getListFlows().add(flow);
                ((DataFlow) flow).setSource((DataPort) sourcePort);
                sourcePort.setOutlink(flow);
            }

            ((DataFlow) flow).getDest().add((DataPort) targetPort);
            targetPort.setInlink(flow);
        }

        else if (flow instanceof ControlFlow)
        {
            // check if the sourcePort has already an outlink
            if (sourcePort.getOutlink() != null)
            {
                // the Flow already exist
                flow = sourcePort.getOutlink();
                ((EMFSemanticModelBridge) getEdge().getSemanticModel()).setElement(flow);
            }
            else
            {
                // add the new Flow to the model
                EObject diagramModelObject = Utils.getDiagramModelObject(getSource());
                ((System) diagramModelObject).getListFlows().add(flow);
                ((ControlFlow) flow).setSource((ControlPort) sourcePort);
                sourcePort.setOutlink(flow);
            }

            ((ControlFlow) flow).setSource((ControlPort) sourcePort);
            ((ControlFlow) flow).getDest().add((ControlPort) targetPort);
            targetPort.setInlink(flow);
        }

        // -------------------------------------------
        // When connecting two ports, the tool should choose a name for the new
        // flow :
        // - copy the name of the flow connected to the other side of the origin
        // port if it exists,
        // - copy the name of the flow connected to the other side of the
        // destination port if it exists (and is unique).
        if (sourcePort.getInlink() != null && sourcePort.getInlink().getName() != null && !"".equals(sourcePort.getInlink().getName()))
        {
            flow.setName(sourcePort.getInlink().getName());
        }
        else if (sourcePort.getName() != null && !"".equals(sourcePort.getName()))
        {
            flow.setName(sourcePort.getName());
        }
        else
        {
            initName();
        }
        // -------------------------------------------
    }

    /**
     * @see org.topcased.modeler.commands.CreateGraphEdgeCommand#undoModel()
     */
    protected void undoModel()
    {
        Port sourcePort = (Port) Utils.getElement(getSource());
        Port targetPort = (Port) Utils.getElement(getTarget());
        Flow flow = (Flow) Utils.getElement(getEdge());

        EObject diagramModelObject = Utils.getDiagramModelObject(getSource());

        if (flow instanceof DataFlow)
        {
            // check if the flow is connected to more than one port
            if (((DataFlow) flow).getDest().size() > 1)
            {
                ((DataFlow) flow).getDest().remove(flow);
            }
            else
            {
                ((System) diagramModelObject).getListFlows().remove(flow);
                sourcePort.setOutlink(null);
                ((DataFlow) flow).setSource(null);
            }
            ((DataFlow) flow).getDest().remove(targetPort);
            targetPort.setInlink(null);
        }
        else if (flow instanceof ControlFlow)
        {
            if (((ControlFlow) flow).getDest().size() > 1)
            {
                ((ControlFlow) flow).getDest().remove(flow);
            }
            else
            {
                ((System) diagramModelObject).getListFlows().remove(flow);
                sourcePort.setOutlink(null);
                ((ControlFlow) flow).setSource(null);
            }
            ((ControlFlow) flow).getDest().remove(targetPort);
            targetPort.setInlink(null);
        }

    }

}
