/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.systemdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.jface.action.IAction;

import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.sam.SAMEditPolicyConstants;
import org.topcased.modeler.sam.edit.policies.CustomEditPolicy;
import org.topcased.modeler.sam.edit.policies.SAMPortLayoutEditPolicy;
import org.topcased.modeler.sam.edit.policies.SamLabelDirectEditPolicy;
import org.topcased.modeler.sam.systemdiagram.commands.RestorePortsCommand;
import org.topcased.modeler.sam.systemdiagram.figures.AutomatonFigure;
import org.topcased.modeler.sam.util.InstancesUtil;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.Automaton;
import org.topcased.sam.SAMPackage;

/**
 * The Automaton object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class AutomatonEditPart extends EMFGraphNodeEditPart
{
    /**
     * Constructor.
     * 
     * Added a check to make sure that instances reflet the current state of the
     * external model on creation.
     * 
     * @param obj the graph node
     * @generated NOT
     */
    public AutomatonEditPart(GraphNode obj)
    {
        super(obj);
        // update the associated automaton if it is an instance
        EObject referencer = Utils.getElement(obj);
        if (InstancesUtil.isAReference(referencer) && (referencer instanceof Automaton))
        {
            Automaton referencerAutomaton = (Automaton) referencer;
            Automaton referencedAutomaton = referencerAutomaton.getIsInstanceOf();

            InstancesUtil.updateAllContents(referencerAutomaton, referencedAutomaton);
        }
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    @Override
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new SAMPortLayoutEditPolicy());

        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());

        installEditPolicy(SAMEditPolicyConstants.CUSTOM_EDITPOLICY, new CustomEditPolicy());

    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated NOT
     */
    @Override
    protected IFigure createFigure()
    {
        // REV weird code ....use a command ?
        RestorePortsCommand restorePortsCommand = new RestorePortsCommand(getViewer().getEditDomain(),
                (GraphNode) getModel(),
                ((ModelerGraphicalViewer) getViewer()).getModelerEditor().getActiveConfiguration().getCreationUtils());
        if ((restorePortsCommand != null) && restorePortsCommand.canExecute())
        {
            restorePortsCommand.execute();
        }

        return new AutomatonFigure();
    }

    // /**
    // * Update the ComposedLabel with the correct Suffix label (if necessary)
    // and
    // * the right icon and font for the Main EditableLabel
    // *
    // * @see
    // org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
    // */
    // protected void refreshHeaderLabel()
    // {
    // AutomatonFigure fig = (AutomatonFigure) getFigure();
    // ComposedLabel lbl = (ComposedLabel) fig.getLabel();
    // if (getModelAutomaton().getName() != null)
    // lbl.setMain(getModelAutomaton().getName());
    //
    // // check if the class is not included in the EPackage of the current
    // diagram
    // if (getModelAutomaton().getParentSystem() != null &&
    // getModelAutomaton().getParentSystem() != Utils.getElement(((Diagram)
    // getParent().getModel()).getSemanticModel().getGraphElement()))
    // {
    // if (getModelAutomaton().getParentSystem().getName() != null)
    // {
    // lbl.setSuffix("<<from
    // "+getModelAutomaton().getParentSystem().getName()+">>");
    // }
    // }
    // else if (getModelAutomaton().getParentModel() != null)
    // {
    // lbl.setSuffix("<<from " + getModelAutomaton().getName() + ".sam>>");
    // }
    // else
    // {
    // lbl.setSuffix("");
    // }
    // }

    /**
     * @see org.topcased.modeler.edit.BaseEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    @Override
    protected void handleModelChanged(Notification msg)
    {
        if (msg.getNotifier() instanceof Automaton)
        {
            switch (msg.getFeatureID(Automaton.class))
            {
                case SAMPackage.AUTOMATON__LIST_PORTS:
                case SAMPackage.AUTOMATON__LIST_MULTI_PORT:
                    // update graphically all the ports that may have been added
                    // in the model so that they are synchronized
                    // REV weird code ....use a command ?
                    RestorePortsCommand restorePortsCommand = new RestorePortsCommand(
                            getViewer().getEditDomain(),
                            (GraphNode) getModel(),
                            ((ModelerGraphicalViewer) getViewer()).getModelerEditor().getActiveConfiguration().getCreationUtils());
                    if ((restorePortsCommand != null) && restorePortsCommand.canExecute())
                    {
                        restorePortsCommand.execute();
                    }
                    break;
            }
        }

        super.handleModelChanged(msg);
    }

    /**
     * Overriden so as to provide custom behaviour when the modelObject is an
     * imported system or automaton When an imported system is double-clicked,
     * it will try to load the associated '.samdi'.
     * 
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#createChangeDiagramAction(org.eclipse.emf.ecore.EObject)
     */
    @Override
    protected IAction createChangeDiagramAction(EObject modelObject)
    {
        if ((modelObject instanceof Automaton) && InstancesUtil.isAReference(modelObject))
        {
            Automaton emptyAutomaton = ((Automaton) modelObject);
            return new OpenReferencedModel(((ModelerGraphicalViewer) getViewer()).getModelerEditor(), emptyAutomaton,
                    emptyAutomaton.getIsInstanceOf());
        }
        return super.createChangeDiagramAction(modelObject);
    }

}
