/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.systemdiagram.edit;

import java.util.Iterator;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPolicy;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.Property;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.listeners.DiagramElementListener;
import org.topcased.modeler.sam.SAMEditPolicyConstants;
import org.topcased.modeler.sam.SAMPropertyConstants;
import org.topcased.modeler.sam.edit.policies.CustomEditPolicy;
import org.topcased.modeler.sam.systemdiagram.figures.DataStoreFigure;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.InDataPort;
import org.topcased.sam.OutDataPort;

/**
 * The DataStore object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class DataStoreEditPart extends EMFGraphNodeEditPart
{
    /**
     * This listener also listens the mutliport state property
     * 
     * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
     */
    protected class DataStoreEditPartListener extends GraphNodeEditPartListener
    {
        /**
         * @see org.topcased.modeler.edit.GraphNodeEditPart.GraphNodeEditPartListener#handlePropertyChanged(org.topcased.modeler.di.model.Property,
         *      org.topcased.modeler.di.model.Property)
         */
        protected void handlePropertyChanged(Property oldProperty, Property newProperty)
        {
            super.handlePropertyChanged(oldProperty, newProperty);
            
            Property property = newProperty != null ? newProperty : oldProperty;
            
            if (SAMPropertyConstants.DATASTORE_TYPE.equals(property.getKey()))
            {
                refreshVisuals();
            }
        }
        
    }
    
    private DiagramElementListener diagramListener = new DataStoreEditPartListener();
    
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public DataStoreEditPart(GraphNode obj)
    {
        super(obj);
    }
    
    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getDiagramElementListener()
     */
    protected DiagramElementListener getDiagramElementListener()
    {
        return diagramListener;
    }
    
    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();
        
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
        
        ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(PositionConstants.EAST_WEST);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);
        
        installEditPolicy(ModelerEditPolicyConstants.CHANGE_BACKGROUND_COLOR_EDITPOLICY, null);
        
        installEditPolicy(SAMEditPolicyConstants.CUSTOM_EDITPOLICY, new CustomEditPolicy());
    }
    
    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#refreshConstraints()
     */
    protected void refreshConstraints()
    {
        super.refreshConstraints();
        
        // place the ports
        for (Iterator i = getModelChildren().iterator(); i.hasNext();)
        {
            GraphElement elt = (GraphElement) i.next();
            
            int dataStoreWidth;
            if (getGraphNode().getSize() == null)
            {
                dataStoreWidth = getDefaultWidth();
            }
            else
            {
                dataStoreWidth = getGraphNode().getSize().width;
            }
            if (Utils.getElement(elt) instanceof InDataPort)
                if (isTypeLeft())
                    elt.setPosition(new Point(dataStoreWidth / 2 - 20, 1));
                else
                    elt.setPosition(new Point(dataStoreWidth / 2 + 10, 1));
            if (Utils.getElement(elt) instanceof OutDataPort)
                if (isTypeLeft())
                    elt.setPosition(new Point(dataStoreWidth / 2 + 10, 1));
                else
                    elt.setPosition(new Point(dataStoreWidth / 2 - 20, 1));
        }
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated NOT
     */
    protected IFigure createFigure()
    {
        return new DataStoreFigure();
    }
    
    /**
     * @return Returns the type_left.
     */
    public boolean isTypeLeft()
    {
        Property property = DIUtils.getProperty(getGraphNode(), SAMPropertyConstants.DATASTORE_TYPE);
        if (property != null && property.getValue().equals(SAMPropertyConstants.DATASTORE_TYPE_RIGHT))
        {
            return false;
        }
        return true;
    }
    
}
