/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.systemdiagram.edit;

import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Orientable;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPolicy;
import org.eclipse.swt.graphics.Color;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.Property;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.listeners.DiagramElementListener;
import org.topcased.modeler.sam.SAMEditPolicyConstants;
import org.topcased.modeler.sam.edit.policies.CustomEditPolicy;
import org.topcased.modeler.sam.systemdiagram.figures.DecompositionFigure;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.Decomposition;
import org.topcased.sam.InDataPort;
import org.topcased.sam.OutDataPort;

/**
 * The Decomposition object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques Lescot </a>
 * @generated NOT
 */
public class DecompositionEditPart extends EMFGraphNodeEditPart
{
    // The resizableEditPolicy that may change when the orientation changes
    private ResizableEditPolicy resizableEditPolicy;

    /**
     * This listener also listens the orientation property of the graphical element
     */
    protected class CompositionEditPartListener extends GraphNodeEditPartListener
    {
        /**
         * @see org.topcased.modeler.edit.GraphNodeEditPart.GraphNodeEditPartListener#handlePropertyChanged(org.topcased.modeler.di.model.Property,
         *      org.topcased.modeler.di.model.Property)
         */
        protected void handlePropertyChanged(Property oldProperty, Property newProperty)
        {
            super.handlePropertyChanged(oldProperty, newProperty);

            Property property = newProperty != null ? newProperty : oldProperty;

            if (ModelerPropertyConstants.DIRECTION.equals(property.getKey()))
            {
                resizableEditPolicy.setResizeDirections(getDirection() == PositionConstants.EAST || getDirection() == PositionConstants.WEST ? PositionConstants.NORTH_SOUTH
                        : PositionConstants.EAST_WEST);
                ((Orientable) getFigure()).setDirection(getDirection());
                refreshVisuals();
            }
        }
    }

    private DiagramElementListener diagramListener = new CompositionEditPartListener();

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj
     * @generated
     */
    public DecompositionEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getDiagramElementListener()
     */
    protected DiagramElementListener getDiagramElementListener()
    {
        return diagramListener;
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());

        resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(getDirection() == PositionConstants.EAST || getDirection() == PositionConstants.WEST ? PositionConstants.NORTH_SOUTH : PositionConstants.EAST_WEST);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

        installEditPolicy(ModelerEditPolicyConstants.CHANGE_BACKGROUND_COLOR_EDITPOLICY, null);
        installEditPolicy(ModelerEditPolicyConstants.CHANGE_FONT_EDITPOLICY, null);

        installEditPolicy(SAMEditPolicyConstants.CUSTOM_EDITPOLICY, new CustomEditPolicy());
    }

    /**
     * Get the orientation
     * 
     * @return return the direction of the graphical element
     */
    public int getDirection()
    {
        Property directionProperty = DIUtils.getProperty(getGraphNode(), ModelerPropertyConstants.DIRECTION);
        return directionProperty == null || "".equals(directionProperty.getValue()) ? PositionConstants.EAST : Integer.parseInt(directionProperty.getValue());
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#refreshConstraints()
     */
    protected void refreshConstraints()
    {
        super.refreshConstraints();

        updatePorts();
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#refreshColors()
     */
    protected void refreshColors()
    {
        // Refresh the foreground and background color of the graphical element
        // that are the same.
        Color color = Utils.getForegroundColor(getGraphNode());
        if (color == null)
        {
            color = getDefaultForegroundColor();
        }
        getFigure().setBackgroundColor(color);
        getFigure().setForegroundColor(color);
    }

    /**
     * Update the DataPorts
     * 
     */
    private void updatePorts()
    {
        int nbOutPorts = getNbOutPorts();

        DecompositionFigure fig = (DecompositionFigure) getFigure();
        if (fig.getNbOutDataPorts() != nbOutPorts)
        {
            fig.setNbOutDataPorts(nbOutPorts);
            fig.repaint();
        }

        int decompositionWidth;
        int decompositionHeight;

        if (getGraphNode().getSize() == null)
        {
            decompositionWidth = getDefaultWidth();
            decompositionHeight = getDefaultHeight();
        }
        else
        {
            decompositionWidth = getGraphNode().getSize().width;
            decompositionHeight = getGraphNode().getSize().height;
        }

        // place the ports
        int cpt = 1;
        switch (getDirection())
        {
            case PositionConstants.SOUTH:
                for (GraphElement elt : (List<GraphElement>) getModelChildren())
                {
                    if (Utils.getElement(elt) instanceof OutDataPort)
                    {
                        elt.setPosition(new Point((decompositionWidth * cpt) / (2 * nbOutPorts) - 5, decompositionHeight - 10));
                        cpt = cpt + 2;
                    }

                    if (Utils.getElement(elt) instanceof InDataPort)
                        elt.setPosition(new Point(decompositionWidth / 2 - 5, 0));
                }
                break;
            case PositionConstants.WEST:
                for (GraphElement elt : (List<GraphElement>) getModelChildren())
                {
                    if (Utils.getElement(elt) instanceof OutDataPort)
                    {
                        elt.setPosition(new Point(0, (decompositionHeight * cpt) / (2 * nbOutPorts) - 5));
                        cpt = cpt + 2;
                    }

                    if (Utils.getElement(elt) instanceof InDataPort)
                        elt.setPosition(new Point(decompositionWidth - 10, decompositionHeight / 2 - 5));
                }
                break;
            case PositionConstants.NORTH:
                for (GraphElement elt : (List<GraphElement>) getModelChildren())
                {
                    if (Utils.getElement(elt) instanceof OutDataPort)
                    {
                        elt.setPosition(new Point((decompositionWidth * cpt) / (2 * nbOutPorts) - 5, 0));
                        cpt = cpt + 2;
                    }

                    if (Utils.getElement(elt) instanceof InDataPort)
                        elt.setPosition(new Point(decompositionWidth / 2 - 5, decompositionHeight - 10));
                }
                break;
            default:
                for (GraphElement elt : (List<GraphElement>) getModelChildren())
                {
                    if (Utils.getElement(elt) instanceof OutDataPort)
                    {
                        elt.setPosition(new Point(decompositionWidth - 10, (decompositionHeight * cpt) / (2 * nbOutPorts) - 5));
                        cpt = cpt + 2;
                    }

                    if (Utils.getElement(elt) instanceof InDataPort)
                        elt.setPosition(new Point(0, decompositionHeight / 2 - 5));
                }
                break;
        }
    }

    private int getNbOutPorts()
    {
        return ((Decomposition) getEObject()).getOut().size();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated NOT
     */
    protected IFigure createFigure()
    {
        DecompositionFigure fig = new DecompositionFigure();
        fig.setDirection(getDirection());
        return fig;
    }

}
