/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.systemdiagram.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Orientable;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.PointList;
import org.topcased.draw2d.figures.IContainerFigure;

/**
 * The figure to display a Decomposition. <!-- begin-user-doc --> <!--
 * end-user-doc -->
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques Lescot </a>
 * @generated NOT
 */
public class DecompositionFigure extends Figure implements Orientable, IContainerFigure
{
    /** The direction this triangle will face. Possible values are 
     * {@link PositionConstants#NORTH}, {@link PositionConstants#SOUTH}, 
     * {@link PositionConstants#EAST} and {@link PositionConstants#WEST}.
     */
    protected int direction = EAST;
    /** The orientation of this triangle.  Possible values are {@link Orientable#VERTICAL} 
     * and {@link Orientable#HORIZONTAL}.
     */
    protected int orientation = HORIZONTAL;

    private static final int ARROW_SIZE = 4;
    
    private int nbOutDataPorts = 2;
    
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    public DecompositionFigure()
    {
        setLayoutManager(new XYLayout());
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
     * @generated NOT
     */
    protected void paintFigure(Graphics graphics)
    {
        super.paintFigure(graphics);
        
        graphics.pushState();
        
        graphics.setLineWidth(2);
        
        int cpt = 1;
        PointList pl = new PointList();

        switch (direction) {
            case NORTH:
                // draw the link for the inDataPort
                graphics.drawLine(bounds.getBottom().getTranslated(0, -10), bounds.getCenter());
                
                // draw the links for all outDataPorts
                for (int i = 0; i < nbOutDataPorts; i++)
                {
                    graphics.drawLine(bounds.getLeft().getTranslated(bounds.width * cpt / (2 * nbOutDataPorts), 0),
                            bounds.getTopLeft().getTranslated(bounds.width * cpt / (2 * nbOutDataPorts), 10));
                    cpt = cpt + 2;
                }
                
                // draw the horizontal line
                graphics.drawLine(bounds.getRight(), bounds.getLeft());
                
                // draw the centered arrow
                pl.addPoint(bounds.getCenter().x, bounds.getCenter().y - ARROW_SIZE);
                pl.addPoint(bounds.getCenter().x - ARROW_SIZE, bounds.getCenter().y + ARROW_SIZE);
                pl.addPoint(bounds.getCenter().x + ARROW_SIZE, bounds.getCenter().y + ARROW_SIZE);
                break;
            case SOUTH:
                // draw the link for the inDataPort
                graphics.drawLine(bounds.getTop().getTranslated(0, 10), bounds.getCenter());
                
                // draw the links for all outDataPorts
                for (int i = 0; i < nbOutDataPorts; i++)
                {
                    graphics.drawLine(bounds.getLeft().getTranslated(bounds.width * cpt / (2 * nbOutDataPorts), 0),
                            bounds.getBottomLeft().getTranslated(bounds.width * cpt / (2 * nbOutDataPorts), -10));
                    cpt = cpt + 2;
                }
                
                // draw the horizontal line
                graphics.drawLine(bounds.getRight(), bounds.getLeft());
                
                // draw the centered arrow
                pl.addPoint(bounds.getCenter().x, bounds.getCenter().y + ARROW_SIZE);
                pl.addPoint(bounds.getCenter().x - ARROW_SIZE, bounds.getCenter().y - ARROW_SIZE);
                pl.addPoint(bounds.getCenter().x + ARROW_SIZE, bounds.getCenter().y - ARROW_SIZE);
                break;
            case WEST:
                // draw the link for the inDataPort
                graphics.drawLine(bounds.getRight().getTranslated(-10, 0), bounds.getCenter());
                
                // draw the links for all outDataPorts
                for (int i = 0; i < nbOutDataPorts; i++)
                {
                    graphics.drawLine(bounds.getTop().getTranslated(0, (bounds.height * cpt) / (2 * nbOutDataPorts)),
                            bounds.getTopLeft().getTranslated(10, (bounds.height * cpt) / (2 * nbOutDataPorts)));
                    cpt = cpt + 2;
                }
                
                // draw the vertical line
                graphics.drawLine(bounds.getTop(), bounds.getBottom());
                
                // draw the centered arrow
                pl.addPoint(bounds.getCenter().x - ARROW_SIZE, bounds.getCenter().y);
                pl.addPoint(bounds.getCenter().x + ARROW_SIZE, bounds.getCenter().y - ARROW_SIZE);
                pl.addPoint(bounds.getCenter().x + ARROW_SIZE, bounds.getCenter().y + ARROW_SIZE);
                break;
            default:
                // draw the link for the inDataPort
                graphics.drawLine(bounds.getLeft().getTranslated(10, 0), bounds.getCenter());
                
                // draw the links for all outDataPorts
                for (int i = 0; i < nbOutDataPorts; i++)
                {
                    graphics.drawLine(bounds.getTop().getTranslated(0, (bounds.height * cpt) / (2 * nbOutDataPorts)),
                            bounds.getTopRight().getTranslated(-10, (bounds.height * cpt) / (2 * nbOutDataPorts)));
                    cpt = cpt + 2;
                }
                
                // draw the vertical line
                graphics.drawLine(bounds.getTop(), bounds.getBottom());
                
                // draw the centered arrow
                pl.addPoint(bounds.getCenter().x + ARROW_SIZE, bounds.getCenter().y);
                pl.addPoint(bounds.getCenter().x - ARROW_SIZE, bounds.getCenter().y - ARROW_SIZE);
                pl.addPoint(bounds.getCenter().x - ARROW_SIZE, bounds.getCenter().y + ARROW_SIZE);
        }
        graphics.fillPolygon(pl);
        
        graphics.popState();
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return null because it can not contain any other Figure
     * @generated
     */
    public IFigure getContentPane()
    {
        return this;
    }
    
    /**
     * Get the number of OutDataPorts
     * 
     * @return nbOutDataPorts
     */
    public int getNbOutDataPorts()
    {
        return nbOutDataPorts;
    }
    
    /**
     * Set the number of OutDataPorts of the Decomposition figure
     * 
     * @param newValue the new value
     */
    public void setNbOutDataPorts(int newValue)
    {
        nbOutDataPorts = newValue;
    }

    /**
     * @see Orientable#setDirection(int)
     */
    public void setDirection(int value) {
        if ((value & (NORTH | SOUTH)) != 0)
            orientation = VERTICAL;
        else
            orientation = HORIZONTAL;
        direction = value;
        revalidate();
        repaint();
    }

    /**
     * @see Orientable#setOrientation(int)
     */
    public void setOrientation(int value) {
        if (orientation == VERTICAL && value == HORIZONTAL) {
            if (direction == NORTH) setDirection(WEST);
            else setDirection(EAST);
        }
        if (orientation == HORIZONTAL && value == VERTICAL) {
            if (direction == WEST) setDirection(NORTH);
            else setDirection(SOUTH);
        }
    }
}
