package org.topcased.modeler.sysml.actions;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.uml2.uml.Profile;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.uml.actions.UMLAction;
import org.topcased.modeler.sysml.command.DefineProfileCommand;
import org.topcased.modeler.sysml.SysMLPlugin;
import org.topcased.modeler.sysml.dialogs.DefineProfileDialog;

/**
 * Action that allows user to define profile, configure stereotypes
 * 
 * 
 * @author Rafik Bouguerra
 */
public class DefineProfileAction extends UMLAction
{
    private Profile profile;

    /**
     * Constructor
     * 
     * @param ed the Modeler object
     * @param prof The profil that should be defined
     */
    public DefineProfileAction(Modeler ed, org.eclipse.uml2.uml.Profile prof)
    {
        super("Define Profile", ed);
        this.profile = prof;
    }

    /**
     * Execute the Action
     * 
     * @see org.eclipse.jface.action.Action#run()
     */
    public void run()
    {
        ((CommandStack) getEditor().getAdapter(CommandStack.class)).execute(createActionCommand(profile));
    }

    /**
     * Create the generation command
     * 
     * @param p the UML profile
     * @return the command that update the profile properties
     */
    protected Command createActionCommand(final org.eclipse.uml2.uml.Profile p)
    {
        Command defineProfilCommand = null;
        if (p != null)
        {
            if (p.getQualifiedName() != null && !"".equals(p.getQualifiedName()))
            {
                DefineProfileDialog defineProfileDlg = new DefineProfileDialog(profile, SysMLPlugin.getActiveWorkbenchShell());
                if (defineProfileDlg.open() == Window.OK)
                {
                    defineProfilCommand = new DefineProfileCommand(profile, defineProfileDlg.getData(), defineProfileDlg.getOldData());
                }
            }
            else
            {
                MessageDialog.openWarning(getEditor().getSite().getShell(), "Error",
                        "Unable to define profile.\nCheck that all parent objects of the profile and the profile itself have their name set.");
            }
        }
        return defineProfilCommand;
    }

}
