/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.blockdefinitiondiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Usage;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;

/**
 * NamedElement restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class NamedElementRestoreConnectionCommand extends ElementRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public NamedElementRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated
     */
    protected void initializeCommands()
    {

        super.initializeCommands();

        GraphElement elt = getGraphElement();
        EObject eltObject = Utils.getElement(elt);

        if (eltObject instanceof NamedElement)
        {
            Iterator< ? > itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement
                        && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(elt);
                    GraphElement elt2 = (GraphElement) obj;
                    EObject eltObject2 = Utils.getElement(elt2);

                    if (eltObject2 instanceof NamedElement)
                    {
                        if (autoRef)
                        {
                            // autoRef not allowed
                        }
                        else
                        {
                            // if the elt is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createDependencyFromNamedElementToNamedElement(elt, elt2);
                            // if elt is the target of the edge or if it is the
                            // source and that the SourceTargetCouple is
                            // reversible
                            createDependencyFromNamedElementToNamedElement(elt2, elt);
                        }
                    }

                    if (eltObject2 instanceof NamedElement)
                    {
                        if (autoRef)
                        {
                            // autoRef not allowed
                        }
                        else
                        {
                            // if the elt is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createUsageFromNamedElementToNamedElement(elt, elt2);
                            // if elt is the target of the edge or if it is the
                            // source and that the SourceTargetCouple is
                            // reversible
                            createUsageFromNamedElementToNamedElement(elt2, elt);
                        }
                    }

                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated
     */
    private void createDependencyFromNamedElementToNamedElement(GraphElement srcElt, GraphElement targetElt)
    {
        NamedElement sourceObject = (NamedElement) Utils.getElement(srcElt);
        NamedElement targetObject = (NamedElement) Utils.getElement(targetElt);

        EList< ? > edgeObjectList = ((org.topcased.sysml.PackageSYSML) Utils.getDiagramModelObject(srcElt)).getOwnedRelationships();
        for (Iterator< ? > it = edgeObjectList.iterator(); it.hasNext();)
        {
            Object obj = it.next();
            if (obj instanceof Dependency)
            {
                Dependency edgeObject = (Dependency) obj;
                if (edgeObject.getSuppliers().contains(targetObject) && edgeObject.getClients().contains(sourceObject))
                {
                    // check if the relation does not exists yet
                    List< ? > existing = getExistingEdges(srcElt, targetElt, Dependency.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            DependencyEdgeCreationCommand cmd = new DependencyEdgeCreationCommand(getEditDomain(),
                                    (GraphEdge) edge, srcElt, false);
                            cmd.setTarget(targetElt);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated
     */
    private void createUsageFromNamedElementToNamedElement(GraphElement srcElt, GraphElement targetElt)
    {
        NamedElement sourceObject = (NamedElement) Utils.getElement(srcElt);
        NamedElement targetObject = (NamedElement) Utils.getElement(targetElt);

        EList< ? > edgeObjectList = ((org.topcased.sysml.PackageSYSML) Utils.getDiagramModelObject(srcElt)).getOwnedRelationships();
        for (Iterator< ? > it = edgeObjectList.iterator(); it.hasNext();)
        {
            Object obj = it.next();
            if (obj instanceof Usage)
            {
                Usage edgeObject = (Usage) obj;
                if (edgeObject.getSuppliers().contains(targetObject) && edgeObject.getClients().contains(sourceObject))
                {
                    // check if the relation does not exists yet
                    List< ? > existing = getExistingEdges(srcElt, targetElt, Usage.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            UsageEdgeCreationCommand cmd = new UsageEdgeCreationCommand(getEditDomain(),
                                    (GraphEdge) edge, srcElt, false);
                            cmd.setTarget(targetElt);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

}