/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.blockdefinitiondiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutManager;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.resource.JFaceResources;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Display;
import org.eclipse.uml2.uml.Element;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.modeler.ModelerColorConstants;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sysml.blockdefinitiondiagram.BlockDefEditPolicyConstants;
import org.topcased.modeler.sysml.blockdefinitiondiagram.commands.BlockRestoreConnectionCommand;
import org.topcased.modeler.sysml.blockdefinitiondiagram.figures.BlockFigure;
import org.topcased.modeler.sysml.blockdefinitiondiagram.policies.BlockLayoutEditPolicy;
import org.topcased.modeler.sysml.blockdefinitiondiagram.policies.ContainmentEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.blockdefinitiondiagram.util.BlockTools;
import org.topcased.modeler.sysml.util.SysMLTools;
import org.topcased.modeler.utils.Utils;
import org.topcased.sysml.Block;
import org.topcased.sysml.Requirement;

/**
 * The Block object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class BlockEditPart extends BehavioredClassifierEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public BlockEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(BlockDefEditPolicyConstants.CONTAINMENT_EDITPOLICY, new ContainmentEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new BlockRestoreConnectionCommand(getHost());
            }
        });

        ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(PositionConstants.EAST_WEST);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new BlockLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated NOT
     */
    protected IFigure createFigure()
    {

        return new BlockFigure(getDefaultWidth());
    }

    /**
     * This method is overriden because of ports which are stored in the figure
     * and not in the content pane
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#reorderChild(EditPart,
     *      int)
     */
    protected void reorderChild(EditPart child, int index)
    {
        // Save the constraint of the child so that it does not
        // get lost during the remove and re-add.
        IFigure childFigure = ((GraphicalEditPart) child).getFigure();
        LayoutManager layout;
        if (child instanceof PortEditPart)
        {
            layout = getFigure().getLayoutManager();
        }
        else
        {
            layout = getContentPane().getLayoutManager();
        }
        Object constraint = null;
        if (layout != null)
            constraint = layout.getConstraint(childFigure);

        super.reorderChild(child, index);
        setLayoutConstraint(child, childFigure, constraint);
    }

    /**
     * This method is overriden because of ports which are stored in the figure
     * and not in the content pane
     * 
     * @see org.eclipse.gef.AbstractGraphicalEditPart#addChildVisual(org.eclipse.gef.EditPart,
     *      int)
     */
    protected void addChildVisual(EditPart childEditPart, int index)
    {
        if (childEditPart instanceof PortEditPart)
        {
            // Ports are added in the figure not in the content pane
            IFigure child = ((GraphicalEditPart) childEditPart).getFigure();
            getFigure().add(child);
        }
        else
        {
            super.addChildVisual(childEditPart, index);
        }
    }

    /**
     * This method is overriden because of ports which are stored in the figure
     * and not in the content pane
     * 
     * @see org.eclipse.gef.AbstractGraphicalEditPart#removeChildVisual(org.eclipse.gef.EditPart)
     */
    protected void removeChildVisual(EditPart childEditPart)
    {
        if (childEditPart instanceof PortEditPart)
        {
            IFigure child = ((GraphicalEditPart) childEditPart).getFigure();
            getFigure().remove(child);
        }
        else
        {
            super.removeChildVisual(childEditPart);
        }
    }

    protected void refreshHeaderLabel()
    {
        BlockFigure fig = (BlockFigure) getFigure();
        ComposedLabel lbl = (ComposedLabel) fig.getLabel();
        Block block = (Block) Utils.getElement(getGraphNode());

        String stereotypes = SysMLTools.getStereotypesList(block);
        if (stereotypes.length() != 0)
        {
            stereotypes = ", " + stereotypes;
        }
        lbl.setPrefix("<<block" + stereotypes + ">>");

        if (block.getName() != null)
        {
            lbl.setMain(block.getName());
        }

        lbl.setSuffix(BlockTools.getFromNotation(block,
                (Element) Utils.getElement((GraphElement) getParent().getModel())));
    }

    /**
     * This method is overriden because BlockEditPart inherits from
     * BehavioredClassifierEditPart which has been modified from the generate
     * code.
     */
    protected Color getDefaultBackgroundColor()
    {
        return ModelerColorConstants.lightBlue;
    }
    
    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getDefaultFont()
     */
    protected Font getDefaultFont()
    {
        Block  block = (Block) Utils.getElement(getGraphNode());
        FontData[] fData = Display.getDefault().getSystemFont().getFontData();
        if (block.isAbstract())
        {
            fData[0].setStyle(SWT.BOLD | SWT.ITALIC);
        }
        else
        {
            fData[0].setStyle(SWT.BOLD);
        }
        JFaceResources.getFontRegistry().put("ItalicFont", fData);
        return JFaceResources.getFontRegistry().get("ItalicFont");
    }

}