/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.blockdefinitiondiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.TypedElement;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.figures.DiagramFigure;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sysml.blockdefinitiondiagram.BlockDefEditPolicyConstants;
import org.topcased.modeler.sysml.blockdefinitiondiagram.commands.PortRestoreConnectionCommand;
import org.topcased.modeler.sysml.blockdefinitiondiagram.figures.PortFigure;
import org.topcased.modeler.sysml.blockdefinitiondiagram.figures.PortProvidedInterfaceFigure;
import org.topcased.modeler.sysml.blockdefinitiondiagram.figures.PortRequiredInterfaceFigure;
import org.topcased.modeler.sysml.blockdefinitiondiagram.policies.CommentLinkEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.blockdefinitiondiagram.policies.ConnectorEdgeCreationEditPolicy;
import org.topcased.modeler.utils.Utils;

/**
 * The Port object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */
public class PortEditPart extends org.topcased.modeler.edit.PortEditPart
{

    protected PortProvidedInterfaceFigure _providedFigure;

    protected PortRequiredInterfaceFigure _requiredFigure;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public PortEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        // CommentLinkEdgeCreationEditPolicy has been added because this class
        // cannot inherit
        // from PortEditPart and from ElementEditPart
        installEditPolicy(BlockDefEditPolicyConstants.COMMENTLINK_EDITPOLICY, new CommentLinkEdgeCreationEditPolicy());

        installEditPolicy(BlockDefEditPolicyConstants.CONNECTOR_EDITPOLICY, new ConnectorEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new PortRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new NonResizableEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated NOT
     */
    protected IFigure createFigure()
    {

        return new PortFigure();
    }

    /**
     * Return the model port
     * 
     * @return the model port
     */
    public Port getModelPort()
    {
        return (Port) Utils.getElement(getGraphNode());
    }

    /**
     * @see org.eclipse.gef.EditPart#activate()
     */
    public void activate()
    {
        super.activate();
        if (getModelPort().getType() != null)
        {
            getModelPort().getType().eAdapters().add(getModelListener());
        }
    }

    /**
     * @see org.eclipse.gef.EditPart#deactivate()
     */
    public void deactivate()
    {
        IFigure interfacesContainerFigure = getFigure().getParent().getParent();

        if (_providedFigure != null)
        {
            getFigure().removeFigureListener(_providedFigure.getPortDeplacementListener());
            interfacesContainerFigure.remove(_providedFigure);
            _providedFigure = null;
        }

        if (_requiredFigure != null)
        {
            getFigure().removeFigureListener(_requiredFigure.getPortDeplacementListener());
            interfacesContainerFigure.remove(_requiredFigure);
            _requiredFigure = null;
        }

        if (getModelPort().getType() != null)
        {
            getModelPort().getType().eAdapters().remove(getModelListener());
        }

        super.deactivate();
    }

    protected void handleModelChanged(Notification msg)
    {
        // Handle type change
        Object newObject = msg.getNewValue();
        Object oldObject = msg.getOldValue();

        if (msg.getNotifier().equals(getEObject()))
        {
            if (msg.getFeatureID(TypedElement.class) == UMLPackage.TYPED_ELEMENT__TYPE)
            {
                if (oldObject != null && oldObject instanceof EObject)
                {
                    ((EObject) oldObject).eAdapters().remove(getModelListener());
                }
                if (newObject != null && newObject instanceof EObject)
                {
                    ((EObject) newObject).eAdapters().add(getModelListener());
                }
            }
        }

        if (msg.getEventType() != Notification.REMOVING_ADAPTER)
            super.handleModelChanged(msg);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     * @generated NOT
     */
    protected void refreshVisuals()
    {
        super.refreshVisuals();
        if (!(getFigure().getParent() instanceof DiagramFigure))
        {
            refreshProvidedInterfaces();
            refreshRequiredInterfaces();
        }
    }

    protected void refreshProvidedInterfaces()
    {
        IFigure interfacesContainerFigure = getFigure().getParent().getParent();
        if (getModelPort().getProvideds().size() != 0)
        {
            if (_providedFigure == null)
            {
                _providedFigure = new PortProvidedInterfaceFigure();
                interfacesContainerFigure.add(_providedFigure);
                getFigure().addFigureListener(_providedFigure.getPortDeplacementListener());
            }
            refreshProvidedInterfacesNames();
            _providedFigure.drawInterface(getFigure());
        }
        else
        {
            if (_providedFigure != null)
            {
                getFigure().removeFigureListener(_providedFigure.getPortDeplacementListener());
                interfacesContainerFigure.remove(_providedFigure);
                _providedFigure = null;
            }
        }
    }

    protected void refreshRequiredInterfaces()
    {
        IFigure interfacesContainerFigure = getFigure().getParent().getParent();
        if (getModelPort().getRequireds().size() != 0)
        {
            if (_requiredFigure == null)
            {
                _requiredFigure = new PortRequiredInterfaceFigure();
                interfacesContainerFigure.add(_requiredFigure);
                getFigure().addFigureListener(_requiredFigure.getPortDeplacementListener());
            }
            refreshRequiredInterfacesNames();
            _requiredFigure.drawInterface(getFigure());
        }
        else
        {
            if (_requiredFigure != null)
            {
                getFigure().removeFigureListener(_requiredFigure.getPortDeplacementListener());
                interfacesContainerFigure.remove(_requiredFigure);
                _requiredFigure = null;
            }
        }
    }

    public void refreshProvidedInterfacesNames()
    {
        String labelText = "";
        for (int i = 0; i < getModelPort().getProvideds().size(); i++)
        {
            Interface inter = (Interface) getModelPort().getProvideds().get(i);
            labelText += (i == 0 ? "" : ", ") + inter.getName();
        }
        _providedFigure.getNameLabel().setText(labelText);
    }

    public void refreshRequiredInterfacesNames()
    {
        String labelText = "";
        for (int i = 0; i < getModelPort().getRequireds().size(); i++)
        {
            Interface inter = (Interface) getModelPort().getRequireds().get(i);
            labelText += (i == 0 ? "" : ", ") + inter.getName();
        }
        _requiredFigure.getNameLabel().setText(labelText);
    }

}