/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.blockdefinitiondiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Display;
import org.eclipse.uml2.uml.Element;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sysml.blockdefinitiondiagram.BlockDefEditPolicyConstants;
import org.topcased.modeler.sysml.blockdefinitiondiagram.commands.UnitRestoreConnectionCommand;
import org.topcased.modeler.sysml.blockdefinitiondiagram.figures.UnitFigure;
import org.topcased.modeler.sysml.blockdefinitiondiagram.policies.CommentLinkEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.blockdefinitiondiagram.policies.UnitLayoutEditPolicy;
import org.topcased.modeler.sysml.blockdefinitiondiagram.util.BlockTools;
import org.topcased.modeler.sysml.util.SysMLTools;
import org.topcased.modeler.utils.Utils;
import org.topcased.sysml.Unit;

/**
 * The Unit object controller
 * 
 * @generated
 */
public class UnitEditPart extends EMFGraphNodeEditPart
{
    /**
     * Constructor
     * 
     * @param obj the graph node
     * @generated
     */
    public UnitEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles
     * 
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();
        
        installEditPolicy(BlockDefEditPolicyConstants.COMMENTLINK_EDITPOLICY, new CommentLinkEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new UnitRestoreConnectionCommand(getHost());
            }
        });

        ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(PositionConstants.EAST_WEST);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new UnitLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new UnitFigure();
    }

    protected void refreshHeaderLabel()
    {
        UnitFigure fig = (UnitFigure) getFigure();
        ComposedLabel lbl = (ComposedLabel) fig.getLabel();
        Unit unit = (Unit) Utils.getElement(getGraphNode());

        String stereotypes = SysMLTools.getStereotypesList(unit);
        if (stereotypes.length() != 0)
        {
            stereotypes = ", " + stereotypes;
        }
        lbl.setPrefix("<<unit" + stereotypes + ">>");

        if (unit.getName() != null)
        {
            lbl.setMain(unit.getName());
        }

        lbl.setSuffix(BlockTools.getFromNotation(unit,
                (Element) Utils.getElement((GraphElement) getParent().getModel())));
    }

    protected Font getDefaultFont()
    {
        FontData[] fData = Display.getDefault().getSystemFont().getFontData();
        fData[0].setStyle(SWT.BOLD);
        JFaceResources.getFontRegistry().put("BoldFont", fData);
        return JFaceResources.getFontRegistry().get("BoldFont");
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshVisuals()
     */
    protected void refreshVisuals()
    {
        super.refreshVisuals();
        refreshUnit();
    }

    protected void refreshUnit()
    {
        UnitFigure fig = (UnitFigure) getFigure();
        Unit unit = (Unit) Utils.getElement(getGraphNode());
        fig.setDimension(unit.getDimension() != null ? unit.getDimension().getName() : "");
    }

}