/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.blockdefinitiondiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.eclipse.uml2.uml.Element;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sysml.blockdefinitiondiagram.commands.ValueTypeRestoreConnectionCommand;
import org.topcased.modeler.sysml.blockdefinitiondiagram.figures.ValueTypeFigure;
import org.topcased.modeler.sysml.blockdefinitiondiagram.policies.ValueTypeLayoutEditPolicy;
import org.topcased.modeler.sysml.blockdefinitiondiagram.util.BlockTools;
import org.topcased.modeler.sysml.util.SysMLTools;
import org.topcased.modeler.utils.Utils;
import org.topcased.sysml.ValueType;

/**
 * The ValueType object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ValueTypeEditPart extends DataTypeEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public ValueTypeEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new ValueTypeRestoreConnectionCommand(getHost());
            }
        });

        ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(PositionConstants.EAST_WEST);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new ValueTypeLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new ValueTypeFigure();
    }

    /**
     * @see org.topcased.modeler.sysml.blockdefinitiondiagram.edit.DataTypeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        ValueTypeFigure fig = (ValueTypeFigure) getFigure();
        ComposedLabel lbl = (ComposedLabel) fig.getLabel();
        ValueType valueType = (ValueType) Utils.getElement(getGraphNode());

        String stereotypes = SysMLTools.getStereotypesList(valueType);
        if (stereotypes.length() != 0)
        {
            stereotypes = ", " + stereotypes;
        }
        lbl.setPrefix("<<valueType" + stereotypes + ">>");

        if (valueType.getName() != null)
        {
            lbl.setMain(valueType.getName());
        }

        lbl.setSuffix(BlockTools.getFromNotation(valueType,
                (Element) Utils.getElement((GraphElement) getParent().getModel())));
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshVisuals()
     */
    protected void refreshVisuals()
    {
        super.refreshVisuals();
        refreshUnitAndDimension();
    }

    protected void refreshUnitAndDimension()
    {
        ValueTypeFigure fig = (ValueTypeFigure) getFigure();
        ValueType valueType = (ValueType) Utils.getElement(getGraphNode());
        fig.setUnit(valueType.getUnit() != null ? valueType.getUnit().getName() : "");
        fig.setDimension(valueType.getDimension() != null ? valueType.getDimension().getName() : "");
    }
}