/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation, Thomas Friol (Anyware Technologies)
 ******************************************************************************/

package org.topcased.modeler.sysml.blockdefinitiondiagram.figures;

import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Rectangle;

public class PortProvidedInterfaceFigure extends PortInterfaceFigure
{
    public PortProvidedInterfaceFigure()
    {
        super();
        _interfaceFigure = new ProvidedInterfaceRepresentation();
        _interfaceFigure.setSize(INTERFACE_WIDTH, INTERFACE_HEIGHT);
        add(_interfaceFigure);
    }

    protected void updateFigureArrangement()
    {
        switch (_attachedPortPosition)
        {
            case PositionConstants.TOP:
                setConstraint(_interfacesNames, BorderLayout.TOP);
                setConstraint(_interfaceFigure, BorderLayout.LEFT);
                setConstraint(_emptySpace, BorderLayout.RIGHT);
                break;
            case PositionConstants.BOTTOM:
                setConstraint(_interfacesNames, BorderLayout.BOTTOM);
                setConstraint(_interfaceFigure, BorderLayout.RIGHT);
                setConstraint(_emptySpace, BorderLayout.LEFT);
                break;
            case PositionConstants.LEFT:
                setConstraint(_interfacesNames, BorderLayout.TOP);
                setConstraint(_interfaceFigure, BorderLayout.RIGHT);
                setConstraint(_emptySpace, BorderLayout.LEFT);
                break;
            case PositionConstants.RIGHT:
                setConstraint(_interfacesNames, BorderLayout.BOTTOM);
                setConstraint(_interfaceFigure, BorderLayout.LEFT);
                setConstraint(_emptySpace, BorderLayout.RIGHT);
        }
    }

    protected void updateFigureLocation()
    {
        Rectangle rect = new Rectangle();
        rect.setSize(Math.max(_interfacesNames.getPreferredSize().width, INTERFACE_WIDTH), INTERFACE_HEIGHT
                + _interfacesNames.getPreferredSize().height);
        switch (_attachedPortPosition)
        {
            case PositionConstants.TOP:
                rect.setLocation(_attachedPortBounds.getTop().getCopy().translate(0, -rect.height));
                break;
            case PositionConstants.BOTTOM:
                rect.setLocation(_attachedPortBounds.getBottom().getCopy().translate(-rect.width, 0));
                break;
            case PositionConstants.LEFT:
                rect.setLocation(_attachedPortBounds.getLeft().getCopy().translate(-rect.width, -rect.height));
                break;
            case PositionConstants.RIGHT:
                rect.setLocation(_attachedPortBounds.getRight().getCopy());
        }
        setBounds(rect);
    }

    private class ProvidedInterfaceRepresentation extends InterfaceRepresentation
    {

        protected void paintFigure(Graphics graphics)
        {
            graphics.pushState();
            graphics.setBackgroundColor(graphics.getForegroundColor());
            // Draw a line with a circle at end
            int circleDiameter = Math.min(INTERFACE_WIDTH, INTERFACE_HEIGHT) / 2;
            switch (_attachedPortPosition)
            {
                case PositionConstants.TOP:
                    graphics.fillArc(bounds.getRight().x - circleDiameter, bounds.y, circleDiameter, circleDiameter,
                            90, 360);
                    graphics.drawLine(getBounds().getBottomLeft(), getBounds().getTopRight().getCopy().translate(
                            -circleDiameter / 2, circleDiameter / 2));
                    break;
                case PositionConstants.BOTTOM:
                    graphics.fillArc(bounds.x, bounds.getBottom().y - circleDiameter, circleDiameter, circleDiameter,
                            90, 360);
                    graphics.drawLine(getBounds().getTopRight(), getBounds().getBottomLeft().getCopy().translate(
                            circleDiameter / 2, -circleDiameter / 2));
                    break;
                case PositionConstants.LEFT:
                    graphics.fillArc(bounds.x, bounds.y, circleDiameter, circleDiameter, 90, 360);
                    graphics.drawLine(getBounds().getBottomRight(), getBounds().getTopLeft().getCopy().translate(
                            circleDiameter / 2, circleDiameter / 2));
                    break;
                case PositionConstants.RIGHT:
                    graphics.fillArc(bounds.getRight().x - circleDiameter, bounds.getBottom().y - circleDiameter,
                            circleDiameter, circleDiameter, 90, 360);
                    graphics.drawLine(getBounds().getTopLeft(), getBounds().getBottomRight().getCopy().translate(
                            -circleDiameter / 2, -circleDiameter / 2));
            }

            graphics.popState();
        }

    }
}
