/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation, Thomas Friol (Anyware Technologies)
 ******************************************************************************/

package org.topcased.modeler.sysml.blockdefinitiondiagram.figures;

import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Rectangle;

public class PortRequiredInterfaceFigure extends PortInterfaceFigure
{
    public PortRequiredInterfaceFigure()
    {
        super();
        _interfaceFigure = new RequiredInterfaceRepresentation();
        _interfaceFigure.setSize(INTERFACE_WIDTH, INTERFACE_HEIGHT);
        add(_interfaceFigure);
    }

    protected void updateFigureArrangement()
    {
        switch (_attachedPortPosition)
        {
            case PositionConstants.TOP:
                setConstraint(_interfacesNames, BorderLayout.TOP);
                setConstraint(_interfaceFigure, BorderLayout.RIGHT);
                setConstraint(_emptySpace, BorderLayout.LEFT);
                break;
            case PositionConstants.BOTTOM:
                setConstraint(_interfacesNames, BorderLayout.BOTTOM);
                setConstraint(_interfaceFigure, BorderLayout.LEFT);
                setConstraint(_emptySpace, BorderLayout.RIGHT);
                break;
            case PositionConstants.LEFT:
                setConstraint(_interfacesNames, BorderLayout.BOTTOM);
                setConstraint(_interfaceFigure, BorderLayout.RIGHT);
                setConstraint(_emptySpace, BorderLayout.LEFT);
                break;
            case PositionConstants.RIGHT:
                setConstraint(_interfacesNames, BorderLayout.TOP);
                setConstraint(_interfaceFigure, BorderLayout.LEFT);
                setConstraint(_emptySpace, BorderLayout.RIGHT);
        }
    }

    protected void updateFigureLocation()
    {
        Rectangle rect = new Rectangle();
        rect.setSize(Math.max(_interfacesNames.getPreferredSize().width, INTERFACE_WIDTH), INTERFACE_HEIGHT
                + _interfacesNames.getPreferredSize().height);
        switch (_attachedPortPosition)
        {
            case PositionConstants.TOP:
                rect.setLocation(_attachedPortBounds.getTop().getCopy().translate(-rect.width, -rect.height));
                break;
            case PositionConstants.BOTTOM:
                rect.setLocation(_attachedPortBounds.getBottom().getCopy());
                break;
            case PositionConstants.LEFT:
                rect.setLocation(_attachedPortBounds.getLeft().getCopy().translate(-rect.width, 0));
                break;
            case PositionConstants.RIGHT:
                rect.setLocation(_attachedPortBounds.getRight().getCopy().translate(0, -rect.height));
        }
        setBounds(rect);
    }

    private class RequiredInterfaceRepresentation extends InterfaceRepresentation
    {

        protected void paintFigure(Graphics graphics)
        {
            graphics.pushState();
            // Draw a line with an arc at end
            int arcDiameter = Math.min(INTERFACE_WIDTH, INTERFACE_HEIGHT) / 2;
            int lineEndToSide = (int) (arcDiameter / 2 + arcDiameter / (2 * Math.sqrt(2)));
            switch (_attachedPortPosition)
            {
                case PositionConstants.TOP:
                    graphics.drawArc(bounds.x, bounds.y, arcDiameter, arcDiameter, 225, 180);
                    graphics.drawLine(getBounds().getBottomRight(), getBounds().getTopLeft().getCopy().translate(
                            lineEndToSide, lineEndToSide));
                    break;
                case PositionConstants.BOTTOM:
                    graphics.drawArc(bounds.getRight().x - arcDiameter, bounds.getBottom().y - arcDiameter,
                            arcDiameter, arcDiameter, 45, 180);
                    graphics.drawLine(getBounds().getTopLeft(), getBounds().getBottomRight().getCopy().translate(
                            -lineEndToSide, -lineEndToSide));
                    break;
                case PositionConstants.LEFT:
                    graphics.drawArc(bounds.x, bounds.getBottom().y - arcDiameter, arcDiameter, arcDiameter, 315, 180);
                    graphics.drawLine(getBounds().getTopRight(), getBounds().getBottomLeft().getCopy().translate(
                            lineEndToSide, -lineEndToSide));
                    break;
                case PositionConstants.RIGHT:
                    graphics.drawArc(bounds.getRight().x - arcDiameter, bounds.y, arcDiameter, arcDiameter, 135, 180);
                    graphics.drawLine(getBounds().getBottomLeft(), getBounds().getTopRight().getCopy().translate(
                            -lineEndToSide, lineEndToSide));
            }

            graphics.popState();
        }

    }
}
