/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Laleve (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.sysml.blockdefinitiondiagram.policies;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;

/**
 * A Layout that allow creation of eObjects in relation with an
 * eStructuralFeature of the parent eObject or not.
 * 
 * Contrary to org.topcased.modeler.edit.policies.EListLayoutEditPolicy, this
 * edit policy allows only creation of childs which are of type of the
 * eStructuralFeature
 */
public class EListLayoutEditPolicy extends org.topcased.modeler.edit.policies.EListLayoutEditPolicy
{
    protected EStructuralFeature _feature;

    /**
     * The Constructor
     * 
     * @param feature the EStructural feature that should be listed
     */
    public EListLayoutEditPolicy(EStructuralFeature feature)
    {
        super(feature);
        _feature = feature;
    }

    /**
     * Check if the current EObject is a valid child for the parent EObject.
     * It's valid if the class of the child is the same than the type of the
     * EStructuralFeature Derived classes are not allowed (because of Ports
     * which are derived from Properties)
     * 
     * @param child the child EObject
     * @param parent the parent EObject
     * @return true if the child can be added
     */
    protected boolean isValid(EObject child, EObject parent)
    {
        if (_feature != null)
        {
            return child != null && parent != null
                    && _feature.getEType().getInstanceClassName().equals(child.eClass().getInstanceClassName());
        }
        return false;
    }

}
