package org.topcased.modeler.sysml.dialogs;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.edit.ui.celleditor.FeatureEditorDialog;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.resource.UMLResource;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;
import org.topcased.modeler.uml.UMLPlugin;

/**
 * 
 * @author Rafik Bouguerra
 */
public class DefineProfileDialog extends Dialog implements ITopcasedDialogConstants
{
    /** th ID of the profile name */
    public static final String PROFILE_NAME = "profile name";

    /** Current edited package */
    private Profile profile;

    // SWT Objects
    private Composite dialogComposite;

    private ILabelProvider labelProvider;

    private Map data;

    private Map oldData;

    private Text profileNameText;
    
    /**
     * Create the dialog for a given profile
     * 
     * @param prof the profile to edit
     * @param parentShell the owing shell
     */
    public DefineProfileDialog(Profile prof, Shell parentShell)
    {
        super(parentShell);
        setBlockOnOpen(true);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        profile = prof;
        labelProvider = new LabelProvider()
        {
            public String getText(Object element)
            {
                if (element instanceof NamedElement)
                {
                    return ((NamedElement) element).getName();
                }
                else
                {
                    return "no name";
                }

            }
        };
    }

    /**
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("Profile " + profile.getName());
        newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

        super.configureShell(newShell);
    }

    /**
     * Create the Dialog area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        // Dialog
        dialogComposite = (Composite) super.createDialogArea(parent);
        createDialogComposite();
        createProfileGroup(dialogComposite);

        return dialogComposite;
    }

    /**
     * Create the Dialog Contents
     */
    private void createDialogComposite()
    {
        GridLayout dialogLayout = new GridLayout();
        GridData dialogLayoutData = new GridData(GridData.FILL_BOTH);
        dialogLayoutData.widthHint = DEFAULT_DIALOG_WIDTH;
        dialogLayoutData.heightHint = DEFAULT_DIALOG_HEIGHT;
        dialogComposite.setLayout(dialogLayout);
        dialogComposite.setLayoutData(dialogLayoutData);
    }

    /**
     * Creates the group
     * 
     * @param parent the parent Composite
     */
    protected void createProfileGroup(Composite parent)
    {
        TabFolder tabFolder = new TabFolder(parent, SWT.TOP);
        GridData layoutData = new GridData(GridData.FILL_BOTH);
        tabFolder.setLayoutData(layoutData);
        data = prepareData();
        oldData = prepareData();
        createFirstTabItem(tabFolder);
        Iterator stereotypeIterator = data.values().iterator();
        while (stereotypeIterator.hasNext())
        {
            createTabItem(tabFolder, (SysMLStereoStructure) stereotypeIterator.next());
        }
        fillFields();
    }

    /**
     * 
     * @param parent
     */
    private void createFirstTabItem(TabFolder parent)
    {

        // Create tab item and add it composite that fills it
        TabItem firstItem = new TabItem(parent, SWT.NONE);
        firstItem.setText("Profile data");
        Composite composite = new Composite(parent, SWT.NONE);
        firstItem.setControl(composite);

        // Add layout on composite
        GridLayout layout = new GridLayout(2, false);
        composite.setLayout(layout);
        GridData layoutData = new GridData(GridData.FILL_BOTH);
        composite.setLayoutData(layoutData);

        new Label(composite, SWT.NONE).setText("Name : ");

        profileNameText = new Text(composite, SWT.BORDER);
        GridData labelLayoutData = new GridData(GridData.FILL_HORIZONTAL);
        profileNameText.setLayoutData(labelLayoutData);
    }

    /**
     * Prepare data by filling the StereotypeStructures
     * 
     * @return A Map that contain all the profile informations
     */
    private HashMap prepareData()
    {
        HashMap datas = new HashMap();
        Iterator stereotypeIterator = profile.getOwnedStereotypes().iterator();
        while (stereotypeIterator.hasNext())
        {
            Stereotype stereotype = (Stereotype) stereotypeIterator.next();
            datas.put(stereotype, new SysMLStereoStructure(stereotype));
        }
        return datas;
    }

    /**
     * Return the label provider
     * 
     * @return the LabelProvider
     */
    public ILabelProvider getLabelProvider()
    {
        return labelProvider;
    }

    /**
     * Dialog for metaclasses
     * 
     * @param currentValues this the current select value
     * @return the new selection as a List
     */
    protected List getReferencedMetaClass(List currentValues)
    {
        List choiceOfValues = new ArrayList();
        List choiceOfValuesUML = new ArrayList();
        try
        {
        	URI uriUML = URI.createURI(UMLResource.UML_METAMODEL_URI);
        	URI uri = URI.createURI("pathmap://SYSML_META_ELTS/metamodel.uml");
        	
        	Resource resource = profile.eResource().getResourceSet().getResource(uri, true);
        	Resource resourceUML = profile.eResource().getResourceSet().getResource(uriUML, true);
            Iterator contents = resource.getAllContents();
            while (contents.hasNext())
            {
                Object object = contents.next();
                if (org.eclipse.uml2.uml.Class.class.isInstance(object) && ((org.eclipse.uml2.uml.Class) object).isMetaclass())
                {
                    choiceOfValues.add(object);
                }
            }
            contents = resourceUML.getAllContents();
            while (contents.hasNext())
            {
                Object object = contents.next();
                if (org.eclipse.uml2.uml.Class.class.isInstance(object) && ((org.eclipse.uml2.uml.Class) object).isMetaclass())
                {
                    choiceOfValuesUML.add(object);
                }
            }
        }
        catch (Exception e)
        {
            // ignore
            // FIXME certainly a very bad code
        }

        Collections.sort(choiceOfValues, new Comparator()
        {
            public int compare(Object o1, Object o2)
            {
                return getLabelProvider().getText(o1).compareTo(getLabelProvider().getText(o2));
            }
        });
        Collections.sort(choiceOfValuesUML, new Comparator()
        {
            public int compare(Object o1, Object o2)
            {
                return getLabelProvider().getText(o1).compareTo(getLabelProvider().getText(o2));
            }
        });
        choiceOfValues.addAll(choiceOfValuesUML);

        final FeatureEditorDialog dialog = new FeatureEditorDialog(UMLPlugin.getActiveWorkbenchWindow().getShell(), getLabelProvider(), profile, UMLPackage.eINSTANCE.getClass_(), currentValues,
                "Metaclasses", choiceOfValues);
        dialog.open();
        if (Window.OK == dialog.getReturnCode())
        {
            return dialog.getResult();
        }
        return null;
    }

    /**
     * 
     * @param parent
     * @param stereotypeStructure
     */
    private void createTabItem(TabFolder parent, final SysMLStereoStructure stereotypeStructure)
    {
        // Create tab item and add it composite that fills it
        TabItem secondItem = new TabItem(parent, SWT.NONE);
        secondItem.setText(stereotypeStructure.getStereotype().getName());
        Composite composite = new Composite(parent, SWT.NONE);
        secondItem.setControl(composite);

        // Add layout on composite
        GridLayout layout = new GridLayout();
        composite.setLayout(layout);
        GridData layoutData = new GridData(GridData.FILL_BOTH);
        composite.setLayoutData(layoutData);

        Composite nameComp = new Composite(composite, SWT.NONE);
        layout = new GridLayout(2, false);
        nameComp.setLayout(layout);
        layoutData = new GridData(GridData.FILL_HORIZONTAL);
        nameComp.setLayoutData(layoutData);

        new Label(nameComp, SWT.NONE).setText("Name : ");

        stereotypeStructure.setNameText(new Text(nameComp, SWT.BORDER));
        stereotypeStructure.getNameText().setText(stereotypeStructure.getStereotype().getName());
        layoutData = new GridData(GridData.FILL_HORIZONTAL);
        stereotypeStructure.getNameText().setLayoutData(layoutData);

        Label applicableLbl = new Label(composite, SWT.NONE);
        applicableLbl.setText("This stereotype is applicable to : ");
        layoutData = new GridData(GridData.FILL_HORIZONTAL);
        applicableLbl.setLayoutData(layoutData);

        Composite extensionComp = new Composite(composite, SWT.NONE);
        layout = new GridLayout(2, false);
        extensionComp.setLayout(layout);
        layoutData = new GridData(GridData.FILL_HORIZONTAL);
        extensionComp.setLayoutData(layoutData);

        stereotypeStructure.setExtensionTxt(new Text(extensionComp, SWT.BORDER));
        stereotypeStructure.getExtensionTxt().setEditable(false);
        layoutData = new GridData(GridData.FILL_HORIZONTAL);
        stereotypeStructure.getExtensionTxt().setLayoutData(layoutData);
        stereotypeStructure.updateLabel();

        Button update = new Button(extensionComp, SWT.PUSH | SWT.CENTER);
        update.setText("...");
        update.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                List result = getReferencedMetaClass(stereotypeStructure.getMetaclasses());
                if (result != null)
                {
                    stereotypeStructure.updateMetaclasses(result);
                    stereotypeStructure.updateLabel();
                }
            }
        });
    }

    /**
     * Fill fields
     * 
     */
    private void fillFields()
    {
        profileNameText.setText(profile.getName());
    }

    /**
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        Iterator iterator = data.values().iterator();
        while (iterator.hasNext())
        {
            SysMLStereoStructure stSt = (SysMLStereoStructure) iterator.next();
            stSt.setName(stSt.getName());
        }
        data.put(PROFILE_NAME, profileNameText.getText());

        super.okPressed();
    }

    /**
     * Return a map containing property data that may changed
     * 
     * @return a Map
     */
    public Map getData()
    {
        return data;
    }

    /**
     * Return a map containing old data
     * 
     * @return a Map
     */
    public Map getOldData()
    {
        oldData.put(PROFILE_NAME, profile.getName());
        return oldData;
    }

}
