
package org.topcased.modeler.sysml.editor;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.FeatureMap;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.provider.IWrapperItemProvider;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.IPageSite;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Profile;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.outline.IOutlineMenuConstants;
import org.topcased.modeler.editor.outline.ModelNavigator;
import org.topcased.modeler.uml.actions.ApplyProfileAction;
import org.topcased.modeler.uml.actions.ApplyStereotypeAction;
import org.topcased.modeler.sysml.actions.DefineProfileAction;
import org.topcased.modeler.uml.actions.UnapplyProfileAction;
import org.topcased.modeler.uml.actions.UnapplyStereotypeAction;
import org.topcased.sysml.PackageSYSML;

/**
 * This custom navigator changes the context menu and adds some UML actions.
 * 
 */
public class SysMLNavigator extends ModelNavigator
{
    /**
     * Constructor
     * 
     * @param parent the parent composite
     * @param editor the modeler
     * @param pageSite the outline site
     */
    public SysMLNavigator(Composite parent, Modeler editor, IPageSite pageSite)
    {
        super(parent, editor, pageSite);
    }

    /**
     * @see org.topcased.modeler.editor.outline.ModelNavigator#createSingleSelectionMenu(org.eclipse.jface.action.IMenuManager,
     *      java.lang.Object)
     */
    protected void createSingleSelectionMenu(IMenuManager manager, Object selection)
    {
        super.createSingleSelectionMenu(manager, selection);

        EObject selectedObject = null;

        if (selection instanceof EObject)
        {
            selectedObject = (EObject) selection;
        }
        else if (selection instanceof IWrapperItemProvider || selection instanceof FeatureMap.Entry)
        {
            selectedObject = (EObject) AdapterFactoryEditingDomain.unwrap(selection);
        }

//        if (selectedObject instanceof Model)
//        {
//            IMenuManager generateTypesMenu = new MenuManager("Generate Primitive Types");
//
//            generateTypesMenu.add(new GenerateUMLPrimitiveTypesAction(getModeler(), (Model) selectedObject));
//            generateTypesMenu.add(new GenerateJavaPrimitiveTypesAction(getModeler(), (Model) selectedObject));
//
//            manager.appendToGroup(IOutlineMenuConstants.NEW_GROUP, generateTypesMenu);
//        }
        if (selectedObject instanceof PackageSYSML || selectedObject instanceof org.eclipse.uml2.uml.Package)
        {
//            manager.appendToGroup(IOutlineMenuConstants.NEW_GROUP, new ImportPrimitiveTypesAction(getModeler(), (org.eclipse.uml2.uml.Package) selectedObject));

            manager.appendToGroup(IOutlineMenuConstants.EDIT_GROUP, new ApplyProfileAction(getModeler(), (org.eclipse.uml2.uml.Package) selectedObject));
            manager.appendToGroup(IOutlineMenuConstants.EDIT_GROUP, new UnapplyProfileAction(getModeler(), (org.eclipse.uml2.uml.Package) selectedObject));
        }
        if (selectedObject instanceof Element)
        {
            manager.appendToGroup(IOutlineMenuConstants.EDIT_GROUP, new ApplyStereotypeAction(getModeler(), (Element) selectedObject));
            manager.appendToGroup(IOutlineMenuConstants.EDIT_GROUP, new UnapplyStereotypeAction(getModeler(), (Element) selectedObject));
        }
        if (selectedObject instanceof Profile)
        {
            manager.appendToGroup(IOutlineMenuConstants.EDIT_GROUP, new DefineProfileAction(getModeler(), (Profile) selectedObject));
        }
    }
}
