package org.topcased.modeler.sysml.editor.properties.sections;

import java.util.List;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.topcased.modeler.listeners.UIAdapterImpl;
import org.topcased.sysml.FlowDirectionKind;
import org.topcased.sysml.FlowPort;
import org.topcased.sysml.SysMLPackage;
import org.topcased.tabbedproperties.sections.AbstractEnumerationPropertySection;

public class FlowPortDirectionPropertySection extends AbstractEnumerationPropertySection
{

    protected UIAdapterImpl flowPortListener = new UIAdapterImpl()
    {

        protected void safeNotifyChanged(Notification msg)
        {
            updateComboState();
        }
    };

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractEnumerationPropertySection#getFeature()
     */
    protected EStructuralFeature getFeature()
    {
        return SysMLPackage.eINSTANCE.getFlowPort_Direction();
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractEnumerationPropertySection#getFeatureAsText()
     */
    protected String getFeatureAsText()
    {
        return ((FlowPort) getEObject()).getDirection().getName();
        // return ((FlowPort) eObject).getDirection().getName();
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractEnumerationPropertySection#getFeatureValue(int)
     */
    protected Object getFeatureValue(int index)
    {
        return FlowDirectionKind.get(index);
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractEnumerationPropertySection#getLabelText()
     */
    protected String getLabelText()
    {
        return "Direction:";
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractEnumerationPropertySection#isEqual(int)
     */
    protected boolean isEqual(int index)
    {
        return FlowDirectionKind.VALUES.get(index).equals(((FlowPort) getEObject()).getDirection());
        // return FlowDirectionKind.VALUES.get(index).equals(((FlowPort)
        // eObject).getDirection());
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractEnumerationPropertySection#getEnumerationFeatureValues()
     */
    protected String[] getEnumerationFeatureValues()
    {
        List< ? > values = FlowDirectionKind.VALUES;
        String[] ret = new String[values.size()];
        for (int i = 0; i < values.size(); i++)
        {
            ret[i] = ((FlowDirectionKind) values.get(i)).getName();
        }
        return ret;
    }

    /**
     * Listen to flow port when the controls are about to be shown
     */
    public void aboutToBeShown()
    {
        // if(eObject != null &&
        // !eObject.eAdapters().contains(flowPortListener))
        // {
        // eObject.eAdapters().add(flowPortListener);
        // }
        if (getEObject() != null && !getEObject().eAdapters().contains(flowPortListener))
        {
            getEObject().eAdapters().add(flowPortListener);
        }
        super.aboutToBeShown();
    }

    /**
     * Unlisten to flow port when the controls are about to be hidden
     */
    public void aboutToBeHidden()
    {
        if (getEObject() != null && getEObject().eAdapters().contains(flowPortListener))
        // if(eObject != null && eObject.eAdapters().contains(flowPortListener))
        {
            getEObject().eAdapters().remove(flowPortListener);
            // eObject.eAdapters().remove(flowPortListener);
        }
        super.aboutToBeHidden();
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractEnumerationPropertySection#refresh()
     */
    public void refresh()
    {
        super.refresh();
        updateComboState();
    }

    private void updateComboState()
    {
        try
        {
            if (((FlowPort) getEObject()).isIsAtomic())
            // if(((FlowPort) eObject).isIsAtomic())
            {
                getCombo().setEnabled(true);
                // combo.setEnabled(true);
            }
            else
            {
                getCombo().setEnabled(true);
                // combo.setEnabled(false);
            }
        }
        catch (UnsupportedOperationException e)
        {
            getCombo().setEnabled(true);
            // combo.setEnabled(true);
        }
    }

    // @Override
    protected Object getOldFeatureValue()
    {
        // TODO Auto-generated method stub
        return null;
    }
}
