package org.topcased.modeler.sysml.editor.properties.sections;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.topcased.modeler.listeners.UIAdapterImpl;
import org.topcased.sysml.FlowPort;
import org.topcased.sysml.SysMLPackage;
import org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection;

public class FlowPortIsConjugatedPropertySection extends AbstractBooleanPropertySection
{

    protected UIAdapterImpl flowPortListener = new UIAdapterImpl()
    {

        protected void safeNotifyChanged(Notification msg)
        {
            updateComboState();
        }
    };

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection#getFeature()
     */
    protected EStructuralFeature getFeature()
    {
        return SysMLPackage.eINSTANCE.getFlowPort_IsConjugated();
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection#getFeatureValue()
     */
    protected boolean getFeatureValue()
    {
        return ((FlowPort) getEObject()).isIsConjugated();
        // return ((FlowPort) eObject).isIsConjugated();
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection#getCheckButtonText()
     */
    protected String getCheckButtonText()
    {
        return "isConjugated";
    }

    /**
     * Listen to flow port when the controls are about to be shown
     */
    public void aboutToBeShown()
    {
        if (getEObject() != null && !getEObject().eAdapters().contains(flowPortListener))
        {
            getEObject().eAdapters().add(flowPortListener);
        }
        // if(eObject != null &&
        // !eObject.eAdapters().contains(flowPortListener))
        // {
        // eObject.eAdapters().add(flowPortListener);
        // }
        super.aboutToBeShown();
    }

    /**
     * Unlisten to flow port when the controls are about to be hidden
     */
    public void aboutToBeHidden()
    {
        // if(eObject != null && eObject.eAdapters().contains(flowPortListener))
        // {
        // eObject.eAdapters().remove(flowPortListener);
        // }
        if (getEObject() != null && getEObject().eAdapters().contains(flowPortListener))
        {
            getEObject().eAdapters().remove(flowPortListener);
        }
        super.aboutToBeHidden();
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractEnumerationPropertySection#refresh()
     */
    public void refresh()
    {
        super.refresh();
        updateComboState();
    }

    private void updateComboState()
    {
        try
        {
            if (((FlowPort) getEObject()).isIsAtomic())
                getCheckButton().setEnabled(false);
            else
                getCheckButton().setEnabled(true);
            // if(((FlowPort) eObject).isIsAtomic())
            // {
            // checkButton.setEnabled(false);
            // }
            // else
            // {
            // checkButton.setEnabled(true);
            // }
        }
        catch (UnsupportedOperationException e)
        {
            getCheckButton().setEnabled(false);
            // checkButton.setEnabled(false);
        }
    }

    // @Override
    protected String getLabelText()
    {
        // TODO Auto-generated method stub
        return getCheckButtonText();
    }
}
