/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation, Thomas Friol (Anyware Technologies), Nicolas Laleve (Anyware
 * Technologies)
 ******************************************************************************/

package org.topcased.modeler.sysml.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.layout.BorderAttachedLayout;
import org.topcased.modeler.figures.DiagramFigure;

public class SysMLDiagramFigure extends DiagramFigure
{
    private static final int HEADER_MARGIN = 5;

    public SysMLDiagramFigure()
    {
        super();

        setOpaque(true);
        setBackgroundColor(Display.getDefault().getSystemColor(SWT.COLOR_WHITE));
        setBorder(new LineBorder());
    }

    public void createContents()
    {
        BorderAttachedLayout borderLayout = new BorderAttachedLayout();
        setLayoutManager(borderLayout);

        setLabel(createLabel());

        setContentPane(this);
        add(getLabel(), new Rectangle(HEADER_MARGIN, HEADER_MARGIN, -1, -1));
    }

    protected ILabel createLabel()
    {
        return new EditableLabel();
    }

    protected void paintFigure(Graphics graphics)
    {
        super.paintFigure(graphics);
        graphics.pushState();

        Rectangle headerBounds = getLabel().getBounds();

        int x1 = headerBounds.x - HEADER_MARGIN;
        int x2 = headerBounds.x + headerBounds.width;
        int y1 = headerBounds.y - HEADER_MARGIN;
        int y2 = headerBounds.y + headerBounds.height + HEADER_MARGIN;

        int corner = (headerBounds.height + 2 * HEADER_MARGIN) / 2;

        graphics.setLineWidth(1);
        graphics.drawLine(x1, y2, x2, y2);
        graphics.drawLine(x2, y2, x2 + corner, y2 - corner);
        graphics.drawLine(x2 + corner, y2 - corner, x2 + corner, y1);
        graphics.popState();
    }
}
