/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.sysml.internalblockdiagram.commands;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.eclipse.uml2.uml.Type;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.commands.IDialogCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.sysml.internalblockdiagram.InternalBlockPlugin;
import org.topcased.modeler.sysml.internalblockdiagram.dialogs.ReferencePropertyDialog;
import org.topcased.modeler.utils.LabelHelper;
import org.topcased.modeler.utils.Utils;
import org.topcased.sysml.SysMLFactory;

public class CreatePropertyCommand extends CreateGraphNodeCommand implements IDialogCommand
{
	/** The Type to affect */
	protected Type refType;

	/** The name of the class to create */
	protected String typeName;

	/** Indicates if the new class has to be created */
	protected boolean createClassifier;

	/**
	 * The Constructor
	 * 
	 * @param domain the EditDomain (used to init the name)
	 * @param newObject the node to create
	 * @param newParent the parent node
	 * @param location the location of the created node
	 * @param dimension the size of the created node
	 * @param pos
	 * @param needModelUpdate <code>true</code> if the model must be modified during this command, <code>false</code> if objects already exist in the mode.
	 */
	public CreatePropertyCommand(EditDomain domain, GraphNode newObject, GraphNode newParent, Point location, Dimension dimension, int pos,
			boolean needModelUpdate)
	{
		super(domain, newObject, newParent, Utils.getElement(newParent), location, dimension, pos, null, needModelUpdate);
	}

	/**
	 * @see org.topcased.modeler.commands.CreateGraphNodeCommand#redoModel()
	 */
	protected void redoModel()
	{
		if (createClassifier)
		{
			// creates a model object 'Block' and adds it to the nearest Package element.
			refType = SysMLFactory.eINSTANCE.createBlock();
			refType.setName(typeName);
			((StructuredClassifier) getParentEObject()).getNearestPackage().getPackagedElements().add(refType);
		}

		// initializes the name given for the CollaborationUse
		String curName = LabelHelper.INSTANCE.getName(getEditDomain(), getChildEObject());
		if (curName == null || "".equals(curName))
		{
			LabelHelper.INSTANCE.initName(getEditDomain(), getParentEObject(), getChildEObject());
		}

		// the current Property is linked to the new block ref
		((Property) getChildEObject()).setType(refType);

		// the Property is attached to the current StructuredClassifier
		((StructuredClassifier) getParentEObject()).getOwnedAttributes().add((Property) getChildEObject());
	}

	/**
	 * @see org.topcased.modeler.commands.CreateGraphNodeCommand#undoModel()
	 */
	protected void undoModel()
	{
		if (createClassifier)
		{
			((StructuredClassifier) getParentEObject()).getNearestPackage().getPackagedElements().remove(refType);
		}
		((StructuredClassifier) getParentEObject()).getOwnedAttributes().remove(getChildEObject());
	}

	/**
	 * @see org.topcased.modeler.commands.IDialogCommand#openDialog()
	 */
	public boolean openDialog()
	{
		Shell currentShell = InternalBlockPlugin.getActiveWorkbenchShell();
		ReferencePropertyDialog dialog = new ReferencePropertyDialog((StructuredClassifier) getParentEObject(), currentShell);
		if (dialog.open() == Window.OK)
		{
			createClassifier = dialog.getCreateType();
			refType = dialog.getType();
			typeName = dialog.getNewTypeName();

			return true;
		}

		return false;
	}

	/**
	 * @see org.topcased.modeler.commands.CreateGraphNodeCommand#getParentEObject()
	 */
	protected EObject getParentEObject()
	{
		// Case where the parent is a Block Property
		if (super.getParentEObject() instanceof Property)
		{
			// The element must be added to the Type of the parent
			return ((Property) super.getParentEObject()).getType();
		}

		return super.getParentEObject();
	}
}
