/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.internalblockdiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;

/**
 * Property restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class PropertyRestoreConnectionCommand extends NamedElementRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public PropertyRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated
     */
    protected void initializeCommands()
    {

        super.initializeCommands();

        GraphElement elt = getGraphElement();
        EObject eltObject = Utils.getElement(elt);

        if (eltObject instanceof Property)
        {
            Iterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement
                        && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(elt);
                    GraphElement elt2 = (GraphElement) obj;
                    EObject eltObject2 = Utils.getElement(elt2);

                    if (eltObject2 instanceof Property)
                    {
                        if (autoRef)
                        {
                            // autoRef not allowed
                        }
                        else
                        {
                            // if the elt is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createConnectorFromPropertyToProperty(elt, elt2);
                            // if elt is the target of the edge or if it is the
                            // source and that the SourceTargetCouple is
                            // reversible
                            createConnectorFromPropertyToProperty(elt2, elt);
                        }
                    }

                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createConnectorFromPropertyToProperty(GraphElement srcNode, GraphElement targetNode)
    {
        Property sourceObject = (Property) Utils.getElement(srcNode);
        Property targetObject = (Property) Utils.getElement(targetNode);

        if (sourceObject.eContainer() instanceof StructuredClassifier)
        {
            EList< ? > edgeObjectList = ((StructuredClassifier) sourceObject.eContainer()).getOwnedConnectors();
            for (Iterator< ? > it = edgeObjectList.iterator(); it.hasNext();)
            {
                Object obj = it.next();
                if (obj instanceof Connector)
                {
                    Connector edgeObject = (Connector) obj;
                    EList< ? > ends = edgeObject.getEnds();
                    if (ends.size() >= 2 && sourceObject.equals(((ConnectorEnd) ends.get(0)).getRole())
                            && targetObject.equals(((ConnectorEnd) ends.get(1)).getRole()))
                    {
                        // check if the relation does not exists yet
                        List< ? > existing = getExistingEdges(srcNode, targetNode, Connector.class);
                        if (!isAlreadyPresent(existing, edgeObject))
                        {
                            ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                            // restore the link with its default presentation
                            GraphElement edge = factory.createGraphElement(edgeObject);
                            if (edge instanceof GraphEdge)
                            {
                                ConnectorEdgeCreationCommand cmd = new ConnectorEdgeCreationCommand(getEditDomain(),
                                        (GraphEdge) edge, srcNode, false);
                                cmd.setTarget(targetNode);
                                add(cmd);
                            }
                        }
                    }
                }
            }
        }
    }

}