package org.topcased.modeler.sysml.internalblockdiagram.dialogs;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;
import org.topcased.facilities.widgets.SingleObjectChooser;
import org.topcased.modeler.figures.EdgeObjectOffsetEditableLabel;
import org.topcased.modeler.sysml.SysMLPlugin;
import org.topcased.modeler.sysml.internalblockdiagram.figures.ConnectorFigure;
import org.topcased.modeler.sysml.internalblockdiagram.util.BlockTools;
import org.topcased.sysml.provider.SysMLItemProviderAdapterFactory;


public class ItemFlowNameEditDialog extends Dialog implements ITopcasedDialogConstants
{
    private Connector _connector;
    
    private ConnectorFigure _connectorFigure;
    
    private Text _itemFlowNameRightToLeft;
    
    private Text _itemFlowNameLeftToRight;
    
    private SingleObjectChooser _typeRightToLeft;
    
    private SingleObjectChooser _typeLeftToRight;
    
    private Map _data;
    
    private String _itemflowIconDirection; 
    
    private final int ITEMFLOWNAME_NAME = 0;
    
    private final int ITEMFLOWNAME_TYPE = 1;
    
    private final int ITEMFLOWNAMESECOND_NAME = 2;
    
    private  final int ITEMFLOWNAMESECOND_TYPE = 3;
    
    /** The ID of the itemFlow return name */
    public static final String  ITEMFLOWNAME_RETURN_NAME = "itemFlow name return name";
    
    /** The ID of the itemFlow return type */
    public static final String  ITEMFLOWNAME_RETURN_TYPE = "itemFlow name return type";

    /** The ID of the itemFlow second return name */
    public static final String  ITEMFLOWNAMESECOND_RETURN_NAME = "itemFlow second name return name";
    
    /** The ID of the itemFlow second return type */
    public static final String  ITEMFLOWNAMESECOND_RETURN_TYPE = "itemFlow second name return type";

    
    
    
    public ItemFlowNameEditDialog(Connector connector,ConnectorFigure connectorFigure, Shell parentShell, String direction)
    {
        super(parentShell);
        setBlockOnOpen(true);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        _connector = connector;
        _connectorFigure = connectorFigure;
        _itemflowIconDirection = direction;
    }
    
    /**
     * Set the title of the new shell
     * 
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("ItemFlow of Connector " + _connector.getName());
        newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

        super.configureShell(newShell);
    }
    
    /**
     * Create the Dialog area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        // Dialog
        Composite dialogComposite = (Composite) super.createDialogArea(parent);
        GridData dialogLayoutData = new GridData(GridData.FILL_BOTH);
        dialogLayoutData.widthHint = DEFAULT_DIALOG_WIDTH;
        dialogLayoutData.heightHint = DEFAULT_DIALOG_HEIGHT;
        dialogComposite.setLayoutData(dialogLayoutData);

        createItemFlowGroup(dialogComposite);

        loadData();

        return dialogComposite;
    }

    /**
     * Creates the group
     * 
     * @param parent the parent Composite
     */
    private void createItemFlowGroup(Composite parent)
    {
        Composite composite = new Composite(parent, SWT.NONE);
        composite.setLayout(new GridLayout(2, false));
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        ComposedAdapterFactory factory = new ComposedAdapterFactory(new AdapterFactory[] {
                new UMLItemProviderAdapterFactory(), new SysMLItemProviderAdapterFactory()});
        ILabelProvider labelProvider = new AdapterFactoryLabelProvider(factory);
        if(isFlowFromRightToLeft())
        {  
            Label itemTitlelbl = new Label(composite, SWT.NONE);
            itemTitlelbl.setText(getTitleLabelUpAndLeft());
            Label fake = new Label(composite,SWT.NONE); 
            Label itemFlowNameLbl = new Label(composite, SWT.NONE);
            itemFlowNameLbl.setText("Name : ");
            itemFlowNameLbl.setToolTipText("Character \":\" are forbidden in the name label");
            _itemFlowNameRightToLeft = new Text(composite, SWT.BORDER);
            _itemFlowNameRightToLeft.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

            Label itemFlowTypeLbl = new Label(composite, SWT.NONE);
            itemFlowTypeLbl.setText("Type : ");
            _typeRightToLeft = new SingleObjectChooser(composite, SWT.NONE);
            _typeRightToLeft.setLabelProvider(labelProvider);
            _typeRightToLeft.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        }
        
        if(isFlowFromLeftToRight())
        {
            Label itemTitleSecondlbl = new Label(composite, SWT.NONE);
            itemTitleSecondlbl.setText(getTitleLabelDonwAndRight());
        //    itemTitleSecondlbl.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
            Label fake = new Label(composite,SWT.NONE); 
            Label itemFlowNameSecondLbl = new Label(composite, SWT.NONE);
            itemFlowNameSecondLbl.setText("Name : ");
            itemFlowNameSecondLbl.setToolTipText("Character \":\" are forbidden in the name label");
            _itemFlowNameLeftToRight = new Text(composite, SWT.BORDER);
            _itemFlowNameLeftToRight.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

   
            Label itemFlowTypeSecondLbl = new Label(composite, SWT.NONE);
            itemFlowTypeSecondLbl.setText("Type : ");
            _typeLeftToRight = new SingleObjectChooser(composite, SWT.NONE);
            _typeLeftToRight.setLabelProvider(labelProvider);
            _typeLeftToRight.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        }

    }

    /**
     * Initialize the content of the widgets
     */
    private void loadData()
    {
        Collection< ? > reachableTypes = ItemPropertyDescriptor.getReachableObjectsOfType(_connector,
                UMLPackage.eINSTANCE.getType());
      
        if (isFlowFromRightToLeft())
        {
            if ((EdgeObjectOffsetEditableLabel) (_connectorFigure.getConnectorItemFlowNameEdgeObjectFigure()) != null)
                _itemFlowNameRightToLeft.setText(getCheckedTextBiDirectionnal(
                        (((EdgeObjectOffsetEditableLabel) (_connectorFigure.getConnectorItemFlowNameEdgeObjectFigure())).getText()),
                        ITEMFLOWNAME_NAME));

            _typeRightToLeft.setChoices(reachableTypes.toArray());
            if (BlockTools.getOldType(
                    getCheckedTextBiDirectionnal(
                            (((EdgeObjectOffsetEditableLabel) (_connectorFigure.getConnectorItemFlowNameEdgeObjectFigure())).getText()),
                            ITEMFLOWNAME_TYPE), _connector) != null)
            {
                _typeRightToLeft.setSelection(BlockTools.getOldType(
                        getCheckedTextBiDirectionnal(
                                (((EdgeObjectOffsetEditableLabel) (_connectorFigure.getConnectorItemFlowNameEdgeObjectFigure())).getText()),
                                ITEMFLOWNAME_TYPE), _connector));
            }
        }

        if (isFlowFromLeftToRight())
        {
            if ((EdgeObjectOffsetEditableLabel) (_connectorFigure.getConnectorItemFlowNameEdgeObjectFigure()) != null)
                _itemFlowNameLeftToRight.setText(getCheckedTextBiDirectionnal(
                        (((EdgeObjectOffsetEditableLabel) (_connectorFigure.getConnectorItemFlowNameEdgeObjectFigure())).getText()),
                        ITEMFLOWNAMESECOND_NAME));

            _typeLeftToRight.setChoices(reachableTypes.toArray());
            if (BlockTools.getOldType(
                    getCheckedTextBiDirectionnal(
                            (((EdgeObjectOffsetEditableLabel) (_connectorFigure.getConnectorItemFlowNameEdgeObjectFigure())).getText()),
                            ITEMFLOWNAMESECOND_TYPE), _connector) != null)
            {
                _typeLeftToRight.setSelection(BlockTools.getOldType(
                        getCheckedTextBiDirectionnal(
                                (((EdgeObjectOffsetEditableLabel) (_connectorFigure.getConnectorItemFlowNameEdgeObjectFigure())).getText()),
                                ITEMFLOWNAMESECOND_TYPE), _connector));
            }
        }
    }

    /**
     * Save the values before the widgets are disposed
     * 
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        _data = new HashMap();
        try
        {
            if(isFlowFromRightToLeft())
            {
                _data.put(ITEMFLOWNAME_RETURN_NAME, getCheckedTextBiDirectionnal(_itemFlowNameRightToLeft.getText(),ITEMFLOWNAME_NAME));
                _data.put(ITEMFLOWNAME_RETURN_TYPE, _typeRightToLeft.getSelection());
            }
            if(isFlowFromLeftToRight())
            {
                _data.put(ITEMFLOWNAMESECOND_RETURN_NAME, getCheckedTextBiDirectionnal(_itemFlowNameLeftToRight.getText(),ITEMFLOWNAMESECOND_NAME));
                _data.put(ITEMFLOWNAMESECOND_RETURN_TYPE, _typeLeftToRight.getSelection());
            }
            super.okPressed();
        }
        catch (Exception e)
        {
            // TODO change this with a validation listener that disable the ok
            // button until the widgets are valid
            SysMLPlugin.log("Required fields", IStatus.WARNING);
            MessageDialog.openWarning(getShell(), "Required parameters",
                    "Some parameters are not set.\nPlease, fill those fields before validating.");
        }
    }
    
    /**
     * 
     * 
     */
    private String getCheckedTextBiDirectionnal(String text, int position)
    {
        String[] textPart = text.split("        ");
        String textReturn = "";
        if(textPart.length == 1)
        {
            if(position == ITEMFLOWNAME_NAME || position == ITEMFLOWNAMESECOND_NAME)
                textReturn = getCheckedText(textPart[0], 0);
            else if(position == ITEMFLOWNAME_TYPE || position == ITEMFLOWNAMESECOND_TYPE)
                textReturn = getCheckedText(textPart[0], 1);          
        }
        else if(textPart.length == 2)
        {
            if(position == ITEMFLOWNAME_NAME)
                textReturn = getCheckedText(textPart[0], 0);
            else if(position == ITEMFLOWNAME_TYPE)
                textReturn = getCheckedText(textPart[0], 1);
            else if(position == ITEMFLOWNAMESECOND_NAME)
                textReturn = getCheckedText(textPart[1], 0);
            else if(position == ITEMFLOWNAMESECOND_TYPE)
                textReturn = getCheckedText(textPart[1], 1);
        }
        return textReturn;
    }
    
    /**
     * This method gets the different elements composing the name of an itemFlow and remove the :Type.
     * @param String text the text to check
     * @return String checkedText : the right name
     */
    private String getCheckedText(String text, int position)
    {
        String[] textPart =  text.split(":");
        if(textPart.length >position)
        {
            return textPart[position];
        }
        return text;
    }
    /**
     * Return a map containing property data that may changed
     * 
     * @return a Map
     */
    public Map getData()
    {
        return _data;
    }
    
    /**
     * Check if the flux is bidirectionnal 
     * 
     * @return boolean
     */
//    private boolean isFlowBidirectionnal()
//    {
//        if(_itemflowIconDirection == "")
//        {
//            return true;
//        }
//        return false;
//    }
    
    private boolean isFlowFromRightToLeft()
    {
        if(_itemflowIconDirection.equals("ITEMFLOWUP"))
        {
            return true;
        }
        else if(_itemflowIconDirection.equals("ITEMFLOWLEFT"))
        {
            return true;
        }
        else if(_itemflowIconDirection.equals("ITEMFLOWBOTH"))
        {
            return true;
        }
        else if(_itemflowIconDirection.equals("ITEMFLOWBOTHUPDOWN"))
        {
            return true;
        }
        return false;
    }
    
    private boolean isFlowFromLeftToRight()
    {
        if(_itemflowIconDirection.equals("ITEMFLOWDOWN"))
        {
            return true;
        }
        else if(_itemflowIconDirection.equals("ITEMFLOWRIGHT"))
        {
            return true;
        }
        else if(_itemflowIconDirection.equals("ITEMFLOWBOTH"))
        {
            return true;
        }
        else if(_itemflowIconDirection.equals("ITEMFLOWBOTHUPDOWN"))
        {
            return true;
        }
        return false;
    }
    
    private String getTitleLabelUpAndLeft()
    {
        if(_itemflowIconDirection.equals("ITEMFLOWUP"))
        {
            return "Up Direction Arrow :";
        }
        else if(_itemflowIconDirection.equals("ITEMFLOWLEFT"))
        {
            return "Left Direction Arrow :";
        }
        else if(_itemflowIconDirection.equals("ITEMFLOWBOTH"))
        {
            return "Left Direction Arrow :";
        }
        else if(_itemflowIconDirection.equals("ITEMFLOWBOTHUPDOWN"))
        {
            return "Up Direction Arrow :";
        }
        return "";
    }
    
    private String getTitleLabelDonwAndRight()
    {
        if(_itemflowIconDirection.equals("ITEMFLOWDOWN"))
        {
            return "Down Direction Arrow :";
        }
        else if(_itemflowIconDirection.equals("ITEMFLOWRIGHT"))
        {
            return "Right Direction Arrow :";
        }
        else if(_itemflowIconDirection.equals("ITEMFLOWBOTH"))
        {
            return "Right Direction Arrow :";
        }
        else if(_itemflowIconDirection.equals("ITEMFLOWBOTHUPDOWN"))
        {
            return "Down Direction Arrow :";
        }
        return "";
    }
  
}
