/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.internalblockdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.uml2.uml.Actor;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.TypedElement;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.commands.DeleteGraphElementCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sysml.internalblockdiagram.commands.BlockPropertyRestoreConnectionCommand;
import org.topcased.modeler.sysml.internalblockdiagram.figures.BlockPropertyFigure;
import org.topcased.modeler.sysml.internalblockdiagram.policies.BlockPropertyLayoutEditPolicy;
import org.topcased.modeler.sysml.util.SysMLTools;

/**
 * The BlockProperty object controller <!-- begin-user-doc --> <!-- end-user-doc
 * -->
 * 
 * @generated
 */
public class BlockPropertyEditPart extends PropertyEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public BlockPropertyEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new BlockPropertyRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new BlockPropertyLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated NOT
     */
    protected IFigure createFigure()
    {

        int type = BlockPropertyFigure.NON_COMPOSITE_PROPERTY;
        Association asso = ((Property) getEObject()).getAssociation();
        Property elt = (asso!=null)?asso.getMemberEnds().get(0):null;
        
        if (asso == null || (elt!=null && elt.getAggregation() == AggregationKind.COMPOSITE_LITERAL) || ((Property) getEObject()).getAggregation() == AggregationKind.COMPOSITE_LITERAL )
            type = BlockPropertyFigure.COMPOSITE_PROPERTY;
        return new BlockPropertyFigure(type);
//        int type = BlockPropertyFigure.NON_COMPOSITE_PROPERTY;
//        
//        if (((Property) getEObject()).getAggregation() == AggregationKind.COMPOSITE_LITERAL)
//            type = BlockPropertyFigure.COMPOSITE_PROPERTY;
//        return new BlockPropertyFigure(type);
    }

    /**
     * Handle property type or aggregation changes
     * 
     * @see org.topcased.modeler.edit.BaseEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {
        Object newObject = msg.getNewValue();

        if (msg.getNotifier() instanceof Property)
        {
            // Handle aggregation change (composite or not)
            if (msg.getFeatureID(Property.class) == UMLPackage.PROPERTY__AGGREGATION
                    && ((Property) msg.getNotifier()).getAggregation() != msg.getOldValue())
            {
                handlePropertyAggregationChanged((AggregationKind) newObject);
            }
            // Handle type change
            if (msg.getFeatureID(TypedElement.class) == UMLPackage.TYPED_ELEMENT__TYPE
                    && ((Property) msg.getNotifier()).getType() != msg.getOldValue())
            {
                handlePropertyTypeChanged();
            }
        }
        super.handleModelChanged(msg);
    }

    private void handlePropertyAggregationChanged(AggregationKind newValue)
    {
        int aggregType;
        if (newValue == AggregationKind.COMPOSITE_LITERAL)
        {
            aggregType = BlockPropertyFigure.COMPOSITE_PROPERTY;
        }
        else
        {
            aggregType = BlockPropertyFigure.NON_COMPOSITE_PROPERTY;
        }
        ((BlockPropertyFigure) getFigure()).setType(aggregType);
    }

    private void handlePropertyTypeChanged()
    {
        // Childs graph nodes for which model objects are stored
        // in the old property type need to be deleted
        CompoundCommand command = new CompoundCommand();
        for (int i = 0; i < getGraphNode().getContained().size(); i++)
        {
            Object child = getGraphNode().getContained().get(i);
            if (child instanceof GraphNode)
            {
                command.add(new DeleteGraphElementCommand((GraphNode) child, true));
            }
        }
        getViewer().getEditDomain().getCommandStack().execute(command);
    }

    /**
     * This method is overriden because of properties which are stored in a
     * specific figure
     * 
     * @see org.eclipse.gef.AbstractGraphicalEditPart#addChildVisual(org.eclipse.gef.EditPart,
     *      int)
     */
    protected void addChildVisual(EditPart childEditPart, int index)
    {
        if (childEditPart.getClass().equals(PropertyEditPart.class))
        {
            // Properties are added in a specific figure
            IFigure child = ((GraphicalEditPart) childEditPart).getFigure();
            IFigure attributesFigure = ((BlockPropertyFigure) getFigure()).getContentPane();
            attributesFigure.add(child);
            if (!attributesFigure.isVisible())
                attributesFigure.setVisible(true);
        }
        else
        {
            super.addChildVisual(childEditPart, -1);
        }
    }

    /**
     * This method is overriden because of properties which are stored in a
     * specific figure
     * 
     * @see org.eclipse.gef.AbstractGraphicalEditPart#removeChildVisual(org.eclipse.gef.EditPart)
     */
    protected void removeChildVisual(EditPart childEditPart)
    {
        if (childEditPart.getClass().equals(PropertyEditPart.class))
        {
            IFigure child = ((GraphicalEditPart) childEditPart).getFigure();
            IFigure attributesFigure = ((BlockPropertyFigure) getFigure()).getContentPane();
            attributesFigure.remove(child);
            if (attributesFigure.getChildren().size() == 0)
                attributesFigure.setVisible(false);
        }
        else
        {
            super.removeChildVisual(childEditPart);
        }
    }

    /**
     * @see org.topcased.modeler.sysml.internalblockdiagram.edit.PropertyEditPart#getPropertyPrefix(org.eclipse.uml2.uml.Property)
     */
    protected String getPropertyPrefix(Property property)
    {
        StringBuffer prefix = new StringBuffer();

        String stereotypes = SysMLTools.getStereotypesList(property);
        if (stereotypes.length() != 0)
            prefix.append("<<");

        if (property.getType() instanceof Actor)
        {
            if (stereotypes.length() != 0)
            {
                stereotypes = "actor, " + stereotypes;
            }
            else
            {
                stereotypes = "<<actor";
            }
        }
        prefix.append(stereotypes);

        if (stereotypes.length() != 0)
            prefix.append(">>");

        if (property.isDerived())
        {
            prefix.append('/');
        }

        return prefix.toString();
    }
}