/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.internalblockdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.TypedElement;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.NonResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.figures.DiagramFigure;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sysml.internalblockdiagram.InternalBlockEditPolicyConstants;
import org.topcased.modeler.sysml.internalblockdiagram.commands.PortRestoreConnectionCommand;
import org.topcased.modeler.sysml.internalblockdiagram.figures.PortFigure;
import org.topcased.modeler.sysml.internalblockdiagram.figures.PortLabelledFigure;
import org.topcased.modeler.sysml.internalblockdiagram.figures.PortProvidedInterfaceFigure;
import org.topcased.modeler.sysml.internalblockdiagram.figures.PortRequiredInterfaceFigure;
import org.topcased.modeler.sysml.internalblockdiagram.policies.CommentLinkEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.internalblockdiagram.policies.ConnectorEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.internalblockdiagram.policies.DependencyEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.util.SysMLTools;
import org.topcased.modeler.utils.Utils;

/**
 * The Port object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class PortEditPart extends PropertyEditPart
{

	protected PortProvidedInterfaceFigure _providedFigure;

	protected PortRequiredInterfaceFigure _requiredFigure;

	/**
	 * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @param obj the graph node
	 * @generated
	 */
	public PortEditPart(GraphNode obj)
	{
		super(obj);
	}

	/**
	 * Creates edit policies and associates these with roles <!-- begin-user-doc
	 * --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	protected void createEditPolicies()
	{
		super.createEditPolicies();

		// The three following policies have been added because this class
		// cannot inherit
		// from PortEditPart and from PropertyEditPart
		installEditPolicy(InternalBlockEditPolicyConstants.COMMENTLINK_EDITPOLICY,
				new CommentLinkEdgeCreationEditPolicy());
		installEditPolicy(InternalBlockEditPolicyConstants.DEPENDENCY_EDITPOLICY,
				new DependencyEdgeCreationEditPolicy());
		installEditPolicy(InternalBlockEditPolicyConstants.CONNECTOR_EDITPOLICY, new ConnectorEdgeCreationEditPolicy());

		installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
		{
			protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
			{
				return new PortRestoreConnectionCommand(getHost());
			}
		});

		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());

		installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new NonResizableEditPolicy());
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * @generated NOT
	 */
	protected IFigure createFigure()
	{

		return new PortLabelledFigure(new PortFigure());

	}

	/**
	 * Return the model port
	 * 
	 * @return the model port
	 */
	private Port getModelPort()
	{
		return (Port) Utils.getElement(getGraphNode());
	}

	/**
	 * Return the figure that contains provided and required interfaces figures
	 * 
	 * @return the figure
	 */
	private IFigure getInterfaceContainerFigure()
	{
		return getDiagramFigure(getFigure());
	}

	/**
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	public void activate()
	{
		super.activate();
		if (getModelPort().getType() != null)
		{
			getModelPort().getType().eAdapters().add(getModelListener());
		}
	}

	/**
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	public void deactivate()
	{
		IFigure interfacesContainerFigure = getInterfaceContainerFigure();

		if (_providedFigure != null)
		{
			getFigure().removeFigureListener(_providedFigure.getPortDeplacementListener());
			interfacesContainerFigure.remove(_providedFigure);
			_providedFigure = null;
		}

		if (_requiredFigure != null)
		{
			getFigure().removeFigureListener(_requiredFigure.getPortDeplacementListener());
			interfacesContainerFigure.remove(_requiredFigure);
			_requiredFigure = null;
		}

		if (getModelPort().getType() != null)
		{
			getModelPort().getType().eAdapters().remove(getModelListener());
		}

		super.deactivate();
	}

	protected void handleModelChanged(Notification msg)
	{
		// Handle type change
		Object newObject = msg.getNewValue();
		Object oldObject = msg.getOldValue();

		if (msg.getNotifier().equals(getEObject()))
		{
			if (msg.getFeatureID(TypedElement.class) == UMLPackage.TYPED_ELEMENT__TYPE)
			{
				updateModelListening(oldObject, newObject);
			}
		}

		if (msg.getEventType() != Notification.REMOVING_ADAPTER)
			super.handleModelChanged(msg);
	}

	/**
	 * @see org.topcased.modeler.edit.PortEditPart#refreshHeaderLabel()
	 */
	protected void refreshHeaderLabel()
	{
		ComposedLabel lbl = (ComposedLabel) ((ILabelFigure) getFigure()).getLabel();
		Port port = (Port) Utils.getElement(getGraphNode());
		if (port.getName() != null /*&& port.getName().length() != 0*/)
		{
			lbl.setPrefix(SysMLTools.getStereotypeNotation(port));
			lbl.setMain(port.getName());
			if (port.getType() != null)
			{
				lbl.setSuffix(" : " + port.getType().getName());
			}
			else 
			{
				lbl.setSuffix("");
			}
			if (port.getLower() == 0 && port.getUpper() == -1)
			{
				lbl.setSuffix(lbl.getSuffix().getText().concat(" [*]"));                
			}
			else if(port.getLower() != port.getUpper() )
			{
				if(port.getUpper() == -1)
				{
					lbl.setSuffix(lbl.getSuffix().getText().concat(" ["+port.getLower()+".. *]"));

				}
				else
				{
					lbl.setSuffix(lbl.getSuffix().getText().concat(" ["+port.getLower()+".."+port.getUpper()+"]"));
				}
			}
			else if (port.getLower() == port.getUpper() && port.getLower() != 1)
			{
				lbl.setSuffix(lbl.getSuffix().getText().concat(" ["+port.getLower()+"]"));
			}

		}
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals()
	{
		super.refreshVisuals();
		if (!(getFigure().getParent() instanceof DiagramFigure))
		{
			refreshProvidedInterfaces();
			refreshRequiredInterfaces();
		}
	}

	protected void refreshProvidedInterfaces()
	{
		IFigure interfacesContainerFigure = getInterfaceContainerFigure();
		if (getModelPort().getProvideds().size() != 0)
		{
			if (_providedFigure == null)
			{
				_providedFigure = new PortProvidedInterfaceFigure();
				interfacesContainerFigure.add(_providedFigure);
				getFigure().addFigureListener(_providedFigure.getPortDeplacementListener());
			}
			refreshProvidedInterfacesNames();
			_providedFigure.drawInterface(getFigure());
		}
		else
		{
			if (_providedFigure != null)
			{
				getFigure().removeFigureListener(_providedFigure.getPortDeplacementListener());
				interfacesContainerFigure.remove(_providedFigure);
				_providedFigure = null;
			}
		}
	}

	protected void refreshRequiredInterfaces()
	{
		IFigure interfacesContainerFigure = getInterfaceContainerFigure();
		if (getModelPort().getRequireds().size() != 0)
		{
			if (_requiredFigure == null)
			{
				_requiredFigure = new PortRequiredInterfaceFigure();
				interfacesContainerFigure.add(_requiredFigure);
				getFigure().addFigureListener(_requiredFigure.getPortDeplacementListener());
			}
			refreshRequiredInterfacesNames();
			_requiredFigure.drawInterface(getFigure());
		}
		else
		{
			if (_requiredFigure != null)
			{
				getFigure().removeFigureListener(_requiredFigure.getPortDeplacementListener());
				interfacesContainerFigure.remove(_requiredFigure);
				_requiredFigure = null;
			}
		}
	}

	public void refreshProvidedInterfacesNames()
	{
		String labelText = "";
		for (int i = 0; i < getModelPort().getProvideds().size(); i++)
		{
			Interface inter = (Interface) getModelPort().getProvideds().get(i);
			labelText += (i == 0 ? "" : ", ") + inter.getName();
		}
		_providedFigure.getNameLabel().setText(labelText);
	}

	public void refreshRequiredInterfacesNames()
	{
		String labelText = "";
		for (int i = 0; i < getModelPort().getRequireds().size(); i++)
		{
			Interface inter = (Interface) getModelPort().getRequireds().get(i);
			labelText += (i == 0 ? "" : ", ") + inter.getName();
		}
		_requiredFigure.getNameLabel().setText(labelText);
	}

	private DiagramFigure getDiagramFigure(IFigure figure)
	{
		if (figure == null || figure instanceof DiagramFigure)
			return (DiagramFigure) figure;
		return getDiagramFigure(figure.getParent());
	}
}