/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.internalblockdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.ValueSpecification;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sysml.internalblockdiagram.InternalBlockEditPolicyConstants;
import org.topcased.modeler.sysml.internalblockdiagram.commands.PropertyRestoreConnectionCommand;
import org.topcased.modeler.sysml.internalblockdiagram.figures.PropertyFigure;
import org.topcased.modeler.sysml.internalblockdiagram.policies.ConnectorEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.util.SysMLTools;
import org.topcased.modeler.utils.Utils;

/**
 * The Property object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class PropertyEditPart extends NamedElementEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public PropertyEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(InternalBlockEditPolicyConstants.CONNECTOR_EDITPOLICY, new ConnectorEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new PropertyRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());

        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new PropertyFigure();
    }

    /**
     * Set the name of the property with it's type and the stereotypes
     * 
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        // <property> ::= [<visibility>][/] <name> [ : <prop-type>] [[
        // <multiplicity> ]] [ = <default>]
        ComposedLabel lbl = (ComposedLabel) ((ILabelFigure) getFigure()).getLabel();
        Property property = (Property) Utils.getElement(getGraphNode());
        if (property.getName() != null && property.getName().length() != 0)
        {
            lbl.setPrefix(getPropertyPrefix(property));

            lbl.setMain(property.getName());

            lbl.setSuffix(getPropertySuffix(property));
        }
    }

    protected String getPropertyPrefix(Property property)
    {
        StringBuffer prefix = new StringBuffer();

        prefix.append(SysMLTools.getStereotypeNotation(property));

        prefix.append(SysMLTools.getVisibilityNotation(property.getVisibility()));

        if (property.isDerived())
        {
            prefix.append('/');
        }

        return prefix.toString();
    }

    protected String getPropertySuffix(Property property)
    {
        // [ : <prop-type>] [[ <multiplicity> ]] [ = <default>]
        StringBuffer suffix = new StringBuffer();

        // [ : <prop-type>]
        Type type = property.getType();
        if (type != null)
        {
            suffix.append(" : ");
            suffix.append(type.getName());
        }

        // [[<multiplicity>]]
        suffix.append(SysMLTools.getMultiplicityText(property));

        // [ = <default>]
        String defaultValue = property.getDefault();
        if (defaultValue != null && !"".equals(defaultValue))
        {
            suffix.append(" = ");
            suffix.append(defaultValue);
        }

        return suffix.toString();
    }

    protected void append(StringBuffer text, String fragment)
    {
        if (fragment != null && !"".equals(fragment))
        {
            if (text.length() > 0)
            {
                text.append(", ");
            }
            text.append(fragment);
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {

        Object notifier = msg.getNotifier();
        Object newObject = msg.getNewValue();
        Object oldObject = msg.getOldValue();

        if (notifier instanceof Property)
        {
            if (notifier == getEObject())
            {
                switch (msg.getFeatureID(Property.class))
                {
                    case UMLPackage.PROPERTY__TYPE:
                    case UMLPackage.PROPERTY__LOWER_VALUE:
                    case UMLPackage.PROPERTY__UPPER_VALUE:
                    case UMLPackage.PROPERTY__DEFAULT_VALUE:
                        updateModelListening(oldObject, newObject);
                        break;
                }
                super.handleModelChanged(msg);
            }
            else
            {
                refreshHeaderLabel();
            }
        }
        else if (notifier instanceof Type)
        {
            switch (msg.getFeatureID(NamedElement.class))
            {
                case UMLPackage.NAMED_ELEMENT__NAME:
                    refreshHeaderLabel();
                    break;
            }
        }
        else if (notifier instanceof ValueSpecification)
        {
            refreshHeaderLabel();
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
     */
    public void activate()
    {
        super.activate();
        EObject model = getEObject();
        if (model instanceof Property)
        {
            listenType((Property) getEObject());
            listenValues((Property) getEObject());
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
     */
    public void deactivate()
    {
        EObject model = getEObject();
        if (model instanceof Property)
        {
            unlistenValues((Property) getEObject());
            unlistenType((Property) getEObject());
        }
        super.deactivate();
    }

    /**
     * Listen to the bounds of the property
     * 
     * @param property the model object
     */
    private void listenValues(Property property)
    {
        // Only listen to object that are not yet listened
        if (property.getLowerValue() != null && !property.getLowerValue().eAdapters().contains(getModelListener()))
        {
            property.getLowerValue().eAdapters().add(getModelListener());
        }
        if (property.getUpperValue() != null && !property.getUpperValue().eAdapters().contains(getModelListener()))
        {
            property.getUpperValue().eAdapters().add(getModelListener());
        }
        if (property.getDefaultValue() != null && !property.getDefaultValue().eAdapters().contains(getModelListener()))
        {
            property.getDefaultValue().eAdapters().add(getModelListener());
        }
    }

    /**
     * Stop listening to the bounds of the property
     * 
     * @param property the model object
     */
    private void unlistenValues(Property property)
    {
        if (property.getLowerValue() != null)
        {
            property.getLowerValue().eAdapters().remove(getModelListener());
        }
        if (property.getUpperValue() != null)
        {
            property.getUpperValue().eAdapters().remove(getModelListener());
        }
        if (property.getDefaultValue() != null)
        {
            property.getDefaultValue().eAdapters().remove(getModelListener());
        }
    }

    /**
     * Listen to the type of the property
     * 
     * @param property the model object
     */
    private void listenType(Property property)
    {
        // Only listen to object that are not yet listened
        if (property.getType() != null && !property.getType().eAdapters().contains(getModelListener()))
        {
            property.getType().eAdapters().add(getModelListener());
        }
    }

    /**
     * Stop listening to the type of the property
     * 
     * @param property the model object
     */
    private void unlistenType(Property property)
    {
        if (property.getType() != null)
        {
            property.getType().eAdapters().remove(getModelListener());
        }
    }

    // stereotype actor pour les property de type acteur
    // tirets pour les property non composite
}