/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.internalblockdiagram.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Image;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.modeler.sysml.internalblockdiagram.InternalBlockImageRegistry;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */
public class FlowPortFigure extends PortFigure implements ILabelFigure
{

    /** The direction of the flow port * */
    private int direction;

    /** If the flow port is atomic * */
    private boolean isAtomic = true;

    /** If the flow port is conjugated * */
    private boolean isConjugated = false;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public FlowPortFigure()
    {
        // TODO : here, you can specify the type of port (IN, OUT or INOUT) to
        // have a better display
        super();
    }

    /**
     * @see org.topcased.draw2d.figures.ILabelFigure#getLabel()
     * @generated
     */
    public ILabel getLabel()
    {
        // TODO : return the Figure that represent the Label
        return null;
    }

    /**
     * Change the direction of the port (in, out or in/out port)
     * 
     * @param portDirection the new direction of the port
     */
    public void setDirection(int portDirection)
    {
        direction = portDirection;
    }

    /**
     * Change port atomicity
     * 
     * @param atomic true if the port is atomic
     */
    public void setIsAtomic(boolean atomic)
    {
        isAtomic = atomic;
    }

    /**
     * Change port conjugation
     * 
     * @param conjugated true if the port is conjugated
     */
    public void setIsConjugated(boolean conjugated)
    {
        isConjugated = conjugated;
    }

    /**
     * Draw the figure that should represent the Port.
     * 
     * @param graphics
     */
    protected void fillShape(Graphics graphics)
    {
        graphics.pushState();
        graphics.setLineWidth(1);

        if (isAtomic)
        {
            switch (direction)
            {
                case IN_PORT:
                    paintInPort(graphics);
                    break;
                case OUT_PORT:
                    paintOutPort(graphics);
                    break;
                case IN_OUT_PORT:
                    paintInOutPort(graphics);
                    break;
            }
        }
        else
        {
            paintNonAtomicFlowPort(graphics);
        }
        graphics.popState();
    }

    /**
     * Paint an InPort
     * 
     * 
     * @param graphics
     */
    protected void paintInPort(Graphics graphics)
    {
        if (getPosition() == PositionConstants.LEFT)
        {
            paintPortRight(graphics);
        }
        else if (getPosition() == PositionConstants.RIGHT)
        {
            paintPortLeft(graphics);
        }
        else if (getPosition() == PositionConstants.BOTTOM)
        {
            paintPortTop(graphics);
        }
        else if (getPosition() == PositionConstants.TOP)
        {
            paintPortBottom(graphics);
        }
    }

    /**
     * Paint an OutPort
     * 
     * 
     * @param graphics
     */
    protected void paintOutPort(Graphics graphics)
    {
        if (getPosition() == PositionConstants.LEFT)
        {
            paintPortLeft(graphics);
        }
        else if (getPosition() == PositionConstants.RIGHT)
        {
            paintPortRight(graphics);
        }
        else if (getPosition() == PositionConstants.BOTTOM)
        {
            paintPortBottom(graphics);
        }
        else if (getPosition() == PositionConstants.TOP)
        {
            paintPortTop(graphics);
        }
    }

    /**
     * Paint an InOutPort
     * 
     * 
     * @param graphics
     */

    protected void paintInOutPort(Graphics graphics)
    {
        if (getPosition() == PositionConstants.LEFT || getPosition() == PositionConstants.RIGHT)
        {
            paintInOutPortAtLeft(graphics);
        }
        else if (getPosition() == PositionConstants.BOTTOM || getPosition() == PositionConstants.TOP)
        {
            paintInOutPortAtBottom(graphics);
        }
    }

    /**
     * Draw an input port when its position is on the left border of the
     * container
     * 
     * @param graphics
     */
    protected void paintPortLeft(Graphics graphics)
    {
        Image image = InternalBlockImageRegistry.getImage("LEFTARROW");
        graphics.drawImage(image, new Rectangle(image.getBounds()), bounds);
    }

    /**
     * Draw an input port when its position is on the bottom border of the
     * container
     * 
     * @param graphics
     */
    protected void paintPortBottom(Graphics graphics)
    {
        Image image = InternalBlockImageRegistry.getImage("BOTTOMARROW");
        graphics.drawImage(image, new Rectangle(image.getBounds()), bounds);
    }

    /**
     * Draw an input port when its position is on the right border of the
     * container
     * 
     * @param graphics
     */
    protected void paintPortRight(Graphics graphics)
    {
        Image image = InternalBlockImageRegistry.getImage("RIGHTARROW");
        graphics.drawImage(image, new Rectangle(image.getBounds()), bounds);
    }

    /**
     * Draw an input port when its position is on the top border of the
     * container
     * 
     * @param graphics
     */
    protected void paintPortTop(Graphics graphics)
    {
        Image image = InternalBlockImageRegistry.getImage("TOPARROW");
        graphics.drawImage(image, new Rectangle(image.getBounds()), bounds);
    }

    /**
     * Draw an input/ouput port when its position is on the left border of the
     * container
     * 
     * @param graphics
     */
    protected void paintInOutPortAtLeft(Graphics graphics)
    {
        Image image = InternalBlockImageRegistry.getImage("LEFTRIGHTARROW");
        graphics.drawImage(image, new Rectangle(image.getBounds()), bounds);
    }

    /**
     * Draw an input/ouput port when its position is on the top border of the
     * container
     * 
     * @param graphics
     */
    protected void paintInOutPortAtTop(Graphics graphics)
    {
        Image image = InternalBlockImageRegistry.getImage("TOPBOTTOMARROW");
        graphics.drawImage(image, new Rectangle(image.getBounds()), bounds);
    }

    /**
     * Draw an input/ouput port when its position is on the bottom border of the
     * container
     * 
     * @param graphics
     */
    protected void paintInOutPortAtBottom(Graphics graphics)
    {
        paintInOutPortAtTop(graphics);
    }

    /**
     * Draw a non atomic flow port according its position
     * 
     * @param graphics
     */
    protected void paintNonAtomicFlowPort(Graphics graphics)
    {
        if (getPosition() == PositionConstants.LEFT || getPosition() == PositionConstants.RIGHT)
            paintHorizontalNonAtomicPort(graphics);
        else
            paintVerticalNonAtomicPort(graphics);
    }

    /**
     * Draw a non atomic flow port when its position is on the left or right
     * border of the container
     * 
     * @param graphics
     */
    protected void paintHorizontalNonAtomicPort(Graphics graphics)
    {
        Image image = InternalBlockImageRegistry.getImage((isConjugated ? "CONJUGATED" : "")
                + "NONATOMICLEFTRIGHTARROW");
        graphics.drawImage(image, new Rectangle(image.getBounds()), bounds);
    }

    /**
     * Draw a non atomic flow port when its position is on the top or left
     * border of the container
     * 
     * @param graphics
     */
    protected void paintVerticalNonAtomicPort(Graphics graphics)
    {
        Image image = InternalBlockImageRegistry.getImage((isConjugated ? "CONJUGATED" : "")
                + "NONATOMICTOPBOTTOMARROW");
        graphics.drawImage(image, new Rectangle(image.getBounds()), bounds);
    }
}