/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation, Thomas Friol (Anyware Technologies)
 ******************************************************************************/

package org.topcased.modeler.sysml.internalblockdiagram.figures;

import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FigureListener;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.geometry.Rectangle;
import org.topcased.draw2d.figures.IPortFigure;

/**
 * Represents a set of required or provided interfaces
 */
public abstract class PortInterfaceFigure extends Figure
{
    protected Rectangle _attachedPortBounds;

    protected int _attachedPortPosition;

    protected Label _interfacesNames;

    protected IFigure _interfaceFigure;

    protected IFigure _emptySpace;

    protected PortDeplacementListener _portDeplacementListener;

    protected final static int INTERFACE_WIDTH = 20;

    protected final static int INTERFACE_HEIGHT = 20;

    protected final static int NAMELABEL_XMARGIN = 10;

    protected final static int NAMELABEL_YMARGIN = 0;

    public PortInterfaceFigure()
    {
        setLayoutManager(new BorderLayout());
        _interfacesNames = new Label();
        _interfacesNames.setBorder(new MarginBorder(NAMELABEL_YMARGIN, NAMELABEL_XMARGIN, NAMELABEL_YMARGIN,
                NAMELABEL_XMARGIN));
        _emptySpace = new Figure();
        _emptySpace.setSize(-1, INTERFACE_HEIGHT);
        add(_interfacesNames);
        add(_emptySpace);
    }

    public Label getNameLabel()
    {
        return _interfacesNames;
    }

    public void drawInterface(IFigure attachedPort)
    {
        IPortFigure figure = null;
        if (attachedPort instanceof PortLabelledFigure)
        {
            figure = ((PortLabelledFigure) attachedPort).getPortFigure();
        }
        else if (attachedPort instanceof PortFigure)
        {
            figure = (PortFigure) attachedPort;
        }
        if (figure != null)
        {
            _attachedPortBounds = attachedPort.getBounds();
            _attachedPortPosition = figure.getPosition();
            updateFigureArrangement();
            updateFigureLocation();
            _interfaceFigure.repaint();
        }
    }

    public PortDeplacementListener getPortDeplacementListener()
    {
        if (_portDeplacementListener == null)
        {
            _portDeplacementListener = new PortDeplacementListener();
        }
        return _portDeplacementListener;
    }

    protected class PortDeplacementListener implements FigureListener
    {
        public void figureMoved(IFigure source)
        {
            drawInterface(source);
        }
    }

    protected abstract void updateFigureArrangement();

    protected abstract void updateFigureLocation();

    protected abstract class InterfaceRepresentation extends Figure
    {
    }
}
