package org.topcased.modeler.sysml.internalblockdiagram.policies;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.BendpointRequest;
import org.topcased.modeler.ModelerRequestConstants;
import org.topcased.modeler.commands.MoveEdgeObjectOffsetCommand;
import org.topcased.modeler.di.model.EdgeObjectOffset;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.policies.EdgeObjectOffsetEditPolicy;
import org.topcased.modeler.edit.requests.MoveEdgeObjectRequest;
import org.topcased.modeler.sysml.internalblockdiagram.InternalBlockEdgeObjectConstants;
import org.topcased.modeler.sysml.internalblockdiagram.figures.ConnectorFigure;
import org.topcased.modeler.sysml.internalblockdiagram.edit.ConnectorEditPart;

public class ConnectorEdgeObjectOffSetEditPolicy extends EdgeObjectOffsetEditPolicy
{
    /**
     * @see org.eclipse.gef.editpolicies.AbstractEditPolicy#getCommand(org.eclipse.gef.Request)
     */
    public Command getCommand(Request request)
    {   
        if (ModelerRequestConstants.REQ_CHANGE_EO_OFFSET.equals(request.getType()))
        {
            MoveEdgeObjectRequest moveRequest = (MoveEdgeObjectRequest) request;
            return getMoveEdgeObjectCommand(moveRequest);
        }
        else if (ModelerRequestConstants.REQ_MOVE_BENDPOINT.equals(request.getType()) || ModelerRequestConstants.REQ_CREATE_BENDPOINT.equals(request.getType()))
        {
          ((ConnectorEditPart)((BendpointRequest)request).getSource()).updateItemFlowIcon((ConnectorFigure)((BendpointRequest)request).getSource().getFigure());
          return null;
        }
        else
        {
            return null;
        }
   
    }

    /**
     * Gets the command to move an edge object.
     * @param request the move edge object request
     * @return a comand to move an edge object
     */
    protected Command getMoveEdgeObjectCommand(MoveEdgeObjectRequest request)
    {
        EdgeObjectOffset edgeObject = (EdgeObjectOffset) request.getEdgeObject();
        if(((edgeObject.getId()).equals(InternalBlockEdgeObjectConstants.CONNECTORITEMFLOWICON_EDGE_OBJECT_ID)))
        {
            return null;
        }
        Dimension newOffset = computeNewOffset(request.getMoveDelta(), edgeObject);
        return new MoveEdgeObjectOffsetCommand(edgeObject, newOffset);
    }
    
    /**
     * Compute the new edge object offset depending on the move delta.
     * 
     * @param moveDelta the move delta
     * @param edgeObject the edge object for the one the offset must be computed
     * @return a new edje object offset
     */
    private Dimension computeNewOffset(Dimension moveDelta, EdgeObjectOffset edgeObject)
    {
        Connection connection = ((GraphEdgeEditPart) getHost()).getConnectionFigure();
        Dimension delta = moveDelta.getCopy();
        connection.translateToRelative(delta);

        Dimension initialOffset = edgeObject.getOffset().getCopy();
        initialOffset.width += delta.width;
        initialOffset.height += delta.height;
        return initialOffset;
    }

}


