/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.sysml.internalblockdiagram.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.uml2.uml.DataType;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.Type;
import org.topcased.modeler.sysml.util.SysMLTools;
import org.topcased.sysml.Block;
import org.topcased.sysml.Dimension;
import org.topcased.sysml.FlowSpecification;
import org.topcased.sysml.ModelSYSML;
import org.topcased.sysml.Unit;
import org.topcased.sysml.ValueType;

/**
 * This class will be useful to find all the objects instance of 'Type' in the current model.<br>
 * It takes into account the full model exploration<br>
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public final class TypeHelper
{
	/**
	 * The default constructor
	 */
	public TypeHelper()
	{
		// no default constructor
	}
	
	/**
	 * Gets all the 'Type' objects of the model
	 * 
	 * @param container The start element to search in
	 * @return The list containing all the 'Type' objects referenced in the current model.
	 */
	public static List<?> getAvailableType(ModelSYSML model)
	{
		List<?> listAllAvailableObjects = SysMLTools.getAllObjects(model, Type.class);
		ArrayList<Object> listObjectsFiltered = new ArrayList<Object>();
		
		Iterator<?> iterator = listAllAvailableObjects.iterator();
		while(iterator.hasNext())
		{	
				listObjectsFiltered.add(iterator.next());	
		}
		return listObjectsFiltered;
	}
	
	/**
	 * Gets all the 'Type' objects of the model
	 * 
	 * @param container The start element to search in
	 * @return The list containing all the 'Type' objects referenced in the current model.
	 */
	public static List<?> getAvailablePortType(ModelSYSML model)
	{
		List<?> listAllAvailableObjects = SysMLTools.getAllObjects(model, Type.class);
		ArrayList<Object> listObjectsFiltered = new ArrayList<Object>();
		
		Iterator<?> iterator = listAllAvailableObjects.iterator();
		while(iterator.hasNext())
		{
			Object element = iterator.next();
			if(element instanceof Dimension || element instanceof Unit || element instanceof Enumeration)
			{
				
			}
			else if(element instanceof Block || element instanceof ValueType || element instanceof DataType || element instanceof FlowSpecification)
			{
				listObjectsFiltered.add(element);
			}			
		}
		return listObjectsFiltered;
	}
}
