/*******************************************************************************
 * 
 * Copyright AIRBUS FRANCE, 2005. All rights reserved.
 * 
 * This document and all information contained herein is the sole property of
 * AIRBUS FRANCE. No intellectual property rights are granted by the delivery of
 * this document or the disclosure of its content.
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.menus;

import java.util.Collection;
import java.util.Iterator;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.topcased.modeler.actions.CreateChildAction;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.editor.outline.ICreateChildMenu;
import org.topcased.modeler.sysml.util.ExactSysMLSwitch;

/**
 * A customized 'Create child' menu manager.
 * <!-- begin-user-doc --> <!--
 * end-user-doc -->
 * @generated
 */
public class CreateChildMenu extends MenuManager implements ICreateChildMenu
{
    /**
     * @generated
     */
    private static final String UNAPPLIED_ACTIONS_GROUP = "unappliedActions";

    /**
     * @generated
     */
    private MixedEditDomain domain;

    /**
     * @generated
     */
    private EObject selectedObject;

    /**
     * @generated
     */
    private Collection< ? > descriptors;

    /**
     * @generated
     */
    private MenuManager thisMenu;

    /**
     * The 'Packageable' menu manager.
     * @generated
     */
    private MenuManager packageableMenu;

    /**
     * The 'Objects' menu manager.
     * @generated
     */
    private MenuManager objectsMenu;

    /**
     * The 'Connections' menu manager.
     * @generated
     */
    private MenuManager connectionsMenu;

    /**
     * The 'Comment' menu manager.
     * @generated
     */
    private MenuManager commentMenu;

    /**
     * The 'UML' menu manager.
     * add to regroup all uml elements
     */
    private MenuManager umlMenu;
    
    /**
     * The 'Deprecated' menu manager.
     * add to contain RequirementDefinition and BlockDefinition elements which are deprecated
     */
    private MenuManager deprecatedMenu;

    /**
     * Constructor.
     * 
     * @generated
     */
    public CreateChildMenu()
    {
        super("Create child");
        thisMenu = this;
    }

    /**
     * @see org.topcased.modeler.editor.outline.ICreateChildMenu#setMixedEditDomain(org.topcased.modeler.editor.MixedEditDomain)
     */
    public void setMixedEditDomain(MixedEditDomain domain)
    {
        this.domain = domain;
    }

    /**
     * @see org.topcased.modeler.editor.outline.ICreateChildMenu#setSelectedEObject(org.eclipse.emf.ecore.EObject)
     */
    public void setSelectedEObject(EObject object)
    {
        selectedObject = object;
    }

    /**
     * Creates this menu contents.<br>
     * It creates the menu structure and add all the create child actions.<br> 
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated NOT
     */
    public void createMenuContents()
    {
        this.descriptors = domain.getEMFEditingDomain().getNewChildDescriptors(selectedObject, null);
        createMenus(this);

        for (Iterator it = descriptors.iterator(); it.hasNext();)
        {
            CommandParameter descriptor = (CommandParameter) it.next();
            EObject child = (EObject) descriptor.getValue();
            CreateChildAction action = new CreateChildAction(domain, selectedObject, descriptor);

            boolean added = false;

            added = addTopackageableMenu(child, action) || added;

            added = addToobjectsMenu(child, action) || added;

            added = addToconnectionsMenu(child, action) || added;

            added = addTocommentMenu(child, action) || added;
            
            added = addTodeprecatedMenu(child, action)|| added;

            if (!added)
            {
                umlMenu.add(action);
            }
        }
    }

    /**
     * Create this menu structure. <br>
     * It creates all the menus and groups.<br>
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     *
     * @param menu the create child menu manager 
     * @generated NOT
     */
    protected void createMenus(MenuManager menu)
    {

        packageableMenu = new MenuManager("Packageable");

        menu.add(packageableMenu);

        objectsMenu = new MenuManager("Objects");

        menu.add(objectsMenu);

        connectionsMenu = new MenuManager("Connections");

        menu.add(connectionsMenu);

        commentMenu = new MenuManager("Comment");

        menu.add(commentMenu);

        umlMenu = new MenuManager("UML");

        menu.add(umlMenu);
        
        deprecatedMenu = new MenuManager("Deprecated");
        
        menu.add(deprecatedMenu);

        // Group containing non sorted actions. 
        menu.add(new Separator(UNAPPLIED_ACTIONS_GROUP));
    }

    /**
     * Tries to add for the given child object the associated action to the
     * 'packageable' menu.<br>
     * If this menu has groups or submenus, it also tries to add the action in each
     * of them.<br>
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     *
     * @param child the child object for the one the action must be added
     * @param childAction the action to add
     * @return <code>true</code> if the given action has been added to this menu or one
     * of its groups or submenus, or <code>false</code> otherwise.
     * @generated
     */
    private boolean addTopackageableMenu(EObject child, CreateChildAction childAction)
    {
        boolean added = false;
        final CreateChildAction action = childAction;

        Object exactResult = new ExactSysMLSwitch()
        {

            public Object caseModelSYSML(org.topcased.sysml.ModelSYSML object)
            {

                packageableMenu.add(action);

                return action;
            }

            public Object casePackageSYSML(org.topcased.sysml.PackageSYSML object)
            {

                packageableMenu.add(action);

                return action;
            }

        }.doSwitch(child);
        added = added || (exactResult == action);

        return added;
    }

    /**
     * Tries to add for the given child object the associated action to the
     * 'objects' menu.<br>
     * If this menu has groups or submenus, it also tries to add the action in each
     * of them.<br>
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     *
     * @param child the child object for the one the action must be added
     * @param childAction the action to add
     * @return <code>true</code> if the given action has been added to this menu or one
     * of its groups or submenus, or <code>false</code> otherwise.
     * @generated
     */
    private boolean addToobjectsMenu(EObject child, CreateChildAction childAction)
    {
        boolean added = false;
        final CreateChildAction action = childAction;

        Object exactResult = new ExactSysMLSwitch()
        {

            public Object caseBlock(org.topcased.sysml.Block object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseRequirement(org.topcased.sysml.Requirement object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseField(org.topcased.sysml.Field object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseTestCase(org.topcased.sysml.TestCase object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object casePort(org.eclipse.uml2.uml.Port object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseFlowPort(org.topcased.sysml.FlowPort object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseFlowSpecification(org.topcased.sysml.FlowSpecification object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseValueType(org.topcased.sysml.ValueType object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseConstraintProperty(org.topcased.sysml.ConstraintProperty object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseConstraintBlock(org.topcased.sysml.ConstraintBlock object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseEnumeration(org.eclipse.uml2.uml.Enumeration object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseInterface(org.eclipse.uml2.uml.Interface object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseDataType(org.eclipse.uml2.uml.DataType object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseActor(org.eclipse.uml2.uml.Actor object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseEnumerationLiteral(org.eclipse.uml2.uml.EnumerationLiteral object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseOperation(org.eclipse.uml2.uml.Operation object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseConstraint(org.eclipse.uml2.uml.Constraint object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseProperty(org.eclipse.uml2.uml.Property object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseDimension(org.topcased.sysml.Dimension object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseUnit(org.topcased.sysml.Unit object)
            {

                objectsMenu.add(action);

                return action;
            }

            public Object caseItemFlow(org.topcased.sysml.ItemFlow object)
            {

                objectsMenu.add(action);

                return action;
            }
            
            public Object caseStereotype(org.eclipse.uml2.uml.Stereotype object)
            {
            	objectsMenu.add(action);
            	
            	return action;
            }

        }.doSwitch(child);
        added = added || (exactResult == action);

        return added;
    }

    /**
     * Tries to add for the given child object the associated action to the
     * 'connections' menu.<br>
     * If this menu has groups or submenus, it also tries to add the action in each
     * of them.<br>
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     *
     * @param child the child object for the one the action must be added
     * @param childAction the action to add
     * @return <code>true</code> if the given action has been added to this menu or one
     * of its groups or submenus, or <code>false</code> otherwise.
     * @generated
     */
    private boolean addToconnectionsMenu(EObject child, CreateChildAction childAction)
    {
        boolean added = false;
        final CreateChildAction action = childAction;

        Object exactResult = new ExactSysMLSwitch()
        {

            public Object caseUsage(org.eclipse.uml2.uml.Usage object)
            {

                connectionsMenu.add(action);

                return action;
            }

            public Object caseAssociation(org.eclipse.uml2.uml.Association object)
            {

                connectionsMenu.add(action);

                return action;
            }

            public Object caseSatisfy(org.topcased.sysml.Satisfy object)
            {

                connectionsMenu.add(action);

                return action;
            }

            public Object caseDeriveReqt(org.topcased.sysml.DeriveReqt object)
            {

                connectionsMenu.add(action);

                return action;
            }

            public Object caseVerify(org.topcased.sysml.Verify object)
            {

                connectionsMenu.add(action);

                return action;
            }

            public Object caseTrace(org.topcased.sysml.Trace object)
            {

                connectionsMenu.add(action);

                return action;
            }

            public Object caseRefine(org.topcased.sysml.Refine object)
            {

                connectionsMenu.add(action);

                return action;
            }

            public Object caseCopy(org.topcased.sysml.Copy object)
            {

                connectionsMenu.add(action);

                return action;
            }

            public Object caseConnector(org.eclipse.uml2.uml.Connector object)
            {

                connectionsMenu.add(action);

                return action;
            }

            public Object caseGeneralization(org.eclipse.uml2.uml.Generalization object)
            {

                connectionsMenu.add(action);

                return action;
            }

            public Object caseDependency(org.eclipse.uml2.uml.Dependency object)
            {

                connectionsMenu.add(action);

                return action;
            }

            public Object caseInterfaceRealization(org.eclipse.uml2.uml.InterfaceRealization object)
            {

                connectionsMenu.add(action);

                return action;
            }

            public Object caseContainment(org.topcased.sysml.Containment object)
            {

                connectionsMenu.add(action);

                return action;
            }

        }.doSwitch(child);
        added = added || (exactResult == action);

        return added;
    }

    /**
     * Tries to add for the given child object the associated action to the
     * 'comment' menu.<br>
     * If this menu has groups or submenus, it also tries to add the action in each
     * of them.<br>
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     *
     * @param child the child object for the one the action must be added
     * @param childAction the action to add
     * @return <code>true</code> if the given action has been added to this menu or one
     * of its groups or submenus, or <code>false</code> otherwise.
     * @generated
     */
    private boolean addTocommentMenu(EObject child, CreateChildAction childAction)
    {
        boolean added = false;
        final CreateChildAction action = childAction;

        Object exactResult = new ExactSysMLSwitch()
        {

            public Object caseComment(org.eclipse.uml2.uml.Comment object)
            {

                commentMenu.add(action);

                return action;
            }

            public Object caseRationale(org.topcased.sysml.Rationale object)
            {

                commentMenu.add(action);

                return action;
            }

            public Object caseProblem(org.topcased.sysml.Problem object)
            {

                commentMenu.add(action);

                return action;
            }

        }.doSwitch(child);
        added = added || (exactResult == action);

        return added;
    }

    /**
     * Tries to add for the given child object the associated action to the
     * 'deprecated' menu.<br>
     * If this menu has groups or submenus, it also tries to add the action in each
     * of them.<br>
     * <!-- begin-user-doc -->
     * <!-- end-user-doc -->
     *
     * @param child the child object for the one the action must be added
     * @param childAction the action to add
     * @return <code>true</code> if the given action has been added to this menu or one
     * of its groups or submenus, or <code>false</code> otherwise.
     * @generated
     */
    private boolean addTodeprecatedMenu(EObject child, CreateChildAction childAction)
    {
        boolean added = false;
        final CreateChildAction action = childAction;

        Object exactResult = new ExactSysMLSwitch()
        {
        	
            public Object caseBlockDefinition(org.topcased.sysml.BlockDefinition object)
            {

            	deprecatedMenu.add(action);

                return action;
            }

            public Object caseRequirementDefinition(org.topcased.sysml.RequirementDefinition object)
            {

            	deprecatedMenu.add(action);

                return action;
            }


        }.doSwitch(child);
        added = added || (exactResult == action);

        return added;
    }

}
