/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.parametricdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Display;
import org.eclipse.uml2.uml.Property;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sysml.parametricdiagram.commands.ConstraintPropertyRestoreConnectionCommand;
import org.topcased.modeler.sysml.parametricdiagram.figures.ConstraintPropertyFigure;
import org.topcased.modeler.sysml.parametricdiagram.policies.ConstraintPropertyLayoutEditPolicy;
import org.topcased.modeler.sysml.util.SysMLTools;
import org.topcased.modeler.utils.Utils;

/**
 * The ConstraintProperty object controller
 * 
 * @generated
 */
public class ConstraintPropertyEditPart extends PropertyEditPart
{
    /**
     * Constructor
     * 
     * @param obj the graph node
     * @generated
     */
    public ConstraintPropertyEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new ConstraintPropertyRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new ConstraintPropertyLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new ConstraintPropertyFigure();
    }

    /**
     * @see org.topcased.modeler.edit.PropertyEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        ComposedLabel lbl = (ComposedLabel) ((ILabelFigure) getFigure()).getLabel();
        Property property = (Property) Utils.getElement(getGraphNode());
        if (property.getName() != null && property.getName().length() != 0)
        {
            lbl.setPrefix(SysMLTools.getStereotypeNotation(property));
            lbl.setMain(property.getName());
            if (property.getType() != null)
            {
                lbl.setSuffix(" : " + property.getType().getName());

            }
        }
        lbl.setFont(JFaceResources.getFontRegistry().get("BoldFont"));
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getDefaultFont()
     */
    protected Font getDefaultFont()
    {
        FontData[] fData = Display.getDefault().getSystemFont().getFontData();
        fData[0].setStyle(SWT.BOLD);

        JFaceResources.getFontRegistry().put("BoldFont", fData);
        return JFaceResources.getFontRegistry().get("BoldFont");
    }

}