package org.topcased.modeler.sysml.parametricdiagram.figures;

import java.util.ListIterator;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.topcased.draw2d.layout.BorderAttachedLayout;
import org.topcased.draw2d.layout.PortConstraint;

public class ConstraintPropertyLayout extends BorderAttachedLayout
{

    private static final int BORDER_SPACE = 1;

    /**
     * @see org.topcased.draw2d.layout.BorderAttachedLayout#calculatePreferredSize(org.eclipse.draw2d.IFigure,
     *      int, int)
     */
    protected Dimension calculatePreferredSize(IFigure f, int wHint, int hHint)
    {
        Rectangle rect = new Rectangle();
        ListIterator children = f.getChildren().listIterator();
        while (children.hasNext())
        {
            IFigure child = (IFigure) children.next();

            Object currentConstraint = constraints.get(child);

            Rectangle r;

            if (currentConstraint instanceof PortConstraint)
            {
                PortConstraint portConstraint = (PortConstraint) currentConstraint;
                r = portConstraint.getRectConstraints().getCopy();
                switch (portConstraint.getAttachment())
                {
                    case PositionConstants.LEFT:
                    case PositionConstants.RIGHT:
                        r.x = BORDER_SPACE;
                        r.y = Math.min(r.y, f.getClientArea().height - r.height - BORDER_SPACE);
                        break;
                    case PositionConstants.TOP:
                    case PositionConstants.BOTTOM:
                        r.x = Math.min(r.x, f.getClientArea().width - r.width - BORDER_SPACE);
                        r.y = BORDER_SPACE;
                        break;
                    default:
                        break;
                }
            }
            else
            {
                r = (Rectangle) currentConstraint;
            }

            if (r == null)
            {
                continue;
            }

            // if (r.width < 0 || r.height < 0) {
            Dimension prefSize = child.getPreferredSize(r.width, r.height);
            r = r.getCopy();
            // if (r.width < 0)
            r.width = prefSize.width;
            // if (r.height < 0)
            r.height = prefSize.height;
            // }
            rect.union(r);
        }
        Dimension d = rect.getSize();
        Insets insets = f.getInsets();
        return new Dimension(d.width + insets.getWidth(), d.height + insets.getHeight()).union(getBorderPreferredSize(f));
    }

}
