/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.parametricdiagram.policies;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.uml2.uml.EncapsulatedClassifier;
import org.eclipse.uml2.uml.Property;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.sysml.SysMLPlugin;
import org.topcased.modeler.sysml.parametricdiagram.commands.DisplayErrorMessageCommand;
import org.topcased.modeler.utils.Utils;
import org.topcased.sysml.ConstraintProperty;

/**
 * @generated
 */
public class ConstraintPropertyLayoutEditPolicy extends org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy
{
    /**
     * Default contructor.
     * 
     * @generated
     */
    public ConstraintPropertyLayoutEditPolicy()
    {
        super();
    }

    /**
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isAttachedToBorder(org.topcased.modeler.di.model.GraphNode)
     * @generated NOT
     */
    protected boolean isAttachedToBorder(GraphNode node)
    {
        // if (!(Utils.getElement(node) instanceof ConstraintProperty)){
        if (Utils.getElement(node) instanceof Property)
        {
            return true;
        }
        // }
        return super.isAttachedToBorder(node);
    }

    /**
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isValid(org.eclipse.emf.ecore.EObject,
     *      org.eclipse.emf.ecore.EObject)
     * @generated
     */
    protected boolean isValid(EObject child, EObject parent)
    {
        if (child instanceof Property)
        {
            return true;
        }
        return false;
    }

    /**
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#getParentContainerEObject(GraphElement,DiagramElement)
     */
    protected EObject getParentContainerEObject(GraphElement parent, DiagramElement child)
    {
        Property prop = (Property) Utils.getElement(parent);
        return prop.getType();
    }

    /**
     * Check if the current EObject is a valid child for the parent EObject.
     * 
     * @param child the child EObject
     * @param parent the parent EObject
     * @return null if the child can be added or the command to execute if it
     *         isn't valid
     */
    protected Command checkValidality(EObject child, EObject parent)
    {
        if (parent instanceof Property)
        {

            if (!(child instanceof ConstraintProperty))
            {
                // The type of the property that will store ports must be an
                // EncapsulatedClassifier
                if (!(((Property) parent).getType() instanceof EncapsulatedClassifier))
                {
                    return new DisplayErrorMessageCommand("The Constraint Property must be correctly typed",
                            SysMLPlugin.getActiveWorkbenchShell());
                }
                return null;
            }
        }
        return UnexecutableCommand.INSTANCE;

    }

    /**
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#getCreateCommand(org.topcased.modeler.di.model.GraphNode,
     *      org.topcased.modeler.di.model.GraphNode,
     *      org.eclipse.draw2d.geometry.Point,
     *      org.eclipse.draw2d.geometry.Dimension, int, boolean)
     */
    protected Command getCreateCommand(GraphNode parent, GraphNode child, Point loc, Dimension dim, int pos,
            boolean needModelUpdate)
    {
        if (parent != null && child != null)
        {
            EditDomain domain = getHost().getViewer().getEditDomain();

            // get the EObjects of the model
            EObject parentEObject = Utils.getElement(parent);
            EObject childEObject = Utils.getElement(child);

            // ----------------------------------
            // 1- Parent and Child cannot be null and must the child should be
            // contained by the parent
            Command notValidCommand = checkValidality(childEObject, parentEObject);
            if (notValidCommand == null)
            {
                // ----------------------------------

                // ----------------------------------
                // 2- Check if this object does not already belong to this
                // container
                GraphElement existingElement = Utils.getGraphElement(parent, childEObject);
                if (existingElement != null && !isSeveralDisplayAllowed(parent, child, needModelUpdate))
                {
                    return UnexecutableCommand.INSTANCE;
                }
                // ----------------------------------

                // Get the model object that should really contain the child (it
                // is not necessary the model object associated with the parent
                // GraphNode
                EObject containerEObject = getParentContainerEObject(parent, child);

                // ----------------------------------
                // 3- Check if this object is external (only done if the model
                // is not modified)
                if (!needModelUpdate)
                {
                    EObject existingContainer = childEObject.eContainer();
                    if (!containerEObject.equals(existingContainer) && !isExternalObjectAllowed(parent, child))
                    {
                        return UnexecutableCommand.INSTANCE;
                    }
                }
                // ----------------------------------

                // ----------------------------------
                // 4- Check that a StructuralFeature can host this child
                // the List where we save the possible EStructuralFeatures
                List referencesList = new ArrayList();

                // we check all the EStructuralFeatures of type Containment
                for (Iterator i = containerEObject.eClass().getEAllContainments().iterator(); i.hasNext();)
                {
                    EReference eRef = (EReference) i.next();
                    // Check if the Reference is of the expected type
                    if (eRef.getEReferenceType().isInstance(childEObject))
                    {
                        referencesList.add(eRef);
                    }

                }

                if (!referencesList.isEmpty())
                {
                    // create the command associated with these parameters
                    return getCreateCommand(domain, child, parent, containerEObject, loc, dim, pos, referencesList,
                            needModelUpdate);
                }
            }
            else
            {
                return notValidCommand;
            }
        }

        return UnexecutableCommand.INSTANCE;
    }

}