/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.requirementdiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.NamedElement;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;
import org.topcased.sysml.Refine;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.Satisfy;
import org.topcased.sysml.SysMLPackage;
import org.topcased.sysml.Trace;

/**
 * NamedElement restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class NamedElementRestoreConnectionCommand extends ElementRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public NamedElementRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated
     */
    protected void initializeCommands()
    {

        super.initializeCommands();

        GraphElement elt = getGraphElement();
        EObject eltObject = Utils.getElement(elt);

        if (eltObject instanceof NamedElement)
        {
            Iterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement
                        && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(elt);
                    GraphElement elt2 = (GraphElement) obj;
                    EObject eltObject2 = Utils.getElement(elt2);

                    if (eltObject2 instanceof Requirement)
                    {
                        if (autoRef)
                        {
                            // autoRef not allowed
                        }
                        else
                        {
                            // if the elt is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createSatisfyFromNamedElementToRequirement(elt, elt2);
                        }
                    }

                    if (eltObject2 instanceof Requirement)
                    {
                        if (autoRef)
                        {
                            // autoRef not allowed
                        }
                        else
                        {
                            // if the elt is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createRefineFromNamedElementToRequirement(elt, elt2);
                        }
                    }

                    if (eltObject2 instanceof Requirement)
                    {
                        if (autoRef)
                        {
                            // autoRef not allowed
                        }
                        else
                        {
                            // if the elt is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createTraceFromNamedElementToRequirement(elt, elt2);
                        }
                    }

                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated
     */
    private void createSatisfyFromNamedElementToRequirement(GraphElement srcElt, GraphElement targetElt)
    {
        NamedElement sourceObject = (NamedElement) Utils.getElement(srcElt);
        Requirement targetObject = (Requirement) Utils.getElement(targetElt);

        EList edgeObjectList = ((org.topcased.sysml.PackageSYSML) Utils.getDiagramModelObject(srcElt)).getOwnedSatisfy();
        for (Iterator it = edgeObjectList.iterator(); it.hasNext();)
        {
            Object obj = it.next();
            if (obj instanceof Satisfy)
            {
                Satisfy edgeObject = (Satisfy) obj;
                if (edgeObject.getSuppliers().contains(targetObject) && edgeObject.getClients().contains(sourceObject)
                        && sourceObject.getClientDependencies().contains(edgeObject))
                {
                    // check if the relation does not exists yet
                    List existing = getExistingEdges(srcElt, targetElt, Satisfy.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            SatisfyEdgeCreationCommand cmd = new SatisfyEdgeCreationCommand(getEditDomain(),
                                    (GraphEdge) edge, srcElt, false);
                            cmd.setTarget(targetElt);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated
     */
    private void createRefineFromNamedElementToRequirement(GraphElement srcElt, GraphElement targetElt)
    {
        NamedElement sourceObject = (NamedElement) Utils.getElement(srcElt);
        Requirement targetObject = (Requirement) Utils.getElement(targetElt);

        EList edgeObjectList = ((org.topcased.sysml.PackageSYSML) Utils.getDiagramModelObject(srcElt)).getOwnedTraceDependency();
        for (Iterator it = edgeObjectList.iterator(); it.hasNext();)
        {
            Object obj = it.next();
            if (obj instanceof Refine)
            {
                Refine edgeObject = (Refine) obj;
                if (edgeObject.getSuppliers().contains(targetObject) && edgeObject.getClients().contains(sourceObject)
                        && sourceObject.getClientDependencies().contains(edgeObject))
                {
                    // check if the relation does not exists yet
                    List existing = getExistingEdges(srcElt, targetElt, Refine.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            RefineEdgeCreationCommand cmd = new RefineEdgeCreationCommand(getEditDomain(),
                                    (GraphEdge) edge, srcElt, false);
                            cmd.setTarget(targetElt);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createTraceFromNamedElementToRequirement(GraphElement srcNode, GraphElement targetNode)
    {
        NamedElement sourceObject = (NamedElement) Utils.getElement(srcNode);
        Requirement targetObject = (Requirement) Utils.getElement(targetNode);

        EList edgeObjectList = ((org.topcased.sysml.PackageSYSML) Utils.getDiagramModelObject(srcNode)).getOwnedTraceDependency();
        for (Iterator it = edgeObjectList.iterator(); it.hasNext();)
        {
            Object obj = it.next();
            // The type of the edge must be Trace and not a derived type
            if (obj instanceof EObject && ((EObject) obj).eClass() == SysMLPackage.eINSTANCE.getTrace())
            {
                Trace edgeObject = (Trace) obj;
                if (edgeObject.getSuppliers().contains(targetObject) && edgeObject.getClients().contains(sourceObject)
                        && sourceObject.getClientDependencies().contains(edgeObject))
                {
                    // check if the relation does not exists yet
                    List existing = getExistingEdges(srcNode, targetNode, Trace.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            TraceEdgeCreationCommand cmd = new TraceEdgeCreationCommand(getEditDomain(),
                                    (GraphEdge) edge, srcNode, false);
                            cmd.setTarget(targetNode);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

}