/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Laleve (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.sysml.requirementdiagram.commands;

import java.util.Map;

import org.eclipse.gef.commands.Command;
import org.topcased.modeler.sysml.requirementdiagram.dialogs.RequirementEditDialog;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.RequirementKind;
import org.topcased.sysml.RiskKind;
import org.topcased.sysml.VerifyMethodKind;

/**
 * Class that create a command in order to update requirement parameters
 */
public class RequirementUpdateCommand extends Command
{
    /** Current requirement */
    private Requirement requirement;

    /** Old values */
    private String oldName;

    private String oldId;

    private String oldSource;

    private String oldText;

    private RequirementKind oldKind;

    private RiskKind oldRisk;

    private VerifyMethodKind oldVerifyMethod;

    /** New values */
    private String name;

    private String id;

    private String source;

    private String text;

    private RequirementKind kind;

    private RiskKind risk;

    private VerifyMethodKind verifyMethod;

    /**
     * Create a command for updating parameters on a given requirement
     * 
     * @param req the requirement to update
     * @param data the datas to set
     */
    public RequirementUpdateCommand(Requirement req, Map data)
    {
        requirement = req;
        name = (String) data.get(RequirementEditDialog.REQUIREMENT_NAME);
        id = (String) data.get(RequirementEditDialog.REQUIREMENT_ID);
        source = (String) data.get(RequirementEditDialog.REQUIREMENT_SOURCE);
        text = (String) data.get(RequirementEditDialog.REQUIREMENT_TEXT);
        kind = (RequirementKind) data.get(RequirementEditDialog.REQUIREMENT_KIND);
        risk = (RiskKind) data.get(RequirementEditDialog.REQUIREMENT_RISK);
        verifyMethod = (VerifyMethodKind) data.get(RequirementEditDialog.REQUIREMENT_VERIFY);
    }

    /**
     * Get the old values
     */
    protected void getOldValues()
    {
        oldName = requirement.getName();
        oldId = requirement.getId();
        oldSource = requirement.getSource();
        oldText = requirement.getText();
        oldKind = requirement.getKind();
        oldRisk = requirement.getRisk();
        oldVerifyMethod = requirement.getVerifyMethod();
    }

    /**
     * Set the values on the Operation
     */
    protected void setValues()
    {
        // Perform update
        requirement.setName(name);
        requirement.setId(id);
        requirement.setSource(source);
        requirement.setText(text);
        requirement.setKind(kind);
        requirement.setRisk(risk);
        requirement.setVerifyMethod(verifyMethod);
    }

    /**
     * Switch the old and new values
     */
    protected void switchValues()
    {
        // Reverse old and new value
        String tempName = name;
        name = oldName;
        oldName = tempName;

        String tempId = id;
        id = oldId;
        oldId = tempId;

        String tempSource = source;
        source = oldSource;
        oldSource = tempSource;

        String tempText = text;
        text = oldText;
        oldText = tempText;

        RequirementKind tempKind = kind;
        kind = oldKind;
        oldKind = tempKind;

        RiskKind tempRisk = risk;
        risk = oldRisk;
        oldRisk = tempRisk;

        VerifyMethodKind tempVerifyMethod = verifyMethod;
        verifyMethod = oldVerifyMethod;
        oldVerifyMethod = tempVerifyMethod;
    }

    /**
     * Get the old values and set the new ones
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        getOldValues();
        setValues();
    }

    /**
     * Set the new values
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        switchValues();
        setValues();
    }

    /**
     * Set the old values
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        switchValues();
        setValues();
    }
}
