/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.requirementdiagram.edit;

import java.util.Iterator;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sysml.requirementdiagram.RequirementEditPolicyConstants;
import org.topcased.modeler.sysml.requirementdiagram.commands.ElementRestoreConnectionCommand;
import org.topcased.modeler.sysml.requirementdiagram.figures.ElementFigure;
import org.topcased.modeler.sysml.requirementdiagram.policies.CommentLinkEdgeCreationEditPolicy;

/**
 * The Element object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ElementEditPart extends EMFGraphNodeEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public ElementEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(RequirementEditPolicyConstants.COMMENTLINK_EDITPOLICY,
                new CommentLinkEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new ElementRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());

        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new ElementFigure();
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
     */
    public void activate()
    {
        super.activate();
        EObject model = getEObject();
        if (model instanceof Element)
        {
            listenEAnnotations((Element) model);
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
     */
    public void deactivate()
    {
        EObject model = getEObject();
        if (model instanceof Element)
        {
            unlistenEAnnotations((Element) model);
        }
        super.deactivate();
    }

    /**
     * Listen to all the EAnnotations associated with the element
     * 
     * @param element the model object
     */
    private void listenEAnnotations(Element element)
    {
        Iterator it = element.getEAnnotations().iterator();
        while (it.hasNext())
        {
            EAnnotation eAnnot = (EAnnotation) it.next();

            // Only listen to object that are not yet listened
            if (!eAnnot.eAdapters().contains(getModelListener()))
            {
                eAnnot.eAdapters().add(getModelListener());
            }
        }
    }

    /**
     * Stop listening to all the EAnnotations of the element
     * 
     * @param element the model object
     */
    private void unlistenEAnnotations(Element element)
    {
        Iterator it = element.getEAnnotations().iterator();
        while (it.hasNext())
        {
            EAnnotation eAnnot = (EAnnotation) it.next();

            eAnnot.eAdapters().remove(getModelListener());
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {

        Object notifier = msg.getNotifier();
        Object newObject = msg.getNewValue();
        Object oldObject = msg.getOldValue();

        if (notifier instanceof Element)
        {
            if (notifier == getEObject())
            {
                switch (msg.getFeatureID(Element.class))
                {
                    case UMLPackage.ELEMENT__EANNOTATIONS:
                        updateModelListening(oldObject, newObject);
                        break;
                }
                super.handleModelChanged(msg);
            }
        }
        else if (notifier instanceof EAnnotation)
        {
            super.handleModelChanged(msg);
        }
    }

}