/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Laleve (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.sysml.requirementdiagram.edit;

import java.util.Iterator;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Relationship;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.sysml.requirementdiagram.RequirementEditPolicyConstants;
import org.topcased.modeler.sysml.requirementdiagram.policies.CommentLinkEdgeCreationEditPolicy;

/**
 * Base class for dependencies associated to requirements
 */
public class RelationshipEditPart extends EMFGraphEdgeEditPart
{

    /**
     * Constructor
     * 
     * @param model the graph object
     */
    public RelationshipEditPart(GraphEdge model)
    {
        super(model);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(RequirementEditPolicyConstants.COMMENTLINK_EDITPOLICY,
                new CommentLinkEdgeCreationEditPolicy());
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphEdgeEditPart#activate()
     */
    public void activate()
    {
        super.activate();
        EObject model = getEObject();
        if (model instanceof Relationship)
        {
            listenEAnnotations((Relationship) model);
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphEdgeEditPart#deactivate()
     */
    public void deactivate()
    {
        EObject model = getEObject();
        if (model instanceof Relationship)
        {
            unlistenEAnnotations((Relationship) model);
        }
        super.deactivate();
    }

    /**
     * Listen to all the EAnnotations associated with the relationship
     * 
     * @param relation the model object
     */
    private void listenEAnnotations(Relationship relation)
    {
        Iterator it = relation.getEAnnotations().iterator();
        while (it.hasNext())
        {
            EAnnotation eAnnot = (EAnnotation) it.next();

            // Only listen to object that are not yet listened
            if (!eAnnot.eAdapters().contains(getModelListener()))
            {
                eAnnot.eAdapters().add(getModelListener());
            }
        }
    }

    /**
     * Stop listening to all the EAnnotations of the relationship
     * 
     * @param relation the model object
     */
    private void unlistenEAnnotations(Relationship relation)
    {
        Iterator it = relation.getEAnnotations().iterator();
        while (it.hasNext())
        {
            EAnnotation eAnnot = (EAnnotation) it.next();

            eAnnot.eAdapters().remove(getModelListener());
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphEdgeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {

        Object notifier = msg.getNotifier();
        Object newObject = msg.getNewValue();
        Object oldObject = msg.getOldValue();

        if (notifier instanceof Relationship)
        {
            if (notifier == getEObject())
            {
                switch (msg.getFeatureID(Element.class))
                {
                    case UMLPackage.ELEMENT__EANNOTATIONS:
                        updateModelListening(oldObject, newObject);
                        break;
                }
                super.handleModelChanged(msg);
            }
        }
        else if (notifier instanceof EAnnotation)
        {
            super.handleModelChanged(msg);
        }
    }
}
