/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.requirementdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Display;
import org.eclipse.uml2.uml.Element;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.modeler.ModelerColorConstants;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sysml.SysMLPlugin;
import org.topcased.modeler.sysml.requirementdiagram.RequirementEditPolicyConstants;
import org.topcased.modeler.sysml.requirementdiagram.commands.RequirementRestoreConnectionCommand;
import org.topcased.modeler.sysml.requirementdiagram.commands.RequirementUpdateCommand;
import org.topcased.modeler.sysml.requirementdiagram.dialogs.RequirementEditDialog;
import org.topcased.modeler.sysml.requirementdiagram.figures.RequirementFigure;
import org.topcased.modeler.sysml.requirementdiagram.policies.ContainmentEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.requirementdiagram.policies.CopyEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.requirementdiagram.policies.DeriveReqtEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.requirementdiagram.policies.RefineEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.requirementdiagram.policies.RequirementLayoutEditPolicy;
import org.topcased.modeler.sysml.requirementdiagram.policies.SatisfyEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.requirementdiagram.policies.TraceEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.requirementdiagram.policies.VerifyEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.requirementdiagram.util.RequirementTools;
import org.topcased.modeler.sysml.util.SysMLTools;
import org.topcased.modeler.utils.Utils;
import org.topcased.sysml.Requirement;

/**
 * The Requirement object controller <!-- begin-user-doc --> <!-- end-user-doc
 * -->
 * 
 * @generated
 */
public class RequirementEditPart extends NamedElementEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public RequirementEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(RequirementEditPolicyConstants.CONTAINMENT_EDITPOLICY,
                new ContainmentEdgeCreationEditPolicy());

        installEditPolicy(RequirementEditPolicyConstants.SATISFY_EDITPOLICY, new SatisfyEdgeCreationEditPolicy());

        installEditPolicy(RequirementEditPolicyConstants.DERIVEREQT_EDITPOLICY, new DeriveReqtEdgeCreationEditPolicy());

        installEditPolicy(RequirementEditPolicyConstants.VERIFY_EDITPOLICY, new VerifyEdgeCreationEditPolicy());

        installEditPolicy(RequirementEditPolicyConstants.REFINE_EDITPOLICY, new RefineEdgeCreationEditPolicy());

        installEditPolicy(RequirementEditPolicyConstants.TRACE_EDITPOLICY, new TraceEdgeCreationEditPolicy());

        installEditPolicy(RequirementEditPolicyConstants.COPY_EDITPOLICY, new CopyEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new RequirementRestoreConnectionCommand(getHost());
            }
        });

        ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(PositionConstants.EAST_WEST);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new RequirementLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new RequirementFigure();
    }

    /**
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     */
    protected void refreshVisuals()
    {
        super.refreshVisuals();
        refreshAttributes();
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        RequirementFigure fig = (RequirementFigure) getFigure();
        ComposedLabel lbl = (ComposedLabel) fig.getLabel();
        Requirement requirement = (Requirement) Utils.getElement(getGraphNode());

        String stereotypes = SysMLTools.getStereotypesList(requirement);
        if (stereotypes.length() != 0)
        {
            stereotypes = ", " + stereotypes;
        }
        lbl.setPrefix("<<requirement" + stereotypes + ">>");

        if (requirement.getName() != null)
        {
            lbl.setMain(requirement.getName());
        }

        lbl.setSuffix(RequirementTools.getFromNotation(requirement,
                (Element) Utils.getElement((GraphElement) getParent().getModel())));
    }

    protected void refreshAttributes()
    {
        RequirementFigure fig = (RequirementFigure) getFigure();
        Requirement req = (Requirement) getEObject();
        fig.setId(req.getId());
        fig.setSource(req.getSource());
        fig.setText(req.getText());
        fig.setKind(req.getKind().getName());
        fig.setRisk(req.getRisk().getName());
        fig.setVerifyMethod(req.getVerifyMethod().getName());
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getDefaultFont()
     */
    protected Font getDefaultFont()
    {
        Requirement req = (Requirement) Utils.getElement(getGraphNode());
        FontData[] fData = Display.getDefault().getSystemFont().getFontData();
        if (req.isAbstract())
        {
            fData[0].setStyle(SWT.BOLD | SWT.ITALIC);
        }
        else
        {
            fData[0].setStyle(SWT.BOLD);
        }
        JFaceResources.getFontRegistry().put("ItalicFont", fData);
        return JFaceResources.getFontRegistry().get("ItalicFont");
    }

    /**
     * @see org.eclipse.gef.editparts.AbstractEditPart#performRequest(Request)
     */
    public void performRequest(Request request)
    {
        if (request.getType() == RequestConstants.REQ_OPEN)
        {
            Requirement requirement = (Requirement) Utils.getElement(getGraphNode());

            RequirementEditDialog requirementDlg = new RequirementEditDialog(requirement,
                    SysMLPlugin.getActiveWorkbenchShell());
            if (requirementDlg.open() == Window.OK)
            {
                RequirementUpdateCommand command = new RequirementUpdateCommand(requirement, requirementDlg.getData());
                getViewer().getEditDomain().getCommandStack().execute(command);
            }
        }
        else
        {
            super.performRequest(request);
        }
    }

    /**
     * @see org.topcased.modeler.sysml.requirementdiagram.edit.NamedElementEditPart#getDefaultBackgroundColor()
     */
    protected Color getDefaultBackgroundColor()
    {
        return ModelerColorConstants.lightBlue;
    }
}