/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.requirementdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Display;
import org.eclipse.uml2.uml.Element;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.modeler.ModelerColorConstants;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sysml.requirementdiagram.RequirementEditPolicyConstants;
import org.topcased.modeler.sysml.requirementdiagram.commands.TestCaseRestoreConnectionCommand;
import org.topcased.modeler.sysml.requirementdiagram.figures.TestCaseFigure;
import org.topcased.modeler.sysml.requirementdiagram.policies.VerifyEdgeCreationEditPolicy;
import org.topcased.modeler.sysml.requirementdiagram.util.RequirementTools;
import org.topcased.modeler.sysml.util.SysMLTools;
import org.topcased.modeler.utils.Utils;
import org.topcased.sysml.TestCase;

/**
 * The TestCase object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class TestCaseEditPart extends NamedElementEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public TestCaseEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc
     * --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(RequirementEditPolicyConstants.VERIFY_EDITPOLICY, new VerifyEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new TestCaseRestoreConnectionCommand(getHost());
            }
        });

        ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(PositionConstants.EAST_WEST);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new TestCaseFigure();
    }
    
    /**
     * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
     */
    protected void refreshVisuals()
    {
        super.refreshVisuals();
        refreshAttributes();
    }

    protected void refreshAttributes()
    {
        TestCaseFigure fig = (TestCaseFigure) getFigure();
        TestCase testcase = (TestCase) getEObject();
        fig.setResult(testcase.getResult().getName());
    }

    protected void refreshHeaderLabel()
    {
        TestCaseFigure fig = (TestCaseFigure) getFigure();
        ComposedLabel lbl = (ComposedLabel) fig.getLabel();
        TestCase testcase = (TestCase) Utils.getElement(getGraphNode());

        String stereotypes = SysMLTools.getStereotypesList(testcase);
        if (stereotypes.length() != 0)
        {
            stereotypes = ", " + stereotypes;
        }
        lbl.setPrefix("<<testcase" + stereotypes + ">>");

        if (testcase.getName() != null)
        {
            lbl.setMain(testcase.getName());
        }

        lbl.setSuffix(RequirementTools.getFromNotation(testcase,
                (Element) Utils.getElement((GraphElement) getParent().getModel())));
    }

    protected Font getDefaultFont()
    {
        TestCase testcase = (TestCase) Utils.getElement(getGraphNode());

        FontData[] fData = Display.getDefault().getSystemFont().getFontData();
        if (testcase.isAbstract())
        {
            fData[0].setStyle(SWT.BOLD | SWT.ITALIC);
        }
        else
        {
            fData[0].setStyle(SWT.BOLD);
        }
        JFaceResources.getFontRegistry().put("ItalicFont", fData);
        return JFaceResources.getFontRegistry().get("ItalicFont");
    }

    protected Color getDefaultBackgroundColor()
    {
        return ModelerColorConstants.lightYellow;
    }
}