/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Laleve (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.sysml.requirementdiagram.util;

import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.NamedElement;
import org.topcased.sysml.PackageSYSML;;

public class RequirementTools
{
    /**
     * Return the requirement definition in which an element is contained
     * 
     * @param element
     * @return the requirement definition
     */
    public static PackageSYSML getPackageSYSML(Element element)
    {
        if (element == null || element instanceof PackageSYSML)
            return (PackageSYSML) element;
        return getPackageSYSML(element.getOwner());
    }

    /**
     * Get a string representation of the origin of a SysML element
     * 
     * @param element an SysML element
     * @param parent the element which graphically contains the SysML Type
     *            element
     * @return a fully qualified string representation or empty string if
     *         element isn't contained in an external requirement definition
     */
    public static String getFromNotation(Element element, Element parent)
    {
        String name = "";
        PackageSYSML rdElement = getPackageSYSML(element);
        PackageSYSML rdParent = getPackageSYSML(parent);
        if (rdElement != rdParent && element.getOwner() instanceof NamedElement)
        {
            name = "(from " + ((NamedElement) element.getOwner()).getName() + ")";
        }
        return name;
    }
}
