/*******************************************************************************
 * Copyright (c) 2005 Anyware Technologies
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies) - initial API and implementation
 *******************************************************************************/

package org.topcased.modeler.uml;

import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.DiagnosticChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.Diagnostician;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.topcased.validation.core.IValidator;

/**
 * TODO comment this class
 * 
 * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class UMLValidator implements IValidator
{

    /**
     * @see org.topcased.validation.core.IValidator#validate(org.eclipse.emf.ecore.EObject,
     *      org.eclipse.emf.common.util.DiagnosticChain, org.eclipse.core.runtime.IProgressMonitor)
     */
    public boolean validate(EObject eObject, DiagnosticChain diagnostics, final IProgressMonitor monitor)
    {
        int count = 0;
        for (Iterator i = eObject.eAllContents(); i.hasNext(); i.next())
        {
            ++count;
        }

        monitor.beginTask("UML Validation", count);

        Diagnostician diagnostician = new Diagnostician()
        {

            public boolean validate(EClass eClass, EObject eObj, DiagnosticChain diag, Map context)
            {
                monitor.worked(1);

                boolean result = org.eclipse.uml2.uml.util.UMLValidator.INSTANCE.validate(eClass, eObj, diag, context);
                if (result || diag != null)
                {
                    result &= doValidateContents(eObj, diag, context);
                }
                return result;
            }
        };

        monitor.setTaskName("Validation of " + EcoreUtil.getURI(eObject));

        Diagnostic result = diagnostician.validate(eObject);
        diagnostics.merge(result);

        return true;
    }

}
