/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.actions;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.uml2.uml.Profile;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.uml.UMLPlugin;
import org.topcased.modeler.uml.commands.DefineProfileCommand;
import org.topcased.modeler.uml.dialogs.DefineProfileDialog;

/**
 * Action that allows user to define profile, configure stereotypes
 * 
 * Creation 25 août 06
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class DefineProfileAction extends UMLAction
{
    private Profile profile;

    /**
     * Constructor
     * 
     * @param ed the Modeler object
     * @param prof The profil that should be defined
     */
    public DefineProfileAction(Modeler ed, org.eclipse.uml2.uml.Profile prof)
    {
        super("Define Profile", ed);
        this.profile = prof;
    }

    /**
     * Execute the Action
     * 
     * @see org.eclipse.jface.action.Action#run()
     */
    public void run()
    {
        ((CommandStack) getEditor().getAdapter(CommandStack.class)).execute(createActionCommand(profile));
    }

    /**
     * Create the generation command
     * 
     * @param p the UML profile
     * @return the command that update the profile properties
     */
    protected Command createActionCommand(final org.eclipse.uml2.uml.Profile p)
    {
        Command defineProfilCommand = null;
        if (p != null)
        {
            if (p.getQualifiedName() != null && !"".equals(p.getQualifiedName()))
            {
                DefineProfileDialog defineProfileDlg = new DefineProfileDialog(profile, UMLPlugin.getActiveWorkbenchShell());
                if (defineProfileDlg.open() == Window.OK)
                {
                    defineProfilCommand = new DefineProfileCommand(profile, defineProfileDlg.getData(), defineProfileDlg.getOldData());
                }
            }
            else
            {
                MessageDialog.openWarning(getEditor().getSite().getShell(), "Error",
                        "Unable to define profile.\nCheck that all parent objects of the profile and the profile itself have their name set.");
            }
        }
        return defineProfilCommand;
    }

}
