/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.activitydiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.ControlFlow;
import org.eclipse.uml2.uml.ObjectFlow;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.uml.alldiagram.commands.ElementRestoreConnectionCommand;
import org.topcased.modeler.utils.Utils;

/**
 * ActivityNode restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ActivityNodeRestoreConnectionCommand extends ElementRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public ActivityNodeRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated
     */
    protected void initializeCommands()
    {

        super.initializeCommands();

        GraphElement elt = getGraphElement();
        EObject eltObject = Utils.getElement(elt);

        if (eltObject instanceof ActivityNode)
        {
            Iterator<EObject> itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(elt);
                    GraphElement elt2 = (GraphElement) obj;
                    EObject eltObject2 = Utils.getElement(elt2);

                    if (eltObject2 instanceof ActivityNode && !autoRef)
                    {
                        createControlFlowFromActivityNodeToActivityNode(elt, elt2);
                        createControlFlowFromActivityNodeToActivityNode(elt2, elt);

                        createObjectFlowFromActivityNodeToActivityNode(elt, elt2);
                        createObjectFlowFromActivityNodeToActivityNode(elt2, elt);
                    }

                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createControlFlowFromActivityNodeToActivityNode(GraphElement srcNode, GraphElement targetNode)
    {
        ActivityNode sourceObject = (ActivityNode) Utils.getElement(srcNode);
        ActivityNode targetObject = (ActivityNode) Utils.getElement(targetNode);

        for (Object obj : ((org.eclipse.uml2.uml.Activity) Utils.getDiagramModelObject(srcNode)).getEdges())
        {
            if (obj instanceof ControlFlow)
            {
                ControlFlow edgeObject = (ControlFlow) obj;
                if (targetObject.equals(edgeObject.getTarget()) && sourceObject.equals(edgeObject.getSource()) && sourceObject.getOutgoings().contains(edgeObject)
                        && targetObject.getIncomings().contains(edgeObject))
                {
                    // check if the relation does not exists yet
                    List<GraphEdge> existing = getExistingEdges(srcNode, targetNode, ControlFlow.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            ControlFlowEdgeCreationCommand cmd = new ControlFlowEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                            cmd.setTarget(targetNode);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createObjectFlowFromActivityNodeToActivityNode(GraphElement srcNode, GraphElement targetNode)
    {
        ActivityNode sourceObject = (ActivityNode) Utils.getElement(srcNode);
        ActivityNode targetObject = (ActivityNode) Utils.getElement(targetNode);

        for (Object obj : ((org.eclipse.uml2.uml.Activity) Utils.getDiagramModelObject(srcNode)).getEdges())
        {
            if (obj instanceof ObjectFlow)
            {
                ObjectFlow edgeObject = (ObjectFlow) obj;
                if (targetObject.equals(edgeObject.getTarget()) && sourceObject.equals(edgeObject.getSource()) && sourceObject.getOutgoings().contains(edgeObject)
                        && targetObject.getIncomings().contains(edgeObject))
                {
                    // check if the relation does not exists yet
                    List<GraphEdge> existing = getExistingEdges(srcNode, targetNode, ObjectFlow.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            ObjectFlowEdgeCreationCommand cmd = new ObjectFlowEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                            cmd.setTarget(targetNode);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

}