/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.activitydiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.ExceptionHandler;
import org.eclipse.uml2.uml.ExecutableNode;
import org.eclipse.uml2.uml.ObjectNode;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;

/**
 * ObjectNode restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ObjectNodeRestoreConnectionCommand extends ActivityNodeRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public ObjectNodeRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated
     */
    protected void initializeCommands()
    {

        super.initializeCommands();

        GraphElement elt = getGraphElement();
        EObject eltObject = Utils.getElement(elt);

        if (eltObject instanceof ObjectNode)
        {
            Iterator<EObject> itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(elt);
                    GraphElement elt2 = (GraphElement) obj;
                    EObject eltObject2 = Utils.getElement(elt2);

                    if (eltObject2 instanceof ExecutableNode && !autoRef)
                    {
                        // if elt is the target of the edge or if it is the source and that the SourceTargetCouple
                        // is reversible
                        createExceptionHandlerFromExecutableNodeToObjectNode(elt2, elt);
                    }

                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createExceptionHandlerFromExecutableNodeToObjectNode(GraphElement srcNode, GraphElement targetNode)
    {
        ExecutableNode sourceObject = (ExecutableNode) Utils.getElement(srcNode);
        ObjectNode targetObject = (ObjectNode) Utils.getElement(targetNode);

        for (ExceptionHandler edgeObject : sourceObject.getHandlers())
        {
            if (targetObject.equals(edgeObject.getExceptionInput()) && sourceObject.equals(edgeObject.getProtectedNode()))
            {
                // check if the relation does not exists yet
                List<GraphEdge> existing = getExistingEdges(srcNode, targetNode, ExceptionHandler.class);
                if (!isAlreadyPresent(existing, edgeObject))
                {
                    ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                    // restore the link with its default presentation
                    GraphElement edge = factory.createGraphElement(edgeObject);
                    if (edge instanceof GraphEdge)
                    {
                        ExceptionHandlerEdgeCreationCommand cmd = new ExceptionHandlerEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                        cmd.setTarget(targetNode);
                        add(cmd);
                    }
                }
            }
        }
    }

}