/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.activitydiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.PolylineDecoration;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.ActivityEdge;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.LiteralBoolean;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralNull;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.ValueSpecification;
import org.topcased.draw2d.figures.Label;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.policies.EdgeObjectOffsetEditPolicy;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.uml.UMLLabel;
import org.topcased.modeler.uml.activitydiagram.ActivityEdgeObjectConstants;
import org.topcased.modeler.uml.activitydiagram.ActivityEditPolicyConstants;
import org.topcased.modeler.uml.activitydiagram.figures.ActivityEdgeFigure;
import org.topcased.modeler.uml.activitydiagram.policies.ActivityEdgeEdgeObjectUVEditPolicy;
import org.topcased.modeler.uml.alldiagram.policies.CommentLinkEdgeCreationEditPolicy;

/**
 * ActivityEdge controller : this is used to regroup as well the ControlFlow behavior than the ObjectFlow behavior.
 * 
 * @generated
 */
public class ActivityEdgeEditPart extends EMFGraphEdgeEditPart
{

    /**
     * Constructor
     * 
     * @param model the graph object
     * @generated
     */
    public ActivityEdgeEditPart(GraphEdge model)
    {
        super(model);
    }

    /**
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ActivityEditPolicyConstants.COMMENTLINK_EDITPOLICY, new CommentLinkEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_UV_EDITPOLICY, new ActivityEdgeEdgeObjectUVEditPolicy());
        installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_OFFSET_EDITPOLICY, new EdgeObjectOffsetEditPolicy());

    }

    /**
     * @return the Figure
     * @generated NOT
     */
    protected IFigure createFigure()
    {
        ActivityEdgeFigure connection = new ActivityEdgeFigure();

        createTargetDecoration(connection);

        return connection;
    }

    /**
     * @param connection the PolylineConnection
     * @generated
     */
    private void createTargetDecoration(PolylineConnection connection)
    {

        PolylineDecoration decoration = new PolylineDecoration();
        decoration.setScale(10, 5);
        connection.setTargetDecoration(decoration);

    }

    /**
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#getEdgeObjectFigure(org.topcased.modeler.di.model.EdgeObject)
     * @generated
     */
    public IEdgeObjectFigure getEdgeObjectFigure(EdgeObject edgeObject)
    {
        if (ActivityEdgeObjectConstants.STEREOTYPE_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((ActivityEdgeFigure) getFigure()).getStereotypeEdgeObjectFigure();
        }
        if (ActivityEdgeObjectConstants.GUARD_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((ActivityEdgeFigure) getFigure()).getGuardEdgeObjectFigure();
        }
        if (ActivityEdgeObjectConstants.NAME_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((ActivityEdgeFigure) getFigure()).getNameEdgeObjectFigure();
        }
        if (ActivityEdgeObjectConstants.WEIGHT_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((ActivityEdgeFigure) getFigure()).getWeightEdgeObjectFigure();
        }
        return null;
    }

    /**
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#refreshEdgeObjects()
     * @generated
     */
    protected void refreshEdgeObjects()
    {
        super.refreshEdgeObjects();
        updateStereotypeLabel();
        updateGuardLabel();
        updateNameLabel();
        updateWeightLabel();
    }

    /**
     * @generated NOT
     */
    private void updateStereotypeLabel()
    {
        ((Label) ((ActivityEdgeFigure) getFigure()).getStereotypeEdgeObjectFigure()).setText(UMLLabel.getStereotypesNotation((Element) getEObject()));
    }

    /**
     * @generated NOT
     */
    private void updateGuardLabel()
    {
        String guardText = "";

        ValueSpecification guard = ((ActivityEdge) getEObject()).getGuard();
        if (guard != null)
        {
            String guardValue = guard.stringValue();
            if (guardValue != null)
            {
                guardText = "[".concat(guardValue).concat("]");
            }
        }

        ((Label) ((ActivityEdgeFigure) getFigure()).getGuardEdgeObjectFigure()).setText(guardText);
    }

    /**
     * @generated NOT
     */
    private void updateNameLabel()
    {
        ((Label) ((ActivityEdgeFigure) getFigure()).getNameEdgeObjectFigure()).setText(((NamedElement) getEObject()).getName());
    }

    /**
     * The weight of the edge may be shown in curly braces that contain the weight. The weight is a value specification
     * that is a positive integer or null, which may be a constant. A weight of null is notated as "all".
     * 
     * @generated NOT
     */
    private void updateWeightLabel()
    {
        String guardText = "";

        ValueSpecification weight = ((ActivityEdge) getEObject()).getWeight();
        if (weight != null)
        {
            String weightValue = weight.stringValue();
            if (weight instanceof LiteralNull)
            {
                weightValue = "all";
            }
            if (weightValue != null)
            {
                guardText = "{weight=".concat(weightValue).concat("}");
            }
        }

        ((Label) ((ActivityEdgeFigure) getFigure()).getWeightEdgeObjectFigure()).setText(guardText);
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphEdgeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {

        Object notifier = msg.getNotifier();
        Object newObject = msg.getNewValue();
        Object oldObject = msg.getOldValue();

        if (notifier instanceof ActivityEdge)
        {
            if (notifier == getEObject())
            {
                switch (msg.getFeatureID(ActivityEdge.class))
                {
                    case UMLPackage.ACTIVITY_EDGE__GUARD:
                    case UMLPackage.ACTIVITY_EDGE__WEIGHT:
                        updateModelListening(oldObject, newObject);
                        break;
                    default:
                        break;
                }
                super.handleModelChanged(msg);
            }
        }
        else if (notifier instanceof LiteralBoolean)
        {
            switch (msg.getFeatureID(LiteralBoolean.class))
            {
                case UMLPackage.LITERAL_BOOLEAN__VALUE:
                    updateGuardLabel();
                    break;
                default:
                    break;
            }
        }
        else if (notifier instanceof LiteralInteger)
        {
            switch (msg.getFeatureID(LiteralInteger.class))
            {
                case UMLPackage.LITERAL_INTEGER__VALUE:
                    updateGuardLabel();
                    updateWeightLabel();
                    break;
                default:
                    break;
            }
        }
        else if (notifier instanceof LiteralString)
        {
            switch (msg.getFeatureID(LiteralString.class))
            {
                case UMLPackage.LITERAL_STRING__VALUE:
                    updateGuardLabel();
                    break;
                default:
                    break;
            }
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
     */
    public void activate()
    {
        super.activate();
        EObject model = getEObject();
        if (model instanceof ActivityEdge)
        {
            listenGuard((ActivityEdge) model);
            listenWeight((ActivityEdge) model);
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
     */
    public void deactivate()
    {
        EObject model = getEObject();
        if (model instanceof ActivityEdge)
        {
            unlistenGuard((ActivityEdge) model);
            unlistenWeight((ActivityEdge) model);
        }
        super.deactivate();
    }

    /**
     * Listen to the Guard element
     * 
     * @param actEdge the model object
     */
    private void listenGuard(ActivityEdge actEdge)
    {
        ValueSpecification guard = actEdge.getGuard();
        // Only listen to object that are not yet listened
        if (guard != null && !guard.eAdapters().contains(getModelListener()))
        {
            guard.eAdapters().add(getModelListener());
        }
    }

    /**
     * Stop listening to the Guard element
     * 
     * @param actEdge the model object
     */
    private void unlistenGuard(ActivityEdge actEdge)
    {
        ValueSpecification guard = actEdge.getGuard();
        if (guard != null)
        {
            guard.eAdapters().remove(getModelListener());
        }
    }

    /**
     * Listen to the Weight element
     * 
     * @param actEdge the model object
     */
    private void listenWeight(ActivityEdge actEdge)
    {
        ValueSpecification weight = actEdge.getWeight();
        // Only listen to object that are not yet listened
        if (weight != null && !weight.eAdapters().contains(getModelListener()))
        {
            weight.eAdapters().add(getModelListener());
        }
    }

    /**
     * Stop listening to the Weight element
     * 
     * @param actEdge the model object
     */
    private void unlistenWeight(ActivityEdge actEdge)
    {
        ValueSpecification weight = actEdge.getWeight();
        if (weight != null)
        {
            weight.eAdapters().remove(getModelListener());
        }
    }
}