/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.activitydiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.listeners.DiagramElementListener;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.activitydiagram.commands.ForkNodeRestoreConnectionCommand;
import org.topcased.modeler.uml.activitydiagram.figures.ForkNodeFigure;
import org.topcased.modeler.utils.Utils;

/**
 * The ForkNode object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ForkNodeEditPart extends ControlNodeEditPart
{
    // The resizableEditPolicy that may change when the orientation changes
    private ResizableEditPolicy resizableEditPolicy;

    /**
     * This listener also listens the orientation property of the graphical element
     */
    protected class ForkNodeEditPartListener extends GraphNodeEditPartListener
    {
        /**
         * @see org.topcased.modeler.edit.GraphNodeEditPart.GraphNodeEditPartListener#handlePresentationChanged()
         */
        protected void handlePresentationChanged()
        {
            updateNodeGraphConf();
            resizableEditPolicy.setResizeDirections(isVertical() ? PositionConstants.NORTH_SOUTH : PositionConstants.EAST_WEST);
            refreshVisuals();
        }
    }

    private DiagramElementListener diagramListener = new ForkNodeEditPartListener();

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public ForkNodeEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getDiagramElementListener()
     */
    protected DiagramElementListener getDiagramElementListener()
    {
        return diagramListener;
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new ForkNodeRestoreConnectionCommand(getHost());
            }
        });

        resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(isVertical() ? PositionConstants.NORTH_SOUTH : PositionConstants.EAST_WEST);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new ForkNodeFigure();
    }

    /**
     * Get the orientation type
     * 
     * @return return true if the Join graphical element should be represented vertically.
     */
    public boolean isVertical()
    {
        return "default".equals(getGraphNode().getSemanticModel().getPresentation());
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getPresentation()
     */
    protected String getPresentation()
    {
        return isVertical() ? "default" : "horizontal";
    }

    /**
     * Update the NodeGraphConf when its orientation changes
     */
    private void updateNodeGraphConf()
    {
        DiagramGraphConf diagGraphConf = ((ModelerGraphicalViewer) getViewer()).getModelerEditor().getActiveConfiguration().getDiagramGraphConf();
        if (diagGraphConf != null)
        {
            if (isVertical())
            {
                setNodeGraphConf(Utils.getNodeGraphConf(diagGraphConf, getEObject(), "default"));
            }
            else
            {
                setNodeGraphConf(Utils.getNodeGraphConf(diagGraphConf, getEObject(), "horizontal"));
            }
        }
    }
}