/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.activitydiagram.figures;

import org.eclipse.draw2d.AbstractBorder;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Insets;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.IContainerFigure;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.draw2d.figures.Label;
import org.topcased.draw2d.layout.GridData;
import org.topcased.draw2d.layout.GridLayout;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */
public class ActivityPartitionFigure extends Figure implements IContainerFigure, ILabelFigure
{
    private ILabel header;

    private IFigure contentPane;

    /**
     * Constructor
     */
    public ActivityPartitionFigure()
    {
        setLayoutManager(new GridLayout());

        header = createLabel();
        add(header, new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_CENTER));

        contentPane = createContainer();
        add(contentPane, new GridData(GridData.FILL_BOTH));

        setBorder(new PartitionBorder());
    }

    /**
     * Creates the header figure
     * 
     * @return the header figure
     */
    protected ILabel createLabel()
    {
        return new ComposedLabel(new Label(), new EditableLabel(), null, false);
    }

    /**
     * Creates the container figure
     * 
     * @return the container figure
     */
    protected IFigure createContainer()
    {
        Figure container = new Figure();
        container.setLayoutManager(new XYLayout());
        container.setOpaque(true);
        container.setBorder(new SingleTopBorder());
        return container;
    }

    /**
     * @see org.topcased.draw2d.figures.ILabelFigure#getLabel()
     */
    public ILabel getLabel()
    {
        return header;
    }

    /**
     * Return the contentPane figure
     * 
     * @return the Container Figure
     */
    public IFigure getContentPane()
    {
        return contentPane;
    }

    /**
     * A Border at the left, top and right of the Figure
     */
    public static class PartitionBorder extends AbstractBorder
    {
        private static final Insets INSETS = new Insets(0);

        /**
         * The Constructor
         */
        public PartitionBorder()
        {
            // Do nothing
        }

        /**
         * @see org.eclipse.draw2d.Border#getInsets(org.eclipse.draw2d.IFigure)
         */
        public Insets getInsets(IFigure figure)
        {
            return INSETS;
        }

        /**
         * Draw the 3 borders
         * 
         * @see org.eclipse.draw2d.Border#paint(org.eclipse.draw2d.IFigure, org.eclipse.draw2d.Graphics,
         *      org.eclipse.draw2d.geometry.Insets)
         */
        public void paint(IFigure figure, Graphics graphics, Insets insets)
        {
            graphics.pushState();

            graphics.setLineWidth(2);

            graphics.drawLine(getPaintRectangle(figure, insets).getBottomLeft().getTranslated(1, 0), getPaintRectangle(figure, insets).getTopLeft().getTranslated(1, 0));
            graphics.drawLine(getPaintRectangle(figure, insets).getTopLeft().getTranslated(0, 1), getPaintRectangle(figure, insets).getTopRight().getTranslated(0, 1));
            graphics.drawLine(getPaintRectangle(figure, insets).getTopRight().getTranslated(-1, 0), getPaintRectangle(figure, insets).getBottomRight().getTranslated(-1, 0));

            graphics.popState();
        }
    }

    /**
     * A Border at bottom of the Figure
     */
    public static class SingleTopBorder extends AbstractBorder
    {
        private static final Insets INSETS = new Insets(0);

        /**
         * The Constructor
         */
        public SingleTopBorder()
        {
            // Do nothing
        }

        /**
         * @see org.eclipse.draw2d.Border#getInsets(org.eclipse.draw2d.IFigure)
         */
        public Insets getInsets(IFigure figure)
        {
            return INSETS;
        }

        /**
         * Draw the 3 borders
         * 
         * @see org.eclipse.draw2d.Border#paint(org.eclipse.draw2d.IFigure, org.eclipse.draw2d.Graphics,
         *      org.eclipse.draw2d.geometry.Insets)
         */
        public void paint(IFigure figure, Graphics graphics, Insets insets)
        {
            graphics.pushState();

            graphics.setLineWidth(2);

            graphics.drawLine(getPaintRectangle(figure, insets).getTopLeft().getTranslated(0, 1), getPaintRectangle(figure, insets).getTopRight().getTranslated(0, 1));

            graphics.popState();
        }
    }
}