/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.activitydiagram.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutManager;
import org.eclipse.draw2d.RoundedRectangle;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.swt.SWT;
import org.eclipse.uml2.uml.ExpansionKind;
import org.topcased.draw2d.figures.IContainerFigure;
import org.topcased.draw2d.layout.BorderAttachedLayout;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */
public class ExpansionRegionFigure extends Figure implements IContainerFigure
{
    private static final int LINE_ON = 8;

    private static final int LINE_OFF = 7;

    private static final int INSETS = 10;

    private String mode;

    private IFigure anchorableFigure;

    private IFigure contentPane;

    /**
     * Constructor
     */
    public ExpansionRegionFigure()
    {
        this(ExpansionKind.ITERATIVE_LITERAL.getName());
    }

    /**
     * Constructor with the initialMode
     * 
     * @param initialMode the Mode of the ExpansionRegion. Indicate the text to display at the upper left corcer of the
     *        figure
     */
    public ExpansionRegionFigure(String initialMode)
    {
        this.mode = initialMode;
        createContents();
    }

    /**
     * Creates the contents of the figure : by defauft, it creates a layout manager, a header and a container
     */
    protected void createContents()
    {
        LayoutManager layout = new StackLayout();
        setLayoutManager(layout);

        anchorableFigure = new RoundedRectangle()
        {
            /**
             * @see org.eclipse.draw2d.Shape#outlineShape(org.eclipse.draw2d.Graphics)
             */
            protected void outlineShape(Graphics graphics)
            {
                graphics.pushState();

                graphics.drawText("<<" + mode + ">>", getBounds().getTopLeft().getTranslated(5, 15));

                graphics.setLineStyle(SWT.LINE_CUSTOM);
                graphics.setLineDash(new int[] {LINE_ON, LINE_OFF});
                graphics.drawRoundRectangle(getBounds().getResized(-1, -1), corner.width, corner.height);

                graphics.popState();
            }

            /**
             * @see org.eclipse.draw2d.Shape#fillShape(org.eclipse.draw2d.Graphics)
             */
            protected void fillShape(Graphics graphics)
            {
                graphics.fillRoundRectangle(getBounds(), corner.width, corner.height);
            }

        };
        anchorableFigure.setOpaque(true);

        IFigure containerOfAnchorableFigure = new Figure()
        {
            public Insets getInsets()
            {
                // Set the Insets between the global figure bounds and the inner anchorableFigure
                return new Insets(INSETS);
            }
        };
        containerOfAnchorableFigure.setLayoutManager(new StackLayout());
        containerOfAnchorableFigure.add(anchorableFigure);
        add(containerOfAnchorableFigure);

        contentPane = new Figure();
        contentPane.setLayoutManager(new BorderAttachedLayout());
        add(contentPane);
    }

    /**
     * Return the contentPane figure
     * 
     * @return the Container Figure
     */
    public IFigure getContentPane()
    {
        return contentPane;
    }

    /**
     * Update the mode of the ExpansionRegion
     * 
     * @param expansionKind the new ExpansionKind
     */
    public void setMode(ExpansionKind expansionKind)
    {
        switch (expansionKind.getValue())
        {
            case ExpansionKind.ITERATIVE:
                mode = ExpansionKind.ITERATIVE_LITERAL.getName();
                break;
            case ExpansionKind.PARALLEL:
                mode = ExpansionKind.PARALLEL_LITERAL.getName();
                break;
            case ExpansionKind.STREAM:
                mode = ExpansionKind.STREAM_LITERAL.getName();
                break;

            default:
                break;
        }
    }

    /**
     * Return the graphical figure that should be used to attach the connections.
     * 
     * @return the rounded rectangle on which connections should be attached.
     */
    public IFigure getAnchorableFigure()
    {
        return anchorableFigure;
    }
}