/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.activitydiagram.internal.properties.sections;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.uml2.uml.ActivityEdge;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.uml.properties.sections.AbstractValueSpecificationSection;

/**
 * The Section used to edit the Weight associated with an ActivityEdge
 * 
 * Creation 9 nov. 06
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class ActivityEdgeWeightSection extends AbstractValueSpecificationSection
{
    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getFeature()
     */
    protected EStructuralFeature getFeature()
    {
        return UMLPackage.eINSTANCE.getActivityEdge_Weight();
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getLabelText()
     */
    protected String getLabelText()
    {
        return "Weight:";
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractDetailedObjectPropertySection#getFeatureAsText()
     */
    protected String getFeatureAsText()
    {
        if (((ActivityEdge) getEObject()).getWeight() != null)
        {
            String featureTextValue = ((ActivityEdge) getEObject()).getWeight().stringValue();
            return featureTextValue == null ? "" : featureTextValue;
        }
        return "";
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractDetailedObjectPropertySection#getRelatedEObject()
     */
    protected EObject getRelatedEObject()
    {
        return ((ActivityEdge) getEObject()).getWeight();
    }
}
