/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.activitydiagram.policies;

import java.util.List;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.ActivityEdge;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.ActivityParameterNode;
import org.eclipse.uml2.uml.ActivityPartition;
import org.eclipse.uml2.uml.ExpansionNode;
import org.eclipse.uml2.uml.InputPin;
import org.eclipse.uml2.uml.OutputPin;
import org.topcased.draw2d.layout.PortConstraint;
import org.topcased.modeler.commands.ChangeBoundsCommand;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy;
import org.topcased.modeler.uml.activitydiagram.commands.ActivityPartitionChangeBoundsCommand;
import org.topcased.modeler.uml.activitydiagram.commands.ActivityPartitionCreationCommand;
import org.topcased.modeler.uml.activitydiagram.edit.ActivityPartitionEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ActivityDiagramLayoutEditPolicy extends ModelerLayoutEditPolicy
{
    /**
     * Default contructor.<br>
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public ActivityDiagramLayoutEditPolicy()
    {
        super();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isAttachedToBorder(org.topcased.modeler.di.model.GraphNode)
     * @generated
     */
    protected boolean isAttachedToBorder(GraphNode node)
    {
        if (Utils.getElement(node) instanceof ActivityParameterNode)
        {
            return true;
        }
        if (Utils.getElement(node) instanceof ExpansionNode)
        {
            return true;
        }
        if (Utils.getElement(node) instanceof InputPin)
        {
            return true;
        }
        if (Utils.getElement(node) instanceof OutputPin)
        {
            return true;
        }
        return super.isAttachedToBorder(node);
    }

    /**
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#getCreateCommand(org.eclipse.gef.EditDomain,
     *      org.topcased.modeler.di.model.GraphNode, org.topcased.modeler.di.model.GraphNode,
     *      org.eclipse.emf.ecore.EObject, org.eclipse.draw2d.geometry.Point, org.eclipse.draw2d.geometry.Dimension,
     *      int, java.util.List, boolean)
     */
    protected Command getCreateCommand(EditDomain domain, GraphNode newObject, GraphNode newParent, EObject newContainerParent, Point location, Dimension dimension, int attach, List featuresList,
            boolean needModelUpdate)
    {
        if (Utils.getElement(newObject) instanceof ActivityNode)
        {
            return new CreateGraphNodeCommand(domain, newObject, newParent, newContainerParent, location, dimension, attach, featuresList, needModelUpdate);
        }
        else if (Utils.getElement(newObject) instanceof ActivityEdge)
        {
            return new CreateGraphNodeCommand(domain, newObject, newParent, newContainerParent, location, dimension, attach, featuresList, needModelUpdate);
        }
        else if (Utils.getElement(newObject) instanceof ActivityPartition)
        {
            return new ActivityPartitionCreationCommand(domain, newObject, newParent, newContainerParent, location, dimension, attach, featuresList, needModelUpdate, getHost());
        }
        return new CreateGraphNodeCommand(domain, newObject, newParent, newContainerParent, location, dimension, attach, featuresList, needModelUpdate);
    }

    /**
     * When an ActivityPartition is resized or moved, we recalculate its contained ActivityNodes and ActivityEdges.
     * 
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart,
     *      java.lang.Object, boolean)
     */
    protected Command createChangeConstraintCommand(EditPart child, Object constraint, boolean checkConstraint)
    {
        GraphNode node = (GraphNode) child.getModel();

        if (constraint instanceof PortConstraint)
        {
            return new ChangeBoundsCommand(node, (PortConstraint) constraint);
        }
        else if (!checkConstraint || isValidConstraint((GraphicalEditPart) getHost(), (Rectangle) constraint))
        {
            if (child instanceof ActivityPartitionEditPart)
            {
                return new ActivityPartitionChangeBoundsCommand(node, (Rectangle) constraint, child, getHost());
            }
            // TODO We should update the ActivityPartition.getNodes() list during the resize/move of an ActivityNode
            // TODO We should update the ActivityPartition.getEdges() list during the reconnection of an ActivityEdge
            return new ChangeBoundsCommand(node, (Rectangle) constraint);
        }

        return null;

    }
}