/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.classdiagram;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.util.UMLSwitch;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.edit.EListEditPart;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.uml.alldiagram.edit.CommentEditPart;
import org.topcased.modeler.uml.alldiagram.edit.CommentLinkEditPart;
import org.topcased.modeler.uml.classdiagram.edit.AssociationClassEditPart;
import org.topcased.modeler.uml.classdiagram.edit.AssociationEditPart;
import org.topcased.modeler.uml.classdiagram.edit.BehavioredClassifierEditPart;
import org.topcased.modeler.uml.classdiagram.edit.ClassDiagramEditPart;
import org.topcased.modeler.uml.classdiagram.edit.ClassEditPart;
import org.topcased.modeler.uml.classdiagram.edit.ClassFromAssociationClassEditPart;
import org.topcased.modeler.uml.classdiagram.edit.ClassifierEditPart;
import org.topcased.modeler.uml.classdiagram.edit.DataTypeEditPart;
import org.topcased.modeler.uml.classdiagram.edit.DependencyEditPart;
import org.topcased.modeler.uml.classdiagram.edit.EnumerationEditPart;
import org.topcased.modeler.uml.classdiagram.edit.EnumerationLiteralEditPart;
import org.topcased.modeler.uml.classdiagram.edit.GeneralizationEditPart;
import org.topcased.modeler.uml.classdiagram.edit.InstanceSpecificationEditPart;
import org.topcased.modeler.uml.classdiagram.edit.InterfaceEditPart;
import org.topcased.modeler.uml.classdiagram.edit.InterfaceRealizationEditPart;
import org.topcased.modeler.uml.classdiagram.edit.NamedElementEditPart;
import org.topcased.modeler.uml.classdiagram.edit.OperationEditPart;
import org.topcased.modeler.uml.classdiagram.edit.PackageEditPart;
import org.topcased.modeler.uml.classdiagram.edit.PackageImportEditPart;
import org.topcased.modeler.uml.classdiagram.edit.PackageMergeEditPart;
import org.topcased.modeler.uml.classdiagram.edit.PrimitiveTypeEditPart;
import org.topcased.modeler.uml.classdiagram.edit.PropertyEListEditPart;
import org.topcased.modeler.uml.classdiagram.edit.PropertyEditPart;
import org.topcased.modeler.uml.classdiagram.edit.SlotEditPart;
import org.topcased.modeler.uml.classdiagram.edit.TypeEditPart;
import org.topcased.modeler.uml.classdiagram.edit.UsageEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * Part Factory : associates a model object to its controller. <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ClassEditPartFactory implements EditPartFactory
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
     * @generated
     */
    public EditPart createEditPart(EditPart context, Object model)
    {
        if (model instanceof Diagram)
        {
            return new ClassDiagramEditPart((Diagram) model);
        }
        else if (model instanceof GraphNode)
        {
            final GraphNode node = (GraphNode) model;
            EObject element = Utils.getElement(node);
            if (element != null)
            {
                if ("http://www.eclipse.org/uml2/2.1.0/UML".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new NodeUMLSwitch(node).doSwitch(element);
                }
                // A default GraphNodeEditPart used to represent elements that are not associated with a model element anymore 
                return new EMFGraphNodeEditPart(node); 
            }
            return new GraphNodeEditPart(node);
        }
        else if (model instanceof GraphEdge)
        {
            final GraphEdge edge = (GraphEdge) model;
            EObject element = Utils.getElement(edge);
            if (element != null)
            {
                if ("http://www.eclipse.org/uml2/2.1.0/UML".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new EdgeUMLSwitch(edge).doSwitch(element);
                }
                // A default GraphEdgeEditPart used to represent elements that are not associated with a model element anymore (desynchronization problem)
                return new EMFGraphEdgeEditPart(edge);
            }
            if (edge.getSemanticModel() instanceof SimpleSemanticModelElement)
            {
                if (ClassSimpleObjectConstants.SIMPLE_OBJECT_COMMENTLINK.equals(((SimpleSemanticModelElement) edge.getSemanticModel()).getTypeInfo()))
                {
                    return new CommentLinkEditPart(edge);
                }
            }
            return new GraphEdgeEditPart(edge);
        }

        throw new IllegalStateException("No edit part matches with the '" + model.getClass().getName() + "' model element. Check ClassEditPartFactory#createEditPart(EditPart,Object) class");
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class NodeUMLSwitch extends UMLSwitch<Object>
    {
        /** The graphical node */
        private GraphNode node;

        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param node the graphical node
         * @generated
         */
        public NodeUMLSwitch(GraphNode node)
        {
            this.node = node;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseNamedElement(org.eclipse.uml2.uml.NamedElement)
         * @generated
         */
        public Object caseNamedElement(org.eclipse.uml2.uml.NamedElement object)
        {
            return new NamedElementEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseType(org.eclipse.uml2.uml.Type)
         * @generated
         */
        public Object caseType(org.eclipse.uml2.uml.Type object)
        {
            return new TypeEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseClassifier(org.eclipse.uml2.uml.Classifier)
         * @generated
         */
        public Object caseClassifier(org.eclipse.uml2.uml.Classifier object)
        {
            return new ClassifierEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseBehavioredClassifier(org.eclipse.uml2.uml.BehavioredClassifier)
         * @generated
         */
        public Object caseBehavioredClassifier(org.eclipse.uml2.uml.BehavioredClassifier object)
        {
            return new BehavioredClassifierEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#casePackage(org.eclipse.uml2.uml.Package)
         * @generated
         */
        public Object casePackage(org.eclipse.uml2.uml.Package object)
        {
            return new PackageEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseClass(org.eclipse.uml2.uml.Class)
         * @generated NOT
         */
        public Object caseClass(org.eclipse.uml2.uml.Class object)
        {
            String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
            if (!"".equals(feature))
            {
                int featureID = Integer.parseInt(feature);

                // add a customized PropertyEListEditPart to display
                // Properties and filtering those associated with an
                // Association
                if (UMLPackage.CLASS__OWNED_ATTRIBUTE == featureID)
                {
                    return new PropertyEListEditPart(node, object.eClass().getEStructuralFeature(featureID));
                }

                return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
            }
            else
            {
                return new ClassEditPart(node);
            }
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseInterface(org.eclipse.uml2.uml.Interface)
         * @generated
         */
        public Object caseInterface(org.eclipse.uml2.uml.Interface object)
        {
            String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
            if (!"".equals(feature))
            {
                int featureID = Integer.parseInt(feature);
                return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
            }
            else
            {
                return new InterfaceEditPart(node);
            }
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseDataType(org.eclipse.uml2.uml.DataType)
         * @generated
         */
        public Object caseDataType(org.eclipse.uml2.uml.DataType object)
        {
            String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
            if (!"".equals(feature))
            {
                int featureID = Integer.parseInt(feature);
                return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
            }
            else
            {
                return new DataTypeEditPart(node);
            }
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseOperation(org.eclipse.uml2.uml.Operation)
         * @generated
         */
        public Object caseOperation(org.eclipse.uml2.uml.Operation object)
        {
            return new OperationEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseProperty(org.eclipse.uml2.uml.Property)
         * @generated
         */
        public Object caseProperty(org.eclipse.uml2.uml.Property object)
        {
            return new PropertyEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseInstanceSpecification(org.eclipse.uml2.uml.InstanceSpecification)
         * @generated
         */
        public Object caseInstanceSpecification(org.eclipse.uml2.uml.InstanceSpecification object)
        {
            String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
            if (!"".equals(feature))
            {
                int featureID = Integer.parseInt(feature);
                return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
            }
            else
            {
                return new InstanceSpecificationEditPart(node);
            }
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseComment(org.eclipse.uml2.uml.Comment)
         * @generated
         */
        public Object caseComment(org.eclipse.uml2.uml.Comment object)
        {
            return new CommentEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseSlot(org.eclipse.uml2.uml.Slot)
         * @generated
         */
        public Object caseSlot(org.eclipse.uml2.uml.Slot object)
        {
            return new SlotEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseEnumeration(org.eclipse.uml2.uml.Enumeration)
         * @generated
         */
        public Object caseEnumeration(org.eclipse.uml2.uml.Enumeration object)
        {
            String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
            if (!"".equals(feature))
            {
                int featureID = Integer.parseInt(feature);
                return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
            }
            else
            {
                return new EnumerationEditPart(node);
            }
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseEnumerationLiteral(org.eclipse.uml2.uml.EnumerationLiteral)
         * @generated
         */
        public Object caseEnumerationLiteral(org.eclipse.uml2.uml.EnumerationLiteral object)
        {
            return new EnumerationLiteralEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#casePrimitiveType(org.eclipse.uml2.uml.PrimitiveType)
         * @generated
         */
        public Object casePrimitiveType(org.eclipse.uml2.uml.PrimitiveType object)
        {
            String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
            if (!"".equals(feature))
            {
                int featureID = Integer.parseInt(feature);
                return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
            }
            else
            {
                return new PrimitiveTypeEditPart(node);
            }
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseAssociationClass(org.eclipse.uml2.uml.AssociationClass)
         */
        public Object caseAssociationClass(org.eclipse.uml2.uml.AssociationClass object)
        {
            String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
            if (!"".equals(feature))
            {
                int featureID = Integer.parseInt(feature);

                // add a customized PropertyEListEditPart to display
                // Properties and filtering those associated with an
                // Association
                if (UMLPackage.ASSOCIATION_CLASS__OWNED_ATTRIBUTE == featureID)
                {
                    return new PropertyEListEditPart(node, object.eClass().getEStructuralFeature(featureID));
                }

                return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
            }
            else
            {
                return new ClassFromAssociationClassEditPart(node);
            }
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphNodeEditPart(node);
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class EdgeUMLSwitch extends UMLSwitch
    {
        /** The graphical edge */
        private GraphEdge edge;

        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param edge the graphical edge
         * @generated
         */
        public EdgeUMLSwitch(GraphEdge edge)
        {
            this.edge = edge;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseDependency(org.eclipse.uml2.uml.Dependency)
         * @generated
         */
        public Object caseDependency(org.eclipse.uml2.uml.Dependency object)
        {
            return new DependencyEditPart(edge);
        }
        
        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseDependency(org.eclipse.uml2.uml.Usage)
         * @generated
         */
        public Object caseUsage(org.eclipse.uml2.uml.Usage object)
        {
            return new UsageEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseAssociation(org.eclipse.uml2.uml.Association)
         * @generated
         */
        public Object caseAssociation(org.eclipse.uml2.uml.Association object)
        {
            return new AssociationEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseGeneralization(org.eclipse.uml2.uml.Generalization)
         * @generated
         */
        public Object caseGeneralization(org.eclipse.uml2.uml.Generalization object)
        {
            return new GeneralizationEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseInterfaceRealization(org.eclipse.uml2.uml.InterfaceRealization)
         * @generated
         */
        public Object caseInterfaceRealization(org.eclipse.uml2.uml.InterfaceRealization object)
        {
            return new InterfaceRealizationEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#casePackageImport(org.eclipse.uml2.uml.PackageImport)
         * @generated
         */
        public Object casePackageImport(org.eclipse.uml2.uml.PackageImport object)
        {
            return new PackageImportEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#casePackageMerge(org.eclipse.uml2.uml.PackageMerge)
         * @generated
         */
        public Object casePackageMerge(org.eclipse.uml2.uml.PackageMerge object)
        {
            return new PackageMergeEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseAssociationClass(org.eclipse.uml2.uml.AssociationClass)
         * @generated
         */
        public Object caseAssociationClass(org.eclipse.uml2.uml.AssociationClass object)
        {
            return new AssociationClassEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphEdgeEditPart(edge);
        }
    }

}