/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.classdiagram.commands;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Type;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.uml.alldiagram.commands.ElementRestoreConnectionCommand;
import org.topcased.modeler.utils.Utils;

/**
 * NamedElement restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class NamedElementRestoreConnectionCommand extends ElementRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public NamedElementRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated
     */
    protected void initializeCommands()
    {

        super.initializeCommands();

        GraphElement elt = getGraphElement();
        EObject eltObject = Utils.getElement(elt);

        if (eltObject instanceof NamedElement)
        {
            Iterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(elt);
                    GraphElement elt2 = (GraphElement) obj;
                    EObject eltObject2 = Utils.getElement(elt2);

                    if (eltObject2 instanceof NamedElement)
                    {
                        if (autoRef)
                        {
                            // autoRef not allowed
                        }
                        else
                        {
                            // if the elt is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createDependencyFromNamedElementToNamedElement(elt, elt2);
                            // if elt is the target of the edge or if it is the
                            // source and that the SourceTargetCouple is
                            // reversible
                            createDependencyFromNamedElementToNamedElement(elt2, elt);
                        }
                    }

                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createDependencyFromNamedElementToNamedElement(GraphElement srcNode, GraphElement targetNode)
    {
        NamedElement sourceObject = (NamedElement) Utils.getElement(srcNode);
        NamedElement targetObject = (NamedElement) Utils.getElement(targetNode);

        org.eclipse.uml2.uml.Package srcPack = getPackage(sourceObject);
        org.eclipse.uml2.uml.Package trgPack = getPackage(targetObject);
        
        srcPack = srcPack == null ?(Package) Utils.getDiagramModelObject(srcNode) : srcPack;
        trgPack = trgPack == null ?(Package) Utils.getDiagramModelObject(targetNode) : trgPack;
        
        List edgeObjectList = new ArrayList();
        edgeObjectList.addAll(srcPack.getPackagedElements());
        if (trgPack != srcPack)
        {
            edgeObjectList.addAll(trgPack.getPackagedElements()); 
        }
        for (Iterator it = edgeObjectList.iterator(); it.hasNext();)
        {
            Object obj = it.next();
            if (obj instanceof Dependency)
            {
                Dependency edgeObject = (Dependency) obj;
                if (edgeObject.getSuppliers().contains(targetObject) && edgeObject.getClients().contains(sourceObject))
                {
                    // check if the relation does not exists yet
                    List existing = getExistingEdges(srcNode, targetNode, Dependency.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            DependencyEdgeCreationCommand cmd = new DependencyEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                            cmd.setTarget(targetNode);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

    private Package getPackage(NamedElement element)
    {
        if (element instanceof Package)
        {
            return ((Package) element).getNestingPackage();
        }
        else if (element instanceof Type)
        {
            return ((Type) element).getPackage();
        }
        
        return null;
    }
}