/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.classdiagram.commands;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.AssociationClass;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;

/**
 * Type restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class TypeRestoreConnectionCommand extends NamedElementRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public TypeRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated
     */
    protected void initializeCommands()
    {

        super.initializeCommands();

        GraphElement elt = getGraphElement();
        EObject eltObject = Utils.getElement(elt);

        if (eltObject instanceof Type)
        {
            Iterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(elt);
                    GraphElement elt2 = (GraphElement) obj;
                    EObject eltObject2 = Utils.getElement(elt2);

                    if (eltObject2 instanceof Type)
                    {
                        if (autoRef)
                        {
                            createAssociationFromTypeToType(elt, elt);
                        }
                        else
                        {
                            // if the elt is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createAssociationFromTypeToType(elt, elt2);
                            // if elt is the target of the edge or if it is the
                            // source and that the SourceTargetCouple is
                            // reversible
                            createAssociationFromTypeToType(elt2, elt);
                        }
                    }

                    if (eltObject2 instanceof Type)
                    {
                        if (autoRef)
                        {
                            createAssociationClassFromTypeToType(elt, elt);
                        }
                        else
                        {
                            // if the elt is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createAssociationClassFromTypeToType(elt, elt2);
                            // if elt is the target of the edge or if it is the
                            // source and that the SourceTargetCouple is
                            // reversible
                            createAssociationClassFromTypeToType(elt2, elt);
                        }
                    }

                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createAssociationFromTypeToType(GraphElement srcNode, GraphElement targetNode)
    {
        Type sourceObject = (Type) Utils.getElement(srcNode);
        Type targetObject = (Type) Utils.getElement(targetNode);

        
        List edgeObjectList = new ArrayList();
        org.eclipse.uml2.uml.Package srcPack = sourceObject.getPackage();
        edgeObjectList.addAll(srcPack.getPackagedElements());
        
        org.eclipse.uml2.uml.Package trgPack = targetObject.getPackage();
        if (trgPack != srcPack)
        {
            edgeObjectList.addAll(trgPack.getPackagedElements()); 
        }
        
        Iterator it = edgeObjectList.iterator();
        while (it.hasNext())
        {
            Object obj = it.next();
            if (obj instanceof Association)
            {
                Association edgeObject = (Association) obj;

                if (edgeObject.getMemberEnds().size() == 2)
                {
                    if (sourceObject.equals(((Property) edgeObject.getMemberEnds().get(0)).getType()) && targetObject.equals(((Property) edgeObject.getMemberEnds().get(1)).getType()))
                    {
                        // check if the relation does not exists yet
                        List existing = getExistingEdges(srcNode, targetNode, Association.class);
                        if (!isAlreadyPresent(existing, edgeObject))
                        {
                            ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                            GraphElement edge = factory.createGraphElement(edgeObject);
                            if (edge instanceof GraphEdge)
                            {
                                AssociationEdgeCreationCommand cmd = new AssociationEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                                cmd.setTarget(targetNode);
                                add(cmd);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcElt the source element
     * @param targetElt the target element
     * @generated NOT
     */
    private void createAssociationClassFromTypeToType(GraphElement srcElt, GraphElement targetElt)
    {
//        Type sourceObject = (Type) Utils.getElement(srcElt);
//        Type targetObject = (Type) Utils.getElement(targetElt);

        EList edgeObjectList = ((org.eclipse.uml2.uml.Package) Utils.getDiagramModelObject(srcElt)).getPackagedElements();
        for (Iterator it = edgeObjectList.iterator(); it.hasNext();)
        {
            Object obj = it.next();
            if (obj instanceof AssociationClass)
            {
                AssociationClass edgeObject = (AssociationClass) obj;
                if (false)
                {
                    // check if the relation does not exists yet
                    List existing = getExistingEdges(srcElt, targetElt, AssociationClass.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            AssociationClassEdgeCreationCommand cmd = new AssociationClassEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcElt, false);
                            cmd.setTarget(targetElt);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

}