/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *   Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware Technologies),
 *   Nicolas Lalevée (Anyware Technologies) - initial API and implementation 
 ******************************************************************************/

package org.topcased.modeler.uml.classdiagram.commands.update;

import java.util.Map;

import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.VisibilityKind;
import org.topcased.modeler.uml.classdiagram.util.AssociationHelper;

/**
 * Class that create a command in order to update operation parameters <br>
 * creation : 12 mai 2005
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * @deprecated The Association should be updated step by step
 */
public class AssociationUpdateCommand extends Command
{

    /** The association currently updated */
    private Association association;

    /** Map containing new association data */
    private Map newAssociationData;

    /** Map containing old association data */
    private Map oldAssociationData;

    /**
     * Create a command for updating parameters on a given operation
     * 
     * @param assoc the association to update
     * @param prop Map containing association data
     */
    public AssociationUpdateCommand(Association assoc, Map prop)
    {
        this.association = assoc;
        this.newAssociationData = prop;
    }

    /**
     * Get the old values and set the new ones
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        AssociationHelper associationHelper = new AssociationHelper(association);
        oldAssociationData = associationHelper.extractAssociationData();

        redo();
    }

    /**
     * Set the new values
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        // Update the Association properties
        association.setName((String) newAssociationData.get(AssociationHelper.ASSOCIATION_NAME));

        // Update the Association First End properties
        Property firstEndProperty = (Property) association.getMemberEnds().get(0);
        firstEndProperty.setAggregation((AggregationKind) newAssociationData.get(AssociationHelper.FIRST_END_AGGREGATION_KIND));
        firstEndProperty.setIsOrdered(((Boolean) newAssociationData.get(AssociationHelper.FIRST_END_IS_ORDERED)).booleanValue());
        firstEndProperty.setIsNavigable(((Boolean) newAssociationData.get(AssociationHelper.FIRST_END_IS_NAVIGABLE)).booleanValue());

        firstEndProperty.setLower(Integer.parseInt((String) newAssociationData.get(AssociationHelper.FIRST_END_LOWER_BOUND)));
        firstEndProperty.setUpper(Integer.parseInt((String) newAssociationData.get(AssociationHelper.FIRST_END_UPPER_BOUND)));
        firstEndProperty.setName((String) newAssociationData.get(AssociationHelper.FIRST_END_ROLE));
        firstEndProperty.setVisibility((VisibilityKind) newAssociationData.get(AssociationHelper.FIRST_END_VISIBILITY_KIND));

        // Update the Association Second End properties
        Property secondEndProperty = (Property) association.getMemberEnds().get(1);
        secondEndProperty.setAggregation((AggregationKind) newAssociationData.get(AssociationHelper.SECOND_END_AGGREGATION_KIND));
        secondEndProperty.setIsOrdered(((Boolean) newAssociationData.get(AssociationHelper.SECOND_END_IS_ORDERED)).booleanValue());
        secondEndProperty.setIsNavigable(((Boolean) newAssociationData.get(AssociationHelper.SECOND_END_IS_NAVIGABLE)).booleanValue());

        secondEndProperty.setLower(Integer.parseInt((String) newAssociationData.get(AssociationHelper.SECOND_END_LOWER_BOUND)));
        secondEndProperty.setUpper(Integer.parseInt((String) newAssociationData.get(AssociationHelper.SECOND_END_UPPER_BOUND)));
        secondEndProperty.setName((String) newAssociationData.get(AssociationHelper.SECOND_END_ROLE));
        secondEndProperty.setVisibility((VisibilityKind) newAssociationData.get(AssociationHelper.SECOND_END_VISIBILITY_KIND));
    }

    /**
     * Set the old values
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        // Update the Association properties
        association.setName((String) oldAssociationData.get(AssociationHelper.ASSOCIATION_NAME));

        // Update the Association First End properties
        Property firstEndProperty = (Property) association.getMembers().get(0);
        firstEndProperty.setAggregation((AggregationKind) oldAssociationData.get(AssociationHelper.FIRST_END_AGGREGATION_KIND));
        firstEndProperty.setIsOrdered(((Boolean) oldAssociationData.get(AssociationHelper.FIRST_END_IS_ORDERED)).booleanValue());
        firstEndProperty.setIsNavigable(((Boolean) oldAssociationData.get(AssociationHelper.FIRST_END_IS_NAVIGABLE)).booleanValue());

        firstEndProperty.setLower(Integer.parseInt((String) oldAssociationData.get(AssociationHelper.FIRST_END_LOWER_BOUND)));
        firstEndProperty.setUpper(Integer.parseInt((String) oldAssociationData.get(AssociationHelper.FIRST_END_UPPER_BOUND)));

        firstEndProperty.setName((String) oldAssociationData.get(AssociationHelper.FIRST_END_ROLE));
        firstEndProperty.setVisibility((VisibilityKind) oldAssociationData.get(AssociationHelper.FIRST_END_VISIBILITY_KIND));

        // Update the Association Second End properties
        Property secondEndProperty = (Property) association.getMembers().get(1);
        secondEndProperty.setAggregation((AggregationKind) oldAssociationData.get(AssociationHelper.SECOND_END_AGGREGATION_KIND));
        secondEndProperty.setIsOrdered(((Boolean) oldAssociationData.get(AssociationHelper.SECOND_END_IS_ORDERED)).booleanValue());
        secondEndProperty.setIsNavigable(((Boolean) oldAssociationData.get(AssociationHelper.SECOND_END_IS_NAVIGABLE)).booleanValue());

        secondEndProperty.setLower(Integer.parseInt((String) oldAssociationData.get(AssociationHelper.SECOND_END_LOWER_BOUND)));
        secondEndProperty.setUpper(Integer.parseInt((String) oldAssociationData.get(AssociationHelper.SECOND_END_UPPER_BOUND)));

        secondEndProperty.setName((String) oldAssociationData.get(AssociationHelper.SECOND_END_ROLE));
        secondEndProperty.setVisibility((VisibilityKind) oldAssociationData.get(AssociationHelper.SECOND_END_VISIBILITY_KIND));
    }
}
