/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *   Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware Technologies),
 *   Nicolas Lalevée (Anyware Technologies) - initial API and implementation 
 ******************************************************************************/

package org.topcased.modeler.uml.classdiagram.commands.update;

import java.util.Map;

import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.VisibilityKind;
import org.topcased.modeler.uml.classdiagram.dialogs.PropertyEditDialog;

/**
 * Class that create a command in order to update property parameters <br>
 * creation : 31 may 2005
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 */
public class PropertyUpdateCommand extends Command
{
    /** Current property */
    private Property property;

    /** Old values */
    private String oldName;

    private Type oldType;

    private VisibilityKind oldVisibility;

    /** New values */
    private String name;

    private Type type;

    private VisibilityKind visibility;

    /**
     * Create a command for updating parameters on a given property
     * 
     * @param prop the property to update
     * @param data the datas to set
     */
    public PropertyUpdateCommand(Property prop, Map data)
    {
        property = prop;
        name = (String) data.get(PropertyEditDialog.PROPERTY_NAME);
        type = (Type) data.get(PropertyEditDialog.PROPERTY_TYPE);
        visibility = (VisibilityKind) data.get(PropertyEditDialog.PROPERTY_VISIBILITY);
    }

    /**
     * Get the old values
     */
    protected void getOldValues()
    {
        oldName = property.getName();
        oldType = property.getType();
        oldVisibility = property.getVisibility();
    }

    /**
     * Set the values on the Operation
     */
    protected void setValues()
    {
        // Perform update
        property.setName(name);
        property.setType(type);
        property.setVisibility(visibility);
    }

    /**
     * Switch the old and new values
     */
    protected void switchValues()
    {
        // Reverse old and new value for name
        String tempString = name;
        name = oldName;
        oldName = tempString;

        // Reverse old and new value for type
        Type tempType = type;
        type = oldType;
        oldType = tempType;

        // Reverse old and new value for visibility
        VisibilityKind tempVisibility = visibility;
        visibility = oldVisibility;
        oldVisibility = tempVisibility;
    }

    /**
     * Get the old values and set the new ones
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        getOldValues();
        setValues();
    }

    /**
     * Set the new values
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        switchValues();
        setValues();
    }

    /**
     * Set the old values
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        switchValues();
        setValues();
    }

}
