/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.classdiagram.edit;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.AssociationClass;
import org.topcased.draw2d.figures.Label;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.policies.EdgeObjectOffsetEditPolicy;
import org.topcased.modeler.figures.EdgeObjectOffsetEditableLabel;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.uml.UMLTools;
import org.topcased.modeler.uml.classdiagram.ClassEdgeObjectConstants;
import org.topcased.modeler.uml.classdiagram.figures.AssociationClassFigure;
import org.topcased.modeler.uml.classdiagram.policies.AssociationClassEdgeObjectUVEditPolicy;
import org.topcased.modeler.uml.classdiagram.util.AssociationHelper;
import org.topcased.modeler.utils.Utils;

/**
 * AssociationClass controller <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class AssociationClassEditPart extends EMFGraphEdgeEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param model the graph object
     * @generated
     */
    public AssociationClassEditPart(GraphEdge model)
    {
        super(model);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_UV_EDITPOLICY, new AssociationClassEdgeObjectUVEditPolicy());
        installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_OFFSET_EDITPOLICY, new EdgeObjectOffsetEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Figure
     * @generated
     */
    protected IFigure createFigure()
    {
        AssociationClassFigure connection = new AssociationClassFigure();

        return connection;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#getEdgeObjectFigure(org.topcased.modeler.di.model.EdgeObject)
     * @generated
     */
    public IEdgeObjectFigure getEdgeObjectFigure(EdgeObject edgeObject)
    {
        if (ClassEdgeObjectConstants.SRCNAME_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((AssociationClassFigure) getFigure()).getSrcNameEdgeObjectFigure();
        }
        if (ClassEdgeObjectConstants.SRCCOUNT_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((AssociationClassFigure) getFigure()).getSrcCountEdgeObjectFigure();
        }
        if (ClassEdgeObjectConstants.TARGETNAME_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((AssociationClassFigure) getFigure()).getTargetNameEdgeObjectFigure();
        }
        if (ClassEdgeObjectConstants.TARGETCOUNT_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((AssociationClassFigure) getFigure()).getTargetCountEdgeObjectFigure();
        }
        if (ClassEdgeObjectConstants.MIDDLENAME_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((AssociationClassFigure) getFigure()).getMiddleNameEdgeObjectFigure();
        }
        return null;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#refreshEdgeObjects()
     * @generated
     */
    protected void refreshEdgeObjects()
    {
        super.refreshEdgeObjects();
        updateSrcNameLabel();
        updateSrcCountLabel();
        updateTargetNameLabel();
        updateTargetCountLabel();
        updateMiddleNameLabel();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the srcName Label
     * 
     * @generated NOT
     */
    private void updateSrcNameLabel()
    {
        Label srcName = (Label) ((AssociationClassFigure) getFigure()).getSrcNameEdgeObjectFigure();

        AssociationClass associationClass = (AssociationClass) Utils.getElement(getGraphEdge());
        AssociationHelper associationHelper = new AssociationHelper(associationClass);

        String visibility = UMLTools.getVisibilityNotation(associationHelper.getAssociationFirstEndVisibilityKind());
        srcName.setText(visibility + " " + associationHelper.getAssociationFirstEndRole());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the srcCount Label
     * 
     * @generated NOT
     */
    private void updateSrcCountLabel()
    {
        Label srcCount = (Label) ((AssociationClassFigure) getFigure()).getSrcCountEdgeObjectFigure();

        AssociationClass associationClass = (AssociationClass) Utils.getElement(getGraphEdge());
        AssociationHelper associationHelper = new AssociationHelper(associationClass);

        String lowerBound = associationHelper.getAssociationFirstEndLowerBound();
        String upperBound = associationHelper.getAssociationFirstEndUpperBound();
        boolean isOrdered = associationHelper.getAssociationFirstEndIsOrdered();

        if (isOrdered)
        {
            srcCount.setText("{" + getMultiplicityLabel(lowerBound, upperBound) + "}");
        }
        else
        {
            srcCount.setText(getMultiplicityLabel(lowerBound, upperBound));
        }

    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the targetName Label
     * 
     * @generated NOT
     */
    private void updateTargetNameLabel()
    {
        Label targetName = (Label) ((AssociationClassFigure) getFigure()).getTargetNameEdgeObjectFigure();

        AssociationClass associationClass = (AssociationClass) Utils.getElement(getGraphEdge());
        AssociationHelper associationHelper = new AssociationHelper(associationClass);

        String visibility = UMLTools.getVisibilityNotation(associationHelper.getAssociationSecondEndVisibilityKind());
        targetName.setText(visibility + " " + associationHelper.getAssociationSecondEndRole());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the targetCount Label
     * 
     * @generated NOT
     */
    private void updateTargetCountLabel()
    {
        Label targetCount = (Label) ((AssociationClassFigure) getFigure()).getTargetCountEdgeObjectFigure();

        AssociationClass associationClass = (AssociationClass) Utils.getElement(getGraphEdge());
        AssociationHelper associationHelper = new AssociationHelper(associationClass);

        String lowerBound = associationHelper.getAssociationSecondEndLowerBound();
        String upperBound = associationHelper.getAssociationSecondEndUpperBound();
        boolean isOrdered = associationHelper.getAssociationSecondEndIsOrdered();

        if (isOrdered)
        {
            targetCount.setText("{" + getMultiplicityLabel(lowerBound, upperBound) + "}");
        }
        else
        {
            targetCount.setText(getMultiplicityLabel(lowerBound, upperBound));
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the middleName Label
     * 
     * @generated NOT
     */
    private void updateMiddleNameLabel()
    {
        AssociationClass associationClass = (AssociationClass) Utils.getElement(getGraphEdge());

        AssociationClassFigure fig = (AssociationClassFigure) getFigure();
        EdgeObjectOffsetEditableLabel label = (EdgeObjectOffsetEditableLabel) fig.getMiddleNameEdgeObjectFigure();
        label.setText(UMLTools.getStereotypesNotation(associationClass));
    }

    /**
     * Retrieve the multiplicity text that should be displayed
     * 
     * @param lowerBound the LowerBound text value
     * @param upperBound the UpperBound text value ("*" represent an Unbounded value)
     * @return the string value to display
     */
    private String getMultiplicityLabel(String lowerBound, String upperBound)
    {
        if (lowerBound.equals(upperBound))
        {
            return lowerBound;
        }
        else if ("0".equals(lowerBound) && "*".equals(upperBound))
        {
            return "*";
        }
        return lowerBound + ".." + upperBound;
    }

    /**
     * @see org.eclipse.gef.editparts.AbstractEditPart#createChild(java.lang.Object)
     */
    protected EditPart createChild(Object model)
    {
        return super.createChild(model);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
     * @generated
     */
    protected List getModelChildren()
    {
        List graphNodeChildren = new ArrayList();
        Iterator it = getGraphEdge().getContained().iterator();
        while (it.hasNext())
        {
            DiagramElement elt = (DiagramElement) it.next();
            if (elt instanceof GraphNode)
            {
                graphNodeChildren.add(elt);
            }
        }
        return graphNodeChildren;
    }

}