/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.classdiagram.edit;

import java.util.Iterator;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.PolylineDecoration;
import org.eclipse.draw2d.RotatableDecoration;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.Request;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.ValueSpecification;
import org.topcased.draw2d.figures.Label;
import org.topcased.modeler.ModelerColorConstants;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.policies.EdgeObjectOffsetEditPolicy;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.uml.UMLLabel;
import org.topcased.modeler.uml.alldiagram.policies.CommentLinkEdgeCreationEditPolicy;
import org.topcased.modeler.uml.classdiagram.ClassEdgeObjectConstants;
import org.topcased.modeler.uml.classdiagram.ClassEditPolicyConstants;
import org.topcased.modeler.uml.classdiagram.figures.AssociationFigure;
import org.topcased.modeler.uml.classdiagram.policies.AssociationEdgeObjectUVEditPolicy;
import org.topcased.modeler.uml.classdiagram.util.AssociationHelper;
import org.topcased.modeler.utils.Utils;

/**
 * Association controller <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class AssociationEditPart extends EMFGraphEdgeEditPart
{
    private RotatableDecoration srcDecor;

    private RotatableDecoration targetDecor;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param model the graph object
     * @generated
     */
    public AssociationEditPart(GraphEdge model)
    {
        super(model);
    }

    /**
     * Create a decoration for the composition
     * 
     * @return the new decoration
     */
    private PolygonDecoration createCompositionDecoration()
    {
        PolygonDecoration decoration = new PolygonDecoration();
        PointList decorationPointList = new PointList();
        decorationPointList.addPoint(0, 0);
        decorationPointList.addPoint(-1, 1);
        decorationPointList.addPoint(-2, 0);
        decorationPointList.addPoint(-1, -1);
        decoration.setTemplate(decorationPointList);
        decoration.setScale(10, 5);
        return decoration;
    }

    /**
     * Create a decoration for the aggregation
     * 
     * @return the new decoration
     */
    private PolygonDecoration createAggregationDecoration()
    {
        PolygonDecoration decoration = new PolygonDecoration();
        PointList decorationPointList = new PointList();
        decorationPointList.addPoint(0, 0);
        decorationPointList.addPoint(-1, 1);
        decorationPointList.addPoint(-2, 0);
        decorationPointList.addPoint(-1, -1);
        decoration.setBackgroundColor(ModelerColorConstants.white);
        decoration.setTemplate(decorationPointList);
        decoration.setScale(10, 5);
        return decoration;
    }

    /**
     * Create a decoration for the navigable association
     * 
     * @return the new decoration
     */
    private PolylineDecoration createNavigableDecoration()
    {
        PolylineDecoration decoration = new PolylineDecoration();
        decoration.setScale(10, 5);
        return decoration;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ClassEditPolicyConstants.COMMENTLINK_EDITPOLICY, new CommentLinkEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_UV_EDITPOLICY, new AssociationEdgeObjectUVEditPolicy());
        installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_OFFSET_EDITPOLICY, new EdgeObjectOffsetEditPolicy());

    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Figure
     * @generated NOT
     */
    protected IFigure createFigure()
    {
        return new AssociationFigure();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#getEdgeObjectFigure(org.topcased.modeler.di.model.EdgeObject)
     * @generated
     */
    public IEdgeObjectFigure getEdgeObjectFigure(EdgeObject edgeObject)
    {
        if (ClassEdgeObjectConstants.SRCNAME_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((AssociationFigure) getFigure()).getSrcNameEdgeObjectFigure();
        }
        if (ClassEdgeObjectConstants.SRCCOUNT_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((AssociationFigure) getFigure()).getSrcCountEdgeObjectFigure();
        }
        if (ClassEdgeObjectConstants.TARGETNAME_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((AssociationFigure) getFigure()).getTargetNameEdgeObjectFigure();
        }
        if (ClassEdgeObjectConstants.TARGETCOUNT_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((AssociationFigure) getFigure()).getTargetCountEdgeObjectFigure();
        }
        if (ClassEdgeObjectConstants.MIDDLENAME_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((AssociationFigure) getFigure()).getMiddleNameEdgeObjectFigure();
        }
        if (ClassEdgeObjectConstants.STEREOTYPE_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((AssociationFigure) getFigure()).getStereotypeEdgeObjectFigure();
        }
        return null;
    }

    /**
     * Handle the double-click to edit the association
     * 
     * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
     */
    public void performRequest(Request request)
    {
        super.performRequest(request);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void refreshEdgeObjects()
    {
        super.refreshEdgeObjects();

        Association association = (Association) Utils.getElement(getGraphEdge());

        // Check whether the association has its two Properties set
        if (association.isBinary())
        {
            if (((GraphEdge) getModel()).getAnchor().size() != 0)
            {
                updateSourceDecoration();
                updateTargetDecoration();
                updateSrcCountLabel();
                updateSrcNameLabel();
                updateTargetCountLabel();
                updateTargetNameLabel();
            }

            updateMiddleNameLabel();
            updateStereotypeLabel();
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the srcName Label
     * 
     * @generated NOT
     */
    private void updateSrcNameLabel()
    {
        Label srcNameLbl = (Label) ((AssociationFigure) getFigure()).getSrcNameEdgeObjectFigure();
        srcNameLbl.setText(getAssociationEndName((Property) ((Association) getEObject()).getMemberEnds().get(0)));
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the srcCount Label
     * 
     * @generated NOT
     */
    private void updateSrcCountLabel()
    {
        Label srcCount = (Label) ((AssociationFigure) getFigure()).getSrcCountEdgeObjectFigure();
        srcCount.setText(getMultiplicityLabel((Property) ((Association) getEObject()).getMemberEnds().get(0)));
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the targetName Label
     * 
     * @generated NOT
     */
    private void updateTargetNameLabel()
    {
        Label srcNameLbl = (Label) ((AssociationFigure) getFigure()).getTargetNameEdgeObjectFigure();
        srcNameLbl.setText(getAssociationEndName((Property) ((Association) getEObject()).getMemberEnds().get(1)));
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the targetCount Label
     * 
     * @generated NOT
     */
    private void updateTargetCountLabel()
    {
        Label targetCount = (Label) ((AssociationFigure) getFigure()).getTargetCountEdgeObjectFigure();
        targetCount.setText(getMultiplicityLabel((Property) ((Association) getEObject()).getMemberEnds().get(1)));
    }

    private String getAssociationEndName(Property property)
    {
        StringBuffer srcName = new StringBuffer();

        srcName.append(UMLLabel.getVisibilityNotation(property.getVisibility()));

        if (property.isDerived())
        {
            srcName.append(" /");
        }

        srcName.append(property.getName());

        String modifiersText = UMLLabel.getPropertyModifiersText(property);
        if (modifiersText != null && !"".equals(modifiersText))
        {
            srcName.append(" { ");
            srcName.append(modifiersText);
            srcName.append(" }");
        }

        return srcName.toString();
    }

    /**
     * Retrieve the multiplicity text that should be displayed
     * 
     * @param prop the Property on which the multiplicity label should be calculated
     * @return the string value to display
     */
    private String getMultiplicityLabel(Property prop)
    {
        StringBuffer text = new StringBuffer();

        int lower = prop.getLower();
        int upper = prop.getUpper();
        if (lower == upper)
        {
            text.append(lower);
        }
        else if (upper == -1)
        {
            if (lower == 0)
            {
                text.append("*");
            }
            else
            {
                text.append(lower);
                text.append("..*");
            }
        }
        else
        {
            text.append(lower);
            text.append("..");
            text.append(upper);
        }

        return text.toString();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the middleName Label
     * 
     * @generated NOT
     */
    private void updateMiddleNameLabel()
    {
        Association association = (Association) getEObject();
        String associationName = association.getName() == null ? "" : association.getName();
        if (association.isDerived())
        {
            // A slash appearing in front of the name of an association, or in place of the name if no name is shown,
            // marks the association as being derived.
            associationName = "/ ".concat(associationName);
        }
        ((Label) ((AssociationFigure) getFigure()).getMiddleNameEdgeObjectFigure()).setText(associationName);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the middleName Label
     * 
     * @generated NOT
     */
    private void updateStereotypeLabel()
    {
        ((Label) ((AssociationFigure) getFigure()).getStereotypeEdgeObjectFigure()).setText(UMLLabel.getStereotypesNotation((Element) getEObject()));
    }

    /**
     * Update the source decoration
     */
    private void updateSourceDecoration()
    {
        Association association = (Association) Utils.getElement(getGraphEdge());
        AssociationHelper associationHelper = new AssociationHelper(association);

        boolean isNavigable = associationHelper.getAssociationFirstEndIsNavigable();
        AggregationKind aggregationKind = associationHelper.getAssociationSecondEndAggregationKind();

        if (AggregationKind.COMPOSITE_LITERAL.equals(aggregationKind))
        {
            srcDecor = createCompositionDecoration();
        }
        else if (AggregationKind.SHARED_LITERAL.equals(aggregationKind))
        {
            srcDecor = createAggregationDecoration();
        }
        else if (isNavigable)
        {
            srcDecor = createNavigableDecoration();
        }
        else
        {
            srcDecor = null;
        }
        ((PolylineConnection) getFigure()).setSourceDecoration(srcDecor);
    }

    /**
     * Update the target decoration
     */
    private void updateTargetDecoration()
    {
        Association association = (Association) Utils.getElement(getGraphEdge());
        AssociationHelper associationHelper = new AssociationHelper(association);

        boolean isNavigable = associationHelper.getAssociationSecondEndIsNavigable();
        AggregationKind aggregationKind = associationHelper.getAssociationFirstEndAggregationKind();

        if (AggregationKind.COMPOSITE_LITERAL.equals(aggregationKind))
        {
            targetDecor = createCompositionDecoration();
        }
        else if (AggregationKind.SHARED_LITERAL.equals(aggregationKind))
        {
            targetDecor = createAggregationDecoration();
        }
        else if (isNavigable)
        {
            targetDecor = createNavigableDecoration();
        }
        else
        {
            targetDecor = null;
        }
        ((PolylineConnection) getFigure()).setTargetDecoration(targetDecor);
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphEdgeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {

        Object notifier = msg.getNotifier();
        Object newObject = msg.getNewValue();
        Object oldObject = msg.getOldValue();

        if (notifier instanceof Association)
        {
            if (notifier == getEObject())
            {
                switch (msg.getFeatureID(Association.class))
                {
                    case UMLPackage.ASSOCIATION__MEMBER_END:
                    case UMLPackage.ASSOCIATION__EANNOTATIONS:
                        updateModelListening(oldObject, newObject);
                        break;
                    default:
                        break;
                }
                super.handleModelChanged(msg);
            }
        }
        else if (notifier instanceof Property)
        {
            switch (msg.getFeatureID(Property.class))
            {
                case UMLPackage.PROPERTY__AGGREGATION:
                    updateSourceDecoration();
                    updateTargetDecoration();
                    break;
                case UMLPackage.PROPERTY__VISIBILITY:
                case UMLPackage.PROPERTY__IS_DERIVED:
                case UMLPackage.PROPERTY__NAME:
                case UMLPackage.PROPERTY__IS_READ_ONLY:
                case UMLPackage.PROPERTY__IS_DERIVED_UNION:
                case UMLPackage.PROPERTY__SUBSETTED_PROPERTY:
                case UMLPackage.PROPERTY__REDEFINED_PROPERTY:
                case UMLPackage.PROPERTY__IS_ORDERED:
                case UMLPackage.PROPERTY__IS_UNIQUE:
                    updateSrcNameLabel();
                    updateTargetNameLabel();
                    break;
                case UMLPackage.PROPERTY__LOWER_VALUE:
                case UMLPackage.PROPERTY__UPPER_VALUE:
                case UMLPackage.PROPERTY__DEFAULT_VALUE:
                    updateModelListening(oldObject, newObject);
                    break;
                default:
                    break;
            }
        }
        else if (notifier instanceof EAnnotation)
        {
            updateMiddleNameLabel();
        }
        else if (notifier instanceof ValueSpecification)
        {
            updateSrcNameLabel();
            updateTargetNameLabel();
            updateSrcCountLabel();
            updateTargetCountLabel();
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
     */
    public void activate()
    {
        super.activate();
        EObject model = getEObject();
        if (model instanceof Association)
        {
            listenProperties((Association) model);
            listenEAnnotations((Association) model);
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
     */
    public void deactivate()
    {
        EObject model = getEObject();
        if (model instanceof Association)
        {
            unlistenEAnnotations((Association) model);
            unlistenProperties((Association) model);
        }
        super.deactivate();
    }

    /**
     * Listen to all the properties associated with the association
     * 
     * @param association the model object
     */
    private void listenProperties(Association association)
    {
        Iterator it = association.getMemberEnds().iterator();
        while (it.hasNext())
        {
            Property prop = (Property) it.next();

            // Only listen to object that are not yet listened
            if (!prop.eAdapters().contains(getModelListener()))
            {
                prop.eAdapters().add(getModelListener());
            }

            listenValues(prop);
        }
    }

    /**
     * Stop listening to all the redefined operations of the operation
     * 
     * @param association the model object
     */
    private void unlistenProperties(Association association)
    {
        Iterator it = association.getMemberEnds().iterator();
        while (it.hasNext())
        {
            Property prop = (Property) it.next();

            unlistenValues(prop);

            prop.eAdapters().remove(getModelListener());
        }
    }

    /**
     * Listen to the bounds of the property
     * 
     * @param property the model object
     */
    private void listenValues(Property property)
    {
        // Only listen to object that are not yet listened
        if (property.getLowerValue() != null && !property.getLowerValue().eAdapters().contains(getModelListener()))
        {
            property.getLowerValue().eAdapters().add(getModelListener());
        }
        if (property.getUpperValue() != null && !property.getUpperValue().eAdapters().contains(getModelListener()))
        {
            property.getUpperValue().eAdapters().add(getModelListener());
        }
        if (property.getDefaultValue() != null && !property.getDefaultValue().eAdapters().contains(getModelListener()))
        {
            property.getDefaultValue().eAdapters().add(getModelListener());
        }
    }

    /**
     * Stop listening to the bounds of the property
     * 
     * @param property the model object
     */
    private void unlistenValues(Property property)
    {
        if (property.getLowerValue() != null)
        {
            property.getLowerValue().eAdapters().remove(getModelListener());
        }
        if (property.getUpperValue() != null)
        {
            property.getUpperValue().eAdapters().remove(getModelListener());
        }
        if (property.getDefaultValue() != null)
        {
            property.getDefaultValue().eAdapters().remove(getModelListener());
        }
    }

    /**
     * Listen to all the EAnnotations associated with the association
     * 
     * @param association the model object
     */
    private void listenEAnnotations(Association association)
    {
        Iterator it = association.getEAnnotations().iterator();
        while (it.hasNext())
        {
            EAnnotation eAnnot = (EAnnotation) it.next();

            // Only listen to object that are not yet listened
            if (!eAnnot.eAdapters().contains(getModelListener()))
            {
                eAnnot.eAdapters().add(getModelListener());
            }
        }
    }

    /**
     * Stop listening to all the EAnnotations of the association
     * 
     * @param association the model object
     */
    private void unlistenEAnnotations(Association association)
    {
        Iterator it = association.getEAnnotations().iterator();
        while (it.hasNext())
        {
            EAnnotation eAnnot = (EAnnotation) it.next();

            eAnnot.eAdapters().remove(getModelListener());
        }
    }
}