/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.classdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.gef.EditPolicy;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Element;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.uml.UMLTools;
import org.topcased.modeler.uml.classdiagram.figures.ClassFigure;
import org.topcased.modeler.uml.classdiagram.policies.ClassLayoutEditPolicy;
import org.topcased.modeler.utils.Utils;

/**
 * The Class element associated with an AssociationClass object controller
 */
public class ClassFromAssociationClassEditPart extends EMFGraphNodeEditPart
{
    /**
     * Constructor
     * 
     * @param obj the graph node
     */
    public ClassFromAssociationClassEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(PositionConstants.EAST_WEST);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new ClassLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {
        return new ClassFigure();
    }

    /**
     * Set the name of Class with the stereotypes and the "from"
     * 
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        ClassFigure fig = (ClassFigure) getFigure();
        ComposedLabel lbl = (ComposedLabel) fig.getLabel();
        Class clazz = (Class) Utils.getElement(getGraphNode());

        lbl.setPrefix(UMLTools.getStereotypesNotation(clazz));

        if (clazz.getName() != null)
        {
            lbl.setMain(clazz.getName());
        }

        lbl.setSuffix(UMLTools.getFromPackageNotation(clazz, (Element) Utils.getElement((GraphElement) getParent().getModel())));
    }

}