/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.classdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Enumeration;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.UMLTools;
import org.topcased.modeler.uml.classdiagram.commands.EnumerationRestoreConnectionCommand;
import org.topcased.modeler.uml.classdiagram.figures.EnumerationFigure;
import org.topcased.modeler.uml.classdiagram.policies.EnumerationLayoutEditPolicy;
import org.topcased.modeler.utils.Utils;

/**
 * The Enumeration object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class EnumerationEditPart extends ClassifierEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public EnumerationEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new EnumerationRestoreConnectionCommand(getHost());
            }
        });

        ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(PositionConstants.EAST_WEST);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new EnumerationLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new EnumerationFigure();
    }

    /**
     * Set the name of the enumeration, with the stereotype and the "from"
     * 
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        EnumerationFigure fig = (EnumerationFigure) getFigure();
        ComposedLabel lbl = (ComposedLabel) fig.getLabel();
        Enumeration enumeration = (Enumeration) Utils.getElement(getGraphNode());

        String stereotype = UMLTools.getStereotypesNotation(enumeration);
        if (stereotype.length() != 0)
        {
            stereotype += "\n";
        }
        lbl.setPrefix(stereotype + "<<enumeration>>");

        if (enumeration.getName() != null)
        {
            lbl.setMain(enumeration.getName());
        }

        lbl.setSuffix(UMLTools.getFromPackageNotation(enumeration, (Element) Utils.getElement((GraphElement) getParent().getModel())));
    }
}