/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.classdiagram.edit;

import java.util.Iterator;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Constraint;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.ParameterDirectionKind;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.ValueSpecification;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.UMLLabel;
import org.topcased.modeler.uml.classdiagram.commands.OperationRestoreConnectionCommand;
import org.topcased.modeler.uml.classdiagram.figures.OperationFigure;
import org.topcased.modeler.utils.Utils;

/**
 * The Operation object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class OperationEditPart extends EMFGraphNodeEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public OperationEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new OperationRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new OperationFigure();
    }

    /**
     * Set the name of the operation, with its argument and its stereotypes
     * 
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        ComposedLabel lbl = (ComposedLabel) ((ILabelFigure) getFigure()).getLabel();

        Operation operation = (Operation) Utils.getElement(getGraphNode());

        if (operation.getName() != null && operation.getName().length() != 0)
        {
            StringBuffer prefix = new StringBuffer();

            prefix.append(UMLLabel.getStereotypesNotation(operation));

            prefix.append(UMLLabel.getVisibilityNotation(operation.getVisibility()));

            lbl.setPrefix(prefix.toString());

            lbl.setMain(operation.getName());

            lbl.setSuffix(getOperationSuffix(operation));

        }
    }

    /**
     * Compose the suffix of an operation using the UML2 Superstructure specification.<br>
     * 
     * @param operation the operation to format
     * @return the operation suffix
     */
    private String getOperationSuffix(Operation operation)
    {

        // �(� [<parameter-list>] �)� [�:� [<return-type>] �{� <oper-property>
        // [�,� <oper-property>]* �}�]
        StringBuffer suffix = new StringBuffer();

        // �(� [<parameter-list>] �)�
        // <parameter-list> ::= <parameter> [�,�<parameter>]*
        suffix.append('(');

        for (Iterator<Parameter> itParameters = operation.getOwnedParameters().iterator(); itParameters.hasNext();)
        {
            Parameter currentParameter = itParameters.next();
            suffix.append(getParameterText(currentParameter));

            if (itParameters.hasNext())
            {
                suffix.append(", ");
            }

            // Remove the ", " String that may have been unnecessarily added
            if (currentParameter.getDirection().getValue() == ParameterDirectionKind.RETURN && suffix.length() > 1)
            {
                suffix.delete(suffix.length() - 2, suffix.length());
            }
        }

        suffix.append(')');

        // �:� [<return-type>]
        Type returnType = operation.getType();
        if (returnType != null)
        {
            suffix.append(" : ");
            suffix.append(returnType.getName());

            // �{� <oper-property> [�,� <oper-property>]* �}�
            // <oper-property> ::= �redefines� <oper-name> | �query� | �ordered�
            // | �unique� | <oper-constraint>
            String propertiesText = getOperationPropertiesText(operation);
            if (propertiesText != null && !"".equals(propertiesText))
            {
                suffix.append(" { ");
                suffix.append(propertiesText);
                suffix.append(" }");
            }
        }

        return suffix.toString();
    }

    private String getOperationPropertiesText(Operation operation)
    {
        // <oper-property> [�,� <oper-property>]*
        // <oper-property> ::= �redefines� <oper-name> | �query� | �ordered� |
        // �unique� | <oper-constraint>
        StringBuffer text = new StringBuffer();

        append(text, getRedefinesText(operation));
        append(text, getQueryText(operation));
        append(text, getOrderedText(operation));
        append(text, getUniqueText(operation));
        append(text, getConstraintText(operation));

        return text.toString();
    }

    private void append(StringBuffer text, String fragment)
    {
        if (fragment != null && !"".equals(fragment))
        {
            if (text.length() > 0)
            {
                text.append(", ");
            }
            text.append(fragment);
        }
    }

    private String getRedefinesText(Operation operation)
    {
        StringBuffer text = new StringBuffer();

        if (operation.getRedefinedOperations() != null)
        {
            Iterator itRedifines = operation.getRedefinedOperations().iterator();
            while (itRedifines.hasNext())
            {
                Operation redefinedOp = (Operation) itRedifines.next();
                text.append("redefines ");
                text.append(redefinedOp.getName());

                if (itRedifines.hasNext())
                {
                    text.append(", ");
                }
            }
        }

        return text.toString();
    }

    private String getConstraintText(Operation operation)
    {
        StringBuffer text = new StringBuffer();

        if (operation.getBodyCondition() != null)
        {
            text.append(operation.getBodyCondition().getName());
        }

        return text.toString();
    }

    private String getUniqueText(Operation operation)
    {
        StringBuffer text = new StringBuffer();
        if (operation.getReturnResult() != null)
        {
            Parameter returnResult = operation.getReturnResult();
            if (returnResult.isMultivalued() && returnResult.isUnique())
            {
                text.append("unique");
            }
        }

        return text.toString();
    }

    private String getOrderedText(Operation operation)
    {
        StringBuffer text = new StringBuffer();

        if (operation.isOrdered())
        {
            text.append("ordered");
        }

        return text.toString();
    }

    private String getQueryText(Operation operation)
    {
        StringBuffer text = new StringBuffer();

        if (operation.isQuery())
        {
            text.append("query");
        }

        return text.toString();
    }

    private String getParameterText(Parameter param)
    {
        StringBuffer text = new StringBuffer();

        // [<direction>] ::= �in� | �out� | �inout� (defaults to �in� if
        // omitted).
        ParameterDirectionKind direction = param.getDirection();

        // Do not display the ReturnType parameter in the list of parameters of
        // the Operation
        if (ParameterDirectionKind.RETURN != direction.getValue())
        {
            if (ParameterDirectionKind.IN != direction.getValue())
            {
                text.append(direction.getName());
                text.append(" ");
            }
            // <parameter-name>
            text.append(param.getName());
            // �:�
            text.append(" : ");
            // <type-expression>
            Type paramType = param.getType();
            if (paramType != null)
            {
                text.append(paramType.getName());
            }
            else
            {
                text.append("null");
            }
            // [�[�<multiplicity>�]�]
            text.append(UMLLabel.getMultiplicityText(param));

            // [�=� <default>]
            String defaultValue = param.getDefault();
            if (defaultValue != null && !"".equals(defaultValue))
            {
                text.append(" = ");
                text.append(defaultValue);
            }
            // [�{� <parm-property> [�,� <parm-property>]* �}�]
            // TODO
        }

        return text.toString();
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {

        Object notifier = msg.getNotifier();
        Object newObject = msg.getNewValue();
        Object oldObject = msg.getOldValue();

        if (notifier instanceof Operation)
        {
            if (notifier == getEObject())
            {
                switch (msg.getFeatureID(Operation.class))
                {
                    case UMLPackage.OPERATION__OWNED_PARAMETER:
                    case UMLPackage.OPERATION__REDEFINED_OPERATION:
                    case UMLPackage.OPERATION__BODY_CONDITION:
                        updateModelListening(oldObject, newObject);
                        break;
                    default:
                        break;
                }
                super.handleModelChanged(msg);
            }
            else
            {
                refreshHeaderLabel();
            }
        }
        else if (notifier instanceof Parameter)
        {
            switch (msg.getFeatureID(Parameter.class))
            {
                case UMLPackage.PARAMETER__TYPE:
                case UMLPackage.PARAMETER__LOWER_VALUE:
                case UMLPackage.PARAMETER__UPPER_VALUE:
                case UMLPackage.PARAMETER__DEFAULT_VALUE:
                    updateModelListening(oldObject, newObject);
                    break;
                default:
                    break;
            }
            refreshHeaderLabel();
        }
        else if (notifier instanceof Type)
        {
            switch (msg.getFeatureID(NamedElement.class))
            {
                case UMLPackage.NAMED_ELEMENT__NAME:
                    refreshHeaderLabel();
                    break;
                default:
                    break;
            }
        }
        else if (notifier instanceof Constraint)
        {
            refreshHeaderLabel();
        }
        else if (notifier instanceof ValueSpecification)
        {
            refreshHeaderLabel();
        }

    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
     */
    public void activate()
    {
        super.activate();
        EObject model = getEObject();
        if (model instanceof Operation)
        {
            listenParameters((Operation) getEObject());
            listenRedefinedOperations((Operation) getEObject());
            listenBodyCondition((Operation) getEObject());
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
     */
    public void deactivate()
    {
        EObject model = getEObject();
        if (model instanceof Operation)
        {
            unlistenBodyCondition((Operation) getEObject());
            unlistenRedefinedOperations((Operation) getEObject());
            unlistenParameters((Operation) getEObject());
        }
        super.deactivate();
    }

    /**
     * Listen to the constraint of the operation
     * 
     * @param operation the model object
     */
    private void listenBodyCondition(Operation operation)
    {
        // Only listen to object that are not yet listened
        if (operation.getBodyCondition() != null && !operation.getBodyCondition().eAdapters().contains(getModelListener()))
        {
            operation.getBodyCondition().eAdapters().add(getModelListener());
        }
    }

    /**
     * Stop listening to the type of the parameter
     * 
     * @param operation the model object
     */
    private void unlistenBodyCondition(Operation operation)
    {
        if (operation.getBodyCondition() != null)
        {
            operation.getBodyCondition().eAdapters().remove(getModelListener());
        }
    }

    /**
     * Listen to all the redefined operations of the operation
     * 
     * @param operation the model object
     */
    private void listenRedefinedOperations(Operation operation)
    {
        Iterator it = operation.getRedefinedOperations().iterator();
        while (it.hasNext())
        {
            Operation redifinedOp = (Operation) it.next();

            // Only listen to object that are not yet listened
            if (!redifinedOp.eAdapters().contains(getModelListener()))
            {
                redifinedOp.eAdapters().add(getModelListener());
            }
        }
    }

    /**
     * Stop listening to all the redefined operations of the operation
     * 
     * @param operation the model object
     */
    private void unlistenRedefinedOperations(Operation operation)
    {
        Iterator it = operation.getRedefinedOperations().iterator();
        while (it.hasNext())
        {
            Operation redifinedOp = (Operation) it.next();

            redifinedOp.eAdapters().remove(getModelListener());
        }
    }

    /**
     * Listen to all the parameters of the operation
     * 
     * @param operation the model object
     */
    private void listenParameters(Operation operation)
    {
        Iterator it = operation.getOwnedParameters().iterator();
        while (it.hasNext())
        {
            Parameter param = (Parameter) it.next();

            // Only listen to object that are not yet listened
            if (!param.eAdapters().contains(getModelListener()))
            {
                param.eAdapters().add(getModelListener());
            }
            listenType(param);
            listenValues(param);
        }
    }

    /**
     * Stop listening to all the parameters of the operation
     * 
     * @param operation the model object
     */
    private void unlistenParameters(Operation operation)
    {
        Iterator it = operation.getOwnedParameters().iterator();
        while (it.hasNext())
        {
            Parameter param = (Parameter) it.next();

            unlistenValues(param);
            unlistenType(param);
            param.eAdapters().remove(getModelListener());
        }
    }

    /**
     * Listen to the type of the parameter
     * 
     * @param param the model object
     */
    private void listenType(Parameter param)
    {
        // Only listen to object that are not yet listened
        if (param.getType() != null && !param.getType().eAdapters().contains(getModelListener()))
        {
            param.getType().eAdapters().add(getModelListener());
        }
    }

    /**
     * Stop listening to the type of the parameter
     * 
     * @param param the model object
     */
    private void unlistenType(Parameter param)
    {
        if (param.getType() != null)
        {
            param.getType().eAdapters().remove(getModelListener());
        }
    }

    /**
     * Listen to the bounds of the parameter
     * 
     * @param param the model object
     */
    private void listenValues(Parameter param)
    {
        // Only listen to object that are not yet listened
        if (param.getLowerValue() != null && !param.getLowerValue().eAdapters().contains(getModelListener()))
        {
            param.getLowerValue().eAdapters().add(getModelListener());
        }
        if (param.getUpperValue() != null && !param.getUpperValue().eAdapters().contains(getModelListener()))
        {
            param.getUpperValue().eAdapters().add(getModelListener());
        }
        if (param.getDefaultValue() != null && !param.getDefaultValue().eAdapters().contains(getModelListener()))
        {
            param.getDefaultValue().eAdapters().add(getModelListener());
        }
    }

    /**
     * Stop listening to the bounds of the parameter
     * 
     * @param param the model object
     */
    private void unlistenValues(Parameter param)
    {
        if (param.getLowerValue() != null)
        {
            param.getLowerValue().eAdapters().remove(getModelListener());
        }
        if (param.getUpperValue() != null)
        {
            param.getUpperValue().eAdapters().remove(getModelListener());
        }
        if (param.getDefaultValue() != null)
        {
            param.getDefaultValue().eAdapters().remove(getModelListener());
        }
    }
}