/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.commands;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Extension;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.topcased.modeler.uml.dialogs.DefineProfileDialog;
import org.topcased.modeler.uml.dialogs.StereotypeStructure;

/**
 * TODO completly refactor and redesign this class Class that create a command in order to update profile settings <br>
 * creation : 31 may 2005
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 */
public class DefineProfileCommand extends Command
{
    /** Current profile */
    private Profile profile;

    /** Old values */
    private Map oldDatas;

    /** New values */
    private Map datas;

    /**
     * Create a command for updating parameters on a given profile
     * 
     * @param prof the profile
     * @param data the new datas
     * @param oldData the old datas
     */
    public DefineProfileCommand(Profile prof, Map data, Map oldData)
    {
        profile = prof;
        datas = data;
        oldDatas = oldData;
    }

    /**
     * Set the values
     */
    protected void setValues()
    {
        // Perform update
        profile.setName((String) datas.get(DefineProfileDialog.PROFILE_NAME));

        // Remove imported classes
        // TODO check if the following code line is clean
        profile.getElementImports().clear();

        // Look for properties that handle extension
        ArrayList propertyToRemove = new ArrayList();
        Iterator itStereotype = profile.getOwnedStereotypes().iterator();
        while (itStereotype.hasNext())
        {
            Stereotype stereotype = (Stereotype) itStereotype.next();
            Iterator itAttribute = stereotype.getOwnedAttributes().iterator();
            while (itAttribute.hasNext())
            {
                Property property = (Property) itAttribute.next();
                if (property.getAssociation() instanceof Extension)
                {
                    propertyToRemove.add(property);
                }
            }
        }

        // Remove the found properties and extension associated to
        Iterator itProperty = propertyToRemove.iterator();
        while (itProperty.hasNext())
        {
            Property property = (Property) itProperty.next();
            if (property.getAssociation() != null)
            {
                property.getAssociation().destroy();
            }
            property.destroy();
        }

        // Update each stereotype
        Iterator itValues = datas.values().iterator();
        while (itValues.hasNext())
        {
            Object object = itValues.next();
            if (object instanceof StereotypeStructure)
            {
                StereotypeStructure stSt = (StereotypeStructure) object;
                Stereotype stereotype = stSt.getStereotype();

                // update name
                stereotype.setName(stSt.getNewName());

                // update metaclass
                for (Iterator metaIterator = stSt.getMetaclasses().iterator(); metaIterator.hasNext();)
                {
                    Class metaclass = (Class) metaIterator.next();
                    if (!profile.getMetaclassReferences().contains(metaclass))
                    {
                        profile.createMetaclassReference(metaclass);
                    }
                    stereotype.createExtension(metaclass, false);
                }
            }
        }

        profile.define();
    }

    /**
     * Switch the old and new values
     */
    protected void switchValues()
    {
        Map temp = datas;
        datas = oldDatas;
        oldDatas = temp;
    }

    /**
     * Set the values
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        setValues();
    }

    /**
     * set the old values
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        switchValues();
        setValues();
    }

    /**
     * Set the new values
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        switchValues();
        setValues();
    }
}
