/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.componentdiagram;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.editor.GraphElementCreationFactory;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.palette.ModelerConnectionCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerPaletteManager;

/**
 * Generated Palette Manager <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ComponentPaletteManager extends ModelerPaletteManager
{
    // declare all the palette categories of the diagram
    private PaletteDrawer objectsDrawer;

    private PaletteDrawer connectionsDrawer;

    private PaletteDrawer commentDrawer;

    private ICreationUtils creationUtils;

    /**
     * The Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param utils the creation utils for the tools of the palette
     * @generated
     */
    public ComponentPaletteManager(ICreationUtils utils)
    {
        super();
        this.creationUtils = utils;
    }

    /**
     * Creates the main categories of the palette <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createCategories()
    {
        createObjectsDrawer();
        createConnectionsDrawer();
        createCommentDrawer();
    }

    /**
     * Updates the main categories of the palette <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void updateCategories()
    {
        // deletion of the existing categories and creation of the updated
        // categories

        getRoot().remove(objectsDrawer);
        createObjectsDrawer();

        getRoot().remove(connectionsDrawer);
        createConnectionsDrawer();

        getRoot().remove(commentDrawer);
        createCommentDrawer();
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private void createObjectsDrawer()
    {
        objectsDrawer = new PaletteDrawer("Objects", null);
        List<PaletteEntry> entries = new ArrayList<PaletteEntry>();

        CreationFactory factory;

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getComponent(), "default");
        entries.add(new ModelerCreationToolEntry("Component", "Component", factory, ComponentImageRegistry.getImageDescriptor("COMPONENT"),
                ComponentImageRegistry.getImageDescriptor("COMPONENT_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getInterface(), "default");
        entries.add(new ModelerCreationToolEntry("Interface", "Interface", factory, ComponentImageRegistry.getImageDescriptor("INTERFACE"),
                ComponentImageRegistry.getImageDescriptor("INTERFACE_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getPort(), "default")
        {
            public EObject getNewModelObject()
            {
                Port element = (Port) super.getNewModelObject();
                // A Port should always be a Composite
                element.setAggregation(AggregationKind.COMPOSITE_LITERAL);
                return element;
            }
        };
        entries.add(new ModelerCreationToolEntry("Port", "Port", factory, ComponentImageRegistry.getImageDescriptor("PORT"), ComponentImageRegistry.getImageDescriptor("PORT_LARGE")));

        objectsDrawer.addAll(entries);
        getRoot().add(objectsDrawer);
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private void createConnectionsDrawer()
    {
        connectionsDrawer = new PaletteDrawer("Connections", null);
        List<PaletteEntry> entries = new ArrayList<PaletteEntry>();

        CreationFactory factory;

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getComponentRealization(), "default");
        entries.add(new ModelerConnectionCreationToolEntry("Component Realization", "Component Realization", factory, ComponentImageRegistry.getImageDescriptor("COMPONENTREALIZATION"),
                ComponentImageRegistry.getImageDescriptor("COMPONENTREALIZATION_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getInterfaceRealization(), "default");
        entries.add(new ModelerConnectionCreationToolEntry("Interface Realization", "Interface Realization", factory, ComponentImageRegistry.getImageDescriptor("INTERFACEREALIZATION"),
                ComponentImageRegistry.getImageDescriptor("INTERFACEREALIZATION_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getUsage(), "default");
        entries.add(new ModelerConnectionCreationToolEntry("Usage", "Usage", factory, ComponentImageRegistry.getImageDescriptor("USAGE"), ComponentImageRegistry.getImageDescriptor("USAGE_LARGE")));

        connectionsDrawer.addAll(entries);
        getRoot().add(connectionsDrawer);
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private void createCommentDrawer()
    {
        commentDrawer = new PaletteDrawer("Comment", null);
        List<PaletteEntry> entries = new ArrayList<PaletteEntry>();

        CreationFactory factory;

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getComment(), "default");
        entries.add(new ModelerCreationToolEntry("Comment", "Comment", factory, ComponentImageRegistry.getImageDescriptor("COMMENT"), ComponentImageRegistry.getImageDescriptor("COMMENT_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, ComponentSimpleObjectConstants.SIMPLE_OBJECT_COMMENTLINK, "default", false);
        entries.add(new ModelerConnectionCreationToolEntry("Comment Link", "Comment Link", factory, ComponentImageRegistry.getImageDescriptor("COMMENTLINK"),
                ComponentImageRegistry.getImageDescriptor("COMMENTLINK_LARGE")));

        commentDrawer.addAll(entries);
        getRoot().add(commentDrawer);
    }

}
