/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.componentdiagram.commands;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.Component;
import org.eclipse.uml2.uml.ComponentRealization;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.Usage;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.uml.alldiagram.commands.ElementRestoreConnectionCommand;
import org.topcased.modeler.utils.Utils;

/**
 * Component restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class ComponentRestoreConnectionCommand extends ElementRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public ComponentRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated NOT
     */
    protected void initializeCommands()
    {
        super.initializeCommands();

        GraphElement node = getGraphElement();
        EObject nodeObject = Utils.getElement(node);

        if (nodeObject instanceof Component)
        {
            for (Object obj : node.eContainer().eContents())
            {
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(node);
                    GraphElement node2 = (GraphElement) obj;
                    EObject nodeObject2 = Utils.getElement(node2);

                    if (nodeObject2 instanceof Component && !autoRef)
                    {
                        createComponentRealizationFromComponentToComponent(node, node2);
                        createComponentRealizationFromComponentToComponent(node2, node);
                    }

                    if (nodeObject2 instanceof Interface && !autoRef)
                    {
                        createInterfaceRealizationFromComponentToInterface(node, node2);
                        createUsageFromComponentToInterface(node, node2);
                    }

                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createComponentRealizationFromComponentToComponent(GraphElement srcNode, GraphElement targetNode)
    {
        Component sourceObject = (Component) Utils.getElement(srcNode);
        Component targetObject = (Component) Utils.getElement(targetNode);

        for (ComponentRealization edgeObject : targetObject.getRealizations())
        {
            if (sourceObject.equals(edgeObject.getRealizingClassifier()))
            {
                // check if the relation does not exists yet
                List<GraphEdge> existing = getExistingEdges(srcNode, targetNode, ComponentRealization.class);
                if (!isAlreadyPresent(existing, edgeObject))
                {
                    ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                    // restore the link with its default presentation
                    GraphElement edge = factory.createGraphElement(edgeObject);
                    if (edge instanceof GraphEdge)
                    {
                        ComponentRealizationEdgeCreationCommand cmd = new ComponentRealizationEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                        cmd.setTarget(targetNode);
                        add(cmd);
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createInterfaceRealizationFromComponentToInterface(GraphElement srcNode, GraphElement targetNode)
    {
        Component sourceObject = (Component) Utils.getElement(srcNode);
        Interface targetObject = (Interface) Utils.getElement(targetNode);

        for (InterfaceRealization edgeObject : sourceObject.getInterfaceRealizations())
        {
            if (targetObject.equals(edgeObject.getContract()))
            {
                // check if the relation does not exists yet
                List<GraphEdge> existing = getExistingEdges(srcNode, targetNode, InterfaceRealization.class);
                if (!isAlreadyPresent(existing, edgeObject))
                {
                    ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                    // restore the link with its default presentation
                    GraphElement edge = factory.createGraphElement(edgeObject);
                    if (edge instanceof GraphEdge)
                    {
                        InterfaceRealizationEdgeCreationCommand cmd = new InterfaceRealizationEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                        cmd.setTarget(targetNode);
                        add(cmd);
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createUsageFromComponentToInterface(GraphElement srcNode, GraphElement targetNode)
    {
        Component sourceObject = (Component) Utils.getElement(srcNode);
        Interface targetObject = (Interface) Utils.getElement(targetNode);

        for (Object obj : sourceObject.getPackagedElements())
        {
            if (obj instanceof Usage)
            {
                Usage edgeObject = (Usage) obj;
                if (edgeObject.getSuppliers().contains(targetObject) && edgeObject.getClients().contains(sourceObject))
                {
                    // check if the relation does not exists yet
                    List<GraphEdge> existing = getExistingEdges(srcNode, targetNode, Usage.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            UsageEdgeCreationCommand cmd = new UsageEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                            cmd.setTarget(targetNode);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

}