/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.componentdiagram.commands;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.Port;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.di.model.DiagramInterchangeFactory;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.uml.componentdiagram.ComponentSimpleObjectConstants;
import org.topcased.modeler.utils.Utils;

/**
 * InterfaceRealization edge creation command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class InterfaceRealizationEdgeCreationCommand extends CreateTypedEdgeCommand
{

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param domain the edit domain
     * @param newObj the graph edge of the new connection
     * @param src the graph element of the source
     * @generated
     */
    public InterfaceRealizationEdgeCreationCommand(EditDomain domain, GraphEdge newObj, GraphElement src)
    {
        this(domain, newObj, src, true);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param domain the edit domain
     * @param newObj the graph edge of the new connection
     * @param src the graph element of the source
     * @param needModelUpdate set it to true if the model need to be updated
     * @generated
     */
    public InterfaceRealizationEdgeCreationCommand(EditDomain domain, GraphEdge newObj, GraphElement src, boolean needModelUpdate)
    {
        super(domain, newObj, src, needModelUpdate);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void redoModel()
    {
        // To specify that a Port (say P1) "provides" an interface (say I1), two cases :
        // 1. The source Port P1 has no type : we can type it with the interface I1
        // 2. The source Port P1 is already typesd with a Class (say C1). Create an "InterfaceRealization" relationship
        // between C1 and the target Interface I1
        EObject srcElt = Utils.getElement(getSource());
        Interface interfaceElt = (Interface) Utils.getElement(getTarget());
        InterfaceRealization interfaceRealizationElt = (InterfaceRealization) Utils.getElement(getEdge());

        // Case 1
        if (srcElt instanceof Port && ((Port) srcElt).getType() == null)
        {
            ((Port) srcElt).setType(interfaceElt);

            // Update the SemanticModelBridge associated with the InterfaceRealization when it is used only to set the
            // Port type.
            SimpleSemanticModelElement element = DiagramInterchangeFactory.eINSTANCE.createSimpleSemanticModelElement();
            element.setTypeInfo(ComponentSimpleObjectConstants.SIMPLE_OBJECT_INTERFACEREALIZATION);
            element.setPresentation("default");
            getEdge().setSemanticModel(element);
        }
        // Case 2
        else if (srcElt instanceof Port && ((Port) srcElt).getType() instanceof org.eclipse.uml2.uml.Class)
        {
            interfaceRealizationElt.getClients().add(((Port) srcElt).getType());
            interfaceRealizationElt.getSuppliers().add(interfaceElt);
            ((Class) ((Port) srcElt).getType()).getInterfaceRealizations().add(interfaceRealizationElt);
        }
        else
        {
            super.redoModel();
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void undoModel()
    {
        EObject srcElt = Utils.getElement(getSource());
        Interface interfaceElt = (Interface) Utils.getElement(getTarget());
        InterfaceRealization interfaceRealizationElt = (InterfaceRealization) Utils.getElement(getEdge());

        // Case 1
        if (srcElt instanceof Port && ((Port) srcElt).getType() == interfaceElt)
        {
            ((Port) srcElt).setType(null);
        }
        // Case 2
        else if (srcElt instanceof Port && ((Port) srcElt).getType() instanceof org.eclipse.uml2.uml.Class)
        {
            interfaceRealizationElt.getClients().remove(((Port) srcElt).getType());
            interfaceRealizationElt.getSuppliers().remove(interfaceElt);
            ((Class) ((Port) srcElt).getType()).getInterfaceRealizations().remove(interfaceRealizationElt);
        }
        else
        {
            super.undoModel();
        }
    }

}