/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.componentdiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Usage;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.uml.alldiagram.commands.ElementRestoreConnectionCommand;
import org.topcased.modeler.uml.componentdiagram.ComponentSimpleObjectConstants;
import org.topcased.modeler.utils.Utils;

/**
 * Port restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class PortRestoreConnectionCommand extends ElementRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public PortRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated NOT
     */
    protected void initializeCommands()
    {
        super.initializeCommands();

        GraphElement node = getGraphElement();
        EObject nodeObject = Utils.getElement(node);

        if (nodeObject instanceof Port)
        {
            Iterator<EObject> itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(node);
                    GraphElement node2 = (GraphElement) obj;
                    EObject nodeObject2 = Utils.getElement(node2);

                    if (nodeObject2 instanceof Interface && !autoRef)
                    {
                        createInterfaceRealizationFromPortToInterface(node, node2);
                        createUsageFromPortToInterface(node, node2);
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createInterfaceRealizationFromPortToInterface(GraphElement srcNode, GraphElement targetNode)
    {
        Port sourceObject = (Port) Utils.getElement(srcNode);
        Interface targetObject = (Interface) Utils.getElement(targetNode);

        if (targetObject.equals(sourceObject.getType()))
        {
            // check if the relation does not exists yet
            if (getExistingEdges(srcNode, targetNode, ComponentSimpleObjectConstants.SIMPLE_OBJECT_INTERFACEREALIZATION).size() == 0)
            {
                GraphEdge edge = Utils.createGraphEdge(ComponentSimpleObjectConstants.SIMPLE_OBJECT_INTERFACEREALIZATION);
                InterfaceRealizationEdgeCreationCommand cmd = new InterfaceRealizationEdgeCreationCommand(null, edge, srcNode, false);
                cmd.setTarget(targetNode);
                add(cmd);
            }
        }
        else if (sourceObject.getType() instanceof org.eclipse.uml2.uml.Class)
        {
            for (InterfaceRealization edgeObject : ((org.eclipse.uml2.uml.Class) sourceObject.getType()).getInterfaceRealizations())
            {
                if (edgeObject.getSuppliers().contains(targetObject) && edgeObject.getClients().contains(sourceObject.getType()))
                {
                    // check if the relation does not exists yet
                    List<GraphEdge> existing = getExistingEdges(srcNode, targetNode, InterfaceRealization.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            InterfaceRealizationEdgeCreationCommand cmd = new InterfaceRealizationEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                            cmd.setTarget(targetNode);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createUsageFromPortToInterface(GraphElement srcNode, GraphElement targetNode)
    {
        Port sourceObject = (Port) Utils.getElement(srcNode);
        Interface targetObject = (Interface) Utils.getElement(targetNode);

        if (sourceObject.getType() instanceof org.eclipse.uml2.uml.Class)
        {
            for (PackageableElement edgeObject : ((org.eclipse.uml2.uml.Class) sourceObject.getType()).getNearestPackage().getPackagedElements())
            {
                if (edgeObject instanceof Usage && ((Usage) edgeObject).getSuppliers().contains(targetObject) && ((Usage) edgeObject).getClients().contains(sourceObject.getType()))
                {
                    // check if the relation does not exists yet
                    List<GraphEdge> existing = getExistingEdges(srcNode, targetNode, Usage.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            UsageEdgeCreationCommand cmd = new UsageEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                            cmd.setTarget(targetNode);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

}