/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.componentdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Element;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.UMLLabel;
import org.topcased.modeler.uml.alldiagram.edit.ElementEditPart;
import org.topcased.modeler.uml.componentdiagram.ComponentEditPolicyConstants;
import org.topcased.modeler.uml.componentdiagram.commands.ComponentRestoreConnectionCommand;
import org.topcased.modeler.uml.componentdiagram.figures.ComponentFigure;
import org.topcased.modeler.uml.componentdiagram.policies.ComponentLayoutEditPolicy;
import org.topcased.modeler.uml.componentdiagram.policies.ComponentRealizationEdgeCreationEditPolicy;
import org.topcased.modeler.uml.componentdiagram.policies.InterfaceRealizationEdgeCreationEditPolicy;
import org.topcased.modeler.uml.componentdiagram.policies.UsageEdgeCreationEditPolicy;

/**
 * The Component object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ComponentEditPart extends ElementEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public ComponentEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ComponentEditPolicyConstants.COMPONENTREALIZATION_EDITPOLICY, new ComponentRealizationEdgeCreationEditPolicy());

        installEditPolicy(ComponentEditPolicyConstants.INTERFACEREALIZATION_EDITPOLICY, new InterfaceRealizationEdgeCreationEditPolicy());

        installEditPolicy(ComponentEditPolicyConstants.USAGE_EDITPOLICY, new UsageEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new ComponentRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new ComponentLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new ComponentFigure();
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getAnchorableFigure()
     */
    protected IFigure getAnchorableFigure()
    {
        return ((ComponentFigure) getFigure()).getBackgroundFigure();
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getEditableLabel()
     */
    public ILabel getEditableLabel()
    {
        return ((ComposedLabel) ((ComponentFigure) getFigure()).getLabel()).getMain();
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        ComposedLabel editableLbl = (ComposedLabel) ((ComponentFigure) getFigure()).getLabel();
        editableLbl.setPrefix(UMLLabel.getStereotypesNotation((Element) getEObject()));
        super.refreshHeaderLabel();
    }

}