/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.componentdiagram.edit;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.EllipseAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Usage;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.EMFSemanticModelBridge;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.NonResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.UMLLabel;
import org.topcased.modeler.uml.alldiagram.edit.ElementEditPart;
import org.topcased.modeler.uml.componentdiagram.ComponentEditPolicyConstants;
import org.topcased.modeler.uml.componentdiagram.ComponentSimpleObjectConstants;
import org.topcased.modeler.uml.componentdiagram.commands.InterfaceRestoreConnectionCommand;
import org.topcased.modeler.uml.componentdiagram.figures.InterfaceFigure;
import org.topcased.modeler.uml.componentdiagram.policies.InterfaceRealizationEdgeCreationEditPolicy;
import org.topcased.modeler.uml.componentdiagram.policies.UsageEdgeCreationEditPolicy;
import org.topcased.modeler.utils.Utils;

/**
 * The Interface object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class InterfaceEditPart extends ElementEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public InterfaceEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ComponentEditPolicyConstants.INTERFACEREALIZATION_EDITPOLICY, new InterfaceRealizationEdgeCreationEditPolicy());

        installEditPolicy(ComponentEditPolicyConstants.USAGE_EDITPOLICY, new UsageEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new InterfaceRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new NonResizableEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated NOT
     */
    protected IFigure createFigure()
    {
        InterfaceFigure interfaceFigure = new InterfaceFigure();
        interfaceFigure.setDisplayBodyFigure(!isOnlyRequiredInterface());
        return interfaceFigure;
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getEditableLabel()
     */
    public ILabel getEditableLabel()
    {
        return ((ComposedLabel) ((InterfaceFigure) getFigure()).getLabel()).getMain();
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        ComposedLabel editableLbl = (ComposedLabel) ((InterfaceFigure) getFigure()).getLabel();
        editableLbl.setPrefix(UMLLabel.getStereotypesNotation((Element) getEObject()));
        super.refreshHeaderLabel();
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getAnchorableFigure()
     */
    protected IFigure getAnchorableFigure()
    {
        return ((InterfaceFigure) getFigure()).getBodyFigure();
    }

    /**
     * Check whether the Interface is only connected to Usage links (at least one). Then the Ellipse of the Figure
     * should not be displayed.
     * 
     * @return true whether the Interface is graphically connected to only Usage links
     */
    private boolean isOnlyRequiredInterface()
    {
        boolean allRequired = true;
        boolean atLeastOneFound = false;

        for (GraphConnector graphConnector : getGraphNode().getAnchorage())
        {
            for (GraphEdge graphEdge : graphConnector.getGraphEdge())
            {
                // Check only the edges associated with a model object
                if (graphEdge.getSemanticModel() instanceof EMFSemanticModelBridge)
                {
                    allRequired = allRequired && (Utils.getElement(graphEdge) instanceof Usage);
                    atLeastOneFound = true;
                }
                else if (graphEdge.getSemanticModel() instanceof SimpleSemanticModelElement &&
                        ComponentSimpleObjectConstants.SIMPLE_OBJECT_INTERFACEREALIZATION.equals(((SimpleSemanticModelElement) graphEdge.getSemanticModel()).getTypeInfo()))
                {
                    allRequired = false;
                }
            }
        }

        return atLeastOneFound && allRequired;
    }

    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#refreshSourceConnections()
     */
    protected void refreshSourceConnections()
    {
        super.refreshSourceConnections();
        ((InterfaceFigure) getFigure()).setDisplayBodyFigure(!isOnlyRequiredInterface());
    }

    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#refreshTargetConnections()
     */
    protected void refreshTargetConnections()
    {
        super.refreshTargetConnections();
        ((InterfaceFigure) getFigure()).setDisplayBodyFigure(!isOnlyRequiredInterface());
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
     */
    public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection)
    {
        final ConnectionEditPart conn = connection;

        return new EllipseAnchor(getAnchorableFigure())
        {
            public Point getLocation(Point reference)
            {
                Rectangle r = Rectangle.SINGLETON;
                r.setBounds(getOwner().getBounds());
                r.translate(-1, -1);
                r.resize(1, 1);
                // Provide a space when the connection is an UsageEditPart
                if (conn instanceof UsageEditPart)
                {
                    r.expand(4, 4);
                }
                getOwner().translateToAbsolute(r);

                Point ref = r.getCenter().negate().translate(reference);

                if (ref.x == 0)
                {
                    return new Point(reference.x, (ref.y > 0) ? r.bottom() : r.y);
                }
                if (ref.y == 0)
                {
                    return new Point((ref.x > 0) ? r.right() : r.x, reference.y);
                }

                float dx = (ref.x > 0) ? 0.5f : -0.5f;
                float dy = (ref.y > 0) ? 0.5f : -0.5f;

                // ref.x, ref.y, r.width, r.height != 0 => safe to proceed

                float k = (float) (ref.y * r.width) / (ref.x * r.height);
                k = k * k;

                return r.getCenter().translate((int) (r.width * dx / Math.sqrt(1 + k)), (int) (r.height * dy / Math.sqrt(1 + 1 / k)));
            }
        };
    }
}