/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.componentdiagram.figures;

import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutManager;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.draw2d.ToolbarLayout;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.IContainerFigure;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.draw2d.figures.Label;
import org.topcased.draw2d.layout.BorderAttachedLayout;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */
public class ComponentFigure extends Figure implements IContainerFigure, ILabelFigure
{
    private ILabel label;

    private IFigure contentPane;

    private IFigure innerRectangle;

    /**
     * Constructor <!-- begin-user-doc --> Creates the contents of the figure : by defauft, it creates a layout manager,
     * a header and a container <!-- end-user-doc -->
     * 
     * @generated
     */
    public ComponentFigure()
    {
        LayoutManager layout = new StackLayout();
        setLayoutManager(layout);

        add(createBackgroundFigure());

        contentPane = new Figure();
        contentPane.setLayoutManager(new BorderAttachedLayout());
        add(contentPane);
    }

    /**
     * Draw an innerRectangle that will be the real background figure. The Anchor will be attached to this inner
     * rectangle.<br>
     * The innerRectangle is also the container of the HeaderComponent and the Label of the Component that are arranged
     * in a toolbarLayout.
     * 
     * @return the figure
     */
    protected IFigure createBackgroundFigure()
    {
        IFigure rectangle = new Figure();
        rectangle.setLayoutManager(new BorderLayout());
        rectangle.setBorder(new MarginBorder(10));

        innerRectangle = new RectangleFigure();
        innerRectangle.setBorder(new MarginBorder(5));
        innerRectangle.setOpaque(true);
        rectangle.add(innerRectangle, new Integer(PositionConstants.CENTER));

        // add the header figure that support the "<<component>>" text and the
        // component icon in the right hand corner of the innerRectangle
        ToolbarLayout toolbarLayout = new ToolbarLayout();
        toolbarLayout.setMinorAlignment(ToolbarLayout.ALIGN_CENTER);
        toolbarLayout.setSpacing(5);
        innerRectangle.setLayoutManager(toolbarLayout);
        innerRectangle.add(new HeaderComponent());
        label = createLabel();
        innerRectangle.add(label);

        return rectangle;
    }

    /**
     * Returns the figure drawn behind the label. In this case, we only return the inner rectangle drawn.
     * 
     * @return the background figure
     */
    public IFigure getBackgroundFigure()
    {
        return innerRectangle;
    }

    /**
     * Creates the label of the figure.<br>
     * <b>Subclasses can override this method to customize the appearance of the label (for example you can return a
     * {@link ComposedLabel} instead)</b>
     * 
     * @return the label of the figure
     */
    protected ILabel createLabel()
    {
        EditableLabel lbl = new EditableLabel();
        lbl.setLabelAlignment(PositionConstants.LEFT);
        ComposedLabel composedLbl = new ComposedLabel(new Label(), lbl, null, false);
        return composedLbl;
    }

    /**
     * @see org.topcased.draw2d.figures.ILabelFigure#getLabel()
     */
    public ILabel getLabel()
    {
        return label;
    }

    /**
     * @see org.topcased.draw2d.figures.IContainerFigure#getContentPane()
     */
    public IFigure getContentPane()
    {
        return contentPane;
    }

}