/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.componentdiagram.internal.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.RotatableDecoration;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Draw an Half Circle figure that will cover the target Figure
 * 
 * Creation 21 juil. 06
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class HalfCircleDecoration extends Figure implements RotatableDecoration
{
    private static final int LINE_WIDTH = 1;

    private PolylineConnection connection;

    private Point location;

    private double rotation = 0;

    /**
     * Constructor
     * 
     * @param conn The PolylineConnection that is the owner of this Decoration
     */
    public HalfCircleDecoration(PolylineConnection conn)
    {
        connection = conn;
        location = new Point();
    }

    /**
     * @see org.eclipse.draw2d.Figure#getBounds()
     */
    public Rectangle getBounds()
    {
        if (bounds == null)
        {
            bounds = getCircleBounds();
            bounds.expand(LINE_WIDTH, LINE_WIDTH);
        }
        return bounds;
    }

    /**
     * Get the bounds of the Figure on which the Decoration should be attached.
     * 
     * @return Rectangle
     */
    private Rectangle getCircleBounds()
    {
        Rectangle circBounds = connection.getTargetAnchor().getOwner().getBounds().getCopy();
        circBounds.expand(5, 5);
        return circBounds;
    }

    /**
     * @see org.eclipse.draw2d.Figure#setLocation(org.eclipse.draw2d.geometry.Point)
     */
    public void setLocation(Point p)
    {
        bounds = null;
        location.setLocation(p);
        super.setLocation(p);
    }

    /**
     * @see org.eclipse.draw2d.RotatableDecoration#setReferencePoint(org.eclipse.draw2d.geometry.Point)
     */
    public void setReferencePoint(Point p)
    {
        bounds = null;
        Point pt = Point.SINGLETON;
        pt.setLocation(p);
        pt.negate().translate(location);
        setRotation(Math.atan2(pt.y, pt.x));
    }

    /**
     * Set the rotation parameter
     * 
     * @param angle set the angle of the halfCircle
     */
    public void setRotation(double angle)
    {
        this.rotation = angle;
        bounds = null;
    }

    /**
     * @see org.eclipse.draw2d.Figure#paintBorder(org.eclipse.draw2d.Graphics)
     */
    protected void paintBorder(Graphics graphics)
    {
        graphics.drawArc(getCircleBounds().getResized(-LINE_WIDTH, -LINE_WIDTH), (int) (90 - Math.toDegrees(rotation)), 180);
    }
}
