/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram;

import java.net.URL;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.gef.EditPartFactory;
import org.topcased.modeler.editor.IConfiguration;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.IPaletteManager;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.graphconf.exceptions.MissingGraphConfFileException;
import org.topcased.modeler.uml.alldiagram.edit.CommentEditPart;
import org.topcased.modeler.uml.alldiagram.edit.ElementEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.CollaborationUseEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.ConnectorEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.DependencyEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.InterfaceEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.PortEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.PropertyEditPart;

/**
 * A diagram configuration : manages Palette, EditPartFactory for this diagram.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class CompositeStructuresConfiguration implements IConfiguration
{
	/**
	 * @generated
	 */
	private CompositeStructuresPaletteManager paletteManager;

	/**
	 * @generated
	 */
	private CompositeStructuresEditPartFactory editPartFactory;

	/**
	 * @generated
	 */
	private CompositeStructuresCreationUtils creationUtils;

	/**
	 * The DiagramGraphConf that contains graphical informations on the configuration
	 * 
	 * @generated
	 */
	private DiagramGraphConf diagramGraphConf;

	/**
	 * Constructor. Initialize Adapter factories.
	 * 
	 * @generated
	 */
	public CompositeStructuresConfiguration()
	{
		registerAdapters();
	}

	/**
	 * Registers the Adapter Factories for all the EditParts
	 * 
	 * @generated
	 */
	private void registerAdapters()
	{
		Platform.getAdapterManager().registerAdapters(
		        new EditPart2ModelAdapterFactory(ConnectorEditPart.class, org.eclipse.uml2.uml.Connector.class), ConnectorEditPart.class);
		Platform.getAdapterManager()
		        .registerAdapters(new EditPart2ModelAdapterFactory(DependencyEditPart.class, org.eclipse.uml2.uml.Dependency.class),
		                DependencyEditPart.class);
		Platform.getAdapterManager().registerAdapters(
		        new EditPart2ModelAdapterFactory(CommentEditPart.class, org.eclipse.uml2.uml.Comment.class), CommentEditPart.class);
		Platform.getAdapterManager().registerAdapters(
		        new EditPart2ModelAdapterFactory(PortEditPart.class, org.eclipse.uml2.uml.Port.class), PortEditPart.class);
		Platform.getAdapterManager().registerAdapters(
		        new EditPart2ModelAdapterFactory(PropertyEditPart.class, org.eclipse.uml2.uml.Property.class), PropertyEditPart.class);
		Platform.getAdapterManager().registerAdapters(
		        new EditPart2ModelAdapterFactory(InterfaceEditPart.class, org.eclipse.uml2.uml.Interface.class), InterfaceEditPart.class);
		Platform.getAdapterManager().registerAdapters(
		        new EditPart2ModelAdapterFactory(ElementEditPart.class, org.eclipse.uml2.uml.Element.class), ElementEditPart.class);
		Platform.getAdapterManager().registerAdapters(
		        new EditPart2ModelAdapterFactory(CollaborationUseEditPart.class, org.eclipse.uml2.uml.CollaborationUse.class),
		        CollaborationUseEditPart.class);
	}

	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getId()
	 * @generated
	 */
	public String getId()
	{
		return new String("org.topcased.modeler.uml.compositestructuresdiagram.internal");
	}

	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getName()
	 * @generated
	 */
	public String getName()
	{
		return new String("Composite Structures Diagram (Class View)");
	}

	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getEditPartFactory()
	 * @generated
	 */
	public EditPartFactory getEditPartFactory()
	{
		if (editPartFactory == null)
		{
			editPartFactory = new CompositeStructuresEditPartFactory();
		}

		return editPartFactory;
	}

	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getPaletteManager()
	 * @generated
	 */
	public IPaletteManager getPaletteManager()
	{
		if (paletteManager == null)
		{
			paletteManager = new CompositeStructuresPaletteManager(getCreationUtils());
		}

		return paletteManager;
	}

	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getCreationUtils()
	 * @generated
	 */
	public ICreationUtils getCreationUtils()
	{
		if (creationUtils == null)
		{
			creationUtils = new CompositeStructuresCreationUtils(getDiagramGraphConf());
		}

		return creationUtils;
	}

	/**
	 * @see org.topcased.modeler.editor.IConfiguration#getDiagramGraphConf()
	 * @generated
	 */
	public DiagramGraphConf getDiagramGraphConf()
	{
		if (diagramGraphConf == null)
		{
			URL url = CompositeStructuresPlugin.getDefault().getBundle().getResource(
			        "org/topcased/modeler/uml/compositestructuresdiagram/common/diagram.graphconf");
			if (url != null)
			{
				URI fileURI = URI.createURI(url.toString());
				ResourceSet resourceSet = new ResourceSetImpl();
				Resource resource = resourceSet.getResource(fileURI, true);
				if (resource != null && resource.getContents().get(0) instanceof DiagramGraphConf)
				{
					diagramGraphConf = (DiagramGraphConf) resource.getContents().get(0);
				}
			}
			else
			{
				new MissingGraphConfFileException(
				        "The *.diagramgraphconf file can not be retrieved. Check if the path is correct in the Configuration class of your diagram.");
			}
		}

		return diagramGraphConf;
	}

}