/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.CollaborationUse;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Usage;
import org.topcased.modeler.di.model.DiagramInterchangeFactory;
import org.topcased.modeler.di.model.EdgeObjectOffset;
import org.topcased.modeler.di.model.EdgeObjectUV;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.editor.AbstractCreationUtils;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.uml.alldiagram.ExactUMLSwitch;

/**
 * This utility class allows to create a GraphElement associated with a Model Object
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class CompositeStructuresCreationUtils extends AbstractCreationUtils
{
	private static final int LABEL_OFFSET = 5;

	/**
	 * Constructor
	 * 
	 * @param diagramConf the Diagram Graphical Configuration
	 * @generated
	 */
	public CompositeStructuresCreationUtils(DiagramGraphConf diagramConf)
	{
		super(diagramConf);
	}

	/**
	 * @generated
	 */
	private class GraphicUMLSwitch extends ExactUMLSwitch
	{
		/**
		 * The presentation of the graphical element
		 * 
		 * @generated
		 */
		private String presentation;

		/**
		 * Constructor
		 * 
		 * @param presentation the presentation of the graphical element
		 * @generated
		 */
		public GraphicUMLSwitch(String presentation)
		{
			this.presentation = presentation;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseProperty(org.eclipse.uml2.uml.Property)
		 * @generated
		 */
		public Object caseProperty(Property object)
		{
			if ("default".equals(presentation))
			{
				return createGraphElementProperty(object, presentation);
			}
			return null;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseInterface(org.eclipse.uml2.uml.Interface)
		 * @generated
		 */
		public Object caseInterface(Interface object)
		{
			if ("default".equals(presentation))
			{
				return createGraphElementInterface(object, presentation);
			}
			return null;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#casePort(org.eclipse.uml2.uml.Port)
		 * @generated
		 */
		public Object casePort(Port object)
		{
			if ("default".equals(presentation))
			{
				return createGraphElementPort(object, presentation);
			}
			return null;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseClass(org.eclipse.uml2.uml.Class)
		 * @generated
		 */
		public Object caseClass(Class object)
		{
			if ("default".equals(presentation))
			{
				return createGraphElementClass(object, presentation);
			}
			return null;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseCollaboration(org.eclipse.uml2.uml.Collaboration)
		 * @generated
		 */
		public Object caseCollaboration(Collaboration object)
		{
			if ("default".equals(presentation))
			{
				return createGraphElementCollaboration(object, presentation);
			}
			return null;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseCollaborationUse(org.eclipse.uml2.uml.CollaborationUse)
		 * @generated
		 */
		public Object caseCollaborationUse(CollaborationUse object)
		{
			if ("default".equals(presentation))
			{
				return createGraphElementCollaborationUse(object, presentation);
			}
			return null;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseConnector(org.eclipse.uml2.uml.Connector)
		 * @generated
		 */
		public Object caseConnector(Connector object)
		{
			if ("default".equals(presentation))
			{
				return createGraphElementConnector(object, presentation);
			}
			return null;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseInterfaceRealization(org.eclipse.uml2.uml.InterfaceRealization)
		 * @generated
		 */
		public Object caseInterfaceRealization(InterfaceRealization object)
		{
			if ("default".equals(presentation))
			{
				return createGraphElementInterfaceRealization(object, presentation);
			}
			return null;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseUsage(org.eclipse.uml2.uml.Usage)
		 * @generated
		 */
		public Object caseUsage(Usage object)
		{
			if ("default".equals(presentation))
			{
				return createGraphElementUsage(object, presentation);
			}
			return null;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseDependency(org.eclipse.uml2.uml.Dependency)
		 * @generated
		 */
		public Object caseDependency(Dependency object)
		{
			if ("default".equals(presentation))
			{
				return createGraphElementDependency(object, presentation);
			}
			return null;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseComment(org.eclipse.uml2.uml.Comment)
		 * @generated
		 */
		public Object caseComment(Comment object)
		{
			if ("default".equals(presentation))
			{
				return createGraphElementComment(object, presentation);
			}
			return null;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
		 * @generated
		 */
		public Object defaultCase(EObject object)
		{
			return null;
		}
	}

	/**
	 * @see org.topcased.modeler.editor.ICreationUtils#createGraphElement(org.eclipse.emf.ecore.EObject, java.lang.String)
	 * @generated
	 */
	public GraphElement createGraphElement(EObject obj, String presentation)
	{
		Object graphElt = null;

		if ("http://www.eclipse.org/uml2/2.1.0/UML".equals(obj.eClass().getEPackage().getNsURI()))
		{
			graphElt = new GraphicUMLSwitch(presentation).doSwitch(obj);
		}

		if (graphElt instanceof GraphElement)
		{
			return (GraphElement) graphElt;
		}

		return null;
	}

	/**
	 * @param element the model element
	 * @param presentation the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementProperty(org.eclipse.uml2.uml.Property element, String presentation)
	{
		return createGraphNode(element, presentation);
	}

	/**
	 * @param element the model element
	 * @param presentation the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementInterface(Interface element, String presentation)
	{
		return createGraphNode(element, presentation);
	}

	/**
	 * @param element the model element
	 * @param presentation the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementPort(Port element, String presentation)
	{
		return createGraphNode(element, presentation);
	}

	/**
	 * @param element the model element
	 * @param presentation the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementCollaborationUse(CollaborationUse element, String presentation)
	{
		return createGraphNode(element, presentation);
	}

	/**
	 * @param element the model element
	 * @param presentation the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementClass(Class element, String presentation)
	{
		return createGraphNode(element, presentation);
	}

	/**
	 * @param element the model element
	 * @param presentation the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementCollaboration(Collaboration element, String presentation)
	{
		return createGraphNode(element, presentation);
	}

	/**
	 * @param element the model element
	 * @param presentation the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementConnector(org.eclipse.uml2.uml.Connector element, String presentation)
	{
		GraphEdge graphEdge = createGraphEdge(element, presentation);
		EdgeObjectUV srccountEdgeObjectUV = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectUV();
		srccountEdgeObjectUV.setId(CompositeStructuresEdgeObjectConstants.SRCCOUNT_EDGE_OBJECT_ID);
		srccountEdgeObjectUV.setUDistance(LABEL_OFFSET);
		srccountEdgeObjectUV.setVDistance(LABEL_OFFSET);
		srccountEdgeObjectUV.setVisible(true);
		graphEdge.getContained().add(srccountEdgeObjectUV);
		EdgeObjectUV targetcountEdgeObjectUV = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectUV();
		targetcountEdgeObjectUV.setId(CompositeStructuresEdgeObjectConstants.TARGETCOUNT_EDGE_OBJECT_ID);
		targetcountEdgeObjectUV.setUDistance(LABEL_OFFSET);
		targetcountEdgeObjectUV.setVDistance(LABEL_OFFSET);
		targetcountEdgeObjectUV.setVisible(true);
		graphEdge.getContained().add(targetcountEdgeObjectUV);
		EdgeObjectOffset middlenameEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
		middlenameEdgeObjectOffset.setId(CompositeStructuresEdgeObjectConstants.MIDDLENAME_EDGE_OBJECT_ID);
		middlenameEdgeObjectOffset.setOffset(new Dimension(0, 10));
		middlenameEdgeObjectOffset.setVisible(true);
		graphEdge.getContained().add(middlenameEdgeObjectOffset);
		return graphEdge;
	}

	/**
	 * @param element the model element
	 * @param presentation the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementInterfaceRealization(InterfaceRealization element, String presentation)
	{
		GraphEdge graphEdge = createGraphEdge(element, presentation);
		return graphEdge;
	}

	/**
	 * @param element the model element
	 * @param presentation the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementUsage(Usage element, String presentation)
	{
		GraphEdge graphEdge = createGraphEdge(element, presentation);
		return graphEdge;
	}

	/**
	 * @param element the model element
	 * @param presentation the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementDependency(Dependency element, String presentation)
	{
		GraphEdge graphEdge = createGraphEdge(element, presentation);
		EdgeObjectUV targetnameEdgeObjectUV = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectUV();
		targetnameEdgeObjectUV.setId(CompositeStructuresEdgeObjectConstants.TARGETNAME_EDGE_OBJECT_ID);
		targetnameEdgeObjectUV.setUDistance(LABEL_OFFSET);
		targetnameEdgeObjectUV.setVDistance(LABEL_OFFSET);
		graphEdge.getContained().add(targetnameEdgeObjectUV);
		return graphEdge;
	}

	/**
	 * @param element the model element
	 * @param presentation the presentation of the graphical element
	 * @return the complete GraphElement
	 * @generated
	 */
	protected GraphElement createGraphElementComment(Comment element, String presentation)
	{
		return createGraphNode(element, presentation);
	}

	/**
	 * Create the ModelObject with its initial children
	 * 
	 * @param obj the model object
	 * @return the model object with its children
	 * @generated
	 */
	public EObject createModelObject(EObject obj)
	{
		return obj;
	}

}
