/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.uml2.uml.CollaborationUse;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Usage;
import org.eclipse.uml2.uml.util.UMLSwitch;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.uml.alldiagram.AllSimpleObjectConstants;
import org.topcased.modeler.uml.alldiagram.edit.CommentEditPart;
import org.topcased.modeler.uml.alldiagram.edit.CommentLinkEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.CollaborationUseEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.CompositeStructuresDiagramEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.ConnectorEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.DependencyEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.InterfaceEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.InterfaceRealizationEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.PortEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.PropertyEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.edit.UsageEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * Part Factory : associates a model object to its controller. <br>
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class CompositeStructuresEditPartFactory implements EditPartFactory
{
	/**
	 * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
	 * @generated
	 */
	public EditPart createEditPart(EditPart context, Object model)
	{
		if (model instanceof Diagram)
		{
			return new CompositeStructuresDiagramEditPart((Diagram) model);
		}
		else if (model instanceof GraphNode)
		{
			final GraphNode node = (GraphNode) model;
			EObject element = Utils.getElement(node);
			if (element != null)
			{
				if ("http://www.eclipse.org/uml2/2.1.0/UML".equals(element.eClass().getEPackage().getNsURI()))
				{
					return (EditPart) new NodeUMLSwitch(node).doSwitch(element);
				}
	            return new EMFGraphNodeEditPart(node);
			}

			return new GraphNodeEditPart(node);
		}
		else if (model instanceof GraphEdge)
		{
			final GraphEdge edge = (GraphEdge) model;
			EObject element = Utils.getElement(edge);
			if (element != null)
			{
				if ("http://www.eclipse.org/uml2/2.1.0/UML".equals(element.eClass().getEPackage().getNsURI()))
				{
					return (EditPart) new EdgeUMLSwitch(edge).doSwitch(element);
				}
	            return new EMFGraphEdgeEditPart(edge);
			}

			if (edge.getSemanticModel() instanceof SimpleSemanticModelElement)
			{
				// Manage the Element that are not associated with a model object
				if (AllSimpleObjectConstants.SIMPLE_OBJECT_COMMENTLINK.equals(((SimpleSemanticModelElement) edge
						.getSemanticModel()).getTypeInfo()))
				{
					return new CommentLinkEditPart(edge);
				}
			}

			return new GraphEdgeEditPart(edge);
		}

		throw new IllegalStateException("No edit part matches with the '" + model.getClass().getName()
				+ "' model element. Check CompositeStructuresEditPartFactory#createEditPart(EditPart,Object) class");
	}

	/**
	 * @generated
	 */
	private class NodeUMLSwitch extends UMLSwitch<Object>
	{
		/**
		 * The graphical node
		 * 
		 * @generated
		 */
		private GraphNode node;

		/**
		 * Constructor
		 * 
		 * @param node the graphical node
		 * @generated
		 */
		public NodeUMLSwitch(GraphNode node)
		{
			this.node = node;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseComment(org.eclipse.uml2.uml.Comment)
		 * @generated
		 */
		public Object caseComment(Comment object)
		{
			return new CommentEditPart(node);
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#casePort(org.eclipse.uml2.uml.Port)
		 * @generated
		 */
		public Object casePort(Port object)
		{
			return new PortEditPart(node);
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseProperty(org.eclipse.uml2.uml.Property)
		 * @generated
		 */
		public Object caseProperty(Property object)
		{
			return new PropertyEditPart(node);
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseInterface(org.eclipse.uml2.uml.Interface)
		 * @generated
		 */
		public Object caseInterface(Interface object)
		{
			return new InterfaceEditPart(node);
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseCollaborationUse(org.eclipse.uml2.uml.CollaborationUse)
		 * @generated
		 */
		public Object caseCollaborationUse(CollaborationUse object)
		{
			return new CollaborationUseEditPart(node);
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
		 * @generated
		 */
		public Object defaultCase(EObject object)
		{
			return new EMFGraphNodeEditPart(node);
		}
	}

	/**
	 * @generated
	 */
	private class EdgeUMLSwitch extends UMLSwitch<Object>
	{
		/**
		 * The graphical edge
		 * 
		 * @generated
		 */
		private GraphEdge edge;

		/**
		 * Constructor
		 * 
		 * @param edge the graphical edge
		 * @generated
		 */
		public EdgeUMLSwitch(GraphEdge edge)
		{
			this.edge = edge;
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseConnector(org.eclipse.uml2.uml.Connector)
		 * @generated
		 */
		public Object caseConnector(Connector object)
		{
			return new ConnectorEditPart(edge);
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseInterfaceRealization(org.eclipse.uml2.uml.InterfaceRealization)
		 * @generated
		 */
		public Object caseInterfaceRealization(InterfaceRealization object)
		{
			return new InterfaceRealizationEditPart(edge);
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseUsage(org.eclipse.uml2.uml.Usage)
		 * @generated
		 */
		public Object caseUsage(Usage object)
		{
			return new UsageEditPart(edge);
		}
		
		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#caseDependency(org.eclipse.uml2.uml.Dependency)
		 * @generated
		 */
		public Object caseDependency(Dependency object)
		{
			return new DependencyEditPart(edge);
		}

		/**
		 * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
		 * @generated
		 */
		public Object defaultCase(EObject object)
		{
			return new EMFGraphEdgeEditPart(edge);
		}
	}

}