/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.palette.PaletteStack;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.eclipse.uml2.uml.ConnectorKind;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.editor.GraphElementCreationFactory;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.palette.ModelerConnectionCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerPaletteManager;
import org.topcased.modeler.uml.alldiagram.AllSimpleObjectConstants;

/**
 * Generated Palette Manager
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class CompositeStructuresPaletteManager extends ModelerPaletteManager
{
	// declares all the palette categories of the diagram
	/**
	 * @generated
	 */
	private PaletteDrawer objectsDrawer;

	/**
	 * @generated
	 */
	private PaletteDrawer connectionsDrawer;

	/**
	 * @generated
	 */
	private PaletteDrawer commentDrawer;

	/**
	 * @generated
	 */
	private ICreationUtils creationUtils;

	/**
	 * The Constructor
	 * 
	 * @param utils the creation utils for the tools of the palette
	 * @generated
	 */
	public CompositeStructuresPaletteManager(ICreationUtils utils)
	{
		super();
		this.creationUtils = utils;
	}

	/**
	 * Creates the main categories of the palette
	 * 
	 * @generated
	 */
	protected void createCategories()
	{
		createObjectsDrawer();
		createConnectionsDrawer();
		createCommentDrawer();
	}

	/**
	 * Updates the main categories of the palette
	 * 
	 * @generated
	 */
	protected void updateCategories()
	{
		// deletion of the existing categories and creation of the updated categories
		getRoot().remove(objectsDrawer);
		createObjectsDrawer();

		getRoot().remove(connectionsDrawer);
		createConnectionsDrawer();

		getRoot().remove(commentDrawer);
		createCommentDrawer();
	}

	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated
	 */
	private void createObjectsDrawer()
	{
		objectsDrawer = new PaletteDrawer("Objects", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();

		CreationFactory factory;

		PaletteStack propertyStack = new PaletteStack("Property", "Property", CompositeStructuresImageRegistry.getImageDescriptor("PART"));
		entries.add(propertyStack);

		factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getProperty(), "default")
		{
			public EObject getNewModelObject()
			{
				Property element = (Property) super.getNewModelObject();
				// UML Constraint : Property.aggregation must be Composite for a Part
				element.setAggregation(AggregationKind.COMPOSITE_LITERAL);
				return element;
			}
		};
		propertyStack.add(new ModelerCreationToolEntry("Part", "Part", factory,
				CompositeStructuresImageRegistry.getImageDescriptor("PART"), CompositeStructuresImageRegistry
						.getImageDescriptor("PART_LARGE")));

		factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getProperty(), "default")
		{
			public EObject getNewModelObject()
			{
				Property element = (Property) super.getNewModelObject();
				// UML Constraint : Property.aggregation must be NONE for a Role
				element.setAggregation(AggregationKind.NONE_LITERAL);
				return element;
			}
		};
		propertyStack.add(new ModelerCreationToolEntry("Role", "Role", factory,
				CompositeStructuresImageRegistry.getImageDescriptor("ROLE"), CompositeStructuresImageRegistry
						.getImageDescriptor("ROLE_LARGE")));

		factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getInterface(), "default");
		entries.add(new ModelerCreationToolEntry("Interface", "Interface", factory, CompositeStructuresImageRegistry
				.getImageDescriptor("INTERFACE"), CompositeStructuresImageRegistry.getImageDescriptor("INTERFACE_LARGE")));


		factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getPort(), "default")
		{
			public EObject getNewModelObject()
			{
				Port element = (Port) super.getNewModelObject();
				// UML Constraint : Port.aggregation must be composite
				element.setAggregation(AggregationKind.COMPOSITE_LITERAL);
				return element;
			}
		};
		entries.add(new ModelerCreationToolEntry("Port", "Port", factory, CompositeStructuresImageRegistry.getImageDescriptor("PORT"),
				CompositeStructuresImageRegistry.getImageDescriptor("PORT_LARGE")));

		factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getCollaborationUse(), "default");
		entries.add(new ModelerCreationToolEntry("CollaborationUse", "CollaborationUse", factory, CompositeStructuresImageRegistry
				.getImageDescriptor("COLLABORATIONUSE"), CompositeStructuresImageRegistry.getImageDescriptor("COLLABORATIONUSE_LARGE")));

		objectsDrawer.addAll(entries);
		getRoot().add(objectsDrawer);
	}

	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated
	 */
	private void createConnectionsDrawer()
	{
		connectionsDrawer = new PaletteDrawer("Connections", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();

		CreationFactory factory;

		factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getConnector(), "default")
		{
			public EObject getNewModelObject()
			{
				// Creates Connector and ConnectorEnd
				Connector element = (Connector) super.getNewModelObject();
				ConnectorEnd sourceEnd = UMLFactory.eINSTANCE.createConnectorEnd();
				ConnectorEnd targetEnd = UMLFactory.eINSTANCE.createConnectorEnd();

				// Sets connection kind
				/** NOTE : the connector kind should be ASSEMBLY_LITERAL otherwise it generates a warning during validation step */
				element.setKind(ConnectorKind.DELEGATION_LITERAL);

				// Adds ConnectorEnds objects
				element.getEnds().add(sourceEnd);
				element.getEnds().add(targetEnd);
				return element;
			}
		};
		entries.add(new ModelerConnectionCreationToolEntry("Connector", "Connector", factory, CompositeStructuresImageRegistry
				.getImageDescriptor("CONNECTOR"), CompositeStructuresImageRegistry.getImageDescriptor("CONNECTOR_LARGE")));


		factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getInterfaceRealization(), "default");
		entries.add(new ModelerConnectionCreationToolEntry("InterfaceRealization", "InterfaceRealization", factory,
				CompositeStructuresImageRegistry.getImageDescriptor("INTERFACEREALIZATION"), CompositeStructuresImageRegistry
						.getImageDescriptor("INTERFACEREALIZATION_LARGE")));

		factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getUsage(), "default");
		entries.add(new ModelerConnectionCreationToolEntry("Usage", "Usage", factory, CompositeStructuresImageRegistry
				.getImageDescriptor("USAGE"), CompositeStructuresImageRegistry.getImageDescriptor("USAGE_LARGE")));
				
		factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getDependency(), "default");
		entries.add(new ModelerConnectionCreationToolEntry("Role Binding", "Role Binding", factory, CompositeStructuresImageRegistry
				.getImageDescriptor("DEPENDENCY"), CompositeStructuresImageRegistry.getImageDescriptor("DEPENDENCY_LARGE")));

		connectionsDrawer.addAll(entries);
		getRoot().add(connectionsDrawer);
	}

	/**
	 * Creates the Palette container containing all the Palette entries for each figure.
	 * 
	 * @generated
	 */
	private void createCommentDrawer()
	{
		commentDrawer = new PaletteDrawer("Comment", null);
		List<PaletteEntry> entries = new ArrayList<PaletteEntry>();

		CreationFactory factory;

		factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getComment(), "default");
		entries.add(new ModelerCreationToolEntry("Comment", "Comment", factory, CompositeStructuresImageRegistry
				.getImageDescriptor("COMMENT"), CompositeStructuresImageRegistry.getImageDescriptor("COMMENT_LARGE")));

		factory = new GraphElementCreationFactory(creationUtils, AllSimpleObjectConstants.SIMPLE_OBJECT_COMMENTLINK, "default", false);
		entries.add(new ModelerConnectionCreationToolEntry("CommentLink", "CommentLink", factory, CompositeStructuresImageRegistry
				.getImageDescriptor("COMMENTLINK"), CompositeStructuresImageRegistry.getImageDescriptor("COMMENTLINK_LARGE")));

		commentDrawer.addAll(entries);
		getRoot().add(commentDrawer);
	}

}
