/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.CollaborationUse;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Property;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.uml.alldiagram.commands.ElementRestoreConnectionCommand;
import org.topcased.modeler.utils.Utils;

/**
 * CollaborationUse restore connection command
 * 
 * @generated
 */
public class CollaborationUseRestoreConnectionCommand extends ElementRestoreConnectionCommand
{
	/**
	 * @param part the EditPart that is restored
	 * @generated
	 */
	public CollaborationUseRestoreConnectionCommand(EditPart part)
	{
		super(part);
	}

	/**
	 * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
	 * @generated
	 */
	protected void initializeCommands()
	{

		GraphElement elt = getGraphElement();
		EObject eltObject = Utils.getElement(elt);

		if (eltObject instanceof CollaborationUse)
		{
			Iterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
			while (itDiagContents.hasNext())
			{
				Object obj = itDiagContents.next();
				// FIXME Change the way to handle EList GraphNodes
				if (obj instanceof GraphElement
						&& DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
				{
					boolean autoRef = obj.equals(elt);
					GraphElement elt2 = (GraphElement) obj;
					EObject eltObject2 = Utils.getElement(elt2);

					if (eltObject2 instanceof Property)
					{
						if (autoRef)
						{
							// autoRef not allowed
						}
						else
						{
							// if the elt is the source of the edge or if it is the target and that the SourceTargetCouple is reversible
							createDependencyFromCollaborationUseToProperty(elt, elt2);
						}
					}

				}
			}
		}
	}

	/**
	 * @param srcElt the source element
	 * @param targetElt the target element
	 * @generated
	 */
	private void createDependencyFromCollaborationUseToProperty(GraphElement srcElt, GraphElement targetElt)
	{
		CollaborationUse sourceObject = (CollaborationUse) Utils.getElement(srcElt);
		Property refSourceObject = (Property) sourceObject.getType().getAllAttributes().get(0);
		Property targetObject = (Property) Utils.getElement(targetElt);

		for (Dependency dependency : sourceObject.getRoleBindings())
		{
			if (dependency.getSuppliers().contains(refSourceObject) && dependency.getClients().contains(targetObject))
			{
				// check if the relation does not exists yet
				List existing = getExistingEdges(srcElt, targetElt, Dependency.class);
				if (!isAlreadyPresent(existing, dependency))
				{
					ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
					// restore the link with its default presentation
					GraphElement edge = factory.createGraphElement(dependency);
					if (edge instanceof GraphEdge)
					{
						DependencyEdgeCreationCommand cmd = new DependencyEdgeCreationCommand(getEditDomain(), (GraphEdge) edge,
								srcElt, false);
						cmd.setTarget(targetElt);
						add(cmd);
					}
				}
			}
		}
	}
}