/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.commands;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.uml2.uml.ConnectableElement;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.utils.Utils;

/**
 * Connector edge creation command
 * 
 * @generated
 */
public class ConnectorEdgeCreationCommand extends CreateTypedEdgeCommand
{

	/**
	 * @param domain the edit domain
	 * @param newObj the graph edge of the new connection
	 * @param src the graph element of the source
	 * @generated
	 */
	public ConnectorEdgeCreationCommand(EditDomain domain, GraphEdge newObj, GraphElement src)
	{
		this(domain, newObj, src, true);
	}

	/**
	 * @param domain the edit domain
	 * @param newObj the graph edge of the new connection
	 * @param src the graph element of the source
	 * @param needModelUpdate set it to true if the model need to be updated
	 * @generated
	 */
	public ConnectorEdgeCreationCommand(EditDomain domain, GraphEdge newObj, GraphElement src, boolean needModelUpdate)
	{
		super(domain, newObj, src, needModelUpdate);
	}

	/**
	 * @see org.topcased.modeler.commands.CreateGraphEdgeCommand#execute()
	 * @generated NOT
	 */
	public void execute()
	{
		// Creates the connection and adds it to the current classifier
		Connector connector = (Connector) Utils.getElement(getEdge());

		// Retrieves source object and target object
		ConnectableElement sourceObject = (ConnectableElement) Utils.getElement(getSource());
		ConnectableElement targetObject = (ConnectableElement) Utils.getElement(getTarget());

		// Sets the 'Role' attribute
		((ConnectorEnd) connector.getEnds().get(0)).setRole(sourceObject);
		((ConnectorEnd) connector.getEnds().get(1)).setRole(targetObject);

		// Sets the 'PartWithRole' attribute for the source
		EObject sourceContainer = Utils.getElement((GraphNode) getSource().eContainer());
		if (sourceContainer instanceof Property)
		{
			if (sourceObject instanceof Port)
			{
				Property property = (Property) Utils.getElement((GraphNode) getSource().eContainer());
				((ConnectorEnd) connector.getEnds().get(0)).setPartWithPort(property);
			}
			else
			{
				((ConnectorEnd) connector.getEnds().get(0)).setPartWithPort((Property) sourceObject);
			}
		}

		// Sets the 'PartWithRole' attribute for the target
		EObject targetContainer = Utils.getElement((GraphNode) getTarget().eContainer());
		if (targetContainer instanceof Property)
		{
			if (targetObject instanceof Port)
			{
				Property property = (Property) Utils.getElement((GraphNode) getTarget().eContainer());
				((ConnectorEnd) connector.getEnds().get(1)).setPartWithPort(property);
			}
			else
			{
				((ConnectorEnd) connector.getEnds().get(1)).setPartWithPort((Property) targetObject);
			}
		}

		super.execute();
	}

	/**
	 * Retrieves the container EObject from the container parameter
	 * 
	 * @return the EObject
	 * @generated NOT
	 */
	protected EObject getContainerEObject()
	{
		// Calls the super method to get the container object
		EObject parentObject = super.getContainerEObject();
		EObject childObject = Utils.getElement(getSource());

		// Handles cases Connector from Port -> Port
		// childObject must be a Port
		// parentObject must be a Property
		if (childObject instanceof Port && parentObject instanceof Property)
		{
			return parentObject.eContainer();
		}

		// all the others cases are managed by the standard way
		return parentObject;
	}

}