/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.commands;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditDomain;
import org.eclipse.jface.window.Window;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.CollaborationUse;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.eclipse.uml2.uml.UMLFactory;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.commands.IDialogCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.uml.compositestructuresdiagram.CompositeStructuresPlugin;
import org.topcased.modeler.uml.compositestructuresdiagram.common.dialog.ReferenceCollaborationDialog;
import org.topcased.modeler.utils.LabelHelper;
import org.topcased.modeler.utils.Utils;

public class CreateCollaborationUseCommand extends CreateGraphNodeCommand implements IDialogCommand
{
	/** The Collaboration */
	private Collaboration collaboration;

	/** The name of the collaboration to create */
	private String collaborationName;

	/** Indicate if the collaboration has to be create */
	private boolean createCollaboration;

	/**
	 * The Constructor
	 * 
	 * @param domain the EditDomain (used to init the name)
	 * @param newObject the node to create
	 * @param newParent the parent node
	 * @param location the location of the created node
	 * @param dimension the size of the created node
	 * @param pos
	 * @param needModelUpdate <code>true</code> if the model must be modified during this command, <code>false</code> if objects already exist in the mode.
	 */
	public CreateCollaborationUseCommand(EditDomain domain, GraphNode newObject, GraphNode newParent, Point location, Dimension dimension,
			int pos, boolean needModelUpdate)
	{
		super(domain, newObject, newParent, Utils.getElement(newParent), location, dimension, pos, null, needModelUpdate);
	}

	/**
	 * @see org.topcased.modeler.commands.CreateGraphNodeCommand#redoModel()
	 */
	protected void redoModel()
	{
		if (createCollaboration)
		{
			// creates a model object 'Collaboration'
			collaboration = UMLFactory.eINSTANCE.createCollaboration();
			collaboration.setName(collaborationName);

			((StructuredClassifier) getParentEObject()).getNearestPackage().getPackagedElements().add(collaboration);
		}

		// Initialize the name of the CollaborationUse
		String curName = LabelHelper.INSTANCE.getName(getEditDomain(), getChildEObject());
		if (curName == null || "".equals(curName))
		{
			LabelHelper.INSTANCE.initName(getEditDomain(), getParentEObject(), getChildEObject());
		}

		// the collaborationUse is attached to the current classifier
		((StructuredClassifier) getParentEObject()).getCollaborationUses().add((CollaborationUse) getChildEObject());

		// the current collaborationUse is linked to the collaboration
		((CollaborationUse) getChildEObject()).setType(collaboration);
	}

	/**
	 * @see org.topcased.modeler.commands.CreateGraphNodeCommand#undoModel()
	 */
	protected void undoModel()
	{
		if (createCollaboration)
		{
			((StructuredClassifier) getParentEObject()).getNearestPackage().getPackagedElements().remove(collaboration);
		}
		((StructuredClassifier) getParentEObject()).getCollaborationUses().remove(getChildEObject());
	}

	/**
	 * @see org.topcased.modeler.commands.IDialogCommand#openDialog()
	 */
	public boolean openDialog()
	{
		// Display a dialog that allow the user to reference the Collaboration to
		// associate with the CollaborationUse
		ReferenceCollaborationDialog dialog = new ReferenceCollaborationDialog((StructuredClassifier) getParentEObject(),
				CompositeStructuresPlugin.getActiveWorkbenchShell());
		if (dialog.open() == Window.OK)
		{
			createCollaboration = dialog.getCreateCollaboration();
			collaboration = dialog.getNewCollaboration();
			collaborationName = dialog.getNewCollaborationName();

			return true;
		}

		return false;
	}

}
