/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.commands;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.utils.LabelHelper;
import org.topcased.modeler.utils.Utils;

/**
 * This command allows to create a new object 'Interface' not in the StructuredClassifier element (the composite diagram) but in the current package based one
 * level higher.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class CreateInterfaceCommand extends CreateGraphNodeCommand
{
	/**
	 * The Constructor
	 * 
	 * @param domain the EditDomain (used to init the name)
	 * @param newObject the node to create
	 * @param newParent the parent node
	 * @param location the location of the created node
	 * @param dimension the size of the created node
	 * @param pos
	 * @param needModelUpdate <code>true</code> if the model must be modified during this command, <code>false</code> if objects already exist in the mode.
	 */
	public CreateInterfaceCommand(EditDomain domain, GraphNode newObject, GraphNode newParent, Point location, Dimension dimension,
			int pos, boolean needModelUpdate)
	{
		super(domain, newObject, newParent, Utils.getElement(newParent), location, dimension, pos, null, needModelUpdate);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.topcased.modeler.commands.CreateGraphNodeCommand#redoModel()
	 */
	public void redoModel()
	{
		// Initializes the name of the Interface
		String curName = LabelHelper.INSTANCE.getName(getEditDomain(), getChildEObject());
		if (curName == null || "".equals(curName))
		{
			LabelHelper.INSTANCE.initName(getEditDomain(), getParentEObject(), getChildEObject());
		}

		// The interface (the child) is directly contained by the current package and not the clasifier
		((Package) getParentEObject()).getPackagedElements().add((PackageableElement) getChildEObject());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.topcased.modeler.commands.CreateGraphNodeCommand#undoModel()
	 */
	protected void undoModel()
	{
		((Package) getParentEObject()).getPackagedElements().remove(getChildEObject());
	}
	
	/**
	 * Instead of returning the StructuredClassifier, it is the container (the current package) which is returned.
	 * 
	 * @see org.topcased.modeler.commands.CreateGraphNodeCommand#getParentEObject()
	 */
	protected EObject getParentEObject()
	{
		if (super.getParentEObject() instanceof StructuredClassifier)
		{
			// the current package is returned
			return ((StructuredClassifier) super.getParentEObject()).getPackage();
		}
		return super.getParentEObject();
	}
}
