/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.commands;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.uml.compositestructuresdiagram.CompositeStructuresPlugin;
import org.topcased.modeler.uml.compositestructuresdiagram.common.dialog.ReferencePortDialog;

public class CreatePortCommand extends CreatePropertyCommand
{
	/** Object type involved in this command */
	protected static final String TYPE = "Port";

	/**
	 * The Constructor
	 * 
	 * @param domain the EditDomain (used to init the name)
	 * @param newObject the node to create
	 * @param newParent the parent node
	 * @param location the location of the created node
	 * @param dimension the size of the created node
	 * @param pos
	 * @param needModelUpdate <code>true</code> if the model must be modified during this command, <code>false</code> if objects already exist in the mode.
	 */
	public CreatePortCommand(EditDomain domain, GraphNode newObject, GraphNode newParent, Point location, Dimension dimension, int pos,
			boolean needModelUpdate)
	{
		super(domain, newObject, newParent, location, dimension, pos, needModelUpdate);
	}

	/**
	 * @see org.topcased.modeler.commands.CreateGraphNodeCommand#getParentEObject()
	 * @generated NOT
	 */
	protected EObject getParentEObject()
	{
		// Case where the parent is a Property
		if (super.getParentEObject() instanceof Property)
		{
			if (((Property) super.getParentEObject()).getType() != null) 
			{
				// The element must be added to the Type of the parent
				return ((Property) super.getParentEObject()).getType();
			}
		}

		return super.getParentEObject();
	}
	
	/**
	 * @see org.topcased.modeler.commands.IDialogCommand#openDialog()
	 */
	public boolean openDialog()
	{
		Shell currentShell = CompositeStructuresPlugin.getActiveWorkbenchShell();
		ReferencePortDialog dialog = new ReferencePortDialog((StructuredClassifier) getParentEObject(), currentShell);
		if (dialog.open() == Window.OK)
		{
			createClassifier = dialog.getCreateType();
			refType = dialog.getType();
			typeName = dialog.getNewTypeName();

			return true;
		}

		return false;
	}
}
