/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.commands;

import org.eclipse.gef.EditDomain;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.CollaborationUse;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.VisibilityKind;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.utils.Utils;

/**
 * Dependency edge creation command
 * 
 * @generated
 */
public class DependencyEdgeCreationCommand extends CreateTypedEdgeCommand
{

	/**
	 * @param domain the edit domain
	 * @param newObj the graph edge of the new connection
	 * @param src the graph element of the source
	 * @generated
	 */
	public DependencyEdgeCreationCommand(EditDomain domain, GraphEdge newObj, GraphElement src)
	{
		this(domain, newObj, src, true);
	}

	/**
	 * @param domain the edit domain
	 * @param newObj the graph edge of the new connection
	 * @param src the graph element of the source
	 * @param needModelUpdate set it to true if the model need to be updated
	 * @generated
	 */
	public DependencyEdgeCreationCommand(EditDomain domain, GraphEdge newObj, GraphElement src, boolean needModelUpdate)
	{
		super(domain, newObj, src, needModelUpdate);
	}

	/**
	 * @generated
	 */
	protected void redoModel()
	{
		super.redoModel();

		// Add a role property in the collaboration
		Collaboration collaboration = ((CollaborationUse) Utils.getElement(getSource())).getType();

		// The property is only created once
		Property property = null;
		if (collaboration.getOwnedAttributes().isEmpty())
		{
			property = UMLFactory.eINSTANCE.createProperty();
			property.setVisibility(VisibilityKind.PRIVATE_LITERAL);
			property.setName("role");
			collaboration.getOwnedAttributes().add(property);
		}
		else
		{
			// Takes the first
			property = (Property) collaboration.getOwnedAttributes().get(0);
		}

		// Then Supplier is updated : the source node is removed and the property is added
		Dependency dependency = (Dependency) Utils.getElement(getEdge());
		dependency.getSuppliers().remove(Utils.getElement(getSource()));
		dependency.getSuppliers().add(property);
	}
}