/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.commands;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Display;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Usage;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.topcased.facilities.dialogs.ChooseDialog;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.uml.alldiagram.commands.ElementRestoreConnectionCommand;
import org.topcased.modeler.utils.Utils;
import org.topcased.tabbedproperties.providers.TabbedPropertiesLabelProvider;

/**
 * Interface restore connection command
 * 
 * @generated
 */
public class InterfaceRestoreConnectionCommand extends ElementRestoreConnectionCommand
{
	/**
	 * @param part the EditPart that is restored
	 * @generated
	 */
	public InterfaceRestoreConnectionCommand(EditPart part)
	{
		super(part);
	}

	/**
	 * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
	 * @generated
	 */
	protected void initializeCommands()
	{

		super.initializeCommands();

		GraphElement elt = getGraphElement();
		EObject eltObject = Utils.getElement(elt);

		if (eltObject instanceof Interface)
		{
			// try to recreate potential connection between the interface and the diagram (normally a class)
			createInterfaceRealizationFromInterfaceToPort(elt, getModeler().getActiveDiagram());

			createDependencyFromPortToInterface(getModeler().getActiveDiagram(), elt);
			
			Iterator<EObject> itDiagContents = getModeler().getActiveDiagram().eAllContents();
			while (itDiagContents.hasNext())
			{
				Object obj = itDiagContents.next();
				if (obj instanceof GraphElement
				        && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
				{
					boolean autoRef = obj.equals(elt);
					GraphElement elt2 = (GraphElement) obj;
					EObject eltObject2 = Utils.getElement(elt2);

					if (eltObject2 instanceof Property && !(eltObject2 instanceof Port))
					{
						if (!autoRef)
						{
							createInterfaceRealizationFromInterfaceToPort(elt, elt2);

							createDependencyFromPortToInterface(elt2, elt);
						}
					}
				}
			}
		}
	}

	/**
	 * @param srcElt the source element
	 * @param targetElt the target element
	 * @generated
	 */
	private void createInterfaceRealizationFromInterfaceToPort(GraphElement srcElt, GraphElement targetElt)
	{
		Interface sourceObject = (Interface) Utils.getElement(srcElt);
		Class targetObject = null;

		EObject genericTargetObject = Utils.getElement(targetElt);

		if (genericTargetObject instanceof Class)
		{
			targetObject = (Class) genericTargetObject;
		}

		if (genericTargetObject instanceof Property)
		{
			targetObject = (Class) ((Property) genericTargetObject).getType();
		}

		if (targetObject != null) 
		{
			// Restore external connections (may come from other diagram)
			for (InterfaceRealization interfaceRealization : targetObject.getInterfaceRealizations())
			{
				if (interfaceRealization.getContract().equals(sourceObject))
				{
					// must propose only Ports which are drawn on the object container
					List<GraphElement> graphElementList = findGraphElementPort(targetElt);
	
					if (graphElementList.size() > 0)
					{
						if (graphElementList.size() == 1)
						{
							drawInterfaceRealizationEdge(interfaceRealization, srcElt, graphElementList.get(0));
						}
						else
						{
							List<Port> portList = new ArrayList<Port>();
	
							// convert the list of GraphElement in list of Port
							for (GraphElement elt : graphElementList)
							{
								portList.add((Port) Utils.getElement(elt));
							}
	
							// display the dialog to choose a port if nbPorts > 1
							ChooseDialog dialog = new ChooseDialog(Display.getCurrent().getActiveShell(), portList.toArray());
							dialog.setTitle("Interface Realization : " + sourceObject.getName() + " --> " + targetObject.getName());
							dialog.setLabelProvider(new TabbedPropertiesLabelProvider(new UMLItemProviderAdapterFactory()));
							if (dialog.open() == Window.OK)
							{
								Object[] selection = dialog.getResult();
	
								if (selection != null && selection.length > 0)
								{
									drawInterfaceRealizationEdge(interfaceRealization, srcElt, Utils.getGraphElement(targetElt,
									        (EObject) selection[0]));
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * @param srcElt the source element
	 * @param targetElt the target element
	 * @generated
	 */
	private void createDependencyFromPortToInterface(GraphElement srcElt, GraphElement targetElt)
	{
		Interface targetObject = (Interface) Utils.getElement(targetElt);
		Class sourceObject = null;

		EObject genericSourceObject = Utils.getElement(srcElt);

		if (genericSourceObject instanceof Class)
		{
			sourceObject = (Class) genericSourceObject;
		}

		if (genericSourceObject instanceof Property)
		{
			sourceObject = (Class) ((Property) genericSourceObject).getType();
		}

		if (sourceObject != null)
		{
			// Restore external connections (may come from other diagram)
			for (Dependency usage : sourceObject.getClientDependencies())
			{
				if (usage instanceof Usage && usage.getSuppliers().contains(targetObject))
				{
					// must propose only Ports which are drawn on the object container
					List<GraphElement> graphElementList = findGraphElementPort(srcElt);

					if (graphElementList.size() > 0)
					{
						if (graphElementList.size() == 1)
						{
							drawUsageEdge((Usage) usage, graphElementList.get(0), targetElt);
						}
						else
						{
							List<Port> portList = new ArrayList<Port>();

							// convert the list of GraphElement in list of Port
							for (GraphElement elt : graphElementList)
							{
								portList.add((Port) Utils.getElement(elt));
							}

							// display the dialog to choose a port if nbPorts > 1
							ChooseDialog dialog = new ChooseDialog(Display.getCurrent().getActiveShell(), portList.toArray());
							dialog.setTitle("Usage : " + sourceObject.getName() + " --> " + targetObject.getName());
							dialog.setLabelProvider(new TabbedPropertiesLabelProvider(new UMLItemProviderAdapterFactory()));
							if (dialog.open() == Window.OK)
							{
								Object[] selection = dialog.getResult();

								if (selection != null && selection.length > 0)
								{
									drawUsageEdge((Usage) usage, Utils.getGraphElement(srcElt, (EObject) selection[0]), targetElt);
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Browses all graphical element contained in the given container.<br>
	 * Builds a list with children that are represented on the diagram.
	 * 
	 * @param container
	 * @return a list of GraphElement representing 'Port' element that are drawn on the current diagram.
	 */
	private List<GraphElement> findGraphElementPort(GraphElement container)
	{
		List<GraphElement> portList = new ArrayList<GraphElement>();
		for (EObject obj : container.eContents())
		{
			if (obj instanceof GraphElement && Utils.getElement((GraphElement) obj) instanceof Port)
			{
				portList.add((GraphElement) obj);
			}
		}

		return portList;
	}

	/**
	 * Draws the InterfaceRealization edge
	 * 
	 * @param interRealization
	 * @param srcElt
	 * @param targetElt
	 */
	private void drawInterfaceRealizationEdge(InterfaceRealization interRealization, GraphElement srcElt, GraphElement targetElt)
	{
		ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
		// restore the link with its default presentation
		GraphElement edge = factory.createGraphElement(interRealization);
		InterfaceRealizationEdgeCreationCommand cmd = new InterfaceRealizationEdgeCreationCommand(getEditDomain(), (GraphEdge) edge,
		        srcElt, false);
		cmd.setTarget(targetElt);
		add(cmd);
	}

	/**
	 * Draws a usage dependency
	 *  
	 * @param usage
	 * @param srcElt
	 * @param targetElt
	 */
	private void drawUsageEdge(Usage usage, GraphElement srcElt, GraphElement targetElt)
	{
		ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
		// restore the link with its default presentation
		GraphElement edge = factory.createGraphElement(usage);
		UsageEdgeCreationCommand cmd = new UsageEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcElt, false);
		cmd.setTarget(targetElt);
		add(cmd);
	}
}