/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.eclipse.uml2.uml.Usage;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;

/**
 * Port restore connection command
 * 
 * @generated
 */
public class PortRestoreConnectionCommand extends PropertyRestoreConnectionCommand
{
	/**
	 * @param part the EditPart that is restored
	 * @generated
	 */
	public PortRestoreConnectionCommand(EditPart part)
	{
		super(part);
	}

	/**
	 * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
	 * @generated
	 */
	protected void initializeCommands()
	{
		super.initializeCommands();

		GraphElement elt = getGraphElement();
		EObject eltObject = Utils.getElement(elt);

		if (eltObject instanceof Port)
		{
			Iterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
			while (itDiagContents.hasNext())
			{
				Object obj = itDiagContents.next();
				if (obj instanceof GraphElement
						&& DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
				{
					boolean autoRef = obj.equals(elt);
					GraphElement elt2 = (GraphElement) obj;
					EObject eltObject2 = Utils.getElement(elt2);

					if (eltObject2 instanceof Interface)
					{
						if (!autoRef)
						{
							// if elt is the target of the edge or if it is the source and that the SourceTargetCouple is reversible
							createInterfaceRealizationFromInterfaceToPort(elt2, elt);
						}
					}

					if (eltObject2 instanceof Interface)
					{
						if (!autoRef)
						{
							// if the elt is the source of the edge or if it is the target and that the SourceTargetCouple is reversible
							createUsageFromPortToInterface(elt, elt2);
						}
					}
				}
			}
		}
	}

	/**
	 * @param srcElt the source element
	 * @param targetElt the target element
	 * @generated
	 */
	private void createInterfaceRealizationFromInterfaceToPort(GraphElement srcElt, GraphElement targetElt)
	{
		Interface sourceObject = (Interface) Utils.getElement(srcElt);
		Port targetObject = (Port) Utils.getElement(targetElt);

		for (InterfaceRealization interfaceRealization : ((Class) targetObject.getType()).getInterfaceRealizations())
		{
			if (sourceObject.equals(interfaceRealization.getContract()))
			{
				// check if the relation does not exists yet
				List existing = getExistingEdges(srcElt, targetElt, InterfaceRealization.class);
				if (!isAlreadyPresent(existing, interfaceRealization))
				{
					ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
					// restore the link with its default presentation
					GraphElement edge = factory.createGraphElement(interfaceRealization);
					if (edge instanceof GraphEdge)
					{
//						InterfaceRealizationEdgeCreationCommand cmd = new InterfaceRealizationEdgeCreationCommand(getEditDomain(),
//								(GraphEdge) edge, srcElt, false);
//						cmd.setTarget(targetElt);
//						add(cmd);
					}
				}
			}
		}
	}

	/**
	 * @param srcElt the source element
	 * @param targetElt the target element
	 * @generated
	 */
	private void createUsageFromPortToInterface(GraphElement srcElt, GraphElement targetElt)
	{
		Port sourceObject = (Port) Utils.getElement(srcElt);
		Interface targetObject = (Interface) Utils.getElement(targetElt);

		EList<PackageableElement> listPackageableElement = null;
		
		// The following code handles both external and internal restore command.
		/** FIXME : remove this code if one the two diagrams is removed !!.*/
		
		// for internal view 
		if (Utils.getDiagramModelObject(srcElt) instanceof StructuredClassifier)
		{
			listPackageableElement = ((StructuredClassifier) Utils.getDiagramModelObject(srcElt)).getNearestPackage().getPackagedElements();
		}
		
		// for external view 
		if (Utils.getDiagramModelObject(srcElt) instanceof Package)
		{
			listPackageableElement = ((Package) Utils.getDiagramModelObject(srcElt)).getPackagedElements();
		}
		
		for (PackageableElement packageableElement : listPackageableElement)
		{
			if (packageableElement instanceof Usage)
			{
				Usage edgeObject = (Usage) packageableElement;
				if (edgeObject.getSuppliers().contains(targetObject) && edgeObject.getClients().contains(sourceObject))
				{
					// check if the relation does not exists yet
					List existing = getExistingEdges(srcElt, targetElt, Usage.class);
					if (!isAlreadyPresent(existing, edgeObject))
					{
						ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
						// restore the link with its default presentation
						GraphElement edge = factory.createGraphElement(edgeObject);
						if (edge instanceof GraphEdge)
						{
//							UsageEdgeCreationCommand cmd = new UsageEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcElt, false);
//							cmd.setTarget(targetElt);
//							add(cmd);
						}
					}
				}
			}
		}
	}
}