/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.CollaborationUse;
import org.eclipse.uml2.uml.ConnectableElement;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.uml.alldiagram.commands.ElementRestoreConnectionCommand;
import org.topcased.modeler.uml.compositestructuresdiagram.common.util.ConnectorHelper;
import org.topcased.modeler.utils.Utils;

/**
 * Property restore connection command
 * 
 * @generated
 */
public class PropertyRestoreConnectionCommand extends ElementRestoreConnectionCommand
{
	/**
	 * @param part the EditPart that is restored
	 * @generated
	 */
	public PropertyRestoreConnectionCommand(EditPart part)
	{
		super(part);
	}

	/**
	 * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
	 * @generated
	 */
	protected void initializeCommands()
	{
		super.initializeCommands();

		GraphElement elt = getGraphElement();
		EObject eltObject = Utils.getElement(elt);

		if (eltObject instanceof Property)
		{
			Iterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
			while (itDiagContents.hasNext())
			{
				Object obj = itDiagContents.next();
				if (obj instanceof GraphElement
						&& DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
				{
					boolean autoRef = obj.equals(elt);
					GraphElement elt2 = (GraphElement) obj;
					EObject eltObject2 = Utils.getElement(elt2);

					if (eltObject2 instanceof Property)
					{
						if (!autoRef)
						{
							// if the elt is the source of the edge or if it is the target and that the SourceTargetCouple is reversible
							createConnectorFromPropertyToProperty(elt, elt2);
							// if elt is the target of the edge or if it is the source and that the SourceTargetCouple is reversible
							createConnectorFromPropertyToProperty(elt2, elt);
						}
					}

					if (eltObject2 instanceof CollaborationUse)
					{
						if (!autoRef)
						{
							// if elt is the target of the edge or if it is the source and that the SourceTargetCouple is reversible
							createDependencyFromCollaborationUseToProperty(elt2, elt);
						}
					}

				}
			}
		}
	}
	
	/**
	 * @param srcElt the source element
	 * @param targetElt the target element
	 * @generated
	 */
	private void createConnectorFromPropertyToProperty(GraphElement srcElt, GraphElement targetElt)
	{
		List edgeObjectList = null;
		
		// The following code handles both external and internal restore command.
		/** FIXME remove a part of this code if one the two diagrams is removed !!.*/
		
		// for internal view 
		if (Utils.getDiagramModelObject(srcElt) instanceof StructuredClassifier)
		{
			edgeObjectList = ((StructuredClassifier) Utils.getDiagramModelObject(srcElt)).getOwnedConnectors();
		}
		
		// for external view 
		if (Utils.getDiagramModelObject(srcElt) instanceof Package)
		{
			edgeObjectList = ConnectorHelper.getAvailableConnector((Package) Utils.getDiagramModelObject(srcElt));
		}
		
		ConnectableElement source = (ConnectableElement) Utils.getElement(srcElt);
		ConnectableElement target = (ConnectableElement) Utils.getElement(targetElt);
		
		for (Iterator it = edgeObjectList.iterator(); it.hasNext();)
		{
			Object obj = it.next();
			if (obj instanceof Connector)
			{
				Connector edgeObject = (Connector) obj;
				ConnectorEnd end1 = (ConnectorEnd) edgeObject.getEnds().get(0);
				ConnectorEnd end2 = (ConnectorEnd) edgeObject.getEnds().get(1);
				
				if (end1.getRole().equals(source) && end2.getRole().equals(target))
				{
					// additional controls are necessary here when end1 or end2 is a Port
					boolean controlOK = true; 
					
					if (end1.getRole() instanceof Port)
					{
						EObject objSrc = Utils.getElement((GraphElement) srcElt.eContainer());
						if (!end1.getPartWithPort().equals(objSrc))
						{
							controlOK = false;
						}
					}
					
					if (end2.getRole() instanceof Port)
					{
						EObject objTarget = Utils.getElement((GraphElement) targetElt.eContainer());
						if (!end2.getPartWithPort().equals(objTarget))
						{
							controlOK = false;
						}
					}
					
					if (controlOK)
					{
						// check if the relation does not exists yet
						List existing = getExistingEdges(srcElt, targetElt, Connector.class);
						if (!isAlreadyPresent(existing, edgeObject))
						{
							ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
							// restore the link with its default presentation
							GraphElement edge = factory.createGraphElement(edgeObject);
							if (edge instanceof GraphEdge)
							{
								ConnectorEdgeCreationCommand cmd = new ConnectorEdgeCreationCommand(getEditDomain(), (GraphEdge) edge,
										srcElt, false);
								cmd.setTarget(targetElt);
								add(cmd);
							}
						}
					}
				}
			}
		}
	}

	/**
	 * @param srcElt the source element
	 * @param targetElt the target element
	 * @generated
	 */
	private void createDependencyFromCollaborationUseToProperty(GraphElement srcElt, GraphElement targetElt)
	{
		CollaborationUse sourceObject = (CollaborationUse) Utils.getElement(srcElt);
		Property refSourceObject = (Property) sourceObject.getType().getAllAttributes().get(0);
		Property targetObject = (Property) Utils.getElement(targetElt);

		for (Dependency dependency : sourceObject.getRoleBindings())
		{
			if (dependency.getSuppliers().contains(refSourceObject) && dependency.getClients().contains(targetObject))
			{
				// check if the relation does not exists yet
				List existing = getExistingEdges(srcElt, targetElt, Dependency.class);
				if (!isAlreadyPresent(existing, dependency))
				{
					ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
					// restore the link with its default presentation
					GraphElement edge = factory.createGraphElement(dependency);
					if (edge instanceof GraphEdge)
					{
						DependencyEdgeCreationCommand cmd = new DependencyEdgeCreationCommand(getEditDomain(), (GraphEdge) edge,
								srcElt, false);
						cmd.setTarget(targetElt);
						add(cmd);
					}
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.commands.CompoundCommand#canExecute()
	 */
	public boolean canExecute()
	{
		return true;
	}
}