/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.common.dialog;

import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;
import org.topcased.modeler.uml.compositestructuresdiagram.common.util.CollaborationHelper;

/**
 * A Dialog used to reference or create a 'Collaboration' object during the creation of a 'CollaborationUse'.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class ReferenceCollaborationDialog extends Dialog implements ITopcasedDialogConstants
{
	// The structured classifier that will contain the Collaboration use
	private StructuredClassifier structuredClassifier;

	// Stores the collaboration that was choosed
	private Collaboration collaboration;

	private String collaborationName;

	private boolean createCollaboration;

	// SWT Objects
	private Button existingCollaborationBt;

	private Button newCollaborationBt;

	private Combo collaborationCb;

	private ComboViewer collaborationCbViewer;

	private Text collaborationFd;

	/**
	 * Creates the Dialog window to select a Collaboration to associate with the current CollaborationUse.
	 * 
	 * @param class_ the class that contains the State
	 * @param parentShell the parent Shell
	 */
	public ReferenceCollaborationDialog(StructuredClassifier class_, Shell parentShell)
	{
		super(parentShell);
		setShellStyle(getShellStyle() | SWT.RESIZE);
		setBlockOnOpen(true);
		structuredClassifier = class_;
	}

	/**
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell newShell)
	{
		newShell.setText("CollaborationUse Reference");
		newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

		super.configureShell(newShell);
	}

	/**
	 * Creates the Dialog area
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent)
	{
		final Composite dialogComposite = (Composite) super.createDialogArea(parent);
		createDialogComposite(dialogComposite);
		createMainGroup(dialogComposite);
		hookListeners();
		loadData();
		return dialogComposite;
	}

	/**
	 * Creates the Dialog Contents
	 * 
	 * @param composite
	 */
	private void createDialogComposite(Composite composite)
	{
		final GridLayout dialogLayout = new GridLayout();
		final GridData dialogLayoutData = new GridData(SWT.FILL, SWT.FILL, true, true);
		composite.setLayout(dialogLayout);
		composite.setLayoutData(dialogLayoutData);
	}

	/**
	 * Creates the SWT Objects of the Dialog
	 * 
	 * @param parent the parent Composite
	 */
	protected void createMainGroup(Composite parent)
	{
		Composite composite = new Composite(parent, SWT.NONE);

		// Add layout on composite
		composite.setLayout(new GridLayout());
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		existingCollaborationBt = new Button(composite, SWT.RADIO);
		existingCollaborationBt.setText("Choose an existing Collaboration to reference from the CollaborationUse :");

		collaborationCb = new Combo(composite, SWT.NONE);
		collaborationCb.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		collaborationCbViewer = new ComboViewer(collaborationCb);

		final Label separationLbl = new Label(composite, SWT.SEPARATOR | SWT.HORIZONTAL);
		separationLbl.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		newCollaborationBt = new Button(composite, SWT.RADIO);
		newCollaborationBt.setText("Create a new Collaboration to reference from the CollaborationUse :");

		collaborationFd = new Text(composite, SWT.BORDER);
		collaborationFd.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
	}

	/**
	 * Initializes widgets value
	 */
	private void loadData()
	{
		collaborationCbViewer.setContentProvider(new CollaborationContentProvider());
		collaborationCbViewer.setLabelProvider(new AdapterFactoryLabelProvider(new UMLItemProviderAdapterFactory()));
		// Retrieve all the available collaborations in the model
		collaborationCbViewer.setInput(structuredClassifier.getModel());

		existingCollaborationBt.setSelection(true);
		newCollaborationBt.setSelection(false);
		collaborationFd.setEnabled(false);
	}

	/**
	 * Hooks the listeners
	 */
	private void hookListeners()
	{
		existingCollaborationBt.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				handleOKButton();
			}
		});

		newCollaborationBt.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				handleOKButton();
			}
		});

		collaborationCbViewer.addSelectionChangedListener(new ISelectionChangedListener()
		{
			// This ensures that we handle selections correctly.
			public void selectionChanged(SelectionChangedEvent event)
			{
				if (event.getSelection() instanceof IStructuredSelection)
				{
					collaboration = (Collaboration) ((IStructuredSelection) event.getSelection()).getFirstElement();
				}
			}
		});

		collaborationFd.addModifyListener(new ModifyListener()
		{
			// Check if the OK button can be enabled
			public void modifyText(ModifyEvent e)
			{
				handleOKButton();
			}
		});

		collaborationCb.addModifyListener(new ModifyListener()
		{
			// Check if the OK button can be enabled
			public void modifyText(ModifyEvent e)
			{
				handleOKButton();
			}
		});
	}

	/**
	 * Returns the new collaboration
	 * 
	 * @return a collaboration
	 */
	public Collaboration getNewCollaboration()
	{
		return collaboration;
	}

	/**
	 * Returns the name of the collaboration to create
	 * 
	 * @return String
	 */
	public String getNewCollaborationName()
	{
		return collaborationName;
	}

	/**
	 * Returns <code>true</code> if a collaboration should be created
	 * 
	 * @return boolean
	 */
	public boolean getCreateCollaboration()
	{
		return createCollaboration;
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed()
	{
		createCollaboration = newCollaborationBt.getSelection();
		collaborationName = collaborationFd.getText();

		super.okPressed();
	}

	/**
	 * Determines whether the OK button should be enable or disable.
	 */
	private void handleOKButton()
	{
		if (newCollaborationBt.getSelection())
		{
			collaborationCb.setEnabled(false);
			collaborationFd.setEnabled(true);
			if (!"".equals(collaborationFd.getText()))
			{
				getButton(Dialog.OK).setEnabled(true);
			}
			else
			{
				getButton(Dialog.OK).setEnabled(false);
			}
		}

		if (existingCollaborationBt.getSelection())
		{
			collaborationCb.setEnabled(true);
			collaborationFd.setEnabled(false);
			if (!"".equals(collaborationCb.getText()))
			{
				getButton(Dialog.OK).setEnabled(true);
			}
			else
			{
				getButton(Dialog.OK).setEnabled(false);
			}
		}
	}

	/**
	 * Overrides the initial method to allow the OK button to be created with a disabled status.
	 *  
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent)
	{
		super.createButtonsForButtonBar(parent);
		getButton(Dialog.OK).setEnabled(false);
	}

	/**
	 * This Content Provider displays contents of collaborations. For the model, this provider delegates contents computing to the model content provider. <br>
	 */
	public class CollaborationContentProvider implements IStructuredContentProvider
	{
		/**
		 * Constructor
		 */
		public CollaborationContentProvider()
		{
			// Do nothing
		}

		/**
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement)
		{
			return CollaborationHelper.getAvailableCollaboration((Model) inputElement).toArray();
		}

		/**
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose()
		{
			// Do nothing
		}

		/**
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
			// Do nothing
		}
	}
}
