/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.edit;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.EllipseAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.action.IAction;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.CollaborationUse;
import org.eclipse.uml2.uml.Type;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.alldiagram.edit.ElementEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.CompositeStructuresEditPolicyConstants;
import org.topcased.modeler.uml.compositestructuresdiagram.CompositeStructuresPlugin;
import org.topcased.modeler.uml.compositestructuresdiagram.commands.CollaborationUseRestoreConnectionCommand;
import org.topcased.modeler.uml.compositestructuresdiagram.figures.CollaborationUseFigure;
import org.topcased.modeler.uml.compositestructuresdiagram.policies.DependencyEdgeCreationEditPolicy;

/**
 * The CollaborationUse object controller
 * 
 * @generated
 */
public class CollaborationUseEditPart extends ElementEditPart
{
	/**
	 * Constructor
	 * 
	 * @param obj the graph node
	 * @generated
	 */
	public CollaborationUseEditPart(GraphNode obj)
	{
		super(obj);
	}
	
	/**
	 * Creates edit policies and associates these with roles
	 * 
	 * @generated NOT
	 */
	protected void createEditPolicies()
	{
		super.createEditPolicies();

		installEditPolicy(CompositeStructuresEditPolicyConstants.DEPENDENCY_EDITPOLICY, new DependencyEdgeCreationEditPolicy());

		installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
		{
			protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
			{
				return new CollaborationUseRestoreConnectionCommand(getHost());
			}
		});
		
		installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());
		
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
	}

	/**
	 * Creates the CollaborationUse Figure (an ellipse with a dash border)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure()
	{
		return new CollaborationUseFigure();
	}
	
	/**
	 * The editable label is its prefix part
	 * 
	 * @returns the editable label 
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getEditableLabel()
	 */
	public ILabel getEditableLabel()
	{
		return ((ComposedLabel) getLabel()).getPrefix();
	}
	
	/**
	 * Updates the CollaborationUse label
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
	 */
	protected void refreshHeaderLabel()
	{
		ComposedLabel collaborationUseLabel = ((ComposedLabel) getLabel());
		CollaborationUse collaborationUse = (CollaborationUse) getEObject();

		// Updates the name of the Collaboration Use
		collaborationUseLabel.setPrefix(collaborationUse.getName());
		collaborationUseLabel.setMain("");
		collaborationUseLabel.setSuffix("");
		
		if (collaborationUse.getType() != null)
		{
			String classifierName = collaborationUse.getType().getName();

			// Updates the Classifier type of the Connector if necessary
			collaborationUseLabel.setMain(" : ");
			collaborationUseLabel.setSuffix(classifierName);
		}
	}
	
	/**
	 * Activates
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
	 */
	public void activate()
	{
		super.activate();
		EObject model = getEObject();
		if (model instanceof CollaborationUse)
		{
			if (((CollaborationUse) model).getType() instanceof Collaboration)
			{
				Collaboration collaboration = ((CollaborationUse) model).getType();
				if (!collaboration.eAdapters().contains(getModelListener()))
				{
					collaboration.eAdapters().add(getModelListener());
				}
			}
		}
	}

	/**
	 * Desactivates
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
	 */
	public void deactivate()
	{
		EObject model = getEObject();
		if (getEObject() instanceof CollaborationUse)
		{
			CollaborationUse collaborationUse = (CollaborationUse) model;
			if (collaborationUse.getType() instanceof Collaboration)
			{
				Collaboration collaboration = ((CollaborationUse) model).getType();
				if (!collaboration.eAdapters().contains(getModelListener()))
				{
					collaboration.eAdapters().remove(getModelListener());
				}
			}
		}
		super.deactivate();
	}
	
    /**
     * Uses Ellipse anchor
     * 
     * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
     */
    public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection)
    {
        return new EllipseAnchor(getFigure());
    }

    /**
     * Uses Ellipse anchor
     * 
     * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
     */
    public ConnectionAnchor getSourceConnectionAnchor(Request request)
    {
        return new EllipseAnchor(getFigure());
    }

    /**
     * Uses Ellipse anchor
     * 
     * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
     */
    public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection)
    {
        return new EllipseAnchor(getFigure());
    }

    /**
     * Uses Ellipse anchor
     * 
     * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
     */
    public ConnectionAnchor getTargetConnectionAnchor(Request request)
    {
        return new EllipseAnchor(getFigure());
    }
    
	/**
	 * This method enables to create a new diagram or view an existing model diagrams.<br>
	 * It will be used when a <b>CollaborationUse Figure</b>will be clicked and will propose to detail the classifier representing by this type. 
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#createChangeDiagramAction(org.eclipse.emf.ecore.EObject)
	 */
	protected IAction createChangeDiagramAction(EObject modelObject)
	{
		Type propertyType = ((CollaborationUse) getEObject()).getType();
        if (propertyType != null && !"".equals(propertyType.getName()))
        {
            return super.createChangeDiagramAction(propertyType);
        }
        return null;
	}
	
	/* (non-Javadoc)
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
	 */
	protected void handleModelChanged(Notification msg)
	{
		// The Type of a CollaborationUse can not set to null once the CollaborationUse is created.
		if (msg.getOldValue() instanceof Collaboration && msg.getNewValue() == null)
		{
			((CollaborationUse) msg.getNotifier()).setType((Collaboration) msg.getOldValue());
			
			// Dialog Box for preventing the user
			CompositeStructuresPlugin.log("The attribute 'Type' of a 'CollaborationUse' object cannot be set to null.", IStatus.WARNING);
		}
		super.handleModelChanged(msg);
	}
}