/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.topcased.draw2d.figures.Label;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.policies.EdgeObjectOffsetEditPolicy;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.uml.compositestructuresdiagram.CompositeStructuresEdgeObjectConstants;
import org.topcased.modeler.uml.compositestructuresdiagram.figures.ConnectorFigure;
import org.topcased.modeler.uml.compositestructuresdiagram.policies.ConnectorEdgeObjectUVEditPolicy;
import org.topcased.modeler.utils.Utils;

/**
 * Connector controller
 * 
 * @generated NOT
 */
public class ConnectorEditPart extends EMFGraphEdgeEditPart
{

	/**
	 * Constructor
	 * 
	 * @param model the graph object
	 * @generated
	 */
	public ConnectorEditPart(GraphEdge model)
	{
		super(model);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 * @generated
	 */
	protected void createEditPolicies()
	{
		super.createEditPolicies();

		installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_UV_EDITPOLICY, new ConnectorEdgeObjectUVEditPolicy());
		
		installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_OFFSET_EDITPOLICY, new EdgeObjectOffsetEditPolicy());

	}

	/**
	 * @return the Figure
	 * @generated
	 */
	protected IFigure createFigure()
	{
		ConnectorFigure connection = new ConnectorFigure();

		return connection;
	}

	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getEdgeObjectFigure(org.topcased.modeler.di.model.EdgeObject)
	 * @generated
	 */
	public IEdgeObjectFigure getEdgeObjectFigure(EdgeObject edgeObject)
	{
		if (CompositeStructuresEdgeObjectConstants.SRCCOUNT_EDGE_OBJECT_ID.equals(edgeObject.getId()))
		{
			return ((ConnectorFigure) getFigure()).getSrcCountEdgeObjectFigure();
		}
		if (CompositeStructuresEdgeObjectConstants.TARGETCOUNT_EDGE_OBJECT_ID.equals(edgeObject.getId()))
		{
			return ((ConnectorFigure) getFigure()).getTargetCountEdgeObjectFigure();
		}
        if (CompositeStructuresEdgeObjectConstants.MIDDLENAME_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((ConnectorFigure) getFigure()).getMiddleNameEdgeObjectFigure();
        }
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#refreshEdgeObjects()
	 */
	protected void refreshEdgeObjects()
	{
		super.refreshEdgeObjects();

		Connector connector = (Connector) Utils.getElement(getGraphEdge());

		// Check whether the connector name is set
		if (connector.isSetName())
		{
			if (((GraphEdge) getModel()).getAnchor().size() != 0)
			{
				updateSrcCountLabel();
				updateTargetCountLabel();
				updateMiddleNameLabel();
			}
		}
	}

	/**
	 * Retrieves the multiplicity text that should be displayed
	 * 
	 * @param lowerBound the LowerBound text value
	 * @param upperBound the UpperBound text value ("*" represent an Unbounded value)
	 * @return the string value to display
	 */
	private String getMultiplicityLabel(String lowerBound, String upperBound)
	{
		if ("-1".equals(upperBound))
		{
			upperBound = "*";
		}

		if (lowerBound.equals(upperBound))
		{
			return lowerBound;
		}
		else if ("0".equals(lowerBound) && "*".equals(upperBound))
		{
			return "*";
		}
		return lowerBound + ".." + upperBound;
	}

	/**
	 * Updates the srcCount Label
	 * 
	 * @generated NOT
	 */
	private void updateSrcCountLabel()
	{
		Label sourceCount = (Label) ((ConnectorFigure) getFigure()).getSrcCountEdgeObjectFigure();
		Connector theConnector = (Connector) Utils.getElement(getGraphEdge());
		ConnectorEnd srcConnectorEnd = ((ConnectorEnd) theConnector.getEnds().get(0));
		String upperBound = new Integer(srcConnectorEnd.getUpper()).toString();
		String lowerBound = new Integer(srcConnectorEnd.getLower()).toString();
		sourceCount.setText(getMultiplicityLabel(lowerBound, upperBound));
	}

	/**
	 * Updates the targetCount Label
	 * 
	 * @generated NOT
	 */
	private void updateTargetCountLabel()
	{
		Label sourceCount = (Label) ((ConnectorFigure) getFigure()).getTargetCountEdgeObjectFigure();
		Connector theConnector = (Connector) Utils.getElement(getGraphEdge());
		ConnectorEnd targetConnectorEnd = ((ConnectorEnd) theConnector.getEnds().get(1));
		String upperBound = new Integer(targetConnectorEnd.getUpper()).toString();
		String lowerBound = new Integer(targetConnectorEnd.getLower()).toString();
		sourceCount.setText(getMultiplicityLabel(lowerBound, upperBound));
	}
	
    /**
     * Updates the label of the connector name<br>
     * 
     * Pattern => Name : Type
     * 
     * 
     * 
     * @generated NOT
     */
    private void updateMiddleNameLabel()
    {
        Connector connector = (Connector) getEObject();
        String connectorName = connector.getName() == null ? "" : connector.getName();
        
        String connectorType = "";
        if (connector.getType() != null) 
        {
        	connectorType = " : " + connector.getType().getLabel();
        }
        else 
        {
        	connectorType = "";
        }
        
        ((Label) ((ConnectorFigure) getFigure()).getMiddleNameEdgeObjectFigure()).setText(connectorName + connectorType);
    }

	/**
	 * Activates
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
	 */
	public void activate()
	{
		super.activate();
		EObject model = getEObject();
		if (model instanceof Connector)
		{
			listenProperties((Connector) model);
		}
	}

	/**
	 * Desactivates
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
	 */
	public void deactivate()
	{
		EObject model = getEObject();
		if (model instanceof Connector)
		{
			unlistenProperties((Connector) model);
		}
		super.deactivate();
	}

	/**
	 * Listens to all the properties associated with the association
	 * 
	 * @param connector the model object
	 */
	private void listenProperties(Connector connector)
	{
		if (connector.getType() != null && !connector.getType().eAdapters().contains(getModelListener()))
		{
			connector.getType().eAdapters().add(getModelListener());
		}
		
		for (ConnectorEnd end : connector.getEnds())
		{
			// Only listen to object that are not yet listened
			if (!end.eAdapters().contains(getModelListener()))
			{
				end.eAdapters().add(getModelListener());
			}
			listenValues(end);
		}
	}

	/**
	 * Stops listening to all the redefined operations of the operation
	 * 
	 * @param connector the model object
	 */
	private void unlistenProperties(Connector connector)
	{
		for (ConnectorEnd end : connector.getEnds())
		{
			unlistenValues(end);
			end.eAdapters().remove(getModelListener());
		}
		
		if (connector.getType() != null)
		{
			connector.getType().eAdapters().remove(getModelListener());
		}
	}

	/**
	 * Listens to the bounds of the property
	 * 
	 * @param property the model object
	 */
	private void listenValues(ConnectorEnd end)
	{
		// Only listen to object that are not yet listened
		if (end.getLowerValue() != null && !end.getLowerValue().eAdapters().contains(getModelListener()))
		{
			end.getLowerValue().eAdapters().add(getModelListener());
		}
		if (end.getUpperValue() != null && !end.getUpperValue().eAdapters().contains(getModelListener()))
		{
			end.getUpperValue().eAdapters().add(getModelListener());
		}
	}

	/**
	 * Stops listening to the bounds of the property
	 * 
	 * @param property the model object
	 */
	private void unlistenValues(ConnectorEnd end)
	{
		if (end.getLowerValue() != null)
		{
			end.getLowerValue().eAdapters().remove(getModelListener());
		}
		if (end.getUpperValue() != null)
		{
			end.getUpperValue().eAdapters().remove(getModelListener());
		}
	}
}