/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.PolylineDecoration;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.CollaborationUse;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.VisibilityKind;
import org.topcased.draw2d.figures.Label;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.policies.EdgeObjectOffsetEditPolicy;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.uml.UMLLabel;
import org.topcased.modeler.uml.compositestructuresdiagram.CompositeStructuresEdgeObjectConstants;
import org.topcased.modeler.uml.compositestructuresdiagram.figures.DependencyFigure;
import org.topcased.modeler.uml.compositestructuresdiagram.policies.DependencyEdgeObjectUVEditPolicy;
import org.topcased.modeler.utils.Utils;

/**
 * Dependency controller
 * 
 * @generated
 */
public class DependencyEditPart extends EMFGraphEdgeEditPart
{

	/**
	 * Constructor
	 * 
	 * @param model the graph object
	 * @generated
	 */
	public DependencyEditPart(GraphEdge model)
	{
		super(model);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 * @generated NOT
	 */
	protected void createEditPolicies()
	{
		super.createEditPolicies();

		installEditPolicy(ModelerEditPolicyConstants.CHANGE_FONT_EDITPOLICY, null);

		installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_UV_EDITPOLICY, new DependencyEdgeObjectUVEditPolicy());

		installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_OFFSET_EDITPOLICY, new EdgeObjectOffsetEditPolicy());
	}

	/**
	 * @param connection the PolylineConnection
	 * @generated NOT
	 */
	private void createTargetDecoration(PolylineConnection connection)
	{
		PolylineDecoration decoration = new PolylineDecoration();
		decoration.setScale(10, 5);
		connection.setTargetDecoration(decoration);
	}

	/**
	 * @return the Figure
	 * @generated
	 */
	protected IFigure createFigure()
	{
		DependencyFigure connection = new DependencyFigure();
		createTargetDecoration(connection);
		return connection;
	}

	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#getEdgeObjectFigure(org.topcased.modeler.di.model.EdgeObject)
	 * @generated
	 */
	public IEdgeObjectFigure getEdgeObjectFigure(EdgeObject edgeObject)
	{
		if (CompositeStructuresEdgeObjectConstants.TARGETNAME_EDGE_OBJECT_ID.equals(edgeObject.getId()))
		{
			return ((DependencyFigure) getFigure()).getTargetNameEdgeObjectFigure();
		}
		return null;
	}

	/**
	 * @see org.topcased.modeler.edit.GraphEdgeEditPart#refreshEdgeObjects()
	 * 
	 * @generated NOT
	 */
	protected void refreshEdgeObjects()
	{
		super.refreshEdgeObjects();

		Dependency dependency = (Dependency) Utils.getElement(getGraphEdge());

		// Check whether the connector name is set
		if (dependency.isSetName())
		{
			if (((GraphEdge) getModel()).getAnchor().size() != 0)
			{
				updateTargetNameLabel();
			}
		}
	}

	/**
	 * Updates the targetCount Label
	 * 
	 * @generated NOT
	 */
	private void updateTargetNameLabel()
	{
		Label labelTarget = (Label) ((DependencyFigure) getFigure()).getTargetNameEdgeObjectFigure();
		Dependency dependency = (Dependency) Utils.getElement(getGraphEdge());
		Collaboration collaboration = (Collaboration) ((CollaborationUse) dependency.eContainer()).getType();
		if (!collaboration.getOwnedAttributes().isEmpty())
		{
			labelTarget.setText(getFullPropertyName((Property) collaboration.getOwnedAttributes().get(0)));
		}

	}

	/**
	 * Returns the full property name including visibility
	 * 
	 * @return the full name with the visibility
	 */
	private String getFullPropertyName(Property property)
	{
		String name = property.getName();
		VisibilityKind visibility = property.getVisibility();
		return UMLLabel.getVisibilityNotation(visibility).concat(" ").concat(name);
	}

	/**
	 * Activates
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
	 */
	public void activate()
	{
		super.activate();
		EObject model = getEObject();
		if (model instanceof Dependency)
		{
			listenProperties((Dependency) model);
		}
	}

	/**
	 * Desactivates
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
	 */
	public void deactivate()
	{
		EObject model = getEObject();
		if (model instanceof Dependency)
		{
			unlistenProperties((Dependency) model);
		}
		super.deactivate();
	}

	/**
	 * Listens to all the properties associated with the association
	 * 
	 * @param connector the model object
	 */
	private void listenProperties(Dependency dependency)
	{
		for (NamedElement elt : dependency.getSuppliers())
		{
			Property prop = (Property) elt;

			// Only listen to object that are not yet listened
			if (!prop.eAdapters().contains(getModelListener()))
			{
				prop.eAdapters().add(getModelListener());
			}

			listenValues(prop);
		}
	}

	/**
	 * Stops listening to all the redefined operations of the operation
	 * 
	 * @param connector the model object
	 */
	private void unlistenProperties(Dependency dependency)
	{
		for (NamedElement elt : dependency.getSuppliers())
		{
			Property prop = (Property) elt;

			unlistenValues(prop);

			prop.eAdapters().remove(getModelListener());
		}
	}

	/**
	 * Listens to the bounds of the property
	 * 
	 * @param property the model object
	 */
	private void listenValues(Property prop)
	{
		// Only listen to object that are not yet listened
		if (prop != null && !prop.eAdapters().contains(getModelListener()))
		{
			prop.eAdapters().add(getModelListener());
		}
	}

	/**
	 * Stops listening to the bounds of the property
	 * 
	 * @param property the model object
	 */
	private void unlistenValues(Property prop)
	{
		if (prop != null)
		{
			prop.eAdapters().remove(getModelListener());
		}
	}

}