/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Port;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.alldiagram.edit.ElementEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.CompositeStructuresEditPolicyConstants;
import org.topcased.modeler.uml.compositestructuresdiagram.commands.PortRestoreConnectionCommand;
import org.topcased.modeler.uml.compositestructuresdiagram.figures.PortFigure;
import org.topcased.modeler.uml.compositestructuresdiagram.figures.internal.InternalPortFigure;
import org.topcased.modeler.uml.compositestructuresdiagram.policies.ConnectorEdgeCreationEditPolicy;
import org.topcased.modeler.uml.compositestructuresdiagram.policies.DependencyEdgeCreationEditPolicy;
import org.topcased.modeler.uml.compositestructuresdiagram.policies.InterfaceRealizationEdgeCreationEditPolicy;
import org.topcased.modeler.uml.compositestructuresdiagram.policies.UsageEdgeCreationEditPolicy;

/**
 * The Port object controller
 * 
 * @generated
 */
public class PortEditPart extends ElementEditPart
{
	/**
	 * Constructor
	 * 
	 * @param obj the graph node
	 * @generated
	 */
	public PortEditPart(GraphNode obj)
	{
		super(obj);
	}

	/**
	 * Creates edit policies and associates these with roles
	 * 
	 * @generated
	 */
	protected void createEditPolicies()
	{
		super.createEditPolicies();

		installEditPolicy(CompositeStructuresEditPolicyConstants.CONNECTOR_EDITPOLICY, new ConnectorEdgeCreationEditPolicy());
		
		installEditPolicy(CompositeStructuresEditPolicyConstants.DEPENDENCY_EDITPOLICY, new DependencyEdgeCreationEditPolicy());
		
		installEditPolicy(CompositeStructuresEditPolicyConstants.INTERFACEREALIZATION_EDITPOLICY,
				new InterfaceRealizationEdgeCreationEditPolicy());

		installEditPolicy(CompositeStructuresEditPolicyConstants.USAGE_EDITPOLICY, new UsageEdgeCreationEditPolicy());
		
		installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
		{
			protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
			{
				return new PortRestoreConnectionCommand(getHost());
			}
		});

		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * @generated NOT
	 */
	protected IFigure createFigure()
	{
		return new PortFigure();
	}
	
	/**
	 * Refreshes the Port Figure when property value of <b>'isBehavior'</b> change.
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshVisuals()
	 */
	protected void refreshVisuals()
	{
		super.refreshVisuals();
		
		Port port = (Port) getEObject();
		
		PortFigure labelledPort = (PortFigure) getFigure();
		InternalPortFigure internalPortFigure = (InternalPortFigure) labelledPort.getPortFigure();
		internalPortFigure.setIsBehavior(port.isBehavior());
	}
	
	/**
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
	 */
	protected void refreshHeaderLabel()
	{
		ComposedLabel portLabel = ((ComposedLabel) getLabel());
		
		Port port = (Port) getEObject();

		// Updates the name of the Port
		portLabel.setPrefix(port.getName());

		// Updates the Type
		if (port.getType() != null) 
		{
			portLabel.setMain(" : ".concat(port.getType().getName()));
		}
		else
		{
			portLabel.setMain("");
		}
	}
	
	/**
	 * The editable part of the label is the prefix
	 * 
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getEditableLabel()
	 */
	public ILabel getEditableLabel()
	{
		return ((ComposedLabel) getLabel()).getPrefix();
	}
	
	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getAnchorableFigure()
	 */
	protected IFigure getAnchorableFigure()
	{
		return ((PortFigure) getFigure()).getPortFigure();
	}
	
	/**
	 * Activates
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
	 */
	public void activate()
	{
		super.activate();
		EObject model = getEObject();
		if (model instanceof Port)
		{
			listenValues((Port) model);
		}
	}

	/**
	 * Desactivates
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
	 */
	public void deactivate()
	{
		EObject model = getEObject();
		if (model instanceof Port)
		{
			unlistenValues((Port) model);
		}
		super.deactivate();
	}

	/**
	 * Listens to the bounds of the property
	 * 
	 * @param property the model object
	 */
	private void listenValues(Port port)
	{
		if (port.getType() != null && !port.getType().eAdapters().contains(getModelListener()))
		{
			port.getType().eAdapters().add(getModelListener());
		}
	}

	/**
	 * Stops listening to the bounds of the property
	 * 
	 * @param property the model object
	 */
	private void unlistenValues(Port port)
	{
		if (port.getType() != null && !port.getType().eAdapters().contains(getModelListener()))
		{
			port.getType().eAdapters().remove(getModelListener());
		}
	}
	
}