/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.edit;

import java.util.Iterator;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.SWT;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.commands.CompoundCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.alldiagram.edit.ElementEditPart;
import org.topcased.modeler.uml.compositestructuresdiagram.CompositeStructuresEditPolicyConstants;
import org.topcased.modeler.uml.compositestructuresdiagram.commands.PropertyRestoreConnectionCommand;
import org.topcased.modeler.uml.compositestructuresdiagram.commands.UpdatePortCommand;
import org.topcased.modeler.uml.compositestructuresdiagram.figures.PropertyFigure;
import org.topcased.modeler.uml.compositestructuresdiagram.policies.ConnectorEdgeCreationEditPolicy;
import org.topcased.modeler.uml.compositestructuresdiagram.policies.DependencyEdgeCreationEditPolicy;
import org.topcased.modeler.uml.compositestructuresdiagram.policies.PropertyLayoutEditPolicy;

/**
 * The Property object controller
 * 
 * @generated
 */
public class PropertyEditPart extends ElementEditPart
{
	/**
	 * Constructor
	 * 
	 * @param obj the graph node
	 * @generated
	 */
	public PropertyEditPart(GraphNode obj)
	{
		super(obj);
	}

	/**
	 * Creates edit policies and associates these with roles
	 * 
	 * @generated
	 */
	protected void createEditPolicies()
	{
		super.createEditPolicies();

		installEditPolicy(CompositeStructuresEditPolicyConstants.CONNECTOR_EDITPOLICY, new ConnectorEdgeCreationEditPolicy());

		installEditPolicy(CompositeStructuresEditPolicyConstants.DEPENDENCY_EDITPOLICY, new DependencyEdgeCreationEditPolicy());

		installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
		{
			protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
			{
				// restore Ports
				UpdatePortCommand cmdNode = new UpdatePortCommand(getHost(), getGraphNode());
				if (cmdNode != null && cmdNode.canExecute())
				{
					cmdNode.execute();
				}
				
				CompoundCommand compCmd = new CompoundCommand();
				compCmd.add(new PropertyRestoreConnectionCommand(getHost()));
				for (Iterator<?> iter = getHost().getChildren().iterator(); iter.hasNext();)
		        {
		            EditPart editPart = (EditPart) iter.next();
		            Command cmd = editPart.getCommand(request);
		            if (cmd != null && cmd.canExecute())
		            {
		            	compCmd.add(cmd);
		            }
		        }
		        return compCmd;
			}
		});

		installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());

		installEditPolicy(EditPolicy.LAYOUT_ROLE, new PropertyLayoutEditPolicy());

		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * @generated
	 */
	protected IFigure createFigure()
	{
		return new PropertyFigure();
	}

	/**
	 * The editable part of the label is the prefix
	 * 
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getEditableLabel()
	 */
	public ILabel getEditableLabel()
	{
		return ((ComposedLabel) getLabel()).getPrefix();
	}

	/**
	 * Refreshes the border of the Figure according to the aggregation kind of the Property.
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshVisuals()
	 */
	protected void refreshVisuals()
	{
		super.refreshVisuals();

		Property property = (Property) getEObject();

		// Modify the style border of the figure if necessary
		switch (property.getAggregation().getValue())
		{
		case AggregationKind.NONE:
			((PropertyFigure) getFigure()).setBorderStyle(SWT.LINE_CUSTOM); // Role
			break;
		case AggregationKind.SHARED:
			((PropertyFigure) getFigure()).setBorderStyle(SWT.LINE_CUSTOM); // Role
			break;
		case AggregationKind.COMPOSITE:
			((PropertyFigure) getFigure()).setBorderStyle(SWT.LINE_SOLID); // Part
			break;
		default:
			break;
		}
	}

	/**
	 * Refresh the Property label
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
	 */
	protected void refreshHeaderLabel()
	{
		ComposedLabel propertyLabel = ((ComposedLabel) getLabel());

		Property property = (Property) getEObject();

		// Updates the name of the Property
		propertyLabel.setPrefix(property.getName());
		propertyLabel.setMain("");
		propertyLabel.setSuffix("");

		// Updates the Type
		if (property.getType() != null)
		{
			propertyLabel.setMain(" : ".concat(property.getType().getName()));

			// Updates the cardinalities
			if (property.getLower() >= 0 && property.getUpper() != 1)
			{
				String lowerBound = new Integer(property.getLower()).toString();
				String upperBound = null;
				if (property.getUpper() < 0)
				{
					upperBound = "*";
				}
				else
				{
					upperBound = new Integer(property.getUpper()).toString();
				}
				propertyLabel.setSuffix(" ".concat(getMultiplicityLabel(lowerBound, upperBound)));
			}
		}
	}

	/**
	 * @see org.topcased.modeler.edit.GraphNodeEditPart#getAnchorableFigure()
	 */
	protected IFigure getAnchorableFigure()
	{
		return ((PropertyFigure) getFigure()).getBackgroundFigure();
	}

	/**
	 * Retrieves the multiplicity text that should be displayed
	 * 
	 * @param lowerBound the LowerBound text value
	 * @param upperBound the UpperBound text value ("*" represent an Unbounded value)
	 * @return the string value to display
	 */
	private String getMultiplicityLabel(String lowerBound, String upperBound)
	{
		if (lowerBound.equals(upperBound))
		{
			return "[" + lowerBound + "]";
		}
		else if ("0".equals(lowerBound) && "*".equals(upperBound))
		{
			return "[*]";
		}
		return "[" + lowerBound + ".." + upperBound + "]";
	}

	/**
	 * Activates
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
	 */
	public void activate()
	{
		super.activate();
		EObject model = getEObject();
		if (model instanceof Property)
		{
			listenValues((Property) model);
		}
	}

	/**
	 * Desactivates
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
	 */
	public void deactivate()
	{
		EObject model = getEObject();
		if (model instanceof Property)
		{
			unlistenValues((Property) model);
		}
		super.deactivate();
	}

	/**
	 * Listens to the bounds of the property
	 * 
	 * @param property the model object
	 */
	private void listenValues(Property property)
	{
		// Only listen to object that are not yet listened
		if (property.getLowerValue() != null && !property.getLowerValue().eAdapters().contains(getModelListener()))
		{
			property.getLowerValue().eAdapters().add(getModelListener());
		}
		if (property.getUpperValue() != null && !property.getUpperValue().eAdapters().contains(getModelListener()))
		{
			property.getUpperValue().eAdapters().add(getModelListener());
		}
		if (property.getType() != null && !property.getType().eAdapters().contains(getModelListener()))
		{
			property.getType().eAdapters().add(getModelListener());
		}
	}

	/**
	 * Stops listening to the bounds of the property
	 * 
	 * @param property the model object
	 */
	private void unlistenValues(Property property)
	{
		if (property.getLowerValue() != null)
		{
			property.getLowerValue().eAdapters().remove(getModelListener());
		}
		if (property.getUpperValue() != null)
		{
			property.getUpperValue().eAdapters().remove(getModelListener());
		}
		if (property.getType() != null && !property.getType().eAdapters().contains(getModelListener()))
		{
			property.getType().eAdapters().remove(getModelListener());
		}
	}

	/**
	 * Handles Port insertion in Property
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
	 */
	protected void handleModelChanged(Notification msg)
	{
		// The Port Update Command will be called only if the Type affected change.
		if (msg.getOldValue() instanceof Type || msg.getNewValue() instanceof Type)
		{
			// Handles the refresh of the Port attached to a Property
			UpdatePortCommand updatePortGroupFeaturesCommand = new UpdatePortCommand(this, getGraphNode());
			if (updatePortGroupFeaturesCommand != null && updatePortGroupFeaturesCommand.canExecute())
			{
				updatePortGroupFeaturesCommand.execute();
			}
		}
		super.handleModelChanged(msg);
	}
	
	/**
	 * This method enables to create a new diagram or view an existing model diagrams.<br>
	 * It will be used when a <b>Property Figure</b> will be clicked and will propose to detail the classifier representing by this type. 
	 * 
	 * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#createChangeDiagramAction(org.eclipse.emf.ecore.EObject)
	 */
	protected IAction createChangeDiagramAction(EObject modelObject)
	{
		Type propertyType = ((Property) getEObject()).getType();
        if (propertyType != null && !"".equals(propertyType.getName()))
        {
            return super.createChangeDiagramAction(propertyType);
        }
        return null;
	}
}