/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.figures;

import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutManager;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.draw2d.ToolbarLayout;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.IContainerFigure;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.draw2d.figures.Label;
import org.topcased.draw2d.layout.BorderAttachedLayout;

/**
 * Represents a <b>Class Figure</b> that is a simple rectangle aimed to contain other objects such as <b>Interface, Port, Property and CollaborationUse Figures</b>.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 * @generated
 */
public class ClassFigure extends org.eclipse.draw2d.Figure implements IContainerFigure, ILabelFigure
{
	private ILabel label;

	private IFigure contentPane;

	private IFigure innerRectangle;

	/**
	 * Creates the contents of the figure : by default, it creates a layout manager, a header and a container
	 * 
	 * @generated NOT
	 */
	public ClassFigure()
	{
		LayoutManager layout = new StackLayout();
		setLayoutManager(layout);

		add(createBackgroundFigure());

		contentPane = new Figure();
		contentPane.setLayoutManager(new BorderAttachedLayout());
		add(contentPane);
	}

	/**
	 * Draws an inner rectangle that will be the real background figure. The Anchor will be attached to this inner rectangle.<br>
	 * 
	 * @return the figure
	 */
	protected IFigure createBackgroundFigure()
	{
		IFigure rectangle = new Figure();
		rectangle.setLayoutManager(new BorderLayout());
		rectangle.setBorder(new MarginBorder(7));

		innerRectangle = new RectangleFigure();
		innerRectangle.setOpaque(true);
		rectangle.add(innerRectangle, new Integer(PositionConstants.CENTER));

		ToolbarLayout toolbarLayout = new ToolbarLayout();
		toolbarLayout.setMinorAlignment(ToolbarLayout.ALIGN_CENTER);
		innerRectangle.setLayoutManager(toolbarLayout);

		IFigure innerLabelRectangle = new RectangleFigure();
		innerLabelRectangle.setBorder(new LineBorder());
		innerLabelRectangle.setLayoutManager(new BorderLayout());

		label = createLabel();
		innerLabelRectangle.add(label, new Integer(PositionConstants.CENTER));
		innerRectangle.add(innerLabelRectangle);
		return rectangle;
	}

	/**
	 * Returns the figure drawn behind the label. In this case, we only return the inner rectangle drawn.
	 * 
	 * @return the background figure
	 */
	public IFigure getBackgroundFigure()
	{
		return innerRectangle;
	}

	/**
	 * Creates the label of the Figure.<br>
	 * 
	 * @return the label of the Figure
	 */
	protected ILabel createLabel()
	{
		EditableLabel lbl = new EditableLabel();
		lbl.setBorder(new MarginBorder(5));
		lbl.setLabelAlignment(PositionConstants.MIDDLE);
		ComposedLabel composedLbl = new ComposedLabel(new Label(), lbl, null, false);
		return composedLbl;
	}

	/*
	 * @see org.topcased.draw2d.figures.ILabelFigure#getLabel()
	 */
	public ILabel getLabel()
	{
		return label;
	}

	/*
	 * @see org.topcased.draw2d.figures.IContainerFigure#getContentPane()
	 */
	public IFigure getContentPane()
	{
		return contentPane;
	}

}