/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.figures;

import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutManager;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.draw2d.ToolbarLayout;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.IContainerFigure;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.draw2d.layout.BorderAttachedLayout;

/**
 * Represents a <b>Collaboration Figure</b> that is a simple ellipse aimed to contain other objects such as <b>Interface, Property and CollaborationUse Figures</b>.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 * @generated
 */
public class CollaborationFigure extends Ellipse implements IContainerFigure, ILabelFigure
{
	/** Name given to the collaboration */
	private ILabel label;
	
	/** Border constant line ON */
	private static final int LINE_ON = 8;

	/** Border constant line OFF */
	private static final int LINE_OFF = 7;

	/**The custom border style */
	private int[] dashes = new int[] { LINE_ON, LINE_OFF };

	/** The client area */
	private IFigure contentPane;

	/**
	 * Constructor
	 * 
	 * @generated NOT
	 */
	public CollaborationFigure()
	{
		LayoutManager layout = new StackLayout();
		setLayoutManager(layout);

		add(createBackgroundFigure());
		
		contentPane = new Figure();
		contentPane.setLayoutManager(new BorderAttachedLayout());
		
		add(contentPane);
		
	}

	/**
	 * Draw an innerRectangle that will be the real background figure. The Anchor will be attached to this inner rectangle.<br>
	 * The innerRectangle is also the container of the HeaderComponent and the Label of the Component that are arranged in a toolbarLayout.
	 * 
	 * @return the figure
	 */
	protected IFigure createBackgroundFigure()
	{
		label = new EditableLabel();
		
		ToolbarLayout toolbarLayout = new ToolbarLayout(false);
		toolbarLayout.setMinorAlignment(ToolbarLayout.ALIGN_CENTER);
		
		Figure innerEllipse = new Figure();
		innerEllipse.setBorder(new MarginBorder(10));
		innerEllipse.setLayoutManager(toolbarLayout);
		innerEllipse.setOpaque(false);
		innerEllipse.add(label);
		
		return innerEllipse;
	}
	
	/**
	 * @see org.topcased.draw2d.figures.ILabelFigure#getLabel()
	 * @generated
	 */
	public ILabel getLabel()
	{
		return label;
	}

	/**
	 * @see org.topcased.draw2d.figures.IContainerFigure#getContentPane()
	 */
	public IFigure getContentPane()
	{
		return contentPane;
	}

	/**
	 * The border of the ellipse is represented using a dash pattern
	 * 
	 * @see org.eclipse.draw2d.Ellipse#outlineShape(org.eclipse.draw2d.Graphics)
	 */
	protected void outlineShape(Graphics graphics)
	{
		graphics.setLineDash(dashes);
		super.outlineShape(graphics);
	}
}