/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.figures;

import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.swt.SWT;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.draw2d.figures.Label;
import org.topcased.draw2d.layout.CenterLayout;

/**
 * Represents a <b>'CollaborationUse'</b> Figure.<br>
 * This figure is represented with a simple ellipse pattern.<br>
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class CollaborationUseFigure extends Ellipse implements ILabelFigure
{
	/** Border constant for line ON */
	private static final int LINE_ON = 8;

	/** Border constant for line OFF */
	private static final int LINE_OFF = 7;

	/** Defines the border style */
	private int[] dashes = new int[] { LINE_ON, LINE_OFF };
	
	/** Value for the border margin */
	private static final int MARGIN_CONSTANT = 15; 
	
	/** Centered label of this Ellipse */
	private ILabel label;

	/**
	 * Constructor
	 * 
	 * @generated
	 */
	public CollaborationUseFigure()
	{
		setLayoutManager(new CenterLayout());
		setLineStyle(SWT.LINE_DASH);
        setBorder(new MarginBorder(MARGIN_CONSTANT));
		label = createLabel();
        add(label);
	}

	/**
	 * Creates the label of the CollaborationUse figure.<br>
	 * 
	 * @return the label of the figure
	 * @generated NOT
	 */
	protected ILabel createLabel()
	{
		// Prefix : the collaborationUse name
		// Main : only the character ':'
		// Suffix : the Type affected to the collaborationUse i.e a Collaboration
		return new ComposedLabel(new EditableLabel(), new Label(), new Label(), true);
	}
	
	/**
	 * @see org.topcased.draw2d.figures.ILabelFigure#getLabel()
	 * @generated
	 */
	public ILabel getLabel()
	{
		return label;
	}
	
    /**
     * Changes the label object
     * 
     * @param newLabel the new object
     */
    public void setLabel(ILabel newLabel)
    {
        label = newLabel;
    }
    
	/** 
	 * The outline of the ellipse is represented using a custom dash border
	 * 
	 * @see org.eclipse.draw2d.Ellipse#outlineShape(org.eclipse.draw2d.Graphics)
	 */
	protected void outlineShape(Graphics graphics)
	{
		graphics.setLineDash(dashes);
		super.outlineShape(graphics);
	}
}