/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.figures;

import org.eclipse.draw2d.ConnectionEndpointLocator;
import org.eclipse.draw2d.Locator;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.handles.HandleBounds;
import org.eclipse.gmf.runtime.draw2d.ui.figures.PolylineConnectionEx;
import org.eclipse.swt.SWT;
import org.topcased.modeler.figures.EdgeObjectEditableLabel;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.figures.IGraphEdgeFigure;

/**
 * Represents the Dependency Figure.<br>
 * 
 * The 'Dependency' has a role label attached to the target point.<br>
 * The Figure is drawn using a dash style.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class DependencyFigure extends PolylineConnectionEx implements IGraphEdgeFigure, HandleBounds
{
	/**
	 * @generated
	 */
	private static final int LINE_ON = 8;

	/**
	 * @generated
	 */
	private static final int LINE_OFF = 7;

	/**
	 * @generated
	 */
	private IEdgeObjectFigure targetNameEdgeObject;

	/**
	 * @generated
	 */
	private Locator targetNameLocator;

	/**
	 * The constructor
	 * 
	 * @generated
	 */
	public DependencyFigure()
	{
		super();

		setLineStyle(SWT.LINE_CUSTOM);
		setLineDash(new int[] { LINE_ON, LINE_OFF });

		targetNameEdgeObject = new EdgeObjectEditableLabel(this);
		targetNameLocator = new ConnectionEndpointLocator(this, true);
		add(targetNameEdgeObject, targetNameLocator);
	}

	/**
	 * @see org.topcased.modeler.figures.IGraphEdgeFigure#getLocator(org.topcased.modeler.figures.IEdgeObjectFigure)
	 * @generated
	 */
	public Locator getLocator(IEdgeObjectFigure edgeObjectfigure)
	{
		if (edgeObjectfigure == targetNameEdgeObject)
		{
			return targetNameLocator;
		}
		return null;
	}

	/**
	 * @see org.eclipse.gef.handles.HandleBounds#getHandleBounds()
	 * @generated
	 */
	public Rectangle getHandleBounds()
	{
		return getPoints().getBounds();
	}

	/**
	 * @return
	 */
	public IEdgeObjectFigure getTargetNameEdgeObjectFigure()
	{
		return targetNameEdgeObject;
	}
}