/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.figures;

import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.GraphicWithLabelFigure;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.Label;

/**
 * Represents the 'Interface' Figure.<br>
 * The figure is composed by a label (top) and a circle figure (bottom).
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class InterfaceFigure extends GraphicWithLabelFigure
{
	/**
	 * Constructor
	 */
	public InterfaceFigure()
	{
		super(PositionConstants.TOP);
	}

	/**
	 * Creates an Ellipse and set its default size to {20, 20}
	 * 
	 * @see org.topcased.draw2d.figures.GraphicWithLabelFigure#createBodyFigure()
	 */
	protected IFigure createBodyFigure()
	{
		return new Ellipse()
		{
			/**
			 * @see org.eclipse.draw2d.Figure#getPreferredSize(int, int)
			 */
			public Dimension getPreferredSize(int wHint, int hHint)
			{
				return new Dimension(20, 20);
			}
		};
	}

	/**
	 * Indicate whether the body figure should be visible or not
	 * 
	 * @param isVisible
	 */
	public void setDisplayBodyFigure(boolean isVisible)
	{
		getBodyFigure().setVisible(isVisible);
	}

	/**
	 * @see org.topcased.draw2d.figures.GraphicWithLabelFigure#createLabel()
	 */
	protected ILabel createLabel()
	{
		ComposedLabel composedLbl = new ComposedLabel(new Label(), new EditableLabel(), null, false);
		composedLbl.setBorder(new MarginBorder(10));
		return composedLbl;
	}
}