/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.figures.internal;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Represents a <b>'Port'</b> Figure.<br>
 * The figure is represented with a white square attached to the border of its container.<br>
 * A small state symbol is drawn inside the Port Figure when the Port property <b>'isBehavior'</b> is set to <b>true</b>.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class InternalPortFigure extends org.topcased.draw2d.figures.PortFigure
{
	/** Kind of the Port */
	private boolean isBehavior = false; 
	
    /** Constant for drawing a rounded rectangle figure */
	private static final int ROUND_CORNER = 5;
    
	/**
	 * Constructor 
	 */
	public InternalPortFigure()
	{
		super();
	}
	
	/*
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint)
	{
		return new Dimension(14, 14);
	}
	
	/**
	 * Setter for the attribute <b>'isBehavior'</b> of a Port
	 * 
	 * @param state The current state of a Port (structure or behavior)
	 */
	public void setIsBehavior(boolean state)
	{
		isBehavior = state;
	}
	
	/**
	 * An internal Figure is drawn inside the Port Figure when the attribute <b>'isBehavior'</b> is set to <b>true</b>.
	 * 
	 * @see org.topcased.draw2d.figures.PortFigure#outlineShape(org.eclipse.draw2d.Graphics)
	 */
	protected void outlineShape(Graphics graphics)
	{
		super.outlineShape(graphics);
		
		// draws an internal symbol when the Port is behavior
		/** FIXME : Temporary representation of behavior Port : it will evolve in the next future. 
		 * The UML2 norm foresee another representation of Port behavior */
		if (isBehavior)
		{
	        Rectangle outer = Rectangle.SINGLETON;
	        outer.x = getBounds().x + lineWidth / 2 + 2;
	        outer.y = getBounds().y + lineWidth / 2 + 5;
	        outer.width =  10 - lineWidth;
	        outer.height = 5 - lineWidth;
	        graphics.drawRoundRectangle(outer, ROUND_CORNER, ROUND_CORNER);
		}
	}
}