/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.policies;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.uml2.uml.CollaborationUse;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy;
import org.topcased.modeler.uml.compositestructuresdiagram.commands.CreateCollaborationUseCommand;
import org.topcased.modeler.uml.compositestructuresdiagram.commands.CreateInterfaceCommand;
import org.topcased.modeler.uml.compositestructuresdiagram.commands.CreatePortCommand;
import org.topcased.modeler.uml.compositestructuresdiagram.commands.CreatePropertyCommand;
import org.topcased.modeler.utils.Utils;

/**
 * @generated
 */
public class CompositeStructuresDiagramLayoutEditPolicy extends ModelerLayoutEditPolicy
{
	/**
	 * Default contructor.
	 * 
	 * @generated
	 */
	public CompositeStructuresDiagramLayoutEditPolicy()
	{
		super();
	}

	/**
	 * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isAttachedToBorder(org.topcased.modeler.di.model.GraphNode)
	 * @generated
	 */
	protected boolean isAttachedToBorder(GraphNode node)
	{
		if (Utils.getElement(node) instanceof Port)
		{
			return true;
		}
		return super.isAttachedToBorder(node);
	}
	
	/**
	 * Override the behavior were the Features component may not exist yet and should be created by the customized creation command
	 * 
	 * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#getCreateCommand(org.topcased.modeler.di.model.GraphNode,
	 *      org.topcased.modeler.di.model.GraphNode, org.eclipse.draw2d.geometry.Point, org.eclipse.draw2d.geometry.Dimension, int, boolean)
	 */
	protected Command getCreateCommand(GraphNode parent, GraphNode child, Point loc, Dimension dim, int pos, boolean needModelUpdate)
	{
		if (isValid(child, parent))
		{
			EditDomain domain = getHost().getViewer().getEditDomain();

			// get the EObjects of the model
			EObject parentEObject = Utils.getElement(parent);
			EObject childEObject = Utils.getElement(child);

			if (isValid(childEObject, parentEObject))
			{
				// handles Interface object
				if (childEObject instanceof Interface)
				{
					return new CreateInterfaceCommand(domain, child, parent, loc, dim, pos, needModelUpdate);
				}			
				
				// handles Port object
				if (childEObject instanceof Port)
				{
					return new CreatePortCommand(domain, child, parent, loc, dim, pos, needModelUpdate);
				}
				
				// handles Property object
				if (childEObject instanceof Property)
				{
					return new CreatePropertyCommand(domain, child, parent, loc, dim, pos, needModelUpdate);
				}
				
				// handles CollaborationUse object 
				if (childEObject instanceof CollaborationUse)
				{
					// creates the command associated with these parameters
					return new CreateCollaborationUseCommand(domain, child, parent, loc, dim, pos, needModelUpdate);
				}
				
				// handles CollaborationUse object
//				if (childEObject instanceof Class)
//				{
//					return new CreateNewPropertyCommand(domain, child, parent, loc, dim, pos, needModelUpdate);
//				}
				
				// default
				return super.getCreateCommand(parent, child, loc, dim, pos, needModelUpdate);
			}
		}

		return UnexecutableCommand.INSTANCE;
	}
	
	@Override
	protected boolean isSeveralDisplayAllowed(GraphNode parent, GraphNode child, boolean needModelUpdate)
	{
	    return false;
	}
}