/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.policies;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.InterfaceRealization;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy;
import org.topcased.modeler.uml.compositestructuresdiagram.commands.InterfaceRealizationEdgeCreationCommand;
import org.topcased.modeler.utils.SourceTargetData;
import org.topcased.modeler.utils.Utils;

/**
 * InterfaceRealization edge creation
 * 
 * @generated
 */
public class InterfaceRealizationEdgeCreationEditPolicy extends AbstractEdgeCreationEditPolicy
{
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#createCommand(org.eclipse.gef.EditDomain, org.topcased.modeler.di.model.GraphEdge,
	 *      org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected CreateTypedEdgeCommand createCommand(EditDomain domain, GraphEdge edge, GraphElement source)
	{
		return new InterfaceRealizationEdgeCreationCommand(domain, edge, source);
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkEdge(org.topcased.modeler.di.model.GraphEdge)
	 * @generated
	 */
	protected boolean checkEdge(GraphEdge edge)
	{
		return Utils.getElement(edge) instanceof InterfaceRealization;
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkSource(org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected boolean checkSource(GraphElement source)
	{
		return Utils.getElement(source) instanceof Interface;
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkTargetForSource(org.topcased.modeler.di.model.GraphElement,
	 *      org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected boolean checkTargetForSource(GraphElement source, GraphElement target)
	{
		EObject sourceObject = Utils.getElement(source);
		EObject targetObject = Utils.getElement(target);

		if (sourceObject instanceof Interface && targetObject instanceof Port)
		{
			if (!sourceObject.equals(targetObject))
			{
				if (Utils.getDiagramModelObject(target) instanceof StructuredClassifier)
				{
					return targetIsPortOfProperty(target) || targetIsPortOfStructuredClassifier(target);
				}
			}
		}
		return false;
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkCommand(org.eclipse.gef.commands.Command)
	 * @generated
	 */
	protected boolean checkCommand(Command command)
	{
		return command instanceof InterfaceRealizationEdgeCreationCommand;
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#getSourceTargetData(org.topcased.modeler.di.model.GraphElement,
	 *      org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected SourceTargetData getSourceTargetData(GraphElement source, GraphElement target)
	{
		EObject sourceObject = Utils.getElement(source);
		EObject targetObject = Utils.getElement(target);

		if (sourceObject instanceof Interface && targetObject instanceof Port)
		{
			return new SourceTargetData(false, false, SourceTargetData.TARGET, "org.eclipse.uml2.uml.Class", "interfaceRealization",
					"contract", null, null, null, null, null);
		}
		return null;
	}

	/**
	 * Method calls only for an internal approach.
	 * Checks the following condition :
	 * 
	 * <ul>
	 * <li>the target Port must be on a Property.</li>
	 * </ul>
	 * 
	 * @param target The target node
	 * @return <code>true</code> if the test is valid, <code>false</code> otherwise.
	 */
	private boolean targetIsPortOfProperty(GraphElement target)
	{
		// Gets the graphical container of the target graph node
		return Utils.getElement((GraphNode) target.eContainer()) instanceof Property;
	}
	
	/**
	 * Method calls only for an internal approach.
	 * Checks the following condition :
	 * 
	 * <ul>
	 * <li>the target Port must be on a StructuredClassifier.</li>
	 * </ul>
	 * 
	 * @param target The target node
	 * @return <code>true</code> if the test is valid, <code>false</code> otherwise.
	 */
	private boolean targetIsPortOfStructuredClassifier(GraphElement target)
	{
		// Gets the graphical container of the target graph node
		return Utils.getElement((GraphNode) target.eContainer()) instanceof StructuredClassifier;
	}
}