/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.policies;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.uml2.uml.EncapsulatedClassifier;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy;
import org.topcased.modeler.uml.compositestructuresdiagram.commands.CreatePortCommand;
import org.topcased.modeler.utils.Utils;

/**
 * Property Layout policies.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 * @generated
 */
public class PropertyLayoutEditPolicy extends ModelerLayoutEditPolicy
{
	/**
	 * Default contructor.
	 * 
	 * @generated
	 */
	public PropertyLayoutEditPolicy()
	{
		super();
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isAttachedToBorder(org.topcased.modeler.di.model.GraphNode)
	 * @generated NOT
	 */
	protected boolean isAttachedToBorder(GraphNode node)
	{
		return (Utils.getElement(node) instanceof Port);
	}
	
	/**
	 * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isValid(org.eclipse.emf.ecore.EObject, org.eclipse.emf.ecore.EObject)
	 * @generated NOT
	 */
	protected boolean isValid(EObject child, EObject parent)
	{
		// check if the child and the parent EObject are not null
		if (super.isValid(child, parent))
		{
			if (child instanceof Port)
			{
				Type propertyType = ((Property) parent).getType();
				return (propertyType != null && propertyType instanceof EncapsulatedClassifier);
			}
		}
		return false;
	}
	
	/**
	 * Override the behavior were the Features component may not exist yet and should be created by the customized creation command
	 * 
	 * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#getCreateCommand(org.topcased.modeler.di.model.GraphNode,
	 *      org.topcased.modeler.di.model.GraphNode, org.eclipse.draw2d.geometry.Point, org.eclipse.draw2d.geometry.Dimension, int, boolean)
	 */
	protected Command getCreateCommand(GraphNode parent, GraphNode child, Point loc, Dimension dim, int pos, boolean needModelUpdate)
	{
		EditDomain domain = getHost().getViewer().getEditDomain();

		// Get the EObjects of the model
		EObject parentEObject = Utils.getElement(parent);
		EObject childEObject = Utils.getElement(child);

		// The parent EObject must be a valid container of the child EObject
		if (isValid(childEObject, parentEObject))
		{
			// create the command associated with these parameters
			return new CreatePortCommand(domain, child, parent, loc, dim, pos, needModelUpdate);
		}

		return UnexecutableCommand.INSTANCE;
	}
}