/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.uml2.uml.util.UMLSwitch;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.uml.alldiagram.edit.CommentEditPart;
import org.topcased.modeler.uml.alldiagram.edit.CommentLinkEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.ArtifactEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.CommunicationPathEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.DependencyEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.DeployedArtifactEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.DeploymentDiagramEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.DeploymentEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.DeploymentSpecificationEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.DeploymentSpecificationLinkEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.DeploymentTargetEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.DeviceEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.ExecutionEnvironmentEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.ManifestationEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.NodeEditPart;
import org.topcased.modeler.uml.deploymentdiagram.edit.PackageableElementEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * Part Factory : associates a model object to its controller. <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class DeploymentEditPartFactory implements EditPartFactory
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
     * @generated
     */
    public EditPart createEditPart(EditPart context, Object model)
    {
        if (model instanceof Diagram)
        {
            return new DeploymentDiagramEditPart((Diagram) model);
        }
        else if (model instanceof GraphNode)
        {
            final GraphNode node = (GraphNode) model;
            EObject element = Utils.getElement(node);
            if (element != null)
            {
                if ("http://www.eclipse.org/uml2/2.1.0/UML".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new NodeUMLSwitch(node).doSwitch(element);
                }
                return new EMFGraphNodeEditPart(node);
            }
            return new GraphNodeEditPart(node);
        }
        else if (model instanceof GraphEdge)
        {
            final GraphEdge edge = (GraphEdge) model;
            EObject element = Utils.getElement(edge);
            if (element != null)
            {
                if ("http://www.eclipse.org/uml2/2.1.0/UML".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new EdgeUMLSwitch(edge).doSwitch(element);
                }
                return new EMFGraphEdgeEditPart(edge);
            }

            if (edge.getSemanticModel() instanceof SimpleSemanticModelElement)
            {
                // Manage the Element that are not associated with a model object
                if (DeploymentSimpleObjectConstants.SIMPLE_OBJECT_COMMENTLINK.equals(((SimpleSemanticModelElement) edge.getSemanticModel()).getTypeInfo()))
                {
                    return new CommentLinkEditPart(edge);
                }
                if (DeploymentSimpleObjectConstants.SIMPLE_OBJECT_DEPLOYMENTSPECIFICATIONLINK.equals(((SimpleSemanticModelElement) edge.getSemanticModel()).getTypeInfo()))
                {
                    return new DeploymentSpecificationLinkEditPart(edge);
                }
            }

            return new GraphEdgeEditPart(edge);
        }

        throw new IllegalStateException("No edit part matches with the '" + model.getClass().getName() + "' model element. Check DeploymentEditPartFactory#createEditPart(EditPart,Object) class");
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class NodeUMLSwitch extends UMLSwitch<Object>
    {
        /** The graphical node */
        private GraphNode node;

        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param node the graphical node
         * @generated
         */
        public NodeUMLSwitch(GraphNode node)
        {
            this.node = node;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseDeployedArtifact(org.eclipse.uml2.uml.DeployedArtifact)
         * @generated
         */
        public Object caseDeployedArtifact(org.eclipse.uml2.uml.DeployedArtifact object)
        {
            return new DeployedArtifactEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseDeploymentTarget(org.eclipse.uml2.uml.DeploymentTarget)
         * @generated
         */
        public Object caseDeploymentTarget(org.eclipse.uml2.uml.DeploymentTarget object)
        {
            return new DeploymentTargetEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#casePackageableElement(org.eclipse.uml2.uml.PackageableElement)
         * @generated
         */
        public Object casePackageableElement(org.eclipse.uml2.uml.PackageableElement object)
        {
            return new PackageableElementEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseNode(org.eclipse.uml2.uml.Node)
         * @generated
         */
        public Object caseNode(org.eclipse.uml2.uml.Node object)
        {
            return new NodeEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseDevice(org.eclipse.uml2.uml.Device)
         * @generated
         */
        public Object caseDevice(org.eclipse.uml2.uml.Device object)
        {
            return new DeviceEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseExecutionEnvironment(org.eclipse.uml2.uml.ExecutionEnvironment)
         * @generated
         */
        public Object caseExecutionEnvironment(org.eclipse.uml2.uml.ExecutionEnvironment object)
        {
            return new ExecutionEnvironmentEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseArtifact(org.eclipse.uml2.uml.Artifact)
         * @generated
         */
        public Object caseArtifact(org.eclipse.uml2.uml.Artifact object)
        {
            return new ArtifactEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseDeploymentSpecification(org.eclipse.uml2.uml.DeploymentSpecification)
         * @generated
         */
        public Object caseDeploymentSpecification(org.eclipse.uml2.uml.DeploymentSpecification object)
        {
            return new DeploymentSpecificationEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseComment(org.eclipse.uml2.uml.Comment)
         * @generated
         */
        public Object caseComment(org.eclipse.uml2.uml.Comment object)
        {
            return new CommentEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphNodeEditPart(node);
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class EdgeUMLSwitch extends UMLSwitch<Object>
    {
        /** The graphical edge */
        private GraphEdge edge;

        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param edge the graphical edge
         * @generated
         */
        public EdgeUMLSwitch(GraphEdge edge)
        {
            this.edge = edge;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseDeployment(org.eclipse.uml2.uml.Deployment)
         * @generated
         */
        public Object caseDeployment(org.eclipse.uml2.uml.Deployment object)
        {
            return new DeploymentEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseManifestation(org.eclipse.uml2.uml.Manifestation)
         * @generated
         */
        public Object caseManifestation(org.eclipse.uml2.uml.Manifestation object)
        {
            return new ManifestationEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseCommunicationPath(org.eclipse.uml2.uml.CommunicationPath)
         * @generated
         */
        public Object caseCommunicationPath(org.eclipse.uml2.uml.CommunicationPath object)
        {
            return new CommunicationPathEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseDependency(org.eclipse.uml2.uml.Dependency)
         * @generated
         */
        public Object caseDependency(org.eclipse.uml2.uml.Dependency object)
        {
            return new DependencyEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphEdgeEditPart(edge);
        }
    }

}