/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.editor.GraphElementCreationFactory;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.palette.ModelerConnectionCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerPaletteManager;

/**
 * Generated Palette Manager <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class DeploymentPaletteManager extends ModelerPaletteManager
{
    // declare all the palette categories of the diagram
    private PaletteDrawer objectsDrawer;

    private PaletteDrawer connectionsDrawer;

    private PaletteDrawer commentDrawer;

    private ICreationUtils creationUtils;

    /**
     * The Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param utils the creation utils for the tools of the palette
     * @generated
     */
    public DeploymentPaletteManager(ICreationUtils utils)
    {
        super();
        this.creationUtils = utils;
    }

    /**
     * Creates the main categories of the palette <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createCategories()
    {
        createObjectsDrawer();
        createConnectionsDrawer();
        createCommentDrawer();
    }

    /**
     * Updates the main categories of the palette <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void updateCategories()
    {
        // deletion of the existing categories and creation of the updated
        // categories

        getRoot().remove(objectsDrawer);
        createObjectsDrawer();

        getRoot().remove(connectionsDrawer);
        createConnectionsDrawer();

        getRoot().remove(commentDrawer);
        createCommentDrawer();
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private void createObjectsDrawer()
    {
        objectsDrawer = new PaletteDrawer("Objects", null);
        List entries = new ArrayList();

        CreationFactory factory;

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getNode(), "default");
        entries.add(new ModelerCreationToolEntry("Node", "Node", factory, DeploymentImageRegistry.getImageDescriptor("NODE"), DeploymentImageRegistry.getImageDescriptor("NODE_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getDevice(), "default");
        entries.add(new ModelerCreationToolEntry("Device", "Device", factory, DeploymentImageRegistry.getImageDescriptor("DEVICE"), DeploymentImageRegistry.getImageDescriptor("DEVICE_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getExecutionEnvironment(), "default");
        entries.add(new ModelerCreationToolEntry("Execution Environment", "Execution Environment", factory, DeploymentImageRegistry.getImageDescriptor("EXECUTIONENVIRONMENT"),
                DeploymentImageRegistry.getImageDescriptor("EXECUTIONENVIRONMENT_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getArtifact(), "default");
        entries.add(new ModelerCreationToolEntry("Artifact", "Artifact", factory, DeploymentImageRegistry.getImageDescriptor("ARTIFACT"), DeploymentImageRegistry.getImageDescriptor("ARTIFACT_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getDeploymentSpecification(), "default");
        entries.add(new ModelerCreationToolEntry("Deployment Specification", "Deployment Specification", factory, DeploymentImageRegistry.getImageDescriptor("DEPLOYMENTSPECIFICATION"),
                DeploymentImageRegistry.getImageDescriptor("DEPLOYMENTSPECIFICATION_LARGE")));

        objectsDrawer.addAll(entries);
        getRoot().add(objectsDrawer);
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private void createConnectionsDrawer()
    {
        connectionsDrawer = new PaletteDrawer("Connections", null);
        List entries = new ArrayList();

        CreationFactory factory;

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getDeployment(), "default");
        entries.add(new ModelerConnectionCreationToolEntry("Deployment", "Deployment", factory, DeploymentImageRegistry.getImageDescriptor("DEPLOYMENT"),
                DeploymentImageRegistry.getImageDescriptor("DEPLOYMENT_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getManifestation(), "default");
        entries.add(new ModelerConnectionCreationToolEntry("Manifestation", "Manifestation", factory, DeploymentImageRegistry.getImageDescriptor("MANIFESTATION"),
                DeploymentImageRegistry.getImageDescriptor("MANIFESTATION_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getCommunicationPath(), "default");
        entries.add(new ModelerConnectionCreationToolEntry("CommunicationPath", "CommunicationPath", factory, DeploymentImageRegistry.getImageDescriptor("COMMUNICATIONPATH"),
                DeploymentImageRegistry.getImageDescriptor("COMMUNICATIONPATH_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getDependency(), "default");
        entries.add(new ModelerConnectionCreationToolEntry("Dependency", "Dependency", factory, DeploymentImageRegistry.getImageDescriptor("DEPENDENCY"),
                DeploymentImageRegistry.getImageDescriptor("DEPENDENCY_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, DeploymentSimpleObjectConstants.SIMPLE_OBJECT_DEPLOYMENTSPECIFICATIONLINK, "default", false);
        entries.add(new ModelerConnectionCreationToolEntry("Deployment Specification Link", "Deployment Specification Link", factory,
                DeploymentImageRegistry.getImageDescriptor("DEPLOYMENTSPECIFICATIONLINK"), DeploymentImageRegistry.getImageDescriptor("DEPLOYMENTSPECIFICATIONLINK_LARGE")));

        connectionsDrawer.addAll(entries);
        getRoot().add(connectionsDrawer);
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private void createCommentDrawer()
    {
        commentDrawer = new PaletteDrawer("Comment", null);
        List entries = new ArrayList();

        CreationFactory factory;

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getComment(), "default");
        entries.add(new ModelerCreationToolEntry("Comment", "Comment", factory, DeploymentImageRegistry.getImageDescriptor("COMMENT"), DeploymentImageRegistry.getImageDescriptor("COMMENT_LARGE")));

        factory = new GraphElementCreationFactory(creationUtils, DeploymentSimpleObjectConstants.SIMPLE_OBJECT_COMMENTLINK, "default", false);
        entries.add(new ModelerConnectionCreationToolEntry("Comment Link", "Comment Link", factory, DeploymentImageRegistry.getImageDescriptor("COMMENTLINK"),
                DeploymentImageRegistry.getImageDescriptor("COMMENTLINK_LARGE")));

        commentDrawer.addAll(entries);
        getRoot().add(commentDrawer);
    }

}
