/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram.commands;

import org.eclipse.gef.EditDomain;
import org.eclipse.uml2.uml.CommunicationPath;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.utils.LabelHelper;
import org.topcased.modeler.utils.Utils;

/**
 * CommunicationPath edge creation command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class CommunicationPathEdgeCreationCommand extends CreateTypedEdgeCommand
{
    private CommunicationPath communicationPath;

    private Type sourceObject;

    private Type targetObject;

    private Property property1;

    private Property property2;

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param domain the edit domain
     * @param newObj the graph edge of the new connection
     * @param src the graph element of the source
     * @generated
     */
    public CommunicationPathEdgeCreationCommand(EditDomain domain, GraphEdge newObj, GraphElement src)
    {
        this(domain, newObj, src, true);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param domain the edit domain
     * @param newObj the graph edge of the new connection
     * @param src the graph element of the source
     * @param needModelUpdate set it to true if the model need to be updated
     * @generated
     */
    public CommunicationPathEdgeCreationCommand(EditDomain domain, GraphEdge newObj, GraphElement src, boolean needModelUpdate)
    {
        super(domain, newObj, src, needModelUpdate);
    }

    /**
     * Create the model objects that will be associated with the CommunicationPath
     * 
     * @see org.topcased.modeler.commands.CreateGraphEdgeCommand#execute()
     */
    public void execute()
    {
        // Create the communicationPath and add it to the current package
        communicationPath = (CommunicationPath) Utils.getElement(getEdge());

        // Retrieve source object and target object
        sourceObject = (Type) Utils.getElement(getSource());
        targetObject = (Type) Utils.getElement(getTarget());

        // Create two properties that handle the link between the two object
        property1 = UMLFactory.eINSTANCE.createProperty();
        property1.setName("source");

        property2 = UMLFactory.eINSTANCE.createProperty();
        property2.setName("target");

        // Add the Multiplicity of the Lower and Upper bound of the Property
        property1.setLower(1);
        property1.setUpper(1);

        property2.setLower(1);
        property2.setUpper(1);

        super.execute();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void redoModel()
    {
        // Add the CommunicationPath to the current package
        sourceObject.getNearestPackage().getPackagedElements().add(communicationPath);

        // init the name of the communicationPath contained by the package
        String curName = LabelHelper.INSTANCE.getName(getEditDomain(), communicationPath);
        if (curName == null || curName.length() == 0)
        {
            LabelHelper.INSTANCE.initName(getEditDomain(), Utils.getElement(getEdge()).eContainer(), communicationPath);
        }

        property1.setType(sourceObject);
        property2.setType(targetObject);

        // Add the properties to the communicationPath
        communicationPath.getOwnedEnds().add(property1);
        communicationPath.getOwnedEnds().add(property2);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void undoModel()
    {
        // Remove the Properties associated with the CommunicationPath
        communicationPath.getOwnedEnds().clear();

        property1.setType(null);
        property2.setType(null);

        // Remove the communicationPath from the package
        sourceObject.getNearestPackage().getPackagedElements().remove(communicationPath);
    }

}