/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *   Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware Technologies),
 *   Nicolas Lalevée (Anyware Technologies) - initial API and implementation 
 ******************************************************************************/

package org.topcased.modeler.uml.deploymentdiagram.commands;

import java.util.Iterator;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.Deployment;
import org.eclipse.uml2.uml.DeploymentSpecification;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.uml.alldiagram.commands.ElementRestoreConnectionCommand;
import org.topcased.modeler.uml.deploymentdiagram.DeploymentSimpleObjectConstants;
import org.topcased.modeler.utils.Utils;

/**
 * DeploymentSpecification restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class DeploymentRestoreConnectionCommand extends ElementRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public DeploymentRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated
     */
    protected void initializeCommands()
    {
        super.initializeCommands();

        GraphElement elt = getGraphElement();
        EObject eltObject = Utils.getElement(elt);

        if (eltObject instanceof Deployment)
        {
            Iterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(elt);
                    GraphElement elt2 = (GraphElement) obj;
                    EObject eltObject2 = Utils.getElement(elt2);
                    if (eltObject2 instanceof DeploymentSpecification)
                    {
                        if (autoRef)
                        {
                            // autoRef not allowed
                        }
                        else
                        {
                            // if the node is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createDeploymentSpecificationLinkFromDeploymentSpecificationToDeployment(elt2, elt);
                        }
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcElt the source element
     * @param targetElt the target element
     * @generated NOT
     */
    private void createDeploymentSpecificationLinkFromDeploymentSpecificationToDeployment(GraphElement srcElt, GraphElement targetElt)
    {
        DeploymentSpecification sourceObject = (DeploymentSpecification) Utils.getElement(srcElt);
        Deployment targetObject = (Deployment) Utils.getElement(targetElt);

        if (targetObject.getConfigurations().contains(sourceObject))
        {
            // check if the relation does not exists yet
            if (getExistingEdges(srcElt, targetElt, DeploymentSimpleObjectConstants.SIMPLE_OBJECT_DEPLOYMENTSPECIFICATIONLINK).size() == 0)
            {
                GraphEdge edge = Utils.createGraphEdge(DeploymentSimpleObjectConstants.SIMPLE_OBJECT_DEPLOYMENTSPECIFICATIONLINK);
                DeploymentSpecificationLinkEdgeCreationCommand cmd = new DeploymentSpecificationLinkEdgeCreationCommand(null, edge, srcElt, false);
                cmd.setTarget(targetElt);
                add(cmd);
            }
        }
    }

}