/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram.commands;

import java.util.Iterator;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.Deployment;
import org.eclipse.uml2.uml.DeploymentSpecification;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.uml.deploymentdiagram.DeploymentSimpleObjectConstants;
import org.topcased.modeler.utils.Utils;

/**
 * DeploymentSpecification restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class DeploymentSpecificationRestoreConnectionCommand extends ArtifactRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public DeploymentSpecificationRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated
     */
    protected void initializeCommands()
    {

        super.initializeCommands();

        GraphElement node = getGraphElement();
        EObject nodeObject = Utils.getElement(node);

        if (nodeObject instanceof DeploymentSpecification)
        {
            Iterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(node);
                    GraphElement node2 = (GraphElement) obj;
                    EObject nodeObject2 = Utils.getElement(node2);
                    if (nodeObject2 instanceof Deployment)
                    {
                        if (autoRef)
                        {
                            // autoRef not allowed
                        }
                        else
                        {
                            // if the node is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createDeploymentSpecificationLinkFromDeploymentSpecificationToDeployment(node, node2);
                        }
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createDeploymentSpecificationLinkFromDeploymentSpecificationToDeployment(GraphElement srcNode, GraphElement targetNode)
    {
        DeploymentSpecification sourceObject = (DeploymentSpecification) Utils.getElement(srcNode);
        Deployment targetObject = (Deployment) Utils.getElement(targetNode);

        if (targetObject.getConfigurations().contains(sourceObject))
        {
            // check if the relation does not exists yet
            if (getExistingEdges(srcNode, targetNode, DeploymentSimpleObjectConstants.SIMPLE_OBJECT_DEPLOYMENTSPECIFICATIONLINK).size() == 0)
            {
                GraphEdge edge = Utils.createGraphEdge(DeploymentSimpleObjectConstants.SIMPLE_OBJECT_DEPLOYMENTSPECIFICATIONLINK);
                DeploymentSpecificationLinkEdgeCreationCommand cmd = new DeploymentSpecificationLinkEdgeCreationCommand(null, edge, srcNode, false);
                cmd.setTarget(targetNode);
                add(cmd);
            }
        }
    }

}