/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.CommunicationPath;
import org.eclipse.uml2.uml.DeployedArtifact;
import org.eclipse.uml2.uml.Deployment;
import org.eclipse.uml2.uml.DeploymentTarget;
import org.eclipse.uml2.uml.Property;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.uml.alldiagram.commands.ElementRestoreConnectionCommand;
import org.topcased.modeler.utils.Utils;

/**
 * DeploymentTarget restore connection command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class DeploymentTargetRestoreConnectionCommand extends ElementRestoreConnectionCommand
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param part the EditPart that is restored
     * @generated
     */
    public DeploymentTargetRestoreConnectionCommand(EditPart part)
    {
        super(part);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.commands.AbstractRestoreConnectionCommand#initializeCommands()
     * @generated
     */
    protected void initializeCommands()
    {

        super.initializeCommands();

        GraphElement node = getGraphElement();
        EObject nodeObject = Utils.getElement(node);

        if (nodeObject instanceof DeploymentTarget)
        {
            Iterator itDiagContents = getModeler().getActiveDiagram().eAllContents();
            while (itDiagContents.hasNext())
            {
                Object obj = itDiagContents.next();
                // FIXME Change the way to handle EList GraphNodes
                if (obj instanceof GraphElement && DIUtils.getProperty((GraphElement) obj, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID) == null)
                {
                    boolean autoRef = obj.equals(node);
                    GraphElement node2 = (GraphElement) obj;
                    EObject nodeObject2 = Utils.getElement(node2);

                    if (nodeObject2 instanceof DeployedArtifact)
                    {
                        if (autoRef)
                        {
                            // autoRef not allowed
                        }
                        else
                        {
                            // if node is the target of the edge or if it is the
                            // source and that the SourceTargetCouple is
                            // reversible
                            createDeploymentFromDeployedArtifactToDeploymentTarget(node2, node);
                        }
                    }

                    if (nodeObject2 instanceof DeploymentTarget)
                    {
                        if (autoRef)
                        {
                            // autoRef not allowed
                        }
                        else
                        {
                            // if the node is the source of the edge or if it is
                            // the target and that the SourceTargetCouple is
                            // reversible
                            createCommunicationPathFromDeploymentTargetToDeploymentTarget(node, node2);
                            // if node is the target of the edge or if it is the
                            // source and that the SourceTargetCouple is
                            // reversible
                            createCommunicationPathFromDeploymentTargetToDeploymentTarget(node2, node);
                        }
                    }

                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createDeploymentFromDeployedArtifactToDeploymentTarget(GraphElement srcNode, GraphElement targetNode)
    {
        DeployedArtifact sourceObject = (DeployedArtifact) Utils.getElement(srcNode);
        DeploymentTarget targetObject = (DeploymentTarget) Utils.getElement(targetNode);

        EList edgeObjectList = targetObject.getDeployments();
        for (Iterator it = edgeObjectList.iterator(); it.hasNext();)
        {
            Object obj = it.next();
            if (obj instanceof Deployment)
            {
                Deployment edgeObject = (Deployment) obj;
                if (edgeObject.getDeployedArtifacts().contains(sourceObject))
                {
                    // check if the relation does not exists yet
                    List existing = getExistingEdges(srcNode, targetNode, Deployment.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            DeploymentEdgeCreationCommand cmd = new DeploymentEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                            cmd.setTarget(targetNode);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param srcNode the source node
     * @param targetNode the target node
     * @generated NOT
     */
    private void createCommunicationPathFromDeploymentTargetToDeploymentTarget(GraphElement srcNode, GraphElement targetNode)
    {
        DeploymentTarget sourceObject = (DeploymentTarget) Utils.getElement(srcNode);
        DeploymentTarget targetObject = (DeploymentTarget) Utils.getElement(targetNode);

        EList edgeObjectList = ((org.eclipse.uml2.uml.Package) Utils.getDiagramModelObject(srcNode)).getPackagedElements();
        for (Iterator it = edgeObjectList.iterator(); it.hasNext();)
        {
            Object obj = it.next();
            if (obj instanceof CommunicationPath)
            {
                CommunicationPath edgeObject = (CommunicationPath) obj;

                if (edgeObject.getMemberEnds().size() == 2 && sourceObject.equals(((Property) edgeObject.getMemberEnds().get(0)).getType())
                        && targetObject.equals(((Property) edgeObject.getMemberEnds().get(1)).getType()))
                {
                    // check if the relation does not exists yet
                    List existing = getExistingEdges(srcNode, targetNode, CommunicationPath.class);
                    if (!isAlreadyPresent(existing, edgeObject))
                    {
                        ICreationUtils factory = getModeler().getActiveConfiguration().getCreationUtils();
                        // restore the link with its default presentation
                        GraphElement edge = factory.createGraphElement(edgeObject);
                        if (edge instanceof GraphEdge)
                        {
                            CommunicationPathEdgeCreationCommand cmd = new CommunicationPathEdgeCreationCommand(getEditDomain(), (GraphEdge) edge, srcNode, false);
                            cmd.setTarget(targetNode);
                            add(cmd);
                        }
                    }
                }
            }
        }
    }

}