/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.uml2.uml.CommunicationPath;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.topcased.modeler.commands.ReconnectGraphEdgeCommand;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.utils.Utils;

/**
 * This command handles the reconnection of associations links.
 * Indeed they connect Type by updating their properties. 
 *
 * @Created 2 févr. 07
 *
 * @author <a href="mailto:alfredo@anyware-tech.com">Jose Alfredo SERRANO</a>
 *
 */
public class ReconnectCommunicationPathEdgeCommand extends ReconnectGraphEdgeCommand
{
    /** Reconnecting the source end */
    public static final int SOURCE = 1;
    
    /** Reconnectiong the target end */
    public static final int TARGET = 2;
    
    /** Memorize the end which is reconnected */
    private int reconnectionEnd = 0;
    
    /** Keeping a trace of the old referenced type */
    private Type oldType;
    
    /** Keeping a trace of the old property */
    private Property property;
    
    public ReconnectCommunicationPathEdgeCommand(ConnectionEditPart connection, int reconnectionEnd)
    {
        super(connection);
        this.reconnectionEnd = reconnectionEnd;
    }

    /* (non-Javadoc)
     * @see org.topcased.modeler.commands.ReconnectGraphEdgeCommand#getOldGraphElement(org.eclipse.gef.ConnectionEditPart)
     */
    protected GraphElement getOldGraphElement(ConnectionEditPart connection)
    {
        return null;
    }

    /*
     * (non-Javadoc)
     * @see org.topcased.modeler.commands.ReconnectGraphEdgeCommand#initializeConnectors(org.eclipse.gef.ConnectionEditPart)
     */
    protected void initializeConnectors(ConnectionEditPart connection)
    {
        super.initializeConnectors(connection);
        oldType = (Type) Utils.getElement(getOldGraphElement(connection));
    }
    
    /* (non-Javadoc)
     * @see org.topcased.modeler.commands.ReconnectGraphEdgeCommand#redoModel()
     */
    protected void redoModel()
    {
        CommunicationPath communicationPath = (CommunicationPath) Utils.getElement(getEdge());
        
        Type type = (Type) Utils.getElement(getNewElement());
        List ends = communicationPath.getOwnedEnds();
        Iterator it = ends.iterator();
        while (it.hasNext())
        {
            property = (Property) it.next();
            Type currentType = property.getType();
            if (currentType == oldType)
            {
                property.setType(type);
                break;
            }
        }
    }

    /* (non-Javadoc)
     * @see org.topcased.modeler.commands.ReconnectGraphEdgeCommand#undoModel()
     */
    protected void undoModel()
    {
        property.setType(oldType);
    }

    /* (non-Javadoc)
     * @see org.topcased.modeler.commands.ReconnectGraphEdgeCommand#updateConnectors(org.topcased.modeler.di.model.GraphConnector, org.topcased.modeler.di.model.GraphConnector, org.topcased.modeler.di.model.GraphConnector)
     */
    protected void updateConnectors(GraphConnector newConnector, GraphConnector oldConnector, GraphConnector attachedConnector)
    {
        GraphEdge edge = getEdge();
        
        if (reconnectionEnd == SOURCE)
        {
            oldConnector.getGraphEdge().remove(edge);
            attachedConnector.getGraphEdge().remove(edge);
            edge.getAnchor().clear();
            newConnector.getGraphEdge().add(edge);
            attachedConnector.getGraphEdge().add(edge);
        }
        else if (reconnectionEnd == TARGET)
        {
            oldConnector.getGraphEdge().remove(edge);
            newConnector.getGraphEdge().add(edge);
        }
    }
}
