/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Element;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.UMLTools;
import org.topcased.modeler.uml.deploymentdiagram.DeploymentEditPolicyConstants;
import org.topcased.modeler.uml.deploymentdiagram.commands.ArtifactRestoreConnectionCommand;
import org.topcased.modeler.uml.deploymentdiagram.figures.ArtifactFigure;
import org.topcased.modeler.uml.deploymentdiagram.policies.ArtifactLayoutEditPolicy;
import org.topcased.modeler.uml.deploymentdiagram.policies.DependencyEdgeCreationEditPolicy;
import org.topcased.modeler.uml.deploymentdiagram.policies.ManifestationEdgeCreationEditPolicy;

/**
 * The Artifact object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ArtifactEditPart extends DeployedArtifactEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public ArtifactEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(DeploymentEditPolicyConstants.MANIFESTATION_EDITPOLICY, new ManifestationEdgeCreationEditPolicy());

        installEditPolicy(DeploymentEditPolicyConstants.DEPENDENCY_EDITPOLICY, new DependencyEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new ArtifactRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new ArtifactLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new ArtifactFigure();
    }

    /**
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getEditableLabel()
     */
    public ILabel getEditableLabel()
    {
        return ((ComposedLabel) ((ArtifactFigure) getFigure()).getLabel()).getMain();
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        ComposedLabel editableLbl = (ComposedLabel) ((ArtifactFigure) getFigure()).getLabel();
        editableLbl.setPrefix(UMLTools.getStereotypesNotation((Element) getEObject()));
        super.refreshHeaderLabel();
    }

}