/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.requests.DirectEditRequest;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.uml2.uml.CommunicationPath;
import org.eclipse.uml2.uml.Element;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.Label;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.LabelCellEditorLocator;
import org.topcased.modeler.edit.ModelerLabelDirectEditManager;
import org.topcased.modeler.edit.policies.EdgeObjectOffsetEditPolicy;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.uml.UMLTools;
import org.topcased.modeler.uml.deploymentdiagram.DeploymentEdgeObjectConstants;
import org.topcased.modeler.uml.deploymentdiagram.figures.CommunicationPathFigure;

/**
 * CommunicationPath controller <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class CommunicationPathEditPart extends EMFGraphEdgeEditPart
{

    /** The DirectEditManager */
    private DirectEditManager manager;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param model the graph object
     * @generated
     */
    public CommunicationPathEditPart(GraphEdge model)
    {
        super(model);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_OFFSET_EDITPOLICY, new EdgeObjectOffsetEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());

    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Figure
     * @generated
     */
    protected IFigure createFigure()
    {
        CommunicationPathFigure connection = new CommunicationPathFigure();

        return connection;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#getEdgeObjectFigure(org.topcased.modeler.di.model.EdgeObject)
     * @generated
     */
    public IEdgeObjectFigure getEdgeObjectFigure(EdgeObject edgeObject)
    {
        if (DeploymentEdgeObjectConstants.NAME_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((CommunicationPathFigure) getFigure()).getNameEdgeObjectFigure();
        }
        if (DeploymentEdgeObjectConstants.STEREOTYPE_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((CommunicationPathFigure) getFigure()).getStereotypeEdgeObjectFigure();
        }
        return null;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#refreshEdgeObjects()
     * @generated
     */
    protected void refreshEdgeObjects()
    {
        super.refreshEdgeObjects();
        updateNameLabel();
        updateStereotypeLabel();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the name Label
     * 
     * @generated
     */
    private void updateNameLabel()
    {
        ((Label) ((CommunicationPathFigure) getFigure()).getNameEdgeObjectFigure()).setText(((CommunicationPath) getEObject()).getName());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the stereotype Label
     * 
     * @generated NOT
     */
    private void updateStereotypeLabel()
    {
        ((Label) ((CommunicationPathFigure) getFigure()).getStereotypeEdgeObjectFigure()).setText(UMLTools.getStereotypesNotation((Element) getEObject()));
    }

    // ---------------------------------------
    // Ajout des fonctions pour le DirectEdit
    // ---------------------------------------

    /**
     * Handle selection <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param value The value of the selection
     * @generated
     */
    public void setSelected(int value)
    {
        super.setSelected(value);
        ((EditableLabel) ((CommunicationPathFigure) getFigure()).getNameEdgeObjectFigure()).setSelected(value == 0 ? false : true);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
     * @generated
     */
    public void performRequest(Request request)
    {
        if (request.getType() == RequestConstants.REQ_DIRECT_EDIT)
        {
            if (request instanceof DirectEditRequest && !(directEditHitTest(((DirectEditRequest) request).getLocation().getCopy())))
            {
                return;
            }
            performDirectEdit();
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param requestLoc the location of the cursor
     * @return true if the cursor is over the EditableLabel
     * @generated
     */
    private boolean directEditHitTest(Point requestLoc)
    {
        Label nameName = (Label) ((CommunicationPathFigure) getFigure()).getNameEdgeObjectFigure();
        if (nameName != null)
        {
            nameName.translateToRelative(requestLoc);
            return nameName.containsPoint(requestLoc);
        }
        return false;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private void performDirectEdit()
    {
        if (manager == null)
        {
            Label nameName = (Label) ((CommunicationPathFigure) getFigure()).getNameEdgeObjectFigure();
            manager = new ModelerLabelDirectEditManager(this, TextCellEditor.class, new LabelCellEditorLocator(nameName), nameName);
        }
        manager.show();
    }

}