/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutManager;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.XYLayout;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.IContainerFigure;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.draw2d.figures.Label;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */
public class ArtifactFigure extends Figure implements IContainerFigure, ILabelFigure
{
    private ILabel label;

    private IFigure contentPane;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public ArtifactFigure()
    {
        LayoutManager layout = new StackLayout();
        setLayoutManager(layout);

        add(createBackgroundFigure());

        contentPane = new Figure();
        contentPane.setLayoutManager(new XYLayout());
        add(contentPane);
    }

    /**
     * Draw an innerRectangle that will be the real background figure. The Anchor will be attached to this inner
     * rectangle.<br>
     * The innerRectangle is also the container of the HeaderComponent and the Label of the Component that are arranged
     * in a toolbarLayout.
     * 
     * @return the figure
     */
    protected IFigure createBackgroundFigure()
    {
        RectangleFigure rectangle = new RectangleFigure();
        rectangle.setOpaque(true);

        ToolbarLayout toolbarLayout = new ToolbarLayout();
        toolbarLayout.setMinorAlignment(ToolbarLayout.ALIGN_CENTER);
        toolbarLayout.setSpacing(5);

        rectangle.setLayoutManager(toolbarLayout);
        rectangle.setBorder(new MarginBorder(5));

        // add the header figure that support the "<<artifact>>" text and the
        // artifact icon in the right hand corner of the rectangle
        rectangle.add(new HeaderArtifact());

        label = createLabel();
        rectangle.add(label);

        return rectangle;
    }

    /**
     * Creates the label of the figure.<br>
     * <b>Subclasses can override this method to customize the appearance of the label (for example you can return a
     * {@link ComposedLabel} instead)</b>
     * 
     * @return the label of the figure
     */
    protected ILabel createLabel()
    {
        EditableLabel lbl = new EditableLabel();
        lbl.setLabelAlignment(PositionConstants.LEFT);
        ComposedLabel composedLbl = new ComposedLabel(new Label(), lbl, null, false);
        return composedLbl;
    }

    /**
     * @see org.topcased.draw2d.figures.ILabelFigure#getLabel()
     */
    public ILabel getLabel()
    {
        return label;
    }

    /**
     * @see org.topcased.draw2d.figures.IContainerFigure#getContentPane()
     */
    public IFigure getContentPane()
    {
        return contentPane;
    }
}