/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * This shape is used to draw the artifact icon that is displayed in the right hand corner of an artifact.
 * 
 * Creation 24 juil. 06
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class ArtifactIcon extends Shape
{
    /**
     * Constructor
     */
    public ArtifactIcon()
    {
        // Do nothing
    }

    /**
     * @see org.eclipse.draw2d.Shape#fillShape(org.eclipse.draw2d.Graphics)
     */
    protected void fillShape(Graphics graphics)
    {
        // No fill needed
    }

    /**
     * @see org.eclipse.draw2d.Shape#outlineShape(org.eclipse.draw2d.Graphics)
     */
    protected void outlineShape(Graphics graphics)
    {
        Rectangle r = new Rectangle();
        r.setBounds(getBounds());
        r.crop(getInsets());
        r.resize(-1, -1);

        PointList icon = new PointList();

        icon.addPoint(r.getTopRight().translate(-r.width / 3, 0));
        icon.addPoint(r.getTopLeft());
        icon.addPoint(r.getBottomLeft());
        icon.addPoint(r.getBottomRight());
        icon.addPoint(r.getTopRight().translate(0, r.height / 4));
        icon.addPoint(r.getTopRight().translate(-r.width / 3, 0));
        icon.addPoint(r.getTopRight().translate(-r.width / 3, r.height / 4));
        icon.addPoint(r.getTopRight().translate(0, r.height / 4));

        graphics.drawPolyline(icon);
    }

}
