/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram.figures;

import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutManager;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.draw2d.XYLayout;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.IContainerFigure;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.draw2d.figures.Label;
import org.topcased.draw2d.layout.CenterLayout;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */
public class NodeFigure extends Figure implements IContainerFigure, ILabelFigure
{
    private ILabel label;

    private IFigure contentPane;

    private IFigure bgdFigure;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public NodeFigure()
    {
        createContents();
    }

    /**
     * Creates the contents of the figure : by default, it creates a layout manager, a header and a container
     */
    protected void createContents()
    {
        LayoutManager layout = new StackLayout();
        setLayoutManager(layout);

        // Draw the background Figure
        bgdFigure = new NodeShape();
        bgdFigure.setOpaque(true);
        add(bgdFigure);

        // Create the top Figure
        IFigure topFigure = new Figure();
        topFigure.setLayoutManager(new CenterLayout());
        // Add margin. At the top, the margin is extended of the deep of the
        // cube (see the NodeShape)
        topFigure.setBorder(new MarginBorder(15, 5, 5, 5));
        label = createLabel();
        topFigure.add(label);

        // Create the contentPane figure
        contentPane = new Figure();
        contentPane.setLayoutManager(new XYLayout());

        // Create the global figure of the Node
        Figure contentContainer = new Figure();
        contentContainer.setLayoutManager(new BorderLayout());

        contentContainer.add(topFigure, new Integer(PositionConstants.TOP));
        contentContainer.add(contentPane, new Integer(PositionConstants.CENTER));

        add(contentContainer);
    }

    /**
     * Creates the label of the figure.<br>
     * <b>Subclasses can override this method to customize the appearance of the label (for example you can return a
     * {@link ComposedLabel} instead)</b>
     * 
     * @return the label of the figure
     */
    protected ILabel createLabel()
    {
        EditableLabel lbl = new EditableLabel();
        lbl.setLabelAlignment(PositionConstants.LEFT);
        ComposedLabel composedLbl = new ComposedLabel(new Label(), lbl, null, false);
        return composedLbl;
    }

    /**
     * @see org.topcased.draw2d.figures.ILabelFigure#getLabel()
     */
    public ILabel getLabel()
    {
        return label;
    }

    /**
     * @see org.topcased.draw2d.figures.IContainerFigure#getContentPane()
     */
    public IFigure getContentPane()
    {
        return contentPane;
    }

    /**
     * Returns the figure drawn behind the label
     * 
     * @return the background figure
     */
    public IFigure getBackgroundFigure()
    {
        return bgdFigure;
    }

}