/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * This shape is used to draw the graphical representation of a Node. This is a figure that looks like a 3-dimensional
 * view of a cube.
 * 
 * Creation 24 juil. 06
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class NodeShape extends Shape
{
    private static final int CUBE_DEEP = 10;

    /**
     * Constructor
     */
    public NodeShape()
    {
        // Do nothing
    }

    /**
     * @see org.eclipse.draw2d.Shape#fillShape(org.eclipse.draw2d.Graphics)
     */
    protected void fillShape(Graphics graphics)
    {
        PointList deepCube = new PointList();

        deepCube.addPoint(getBounds().getBottomLeft());
        deepCube.addPoint(getBounds().getTopLeft().getTranslated(0, CUBE_DEEP));
        deepCube.addPoint(getBounds().getTopLeft().getTranslated(CUBE_DEEP, 0));
        deepCube.addPoint(getBounds().getTopRight());
        deepCube.addPoint(getBounds().getBottomRight().getTranslated(0, -CUBE_DEEP));
        deepCube.addPoint(getBounds().getBottomRight().getTranslated(-CUBE_DEEP, 0));
        graphics.fillPolygon(deepCube);
    }

    /**
     * @see org.eclipse.draw2d.Shape#outlineShape(org.eclipse.draw2d.Graphics)
     */
    protected void outlineShape(Graphics graphics)
    {
        Rectangle r = new Rectangle();
        r.setBounds(getBounds());
        r.crop(getInsets());
        r.resize(-1, -1);

        graphics.drawRectangle(r.getResized(-CUBE_DEEP, -CUBE_DEEP).getTranslated(0, CUBE_DEEP));

        PointList deepCube = new PointList();

        deepCube.addPoint(r.getTopLeft().getTranslated(0, CUBE_DEEP));
        deepCube.addPoint(r.getTopLeft().getTranslated(CUBE_DEEP, 0));
        deepCube.addPoint(r.getTopRight());
        deepCube.addPoint(r.getBottomRight().getTranslated(0, -CUBE_DEEP));
        deepCube.addPoint(r.getBottomRight().getTranslated(-CUBE_DEEP, 0));
        graphics.drawPolyline(deepCube);

        graphics.drawLine(r.getTopRight(), r.getTopRight().getTranslated(-CUBE_DEEP, CUBE_DEEP));
    }

}
