/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.deploymentdiagram.policies;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.uml2.uml.Deployment;
import org.eclipse.uml2.uml.DeploymentSpecification;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.commands.ReconnectNodeToNodeCommand;
import org.topcased.modeler.commands.ReconnectTargetToSourceCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy;
import org.topcased.modeler.uml.deploymentdiagram.DeploymentSimpleObjectConstants;
import org.topcased.modeler.uml.deploymentdiagram.commands.DeploymentSpecificationLinkEdgeCreationCommand;
import org.topcased.modeler.uml.deploymentdiagram.edit.DeploymentSpecificationLinkEditPart;
import org.topcased.modeler.utils.SourceTargetData;
import org.topcased.modeler.utils.Utils;

/**
 * DeploymentSpecificationLink edge creation <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class DeploymentSpecificationLinkEdgeCreationEditPolicy extends AbstractEdgeCreationEditPolicy
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#createCommand(org.eclipse.gef.EditDomain,
     *      org.topcased.modeler.di.model.GraphEdge, org.topcased.modeler.di.model.GraphElement)
     * @generated
     */
    protected CreateTypedEdgeCommand createCommand(EditDomain domain, GraphEdge edge, GraphElement source)
    {
        return new DeploymentSpecificationLinkEdgeCreationCommand(domain, edge, source);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkEdge(org.topcased.modeler.di.model.GraphEdge)
     * @generated
     */
    protected boolean checkEdge(GraphEdge edge)
    {
        if (edge.getSemanticModel() instanceof SimpleSemanticModelElement)
        {
            return DeploymentSimpleObjectConstants.SIMPLE_OBJECT_DEPLOYMENTSPECIFICATIONLINK.equals(((SimpleSemanticModelElement) edge.getSemanticModel()).getTypeInfo());
        }
        return false;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkSource(org.topcased.modeler.di.model.GraphElement)
     * @generated
     */
    protected boolean checkSource(GraphElement source)
    {
        EObject object = Utils.getElement(source);
        if (object instanceof org.eclipse.uml2.uml.DeploymentSpecification && !isDeploymentSpecificationLinked(source))
        {
            return true;
        }
        return false;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkTargetForSource(org.topcased.modeler.di.model.GraphElement,
     *      org.topcased.modeler.di.model.GraphElement)
     * @generated NOT
     */
    protected boolean checkTargetForSource(GraphElement source, GraphElement target)
    {
        EObject sourceObject = Utils.getElement(source);
        EObject targetObject = Utils.getElement(target);

        if (sourceObject instanceof org.eclipse.uml2.uml.DeploymentSpecification && targetObject instanceof org.eclipse.uml2.uml.Deployment)
        {
            if (!sourceObject.equals(targetObject))
            {
                return true;
            }
        }
        return false;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkCommand(org.eclipse.gef.commands.Command)
     * @generated
     */
    protected boolean checkCommand(Command command)
    {
        return command instanceof DeploymentSpecificationLinkEdgeCreationCommand;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#getSourceTargetData(org.topcased.modeler.di.model.GraphElement,
     *      org.topcased.modeler.di.model.GraphElement)
     * @generated
     */
    protected SourceTargetData getSourceTargetData(GraphElement source, GraphElement target)
    {
        EObject sourceObject = Utils.getElement(source);
        EObject targetObject = Utils.getElement(target);

        if (sourceObject instanceof org.eclipse.uml2.uml.DeploymentSpecification && targetObject instanceof org.eclipse.uml2.uml.Deployment)
        {
            return new SourceTargetData(false, false, SourceTargetData.NONE, null, null, null, null, null, null, null, "configuration");
        }
        return null;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
     * @generated
     */
    protected Command getReconnectSourceCommand(ReconnectRequest request) 
    {
        ConnectionEditPart connection = request.getConnectionEditPart();
        if (connection instanceof DeploymentSpecificationLinkEditPart)
        {
            final DeploymentSpecificationLinkEditPart edit = (DeploymentSpecificationLinkEditPart) connection;
            
            final GraphElement newSource = (GraphElement) getHost().getModel();
            
            if (checkSource(newSource))
            {
                final GraphElement graphElt = (GraphElement) edit.getTarget().getModel();
                ReconnectNodeToNodeCommand command = new ReconnectTargetToSourceCommand(edit, "configuration", ReconnectNodeToNodeCommand.SOURCE)
                {
                    protected EObject getObjectToUpdate()
                    {
                        return Utils.getElement(graphElt);
                    }
                };
                command.setNewElement(newSource);
                return command;
            }
        }

        return null;
    }
    
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
     * 
     * @generated
     */
    protected Command getReconnectTargetCommand(ReconnectRequest request)
    {
        ConnectionEditPart connection = request.getConnectionEditPart();
        if (connection instanceof DeploymentSpecificationLinkEditPart)
        {
            final DeploymentSpecificationLinkEditPart edit = (DeploymentSpecificationLinkEditPart) connection;
            
            final GraphElement newTarget = (GraphElement) getHost().getModel();
            final GraphElement source = (GraphElement) edit.getSource().getModel();
            
            if (checkTargetForSource(source, newTarget))
            {
                final GraphElement graphElt = source;
                ReconnectNodeToNodeCommand command = new ReconnectTargetToSourceCommand(edit, "configuration", ReconnectNodeToNodeCommand.TARGET)
                {
                    protected EObject getObjectToUpdate()
                    {
                        return Utils.getElement(graphElt);
                    }
                };
                command.setNewElement(newTarget);
                return command;
            }
        }

        return null;
    }
    
    /**
     * Check if the EAnnotation is already associated with a model object
     * 
     * @param src the EAnnotation GraphNode
     * @return true if the EAnnotation is already associated with a model object
     */
    private boolean isDeploymentSpecificationLinked(GraphElement src)
    {
        // get the EAnnotation model object
        DeploymentSpecification deploymentSpecification = (DeploymentSpecification) Utils.getElement(src);

        // get the model Object that is currently associated with the
        // EAnnotation
        Deployment deployment = deploymentSpecification.getDeployment();

        // if the model Object is the Package associated with the diagram, that
        // means that the EAnnotation is not currently associated with a model
        // object
        return deployment != null;
    }
}