/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.dialogs;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.Extension;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.Type;

/**
 * Structure that stores stereotype data
 */
public class StereotypeStructure
{
    /** */
    private Stereotype stereotype;

    /** */
    private Text nameText;

    /** */
    private String name;

    /** */
    private Text extensionTxt;

    /** */
    private List metaclasses;

    /**
     * 
     * @param stereo
     */
    public StereotypeStructure(Stereotype stereo)
    {
        stereotype = stereo;
        name = stereo.getName();
        metaclasses = new ArrayList();
        Iterator iterator = stereo.getOwnedAttributes().iterator();
        while (iterator.hasNext())
        {
            Object o = iterator.next();
            if (o instanceof Property)
            {
                Property property = (Property) o;
                if (property.getAssociation() instanceof Extension)
                {
                    metaclasses.add(property.getType());
                }
            }
        }
    }

    /**
     * 
     * @return Stereotype the Stereotype
     */
    public Stereotype getStereotype()
    {
        return stereotype;
    }

    /**
     * 
     * @return List the list of MetaClasses
     */
    public List getMetaclasses()
    {
        return metaclasses;
    }

    /**
     * 
     * @return String the new name
     */
    public String getNewName()
    {
        return name;
    }

    /**
     * 
     * @param metas
     */
    public void updateMetaclasses(List metas)
    {
        metaclasses = metas;
    }

    /**
     * 
     * 
     */
    public void updateLabel()
    {
        boolean first = true;
        StringBuffer result = new StringBuffer();
        Iterator iterator = metaclasses.iterator();
        int counter = 0;
        while (iterator.hasNext())
        {
            if (first)
            {
                first = false;
            }
            else
            {
                result.append(", ");
            }

            result.append(((Type) iterator.next()).getName());
            counter++;
        }

        if (metaclasses.size() == 0)
        {
            result.append("none");
        }

        if (extensionTxt != null)
        {
            extensionTxt.setText(result.toString());
        }
        else
        {
            throw new IllegalStateException("Label cannot be null at this point.");
        }
    }

    /**
     * 
     * @return String the name
     */
    protected String getName()
    {
        if (nameText != null)
        {
            return nameText.getText();
        }
        else
        {
            throw new IllegalStateException("Text cannot be null at this point.");
        }
    }

    /**
     * 
     * @param name
     */
    protected void setName(String name)
    {
        this.name = name;
    }

    /**
     * 
     * @return Text the Text widget associated with the extension
     */
    protected Text getExtensionTxt()
    {
        return extensionTxt;
    }

    /**
     * 
     * @param extensionTxt
     */
    protected void setExtensionTxt(Text extensionTxt)
    {
        this.extensionTxt = extensionTxt;
    }

    /**
     * @return Text the Text widget associated with the name
     */
    protected Text getNameText()
    {
        return nameText;
    }

    /**
     * 
     * @param nameText
     */
    protected void setNameText(Text nameText)
    {
        this.nameText = nameText;
    }

}