/*******************************************************************************
 * Copyright (c) 2005 David Sciamma.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies) - initial API and implementation
 *******************************************************************************/

package org.topcased.modeler.uml.importer.internal.actions;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWizard;
import org.topcased.modeler.uml.importer.internal.Activator;
import org.topcased.modeler.uml.importer.internal.wizards.Java2UMLWizard;

/**
 * This action launch the convertion wizard.
 * 
 * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class UMLFromJavaAction implements IObjectActionDelegate
{
    private IJavaProject javaProject;

    private IWorkbenchWindow window;

    /**
     * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction,
     *      org.eclipse.ui.IWorkbenchPart)
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart)
    {
        window = targetPart.getSite().getWorkbenchWindow();
    }
    
    /**
     * Return the container associated with the selected IJavaElement
     * @return the initial container
     */
    protected IContainer getInitialContainer()
    {
        IContainer container = null;
        if (javaProject != null)
        {
            try
            {
                container = (IContainer) javaProject.getCorrespondingResource();
            }
            catch (JavaModelException e)
            {
                Activator.log(e);
            }
        }
        
        return container;
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action)
    {
        if (window == null)
        {
            return;
        }

        Shell shell = window.getShell();
        try
        {
            Wizard wizard = createWizard();
            if (wizard instanceof IWorkbenchWizard)
            {
                ((IWorkbenchWizard) wizard).init(getWorkbench(), new StructuredSelection(getInitialContainer()));
            }

            WizardDialog dialog = new WizardDialog(shell, wizard);
            dialog.create();
            int result = dialog.open();
            postWizard(result);
        }
        catch (CoreException e)
        {
            String title = "Error";
            String message = "An error occured during wizard execution.\n"
                    + "See error logs for more details.";

            ErrorDialog.openError(shell, title, message, e.getStatus());
        }
    }

    /**
     * Returns the Eclipse workbench
     * @return the workbench
     */
    protected IWorkbench getWorkbench()
    {
        return Activator.getDefault().getWorkbench();
    }
    
    /**
     * Create the wizard
     * @return the wizard that creates the reverse engineered model
     * @throws CoreException if an error occured during wizard
     */
    protected Wizard createWizard() throws CoreException
    {
        Java2UMLWizard wizard = new Java2UMLWizard();
        wizard.setJavaProject(javaProject);
        return wizard;
    }

    /**
     * Execute the post action after the wizard
     * @param code the return code of the wizard
     * @return <code>true</code> if the action is executed
     */
    protected boolean postWizard(int code)
    {
        if (code == Window.OK)
        {
            // TODO open the editor
            return true;
        }
        return false;
    }


    /**
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
     *      org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection)
    {
        javaProject = null;
        if (selection instanceof IStructuredSelection)
        {
            Object selectedObj = ((IStructuredSelection) selection).getFirstElement();
//            if (selectedObj instanceof IPackageFragment)
//            {
//                javaElement = (IPackageFragment) selectedObj;
//            }
//            else 
                if (selectedObj instanceof IJavaProject)
            {
                javaProject = (IJavaProject) selectedObj;
            }
            // TODO manage adaptable objects
        }

        action.setEnabled(javaProject != null);
    }

}
