/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.internal.properties.sections.association;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Property;
import org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection;

/**
 * The section for the isNavigable property of a Property associated with an Association Object.
 * 
 * Creation 10 avr. 2006
 * 
 * @author jlescot
 */
public abstract class AbstractIsNavigableSection extends AbstractBooleanPropertySection
{
    /**
     * Handle the checkbutton modified event.
     */
    protected void handleCheckButtonModified()
    {
        // REV model modifications should be done through a Command
        if (getEObjectList().size() == 1)
        {
            /* apply the property change to single selected object */
            getProperty((Association) getEObject()).setIsNavigable(getCheckButton().getSelection());
        }
        else
        {
            /* apply the property change to all selected elements */
            for (EObject nextObject : getEObjectList())
            {
                getProperty((Association) nextObject).setIsNavigable(getCheckButton().getSelection());
            }
        }
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection#getFeature()
     */
    protected EStructuralFeature getFeature()
    {
        return null;
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection#getFeatureValue()
     */
    protected boolean getFeatureValue()
    {
        return getProperty((Association) getEObject()).isNavigable();
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractBooleanPropertySection#getLabelText()
     */
    protected String getLabelText()
    {
        return "isNavigable";
    }

    /**
     * Get the property associated with the section
     * 
     * @param association the Association model object
     * @return the Property (first end or second end) or null if not found
     */
    protected abstract Property getProperty(Association association);

}