/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.profilediagram;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.uml2.uml.util.UMLSwitch;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.edit.EListEditPart;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.uml.classdiagram.edit.PropertyEditPart;
import org.topcased.modeler.uml.profilediagram.edit.ProfileDiagramEditPart;
import org.topcased.modeler.uml.profilediagram.edit.StereotypeEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * Part Factory : associates a model object to its controller. <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class ProfileEditPartFactory implements EditPartFactory
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
     * @generated
     */
    public EditPart createEditPart(EditPart context, Object model)
    {
        if (model instanceof Diagram)
        {
            return new ProfileDiagramEditPart((Diagram) model);
        }
        else if (model instanceof GraphNode)
        {
            final GraphNode node = (GraphNode) model;
            EObject element = Utils.getElement(node);
            if (element != null)
            {
                if ("http://www.eclipse.org/uml2/2.1.0/UML".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new NodeUMLSwitch(node).doSwitch(element);
                }
                return new EMFGraphNodeEditPart(node);
            }
            return new GraphNodeEditPart(node);
        }
        else if (model instanceof GraphEdge)
        {
            final GraphEdge edge = (GraphEdge) model;
            EObject element = Utils.getElement(edge);
            if (element != null)
            {
                if ("http://www.eclipse.org/uml2/2.1.0/UML".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new EdgeUMLSwitch(edge).doSwitch(element);
                }
                return new EMFGraphEdgeEditPart(edge);
            }
            return new GraphEdgeEditPart(edge);
        }

        throw new IllegalStateException("No edit part matches with the '" + model.getClass().getName() + "' model element. Check ProfileEditPartFactory#createEditPart(EditPart,Object) class");
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class NodeUMLSwitch extends UMLSwitch<Object>
    {
        /** The graphical node */
        private GraphNode node;

        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param node the graphical node
         * @generated
         */
        public NodeUMLSwitch(GraphNode node)
        {
            this.node = node;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseStereotype(org.eclipse.uml2.uml.Stereotype)
         * @generated
         */
        public Object caseStereotype(org.eclipse.uml2.uml.Stereotype object)
        {
            String feature = DIUtils.getPropertyValue(node, ModelerPropertyConstants.ESTRUCTURAL_FEATURE_ID);
            if (!"".equals(feature))
            {
                int featureID = Integer.parseInt(feature);
                return new EListEditPart(node, object.eClass().getEStructuralFeature(featureID));
            }
            else
            {
                return new StereotypeEditPart(node);
            }
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseProperty(org.eclipse.uml2.uml.Property)
         * @generated
         */
        public Object caseProperty(org.eclipse.uml2.uml.Property object)
        {
            return new PropertyEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphNodeEditPart(node);
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class EdgeUMLSwitch extends UMLSwitch<Object>
    {
        /** The graphical edge */
        private GraphEdge edge;

        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param edge the graphical edge
         * @generated
         */
        public EdgeUMLSwitch(GraphEdge edge)
        {
            this.edge = edge;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphEdgeEditPart(edge);
        }
    }

}