/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageSort;
import org.eclipse.uml2.uml.UMLFactory;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.editor.ICreationUtils;

/**
 * Factory that creates messages with the specified sort
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * @deprecated should not be used anymore. The MessageSort property should be directly initialized in the PaletteManager
 */
public class MessageCreationFactory implements CreationFactory
{
    private ICreationUtils creation;

    private int messageSort;

    /**
     * Constructor. The allowed specified sort of messsage is the different values specified in
     * org.eclipse.uml2.MessageSort.
     * 
     * @param creationUtils the creation utils
     * @param msgSort the sort of the message
     */
    public MessageCreationFactory(ICreationUtils creationUtils, int msgSort)
    {
        super();
        this.creation = creationUtils;
        this.messageSort = msgSort;
    }

    /**
     * Create a message with the specified sort
     * 
     * @see org.eclipse.gef.requests.CreationFactory#getNewObject()
     */
    public Object getNewObject()
    {
        Message message = UMLFactory.eINSTANCE.createMessage();
        message.setMessageSort(MessageSort.get(messageSort));

        EObject commandObj = creation.createModelObject(message);
        GraphElement createdObject = creation.createGraphElement(commandObj);

        return createdObject;
    }

    /**
     * No type
     * 
     * @see org.eclipse.gef.requests.CreationFactory#getObjectType()
     */
    public Object getObjectType()
    {
        return null;
    }

}
