/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.BehaviorExecutionSpecification;
import org.eclipse.uml2.uml.CombinedFragment;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.DestructionEvent;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Message;
import org.topcased.modeler.di.model.DiagramInterchangeFactory;
import org.topcased.modeler.di.model.EdgeObjectOffset;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.editor.AbstractCreationUtils;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.uml.alldiagram.ExactUMLSwitch;

/**
 * This utility class allows to create a GraphElement associated with a Model Object <!-- begin-user-doc --> <!--
 * end-user-doc -->
 * 
 * @generated
 */
public class SequenceCreationUtils extends AbstractCreationUtils
{
    private static final int LABEL_OFFSET = 10;

    /**
     * Constructor
     * 
     * @param diagramConf the Diagram Graphical Configuration
     */
    public SequenceCreationUtils(DiagramGraphConf diagramConf)
    {
        super(diagramConf);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    private class GraphicSwitch extends ExactUMLSwitch
    {
        /** The presentation of the graphical element */
        private String presentation;

        /**
         * The constructor
         * 
         * @param presentation
         */
        public GraphicSwitch(String presentation)
        {
            this.presentation = presentation;
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseLifeline(org.eclipse.uml2.uml.Lifeline)
         */
        public Object caseLifeline(Lifeline object)
        {
            return createGraphElementLifeline(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseBehaviorExecutionSpecification(org.eclipse.uml2.uml.BehaviorExecutionSpecification)
         */
        public Object caseBehaviorExecutionSpecification(BehaviorExecutionSpecification object)
        {
            return createGraphElementBehaviorExecutionSpecification(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseDestructionEvent(org.eclipse.uml2.uml.DestructionEvent)
         */
        public Object caseDestructionEvent(DestructionEvent object)
        {
            return createGraphElementDestructionEvent(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseMessage(org.eclipse.uml2.uml.Message)
         */
        public Object caseMessage(Message object)
        {
            return createGraphElementMessage(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseComment(org.eclipse.uml2.uml.Comment)
         */
        public Object caseComment(Comment object)
        {
            return createGraphElementComment(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseCombinedFragment(org.eclipse.uml2.uml.CombinedFragment)
         */
        public Object caseCombinedFragment(CombinedFragment object)
        {
            return createGraphElementCombinedFragment(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         */
        public Object defaultCase(EObject object)
        {
            return null;
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.ICreationUtils#createGraphElement(org.eclipse.emf.ecore.EObject,
     *      java.lang.String)
     * @generated
     */
    public GraphElement createGraphElement(EObject obj, String presentation)
    {
        Object graphElt = new GraphicSwitch(presentation).doSwitch(obj);
        if (graphElt instanceof GraphElement)
        {
            return (GraphElement) graphElt;
        }

        return null;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementLifeline(Lifeline element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementBehaviorExecutionSpecification(BehaviorExecutionSpecification element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementDestructionEvent(DestructionEvent element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated NOT
     */
    protected GraphElement createGraphElementMessage(Message element, String presentation)
    {
        if (SequenceUtils.isSignal(element))
        {
            return createGraphElementSignal(element, presentation);
        }
        else
        {
            if (SequenceUtils.isSynchCallMessage(element))
            {
                return createGraphElementSyncCall(element, presentation);
            }
            else
            {
                return createGraphElementAsyncCall(element, presentation);
            }
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated NOT
     */
    private GraphElement createGraphElementSignal(Message element, String presentation)
    {
        GraphEdge graphEdge = createGraphEdge(element, presentation);
        EdgeObjectOffset nameEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
        nameEdgeObjectOffset.setId(SequenceEdgeObjectConstants.NAME_EDGE_OBJECT_ID);
        nameEdgeObjectOffset.setOffset(new Dimension(0, -LABEL_OFFSET));
        graphEdge.getContained().add(nameEdgeObjectOffset);
        return graphEdge;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated NOT
     */
    private GraphElement createGraphElementSyncCall(Message element, String presentation)
    {
        GraphEdge graphEdge = createGraphEdge(element, presentation);
        EdgeObjectOffset nameEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
        nameEdgeObjectOffset.setId(SequenceEdgeObjectConstants.NAME_EDGE_OBJECT_ID);
        nameEdgeObjectOffset.setOffset(new Dimension(0, -LABEL_OFFSET));
        graphEdge.getContained().add(nameEdgeObjectOffset);
        return graphEdge;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated NOT
     */
    private GraphElement createGraphElementAsyncCall(Message element, String presentation)
    {
        GraphEdge graphEdge = createGraphEdge(element, presentation);
        EdgeObjectOffset nameEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
        nameEdgeObjectOffset.setId(SequenceEdgeObjectConstants.NAME_EDGE_OBJECT_ID);
        nameEdgeObjectOffset.setOffset(new Dimension(0, -LABEL_OFFSET));
        graphEdge.getContained().add(nameEdgeObjectOffset);
        return graphEdge;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementComment(Comment element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementCombinedFragment(CombinedFragment element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * @see org.topcased.modeler.editor.ICreationUtils#createModelObject(org.eclipse.emf.ecore.EObject)
     */
    public EObject createModelObject(EObject obj)
    {
        return obj;
    }

}
