/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.uml2.uml.BehaviorExecutionSpecification;
import org.eclipse.uml2.uml.CombinedFragment;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.DestructionEvent;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.util.UMLSwitch;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.uml.alldiagram.edit.CommentEditPart;
import org.topcased.modeler.uml.alldiagram.edit.CommentLinkEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.BehaviorExecutionSpecificationEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.CombinedFragmentEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.DestructionEventEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.LifelineEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.MessageEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.SequenceDiagramEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * Part Factory : associates a model object to its controller. <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class SequenceEditPartFactory implements EditPartFactory
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
     * @generated
     */
    public EditPart createEditPart(EditPart context, Object model)
    {
        if (model instanceof Diagram)
        {
            return new SequenceDiagramEditPart((Diagram) model);
        }
        else if (model instanceof GraphNode)
        {
            final GraphNode node = (GraphNode) model;
            EObject element = Utils.getElement(node);
            if (element != null)
            {
                Object editPart = new UMLSwitch<Object>()
                {
                    public Object caseLifeline(Lifeline object)
                    {
                        return new LifelineEditPart(node);
                    }

                    public Object caseBehaviorExecutionSpecification(BehaviorExecutionSpecification object)
                    {
                        return new BehaviorExecutionSpecificationEditPart(node);
                    }

                    public Object caseDestructionEvent(DestructionEvent object)
                    {
                        return new DestructionEventEditPart(node);
                    }

                    public Object caseComment(Comment object)
                    {
                        return new CommentEditPart(node);
                    }

                    public Object caseCombinedFragment(CombinedFragment object)
                    {
                        return new CombinedFragmentEditPart(node);
                    }

                    public Object defaultCase(EObject object)
                    {
                        return new EMFGraphNodeEditPart(node);
                    }
                }.doSwitch(element);
                return (EditPart) editPart;
            }
            return new GraphNodeEditPart(node);
        }
        else if (model instanceof GraphEdge)
        {
            final GraphEdge edge = (GraphEdge) model;
            EObject element = Utils.getElement(edge);
            if (element != null)
            {
                Object editPart = new UMLSwitch()
                {
                    public Object caseMessage(Message object)
                    {
                        return new MessageEditPart(edge);
                    }

                    public Object defaultCase(EObject object)
                    {
                        return new EMFGraphEdgeEditPart(edge);
                    }
                }.doSwitch(element);
                return (EditPart) editPart;
            }
            if (edge.getSemanticModel() instanceof SimpleSemanticModelElement)
            {
                if (SequenceSimpleObjectConstants.SIMPLE_OBJECT_COMMENTLINK.equals(((SimpleSemanticModelElement) edge.getSemanticModel()).getTypeInfo()))
                {
                    return new CommentLinkEditPart(edge);
                }
            }
            return new GraphEdgeEditPart((GraphEdge) model);
        }

        throw new IllegalStateException("No edit part matches with the '" + model.getClass().getName() + "' model element. Check SequenceEditPartFactory#createEditPart(EditPart,Object) class");
    }

}