/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteSeparator;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.uml2.uml.CombinedFragment;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageSort;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.editor.GraphElementCreationFactory;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.palette.ModelerConnectionCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerCreationToolEntry;
import org.topcased.modeler.editor.palette.ModelerPaletteManager;

/**
 * Generated Palette Manager <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class SequencePaletteManager extends ModelerPaletteManager
{
    // declare all the palette categories of the diagram
    private PaletteDrawer objectsDrawer;

    private PaletteDrawer messagesDrawer;

    private PaletteDrawer commentDrawer;

    private PaletteDrawer fragmentsDrawer;

    private ICreationUtils creationUtils;

    /**
     * The Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param utils the creation utils for the tools of the palette
     * @generated
     */
    public SequencePaletteManager(ICreationUtils utils)
    {
        super();
        this.creationUtils = utils;
    }

    /**
     * Creates the main categories of the palette <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createCategories()
    {
        createObjectsDrawer();
        createMessagesDrawer();
        createCommentDrawer();
        createFragmentsDrawer();
    }

    /**
     * Updates the main categories of the palette <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void updateCategories()
    {
        // deletion of the existing categories and creation of the updated
        // categories

        getRoot().remove(objectsDrawer);
        createObjectsDrawer();

        getRoot().remove(messagesDrawer);
        createMessagesDrawer();

        getRoot().remove(commentDrawer);
        createCommentDrawer();

        getRoot().remove(fragmentsDrawer);
        createFragmentsDrawer();
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private void createObjectsDrawer()
    {
        objectsDrawer = new PaletteDrawer("Objects", null);
        List entries = new ArrayList();

        CreationFactory factory;
        String newObjTxt;

        ModelerCreationToolEntry objectTool;

        newObjTxt = "Lifeline";
        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getLifeline(), "default");
        objectTool = new ModelerCreationToolEntry(newObjTxt, newObjTxt, factory, SequenceImageRegistry.getImageDescriptor("LIFELINE"), SequenceImageRegistry.getImageDescriptor("LIFELINE_LARGE"));
        entries.add(objectTool);

        newObjTxt = "Execution";
        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getBehaviorExecutionSpecification(), "default");
        objectTool = new ModelerCreationToolEntry(newObjTxt, newObjTxt, factory, SequenceImageRegistry.getImageDescriptor("BEHAVIOREXECUTIONSPECIFICATION"),
                SequenceImageRegistry.getImageDescriptor("BEHAVIOREXECUTIONSPECIFICATION_LARGE"));
        entries.add(objectTool);

        objectsDrawer.addAll(entries);
        getRoot().add(objectsDrawer);
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated NOT
     */
    private void createMessagesDrawer()
    {
        messagesDrawer = new PaletteDrawer("Messages", null);
        List entries = new ArrayList();

        CreationFactory factory;
        String newObjTxt;

        ModelerConnectionCreationToolEntry connectionTool;

        newObjTxt = "Call (synch.)";
        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getMessage())
        {
            public EObject getNewModelObject()
            {
                Message element = (Message) super.getNewModelObject();
                element.setMessageSort(MessageSort.SYNCH_CALL_LITERAL);
                return element;
            }
        };
        connectionTool = new ModelerConnectionCreationToolEntry(newObjTxt, newObjTxt, factory, SequenceImageRegistry.getImageDescriptor("MESSAGE_SYNCHCALL"),
                SequenceImageRegistry.getImageDescriptor("MESSAGE_SYNCHCALL_LARGE"));
        entries.add(connectionTool);

        newObjTxt = "Reply Message";
        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getMessage())
        {
            public EObject getNewModelObject()
            {
                Message element = (Message) super.getNewModelObject();
                element.setMessageSort(MessageSort.REPLY_LITERAL);
                return element;
            }
        };
        connectionTool = new ModelerConnectionCreationToolEntry(newObjTxt, newObjTxt, factory, SequenceImageRegistry.getImageDescriptor("MESSAGE_REPLY"),
                SequenceImageRegistry.getImageDescriptor("MESSAGE_REPLY_LARGE"));
        entries.add(connectionTool);

        newObjTxt = "Call (asynch.)";
        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getMessage())
        {
            public EObject getNewModelObject()
            {
                Message element = (Message) super.getNewModelObject();
                element.setMessageSort(MessageSort.ASYNCH_CALL_LITERAL);
                return element;
            }
        };
        connectionTool = new ModelerConnectionCreationToolEntry(newObjTxt, newObjTxt, factory, SequenceImageRegistry.getImageDescriptor("MESSAGE_ASYNCHCALL"),
                SequenceImageRegistry.getImageDescriptor("MESSAGE_ASYNCHCALL_LARGE"));
        entries.add(connectionTool);

        entries.add(new PaletteSeparator());

        newObjTxt = "Signal";
        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getMessage())
        {
            public EObject getNewModelObject()
            {
                Message element = (Message) super.getNewModelObject();
                element.setMessageSort(MessageSort.ASYNCH_SIGNAL_LITERAL);
                return element;
            }
        };
        connectionTool = new ModelerConnectionCreationToolEntry(newObjTxt, newObjTxt, factory, SequenceImageRegistry.getImageDescriptor("MESSAGE_ASYNCHSIGNAL"),
                SequenceImageRegistry.getImageDescriptor("MESSAGE_ASYNCHSIGNAL_LARGE"));
        entries.add(connectionTool);

        // TODO Reactivate the CreateMessage when it will have been correctly
        // implemented
        // newObjTxt = "Create Message";
        // factory = new GraphElementCreationFactory(creationUtils,
        // UMLPackage.eINSTANCE.getMessage())
        // {
        // public EObject getNewModelObject()
        // {
        // Message element = (Message) super.getNewModelObject();
        // element.setMessageSort(MessageSort.CREATE_MESSAGE_LITERAL);
        // return element;
        // }
        // };
        // connectionTool = new ModelerConnectionCreationToolEntry(newObjTxt,
        // newObjTxt, factory,
        // SequenceImageRegistry.getImageDescriptor("MESSAGE_CREATEMESSAGE"),
        // null);
        // entries.add(connectionTool);

        newObjTxt = "Delete Message";
        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getMessage())
        {
            public EObject getNewModelObject()
            {
                Message element = (Message) super.getNewModelObject();
                element.setMessageSort(MessageSort.DELETE_MESSAGE_LITERAL);
                return element;
            }
        };
        connectionTool = new ModelerConnectionCreationToolEntry(newObjTxt, newObjTxt, factory, SequenceImageRegistry.getImageDescriptor("MESSAGE_DELETEMESSAGE"),
                SequenceImageRegistry.getImageDescriptor("MESSAGE_DELETEMESSAGE_LARGE"));
        entries.add(connectionTool);

        messagesDrawer.addAll(entries);
        getRoot().add(messagesDrawer);
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private void createCommentDrawer()
    {
        commentDrawer = new PaletteDrawer("Comment", null);
        List entries = new ArrayList();

        CreationFactory factory;
        String newObjTxt;

        ModelerCreationToolEntry objectTool;
        ModelerConnectionCreationToolEntry connectionTool;

        newObjTxt = "Comment";
        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getComment(), "default");
        objectTool = new ModelerCreationToolEntry(newObjTxt, newObjTxt, factory, SequenceImageRegistry.getImageDescriptor("COMMENT"), SequenceImageRegistry.getImageDescriptor("COMMENT_LARGE"));
        entries.add(objectTool);

        newObjTxt = "Comment Link";
        factory = new GraphElementCreationFactory(creationUtils, SequenceSimpleObjectConstants.SIMPLE_OBJECT_COMMENTLINK, "default", false);
        connectionTool = new ModelerConnectionCreationToolEntry(newObjTxt, newObjTxt, factory, SequenceImageRegistry.getImageDescriptor("COMMENTLINK"),
                SequenceImageRegistry.getImageDescriptor("COMMENTLINK_LARGE"));
        entries.add(connectionTool);

        commentDrawer.addAll(entries);
        getRoot().add(commentDrawer);
    }

    /**
     * Creates the Palette container containing all the Palette entries for each figure. <!-- begin-user-doc --> <!--
     * end-user-doc -->
     * 
     * @generated
     */
    private void createFragmentsDrawer()
    {
        fragmentsDrawer = new PaletteDrawer("Fragments", null);
        List entries = new ArrayList();

        CreationFactory factory;

        factory = new GraphElementCreationFactory(creationUtils, UMLPackage.eINSTANCE.getCombinedFragment(), "default")
        {
            public EObject getNewModelObject()
            {
                CombinedFragment element = (CombinedFragment) super.getNewModelObject();
                element.getOperands().add(UMLFactory.eINSTANCE.createInteractionOperand());
                return element;
            }
        };
        entries.add(new ModelerCreationToolEntry("CombinedFragment", "CombinedFragment", factory, SequenceImageRegistry.getImageDescriptor("COMBINEDFRAGMENT"), null));

        fragmentsDrawer.addAll(entries);
        getRoot().add(fragmentsDrawer);
    }

}
