/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditDomain;
import org.eclipse.uml2.uml.Event;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.UMLFactory;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.edit.BaseEditPart;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.uml.sequencediagram.SequenceUtils;
import org.topcased.modeler.uml.sequencediagram.edit.BehaviorExecutionSpecificationEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.SequenceDiagramEditPart;
import org.topcased.modeler.utils.LabelHelper;
import org.topcased.modeler.utils.Utils;

/**
 * Message edge creation command<br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class MessageEdgeCreationCommand extends CreateTypedEdgeCommand
{

    /** The position of the source on the source lifeline */
    private Point sourcePos;

    /** The position of the target on the target lifeline */
    private Point targetPos;

    private GraphConnector srcConnector;

    private GraphConnector targetConnector;

    private BaseEditPart sourceEdit;

    private BaseEditPart targetEdit;

    /** The Message to create */
    private Message message;

    /**
     * The MessageOccurrenceSpecification to associate on the start end of the Message
     */
    private MessageOccurrenceSpecification eventSend;

    /**
     * The MessageOccurrenceSpecification to associate on the target end of the Message
     */
    private MessageOccurrenceSpecification eventReceive;

    /**
     * The Event to associate with the MessageOccurrenceSpecification(s) of the Message
     */
    private Event event;

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param domain the edit domain
     * @param newObj the graph edge of the new connection
     * @param src the graph element of the source
     * @generated
     */
    public MessageEdgeCreationCommand(EditDomain domain, GraphEdge newObj, GraphElement src)
    {
        this(domain, newObj, src, true);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param domain the edit domain
     * @param newObj the graph edge of the new connection
     * @param src the graph element of the source
     * @param needModelUpdate set it to true if the model need to be updated
     * @generated NOT
     */
    public MessageEdgeCreationCommand(EditDomain domain, GraphEdge newObj, GraphElement src, boolean needModelUpdate)
    {
        super(domain, newObj, src, needModelUpdate);
        message = (Message) Utils.getElement(getEdge());
        eventSend = null;
        eventReceive = null;
    }

    /**
     * Return the created message
     * 
     * @return the message
     */
    public Message getMessage()
    {
        return message;
    }

    /**
     * Set the source position of the edge
     * 
     * @param srcPos the source position on the lifeline
     */
    public void setSourcePos(Point srcPos)
    {
        this.sourcePos = srcPos;
    }

    /**
     * Get the source position on the lifeline
     * 
     * @return the source position on the lifeline
     */
    public Point getSourcePos()
    {
        return sourcePos;
    }

    /**
     * Set the target position of the edge
     * 
     * @param tgtPos the target position on the lifeline
     */
    public void setTargetPos(Point tgtPos)
    {
        this.targetPos = tgtPos;
    }

    /**
     * Get the target position on the lifeline
     * 
     * @return the target position on the lifeline
     */
    public Point getTargetPos()
    {
        return targetPos;
    }

    /**
     * Set the edit part of the source node
     * 
     * @param source the edit part of the source node
     */
    public void setSourceEditPart(BaseEditPart source)
    {
        this.sourceEdit = source;
    }

    /**
     * Get the edit part of the source node
     * 
     * @return the edit part of the source node
     */
    public BaseEditPart getSourceEditPart()
    {
        return sourceEdit;
    }

    /**
     * Set the edit part of the target node
     * 
     * @param target the edit part of the target node
     */
    public void setTargetEditPart(BaseEditPart target)
    {
        this.targetEdit = target;
    }

    /**
     * Get the edit part of the target node
     * 
     * @return the edit part of the target node
     */
    public BaseEditPart getTargetEditPart()
    {
        return targetEdit;
    }

    /**
     * Get the source lifeline. It can be null if it is a found message.
     * 
     * @return the source lifeline
     */
    public Lifeline getSrcLifeline()
    {
        if (getSource() instanceof Diagram)
        {
            return null;
        }

        if (SequenceUtils.isCall(message) || SequenceUtils.isReplyMessage(message))
        {
            return (Lifeline) Utils.getElement(getSource().getContainer());
        }
        else
        {
            return (Lifeline) Utils.getElement(getSource());
        }
    }

    /**
     * Get the target lifeline. It can be null if it is a lost message.
     * 
     * @return the target lifeline
     */
    public Lifeline getTargetLifeline()
    {
        if (getTarget() instanceof Diagram)
        {
            return null;
        }

        if (SequenceUtils.isCall(message) || SequenceUtils.isReplyMessage(message))
        {
            return (Lifeline) Utils.getElement(getTarget().getContainer());
        }
        else
        {
            return (Lifeline) Utils.getElement(getTarget());
        }
    }

    /**
     * Set graphic properties
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        Point srcPos = getSourcePos().getCopy();
        IFigure scrFig;
        if (sourceEdit instanceof SequenceDiagramEditPart)
        {
            scrFig = (IFigure) sourceEdit.getFigure().getChildren().get(0);
        }
        else
        {
            scrFig = sourceEdit.getFigure();
        }
        srcPos.translate(scrFig.getBounds().getLocation().getNegated());

        Point tgtPos = getTargetPos().getCopy();
        IFigure tgtFig;
        if (targetEdit instanceof SequenceDiagramEditPart)
        {
            tgtFig = (IFigure) targetEdit.getFigure().getChildren().get(0);
        }
        else
        {
            tgtFig = targetEdit.getFigure();
        }
        tgtPos.translate(tgtFig.getBounds().getLocation().getNegated());

        srcConnector = SequenceUtils.createGraphConnector(srcPos, getSource(), getEdge());
        targetConnector = SequenceUtils.createGraphConnector(tgtPos, getTarget(), getEdge());

        if (getContainer() != null)
        {
            // add the edge to the graphical view
            getContainer().getContained().add(getEdge());
        }

        redoModel();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void redoModel()
    {
        // Create the Event to associate with the Message
        if (SequenceUtils.isSignal(message))
        {
            event = UMLFactory.eINSTANCE.createSignalEvent();
        }
        else if (SequenceUtils.isCall(message) || SequenceUtils.isReplyMessage(message))
        {
            event = UMLFactory.eINSTANCE.createCallEvent();
        }
        else if (SequenceUtils.isCreateMessage(message))
        {
            event = UMLFactory.eINSTANCE.createCreationEvent();
        }
        else if (SequenceUtils.isDeleteMessage(message))
        {
            event = UMLFactory.eINSTANCE.createDestructionEvent();
        }

        // If the sourceNode is the Lifeline or the ExecutionSpecification of a
        // Lifeline, we create the MessageOccurrenceSpecification to associate
        // with the SendEvent of the Message
        if (sourceEdit instanceof GraphNodeEditPart)
        {
            eventSend = UMLFactory.eINSTANCE.createMessageOccurrenceSpecification();
            message.setSendEvent(eventSend);
            eventSend.setMessage(message);
            // Do not reference the DestructionEvent in the
            // MessageOccurrenceSpecification of the source Lifeline
            if (!SequenceUtils.isDeleteMessage(message))
            {
                eventSend.setEvent(event);
            }
            SequenceUtils.insertOccurrenceSpecification(getEdge(), sourceEdit.getFigure(), eventSend, getSrcLifeline(), getSourcePos());
            eventSend.getCovereds().add(getSrcLifeline());
        }

        // If the targetNode is the Lifeline or the ExecutionSpecification of a
        // Lifeline, we create the MessageOccurrenceSpecification to associate
        // with the ReceiveEvent of the Message
        if (targetEdit instanceof GraphNodeEditPart)
        {
            eventReceive = UMLFactory.eINSTANCE.createMessageOccurrenceSpecification();
            message.setReceiveEvent(eventReceive);
            eventReceive.setMessage(message);
            eventReceive.setEvent(event);
            SequenceUtils.insertOccurrenceSpecification(getEdge(), sourceEdit.getFigure(), eventReceive, getTargetLifeline(), getTargetPos());
            eventReceive.getCovereds().add(getTargetLifeline());
        }

        // Add the Event into the nearest Package containing the Interaction
        Package nearestPackage = getInteraction().getNearestPackage();
        nearestPackage.getPackagedElements().add(event);

        // Initialize the names of the created model objects
        LabelHelper.INSTANCE.initName(getEditDomain(), getInteraction(), message);
        LabelHelper.INSTANCE.initName(getEditDomain(), getInteraction(), eventSend);
        LabelHelper.INSTANCE.initName(getEditDomain(), getInteraction(), eventReceive);
        LabelHelper.INSTANCE.initName(getEditDomain(), nearestPackage, event);

        // Update the remaining references between the model objects that were
        // created
        message.setInteraction(getInteraction());
        if (SequenceUtils.isCall(message) || SequenceUtils.isReplyMessage(message))
        {
            if (eventSend != null)
            {
                SequenceUtils.updateStartAndFinishOccurrenceSpecification((BehaviorExecutionSpecificationEditPart) sourceEdit);
            }
            if (eventReceive != null)
            {
                SequenceUtils.updateStartAndFinishOccurrenceSpecification((BehaviorExecutionSpecificationEditPart) targetEdit);
            }
        }
    }

    /**
     * Unset graphic properties
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        SequenceUtils.removeGraphConnector(srcConnector, getSource(), getEdge());
        SequenceUtils.removeGraphConnector(targetConnector, getTarget(), getEdge());

        if (getContainer() != null)
        {
            // remove the edge to the graphical view
            getContainer().getContained().remove(getEdge());
        }

        undoModel();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void undoModel()
    {
        Interaction interaction = getInteraction();

        EList list = interaction.getFragments();

        if (eventSend != null)
        {
            list.remove(eventSend);
            eventSend.getCovereds().remove(getSrcLifeline());
        }

        if (eventReceive != null)
        {
            list.remove(eventReceive);
            eventReceive.getCovereds().remove(getTargetLifeline());
        }

        // Remove the created Event
        Package nearestPackage = getInteraction().getNearestPackage();
        nearestPackage.getPackagedElements().remove(event);

        message.setInteraction(null);

        message.setReceiveEvent(null);
        message.setSendEvent(null);

        if (SequenceUtils.isCall(message) || SequenceUtils.isReplyMessage(message))
        {
            if (eventSend != null)
            {
                SequenceUtils.updateStartAndFinishOccurrenceSpecification((BehaviorExecutionSpecificationEditPart) sourceEdit);
            }
            if (eventReceive != null)
            {
                SequenceUtils.updateStartAndFinishOccurrenceSpecification((BehaviorExecutionSpecificationEditPart) targetEdit);
            }
        }
    }

    /**
     * Get the Interaction linked with the current diagram
     * 
     * @return the Interaction model object
     */
    private Interaction getInteraction()
    {
        if (getSrcLifeline() != null)
        {
            return getSrcLifeline().getInteraction();
        }

        return getTargetLifeline().getInteraction();
    }
}