/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.creation;

import java.util.List;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.uml2.uml.BehaviorExecutionSpecification;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Lifeline;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.utils.LabelHelper;
import org.topcased.modeler.utils.Utils;

/**
 * Command to create an Execution Occurrence
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class BehaviorExecutionSpecificationCreationCommand extends CreateGraphNodeCommand
{
    private GraphNode lifelineNode;

    private GraphNode execNode;

    /**
     * Constructor
     * 
     * @param domain the EditDomain (used to init the name)
     * @param newObject the node to create
     * @param newParent the parentNode node
     * @param newContainerParent the model object that should contain the new childNode object
     * @param location the location of the created node
     * @param dimension the size of the created node
     * @param attach the newAttachment of the created node
     * @param featuresList the avalaible model element feature of the parentNode node where the model element of the
     *        childNode node can be added
     * @param needModelUpdate <code>true</code> if the model must be modified during this command, <code>false</code>
     *        if objects already exist in the mode.
     */
    public BehaviorExecutionSpecificationCreationCommand(EditDomain domain, GraphNode newObject, GraphNode newParent, EObject newContainerParent, Point location, Dimension dimension, int attach,
            List featuresList, boolean needModelUpdate)
    {
        super(domain, newObject, newParent, newContainerParent, location, dimension, attach, featuresList, needModelUpdate);
        this.lifelineNode = newParent;
        execNode = newObject;
    }

    /**
     * Create the Execution Occurrence
     * 
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#redoModel()
     */
    protected void redoModel()
    {
        LabelHelper.INSTANCE.initName(getEditDomain(), getParentEObject(), getChildEObject());

        Interaction interaction = (Interaction) getParentEObject();

        BehaviorExecutionSpecification exec = (BehaviorExecutionSpecification) getChildEObject();
        Lifeline lifeline = (Lifeline) Utils.getElement(lifelineNode);

        exec.setEnclosingInteraction(interaction);

        exec.getCovereds().add(lifeline);

    }

    /**
     * Remove the Execution Occurrence
     * 
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#undoModel()
     */
    protected void undoModel()
    {
        super.undoModel();
        BehaviorExecutionSpecification exec = (BehaviorExecutionSpecification) getChildEObject();
        Lifeline lifeline = (Lifeline) Utils.getElement(lifelineNode);
        exec.getCovereds().remove(lifeline);
    }

    /**
     * Return the graph node of the Execution Occurrence
     * 
     * @return the graph node of the Execution Occurrence
     */
    public GraphNode getExecutionOccurrenceGraphNode()
    {
        return execNode;
    }
}
