/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.creation;

import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Package;

/**
 * Command of creation of a Classifier in a given Package
 * 
 * Creation 13 juin 06
 * 
 * @author jlescot
 */
public class ClassifierCreationCommand extends Command
{
    private final Package owningPackage;

    private Classifier classifier;

    /**
     * Constructor
     * 
     * @param owningPackage the package owning the classifier
     * @param classifier the newly created classifier
     */
    public ClassifierCreationCommand(Package owningPackage, Classifier classifier)
    {
        this.owningPackage = owningPackage;
        this.classifier = classifier;
    }

    /**
     * Just redo
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        redo();
    }

    /**
     * Create the classifier into the owningPackage
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        owningPackage.getPackagedElements().add(classifier);
    }

    /**
     * Remove the classifier from the owningPackage
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        owningPackage.getPackagedElements().remove(classifier);
    }
}
