/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.creation;

import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.CombinedFragment;
import org.eclipse.uml2.uml.Lifeline;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.uml.sequencediagram.edit.LifelineEditPart;
import org.topcased.modeler.utils.LabelHelper;

/**
 * Command to create a CombinedFragment
 * 
 * Creation 8 juin 06
 * 
 * @author jlescot
 */
public class CombinedFragmentCreationCommand extends CreateGraphNodeCommand
{
    /** The EditPart that will contain the CombinedFragment EditPart */
    private EditPart interactionEditPart;

    /** The bounds of the CombinedFragment node */
    private Rectangle combinedFragmentBounds;

    /**
     * Constructor
     * 
     * @param domain the EditDomain (used to init the name)
     * @param newObject the node to create
     * @param newParent the parentNode node
     * @param newContainerParent the model object that should contain the new childNode object
     * @param location the location of the created node
     * @param dimension the size of the created node
     * @param attach the newAttachment of the created node
     * @param featuresList the avalaible model element feature of the parentNode node where the model element of the
     *        childNode node can be added
     * @param needModelUpdate <code>true</code> if the model must be modified during this command, <code>false</code>
     *        if objects already exist in the mode.
     * @param parentEditPart the EditPart that will contain the CombinedFragment EditPart
     */
    public CombinedFragmentCreationCommand(EditDomain domain, GraphNode newObject, GraphNode newParent, EObject newContainerParent, Point location, Dimension dimension, int attach, List featuresList,
            boolean needModelUpdate, EditPart parentEditPart)
    {
        super(domain, newObject, newParent, newContainerParent, location, dimension, attach, featuresList, needModelUpdate);

        this.combinedFragmentBounds = new Rectangle(location == null ? new Point() : location, dimension == null ? new Dimension() : dimension);
        this.interactionEditPart = parentEditPart;
    }

    /**
     * Create the Execution Occurrence
     * 
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#redoModel()
     */
    protected void redoModel()
    {
        super.redoModel();

        LabelHelper.INSTANCE.initName(getEditDomain(), getParentEObject(), getChildEObject());

        Iterator iterEditParts = interactionEditPart.getChildren().iterator();
        while (iterEditParts.hasNext())
        {
            EditPart editPart = (EditPart) iterEditParts.next();
            if (editPart instanceof LifelineEditPart && ((LifelineEditPart) editPart).getFigure().intersects(combinedFragmentBounds))
            {
                ((CombinedFragment) getChildEObject()).getCovereds().add((Lifeline) ((LifelineEditPart) editPart).getEObject());
            }
        }
    }

    /**
     * Remove the Execution Occurrence
     * 
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#undoModel()
     */
    protected void undoModel()
    {
        super.undoModel();

        ((CombinedFragment) getChildEObject()).getCovereds().clear();
    }

}
