/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.creation;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageSort;
import org.eclipse.uml2.uml.UMLFactory;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.uml.sequencediagram.SequenceCreationUtils;
import org.topcased.modeler.uml.sequencediagram.commands.MessageEdgeCreationCommand;

/**
 * Command to create a synchronous call message
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * @deprecated Stop using this command. Now the two messages are created separately with the MessageEdgeCreationCommand
 *             and the MessageAndExecutionCreationCommand
 */
public class SyncCallEdgeCreationCommand extends MessageEdgeCreationCommand
{

    private static final int RETURN_MESSAGE_OFFSET = 30;

    private static final int RETURN_SELF_MESSAGE_OFFSET = 30;

    private EditDomain domain;

    private EditPartViewer viewer;

    private MessageEdgeCreationCommand returnedMessageCmd;

    private boolean isSelf;

    /**
     * Contructor
     * 
     * @param v the editPart viewer
     * @param newObj the graph edge of the new message
     * @param src the source graph node
     */
    public SyncCallEdgeCreationCommand(EditPartViewer v, GraphEdge newObj, GraphNode src)
    {
        super(v.getEditDomain(), newObj, src);
        viewer = v;
        domain = v.getEditDomain();
    }

    /**
     * Set if it's a self edge
     * 
     * @param self set it to true if it's a self edge
     */
    public void setSelf(boolean self)
    {
        isSelf = self;
    }

    /**
     * Create the return message command
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        SequenceCreationUtils interactionCreationUtils = (SequenceCreationUtils) ((ModelerGraphicalViewer) viewer).getModelerEditor().getActiveConfiguration().getCreationUtils();

        Message replyMessage = UMLFactory.eINSTANCE.createMessage();
        replyMessage.setMessageSort(MessageSort.REPLY_LITERAL);

        GraphEdge edgeReplyMessage = (GraphEdge) interactionCreationUtils.createGraphElement(replyMessage);

        Point sourcePos2 = getTargetPos().getCopy();
        sourcePos2.y += RETURN_MESSAGE_OFFSET;

        Point targetPos2 = getSourcePos().getCopy();
        targetPos2.y += RETURN_MESSAGE_OFFSET;
        if (isSelf)
        {
            targetPos2.y += RETURN_SELF_MESSAGE_OFFSET;
        }

        returnedMessageCmd = new MessageEdgeCreationCommand(domain, edgeReplyMessage, getTarget());
        returnedMessageCmd.setSourceEditPart(getTargetEditPart());
        returnedMessageCmd.setTargetEditPart(getSourceEditPart());
        // returnedMessageCmd.setContainer(getContainer());
        returnedMessageCmd.setSourcePos(sourcePos2);
        returnedMessageCmd.setTargetPos(targetPos2);
        returnedMessageCmd.setTarget(getSource());

        redo();
    }

    /**
     * Create the complete message
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        super.redo();
        returnedMessageCmd.redo();
    }

    /**
     * Remove the complete message
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        returnedMessageCmd.undo();
        super.undo();
    }

}