/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.reconnect;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.topcased.modeler.edit.BaseEditPart;
import org.topcased.modeler.uml.sequencediagram.SequenceUtils;
import org.topcased.modeler.uml.sequencediagram.edit.MessageEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.SequenceDiagramEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * Comand to reconnect the messages
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 */
public class MessageEdgeReconnectCommand extends GraphEdgeReconnectCommand
{

    /**
     * Contructor
     * 
     * @param newEdit the edit part of the new receiving node
     * @param edgeEdit the edit part of the graph edge
     * @param isStart set it to true if the source is reconnected
     * @param newP the new position of the connector
     */
    public MessageEdgeReconnectCommand(BaseEditPart newEdit, MessageEditPart edgeEdit, boolean isStart, Point newP)
    {
        super(newEdit, edgeEdit, isStart, newP);
    }

    /**
     * Set the correct new position
     * 
     * @see org.topcased.modeler.uml.sequencediagram.commands.reconnect.GraphEdgeReconnectCommand#getOldValues()
     */
    protected void getOldValues()
    {
        super.getOldValues();

        Point oppositePos;

        if (isMovingStartAnchor())
        {
            oppositePos = SequenceUtils.getTargetAbsolutePosition((MessageEditPart) getGraphEdgeEdit());
        }
        else
        {
            oppositePos = SequenceUtils.getSourceAbsolutePosition((MessageEditPart) getGraphEdgeEdit());
        }

        if (getNewNodeEdit() instanceof SequenceDiagramEditPart)
        {
            getNewPos().y = oppositePos.y;
            getNewPos().translate(((IFigure) getNewNodeEdit().getFigure().getChildren().get(0)).getBounds().getLocation().getNegated());
        }
        else
        {
            SequenceUtils.alignPosAnchor(getNewPos(), oppositePos);
            getNewPos().translate(getNewNodeEdit().getFigure().getBounds().getLocation().getNegated());
        }
    }

    /**
     * Update the events
     * 
     * @see org.topcased.modeler.uml.sequencediagram.commands.reconnect.GraphEdgeReconnectCommand#setModelValues()
     */
    protected void setModelValues()
    {
        Message message = (Message) Utils.getElement(getGraphEdge());

        MessageOccurrenceSpecification changedEvent;

        if (isMovingStartAnchor())
        {
            changedEvent = (MessageOccurrenceSpecification) message.getSendEvent();
        }
        else
        {
            changedEvent = (MessageOccurrenceSpecification) message.getReceiveEvent();
        }

        if (changedEvent != null)
        {
            SequenceUtils.updateOccurrenceSpecification(getGraphEdge(), changedEvent, getNewPos(), getOldNodeEdit(), getNewNodeEdit());
        }
    }
}