/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.reconnect;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.edit.BaseEditPart;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.uml.sequencediagram.SequenceUtils;
import org.topcased.modeler.utils.Utils;

/**
 * Command that moves a message
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class MessageMoveCommand extends GraphEdgeMoveCommand
{

    /**
     * Constructor
     * 
     * @param edgeEdit the edit part of the graph edge to move
     * @param initialPos the initial position of the connection
     * @param newPos the new position of the connection
     */
    public MessageMoveCommand(GraphEdgeEditPart edgeEdit, Point initialPos, Point newPos)
    {
        super(edgeEdit, initialPos, newPos);
    }

    /**
     * Set the correct new positions
     * 
     * @see org.topcased.modeler.uml.sequencediagram.commands.reconnect.GraphEdgeMoveCommand#getValues()
     */
    protected void getValues()
    {
        super.getValues();

        BaseEditPart sourceEdit = (BaseEditPart) getGraphEdgeEdit().getSource();

        Point src = getNewSourcePosition();
        if (src.y < 0)
        {
            src.y = 0;
        }
        if (src.y > sourceEdit.getFigure().getBounds().height)
        {
            src.y = sourceEdit.getFigure().getBounds().height;
        }

        BaseEditPart targetEdit = (BaseEditPart) getGraphEdgeEdit().getTarget();

        Point tgt = getNewTargetPosition();
        if (tgt.y < 0)
        {
            tgt.y = 0;
        }
        if (tgt.y > targetEdit.getFigure().getBounds().height)
        {
            tgt.y = targetEdit.getFigure().getBounds().height;
        }

    }

    /**
     * Update the events
     * 
     * @see org.topcased.modeler.uml.sequencediagram.commands.reconnect.GraphEdgeMoveCommand#setModelValues()
     */
    protected void setModelValues()
    {
        GraphEdge edge = (GraphEdge) getGraphEdgeEdit().getModel();
        Message message = (Message) Utils.getElement(edge);

        MessageOccurrenceSpecification sourceEvent = (MessageOccurrenceSpecification) message.getSendEvent();
        MessageOccurrenceSpecification targetEvent = (MessageOccurrenceSpecification) message.getReceiveEvent();

        if (sourceEvent != null)
        {
            BaseEditPart nodeEdit = (BaseEditPart) getGraphEdgeEdit().getSource();
            SequenceUtils.updateOccurrenceSpecification(edge, sourceEvent, getNewSourcePosition(), nodeEdit, nodeEdit);
        }

        if (targetEvent != null)
        {
            BaseEditPart nodeEdit = (BaseEditPart) getGraphEdgeEdit().getTarget();
            SequenceUtils.updateOccurrenceSpecification(edge, targetEvent, getNewTargetPosition(), nodeEdit, nodeEdit);
        }
    }

}
