/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.update;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.CallEvent;
import org.eclipse.uml2.uml.Event;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.SignalEvent;

/**
 * Command to update the message parameters
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 */
public class MessageUpdateCommand extends Command
{

    private Message message;

    private NamedElement signature;

    private Collection valueSpecs;

    private Collection oldValueSpecs;

    private NamedElement oldSignature;

    /**
     * Constructor
     * 
     * @param msg the message to update
     * @param sig the signature
     * @param values the list of ValueSpecification
     */
    public MessageUpdateCommand(Message msg, NamedElement sig, Collection values)
    {
        message = msg;
        signature = sig;
        valueSpecs = values;
    }

    /**
     * Get the old values and set the new ones
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        getOldValues();
        setValues();
    }

    /**
     * Get the old values
     */
    protected void getOldValues()
    {
        oldValueSpecs = new ArrayList(message.getArguments());

        oldSignature = message.getSignature();
    }

    /**
     * Switch the old and new values
     */
    protected void switchValues()
    {
        Collection tempValues = valueSpecs;
        valueSpecs = oldValueSpecs;
        oldValueSpecs = tempValues;

        NamedElement tempSignature = signature;
        signature = oldSignature;
        oldSignature = tempSignature;
    }

    /**
     * Set the values on the message
     */
    protected void setValues()
    {
        message.getArguments().removeAll(oldValueSpecs);

        message.getArguments().addAll(valueSpecs);

        // update the Signture of the Message
        if (signature == null)
        {
            Event event = ((MessageOccurrenceSpecification) message.getSendEvent()).getEvent();
            if (event instanceof CallEvent)
            {
                ((CallEvent) event).setOperation(null);
            }
            else if (event instanceof SignalEvent)
            {
                ((SignalEvent) event).setSignal(null);
            }

        }
        else if (signature instanceof Operation)
        {
            ((CallEvent) ((MessageOccurrenceSpecification) message.getSendEvent()).getEvent()).setOperation((Operation) signature);
        }
        else if (signature instanceof Signal)
        {
            ((SignalEvent) ((MessageOccurrenceSpecification) message.getSendEvent()).getEvent()).setSignal((Signal) signature);
        }
    }

    /**
     * Update the message parameters
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        switchValues();
        setValues();
    }

    /**
     * Undo the update
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        switchValues();
        setValues();
    }

}
