/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.update;

import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.LiteralBoolean;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.ValueSpecification;

/**
 * Command for updating the Value property of a ValueSpecification model object
 * 
 * Creation 16 juin 06
 * 
 * @author jlescot
 */
public class UpdateValueSpecificationValueCommand extends Command
{
    /** The ValueSpecification to update */
    private ValueSpecification valueSpecification;

    private String oldValue;

    private String newValue;

    /**
     * Create a command for updating the value of a given ValueSpecification
     * 
     * @param valueSpec the ValueSpecification model object
     * @param newValue the new value represented as a String
     */
    public UpdateValueSpecificationValueCommand(ValueSpecification valueSpec, String newValue)
    {
        this.valueSpecification = valueSpec;
        this.newValue = newValue;
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        oldValue = valueSpecification.stringValue();

        redo();
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        updateValue(newValue);
    }

    /**
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        updateValue(oldValue);
    }

    /**
     * This method change the value of the ValueSpecification
     * 
     * @param value the new value to set
     */
    private void updateValue(String value)
    {
        if (valueSpecification instanceof LiteralString)
        {
            ((LiteralString) valueSpecification).setValue(value);
        }
        else if (valueSpecification instanceof LiteralInteger)
        {
            ((LiteralInteger) valueSpecification).setValue(Integer.parseInt(value));
        }
        else if (valueSpecification instanceof LiteralBoolean)
        {
            ((LiteralBoolean) valueSpecification).setValue(Boolean.getBoolean(value));
        }
    }
}
