/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.edit;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.uml2.uml.Message;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.BaseEditPart;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.alldiagram.edit.ElementEditPart;
import org.topcased.modeler.uml.sequencediagram.SequenceEditPolicyConstants;
import org.topcased.modeler.uml.sequencediagram.SequenceUtils;
import org.topcased.modeler.uml.sequencediagram.anchors.BehaviorExecutionSpecificationAnchor;
import org.topcased.modeler.uml.sequencediagram.commands.BehaviorExecutionSpecificationRestoreConnectionCommand;
import org.topcased.modeler.uml.sequencediagram.commands.MessageEdgeCreationCommand;
import org.topcased.modeler.uml.sequencediagram.figures.BehaviorExecutionSpecificationFigure;
import org.topcased.modeler.uml.sequencediagram.policies.MessageEdgeCreationEditPolicy;
import org.topcased.modeler.uml.sequencediagram.policies.SequenceDeleteModelObjectEditPolicy;
import org.topcased.modeler.uml.sequencediagram.requests.GraphEdgeMoveRequest;
import org.topcased.modeler.utils.Utils;

/**
 * The BehaviorExecutionSpecification object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class BehaviorExecutionSpecificationEditPart extends ElementEditPart
{
    /** the default width of a Execution Occurence */
    public static final int DEFAULT_WIDTH = 20;

    /** the default height of a Execution Occurence */
    public static final int DEFAULT_HEIGHT = 40;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public BehaviorExecutionSpecificationEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(SequenceEditPolicyConstants.MESSAGE_EDITPOLICY, new MessageEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new BehaviorExecutionSpecificationRestoreConnectionCommand(getHost());
            }
        });

        ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(PositionConstants.NORTH_SOUTH);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

        // REV Check whether the removeEditPolicy() method is necessary or not
        // removing default edit policy for deleting model objects for messages
        // and behaviorExecutionSpecification.
        removeEditPolicy(ModelerEditPolicyConstants.DELETE_MODEL_OBJECT_EDITPOLICY);
        // and adding our custom one.
        installEditPolicy(ModelerEditPolicyConstants.DELETE_MODEL_OBJECT_EDITPOLICY, new SequenceDeleteModelObjectEditPolicy());

    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new BehaviorExecutionSpecificationFigure();
    }

    /**
     * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
     */
    public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection)
    {
        GraphEdge edge = (GraphEdge) connection.getModel();
        EList<GraphConnector> connectors = edge.getAnchor();
        if (!connectors.isEmpty())
        {
            GraphConnector connector = connectors.get(0);
            return new BehaviorExecutionSpecificationAnchor(getFigure(), connector.getPosition());
        }
        return super.getSourceConnectionAnchor(connection);
    }

    /**
     * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
     */
    public ConnectionAnchor getSourceConnectionAnchor(Request request)
    {
        BehaviorExecutionSpecificationFigure fig = (BehaviorExecutionSpecificationFigure) getFigure();
        Point pos = null;
        Point oppositePoint = null;

        if (request instanceof CreateConnectionRequest)
        {
            CreateConnectionRequest createRequest = (CreateConnectionRequest) request;

            pos = createRequest.getLocation().getCopy();

            BaseEditPart sourceEdit = (BaseEditPart) createRequest.getSourceEditPart();
            sourceEdit.getFigure().translateToRelative(pos);
        }
        else if (request instanceof ReconnectRequest)
        {
            ReconnectRequest reconnectRequest = (ReconnectRequest) request;
            oppositePoint = SequenceUtils.getTargetAbsolutePosition((MessageEditPart) reconnectRequest.getConnectionEditPart());
            pos = reconnectRequest.getLocation().getCopy();

            BaseEditPart sourceEdit = (BaseEditPart) reconnectRequest.getTarget();
            sourceEdit.getFigure().translateToRelative(pos);
        }
        else if (request instanceof GraphEdgeMoveRequest)
        {
            GraphEdgeMoveRequest moveRequest = (GraphEdgeMoveRequest) request;
            pos = SequenceUtils.getSourceAbsolutePosition((MessageEditPart) moveRequest.getGraphEdgeEditPart());
            pos.y += moveRequest.getDelta();
        }

        return new BehaviorExecutionSpecificationAnchor(fig, pos, oppositePoint);
    }

    /**
     * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
     */
    public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection)
    {
        GraphEdge edge = (GraphEdge) connection.getModel();
        EObject eobj = Utils.getElement(edge);
        if (eobj instanceof Message)
        {
            EList<GraphConnector> connectors = edge.getAnchor();

            if (edge.getAnchor().size() >= 2)
            {
                GraphConnector connector = connectors.get(1);
                return new BehaviorExecutionSpecificationAnchor(getFigure(), connector.getPosition());
            }
        }
        return super.getTargetConnectionAnchor(connection);
    }

    /**
     * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
     */
    public ConnectionAnchor getTargetConnectionAnchor(Request request)
    {
        BehaviorExecutionSpecificationFigure fig = (BehaviorExecutionSpecificationFigure) getFigure();

        Point pos = null;
        Point oppositePoint = null;

        if (request instanceof CreateConnectionRequest)
        {
            CreateConnectionRequest createRequest = (CreateConnectionRequest) request;

            Command command = createRequest.getStartCommand();

            if (!(command instanceof MessageEdgeCreationCommand))
            {
                return super.getTargetConnectionAnchor(request);
            }

            if (SequenceUtils.isCall((Message) Utils.getElement((GraphElement) createRequest.getNewObject()))
                    || SequenceUtils.isReplyMessage((Message) Utils.getElement((GraphElement) createRequest.getNewObject())))
            {
                return null;
            }

            MessageEdgeCreationCommand callCommand = (MessageEdgeCreationCommand) command;

            oppositePoint = callCommand.getSourcePos();
            pos = createRequest.getLocation().getCopy();

            BaseEditPart targetEdit = (BaseEditPart) createRequest.getTargetEditPart();
            targetEdit.getFigure().translateToRelative(pos);
        }
        else if (request instanceof ReconnectRequest)
        {
            ReconnectRequest reconnectRequest = (ReconnectRequest) request;

            ConnectionEditPart edit = reconnectRequest.getConnectionEditPart();
            if (!(edit instanceof MessageEditPart))
            {
                return super.getTargetConnectionAnchor(request);
            }

            Message message = (Message) Utils.getElement((GraphElement) edit.getModel());
            if (SequenceUtils.isSignal(message))
            {
                return null;
            }

            oppositePoint = SequenceUtils.getSourceAbsolutePosition((MessageEditPart) reconnectRequest.getConnectionEditPart());
            pos = reconnectRequest.getLocation().getCopy();

            BaseEditPart targetEdit = (BaseEditPart) reconnectRequest.getTarget();
            targetEdit.getFigure().translateToRelative(pos);
        }
        else if (request instanceof GraphEdgeMoveRequest)
        {
            GraphEdgeMoveRequest moveRequest = (GraphEdgeMoveRequest) request;

            pos = SequenceUtils.getTargetAbsolutePosition((MessageEditPart) moveRequest.getGraphEdgeEditPart());
            pos.y += moveRequest.getDelta();
        }

        return new BehaviorExecutionSpecificationAnchor(fig, pos, oppositePoint);
    }

}