/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.edit;

import java.util.Iterator;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.uml2.uml.ConnectableElement;
import org.eclipse.uml2.uml.DestructionEvent;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.edit.BaseEditPart;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.figures.NodeAnchor;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.UMLLabel;
import org.topcased.modeler.uml.alldiagram.commands.CommentLinkEdgeCreationCommand;
import org.topcased.modeler.uml.alldiagram.edit.ElementEditPart;
import org.topcased.modeler.uml.sequencediagram.SequenceEditPolicyConstants;
import org.topcased.modeler.uml.sequencediagram.SequenceSimpleObjectConstants;
import org.topcased.modeler.uml.sequencediagram.SequenceUtils;
import org.topcased.modeler.uml.sequencediagram.anchors.LifelineAnchor;
import org.topcased.modeler.uml.sequencediagram.commands.LifelineRestoreConnectionCommand;
import org.topcased.modeler.uml.sequencediagram.commands.MessageEdgeCreationCommand;
import org.topcased.modeler.uml.sequencediagram.figures.LifelineFigure;
import org.topcased.modeler.uml.sequencediagram.policies.LifelineLayoutEditPolicy;
import org.topcased.modeler.uml.sequencediagram.policies.MessageEdgeCreationEditPolicy;
import org.topcased.modeler.uml.sequencediagram.requests.GraphEdgeMoveRequest;
import org.topcased.modeler.utils.Utils;

/**
 * The Lifeline object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class LifelineEditPart extends ElementEditPart
{

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public LifelineEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(SequenceEditPolicyConstants.MESSAGE_EDITPOLICY, new MessageEdgeCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new LifelineRestoreConnectionCommand(getHost());
            }
        });

        // prevent the Resizing of the Lifeline in the NORTH direction
        ResizableEditPolicy resizableEditPolicy = new ResizableEditPolicy();
        resizableEditPolicy.setResizeDirections(PositionConstants.EAST_WEST | PositionConstants.SOUTH);
        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, resizableEditPolicy);

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new LifelineLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new LifelineFigure();
    }

    /**
     * The editable part of the label is the prefix
     * 
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getEditableLabel()
     */
    public ILabel getEditableLabel()
    {
        ComposedLabel label = (ComposedLabel) ((ComposedLabel) getLabel()).getMain();
        return label.getPrefix();
    }

    /**
     * Refresh the Lifeline label
     * 
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        ComposedLabel fullLabel = (ComposedLabel) getLabel();
        ComposedLabel lifelineLabel = (ComposedLabel) fullLabel.getMain();

        Lifeline lifeline = (Lifeline) getEObject();

        // Update the Stereotype Label
        fullLabel.setPrefix(UMLLabel.getStereotypesNotation(lifeline));

        // Update the name of the Lifeline
        lifelineLabel.setPrefix(lifeline.getName());

        Property prop = (Property) lifeline.getRepresents();
        String classifierName = "";
        if (prop != null && prop.getType() != null)
        {
            classifierName = prop.getType().getName();
        }

        // Update the Classifier type of the Lifeline
        lifelineLabel.setSuffix(classifierName);
    }

    /**
     * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
     */
    public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection)
    {
        GraphEdge edge = (GraphEdge) connection.getModel();
        EList<GraphConnector> connectors = edge.getAnchor();

        if (connectors != null && !connectors.isEmpty())
        {
            GraphConnector connector = connectors.get(0);
            return new LifelineAnchor(getFigure(), connector.getPosition());
        }
        return super.getSourceConnectionAnchor(connection);
    }

    /**
     * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
     */
    public ConnectionAnchor getSourceConnectionAnchor(Request request)
    {
        LifelineFigure fig = (LifelineFigure) getFigure();

        Point pos = null;
        Point oppositePoint = null;

        if (request instanceof CreateConnectionRequest)
        {
            CreateConnectionRequest createRequest = (CreateConnectionRequest) request;

            pos = createRequest.getLocation().getCopy();

            BaseEditPart sourceEdit = (BaseEditPart) createRequest.getSourceEditPart();
            sourceEdit.getFigure().translateToRelative(pos);
        }
        else if (request instanceof ReconnectRequest)
        {
            ReconnectRequest reconnectRequest = (ReconnectRequest) request;

            oppositePoint = SequenceUtils.getTargetAbsolutePosition((MessageEditPart) reconnectRequest.getConnectionEditPart());

            pos = reconnectRequest.getLocation().getCopy();

            BaseEditPart sourceEdit = (BaseEditPart) reconnectRequest.getTarget();
            sourceEdit.getFigure().translateToRelative(pos);
        }
        else if (request instanceof GraphEdgeMoveRequest)
        {
            GraphEdgeMoveRequest moveRequest = (GraphEdgeMoveRequest) request;
            pos = SequenceUtils.getSourceAbsolutePosition((MessageEditPart) moveRequest.getGraphEdgeEditPart());
            pos.y += moveRequest.getDelta();
        }

        return new LifelineAnchor(fig, pos, oppositePoint);
    }

    /**
     * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
     */
    public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection)
    {
        GraphEdge edge = (GraphEdge) connection.getModel();
        EObject eobj = Utils.getElement(edge);
        if (eobj instanceof Message)
        {
            EList<GraphConnector> connectors = edge.getAnchor();
            if (connectors != null && connectors.size() > 1)
            {
                GraphConnector connector = connectors.get(1);
                return new LifelineAnchor(getFigure(), connector.getPosition());
            }
        }
        else if (edge.getSemanticModel() instanceof SimpleSemanticModelElement)
        {
            if (((SimpleSemanticModelElement) edge.getSemanticModel()).getTypeInfo().equals(SequenceSimpleObjectConstants.SIMPLE_OBJECT_COMMENTLINK))
            {
                return new NodeAnchor(((LifelineFigure) getFigure()).getHeaderFigure());
            }
        }

        return super.getTargetConnectionAnchor(connection);
    }

    /**
     * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
     */
    public ConnectionAnchor getTargetConnectionAnchor(Request request)
    {
        LifelineFigure fig = (LifelineFigure) getFigure();

        Point pos = null;
        Point oppositePoint = null;

        if (request instanceof CreateConnectionRequest)
        {
            CreateConnectionRequest createRequest = (CreateConnectionRequest) request;

            Command command = createRequest.getStartCommand();

            if (command instanceof MessageEdgeCreationCommand)
            {
                CreateTypedEdgeCommand typedEdgeCommand = (CreateTypedEdgeCommand) command;
                oppositePoint = typedEdgeCommand.getSource().getPosition();
                pos = createRequest.getLocation().getCopy();

                return new LifelineAnchor(fig, pos, oppositePoint);
            }
            else if (command instanceof CommentLinkEdgeCreationCommand)
            {

                return new NodeAnchor(((LifelineFigure) getFigure()).getHeaderFigure());
            }
            else
            {
                return super.getTargetConnectionAnchor(createRequest);
            }

        }
        else if (request instanceof ReconnectRequest)
        {
            ReconnectRequest reconnectRequest = (ReconnectRequest) request;
            ConnectionEditPart edit = reconnectRequest.getConnectionEditPart();

            if (edit instanceof MessageEditPart)
            {
                Message message = (Message) Utils.getElement((GraphElement) edit.getModel());
                if (SequenceUtils.isCall(message) || SequenceUtils.isReplyMessage(message))
                {
                    return null;
                }

                oppositePoint = SequenceUtils.getSourceAbsolutePosition((MessageEditPart) edit);
                pos = reconnectRequest.getLocation().getCopy();

                BaseEditPart targetEdit = (BaseEditPart) reconnectRequest.getTarget();
                targetEdit.getFigure().translateToRelative(pos);
            }
            else
            {
                return super.getTargetConnectionAnchor(request);
            }
        }
        else if (request instanceof GraphEdgeMoveRequest)
        {
            GraphEdgeMoveRequest moveRequest = (GraphEdgeMoveRequest) request;
            if (moveRequest.getGraphEdgeEditPart() instanceof MessageEditPart)
            {
                pos = SequenceUtils.getTargetAbsolutePosition((MessageEditPart) moveRequest.getGraphEdgeEditPart());
                pos.y += moveRequest.getDelta();
            }
            else
            {
                return super.getTargetConnectionAnchor(moveRequest);
            }
        }

        return new LifelineAnchor(fig, pos, oppositePoint);
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {

        Object notifier = msg.getNotifier();
        Object newObject = msg.getNewValue();
        Object oldObject = msg.getOldValue();

        if (notifier instanceof Lifeline)
        {
            if (notifier == getEObject())
            {
                switch (msg.getFeatureID(Lifeline.class))
                {
                    case UMLPackage.LIFELINE__REPRESENTS:
                        updateModelListening(oldObject, newObject);
                        break;
                    case UMLPackage.LIFELINE__COVERED_BY:
                        updateDestructionEventRepresentation();
                        break;
                    default:
                        break;
                }
                super.handleModelChanged(msg);
            }
            else
            {
                refreshHeaderLabel();
            }
        }
        else if (notifier instanceof ConnectableElement)
        {
            switch (msg.getFeatureID(ConnectableElement.class))
            {
                case UMLPackage.CONNECTABLE_ELEMENT__TYPE:
                    updateModelListening(oldObject, newObject);
                    break;
                default:
                    break;
            }
            refreshHeaderLabel();
        }
    }

    /**
     * Check if the Lifeline is associated with a DestructionEvent and update the LifelineFigure in consequence
     */
    private void updateDestructionEventRepresentation()
    {
        boolean destructionEventFound = false;
        Iterator itFragments = ((Lifeline) getEObject()).getCoveredBys().iterator();
        while (itFragments.hasNext() && !destructionEventFound)
        {
            InteractionFragment fragment = (InteractionFragment) itFragments.next();
            if (fragment instanceof MessageOccurrenceSpecification && ((MessageOccurrenceSpecification) fragment).getEvent() instanceof DestructionEvent)
            {
                destructionEventFound = true;
            }
        }

        ((LifelineFigure) getFigure()).setHasDestructionEvent(destructionEventFound);
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshVisuals()
     */
    protected void refreshVisuals()
    {
        super.refreshVisuals();
        updateDestructionEventRepresentation();
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
     */
    public void activate()
    {
        super.activate();
        EObject model = getEObject();
        if (model instanceof Lifeline)
        {
            listenRepresents((Lifeline) getEObject());
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
     */
    public void deactivate()
    {
        EObject model = getEObject();
        if (model instanceof Operation)
        {
            unlistenRepresents((Lifeline) getEObject());
        }
        super.deactivate();
    }

    /**
     * Listen to the represents property of the Lifeline
     * 
     * @param lifeline the model object
     */
    private void listenRepresents(Lifeline lifeline)
    {
        // Only listen to object that are not yet listened
        if (lifeline.getRepresents() != null && !lifeline.getRepresents().eAdapters().contains(getModelListener()))
        {
            lifeline.getRepresents().eAdapters().add(getModelListener());
        }
    }

    /**
     * Stop listening to the represents property of the Lifeline
     * 
     * @param lifeline the model object
     */
    private void unlistenRepresents(Lifeline lifeline)
    {
        if (lifeline.getRepresents() != null)
        {
            lifeline.getRepresents().eAdapters().remove(getModelListener());
        }
    }

}