/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.edit;

import java.util.Iterator;

import org.eclipse.draw2d.ConnectionRouter;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.PolylineDecoration;
import org.eclipse.draw2d.RotatableDecoration;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.Request;
import org.eclipse.gmf.runtime.draw2d.ui.figures.PolylineConnectionEx;
import org.eclipse.gmf.runtime.draw2d.ui.internal.figures.ConnectionLayerEx;
import org.eclipse.swt.SWT;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageKind;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.ValueSpecification;
import org.topcased.draw2d.figures.Label;
import org.topcased.draw2d.figures.OvalDecoration;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.policies.EdgeObjectOffsetEditPolicy;
import org.topcased.modeler.figures.IEdgeObjectFigure;
import org.topcased.modeler.uml.sequencediagram.SequenceEdgeObjectConstants;
import org.topcased.modeler.uml.sequencediagram.SequenceUtils;
import org.topcased.modeler.uml.sequencediagram.figures.MessageFigure;
import org.topcased.modeler.uml.sequencediagram.figures.decorations.MessageLostDecoration;
import org.topcased.modeler.uml.sequencediagram.policies.MoveGraphEdgeEditPolicy;
import org.topcased.modeler.uml.sequencediagram.policies.MoveMessageEditPolicy;
import org.topcased.modeler.uml.sequencediagram.policies.SequenceDeleteModelObjectEditPolicy;
import org.topcased.modeler.uml.sequencediagram.routers.CallRouter;
import org.topcased.modeler.uml.sequencediagram.routers.SignalRouter;
import org.topcased.modeler.uml.sequencediagram.trackers.MoveGraphEdgeEditPartTracker;

/**
 * Message controller <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class MessageEditPart extends EMFGraphEdgeEditPart
{

    private static final int ARROW_WIDTH = 10;

    private static final int ARROW_HEIGHT = 5;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param model the graph object
     * @generated
     */
    public MessageEditPart(GraphEdge model)
    {
        super(model);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     * @generated NOT
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();
        installEditPolicy(ModelerEditPolicyConstants.EDGE_OBJECTS_OFFSET_EDITPOLICY, new EdgeObjectOffsetEditPolicy());

        // custom edit policy
        installEditPolicy(MoveGraphEdgeEditPolicy.GRAPHEDGE_MOVE_ROLE, new MoveMessageEditPolicy());

        // REV Check whether the removeEditPolicy() method is necessary or not
        // removing default edit policy for deleting model objects for messages
        // and behaviorExecutionSpecification.
        removeEditPolicy(ModelerEditPolicyConstants.DELETE_MODEL_OBJECT_EDITPOLICY);
        // and adding our custom one.
        installEditPolicy(ModelerEditPolicyConstants.DELETE_MODEL_OBJECT_EDITPOLICY,
                new SequenceDeleteModelObjectEditPolicy());
    }

    /**
     * 
     * @see org.eclipse.gef.EditPart#getDragTracker(org.eclipse.gef.Request)
     */
    public DragTracker getDragTracker(Request req)
    {
        return new MoveGraphEdgeEditPartTracker(this);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @return the Figure
     * @generated
     */
    protected IFigure createFigureGen()
    {
        MessageFigure connection = new MessageFigure();
        createSourceDecoration(connection);
        createTargetDecoration(connection);

        return connection;
    }

    /**
     * Customize for the return message
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     */
    protected IFigure createFigure()
    {
        PolylineConnectionEx connection = (PolylineConnectionEx) createFigureGen();
        if (SequenceUtils.isReplyMessage((Message) getEObject())
                || SequenceUtils.isCreateMessage((Message) getEObject()))
        {
            connection.setLineStyle(SWT.LINE_CUSTOM);
            connection.setLineDash(new int[] {10, 5});
        }
        return connection;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param connection the PolylineConnection
     * @generated NOT
     */
    private void createSourceDecoration(PolylineConnection connection)
    {
        updateSourceDecoration(connection);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param connection the PolylineConnection
     * @generated NOT
     */
    private void createTargetDecoration(PolylineConnection connection)
    {
        updateTargetDecoration(connection);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#getEdgeObjectFigure(org.topcased.modeler.di.model.EdgeObject)
     * @generated
     */
    public IEdgeObjectFigure getEdgeObjectFigure(EdgeObject edgeObject)
    {
        if (SequenceEdgeObjectConstants.NAME_EDGE_OBJECT_ID.equals(edgeObject.getId()))
        {
            return ((MessageFigure) getFigure()).getNameEdgeObjectFigure();
        }
        return null;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#getDefaultRouter(org.eclipse.gmf.runtime.draw2d.ui.internal.figures.ConnectionLayerEx)
     * @generated NOT
     */
    protected ConnectionRouter getDefaultRouter(ConnectionLayerEx cLayer)
    {
        if (SequenceUtils.isSignal((Message) getEObject()) || SequenceUtils.isCreateMessage((Message) getEObject())
                || SequenceUtils.isDeleteMessage((Message) getEObject()))
        {
            return new SignalRouter();
        }
        else
        {
            return new CallRouter();
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.GraphEdgeEditPart#refreshEdgeObjects()
     * @generated
     */
    protected void refreshEdgeObjects()
    {
        super.refreshEdgeObjects();
        updateNameLabel();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc --> Update the name Label
     * 
     * @generated NOT
     */
    private void updateNameLabel()
    {
        ((Label) ((MessageFigure) getFigure()).getNameEdgeObjectFigure()).setText(SequenceUtils.getMessageNotation((Message) getEObject()));
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphEdgeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {
        Object notifier = msg.getNotifier();
        Object newObject = msg.getNewValue();
        Object oldObject = msg.getOldValue();

        if (notifier instanceof Message)
        {
            if (notifier == getEObject())
            {
                switch (msg.getFeatureID(Message.class))
                {
                    case UMLPackage.MESSAGE__MESSAGE_KIND:
                    case UMLPackage.MESSAGE__SEND_EVENT:
                    case UMLPackage.MESSAGE__RECEIVE_EVENT:
                        updateSourceDecoration((PolylineConnection) getFigure());
                        updateTargetDecoration((PolylineConnection) getFigure());
                        break;
                    case UMLPackage.MESSAGE__NAME:
                        refreshEdgeObjects();
                        break;
                    case UMLPackage.MESSAGE__SIGNATURE:
                        updateModelListening(oldObject, newObject);
                        refreshEdgeObjects();
                        break;
                    case UMLPackage.MESSAGE__ARGUMENT:
                        listenArguments((Message) getEObject());
                        break;
                    case UMLPackage.MESSAGE__MESSAGE_SORT:
                        updateSourceDecoration((PolylineConnection) getFigure());
                        updateTargetDecoration((PolylineConnection) getFigure());
                        break;
                    default:
                        break;
                }
            }
        }
        else if (notifier instanceof ValueSpecification)
        {
            Message message = (Message) getEObject();
            if (message.getArguments().contains(notifier))
            {
                refreshEdgeObjects();
            }
        }

        super.handleModelChanged(msg);
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
     */
    public void activate()
    {
        super.activate();
        EObject model = getEObject();
        if (model instanceof Message)
        {
            listenArguments((Message) getEObject());
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
     */
    public void deactivate()
    {
        EObject model = getEObject();
        if (model instanceof Message)
        {
            unlistenArguments((Message) getEObject());
        }
        super.deactivate();
    }

    /**
     * Listen to all the arguments of the message
     * 
     * @param message the model object
     */
    private void listenArguments(Message message)
    {
        Iterator it = message.getArguments().iterator();
        while (it.hasNext())
        {
            ValueSpecification arg = (ValueSpecification) it.next();

            // Only listen to object that are not yet listened
            if (!arg.eAdapters().contains(getModelListener()))
            {
                arg.eAdapters().add(getModelListener());
            }
        }
    }

    /**
     * Stop listening to all the arguments of the message
     * 
     * @param message the model object
     */
    private void unlistenArguments(Message message)
    {
        Iterator it = message.getArguments().iterator();
        while (it.hasNext())
        {
            ValueSpecification arg = (ValueSpecification) it.next();

            arg.eAdapters().remove(getModelListener());
        }
    }

    /**
     * Update the source decoration
     * 
     * @param connection the connection figure
     */
    private void updateSourceDecoration(PolylineConnection connection)
    {
        Message message = (Message) getEObject();

        if (MessageKind.FOUND_LITERAL.getName().equals(message.getMessageKind().getName()))
        {
            connection.setSourceDecoration(new OvalDecoration());
        }
    }

    /**
     * Update the target decoration
     * 
     * @param connection the connection figure
     */
    private void updateTargetDecoration(PolylineConnection connection)
    {
        Message message = (Message) getEObject();

        RotatableDecoration decoration;
        if (SequenceUtils.isSynchCallMessage(message) || SequenceUtils.isReplyMessage(message))
        {
            if (MessageKind.LOST_LITERAL.getName().equals(message.getMessageKind().getName()))
            {
                decoration = new MessageLostDecoration(true);
                ((MessageLostDecoration) decoration).setScale(ARROW_WIDTH / 2, ARROW_HEIGHT);
            }
            else
            {
                decoration = new PolygonDecoration();
                ((PolygonDecoration) decoration).setScale(ARROW_WIDTH, ARROW_HEIGHT);
            }
        }
        else
        {
            if (MessageKind.LOST_LITERAL.getName().equals(message.getMessageKind().getName()))
            {
                decoration = new MessageLostDecoration(false);
                ((MessageLostDecoration) decoration).setScale(ARROW_WIDTH / 2, ARROW_HEIGHT);
            }
            else
            {
                decoration = new PolylineDecoration();
                ((PolylineDecoration) decoration).setScale(ARROW_WIDTH, ARROW_HEIGHT);
            }
        }

        connection.setTargetDecoration(decoration);
    }

}