/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.edit;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.uml2.uml.Message;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.BaseEditPart;
import org.topcased.modeler.edit.DiagramEditPart;
import org.topcased.modeler.listeners.DiagramElementListener;
import org.topcased.modeler.uml.sequencediagram.SequenceEditPolicyConstants;
import org.topcased.modeler.uml.sequencediagram.SequenceUtils;
import org.topcased.modeler.uml.sequencediagram.anchors.SequenceDiagramAnchor;
import org.topcased.modeler.uml.sequencediagram.commands.MessageEdgeCreationCommand;
import org.topcased.modeler.uml.sequencediagram.figures.SequenceDiagramFigure;
import org.topcased.modeler.uml.sequencediagram.policies.MessageEdgeCreationEditPolicy;
import org.topcased.modeler.uml.sequencediagram.policies.SequenceDiagramLayoutEditPolicy;
import org.topcased.modeler.uml.sequencediagram.requests.GraphEdgeMoveRequest;
import org.topcased.modeler.utils.Utils;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */
public class SequenceDiagramEditPart extends DiagramEditPart implements NodeEditPart
{
    /**
     * Override the default Listener to update the connections
     */
    protected class SequenceDiagramEditPartListener extends DiagramEditPartListener
    {
        /**
         * @see org.topcased.modeler.edit.DiagramEditPart.DiagramEditPartListener#handleConnectorChanged()
         */
        protected void handleConnectorChanged()
        {
            super.handleConnectorChanged();

            refreshSourceConnections();
            refreshTargetConnections();
        }
    }

    private DiagramElementListener diagramListener = new SequenceDiagramEditPartListener();

    /**
     * The Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param model
     * @generated
     */
    public SequenceDiagramEditPart(Diagram model)
    {
        super(model);
    }

    /**
     * @see org.topcased.modeler.edit.DiagramEditPart#getDiagramElementListener()
     */
    protected DiagramElementListener getDiagramElementListener()
    {
        return diagramListener;
    }

    /**
     * The sequence diagram can recieve messages
     * 
     * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(SequenceEditPolicyConstants.MESSAGE_EDITPOLICY, new MessageEdgeCreationEditPolicy());

    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.DiagramEditPart#getLayoutEditPolicy()
     * @generated
     */
    protected EditPolicy getLayoutEditPolicy()
    {
        return new SequenceDiagramLayoutEditPolicy();
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.DiagramEditPart#createBodyFigure()
     * @generated
     */
    protected IFigure createBodyFigure()
    {
        return new SequenceDiagramFigure();
    }

    /**
     * The label is the name of the diagram
     * 
     * @see org.topcased.modeler.edit.DiagramEditPart#getHeaderDiagram()
     */
    protected String getHeaderDiagram()
    {
        return getDiagram().getName();
    }

    /**
     * 
     * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
     */
    public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection)
    {
        GraphEdge edge = (GraphEdge) connection.getModel();
        EList<GraphConnector> connectors = edge.getAnchor();
        GraphConnector connector = connectors.get(0);
        return new SequenceDiagramAnchor(getFigure(), connector.getPosition());
    }

    /**
     * 
     * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
     */
    public ConnectionAnchor getSourceConnectionAnchor(Request request)
    {
        SequenceDiagramFigure fig = (SequenceDiagramFigure) getFigure().getChildren().get(0);
        Point oppositePoint = null;
        Point pos = null;

        if (request instanceof CreateConnectionRequest)
        {
            CreateConnectionRequest createRequest = (CreateConnectionRequest) request;
            pos = createRequest.getLocation();

            BaseEditPart sourceEdit = (BaseEditPart) createRequest.getSourceEditPart();
            sourceEdit.getFigure().translateToRelative(pos);
        }
        else if (request instanceof ReconnectRequest)
        {
            ReconnectRequest reconnectRequest = (ReconnectRequest) request;
            oppositePoint = SequenceUtils.getTargetAbsolutePosition((MessageEditPart) ((ReconnectRequest) request).getConnectionEditPart());
            pos = reconnectRequest.getLocation();

            BaseEditPart sourceEdit = (BaseEditPart) reconnectRequest.getTarget();
            sourceEdit.getFigure().translateToRelative(pos);
        }
        else if (request instanceof GraphEdgeMoveRequest)
        {
            GraphEdgeMoveRequest moveRequest = (GraphEdgeMoveRequest) request;
            pos = SequenceUtils.getSourceAbsolutePosition((MessageEditPart) moveRequest.getGraphEdgeEditPart());
            pos.y += moveRequest.getDelta();
        }

        return new SequenceDiagramAnchor(fig, pos, oppositePoint);
    }

    /**
     * 
     * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
     */
    public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection)
    {
        GraphEdge edge = (GraphEdge) connection.getModel();
        EObject eobj = Utils.getElement(edge);
        if (eobj instanceof Message)
        {
            EList<GraphConnector> connectors = edge.getAnchor();
            GraphConnector connector = connectors.get(1);
            return new SequenceDiagramAnchor(getFigure(), connector.getPosition());
        }
        return null;
    }

    /**
     * 
     * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
     */
    public ConnectionAnchor getTargetConnectionAnchor(Request request)
    {
        SequenceDiagramFigure fig = (SequenceDiagramFigure) getFigure().getChildren().get(0);

        Point oppositePoint = null;
        Point pos = null;

        if (request instanceof CreateConnectionRequest)
        {
            CreateConnectionRequest createRequest = (CreateConnectionRequest) request;

            Command command = createRequest.getStartCommand();

            if (!(command instanceof MessageEdgeCreationCommand))
            {
                return null;
            }

            MessageEdgeCreationCommand messageCommand = (MessageEdgeCreationCommand) command;

            oppositePoint = messageCommand.getSourcePos();
            pos = createRequest.getLocation().getCopy();

            BaseEditPart targetEdit = (BaseEditPart) createRequest.getTargetEditPart();
            targetEdit.getFigure().translateToRelative(pos);
        }
        else if (request instanceof ReconnectRequest)
        {
            ReconnectRequest reconnectRequest = (ReconnectRequest) request;

            ConnectionEditPart edit = reconnectRequest.getConnectionEditPart();
            if (!(edit instanceof MessageEditPart))
            {
                return null;
            }

            Message message = (Message) Utils.getElement((GraphElement) edit.getModel());
            if (SequenceUtils.isCall(message) || SequenceUtils.isReplyMessage(message))
            {
                return null;
            }

            oppositePoint = SequenceUtils.getSourceAbsolutePosition((MessageEditPart) edit);
            pos = reconnectRequest.getLocation().getCopy();

            BaseEditPart targetEdit = (BaseEditPart) reconnectRequest.getTarget();
            targetEdit.getFigure().translateToRelative(pos);
        }
        else if (request instanceof GraphEdgeMoveRequest)
        {
            GraphEdgeMoveRequest moveRequest = (GraphEdgeMoveRequest) request;

            pos = SequenceUtils.getTargetAbsolutePosition((MessageEditPart) moveRequest.getGraphEdgeEditPart());
            pos.y += moveRequest.getDelta();
        }

        return new SequenceDiagramAnchor(fig, pos, oppositePoint);
    }

    /**
     * Get the graphNode model object (DI) associated with the EditPart
     * 
     * @return GraphNode
     */
    protected GraphNode getGraphNode()
    {
        return (GraphNode) getModel();
    }

    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
     */
    protected List getModelSourceConnections()
    {
        List<GraphEdge> srcConnections = new ArrayList<GraphEdge>();
        for (GraphConnector connector : getGraphNode().getAnchorage())
        {
            for (GraphEdge edge : connector.getGraphEdge())
            {
                if (connector == edge.getAnchor().get(0))
                {
                    srcConnections.add(edge);
                }
            }
        }
        return srcConnections;
    }

    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
     */
    protected List getModelTargetConnections()
    {
        List<GraphEdge> tgtConnections = new ArrayList<GraphEdge>();
        for (GraphConnector connector : getGraphNode().getAnchorage())
        {
            for (GraphEdge edge : connector.getGraphEdge())
            {
                if (edge.getAnchor().size() == 2 && connector == edge.getAnchor().get(1))
                {
                    tgtConnections.add(edge);
                }
            }
        }
        return tgtConnections;
    }

}