/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.sequencediagram.editingsupport;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.widgets.Composite;

/**
 * Editing support for editing text, you should override
 * {@link #getValue(Object)} and {@link #setValue(Object, Object)}.
 * 
 * The methods were made public so that we can access it from the delegating
 * editing support.
 * 
 * @author jrme
 */
public abstract class TextEditingSupport extends EditingSupport
{

    private TextCellEditor cellEditor;

    public TextEditingSupport(TableViewer viewer)
    {
        super(viewer);
        cellEditor = new TextCellEditor(viewer.getTable());

    }

    public TextEditingSupport(ColumnViewer viewer)
    {
        super(viewer);
        cellEditor = new TextCellEditor((Composite) viewer.getControl());
    }

    @Override
    public boolean canEdit(Object element)
    {
        return true;
    }

    @Override
    public CellEditor getCellEditor(Object element)
    {
        return cellEditor;
    }

    @Override
    public abstract Object getValue(Object element);

    @Override
    public abstract void setValue(Object element, Object value);

}
