/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gmf.runtime.draw2d.ui.figures.OneLineBorder;
import org.eclipse.swt.SWT;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.Label;
import org.topcased.draw2d.layout.GridData;
import org.topcased.draw2d.layout.GridLayout;
import org.topcased.modeler.figures.ModelerFigure;
import org.topcased.modeler.uml.sequencediagram.layouts.LifelineLayout;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */
public class LifelineFigure extends ModelerFigure
{
    /** the default height of a DestructionEvent */
    private static final int DEFAULT_HEIGHT = 20;

    /** the default width of a DestructionEvent */
    private static final int DEFAULT_WIDTH = 20;

    private static final int LINE_ON = 10;

    private static final int LINE_OFF = 2;

    private IFigure header;

    private boolean hasDestructionEvent;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public LifelineFigure()
    {
        super();
    }

    /**
     * Complete cutom figure
     * 
     * @see org.topcased.modeler.figures.ModelerFigure#drawFigure()
     */
    protected void drawFigure()
    {
        GridLayout gridLayout = new GridLayout();
        gridLayout.verticalSpacing = 0;
        gridLayout.horizontalSpacing = 0;
        gridLayout.marginWidth = 0;
        gridLayout.marginHeight = 0;
        setLayoutManager(gridLayout);
        setBorder(null);
        setOpaque(false);

        setLabel(new ComposedLabel(new Label(), new ComposedLabel(new EditableLabel(), new Label(" : "), new Label(), true), null, false));
        // to underline the label
        getLabel().setBorder(new OneLineBorder(1, PositionConstants.BOTTOM));

        header = new Figure();
        header.setLayoutManager(new GridLayout());
        header.add(getLabel(), new GridData(GridData.CENTER, GridData.BEGINNING, true, false));
        header.setBorder(new LineBorder());
        header.setOpaque(true);
        add(header, new GridData(GridData.FILL, GridData.BEGINNING, true, false));

        setContentPane(new Figure());
        getContentPane().setLayoutManager(new LifelineLayout());
        getContentPane().setOpaque(false);

        add(getContentPane(), new GridData(GridData.FILL_BOTH));
    }

    /**
     * Add the dashed line of the lifeline and the cross for the destruction event if necessary
     * 
     * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
     */
    protected void paintFigure(Graphics graphics)
    {
        super.paintFigure(graphics);
        graphics.pushState();

        int height = bounds.height;

        if (hasDestructionEvent)
        {
            int x = bounds.x + bounds.width / 2 - DEFAULT_WIDTH / 2;
            int y = bounds.y + bounds.height - DEFAULT_HEIGHT;
            graphics.setLineWidth(1);
            graphics.drawLine(x, y, x + DEFAULT_WIDTH, y + DEFAULT_HEIGHT);
            graphics.drawLine(x + DEFAULT_WIDTH, y, x, y + DEFAULT_HEIGHT);

            height -= DEFAULT_HEIGHT / 2;
        }

        graphics.setLineStyle(SWT.LINE_DASH);
        graphics.setLineDash(new int[] {LINE_ON, LINE_OFF});
        int mid = bounds.x + bounds.width / 2;
        graphics.drawLine(mid, bounds.y + header.getBounds().height, mid, bounds.y + height);
        graphics.popState();
    }

    // /**
    // * Get if there is a Stop
    // *
    // * @return true if there is a Stop
    // */
    // public boolean existStopFigure()
    // {
    // boolean exist = false;
    // Iterator itChildren = getContentPane().getChildren().iterator();
    // while (!exist && itChildren.hasNext())
    // {
    // Object obj = itChildren.next();
    // exist = (obj instanceof DestructionEventFigure);
    // }
    // return exist;
    // }

    /**
     * This method is used to update the DestructionEvent representation inside the Lifeline figure
     * 
     * @param hasOne true if the cross should be displayed
     */
    public void setHasDestructionEvent(boolean hasOne)
    {
        if (hasOne != hasDestructionEvent)
        {
            this.hasDestructionEvent = hasOne;
            repaint();
        }
    }

    /**
     * Return the bounds of the header
     * 
     * @return the bounds of the header
     */
    public Rectangle getHeaderBounds()
    {
        return header.getBounds();
    }

    /**
     * Get the Header figure. Used to link a comment to a Lifeline.
     * 
     * @return IFigure the header figure
     */
    public IFigure getHeaderFigure()
    {
        return header;
    }

}