// ------------------------------------------------
// $Id: ValueSpecificationValueLabelProvider.java,v 1.1 2007-05-24 15:31:56 jako Exp $
// (c) Anyware Technologies 2007 www.anyware-tech.com
// ------------------------------------------------
package org.topcased.modeler.uml.sequencediagram.labelproviders;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.uml2.uml.Duration;
import org.eclipse.uml2.uml.DurationInterval;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.LiteralBoolean;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralNull;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.StringExpression;
import org.eclipse.uml2.uml.TimeExpression;
import org.eclipse.uml2.uml.TimeInterval;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.uml2.uml.util.UMLSwitch;

/**
 * Label provider used for displaying a ValueSpecification value.
 * 
 * @author jrme
 */
public class ValueSpecificationValueLabelProvider extends ColumnLabelProvider
{
    /**
     * Internal switch used in getText for the different types of
     * ValueSpecification.
     * 
     * @author jrme
     */
    protected class LabelSwitch extends UMLSwitch<String>
    {

        @Override
        public String caseDuration(Duration object)
        {
            return (object.getExpr().getName());
        }

        @Override
        public String caseStringExpression(StringExpression object)
        {
            return object.getSubExpressions().toString();
        }

        @Override
        public String caseInstanceValue(InstanceValue object)
        {
            if (object.getInstance() != null)
            {
                return object.getInstance().getName();
            }
            return "";
        }

        @Override
        public String caseDurationInterval(DurationInterval object)
        {
            return "[" + object.getMin().getName() + " " + object.getMax().getName() + "]";
        }

        @Override
        public String caseTimeInterval(TimeInterval object)
        {
            return "[" + object.getMin().getName() + " " + object.getMax().getName() + "]";
        }

        @Override
        public String caseLiteralBoolean(LiteralBoolean object)
        {
            return String.valueOf(object.isValue());
        }

        @Override
        public String caseLiteralInteger(LiteralInteger object)
        {
            return String.valueOf(object.getValue());
        }

        @Override
        public String caseLiteralNull(LiteralNull object)
        {
            return object.getName();
        }

        @Override
        public String caseLiteralString(LiteralString object)
        {
            return object.getValue();
        }

        @Override
        public String caseLiteralUnlimitedNatural(LiteralUnlimitedNatural object)
        {
            return String.valueOf(object.getValue());
        }

        @Override
        public String caseOpaqueExpression(OpaqueExpression object)
        {
            return object.getResult().getName();
        }

        @Override
        public String caseTimeExpression(TimeExpression object)
        {
            return object.getObservations().toString();
        }
    }

    @Override
    public String getText(Object element)
    {
        if (element instanceof ValueSpecification)
        {
            return new LabelSwitch().doSwitch((EObject) element);
        }
        return super.getText(element);
    }
}
