/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.layouts;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.topcased.modeler.uml.sequencediagram.figures.BehaviorExecutionSpecificationFigure;
import org.topcased.modeler.uml.sequencediagram.figures.DestructionEventFigure;

/**
 * The layout of a lifeline that handles the executions position
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class LifelineLayout extends XYLayout
{
    private static final int BORDER_SPACE = 1;

    /**
     * The Constructor
     */
    public LifelineLayout()
    {
        super();
    }

    /**
     * Layout the executions
     * 
     * @see org.eclipse.draw2d.LayoutManager#layout(org.eclipse.draw2d.IFigure)
     */
    public void layout(IFigure container)
    {
        Iterator children = container.getChildren().iterator();
        Rectangle clientarea = container.getClientArea();
        Point offset = clientarea.getLocation();
        Point negatedOffset = offset.getNegated();

        ArrayList execs = new ArrayList();

        // Iterate on all the children and place them
        while (children.hasNext())
        {
            Figure figure = (Figure) children.next();

            Rectangle bounds = (Rectangle) constraints.get(figure);

            if (bounds == null)
            {
                continue;
            }

            if (bounds.width == -1 || bounds.height == -1)
            {
                Dimension prefSize = figure.getPreferredSize(bounds.width, bounds.height);
                bounds = bounds.getCopy();
                if (bounds.width == -1)
                {
                    bounds.width = prefSize.width;
                }
                if (bounds.height == -1)
                {
                    bounds.height = prefSize.height;
                }
            }

            if (figure instanceof DestructionEventFigure)
            {
                bounds.x = (clientarea.width - bounds.width) / 2;
                bounds.y = clientarea.height - bounds.height - BORDER_SPACE;
            }
            else if (figure instanceof BehaviorExecutionSpecificationFigure)
            {
                bounds.x = (clientarea.width - bounds.width) / 2;

                // iterate on each already layouted execution to prevent overlap
                Iterator itExecs = execs.iterator();
                while (itExecs.hasNext())
                {
                    Rectangle r = new Rectangle(((BehaviorExecutionSpecificationFigure) itExecs.next()).getBounds());
                    r.translate(negatedOffset);
                    if (r.y < bounds.y + bounds.height && r.y + r.height > bounds.y)
                    {
                        // these two executions are on a same protion of life
                        // line make a little offset
                        int x = r.x + bounds.width * 2 / 3;
                        if (x > bounds.x)
                        {
                            bounds.x = x;
                        }
                    }
                }
                execs.add(figure);
            }

            bounds = bounds.getTranslated(offset);
            figure.setBounds(bounds);
        }
    }

}
