/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.policies;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.FeedbackHelper;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.uml2.uml.BehaviorExecutionSpecification;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageKind;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.BaseEditPart;
import org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy;
import org.topcased.modeler.uml.sequencediagram.SequenceUtils;
import org.topcased.modeler.uml.sequencediagram.commands.MessageEdgeCreationCommand;
import org.topcased.modeler.uml.sequencediagram.commands.creation.MessageAndExecutionCreationCommand;
import org.topcased.modeler.uml.sequencediagram.commands.reconnect.MessageEdgeReconnectCommand;
import org.topcased.modeler.uml.sequencediagram.edit.LifelineEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.MessageEditPart;
import org.topcased.modeler.uml.sequencediagram.routers.CallRouter;
import org.topcased.modeler.uml.sequencediagram.routers.SignalRouter;
import org.topcased.modeler.utils.SourceTargetData;
import org.topcased.modeler.utils.Utils;

/**
 * Message edge creation <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class MessageEdgeCreationEditPolicy extends AbstractEdgeCreationEditPolicy
{
    /**
     * <!-- begin-user-doc --> !! should not be called !! <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#createCommand(org.eclipse.gef.EditDomain,
     *      org.topcased.modeler.di.model.GraphEdge, org.topcased.modeler.di.model.GraphElement)
     * @generated
     */
    protected CreateTypedEdgeCommand createCommand(EditDomain domain, GraphEdge edge, GraphElement source)
    {
        return new MessageEdgeCreationCommand(domain, edge, source);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkEdge(org.topcased.modeler.di.model.GraphEdge)
     * @generated
     */
    protected boolean checkEdge(GraphEdge edge)
    {
        return Utils.getElement(edge) instanceof Message;
    }

    /**
     * <!-- begin-user-doc --> !! should not be called !! <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkSource(org.topcased.modeler.di.model.GraphElement)
     * @generated
     */
    protected boolean checkSource(GraphElement source)
    {
        EObject object = Utils.getElement(source);
        if (object instanceof org.eclipse.uml2.uml.Lifeline)
        {
            return true;
        }
        return false;
    }

    /**
     * Check if the sourceNode is valid for the given Message. We check the MessageSort value of the Message to
     * determine if the source is valid
     * 
     * @param source the sourceNode
     * @param edge the graphEdge that represent the Message
     * @return true if the sourceNode is valid
     */
    protected boolean checkSource(GraphNode source, GraphEdge edge)
    {
        EObject object = Utils.getElement(source);
        Message message = (Message) Utils.getElement(edge);

        // A Signal Message must always be created between two Lifelines (in the
        // case of a FoundMessage, the sourceNode is directly the diagram
        // associated with the Interaction)
        if (SequenceUtils.isSignal(message))
        {
            return object instanceof Lifeline || object instanceof Interaction;
        }

        // In the case of a CallMessage or a ReplyMessage, the sourceNode must
        // be an ExecutionSpecification (in the case of a FoundMessage, the
        // sourceNode is directly the diagram associated with the Interaction)
        if (SequenceUtils.isCall(message) || SequenceUtils.isReplyMessage(message))
        {
            return object instanceof BehaviorExecutionSpecification || object instanceof Interaction;
        }

        // In all other cases (CreateMessage and DeleteMessage) the sourceNode
        // must be a Lifeline
        return object instanceof Lifeline;
    }

    /**
     * <!-- begin-user-doc --> !! should not be called !! <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkTargetForSource(org.topcased.modeler.di.model.GraphElement,
     *      org.topcased.modeler.di.model.GraphElement)
     * @generated
     */
    protected boolean checkTargetForSource(GraphElement source, GraphElement target)
    {
        EObject sourceObject = Utils.getElement(source);
        EObject targetObject = Utils.getElement(target);

        if (sourceObject instanceof org.eclipse.uml2.uml.Lifeline && targetObject instanceof org.eclipse.uml2.uml.Lifeline)
        {
            if (!sourceObject.equals(targetObject))
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Check if the targetNode is valid for the given sourceNode and Message. We check the MessageSort value of the
     * Message to determine if the target is valid
     * 
     * @param source the sourceNode
     * @param target the targetNode
     * @param edge the graphEdge that represent the Message
     * @return true if the targetNode is valid
     */
    protected boolean checkTargetForSource(GraphNode source, GraphNode target, GraphEdge edge)
    {
        EObject sourceObject = Utils.getElement(source);
        EObject targetObject = Utils.getElement(target);
        Message message = (Message) Utils.getElement(edge);

        // A Signal Message must always be created between two Lifelines (in the
        // case of a LostMessage, the targetNode is directly the diagram
        // associated with the Interaction)
        if (SequenceUtils.isSignal(message))
        {
            return (sourceObject instanceof Lifeline && (targetObject instanceof Lifeline || targetObject instanceof Interaction))
                    || (sourceObject instanceof Interaction && targetObject instanceof Lifeline);
        }

        // A Call Message or a Reply Message must always be created between two
        // ExecutionSpecification (in the case of a LostMessage, the targetNode
        // is
        // directly the diagram associated with the Interaction)
        if (SequenceUtils.isCall(message) || SequenceUtils.isReplyMessage(message))
        {
            return (sourceObject instanceof BehaviorExecutionSpecification && (targetObject instanceof Lifeline || targetObject instanceof BehaviorExecutionSpecification || targetObject instanceof Interaction))
                    || (sourceObject instanceof Interaction && (targetObject instanceof Lifeline || targetObject instanceof BehaviorExecutionSpecification));
        }

        // In all other cases (CreateMessage and DeleteMessage) the targetNode
        // must be a Lifeline
        return sourceObject instanceof Lifeline && targetObject instanceof Lifeline;

    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkCommand(org.eclipse.gef.commands.Command)
     * @generated
     */
    protected boolean checkCommand(Command command)
    {
        return command instanceof MessageEdgeCreationCommand;
    }

    /**
     * <!-- begin-user-doc --> !! should not be called !! <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#getSourceTargetData(org.topcased.modeler.di.model.GraphElement,
     *      org.topcased.modeler.di.model.GraphElement)
     * @generated
     */
    protected SourceTargetData getSourceTargetData(GraphElement source, GraphElement target)
    {
        EObject sourceObject = Utils.getElement(source);
        EObject targetObject = Utils.getElement(target);

        if (sourceObject instanceof org.eclipse.uml2.uml.Lifeline && targetObject instanceof org.eclipse.uml2.uml.Lifeline)
        {
            return new SourceTargetData(false, false, SourceTargetData.DIAGRAM, "org.eclipse.uml2.uml.Interaction", "message", null, null, null, null, null, null);
        }
        return null;
    }

    /**
     * Retrieve the SourceTargetData corresponding to the type (depending on the MessageSort property) of Message that
     * is created
     * 
     * @param edge the edge associated with the Message
     * @return the SourceTargetData that contains all needed informations about the connection that is about to be
     *         created.
     */
    protected SourceTargetData getSourceTargetData(GraphEdge edge)
    {
        Message message = (Message) Utils.getElement(edge);

        if (SequenceUtils.isSignal(message))
        {
            return new SourceTargetData(false, false, SourceTargetData.NONE, null, null, null, null, null, "sendEvent", null, "receiveEvent");
        }
        else
        {
            return new SourceTargetData(false, false, SourceTargetData.NONE, null, null, null, null, null, "sendEvent", null, "receiveEvent");
        }
    }

    /**
     * Handle the various type (depending on the MessageSort property) of the Message
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
     */
    protected Command getConnectionCreateCommand(CreateConnectionRequest request)
    {
        Object newObject = request.getNewObject();
        if (newObject instanceof GraphEdge)
        {
            GraphEdge edge = (GraphEdge) newObject;
            GraphNode sourceNode = (GraphNode) getHost().getModel();
            if (edge != null && sourceNode != null && checkEdge(edge) && checkSource(sourceNode, edge))
            {
                MessageEdgeCreationCommand command = new MessageEdgeCreationCommand(getHost().getViewer().getEditDomain(), edge, sourceNode);

                BaseEditPart edit = (BaseEditPart) getHost();

                command.setSourceEditPart(edit);

                request.setStartCommand(command);

                Point pos = request.getLocation().getCopy();
                edit.getFigure().translateToRelative(pos);

                command.setSourcePos(pos);
                return command;
            }
        }
        return null;
    }

    /**
     * Handle the various type of message
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
     */
    protected Command getConnectionCompleteCommand(CreateConnectionRequest request)
    {
        Command command = null;

        if (checkCommand(request.getStartCommand()))
        {
            GraphNode targetNode = (GraphNode) getHost().getModel();
            GraphNode sourceNode = (GraphNode) request.getSourceEditPart().getModel();
            GraphEdge edge = (GraphEdge) request.getNewObject();
            if (targetNode != null && sourceNode != null && checkTargetForSource(sourceNode, targetNode, edge))
            {
                command = request.getStartCommand();
                if (command != null)
                {
                    MessageEdgeCreationCommand messageCommand = (MessageEdgeCreationCommand) command;
                    messageCommand.setTarget(targetNode);
                    messageCommand.setSourceTargetData(getSourceTargetData(edge));

                    BaseEditPart targetEdit = (BaseEditPart) request.getTargetEditPart();
                    BaseEditPart sourceEdit = (BaseEditPart) request.getSourceEditPart();

                    messageCommand.setTargetEditPart(targetEdit);

                    Point pos = request.getLocation().getCopy();
                    targetEdit.getFigure().translateToRelative(pos);
                    Point oppositePos = messageCommand.getSourcePos();

                    int diff = SequenceUtils.alignPosAnchor(pos, oppositePos);
                    messageCommand.setTargetPos(pos);

                    Message message = (Message) Utils.getElement(edge);

                    // It may be necessary to create the ExecutionSpecification
                    if ((SequenceUtils.isCall(message) || SequenceUtils.isReplyMessage(message)) && (targetEdit instanceof LifelineEditPart || targetEdit == sourceEdit))
                    {
                        Point loc = request.getLocation().getCopy();
                        loc.y += diff;

                        command = new MessageAndExecutionCreationCommand(getHost().getViewer(), loc, messageCommand);
                    }
                }
            }
        }

        return command;
    }

    /**
     * Check if the source can be reconnected
     * 
     * @param source the source of the edge
     * @param edge the edge to reconnect
     * @return true if the source can be reconnected
     */
    protected boolean checkReconnectSource(GraphNode source, GraphEdge edge)
    {
        boolean isValid = false;

        EObject object = Utils.getElement(source);
        Message message = (Message) Utils.getElement(edge);
        int kind = message.getMessageKind().getValue();

        // the sourceNode should be the same type as it was before the
        // reconnection has happened.
        if ((kind == MessageKind.FOUND && object instanceof Interaction)
                || ((SequenceUtils.isCall(message) || SequenceUtils.isReplyMessage(message)) && object instanceof BehaviorExecutionSpecification)
                || ((SequenceUtils.isSignal(message) || SequenceUtils.isCreateMessage(message) || SequenceUtils.isDeleteMessage(message)) && object instanceof Lifeline))
        {
            isValid = true;
        }

        return isValid;
    }

    /**
     * Check if the target can be reconnected
     * 
     * @param target the target of the edge
     * @param edge the edge to reconnect
     * @return true if the target can be reconnected
     */
    protected boolean checkReconnectTarget(GraphNode target, GraphEdge edge)
    {
        boolean isValid = false;

        EObject object = Utils.getElement(target);
        Message message = (Message) Utils.getElement(edge);
        int kind = message.getMessageKind().getValue();

        // the sourceNode should be the same type as it was before the
        // reconnection has happened.
        if ((kind == MessageKind.LOST && object instanceof Interaction)
                || ((SequenceUtils.isCall(message) || SequenceUtils.isReplyMessage(message)) && object instanceof BehaviorExecutionSpecification)
                || ((SequenceUtils.isSignal(message) || SequenceUtils.isCreateMessage(message) || SequenceUtils.isDeleteMessage(message)) && object instanceof Lifeline))
        {
            isValid = true;
        }

        return isValid;
    }

    /**
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
     */
    protected Command getReconnectSourceCommand(ReconnectRequest request)
    {
        ConnectionEditPart connection = request.getConnectionEditPart();
        if (connection instanceof MessageEditPart)
        {
            BaseEditPart target = (BaseEditPart) request.getTarget();
            if (checkReconnectSource((GraphNode) target.getModel(), (GraphEdge) connection.getModel()))
            {
                MessageEditPart edit = (MessageEditPart) connection;

                Point loc = request.getLocation().getCopy();
                target.getFigure().translateToRelative(loc);

                MessageEdgeReconnectCommand command = new MessageEdgeReconnectCommand(target, edit, true, loc);
                return command;
            }
        }

        return null;
    }

    /**
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
     */
    protected Command getReconnectTargetCommand(ReconnectRequest request)
    {
        ConnectionEditPart connection = request.getConnectionEditPart();
        if (connection instanceof MessageEditPart)
        {
            BaseEditPart target = (BaseEditPart) request.getTarget();
            if (checkReconnectTarget((GraphNode) target.getModel(), (GraphEdge) connection.getModel()))
            {
                MessageEditPart edit = (MessageEditPart) connection;

                Point loc = request.getLocation().getCopy();
                target.getFigure().translateToRelative(loc);

                MessageEdgeReconnectCommand command = new MessageEdgeReconnectCommand(target, edit, false, loc);
                return command;
            }
        }

        return null;
    }

    /**
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getFeedbackHelper(org.eclipse.gef.requests.CreateConnectionRequest)
     */
    protected FeedbackHelper getFeedbackHelper(CreateConnectionRequest request)
    {
        FeedbackHelper helper = super.getFeedbackHelper(request);
        Message message = (Message) Utils.getElement((GraphElement) request.getNewObject());

        if (SequenceUtils.isSignal(message) || SequenceUtils.isCreateMessage(message) || SequenceUtils.isDeleteMessage(message))
        {
            connectionFeedback.setConnectionRouter(new SignalRouter());
        }
        else
        {
            connectionFeedback.setConnectionRouter(new CallRouter());
        }
        return helper;
    }

}