/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.policies;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.FeedbackHelper;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.policies.EdgeEndpointEditPolicy;
import org.topcased.modeler.uml.UMLRequestConstants;
import org.topcased.modeler.uml.sequencediagram.commands.reconnect.GraphEdgeMoveCommand;
import org.topcased.modeler.uml.sequencediagram.requests.GraphEdgeMoveRequest;

/**
 * The edit policy that manage the move of graph edge
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class MoveGraphEdgeEditPolicy extends EdgeEndpointEditPolicy
{
    /** the edit policy ID */
    public static final String GRAPHEDGE_MOVE_ROLE = "GraphEdge Move Policy";

    private FeedbackHelper feedbackHelper;

    /**
     * Handle the move graphedge request
     * 
     * @see org.eclipse.gef.EditPolicy#getCommand(org.eclipse.gef.Request)
     */
    public Command getCommand(Request request)
    {
        if (UMLRequestConstants.REQ_MOVE_GRAPHEDGE.equals(request.getType()))
        {
            GraphEdgeMoveRequest connRequest = (GraphEdgeMoveRequest) request;

            return getCommand(connRequest.getGraphEdgeEditPart(), connRequest.getInitialLocation(), connRequest.getLocation());
        }

        return super.getCommand(request);
    }

    /**
     * Return the command
     * 
     * @param edgeEdit the edit part of the graph edge to move
     * @param initialLoc the initial position of the connection
     * @param loc the new position of the connection
     * @return the command
     */
    protected Command getCommand(GraphEdgeEditPart edgeEdit, Point initialLoc, Point loc)
    {
        return new GraphEdgeMoveCommand(edgeEdit, initialLoc, loc);
    }

    /**
     * Return the feedback helper
     * 
     * @param request the request to feedback
     * @return the feedback helper
     */
    protected FeedbackHelper getFeedbackHelper(GraphEdgeMoveRequest request)
    {
        if (feedbackHelper == null)
        {
            feedbackHelper = new FeedbackHelper();
            feedbackHelper.setConnection(getConnection());
        }
        return feedbackHelper;
    }

    /**
     * Show the connection feedback
     * 
     * @param request the request to feedback
     */
    protected void showConnectionMoveFeedback(GraphEdgeMoveRequest request)
    {
        NodeEditPart sourceNode = null;
        NodeEditPart targetNode = null;

        if (request.getSource() instanceof NodeEditPart)
        {
            sourceNode = (NodeEditPart) request.getSource();
        }

        if (request.getTarget() instanceof NodeEditPart)
        {
            targetNode = (NodeEditPart) request.getTarget();
        }

        ConnectionAnchor sourceAnchor = null;
        ConnectionAnchor targetAnchor = null;

        if (sourceNode != null)
        {
            sourceAnchor = sourceNode.getSourceConnectionAnchor(request);
        }

        if (targetNode != null)
        {
            targetAnchor = targetNode.getTargetConnectionAnchor(request);
        }

        FeedbackHelper helper = getFeedbackHelper(request);
        helper.setMovingStartAnchor(true);
        helper.update(sourceAnchor, null);
        helper.setMovingStartAnchor(false);
        helper.update(targetAnchor, null);
    }

    /**
     * Add feedback for the graph edge move
     * 
     * @see org.eclipse.gef.EditPolicy#showSourceFeedback(org.eclipse.gef.Request)
     */
    public void showSourceFeedback(Request request)
    {
        if (UMLRequestConstants.REQ_MOVE_GRAPHEDGE.equals(request.getType()))
        {
            showConnectionMoveFeedback((GraphEdgeMoveRequest) request);
        }
        else
        {
            super.showSourceFeedback(request);
        }
    }

}
