/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.policies;

import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.uml2.uml.CombinedFragment;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Lifeline;
import org.topcased.draw2d.layout.PortConstraint;
import org.topcased.modeler.commands.ChangeBoundsCommand;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy;
import org.topcased.modeler.uml.sequencediagram.commands.CombinedFragmentChangeBoundsCommand;
import org.topcased.modeler.uml.sequencediagram.commands.creation.CombinedFragmentCreationCommand;
import org.topcased.modeler.uml.sequencediagram.edit.CombinedFragmentEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.LifelineEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class SequenceDiagramLayoutEditPolicy extends ModelerLayoutEditPolicy
{
    private static final int LIFELINES_PADDING = 40;

    /**
     * Default contructor. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public SequenceDiagramLayoutEditPolicy()
    {
        super();
    }

    /**
     * Only lifeline and comment are allowed
     * 
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isValid(org.eclipse.emf.ecore.EObject,
     *      org.eclipse.emf.ecore.EObject)
     */
    protected boolean isValid(EObject child, EObject parent)
    {
        return child != null && parent != null && (child instanceof Lifeline || child instanceof Comment || child instanceof CombinedFragment);
    }

    /**
     * The lifelines are an y fixed position
     * 
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#getCreateCommand(org.eclipse.gef.EditDomain,
     *      org.topcased.modeler.di.model.GraphNode, org.topcased.modeler.di.model.GraphNode,
     *      org.eclipse.emf.ecore.EObject, org.eclipse.draw2d.geometry.Point, org.eclipse.draw2d.geometry.Dimension,
     *      int, java.util.List, boolean)
     */
    protected Command getCreateCommand(EditDomain domain, GraphNode newObject, GraphNode newParent, EObject newContainerParent, Point location, Dimension dimension, int attach, List featuresList,
            boolean needModelUpdate)
    {
        if (Utils.getElement(newObject) instanceof Lifeline)
        {
            location.y = LIFELINES_PADDING;
        }
        else if (Utils.getElement(newObject) instanceof CombinedFragment)
        {
            return new CombinedFragmentCreationCommand(domain, newObject, newParent, newContainerParent, location, dimension, attach, featuresList, needModelUpdate, getHost());
        }
        return new CreateGraphNodeCommand(domain, newObject, newParent, newContainerParent, location, dimension, attach, featuresList, needModelUpdate);
    }

    /**
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.requests.ChangeBoundsRequest,
     *      org.eclipse.gef.EditPart, java.lang.Object)
     */
    protected Command createChangeConstraintCommand(ChangeBoundsRequest request, EditPart child, Object constraint)
    {
        return createChangeConstraintCommand(child, constraint);
    }

    /**
     * The lifelines are an y fixed position Don't allow the resize if some message are present
     * 
     * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart,
     *      java.lang.Object)
     */
    protected Command createChangeConstraintCommand(EditPart child, Object constraint)
    {
        GraphNode node = (GraphNode) child.getModel();

        if (child instanceof CombinedFragmentEditPart)
        {
            return new CombinedFragmentChangeBoundsCommand(node, (Rectangle) constraint, child, getHost());
        }
        else if (child instanceof LifelineEditPart)
        {
            int maxy = 0;
            Iterator itAnchors = ((GraphNode) child.getModel()).getAnchorage().iterator();
            while (itAnchors.hasNext())
            {
                GraphConnector connector = (GraphConnector) itAnchors.next();
                int y = connector.getPosition().y;
                if (y > maxy)
                {
                    maxy = y;
                }
            }

            if (((Rectangle) constraint).height < maxy)
            {
                return null;
            }
        }

        if (constraint instanceof PortConstraint)
        {
            return new ChangeBoundsCommand(node, (PortConstraint) constraint);
        }
        else if (isValidConstraint((GraphicalEditPart) getHost(), (Rectangle) constraint))
        {
            if (Utils.getElement((GraphElement) child.getModel()) instanceof Lifeline)
            {
                ((Rectangle) constraint).y = LIFELINES_PADDING;
            }
            return new ChangeBoundsCommand(node, (Rectangle) constraint);
        }

        return null;
    }

}