/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.sequencediagram.properties.sections;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.ConnectableElement;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.uml.sequencediagram.SequencePlugin;
import org.topcased.modeler.uml.sequencediagram.commands.update.UpdateLifelineRepresentsCommand;
import org.topcased.modeler.uml.sequencediagram.dialogs.ClassifierCreateDialog;
import org.topcased.modeler.widgets.CSingleObjectChooser;
import org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection;

/**
 * A section for the represents property (and the linked Classifier) of a selected Lifeline Object.
 * 
 * Creation 13 juin 06
 * 
 * @author jlescot
 */
public class LifelineRepresentsPropertySection extends AbstractTabbedPropertySection
{
    /**
     * A boolean that store if refreshing is happening and no model modifications should be performed
     */
    private boolean isRefreshing = false;

    /**
     * The Label that identify the section
     */
    private CLabel representsLbl;

    /**
     * The combo box control for the section.
     */
    private CSingleObjectChooser cSingleObjectChooser;

    /**
     * The button used to create a new classifier to associate with the Lifeline
     */
    private Button newButton;

    // /**
    // * @see org.eclipse.ui.views.properties.tabbed.ISection#createControls(org.eclipse.swt.widgets.Composite,
    // * org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage)
    // */
    // public void createControls(Composite parent, TabbedPropertySheetPage aTabbedPropertySheetPage)
    // {
    // super.createControls(parent, aTabbedPropertySheetPage);
    // Composite composite = getWidgetFactory().createFlatFormComposite(parent);
    // FormData data;
    //
    // representsLbl = getWidgetFactory().createCLabel(composite, getLabelText(), SWT.NONE);
    //
    // cSingleObjectChooser = new CSingleObjectChooser(composite, getWidgetFactory(), SWT.NONE);
    // cSingleObjectChooser.setLabelProvider(new AdapterFactoryLabelProvider(new UMLItemProviderAdapterFactory()));
    //
    // newButton = getWidgetFactory().createButton(composite, "New", SWT.NONE);
    //
    // data = new FormData();
    // data.left = new FormAttachment(0, 0);
    // data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    // representsLbl.setLayoutData(data);
    //
    // data = new FormData();
    // data.left = new FormAttachment(representsLbl, ITabbedPropertyConstants.HSPACE);
    // data.right = new FormAttachment(newButton, -ITabbedPropertyConstants.HSPACE);
    // data.top = new FormAttachment(representsLbl, 0, SWT.CENTER);
    // cSingleObjectChooser.setLayoutData(data);
    //
    // data = new FormData();
    // data.right = new FormAttachment(100, 0);
    // data.top = new FormAttachment(representsLbl, 0, SWT.CENTER);
    // newButton.setLayoutData(data);
    //
    // hookListeners();
    // }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#createWidgets(org.eclipse.swt.widgets.Composite)
     */
    protected void createWidgets(Composite composite)
    {
        representsLbl = getWidgetFactory().createCLabel(composite, getLabelText(), SWT.NONE);

        cSingleObjectChooser = new CSingleObjectChooser(composite, getWidgetFactory(), SWT.NONE);
        cSingleObjectChooser.setLabelProvider(new AdapterFactoryLabelProvider(new UMLItemProviderAdapterFactory()));

        newButton = getWidgetFactory().createButton(composite, "New", SWT.NONE);
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#setSectionData(org.eclipse.swt.widgets.Composite)
     */
    protected void setSectionData(Composite composite)
    {
        FormData data = new FormData();
        data.left = new FormAttachment(0, 0);
        data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
        representsLbl.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(representsLbl, ITabbedPropertyConstants.HSPACE);
        data.right = new FormAttachment(newButton, -ITabbedPropertyConstants.HSPACE);
        data.top = new FormAttachment(representsLbl, 0, SWT.CENTER);
        cSingleObjectChooser.setLayoutData(data);

        data = new FormData();
        data.right = new FormAttachment(100, 0);
        data.top = new FormAttachment(representsLbl, 0, SWT.CENTER);
        newButton.setLayoutData(data);
    }

    /**
     * Adds the listeners on the widgets
     */
    protected void hookListeners()
    {
        cSingleObjectChooser.addSelectionListener(new SelectionAdapter()
        {
            /**
             * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
             */
            public void widgetSelected(SelectionEvent e)
            {
                handleComboModified();
            }
        });

        newButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent event)
            {
                handleButtonPressed();
            }
        });
    }

    /**
     * Handle the combo modified event.
     */
    protected void handleComboModified()
    {
        if (!isRefreshing && getFeatureValue() != cSingleObjectChooser.getSelection())
        {
            MixedEditDomain mixedEditDomain = (MixedEditDomain) ((Modeler) getPart().getAdapter(Modeler.class)).getAdapter(MixedEditDomain.class);

            if (getEObjectList().size() == 1)
            {
                // Update the Lifeline with the corresponding Property and
                // ConnectorEnd
                mixedEditDomain.getCommandStack().execute(new UpdateLifelineRepresentsCommand((Lifeline) getEObject(), (Classifier) cSingleObjectChooser.getSelection(), mixedEditDomain));
            }
        }
    }

    /**
     * Handle the button pressed event.
     */
    protected void handleButtonPressed()
    {
        MixedEditDomain mixedEditDomain = (MixedEditDomain) ((Modeler) getPart().getAdapter(Modeler.class)).getAdapter(MixedEditDomain.class);

        if (getEObjectList().size() == 1)
        {
            ClassifierCreateDialog dialog = new ClassifierCreateDialog(((Lifeline) getEObject()).getModel(), SequencePlugin.getActiveWorkbenchShell());
            if (dialog.open() == Window.OK)
            {
                CompoundCommand compoundCommand = new CompoundCommand();

                // Create the Classifier
                compoundCommand.add(dialog.getCommand());
                // Update the Lifeline with the corresponding Property and
                // ConnectorEnd
                compoundCommand.add(new UpdateLifelineRepresentsCommand((Lifeline) getEObject(), dialog.getCreatedClassifier(), mixedEditDomain));

                mixedEditDomain.getCommandStack().execute(compoundCommand);

                // refresh the combo so that the new Classifier is added and
                // selected
                this.refresh();
            }
        }
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.ISection#refresh()
     */
    public void refresh()
    {
        isRefreshing = true;
        cSingleObjectChooser.setChoices(getComboFeatureValues());
        cSingleObjectChooser.setSelection(getFeatureValue());
        isRefreshing = false;
    }

    /**
     * Get the List of Object to associate with the CCombo
     * 
     * @return Object[]
     */
    protected Object[] getComboFeatureValues()
    {
        return getChoices(UMLPackage.eINSTANCE.getType());
    }

    /**
     * Returns an array of all reachable objects of a given type from the current selection.
     * 
     * @param type Reachable object which have this type
     * @return An array of objects of the given type
     */
    protected Object[] getChoices(EClassifier type)
    {
        List choices = new ArrayList();
        choices.add("");
        choices.addAll(ItemPropertyDescriptor.getReachableObjectsOfType(getEObject(), type));

        return choices.toArray();
    }

    /**
     * Get the current Feature Value
     * 
     * @return Type
     */
    protected Object getFeatureValue()
    {
        ConnectableElement connectableElement = ((Lifeline) getEObject()).getRepresents();
        if (connectableElement != null)
        {
            return connectableElement.getType();
        }
        return null;
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getFeature()
     */
    protected EStructuralFeature getFeature()
    {
        return null;
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getLabelText()
     */
    protected String getLabelText()
    {
        return "Represents:";
    }

}