/*******************************************************************************
 * Copyright (c) 2006, 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *    Benoit Maggi (Atos Origin) - Show all visible operations [Bug #1462]
 ******************************************************************************/
package org.topcased.modeler.uml.sequencediagram.properties.sections;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.ConnectableElement;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageEnd;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.listeners.UIAdapterImpl;
import org.topcased.modeler.uml.sequencediagram.SequenceUtils;
import org.topcased.modeler.uml.sequencediagram.commands.update.UpdateMessageSignatureCommand;
import org.topcased.modeler.uml.sequencediagram.composites.ArgumentsComposite;
import org.topcased.modeler.widgets.CSingleObjectChooser;
import org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection;

/**
 * The section for the creation of Parameters of an Operation Object.
 * 
 * Creation 23 may 2006
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class MessageSignaturePropertySection extends AbstractTabbedPropertySection
{
    /**
     * A boolean that store if refreshing is happening and no model
     * modifications should be performed
     */
    private boolean isRefreshing = false;

    // The widgets
    /**
     * The composite responsible for choosing arguments.
     */
    private ArgumentsComposite argumentsComposite;

    /**
     * The Label that identify the section
     */
    private CLabel signatureLbl;

    /**
     * The combo box control for the section.
     */
    private CSingleObjectChooser cSingleObjectChooser;

    /**
     * the group arguments for this message section
     */
    private Group theGroup;

    /**
     * A listener on message arguments who refreshes this property section when
     * changes occur.
     */
    private final Adapter messageArgumentsListener = new UIAdapterImpl()
    {
        @Override
        protected void safeNotifyChanged(Notification msg)
        {
            argumentsComposite.updateViewer((Message) getEObject());
        }
    };

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#createWidgets(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void createWidgets(Composite composite)
    {
        theGroup = getWidgetFactory().createGroup(composite, "Signature");
        theGroup.setLayout(new FormLayout());
        signatureLbl = getWidgetFactory().createCLabel(theGroup, "", SWT.NONE);

        cSingleObjectChooser = new CSingleObjectChooser(theGroup, getWidgetFactory(), SWT.NONE);
        cSingleObjectChooser.setLabelProvider(new AdapterFactoryLabelProvider(new UMLItemProviderAdapterFactory()));

        int columnWidth = getStandardLabelWidth(getSectionComposite(), new String[] {signatureLbl.getText()});
        columnWidth = (columnWidth * 2) / 3;
        argumentsComposite = new ArgumentsComposite(theGroup, SWT.NONE, columnWidth);
        getWidgetFactory().adapt(argumentsComposite);
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#setSectionData(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void setSectionData(Composite composite)
    {
        FormData data = new FormData();
        data.left = new FormAttachment(0, 0);
        data.right = new FormAttachment(100, 0);
        data.top = new FormAttachment(0, 0);
        theGroup.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0, 0);
        data.right = new FormAttachment(cSingleObjectChooser, -ITabbedPropertyConstants.HSPACE);
        data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
        signatureLbl.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0, getStandardLabelWidth(getSectionComposite(),
                new String[] {signatureLbl.getText()}));
        data.right = new FormAttachment(100, 0);
        data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
        cSingleObjectChooser.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0, getStandardLabelWidth(getSectionComposite(),
                new String[] {signatureLbl.getText()})
                - ITabbedPropertyConstants.HSPACE);
        data.right = new FormAttachment(100, 0);
        data.top = new FormAttachment(signatureLbl, ITabbedPropertyConstants.VSPACE);
        data.bottom = new FormAttachment(100, 0);
        argumentsComposite.setLayoutData(data);

    }

    /**
     * Adds the listeners on the widgets
     */
    protected void hookListeners()
    {
        cSingleObjectChooser.addSelectionListener(new SelectionAdapter()
        {
            /**
             * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
             */
            public void widgetSelected(SelectionEvent e)
            {
                if (!isRefreshing && ((Message) getEObject()).getSignature() != cSingleObjectChooser.getSelection())
                {
                    MixedEditDomain mixedEditDomain = (MixedEditDomain) ((Modeler) getPart().getAdapter(Modeler.class)).getAdapter(MixedEditDomain.class);

                    if (getEObjectList().size() == 1)
                    {
                        // Update the Message with the new selected Signature
                        UpdateMessageSignatureCommand updateMessageSignatureCommand = new UpdateMessageSignatureCommand(
                                (Message) getEObject(), (NamedElement) cSingleObjectChooser.getSelection(),
                                mixedEditDomain);
                        mixedEditDomain.getCommandStack().execute(updateMessageSignatureCommand);
                        argumentsComposite.updateViewer((Message) getEObject());
                        // update the listeners on the new message arguments
                        updateArgumentListeners((Message) getEObject());
                    }
                }
            }
        });
    }

    /**
     * Hooks listeners on the message's arguments so as to refresh the
     * properties section appropriatly.
     * 
     * @param message
     */
    private void updateArgumentListeners(Message message)
    {
        for (ValueSpecification valueSpecification : message.getArguments())
        {
            if (!valueSpecification.eAdapters().contains(messageArgumentsListener))
            {
                valueSpecification.eAdapters().add(messageArgumentsListener);
            }
        }
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.ISection#refresh()
     */
    @Override
    public void refresh()
    {
        isRefreshing = true;

        if (SequenceUtils.isSignal((Message) getEObject()))
        {
            signatureLbl.setText("Signal:   ");
        }
        else
        {
            signatureLbl.setText("Operation:");
        }

        cSingleObjectChooser.setChoices(getComboFeatureValues());
        cSingleObjectChooser.setSelection(((Message) getEObject()).getSignature());

        MixedEditDomain mixedEditDomain = (MixedEditDomain) ((Modeler) getPart().getAdapter(Modeler.class)).getAdapter(MixedEditDomain.class);
        argumentsComposite.setEditDomain(mixedEditDomain);
        argumentsComposite.updateViewer((Message) getEObject());

        isRefreshing = false;
    }

    /**
     * Returns an array of all reachable objects of a given type from the
     * current selection.
     * 
     * @param type Reachable object which have this type
     * @return An array of objects of the given type
     */
    protected Object[] getChoices(EClassifier type)
    {
        List<Object> choices = new ArrayList<Object>();
        choices.add("");
        choices.addAll(ItemPropertyDescriptor.getReachableObjectsOfType(getEObject(), type));

        return choices.toArray();
    }

    /**
     * Get the List of Object to associate with the CCombo
     * 
     * @return Object[]
     */
    protected Object[] getComboFeatureValues()
    {
        Message message = (Message) getEObject();

        if (SequenceUtils.isSignal(message))
        {
            // TODO filters on the Signals that are only applicable for this
            // Message
            ArrayList<EObject> comboItemsList = new ArrayList<EObject>(
                    ItemPropertyDescriptor.getReachableObjectsOfType(getEObject(), UMLPackage.eINSTANCE.getSignal()));

            return comboItemsList.toArray();

        }
        if (SequenceUtils.isReplyMessage(message))
        {
            return new Object[0];
        }
        // we're here so the message is a call (synch or asynch) one
        // so we'll get the applicable operations for this message.

        // used to keep only the operations applicable for this message.
        HashSet<Operation> possibleOperations = new HashSet<Operation>();

        // used to know the lifeline where to get all the allowed operations
        MessageEnd lSendEvent = message.getReceiveEvent();

        // retrieve the classifier for this message's lifeline.
        for (Lifeline lifeline : message.getInteraction().getLifelines())
        {
            for (InteractionFragment lInteractionFragment : lifeline.getCoveredBys())
            {
                if (lInteractionFragment.equals(lSendEvent))
                {
                    ConnectableElement element = lifeline.getRepresents();
                    if (element != null)
                    {
                        Type elementType = element.getType();
                        possibleOperations.addAll(this.getAllVisibleOperations((Classifier) elementType));
                    }
                }
            }
        }
        return possibleOperations.toArray();
    }

    /**
     * Return all the operations usable by a class or an interface
     * 
     * @param pClassifier Class or Interface
     * @return HashSet<Operation>
     */
    private HashSet<Operation> getAllVisibleOperations(Classifier pClassifier)
    {
        HashSet<Operation> possibleOperations = new HashSet<Operation>();

        if (pClassifier != null && pClassifier instanceof Class)
        {
            Class lClass = (Class) pClassifier;
            possibleOperations.addAll(lClass.getAllOperations());
            for (Interface lImplementedInterface : lClass.getAllImplementedInterfaces())
            {
                possibleOperations.addAll(getAllVisibleOperations(lImplementedInterface));
            }
        }
        if (pClassifier != null && pClassifier instanceof Interface)
        {
            possibleOperations.addAll(pClassifier.getAllOperations());
        }
        return possibleOperations;
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getFeature()
     */
    protected EStructuralFeature getFeature()
    {
        return null;
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getLabelText()
     */
    protected String getLabelText()
    {
        return null;
    }

    /**
     * Overriden so as to hook our own listener to signature changes in the
     * message since we can't benefit from the mechanism from the super class
     * (no 'real' feature).
     * 
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#setInput(org.eclipse.ui.IWorkbenchPart,
     *      org.eclipse.jface.viewers.ISelection)
     */
    @Override
    public void setInput(IWorkbenchPart part, ISelection selection)
    {
        super.setInput(part, selection);
        if (getEObject() instanceof Message)
        {
            updateArgumentListeners((Message) getEObject());
        }

    }
}