/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.requests;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.requests.LocationRequest;
import org.topcased.modeler.edit.GraphEdgeEditPart;

/**
 * Request to handle the connection move
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class GraphEdgeMoveRequest extends LocationRequest
{
    private GraphEdgeEditPart edgeEditPart;

    private Point initialLocation;

    /**
     * Set the edit part of the graph edge
     * 
     * @param part the new graph edge edit part
     */
    public void setGraphEdgeEditPart(GraphEdgeEditPart part)
    {
        this.edgeEditPart = part;

    }

    /**
     * Return the graph edge edit part
     * 
     * @return the edit part of the graph edge
     */
    public GraphEdgeEditPart getGraphEdgeEditPart()
    {
        return edgeEditPart;
    }

    /**
     * Return the edit part of the source of the edge
     * 
     * @return the source edit part
     */
    public EditPart getSource()
    {
        return getGraphEdgeEditPart().getSource();
    }

    /**
     * Return the edit part of the target of the edge
     * 
     * @return the target edit part
     */
    public EditPart getTarget()
    {
        return getGraphEdgeEditPart().getTarget();
    }

    /**
     * Get the initial position
     * 
     * @return the initial position
     */
    public Point getInitialLocation()
    {
        return initialLocation;
    }

    /**
     * Set the initial position
     * 
     * @param p the initial position
     */
    public void setInitialLocation(Point p)
    {
        this.initialLocation = p;
    }

    /**
     * Return the difference of position
     * 
     * @return the difference of position
     */
    public int getDelta()
    {
        return getLocation().y - getInitialLocation().y;
    }

}
