/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.routers;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.geometry.Point;
import org.topcased.modeler.uml.sequencediagram.edit.BehaviorExecutionSpecificationEditPart;

/**
 * the router for the calls
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class CallRouter extends MessageRouter
{
    /**
     * A self connection is between two executions on the same lifeline
     * 
     * @see org.topcased.modeler.uml.sequencediagram.routers.MessageRouter#isSelfConnection(org.eclipse.draw2d.Connection)
     */
    protected boolean isSelfConnection(Connection conn)
    {
        if (conn.getSourceAnchor().getOwner() == null || conn.getTargetAnchor().getOwner() == null)
        {
            return false;
        }
        return conn.getSourceAnchor().getOwner().getParent() == conn.getTargetAnchor().getOwner().getParent() || conn.getSourceAnchor().getOwner() == conn.getTargetAnchor().getOwner().getParent()
                || conn.getSourceAnchor().getOwner().getParent() == conn.getTargetAnchor().getOwner();
    }

    /**
     * The start point depends on the way to route (self or not)
     * 
     * @see org.eclipse.draw2d.AbstractRouter#getStartPoint(org.eclipse.draw2d.Connection)
     */
    protected Point getStartPoint(Connection conn)
    {
        Point start = super.getStartPoint(conn);

        if (isSelfConnection(conn))
        {
            start.x += BehaviorExecutionSpecificationEditPart.DEFAULT_WIDTH / 2;
        }
        else
        {
            Point end = conn.getTargetAnchor().getReferencePoint();
            if (end.x > start.x)
            {
                start.x += BehaviorExecutionSpecificationEditPart.DEFAULT_WIDTH / 2;
            }
            else
            {
                start.x -= BehaviorExecutionSpecificationEditPart.DEFAULT_WIDTH / 2;
            }
        }
        return start;
    }

    /**
     * The end point depends on the way to route (self or not)
     * 
     * @see org.eclipse.draw2d.AbstractRouter#getEndPoint(org.eclipse.draw2d.Connection)
     */
    protected Point getEndPoint(Connection conn)
    {
        Point end = super.getEndPoint(conn);

        if (isSelfConnection(conn))
        {
            end.x += BehaviorExecutionSpecificationEditPart.DEFAULT_WIDTH / 2;
        }
        else
        {
            Point start = conn.getSourceAnchor().getReferencePoint();
            if (start.x > end.x)
            {
                end.x += BehaviorExecutionSpecificationEditPart.DEFAULT_WIDTH / 2;
            }
            else
            {
                end.x -= BehaviorExecutionSpecificationEditPart.DEFAULT_WIDTH / 2;
            }
        }
        return end;
    }
}
