/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.routers;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.gmf.runtime.draw2d.ui.internal.routers.RectilinearRouter;

/**
 * The router for the messages
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public abstract class MessageRouter extends RectilinearRouter
{
    private static final int DEFAULT_WIDTH = 30;

    /**
     * The route line is straigth or in 3 rectilinear parts if it's a self connection
     * 
     * @see org.eclipse.gmf.runtime.draw2d.ui.internal.routers.ObliqueRouter#routeLine(org.eclipse.draw2d.Connection,
     *      int, org.eclipse.draw2d.geometry.PointList)
     */
    public void routeLine(Connection conn, int nestedRoutingDepth, PointList newLine)
    {
        if (isSelfConnection(conn))
        {
            if (conn.getSourceAnchor().getOwner().getParent() == null)
            {
                return;
            }
            if (newLine.size() < 4)
            {
                setSelfPoints(conn, DEFAULT_WIDTH, newLine);
            }
            else
            {
                int width = newLine.getMidpoint().x - newLine.getFirstPoint().x;
                setSelfPoints(conn, width, newLine);
            }
        }
        else
        {
            newLine.removeAllPoints();
            Point p = getStartPoint(conn);
            conn.translateToRelative(p);
            newLine.addPoint(p);
            p = getEndPoint(conn);
            conn.translateToRelative(p);
            newLine.addPoint(p);
        }
    }

    /**
     * Say if the connection is self
     * 
     * @param conn the connection
     * @return true if it's a self connection
     */
    protected abstract boolean isSelfConnection(Connection conn);

    /**
     * Set the points for a self connection
     * 
     * @param conn the connection
     * @param width the width between the lifeline and the edge
     * @param newLine the list of point to update
     */
    protected void setSelfPoints(Connection conn, int width, PointList newLine)
    {
        newLine.removeAllPoints();

        Point pStart = getStartPoint(conn);
        Point pEnd = getEndPoint(conn);

        conn.translateToRelative(pStart);

        newLine.addPoint(pStart);

        Point p2 = Point.SINGLETON;
        p2.setLocation(pStart);
        p2.x += width;

        newLine.addPoint(p2);

        conn.translateToRelative(pEnd);

        p2.setLocation(pEnd);
        p2.x += width + (pStart.x - pEnd.x);

        newLine.addPoint(p2);

        newLine.addPoint(pEnd);

    }

}
