/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.trackers;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.Request;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.dragtrackers.SelectGraphEdgeEditPartTracker;
import org.topcased.modeler.uml.UMLRequestConstants;
import org.topcased.modeler.uml.sequencediagram.requests.GraphEdgeMoveRequest;

/**
 * This tracker will manage the moving of the graph edge
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 */
public class MoveGraphEdgeEditPartTracker extends SelectGraphEdgeEditPartTracker
{

    private Point initialLoc;

    /**
     * Default contructor
     * 
     * @param owner the edge edit part
     */
    public MoveGraphEdgeEditPartTracker(GraphEdgeEditPart owner)
    {
        super(owner);
    }

    /**
     * Set the request to be a REQ_MOVE_GRAPHEDGE
     * 
     * @see org.eclipse.gef.tools.AbstractTool#handleButtonDown(int)
     */
    protected boolean handleButtonDown(int button)
    {
        if (!super.handleButtonDown(button))
        {
            return false;
        }

        setType(UMLRequestConstants.REQ_MOVE_GRAPHEDGE);
        initialLoc = getLocation();
        return true;
    }

    /**
     * Create a MessageMoveRequest
     * 
     * @see org.topcased.modeler.edit.dragtrackers.SelectGraphEdgeEditPartTracker#createSourceRequest()
     */
    protected Request createSourceRequest()
    {
        if (getType() == UMLRequestConstants.REQ_MOVE_GRAPHEDGE)
        {
            GraphEdgeMoveRequest request = new GraphEdgeMoveRequest();
            request.setInitialLocation(initialLoc);
            request.setGraphEdgeEditPart((GraphEdgeEditPart) getSourceEditPart());
            request.setType(UMLRequestConstants.REQ_MOVE_GRAPHEDGE);
            return request;
        }
        return super.createSourceRequest();
    }

}
