/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.util;

import org.eclipse.uml2.uml.Expression;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.LiteralBoolean;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.TypedElement;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.ValueSpecification;
import org.topcased.modeler.uml.UMLTools;

/**
 * This class is used to manipulate ValueSpecification via the MessageArgumentsTableViewer
 * 
 * @see org.topcased.modeler.uml.sequencediagram.properties.sections.MessageArgumentsTableViewer
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class MessageArgument
{
    private final Model rootElement;

    private ValueSpecification valueSpec;

    private Type type;

    /**
     * The constructor
     * 
     * @param valueSpec the ValueSpecification
     * @param typedElement the TypedElement (Parameter or Property)
     */
    public MessageArgument(ValueSpecification valueSpec, TypedElement typedElement)
    {
        this.rootElement = typedElement.getModel();
        this.valueSpec = valueSpec;
        this.type = typedElement.getType();
        valueSpec.setName(typedElement.getName());
    }

    /**
     * Return the ValueSpecification corresponding to the argument
     * 
     * @return a ValueSpecification
     */
    public ValueSpecification getValueSpecification()
    {
        return valueSpec;
    }

    /**
     * Set the valueSpec of the ValueSpecification of the argument. If the valueSpec is null, the argument's
     * ValueSpecification is now a LitteralNull.
     * 
     * @param textValue the valueSpec as a string
     */
    public void setStringValue(String textValue)
    {
        if (textValue == null)
        {
            String name = valueSpec.getName();
            valueSpec = UMLFactory.eINSTANCE.createLiteralNull();
            valueSpec.setName(name);
        }
        else
        {
            if (type instanceof PrimitiveType)
            {
                if (type.getName().equals("Boolean"))
                {
                    ((LiteralBoolean) valueSpec).setValue(Boolean.parseBoolean(textValue));
                }
                else if (type.getName().equals("Integer"))
                {
                    ((LiteralInteger) valueSpec).setValue(Integer.parseInt(textValue));
                }
                else if (type.getName().equals("String"))
                {
                    ((LiteralString) valueSpec).setValue(textValue);
                }
                else if (valueSpec instanceof Expression)
                {
                    // TODO use a parser to create the String associated with
                    // the Expression. Before : ((Expression)
                    // valueSpec).setBody(textValue);
                }
            }
            else if (type instanceof org.eclipse.uml2.uml.Class)
            {
                ((InstanceValue) valueSpec).setInstance((InstanceSpecification) UMLTools.getElementFromPath(rootElement, textValue));
            }
        }
    }

    /**
     * Get the type of the arguent
     * 
     * @return the type
     */
    public Type getType()
    {
        return type;
    }

    /**
     * Return the name of tha argument
     * 
     * @return the name
     */
    public String getName()
    {
        return valueSpec.getName();
    }

    /**
     * Set the name of the argument
     * 
     * @param name the new name
     */
    public void setName(String name)
    {
        valueSpec.setName(name);
    }

}
