/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram;

import java.net.URL;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.gef.EditPartFactory;
import org.topcased.modeler.editor.EditPart2ModelAdapterFactory;
import org.topcased.modeler.editor.IConfiguration;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.IPaletteManager;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.graphconf.exceptions.MissingGraphConfFileException;
import org.topcased.modeler.uml.alldiagram.edit.CommentEditPart;
import org.topcased.modeler.uml.alldiagram.edit.ElementEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.CompositeStateEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.ConnectionPointReferenceEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.FinalStateEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateChoiceEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateDeepHistoryEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateEntryPointEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateExitPointEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateForkEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateInitialEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateJoinEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateJunctionEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateShallowHistoryEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateTerminateEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.RegionEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.StateEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.SubmachineStateEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.TransitionEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.VertexEditPart;

/**
 * A diagram configuration : manages Palette, EditPartFactory for this diagram. <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class STMConfiguration implements IConfiguration
{
    private STMPaletteManager paletteManager;

    private STMEditPartFactory editPartFactory;

    private STMCreationUtils creationUtils;

    /**
     * The DiagramGraphConf that contains graphical informations on the configuration
     */
    private DiagramGraphConf diagramGraphConf;

    /**
     * Constructor. Initialize Adapter factories.
     * 
     * @generated
     */
    public STMConfiguration()
    {
        registerAdapters();
    }

    /**
     * Registers the Adapter Factories for all the EditParts
     * 
     * @generated
     */
    private void registerAdapters()
    {
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(ElementEditPart.class, org.eclipse.uml2.uml.Element.class), ElementEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(RegionEditPart.class, org.eclipse.uml2.uml.Region.class), RegionEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(VertexEditPart.class, org.eclipse.uml2.uml.Vertex.class), VertexEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(StateEditPart.class, org.eclipse.uml2.uml.State.class), StateEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(CompositeStateEditPart.class, org.eclipse.uml2.uml.State.class), CompositeStateEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(SubmachineStateEditPart.class, org.eclipse.uml2.uml.State.class), SubmachineStateEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(FinalStateEditPart.class, org.eclipse.uml2.uml.FinalState.class), FinalStateEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(TransitionEditPart.class, org.eclipse.uml2.uml.Transition.class), TransitionEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(PseudostateInitialEditPart.class, org.eclipse.uml2.uml.Pseudostate.class), PseudostateInitialEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(PseudostateDeepHistoryEditPart.class, org.eclipse.uml2.uml.Pseudostate.class),
                PseudostateDeepHistoryEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(PseudostateShallowHistoryEditPart.class, org.eclipse.uml2.uml.Pseudostate.class),
                PseudostateShallowHistoryEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(PseudostateJoinEditPart.class, org.eclipse.uml2.uml.Pseudostate.class), PseudostateJoinEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(PseudostateForkEditPart.class, org.eclipse.uml2.uml.Pseudostate.class), PseudostateForkEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(PseudostateJunctionEditPart.class, org.eclipse.uml2.uml.Pseudostate.class), PseudostateJunctionEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(PseudostateChoiceEditPart.class, org.eclipse.uml2.uml.Pseudostate.class), PseudostateChoiceEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(PseudostateEntryPointEditPart.class, org.eclipse.uml2.uml.Pseudostate.class),
                PseudostateEntryPointEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(PseudostateExitPointEditPart.class, org.eclipse.uml2.uml.Pseudostate.class), PseudostateExitPointEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(PseudostateTerminateEditPart.class, org.eclipse.uml2.uml.Pseudostate.class), PseudostateTerminateEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(ConnectionPointReferenceEditPart.class, org.eclipse.uml2.uml.ConnectionPointReference.class),
                ConnectionPointReferenceEditPart.class);
        Platform.getAdapterManager().registerAdapters(new EditPart2ModelAdapterFactory(CommentEditPart.class, org.eclipse.uml2.uml.Comment.class), CommentEditPart.class);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getId()
     * @generated
     */
    public String getId()
    {
        return new String("org.topcased.modeler.uml.statemachinediagram");
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getName()
     * @generated
     */
    public String getName()
    {
        return new String("State Machine Diagram");
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getEditPartFactory()
     * @generated
     */
    public EditPartFactory getEditPartFactory()
    {
        if (editPartFactory == null)
        {
            editPartFactory = new STMEditPartFactory();
        }

        return editPartFactory;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getPaletteManager()
     * @generated
     */
    public IPaletteManager getPaletteManager()
    {
        if (paletteManager == null)
        {
            paletteManager = new STMPaletteManager(getCreationUtils());
        }

        return paletteManager;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getCreationUtils()
     * @generated
     */
    public ICreationUtils getCreationUtils()
    {
        if (creationUtils == null)
        {
            creationUtils = new STMCreationUtils(getDiagramGraphConf());
        }

        return creationUtils;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.IConfiguration#getDiagramGraphConf()
     * @generated
     */
    public DiagramGraphConf getDiagramGraphConf()
    {
        if (diagramGraphConf == null)
        {
            URL url = STMPlugin.getDefault().getBundle().getResource("org/topcased/modeler/uml/statemachinediagram/diagram.graphconf");
            if (url != null)
            {
                URI fileURI = URI.createURI(url.toString());
                ResourceSet resourceSet = new ResourceSetImpl();
                Resource resource = resourceSet.getResource(fileURI, true);
                if (resource != null && resource.getContents().get(0) instanceof DiagramGraphConf)
                {
                    diagramGraphConf = (DiagramGraphConf) resource.getContents().get(0);
                }
            }
            else
            {
                new MissingGraphConfFileException("The *.diagramgraphconf file can not be retrieved. Check if the path is correct in the Configuration class of your diagram.");
            }
        }

        return diagramGraphConf;
    }

}