/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram;

import java.util.Iterator;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.ecore.EObject;
import org.topcased.modeler.di.model.DiagramInterchangeFactory;
import org.topcased.modeler.di.model.EdgeObjectOffset;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.AbstractCreationUtils;
import org.topcased.modeler.evaluator.EvaluatorException;
import org.topcased.modeler.evaluator.extension.EvaluatorsManager;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.uml.alldiagram.ExactUMLSwitch;

/**
 * This utility class allows to create a GraphElement associated with a Model Object <!-- begin-user-doc --> <!--
 * end-user-doc -->
 * 
 * @generated
 */
public class STMCreationUtils extends AbstractCreationUtils
{

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param diagramConf the Diagram Graphical Configuration
     * @generated
     */
    public STMCreationUtils(DiagramGraphConf diagramConf)
    {
        super(diagramConf);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class GraphicUMLSwitch extends ExactUMLSwitch
    {
        /** The presentation of the graphical element */
        private String presentation;

        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param presentation the presentation of the graphical element
         * @generated
         */
        public GraphicUMLSwitch(String presentation)
        {
            this.presentation = presentation;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseRegion(org.eclipse.uml2.uml.Region)
         * @generated
         */
        public Object caseRegion(org.eclipse.uml2.uml.Region object)
        {
            if ("default".equals(presentation))
            {
                return createGraphElementRegion(object, presentation);
            }
            return null;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseState(org.eclipse.uml2.uml.State)
         * @generated
         */
        public Object caseState(org.eclipse.uml2.uml.State object)
        {
            try
            {
                if ("default".equals(presentation) && EvaluatorsManager.getInstance().evaluate(object, "self.isSimple=true", "ocl")
                        && EvaluatorsManager.getInstance().evaluate(object, "self.isSubmachineState=false", "ocl"))
                {
                    return createGraphElementState(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if ("default".equals(presentation) && EvaluatorsManager.getInstance().evaluate(object, "self.isComposite=true", "ocl"))
                {
                    return createGraphElementCompositeState(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if ("default".equals(presentation) && EvaluatorsManager.getInstance().evaluate(object, "self.isSubmachineState=true", "ocl"))
                {
                    return createGraphElementSubmachineState(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            return null;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseConnectionPointReference(org.eclipse.uml2.uml.ConnectionPointReference)
         * @generated
         */
        public Object caseConnectionPointReference(org.eclipse.uml2.uml.ConnectionPointReference object)
        {
            if ("default".equals(presentation))
            {
                return createGraphElementConnectionPointReference(object, presentation);
            }
            return null;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseFinalState(org.eclipse.uml2.uml.FinalState)
         * @generated
         */
        public Object caseFinalState(org.eclipse.uml2.uml.FinalState object)
        {
            if ("default".equals(presentation))
            {
                return createGraphElementFinalState(object, presentation);
            }
            return null;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#casePseudostate(org.eclipse.uml2.uml.Pseudostate)
         * @generated
         */
        public Object casePseudostate(org.eclipse.uml2.uml.Pseudostate object)
        {
            try
            {
                if ("default".equals(presentation) && EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::initial", "ocl"))
                {
                    return createGraphElementPseudostateInitial(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if ("default".equals(presentation) && EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::deepHistory", "ocl"))
                {
                    return createGraphElementPseudostateDeepHistory(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if ("default".equals(presentation) && EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::shallowHistory", "ocl"))
                {
                    return createGraphElementPseudostateShallowHistory(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (("default".equals(presentation) || "horizontal".equals(presentation)) && EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::join", "ocl"))
                {
                    return createGraphElementPseudostateJoin(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (("default".equals(presentation) || "horizontal".equals(presentation)) && EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::fork", "ocl"))
                {
                    return createGraphElementPseudostateFork(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if ("default".equals(presentation) && EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::junction", "ocl"))
                {
                    return createGraphElementPseudostateJunction(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if ("default".equals(presentation) && EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::choice", "ocl"))
                {
                    return createGraphElementPseudostateChoice(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if ("default".equals(presentation) && EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::entryPoint", "ocl"))
                {
                    return createGraphElementPseudostateEntryPoint(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if ("default".equals(presentation) && EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::exitPoint", "ocl"))
                {
                    return createGraphElementPseudostateExitPoint(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if ("default".equals(presentation) && EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::terminate", "ocl"))
                {
                    return createGraphElementPseudostateTerminate(object, presentation);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            return null;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseTransition(org.eclipse.uml2.uml.Transition)
         * @generated
         */
        public Object caseTransition(org.eclipse.uml2.uml.Transition object)
        {
            if ("default".equals(presentation))
            {
                return createGraphElementTransition(object, presentation);
            }
            return null;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseComment(org.eclipse.uml2.uml.Comment)
         * @generated
         */
        public Object caseComment(org.eclipse.uml2.uml.Comment object)
        {
            if ("default".equals(presentation))
            {
                return createGraphElementComment(object, presentation);
            }
            return null;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return null;
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.ICreationUtils#createGraphElement(org.eclipse.emf.ecore.EObject,
     *      java.lang.String)
     * @generated
     */
    public GraphElement createGraphElement(EObject obj, String presentation)
    {
        Object graphElt = null;

        if ("http://www.eclipse.org/uml2/2.1.0/UML".equals(obj.eClass().getEPackage().getNsURI()))
        {
            graphElt = new GraphicUMLSwitch(presentation).doSwitch(obj);
        }

        if (graphElt instanceof GraphElement)
        {
            return (GraphElement) graphElt;
        }

        return null;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementRegion(org.eclipse.uml2.uml.Region element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementState(org.eclipse.uml2.uml.State element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated NOT
     */
    protected GraphElement createGraphElementCompositeState(org.eclipse.uml2.uml.State element, String presentation)
    {
        GraphNode nodeParent = createGraphNode(element, presentation);

        for (Iterator itRegions = element.getRegions().iterator(); itRegions.hasNext();)
        {
            GraphNode region = createGraphNode((EObject) itRegions.next());
            region.setContainer(nodeParent);
        }

        return nodeParent;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementSubmachineState(org.eclipse.uml2.uml.State element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementConnectionPointReference(org.eclipse.uml2.uml.ConnectionPointReference element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementFinalState(org.eclipse.uml2.uml.FinalState element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementPseudostateInitial(org.eclipse.uml2.uml.Pseudostate element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementPseudostateDeepHistory(org.eclipse.uml2.uml.Pseudostate element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementPseudostateShallowHistory(org.eclipse.uml2.uml.Pseudostate element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementPseudostateJoin(org.eclipse.uml2.uml.Pseudostate element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementPseudostateFork(org.eclipse.uml2.uml.Pseudostate element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementPseudostateJunction(org.eclipse.uml2.uml.Pseudostate element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementPseudostateChoice(org.eclipse.uml2.uml.Pseudostate element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementPseudostateEntryPoint(org.eclipse.uml2.uml.Pseudostate element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementPseudostateExitPoint(org.eclipse.uml2.uml.Pseudostate element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementPseudostateTerminate(org.eclipse.uml2.uml.Pseudostate element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementTransition(org.eclipse.uml2.uml.Transition element, String presentation)
    {
        GraphEdge graphEdge = createGraphEdge(element, presentation);
        EdgeObjectOffset nameEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
        nameEdgeObjectOffset.setId(STMEdgeObjectConstants.NAME_EDGE_OBJECT_ID);
        nameEdgeObjectOffset.setOffset(new Dimension(0, 0));
        graphEdge.getContained().add(nameEdgeObjectOffset);
        return graphEdge;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementComment(org.eclipse.uml2.uml.Comment element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * Create the ModelObject with its initial children <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the model object
     * @return the model object with its children
     * @generated
     */
    public EObject createModelObject(EObject obj)
    {
        return obj;
    }

}
