/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.uml2.uml.util.UMLSwitch;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.evaluator.EvaluatorException;
import org.topcased.modeler.evaluator.extension.EvaluatorsManager;
import org.topcased.modeler.uml.alldiagram.edit.CommentEditPart;
import org.topcased.modeler.uml.alldiagram.edit.CommentLinkEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.CompositeStateEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.ConnectionPointReferenceEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.FinalStateEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateChoiceEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateDeepHistoryEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateEntryPointEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateExitPointEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateForkEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateInitialEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateJoinEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateJunctionEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateShallowHistoryEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.PseudostateTerminateEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.RegionEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.STMDiagramEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.StateEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.SubmachineStateEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.TransitionEditPart;
import org.topcased.modeler.uml.statemachinediagram.edit.VertexEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * Part Factory : associates a model object to its controller. <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class STMEditPartFactory implements EditPartFactory
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
     * @generated
     */
    public EditPart createEditPart(EditPart context, Object model)
    {
        if (model instanceof Diagram)
        {
            return new STMDiagramEditPart((Diagram) model);
        }
        else if (model instanceof GraphNode)
        {
            final GraphNode node = (GraphNode) model;
            EObject element = Utils.getElement(node);
            if (element != null)
            {
                if ("http://www.eclipse.org/uml2/2.1.0/UML".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new NodeUMLSwitch(node).doSwitch(element);
                }
                return new EMFGraphNodeEditPart(node);
            }
            return new GraphNodeEditPart(node);
        }
        else if (model instanceof GraphEdge)
        {
            final GraphEdge edge = (GraphEdge) model;
            EObject element = Utils.getElement(edge);
            if (element != null)
            {
                if ("http://www.eclipse.org/uml2/2.1.0/UML".equals(element.eClass().getEPackage().getNsURI()))
                {
                    return (EditPart) new EdgeUMLSwitch(edge).doSwitch(element);
                }
                return new EMFGraphEdgeEditPart(edge);
            }
            if (edge.getSemanticModel() instanceof SimpleSemanticModelElement)
            {
                if (STMSimpleObjectConstants.SIMPLE_OBJECT_COMMENTLINK.equals(((SimpleSemanticModelElement) edge.getSemanticModel()).getTypeInfo()))
                {
                    return new CommentLinkEditPart(edge);
                }
            }
            return new GraphEdgeEditPart(edge);
        }

        throw new IllegalStateException("No edit part matches with the '" + model.getClass().getName() + "' model element. Check STMEditPartFactory#createEditPart(EditPart,Object) class");
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class NodeUMLSwitch extends UMLSwitch<Object>
    {
        /** The graphical node */
        private GraphNode node;

        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param node the graphical node
         * @generated
         */
        public NodeUMLSwitch(GraphNode node)
        {
            this.node = node;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseRegion(org.eclipse.uml2.uml.Region)
         * @generated
         */
        public Object caseRegion(org.eclipse.uml2.uml.Region object)
        {
            return new RegionEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseVertex(org.eclipse.uml2.uml.Vertex)
         * @generated
         */
        public Object caseVertex(org.eclipse.uml2.uml.Vertex object)
        {
            return new VertexEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseState(org.eclipse.uml2.uml.State)
         * @generated NOT
         */
        public Object caseState(org.eclipse.uml2.uml.State object)
        {
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.isSimple=true", "ocl") && EvaluatorsManager.getInstance().evaluate(object, "self.isSubmachineState=false", "ocl"))
                {
                    return new StateEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.isComposite=true", "ocl"))
                {
                    return new CompositeStateEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.isSubmachineState=true", "ocl"))
                {
                    return new SubmachineStateEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            return new EMFGraphNodeEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseFinalState(org.eclipse.uml2.uml.FinalState)
         * @generated
         */
        public Object caseFinalState(org.eclipse.uml2.uml.FinalState object)
        {
            return new FinalStateEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#casePseudostate(org.eclipse.uml2.uml.Pseudostate)
         * @generated
         */
        public Object casePseudostate(org.eclipse.uml2.uml.Pseudostate object)
        {
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::initial", "ocl"))
                {
                    return new PseudostateInitialEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::deepHistory", "ocl"))
                {
                    return new PseudostateDeepHistoryEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::shallowHistory", "ocl"))
                {
                    return new PseudostateShallowHistoryEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::join", "ocl"))
                {
                    return new PseudostateJoinEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::fork", "ocl"))
                {
                    return new PseudostateForkEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::junction", "ocl"))
                {
                    return new PseudostateJunctionEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::choice", "ocl"))
                {
                    return new PseudostateChoiceEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::entryPoint", "ocl"))
                {
                    return new PseudostateEntryPointEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::exitPoint", "ocl"))
                {
                    return new PseudostateExitPointEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            try
            {
                if (EvaluatorsManager.getInstance().evaluate(object, "self.kind=PseudostateKind::terminate", "ocl"))
                {
                    return new PseudostateTerminateEditPart(node);
                }
            }
            catch (EvaluatorException ee)
            {
                STMPlugin.log(ee);
            }
            return new EMFGraphNodeEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseConnectionPointReference(org.eclipse.uml2.uml.ConnectionPointReference)
         * @generated
         */
        public Object caseConnectionPointReference(org.eclipse.uml2.uml.ConnectionPointReference object)
        {
            return new ConnectionPointReferenceEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseComment(org.eclipse.uml2.uml.Comment)
         * @generated
         */
        public Object caseComment(org.eclipse.uml2.uml.Comment object)
        {
            return new CommentEditPart(node);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphNodeEditPart(node);
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class EdgeUMLSwitch extends UMLSwitch<Object>
    {
        /** The graphical edge */
        private GraphEdge edge;

        /**
         * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @param edge the graphical edge
         * @generated
         */
        public EdgeUMLSwitch(GraphEdge edge)
        {
            this.edge = edge;
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseTransition(org.eclipse.uml2.uml.Transition)
         * @generated
         */
        public Object caseTransition(org.eclipse.uml2.uml.Transition object)
        {
            return new TransitionEditPart(edge);
        }

        /**
         * <!-- begin-user-doc --> <!-- end-user-doc -->
         * 
         * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         * @generated
         */
        public Object defaultCase(EObject object)
        {
            return new EMFGraphEdgeEditPart(edge);
        }
    }

}