/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram.commands;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Region;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.UMLFactory;
import org.topcased.modeler.extensions.ICreationDiagram;

/**
 * Creation 3 mai 07
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class CreateSTMDiagramFromPackage extends Command implements ICreationDiagram
{
    private Package thePackage;

    private StateMachine stateMachine;

    private Region region;

    /**
     * Constructor
     */
    public CreateSTMDiagramFromPackage()
    {
        // Do nothing
    }

    /**
     * @see org.topcased.modeler.extensions.ICreationDiagram#getLinkedElementWithDiagram(org.eclipse.emf.ecore.EObject)
     */
    public EObject getLinkedElementWithDiagram(EObject initialObject)
    {
        if (initialObject instanceof Package)
        {
            thePackage = (Package) initialObject;
            stateMachine = UMLFactory.eINSTANCE.createStateMachine();
            region = UMLFactory.eINSTANCE.createRegion();
            stateMachine.getRegions().add(region);
            return stateMachine;
        }

        return initialObject;
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    @Override
    public void execute()
    {
        redo();
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    @Override
    public void redo()
    {
        thePackage.getPackagedElements().add(stateMachine);
    }

    /**
     * @see org.eclipse.gef.commands.Command#undo()
     */
    @Override
    public void undo()
    {
        thePackage.getPackagedElements().remove(stateMachine);
    }

}
