/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram.edit;

import java.util.Iterator;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.OpaqueBehavior;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.UMLLabel;
import org.topcased.modeler.uml.statemachinediagram.commands.StateRestoreConnectionCommand;
import org.topcased.modeler.uml.statemachinediagram.figures.StateFigure;

/**
 * The State object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class StateEditPart extends VertexEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public StateEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new StateRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());

        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new StateFigure();
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        if (getLabel() != null)
        {
            ComposedLabel composedLabel = (ComposedLabel) getLabel();

            // Update the Stereotype Label
            composedLabel.setPrefix(UMLLabel.getStereotypesNotation((Element) getEObject()));
        }

        super.refreshHeaderLabel();
    }

    @Override
    protected void refreshVisuals()
    {
        super.refreshVisuals();

        // Handle FinalStateEditPart (which is not a StateFigure)
        if (getFigure() instanceof StateFigure)
        {
            refreshEntry();
            refreshDo();
            refreshExit();
        }
    }

    private void refreshEntry()
    {
        StateFigure stateFigure = (StateFigure) getFigure();
        State state = (State) getEObject();
        stateFigure.setEntry(getBehaviorString(state.getEntry()));
    }

    private void refreshDo()
    {
        StateFigure stateFigure = (StateFigure) getFigure();
        State state = (State) getEObject();
        stateFigure.setDo(getBehaviorString(state.getDoActivity()));
    }

    private void refreshExit()
    {
        StateFigure stateFigure = (StateFigure) getFigure();
        State state = (State) getEObject();
        stateFigure.setExit(getBehaviorString(state.getExit()));
    }

    private String getBehaviorString(Behavior b)
    {
        String label = "";

        if (b != null)
        {
            if (b instanceof OpaqueBehavior) // display the body of the action/behavior
            {
                for (Iterator<String> iterator = ((OpaqueBehavior) b).getBodies().iterator(); iterator.hasNext();)
                {
                    String body = iterator.next();
                    label = label.concat(body);
                    if (iterator.hasNext())
                    {
                        label = label.concat(" "); // insert space as a delimiter; '\n' would be another option
                    }
                }
            }
            if (label.equals("")) // display action/behavior name only
            {
                label = b.getName();
            }
        }
        return label;
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {
        Object notifier = msg.getNotifier();
        Object newObject = msg.getNewValue();
        Object oldObject = msg.getOldValue();

        if (notifier instanceof State)
        {
            if (notifier == getEObject())
            {
                switch (msg.getFeatureID(State.class))
                {
                    case UMLPackage.STATE__ENTRY:
                    case UMLPackage.STATE__DO_ACTIVITY:
                    case UMLPackage.STATE__EXIT:
                        updateModelListening(oldObject, newObject);
                        break;
                    default:
                        break;
                }
                super.handleModelChanged(msg);
            }
        }
        else if (notifier instanceof Behavior)
        {
            switch (msg.getFeatureID(Behavior.class))
            {
                case UMLPackage.BEHAVIOR__NAME:
                    refreshEntry();
                    refreshDo();
                    refreshExit();
                    break;
                default:
                    break;
            }
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#activate()
     */
    public void activate()
    {
        super.activate();
        EObject model = getEObject();
        if (model instanceof State)
        {
            updateModelListening(null, ((State) model).getEntry());
            updateModelListening(null, ((State) model).getDoActivity());
            updateModelListening(null, ((State) model).getExit());
        }
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#deactivate()
     */
    public void deactivate()
    {
        EObject model = getEObject();
        if (model instanceof State)
        {
            updateModelListening(((State) model).getEntry(), null);
            updateModelListening(((State) model).getDoActivity(), null);
            updateModelListening(((State) model).getExit(), null);
        }
        super.deactivate();
    }

}