/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.action.IAction;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.StateMachine;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.ResizableEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.uml.UMLTools;
import org.topcased.modeler.uml.statemachinediagram.commands.SubmachineStateRestoreConnectionCommand;
import org.topcased.modeler.uml.statemachinediagram.figures.SubmachineStateFigure;
import org.topcased.modeler.uml.statemachinediagram.policies.SubmachineStateLayoutEditPolicy;

/**
 * The SubmachineState object controller <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class SubmachineStateEditPart extends VertexEditPart
{
    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the graph node
     * @generated
     */
    public SubmachineStateEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new SubmachineStateRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(ModelerEditPolicyConstants.RESIZABLE_EDITPOLICY, new ResizableEditPolicy());

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new SubmachineStateLayoutEditPolicy());
        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     * @generated
     */
    protected IFigure createFigure()
    {

        return new SubmachineStateFigure();
    }

    /**
     * The Editable part of the label is the prefix of the prefix ComposedLabel
     * 
     * @see org.topcased.modeler.edit.GraphNodeEditPart#getEditableLabel()
     */
    public ILabel getEditableLabel()
    {
        ComposedLabel label = (ComposedLabel) ((ComposedLabel) getLabel()).getMain();
        return label.getMain();
    }

    /**
     * Refresh the SubmachineState label
     * 
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        ComposedLabel label = (ComposedLabel) getLabel();
        ComposedLabel mainLabel = (ComposedLabel) label.getMain();
        State state = (State) getEObject();

        // Update the Stereotype label
        label.setPrefix(UMLTools.getStereotypesNotation(state));

        mainLabel.setMain(state.getName());

        StateMachine stm = state.getSubmachine();
        String stmName = "";
        if (stm != null && stm.getName() != null)
        {
            stmName = stm.getName();
        }
        label.setSuffix(stmName);
    }

    /**
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#createChangeDiagramAction(org.eclipse.emf.ecore.EObject)
     */
    protected IAction createChangeDiagramAction(EObject modelObject)
    {
        IAction action = null;
        if (((State) getEObject()).getSubmachine() != null)
        {
            action = super.createChangeDiagramAction(((State) getEObject()).getSubmachine());
        }
        return action;
    }

}