/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram.figures;

import org.eclipse.draw2d.AbstractBorder;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.RoundedRectangle;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.Label;
import org.topcased.draw2d.layout.GridData;
import org.topcased.draw2d.layout.RegionLayout;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 *
 * @generated
 */
public class CompositeStateFigure extends org.topcased.draw2d.figures.ContainerFigure
{
    private ComposedLabel composedLabel;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     *
     * @generated
     */
    public CompositeStateFigure()
    {
        super();

        GridData data = new GridData();
        data.horizontalIndent = 20;
        setConstraint(composedLabel, data);
    }

    /**
     * @see org.topcased.draw2d.figures.ContainerFigure#createContainer()
     */
    protected IFigure createContainer()
    {
        final int roundCorner = 20;

        RoundedRectangle container = new CompositeStateRoundedRectangle();
        container.setCornerDimensions(new Dimension(roundCorner, roundCorner));
        container.setOpaque(true);
        container.setLayoutManager(new RegionLayout(false));
        return container;
    }

    /**
     * @see org.topcased.draw2d.figures.ContainerFigure#createLabel()
     */
    protected ILabel createLabel()
    {
        ILabel label = new EditableLabel();
        ((EditableLabel) label).setLabelAlignment(PositionConstants.LEFT);

        composedLabel = new ComposedLabel(new Label(), label, null, false);
        composedLabel.setBorder(new CompositeStateHeaderBorder());

        return composedLabel;
    }

    /**
     * A rounded rectangle, that draws 10px smaller on left, right, and bottom
     */
    private static class CompositeStateRoundedRectangle extends RoundedRectangle
    {
        @Override
        protected void outlineShape(Graphics graphics)
        {
            Rectangle f = Rectangle.SINGLETON;
            Rectangle r = getBounds();
            f.x = r.x + lineWidth / 2 + 10;
            f.y = r.y + lineWidth / 2;
            f.width = r.width - lineWidth - 20;
            f.height = r.height - lineWidth - 10;
            graphics.drawRoundRectangle(f, corner.width, corner.height);
        }
    }

    /**
     * A Border at the left, top and right of the label
     */
    public static class CompositeStateHeaderBorder extends AbstractBorder
    {

        private static final Insets INSETS = new Insets(1, 3, 3, 3);

        /**
         * The Constructor
         */
        public CompositeStateHeaderBorder()
        {
            // Do nothing
        }

        /**
         * @see org.eclipse.draw2d.Border#getInsets(org.eclipse.draw2d.IFigure)
         */
        public Insets getInsets(IFigure figure)
        {
            return INSETS;
        }

        /**
         * Draw the 3 borders
         *
         * @see org.eclipse.draw2d.Border#paint(org.eclipse.draw2d.IFigure, org.eclipse.draw2d.Graphics,
         *      org.eclipse.draw2d.geometry.Insets)
         */
        public void paint(IFigure figure, Graphics graphics, Insets insets)
        {
            graphics.pushState();

            graphics.setLineWidth(2);
            graphics.drawRectangle(getPaintRectangle(figure, insets).getExpanded(-1, -1));

            graphics.popState();
        }
    }

    /**
     * Change the orientation of the children Region Figures
     *
     * @param isVertical Indicate whether Regions should be arranged vertically
     */
    public void setVerticalLayout(boolean isVertical)
    {
        ((RegionLayout) getContentPane().getLayoutManager()).setVertical(isVertical);
        invalidateTree();
    }
}