/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram.figures;

import org.eclipse.draw2d.CompoundBorder;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gmf.runtime.draw2d.ui.figures.OneLineBorder;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.draw2d.figures.Label;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated NOT
 */
public class StateFigure extends Shape implements ILabelFigure
{
    private static final int ROUND_CORNER = 20;

    private ILabel header;

    private IFigure activitiesCompartment;

    private Label entryLbl;

    private Label doLbl;

    private Label exitLbl;

    /**
     * Constructor <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public StateFigure()
    {
        super();

        drawFigure();
    }

    /**
     * Initialize ToolbarLayout and its children Figures Suclasses may overrides this to provides their own background
     * figure.
     */
    protected void drawFigure()
    {
        ToolbarLayout layout = new ToolbarLayout();
        setLayoutManager(layout);
        setOpaque(true);

        header = createLabel();
        add(header);

    }

    private void createActivitiesCompartment()
    {
        activitiesCompartment = new Figure();
        activitiesCompartment.setBorder(new CompoundBorder(new OneLineBorder(1, PositionConstants.TOP), new MarginBorder(5)));
        ToolbarLayout layout = new ToolbarLayout();
        activitiesCompartment.setLayoutManager(layout);
        add(activitiesCompartment);
    }

    public void setDo(String doString)
    {
        if ("".equals(doString))
        {
            removeActivityLabel(doLbl);
            doLbl = null;
        }
        else
        {
            doLbl = updateActivityLabel(doLbl, "do / " + doString, 1);
        }
    }

    public void setExit(String exitString)
    {
        if ("".equals(exitString))
        {
            removeActivityLabel(exitLbl);
            exitLbl = null;
        }
        else
        {
            exitLbl = updateActivityLabel(exitLbl, "exit / " + exitString, 2);
        }
    }

    public void setEntry(String entry)
    {
        if ("".equals(entry))
        {
            removeActivityLabel(entryLbl);
            entryLbl = null;
        }
        else
        {
            entryLbl = updateActivityLabel(entryLbl, "entry / " + entry, 0);
        }
    }

    /**
     * Add a label to the ActivitiesCompartment : <br>
     * - If the compartment does not exist, it is created.<br>
     * - Then if (the label is not already in the list, it is added at the given index (modulo the current list size) 
     * - And finally, the text is updated
     * @param activityLabel the label to update
     * @param label the new text
     * @param index the absolute position
     * @return the updated label
     */
    private Label updateActivityLabel(Label activityLabel, String label, int index)
    {
        if (activitiesCompartment == null)
        {
            createActivitiesCompartment();
        }
        if (activityLabel == null || "".equals(activityLabel.getText()))
        {
            activityLabel = new Label();
            
            int modIndex = 0;
            if (activitiesCompartment.getChildren().size() != 0)
            {
                modIndex = index % activitiesCompartment.getChildren().size();
            }
            activitiesCompartment.add(activityLabel, modIndex);
            
        }
        activityLabel.setText(label);
        
        return activityLabel;
    }

    /**
     * Remove a label from the ActivitiesCompartement
     * @param activityLabel the label to remove
     */
    private void removeActivityLabel(Label activityLabel)
    {
        if (activitiesCompartment != null && activityLabel != null)
        {
            activitiesCompartment.remove(activityLabel);
            updateActivitiesCompartment();
        }
    }

    /**
     * Remove the ActivitiesCompartment if there is no content
     */
    private void updateActivitiesCompartment()
    {
        if (entryLbl == null && doLbl == null && exitLbl == null)
        {
            remove(activitiesCompartment);
            activitiesCompartment = null;
        }
    }

    /**
     * @see org.topcased.draw2d.figures.ILabelFigure#getLabel()
     */
    public ILabel getLabel()
    {
        return header;
    }

    /**
     * Set the label figure
     * 
     * @param newLabel the new label Figure
     */
    protected void setLabel(ILabel newLabel)
    {
        header = newLabel;
    }

    /**
     * @see org.topcased.draw2d.figures.BorderedLabel#fillShape(org.eclipse.draw2d.Graphics)
     */
    protected void fillShape(Graphics graphics)
    {
        graphics.fillRoundRectangle(getBounds(), ROUND_CORNER, ROUND_CORNER);
    }

    /**
     * @see org.topcased.draw2d.figures.BorderedLabel#outlineShape(org.eclipse.draw2d.Graphics)
     */
    protected void outlineShape(Graphics graphics)
    {
        Rectangle r = getBounds();

        Rectangle outer = Rectangle.SINGLETON;
        outer.x = r.x + lineWidth / 2;
        outer.y = r.y + lineWidth / 2;
        outer.width = r.width - lineWidth;
        outer.height = r.height - lineWidth;

        graphics.drawRoundRectangle(outer, ROUND_CORNER, ROUND_CORNER);
    }

    /**
     * @see org.topcased.draw2d.figures.BorderedLabel#createLabel()
     */
    protected ILabel createLabel()
    {
        ComposedLabel label = new ComposedLabel(new Label(), new EditableLabel(), null, false);
        return label;
    }
}