/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram.internal.properties.sections;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.uml2.uml.Transition;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.uml.properties.sections.AbstractConstraintSection;

/**
 * The Section used to edit the Guard Constraint associated with a Transition
 * 
 * Creation 10 nov. 06
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class TransitionGuardSection extends AbstractConstraintSection
{
    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getFeature()
     */
    protected EStructuralFeature getFeature()
    {
        return UMLPackage.eINSTANCE.getTransition_Guard();
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getLabelText()
     */
    protected String getLabelText()
    {
        return "Guard:";
    }

    /**
     * @see org.topcased.modeler.uml.properties.sections.AbstractConstraintSection#getFeatureAsText()
     */
    protected String getFeatureAsText()
    {
        if (((Transition) getEObject()).getGuard() != null)
        {
            String featureTextValue = ((Transition) getEObject()).getGuard().getName();
            return featureTextValue == null ? "" : featureTextValue;
        }
        return "";
    }

    /**
     * @see org.topcased.modeler.uml.properties.sections.AbstractConstraintSection#getRelatedEObject()
     */
    protected EObject getRelatedEObject()
    {
        return ((Transition) getEObject()).getGuard();
    }
}
