package org.topcased.modeler.uml.statemachinediagram.internal.properties.sections;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;
import org.eclipse.uml2.uml.Transition;
import org.eclipse.uml2.uml.Trigger;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.topcased.modeler.editor.properties.TextChangeHelper;
import org.topcased.tabbedproperties.providers.TabbedPropertiesLabelProvider;
import org.topcased.tabbedproperties.sections.AbstractListPropertySection;
import org.topcased.tabbedproperties.sections.widgets.CSingleObjectChooser;

/**
 * TODO : Comment this class
 */
public class TransitionTriggersSection extends AbstractListPropertySection
{

    /** The widgets */
    private Text triggerNameTxt;

    private CSingleObjectChooser eventChooser;

    /**
     * A boolean that store if refreshing is happening and no model modifications should be performed
     */
    private boolean isRefreshing = false;

    /**
     * Groupod detail widgets
     */
    private Group groupDetails;

    // /**
    // * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#createControls(org.eclipse.swt.widgets.Composite, org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage)
    // */
    // public void createControls(Composite parent, TabbedPropertySheetPage aTabbedPropertySheetPage)
    // {
    // super.createControls(parent, aTabbedPropertySheetPage);
    //
    // Composite composite = getSectionComposite();
    // FormData data;
    //
    // Group groupDetails = getWidgetFactory().createGroup(composite, "Trigger Details");
    // groupDetails.setLayout(new GridLayout());
    //
    // data = new FormData();
    // data.left = new FormAttachment(0, 0);
    // data.right = new FormAttachment(100, 0);
    // data.top = new FormAttachment(getTable(), ITabbedPropertyConstants.VSPACE);
    // groupDetails.setLayoutData(data);
    //
    // createGroupContents(groupDetails);
    // hookListeners();
    // }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractListPropertySection#createWidgets(org.eclipse.swt.widgets.Composite)
     */
    protected void createWidgets(Composite composite)
    {
        super.createWidgets(composite);
        groupDetails = getWidgetFactory().createGroup(composite, "Trigger Details");
        groupDetails.setLayout(new GridLayout());

        createGroupContents(groupDetails);
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractListPropertySection#setSectionData(org.eclipse.swt.widgets.Composite)
     */
    protected void setSectionData(Composite composite)
    {
        super.setSectionData(composite);
        FormData data = new FormData();
        data.left = new FormAttachment(0, 0);
        data.right = new FormAttachment(100, 0);
        data.top = new FormAttachment(getTable(), ITabbedPropertyConstants.VSPACE);
        groupDetails.setLayoutData(data);
    }

    /**
     * Create the group contents
     *
     * @param parent the parent Composite
     */
    protected void createGroupContents(Composite parent)
    {
        TabbedPropertySheetWidgetFactory widgetFactory = getWidgetFactory();
        Composite composite = widgetFactory.createComposite(parent);
        composite.setLayout(new GridLayout(2, false));
        composite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        widgetFactory.createLabel(composite, "Name : ");
        triggerNameTxt = widgetFactory.createText(composite, "", SWT.BORDER);
        triggerNameTxt.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        widgetFactory.createLabel(composite, "Event : ");
        eventChooser = new CSingleObjectChooser(composite, getWidgetFactory(), SWT.NONE);
        eventChooser.setLabelProvider((ILabelProvider) getLabelProvider());
        eventChooser.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractListPropertySection#hookListeners()
     */
    protected void hookListeners()
    {
        // Add the Listener for the trigger name
        TextChangeHelper parameterNameListener = new TextChangeHelper()
        {
            public void textChanged(Control control)
            {
                String newText = triggerNameTxt.getText();
                Trigger trigger = (Trigger) getTable().getSelectionItem();
                if (trigger != null && !newText.equals(trigger.getName()))
                {
                    getEditingDomain().getCommandStack().execute(SetCommand.create(getEditingDomain(), trigger, UMLPackage.eINSTANCE.getNamedElement_Name(), newText));
                }
                refresh();
            }
        };
        parameterNameListener.startListeningTo(triggerNameTxt);
        parameterNameListener.startListeningForEnter(triggerNameTxt);

        // Add a listener to the trigger event field
        eventChooser.addSelectionListener(new SelectionAdapter()
        {
            /**
             * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
             */
            public void widgetSelected(SelectionEvent e)
            {
                handleComboModified();
            }
        });

        // Adds a listener that enables the trigger detail fields
        // while a trigger is selected in the list
        getTable().addSelectionListener(new SelectionAdapter()
        {
            @Override
            public void widgetSelected(SelectionEvent e)
            {
                refresh();
            }
        });
    }

    /**
     * This method is called when the combo value is modified
     */
    protected void handleComboModified()
    {
        if (!isRefreshing && getTriggerEvent() != eventChooser.getSelection())
        {
            EditingDomain editingDomain = getEditingDomain();
            Trigger trigger = (Trigger) getTable().getSelectionItem();
            if (getEObjectList().size() == 1)
            {
                /* apply the property change to single selected object */
                editingDomain.getCommandStack().execute(SetCommand.create(editingDomain, trigger, UMLPackage.eINSTANCE.getTrigger_Event(), eventChooser.getSelection()));
            }
            else
            {
                CompoundCommand compoundCommand = new CompoundCommand();
                /* apply the property change to all selected elements */
                for (Iterator<EObject> i = getEObjectList().iterator(); i.hasNext();)
                {
                    EObject nextObject = i.next();
                    compoundCommand.append(SetCommand.create(editingDomain, nextObject, UMLPackage.eINSTANCE.getTrigger_Event(), eventChooser.getSelection()));
                }
                editingDomain.getCommandStack().execute(compoundCommand);
            }
        }

    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractListPropertySection#getLabelProvider()
     */
    protected IBaseLabelProvider getLabelProvider()
    {
        return new TabbedPropertiesLabelProvider(new UMLItemProviderAdapterFactory());
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractListPropertySection#getListValues()
     */
    protected Object getListValues()
    {
        return ((Transition) getEObject()).getTriggers();
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getFeature()
     */
    protected EStructuralFeature getFeature()
    {
        return UMLPackage.eINSTANCE.getTransition_Trigger();
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractTabbedPropertySection#getLabelText()
     */
    protected String getLabelText()
    {
        return "Triggers";
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.ISection#refresh()
     */
    public void refresh()
    {
        isRefreshing = true;
        super.refresh();
        eventChooser.setChoices(getComboFeatureValues());
        eventChooser.setSelection(getTriggerEvent());
        Trigger trigger = (Trigger) getTable().getSelectionItem();
        String nameToDisplay = getTriggerName(trigger);
        triggerNameTxt.setText(nameToDisplay);
        boolean enabled = getTable().getTable().getSelection().length > 0;
        triggerNameTxt.setEnabled(enabled);
        eventChooser.setEnabled(enabled);
        isRefreshing = false;
    }

    /**
     * @see org.topcased.tabbedproperties.sections.AbstractListPropertySection#updateSelection(java.lang.Object)
     */
    public void updateSelection(Object data)
    {
        if (data instanceof Trigger)
        {
            Trigger trigger = (Trigger) data;
            String nameToDisplay = getTriggerName(trigger);
            triggerNameTxt.setText(nameToDisplay);
            eventChooser.setSelection(getTriggerEvent());
        }
    }

    /**
     * Returns the name of the given trigger
     *
     * @param trigger The trigger to know the name
     * @return the triggers name
     */
    private String getTriggerName(Trigger trigger)
    {
        return trigger == null ? "" : trigger.getName() != null ? trigger.getName() : "";
    }

    /**
     * Search for all reachable events in the model
     *
     * @return An array ob events
     */
    private Object[] getComboFeatureValues()
    {
        return getChoices(UMLPackage.eINSTANCE.getEvent());
    }

    /**
     * Returns the trigger event
     *
     * @return the trigger event
     */
    private Object getTriggerEvent()
    {
        Trigger trigger = (Trigger) getTable().getSelectionItem();
        if (trigger != null)
        {
            return trigger.getEvent();
        }
        return null;
    }

    /**
     * Returns an array of all reachable objects of a given type from the current selection.
     *
     * @param type Reachable object which have this type
     * @return An array of objects of the given type
     */
    private Object[] getChoices(EClassifier type)
    {
        List<Object> choices = new ArrayList<Object>();
        choices.add("");
        choices.addAll(ItemPropertyDescriptor.getReachableObjectsOfType(getEObject(), type));

        return choices.toArray();
    }
}
