/*******************************************************************************
 * Copyright (c) 2006 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram.policies;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Pseudostate;
import org.eclipse.uml2.uml.PseudostateKind;
import org.eclipse.uml2.uml.Region;
import org.eclipse.uml2.uml.State;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.uml.statemachinediagram.commands.CreateSubmachineStateCommand;
import org.topcased.modeler.utils.Utils;

/**
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class RegionLayoutEditPolicy extends ModelerLayoutEditPolicy
{
    /**
     * Default contructor.<br>
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public RegionLayoutEditPolicy()
    {
        super();
    }

    /**
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#getCreateCommand(org.topcased.modeler.di.model.GraphNode,
     *      org.topcased.modeler.di.model.GraphNode, org.eclipse.draw2d.geometry.Point,
     *      org.eclipse.draw2d.geometry.Dimension, int, boolean)
     */
    protected Command getCreateCommand(GraphNode parent, GraphNode child, Point loc, Dimension dim, int pos, boolean needModelUpdate)
    {
        if (parent != null && child != null)
        {
            EditDomain domain = getHost().getViewer().getEditDomain();
            ICreationUtils creationUtils = ((ModelerGraphicalViewer) getHost().getViewer()).getModelerEditor().getActiveConfiguration().getCreationUtils();

            // get the EObjects of the model
            EObject parentEObject = Utils.getElement(parent);
            EObject childEObject = Utils.getElement(child);

            if (isValid(childEObject, parentEObject))
            {
                if (childEObject instanceof State && ((State) childEObject).isSubmachineState())
                {
                    return new CreateSubmachineStateCommand(domain, creationUtils, child, parent, loc, dim, pos, needModelUpdate);
                }
                else
                {
                    return super.getCreateCommand(parent, child, loc, dim, pos, needModelUpdate);
                }
            }
        }

        return UnexecutableCommand.INSTANCE;
    }

    /**
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isValid(org.eclipse.emf.ecore.EObject,
     *      org.eclipse.emf.ecore.EObject)
     */
    protected boolean isValid(EObject child, EObject parent)
    {
        return child instanceof State || child instanceof Pseudostate || child instanceof Comment;
    }

    /**
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isAttachedToBorder(org.topcased.modeler.di.model.GraphNode)
     */
    protected boolean isAttachedToBorder(GraphNode node)
    {
        EObject childObject = Utils.getElement(node);

        Region region = (Region) Utils.getElement((GraphElement) getHost().getModel());

        if (region.eContainer() instanceof State && childObject instanceof Pseudostate
                && (PseudostateKind.ENTRY_POINT == ((Pseudostate) childObject).getKind().getValue() || PseudostateKind.EXIT_POINT == ((Pseudostate) childObject).getKind().getValue()))
        {
            return true;
        }
        return super.isAttachedToBorder(node);
    }
}