/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *   Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware Technologies),
 *   Nicolas Lalevée (Anyware Technologies) - initial API and implementation 
 ******************************************************************************/

package org.topcased.modeler.uml.usecasediagram;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Actor;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Extend;
import org.eclipse.uml2.uml.Generalization;
import org.eclipse.uml2.uml.Include;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.UseCase;
import org.eclipse.uml2.uml.util.UMLSwitch;
import org.topcased.modeler.di.model.DiagramInterchangeFactory;
import org.topcased.modeler.di.model.EdgeObjectOffset;
import org.topcased.modeler.di.model.EdgeObjectUV;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.editor.AbstractCreationUtils;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.uml.alldiagram.ExactUMLSwitch;
import org.topcased.modeler.uml.classdiagram.ClassEdgeObjectConstants;

/**
 * This utility class allows to create a GraphElement associated with a Model Object <!-- begin-user-doc --> <!--
 * end-user-doc -->
 * 
 * @generated
 */
public class UseCaseCreationUtils extends AbstractCreationUtils
{

    private static final int LABEL_OFFSET = 10;

    /**
     * Constructor
     * 
     * @param diagramConf the Diagram Graphical Configuration
     */
    public UseCaseCreationUtils(DiagramGraphConf diagramConf)
    {
        super(diagramConf);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class GraphicSwitch extends ExactUMLSwitch
    {
        /** The presentation of the graphical element */
        private String presentation;

        /**
         * Constructor
         * 
         * @param presentation the presentation of the graphical element
         */
        public GraphicSwitch(String presentation)
        {
            this.presentation = presentation;
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#casePackage(org.eclipse.uml2.uml.Package)
         */
        public Object casePackage(Package object)
        {
            return createGraphElementPackage(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseActor(org.eclipse.uml2.uml.Actor)
         */
        public Object caseActor(Actor object)
        {
            return createGraphElementActor(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseUseCase(org.eclipse.uml2.uml.UseCase)
         */
        public Object caseUseCase(UseCase object)
        {
            return createGraphElementUseCase(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseAssociation(org.eclipse.uml2.uml.Association)
         */
        public Object caseAssociation(Association object)
        {
            return createGraphElementAssociation(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseGeneralization(org.eclipse.uml2.uml.Generalization)
         */
        public Object caseGeneralization(Generalization object)
        {
            return createGraphElementGeneralization(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseInclude(org.eclipse.uml2.uml.Include)
         */
        public Object caseInclude(Include object)
        {
            return createGraphElementInclude(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseExtend(org.eclipse.uml2.uml.Extend)
         */
        public Object caseExtend(Extend object)
        {
            return createGraphElementExtend(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseComment(org.eclipse.uml2.uml.Comment)
         */
        public Object caseComment(Comment object)
        {
            return createGraphElementComment(object, presentation);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         */
        public Object defaultCase(EObject object)
        {
            return null;
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.topcased.modeler.editor.ICreationUtils#createGraphElement(org.eclipse.emf.ecore.EObject,
     *      java.lang.String)
     * @generated
     */
    public GraphElement createGraphElement(EObject obj, String presentation)
    {
        Object graphElt = new GraphicSwitch(presentation).doSwitch(obj);
        if (graphElt instanceof GraphElement)
        {
            return (GraphElement) graphElt;
        }
        
        return null;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementPackage(Package element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementActor(Actor element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementUseCase(UseCase element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated NOT
     */
    protected GraphElement createGraphElementAssociation(Association element, String presentation)
    {
        GraphEdge graphEdge = createGraphEdge(element, presentation);
        
        EdgeObjectUV srccountEdgeObjectUV = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectUV();
        srccountEdgeObjectUV.setId(ClassEdgeObjectConstants.SRCCOUNT_EDGE_OBJECT_ID);
        srccountEdgeObjectUV.setUDistance(LABEL_OFFSET);
        srccountEdgeObjectUV.setVDistance(-LABEL_OFFSET);
        graphEdge.getContained().add(srccountEdgeObjectUV);
        
        EdgeObjectUV targetcountEdgeObjectUV = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectUV();
        targetcountEdgeObjectUV.setId(ClassEdgeObjectConstants.TARGETCOUNT_EDGE_OBJECT_ID);
        targetcountEdgeObjectUV.setUDistance(LABEL_OFFSET);
        targetcountEdgeObjectUV.setVDistance(-LABEL_OFFSET);
        graphEdge.getContained().add(targetcountEdgeObjectUV);

        EdgeObjectOffset nameEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
        nameEdgeObjectOffset.setId(UseCaseEdgeObjectConstants.NAME_EDGE_OBJECT_ID);
        nameEdgeObjectOffset.setOffset(new Dimension(0, 10));
        nameEdgeObjectOffset.setVisible(false);
        graphEdge.getContained().add(nameEdgeObjectOffset);

        EdgeObjectOffset stereotypeEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
        stereotypeEdgeObjectOffset.setId(UseCaseEdgeObjectConstants.STEREOTYPE_EDGE_OBJECT_ID);
        stereotypeEdgeObjectOffset.setOffset(new Dimension(0, -10));
        graphEdge.getContained().add(stereotypeEdgeObjectOffset);

        return graphEdge;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementGeneralization(Generalization element, String presentation)
    {
        GraphEdge graphEdge = createGraphEdge(element, presentation);
        EdgeObjectOffset stereotypeEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
        stereotypeEdgeObjectOffset.setId(UseCaseEdgeObjectConstants.STEREOTYPE_EDGE_OBJECT_ID);
        stereotypeEdgeObjectOffset.setOffset(new Dimension(0, 0));
        graphEdge.getContained().add(stereotypeEdgeObjectOffset);
        return graphEdge;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementInclude(Include element, String presentation)
    {
        GraphEdge graphEdge = createGraphEdge(element, presentation);
        EdgeObjectOffset nameEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
        nameEdgeObjectOffset.setId(UseCaseEdgeObjectConstants.NAME_EDGE_OBJECT_ID);
        nameEdgeObjectOffset.setOffset(new Dimension(0, 0));
        graphEdge.getContained().add(nameEdgeObjectOffset);
        return graphEdge;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementExtend(Extend element, String presentation)
    {
        GraphEdge graphEdge = createGraphEdge(element, presentation);
        EdgeObjectOffset nameEdgeObjectOffset = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectOffset();
        nameEdgeObjectOffset.setId(UseCaseEdgeObjectConstants.NAME_EDGE_OBJECT_ID);
        nameEdgeObjectOffset.setOffset(new Dimension(0, 0));
        graphEdge.getContained().add(nameEdgeObjectOffset);
        EdgeObjectUV extensionEdgeObjectUV = DiagramInterchangeFactory.eINSTANCE.createEdgeObjectUV();
        extensionEdgeObjectUV.setId(UseCaseEdgeObjectConstants.EXTENSION_EDGE_OBJECT_ID);
        extensionEdgeObjectUV.setUDistance(0);
        extensionEdgeObjectUV.setVDistance(0);
        graphEdge.getContained().add(extensionEdgeObjectUV);
        return graphEdge;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the model element
     * @param presentation the presentation of the graphical element
     * @return the complete GraphElement
     * @generated
     */
    protected GraphElement createGraphElementComment(Comment element, String presentation)
    {
        return createGraphNode(element, presentation);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    private class ModelSwitch extends UMLSwitch
    {
        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#casePackage(org.eclipse.uml2.uml.Package)
         */
        public Object casePackage(Package object)
        {
            return createModelObjectPackage(object);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseActor(org.eclipse.uml2.uml.Actor)
         */
        public Object caseActor(Actor object)
        {
            return createModelObjectActor(object);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseUseCase(org.eclipse.uml2.uml.UseCase)
         */
        public Object caseUseCase(UseCase object)
        {
            return createModelObjectUseCase(object);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseAssociation(org.eclipse.uml2.uml.Association)
         */
        public Object caseAssociation(Association object)
        {
            return createModelObjectAssociation(object);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseGeneralization(org.eclipse.uml2.uml.Generalization)
         */
        public Object caseGeneralization(Generalization object)
        {
            return createModelObjectGeneralization(object);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseInclude(org.eclipse.uml2.uml.Include)
         */
        public Object caseInclude(Include object)
        {
            return createModelObjectInclude(object);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseExtend(org.eclipse.uml2.uml.Extend)
         */
        public Object caseExtend(Extend object)
        {
            return createModelObjectExtend(object);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#caseComment(org.eclipse.uml2.uml.Comment)
         */
        public Object caseComment(Comment object)
        {
            return createModelObjectComment(object);
        }

        /**
         * @see org.eclipse.uml2.uml.util.UMLSwitch#defaultCase(org.eclipse.emf.ecore.EObject)
         */
        public Object defaultCase(EObject object)
        {
            return null;
        }
    }

    /**
     * Create the ModelObject with its initial children <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param obj the model object
     * @return the model object with its children
     * @generated
     */
    public EObject createModelObject(EObject obj)
    {
        Object eObject = new ModelSwitch().doSwitch(obj);
        return (EObject) eObject;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the original model element
     * @return the complete Model Object
     * @generated
     */
    protected Package createModelObjectPackage(Package element)
    {
        return element;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the original model element
     * @return the complete Model Object
     * @generated
     */
    protected Actor createModelObjectActor(Actor element)
    {
        return element;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the original model element
     * @return the complete Model Object
     * @generated
     */
    protected UseCase createModelObjectUseCase(UseCase element)
    {
        return element;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the original model element
     * @return the complete Model Object
     * @generated
     */
    protected Association createModelObjectAssociation(Association element)
    {
        return element;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the original model element
     * @return the complete Model Object
     * @generated
     */
    protected Generalization createModelObjectGeneralization(Generalization element)
    {
        return element;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the original model element
     * @return the complete Model Object
     * @generated
     */
    protected Include createModelObjectInclude(Include element)
    {
        return element;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the original model element
     * @return the complete Model Object
     * @generated
     */
    protected Extend createModelObjectExtend(Extend element)
    {
        return element;
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @param element the original model element
     * @return the complete Model Object
     * @generated
     */
    protected Comment createModelObjectComment(Comment element)
    {
        return element;
    }

}
