/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *   Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware Technologies),
 *   Nicolas Lalev�e (Anyware Technologies) - initial API and implementation 
 ******************************************************************************/

package org.topcased.modeler.uml.usecasediagram;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.uml2.uml.Actor;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Extend;
import org.eclipse.uml2.uml.Generalization;
import org.eclipse.uml2.uml.Include;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.UseCase;
import org.eclipse.uml2.uml.util.UMLSwitch;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.edit.EMFGraphEdgeEditPart;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.uml.alldiagram.edit.CommentEditPart;
import org.topcased.modeler.uml.alldiagram.edit.CommentLinkEditPart;
import org.topcased.modeler.uml.usecasediagram.edit.ActorEditPart;
import org.topcased.modeler.uml.usecasediagram.edit.AssociationEditPart;
import org.topcased.modeler.uml.usecasediagram.edit.ClassifierEditPart;
import org.topcased.modeler.uml.usecasediagram.edit.ExtendEditPart;
import org.topcased.modeler.uml.usecasediagram.edit.GeneralizationEditPart;
import org.topcased.modeler.uml.usecasediagram.edit.IncludeEditPart;
import org.topcased.modeler.uml.usecasediagram.edit.PackageEditPart;
import org.topcased.modeler.uml.usecasediagram.edit.UseCaseDiagramEditPart;
import org.topcased.modeler.uml.usecasediagram.edit.UseCaseEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * Part Factory : associates a model object to its controller. <br>
 * <!-- begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class UseCaseEditPartFactory implements EditPartFactory
{
    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,java.lang.Object)
     * @generated
     */
    public EditPart createEditPart(EditPart context, Object model)
    {
        if (model instanceof Diagram)
        {
            return new UseCaseDiagramEditPart((Diagram) model);
        }
        else if (model instanceof GraphNode)
        {
            final GraphNode node = (GraphNode) model;
            EObject element = Utils.getElement(node);
            if (element != null)
            {
                Object editPart = new UMLSwitch<Object>()
                {
                    public Object caseClassifier(Classifier object)
                    {
                        return new ClassifierEditPart(node);
                    }

                    public Object casePackage(Package object)
                    {
                        return new PackageEditPart(node);
                    }

                    public Object caseActor(Actor object)
                    {
                        return new ActorEditPart(node);
                    }

                    public Object caseUseCase(UseCase object)
                    {
                        return new UseCaseEditPart(node);
                    }

                    public Object caseComment(Comment object)
                    {
                        return new CommentEditPart(node);
                    }

                    public Object defaultCase(EObject object)
                    {
                        return new EMFGraphNodeEditPart(node);
                    }
                }.doSwitch(element);
                return (EditPart) editPart;
            }

            return new GraphNodeEditPart(node);
        }
        else if (model instanceof GraphEdge)
        {
            final GraphEdge edge = (GraphEdge) model;
            EObject element = Utils.getElement(edge);
            if (element != null)
            {
                Object editPart = new UMLSwitch<Object>()
                {
                    public Object caseExtend(Extend object)
                    {
                        return new ExtendEditPart(edge);
                    }

                    public Object caseInclude(Include object)
                    {
                        return new IncludeEditPart(edge);
                    }

                    public Object caseAssociation(Association object)
                    {
                        return new AssociationEditPart(edge);
                    }

                    public Object caseGeneralization(Generalization object)
                    {
                        return new GeneralizationEditPart(edge);
                    }

                    public Object defaultCase(EObject object)
                    {
                        return new EMFGraphEdgeEditPart(edge);
                    }
                }.doSwitch(element);
                return (EditPart) editPart;
            }
            if (edge.getSemanticModel() instanceof SimpleSemanticModelElement)
            {
                if (((SimpleSemanticModelElement) edge.getSemanticModel()).getTypeInfo().equals(UseCaseSimpleObjectConstants.SIMPLE_OBJECT_COMMENTLINK))
                {
                    return new CommentLinkEditPart(edge);
                }
            }
            return new GraphEdgeEditPart((GraphEdge) model);
        }

        throw new IllegalStateException("No edit part matches with the '" + model.getClass().getName() + "' model element. Check UseCaseEditPartFactory#createEditPart(EditPart,Object) class");
    }

}