/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.utils;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.impl.URIConverterImpl;

/**
 * <br>
 * creation : 28 juin 2005 Extension of URIConverter
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 */
public class ModelerURIConverter extends URIConverterImpl
{
    private static ModelerURIConverter _defaultURIConverter;

    /**
     * Returns the default URI converter that's used when there is no resource
     * set.
     * 
     * @return the default URI converter.
     * @see #getURIConverter
     */
    public static ModelerURIConverter getDefaultURIConverter()
    {
        if (_defaultURIConverter == null)
        {
            _defaultURIConverter = new ModelerURIConverter();
        }
        return _defaultURIConverter;
    }

    /**
     * Return the file that matches with a given uri
     * 
     * @param container file container
     * @param uri
     * @return
     */
    public IFile convertToFile(IContainer container, URI uri)
    {
        URI converted = normalize(uri);
        if (converted.isFile())
        {
            String filePath = converted.toFileString();
            return container.getFile(new Path(filePath));
        }
        else
        {
            String scheme = converted.scheme();
            if (isArchiveScheme(scheme))
            {
                return null;
            }
            else if ("platform".equals(scheme) && converted.segmentCount() > 1
                    && "resource".equals(converted.segment(0)))
            {

                StringBuffer platformResourcePath = new StringBuffer();
                int size = converted.segmentCount();
                for (int i = 2; i < size; ++i)
                {
                    platformResourcePath.append('/');
                    platformResourcePath.append(URI.decode(converted.segment(i)));
                }
                return container.getProject().getFile(new Path(platformResourcePath.toString()));
            }
        }
        return null;
    }

}
