/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * AbstractModelExtensibleSearchParameters.java
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.core.engine;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.search.ui.ISearchPageContainer;

/**
 * Describes parameters for a given model search query. Some of the parameters are common to all kind of queries such as queryText,
 *  modelSearchEngineID, selectedScope, selection. Users can describe arbitrary parameters using a Object map. 
 * <br>
 * <br>
 * <b>Attributes:</b>
 * <dl>
 * <dd><b>queryText</b> : Query text
 * <dd><b>modelSearchEngineID</b> : ID of an existing model search engine registred to 
 * <b>org.topcased.search.core.modelSearchEngine</b> extension point.
 * <dd><b>selectedScope</b> : a selection scope among all ISearchPageContainer possible kinds.
 * <ul>{ <i>WORKSPACE_SCOPE</i> | <i>SELECTION_SCOPE</i> | <i>WORKING_SET_SCOPE</i> | <i>SELECTED_PROJECTS_SCOPE</i>}.</ul>
 * <dd><b>selection</b> : selection with which the search dialog has been opened (IResource, IFile, IProject).
 * <dd><b>data</b> : map of arbitrary specific parameters <i>(Map< key:String, value:Object >)</i>
 * <br>
 * </dl>
 * 
 * <br> 
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public abstract class AbstractModelExtensibleSearchParameters {
	private String queryText;
	private String modelSearchEngineID;
	private String queryKind;
	private List<?> participantElements;
	private ISearchPageContainer container;
	private Map<String, Object> data;
	
	public AbstractModelExtensibleSearchParameters() {
		data = new HashMap<String, Object>();
	}
	public String getModelSearchEngineID() {
		return modelSearchEngineID;
	}
	public void setModelSearchEngineID(String modelSearchEngineID) {
		this.modelSearchEngineID = modelSearchEngineID;
	}
	public String getQueryText() {
		return queryText;
	}
	public void setQueryText(String queryText) {
		this.queryText = queryText;
	}
	public Object getData(String key) {
		return data.get(key);
	}
	public void setData(String key, Object data) {
		this.data.put(key, data);
	}
	public ISearchPageContainer getContainer() {
		return container;
	}
	public void setContainer(ISearchPageContainer c) {
		this.container = c;
	}
	public String getQueryKind() {
		return queryKind;
	}
	public void setQueryKind(String queryKind) {
		this.queryKind = queryKind;
	}
	public List<? extends Object> getParticipantElements() {
		return participantElements;
	}
	public void setParticipantElements(List<? extends Object> filteredElemList) {
		participantElements = filteredElemList;
	}
}
