/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.core.engine;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IResource;

/**
 * Entity responsible of model resource validity checkings.
 * 
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public class AbstractModelResourceValidator implements IModelResourceValidator {
	List<String> modelFileExtensions;
	public AbstractModelResourceValidator() {
		modelFileExtensions = new ArrayList<String>();
	}
	public boolean check(IResource resource) {
		return hasSupportedModelFileExtension(resource);
	}
	public void addModelFileExtension(String extension) {
		modelFileExtensions.add(extension);		
	}
	public List<String> getModelFileExtensions() {
		return modelFileExtensions;		
	}
	public void removeModelFileExtension(String extension) {
		modelFileExtensions.remove(extension);		
	}
	public boolean hasSupportedModelFileExtension(IResource resource) {
		if (resource != null) {
			return modelFileExtensions.contains(resource.getFileExtension());
		}
		return false;
	}
}
