/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.core.engine;

/**
 * Describes all possible textual search queries to be implemented by users.
 * 
 * @author lucas.bigeardel@anyware-tech.com
 */
public abstract class AbstractModelSearchQueryTextualExpressionMatchingHelper implements IModelSearchQueryTextualExpressionMatchingHelper {

	/**
	 * Main search entry point. Triggers the right search given a matching strategy kind as a parameter.
	 */
	public boolean match(String text, String pattern, int matchingStrategy) {
		switch (matchingStrategy) {
			case CASE_SENSITIVE:
				return matchWithCaseSensitiveStrategy(text, pattern);
			case REGULAR_EXPRESSION:
				return matchWithRegularExpressionStrategy(text, pattern);
			case NORMAL_TEXT:
				return matchWithNormalTextStrategy(text, pattern);
		}
		return false;
	}

	/**
	 * Case sensitive textual search
	 * @param text a text to match the pattern into
	 * @param pattern a pattern to match a given text with
	 * @return true if pattern matched into given text, false otherwise
	 */
	abstract boolean matchWithCaseSensitiveStrategy(String text, String pattern);

	/**
	 * Normal textual search
	 * @param text a text to match the pattern into
	 * @param pattern a pattern to match a given text with
	 * @return true if pattern matched into given text, false otherwise
	 */
	abstract boolean matchWithNormalTextStrategy(String text, String pattern);

	/**
	 * regular expression textual search
	 * @param text a text to match the pattern into
	 * @param pattern a pattern to match a given text with
	 * @return true if pattern matched into given text, false otherwise
	 */
	abstract boolean matchWithRegularExpressionStrategy(String text, String pattern);
}
