/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.core.engine;

import java.util.List;

import org.eclipse.core.resources.IResource;

/**
 * This interface defines API contracts to be implemented in order to validate 
 * potential search participant resources.
 * 
 * This interface is subject to modifcations as more complex cheking rules
 * will be introduced in newer versions.
 *  
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public interface IModelResourceValidator {
	/**
	 * Check whether the given IResource is valid or not according to user defined rules.
	 * 
	 * Rules have to be handled in an implementaion of {@link IModelResourceValidator}.
	 * 
	 * @param resource The currently explored 
	 * @return true if the evaluated {@link IResource} is valid, false otherwise
	 */
	public boolean check(IResource resource);
	
	/**
	 * Add model file extension used to discriminate as search participate
	 * @param extension model file extension
	 */
	public void addModelFileExtension(String extension);

	/**
	 * Remove model file extension used to discriminate as search participate
	 * @param extension model file extension
	 */
	public void removeModelFileExtension(String extension);
	
	/**
	 * Get all registred model file extensions.
	 * @return All registred model file extensions
	 */
	public List<String> getModelFileExtensions();
	
	/**
	 * @return true if a resource has a valid Model file extension, false otherwise.
	 */
	public boolean hasSupportedModelFileExtension(IResource resource);
}
