/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.core.engine;

/**
 * Defines contract for textual queries.
 * 
 * <p>
 * There are 3 different textual query mode :
 * <ul>
 * 	<li>CASE_SENSITIVE
 * 	<li>REGULAR_EXPRESSION
 * 	<li>NORMAL_TEXT
 * </ul>
 * 
 * All of these modes cannot be <i>xorable</i> .

 * </p>
 * 
 * <ol>
 * 	<li><h3>CASE_SENSITIVE</h3>
 * 	<p>
 * 		Defines a normal textual query supporting wildcard '?' and '*' features.
 * 	</p>
 * 	<li><h3>REGULAR_EXPRESSION</h3>
 * 	<p>
 * 		Defines a regular expression textual query fully supporting Java 1.5 regex.
 * 	</p>
 * 	<li><h3>NORMAL_TEXT</h3>
 * 	<p>
 * 		Defines a case sensitive textual query supporting wildcard '?' and '*' features.
 * 	</p>
 * </ol>
 * 
 * @author lucas.bigeardel@anyware-technologies.com
 *
 */
public interface IModelSearchQueryTextualExpressionMatchingHelper {
	public final static int CASE_SENSITIVE = 0x00000001;
	public final static int REGULAR_EXPRESSION = 0x00000010;
	public final static int NORMAL_TEXT = 0x00000100;
	
	public boolean match(String text, String pattern, int matchingStrategy);
}
