/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ecore.engine;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;

/**
 * ECore convenience class allowing to collect all instances of supported meta elements from
 * a given metamodel & its supported meta elements.
 * 
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public final class ECoreMetaModelIntrospector {
	/**
	 * ECore convenience method allowing to collect all instances of supported meta elements from
	 * a given meta model reference & its supported meta elements EClass list.
	 * 
	 * @param model An ECore meta model reference 
	 * @param supportedElementsList Supoorted meta element EClass list 
	 * @return All instances of supoorted meta elements, a void list otherwise
	 */
	@SuppressWarnings("unchecked")
	public static List<EClassifier> discriminateValidMetaElements(EObject model, List<EClassifier> supportedElementsList) {
		ArrayList validMetaElementsList = new ArrayList();
		for (EClassifier supportedElement : supportedElementsList) {
			Collection instances = ItemPropertyDescriptor.getReachableObjectsOfType(model, supportedElement);
			for (Object instance : instances) {
				if (!validMetaElementsList.contains(instance)) {
					validMetaElementsList.add(instance);
				}
			}
		}
		return validMetaElementsList;
	}
}
