/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ecore.engine;

import org.eclipse.core.resources.IResource;
import org.topcased.search.core.engine.AbstractModelResourceValidator;

/**
 * Convenience class allowing to check whether a resource is valid or not according 
 * to some basic file extension for exemple.
 * 
 * Users can extend this class and override the check method in order to add additional, 
 * exhaustive & pertinent resource checkings.
 *   
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public class ECoreModelResourceValidator extends AbstractModelResourceValidator {
	public ECoreModelResourceValidator() {
		addModelFileExtension("ecore");
	}	
	
	/**
	 * Check if a resource is valid or not for model element search purpose for the current model kind.
	 */
	@Override
	public boolean check(IResource resource) {
		return hasSupportedModelFileExtension(resource);
	}
}
