/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ecore.engine;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EcorePackage;
import org.topcased.search.ecore.Activator;
import org.topcased.search.ecore.l10n.Messages;

/**
 * 
 * Convenience entity offering services to discriminate model search valid ecore meta elements. 
 * 
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public final class ECoreSupportedElements {
	
	private static List<EClassifier> getECoreEClassifiersLiterals() {
		List<EClassifier> eclassifiersLiteralsList = new ArrayList<EClassifier>();
		try {
			for (Object o : EcorePackage.eINSTANCE.getEClassifiers()) {
				if (o instanceof ENamedElement) {
					eclassifiersLiteralsList.add((EClassifier)o);
				}
			}
		} catch (Throwable t) {
			Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, Messages.getString("ECoreSupportedElements.ecoreWalkingErrorMessage"), t)); //$NON-NLS-1$
		}
		return eclassifiersLiteralsList;
	}

	public static List<EClassifier> getSupportedElements(List<?> participantEClassList) {
		ArrayList<EClassifier> definitiveMetaElementParticipantList = new ArrayList<EClassifier>();
		for (EClassifier eClass : getECoreEClassifiersLiterals()) {
			if (participantEClassList.contains(eClass)) {
				definitiveMetaElementParticipantList.add(eClass);
			}
		}
		return definitiveMetaElementParticipantList;
	}
}
