/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ecore.ui.areas;

import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.topcased.search.ui.areas.IModelSearchQueryArea;
import org.topcased.search.ui.pages.ModelExtensibleSearchPage;

/**
 * Defines UI for ECore models <b>XPath</b> based queries.
 * 
 *  @see <a href="http://www.w3.org/TR/xpath">http://www.w3.org/TR/xpath</a>
 * <br>
 * <br>
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public class ECoreXPathModelSearchQueryArea extends Composite implements IModelSearchQueryArea {
	ModelExtensibleSearchPage searchPage;
	Label searchQueryLabel;
	Text searchQueryText;
	List<String> lastQueries; //TODO: LB to add dialog settings handling
	
	public ECoreXPathModelSearchQueryArea(Composite parent, ModelExtensibleSearchPage page, int style) {
		super(parent, style);
		searchPage = page;
		
		setLayout(new GridLayout(2, false));
		setLayoutData(new GridData(GridData.FILL_BOTH));
		
		GridData gridData = new GridData();
		gridData.horizontalSpan = 2;
		
		searchQueryLabel = new Label(this, SWT.NONE);
		searchQueryLabel.setText("Xpath query:");
		searchQueryLabel.setLayoutData(gridData);
		
		gridData = new GridData(GridData.FILL_BOTH);
		
		searchQueryText = new Text(this, SWT.MULTI|SWT.BORDER);
		searchQueryText.setLayoutData(gridData);
	}
	public Control getControl() {
		return this;
	}
	public Map<String, Object> getData() {
		dataMap.clear();
		return dataMap;
	}
	public String getQuery() {
		return searchQueryText.getText();
	}
	public void loadDialogSettings() {}
	public void storeDialogSettings() {}
	public void prepare() {}
	public boolean validateStatus() {
		return true;
	}
}
