package org.topcased.search.ui.areas;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.provider.EcoreItemProviderAdapterFactory;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.dialogs.ContainerCheckedTreeViewer;
import org.topcased.search.ui.Activator;
import org.topcased.search.ui.pages.ModelExtensibleSearchPage;
import org.topcased.search.ui.providers.AbstractMetaModelParticipantsItemProvider;
import org.topcased.search.ui.sorters.AbstractMetaModelParticipantsViewerSorter;

/**
 * <p>
 * Provide a list of all ECore meta elements particpating to the search.
 * </p>
 * <p>
 * Allows user to select into which meta-element the search query will look into.
 * </p>
 * 
 * 
 * @author lucas.bigeardel@anyware-tech.com
 */
public abstract class AbstractModelSearchParticipantArea extends Composite implements
		IModelSearchParticipantArea {
	protected ContainerCheckedTreeViewer metaElementParticipantCheckedTreeViewer;
	protected ComposeableAdapterFactory metaElementParticipantAdapterFactory;
	protected AbstractMetaModelParticipantsItemProvider metaElementsPartictipantsItemProvider;
	protected ILabelProvider metaElementParticipantLabelProvider;
	private Button metaElementParticipantSelectAllButton, metaElementParticipantDeselectAllButton;
	private Group metaElementParticipantGroup;
	
	// Dialog settings
	private static final String META_ELEMENTS_PARTICIPANT_LIST_SETTINGS_ID = "metaElementsParticipantList";
	private static String modelSearchParticipantsDilaogSettingsFile = "modelSearchParticipantArea.settings";
	
	public AbstractModelSearchParticipantArea(Composite parent, ModelExtensibleSearchPage page, int style) {
		super(parent, style);
		parent.setLayout(new GridLayout());
		parent.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		setLayout(new GridLayout());
		setLayoutData(new GridData(GridData.FILL_BOTH));
		
		metaElementParticipantAdapterFactory = getMetaElementComposeableAdapterFactory();
		metaElementsPartictipantsItemProvider = getMetaModelParticipantsItemProvider();
		metaElementParticipantLabelProvider = new AdapterFactoryLabelProvider(metaElementParticipantAdapterFactory);
		
		metaElementParticipantGroup = new Group(this, SWT.SHADOW_ETCHED_IN);
		metaElementParticipantGroup.setText("UML2 Ecore Meta Elements Search Participants: ");
		metaElementParticipantGroup.setLayout(new GridLayout(2, false));
		metaElementParticipantGroup.setLayoutData(new GridData(GridData.FILL_BOTH));

		metaElementParticipantCheckedTreeViewer = new ContainerCheckedTreeViewer(metaElementParticipantGroup, SWT.BORDER);
		metaElementParticipantCheckedTreeViewer.setContentProvider(metaElementsPartictipantsItemProvider);
		metaElementParticipantCheckedTreeViewer.setLabelProvider(new AdapterFactoryLabelProvider(metaElementParticipantAdapterFactory));
		metaElementParticipantCheckedTreeViewer.setInput("# EVERYTHING DEFINED IN THE PROVIDER ACCORDING AN *XYZ* ECORE METAMODEL #");
		metaElementParticipantCheckedTreeViewer.setAllChecked(true);
		metaElementParticipantCheckedTreeViewer.setSorter(new AbstractMetaModelParticipantsViewerSorter());
		metaElementParticipantCheckedTreeViewer.getTree().setLayout(new GridLayout());
		metaElementParticipantCheckedTreeViewer.getTree().setLayoutData(new GridData(GridData.FILL_BOTH));

		Composite rightPaneContainer = new Composite(metaElementParticipantGroup, SWT.NONE);
		rightPaneContainer.setLayout(new GridLayout());
		rightPaneContainer.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		
		// Select/Deselect All butons
		metaElementParticipantSelectAllButton = new Button(rightPaneContainer, SWT.PUSH);
		metaElementParticipantSelectAllButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		metaElementParticipantSelectAllButton.setText("Select All");
		metaElementParticipantSelectAllButton.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				metaElementParticipantCheckedTreeViewer.setAllChecked(true);
			}
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});
		
		metaElementParticipantDeselectAllButton = new Button(rightPaneContainer, SWT.PUSH);
		metaElementParticipantDeselectAllButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		metaElementParticipantDeselectAllButton.setText("Deselect All");
		metaElementParticipantDeselectAllButton.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				metaElementParticipantCheckedTreeViewer.setAllChecked(false);
			}
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});
	}
	
	/**
	 * Implement at least ECore basic composeable Adapter Factories.
	 * This will be overiden by user. This would allow to avoid void lists ... ^^
	 */
	public List<AdapterFactory> getMetaElementComposeableAdapterFactories() {
		List<AdapterFactory> composeableAdapterFactories = new ArrayList<AdapterFactory>();
		composeableAdapterFactories.add(new EcoreItemProviderAdapterFactory());
		composeableAdapterFactories.add(new ResourceItemProviderAdapterFactory());
		return composeableAdapterFactories;
	}

	/**
	 * Returns current meta-elements participants list.
	 */
	public final List<EClassifier> getSelectedParticipants() {
		ArrayList<EClassifier> eClassList = new ArrayList<EClassifier>();
		for (Object o : metaElementParticipantCheckedTreeViewer.getCheckedElements()) {
			if (o instanceof EClassifier) {
				eClassList.add((EClassifier)o);
			}
		}
		return eClassList;
	}
	
	/**
	 * User wanting to propose a meta-element participant list must implement this method.
	 */
	public AbstractMetaModelParticipantsItemProvider getMetaModelParticipantsItemProvider() {
		return new AbstractMetaModelParticipantsItemProvider();
	}
	
	/**
	 * Composeable Adapter Factory containing a list of pertinent other ECore Adapter Factories according to currenttly introspected model.
	 * 
	 * @return a Composeable Adapter Factory containing a list of pertinent other ECore Adapter Factories
	 * 
	 */
	public final ComposeableAdapterFactory getMetaElementComposeableAdapterFactory() {
		return new ComposedAdapterFactory(getMetaElementComposeableAdapterFactories());
	}
	
	/**
	 * Current area control.
	 */
	public Control getControl() {
		return this;
	}
	
	public Map<String, Object> getData() {
		dataMap.clear();
		return dataMap;
	}
	
	/**
	 * User wanting to access current Model Search Query from participant area should override 
	 * this method, although this doesn't seem very safe architectually speaking ^^
	 * But sometimes, in some special situations, technical needs create some coincidental 
	 * solutions and may also lead to innovation. (Isn't it ?)
	 */
	public String getQuery() {
		return null;
	}
	
	// dialog settings
	public void loadDialogSettings() {
		//Dialog settings for Query Tab
    	IDialogSettings settings = Activator.getDefault().getDialogSettings();
    	
		try {
			settings.load(modelSearchParticipantsDilaogSettingsFile);
		} catch (IOException e) {
			Activator.getDefault().getLog().log(new Status(IStatus.INFO, Activator.PLUGIN_ID, 0, "Impossible to load dialog settings !", e));
		}

        IDialogSettings ecoreTextQuerySectionDialogSettings;
        
        if ((ecoreTextQuerySectionDialogSettings = settings.getSection(getClass().getCanonicalName()))==null) {
        	ecoreTextQuerySectionDialogSettings = settings.addNewSection(getClass().getCanonicalName());
        }
        
        String[] previousSelection = ecoreTextQuerySectionDialogSettings.getArray(META_ELEMENTS_PARTICIPANT_LIST_SETTINGS_ID);
        if (previousSelection != null) {
	    	List<String> lastMetaElementsSelectionList = Arrays.asList(previousSelection);
	    	for (Object o : metaElementsPartictipantsItemProvider.getElements("WHATEVER") ) {
	    		if (o instanceof EClassifier) {
	    			metaElementParticipantCheckedTreeViewer.setChecked(o, lastMetaElementsSelectionList.contains(((EClassifier)o).getClassifierID()+""));
	    		}
	    	}
	    }
	}
	
	public void storeDialogSettings() {
		//Dialog settings for Participant Tab
    	IDialogSettings settings = Activator.getDefault().getDialogSettings();
        IDialogSettings participantListSectionDialogSettings;
        
        if ((participantListSectionDialogSettings = settings.getSection(getClass().getCanonicalName()))==null) {
        	participantListSectionDialogSettings = settings.addNewSection(getClass().getCanonicalName());
        }
        
        List<String> metaElementSelectionCanonicalNamesList = new ArrayList<String>();
        for (Object o : metaElementParticipantCheckedTreeViewer.getCheckedElements()) {
        	if (o instanceof EClassifier) {
        		metaElementSelectionCanonicalNamesList.add(((EClassifier)o).getClassifierID()+"");
        	}
        }
        
    	participantListSectionDialogSettings.put(META_ELEMENTS_PARTICIPANT_LIST_SETTINGS_ID, metaElementSelectionCanonicalNamesList.toArray(new String[0]));
    	
    	try {
			settings.save(modelSearchParticipantsDilaogSettingsFile);
		} catch (IOException e) {
			Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, "Impossible to store dialog settings !", e));
		}
	}
	
	public void prepare() {}
	
	public boolean validateStatus() {
		return true;
	}
}