/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ui.services;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.osgi.framework.Bundle;
import org.topcased.search.ui.areas.IModelElementEditorSelectionHandler;
import org.topcased.search.ui.areas.IModelSearchArea;
import org.topcased.search.ui.areas.IModelSearchAreaFactory;
import org.topcased.search.ui.pages.ModelExtensibleSearchPage;
import org.topcased.search.ui.utils.ModelSearchImages;

/**
 * Encapsulate data comming from modelSearchParticipantTab an extension point contribution.
 * 
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public class ParticipantTabDescriptor {
	public final static String PARTICIPANT_AREA_ID = "id";
	public final static String SEARCH_ENGINE_ID = "searchEngineID";
	public final static String PARTICIPANT_AREA_LABEL_ID = "label";
	public final static String PARTICIPANT_AREA_TOOLTIP_ID = "tooltip";
	public final static String PARTICIPANT_IMAGE_ID = "image";
	public final static String PARTICIPANT_COMPOSITE_AREA_CLASS = "participantModelSearchArea";
	public final static String PARTICIPANT_ELEMENT_EDITOR_SELECTION_HANDLER_CLASS = "elementModelEditorSelectionHandler";
	public final static String PARTICIPANT_ELEMENT_COMPOSEABLE_FACTORY_ADAPTER_CLASS = "elementComposeableAdapterFactory";
	public final static String PARTICIPANT_COMPOSITE_AREA_FACTORY_CLASS = "participantCompositeAreaFactory";
	public final static String PARTICIPANT_REFERENCED_QUERY_TABS_ID = "referencedQueryTabIDs";
	
	public String ID;
	public String label;
	public String tooltip;
	public String searchEngineID;
	public IModelSearchArea participantModelSearchArea;
	public IModelSearchAreaFactory participantCompositeAreaFactory;
	public IModelElementEditorSelectionHandler modelElementEditorSelectionHandler;
	public ComposeableAdapterFactory elementComposeableAdapterFactory;
	public List<String> referencedQueryTabIDs;
	public Image image;
	
	private IConfigurationElement configElement;
	
	public ParticipantTabDescriptor(IConfigurationElement element) throws CoreException {
		super();
		configElement = element;
		referencedQueryTabIDs = new ArrayList<String>();
		load();
	}
	private void load()throws CoreException {
		Bundle bundle = Platform.getBundle(configElement.getNamespaceIdentifier());
		
		ID = configElement.getAttribute(PARTICIPANT_AREA_ID);
		label = configElement.getAttribute(PARTICIPANT_AREA_LABEL_ID);
		tooltip = configElement.getAttribute(PARTICIPANT_AREA_TOOLTIP_ID);
		searchEngineID = configElement.getAttribute(SEARCH_ENGINE_ID);
		image = ModelSearchImages.getImageDescriptor(bundle, configElement.getAttribute(PARTICIPANT_IMAGE_ID)).createImage();
		participantCompositeAreaFactory = (IModelSearchAreaFactory) configElement.createExecutableExtension(PARTICIPANT_COMPOSITE_AREA_FACTORY_CLASS);
		elementComposeableAdapterFactory = (ComposeableAdapterFactory) configElement.createExecutableExtension(PARTICIPANT_ELEMENT_COMPOSEABLE_FACTORY_ADAPTER_CLASS);
		modelElementEditorSelectionHandler = (IModelElementEditorSelectionHandler) configElement.createExecutableExtension(PARTICIPANT_ELEMENT_EDITOR_SELECTION_HANDLER_CLASS);
		IConfigurationElement[] queryTabElements = configElement.getChildren("queryTab");
		
		if (queryTabElements!=null) {
			for (IConfigurationElement queryTabConfigurationElement : queryTabElements) {
				referencedQueryTabIDs.add(queryTabConfigurationElement.getAttribute("id"));
			}
		}
		
        // Sanity check.
        if (ID == null
        		|| searchEngineID == null
                || participantCompositeAreaFactory == null
                || elementComposeableAdapterFactory == null
                || modelElementEditorSelectionHandler == null
                || label == null
                || image == null
                ) 
        {
            throw new CoreException(new Status(IStatus.ERROR, configElement
                    .getContributor().getName(), IStatus.OK,
                    "Invalid extension (missing id, label or class name): " + ID,
                    null));
        }
	}
	public String getID() {
		return ID;
	}
	public IModelSearchAreaFactory getQueryCompositeAreaFactory() {
		return participantCompositeAreaFactory;
	}
	public String getLabel() {
		return label;
	}
	public String getTooltip() {
		return tooltip;
	}
	public String getSearchEngineID() {
		return searchEngineID;
	}
	public List<String> getReferencedQueryTabIDs() {
		return referencedQueryTabIDs;
	}
	public IModelSearchArea createArea(Composite parent, ModelExtensibleSearchPage searchPage) {
		return participantModelSearchArea = participantCompositeAreaFactory.createArea(parent, searchPage);
	}
	public IModelSearchArea getParticipantModelSearchArea() {
		return participantModelSearchArea;
	}
	public ComposeableAdapterFactory getElementComposeableAdapterFactory() {
		return elementComposeableAdapterFactory;
	}
	public IModelElementEditorSelectionHandler getModelElementEditorSelectionHandler() {
		return modelElementEditorSelectionHandler;
	}
	public Image getImage() {
		return image;
	}
}
