/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ui.services;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.osgi.framework.Bundle;
import org.topcased.search.ui.areas.IModelSearchArea;
import org.topcased.search.ui.areas.IModelSearchAreaFactory;
import org.topcased.search.ui.pages.ModelExtensibleSearchPage;
import org.topcased.search.ui.utils.ModelSearchImages;

/**
 * Encapsulate data comming from modelSearchQueryTab an extension point contribution.
 * 
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public class QueryTabDescriptor {
	public final static String QUERY_AREA_ID = "id";
	public final static String QUERY_AREA_LABEL_ID = "label";
	public final static String QUERY_AREA_TOOLTIP_ID = "tooltip";
	public final static String QUERY_COMPOSITE_AREA_CLASS = "queryCompositeArea";
	public final static String QUERY_COMPOSITE_AREA_FACTORY_CLASS = "queryCompositeAreaFactory";
	public final static String QUERY_IMAGE_ID = "image";
	
	public String ID;
	public String label;
	public String tooltip;
	public IModelSearchArea queryModelSearchArea;
	public IModelSearchAreaFactory queryCompositeAreaFactory;
	public Image image;

	private IConfigurationElement configElement;
	
	public QueryTabDescriptor(IConfigurationElement element) throws CoreException {
		super();
		configElement = element;
		load();
	}

	private void load()throws CoreException {
		Bundle bundle = Platform.getBundle(configElement.getNamespaceIdentifier());

		ID = configElement.getAttribute(QUERY_AREA_ID);
		label = configElement.getAttribute(QUERY_AREA_LABEL_ID);
		tooltip = configElement.getAttribute(QUERY_AREA_TOOLTIP_ID);
		queryCompositeAreaFactory = (IModelSearchAreaFactory) configElement.createExecutableExtension(QUERY_COMPOSITE_AREA_FACTORY_CLASS);
		image = ModelSearchImages.getImageDescriptor(bundle, configElement.getAttribute(QUERY_IMAGE_ID)).createImage();

        // Sanity check.
        if (ID == null
                || queryCompositeAreaFactory == null
                || label == null
                || image == null
                ) 
        {
            throw new CoreException(new Status(IStatus.ERROR, configElement
                    .getContributor().getName(), IStatus.OK,
                    "Invalid extension (missing id, label or class name): " + ID,
                    null));
        }
	}
	public String getID() {
		return ID;
	}
	public IModelSearchAreaFactory getQueryCompositeAreaFactory() {
		return queryCompositeAreaFactory;
	}
	public String getLabel() {
		return label;
	}
	public String getTooltip() {
		return tooltip;
	}
	public IModelSearchArea createArea(Composite parent, ModelExtensibleSearchPage searchPage) {
		return queryModelSearchArea = queryCompositeAreaFactory.createArea(parent, searchPage);
	}

	public IModelSearchArea getQueryModelSearchArea() {
		return queryModelSearchArea;
	}

	public Image getImage() {
		return image;
	}
}
