/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ui.utils;

import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.osgi.framework.Bundle;
import org.topcased.search.ui.Activator;

/**
 * A class defining a bundle of all images used by the System Editor plugin.
 * 
 * @author Sbastien Moran : <a
 *         href="mailto:sebastien.moran@anyware-tech.com">sebastien.moran@anyware-tech.com
 *         </a>
 * @author Lucas Bigeardel : <a
 *         href="mailto:sebastien.moran@anyware-tech.com">lucas.bigeardel@anyware-tech.com
 *         </a>
 *         
 */
public class ModelSearchImages {
    public final static String ICONS_PATH = "icons/full/"; //$NON-NLS-1$

    public final static String ERROR_IMAGE_PATH = ICONS_PATH + "error_log.gif"; //$NON-NLS-1$
    static Image ERROR_IMAGE = null;
    
    public final static String FILE_IMAGE_PATH = ICONS_PATH + "esearch.gif"; //$NON-NLS-1$
    static Image FILE_IMAGE = null;
    
    public static ImageDescriptor getImageDescriptor(String imagePath) {
	    ImageDescriptor desc = ImageDescriptor.getMissingImageDescriptor();
	    Bundle bundle = Platform.getBundle(Activator.getDefault().getBundle().getSymbolicName());
	    URL url = null;
	    if (bundle != null){
	        url = FileLocator.find(bundle, new Path(imagePath), null);
	        desc = ImageDescriptor.createFromURL(url);
	    }
	    return desc;
    }
    
    /**
     * Get an ImageDescriptor from a bundle ID and image relative path to the given bundle location
     * @param bundle Given bundle ID wich location will be resolve as root to search images into
     * @param imagePath Relative path to the wanted image
     * @return ImageDescriptor built from the image file, Empty non null ImageDescriptor otherwise
     * @see ImageDescriptor
     */
    public static ImageDescriptor getImageDescriptor(Bundle bundle, String imagePath) {
	    ImageDescriptor desc = ImageDescriptor.getMissingImageDescriptor();
	    URL url = null;
	    if (bundle != null){
	        url = FileLocator.find(bundle, new Path(imagePath), null);
	        desc = ImageDescriptor.createFromURL(url);
	    }
	    return desc;
    }
    
    public static Image getErrorImage() {
    	return ERROR_IMAGE==null?ERROR_IMAGE=getImageDescriptor(ERROR_IMAGE_PATH).createImage():ERROR_IMAGE;
    }
    public static Image getFileImage() {
    	return FILE_IMAGE==null?FILE_IMAGE=getImageDescriptor(FILE_IMAGE_PATH).createImage():FILE_IMAGE;
    }
}