/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ui.utils;

import java.util.Collection;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.topcased.search.core.engine.ModelExtensibleSearchResult;
import org.topcased.search.core.engine.ModelExtensibleSearchResultEntry;

public class ModelSearchResultItemProvider implements ITreeContentProvider {
    private ModelExtensibleSearchResult input;
    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
     */
    public Object[] getChildren(Object parentElement) {
        Object[] result = new Object[0];
        if (parentElement instanceof IFile) {
            Collection < ModelExtensibleSearchResultEntry > item = input.getItems().get((IFile) parentElement);
            result = item.toArray();
        }
        return result;
    }
    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
     */
    public Object getParent(Object element) {
        Object result = null;
        if (element instanceof IFile) {
            result = input;
        } else if (element instanceof ModelExtensibleSearchResultEntry) {
            result = ((ModelExtensibleSearchResultEntry) element).getFile();
        }
        return result;
    }
    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
     */
    public boolean hasChildren(Object element) {
        return element instanceof IFile;
    }
    /**
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement) {
        Object[] result = new Object[0];
        if (inputElement instanceof ModelExtensibleSearchResult) {
        	ModelExtensibleSearchResult input = (ModelExtensibleSearchResult) inputElement;
            Set < IFile > files = input.getItems().keySet();
            result = files.toArray();
        } else if (inputElement instanceof Collection) {
            Collection input = (Collection) inputElement;
            result = input.toArray();
        }
        return result;
    }
    /**
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
     *      java.lang.Object, java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
        if (newInput instanceof ModelExtensibleSearchResult) {
            input = (ModelExtensibleSearchResult) newInput;
        }
    }
	public void dispose() {}
}
