/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.changeuri.services;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;

/**
 * This class defines the "Change URI Service".<br>
 * Three kinds of parameters are expected to run this service :
 * <ul>
 * <li>an input model to parse</li>
 * <li>an output model to create</li>
 * <li>a map containing the couple namespace/namespace value</li>
 * </ul>
 */
public class ChangeURIService implements IService
{
	/**
	 * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{
		if (!parameters.containsKey("InFilePath"))
		{
			CorePlugin.log("The parameter InFilePath is not set.", IStatus.ERROR);
			return null;
		}

		if (!parameters.containsKey("OutFilePath"))
		{
			CorePlugin.log("The parameter OutFilePath is not set.", IStatus.ERROR);
			return null;
		}

		if (!parameters.containsKey("NSURI"))
		{
			CorePlugin.log("The parameter NSURI is not set.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("InFilePath") instanceof String))
		{
			CorePlugin.log("The parameter InFilePath is not a String.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("OutFilePath") instanceof String))
		{
			CorePlugin.log("The parameter OutFilePath is not a String.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("NSURI") instanceof Map))
		{
			CorePlugin.log("The parameter NSURI is not a Map.", IStatus.ERROR);
			return null;
		}

		CorePlugin.log("Entering Change URI Service.", IStatus.INFO);

		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IResource inResource = root.findMember(new Path((String) parameters.get("InFilePath")));
		File inFile;
		if (inResource != null)
		{
			inFile = inResource.getLocation().toFile();
		}
		else
		{
			inFile = new Path((String) parameters.get("InFilePath")).toFile();
		}

		IResource outResource = root.findMember(new Path((String) parameters.get("OutFilePath")).removeLastSegments(1));
		File outFile;
		if (outResource != null)
		{
			outFile = root.getLocation().append((String) parameters.get("OutFilePath")).toFile();
		}
		else
		{
			outFile = new Path((String) parameters.get("OutFilePath")).toFile();
		}

		Map<String, Object> nsuri = (Map<String, Object>) parameters.get("NSURI");
		Set<String> listUri = nsuri.keySet();
		try
		{
			BufferedReader br = new BufferedReader(new FileReader(inFile));
			BufferedWriter bw = new BufferedWriter(new FileWriter(outFile));

			String line = br.readLine();
			while (line != null)
			{
				for (String uri : listUri)
				{
					if (line.indexOf("xmlns:".concat(uri)) != -1)
					{
						int startIndex = line.indexOf('"', line.indexOf("xmlns:" + uri));
						if (startIndex != -1)
						{
							int endIndex = line.indexOf('"', startIndex + 1);
							if (endIndex != -1)
							{
								line = line.substring(0, startIndex + 1) + nsuri.get(uri) + line.substring(endIndex);
							}
						}
					}
				}
				bw.write(line + '\n');
				line = br.readLine();
			}
			br.close();
			bw.close();
		}
		catch (Exception e)
		{
			CorePlugin.log(e);
			return null;
		}

		CorePlugin.log("Leaving Change URI Service.", IStatus.INFO);

		return null;
	}
}
