/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.comparison;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

public class ComparedETreeModel {
	private ComparedETreeModel eparent = null;
	private ObjectComparison compareObj = null;
	private List allAttributes = null;
	private List allContainedSons = new ArrayList();
	 
    public ComparedETreeModel (ComparedETreeModel parent, ObjectComparison newObject, boolean leaf) {
        if (leaf) {
            this.eparent = parent;
            this.compareObj  = newObject;     
        }
    }
    
    
	
	public ComparedETreeModel (ComparedETreeModel parent, ObjectComparison newObject) {
		this.eparent = parent;
		this.compareObj = newObject; 
		if (newObject != null) {
			List sons = newObject.getSons();
			for (Iterator itSons = sons.iterator(); itSons.hasNext();) {
				ObjectComparison obj = (ObjectComparison)itSons.next();
				EObject anObject = obj.getEObject();
				anObject.eClass().getEAllStructuralFeatures();
				Iterator leafs = obj.getEObject().eAllContents();
				if (leafs.hasNext()) {
					allContainedSons.add(new ComparedETreeModel(this,obj,true));
				}
				else {
					// dans ce cas les elements sont decomposables
					allContainedSons.add(new ComparedETreeModel(this,obj));
				}	
			}
		}
		else {
			this.compareObj = new ObjectComparison();
		}
		/* if (this.compareObj.getEObject() != null) {
			allAttributes = this.setAttributes();
			
			for (Iterator it = allAttributes.iterator(); it.hasNext();) {
				EObject currentObj = (EObject) it.next();
				allContainedSons.add(new ComparedETreeModel(this,new ObjectComparison(currentObj,false,false),true));
			}
			allContainedSons.addAll(newObject.getSons());
		}
		allContainedSons.addAll(newObject.getSons());
		*/
	}

	/*private List setAttributes() {
		EList attDescription = compareObj.getEObject().eClass().getEAllAttributes();
		List attModel = new ArrayList();
		for (Iterator it = attDescription.iterator(); it.hasNext();) {
			EAttribute currentObj = ((EAttribute) it.next());
            // valeur : Object value = eobject.eGet(currentObj);
			attModel.add(currentObj);
		}
		return attModel;
	}*/
	
	public List getAttributes() {
		return allAttributes;
	} 
	
	public ObjectComparison getObjectComparison () {
		return this.compareObj;
	}
	
	
	public ComparedETreeModel getParent() {
		return this.eparent;
	}

	public List getSons() {
		return allContainedSons;
		// return this.compareObj.getSons();
	}
	
	public String getTitle() {
		if (compareObj != null) {
			return this.compareObj.getTitle();
		}
		else {
			return "Comparison Results";
		}
	}
}
