/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.comparison;

import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;

public class ComparedETreeModelLabelProvider extends LabelProvider implements ComparisonImageConstantes
{
	/**
	 * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
	 */
	public Image getImage(Object element)
	{
		Image image = null;
		if (element instanceof ComparedETreeModel)
		{
			ComparedETreeModel treeElement = (ComparedETreeModel) element;
			ObjectComparison elementComp = treeElement.getObjectComparison();
			if (elementComp.getEObject() == null)
			{
				image = ComparisonPlugin.getImageDescriptor(COMPARISON).createImage();
				return image;
			}
			if (elementComp.getOrigin())
			{
				if (elementComp.getReason() == ModelComparator.ADDITION)
				{
					image = ComparisonPlugin.getImageDescriptor(ADD_FROM_LEFT).createImage();
				}
				else
				{
					if (elementComp.getReason() == ModelComparator.DELETION)
					{
						image = ComparisonPlugin.getImageDescriptor(DEL_FROM_LEFT).createImage();
					}
					else
					{
						image = ComparisonPlugin.getImageDescriptor(MODIFIED).createImage();
					}
				}
			}
			else
			{
				if (elementComp.getReason() == ModelComparator.ADDITION)
				{
					image = ComparisonPlugin.getImageDescriptor(ADD_FROM_RIGHT).createImage();
				}
				else
				{
					if (elementComp.getReason() == ModelComparator.DELETION)
					{
						image = ComparisonPlugin.getImageDescriptor(DEL_FROM_RIGHT).createImage();
					}
					else
					{
						image = ComparisonPlugin.getImageDescriptor(MODIFIED).createImage();
					}
				}
			}
		}
		return image;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
	 */
	public String getText(Object element)
	{
		if (element == null)
		{
			return "Comparison Results";
		}

		if (element instanceof ComparedETreeModel)
		{
			if (((ComparedETreeModel) element).getParent() == null)
			{
				if (element != null)
				{
					return ((ComparedETreeModel) element).getTitle();
				}
				else
				{
					return "Comparison Results";
				}
			}
		}

		EObject elementComp = null;
		if (element instanceof ComparedETreeModel)
		{
			ObjectComparison castElem = ((ComparedETreeModel) element).getObjectComparison();
			elementComp = castElem.getEObject();
		}

		String name = elementComp.eClass().getName();
		String name2 = new String();

		if (!(elementComp instanceof EAttribute))
		{
			EList<EAttribute> attList = elementComp.eClass().getEAllAttributes();
			boolean endSearch = false;

			Iterator<EAttribute> itAtt = attList.iterator();
			while (itAtt.hasNext() && !endSearch)
			{
				EAttribute att = itAtt.next();
				if (att.getName().compareTo("name") == 0)
				{
					endSearch = !endSearch;
					Object attObj = ((ComparedETreeModel) element).getObjectComparison().getEObject().eGet(att);
					if (!(attObj == null))
					{
						name2 = attObj.toString().concat(":").concat(name);
					}
				}
			}
		}
		else
		{
			if (elementComp instanceof EAttribute)
			{
				EAttribute att = (EAttribute) ((ComparedETreeModel) element).getObjectComparison().getEObject();
				if (att.getName() == null)
				{
					name2 = name2.concat("Unknown");
				}
				else
				{
					String value = new String("\"\"");
					Object attObj;
					try
					{
						attObj = ((ComparedETreeModel) element).getObjectComparison().getEObject().eGet(att, true);
					}
					catch (RuntimeException e)
					{
						attObj = att.getEType().getName();
					}

					if (attObj != null)
					{
						value = attObj.toString();
					}
					name2 = att.getName().concat(":").concat(value);
				}
			}
			else
			{
				((EAttribute) elementComp).getName();
				EAttribute anAttribute = (EAttribute) elementComp;
				name2 = anAttribute.getName().concat(":").concat(anAttribute.getEType().getName());
			}
		}

		if (name2.compareTo("") == 0)
		{
			return (name);
		}
		else
		{
			return name2;
		}
	}

}
