/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.comparison;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.topcased.comparisonresultmodel.ComparisonObject;
import org.topcased.comparisonresultmodel.ComparisonresultmodelFactory;
import org.topcased.comparisonresultmodel.Container;
 
public class ComparisonTransformation
{
  
	public static EObject transform(ObjectComparison root)
	{
		Container c = ComparisonresultmodelFactory.eINSTANCE.createContainer();
		List children = root.getSons();
		for(Iterator ite = children.iterator(); ite.hasNext();)
		{
			ObjectComparison child = (ObjectComparison) ite.next();
			ComparisonObject modelChild = ComparisonresultmodelFactory.eINSTANCE.createComparisonObject();
			modelChild.setLabel(getName(child));
			modelChild.setReason(child.getReason());
			modelChild.setSource(child.getOrigin());
			modelChild.setSourceObject(EcoreUtil.copy(child.getEObject()));
			c.getChildren().add(modelChild);
		}
		return c;
	}
	
	private static String getName(ObjectComparison child)
	{
		//TODO check if this is valid for every element possible
		String name = child.getEObject().eClass().getName();
		
		EList attList = child.getEObject().eClass().getEAllAttributes();
        boolean endSearch = false;

        Iterator it = attList.iterator();
        while (it.hasNext() && !endSearch)
        {
            EAttribute att = (EAttribute) it.next();
            if (att.getName().compareTo("name") == 0) 
            {
                endSearch = !endSearch;
				Object attObj = child.getEObject().eGet(att);
                if (attObj != null) 
                {
                    name = attObj.toString().concat(":").concat(name); 
                }
            }
        }
		return name;
	}
}
