/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/ 
package org.topcased.service.comparison;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.topcased.service.comparison.dialogs.ReferencesDialog;

public class ETreeModel 
{
	private boolean firstModel = false;
	private ETreeModel eparent = null;
	private EObject eobject = null;
	private List allContainedSons = new ArrayList();

	public ETreeModel (ETreeModel parent, EObject newObject, boolean leaf, boolean firstModel)
	{
		if (leaf) 
		{
			this.eparent = parent;
			this.eobject = newObject;        
			this.firstModel = firstModel;
		}
	}


	public ETreeModel (ETreeModel parent, EObject newObject, boolean firstModel)
	{
		List allAttributes = new ArrayList();
		List allOperations = new ArrayList();
		List allReferences = new ArrayList();
		List allAnnotations = new ArrayList();

		this.eparent = parent;
		this.eobject = newObject;
		this.firstModel = firstModel;
		allAttributes = this.setAttributes();
		allAnnotations = this.setAnnotations();
		allReferences = this.setReferredContainedObjects();

		for (Iterator it = allAttributes.iterator(); it.hasNext();)
		{
			EObject currentObj = (EObject) it.next();
			allContainedSons.add(new ETreeModel(this,currentObj,true, this.firstModel));
		}

		addToAllContainedSonsNotLeaf(allOperations);
		addToAllContainedSonsNotLeaf(allAnnotations);
		addToAllContainedSonsNotLeaf(allReferences);
	}

	private void addToAllContainedSonsNotLeaf(List toAdd) 
	{
		for (Iterator it = toAdd.iterator(); it.hasNext();) 
		{
			EObject currentObj = (EObject) it.next();
			allContainedSons.add(new ETreeModel(this,currentObj, this.firstModel));
		}	
	}

	/**
	 * 
	 * @param toAdd the ETreeModel to add
	 * @return true if the ETreeModel to add, has been added, false otherwise
	 */
	public boolean addToAllContainedSons(ETreeModel toAdd)
	{
		boolean addition = allContainedSons.add(toAdd.getSons()); 
		return addition;
	}

	/**
	 * 
	 * @param toSub the ETreeModel to remove
	 * @return true if the ETreeModel has been removed, false otherwise
	 */
	public boolean subToAllContainedSons(ETreeModel toSub)
	{	
		// supprimer l'objet dans son containment
		boolean sub = allContainedSons.removeAll(toSub.getSons()); 
		return sub;
	}

	/**
	 * 
	 * @return : List of EAttributes of the compared model
	 */
	private List setAttributes() 
	{
		List attModel = new ArrayList();
		if (eobject != null) 
		{
			EList attDescription = eobject.eClass().getEAllAttributes();
			for (Iterator it = attDescription.iterator(); it.hasNext();) 
			{
				EAttribute currentObj = ((EAttribute) it.next());
				attModel.add(currentObj);
			}
		}
		return attModel;
	}

	/**
	 * 
	 * @return : List of EAnnotations of the compared model
	 */
	private List setAnnotations() 
	{
		List attModel = new ArrayList();
		if (eobject != null) 
		{
			EList attListAnnotations = eobject.eClass().getEAnnotations();

			for (Iterator it = attListAnnotations.iterator(); it.hasNext();) 
			{
				EAnnotation currentObj = ((EAnnotation) it.next());
				attModel.add(currentObj);
			}
		}
		return attModel;
	}


	private List setReferredContainedObjects() 
	{
		List refModel = new ArrayList();

		if (eobject != null) 
		{
			EList refDescription = eobject.eClass().getEAllReferences();

			for (Iterator it = refDescription.iterator(); it.hasNext();) 
			{
				EReference currentObj = (EReference) it.next();
				if (currentObj.isContainment())
				{
					// rcupre une liste d'objets lis par reference 
					Object objectList = eobject.eGet(currentObj,true);
					if (objectList != null)
					{
						if (objectList instanceof EList) 
						{
							for (Iterator it1 = ((EList)objectList).iterator(); it1.hasNext();) 
							{
								EObject addObject = (EObject)it1.next();
								refModel.add(addObject);
							}
						}
						else 
						{
							refModel.add((EObject)objectList);
						}
					}
				}
			}
		}
		return refModel;
	}

	public List getSons() 
	{
		return allContainedSons;
	}

	public EObject getEobject() 
	{
		return this.eobject;
	}

	public ETreeModel getParent()
	{
		return this.eparent;
	}

	public boolean getFirstModel() 
	{
		return this.firstModel;
	}

	public boolean hasParent(ETreeModel element)
	{
		return (element.getParent() != null); 
	}

	public void setEParent(ETreeModel parent) 
	{
		this.eparent = parent;
	}

	public boolean setParent(ETreeModel parent)
	{
		boolean returnValue = true;
		EObject oldParent = this.eparent.getEobject();
		EStructuralFeature findContener = this.eobject.eContainmentFeature();

		if (parent == null)
		{
			return false; 
		}
		this.eparent = parent;

		// references of the parent
		EList listRef = parent.getEobject().eClass().getEAllStructuralFeatures();

		Object localChildren = null;

		if (listRef.contains(this.eobject)) 
		{
			EObject objectTempToReplace = (EObject) EcoreUtil.copy(this.eobject);
			EObject object2replace = (EObject) EcoreUtil.copy((EObject)listRef.get(listRef.indexOf(this.eobject)));//(EObject)listRef.get(listRef.indexOf(this.eobject));
			EcoreUtil.replace(object2replace, objectTempToReplace);
			return returnValue;
		}

		if (!listRef.contains(findContener) && findContener instanceof EReference)
		{
			// get the containment references of the parent where to include the object
			EList parentContainers = parent.getEobject().eClass().getEAllContainments();
			EList proposedConteners = new BasicEList();
			// get the supertypes of the element so that the element can be included via containments references
			// on its supertypes
			EList classesSupertypes = this.eobject.eClass().getEAllSuperTypes();
			for (Iterator itConteners = parentContainers.iterator(); itConteners.hasNext();)
			{
				EReference aRef = (EReference)itConteners.next();
				if (classesSupertypes.contains(aRef.getEType())) 
				{
					proposedConteners.add(aRef);
				}
			}
			if (!proposedConteners.isEmpty()) 
			{
				ReferencesDialog referenceDialog = new ReferencesDialog(new Shell(), proposedConteners);
				referenceDialog.open();
				if (referenceDialog.getReturnCode() == ReferencesDialog.OK)
				{
					findContener = referenceDialog.getSelectionReference();
				}
				else 
				{
					MessageDialog.openInformation(new Shell(), "Merge operation rollback", "Operation cancelled.");
					return false;
				}
			}
			else 
			{
				MessageDialog.openError(new Shell(), "Merge operation forbidden", "Operation cancelled.");
				return false;
			}
		}

		if (listRef.contains(findContener))
		{
			localChildren = parent.getEobject().eGet(findContener,true);
			if (localChildren instanceof EList) 
			{
				EList childrenList = (EList)localChildren;
				if (this.getEobject() instanceof EModelElement) 
				{
					String uuidNewObject = ModelComparator.getInstance().getUid(((EModelElement)this.getEobject()));
					if (uuidNewObject != null) 
					{
						int j = 0;
						int rang = -1;
						boolean foundItem = false;
						while (j<childrenList.size() && !foundItem) 
						{
							if (ModelComparator.getInstance().getUid((EModelElement)childrenList.get(j))!= null) 
							{
								if (uuidNewObject.compareTo(ModelComparator.getInstance().getUid((EModelElement)childrenList.get(j))) == 0)
								{
									foundItem =! foundItem;
									rang = j;
								}
							}
							j++;
						}
						if (rang > -1)
						{
							EObject child2replace = (EObject)childrenList.get(rang);
							// Temporary copy to avoid deletion in the source model 
							EObject tempObjectToCopy = EcoreUtil.copy(this.getEobject());
							EcoreUtil.replace(child2replace, tempObjectToCopy);
							findContener.eContainer();
						}
						else 
						{
							childrenList.add(EcoreUtil.copy(this.getEobject()));
						}
					}
					else 
					{
						childrenList.add(this.getEobject());
					}
				}
			}
			else
			{
				try 
				{
					parent.getEobject().eSet(findContener, oldParent.eGet(findContener,true));
				}
				catch (Exception e) 
				{
					MessageDialog.openInformation(new Shell(), "Merge operation rollback", "This feature is unsetable.");
				}
			}
		}
		else 
		{
			//if (findContener != null)
			//{
				MessageDialog.openError(new Shell(), "Merge operation forbidden. End of Merge", "Operation cancelled.");
				return false;
			//}
		}
		return returnValue;
	}

	public String getName() 
	{
		return "NameElement";
	}

	/**
	 * Returns a flat list of all gadgets in this gadget tree.
	 */
	public ETreeModel[] flatten() 
	{
		ArrayList result = new ArrayList();
		result = doFlatten(this);
		return (ETreeModel[])result.toArray(new ETreeModel[result.size()]);
	}

	public ArrayList doFlatten (ETreeModel element) 
	{
		return (ArrayList)this.allContainedSons;
	}

}
