/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.comparison;

import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;

/**
 * 
 * @author clecamus
 */
public class ETreeModelLabelProvider extends LabelProvider implements ComparisonImageConstantes
{

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
	 */
	public Image getImage(Object element)
	{
		ETreeModel castElement = null;
		Image image;

		if (element instanceof ETreeModel)
		{
			castElement = ((ETreeModel) element);
		}
		if (castElement != null)
		{
			if (castElement.getFirstModel())
			{
				image = ComparisonPlugin.getImageDescriptor(OLD_OBJECT).createImage();
			}
			else
			{
				image = ComparisonPlugin.getImageDescriptor(NEW_OBJECT).createImage();
			}
		}
		else
		{ 
			// This case must never happen
			image = ComparisonPlugin.getImageDescriptor(OLD_OBJECT).createImage();
		}

		return image;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
	 */
	public String getText(Object element)
	{
		if (element != null && ((ETreeModel) element).getEobject() == null)
		{
			return "";
		}
		String name = ((ETreeModel) element).getEobject().eClass().getName();
		String name2 = new String();

		if (!(((ETreeModel) element).getEobject() instanceof EAttribute))
		{
			EList<EAttribute> attList = ((ETreeModel) element).getEobject().eClass().getEAllAttributes();
			boolean endSearch = false;

			Iterator<EAttribute> it = attList.iterator();

			while (it.hasNext() && !endSearch)
			{
				EAttribute att = (EAttribute) it.next();
				if (att.getName().compareTo("name") == 0)
				{
					endSearch = !endSearch;
					Object attObj = ((ETreeModel) element).getEobject().eGet(att);
					if (!(attObj == null))
					{
						name2 = attObj.toString().concat(":").concat(name);
					}
					if (name2.length() == 0)
					{
						name2 = "unNamedElement:".concat(name);
					}
				}
			}
		}
		else

		{
			if (((ETreeModel) element).getEobject() instanceof EAttribute)
			{
				EAttribute att = (EAttribute) (((ETreeModel) element).getEobject());
				if (att.getName() == null)
				{
					name2 = name2.concat("Unknown");
				}
				else
				{
					String value = new String("\"\"");

					Object attObj;
					try
					{
						attObj = ((ETreeModel) element).getParent().getEobject().eGet(att, true);
					}
					catch (RuntimeException e)
					{
						attObj = att.getEType().getName();
					}

					if (attObj != null)
					{
						value = attObj.toString();
					}
					name2 = att.getName().concat(":").concat(value);
				}
			}
			else
			{

				// EList attList = ((ETreeModel)element).getEobject().eClass().getEAllAttributes();
				EAttribute anAttribute = (EAttribute) ((ETreeModel) element).getEobject();
				if (!(((ETreeModel) element).getParent().getEobject().eGet(anAttribute) == null))
				{
					name2 = anAttribute.getName() + "=" + ((ETreeModel) element).getParent().getEobject().eGet(anAttribute).toString();
				}
				else
				{
					name2 = anAttribute.getName() + "=null";
				}
			}

		}
		if (name2.compareTo("") == 0)
		{
			return (name);
		}
		else
		{
			return name2;
		}
	}

}
