/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.comparison;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.topcased.bus.core.CorePlugin;
import org.topcased.model.management.UuidHelper;

public class ModelComparator {
	 
	private static ModelComparator comparison = new ModelComparator();
	
	public static final String ADDITION = "Addition";
	public static final String DELETION = "Deletion";
	public static final String MODIFICATION = "Modification";
	
	private EObject oldModel;
	private EObject newModel;
	private ObjectComparison comparisonObj;
	private Map inFirstOnly;
	private Map inSecondOnly;
	
	private Map oldMapObjects;
	private Map newMapObjects;
	private Map comparedObjects;
	private Object[] _pathSelected;
	
	private ModelComparator() {
		oldModel = null;
		newModel = null;
		comparisonObj = new ObjectComparison();
		inFirstOnly = new HashMap();
		inSecondOnly = new HashMap();
		oldMapObjects = new HashMap();
		newMapObjects = new HashMap();
		comparedObjects = new HashMap();
		_pathSelected = new Object[] {null, null};
	}
	
	public static ModelComparator getInstance() {
		return comparison;
	}
	
	public ObjectComparison compareModels(EObject _oldModel, EObject _newModel, IPath pathFile1, IPath pathFile2) {
		// init of comparison params 
		comparisonObj = new ObjectComparison();
		inFirstOnly = new HashMap();
		inSecondOnly = new HashMap();
		oldMapObjects = new HashMap();
		newMapObjects = new HashMap();
		comparedObjects = new HashMap();
		_pathSelected = new Object[] {pathFile1, pathFile2};
		// 
		oldModel = _oldModel;
		newModel = _newModel; 
		oldMapObjects = loadMap(oldModel);
		newMapObjects = loadMap(newModel);
		
//		CorePlugin.log("total old = "+oldMapObjects.size(),IStatus.ERROR);
//		CorePlugin.log("total new = "+newMapObjects.size(),IStatus.ERROR);
		
		for (Iterator gg = newMapObjects.entrySet().iterator();gg.hasNext();) {
			Object key = (Object)gg.next();
            if (oldMapObjects.containsValue(key)) {
            	CorePlugin.log("Object found in the other table",IStatus.ERROR);
            }
        }
//		CorePlugin.log("First side Comparison ...",IStatus.INFO);
		setInFirstModelOnly(comparisonObj, newModel,oldMapObjects);
		
//		CorePlugin.log("Second side Comparison ...",IStatus.INFO);
		setInSecondModelOnly(comparisonObj, oldModel, newMapObjects);
		
//		CorePlugin.log("End of Comparison ...",IStatus.INFO);
		return getModelComparison();
	}
	
	/**
	 * 
	 * @param anObject : the object we need the uid 
	 * @return : the uid of the object as String
	 */
	public String getUid(EObject anObject) {
		UuidHelper uidHelper = new UuidHelper();
		String result= new String();
		if (anObject instanceof EModelElement) {
			String uuidget = uidHelper.getUUID(((EModelElement)anObject));
			if (uuidget == null) {
				result  ="";
			}
		}
		
		if (result.compareTo("")==0) {
			if (result==null || result.compareTo("")==0) {
				URI myUri = EcoreUtil.getURI(anObject);
				if (myUri.hasFragment()) {
					result = myUri.fragment();
				}
				result = Integer.toString(result.hashCode());
			}
		}
		return result;
	}
	
	/**
	 * 
	 * @param _oldModel : the first model selected
	 * @return : Map of old Objects with uid as key
	 */
	private Map loadMap(EObject _aModel) {
		Map mapObjects = new HashMap();
		for (Iterator it = _aModel.eAllContents();it.hasNext();){
			EObject aChild = (EObject)it.next();
			String uid = getUid(aChild);
			mapObjects.put(uid, aChild);
		}
		CorePlugin.log("Number of Elements in the loaded model : "+mapObjects.size(),IStatus.INFO);
		return mapObjects;
	}
	
	
	/**
	 * 
	 * @param secondModel : The second model selected as EObject
	 * @param firstMapModel : the first model selected as Map
	 * @return : a Map with only Eobjets in the first Model
	 */
	private void setInFirstModelOnly(ObjectComparison compObj, EObject secondModel, Map firstMapModel) {
		for (Iterator it = secondModel.eAllContents();it.hasNext();){
			EObject aChild = (EObject)it.next();
			String uid = getUid(aChild);
			if (uid != null && uid.compareTo("NullId")!=0) {
				if (!firstMapModel.containsKey(uid)) { 
					// the object is in the first Model only
					ObjectComparison newObj = new ObjectComparison(aChild,false,ModelComparator.ADDITION);
					compObj.setSons(newObj);
					comparedObjects.put(getUid(aChild),aChild);
					inSecondOnly.put(getUid(aChild),aChild);
//					CorePlugin.log("New Element detected ..."+aChild.eClass().getName(),IStatus.WARNING);
				}
				else 
				{
					// the object exists in both models
					// but atributes and so on may be different. So compare content.
					// If content is different the object is different
					EObject equalsObject = (EObject)firstMapModel.get(uid);
					// compare Attributes
					boolean notTrouve = true;
					notTrouve = compareElements(aChild,equalsObject);
					
					if (!notTrouve) {
						ObjectComparison newObj = new ObjectComparison(aChild,false,ModelComparator.MODIFICATION);
						compObj.setSons(newObj);
						comparedObjects.put(getUid(aChild),aChild);
//						CorePlugin.log("Modified Element detected ..."+aChild.eClass().getName(),IStatus.WARNING);
					}
				}
			} // end If uid null
		}	// end For	
	}
	
	
	
	/** 
	 * 
	 * @param aChild : an EObject
	 * @param aChildAtt : the Attribute of the aChild EObject we want to compare
	 * @param equalsObject : an EObject
	 * @param equalsAtt : the Attribute of the equqlsObject EObject we want to compare
	 * @return
	 */
	private boolean compareAnAttribute (EObject aChild, EAttribute aChildAtt, EObject equalsObject, EAttribute equalsAtt) {
		boolean notTrouve = true;
		if (aChildAtt != null && equalsAtt != null) {
			try {
				Object fromValue = aChild.eGet(aChildAtt,true);
				Object destValue = equalsObject.eGet(equalsAtt,true);
				if (fromValue != null && destValue != null) {
					if (fromValue.toString().compareTo(destValue.toString())!=0) {
						notTrouve = !notTrouve;
//						CorePlugin.log("Different attributes values : "+fromValue.toString()+" and "+destValue.toString(),IStatus.WARNING);
					}
				}
				else {
					if ((fromValue == null && destValue != null) |
							(fromValue != null && destValue == null)) {
						notTrouve = !notTrouve;
						if (fromValue == null) {
//							CorePlugin.log("An attribute value was valued : "+fromValue.toString()+" and is now null",IStatus.WARNING);
						}
						else {
//							CorePlugin.log("An attribute value is null and was valued as : "+destValue.toString(),IStatus.WARNING);
						}
					}
				}
			}
			catch (Exception e) {
//				CorePlugin.log("An attribute is unresolved",IStatus.WARNING);
			}
		}
		return notTrouve;
	}
	/**
	 * 
	 * @param aChild : Examined node in the model
	 * @param equalsObject : Examined node in the opposite Model
	 * @return : boolean if the attributes of these nodes are the same
	 */
	private boolean compareAttributes(EObject aChild, EObject equalsObject) {
		boolean notTrouve = true;
		if (aChild != null && equalsObject != null) {
			// les attributs sont contenus 
			Map featuresMap = new HashMap();
			EList features = equalsObject.eClass().getEAllAttributes();
			
			for (Iterator itFeatures = features.iterator(); itFeatures.hasNext(); ) {
				EAttribute aFeature = (EAttribute)itFeatures.next();
				featuresMap.put(aFeature,aFeature);
			} 
			EList attDescription = aChild.eClass().getEAllAttributes(); 
			if (attDescription.size()!=featuresMap.size()) {
				notTrouve=!notTrouve;
//				CorePlugin.log("Number of Attributes Different: "+attDescription.size()+" and "+featuresMap.size(),IStatus.WARNING);
			}

			Iterator it = attDescription.iterator();
			while (it.hasNext() && notTrouve) {
				EAttribute currentObj = ((EAttribute) it.next());
				if (currentObj != null && (EAttribute)featuresMap.get(currentObj)!= null) {
					notTrouve = compareAnAttribute(aChild, currentObj, equalsObject, (EAttribute)featuresMap.get(currentObj));
					if (notTrouve) {
						notTrouve = compareAnnotations(currentObj,(EAttribute)featuresMap.get(currentObj));
					}
				}
				else {
					if ((currentObj == null && featuresMap.get(getUid(currentObj)) != null) |
							(currentObj != null && featuresMap.get(getUid(currentObj)) == null)) {
//							CorePlugin.log("An Attribute is undefined : "+currentObj.getName()+" and "+((EAttribute)featuresMap.get(currentObj)).getName(),IStatus.WARNING);
						notTrouve = !notTrouve;
					}
				}
			}
		}
		else
		{
			notTrouve = !notTrouve;
//			CorePlugin.log("The container Element of Attributes is undefined",IStatus.ERROR);
		}
		return notTrouve;
	}
	
	/**
	 * 
	 * @param aChild : Examined node in the model
	 * @param equalsObject : Examined node in the opposite Model
	 * @return : boolean if the annotations of these nodes are the same
	 */
	private boolean compareEStringtoMapEntry(EObject aChild, EObject equalsObject) {
		boolean notTrouve = true;
		if ((aChild != null && equalsObject != null) && (aChild instanceof EAnnotation && equalsObject instanceof EAnnotation)){
			EMap featuresMap = ((EAnnotation)equalsObject).getDetails();
			EMap attDescription = ((EAnnotation)aChild).getDetails();

			Iterator it = attDescription.keySet().iterator();
			if (attDescription.size()!=featuresMap.size()) {
				notTrouve=!notTrouve;
//				CorePlugin.log("Number of Entries different ...",IStatus.INFO);
			}
			
		    while (it.hasNext() && notTrouve) {
		        String key=(String)it.next();
		        if (featuresMap.containsKey(key)) {
					if (!featuresMap.containsValue(attDescription.get(key))) {
						notTrouve=!notTrouve;
//						CorePlugin.log("Entry modified ...",IStatus.INFO);
					}
		        }
		        else 
				{
					notTrouve=!notTrouve;
//					CorePlugin.log("Entry inexitent ...",IStatus.INFO);
				}
		    }
		}
		else
		{
			notTrouve = !notTrouve;
//			CorePlugin.log("A container of EStringtoMapEntry is undefined",IStatus.ERROR);
		}
		return notTrouve;
	}
	
	/**
	 * 
	 * @param aChild : Examined node in the model
	 * @param equalsObject : Examined node in the opposite Model
	 * @return : boolean if the annotations of these nodes are the same
	 */
	private boolean compareAnnotations(EObject aChild, EObject equalsObject) {
		
		boolean notTrouve = true;

		if ((aChild != null && equalsObject != null) && (aChild instanceof EModelElement && equalsObject instanceof EModelElement)){
			// les attributs sont contenus 
			Map featuresMap = new HashMap();
			EList features = ((EModelElement)equalsObject).getEAnnotations();
			for (Iterator itFeatures = features.iterator(); itFeatures.hasNext(); ) {
				EAnnotation aFeature = (EAnnotation)itFeatures.next();
				featuresMap.put(aFeature.getSource(),aFeature);
			} 
			EList attDescription = ((EModelElement)aChild).getEAnnotations();
			if (attDescription.size()!=features.size()) {
				notTrouve=!notTrouve;
//				CorePlugin.log("Number of Annotations Different : "+attDescription.size()+" and "+features.size(),IStatus.WARNING);
			}
			Iterator it = attDescription.iterator();
			while (it.hasNext() && notTrouve) {
				EAnnotation currentObj = ((EAnnotation)it.next());
				if (currentObj != null && featuresMap.get(currentObj.getSource()) != null) {
					if (currentObj.getSource().compareTo(((EAnnotation)featuresMap.get(currentObj.getSource())).getSource())!=0) {
						notTrouve = !notTrouve;
//						CorePlugin.log("Annotation Source value different : "+currentObj.getSource()+" and "+((EAnnotation)featuresMap.get(currentObj)).getSource(),IStatus.WARNING);
					}
					if (notTrouve) {
						notTrouve = compareReferences(currentObj,(EAnnotation)featuresMap.get(currentObj.getSource()));
					}
					if (notTrouve) {
						notTrouve = compareEStringtoMapEntry(currentObj,(EAnnotation)featuresMap.get(currentObj.getSource()));
					}

				}
				else {
					if ((currentObj == null && featuresMap.get(currentObj.getSource()) != null) |
							(currentObj != null && featuresMap.get(currentObj.getSource()) == null)) {
						if (currentObj == null ) {
//							CorePlugin.log("An Annotation was valued : "+currentObj.toString()+" and is null now.",IStatus.WARNING);
						}
						else {
//							CorePlugin.log("An Annotation is null and was valued as : "+featuresMap.get(currentObj.getSource()),IStatus.WARNING);
						}
						notTrouve = !notTrouve;
					}
				}
			}
		}
		return notTrouve;
	}
	
	
	/**
	 * 
	 * @param aChild : An EClass Object
	 * @param equalsObject : An EClass Object
	 * @return : if EClass are equals
	 */
	private boolean compareEClass(EObject aChild, EObject equalsObject) {
		boolean notTrouve =true; 
		if (aChild != null && equalsObject != null){
				if (getUid(aChild).compareTo(getUid(equalsObject))==0) {
					if (notTrouve) {
						notTrouve = compareAttributes(aChild, equalsObject);
					}
					if (notTrouve) {
						notTrouve = compareReferences(aChild, equalsObject);
					}
					if (notTrouve) {
						notTrouve = compareAnnotations(aChild, equalsObject);
					}
					if (notTrouve) {
						notTrouve = compareMethods(aChild, equalsObject);
					}
				}
				else
				{
					notTrouve = !notTrouve;
//					CorePlugin.log("Classes ID Different : "+getUid(aChild)+" and "+getUid(equalsObject),IStatus.WARNING);
				}
				return notTrouve;
		}
		else {
			if ((aChild == null && equalsObject != null) |
					(aChild != null && equalsObject == null)) {
				notTrouve = !notTrouve;
//				CorePlugin.log("A Class is undefined",IStatus.ERROR);
			}
		}
		return notTrouve;
	}
	
	
	/**
	 * 
	 * @param aChild : An EReference Object
	 * @param equalsObject : An EReference Object
	 * @return : if EReference are equals
	 */
	private boolean compareReferences(EObject aChild, EObject equalsObject) {
		boolean notTrouve =true;
		if (aChild != null && equalsObject != null){
			// compare only one Reference
			if (aChild instanceof EReference && equalsObject instanceof EReference) {
				if (getUid(aChild).compareTo(getUid(equalsObject))==0) {

					EClassifier aChildType = ((EReference)aChild).getEType();
					EClassifier equalsType = ((EReference)equalsObject).getEType();
					
					if (aChildType != null && equalsType != null) {
						if (getUid(aChildType).compareTo(getUid(equalsType))!=0) {
//							CorePlugin.log("The Type of the reference is Different : "+getUid(aChildType)+" and "+getUid(equalsObject),IStatus.WARNING);
							notTrouve = !notTrouve;
						}
					}
					else {
						if ((aChildType == null && equalsType != null) || (aChildType != null && equalsType == null)) {
							notTrouve = !notTrouve;
//							CorePlugin.log("A Reference is undefined : "+aChildType+" and "+equalsType,IStatus.WARNING);
						}
					}
				}
				else 
				{
					notTrouve = !notTrouve;
//					CorePlugin.log("A Reference is Different : "+getUid(aChild)+" and "+getUid(equalsObject),IStatus.WARNING);				
				}
				return notTrouve;
			}
			else {
				EList refChildList = aChild.eClass().getEReferences();
				EList refEqualsList = equalsObject.eClass().getEReferences();
				Map refEqualsMap = new HashMap();
				for (Iterator it=refEqualsList.iterator(); it.hasNext(); ) {
					EReference aRefEquals = (EReference)it.next();
					refEqualsMap.put(aRefEquals,aRefEquals);
				}
				if (refChildList.size()!=refEqualsList.size()) {
					notTrouve=!notTrouve;
//					CorePlugin.log("Number of References Different : "+refChildList.size()+" and "+refEqualsList.size(),IStatus.WARNING);
				}
				Iterator it1 = refChildList.iterator();
				while (notTrouve && it1.hasNext()) {
					EReference refChild = (EReference)it1.next();
					if (refChild != null && refEqualsMap.containsKey(refChild)) {
						EReference equalsReference = (EReference) refEqualsMap.get(refChild);
						
						if (notTrouve) {
							// Obtient les elements references par la references
							Object valChild = aChild.eCrossReferences();
							Object valEquals = equalsObject.eCrossReferences();

							if (valChild instanceof EList && valEquals instanceof EList) {
								// Conversion de  l'objet en EList
								EList valListChild = (EList)valChild;
								EList valListEquals = (EList)valEquals;
								// Les objets du second objet references sont ranges dans une Map
								Map refMap = new HashMap();
								if (valListChild.size()!=valListEquals.size()) {
									notTrouve=!notTrouve;
//									CorePlugin.log("Number of targetted Element Different for the References : "+valListChild.size()+" and "+valListEquals.size(),IStatus.WARNING);								
								}
								for (Iterator it = valListEquals.iterator(); it.hasNext();) {
									EObject anObject = (EObject)it.next();
									refMap.put(getUid(anObject),anObject);
								}
								// On travaille sur les objets references par le premier objet
								for (Iterator it2 = valListChild.iterator(); it2.hasNext();) {
									EObject aRefChild = (EObject)it2.next();
									// les objets references sont ils les memes
									if (refMap.containsKey(getUid(aRefChild))) {
										EObject aRefEquals = (EObject) refMap.get(getUid(aRefChild));
										// les objets sont != null
										if (aRefChild != null && aRefEquals != null) {
											// les ID sont differents 
											if (getUid(aRefChild).compareTo(getUid(aRefEquals))!=0) {
												notTrouve = ! notTrouve;
//												CorePlugin.log("Referenced Element ID Different : "+getUid(aRefChild)+" and "+getUid(aRefEquals),IStatus.WARNING);								
											}
											else {
												// referenced elements are the same
											}
										}
										else
										{
											if ((aRefChild == null && aRefEquals != null) |
													(aRefChild != null & aRefEquals == null)) {
//												CorePlugin.log("A Reference is undefined.",IStatus.ERROR);								
												notTrouve = !notTrouve;
											}
										}
									}
									else
									{
										notTrouve = !notTrouve;
										if (refMap.get(getUid(aRefChild)) != null) {
//											CorePlugin.log("A Reference is different : "+aRefChild.toString()+" and "+refMap.get(getUid(aRefChild)).toString(),IStatus.WARNING);
										}
										else {
//											CorePlugin.log("A Reference was existant : "+aRefChild.toString()+" and has been deleted",IStatus.WARNING);
										}
									}
								}
							}
							else
							{
								// Compare the EObjects
								if (valChild != null && valEquals != null) {
									if (getUid((EObject)valChild).compareTo(getUid((EObject)valEquals))==0) {
									}
									else {
										notTrouve = !notTrouve;
//										CorePlugin.log("Targetted Object ID different : "+getUid((EObject)valChild)+" and "+getUid((EObject)valEquals).toString(),IStatus.WARNING);								
									}
								}
								else {
									if ((valChild == null && valEquals != null) || (valChild != null && valEquals == null)) {
										notTrouve = !notTrouve;
//										CorePlugin.log("A Targetted Object is undefined",IStatus.WARNING);
									}
								}
							}
						}
						if (notTrouve) {
							notTrouve = compareAnnotations(refChild, equalsReference);
						}			
						
					}
					else 
					{
//						CorePlugin.log("A Reference is missing",IStatus.WARNING);
						notTrouve = !notTrouve;
					}
				}
			}
		}
		else
		{
			if ((aChild == null && equalsObject != null) |
					(aChild != null && equalsObject == null)) {
				notTrouve = !notTrouve;
//				CorePlugin.log("A container of Reference is undefined",IStatus.ERROR);
				}
			
		}
		return notTrouve;
	}
	
	
	/**
	 * 
	 * @param aChild : an object possibly containing parameters or a parameter
	 * @param equalsObject : an object possibly containing parameters or a parameter
	 * @return : true if all parameters are the same
	 */
	private boolean compareParameters(EObject aChild, EObject equalsObject) {
		boolean notTrouve = true;
		// parametres des methodes
		if (aChild != null && equalsObject != null) {
				if (aChild instanceof EOperation && equalsObject instanceof EOperation) {
					EOperation currentObj = (EOperation)aChild;
					EOperation equalsObj = (EOperation)equalsObject;
					EList paramsCurrentObj = currentObj.getEParameters();
					// Extraction of parameters de l'objet distant
					Map paramsEquals = new HashMap();
					for (Iterator it = equalsObj.getEParameters().iterator(); it.hasNext(); ) {
						EParameter aParam = (EParameter)it.next();
						paramsEquals.put(aParam, aParam);
					}
					if (paramsCurrentObj.size()!=paramsEquals.size()) {
//						CorePlugin.log("Number of Parameters Different : "+paramsCurrentObj.size()+" and "+paramsEquals.size(),IStatus.WARNING);
						notTrouve=!notTrouve;
					}
					Iterator itParams = paramsCurrentObj.iterator();
					while (itParams.hasNext() && notTrouve) {
						EParameter aParam = (EParameter)itParams.next();
						if (paramsEquals.containsKey(aParam)) {
							notTrouve = compareAttributes(aParam, (EParameter)paramsEquals.get(aParam));
							if (notTrouve) {
								notTrouve = compareAnnotations(aParam,(EParameter)paramsEquals.get(aParam));
							}
						}
					}
			}
		}
		else
		{
			if ((aChild == null & equalsObject != null) |
					(aChild != null & equalsObject == null)) {
				notTrouve = !notTrouve;
//				CorePlugin.log("A container of parameters is undefined.",IStatus.ERROR);
			}
		}
	return notTrouve;
	}
	
	/**
	 * 
	 * @param aChild : an object possibly containing operations
	 * @param equalsObject : an object possibly containing operations
	 * @return : true if an operation equals another examining its parameters
	 */
	private boolean compareMethods(EObject aChild, EObject equalsObject) {
		boolean notTrouve = true;
		
		if ((aChild != null && equalsObject != null)) {
			Map featuresMap = new HashMap();

			EList features = equalsObject.eClass().getEAllOperations();
			for (Iterator itFeatures = features.iterator(); itFeatures.hasNext(); ) {
				EOperation aFeature = (EOperation)itFeatures.next();
				featuresMap.put(getUid(aFeature),aFeature);
			}
			
			EList attDescription = aChild.eClass().getEAllOperations();
			Iterator it1 = attDescription.iterator();
			if (attDescription.size()!=features.size()) {
				notTrouve=!notTrouve;
//				CorePlugin.log("Number of Operations Different : "+attDescription.size()+" and "+features.size(),IStatus.WARNING);
			}
			while (it1.hasNext() && notTrouve) {
				EOperation currentObj = ((EOperation) it1.next());
				if (featuresMap.containsKey(currentObj)) {
					EOperation method = (EOperation)featuresMap.get(getUid(currentObj));
					if (method != null && currentObj != null)
					{
						notTrouve=compareAttributes(currentObj,method);
						// annotations concernant les methodes
						if (notTrouve) {
							notTrouve = compareParameters(currentObj, method);
						}
						if (notTrouve) {
							notTrouve = compareAnnotations(aChild, equalsObject);
						}
					}
					else {
						if ((method == null && currentObj != null) |
								(method != null & currentObj == null)) {
							notTrouve = !notTrouve;
//							CorePlugin.log("An Operation is undefined",IStatus.WARNING);							
						}
					}
				}
			}
		}
		else {
			if ((aChild == null & equalsObject != null) |
					(aChild != null & equalsObject == null)) {
				notTrouve = !notTrouve;
//				CorePlugin.log("A container of Operations is undefined",IStatus.ERROR);							
			}
		}
		return notTrouve;
	}
	
	
	/**
	 * 
	 * @param firstModel : The first model selected as EObject
	 * @param secondMapModel : the second model selected as Map
	 * @return : a Map with only Eobjets in the second Model
	 */
	private void setInSecondModelOnly(ObjectComparison compObj, EObject firstModel, Map secondMapModel) {
		for (Iterator it = firstModel.eAllContents();it.hasNext();){
			EObject aChild = (EObject)it.next();
			String uid = getUid(aChild);
			Object alreadyDone = comparedObjects.get(getUid(aChild));
			if (alreadyDone == null && uid != null && uid.compareTo("NullId")!=0) {
				if (!secondMapModel.containsKey(uid)) {
					// the object is in the second Model only
					inFirstOnly.put(getUid(aChild),aChild);
					if (comparedObjects.get(getUid(aChild)) == null) {
						ObjectComparison newObj = new ObjectComparison(aChild,true,ModelComparator.DELETION);
						compObj.setSons(newObj);
						comparedObjects.put(getUid(aChild),aChild);
//						CorePlugin.log("New Element detected ..."+aChild.eClass().getName(),IStatus.WARNING);
						}
				}
				else 
				{
					// the object exists in both models. But atributes and so on may be different. So compare content.
					// If content is different the object is different
					EObject equalsObject = (EObject)secondMapModel.get(uid);
					boolean notTrouve = true;
					notTrouve = compareElements(aChild,equalsObject);
					
					if (!notTrouve) 
					{
							ObjectComparison newObj = new ObjectComparison(aChild,true,ModelComparator.MODIFICATION);
							compObj.setSons(newObj);
							comparedObjects.put(getUid(aChild),aChild);
//							CorePlugin.log("Modified Element detected ..."+aChild.eClass().getName(),IStatus.WARNING);
					}
				}
			} // end If uid null
		}		
	}
	
	
	private boolean compareElements(EObject anObject, EObject equalsObject) {
		boolean notTrouve =true;
		
		// Fin Traitement des EObject cas de Ecore Metamodele
		if (anObject.eClass().getName().compareTo(anObject.eClass().getName())==0) {
//			CorePlugin.log("Start of Comparison for : "+anObject.eClass().getName()+" and "+equalsObject.eClass().getName(),IStatus.INFO);
//			CorePlugin.log("Details : "+anObject.toString()+" and "+equalsObject.toString(),IStatus.INFO);
			notTrouve = compareEClass(anObject,equalsObject);
		}
		return notTrouve;
	}
	
	/*
	 * Public Accessors  
	 */
	
	public EObject getFirstModel() {
		return oldModel;
	}
	
	public EObject getSecondModel() {
		return newModel;
	}
	
	public Map getOldMapObjects() {
		return oldMapObjects;
	}
	
	public Map getNewMapObjects() {
		return newMapObjects;
	}
	
	public Map getInFirstOnly() {
		return inFirstOnly;
	}
	
	public Map getInSecondOnly() {
		return inSecondOnly;
	}
	
	public ObjectComparison getModelComparison() {
		return comparisonObj;
	}
	
	public Object[] getPathSelected() {
		return _pathSelected;
	}
	
}
