package org.topcased.service.comparison.dialogs;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

public class ReferencesDialog extends Dialog {
	private Label messageLabel;
	private TableViewer referencesTableViewer;
	private EList proposedReferences = new BasicEList();
	private EReference selectedReference = null;
	
	public ReferencesDialog(Shell parentShell, EList _proposedReferences) {
		super(parentShell);
		proposedReferences.addAll(_proposedReferences);
		setShellStyle(SWT.RESIZE | SWT.MAX | SWT.CLOSE | SWT.SYSTEM_MODAL | SWT.TITLE);
	}
	
	protected void configureShell(Shell shell) {
		
		super.configureShell(shell);
		shell.setText("Merge model Helper");
		shell.setMinimumSize(300,150);
	}
	
	protected Control createDialogArea(Composite parent) {
		GridData layoutData;
		
		Composite topComposite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout(1, false);
		topComposite.setLayout(layout);
		layoutData = new GridData();
		layoutData.grabExcessHorizontalSpace = true;
		layoutData.minimumWidth = 300;
		layoutData.widthHint = SWT.DEFAULT;
		layoutData.minimumHeight = 150;
		layoutData.heightHint = SWT.DEFAULT;
		layoutData.horizontalAlignment = SWT.FILL;
		layoutData.grabExcessVerticalSpace = true;
		layoutData.verticalAlignment = SWT.FILL;
		topComposite.setLayoutData(layoutData);
		
		// Information Message 
		
		messageLabel = new Label(topComposite, SWT.NONE);
		String textInfo = "The object cannot be contained as it was before the Drag & Drop.";
		textInfo = textInfo + "\n New containment possibilities are listed below.";
		textInfo = textInfo + "\n Please select the right containment possibility to attach the drag and dropped element. \n";
		messageLabel.setText(textInfo);
		layoutData = new GridData();
		layoutData.grabExcessHorizontalSpace = true;
		layoutData.minimumWidth = 300;
		layoutData.widthHint = SWT.DEFAULT;
		layoutData.horizontalAlignment = SWT.FILL;
		messageLabel.setLayoutData(layoutData);
		
		// TableViewer
		
		Table referencesTable = new Table(topComposite,SWT.V_SCROLL | SWT.FULL_SELECTION);
		layoutData = new GridData();
		layoutData.minimumWidth = 300;
		layoutData.widthHint = SWT.DEFAULT;
		layoutData.minimumHeight = 150;
		layoutData.heightHint = SWT.DEFAULT;
		layoutData.grabExcessHorizontalSpace = true;
		layoutData.grabExcessVerticalSpace = true;
		layoutData.horizontalSpan = 2;
		layoutData.horizontalAlignment = SWT.FILL;
		layoutData.verticalAlignment = SWT.FILL;
		referencesTable.setLayoutData(layoutData);
		referencesTable.setHeaderVisible(true);
		
		TableColumn referenceNameColumn = new TableColumn(referencesTable,SWT.LEFT);
		referenceNameColumn.setMoveable(false);
		referenceNameColumn.setResizable(true);
		referenceNameColumn.setText("Reference name");
		referenceNameColumn.setWidth(150);
		TableColumn referencedTypeColumn = new TableColumn(referencesTable,SWT.LEFT);
		referencedTypeColumn.setMoveable(false);
		referencedTypeColumn.setResizable(true);
		referencedTypeColumn.setText("Referenced type");
		referencedTypeColumn.setWidth(300);
		referencesTableViewer = new TableViewer(referencesTable);
		referencesTableViewer.setLabelProvider(new ITableLabelProvider(){

			public Image getColumnImage(Object element, int columnIndex) {
				return null;
			}

			public String getColumnText(Object element, int columnIndex) {
				//Query queryElement = (Query) element;
				// Rule rule = (Rule) element;
				EReference reference = (EReference)element;
				switch(columnIndex)
				{
					case 0 :
						return reference.getName(); // ((Context) rule.eContainer()).getContextClass().getName();
					case 1 :
						return reference.getEType().getName(); // rule.getName();
				}
				return "null";
			}

			public void addListener(ILabelProviderListener listener) {
				// Nothing
				
			}

			public void dispose() {
				// Nothing
				
			}

			public boolean isLabelProperty(Object element, String property) {
				// Nothing
				return false;
			}

			public void removeListener(ILabelProviderListener listener) {
				// Nothing
				
			}
			
		});
		referencesTableViewer.add(proposedReferences.toArray());
		return topComposite;
	}

	protected void okPressed() {
		IStructuredSelection selection = (IStructuredSelection)referencesTableViewer.getSelection();
		if (!selection.isEmpty()) {
			Object selectedObject = selection.getFirstElement();
	
			super.okPressed();
			if (selectedObject instanceof EReference) {
				selectedReference = (EReference)selectedObject;
			}
		}
		else
		{
			MessageDialog.openError(new Shell(),"Model structure modification","Please select the right containment and confirm the structure modification.");
		}
	}
 
	public EReference getSelectionReference() {
		return selectedReference; 
	}

	
	
}
