/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/ 
package org.topcased.service.comparison.dnd;

import org.eclipse.emf.ecore.impl.EAttributeImpl;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Tree;
import org.topcased.service.comparison.ETreeModel;

public class DragSourceComparison implements DragSourceListener
{
	private Transfer[] textTransfer = new Transfer[0];

	public void dragStart(DragSourceEvent event)
	{
		event.doit = true;
		Tree source = ((Tree)((DragSource)event.getSource()).getControl());
		SourceTargetFacilities.getInstance().setSourceTree(source);
		SourceTargetFacilities.getInstance().setDestinationTree(source);
		if (source.getSelection()[0].getParentItem() == null)
		{
			event.doit = false;
		}
		
		if ( ((ETreeModel)source.getSelection()[0].getData()).getEobject() instanceof EAttributeImpl)
		{
			event.doit = false;
		}
	}

	/**
	 * Get transfered data
	 */
	public void dragSetData(DragSourceEvent event) 
	{
		Tree source = ((Tree)((DragSource)event.getSource()).getControl());
		event.data = source.getSelection()[0].getText();		
	}

	public void dragFinished(DragSourceEvent event) 
	{
		// Do nothing		
	}

	/**
	 * Specifies the list of data types that can be transferred by this DragSource.
	 * The application must be able to provide data to match each of these types when
	 * a successful drop has occurred.
	 * 
	 * @param transferAgents a list of Transfer objects which define the types of data that can be
	 * dragged from this source
	 */
	public void setTransfer(Transfer[] transferAgents)
	{
		this.textTransfer = transferAgents;
	}

}
