/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/ 
package org.topcased.service.comparison.dnd;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.impl.EStringToStringMapEntryImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.topcased.model.management.IAnnotationConstants;
import org.topcased.service.comparison.ComparisonImageConstantes;
import org.topcased.service.comparison.ComparisonPlugin;
import org.topcased.service.comparison.ETreeModel;
import org.topcased.service.comparison.ModelComparator;

public class DropTargetComparison extends DropTargetAdapter implements ComparisonImageConstantes
{
	private Transfer[] textTransfer;

	public void setTransfer(Transfer[] transferAgents)
	{
		textTransfer = transferAgents;
	}

	/**
	 * dragEnter (DropTargetEvent event)
	 */
	public void dragEnter(DropTargetEvent event)
	{
		if (event.detail == DND.DROP_DEFAULT)
		{
			if ((event.operations & DND.DROP_MOVE) != 0)
			{
				event.detail = DND.DROP_MOVE;
			}
			else
			{
				event.detail = DND.DROP_NONE;
			}
		}
		for (int i = 0; i < event.dataTypes.length; i++)
		{
			if (TextTransfer.getInstance().isSupportedType(event.dataTypes[i]))
			{
				event.currentDataType = event.dataTypes[i];
				// files should only be copied
				if (event.detail != DND.DROP_MOVE)
				{
					event.detail = DND.DROP_NONE;
				}
				break;
			}
		}
	}

	/**
	 * dragLeave(DropTargetEvent event)
	 */
	public void dragLeave(DropTargetEvent event)
	{
		super.dragLeave(event);
	}

	/**
	 * dragOperationChanged(DropTargetEvent event)
	 */
	public void dragOperationChanged(DropTargetEvent event) 
	{
		if (event.detail == DND.DROP_DEFAULT)
		{
			if ((event.detail = (event.operations & DND.DROP_MOVE)) != 0)
			{
				event.detail = DND.DROP_MOVE;
			}
			else
			{
				event.detail = DND.DROP_NONE;
			}
		}
		// allow text to be moved but files should only be copied
		if (TextTransfer.getInstance().isSupportedType(event.currentDataType))
		{
			if (event.detail != DND.DROP_MOVE)
			{
				event.detail = DND.DROP_NONE;
			}
		}
	}

	/**
	 * dragOver(DropTargetEvent event)
	 */
	public void dragOver(DropTargetEvent event)
	{
		TextTransfer textTransfer = TextTransfer.getInstance();
		Tree sourceTree = SourceTargetFacilities.getInstance().getSourceTree();
		// get data to transfer
		textTransfer.nativeToJava(event.currentDataType);
		event.feedback = DND.FEEDBACK_SELECT | DND.FEEDBACK_SCROLL;
		if ((sourceTree != null) && (event.item != null) && !((TreeItem)event.item).getParent().equals(sourceTree)) 
		{
			event.detail = DND.DROP_MOVE;
		}
		else 
		{
			event.detail = DND.DROP_NONE;
		}
	}

	/**
	 * drop(DropTargetEvent event)
	 */
	public void drop(DropTargetEvent event) 
	{
		if (event.item != null)
		{
			// Get the item selected and the targetted item
			TreeItem treeItemSource = SourceTargetFacilities.getInstance().getSourceTree().getSelection()[0];
			TreeItem treeItemTarget = ((TreeItem) event.item);

			// Add a dialog to confirm drop or cancell it
			performDrop(treeItemSource, treeItemTarget);

			// update the trees
			SourceTargetFacilities.getInstance().getSourceTree().update();
			SourceTargetFacilities.getInstance().getDestinationTree().update();
		}
	}

	/**
	 * Performs drop operation : replaces or creates an item
	 * @param treeItemSource : the item to drop
	 * @param treeItemTarget : the item where to include the treeItemSource
	 */
	private void performDrop (TreeItem treeItemSource, TreeItem treeItemTarget) 
	{
		TreeItem destItemToReplace = null;

		int rang = findSourceItemInTargetTree(treeItemSource, treeItemTarget);
		if (rang > -1) 
		{
			destItemToReplace = treeItemTarget.getItems()[rang];
		}
		
		if (treeItemTarget != null && (treeItemTarget.getParentItem() != null || treeItemTarget.getParent() != null))
		{
			// replace the item
			if (destItemToReplace != null) 
			{
				boolean continueDND = setExistentItem (treeItemSource, treeItemTarget, destItemToReplace);
				if (continueDND) 
				{
					treeItemSource.setImage(ComparisonPlugin.getImageDescriptor(MERGE_OBJECT).createImage());
					treeItemTarget.getParent().update();
				}
				else 
				{
					treeItemSource.getParent().update();
					treeItemTarget.getParent().update();
				}
			}
			else 
			{
				// add the new item
				TreeItem treeItemTarget2 = new TreeItem(treeItemTarget, SWT.NONE);
				TreeItem createdItem = createNewItem (treeItemSource, treeItemTarget, treeItemTarget2);
				if (createdItem != null) 
				{
					treeItemSource.setImage(ComparisonPlugin.getImageDescriptor(MERGE_OBJECT).createImage());
				}
				else 
				{
					Tree treeParent = treeItemTarget2.getParent();
					treeItemTarget2.dispose();
					treeParent.update();
				}
			}
		}    		
	}

	public void dropAccept(DropTargetEvent event)
	{
		Tree destinationTree = ((Tree)((DropTarget)event.getSource()).getControl());
		SourceTargetFacilities.getInstance().setDestinationTree(destinationTree);
	}

	/**
	 * Copies an item
	 * @param sourceTree : treeItem from which the treeItems are copied
	 * @param destTree : treeItem that receives the copied treeItems
	 */
	private void copyItems(TreeItem sourceTree, TreeItem destTree)
	{
		TreeItem destItemToCopy = null;
		TreeItem[] children = sourceTree.getItems();

		// destTree cannot be null
		if (destTree != null) 
		{
			for (int i=0; i<children.length; i++) 
			{
				destItemToCopy = null;

				int rang = findSourceItemInTargetTree(children[i], destTree);
				if (rang > -1) 
				{	
					destItemToCopy = destTree.getItems()[rang];
				}

				if (destItemToCopy != null)
				{
					// Preserve the existent item
					setExistentItem(children[i], destTree, destItemToCopy);
					destTree.getParent().update();
				}
				else 
				{
					// Call to createNewItem
					TreeItem treeItemTarget2 = new TreeItem(destTree, SWT.NONE);
					TreeItem createdItem = createNewItem(children[i], destTree, treeItemTarget2);
					if (createdItem!=null) 
					{
						destTree.getItems();
						treeItemTarget2.getParent().update();
					}
					else
					{
						Tree treeParent = destTree.getParent();
						treeItemTarget2.dispose();
						treeParent.update();
					}
				}
			}
		}
		Tree destinationTree = destTree.getParent();
		destinationTree.update();
	}

	/**
	 * Checks that the item exists in a given Tree
	 * @param treeItemSource : treeItem being dragged
	 * @param parentItemTarget : treeItem parent in the new tree
	 * @param destItemToCopy : treeItem that we want to update only 
	 */
	private boolean setExistentItem (TreeItem treeItemSource, TreeItem parentItemTarget, TreeItem destItemToCopy) 
	{
		TreeItem treeItemTarget2 = new TreeItem(parentItemTarget, SWT.NONE); 
		boolean continueDND = false;

		treeItemTarget2.setData(treeItemSource.getData());

		continueDND = ((ETreeModel)treeItemTarget2.getData()).setParent(((ETreeModel)parentItemTarget.getData()));
		if (continueDND) 
		{
			destItemToCopy.removeAll();
			destItemToCopy.setText(treeItemSource.getText());
			destItemToCopy.setImage(treeItemSource.getImage());
			destItemToCopy.setData(treeItemSource.getData());
			if (treeItemSource.getItems().length > 0) 
			{
				copyItems(treeItemSource, destItemToCopy);
			}
			continueDND = ((ETreeModel)destItemToCopy.getData()).setParent(((ETreeModel)parentItemTarget.getData()));	

			Tree destItemParent = treeItemTarget2.getParent();
			treeItemTarget2.dispose();
			destItemParent.update();
		}
		else
		{
			Tree treeParent = parentItemTarget.getParent();
			treeParent.update();
		}
		return continueDND;
	}

	/**
	 * Creates a new Item under another Item  
	 * @param treeItemSource : treeItem being dragged
	 * @param treeItemTarget : treeItem parent in the new tree
	 * @param treeItemToCreate : treeItem to add to parent
	 */
	private TreeItem createNewItem (TreeItem treeItemSource, TreeItem treeItemTarget, TreeItem treeItemToCreate)
	{
		boolean continueDND = false;

		treeItemToCreate.setText(treeItemSource.getText());
		treeItemToCreate.setImage(treeItemSource.getImage());
		treeItemToCreate.setData(treeItemSource.getData());
		continueDND = ((ETreeModel)treeItemToCreate.getData()).setParent(((ETreeModel)treeItemTarget.getData()));
		if (continueDND) 
		{
			// Adds the new item to the parent
			((ETreeModel)treeItemTarget.getData()).addToAllContainedSons(((ETreeModel)treeItemToCreate.getData()));
			// Copy sub Items
			copyItems(treeItemSource, treeItemToCreate);
			treeItemToCreate.getParent().update();
			return treeItemToCreate;
		}
		return null;
	}

	/**
	 * 
	 * @param treeItemSource : treeItem to find
	 * @param treeItemTarget : treeItem to search in
	 * @return : the position of the treeItem in the treeItemTarget else -1
	 */
	private int findSourceItemInTargetTree(TreeItem treeItemSource, TreeItem treeItemTarget)
	{
		TreeItem[]  listItems = treeItemTarget.getItems();
		
		int i = 0;
		int rang = -1;
		boolean notFound = true;
		while (i < listItems.length && notFound)
		{
			if (((ETreeModel)listItems[i].getData()) != null && ((ETreeModel)treeItemSource.getData()) != null) 
			{
				EObject targetData = ((ETreeModel)listItems[i].getData()).getEobject();
				EObject sourceData = ((ETreeModel)treeItemSource.getData()).getEobject();

				// EObject compare
				if (EcoreUtil.equals(sourceData, targetData)) 
				{
					rang = i;
					notFound = false;
				}

				// EAttribute
				if (sourceData instanceof EAttribute)  
				{
					if (targetData instanceof EAttribute) 
					{
						if (((EAttribute)sourceData).getName().compareTo(((EAttribute)targetData).getName()) == 0) 
						{
							rang = i;
							notFound = false;
						}
					}
				}

				// EStringtoString...
				if (sourceData instanceof EStringToStringMapEntryImpl && targetData instanceof EStringToStringMapEntryImpl) 
				{
					EStringToStringMapEntryImpl eStringSource = (EStringToStringMapEntryImpl) sourceData;
					EStringToStringMapEntryImpl eStringTarget = (EStringToStringMapEntryImpl) targetData;
					if (eStringSource.getValue().equals(eStringTarget.getValue())) 
					{
						rang = i;
						notFound = false;
					}
				}

				// EModelElement with UUID comes from Topcased saves.
				if (sourceData instanceof EModelElement && targetData instanceof EModelElement) 
				{

					if (!((EModelElement)sourceData).getEAnnotations().isEmpty() && 
							!((EModelElement)targetData).getEAnnotations().isEmpty() &&	
							((EModelElement)sourceData).getEAnnotation(IAnnotationConstants.UUID_SOURCE) != null &&
							((EModelElement)targetData).getEAnnotation(IAnnotationConstants.UUID_SOURCE) != null )
					{
						String uuidOldParent = new String(ModelComparator.getInstance().getUid(((EModelElement)targetData)));
						String uuidThis = new String(ModelComparator.getInstance().getUid(((EModelElement)sourceData)));
						if (uuidOldParent.compareTo(uuidThis) == 0) 
						{
							rang = i;
							notFound = false;
						}
					}
				}

				// The element has no UUID so ask for UUID for each element 
				if (sourceData.getClass().equals(targetData.getClass())) 
				{ 
					String sourceUid = ModelComparator.getInstance().getUid(sourceData);
					String targetUid = ModelComparator.getInstance().getUid(targetData);
					if (sourceUid.compareTo(targetUid) == 0) 
					{
						rang = i;
						notFound = false;
					}
				}
			}
			i++;
		}
		return rang;
	}
}
