/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.comparison.menuAction;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.Service;
import org.topcased.bus.core.ServicesManager;
import org.topcased.service.comparison.views.ComparisonView;

public class ModelCompareServiceAction implements IActionDelegate
{
	private Object[] _pathSelected;

	/**
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action)
	{

		Shell aShell = new Shell(SWT.SYSTEM_MODAL | SWT.SHELL_TRIM);
		// retrieve the Comparison service among Topcased services
		Map<String, Service> services = ServicesManager.getInstance().getAllServices();
		Map<String, Service> services2 = ServicesManager.getInstance().getServicesPerType("Comparison", services);
		// Declare the parameters service map
		Map<String, Object> mapParams = new HashMap<String, Object>();
		mapParams.put("model1Path", _pathSelected[0]);
		mapParams.put("model2Path", _pathSelected[1]);
		// call the comparison service
		Service s = null;
		for (String serviceName : services2.keySet())
		{
			s = services2.get(serviceName);
		}
		
		// call the comparison service
		IService is = ServicesManager.getInstance().getService(s.getId());
		is.serviceRun(mapParams);

		// display the result view
		try
		{
			IViewPart view = getPage().findView("org.topcased.service.comparison.views.ComparisonView");
			if (view == null)
			{
				view = getPage().showView("org.topcased.service.comparison.views.ComparisonView", null, IWorkbenchPage.VIEW_CREATE);
				this.getPage().activate(view);
			}
			else
			{
				ComparisonView comparisonView = (ComparisonView) view;
				comparisonView.showBusy(true);
				comparisonView.setData();
				comparisonView.showBusy(false);
				view = getPage().showView("org.topcased.service.comparison.views.ComparisonView");
				this.getPage().activate(view);
				// Attention appel  faire
				// comparisonView.selectionChanged(comparisonView.getSite().getPart(),_selection);
			}
		}
		catch (PartInitException e)
		{
			CorePlugin.log("View Creation Exception !", IStatus.ERROR);
			MessageDialog.openError(aShell, "View Creation Error", "The View cannot be created !");
			CorePlugin.log(e);
		}
	}

	private IWorkbenchPage getPage()
	{
		IWorkbench workbench = PlatformUI.getWorkbench();
		IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
		return window.getActivePage();
	}

	/**
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection)
	{
		if (selection instanceof IStructuredSelection)
		{
			IStructuredSelection mySelection = (IStructuredSelection) selection;
			if (mySelection.size() == 1)
			{
				Object selectedObject = mySelection.getFirstElement();
				if (selectedObject instanceof IFile)
				{
					_pathSelected = new Object[] { ((IFile) selectedObject).getLocation() };

				}
				else if (selectedObject instanceof IFile)
				{
					Object adapted = ((IAdaptable) selectedObject).getAdapter(IFile.class);
					if (adapted != null)
					{
						_pathSelected = new Object[] { ((IFile) selectedObject).getLocation() };
					}
				}
			}
			else if (mySelection.size() == 2)
			{
				Object[] selectedObjects = mySelection.toArray();
				if (selectedObjects[0] instanceof IFile && selectedObjects[1] instanceof IFile)
				{
					_pathSelected = new Object[] { ((IFile) selectedObjects[0]).getLocation(), ((IFile) selectedObjects[1]).getLocation() };
				}
			}
		}
	}

}
