/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.comparison.service;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.Service;
import org.topcased.bus.core.ServicesManager;
import org.topcased.service.comparison.ModelComparator;
import org.topcased.service.comparison.ObjectComparison;

public class ModelCompareService implements IService
{
	public Object serviceRun(Map<String, Object> parameters)
	{
		// Declaration bloc
		EObject model1 = null;
		EObject model2 = null;
		IPath pathFile1 = null;
		IPath pathFile2 = null;
		CorePlugin.log(this.getClass(), "Starting model comparison service.", IStatus.OK);
		CorePlugin.log(this.getClass(), "Asking services from the bus.", IStatus.INFO);

		Map<String, Service> services = ServicesManager.getInstance().getAllServices();
		Map<String, Service> services2 = ServicesManager.getInstance().getServicesPerType("Read", services);
		Map<String, Service> services3 = ServicesManager.getInstance().getServicesPerType("Write", services);
		Map<String, Object> param = new HashMap<String, Object>();
		Shell aShell = new Shell(SWT.SYSTEM_MODAL | SWT.SHELL_TRIM);

		// Map null
		if (parameters == null)
		{
			CorePlugin.log(this.getClass(), "Parameters list is null. It must have exactly two parameter", IStatus.ERROR);
			MessageDialog.openError(aShell, "View Initialization Error", "Parameters list is null. It must have exactly two parameters");
			return "Parameters list is null. It must have exactly two parameters";
		}

		if (parameters.size() != 2)
		{
			CorePlugin.log(this.getClass(), "Parameters number is not correct. It must have exactly two parameters : 'Path to models'",
					IStatus.ERROR);
			MessageDialog.openError(aShell, "View Initialization Error",
					"Parameters number is not correct. It must have exactly two parameters : 'Path to models'");
			return "Parameters number is not correct. It must have exactly two parameters : 'Path to models'";
		}

		// Get 'model1' parameter
		if (parameters.containsKey("model1Path"))
		{
			if (parameters.get("model1Path") instanceof IPath)
			{
				pathFile1 = (IPath) parameters.get("model1Path");
				if (pathFile1 == null)
				{
					CorePlugin.log(this.getClass(), "The 'model1Path' parameter must be a Path", IStatus.ERROR);
					MessageDialog.openError(aShell, "View Initialization Error", "The 'model1Path' parameter must be a Path");
					return "The 'model1Path' parameter must be a Path";
				}
			}
		}

		if (parameters.containsKey("model2Path"))
		{
			if (parameters.get("model2Path") instanceof IPath)
			{
				pathFile2 = (IPath) parameters.get("model2Path");
				if (pathFile2 == null)
				{
					CorePlugin.log(this.getClass(), "The 'model2Path' parameter must be a Path", IStatus.ERROR);
					MessageDialog.openError(aShell, "View Initialization Error", "The 'model2Path' parameter must be a Path");
					return "The 'model2Path' parameter must be a Path";
				}
			}
		}
		// Get the read service
		Service s = null;
		for (String serviceName : services2.keySet())
		{
			s = services2.get(serviceName);
		}

		IService is = ServicesManager.getInstance().getService(s.getId());
		// Get the write service
		Service w = null;
		for (String serviceName : services3.keySet())
		{
			w = services3.get(serviceName);
		}

		ServicesManager.getInstance().getService(w.getId());

		// read the first model
		param.put("XMIFile", pathFile1.toOSString());
		try
		{
			model1 = (EObject) is.serviceRun(param);
		}
		catch (Exception e)
		{
			CorePlugin.log("The " + pathFile1 + " model is not valid. Check the model", IStatus.ERROR);
			MessageDialog.openError(aShell, "View Initialization Error", e.getMessage());
			return e.getMessage();
		}

		// read the second model
		param.put("XMIFile", pathFile2.toOSString());
		try
		{
			model2 = (EObject) is.serviceRun(param);
		}
		catch (Exception e)
		{
			CorePlugin.log("The " + pathFile2 + " model is not valid. Check the model", IStatus.ERROR);
			MessageDialog.openError(aShell, "View Initialization Error", e.getMessage());
			return e.getMessage();
		}

		// check the metamodels of the models are the same
		String nsURImodel1 = "";
		String nsURImodel2 = "";
		if (model1.eClass().getEPackage() != null && model2.eClass().getEPackage() != null)
		{

			nsURImodel1 = model1.eClass().getEPackage().getNsURI();
			nsURImodel2 = model2.eClass().getEPackage().getNsURI();

			if (nsURImodel1.compareTo(nsURImodel2) == 0)
			{
				try
				{
					// call the models comparison

				}
				catch (Exception e)
				{
					CorePlugin.log("Metamodels of compared Models are not the same !", IStatus.ERROR);
					MessageDialog.openError(aShell, "View Initialization Error", "Metamodels of compared Models are not the same !");
				}
			}
			else
			{
				CorePlugin.log("Metamodels of compared Models are not the same !", IStatus.ERROR);
				MessageDialog.openError(aShell, "View Initialization Error", "Metamodels of compared Models are not the same !");
				return "Metamodels of compared Models are not the same !";
			}
		}

		CorePlugin.log(this.getClass(), "End of model comparison service.", IStatus.INFO);
		// return the compared model
		try
		{
			ModelComparator comparison = ModelComparator.getInstance();
			ObjectComparison comparisonModel = comparison.compareModels(model1, model2, pathFile1, pathFile2);
			return comparisonModel;
		}
		catch (Exception e)
		{
			CorePlugin.log(this.getClass(), e.getLocalizedMessage(), IStatus.ERROR);
		}
		return null;

	}
}
