/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.comparison.views;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.Service;
import org.topcased.bus.core.ServicesManager;
import org.topcased.service.comparison.ComparedETreeModel;
import org.topcased.service.comparison.ComparedETreeModelContentProvider;
import org.topcased.service.comparison.ComparedETreeModelLabelProvider;
import org.topcased.service.comparison.ComparisonImageConstantes;
import org.topcased.service.comparison.ComparisonPlugin;
import org.topcased.service.comparison.ComparisonTransformation;
import org.topcased.service.comparison.ETreeModel;
import org.topcased.service.comparison.ETreeModelContentProvider;
import org.topcased.service.comparison.ETreeModelLabelProvider;
import org.topcased.service.comparison.ModelComparator;
import org.topcased.service.comparison.ObjectComparison;
import org.topcased.service.comparison.dnd.DragSourceComparison;
import org.topcased.service.comparison.dnd.DropTargetComparison;
import org.topcased.service.comparison.listener.ComparisonListener;


public class ComparisonView extends ViewPart implements ComparisonImageConstantes
{
	//
	private Object[] _pathSelected = new Object[] { null, null };

	//
	ETreeModel firstModel = null;

	ETreeModel secondModel = null;

	ModelComparator comparedModels;

	ObjectComparison modelComp = null;

	ComparedETreeModel resultTree = null;

	Tree tree1 = null;

	Tree tree2 = null;

	Tree tree3 = null;

	// Tree saving the source of Drag
	Tree sourceTree = null;

	Object[] racine1;

	Object[] racine2;

	Object[] racine3;

	TreeViewer treeViewer;

	TreeViewer treeViewer2;

	TreeViewer treeViewer3;

	private final TextTransfer textTransfer = TextTransfer.getInstance();

	private Action saveModel;

	private Action deleteObject;

	boolean isInitialise = false;

	private SashForm firstLine;

	private SashForm secondLine;

	// Listeners for Tree1
	DragSourceComparison tree1Drag = null;

	DragSource sourceDrag = null;

	DropTarget sourceDrop = null;

	// Listeners for Tree2
	DragSourceComparison tree2Drag = null;

	DragSource targetDrag = null;

	DropTarget targetDrop = null;

	// Listener for Tree3
	ComparisonListener compListener = null;

	// DropTragets
	DropTargetComparison dropTarget1 = null;

	DropTargetComparison dropTarget2 = null;

	/**
	 * The constructor.
	 */
	public ComparisonView(ModelComparator _comparedModels, Object[] _pathSelection)
	{
		comparedModels = ModelComparator.getInstance();
		_pathSelected = _pathSelection;
	}

	/**
	 * The constructor.
	 */
	public ComparisonView()
	{
	}

	public void init(IViewSite site) throws PartInitException
	{
		super.init(site);
	}

	public void setData()
	{
		comparedModels = ModelComparator.getInstance();
		_pathSelected = ModelComparator.getInstance().getPathSelected();
		if (_pathSelected[0] != null && _pathSelected[1] != null)
		{
			loadData();
		}
		setFocus();
	}

	public void loadData()
	{

		isInitialise = false;

		// initialise the tree content result model
		if (_pathSelected != null)
		{
			this.resultTree = new ComparedETreeModel(null, ModelComparator.getInstance().getModelComparison());
			racine3 = new Object[] { this.resultTree };
		}
		else
		{
			this.resultTree = new ComparedETreeModel(null, null);
			racine3 = new Object[] { this.resultTree };
		}
		if (treeViewer3.getContentProvider() != null || treeViewer2.getContentProvider() != null || treeViewer.getContentProvider() != null)
		{
			treeViewer3.getContentProvider().dispose();
			treeViewer3.getLabelProvider().dispose();
			treeViewer2.getContentProvider().dispose();
			treeViewer2.getLabelProvider().dispose();
			treeViewer.getContentProvider().dispose();
			treeViewer.getLabelProvider().dispose();
			treeViewer2.remove(racine2);
			treeViewer.remove(racine1);
		}
		treeViewer3.setContentProvider(new ComparedETreeModelContentProvider());
		ComparedETreeModelLabelProvider label = new ComparedETreeModelLabelProvider();
		treeViewer3.setLabelProvider(label);

		treeViewer2.setContentProvider(new ETreeModelContentProvider());
		treeViewer2.setLabelProvider(new ETreeModelLabelProvider());
		treeViewer.setContentProvider(new ETreeModelContentProvider());
		treeViewer.setLabelProvider(new ETreeModelLabelProvider());

		treeViewer3.setExpandedElements(racine3);
		treeViewer3.setAutoExpandLevel(TreeViewer.ALL_LEVELS);
		treeViewer3.collapseToLevel(racine3, TreeViewer.ALL_LEVELS);

		treeViewer3.setInput(racine3);
		tree3 = treeViewer3.getTree();

		// initialise the content of the first(left) model
		if (_pathSelected != null)
		{
			this.firstModel = new ETreeModel(null, ModelComparator.getInstance().getFirstModel(), true);
		}
		else
		{
			EObject nullObject = null;
			this.firstModel = new ETreeModel(null, nullObject, true);
		}
		racine1 = new Object[] { this.firstModel };

		treeViewer.setAutoExpandLevel(TreeViewer.ALL_LEVELS);
		treeViewer.setExpandedElements(racine1);
		treeViewer.collapseToLevel(racine1, TreeViewer.ALL_LEVELS);
		treeViewer.setInput(racine1);
		tree1 = treeViewer.getTree();

		// initialise the content of the second(right) model
		if (_pathSelected != null)
		{
			this.secondModel = new ETreeModel(null, ModelComparator.getInstance().getSecondModel(), false);
		}
		else
		{
			EObject nullObject = null;
			this.secondModel = new ETreeModel(null, nullObject, false);
		}

		racine2 = new Object[] { this.secondModel };

		treeViewer2.setAutoExpandLevel(TreeViewer.ALL_LEVELS);
		treeViewer2.setExpandedElements(racine2);
		treeViewer2.collapseToLevel(racine2, TreeViewer.ALL_LEVELS);
		treeViewer2.setInput(racine2);
		tree2 = treeViewer2.getTree();

		// refresh the view : Regarder ou c'tait appel

		if (_pathSelected != null)
		{

			if (compListener != null)
			{
				tree3.removeSelectionListener(compListener);
			}
			// new Listener for Trees
			compListener = new ComparisonListener(isInitialise, tree1, tree2, tree3, treeViewer, treeViewer2);
			tree3.addSelectionListener(compListener);
			isInitialise = true;
			makeActions();
			hookContextMenu();
			hookDoubleClickAction();
			contributeToActionBars();
		}

		// Drag Listener for Tree1
		if (tree1Drag != null)
		{
			sourceDrag.removeDragListener(tree1Drag);
		}
		// sourceDrag = new DragSource(tree1, operations);
		tree1Drag = new DragSourceComparison();
		sourceDrag.addDragListener(tree1Drag);
		sourceDrag.setTransfer(new Transfer[] { textTransfer });

		// Drop Listener for Tree1

		// Adds Drag operation for Tree2
		if (tree2Drag != null)
		{
			sourceDrag.removeDragListener(tree2Drag);
		}

		tree2Drag = new DragSourceComparison();
		targetDrag.addDragListener(tree2Drag);
		targetDrag.setTransfer(new Transfer[] { textTransfer });

		// Add Drop operation for Tree2

		// DND
		if (_pathSelected != null)
		{
			if (dropTarget1 != null && dropTarget2 != null)
			{
				sourceDrop.removeDropListener(dropTarget1);
				targetDrop.removeDropListener(dropTarget2);
			}
			dropTarget1 = new DropTargetComparison();
			sourceDrop.addDropListener(dropTarget1);
			sourceDrop.setTransfer(new Transfer[] { textTransfer });
		}
		if (_pathSelected != null)
		{
			dropTarget2 = new DropTargetComparison();
			targetDrop.addDropListener(dropTarget2);
			targetDrop.setTransfer(new Transfer[] { textTransfer });
		}

		// show file name + object name in the tree1
		if (null != _pathSelected[0])
		{
			tree1.getItem(0).setText(((IPath) _pathSelected[0]).toString().concat("/#").concat(tree1.getItem(0).getText()));
		}
		else
		{
			tree1.getItem(0).setText("Empty tree model");
		}
		tree1.showItem(tree1.getItem(0));

		// show file name + object name in the tree2
		if (null != _pathSelected[0])
		{
			tree2.getItem(0).setText(((IPath) _pathSelected[1]).toString().concat("/#").concat(tree2.getItem(0).getText()));
		}
		else
		{
			tree2.getItem(0).setText("Empty tree model");
		}

		tree2.showItem(tree2.getItem(0));
		treeViewer.collapseToLevel(racine1, TreeViewer.ALL_LEVELS);
		treeViewer2.collapseToLevel(racine2, TreeViewer.ALL_LEVELS);
		setFocus();

	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent)
	{

		// SourceTargetFacilities treeSelected = SourceTargetFacilities.getInstance();

		// window split
		GridData gridDataFirstLine = new GridData();
		gridDataFirstLine.verticalSpan = 2;
		firstLine = new SashForm(parent, SWT.HORIZONTAL);
		firstLine.setLayoutData(gridDataFirstLine);

		// The tree of the comparison model result
		treeViewer3 = new TreeViewer(firstLine, SWT.BORDER);
		tree3 = treeViewer3.getTree();
		tree3.setSize(520, 130);
		GridData gridData3 = new GridData();
		tree3.setLayoutData(gridData3);

		// window split
		GridData gridDataSecondLine = new GridData();// GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL);
		gridDataSecondLine.horizontalSpan = 2;
		gridDataSecondLine.heightHint = 200;
		secondLine = new SashForm(firstLine, SWT.VERTICAL);
		secondLine.setLayoutData(gridDataSecondLine);

		// First Viewer
		treeViewer = new TreeViewer(secondLine, SWT.BORDER);
		// Second Viewer
		treeViewer2 = new TreeViewer(secondLine, SWT.BORDER);

		// Value the first TreeViewer
		tree1 = treeViewer.getTree();
		tree1.setSize(250, 300);
		GridData gridData = new GridData();
		tree1.setLayoutData(gridData);

		// First Colon : comparison results 25%, Second Colon 75%
		firstLine.setWeights(new int[] { 1, 3 });

		// Adds Drag operations
		int operations = DND.DROP_COPY | DND.DROP_MOVE;

		// Drag Listener for Tree1
		sourceDrag = new DragSource(tree1, operations);

		// Drop Listener for Tree1
		sourceDrop = new DropTarget(tree1, operations);

		// Value the Second TreeViewer

		tree2 = treeViewer2.getTree();
		tree2.setSize(250, 300);
		GridData gridData2 = new GridData();
		tree2.setLayoutData(gridData2);

		// Adds Drag operation for Tree2
		targetDrag = new DragSource(tree2, operations);

		// Add Drop operation for Tree2
		targetDrop = new DropTarget(tree2, operations);
		// ISelection selection = getSite().getPage().getSelection();
		setData();

	}

	private void hookContextMenu()
	{
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener()
		{
			public void menuAboutToShow(IMenuManager manager)
			{
				ComparisonView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(treeViewer3.getControl());
		treeViewer3.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, treeViewer3);
	}

	private void contributeToActionBars()
	{
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager)
	{
		manager.add(saveModel);
		// manager.add(deleteObject);
		manager.add(new Separator());
	}

	private void fillContextMenu(IMenuManager manager)
	{
		manager.add(saveModel);
		manager.add(new Separator());
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void fillLocalToolBar(IToolBarManager manager)
	{
		manager.add(saveModel);
		// manager.add(deleteObject);
		manager.add(new Separator());
	}

	private void makeActions()
	{

		// Action to save the model
		saveModel = new Action()
		{
			public void run()
			{
				Map<String, Service> services = ServicesManager.getInstance().getAllServices();
				Map<String, Service> services2 = ServicesManager.getInstance().getServicesPerType("Write", services);
				Map<String, Object> param = new HashMap<String, Object>();
				Tree selectedTree = null;
				String pathSelected = null;
				if (tree2.isFocusControl())
				{
					selectedTree = tree2;
					pathSelected = ((IPath) _pathSelected[1]).removeLastSegments(1).toOSString().concat(File.separator);
				}
				else
				{
					if (tree1.isFocusControl())
					{
						selectedTree = tree1;
						pathSelected = ((IPath) _pathSelected[0]).removeLastSegments(1).toOSString().concat(File.separator);
					}
					else
					{
						if (tree3.isFocusControl())
						{
							selectedTree = tree3;
							pathSelected = ((IPath) _pathSelected[1]).removeLastSegments(1).toOSString().concat(File.separator);
						}
					}
				}
				if (selectedTree != null)
				{
					// showMessage("Saving...");
					Service s = null;
					for (String serviceName : services2.keySet())
					{
						s = services2.get(serviceName);
					}

					IService is = ServicesManager.getInstance().getService(s.getId());
					// fill the parameters

					InputDialog fileDialog = new InputDialog(new Shell(), "Save to ...", "Give the file name", new String(), new IInputValidator()
					{
						public String isValid(String newText)
						{
							if (newText.equals(null) || newText.compareTo("") == 0)
							{
								return "Give a new file name";
							}
							return null;
						}
					});
					fileDialog.setBlockOnOpen(true);
					fileDialog.create();
					fileDialog.open();
					String fileName = fileDialog.getValue();
					String pathString = pathSelected.concat(fileName);

					if (pathString != null)
					{
						param.put("PathFileString", (new Path(pathString).toOSString()));

						EObject eobj = null;
						if (selectedTree.getItem(0).getData() instanceof ETreeModel)
						{
							eobj = ((ETreeModel) selectedTree.getItem(0).getData()).getEobject();
						}
						else
						{
							eobj = ComparisonTransformation.transform(((ComparedETreeModel) selectedTree.getItem(0).getData())
									.getObjectComparison());
						}

						param.put("Model", eobj);

						// call the write service
						is.serviceRun(param);
					}
					else
					{
						CorePlugin.log("Save cancelled by the user", IStatus.WARNING);
					}
				}
			}
		};
		saveModel.setText("Save the selected model");
		saveModel.setToolTipText("To save selected model");
		saveModel.setImageDescriptor(ComparisonPlugin.getImageDescriptor(SAVE_OBJECT));

		// Action to delete an object in the right model
		deleteObject = new Action()
		{
			public void run()
			{
				Tree selectedTree = null;
				if (tree2.isFocusControl())
				{
					selectedTree = tree2;
				}
				else
				{
					selectedTree = tree1;
				}
				if (selectedTree.getSelection()[0] != null)
				{
					TreeItem itemToDelete = selectedTree.getSelection()[0];
					if (selectedTree.getSelection()[0].getParentItem() != null)
					{
						TreeItem parent = selectedTree.getSelection()[0].getParentItem();
						// Remove the object from the model
						EObject elementObject = ((ETreeModel) itemToDelete.getData()).getEobject();
						EStructuralFeature afeature = elementObject.eContainmentFeature();
						if (afeature != null)
						{
							EObject parentEobject = ((ETreeModel) parent.getData()).getEobject();
							EList<EStructuralFeature> listRef = parentEobject.eClass().getEAllStructuralFeatures();

							Object localChildren = null;
							if (listRef.contains(afeature))
							{
								localChildren = parentEobject.eGet(afeature, true);
							}
							if (afeature != null)
							{
								if (localChildren instanceof EList)
								{
									((EList) localChildren).remove(elementObject);
									selectedTree.update();
								}
								else
								{
									try
									{
										parentEobject.eUnset(afeature);
										selectedTree.update();
									}
									catch (Exception e)
									{
										// The feature is unsettable but was correctly found
									}
								}
							}
							//
						}
						((ETreeModel) parent.getData()).subToAllContainedSons(((ETreeModel) itemToDelete.getData()));
						itemToDelete.dispose();
						selectedTree.update();
					}
				}
			}
		};
		deleteObject.setText("Delete the selected object in the selected model");
		deleteObject.setToolTipText("To delete the selected object in the selected model");
		deleteObject.setImageDescriptor(ComparisonPlugin.getImageDescriptor(DELETE_OBJECT));
	}

	private void hookDoubleClickAction()
	{
		treeViewer3.addDoubleClickListener(new IDoubleClickListener()
		{
			public void doubleClick(DoubleClickEvent event)
			{
				// doubleClickAction.run();
			}
		});
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus()
	{
		treeViewer.getControl().setFocus();
		treeViewer2.getControl().setFocus();
		treeViewer3.getControl().setFocus();
	}
}