/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.getservices;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.Service;
import org.topcased.bus.core.ServicesManager;
import org.topcased.bus.core.parameter.Parameter;

/**
 * 
 * This class represents the dialog box used to call a service.<br>
 * The input parameters, if any, have to be given before calling the service.<br>
 * This allows to test if a service may work.
 */
public class CallServiceShell
{

	private Shell instance;

	private String serviceId;

	private List<Label> inParamLabel = new ArrayList<Label>();

	private List<Label> inParamType = new ArrayList<Label>();

	private List<Text> inParamValue = new ArrayList<Text>();

	private Map<String, String> parametersService = new HashMap<String, String>();

	private List<String> descriptionValue = new ArrayList<String>();

	/**
	 * This contructor makes the window appear.
	 * 
	 * @param id : id of the service to be called
	 * @param parent : the shell invoking this window
	 */
	public CallServiceShell(String id, Shell parent)
	{
		// Grid Data and Grid Layout Definitions
		final GridLayout mainGridLayout = new GridLayout(2, false);
		mainGridLayout.horizontalSpacing = 10;

		// Concerns different left items
		final GridData leftLabelGridData = new GridData();
		leftLabelGridData.widthHint = 80;

		final GridData leftValueGridData = new GridData(SWT.FILL, SWT.NONE, true, false);
		leftValueGridData.minimumWidth = 100;

		// Concerns different right items
		final GridData rightTextGridData = new GridData(SWT.FILL, SWT.CENTER, true, false);
		rightTextGridData.minimumWidth = 140;

		final GridData rightSmallButton = new GridData(SWT.FILL, SWT.NONE, false, false);
		rightSmallButton.widthHint = 25;
		rightSmallButton.minimumWidth = 25;
		rightSmallButton.minimumHeight = 15;

		final GridData tipGridData = new GridData(SWT.FILL, SWT.NONE, true, false);
		tipGridData.heightHint = 35;

		final GridLayout tipLayout = new GridLayout();
		tipLayout.marginWidth = 0;
		tipLayout.marginHeight = 0;

		// Concerns bottom items
		final GridData bottomGridData = new GridData(SWT.FILL, SWT.NONE, true, false);
		bottomGridData.horizontalSpan = 2;

		final GridLayout bottomLayout = new GridLayout(2, false);
		bottomLayout.marginWidth = 0;
		bottomLayout.marginHeight = 0;

		final GridData buttonGridData = new GridData();
		buttonGridData.widthHint = 120;

		/*
		 * MAIN SHELL
		 */
		instance = new Shell(parent, SWT.SYSTEM_MODAL | SWT.SHELL_TRIM);
		instance.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		instance.setLayout(mainGridLayout);
		instance.setText("Call service");
		instance.setMinimumSize(850, 330);
		instance.setSize(850, 330);
		instance.setLocation(50, 50);

		serviceId = id;
		ServicesManager manager = ServicesManager.getInstance();
		Service service = manager.serviceId2Service(id);
		Parameter out = manager.parameterId2Parameter(service.getIdParameterOut());

		/*
		 * LEFT COMPOSITE
		 */
		final Composite leftComposite = new Composite(instance, SWT.NONE);
		leftComposite.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, true));
		leftComposite.setLayout(new GridLayout(2, false));

		final Label ontologyLabel = new Label(leftComposite, SWT.LEFT);
		ontologyLabel.setText("Ontology :");
		ontologyLabel.setLayoutData(leftLabelGridData);

		final Label ontologyValueLabel = new Label(leftComposite, SWT.BORDER);
		ontologyValueLabel.setText(manager.typeId2Type(service.getIdType()).getCategoryType());
		ontologyValueLabel.setLayoutData(leftValueGridData);

		final Label nameLabel = new Label(leftComposite, SWT.NONE);
		nameLabel.setText("Name :");
		nameLabel.setLayoutData(leftLabelGridData);

		final Label nameValueLabel = new Label(leftComposite, SWT.BORDER);
		nameValueLabel.setText(service.getName());
		nameValueLabel.setLayoutData(leftValueGridData);

		final Label outParamLabel = new Label(leftComposite, SWT.NONE);
		outParamLabel.setText("Result :");
		outParamLabel.setLayoutData(leftLabelGridData);

		final Label outParamValueLabel = new Label(leftComposite, SWT.BORDER);
		outParamValueLabel.setLayoutData(leftValueGridData);
		if (out != null)
		{
			outParamValueLabel.setText(out.getNameParam() + " (" + out.getKindParam() + ")");
		}
		else
		{
			outParamValueLabel.setText("None");
		}

		/*
		 * RIGHT COMPOSITE
		 */
		final Composite rightComposite = new Composite(instance, SWT.NONE);
		rightComposite.setLayout(new GridLayout());
		rightComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		// Top composite of right side
		final Composite informationComposite = new Composite(rightComposite, SWT.BORDER);
		informationComposite.setLayout(new GridLayout(6, false));
		informationComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		final Composite tipComposite = new Composite(rightComposite, SWT.NONE);
		tipComposite.setLayoutData(tipGridData);
		tipComposite.setLayout(tipLayout);

		final Label descriptionText = new Label(tipComposite, SWT.BORDER | SWT.WRAP);
		descriptionText.setLayoutData(tipGridData);

		final List<Button> listFileButton = new ArrayList<Button>();
		final List<Button> listDirectoryButton = new ArrayList<Button>();
		final List<Button> listRessourceButton = new ArrayList<Button>();

		final List<String> inputParameters = new ArrayList<String>(service.getIdParameterIn());
		for (String param : inputParameters)
		{
			final int currentParam = inputParameters.indexOf(param);
			
			final Parameter inParam = manager.parameterId2Parameter(param);
			descriptionValue.add(inParam.getDescriptionParam());

			final Label label = new Label(informationComposite, SWT.NONE); 
			label.setText(inParam.getNameParam());
			label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
			inParamLabel.add(label);

			final Label type = new Label(informationComposite, SWT.NONE);
			type.setText("(".concat(inParam.getKindParam()).concat(")"));
			type.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false));
			inParamType.add(type);

			final Text value = new Text(informationComposite, SWT.NONE | SWT.BORDER); 
			value.setLayoutData(rightTextGridData);
			value.addFocusListener(new FocusAdapter()
			{
				public void focusGained(FocusEvent e)
				{
					descriptionText.setText(descriptionValue.get(currentParam));
				}

			});
			inParamValue.add(value);
			parametersService.put(inParam.getNameParam(), "");

			final Button fileBtn = new Button(informationComposite, SWT.PUSH);
			fileBtn.setText("F");
			fileBtn.setLayoutData(rightSmallButton);
			fileBtn.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					openFileDialog(currentParam);
				}
			});
			listFileButton.add(fileBtn);

			final Button directoryBtn = new Button(informationComposite, SWT.PUSH);
			directoryBtn.setText("D");
			directoryBtn.setLayoutData(rightSmallButton);
			directoryBtn.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					openDirectoryDialog(currentParam);
				}
			});
			listDirectoryButton.add(directoryBtn);

			final Button ressourceButton = new Button(informationComposite, SWT.PUSH);
			ressourceButton.setText("R");
			ressourceButton.setLayoutData(rightSmallButton);
			ressourceButton.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					openRessourceDialog(currentParam);
				}
			});
			listRessourceButton.add(ressourceButton);
		}

		/*
		 * BOTTOM COMPOSITE
		 */
		final Composite bottomComposite = new Composite(instance, SWT.NONE);
		bottomComposite.setLayoutData(bottomGridData);
		bottomComposite.setLayout(bottomLayout);

		final Button cancelButton = new Button(bottomComposite, SWT.PUSH | SWT.CENTER);
		cancelButton.setText("Cancel");
		cancelButton.setLayoutData(buttonGridData);
		cancelButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				instance.dispose();
			}
		});

		final Button callServiceButton = new Button(bottomComposite, SWT.PUSH | SWT.CENTER);
		callServiceButton.setText("Call Service");
		callServiceButton.setLayoutData(buttonGridData);
		callServiceButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				Map<String, Object> parameters = new HashMap<String, Object>();
				for (Text textField : inParamValue)
				{
					int ind = inParamValue.indexOf(textField);
					if ("(IPath)".equals(inParamType.get(ind).getText()))
					{
						IPath newValuePath = new Path(inParamValue.get(ind).getText());
						parameters.put(inParamLabel.get(ind).getText(), newValuePath);
					}
					else if ("(EObject)".equals(inParamType.get(ind).getText()))
					{
						String value = inParamValue.get(ind).getText();

						IService readService = ServicesManager.getInstance().getService(IServiceCst.READ_XMI_EMF);
						HashMap<String, Object> paramService = new HashMap<String, Object>();
						paramService.put("XMIFile", value);
						parameters.put(inParamLabel.get(ind).getText(), (EObject) readService.serviceRun(paramService));
					}
					else
					{
						parameters.put(inParamLabel.get(ind).getText(), inParamValue.get(ind).getText());
					}
				}

				try
				{
					ServicesManager.getInstance().getService(serviceId).serviceRun(parameters);
					CorePlugin.log(this.getClass(), "Service '" + serviceId + "' terminated", IStatus.INFO);
				}
				catch (Exception ex)
				{
					MessageDialog.openError(instance, "Exception error", "An exception has been raised while running the service '"
							+ serviceId + "' : " + ex.getMessage());
					CorePlugin.log(this.getClass(), "Exception raised : " + ex.getMessage(), IStatus.ERROR);
				}
			}
		});

		instance.open();
	}

	/**
	 * This function open a FileDialog and change the text in the corresponding paramLabel
	 * 
	 * @param i : number of the parameter to be filled
	 */
	private void openFileDialog(int i)
	{
		FileDialog box = new FileDialog(instance, SWT.OPEN);
		String result = box.open();
		if (result != null)
		{
			inParamValue.get(i).setText(result);
			parametersService.put(inParamLabel.get(i).getText(), inParamValue.get(i).getText());
		}
	}

	/**
	 * This function open a DirectoryDialog and change the text in the corresponding paramLabel
	 * 
	 * @param i : number of the parameter to be filled
	 */
	private void openDirectoryDialog(int i)
	{
		DirectoryDialog box = new DirectoryDialog(instance, SWT.OPEN);
		String result = box.open();
		if (result != null)
		{
			inParamValue.get(i).setText(result);
			parametersService.put(inParamLabel.get(i).getText(), inParamValue.get(i).getText());
		}
	}

	/**
	 * This function open a RessourceSelectionDialog and change the text in the corresponding paramLabel
	 * 
	 * @param i : number of the parameter to be filled
	 */
	private void openRessourceDialog(int i)
	{
		ResourceSelectionDialog box = new ResourceSelectionDialog(instance, ResourcesPlugin.getWorkspace().getRoot(),
				"Select one resource file");
		if (box.open() == Window.OK)
		{
			Object[] result = box.getResult();
			if (result != null && result.length > 0)
			{
				if (result[0] instanceof IFile)
				{
					inParamValue.get(i).setText(((IFile) result[0]).getLocation().toOSString());
					parametersService.put(inParamLabel.get(i).getText(), inParamValue.get(i).getText());
				}
			}
		}
	}
}
