/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.getservices;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.Service;
import org.topcased.bus.core.ServicesManager;
import org.topcased.bus.core.parameter.Parameter;

public class ServicesManagement implements IService
{
	private Shell shell;

	private Display defaultDisplay = Display.getDefault();

	private Table basicServicesList;

	private Table otherServicesList;

	private Set<String> ontologySet;

	private Combo ontologyFilterCombo;

	private Text nameFilter;

	// Constants definition
	private static final int ONTOLOGY_INDEX = 0;

	private static final int NAME_INDEX = 1;

	private static final int OUT_PARAM_INDEX = 2;

	private static final int IN_PARAM_INDEX = 3;

	private static final int ID_INDEX = 4;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.topcased.bus.core.IService#serviceRun(java.lang.Object[])
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{
		// Grid Data Definitions
		final GridData mainGridData = new GridData(SWT.FILL, SWT.FILL, true, true);

		final GridData labelGridData = new GridData(SWT.FILL, SWT.NONE, true, false);
		labelGridData.horizontalSpan = 3;

		final GridData tableGridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		tableGridData.horizontalSpan = 3;

		final GridData buttonsGridData = new GridData(SWT.RIGHT, SWT.NONE, true, false);
		buttonsGridData.horizontalSpan = 3;

		final GridData buttonGridData = new GridData();
		buttonGridData.widthHint = 100;

		GridData textGridData = new GridData();
		textGridData.widthHint = 150;

		final GridLayout buttonsLayout = new GridLayout(2, false);
		buttonsLayout.marginWidth = 0;
		buttonsLayout.marginHeight = 0;

		shell = new Shell(defaultDisplay, SWT.SYSTEM_MODAL | SWT.SHELL_TRIM);
		shell.setText("Topcased Console");
		shell.setMinimumSize(700, 390);
		shell.setLayout(new GridLayout(3, false));
		shell.setLayoutData(mainGridData);
		shell.setLocation(100, 80);

		final Label basicServicesLabel = new Label(shell, SWT.NONE);
		basicServicesLabel.setText("Basic services list :");
		basicServicesLabel.setLayoutData(labelGridData);

		basicServicesList = new Table(shell, SWT.BORDER | SWT.FULL_SELECTION);
		basicServicesList.setLayoutData(tableGridData);

		final TableColumn basicOntologyColumn = new TableColumn(basicServicesList, SWT.LEFT);
		final TableColumn basicNameColumn = new TableColumn(basicServicesList, SWT.LEFT);
		final TableColumn basicOutParamColumn = new TableColumn(basicServicesList, SWT.LEFT);
		final TableColumn basicInParamColumn = new TableColumn(basicServicesList, SWT.CENTER);
		final TableColumn basicServiceIdColumn = new TableColumn(basicServicesList, SWT.CENTER);

		basicOntologyColumn.setText("Ontology");
		basicOntologyColumn.setWidth(200);
		basicOntologyColumn.setResizable(true);

		basicNameColumn.setText("Name");
		basicNameColumn.setWidth(250);
		basicNameColumn.setResizable(true);

		basicOutParamColumn.setText("Result");
		basicOutParamColumn.setWidth(140);
		basicOutParamColumn.setResizable(true);

		basicInParamColumn.setText("Parameters");
		basicInParamColumn.setWidth(80);
		basicInParamColumn.setResizable(true);

		basicServiceIdColumn.setText("");
		basicServiceIdColumn.setWidth(0);
		basicServiceIdColumn.setResizable(false);

		basicServicesList.setLinesVisible(true);
		basicServicesList.setHeaderVisible(true);
		basicServicesList.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				otherServicesList.deselectAll();
			}
		});

		ontologySet = new HashSet<String>();

		fillOntologySet();

		// Insert an empty label separator
		Label emptyLabel = new Label(shell, SWT.NONE);
		emptyLabel.setLayoutData(labelGridData);

		ontologyFilterCombo = new Combo(shell, SWT.READ_ONLY);
		ontologyFilterCombo.setSize(100, 20);
		ontologyFilterCombo.add("No filter");
		for (String ontology : ontologySet)
		{
			ontologyFilterCombo.add(ontology);
		}
		ontologyFilterCombo.select(0);
		ontologyFilterCombo.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				fillServicesList();
			}
		});

		nameFilter = new Text(shell, SWT.SINGLE | SWT.BORDER);
		nameFilter.setLayoutData(textGridData);
		nameFilter.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				fillServicesList();
			}

		});

		final Label comment = new Label(shell, SWT.NONE);
		comment.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
		comment.setText("Display the services list containing the search expression.");

		// Insert an empty label separator
		emptyLabel = new Label(shell, SWT.NONE);
		emptyLabel.setLayoutData(labelGridData);

		final Label otherServicesLabel = new Label(shell, SWT.NONE);
		otherServicesLabel.setText("Other services list :");
		otherServicesLabel.setLayoutData(labelGridData);

		otherServicesList = new Table(shell, SWT.BORDER | SWT.FULL_SELECTION);
		otherServicesList.setLayoutData(tableGridData);
		final TableColumn otherOntologyColumn = new TableColumn(otherServicesList, SWT.LEFT);
		final TableColumn otherNameColumn = new TableColumn(otherServicesList, SWT.LEFT);
		final TableColumn otherOutParamColumn = new TableColumn(otherServicesList, SWT.LEFT);
		final TableColumn otherInParamColumn = new TableColumn(otherServicesList, SWT.CENTER);
		final TableColumn otherServiceIdColumn = new TableColumn(otherServicesList, SWT.CENTER);

		otherOntologyColumn.setText("Ontology");
		otherOntologyColumn.setWidth(200);
		otherOntologyColumn.setResizable(true);

		otherNameColumn.setText("Name");
		otherNameColumn.setWidth(250);
		otherNameColumn.setResizable(true);

		otherOutParamColumn.setText("Result");
		otherOutParamColumn.setWidth(140);
		otherOutParamColumn.setResizable(true);

		otherInParamColumn.setText("Parameters");
		otherInParamColumn.setWidth(80);
		otherInParamColumn.setResizable(true);

		otherServiceIdColumn.setText("");
		otherServiceIdColumn.setWidth(0);
		otherServiceIdColumn.setResizable(false);

		otherServicesList.setLinesVisible(true);
		otherServicesList.setHeaderVisible(true);
		otherServicesList.setSize(550, 100);
		// otherServicesList.setLocation(20,190);
		otherServicesList.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				basicServicesList.deselectAll();
			}
		});

		fillServicesList();

		// Insertion of an empty label
		emptyLabel = new Label(shell, SWT.NONE);
		emptyLabel.setLayoutData(labelGridData);

		// Initialisation of the buttons on a specific composite
		final Composite buttonsComposite = new Composite(shell, SWT.NONE);
		buttonsComposite.setLayoutData(buttonsGridData);
		buttonsComposite.setLayout(buttonsLayout);
		
		final Button callButton = new Button(buttonsComposite, SWT.PUSH | SWT.CENTER);
		callButton.setText("Call service");
		callButton.setLayoutData(buttonGridData);
		callButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				callService();
			}
		});
		
		final Button cancelButton = new Button(buttonsComposite, SWT.PUSH | SWT.CENTER);
		cancelButton.setText("Close");
		cancelButton.setLayoutData(buttonGridData);
		cancelButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				shell.dispose();
			}
		});



		shell.open();
		while (!shell.isDisposed())
		{
			if (!defaultDisplay.readAndDispatch())
				defaultDisplay.sleep();
		}
		// Do not dispose the display or it will result in an error
		return null;
	}

	/**
	 * Fills the ontologies list
	 */
	private void fillOntologySet()
	{
		ServicesManager manager = ServicesManager.getInstance();
		Map<String, Service> services = manager.getAllServices();
		for (String servName : services.keySet())
		{
			Service serv = services.get(servName);
			ontologySet.add(manager.typeId2Type(serv.getIdType()).getCategoryType());
		}
	}

	/**
	 * Fills the services list
	 */
	private void fillServicesList()
	{
		ServicesManager manager = ServicesManager.getInstance();
		Map<String, Service> services = manager.getAllServices();
		basicServicesList.removeAll();
		otherServicesList.removeAll();
		for (String serviceName : services.keySet())
		{
			Service serv = services.get(serviceName);
			String ontology = manager.typeId2Type(serv.getIdType()).getCategoryType();
			if (ontologyFilterCombo.getSelectionIndex() == 0
					|| ontologyFilterCombo.getItem(ontologyFilterCombo.getSelectionIndex()).equals(ontology))
			{
				if (nameFilter.getText() == null || nameFilter.getText().equals("")
						|| serv.getName().matches(".*" + nameFilter.getText() + ".*"))
				{
					TableItem temp;
					if (ServicesManager.isBasicService(serviceName))
					{
						temp = new TableItem(basicServicesList, SWT.NONE);
					}
					else
					{
						temp = new TableItem(otherServicesList, SWT.NONE);
					}
					temp.setText(ONTOLOGY_INDEX, ontology);
					temp.setText(NAME_INDEX, serv.getName());
					Parameter outParam = manager.parameterId2Parameter(serv.getIdParameterOut());
					if (outParam != null)
					{
						temp.setText(OUT_PARAM_INDEX, outParam.getNameParam() + " (" + outParam.getKindParam() + ")");
					}
					else
					{
						temp.setText(OUT_PARAM_INDEX, "None");
					}
					temp.setText(IN_PARAM_INDEX, Integer.toString(serv.getIdParameterIn().size()));
					temp.setText(ID_INDEX, serviceName);

				}
			}
		}
	}

	/**
	 * Calls the corresponding service
	 */
	private void callService()
	{
		TableItem[] selection = basicServicesList.getSelection();
		TableItem[] selectionOther = otherServicesList.getSelection();
		if (selection.length == 0 && selectionOther.length == 0)
		{
			MessageDialog.openError(shell, "Empty selection", "No item has been selected from the list");
		}
		else
		{
			for (int i = 0; i < selection.length; i++)
			{
				String id = selection[i].getText(ID_INDEX);
				new CallServiceShell(id, shell);
			}
			for (int i = 0; i < selectionOther.length; i++)
			{
				String id = selectionOther[i].getText(ID_INDEX);
				new CallServiceShell(id, shell);
			}
		}
	}
}
