/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.importcopymodel.services;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;

/**
 * Defines the Copy Model import service.
 */ 
public class CopyModelImportService implements IService
{
	/**
	 * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{
		if (!parameters.containsKey("IN"))
		{
			CorePlugin.log("The parameter IN must be set.", IStatus.ERROR);
		}

		if (!parameters.containsKey("OUT"))
		{
			CorePlugin.log("The parameter OUT must be set.", IStatus.ERROR);
		}

		if (!parameters.containsKey("Path"))
		{
			CorePlugin.log("The parameter Path must be set.", IStatus.ERROR);
		}

		if (!(parameters.get("IN") instanceof String))
		{
			CorePlugin.log("The parameter IN must be a String.", IStatus.ERROR);
		}

		if (!(parameters.get("OUT") instanceof String))
		{
			CorePlugin.log("The parameter OUT must be a String.", IStatus.ERROR);
		}

		if (!(parameters.get("Path") instanceof IPath))
		{
			CorePlugin.log("The parameter Path must be a IPath.", IStatus.ERROR);
		}

		String beginTime = new SimpleDateFormat("yyyy-MM-dd hh'h'mm").format(new Date());
		CorePlugin.log("Entering Copy Model import service at ".concat(beginTime), IStatus.INFO);

		String inpath = (String) parameters.get("IN");
		IPath outpath = new Path((String) parameters.get("OUT"));
		outpath = outpath.addFileExtension("mapping");
		IPath path = (IPath) parameters.get("Path");

		Map<String, String> models = new HashMap<String, String>();
		models.put("IN", inpath);
		models.put("OUT", outpath.toOSString());

		Map<String, Object> serviceParameters = new HashMap<String, Object>();
		serviceParameters.put("Id", "org.topcased.service.importcopymodel.transformation");
		serviceParameters.put("Models", models);
		serviceParameters.put("IsXml", new Boolean(false));
		serviceParameters.put("ProjectPath", path);

		IService transformation = ServicesManager.getInstance().getService(IServiceCst.TRANSFORMATION);
		transformation.serviceRun(serviceParameters);

		IService readService = ServicesManager.getInstance().getService(IServiceCst.READ_XMI_EMF);
		serviceParameters = new HashMap<String, Object>();
		serviceParameters.put("XMIFile", path.append(outpath).toString());
		Object result = readService.serviceRun(serviceParameters);

		String endTime = new SimpleDateFormat("yyyy-MM-dd hh'h'mm").format(new Date());
		CorePlugin.log("Leaving Copy Model import service at ".concat(endTime), IStatus.INFO);

		return result;
	}
}
