/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.importmappingatl.services;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;

/**
 * Defines the Mapping ATL import service.
 */
public class MappingATLImportService implements IService
{
	/**
	 * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{
		if (!parameters.containsKey("IN"))
		{
			CorePlugin.log("The parameter IN must be set.", IStatus.ERROR);
		}

		if (!parameters.containsKey("OUT"))
		{
			CorePlugin.log("The parameter OUT must be set.", IStatus.ERROR);
		}

		if (!parameters.containsKey("Path"))
		{
			CorePlugin.log("The parameter Path must be set.", IStatus.ERROR);
		}

		if (!(parameters.get("IN") instanceof String))
		{
			CorePlugin.log("The parameter IN must be a String.", IStatus.ERROR);
		}

		if (!(parameters.get("OUT") instanceof String))
		{
			CorePlugin.log("The parameter OUT must be a String.", IStatus.ERROR);
		}

		if (!(parameters.get("Path") instanceof IPath))
		{
			CorePlugin.log("The parameter Path must be a IPath.", IStatus.ERROR);
		}

		String beginTime = new SimpleDateFormat("yyyy-MM-dd hh'h'mm").format(new Date());
		CorePlugin.log("Entering Mapping to ATL import service at ".concat(beginTime), IStatus.INFO);

		String inpath = (String) parameters.get("IN");
		IPath outpath = new Path((String) parameters.get("OUT"));
		outpath = outpath.addFileExtension("atl");
		IPath path = (IPath) parameters.get("Path");

		HashMap<String, String> models = new HashMap<String, String>();
		models.put("IN", inpath);
		models.put("OUT", outpath.toOSString());

		HashMap<String, Object> serviceParameters = new HashMap<String, Object>();
		serviceParameters.put("Id", "org.topcased.service.importmappingatl.transformation");
		serviceParameters.put("Models", models);
		serviceParameters.put("IsXml", new Boolean(false));
		serviceParameters.put("ProjectPath", path);

		IService transformation = ServicesManager.getInstance().getService(IServiceCst.TRANSFORMATION);
		transformation.serviceRun(serviceParameters);

		File temp = new File(System.getProperty("user.dir"), "tempfile.atl");

		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		File target = root.getLocation().append(path.append(outpath)).toFile();
		if (target.exists())
		{
			target.delete();
		}

		if (!temp.renameTo(target))
		{
			CorePlugin.log("Problem during saving rule file.", IStatus.ERROR);
		}
		
		String endTime = new SimpleDateFormat("yyyy-MM-dd hh'h'mm").format(new Date());
		CorePlugin.log("Leaving Mapping to ATL import service at ".concat(endTime), IStatus.INFO);
		
		return null;
	}
}
