/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS)
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *********************************************************************************************/
package org.topcased.service.oclchecker.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.topcased.service.oclchecker.dialogs.CheckerDialog;

public class OCLCheckerAction implements IObjectActionDelegate
{
	/** Path to the model file */
	private IFile modelFile = null;

	/**
	 * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction, org.eclipse.ui.IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart)
	{
		// Nothing to do
	}

	/**
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action)
	{
		CheckerDialog dialog;
		if (modelFile == null)
		{
			dialog = new CheckerDialog(Display.getCurrent().getActiveShell());
		}
		else
		{
			dialog = new CheckerDialog(Display.getCurrent().getActiveShell(), modelFile);
		}
		dialog.open();

	}

	/**
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection)
	{
		IStructuredSelection structured = (IStructuredSelection) selection;
		Object object = structured.getFirstElement();
		if (object instanceof IAdaptable)
		{
			object = ((IAdaptable) object).getAdapter(IResource.class);
		}
		// Retrieves the path of the model file
		if (object != null && object instanceof IResource)
		{
			IResource res = (IResource) object;
			if (res instanceof IFile)
			{
				modelFile = (IFile) res;
			}
			else
			{
				modelFile = null;
			}
		}
	}
}
