/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.oclchecker.dialogs;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.DialogPage;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.window.Window;
import org.eclipse.ocl.Query;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.expressions.OCLExpression;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;
import org.topcased.checkresultmodel.CheckresultmodelFactory;
import org.topcased.checkresultmodel.Context;
import org.topcased.checkresultmodel.Item;
import org.topcased.checkresultmodel.Model;
import org.topcased.checkresultmodel.ModelInformation;
import org.topcased.checkresultmodel.Package;
import org.topcased.checkresultmodel.Rule;
import org.topcased.checkresultmodel.State;
import org.topcased.checkresultmodel.dialog.CheckResultDialog;
import org.topcased.editors.ocleditor.utils.OCLRuleUtil;

public class CheckerDialog extends Dialog
{

	private EObject model;

	private Text modelText;

	private Menu ruleListMenu;

	private CheckboxTableViewer ruleListCheckboxViewer;

	private Table rulesTable;

	private Button oclFileButton;

	private Button oclFileWorkspaceButton;

	private Label messageLabel;

	private static int CHECK_BUTTON_ID = 10;

	private static int SELECT_ALL_BUTTON_ID = 11;

	private static int DESELECT_ALL_BUTTON_ID = 12;

	private static CheckresultmodelFactory resultFactory = CheckresultmodelFactory.eINSTANCE;

	private String oclRuleFilePath = "";

	/**
	 * Constructor
	 * 
	 * @param parent The parent shell
	 */
	public CheckerDialog(Shell parent)
	{
		super(parent);
		this.model = null;
		setShellStyle(SWT.RESIZE | SWT.MAX | SWT.CLOSE | SWT.SYSTEM_MODAL | SWT.TITLE);
	}

	/**
	 * Constructor
	 * 
	 * @param parent The parent Shell
	 * @param modelFile The model file to check
	 */
	public CheckerDialog(Shell parent, IFile modelFile)
	{
		this(parent);
		model = OCLRuleUtil.loadModel(modelFile);
	}

	/**
	 * Constructor
	 * 
	 * @param parent The parent Shell
	 * @param model_ Th emodel to check
	 */
	public CheckerDialog(Shell parent, EObject model_)
	{
		this(parent);
		model = model_;
	}

	/**
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell newShell)
	{
		super.configureShell(newShell);
		newShell.setText("Checker Dialog");
		newShell.setMinimumSize(400, 400);
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent)
	{
		Composite topComposite = new Composite(parent, SWT.NONE);
		GridLayout mainLayout = new GridLayout(2, false);
		topComposite.setLayout(mainLayout);
		GridData mainLayoutData = new GridData(SWT.FILL, SWT.FILL, true, true);
		mainLayoutData.widthHint = 600;
		mainLayoutData.heightHint = 500;
		topComposite.setLayoutData(mainLayoutData);

		messageLabel = new Label(topComposite, SWT.NONE);
		GridData labelLayoutData = new GridData(SWT.FILL, SWT.NONE, true, false);
		labelLayoutData.horizontalSpan = 2;
		messageLabel.setLayoutData(labelLayoutData);

		if (model == null)
		{
			modelText = new Text(topComposite, SWT.BORDER | SWT.SINGLE);
			GridData textLayoutData = new GridData(SWT.FILL, SWT.CENTER, true, false);
			textLayoutData.minimumWidth = 100;
			textLayoutData.widthHint = 300;
			modelText.setLayoutData(textLayoutData);

			Button modelButton = new Button(topComposite, SWT.PUSH);
			modelButton.setText("Browse a Model to check...");
			GridData buttonLayoutData = new GridData();
			buttonLayoutData.minimumWidth = 50;
			buttonLayoutData.widthHint = "Browse a Model to check...".length() * modelButton.getFont().getFontData()[0].getHeight() * 2 / 3;
			buttonLayoutData.horizontalAlignment = SWT.CENTER;
			modelButton.setLayoutData(buttonLayoutData);
			modelButton.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					ResourceSelectionDialog resourceDialog = new ResourceSelectionDialog(getShell(), ResourcesPlugin.getWorkspace()
					        .getRoot(), "Select a model file");
					int result = resourceDialog.open();
					if (result == Window.OK && resourceDialog.getResult().length > 0)
					{
						Object selectedObject = resourceDialog.getResult()[0];
						if (selectedObject instanceof IFile)
						{
							model = OCLRuleUtil.loadModel((IFile) selectedObject);
							modelText.setText(((IFile) selectedObject).getLocation().toString());
							if (model != null)
							{
								rulesTable.removeAll();
							}

							updateMessage();
						}
					}
				}
			});
		}

		Composite browseRuleButtonsComposite = new Composite(topComposite, SWT.NONE);
		browseRuleButtonsComposite.setLayout(new GridLayout(2, false));
		GridData compositeLayoutData = new GridData(SWT.LEFT, SWT.NONE, true, false);
		compositeLayoutData.minimumWidth = 100;
		compositeLayoutData.horizontalSpan = 2;
		browseRuleButtonsComposite.setLayoutData(compositeLayoutData);

		oclFileWorkspaceButton = new Button(browseRuleButtonsComposite, SWT.PUSH);
		oclFileWorkspaceButton.setText("Browse OCL rule files (workspace)...");
		GridData button2LayoutData = new GridData(SWT.LEFT, SWT.NONE, false, false);
		button2LayoutData.minimumWidth = 100;
		button2LayoutData.widthHint = oclFileWorkspaceButton.getText().length()
		        * oclFileWorkspaceButton.getFont().getFontData()[0].getHeight() * 2 / 3;
		oclFileWorkspaceButton.setLayoutData(button2LayoutData);
		oclFileWorkspaceButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent event)
			{
				ResourceSelectionDialog resourceDialog = new ResourceSelectionDialog(getShell(), ResourcesPlugin.getWorkspace().getRoot(),
				        "Select an OCL rule file");
				int result = resourceDialog.open();
				if (result == Window.OK && resourceDialog.getResult().length > 0)
				{
					Object[] selectedArray = resourceDialog.getResult();
					for (int i = 0; i < selectedArray.length; i++)
					{
						if (resourceDialog.getResult()[i] instanceof IFile)
						{
							IFile resourceFile = (IFile) resourceDialog.getResult()[i];
							if (resourceFile.getProjectRelativePath().getFileExtension().equals("ocl"))
							{
								loadRulesInTable(resourceFile.getLocation().toOSString());
							}
						}
					}
				}
				updateMessage();
			}
		});

		oclFileButton = new Button(browseRuleButtonsComposite, SWT.PUSH);
		oclFileButton.setText("Browse an OCL rule file (file system)...");
		GridData button3LayoutData = new GridData(SWT.LEFT, SWT.NONE, false, false);
		button3LayoutData.minimumWidth = 100;
		button3LayoutData.widthHint = oclFileButton.getText().length() * oclFileButton.getFont().getFontData()[0].getHeight() * 2 / 3;
		oclFileButton.setLayoutData(button3LayoutData);
		oclFileButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				String path = new FileDialog(getShell()).open();
				if (path != null)
				{
					loadRulesInTable(path);
				}
				updateMessage();
			}
		});

		final GridData tableLayoutData = new GridData(SWT.FILL, SWT.FILL, true, true);
		tableLayoutData.horizontalSpan = 2;

		rulesTable = new Table(topComposite, SWT.BORDER | SWT.CHECK);
		rulesTable.setLinesVisible(true);
		rulesTable.setHeaderVisible(true);
		rulesTable.getVerticalBar().setEnabled(true);
		rulesTable.setLayoutData(tableLayoutData);
		ruleListCheckboxViewer = new CheckboxTableViewer(rulesTable);

		final TableColumn col1 = new TableColumn(rulesTable, SWT.NONE);
		col1.setWidth(20);
		col1.setResizable(false);

		final TableColumn col2 = new TableColumn(rulesTable, SWT.NONE);
		col2.setText("Context");
		col2.setWidth(100);

		final TableColumn col3 = new TableColumn(rulesTable, SWT.NONE);
		col3.setText("Type");
		col3.setWidth(100);

		final TableColumn col4 = new TableColumn(rulesTable, SWT.NONE);
		col4.setText("Rule");
		col4.setWidth(400);

		ruleListMenu = new Menu(rulesTable);
		MenuItem checkMenuItem = new MenuItem(ruleListMenu, SWT.PUSH);
		checkMenuItem.setText("Check");
		checkMenuItem.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				checkAction();
			}
		});

		rulesTable.setMenu(ruleListMenu);

		Composite selectionButtonsComposite = new Composite(topComposite, SWT.NONE);
		selectionButtonsComposite.setLayout(new GridLayout(2, false));
		GridData button3layoutData = new GridData(SWT.LEFT, SWT.NONE, true, false);
		button3layoutData.minimumWidth = 100;
		button3layoutData.horizontalSpan = 2;
		selectionButtonsComposite.setLayoutData(button3layoutData);

		return topComposite;
	}

	/**
	 * Reads and loads the rule file coming euther from workspace or file system
	 * 
	 * @param fileToRead The path to the rule file to load.
	 */
	private void loadRulesInTable(String fileToRead)
	{
		try
		{
			BufferedReader contentReader = new BufferedReader(new FileReader(fileToRead));
			oclRuleFilePath = fileToRead;

			String buffer = contentReader.readLine();
			String text = "";
			while (buffer != null)
			{
				text = text.concat(buffer + "\n");
				buffer = contentReader.readLine();
			}

			// Add
			if (model.eClass().eContainer() instanceof EPackage)
			{
				EPackage metamodel = (EPackage) model.eClass().eContainer();
				OCLRuleUtil.registerMetaModel(metamodel);
			}

			// Fin Add
			EClassifier booleanType = EcoreEnvironmentFactory.INSTANCE.createEnvironment().getOCLStandardLibrary().getBoolean();
			for (Query<EClassifier, EClass, EObject> query : OCLRuleUtil.loadRulesFromText(text, model, booleanType))
			{
				Constraint constraint = (Constraint) query.getExpression().eContainer().eContainer();
				TableItem line = new TableItem(rulesTable, SWT.NONE);

				// Adds the context (second column of the table)
				line.setText(1, model.eClass().getName());
				line.setText(2, constraint.getStereotype());
				line.setText(3, constraint.getSpecification().getBodyExpression().toString());

				// Sets the query to be able to retrieve it later
				line.setData(query);
			}
		}
		catch (IOException e)
		{
			CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
		}
	}

	/**
	 * Gets the loaded model
	 * 
	 * @return The current model
	 */
	public EObject getModel()
	{
		return model;
	}

	/**
	 * Updates the message if necessary
	 */
	private void updateMessage()
	{
		if (rulesTable.getItemCount() > 0 && model != null)
		{
			getButton(CHECK_BUTTON_ID).setEnabled(true);
			getButton(SELECT_ALL_BUTTON_ID).setEnabled(true);
			getButton(DESELECT_ALL_BUTTON_ID).setEnabled(true);
			ruleListMenu.setEnabled(true);
		}
		else
		{
			getButton(CHECK_BUTTON_ID).setEnabled(false);
			getButton(SELECT_ALL_BUTTON_ID).setEnabled(false);
			getButton(DESELECT_ALL_BUTTON_ID).setEnabled(false);
			ruleListMenu.setEnabled(false);
		}
		if (model == null)
		{
			oclFileButton.setEnabled(false);
			oclFileWorkspaceButton.setEnabled(false);
			if (rulesTable.getItemCount() < 1)
			{
				setMessage("Please select a model and load a rule to check.", DialogPage.ERROR);
			}
			else
			{
				setMessage("Please select a model.", DialogPage.ERROR);
			}
		}
		else
		{
			oclFileButton.setEnabled(true);
			oclFileWorkspaceButton.setEnabled(true);
			if (rulesTable.getItemCount() < 1)
			{
				setMessage("Please load a rule to check.", DialogPage.ERROR);
			}
			else
			{
				setMessage(null, DialogPage.ERROR);
			}
		}
	}

	private void checkAction()
	{
		Object[] checked = ruleListCheckboxViewer.getCheckedElements();
		IService checkerService = ServicesManager.getInstance().getService(IServiceCst.OCL_CHECKER);
		boolean result = true;
		Model resultModel = resultFactory.createModel();
		ModelInformation information = resultFactory.createModelInformation();
		information.setModelContent(model);
		information.setRuleFile(oclRuleFilePath);
		resultModel.setInformation(information);

		for (int ite = 0; ite < checked.length; ite++)
		{
			Query<?, ?, ?> query = (Query<?, ?, ?>) checked[ite];
			OCLExpression<?> rule = query.getExpression();

			Constraint ruleConstraint = (Constraint) rule.eContainer().eContainer();
			EObject constrainedElements = ruleConstraint.getConstrainedElements().get(0);
			EClass ruleContext;
			if (constrainedElements instanceof EClass)
			{
				ruleContext = (EClass) constrainedElements;
			}
			else
			{
				ruleContext = (EClass) constrainedElements.eContainer();
			}

			EPackage rulePackage = ruleContext.getEPackage();

			HashMap<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("Model", model);
			parameters.put("OCLRule", rule);
			try
			{
				Map<?, ?> resultMap = (Map<?, ?>) checkerService.serviceRun(parameters);

				// If the rule was executed with no error, it is added to the resultModel
				// Retrieve or create the package
				Package rulePackageModel = null;
				List<Package> existingPackages = resultModel.getContextPackages();
				for (int i = 0; i < existingPackages.size(); i++)
				{
					Package tempPackage = existingPackages.get(i);
					if (tempPackage.getPackageReference().equals(rulePackage))
					{
						rulePackageModel = tempPackage;
					}
				}
				if (rulePackageModel == null)
				{
					rulePackageModel = resultFactory.createPackage();
					rulePackageModel.setName(rulePackage.getName());
					rulePackageModel.setPackageReference(rulePackage);
					existingPackages.add(rulePackageModel);
				}

				// Retrieves or creates the context
				Context ruleContextModel = null;
				List<Context> existingContexts = rulePackageModel.getContext();
				for (int i = 0; i < existingContexts.size(); i++)
				{
					Context tempContext = existingContexts.get(i);
					if (tempContext.getContextClass().equals(ruleContext))
					{
						ruleContextModel = tempContext;
					}
				}
				
				if (ruleContextModel == null)
				{
					ruleContextModel = resultFactory.createContext();
					ruleContextModel.setName(ruleContext.getName());
					ruleContextModel.setContextClass(ruleContext);
					existingContexts.add(ruleContextModel);
				}

				// Create the rule
				Rule ruleModel = resultFactory.createRule();
				ruleModel.setName(query.queryText());
				ruleModel.setRuleText(query.queryText());
				ruleContextModel.getRules().add(ruleModel);

				// Create and add the rejected items to the rule
				List<?> rejectedList = (List<?>) resultMap.get("rejected");
				for (int i = 0; i < rejectedList.size(); i++)
				{
					Item rejectedItem = resultFactory.createItem();
					rejectedItem.setName(State.REJECTED);
					rejectedItem.setReference((EObject) rejectedList.get(i));
					rejectedItem.setValue(Boolean.FALSE);
					ruleModel.getRejectedItems().add(rejectedItem);
				}

				// Create and add the selected items to the rule
				List<?>  selectedList = (List<?>) resultMap.get("selected");
				for (int i = 0; i < selectedList.size(); i++)
				{
					Item selectedItem = resultFactory.createItem();
					selectedItem.setName(State.SELECTED);
					selectedItem.setReference((EObject) selectedList.get(i));
					selectedItem.setValue(Boolean.TRUE);
					ruleModel.getSelectedItems().add(selectedItem);
				}
				
				// Compute the global result with the current result
				result = result && rejectedList.isEmpty();
				ruleModel.setResult(Boolean.toString(rejectedList.isEmpty()));
			}
			catch (NullPointerException ex)
			{
				CorePlugin.log(getClass(), "Cannot compute the result of the rule : " + query.queryText(), IStatus.WARNING, ex);
			}
		}
		CheckResultDialog resultDialog = new CheckResultDialog(getShell(), resultModel, result, true);
		resultDialog.open();
	}

	private void setMessage(String message, int criticity)
	{
		if (message != null)
		{
			messageLabel.setText(message);
		}
		else
		{
			messageLabel.setText("Select the rules you want to check on the model.");
		}
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent)
	{
		createButton(parent, SELECT_ALL_BUTTON_ID, "Select all", false);

		createButton(parent, DESELECT_ALL_BUTTON_ID, "Deselect all", false);

		createButton(parent, CHECK_BUTTON_ID, "Check...", false);

		createButton(parent, Dialog.CANCEL, "Close", true);

		updateMessage();
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int buttonId)
	{
		if (buttonId == CHECK_BUTTON_ID)
		{
			checkAction();
		}
		else if (buttonId == SELECT_ALL_BUTTON_ID)
		{
			ruleListCheckboxViewer.setAllChecked(true);
		}
		else if (buttonId == DESELECT_ALL_BUTTON_ID)
		{
			ruleListCheckboxViewer.setAllChecked(false);
		}
		else
		{
			super.buttonPressed(buttonId);
		}
	}
}
