package org.topcased.service.oclchecker.dialogs;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.DialogPage;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.ocl.OCL;
import org.eclipse.ocl.ParserException;
import org.eclipse.ocl.Query;
import org.eclipse.ocl.ecore.CallOperationAction;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.ecore.SendSignalAction;
import org.eclipse.ocl.helper.OCLHelper;
import org.eclipse.ocl.types.PrimitiveType;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
import org.eclipse.ui.wizards.newresource.BasicNewFileResourceWizard;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;
import org.topcased.checkresultmodel.CheckresultmodelFactory;
import org.topcased.checkresultmodel.Context;
import org.topcased.checkresultmodel.Model;
import org.topcased.checkresultmodel.ModelInformation;
import org.topcased.checkresultmodel.Package;
import org.topcased.checkresultmodel.Rule;
import org.topcased.editors.ocleditor.editor.OCLEditorDocumentPartitioner;
import org.topcased.editors.ocleditor.utils.OCLRuleUtil;
import org.topcased.service.oclchecker.helper.OCLServiceHelper;
import org.topcased.service.oclchecker.services.OCLStatisticService;

public class StatisticDialog extends Dialog
{

	private static OCLStatisticService statisticService = new OCLStatisticService();

	private EObject model;

	private Text modelText;

	private Button oclFileButton;

	private Button oclFileWorkspaceButton;

	private Label messageLabel;

	private Text descriptionText;

	private TableViewer ruleTableViewer;

	private static final int SAVE = 25;

	private String oclRuleFilePath = "";

	/**
	 * Main Constructor
	 * 
	 * @param parentShell The parent shell
	 */
	public StatisticDialog(Shell parentShell)
	{
		super(parentShell);
		this.model = null;
		setShellStyle(SWT.RESIZE | SWT.MAX | SWT.CLOSE | SWT.SYSTEM_MODAL | SWT.TITLE);
	}

	/**
	 * Second Constructor
	 * 
	 * @param parentShell The parent shell
	 * @param modelFile
	 */
	public StatisticDialog(Shell parentShell, IFile modelFile)
	{
		this(parentShell);
		this.model = OCLRuleUtil.loadModel(modelFile);
	}

	/**
	 * Third Constructor
	 * 
	 * @param parentShell The parent shell
	 * @param model
	 */
	public StatisticDialog(Shell parentShell, EObject model)
	{
		this(parentShell);
		this.model = model;
	}

	/**
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell newShell)
	{
		super.configureShell(newShell);
		newShell.setText("Statistic Dialog");
		newShell.setMinimumSize(400, 600);
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent)
	{
		final Composite topComposite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout(2, false);
		topComposite.setLayout(layout);
		GridData mainLayoutData = new GridData(SWT.FILL, SWT.FILL, true, true);
		topComposite.setLayoutData(mainLayoutData);

		messageLabel = new Label(topComposite, SWT.NONE);
		messageLabel.setText("");
		
		final GridData labelLayoutData = new GridData(SWT.FILL, SWT.NONE, true, false);
		labelLayoutData.minimumWidth = 200;
		labelLayoutData.widthHint = 300;
		labelLayoutData.horizontalSpan = 2;
		messageLabel.setLayoutData(labelLayoutData);

		if (model == null)
		{
			modelText = new Text(topComposite, SWT.BORDER | SWT.SINGLE);
			final GridData textLayoutData = new GridData(SWT.FILL, SWT.CENTER, true, false);
			textLayoutData.minimumWidth = 100;
			textLayoutData.widthHint = 300;
			modelText.setLayoutData(textLayoutData);

			Button modelButton = new Button(topComposite, SWT.PUSH);
			modelButton.setText("Browse a Model to check...");
			final GridData buttonLayoutData = new GridData(SWT.CENTER, SWT.NONE, false, false);
			buttonLayoutData.minimumWidth = 50;
			buttonLayoutData.widthHint = "Browse a Model to check...".length() * modelButton.getFont().getFontData()[0].getHeight() * 2 / 3;
			modelButton.setLayoutData(buttonLayoutData);
			modelButton.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					ResourceSelectionDialog resourceDialog = new ResourceSelectionDialog(getShell(), ResourcesPlugin.getWorkspace()
					        .getRoot(), "Select a model file");
					int result = resourceDialog.open();
					if (result == Window.OK && resourceDialog.getResult().length > 0)
					{
						Object selectedObject = resourceDialog.getResult()[0];
						if (selectedObject instanceof IFile)
						{
							model = OCLRuleUtil.loadModel((IFile) selectedObject);
							modelText.setText(((IFile) selectedObject).getLocation().toString());
							if (model != null)
							{
								ruleTableViewer.getTable().removeAll();
							}
							updateMessage();
						}
					}
				}
			});
		}

		Composite browseRuleButtonsComposite = new Composite(topComposite, SWT.NONE);
		browseRuleButtonsComposite.setLayout(new GridLayout(2, false));
		final GridData compositeLayoutData = new GridData(SWT.LEFT, SWT.NONE, true, false);
		compositeLayoutData.minimumWidth = 100;
		compositeLayoutData.horizontalSpan = 2;
		browseRuleButtonsComposite.setLayoutData(compositeLayoutData);

		oclFileWorkspaceButton = new Button(browseRuleButtonsComposite, SWT.PUSH);
		oclFileWorkspaceButton.setText("Browse OCL rule files (workspace)...");
		final GridData layoutData1 = new GridData(SWT.LEFT, SWT.NONE, false, false);
		layoutData1.minimumWidth = 100;
		layoutData1.widthHint = oclFileWorkspaceButton.getText().length() * oclFileWorkspaceButton.getFont().getFontData()[0].getHeight()
		        * 2 / 3;
		oclFileWorkspaceButton.setLayoutData(layoutData1);
		oclFileWorkspaceButton.addSelectionListener(new SelectionAdapter()
		{

			public void widgetSelected(SelectionEvent event)
			{
				ResourceSelectionDialog resourceDialog = new ResourceSelectionDialog(getShell(), ResourcesPlugin.getWorkspace().getRoot(),
				        "Select an OCL rule file");
				int result = resourceDialog.open();
				if (result == Window.OK && resourceDialog.getResult().length > 0)
				{
					Object[] selectedArray = resourceDialog.getResult();
					for (int i = 0; i < selectedArray.length; i++)
					{
						Object selectedObject = resourceDialog.getResult()[i];
						if (selectedObject instanceof IFile)
						{
							IFile resourceFile = (IFile) selectedObject;
							if (resourceFile.getName().endsWith(".ocl"))
							{
								InputStream content = null;
								try
								{
									content = resourceFile.getContents();
									oclRuleFilePath = resourceFile.getFullPath().toString();
									BufferedReader contentReader = new BufferedReader(new InputStreamReader(content));
									String text = "";

									String buffer = contentReader.readLine();
									while (buffer != null)
									{
										text = text.concat(buffer + "\n");
										buffer = contentReader.readLine();
									}
									content.close();

									loadRulesFromText(text);
								}
								catch (CoreException e)
								{
									CorePlugin.log(e);
								}
								catch (IOException e)
								{
									CorePlugin.log(e);
								}
							}
						}
					}
				}
				updateMessage();
			}

		});

		oclFileButton = new Button(browseRuleButtonsComposite, SWT.PUSH);
		oclFileButton.setText("Browse an OCL rule file (file system)...");
		final GridData layoutData2 = new GridData(SWT.LEFT, SWT.NONE, true, false);
		layoutData2.minimumWidth = 100;
		layoutData2.widthHint = oclFileButton.getText().length() * oclFileButton.getFont().getFontData()[0].getHeight() * 2 / 3;
		oclFileButton.setLayoutData(layoutData2);
		oclFileButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				FileDialog dialog = new FileDialog(getShell());
				String result = dialog.open();
				if (result != null)
				{
					try
					{
						BufferedReader contentReader = new BufferedReader(new FileReader(result));
						String text = "";

						String buffer = contentReader.readLine();
						while (buffer != null)
						{
							text = text.concat(buffer + "\n");
							buffer = contentReader.readLine();
						}

						loadRulesFromText(text);

					}
					catch (IOException ex)
					{
						CorePlugin.log(ex);
					}
				}
				updateMessage();
			}

		});

		Table ruleTable = new Table(topComposite, SWT.V_SCROLL | SWT.FULL_SELECTION);
		final GridData tableLayoutData = new GridData(SWT.FILL, SWT.FILL, true, true);
		tableLayoutData.horizontalSpan = 2;
		ruleTable.setLayoutData(tableLayoutData);
		ruleTable.setHeaderVisible(true);

		TableColumn contextColumn = new TableColumn(ruleTable, SWT.LEFT);
		contextColumn.setMoveable(false);
		contextColumn.setResizable(true);
		contextColumn.setText("Context");
		contextColumn.setWidth(100);

		TableColumn ruleNameColumn = new TableColumn(ruleTable, SWT.LEFT);
		ruleNameColumn.setMoveable(false);
		ruleNameColumn.setResizable(true);
		ruleNameColumn.setText("Rule name");
		ruleNameColumn.setWidth(100);

		TableColumn descriptionColumn = new TableColumn(ruleTable, SWT.LEFT);
		descriptionColumn.setMoveable(false);
		descriptionColumn.setResizable(true);
		descriptionColumn.setText("Description");
		descriptionColumn.setWidth(200);

		TableColumn resultColumn = new TableColumn(ruleTable, SWT.CENTER);
		resultColumn.setMoveable(false);
		resultColumn.setResizable(true);
		resultColumn.setText("Result");
		resultColumn.setWidth(100);

		ruleTableViewer = new TableViewer(ruleTable);
		ruleTableViewer.setLabelProvider(new ITableLabelProvider()
		{

			public Image getColumnImage(Object element, int columnIndex)
			{
				return null;
			}

			public String getColumnText(Object element, int columnIndex)
			{
				// Query queryElement = (Query) element;
				Rule rule = (Rule) element;
				switch (columnIndex)
				{
					case 0:
						return ((Context) rule.eContainer()).getContextClass().getName();
					case 1:
						return rule.getName();
					case 2:
						// TODO change to description
						return rule.getRuleText();
					case 3:
						return rule.getResult().toString();
				}
				return "default";
			}

			public void addListener(ILabelProviderListener listener)
			{
				// Nothing
			}

			public void dispose()
			{
				// Nothing
			}

			public boolean isLabelProperty(Object element, String property)
			{
				// Nothing
				return false;
			}

			public void removeListener(ILabelProviderListener listener)
			{
				// Nothing
			}

		});
		ruleTableViewer.addSelectionChangedListener(new ISelectionChangedListener()
		{

			public void selectionChanged(SelectionChangedEvent event)
			{
				StructuredSelection selection = (StructuredSelection) ruleTableViewer.getSelection();
				Object selected = selection.getFirstElement();
				if (selected != null)
				{
					// TODO change to description
					descriptionText.setText(((Rule) selected).getRuleText());
				}
			}
		});

		Group descriptionGroup = new Group(topComposite, SWT.NONE);
		descriptionGroup.setText("Description");
		final GridData groupLayoutData = new GridData(SWT.FILL, SWT.NONE, true, false);
		groupLayoutData.horizontalSpan = 2;
		groupLayoutData.minimumHeight = 100;
		groupLayoutData.heightHint = 100;
		descriptionGroup.setLayoutData(groupLayoutData);
		descriptionGroup.setLayout(new GridLayout());

		descriptionText = new Text(descriptionGroup, SWT.READ_ONLY | SWT.MULTI);
		descriptionText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		return topComposite;
	}

	private void updateMessage()
	{
		if (model == null)
		{
			oclFileButton.setEnabled(false);
			oclFileWorkspaceButton.setEnabled(false);
			setMessage("Please select a model.", DialogPage.ERROR);
		}
		else
		{
			oclFileButton.setEnabled(true);
			oclFileWorkspaceButton.setEnabled(true);
			if (ruleTableViewer.getTable().getItemCount() < 1)
			{
				setMessage("Please load statistic rules.", DialogPage.ERROR);
			}
			else
			{
				setMessage(null, DialogPage.ERROR);
			}
		}
	}

	private void setMessage(String message, int criticity)
	{
		if (message != null)
		{
			messageLabel.setText(message);
		}
		else
		{
			messageLabel.setText("Select a line to see the description at the bottom of the window.");
		}
	}

	/**
	 * This functions loads every rule found on the string, considering it is the content of a rule file
	 * 
	 * @param text the content of the file
	 */
	private void loadRulesFromText(String text)
	{
		try
		{
			Document oclDocument = new Document(text);
			// If the document has less than 2 lines, then it cannot contain any rule
			// Only documents with rules are interesting here.
			if (oclDocument.getNumberOfLines() > 1)
			{
				String firstLine = oclDocument.get(0, oclDocument.getLineLength(0));
				// If the first line does not begin with "MainModel :", then it cannot be an ocl rule file
				if (firstLine.startsWith("MainModel :"))
				{
					String modelURI = firstLine.substring("MainModel :".length()).trim();
					URI URIModelURI = URI.createURI(modelURI);
					EPackage modelPackage = OCLRuleUtil.findMetamodelPackage(URIModelURI);
					EClass context = null;
					EClassifier contextClassifier = null;
					if (modelPackage != null)
					{
						OCLEditorDocumentPartitioner partitionner = new OCLEditorDocumentPartitioner();
						partitionner.connect(oclDocument);
						ITypedRegion[] regions = partitionner.computePartitioning(0, oclDocument.getLength());
						ITypedRegion packageRegion = regions[0];
						ITypedRegion modelRegion = regions[0];

						for (ITypedRegion typedRegion : Arrays.asList(regions))
						{
							if (typedRegion.getType().equals(OCLEditorDocumentPartitioner.RULE_CONTENT))
							{
								// TODO : Check the meta model is compliant with the model. If not log an ERROR
								if (context != null && (context.getEPackage().getNsURI().equals(model.eClass().getEPackage().getNsURI())))
								{
									String ruleText = oclDocument.get(typedRegion.getOffset(), typedRegion.getLength());
									String name = ruleText.substring(ruleText.indexOf("inv") + 3, ruleText.indexOf(':')).trim();
									ruleText = ruleText.substring(ruleText.indexOf(':') + 1);
									try
									{
										OCL<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> ocl = OCL
										        .newInstance(EcoreEnvironmentFactory.INSTANCE);
										OCLHelper<EClassifier, EOperation, EStructuralFeature, Constraint> helper = ocl.createOCLHelper();
										helper.setContext(context);
										Query<EClassifier, EClass, EObject> query = ocl.createQuery(helper.createQuery(ruleText));

										// The statistic service can only evaluate rules returning an Integer primitive type.
										if ("Integer".equals(query.resultType().getName()))
										{
											Context contextRule = CheckresultmodelFactory.eINSTANCE.createContext();
											contextRule.setContextClass(context);

											Rule rule = CheckresultmodelFactory.eINSTANCE.createRule();
											rule.setRuleText(ruleText.trim());
											rule.setName(name);

											Map<String, Object> parameters = new HashMap<String, Object>();
											parameters.put("OCLRule", query.getExpression());
											parameters.put("Model", model);
											parameters.put("Context", context);
											Object resultObject = statisticService.serviceRun(parameters);
											if (resultObject != null)
											{
												contextRule.getRules().add(rule);
												rule.setResult(resultObject.toString());
												ruleTableViewer.add(rule);
											}
										}
									}
									catch (IllegalArgumentException e)
									{
										CorePlugin.log("Illegal Agument Exception when building the Rule : " + e.getMessage(), IStatus.WARNING);
									}
									catch (ParserException e)
									{
										CorePlugin.log("Error while parsing Rule : " + e.getMessage(), IStatus.ERROR);
									}
								}
							}
							else
							{
								String textTemp = oclDocument.get(typedRegion.getOffset(), typedRegion.getLength());
								if (textTemp.startsWith("package"))
								{
									String modelURIString = textTemp.substring("Package".length()).trim();
									modelURIString = modelURIString.replaceAll("::", "/");
									URI URIpackageURI = URIModelURI.appendFragment(modelURIString);
									modelPackage = OCLRuleUtil.findMetamodelPackage(URIpackageURI);
									packageRegion = typedRegion;
								}
								if (textTemp.startsWith("endpackage"))
								{
									modelPackage = OCLRuleUtil.findMetamodelPackage(URIModelURI);
									packageRegion = modelRegion;
								}
								// try to find a context
								contextClassifier = OCLRuleUtil.computeContextClassifier(oclDocument, modelPackage, typedRegion, packageRegion);
								if (contextClassifier != null && contextClassifier instanceof EClass)
								{
									context = (EClass) contextClassifier;
								}
							}
						}
					}
				}
			}
		}
		catch (BadLocationException e)
		{
			CorePlugin.log(e);
		}
	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent)
	{
		createButton(parent, SAVE, "Save...", false);

		createButton(parent, Dialog.CANCEL, "Close", false);

		updateMessage();

	}

	/**
	 * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
	 */
	protected void buttonPressed(int id)
	{
		if (id == SAVE)
		{
			BasicNewFileResourceWizard wizard = new BasicNewFileResourceWizard();
			wizard.init(PlatformUI.getWorkbench(), StructuredSelection.EMPTY);

			WizardDialog dialog = new WizardDialog(getShell(), wizard)
			{
				protected void finishPressed()
				{
					WizardNewFileCreationPage wizardPage = (WizardNewFileCreationPage) getCurrentPage();
					IPath workspacePath = ResourcesPlugin.getWorkspace().getRoot().getLocation();
					IPath filePath = wizardPage.getContainerFullPath().append(wizardPage.getFileName());
					filePath = workspacePath.append(filePath);

					if (!"checkresultmodel".equals(filePath.getFileExtension()))
					{
						filePath = filePath.addFileExtension("checkresultmodel");
					}

					TableItem[] items = ruleTableViewer.getTable().getItems();
					Model resultModel = CheckresultmodelFactory.eINSTANCE.createModel();
					ModelInformation info = CheckresultmodelFactory.eINSTANCE.createModelInformation();
					resultModel.setInformation(info);
					info.setModelContent(model);
					info.setRuleFile(oclRuleFilePath);
					
					Package contextPackage = CheckresultmodelFactory.eINSTANCE.createPackage();
					resultModel.getContextPackages().add(contextPackage);
					Context context = CheckresultmodelFactory.eINSTANCE.createContext();
					contextPackage.getContext().add(context);
					
					for (int i = 0; i < items.length; i++)
					{
						Rule rule = (Rule) items[i].getData();
						context.getRules().add(rule);
					}

					IService writeService = ServicesManager.getInstance().getService(IServiceCst.WRITE_EMF_XMI);
					Map<String, Object> params = new HashMap<String, Object>();
					params.put("Model", resultModel);
					params.put("PathFileString", filePath.toString());
					writeService.serviceRun(params);

					setReturnCode(OK);
					getWizard().dispose();
					getWizard().setContainer(null);
					super.close();
				}
			};
			dialog.open();
		}
		else
		{
			super.buttonPressed(id);
		}
	}
}
