/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.oclchecker.helper;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.ocl.OCL;
import org.eclipse.ocl.ParserException;
import org.eclipse.ocl.ecore.CallOperationAction;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.ecore.SendSignalAction;
import org.eclipse.ocl.expressions.OCLExpression;
import org.eclipse.ocl.helper.OCLHelper;

/**
* Helper for common code between checker and statistic services.<br>
* Mainly composed by methods accessible by a static way.
* 
* @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
*/
public class OCLServiceHelper
{
	/**
	 * Gets the metamodel corresponding to the URI given in parameter.<br>
	 * The EMF registry is inspected. If the EPackage is not found, the value <b>null</b> is returned..
	 * 
	 * @param mmURI The metamodel of the model
	 * @return The root element of the metamodel
	 */
	public static EPackage getMetaModelPackage(String mmURI)
	{
		return (EPackage) EPackage.Registry.INSTANCE.getEPackage(mmURI);
	}

	/**
	 * Gets the metamodel and looks for the textual context in order to return the corresponding model element.
	 * 
	 * @param mmURI The metamodel of the model
	 * @param context The context of the OCL rule to apply
	 * @return A EClass corresponding to the context in the given metamodel
	 */
	public static EClass getContext(String mmURI, String context)
	{
		return (EClass) getMetaModelPackage(mmURI).getEClassifier(context);
	}

	/**
	 * Creates an OCL Expression 
	 * 
	 * @param mmURI The metamodel of the model
	 * @param context The context of the OCL rule to apply
	 * @param rule The OCL rule to execute
	 * @return An OCL expression
	 * @throws ModelingServiceError If an error occured while the creation of the OCLHelper.
	 */
	public static OCLExpression<EClassifier> createOCLExpression(String mmURI, String context, String rule) throws ParserException
	{
		// Creates the OCL expression
		OCL<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> ocl = OCL
		.newInstance(EcoreEnvironmentFactory.INSTANCE);
		OCLHelper<EClassifier, EOperation, EStructuralFeature, Constraint> helper = ocl.createOCLHelper();
		helper.setContext(getContext(mmURI, context));
		return helper.createInvariant(rule).getSpecification().getBodyExpression();
	}
	
	/**
	 * Creates an OCL Query 
	 * 
	 * @param mmURI The metamodel of the model
	 * @param context The context of the OCL rule to apply
	 * @param rule The OCL rule to execute
	 * @return An OCL expression
	 * @throws ModelingServiceError If an error occured while the creation of the OCLHelper.
	 */
	public static OCLExpression<EClassifier> createOCLQuery(String mmURI, String context, String rule) throws ParserException
	{
		// Creates the OCL query
		OCL<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> ocl = OCL
		.newInstance(EcoreEnvironmentFactory.INSTANCE);
		OCLHelper<EClassifier, EOperation, EStructuralFeature, Constraint> helper = ocl.createOCLHelper();
		helper.setContext(getContext(mmURI, context));
		return helper.createQuery(rule);
	}
}
