/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), S�bastien Gabel (CS)
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *********************************************************************************************/
package org.topcased.service.oclchecker.services;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.ocl.OCL;
import org.eclipse.ocl.Query;
import org.eclipse.ocl.ecore.CallOperationAction;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.ecore.SendSignalAction;
import org.eclipse.ocl.expressions.OCLExpression;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;

/**
 * Represents the checker service.<br>
 * Note : This service is only able to apply one rule.  
 */
public class OCLCheckerService implements IService
{
	/**
	 * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{

		if (parameters.size() != 2)
		{
			CorePlugin.log("The OCL Checker service must have 2 parameters: 'Model' and 'OCLRule'.", IStatus.ERROR);
			return null;
		}

		if (!parameters.containsKey("Model"))
		{
			CorePlugin.log("The 'Model' parameter was not found.", IStatus.ERROR);
			return null;
		}

		if (!parameters.containsKey("OCLRule"))
		{
			CorePlugin.log("The 'OCLRule' parameter was not found.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("Model") instanceof EObject))
		{
			CorePlugin.log("The 'Model' parameter must be an EObject.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("OCLRule") instanceof OCLExpression))
		{
			CorePlugin.log("The 'OCLRule' parameter must be an OCLExpression.", IStatus.ERROR);
			return null;
		}

		CorePlugin.log("Begin Check Service. Parameters are OK.", IStatus.INFO);

		// TODO OCLExpression oclRule = (OCLExpression) parameters.get("OCLRule");
		OCLExpression<EClassifier> oclRule = (OCLExpression<EClassifier>) parameters.get("OCLRule");

		// TODO EClass context = (EClass) ((Constraint) oclRule.eContainer()).getConstrainedElement().get(0);
		EObject constrainedElements = ((Constraint) oclRule.eContainer().eContainer()).getConstrainedElements().get(0);
		EClass context;
		if (constrainedElements instanceof EClass)
		{
			context = (EClass) constrainedElements;
		}
		else
		{
			context = (EClass) constrainedElements.eContainer();
		}

		EObject model = (EObject) parameters.get("Model");
		List<EObject> objects = new ArrayList<EObject>();
		for (Iterator<EObject> ite = model.eAllContents(); ite.hasNext();)
		{
			EObject temp = (EObject) ite.next();
			// TODO see if the comparison can be improved
			if (temp.eClass().equals(context) || temp.eClass().getEAllSuperTypes().contains(context)
			        || EcoreUtil.getURI(context).equals(EcoreUtil.getURI(temp.eClass())))
			{
				objects.add(temp);
			}
		}

		if (model.eClass().equals(context) || model.eClass().getEAllSuperTypes().contains(context))
		{
			objects.add(model);
		}

		// TODO Query query = QueryFactory.eINSTANCE.createQuery(oclRule);
		OCL<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> ocl
		// = OCL.newInstance(EcoreEnvironmentFactory.INSTANCE.createInstanceContext(EcoreEnvironmentFactory.INSTANCE.createEnvironment(), model));
		// = OCL.newInstance(new EcoreEnvironmentFactory(model.eResource().getResourceSet().getPackageRegistry()));
		= OCL.newInstance(EcoreEnvironmentFactory.INSTANCE.createEnvironment());

		Query<EClassifier, EClass, EObject> query = ocl.createQuery(oclRule);

		CorePlugin.log("Checking rule : " + query.queryText(), IStatus.INFO);

		if (query.resultType() == ocl.getEnvironment().getOCLStandardLibrary().getBoolean())
		{
			Map<String, Object> toReturn = new HashMap<String, Object>();
			Map<EClass, ? extends Set<? extends EObject>> extent = query.getExtentMap();
			extent = fillExtentMap(extent, context, model);
			// TODO query.setExtentMap(extent);
			ocl.setExtentMap(extent);
			Query<EClassifier, EClass, EObject> newQuery = ocl.createQuery(oclRule);

			toReturn.put("selected", newQuery.select(objects));
			toReturn.put("rejected", newQuery.reject(objects));

			return toReturn;
		}
		else
		{
			CorePlugin.log("The rule is not boolean. It cannot be checked.", IStatus.WARNING);
			return null;
		}
	}

	public static Map<EClass, ? extends Set<? extends EObject>> fillExtentMap(Map<EClass, ? extends Set<? extends EObject>> extentMap,
	        EClass context, EObject model)
	{

		Map<EClass, Set<? extends EObject>> localExtendMal = new HashMap<EClass, Set<? extends EObject>>();
		localExtendMal.putAll(extentMap);

		EPackage ePackage = context.getEPackage();
		HashSet<EObject> objects;
		List<EClassifier> listClassifiers = ePackage.getEClassifiers();
		for (int i = 0; i < listClassifiers.size(); i++)
		{
			EClassifier currentClassifier = (EClassifier) listClassifiers.get(i);
			objects = new HashSet<EObject>();
			for (Iterator<EObject> ite = model.eAllContents(); ite.hasNext();)
			{
				EObject temp = ite.next();
				if (temp.eClass().equals(currentClassifier) || temp.eClass().getEAllSuperTypes().contains(currentClassifier))
				{
					objects.add(temp);
				}
			}

			if (model.eClass().equals(currentClassifier) || model.eClass().getEAllSuperTypes().contains(currentClassifier))
			{
				objects.add(model);
			}

			// TODO extentMap.put(currentClassifier, objects);
			if (currentClassifier instanceof EClass)
			{
				EClass currentClass = (EClass) currentClassifier;
				localExtendMal.put(currentClass, objects);
			}

		}

		return localExtendMal;
	}

}
