package org.topcased.service.oclchecker.services;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.ocl.OCL;
import org.eclipse.ocl.Query;
import org.eclipse.ocl.ecore.CallOperationAction;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.ecore.SendSignalAction;
import org.eclipse.ocl.expressions.OCLExpression;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.checkresultmodel.CheckresultmodelFactory;
import org.topcased.checkresultmodel.Context;
import org.topcased.checkresultmodel.Item;
import org.topcased.checkresultmodel.Model;
import org.topcased.checkresultmodel.ModelInformation;
import org.topcased.checkresultmodel.Package;
import org.topcased.checkresultmodel.Rule;
import org.topcased.checkresultmodel.State;
import org.topcased.editors.ocleditor.utils.OCLRuleUtil;

public class OCLFileCheckService implements IService
{

	public Object serviceRun(Object[] parameters)
	{
		return null;
	}

	public Object serviceRun(Map<String,Object> parameters)
	{
		if (parameters.size() != 2)
		{
			CorePlugin.log("The OCL Checker service must have 2 parameters: 'Model' and 'OCLFile'.", IStatus.ERROR);
			return null;
		}

		if (!parameters.containsKey("Model"))
		{
			CorePlugin.log("The 'Model' parameter was not found.", IStatus.ERROR);
			return null;
		}

		if (!parameters.containsKey("OCLFile"))
		{
			CorePlugin.log("The 'OCLFile' parameter was not found.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("Model") instanceof EObject))
		{
			CorePlugin.log("The 'Model' parameter must be an EObject.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("OCLFile") instanceof String))
		{
			CorePlugin.log("The 'OCLFile' parameter must be a String.", IStatus.ERROR);
			return null;
		}

		Path filePath = new Path((String) parameters.get("OCLFile"));

		EObject model = (EObject) parameters.get("Model");

		try
		{
			BufferedReader br = new BufferedReader(new FileReader(filePath.toFile()));

			String text = "";
			String line = br.readLine();
			while (line != null)
			{
				text = text.concat(line + "\n");
				line = br.readLine();
			}

			
			EClassifier booleanType = EcoreEnvironmentFactory.INSTANCE.createEnvironment().getOCLStandardLibrary().getBoolean();
			List<Query<EClassifier, EClass, EObject>> rules = OCLRuleUtil.loadRulesFromText(text, model, booleanType);
			
			CheckresultmodelFactory factory = CheckresultmodelFactory.eINSTANCE;
			
			Model resultModel = factory.createModel();
			ModelInformation information = factory.createModelInformation();
			information.setModelContent(model);
			information.setRuleFile(filePath.toOSString());
			resultModel.setInformation(information);

			Package resultPackage = factory.createPackage();
			resultPackage.setPackageReference(model.eClass().getEPackage());
			resultModel.getContextPackages().add(resultPackage);

			for (int i = 0; i < rules.size(); i++)
			{

				Query<EClassifier, EClass, EObject> currentQuery = rules.get(i);
				OCLExpression<EClassifier> rule = currentQuery.getExpression();

				Context resultContext = null;
				Constraint ruleConstraint = (Constraint) rule.eContainer();
				EClass ruleContext = (EClass) ruleConstraint.getConstrainedElements().get(0);

				List<Context> contextList = resultPackage.getContext();
				for (int j = 0; j < contextList.size() && resultContext == null; j++)
				{
					Context currentContext = (Context) contextList.get(j);
					if (currentContext.getContextClass().equals(ruleContext))
					{
						resultContext = currentContext;
					}
				}

				if (resultContext == null)
				{
					resultContext = factory.createContext();
					resultContext.setContextClass(ruleContext);
					resultPackage.getContext().add(resultContext);
				}

				Rule resultRule = factory.createRule();
				resultRule.setName(currentQuery.queryText());
				resultRule.setRuleText(currentQuery.queryText());
				resultContext.getRules().add(resultRule);

				List<EObject> objects = new ArrayList<EObject>();
				for (Iterator<EObject> ite = model.eAllContents(); ite.hasNext();)
				{
					EObject temp = (EObject) ite.next();
					// TODO see if the comparison can be improved
					if (temp.eClass().equals(ruleContext) || temp.eClass().getEAllSuperTypes().contains(ruleContext)
							|| EcoreUtil.getURI(ruleContext).equals(EcoreUtil.getURI(temp.eClass())))
					{
						objects.add(temp);
					}
				}

				if (model.eClass().equals(ruleContext) || model.eClass().getEAllSuperTypes().contains(ruleContext))
				{
					objects.add(model);
				}

				Map<EClass, ? extends Set<? extends EObject>> extent = currentQuery.getExtentMap();
				extent = OCLCheckerService.fillExtentMap(extent, ruleContext, model);
				OCL<EPackage, EClassifier, EOperation, EStructuralFeature,EEnumLiteral, EParameter,	EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> ocl
				= OCL.newInstance(EcoreEnvironmentFactory.INSTANCE.createEnvironment());
				ocl.setExtentMap(extent);				
				currentQuery= ocl.createQuery(currentQuery.getExpression());
		       
				
				List<EObject> resultList = currentQuery.select(objects);
				for (int j = 0; j < resultList.size(); j++)
				{
					Item item = factory.createItem();
					item.setName(State.SELECTED);
					item.setReference((EObject) resultList.get(j));
					item.setValue(Boolean.TRUE);
					resultRule.getSelectedItems().add(item);
				}

				resultList = currentQuery.reject(objects);
				for (int j = 0; j < resultList.size(); j++)
				{
					Item item = factory.createItem();
					item.setName(State.REJECTED);
					item.setReference((EObject) resultList.get(j));
					item.setValue(Boolean.FALSE);
					resultRule.getRejectedItems().add(item);
				}
			}

			return resultModel;

		}
		catch (FileNotFoundException e)
		{
			CorePlugin.log("File not found : " + filePath, IStatus.ERROR);
			CorePlugin.log(e);
		}
		catch (IOException e)
		{
			CorePlugin.log("Error reading file : " + filePath, IStatus.ERROR);
			CorePlugin.log(e);
		}

		return null;
	}

}
