package org.topcased.service.oclchecker.services;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.ocl.OCL;
import org.eclipse.ocl.Query;
import org.eclipse.ocl.ecore.CallOperationAction;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.ecore.SendSignalAction;
import org.eclipse.ocl.expressions.OCLExpression; //import org.eclipse.emf.ocl.expressions.OCLExpression;
//import org.eclipse.emf.ocl.query.Query;
//import org.eclipse.emf.ocl.query.QueryFactory;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;

/**
 * Represents the statistic service.<br>
 * Note : This service is only able to apply one rule.  
 */
public class OCLStatisticService implements IService
{
	/**
	 * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{
		if (parameters.size() != 3)
		{
			CorePlugin.log("The OCL Checker service must have 2 parameters: 'Model' and 'OCLRule'.", IStatus.ERROR);
			return null;
		}

		if (!parameters.containsKey("Model"))
		{
			CorePlugin.log("The 'Model' parameter was not found.", IStatus.ERROR);
			return null;
		}

		if (!parameters.containsKey("OCLRule"))
		{
			CorePlugin.log("The 'OCLRule' parameter was not found.", IStatus.ERROR);
			return null;
		}

		if (!parameters.containsKey("Context"))
		{
			CorePlugin.log("The 'Context' parameter was not found.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("Model") instanceof EObject))
		{
			CorePlugin.log("The 'Model' parameter must be an EObject.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("OCLRule") instanceof OCLExpression))
		{
			CorePlugin.log("The 'OCLRule' parameter must be a EClass.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("Context") instanceof EClass))
		{
			CorePlugin.log("The 'Context' parameter must be a Context.", IStatus.ERROR);
			return null;
		}

		CorePlugin.log("Begin Statistic Service. Parameters are OK.", IStatus.INFO);

		OCLExpression<EClassifier> oclRule = (OCLExpression<EClassifier>) parameters.get("OCLRule");
		EClass context = (EClass) (parameters.get("Context"));// ((Constraint) oclRule.eContainer()).getConstrainedElement().get(0);

		EObject model = (EObject) parameters.get("Model");
		List<EObject> objects = new ArrayList<EObject>();
		for (Iterator<EObject> ite = model.eAllContents(); ite.hasNext();)
		{
			EObject temp = (EObject) ite.next();
			if (temp.eClass().equals(context) || temp.eClass().getEAllSuperTypes().contains(context))
			{
				objects.add(temp);
			}
		}

		if (model.eClass().equals(context) || model.eClass().getEAllSuperTypes().contains(context))
		{
			objects.add(model);
		}

		// TODO Query query = QueryFactory.eINSTANCE.createQuery(oclRule);
		OCL<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> ocl = OCL
		        .newInstance(EcoreEnvironmentFactory.INSTANCE);
		Query<EClassifier, EClass, EObject> query = ocl.createQuery(oclRule);

		CorePlugin.log("Statistic rule : " + query.queryText(), IStatus.INFO);

		// TODO Map extents = query.getExtentMap();
		// extents = OCLCheckerService.fillExtentMap(extents,context,model);
		// query.setExtentMap(extents);
		Map<EClass, ? extends Set<? extends EObject>> extent = query.getExtentMap();
		extent = OCLCheckerService.fillExtentMap(extent, context, model);
		ocl.setExtentMap(extent);
		Query<EClassifier, EClass, EObject> newQuery = ocl.createQuery(oclRule);
		// ????? query.setExtentMap(extent); ????

		try
		{
			return newQuery.evaluate();
		}
		catch (NullPointerException e)
		{
			CorePlugin
			        .log("Rules using 'self' cannot be used for statistics. This rule is ignored : " + query.queryText(), IStatus.WARNING);
			return null;
		}
		catch (Exception e)
		{
			CorePlugin.log("Exception found while computing the result of a statistic rule : " + query.queryText(), IStatus.ERROR);
			CorePlugin.log(e);
			return null;
		}
	}

}
