/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sebastien Gabel (CS)
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *********************************************************************************************/
package org.topcased.service.oclchecker.services.external;

import java.io.File;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.ocl.OCL;
import org.eclipse.ocl.ecore.CallOperationAction;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.ecore.SendSignalAction;
import org.eclipse.ocl.expressions.OCLExpression;
import org.eclipse.ocl.helper.OCLHelper;
import org.osgi.framework.Bundle;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;
import org.topcased.checkresultmodel.CheckresultmodelFactory;
import org.topcased.checkresultmodel.Context;
import org.topcased.checkresultmodel.Item;
import org.topcased.checkresultmodel.Model;
import org.topcased.checkresultmodel.ModelInformation;
import org.topcased.checkresultmodel.Package;
import org.topcased.checkresultmodel.Rule;
import org.topcased.checkresultmodel.State;
import org.topcased.externalbus.core.CorePlugin;
import org.topcased.externalbus.core.IExternalService;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class ExternalCheckService implements IExternalService
{

	private final static String PROJECT_WORKSPACE = "HttpServerWorkspace";

	public Object externalServiceRun(Map<String, Object> parameters)
	{

		if (!parameters.containsKey("Model"))
		{
			CorePlugin.log(this.getClass(), "The 'Model' parameter was not found.", IStatus.ERROR);
			return "";
		}

		if (!parameters.containsKey("Rule"))
		{
			CorePlugin.log(this.getClass(), "The 'Rule' parameter was not found.", IStatus.ERROR);
			return "";
		}

		if (!parameters.containsKey("Context"))
		{
			CorePlugin.log(this.getClass(), "The 'Rule' parameter was not found.", IStatus.ERROR);
			return "";
		}

		if (!parameters.containsKey("MMURI"))
		{
			CorePlugin.log(this.getClass(), "The 'MMURI' parameter was not found.", IStatus.ERROR);
			return "";
		}

		if (!(parameters.get("Model") instanceof Element))
		{
			CorePlugin.log(this.getClass(), "The 'Model' parameter must be a model.", IStatus.ERROR);
			return "";
		}

		if (!(parameters.get("Rule") instanceof String))
		{
			CorePlugin.log(this.getClass(), "The 'Rule' parameter must be a string.", IStatus.ERROR);
			return "";
		}

		if (parameters.containsKey("Package") && !(parameters.get("Package") instanceof String))
		{
			CorePlugin.log(this.getClass(), "The 'Package' parameter must be a string.", IStatus.ERROR);
			return "";
		}

		if (!(parameters.get("Context") instanceof String))
		{
			CorePlugin.log(this.getClass(), "The 'Context' parameter must be a string.", IStatus.ERROR);
			return "";
		}

		if (!(parameters.get("MMURI") instanceof String))
		{
			CorePlugin.log(this.getClass(), "The 'MMURI' parameter must be a string.", IStatus.ERROR);
			return "";
		}

		Element modelAsElement = (Element) parameters.get("Model");
		Document modelDocument;
		String contextPackage = (String) parameters.get("Package");
		String context = (String) parameters.get("Context");
		String ruleText = (String) parameters.get("Rule");
		String mmUri = (String) parameters.get("MMURI");
		try
		{
			modelDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
			modelDocument.appendChild(modelDocument.importNode(modelAsElement, true));

			DOMSource source = new DOMSource(modelDocument);
			StreamResult result = new StreamResult(new File("tempModel.xmi"));

			Transformer transformer = TransformerFactory.newInstance().newTransformer();
			transformer.transform(source, result);

			EPackage metamodelPackage = null;

			IExtensionRegistry registry = Platform.getExtensionRegistry();
			IExtensionPoint extensionPoint = registry.getExtensionPoint("org.eclipse.emf.ecore.generated_package");
			IExtension[] extensions = extensionPoint.getExtensions();
			boolean modelFound = false;
			for (int i = 0; i < extensions.length && !modelFound; i++)
			{
				IExtension extension = extensions[i];
				IConfigurationElement[] elements = extension.getConfigurationElements();
				for (int j = 0; j < elements.length && !modelFound; j++)
				{
					if (elements[j].getAttribute("uri").equals(mmUri))
					{
						modelFound = true;
						String className = elements[j].getAttribute("class");
						Bundle bundle = Platform.getBundle(extension.getNamespaceIdentifier());
						Class<?> packageClass = bundle.loadClass(className);
						Field field = packageClass.getDeclaredField("eINSTANCE");
						metamodelPackage = (EPackage) field.get(null);
					}
				}
			}
			if (!modelFound)
			{
				CorePlugin.log("The user model could not be found", IStatus.ERROR);
				return "";
			}

			if (contextPackage != null && !contextPackage.equals(""))
			{
				String[] subpackages = contextPackage.split(":");
				List<String> packagePrefixes = new ArrayList<String>();
				for (int i = 0; i < subpackages.length; i++)
				{
					if (!subpackages[i].equals(""))
						packagePrefixes.add(subpackages[i]);
				}
				if (packagePrefixes.size() != 0)
				{
					if (metamodelPackage.getNsPrefix().equals(packagePrefixes.get(0)))
					{
						EPackage tempPackage = metamodelPackage;
						boolean packageError = false;
						for (int i = 1; i < packagePrefixes.size() && !packageError; i++)
						{
							List<EPackage> subPackages = tempPackage.getESubpackages();
							boolean packageFound = false;
							for (int j = 0; j < subPackages.size() && !packageFound; j++)
							{
								EPackage subi = subPackages.get(j);
								if (subi.getNsPrefix().equals(packagePrefixes.get(i)))
								{
									packageFound = true;
									tempPackage = subi;
								}
							}
							if (!packageFound)
							{
								packageError = true;
							}
						}
						if (!packageError)
						{
							metamodelPackage = tempPackage;
						}
					}
				}
			}

			EClass contextClass = (EClass) metamodelPackage.getEClassifier(context);

			// TODO OCLExpression rule = ExpressionsUtil.createInvariant(contextClass, ruleText, true);
			OCL<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> ocl = OCL
			        .newInstance(EcoreEnvironmentFactory.INSTANCE);
			OCLHelper<EClassifier, EOperation, EStructuralFeature, Constraint> helper = ocl.createOCLHelper();
			helper.setContext(contextClass);
			OCLExpression<EClassifier> rule = helper.createInvariant(ruleText).getSpecification().getBodyExpression();

			IService readService = ServicesManager.getInstance().getService(IServiceCst.READ_XMI_EMF);

			HashMap<String, Object> readParameters = new HashMap<String, Object>();
			readParameters.put("XMIFile", "tempModel.xmi");

			EObject model = (EObject) readService.serviceRun(readParameters);

			IService checkService = ServicesManager.getInstance().getService(IServiceCst.OCL_CHECKER);
			HashMap<String, Object> checkParameters = new HashMap<String, Object>();
			checkParameters.put("Model", model);
			checkParameters.put("OCLRule", rule);

			Map<?, ?> resultMap = (Map<?, ?>) checkService.serviceRun(checkParameters);
			List<?> rejectedItems = (List<?>) resultMap.get("rejected");

			Model modelResult = CheckresultmodelFactory.eINSTANCE.createModel();

			ModelInformation information = CheckresultmodelFactory.eINSTANCE.createModelInformation();
			information.setModelContent(model);
			modelResult.setInformation(information);

			Package contextPackageModel = CheckresultmodelFactory.eINSTANCE.createPackage();
			contextPackageModel.setPackageReference(metamodelPackage);
			modelResult.getContextPackages().add(contextPackageModel);

			Context contextModel = CheckresultmodelFactory.eINSTANCE.createContext();
			contextModel.setContextClass(contextClass);
			contextPackageModel.getContext().add(contextModel);

			Rule ruleModel = CheckresultmodelFactory.eINSTANCE.createRule();
			ruleModel.setRuleText(ruleText);
			contextModel.getRules().add(ruleModel);

			for (int i = 0; i < rejectedItems.size(); i++)
			{
				Item rejected = CheckresultmodelFactory.eINSTANCE.createItem();
				rejected.setName(State.REJECTED);
				rejected.setReference((EObject) rejectedItems.get(i));
				rejected.setValue(Boolean.FALSE);
				ruleModel.getRejectedItems().add(rejected);
			}

			IService writeService = ServicesManager.getInstance().getService(IServiceCst.WRITE_EMF_XMI);
			HashMap<String, Object> writeParameters = new HashMap<String, Object>();
			writeParameters.put("Model", modelResult);
			writeParameters.put("PathFileString", ResourcesPlugin.getWorkspace().getRoot().getProject(PROJECT_WORKSPACE)
			        .getProjectRelativePath()
			        + "HttpServerWorkspace/result.xmi");
			writeService.serviceRun(writeParameters);

			return ResourcesPlugin.getWorkspace().getRoot().getProject(PROJECT_WORKSPACE).getLocation().toOSString() + File.separator
			        + "result.xmi";

		}
		catch (Exception e)
		{
			CorePlugin.log(e);
		}
		return "";
	}
}
