/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.plugincustomization.epf.exportwizard;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.preferences.IPreferenceFilter;
import org.eclipse.core.runtime.preferences.IPreferencesService;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.internal.WorkbenchPlugin;
import org.eclipse.ui.internal.wizards.preferences.PreferencesMessages;
import org.eclipse.ui.internal.wizards.preferences.WizardPreferencesPage;

/**
 * TODO : WARNING COPY OF THE FILE
 * 'org.eclipse.ui.internal.wizards.preferences.WizardPreferencesExportPage1'
 * PreferencesFileExportPage take the path of the file .epf created
 * 
 */
public class PreferencesFileExportPage extends WizardPreferencesPage implements Listener
{

    // constants
    private static final String PREFERENCESEXPORTPAGE1 = "preferencesExportPage1"; //$NON-NLS-1$

    // TODO Add
    private String pathEpf = "";

    /**
     * Create an instance of this class
     * @param name
     */
    protected PreferencesFileExportPage(String name)
    {
        super(name);
        setTitle(PreferencesMessages.WizardPreferencesExportPage1_exportTitle);
        setDescription(PreferencesMessages.WizardPreferencesExportPage1_exportDescription);
    }

    /**
     * Create an instance of this class
     * 
     */
    public PreferencesFileExportPage()
    {
        this(PREFERENCESEXPORTPAGE1);//$NON-NLS-1$
    }

    /**
     * 
     * @see org.eclipse.ui.internal.wizards.preferences.WizardPreferencesPage#getAllButtonText()
     */
    protected String getAllButtonText()
    {
        return PreferencesMessages.WizardPreferencesExportPage1_all;
    }

    /**
     * 
     * @see org.eclipse.ui.internal.wizards.preferences.WizardPreferencesPage#getChooseButtonText()
     */
    protected String getChooseButtonText()
    {
        return PreferencesMessages.WizardPreferencesExportPage1_choose;
    }

    /**
     * @param composite
     * 
     */
    protected void createTransferArea(Composite composite)
    {
        createTransfersList(composite);
        createDestinationGroup(composite);
        createOptionsGroup(composite);
    }

    /**
     * Answer the string to display in self as the destination type
     * 
     * @return java.lang.String
     */
    protected String getDestinationLabel()
    {
        return PreferencesMessages.WizardPreferencesExportPage1_file;
    }

    /**
     * @param transfers
     * @return <code>true</code> if the transfer was succesful, and
     *         <code>false</code> otherwise
     */
    protected boolean transfer(IPreferenceFilter[] transfers)
    {
        File exportFile = new File(getDestinationValue());
        if (!ensureTargetIsValid(exportFile))
        {
            return false;
        }
        FileOutputStream fos = null;
        try
        {
            if (transfers.length > 0)
            {
                try
                {
                    fos = new FileOutputStream(exportFile);
                }
                catch (FileNotFoundException e)
                {
                    WorkbenchPlugin.log(e.getMessage(), e);
                    return false;
                }
                IPreferencesService service = Platform.getPreferencesService();
                try
                {
                    service.exportPreferences(service.getRootNode(), transfers, fos);
                }
                catch (CoreException e)
                {
                    WorkbenchPlugin.log(e.getMessage(), e);
                    return false;
                }
            }
        }
        finally
        {
            if (fos != null)
                try
                {
                    fos.close();
                }
                catch (IOException e)
                {
                    WorkbenchPlugin.log(e.getMessage(), e);
                }
        }

        // TODO when finish clic
        pathEpf = getDestinationValue();

        return true;
    }

    /**
     * 
     * @see org.eclipse.ui.internal.wizards.preferences.WizardPreferencesPage#getFileDialogTitle()
     */
    protected String getFileDialogTitle()
    {
        return PreferencesMessages.WizardPreferencesExportPage1_title;
    }

    /**
     * 
     * @see org.eclipse.ui.internal.wizards.preferences.WizardPreferencesPage#getFileDialogStyle()
     */
    protected int getFileDialogStyle()
    {
        return SWT.SAVE;
    }

    /**
     * TODO Add
     * @return EPF path
     */
    public String getEpfFilePath()
    {
        String returnPath = "";
        if (pathEpf != null)
        {
            returnPath = pathEpf;
        }
        return returnPath;
    }

	protected String getInvalidDestinationMessage() {
		// TODO Auto-generated method stub
		return null;
	}
}
