/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.plugincustomization.epf.exportwizard;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IExportWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.wizards.preferences.PreferencesMessages;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.topcased.facilities.dialogs.ITopcasedWizardConstants;
import org.topcased.service.plugincustomization.PlugincustomizationPlugin;

/**
 * TODO : WARNING COPY OF THE FILE
 * 'org.eclipse.ui.internal.wizards.preferences.PreferencesExportWizard'
 * PreferencesFileExportWizard call PreferencesFileExportPage wich take the path
 * of the file .epf created
 * 
 */

/**
 * Standard workbench wizard for exporting preferences from the workspace to the
 * local file system.
 * <p>
 * This class may be instantiated and used without further configuration; this
 * class is not intended to be subclassed.
 * </p>
 * <p>
 * Example:
 * 
 * <pre>
 * IWizard wizard = new PreferencesExportWizard();
 * wizard.init(workbench, selection);
 * WizardDialog dialog = new WizardDialog(shell, wizard);
 * dialog.open();
 * </pre>
 * 
 * During the call to <code>open</code>, the wizard dialog is presented to
 * the user. When the user hits Finish, the user-selected workspace preferences
 * are exported to the user-specified location in the local file system, the
 * dialog closes, and the call to <code>open</code> returns.
 * </p>
 * 
 * @since 3.1
 * 
 */
public class PreferencesFileExportWizard extends Wizard implements IExportWizard, ITopcasedWizardConstants
{

    private PreferencesFileExportPage mainPage;

    /**
     * Creates a wizard for exporting workspace preferences to the local file
     * system.
     */
    public PreferencesFileExportWizard()
    {
        AbstractUIPlugin plugin = (AbstractUIPlugin) Platform.getPlugin(PlatformUI.PLUGIN_ID);
        IDialogSettings workbenchSettings = plugin.getDialogSettings();
        IDialogSettings section = workbenchSettings.getSection("PreferencesExportWizard");
        if (section == null)
        {
            section = workbenchSettings.addNewSection("PreferencesExportWizard");
        }
        setDialogSettings(section);
    }

    /**
     * @see org.eclipse.jface.wizard.IWizard#addPages()
     */
    public void addPages()
    {
        super.addPages();
        mainPage = new PreferencesFileExportPage();
        addPage(mainPage);
    }

    /**
     * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
     */
    public void init(IWorkbench workbench, IStructuredSelection currentSelection)
    {
        setWindowTitle(PreferencesMessages.PreferencesExportWizard_export);
        
        setDefaultPageImageDescriptor(PlugincustomizationPlugin.getImageDescriptor(TOPCASED_LOGO_PATH));
        
        setNeedsProgressMonitor(true);
    }

    /**
     * @see org.eclipse.jface.wizard.IWizard#performFinish()
     */
    public boolean performFinish()
    {
        return mainPage.finish();
    }
}
