/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.plugincustomization.services;

import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.service.plugincustomization.PlugincustomizationPlugin;
import org.topcased.service.plugincustomization.epf.TransformEpf;

/**
 * PluginCustomizationCreation is a Service of the Topcased Bus.<br>
 * It createsthe file <b>'plugin_customization.ini'</b> from the Eclipse Preferences file finishing with the extension <b>'*.epf'</b>
 * 
 */
public class PluginCustomizationCreation implements IService
{
	/**
	 * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{
		// Bloc Declaration
		TransformEpf transformEpf = null;
		String pathPCI = "";
		String pathEPF = "";
		boolean isCreatePci = false;

		// Map null
		if (parameters == null)
		{
			CorePlugin.log(this.getClass(), PlugincustomizationPlugin
					.getResourceString("PluginCustomizationCreation.log.parameter.error.number"), IStatus.ERROR);
			return null;
		}

		// Get 'PluginCustomizationPath' parameter
		if (parameters.containsKey("PluginCustomizationPath"))
		{
			if (parameters.get("PluginCustomizationPath") instanceof String)
			{

				pathPCI = (String) parameters.get("PluginCustomizationPath");
			}
			else
			{
				CorePlugin.log(this.getClass(), PlugincustomizationPlugin
						.getResourceString("PluginCustomizationCreation.log.parameter.error.PCIpath.type"), IStatus.ERROR);
			}
		}
		else
		{
			CorePlugin.log(this.getClass(), PlugincustomizationPlugin
					.getResourceString("PluginCustomizationCreation.log.parameter.error.PCIpath"), IStatus.ERROR);
		}

		// Get 'EclipsePreferencesPath' parameter
		if (parameters.containsKey("EclipsePreferencesPath"))
		{
			if (parameters.get("EclipsePreferencesPath") instanceof String)
			{

				pathEPF = (String) parameters.get("EclipsePreferencesPath");
			}
			else
			{
				CorePlugin.log(this.getClass(), PlugincustomizationPlugin
						.getResourceString("PluginCustomizationCreation.log.parameter.error.EPFpath.type"), IStatus.ERROR);
			}
		}
		else
		{
			CorePlugin.log(this.getClass(), PlugincustomizationPlugin
					.getResourceString("PluginCustomizationCreation.log.parameter.error.EPFpath"), IStatus.ERROR);
		}

		// SERVICE CREATE PLUGIN CUSTOMIZATION
		// transform .epf to .ini
		transformEpf = new TransformEpf(pathEPF, pathPCI);
		isCreatePci = transformEpf.CreatePluginCustomization();

		// Display Information
		if (isCreatePci)
		{
			CorePlugin.log(this.getClass(), PlugincustomizationPlugin
					.getResourceString("PluginCustomizationCreation.log.parameter.service.info.PCI")
					+ " "
					+ pathPCI
					+ " "
					+ PlugincustomizationPlugin.getResourceString("PluginCustomizationCreation.log.parameter.service.info.EPF")
					+ " "
					+ pathEPF, IStatus.INFO);
		}
		else
		{
			CorePlugin.log(this.getClass(), PlugincustomizationPlugin
					.getResourceString("PluginCustomizationCreation.log.parameter.service.error")
					+ pathPCI, IStatus.ERROR);
		}

		return null;
	}

}
