/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.plugincustomization.wizard;

import java.io.File;
import java.util.HashMap;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.topcased.service.plugincustomization.PlugincustomizationPlugin;
import org.topcased.service.plugincustomization.services.EclipsePreferencesCreation;

/**
 * EclipsePreferencePage is a Wizard page. <br>
 * This interface permits : 
 * <ul>
 * <li>- to create '.epf' file (using Eclipse Export wizard)</li>
 * <li>- to take the path of the '.epf' file</li>
 * </ul>
 */
public class EclipsePreferencePage extends WizardPage
{

    // Wizard
    private PluginCustomizationWizard wizard;

    // Path of Eclipse Preferences (.epf)
    private String aPathEpf = "";

    // Page Finish
    private boolean finish = false;

    // Text path of Eclipse Preferences (.epf)
    private Text textPathEpf = null;

    /**
     * Constructor
     * 
     * @param pageName : name of the page
     * @param pciWizard : wizard which call this page
     */
    protected EclipsePreferencePage(String pageName, PluginCustomizationWizard pciWizard)
    {
        super(pageName);
        wizard = pciWizard;
    }

    /**
     * Create Control
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent)
    {
        final Composite composite = new Composite(parent, SWT.NONE);
        composite.setLayout(new GridLayout());
        setControl(composite);

        // Section to create a .epf file thanks to Eclipse
        createSection(composite);
        
        // Label
        createLabel(PlugincustomizationPlugin.getResourceString("EclipsePreferencePage.label"), composite);

        // Browse Preferences file (.epf)
        createBrowseEpf(composite);

        setPageComplete(false);

    }

    /**
     * 
     * Finishs
     * 
     * @return true if the page can be finished
     */
    public boolean finish()
    {
        return finish;
    }

    /**
     * Creates Section : path template repository
     * 
     * @param parent
     */
    private void createBrowseEpf(Composite parent)
    {
        final Composite browseComposite = new Composite(parent, SWT.NONE);
        browseComposite.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
        browseComposite.setLayout(new GridLayout(2, false));
        
        textPathEpf = new Text(browseComposite, SWT.BORDER | SWT.READ_ONLY);
        textPathEpf.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        textPathEpf.setText(aPathEpf);
        
        final Button browseButton = new Button(browseComposite, SWT.PUSH);
        browseButton.setText(PlugincustomizationPlugin.getResourceString("EclipsePreferencePage.button.browse"));
        browseButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                handleBrowseEpf();
            }
        });
    }

    /**
     * Creates Label
     * 
     * @param text
     * @param parent
     */
    private void createLabel(String text, Composite parent)
    {
        final Label label = new Label(parent, SWT.NONE);
        label.setText(text);

    }

    /**
     * Creates Section : Create .epf from Elipse
     * 
     * @param parent
     */
    private void createSection(Composite parent)
    {
        final Composite compositeCreateEpf = new Composite(parent, SWT.NONE);
        compositeCreateEpf.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
        compositeCreateEpf.setLayout(new GridLayout());
        final Label epfDescription = new Label(compositeCreateEpf, SWT.NONE);
        epfDescription.setText(PlugincustomizationPlugin.getResourceString("EclipsePreferencePage.section.description"));


        // Button Create : component of 'section'
        final Button buttonCreateEpf = new Button(compositeCreateEpf, SWT.NONE);
        buttonCreateEpf.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, true));
        buttonCreateEpf.setText(PlugincustomizationPlugin.getResourceString("EclipsePreferencePage.button.createEPF"));
        buttonCreateEpf.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                handleCreateEpfFile();
            }
        });
    }

    /**
     * Browses Preferences Eclipse File : '.epf' in a File Dialog
     */
    private void handleBrowseEpf()
    {
        FileDialog dialogBrowse = new FileDialog(getShell());
        dialogBrowse.setFilterPath(aPathEpf);
        dialogBrowse.setFilterExtensions(new String[] {"*.epf"});
        String dialogOpen = dialogBrowse.open();
        // Browse Dialog is opened
        if (dialogOpen != null)
        {
            // Take the path of the file
            String filePath = dialogBrowse.getFilterPath() + File.separator + dialogBrowse.getFileName();
            aPathEpf = filePath;
            handlePathEpf();
        }
    }

    /**
     * Displays the path epf in the text field
     */
    private void handlePathEpf()
    {
        if (!aPathEpf.equals(""))
        {
            // Verify '.epf' (not necessary : it is do in the browse)
            String epfString = aPathEpf.substring(aPathEpf.lastIndexOf('.'));
            if (epfString.equals(".epf"))
            {
                finish = true;
                setPageComplete(true);
                // Display in the text filed
                textPathEpf.setText(aPathEpf);
                // Give parameter to Wizard
                wizard.setEpfPath(aPathEpf);
                // Un-Display message
                wizard.getEclipsePreferencePage().setMessage(PlugincustomizationPlugin.getResourceString("EclipsePreferencePage.message.description"));
            }
            else
            {
                // Display message
                wizard.getEclipsePreferencePage().setMessage(
                        PlugincustomizationPlugin.getResourceString("EclipsePreferencePage.message.error.pathEPF"),
                        WizardPage.ERROR);
            }
        }
        else
        {
            // Display message
            wizard.getEclipsePreferencePage().setMessage(
                    PlugincustomizationPlugin.getResourceString("EclipsePreferencePage.message.missing.pathEPF"), 
                    WizardPage.ERROR);
        }
    }

    /**
     * Creates EPF File Plug with funtionality of Elipse.
     */
    private void handleCreateEpfFile()
    {
        // Call SERVICE CreateEpf
        EclipsePreferencesCreation epfCreation = new EclipsePreferencesCreation();
        aPathEpf = (String) epfCreation.serviceRun(new HashMap<String, Object>());
        handlePathEpf();
    }

}
