/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/
package org.topcased.service.plugincustomization.wizard;

import org.eclipse.jface.wizard.Wizard;
import org.topcased.facilities.dialogs.ITopcasedWizardConstants;
import org.topcased.service.plugincustomization.PlugincustomizationPlugin;

/**
 * PluginCustomizationWizard open a wizard which create the file
 * 'plugin_customization.ini' from Eclipse Preferences file (.epf)
 * 
 */
public class PluginCustomizationWizard extends Wizard implements ITopcasedWizardConstants
{
	// Page plugin-customization.ini
    private PluginCustomizationPage pciPage;

    // Page Eclipse Preferences (.epf)
    private EclipsePreferencePage epfPage;

    // Path of the .epf file
    private String epfPath = "";

    // Path of the plugin_customization.ini
    private String pciPath = "";

    /**
     * Constructor
     */
    public PluginCustomizationWizard()
    {
        super();
        
        // Image
        setDefaultPageImageDescriptor(PlugincustomizationPlugin.getImageDescriptor(TOPCASED_LOGO_PATH));

        // Title
        setWindowTitle(PlugincustomizationPlugin.getResourceString("PluginCustomizationWizard.wizard.title"));
    }

    
    /**
     * Add Pages
     */
    public void addPages()
    {
        epfPage = new EclipsePreferencePage("EPF Page", this);
        epfPage.setTitle(PlugincustomizationPlugin.getResourceString("PluginCustomizationWizard.epfPage.title"));
        epfPage.setDescription(PlugincustomizationPlugin.getResourceString("EclipsePreferencePage.message.description"));
        addPage(epfPage);

        pciPage = new PluginCustomizationPage("PCI Page", this);
        pciPage.setTitle(PlugincustomizationPlugin.getResourceString("PluginCustomizationWizard.pciPage.title"));
        pciPage.setDescription(PlugincustomizationPlugin.getResourceString("PluginCustomizationPage.message.description"));
        addPage(pciPage);
    }

    /**
     * Perform Finish
     */
    public boolean performFinish()
    {
        return epfPage.finish() && pciPage.finish();
    }

    /**
     * @return PluginCustomizationPage
     */
    public PluginCustomizationPage getPluginCustomizationPage()
    {
        return pciPage;
    }

    /**
     * @return EclipsePreferencePage
     */
    public EclipsePreferencePage getEclipsePreferencePage()
    {
        return epfPage;
    }

    /**
     * @return Path of the plugin-customization.ini
     */
    public String getPciPath()
    {
        return pciPath;
    }

    /**
     * @return Path of the Eclipse Preferences file (.epf)
     */
    public String getEpfPath()
    {
        return epfPath;
    }

    /**
     * Set the path of the plugin-customization.ini
     */
    public void setPciPath(String pathPci)
    {
        pciPath = pathPci;
    }

    /**
     * Set the path of the Eclipse Preferences file (.epf)
     */
    public void setEpfPath(String pathEpf)
    {
        epfPath = pathEpf;
    }

}
