/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.projecttotemplate.action;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.topcased.service.projecttotemplate.wizard.ProjectToTemplateWizard;

/**
 * ProjectToTemplateAction is a popup action on a repository.
 * It launch wizard to create a template xml file from current repository
 *
 */
public class ProjectToTemplateAction implements IObjectActionDelegate {
     
    // Project Path selected
    private IPath _pathProject;

    /**
     * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart)
    {
        // Nothing to do
    }
    
    /**
     * Call a wizard in order to create a template xml file from current repository.
     * @see IActionDelegate#run(IAction)
     */
    public void run(IAction action)
    {
        Shell shell = new Shell();
        ProjectToTemplateWizard wizard = new ProjectToTemplateWizard(_pathProject);
        WizardDialog newWizardDialog = new WizardDialog(shell, wizard);
        newWizardDialog.open();
    }
    
    /**
     * Take the repository path selected
     * @see IActionDelegate#selectionChanged(IAction, ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection)
    {
        IStructuredSelection structured = (IStructuredSelection)selection;
        Object object = structured.getFirstElement();
        // Retrieve the path of the project
        if (object instanceof IProject)
        {
            _pathProject = new Path(((IProject)object).getLocation().toOSString());
        }
    }

}
