/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.projecttotemplate.infos;

import java.util.HashMap;
import java.util.Map;

/**
 * InfosDirectory contains : <br>
 * <ul>
 * <li>a directory name 'directoryName',</li>
 * <li>a directory path 'directoryPath'</li>
 * </ul>
 * It takes informations on its sons (directory and file) : -
 * Map 'mapInfosDirectory' (key=name ; value=InfosDirectory), - Map
 * 'mapInfosFile' (key=name ; value=InfosFile)
 * 
 */
public class InfosDirectory
{
    // Infos Directory Map
    private Map<String, InfosDirectory> mapInfosDirectory = new HashMap<String, InfosDirectory>();

    // Infos File Map
    private Map<String, InfosFile> mapInfosFile = new HashMap<String, InfosFile>();

    // Name Directory
    private String directoryName = "";

    // Path Directory
    private String directoryPath = "";

    /**
     * Constructor
     * 
     * @param nameDirectory
     * @param pathDirectory
     */
    public InfosDirectory(String nameDirectory, String pathDirectory)
    {
        directoryName = nameDirectory;
        directoryPath = pathDirectory;
    }

    /**
     * Constructor
     * 
     * @param nameDirectory
     * @param pathDirectory
     * @param infosDirectoryMap
     * @param infosFileMap
     */
    public InfosDirectory(String nameDirectory, String pathDirectory, Map<String, InfosDirectory> infosDirectoryMap, Map<String, InfosFile> infosFileMap)
    {
        mapInfosDirectory = infosDirectoryMap;
        mapInfosFile = infosFileMap;
        directoryName = nameDirectory;
        directoryPath = pathDirectory;
    }

    /**
     * Sets directory name nameDirectory
     * 
     * @param nameDirectory
     */
    public void setDirectoryName(String nameDirectory)
    {
        directoryName = nameDirectory;
    }

    /**
     * Sets directory path pathDirectory
     * 
     * @param pathDirectory
     */
    public void setDirectoryPath(String pathDirectory)
    {
        directoryPath = pathDirectory;
    }

    /**
     * Sets InfosDirectory Map
     * 
     * @param infosDirectoryMap : Map of InfosDirectory (key=name ;
     *            value=InfosDirectory)
     */
    public void setInfosDirectoryMap(Map<String, InfosDirectory> infosDirectoryMap)
    {
        mapInfosDirectory = infosDirectoryMap;
    }

    /**
     * Sets infosFileMap Map
     * 
     * @param infosFileMap : Map of infosFileMap (key=name ; value=InfosFile)
     */
    public void setInfosFileMap(Map<String, InfosFile> infosFileMap)
    {
        mapInfosFile = infosFileMap;
    }

    /**
     * Gets directory name nameDirectory
     * 
     * @return directoryName
     */
    public String getDirectoryName()
    {
        return directoryName;
    }

    /**
     * Gets directory path pathDirectory
     * 
     * @return directoryPath
     */
    public String getDirectoryPath()
    {
        return directoryPath;
    }

    /**
     * Gets mapInfosDirectory
     * 
     * @return mapInfosDirectory (key=name ; value=InfosDirectory)
     */
    public Map<String, InfosDirectory> getMapInfosDirectory()
    {
        return mapInfosDirectory;
    }

    /**
     * Gets mapInfosFile
     * 
     * @return mapInfosFile (key=pluginId ; value=InfosFile)
     */
    public Map<String, InfosFile> getMapInfosFile()
    {
        return mapInfosFile;
    }
}
