/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.projecttotemplate.infos.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.topcased.bus.core.CorePlugin;
import org.topcased.service.projecttotemplate.ProjectToTemplatePlugin;
import org.topcased.service.projecttotemplate.infos.InfosDirectory;
import org.topcased.service.projecttotemplate.infos.InfosFile;
import org.topcased.service.projecttotemplate.infos.InfosTemplateProject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * InfosTemplateProjectToTemplate create a xml file template and a zip file (.xml + .zip) Transform a InfosTemplateProject in a xml file. <template name="..."
 * pathzip="..."> <project> <directory name="..."> [directory * | file *] </directory> * <file name="..."/> * </project> </template> Create a zip file with all
 * files referenced like selected in InfosTemplateProject. InfosTemplateProjectToTemplate is used by InfosTemplateProject.
 * 
 */
public class InfosTemplateProjectToTemplate
{

	// *********************
	// Declaration bloc
	// *********************
	// Infos Directory Map
	private Map<String, InfosDirectory> mapInfosDirectory = new HashMap<String, InfosDirectory>();

	// Infos File Map
	private Map<String, InfosFile> mapInfosFile = new HashMap<String, InfosFile>();

	// Template Name
	private String templateName = "";

	// Template Path ZIP File
	private String templatePathZipFile = "";

	// Template Name
	private String oldProjectName = "";

	// Filtre
	private static String EXTSERVICES = ".services";

	/**
	 * Constructor
	 * 
	 * @param infosTemplateProject : InfosTemplateProject which must transform in Template
	 */
	public InfosTemplateProjectToTemplate(InfosTemplateProject infosTemplateProject, String oldNameProject)
	{
		mapInfosDirectory = infosTemplateProject.getMapInfosDirectory();
		mapInfosFile = infosTemplateProject.getMapInfosFile();
		templateName = infosTemplateProject.getTemplateName();
		templatePathZipFile = infosTemplateProject.getTemplatePathZipFile();
		oldProjectName = oldNameProject;
	}

	// **************************************************************
	//
	// Build XML file with InfosTemplateProject
	//
	// **************************************************************

	/**
	 * Create about the template project templatePath
	 * 
	 * @param templatePath : path of the xml file (.xml)
	 * @param zipFilePath : path of the zip file (.zip)
	 * @return true if '.xml' files is created
	 */
	public boolean createTemplateProject(String templatePath, String zipFilePath)
	{
		boolean isCreate = false;
		boolean continu = true;
		templatePathZipFile = zipFilePath;
		File destination = new File(templatePath);
		// File '.xml' already exist?
		if ((destination != null) && destination.exists())
		{
			continu = MessageDialog.openQuestion(new Shell(), "TOPCASED Question", "The file " + templatePath
					+ " already exists, do you want to overwrite it?");
		}
		if (continu)
		{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			try
			{
				DocumentBuilder builder = factory.newDocumentBuilder();
				Document document = builder.newDocument();
				// Tag creation : <template name="..." pathzip="...">
				Element templateTag = document.createElement(InfosTemplateProject.TEMPLATE);
				document.appendChild(templateTag);
				templateTag.setAttribute(InfosTemplateProject.TEMPLATENAME, templateName);
				templateTag.setAttribute(InfosTemplateProject.TEMPLATEPATHZIPFILE, templatePathZipFile);

				// Tag <project>
				Element projectTag = document.createElement(InfosTemplateProject.PROJECT);
				templateTag.appendChild(projectTag);
				projectTag.setAttribute(InfosTemplateProject.PROJECTOLDNAME, oldProjectName);

				// Tag <file name="...">
				createTagFile(mapInfosFile, projectTag, document);

				// Tag <directory name="...">
				createTagDirectory(mapInfosDirectory, projectTag, document);

				// Xml write
				TransformerFactory tFactory = TransformerFactory.newInstance();
				Transformer transformer = tFactory.newTransformer();
				// Indent the XML document
				transformer.setOutputProperty(OutputKeys.INDENT, "yes");
				transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "3");
				DOMSource sourceDom = new DOMSource(document);
				StreamResult result = new StreamResult(templatePath);
				transformer.transform(sourceDom, result);
				isCreate = true;

			}
			catch (ParserConfigurationException e)
			{
				CorePlugin.log(this.getClass(), ProjectToTemplatePlugin
						.getResourceString("InfosTemplateProjectToTemplate.log.exception.parseconfiguration")
						+ " " + e.getMessage(), IStatus.ERROR);
			}
			catch (TransformerConfigurationException e)
			{
				CorePlugin.log(this.getClass(), ProjectToTemplatePlugin
						.getResourceString("InfosTemplateProjectToTemplate.log.exception.transformerconfiguration")
						+ " " + e.getMessage(), IStatus.ERROR);
			}
			catch (TransformerException e)
			{
				CorePlugin.log(this.getClass(), ProjectToTemplatePlugin
						.getResourceString("InfosTemplateProjectToTemplate.log.exception.transformer")
						+ " " + e.getMessage(), IStatus.ERROR);
			}
			catch (Exception e)
			{
				CorePlugin.log(this.getClass(), ProjectToTemplatePlugin.getResourceString("InfosTemplateProjectToTemplate.log.exception")
						+ " " + e.getMessage(), IStatus.ERROR);
			}
		}
		return isCreate;

	}

	/**
	 * Create tag "DIRECTORY"
	 * 
	 * @param mapInfoDirectory : Map of InfoDirectory (key=name, value = InfosDirectory)
	 * @param tagParent
	 * @param document
	 */
	private void createTagDirectory(Map<String, InfosDirectory> mapInfoDirectory, Element tagParent, Document document)
	{
		for (String infoDir : mapInfoDirectory.keySet())
		{
			// <directory name="...">
			InfosDirectory infosDirectory = mapInfoDirectory.get(infoDir);
			Element directoryTag = document.createElement(InfosTemplateProject.DIRECTORY);
			tagParent.appendChild(directoryTag);
			directoryTag.setAttribute(InfosTemplateProject.DIRECTORYNAME, infosDirectory.getDirectoryName());
			// <directory>
			createTagDirectory(infosDirectory.getMapInfosDirectory(), directoryTag, document);
			// <file>
			createTagFile(infosDirectory.getMapInfosFile(), directoryTag, document);
		}
	}

	/**
	 * Create tag "FILE"
	 * 
	 * @param mapInfoFile : Map of InfoFile (key=name, value = InfosFile)
	 * @param tagParent
	 * @param document
	 */
	private void createTagFile(Map<String, InfosFile> mapInfoFile, Element tagParent, Document document)
	{
		for (String infoFile : mapInfoFile.keySet())
		{
			// <file name="...">
			InfosFile infosFile = mapInfoFile.get(infoFile);
			String fileName = infosFile.getFileName();
			if (isValideExtension(fileName))
			{
				Element fileTag = document.createElement(InfosTemplateProject.FILE);
				tagParent.appendChild(fileTag);
				fileTag.setAttribute(InfosTemplateProject.FILENAME, fileName);
			}
		}
	}

	// **************************************************************
	//
	// Create ZipFile (.zip) with InfosTemplateProject
	//
	// **************************************************************

	/**
	 * Create a zip file from the selected directory
	 * 
	 * @param pathSource : path of the zip file to create(.zip)
	 * @param pathDestination : path of the directory to zip
	 * @return true if is '.zip' file is created
	 */
	public boolean createZipFile(String pathSource, String pathDestination)
	{
		boolean isCreated = false;
		boolean continu = true;
		if (pathSource != null && !pathSource.equals(""))
		{
			File pathFile = new File(pathSource);
			File[] files = pathFile.listFiles();
			File destination = new File(pathDestination);

			// File '.zip' already exist?
			if ((destination != null) && destination.exists())
			{
				continu = MessageDialog.openQuestion(new Shell(), "TOPCASED Question", "The file " + pathDestination
						+ " already exists, do you want to overwrite it?");
			}
			if (continu)
			{
				try
				{
					FileOutputStream outputStream = new FileOutputStream(pathDestination);
					ZipOutputStream output = new ZipOutputStream(outputStream);
					zipFile(files, output, ""); // TODOpathFile.getName() +
					output.close();
					isCreated = true;
				}
				catch (FileNotFoundException e)
				{
					CorePlugin.log(this.getClass(), ProjectToTemplatePlugin
							.getResourceString("InfosTemplateProjectToTemplate.log.exception.filenotfound")
							+ " " + e.getMessage(), IStatus.ERROR);
				}
				catch (IOException e)
				{
					CorePlugin.log(this.getClass(), ProjectToTemplatePlugin
							.getResourceString("InfosTemplateProjectToTemplate.log.exception.io")
							+ " " + e.getMessage(), IStatus.ERROR);
				}
			}
		}
		else
		{
			CorePlugin.log(this.getClass(), ProjectToTemplatePlugin.getResourceString("InfosTemplateProjectToTemplate.log.exception.io")
					+ " " + pathSource + " "
					+ ProjectToTemplatePlugin.getResourceString("InfosTemplateProjectToTemplate.log.error.filenotexist"), IStatus.ERROR);
		}
		return isCreated;

	}

	/**
	 * Create a zip file from the selected directory and with files selected
	 * 
	 * @param pathSource : path of the zip file to create(.zip)
	 * @param pathDestination : path of the directory to zip
	 */
	public void createZipFileSelected(String pathSource, String pathDestination)
	{
		// TODO Put only file selected

	}

	/**
	 * Zip File
	 * 
	 * @param files : all files to zip
	 * @param output : destination jar
	 * @param path : path in the jar
	 * @throws IOException
	 */
	private void zipFile(File[] files, ZipOutputStream output, String path) throws IOException
	{
		for (int i = 0; i < files.length; i++)
		{
			File file = files[i];
			String fileName = file.getName();

			ZipEntry entry = new ZipEntry(path + fileName);

			if (file.isDirectory())
			{
				// DIRECTORY
				zipFile(file.listFiles(), output, path + file.getName() + File.separator);
			}
			else
			{
				// FILE
				if (isValideExtension(fileName))
				{
					FileInputStream input = new FileInputStream(file);
					output.putNextEntry(entry);
					// Copy
					byte[] buffer = new byte[1024];
					int length;
					while ((length = input.read(buffer)) != -1)
					{
						output.write(buffer, 0, length);
					}
					input.close();
				}

			}

		}
	}

	/**
	 * Valide the file name
	 * 
	 * @param fileName
	 * @return true if the file is valid
	 */
	private boolean isValideExtension(String fileName)
	{
		boolean isOK = false;
		String extension = "";
		int index = fileName.indexOf('.');
		// Extension verification
		if (index > 0)
		{
			extension = fileName.substring(index);
			if (extension != null && !extension.equals(EXTSERVICES))
			{
				isOK = true;
			}
		}
		else
		{
			isOK = true;
		}
		return isOK;
	}
}
