/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.projecttotemplate.infos.util;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.topcased.service.projecttotemplate.infos.InfosDirectory;
import org.topcased.service.projecttotemplate.infos.InfosFile;


/**
 * ProjectToInfosTemplateProject build : - a
 * mapInfosDirectory(name/InfosDirectory) - a mapInfosFile(name/InfosFile).
 * There are built tanks to the project path. ProjectToInfosTemplateProject is
 * used by InfosTemplateProject.
 * 
 */
public class ProjectToInfosTemplateProject
{
    /**
     * Declaration bloc
     */
    // Infos Directory Map
    private Map<String, InfosDirectory> mapInfosDirectory = new HashMap<String, InfosDirectory>(); 
                                                   

    // Infos File Map
    private Map<String, InfosFile> mapInfosFile = new HashMap<String, InfosFile>();

    /**
     * Get mapInfosDirectory
     * 
     * @return mapInfosDirectory : Map of InfosDirectory (key=name ;
     *         value=InfosDirectory)
     */
    public Map<String, InfosDirectory> getMapInfosDirectory()
    {
        return mapInfosDirectory;
    }

    /**
     * Get mapInfosFile
     * 
     * @return mapInfosFile : Map of InfosFile (key=name ; value=InfosFile)
     */
    public Map<String, InfosFile> getMapInfosFile()
    {
        return mapInfosFile;
    }

    // **************************************************************
    //
    // Build InfosTemplateProject from a existing project
    //
    // **************************************************************
    /**
     * Get InfosTemplateProject from a repository path. Initialize
     * 'mapInfosDirectory' and 'mapInfosFile'
     * 
     * @param directoryPath
     * @return true if informations of the project are correctly taken 
     */
    public boolean getInfosTemplateProjectFromDirectory(String directoryPath)
    {
        boolean success = true;
        File directoryFile = new File(directoryPath);
        if (directoryFile.exists())
        {
            File[] listFile = directoryFile.listFiles();
            success = getInfosProject(listFile, mapInfosDirectory, mapInfosFile, "");
        }
        return success;
    }

    /**
     * 
     * @param listFile
     * @param infosDirectoryMap : Map of InfoDirectory (key=name, value =
     *            InfosDirectory)
     * @param infosFileMap : Map of InfoFile (key=name, value = InfosFile)
     * @param relativePath
     * @return true if informations are correctly taken
     */
    private boolean getInfosProject(File[] listFile, Map<String, InfosDirectory> infosDirectoryMap, Map<String, InfosFile> infosFileMap, String relativePath)
    {
        boolean success = true;
        String pathRelative = "";
        String name = "";
        int i = 0;
        while (success && (i < listFile.length))
        {
            File file = listFile[i];
            i = i + 1;
            if (file.isDirectory())
            {
                name = file.getName();
                pathRelative = relativePath + File.separator + name;
                Map<String, InfosDirectory> infosDirectoryMap2 = new HashMap<String, InfosDirectory>();
                Map<String, InfosFile> infosFileMap2 = new HashMap<String, InfosFile>();
                success = getInfosProject(file.listFiles(), infosDirectoryMap2, infosFileMap2, pathRelative);
                if (!success)
                {
                    return success;
                }
                InfosDirectory infosD = new InfosDirectory(name, pathRelative, infosDirectoryMap2, infosFileMap2);
                infosDirectoryMap.put(infosD.getDirectoryName(), infosD);
            }
            else if (file.isFile())
            {
                name = file.getName();
                pathRelative = relativePath + File.separator + name;
                InfosFile infosF = new InfosFile(name, pathRelative);
                infosFileMap.put(infosF.getFileName(), infosF);
            }
        }

        return success;
    }
}
