/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.projecttotemplate.preferences;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.topcased.service.projecttotemplate.ProjectToTemplatePlugin;

public class ProjectToTemplatePreferencePage extends PreferencePage implements IWorkbenchPreferencePage{

	private String aTemplateXMLDirectory = ""; 

	private Text repositoryPath;
	
	private IPreferenceStore preferenceStore = ProjectToTemplatePlugin.getDefault().getPreferenceStore();
	
	public static final String TEMPLATE_REPOSITORY_PATH = "TemplateRepositoryPath";
	
	public ProjectToTemplatePreferencePage()
	{
        setPreferenceStore(preferenceStore);
        setDescription("Project to Template preference page");
	}
	
	protected Control createContents(Composite parent)
	{
        // Layout for the main composite  
		final GridLayout mainLayout = new GridLayout();
		mainLayout.numColumns = 1;
		mainLayout.marginHeight = 0;
		mainLayout.marginWidth = 0;
        
        // Grid layout for the text
        final GridData textGridData = new GridData(SWT.FILL, SWT.NONE , true, false);
        textGridData.widthHint = 150;

        final Composite mainComposite = new Composite(parent, SWT.NONE);
        mainComposite.setLayout(mainLayout);
        mainComposite.setLayoutData(new GridData(SWT.FILL, SWT.NONE , true, false));
        
		final Group group = new Group(mainComposite, SWT.SHADOW_OUT);
        group.setText("XML files repository path");
		group.setLayoutData(new GridData(SWT.FILL, SWT.NONE , true, false));
        group.setLayout(new GridLayout(2, false));
  
	    repositoryPath = new Text(group, SWT.BORDER | SWT.READ_ONLY);
        repositoryPath.setText(preferenceStore.getString(TEMPLATE_REPOSITORY_PATH));
        repositoryPath.setLayoutData(textGridData);
        
        Button browseButton = new Button(group, SWT.NONE);
        browseButton.setText("Browse...");
        browseButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                handleBrowseTemplateDirectory(repositoryPath);
            }
        });

        return mainComposite;
	}

	public void init(IWorkbench workbench)
	{
		// TODO Auto-generated method stub
	}
	
	private void handleBrowseTemplateDirectory(Text text)
    {
        DirectoryDialog dialogBrowse = new DirectoryDialog(getShell());
        dialogBrowse.setFilterPath(aTemplateXMLDirectory);
        String dialogOpen = dialogBrowse.open();
        // Browse Dialog is opened
        if (dialogOpen != null)
        {
            // Takes the path of the repository
            text.setText(dialogOpen);
            aTemplateXMLDirectory = dialogOpen;
        }
    }
	
	public boolean performOk()
	{
		//store data now
		preferenceStore.setValue(TEMPLATE_REPOSITORY_PATH,repositoryPath.getText());
		return super.performOk();
	}
	
	public void performDefaults()
	{
		repositoryPath.setText(preferenceStore.getString(TEMPLATE_REPOSITORY_PATH));
		super.performDefaults();
	}
	
	public void performApply()
	{
		preferenceStore.setValue(TEMPLATE_REPOSITORY_PATH,repositoryPath.getText());
		super.performApply();
	}
}
