/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.projecttotemplate.service;

import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.service.projecttotemplate.ProjectToTemplatePlugin;
import org.topcased.service.projecttotemplate.infos.InfosTemplateProject;

/**
 * Service of the BUS : Create a template (.xml + .zip) from a project selected.<br>
 */
public class CreateTemplate implements IService
{
    /**
     * Create a template (.xml + .zip) from a project selected.
     * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
     */
    public Object serviceRun(Map<String, Object> parameters)
    {
        // Bloc Declaration
        InfosTemplateProject infosTemplateProject = null;
        String pathTemplateZip = "";
        String pathTemplateXml = "";
        String pathProject = "";
        String templateName = "";

        // Map null
        if (parameters == null)
        {
            CorePlugin.log(this.getClass(),
                    ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.parameter.error.number"), 
                    IStatus.ERROR);
            return null;
        }

        // Get 'TemplateName' parameter
        if (parameters.containsKey("TemplateName"))
        {
            if (parameters.get("TemplateName") instanceof String)
            {
                templateName = (String) parameters.get("TemplateName");
            }
            else
            {
                CorePlugin.log(this.getClass(), 
                        ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.parameter.error.templateName.type"),
                        IStatus.ERROR);
            }
        }
        else
        {
            CorePlugin.log(this.getClass(),
                    ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.parameter.error.templateName"),
                    IStatus.ERROR);
        }

        // Get 'pathTemplateXml' parameter
        if (parameters.containsKey("PathTemplateXml"))
        {
            if (parameters.get("PathTemplateXml") instanceof String)
            {
                pathTemplateXml = (String) parameters.get("PathTemplateXml");
            }
            else
            {
                CorePlugin.log(this.getClass(), 
                        ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.parameter.error.pathxml.type"), 
                        IStatus.ERROR);
            }
        }
        else
        {
            CorePlugin.log(this.getClass(),
                    ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.parameter.error.pathxml"), 
                    IStatus.ERROR);
        }

        // Get 'pathTemplateZip' parameter
        if (parameters.containsKey("PathTemplateZip"))
        {
            if (parameters.get("PathTemplateZip") instanceof String)
            {
                pathTemplateZip = (String) parameters.get("PathTemplateZip");
            }
            else
            {
                CorePlugin.log(this.getClass(), 
                        ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.parameter.error.pathzip.type"), 
                        IStatus.ERROR);
            }
        }
        else
        {
            CorePlugin.log(this.getClass(),
                    ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.parameter.error.pathzip"), 
                    IStatus.ERROR);
        }

        // Get 'PathProject' parameter
        if (parameters.containsKey("PathProject"))
        {
            if (parameters.get("PathProject") instanceof String)
            {
                pathProject = (String) parameters.get("PathProject");
            }
            else
            {
                CorePlugin.log(this.getClass(), 
                        ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.parameter.error.pathproject.type"), 
                        IStatus.ERROR);
            }
        }
        else
        {
            CorePlugin.log(this.getClass(),
                    ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.parameter.error.pathproject"), 
                    IStatus.ERROR);
        }
        

        // SERVICE CREATE TEMPLATE
        if (!templateName.equals("") && !pathProject.equals(""))
        {
            infosTemplateProject = new InfosTemplateProject(templateName, pathProject);
        }
        if (infosTemplateProject != null && !pathTemplateZip.equals("") && !pathTemplateXml.equals("")
                && !pathProject.equals(""))
        {
            boolean createXml = false;
            boolean createZip = false;
            
            // Create xml file : Template Project
            createXml = infosTemplateProject.createTemplateProject(pathTemplateXml, pathTemplateZip);
            if (createXml)
            {
                CorePlugin.log(this.getClass(), 
                        ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.service.info.xmlcreation") 
                        + " " + pathProject + " "
                        + ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.service.in")
                        + " " + pathTemplateXml, 
                        IStatus.INFO);
            }
            else
            {
                CorePlugin.log(this.getClass(), 
                        ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.service.error.xmlcreation") 
                        + " " + pathProject + " "
                        + ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.service.in")
                        + " " + pathTemplateXml, 
                        IStatus.ERROR);
            }

            // Create zip file
            createZip = infosTemplateProject.createZipFile(pathProject, pathTemplateZip);
            if (createZip)
            {
                CorePlugin.log(this.getClass(), 
                        ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.service.info.zipcreation")
                        + " " + pathProject + " "
                        + ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.service.in")
                        + " " + pathTemplateZip, 
                        IStatus.INFO);
            }
            else
            {
                CorePlugin.log(this.getClass(), 
                        ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.service.error.zipcreation") 
                        + " " + pathProject + " "
                        + ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.service.in")
                        + " " + pathTemplateXml, 
                        IStatus.ERROR);
            }
        }
        else
        {
            CorePlugin.log(this.getClass(), 
                    ProjectToTemplatePlugin.getResourceString("CreateTemplate.log.service.error")
                    + " " + pathProject, 
                    IStatus.ERROR);

        }
        return null;
    }
}
