/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.projecttotemplate.structure;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.topcased.service.projecttotemplate.infos.InfosDirectory;
import org.topcased.service.projecttotemplate.infos.InfosFile;

/**
 * Structure of a Tree : TemplateProjectTree <br>
 * The tree is composed of :
 * <ul>
 * <li>name</li>
 * <li>path</li>
 * <li>infos : is a 'directory' or 'file'</li>
 */
public class TemplateProjectTree
{
	// Parent tree
	private TemplateProjectTree parentTree = null;

	// Name
	private String nameParent = "";

	// Path
	private String pathParent = "";

	// is an Directory or File
	private boolean isDirectoryParent = true;

	// Map Infos TemplateProject : InfosDirectory
	private Map<String, InfosDirectory> mapInfosDirectory = new HashMap<String, InfosDirectory>();

	// Map Infos TemplateProject : InfosFile
	private Map<String, InfosFile> mapInfosFile = new HashMap<String, InfosFile>();

	// Map Infos TemplateProject : on the project tree
	private Map<String, TemplateProjectTree> mapProjectTree = new HashMap<String, TemplateProjectTree>();

	/**
	 * Constructor
	 * 
	 * @param parent
	 * @param mapDirectory : key=name ; value=TemplatePojectTree
	 * @param mapFile : key=name ; value=InfosFile
	 * @param name : parent name
	 * @param path : parent path
	 * @param isDirectory : true if is a Directory, false if is a File
	 */
	public TemplateProjectTree(TemplateProjectTree parent, Map<String, InfosDirectory> mapDirectory, Map<String, InfosFile> mapFile,
			String name, String path, boolean isDirectory)
	{
		parentTree = parent;
		mapInfosDirectory = mapDirectory;
		mapInfosFile = mapFile;
		nameParent = name;
		pathParent = path;
		isDirectoryParent = isDirectory;

		// New TemplateProjectTree
		Iterator<String> iterDir = mapInfosDirectory.keySet().iterator();
		while (iterDir.hasNext())
		{
			InfosDirectory aInfoD = mapInfosDirectory.get(iterDir.next());
			Map<String, InfosDirectory> aMapD = aInfoD.getMapInfosDirectory();
			Map<String, InfosFile> aMapF = aInfoD.getMapInfosFile();
			String aNameD = aInfoD.getDirectoryName();
			String aPath = aInfoD.getDirectoryPath();
			TemplateProjectTree tpt = new TemplateProjectTree(this, aMapD, aMapF, aNameD, aPath, true);
			mapProjectTree.put(aNameD, tpt);
			// Directory
//			if (mapInfosDirectory.get(key) instanceof InfosDirectory)
//			{

//			}
//			// File
//			else if (mapInfosDirectory.get(key) instanceof InfosFile)
//			{
//				InfosFile aInfoF = (InfosFile) mapInfosDirectory.get(key);
//				String aNameF = aInfoF.getFileName();
//				String aPath = aInfoF.getFilePath();
//				TemplateProjectTree tpt = new TemplateProjectTree(this, new HashMap<String, InfosDirectory>(), new HashMap<String, InfosFile>(), aNameF, aPath, false);
//				mapProjectTree.put(aNameF, tpt);
//			}
		}
		
		Iterator<String> iterFile = mapInfosFile.keySet().iterator();
		while (iterFile.hasNext())
		{
			// File
			InfosFile aInfoF = (InfosFile) mapInfosFile.get(iterFile.next());
			String aNameF = aInfoF.getFileName();
			String aPath = aInfoF.getFilePath();
			TemplateProjectTree tpt = new TemplateProjectTree(this, new HashMap<String, InfosDirectory>(),
					new HashMap<String, InfosFile>(), aNameF, aPath, false);
			mapProjectTree.put(aNameF, tpt);
		}
	}

	/**
	 * Returns parent of selected TemplateProjectTree
	 * 
	 * @return TemplateProjectTree
	 */
	public TemplateProjectTree getParent()
	{
		return parentTree;
	}

	/**
	 * Get sons
	 * 
	 * @return map : key=name ; value=TemplatePojectTree
	 */
	public List<TemplateProjectTree> getSons()
	{
		return new ArrayList<TemplateProjectTree>(mapProjectTree.values());
	}

	/**
	 * Get name
	 * 
	 * @return parent name
	 */
	public String getName()
	{
		return nameParent;
	}

	/**
	 * Get name of the project
	 * 
	 * @param nameProjectTree
	 */
	public void setName(String nameProjectTree)
	{
		nameParent = nameProjectTree;
	}

	/**
	 * Get path
	 * 
	 * @return parent path
	 */
	public String getPath()
	{
		return pathParent;
	}

	/**
	 * Get information : is a Directory or a File
	 * 
	 * @return 'true' if is a directory ans 'false' if is a 'File'
	 */
	public boolean isDirectory()
	{
		return isDirectoryParent;
	}

	/**
	 * Get Map Informations Directory
	 * 
	 * @return map : key=name ; value=InfosDirectory
	 */
	public Map<String, InfosDirectory> getMapInfosDirectory()
	{
		return mapInfosDirectory;
	}

	/**
	 * Get Map Informations File
	 * 
	 * @return map : key=name ; value=InfosFile
	 */
	public Map<String, InfosFile> getMapInfosFile()
	{
		return mapInfosFile;
	}
}
