/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.projecttotemplate.wizard;

import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.Wizard;
import org.topcased.facilities.dialogs.ITopcasedWizardConstants;
import org.topcased.service.projecttotemplate.ProjectToTemplatePlugin;

/**
 * Create a new Wizard to create a project from template : "*.xml" file.<br>
 * This wizard is composed of only one page : 'TemplateToProjectPage'.
 * 
 */
public class ProjectToTemplateWizard extends Wizard implements ITopcasedWizardConstants
{
	ProjectToTemplatePage mainPage;

	private IPath pathProject = null;

	/**
	 * Constructor
	 * 
	 * @param projectPath : path of the project which can be transform in xml file
	 */
	public ProjectToTemplateWizard(IPath projectPath)
	{
		pathProject = projectPath;

		// Image
		setDefaultPageImageDescriptor(ProjectToTemplatePlugin.getImageDescriptor(TOPCASED_LOGO_PATH));

	}

	/**
	 * @see IWizard Add pages
	 */
	public void addPages()
	{
		setWindowTitle(ProjectToTemplatePlugin.getResourceString("ProjectToTemplateWizard.wizard.title"));
		mainPage = new ProjectToTemplatePage(pathProject, this);
		mainPage.setTitle(ProjectToTemplatePlugin.getResourceString("ProjectToTemplateWizard.wizard.page.title"));
		mainPage.setDescription(ProjectToTemplatePlugin.getResourceString("ProjectToTemplatePage.message"));
		addPage(mainPage);
	}

	/**
	 * IWizard Perform Finish
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	public boolean performFinish()
	{
		return mainPage.finish();
	}

	/**
	 * @return ProjectToTemplatePage
	 */
	public ProjectToTemplatePage getProjectToTemplatePage()
	{
		return mainPage;
	}
}
