/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.publishmetamodel.service;

import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.ecore.EObject;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.metamodel.MetamodelsManager;

/**
 * Gets MetaModel (EObject) from the uri meta model <br>
 */
public class PublishMetaModel implements IService
{
	/**
	 * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{
		// bloc declaration
		String metaModelId = "";

		// Map null
		if (parameters == null)
		{
			CorePlugin.log(this.getClass(), "Parameters list is null. It must have exactly one parameter", IStatus.ERROR);
			return null;
		}

		// Get 'XMIFile' parameter
		if (parameters.containsKey("MetaModelUri"))
		{
			if (parameters.get("MetaModelUri") instanceof String)
			{
				metaModelId = (String) parameters.get("MetaModelUri");
			}
			else
			{
				CorePlugin.log(this.getClass(), "The 'MetaModelUri' parameter must be a String", IStatus.ERROR);
			}
		}
		else
		{
			CorePlugin.log(this.getClass(), "Parameters number is not correct. It must have exactly one parameter : 'MetaModelUri'", IStatus.ERROR);
		}

		// SERVICE : PUBLISH META MODEL
		if (metaModelId != null && !metaModelId.equals(""))
		{
			String xmiFileMM = MetamodelsManager.getInstance().uriMM2XmiFileMM(metaModelId);
			if (metaModelId != null && !metaModelId.equals(""))
			{
				CorePlugin.log(this.getClass(), "The xmiFile corresponding to the meta model " + metaModelId + " is " + xmiFileMM, IStatus.INFO);
				EObject eObjectMM = MetamodelsManager.getInstance().xmiFileMM2EObjectMM(xmiFileMM);
				CorePlugin.log(this.getClass(), "The model EObject is done.", IStatus.INFO);
				return eObjectMM;
			}
			else
			{
				CorePlugin.log(this.getClass(), "No one xmiFile corresponding to the meta model " + metaModelId, IStatus.INFO);
			}
		}
		return null;
	}
}
