/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.readxmiemf;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;

import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

/**
 * Drives the model. Acts as the model entry point.
 */

public class ReadModelManager
{
    /**
     * In EMF, a resource provides the way to have access to the model content.
     */
    private ResourceSet resourceSet = null;

    /**
     * Returns the resource containing the network. Uses lazy initialization.
     * 
     * @param path
     * @return the associated resource
     */
    public Resource getResource(IPath path)
    {
        Resource resource = null;
        File file = path.toFile();
        if (file.exists())
        {
            resource = getResourceSet().getResource(URI.createFileURI(path.toString()), true);
        }
        else
        {
            resource = getResourceSet().getResource(URI.createPlatformResourceURI(path.toString(), false), true);
        }

        return resource;
    }

    /**
     * Returns the resource set.
     * 
     * @return the stored ResourceSet
     */
    private ResourceSet getResourceSet()
    {
        if (resourceSet == null)
        {
            // Obtains a new resource set
            resourceSet = new ResourceSetImpl();
        }
        return resourceSet;
    }

    /**
     * Loads the content of the model from the file.
     * 
     * @param path
     * @throws IOException
     */
    public void load(IPath path) throws IOException
    {
        Resource resource = getResource(path);
        if (resource != null)
        {
        	resource.load(new HashMap<Object, Object>());
        }
    }

    /**
     * Reloads the content of the model from the file.
     * 
     * @param path
     * @throws IOException
     */
    public void reload(IPath path) throws IOException
    {
        Resource resource = getResource(path);
        if (resource != null)
        {
            resource.unload();
            load(path);
        }
    }

    /**
     * Gets the top level object.
     * 
     * @param path the path where the model is stored
     * @return the edited model
     */
    public EObject getModel(IPath path)
    {
        Resource resource = getResource(path);
        if (resource != null && !resource.getContents().isEmpty())
        {
            return resource.getContents().get(0);
        }
        return null;
    }
}
