/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.readxmiemf.service;

import java.io.IOException;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.parameter.ParameterCst;
import org.topcased.service.readxmiemf.ReadModelManager;

/**
 * Loads an XMI file and return an EMF model
 */
public class ReadXmiEmf implements IService
{
	/**
	 * Loads and gets a model from an XMI file
	 * 
	 * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{
		// Declaration bloc
		EObject model = null;
		IPath pathFile = null;
		ReadModelManager modelManager = null;

		// Map null
		if (parameters == null)
		{
			CorePlugin.log(this.getClass(), "Parameters list is null. It must have exactly one parameter", IStatus.ERROR);
			return null;
		}

		// Get 'XMIFile' parameter
		if (parameters.containsKey(ParameterCst.XMI_FILE))
		{
			if (parameters.get(ParameterCst.XMI_FILE) instanceof String)
			{
				String value = (String) parameters.get(ParameterCst.XMI_FILE);
				pathFile = new Path(value);
			}
			else
			{
				CorePlugin.log(this.getClass(), "The 'XMIFile' parameter must be a String", IStatus.ERROR);
			}
		}
		else
		{
			CorePlugin.log(this.getClass(), "Parameters number is not correct. It must have exactly one parameter : 'XMIFile'",
					IStatus.ERROR);
		}

		// Test pathFile
		if (pathFile != null)
		{
			modelManager = new ReadModelManager();
		}
		else
		{
			CorePlugin.log(this.getClass(), "Error : pathFile not define!", IStatus.ERROR);
			return null;
		}

		// Launch Service Read
		try
		{
			modelManager.load(pathFile);
			CorePlugin.log(this.getClass(), "Load XMI/EMF from : " + pathFile, IStatus.INFO);
			model = modelManager.getModel(pathFile);
			EcoreUtil.resolveAll(model);
			CorePlugin.log(this.getClass(), "Get model XMI/EMF from : " + pathFile, IStatus.INFO);

		}
		catch (IOException e)
		{
			CorePlugin.log(this.getClass(), "IOException : " + e.getMessage(), IStatus.ERROR);
		}
		return model;
	}
}