/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.templatetoproject.service;

import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.service.templatetoproject.infos.InfosTemplateProject;

/**
 * Service of the BUS : Install a project in the workspace from a template (.xml + .zip)
 */
public class InstallProject implements IService
{

	/**
	 * Install a project in the workspace from a template (.xml + .zip)
	 * 
	 * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{
		// Bloc declaration
		InfosTemplateProject infosTemplateProject;
		String pathProject = "";
		String pathTemplateProject = "";

		// Map null
		if (parameters == null)
		{
			CorePlugin.log(this.getClass(), "Parameters list is null. It must have 3 parameters", IStatus.ERROR);
			return null;
		}

		// Get 'PathTemplateProject' parameter
		if (parameters.containsKey("PathTemplateProject"))
		{
			if (parameters.get("PathTemplateProject") instanceof String)
			{
				pathTemplateProject = (String) parameters.get("PathTemplateProject");
			}
			else
			{
				CorePlugin.log(this.getClass(), "The 'PathTemplateProject' parameter must be a String", IStatus.ERROR);
			}
		}
		else
		{
			CorePlugin.log(this.getClass(), "Parameters number is not correct. It must have the parameter : 'InfosTemplateProject'",
					IStatus.ERROR);
		}

		// Get 'PathProject' parameter
		if (parameters.containsKey("PathProject"))
		{
			if (parameters.get("PathProject") instanceof String)
			{
				pathProject = (String) parameters.get("PathProject");
			}
			else
			{
				CorePlugin.log(this.getClass(), "The 'PathProject' parameter must be a String", IStatus.ERROR);
			}
		}
		else
		{
			CorePlugin.log(this.getClass(), "Parameters number is not correct. It must have the parameter : 'PathProject'", IStatus.ERROR);
		}

		// Get InfostemplateProject
		if (parameters.containsKey("InfosTemplateProject"))
		{
			if (parameters.get("InfosTemplateProject") instanceof InfosTemplateProject)
			{
				infosTemplateProject = (InfosTemplateProject) parameters.get("InfosTemplateProject");

				if (!pathProject.equals("") && !pathTemplateProject.equals(""))
				{
					// SERVICE INSTALL PROJECT
					if (infosTemplateProject.installProject(pathProject))
					{
						CorePlugin.log(this.getClass(), "Successfully install this template in : " + pathProject, IStatus.OK);
					}
					else
					{
						CorePlugin.log(this.getClass(), "Not install this template in : " + pathProject, IStatus.WARNING);
					}
				}
				else
				{
					CorePlugin.log(this.getClass(), "The template is not installed like a project in : " + pathProject, IStatus.ERROR);
				}
			}
			else
			{
				CorePlugin
						.log(this.getClass(), "The 'InfosTemplateProject' parameter must be a InfosTemplateProject object", IStatus.ERROR);
			}
		}
		else
		{
			CorePlugin.log(this.getClass(), "Parameters number is not correct. It must have the parameter : 'InfosTemplateProject'",
					IStatus.ERROR);
		}

		return null;
	}
}
