/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.templatetoproject.structure;

import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.topcased.facilities.dialogs.ITopcasedWizardConstants;
import org.topcased.service.templatetoproject.TemplateToProjectPlugin;

/**
 * LabelProvider of TemplateProjectTree Display name of the repository or of the
 * file with the icon corresponding
 * 
 */
public class TemplateProjectTreeLabelProvider extends LabelProvider implements ITopcasedWizardConstants
{
    /**
     * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
     */
    public String getText(Object element)
    {
        String text = "";
        if (element instanceof TemplateProjectTree)
        {
            TemplateProjectTree aSubTree = (TemplateProjectTree) element;
            text = aSubTree.getName();
        }
        return text;
    }

    /**
     * Get inforamtion on the type of element. 'true' if it is a 'Repository'
     * and 'false' if it is a 'File'
     * 
     * @param element
     * @return 'true' for a Repository and 'false' for a File
     */
    public boolean isDirectory(Object element)
    {
        boolean isDirectory = true;
        if (element instanceof TemplateProjectTree)
        {
            TemplateProjectTree aSubTree = (TemplateProjectTree) element;
            isDirectory = aSubTree.isDirectory();
        }
        return isDirectory;
    }

    /**
     * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
     */
    public Image getImage(Object element)
    {
        if (isDirectory(element))
        {
            return TemplateToProjectPlugin.getImageDescriptor(DIRECTORY_IMG).createImage();
        }
        else
        {
            return TemplateToProjectPlugin.getImageDescriptor(FILE_IMG).createImage();
        }
    }
}
