/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.templatetoproject.wizard;

import java.io.File;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class ProjectLocationGroup {

	private Group container;
	private Text pathText;
	private Button browseButton;
	private Button checkDefault;
	private String defaultPath;
	
	public ProjectLocationGroup(Composite parent, String _defaultPath)
	{
		container = new Group(parent,SWT.SHADOW_IN);
		
		checkDefault = new Button(container,SWT.CHECK);
    	
    	Group directoryGroup = new Group(container,SWT.SHADOW_NONE);
    	Label pathLabel = new Label(directoryGroup,SWT.NONE);
    	pathText = new Text(directoryGroup,SWT.SINGLE | SWT.BORDER);
    	browseButton = new Button(directoryGroup,SWT.PUSH);
    	
    	GridLayout verticalLayout = new GridLayout(1,true);
    	//verticalLayout.type = SWT.VERTICAL;
    	GridData checkDefaultData = new GridData(SWT.FILL,SWT.CENTER,true,false);
    	GridData directoryGroupData = new GridData(SWT.FILL,SWT.CENTER,true,false);
    	
    	GridLayout horizontalLayout = new GridLayout(3,false);
    	GridData pathTextData = new GridData(SWT.FILL,SWT.CENTER,true,false);
    	
    	GridData containerData = new GridData(SWT.FILL,SWT.CENTER,true,false);
    	container.setText("Project content :");
    	container.setLayout(verticalLayout);
    	container.setLayoutData(containerData);
    	checkDefault.setText("Use default workspace");
    	checkDefault.setSelection(true);
    	checkDefault.setLayoutData(checkDefaultData);
    	checkDefault.addMouseListener(new MouseAdapter(){
    		public void mouseUp(MouseEvent e){
    			checkButtonPressed();
    		}
    	});
    	
    	defaultPath = _defaultPath;
    	directoryGroup.setLayout(horizontalLayout);
    	directoryGroup.setLayoutData(directoryGroupData);
    	pathLabel.setText("Directory : ");
    	pathLabel.setSize(60,15);
    	pathText.setText(defaultPath);
    	pathText.setLayoutData(pathTextData);
    	pathText.setEditable(false);
    	browseButton.setText("Browse...");
    	browseButton.setEnabled(false);
    	browseButton.addMouseListener(new MouseAdapter(){
    		public void mouseUp(MouseEvent e){
    			openBrowse();
    		}
    	});
	}
	
	private void openBrowse(){
		DirectoryDialog dialog = new DirectoryDialog(container.getShell());
		String path = dialog.open();
		if(path!=null)
		{
			pathText.setText(path);
		}
		
	}
	
	private void checkButtonPressed()
	{
		if (checkDefault.getSelection())
		{
			pathText.setText(defaultPath);
			pathText.setEditable(false);
			browseButton.setEnabled(false);
		}
		else
		{
			pathText.setText("");
			pathText.setEditable(true);
			browseButton.setEnabled(true);
		}
	}
	
	public String getPath()
	{
		String path = pathText.getText();
		if(!path.endsWith(File.separator))
		{
			path = path.concat(File.separator);
		}
		return path;
	}
}
