/******************************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS)
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************************/ 
package org.topcased.service.templatetoproject.wizard;

import java.util.Map;
import java.util.TreeMap;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.topcased.service.templatetoproject.infos.InfosTemplateProject;

/**
 *
 */
public class TemplateToProjectConfigurationPage extends WizardPage
{
    // Wizard TemplateToProject
    private Table variableTable;
    
    // Reference to the InfosTemplateProject object
    private InfosTemplateProject infos;
    
    // Constants definition
    private static final int VARIABLE_NAME = 0; // represents the first column
    
    private static final int VARIABLE_VALUE = 1; // represents the second column 
    
    
    protected TemplateToProjectConfigurationPage(String pageName)
    {
    	super(pageName);
    }
    
    /**
     * Constructor
     * 
     * @param w : workbench
     * @param s : structured selection
     * @param templateToProject : our wizard 
     */
    protected TemplateToProjectConfigurationPage(IWorkbench w, IStructuredSelection s)
    {
        super("templateConfigurationPage");
    }
    
    /**
     * Get the variables table
     * @return the reference to the varuiable table
     */
    public Table getvariableTable()
    {
    	return variableTable;
    }
    
    /**
     * Getter for the object InfosTemplateProject
     * @return the current reference to the object InfosTemplateProject
     */
    public InfosTemplateProject getInfosTemplateProject()
    {
    	return infos;
    }
    
    /**
     * Setter for the object InfosTemplateProject
     * @param i information template project to set
     */
    public void setInfosTemplateProject(InfosTemplateProject i)
    {
    	infos = i;
    }
	/**
     * Create Control
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent)
    {
        final GridData gridDataForTable = new GridData(SWT.FILL, SWT.FILL, true, true);
        gridDataForTable.heightHint = 50;
        
    	Composite mainComposite = new Composite(parent, SWT.NONE);
    	mainComposite.setLayout(new GridLayout());
    	setControl(mainComposite);
    	
    	Label tableLabel = new Label(mainComposite, SWT.NONE);
    	tableLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	tableLabel.setText("Affect values to your template variables thanks to the table below");
    	
    	variableTable = new Table(mainComposite, SWT.BORDER | SWT.FULL_SELECTION);
    	variableTable.setLayoutData(gridDataForTable);
        variableTable.setLinesVisible(true);
        variableTable.setHeaderVisible(true);
        variableTable.getVerticalBar().setEnabled(true);
        
    	final TableColumn variableNameColumn = new TableColumn(variableTable, SWT.LEFT);
    	variableNameColumn.setWidth(250);
    	variableNameColumn.setText("Variables");

    	final TableColumn variableValueColumn = new TableColumn(variableTable, SWT.LEFT);
    	variableValueColumn.setWidth(250);
    	variableValueColumn.setText("Value");
     	
    	final TableEditor editor = new TableEditor(variableTable);
    	editor.horizontalAlignment = SWT.LEFT;
    	editor.grabHorizontal = true;
    	editor.minimumWidth = 250;
    	
    	variableTable.addMouseListener(new MouseAdapter() 
    	{
    	   	public void mouseDown(MouseEvent event) 
    	   	{
    	   		// Clean up any previous editor control
    	   		Control oldEditor = editor.getEditor();
    	   		if (oldEditor != null) oldEditor.dispose();

    	   		// Identify the selected row
    	   		final TableItem item = variableTable.getItem(new Point(event.x, event.y));
    	   		if (item == null) return;

    	   		// The control that will be the editor must be a child of the Table
    	   		final Text newEditor = new Text(variableTable, SWT.NONE);
    	   		newEditor.setText(item.getText(VARIABLE_VALUE));
    	   		newEditor.setForeground(item.getForeground());
    	   		newEditor.selectAll();
    	   		newEditor.setFocus();
    	   		newEditor.addFocusListener(new FocusListener()
    	   		{
    	   			public void focusGained(FocusEvent e) {}
    	   			
    	   			public void focusLost(org.eclipse.swt.events.FocusEvent e)
    	   			{
    	   				updateVariablesMap(item.getText(VARIABLE_NAME), item.getText(VARIABLE_VALUE));
    	   				handleFinish();	
    	   			};
    	   		});
    	   		
    	   		newEditor.addModifyListener(new ModifyListener() 
    	   		{
    	   			public void modifyText(ModifyEvent e) 
    	   			{
    	   				// Set the new value
    	   				item.setText(VARIABLE_VALUE, newEditor.getText());
    	   			}
    	   		});
    	   		editor.setEditor(newEditor, item, VARIABLE_VALUE);
    	   	}
    	});
    	
    	setPageComplete(false);
    	
    }
    
    /**
     * Method which initializes the table with default values
     * 
     */
    public void fillInVariableMap()
    {
    	variableTable.removeAll();
    	Map<String, String> variableMap = (TreeMap<String, String>) infos.getMapTemplateVariables();
    	
    	for (String variable : variableMap.keySet())
    	{
    		TableItem temp = new TableItem(variableTable, SWT.NONE);
    		temp.setText(VARIABLE_NAME, variable);
   			temp.setText(VARIABLE_VALUE, variableMap.get(variable));
    	}
    }
    
    /**
     * Setter for the variable map 
     * @param name the variable name
     * @param value the variable value given by the user
     */
    private void updateVariablesMap(String name, String value)
    {
    	infos.getMapTemplateVariables().put(name, value);
    }
    
   
    /**
     * Install project : install the template in the ProjectPath (workspace)
     * 
     * @return finish value
     */
    public boolean finish()
    {
        return isPageComplete();
    }

    /**
     * Method called to know if all the elements of the second column has been filled.
     * If all is correctly filled then the pageComplete variable is setting.
     */
    public void handleFinish()
    {
    	// This page is completed only if all the elements of the second column are entered.
    	TableItem [] content =  variableTable.getItems();
    	int i=0;
    	boolean notEmpty = true;
    	while (i < content.length && notEmpty)
    	{
    		if (content[i].getText(VARIABLE_VALUE).trim().equals(""))
    		{
    			notEmpty = false;
    		}
    		i++;
    	}
   		setPageComplete(notEmpty);
    }
}