/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.templatetoproject.wizard;

import java.io.File;
import java.io.FileFilter;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IWorkbench;
import org.topcased.bus.core.CorePlugin;
import org.topcased.service.projecttotemplate.ProjectToTemplatePlugin;
import org.topcased.service.projecttotemplate.preferences.ProjectToTemplatePreferencePage;
import org.topcased.service.templatetoproject.infos.InfosDirectory;
import org.topcased.service.templatetoproject.infos.InfosFile;
import org.topcased.service.templatetoproject.infos.InfosTemplateProject;
import org.topcased.service.templatetoproject.service.InstallProject;
import org.topcased.service.templatetoproject.structure.TemplateProjectTree;
import org.topcased.service.templatetoproject.structure.TemplateProjectTreeContentProvider;
import org.topcased.service.templatetoproject.structure.TemplateProjectTreeLabelProvider;

/**
 * TemplateToProjectPage is a WizardPage. Display the wizard to create a project from a template of project in hte current WorkSpace.
 */
public class TemplateToProjectPage extends WizardPage
{
	// Wizard TemplateToProject
	TemplateToProjectWizard wizard;

	boolean finish;

	private TreeViewer treeViewer;

	private Group groupProjectName;

	private ComboViewer combo;

	// Tree TemplateProject
	private TemplateProjectTree aTemplateProjectTree;

	// Map Infos TemplateProject : InfosDirectory
	private Map<String, InfosDirectory> aMapDirectory = new HashMap<String, InfosDirectory>();

	// Map Infos TemplateProject : InfosFile
	private Map<String, InfosFile> aMapFile = new HashMap<String, InfosFile>();

	// Class InfosTemplateProject to construct the project from template : *.xml
	// file
	private InfosTemplateProject aInfosTemplateProject = new InfosTemplateProject();

	// Path of repository of template (all *.xml)
	// "aTemplateXMLDirectory" must come from "Preferences"
	private String aTemplateXMLDirectory = ProjectToTemplatePlugin.getDefault().getPreferenceStore().getString(
			ProjectToTemplatePreferencePage.TEMPLATE_REPOSITORY_PATH);

	// Map of all template (*.xml) : use for the ComboBox
	private Map<String, String> aMapXmlFile = new HashMap<String, String>();

	// Project name to create
	private String projectName = "DefaultNameProject";

	// Project path : Workspace
	private String pathProject = ResourcesPlugin.getWorkspace().getRoot().getLocation().toOSString();// Platform.getInstanceLocation().getURL().getPath();

	// Template xml path
	private String pathFileXml = "";

	private ProjectLocationGroup projectLocation;

	/**
	 * Constructor
	 * 
	 * @param pageName The name givent to the current page
	 * @param templateToProject : wizard
	 */
	protected TemplateToProjectPage(String pageName, TemplateToProjectWizard templateToProject)
	{
		super(pageName);
		finish = false;
		wizard = templateToProject;
	}

	/**
	 * Constructor
	 * 
	 * @param w
	 * @param s
	 * @param templateToProject : wizard
	 */
	protected TemplateToProjectPage(IWorkbench w, IStructuredSelection s, TemplateToProjectWizard templateToProject)
	{
		super("templatePage");
		finish = false;
		wizard = templateToProject;
	}

	/**
	 * Creates Control
	 * 
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent)
	{
		Composite composite = new Composite(parent, SWT.NULL);
		GridLayout gridLayout = new GridLayout(1, false);
		composite.setLayout(gridLayout);
		setControl(composite);

		// Group projet : 'groupProjectName'
		createGroupProject(composite);

		// Project Name : 'sectionProjectName'
		createTextProjectName(groupProjectName);

		// Combo Box : Template choice : 'combo'
		createComboBoxTemplate(groupProjectName);

		createLocationSelector(composite);

		// Tree : 'treeViewer'
		createTreeViewer(composite);

		// Finish ?
		handleFinish();
	}

	/**
	 * Creates the TreeViewer
	 * 
	 * @param parent
	 */
	private void createTreeViewer(Composite parent)
	{
		treeViewer = new TreeViewer(parent, SWT.BORDER);
		final Tree tree = treeViewer.getTree();
		tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		treeViewer.setContentProvider(new TemplateProjectTreeContentProvider());
		treeViewer.setLabelProvider(new TemplateProjectTreeLabelProvider());
		aTemplateProjectTree = new TemplateProjectTree(null, aMapDirectory, aMapFile, projectName, pathProject, true);
	}

	/**
	 * Creates Combo : template choice with the name template
	 * 
	 * @param parent
	 */
	private void createComboBoxTemplate(Composite parent)
	{
		combo = new ComboViewer(parent, SWT.READ_ONLY);
		combo.addSelectionChangedListener(new ISelectionChangedListener()
		{
			public void selectionChanged(SelectionChangedEvent event)
			{
				handleCombo(combo);
			}
		});

		aMapXmlFile = initMapXmlFile(new File(aTemplateXMLDirectory));
		// Initialization with all template name (key of aMapXmlFile)
		for (String file : aMapXmlFile.keySet())
		{
			combo.getCombo().add(file);
		}
	}

	/**
	 * Creates Text : project Name
	 * 
	 * @param parent
	 */
	private void createTextProjectName(Composite parent)
	{
		final Text sectionProjectName = new Text(parent, SWT.BORDER);
		sectionProjectName.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		sectionProjectName.setText(projectName);
		sectionProjectName.selectAll();
		sectionProjectName.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				handleProjectName(sectionProjectName);
			}
		});
	}

	/**
	 * Creates the project group
	 * 
	 * @param parent The composite parent
	 */
	private void createGroupProject(Composite parent)
	{
		groupProjectName = new Group(parent, SWT.NONE);
		groupProjectName.setLayout(new GridLayout());
		groupProjectName.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		groupProjectName.setText("Project name : ");
	}

	/**
	 * Installs project : install the template in the ProjectPath (workspace)
	 * 
	 * @return finish value
	 */
	public boolean finish()
	{
		// Call SERVICE INSTALL PROJECT
		pathProject = projectLocation.getPath();
		InstallProject installProjectService = new InstallProject();
		Map<String, Object> mapInput = new HashMap<String, Object>();
		mapInput.put("PathTemplateProject", pathFileXml);
		mapInput.put("PathProject", pathProject + projectName);
		mapInput.put("InfosTemplateProject", aInfosTemplateProject);
		installProjectService.serviceRun(mapInput);
		return finish;
	}

	/**
	 * Handles Finish action
	 */
	public void handleFinish()
	{
		if (combo != null)
		{
			String comboString = combo.getCombo().getText().trim();
			boolean existingProject = false;

			try
			{
				IResource[] projects = ResourcesPlugin.getWorkspace().getRoot().members();
				for (int i = 0; i < projects.length; i++)
				{
					if (projectName.equalsIgnoreCase(projects[i].getName()))
					{
						existingProject = true;
					}
				}
			}
			catch (CoreException e)
			{
				CorePlugin.log(e);
			}

			if (!"".equals(comboString) && !"".equals(projectName) && !existingProject)
			{
				finish = true;
				setPageComplete(true);
				setMessage(TemplateToProjectWizard.MESSAGEDESCRIPTION);
			}
			else
			{
				finish = false;
				setPageComplete(false);
				// Display Message
				if (comboString.equals(""))
				{
					setMessage("Choose a template in the comboBox!", WizardPage.ERROR);
				}
				
				if (projectName.equals(""))
				{
					setMessage("Get a name to the project!", WizardPage.ERROR);
				}
				
				if (existingProject)
				{
					setMessage("Project name already exists!", WizardPage.ERROR);
				}
			}
		}
		else
		{
			finish = false;
			setPageComplete(false);
			setMessage("ComboxBox is void!", WizardPage.ERROR);
		}
	}

	/**
	 * Takes the value (which is changing) of the project name text area
	 * 
	 * @param nameProject : project name
	 */
	private void handleProjectName(Text nameProject)
	{
		projectName = nameProject.getText();
		aTemplateProjectTree.setName(projectName);
		if (!combo.getCombo().getText().trim().equals(""))
		{
			treeViewer.setInput(new Object[] { aTemplateProjectTree });
		}
		handleFinish();
	}

	/**
	 * Method called when the selection of the combo change Inialize "aInfosTemplateProject", "aMapDirectory", "aMapFile" : Infos about Template
	 * 
	 * @param comboViewer
	 */
	private void handleCombo(ComboViewer comboViewer)
	{
		String templateNameSelected = comboViewer.getCombo().getText();
		if (aMapXmlFile.containsKey(templateNameSelected))
		{
			if (aMapXmlFile.get(templateNameSelected) instanceof String)
			{
				pathFileXml = (String) aMapXmlFile.get(templateNameSelected);
				readTemplate(pathFileXml, new HashMap<String, String>());

				// After reading the xml file, fill in of the table based on the next page
				if (!aInfosTemplateProject.getMapTemplateVariables().isEmpty())
				{
					wizard.getTemplateToProjectConfigurationPage().setInfosTemplateProject(aInfosTemplateProject);
					wizard.getTemplateToProjectConfigurationPage().fillInVariableMap();
				}

				// New TemplateProjectTree
				aTemplateProjectTree = new TemplateProjectTree(null, aMapDirectory, aMapFile, projectName, pathProject, true);
				treeViewer.setInput(new Object[] { aTemplateProjectTree });
			}
		}
		handleFinish();
	}

	/**
	 * Initializes mapXmlFile : Take all templates names in the repository pathRepository
	 * 
	 * @param pathRepository
	 * @return Map of all xml file from the repository pathRepository (key=template name ; value=template path)
	 */
	private Map<String, String> initMapXmlFile(File pathRepository)
	{
		Map<String, String> mapXmlFile = new HashMap<String, String>();
		if (pathRepository != null && pathRepository.exists())
		{
			File[] repositoryContent = pathRepository.listFiles((new FileFilter()
			{
				public boolean accept(File pathname)
				{
					boolean acceptReturn = false;
					if (pathname.isFile())
					{
						IPath jarFile = new Path(pathname.toString());
						if (jarFile.getFileExtension().equals("xml"))
						{
							acceptReturn = true;
						}
					}
					return acceptReturn;
				}
			}));
			// Add xml files in mapXmlFile
			File subRepository = null;
			if (repositoryContent != null)
			{
				for (int indice = 0; indice < repositoryContent.length; indice++)
				{
					subRepository = repositoryContent[indice];
					mapXmlFile = readTemplate(subRepository.getAbsolutePath(), mapXmlFile);
				}
			}
		}
		return mapXmlFile;
	}

	/**
	 * Reads Template project
	 * 
	 * @param templatePath : path of the template (.xml)
	 * @param mapXmlFile (key=template name ; value=template path)
	 * @return map Xml File (key=template name ; value=template path)
	 */
	private Map<String, String> readTemplate(String templatePath, Map<String, String> mapXmlFile)
	{
		aInfosTemplateProject = new InfosTemplateProject();
		// Take information about Template xml file
		aInfosTemplateProject.getInfosTemplateProject(templatePath);
		// Map Infos : Directory + File
		aMapDirectory = aInfosTemplateProject.getMapInfosDirectory();
		aMapFile = aInfosTemplateProject.getMapInfosFile();
		// Map TemplateName
		mapXmlFile.put(aInfosTemplateProject.getTemplateName(), templatePath);

		return mapXmlFile;
	}

	/**
	 * Creates location selector.
	 * 
	 * @param parent The parent composite
	 */
	private void createLocationSelector(Composite parent)
	{
		projectLocation = new ProjectLocationGroup(parent, pathProject);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.WizardPage#canFlipToNextPage()
	 */
	public boolean canFlipToNextPage()
	{
		return (isPageComplete() && !aInfosTemplateProject.getMapTemplateVariables().isEmpty());
	}
}
