/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.templatetoproject.wizard;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.topcased.facilities.dialogs.ITopcasedWizardConstants;
import org.topcased.service.templatetoproject.TemplateToProjectPlugin;

/**
 * Create a new Wizard to create a project with template : "*.xml" file. This wizard is composed of only one page : 'TemplateToProjectPage'.
 * 
 * 
 */
public class TemplateToProjectWizard extends Wizard implements INewWizard, ITopcasedWizardConstants
{
	private IWorkbench workbench;

	private IStructuredSelection selection;

	// Wizard page : TemplateToProjectPage
	TemplateToProjectPage mainPage;

	// Wizard page : TemplateToProjectConfigurationPage
	TemplateToProjectConfigurationPage secondPage;

	/**
	 * MessageDescription TemplateToProjectPage
	 */
	public static String MESSAGEDESCRIPTION = "Create a project in the current Workspace from a template project selected in the comboBox.";

	public static String OPTIONALMESSAGEDESCRIPTION = "Customize your template using template variables.";

	/**
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench w, IStructuredSelection s)
	{
		workbench = w;
		selection = s;

		// Image
		setDefaultPageImageDescriptor(TemplateToProjectPlugin.getImageDescriptor(TOPCASED_LOGO_PATH));

		// Title
		setWindowTitle("TOPCASED : Create new Project from Template");
	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	public void addPages()
	{
		mainPage = new TemplateToProjectPage(workbench, selection, this);
		mainPage.setTitle("Create a project with template");
		mainPage.setDescription(MESSAGEDESCRIPTION);
		addPage(mainPage);

		secondPage = new TemplateToProjectConfigurationPage(workbench, selection);
		secondPage.setTitle("Give more flexibility to your project by configuring your template");
		secondPage.setDescription(OPTIONALMESSAGEDESCRIPTION);
		addPage(secondPage);
	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#canFinish()
	 */
	public boolean canFinish()
	{
		if (mainPage.canFlipToNextPage())
		{
			return secondPage.isPageComplete();
		}
		else
		{
			return mainPage.isPageComplete();
		}
	}

	/**
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	public boolean performFinish()
	{
		return mainPage.finish();
	}

	/**
	 * @return TemplateToProjectPage
	 */
	public TemplateToProjectPage getTemplateToProjectPage()
	{
		return mainPage;
	}

	/**
	 * @return TemplateToProjectConfigurationPage
	 */
	public TemplateToProjectConfigurationPage getTemplateToProjectConfigurationPage()
	{
		return secondPage;
	}
}
