/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.transformation;

/**
 * 
 * The Node adapted to application
 */

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class AdapterNode
{
	Node domNode;

	static final int ELEMENT_TYPE = Node.ELEMENT_NODE;

	static final String[] refNode = { "ref", "port", "instance" };

	static final String[] idNode = { "id", "ID" };

	static final String[] tagName = { "Class", "class", "Direction", "direction", "Type", "type", "Property", "property",
			"PRAGMA_PARAMETER", "VALUE", "ENUMERATION", "ATTRIBUTE", "DEFINITION", "IMPLEMENTED_BY", "MEMBER_OF", "SECTION", "SKETCH",
			"TABLE", "PROPAGATED_EXCEPTION", "RAISED_BY", "PSEUDO_CODE", "CODE", "TYPES_PARAMETER", "CONSTANTS_PARAMETER",
			"OPERATION_SETS_PARAMETER", "OPERATIONS_PARAMETER", "MPLEMENTATION_CONSTRAINTS", "MODE", "CONSTRAINED_OPERATION", "STATE",
			"TRANSITION", "CONSTRAINED_BY", "OBJECT", "DATAFLOW", "EXCEPTION_FLOW", "code", "Code" };

	static final String[] tagAttrib = {
			"value", // "name",
			"value", // "name",
			"value", "value", "value", "value", "value", "value", "value", "value", "value", "value", "value", "value", "value", "value",
			"value", "value", "value", "value", "value", "value", "value", "value", "value", "value", "value", "value", "value", "value",
			"value", "value", "value", "value", "value", "value", "value" };

	public AdapterNode(Node node)
	{
		domNode = node;
	}

	public List<Node> getElements()
	{
		List<Node> elements = new ArrayList<Node>();
		if (domNode.getNodeType() == ELEMENT_TYPE)
		{
			NodeList nodeList = domNode.getChildNodes();
			for (int i = 0; i < nodeList.getLength(); i++)
			{
				Node nodelist = nodeList.item(i);
				if (nodelist.getNodeType() == ELEMENT_TYPE)
					elements.add(nodelist);
			}
		}
		return elements;
	}

	/**
	 * elements of the node
	 * 
	 */
	public Node getElementbyID(String refName)
	{
		List<Node> elementsbyID = new ArrayList<Node>();
		if (domNode.getNodeType() == ELEMENT_TYPE && domNode.hasAttributes())
		{
			NamedNodeMap map = domNode.getAttributes();
			for (int i = 0; i < idNode.length; i++)
			{
				if (map.getNamedItem(idNode[i]) != null)
				{
					elementsbyID.add(map.getNamedItem(idNode[i]));
				}
			}
			for (int i = 0; i < elementsbyID.size(); i++)
			{
				Node node = elementsbyID.get(i);
				if (node.getNodeValue().equals(refName))
				{
					return node;
				}
			}
			return null;
		}
		return null;
	}

	/**
	 * attributes with ref value
	 * 
	 */
	public List<String> getAttribRefName()
	{
		List<String> ref = new ArrayList<String>();
		if (domNode.getNodeType() == ELEMENT_TYPE && domNode.hasAttributes())
		{
			NamedNodeMap map = domNode.getAttributes();
			for (int i = 0; i < refNode.length; i++)
			{
				if (map.getNamedItem(refNode[i]) != null)
				{
					ref.add((String) refNode[i]);
				}
			}
		}
		return ref;
	}

	/**
	 * returns the reference value
	 * 
	 */
	public String getRefValue(String refName)
	{
		String s = null;
		if (domNode.getNodeType() == ELEMENT_TYPE && domNode.hasAttributes())
		{
			NamedNodeMap map = domNode.getAttributes();
			if (map.getNamedItem(refName) != null)
				s = map.getNamedItem(refName).getNodeValue();
		}
		return s;
	}

	public String getTagAttribName()
	{
		if (domNode.getNodeType() == ELEMENT_TYPE)
		{
			for (int i = 0; i < tagName.length; i++)
				if (domNode.getNodeName().equals(tagName[i]))
				{
					return (tagAttrib[i]);
				}
		}
		return null;
	}

	public Node getAttribNode(String refName)
	{
		NamedNodeMap map = domNode.getAttributes();
		Node attribNode = map.getNamedItem(refName);
		if (attribNode != null)
		{
			return attribNode;
		}
		else
			return null;
	}

	public void firstTransf(String metamodel)
	{

		Element e = (Element) domNode;
		NamedNodeMap map = domNode.getAttributes();

		Node attribNode1 = map.getNamedItem("xmi:version");
		if (attribNode1 != null)
			attribNode1.setNodeValue("2.0");
		else
			e.setAttribute("xmi:version", "2.0");
		Node attribNode2 = map.getNamedItem("xmlns:xmi");
		if (attribNode2 != null)
			attribNode2.setNodeValue("http://www.omg.org/XMI");
		else
			e.setAttribute("xmlns:xmi", "http://www.omg.org/XMI");
		Node attribNode3 = map.getNamedItem("xmlns:" + metamodel);
		if (attribNode3 != null)
			attribNode3.setNodeValue("http:///" + metamodel + ".ecore");
		else
			e.setAttribute("xmlns:" + metamodel, "http:///" + metamodel + ".ecore");
	}

	public boolean hasChildElements()
	{
		if (!domNode.hasChildNodes())
			return false;
		else
		{
			NodeList list = domNode.getChildNodes();
			for (int i = 0; i < list.getLength(); i++)
				if (list.item(i).getNodeType() == ELEMENT_TYPE)
					return true;
			return false;
		}
	}
}
