/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.transformation;

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class TransformationInfo
{
	private String id;

	private String name;

	private File ruleFile;

	private Map<String, String> inParams;

	private Map<String, String> outParams;

	private Map<String, String> metamodelHandler;

	private Map<String, String> metamodelId;

	private Map<String, File> metamodelFile;

	private Set<String> missingMetamodelIn;

	private Set<String> missingMetamodelOut;

	public TransformationInfo(String _id, String _name)
	{
		id = _id;
		name = _name;
		inParams = new HashMap<String, String>();
		outParams = new HashMap<String, String>();
		metamodelHandler = new HashMap<String, String>();
		metamodelId = new HashMap<String, String>();
		metamodelFile = new HashMap<String, File>();
		missingMetamodelIn = new HashSet<String>();
		missingMetamodelOut = new HashSet<String>();
		ruleFile = null;
	}

	public String getId()
	{
		return id;
	}

	public String getName()
	{
		return name;
	}

	public Set<String> getListInParams()
	{
		return inParams.keySet();
	}

	public void addInParams(String model, String metamodel)
	{
		inParams.put(model, metamodel);
	}

	public String getInParamMetamodel(String model)
	{
		return inParams.get(model);
	}

	public Set<String> getListOutParams()
	{
		return outParams.keySet();
	}

	public void addOutParams(String model, String metamodel)
	{
		outParams.put(model, metamodel);
	}

	public String getOutParamMetamodel(String model)
	{
		return outParams.get(model);
	}

	public void setMetamodelHandler(String metamodel, String handler)
	{
		metamodelHandler.put(metamodel, handler);
	}

	public String getMetamodelHandler(String metamodel)
	{
		return metamodelHandler.get(metamodel);
	}

	public void setMetamodelId(String metamodel, String id)
	{
		metamodelId.put(metamodel, id);
	}

	public String getMetamodelId(String metamodel)
	{
		return metamodelId.get(metamodel);
	}

	public void setMetamodelFile(String metamodel, File f)
	{
		metamodelFile.put(metamodel, f);
	}

	public File getMetamodelFile(String metamodel)
	{
		return metamodelFile.get(metamodel);
	}

	public void setRuleFile(File file)
	{
		ruleFile = file;
	}

	public File getRuleFile()
	{
		return ruleFile;
	}

	public Set<String> getMissingInMetamodelName()
	{
		return missingMetamodelIn;
	}

	public void setMissingInMetamodelName(String name)
	{
		missingMetamodelIn.add(name);
	}

	public Set<String> getMissingOutMetamodelName()
	{
		return missingMetamodelOut;
	}

	public void setMissingOutMetamodelName(String name)
	{
		missingMetamodelOut.add(name);
	}
}
