/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.transformation;

/**
 * 
 * Class for xml to xmi(ver. 2.0) transformation
 */
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Writer;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.topcased.bus.core.CorePlugin;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import com.sun.org.apache.xml.internal.serializer.Encodings;

public class Xmi1toXmi2
{

	public static Document document;

	private String metamodelName, rootName;

	private String xmiInputName, xmiOutputName;

	private String refNewValue;

	public static String space = "";

	private Writer wr;

	static File fdtd;

	private HashMap<String, Integer> map;

	private String encoding;

	public Xmi1toXmi2(String xmifile1, String xmifile2, String enc, String metamodel)
	{
		xmiInputName = xmifile1;
		xmiOutputName = xmifile2;
		encoding = enc;
		metamodelName = metamodel;
		rootName = "";
	}

	/**
	 * file metods
	 * 
	 */
	public void openOutputFile() throws IOException
	{
		try
		{
			wr = Encodings.getWriter(new FileOutputStream(xmiOutputName), encoding);
		}
		catch (IOException e)
		{
			throw new IOException("Open file " + xmiOutputName + " failed.");
		}
	}

	public void closeOutputFile() throws IOException
	{
		try
		{
			wr.close();
		}
		catch (IOException e)
		{
			throw new IOException("Close file " + xmiOutputName + " failed.");
		}
	}

	public static void createDTDFile(String fileName, String fileToImport)
	{
		Path pathDtd = new Path(fileName);
		String absolutePath = new String();
		if (!pathDtd.isAbsolute())
		{
			absolutePath = absolutePath.concat(fileToImport.substring(0, fileToImport.lastIndexOf(File.separator)));
			absolutePath = absolutePath.concat(File.separator).concat(fileName);
		}
		else
		{
			absolutePath = absolutePath.concat(fileName);
		}
		try
		{
			File dtdFile = new File(absolutePath);
			boolean createFile = dtdFile.createNewFile();
			if (createFile)
			{
				fdtd = dtdFile;
			}
		}
		catch (Exception e)
		{
			CorePlugin.log("Error on dtd file creation : " + absolutePath, IStatus.ERROR);
			CorePlugin.log(e);
		}
	}

	public static void deleteDTDFile(String fileName)
	{
		try
		{
			if (fdtd != null && fdtd.exists())
			{
				fdtd.delete();
			}
		}
		catch (Exception e)
		{
			CorePlugin.log("Error on dtd file deletion : " + fileName, IStatus.ERROR);
			CorePlugin.log(e);
		}
	}

	public void writeLine(String line) throws IOException
	{
		try
		{
			wr.write(line, 0, line.length());
			wr.write('\n');
		}
		catch (IOException e)
		{
			throw new IOException("Write file " + xmiOutputName + " error.");
		}
	}

	public void writeString(String line) throws IOException
	{
		try
		{
			wr.write(line, 0, line.length());
		}
		catch (IOException e)
		{
			throw new IOException("Write file " + xmiOutputName + " error.");
		}
	}

	public String getFileName(String name)
	{
		String path = null;
		StringTokenizer st = new StringTokenizer(name, "" + File.separatorChar);
		while (st.hasMoreTokens())
			path = st.nextToken();
		StringTokenizer st1 = new StringTokenizer(path, ".");
		path = st1.nextToken();
		return path;
	}

	private String getDtd(String fileName)
	{
		File xmlAsTextFile = new File(fileName);
		String dtdFileName = null;
		char[] subtext = new char[200];
		BufferedReader br = null;

		try
		{
			br = new BufferedReader(new InputStreamReader(new FileInputStream(xmlAsTextFile)));
		}
		catch (FileNotFoundException e)
		{
			CorePlugin.log(e);
		}

		boolean notTrouve = false;
		int completeLength = 0;
		try
		{
			completeLength = br.read(subtext);
		}
		catch (IOException e)
		{
			CorePlugin.log(e);
		}

		while (completeLength > 0 && !notTrouve)
		{
			int i = 0;
			while (i + 8 < completeLength && !notTrouve)
			{
				if ((subtext[i] == '<') && (subtext[i + 1] == '!') && (subtext[i + 2] == 'D') && (subtext[i + 3] == 'O')
						&& (subtext[i + 4] == 'C') && (subtext[i + 5] == 'T') && (subtext[i + 6] == 'Y') && (subtext[i + 7] == 'P')
						&& (subtext[i + 8] == 'E'))
				{
					// parse the remaining string
					// notTrouve = !notTrouve;
					int j = i + 9;
					while (j + 7 < completeLength && !notTrouve)
					{
						if ((subtext[j] == 'S') && (subtext[j + 1] == 'Y') && (subtext[j + 2] == 'S') && (subtext[j + 3] == 'T')
								&& (subtext[j + 4] == 'E') && (subtext[j + 5] == 'M') && (subtext[j + 6] == ' ') && (subtext[j + 7] == '"'))
						{
							int k = j + 8;
							char[] dtdFile = new char[1000];
							int l = 0;
							while (subtext[k] != '"')
							{
								dtdFile[l] = subtext[k];
								k = k + 1;
								l = l + 1;
							}
							dtdFileName = new String(dtdFile, 0, l);
							notTrouve = !notTrouve;
						}
						j = j + 1;
					}
				}
				i = i + 1;
			}
			try
			{
				completeLength = br.read(subtext);
			}
			catch (IOException e)
			{
				CorePlugin.log(e);
			}
		}
		return dtdFileName;
	}

	/**
	 * write the antet of the output file
	 * 
	 */
	public void writeAntet() throws IOException
	{
		writeLine("<?xml version = \"1.0\"" + " encoding = \"" + encoding + "\" ?>");
	}

	/**
	 * builds the Dom from input file
	 * 
	 */
	public void buildDom() throws SAXParseException, SAXException, ParserConfigurationException, IOException
	{
		Node dtd;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware(true);

		factory.setValidating(false);
		// factory.setIgnoringElementContentWhitespace(true);
		// Ignore les commentaires ...
		factory.setIgnoringComments(true);
		DocumentBuilder builder = factory.newDocumentBuilder();
		document = builder.parse(new File(xmiInputName));

		dtd = document.getDoctype();

		if (dtd != null)
		{
			document.removeChild(dtd);
		}
	}

	/**
	 * makes modif. in Dom
	 * 
	 */
	public void makeNewDom(Node domNode, String space) throws IOException
	{
		AdapterNode adnode = new AdapterNode(domNode);
		Node firstNode = domNode;
		while (domNode.getFirstChild() != null && Node.COMMENT_NODE == domNode.getFirstChild().getNodeType())
		{
			domNode = domNode.getFirstChild();
			writeLine(domNode.getNodeValue());
		}

		if (adnode.getTagAttribName() != null)
		{
			String attrName = adnode.getTagAttribName();
			Element e = (Element) domNode;
			Node node = e.getFirstChild();
			if (node != null)
			{
				e.setAttribute(attrName, node.getNodeValue());
				e.removeChild(node);
			}
		}
		
		List<String> v = adnode.getAttribRefName();
		if (domNode.hasAttributes() && !v.isEmpty())
		{
			for (int i = 0; i < v.size(); i++)
			{
				makeNodeNewRef(domNode, v.get(i), adnode.getRefValue(v.get(i)));
			}
		}

		writeNode(domNode, space);
		String s = space;
		space = space.concat("  ");
		List<Node> w = adnode.getElements();
		for (int i = 0; i < w.size(); i++)
		{
			makeNewDom(w.get(i), space);
		}
		space = s;

		if (w.size() > 0)
		{
			if (firstNode == document.getFirstChild())
				writeLine(space + "</" + upperCase(rootName) + ">");
			else
				writeLine(space + "</" + upperCase(domNode.getNodeName()) + ">");
		}
	}

	/**
	 * makes new ref.
	 * 
	 */
	public void makeNodeNewRef(Node modifNode, String refName, String refValue)
	{
		List<Node> lavel1Elements = new AdapterNode(document.getFirstChild()).getElements();
		Node refNode = null;
		Map<String, Integer> rootMap = new HashMap<String, Integer>();

		for (int i = 0; i < lavel1Elements.size() && refNode == null; i++)
		{
			Node parentNode = lavel1Elements.get(i);
			initMap(rootMap, parentNode);
			int n = (((Integer) rootMap.get((String) parentNode.getNodeName())).intValue());
			refNewValue = "//@" + upperCase(parentNode.getNodeName()) + "." + n;
			map = new HashMap<String, Integer>();
			initMap(map, parentNode);
			refNode = seekRefNode(parentNode, refValue);
		}

		if (refNode != null)
		{
			Node attribNode = (new AdapterNode(modifNode)).getAttribNode(refName);
			if (attribNode != null)
			{
				attribNode.setNodeValue(refNewValue);
			}
		}
	}

	public int initMap(Map<String, Integer> map, Node node)
	{
		Integer N = map.get(node.getNodeName());
		int n;
		if (N == null)
		{
			map.put(node.getNodeName(), new Integer(0));
			n = 0;
		}
		else
		{
			n = N.intValue() + 1;
			map.remove((String) node.getNodeName());
			map.put(node.getNodeName(), new Integer(n));
		}
		return n;
	}

	/**
	 * seeks ref. to nodes
	 */
	public Node seekRefNode(Node startNode, String refValue)
	{
		String s = refNewValue;
		Node referedNode = (new AdapterNode(startNode)).getElementbyID(refValue);
		if (referedNode != null)
		{
			return referedNode;
		}
		else
		{
			AdapterNode adnode = new AdapterNode(startNode);
			List<Node> v = adnode.getElements();
			for (int i = 0; i < v.size(); i++)
			{
				refNewValue = s;
				Node node = v.get(i);
				// retour de fonction ignore
				initMap(map, node);
				// variable duplique n --> n2
				int n2 = map.get(node.getNodeName()).intValue();
				String n2String = String.valueOf(n2);
				refNewValue = refNewValue.concat("/@").concat(upperCase(node.getNodeName())).concat(".").concat(n2String);
				referedNode = seekRefNode(node, refValue);
				if (referedNode != null)
				{
					return referedNode;
				}
			}
			refNewValue = s;
			return null;
		}
	}

	/**
	 * transf. of the first node
	 * 
	 */
	public void firstElemTransf()
	{
		(new AdapterNode(document.getLastChild())).firstTransf(metamodelName);

	}

	/**
	 * set the root name
	 * 
	 */
	public void setRootName()
	{
		Node node = document.getFirstChild();
		StringTokenizer st = new StringTokenizer(node.getNodeName(), ":");
		String s = "";
		while (st.hasMoreTokens())
		{
			s = st.nextToken();
		}
		rootName = metamodelName.concat(":").concat(s);
	}

	/**
	 * write the inf. of a node
	 * 
	 */
	private void writeNode(Node node, String space) throws IOException
	{
		if (node.getNodeType() != Node.DOCUMENT_TYPE_NODE)
		{
			if (node == document.getDocumentElement())
			{ // TODO check
				writeString(space + "<" + upperCase(rootName));
				NamedNodeMap map = node.getAttributes();
				Node n1 = map.getNamedItem("xmi:version");
				String valAttrib = n1.getNodeValue();
				writeString(" " + upperCaseAttribFirstNode(n1.getNodeName()) + "=\"" + valAttrib + "\"");
				Node n2 = map.getNamedItem("xmlns:xmi");
				valAttrib = n2.getNodeValue();
				writeString(" " + upperCaseAttribFirstNode(n2.getNodeName()) + "=\"" + valAttrib + "\"");
				Node n3 = map.getNamedItem("xmlns:" + metamodelName);
				valAttrib = upperCase2(n3.getNodeValue());
				writeString(" " + upperCaseAttribFirstNode(n3.getNodeName()) + "=\"" + valAttrib + "\"");
				for (int i = 0; i < map.getLength(); i++)
				{
					Node n = map.item(i);
					String s = n.getNodeName();
					if (!s.equals("xmi:version") && !s.equals("xmlns:xmi") && !s.equals("xmlns:" + metamodelName))
					{
						valAttrib = n.getNodeValue();
						writeString(" " + n.getNodeName() + "=\"" + valAttrib + "\"");
					}

				}
			}
			else
			{
				writeString(space + "<" + upperCase(node.getNodeName()));
				if (node.hasAttributes())
				{
					NamedNodeMap map = node.getAttributes();
					for (int i = 0; i < map.getLength(); i++)
					{
						String valAttrib;
						if (map.item(i).getNodeName().equals("value"))
							valAttrib = xmlString(map.item(i).getNodeValue(), true);
						else if (!map.item(i).getNodeName().equals("xml:space"))
							valAttrib = map.item(i).getNodeValue();
						else
							valAttrib = null;
						if (valAttrib != null)
							writeString(" " + map.item(i).getNodeName() + "=\"" + valAttrib + "\"");
					}
				}
			}
			if ((new AdapterNode(node)).hasChildElements())
				writeLine(">");
			else
				writeLine("/>");

		}
	}

	/**
	 * first char uppercase
	 * 
	 */
	public String upperCase(String s)
	{
		StringTokenizer analizor = new StringTokenizer(s, ":");
		String s1 = null;
		String s2 = null;
		int n = 0;
		while (analizor.hasMoreTokens())
		{
			String t = analizor.nextToken();
			if (n == 0)
				s1 = t;
			if (n == 1)
				s2 = t;
			n = n + 1;
		}
		s1 = Character.toUpperCase(s1.charAt(0)) + s1.substring(1);
		s = s1;
		if (s2 != null)
		{
			s2 = Character.toUpperCase(s2.charAt(0)) + s2.substring(1);
			s = s.concat(":").concat(s2);
		}
		return s;
	}

	public String upperCase2(String s)
	{
		StringTokenizer analizor = new StringTokenizer(s, "/\\");
		String s1 = null;
		String s2 = null;
		String s3 = null;
		String s4 = null;
		int n = 0;
		while (analizor.hasMoreTokens())
		{
			s1 = analizor.nextToken();
			n = n + 1;
		}
		if (s1 != null)
		{
			s2 = s1;
			StringTokenizer analizor2 = new StringTokenizer(s1, ".");
			n = 0;
			while (analizor2.hasMoreTokens() && n < 2)
			{
				if (n == 0)
					s3 = analizor2.nextToken();
				if (n == 1)
					s4 = analizor2.nextToken();
				n = n + 1;
			}
			if (s3 != null)
			{
				s3 = Character.toUpperCase(s3.charAt(0)) + s3.substring(1);
			}
			s1 = s3.concat(".").concat(s4);
			s = s.replaceFirst(s2, s1);
		}
		return s;
	}

	public String upperCaseAttribFirstNode(String s)
	{
		StringTokenizer analizor = new StringTokenizer(s, ":");
		String s1 = null;
		String s2 = null;
		int n = 0;
		while (analizor.hasMoreTokens())
		{
			String t = analizor.nextToken();
			if (n == 0)
				s1 = t;
			if (n == 1)
				s2 = t;
			n = n + 1;
		}
		s = s1;
		if (s2 != null)
		{
			if (!s2.equals("version") && !s2.equals("xmi"))
			{
				s2 = Character.toUpperCase(s2.charAt(0)) + s2.substring(1);
			}
			s = s.concat(":").concat(s2);
		}
		return s;
	}

	/**
	 * string to xml string
	 * 
	 */
	public String xmlString(String source, boolean total)
	{
		if (total)
		{
			source = replaceChar('&', source, "&#38;");
			source = replaceChar('<', source, "&lt;");
			source = replaceChar('>', source, "&gt;");
			source = replaceChar('\"', source, "&quot;");
			source = replaceChar('\'', source, "&apos;");
		}
		// additional character encoding Fr ISO-8859-1
		source = replaceChar((char) 0xE0, source, "&#224;");
		source = replaceChar((char) 0xE2, source, "&#226;");
		source = replaceChar((char) 0xE4, source, "&#228;");
		source = replaceChar((char) 0xE7, source, "&#231;");
		source = replaceChar((char) 0xE8, source, "&#232;");
		source = replaceChar((char) 0xE9, source, "&#233;");
		source = replaceChar((char) 0xEA, source, "&#234;");
		source = replaceChar((char) 0xEB, source, "&#235;");
		source = replaceChar((char) 0xEE, source, "&#238;");
		source = replaceChar((char) 0xEF, source, "&#239;");
		source = replaceChar((char) 0xF9, source, "&#249;");
		source = replaceChar((char) 0xFB, source, "&#251;");
		return source;
	}

	public String replaceChar(char c, String source, String s)
	{
		String s1 = "";
		for (int i = 0; i < source.length(); i++)
			if (source.charAt(i) != c)
				s1 = s1 + source.charAt(i);
			else
				s1 = s1.concat(s);
		return s1;
	}

	/**
	 * transformatin action
	 * 
	 */
	public String runImport()
	{

		String ret = "NOK";
		try
		{
			String dtdFileName = getDtd(xmiInputName);
			if (dtdFileName != null)
			{
				Xmi1toXmi2.createDTDFile(dtdFileName, xmiInputName);
			}
			// Xmi1toXmi2.createDTDFile(pathProject + separator + getFileName(xmiInputName));
			buildDom();
			openOutputFile();
			writeAntet();
			firstElemTransf();
			setRootName();
			makeNewDom(Xmi1toXmi2.document.getFirstChild(), Xmi1toXmi2.space);
			closeOutputFile();
			// Xmi1toXmi2.deleteDTDFile(pathProject + separator + getFileName(xmiInputName));
			// Xmi1toXmi2.deleteDTDFile(pathProject + separator + getFileName(xmiInputName));
			if (dtdFileName != null)
			{
				Xmi1toXmi2.deleteDTDFile(dtdFileName);
			}
			ret = "OK";
			return ret;
		}
		catch (IOException e1)
		{
			ret = e1.toString();
			return ret;
		}
		catch (SAXParseException e2)
		{
			ret = "Parsing error, " + e2.getMessage() + " line " + e2.getLineNumber() + " column " + e2.getColumnNumber() + ", uri "
					+ e2.getSystemId();
			return ret;
		}
		catch (SAXException e3)
		{
			ret = e3.toString();
			return ret;
		}
		catch (ParserConfigurationException e4)
		{
			ret = e4.toString();
			return ret;
		}
	}
}
