/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.transformation.services.external;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.topcased.externalbus.core.CorePlugin;
import org.topcased.externalbus.core.IExternalService;
import org.topcased.service.transformation.Transformation;
import org.w3c.dom.Element;

/**
 * External service of Import. It transforms model. <br>
 * creation : 30 juin 2005
 * 
 */
public class TransformationExternal implements IExternalService
{

	private static int currentNumber = 0;

	private IProject project;

	private final static String PROJECT_WORKSPACE = "HttpServerWorkspace";

	/**
	 * Import Model
	 * 
	 * @see org.topcased.externalbus.core.IExternalService#externalServiceRun(java.util.Map)
	 */
	public Object externalServiceRun(Map<String, Object> parameters)
	{
		// Bloc declaration
		String paramTransfoType = "";
		String output = "";
		String currentDirectory = "";

		currentNumber = currentNumber + 1;

		Map<String, Element> models = new HashMap<String, Element>();
		Map<String, String> modelnames = new HashMap<String, String>();

		try
		{
			project = ResourcesPlugin.getWorkspace().getRoot().getProject(PROJECT_WORKSPACE);
			project.open(null);
			currentDirectory = project.getLocation().toOSString();
		}
		catch (CoreException e1)
		{
			CorePlugin.log(e1);
		}

		boolean paramsOk = true;
		String pathReturn = "";

		// Map null
		if (parameters == null)
		{
			CorePlugin.log(this.getClass(), "Parameters list is null. It must have exactly 7 parameters", IStatus.ERROR);
			paramsOk = false;
			return null;
		}

		// get the input models
		if (parameters.containsKey("models"))
		{
			StringTokenizer tokenizer = new StringTokenizer((String) parameters.get("models"), ":", false);
			while (tokenizer.hasMoreTokens())
			{
				modelnames.put(tokenizer.nextToken(), tokenizer.nextToken());
			}
		}

		for (String model : modelnames.keySet())
		{
			String name = modelnames.get(model);
			if (parameters.containsKey(name))
			{
				if (parameters.get(name) instanceof Element)
				{
					models.put(model, (Element) parameters.get(name));

				}
				else
				{
					CorePlugin.log(this.getClass(), "The '" + name + "' parameter must be an Element", IStatus.ERROR);
					paramsOk = false;
				}
			}
			else
			{
				CorePlugin.log(this.getClass(), "Parameters number is not correct. It must have the parameter : '" + name + "'",
						IStatus.ERROR);
				paramsOk = false;
			}
		}

		// get the output name
		if (parameters.containsKey("output"))
		{
			if (parameters.get("output") instanceof String)
			{
				output = (String) parameters.get("output");
			}
			else
			{
				CorePlugin.log(this.getClass(), "The 'output' parameter must be a String", IStatus.ERROR);
				paramsOk = false;
			}
		}

		// Get 'TransformationType' parameter
		if (parameters.containsKey("TransformationType"))
		{
			if (parameters.get("TransformationType") instanceof String)
			{
				paramTransfoType = (String) parameters.get("TransformationType");
			}
			else
			{
				CorePlugin.log(this.getClass(), "The 'TransformationType' parameter must be a String", IStatus.ERROR);
				paramsOk = false;
			}
		}
		else
		{
			CorePlugin.log(this.getClass(), "Parameters number is not correct. It must have the parameter : 'TransformationType'",
					IStatus.ERROR);
			paramsOk = false;
		}

		// EXTERNAL SERVICE : TRANSFORMATION
		if (paramsOk)
		{
			CorePlugin.log(this.getClass(), "All parameters are given to the WebImportAction. Launch Service.", IStatus.INFO);
			
			// Constructor
			String result = "result.xmi";
			String inputFile = "";
			Map<String, String> modelFile = new HashMap<String, String>();
			boolean isAnXmlModel = false;

			// put the result model
			modelFile.put(output, result);
			try
			{
				for (String model : models.keySet())
				{
					inputFile = currentDirectory + File.separator + model + ".xml";
					Element modelNode = models.get(model);

					// Test on the type : xml or xmi ?
					isAnXmlModel = isXmlModel(modelNode);

					modelFile.put(model, inputFile);
					TransformerFactory transFactory = TransformerFactory.newInstance();

					Transformer transformer = transFactory.newTransformer();
					transformer.setOutputProperty(OutputKeys.INDENT, "yes");
					transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "3");
					transformer.transform(new DOMSource(modelNode), new StreamResult(inputFile));
				}
			}
			catch (Exception e)
			{
				CorePlugin.log(e);
			}

			new Transformation(paramTransfoType, modelFile, project.getFullPath(), isAnXmlModel, null);
			if (!inputFile.equals("") && new File(inputFile).exists())
			{
				new File(inputFile).delete();
			}
			return currentDirectory + File.separator + result;
		}
		else
		{
			String message = "All parameters were not given to the WebImportAction. External Service is not Launched.";
			CorePlugin.log(this.getClass(), message, IStatus.ERROR);
		}
		try
		{
			// closes the project opened for the transformation
			project.close(null);
		}
		catch (CoreException e)
		{
			CorePlugin.log(e);
		}
		return pathReturn;
	}

	/**
	 * Reads an input model and determine if it is a XML model or a XMI model. This method is called only by external services.
	 * 
	 * @param modelMap The Map of input models
	 * @return true if it is an XML model, false otherwise
	 */
	private boolean isXmlModel(Element node)
	{
		return (!node.hasAttribute("xmi:version"));
	}
}
