/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.transformation.wizard;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.topcased.facilities.dialogs.ITopcasedWizardConstants;
import org.topcased.service.transformation.TransformationPlugin;

/**
 * An import wizard enabling to choose an ATL transformation (5 availables) to perform.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 *
 */
public class ImportWizard extends Wizard implements INewWizard, ITopcasedWizardConstants
{
	/** path of the current project */
	private Path pathProject = null;

	/** Constant indicating that this wizard concerns export */
	public static final String IMPORT_KIND = "import";
	
	/**
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	public void addPages()
	{
		TransformationPage mainPage = new TransformationPage(pathProject, IMPORT_KIND);
		mainPage.setTitle(TransformationPlugin.getResourceString("ImportWizard.title"));
		mainPage.setDescription(TransformationPlugin.getResourceString("ImportWizard.description"));
		addPage(mainPage);
	}

	/**
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	public boolean performFinish()
	{
		return ((TransformationPage) getPage(IMPORT_KIND)).performFinish();
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench w, IStructuredSelection s)
	{
		// Image
		setDefaultPageImageDescriptor(TransformationPlugin.getImageDescriptor(TOPCASED_LOGO_PATH));

		// Title
		setWindowTitle(TransformationPlugin.getResourceString("ImportWizard.window.title"));

		selectionChanged(s);
	}

	/**
	 * Calls when the selection changes.
	 * 
	 * @param selection The chosen item
	 */
	public void selectionChanged(ISelection selection)
	{
		IStructuredSelection structured = (IStructuredSelection) selection;
		Object object = structured.getFirstElement();
		if (object instanceof IAdaptable)
		{
			object = ((IAdaptable) object).getAdapter(IResource.class);
		}
		// Retrieves the path of the project
		if (object != null && object instanceof IResource)
		{
			IResource res = (IResource) object;
			if (res instanceof IContainer)
			{
				pathProject = new Path(res.getFullPath().toString());
			}
			else
			{
				pathProject = new Path(res.getParent().getFullPath().toString());
			}
		}
	}
}
