/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/
package org.topcased.service.updatesite;

import java.io.File;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

/**
 * Wizard Page which is used to choose the source directory and the updatesite
 * directory Some informations about the features can be saw And a filter permit
 * to show only the features which are not included in other features
 * 
 * @version 1.0.0
 */
public class BrowseUpdateSiteWizardPage extends WizardPage
{
    private Table tableUS;

    private Table tableTA;
    
    private Text updateSiteBrowseText;

    private Button updateSiteBrowseButton;
    
    private Button deleteButton;
    
    private Button addButton;
    
    private BrowseUpdateSiteWizardTreatment treatment;

    private UpdateSiteWizard wizard;

    /**
     * Constructor
     * 
     * @param wizard the parent wizard
     * @param pageName the name of the page
     */
    protected BrowseUpdateSiteWizardPage(UpdateSiteWizard wizard, String pageName)
    {
        super(pageName);
        this.wizard = wizard;
        treatment = new BrowseUpdateSiteWizardTreatment(wizard);
    }

    /**
     * 
     * @return the delete button of the browse page
     */
    public Button getDeleteButton()
    {
    	return deleteButton;
    }
    
    /**
     * 
     * @return the add button of the browse page
     */
    public Button getAddButton()
    {
    	return addButton;
    }
    
    /**
     * 
     * @return the Update Site directory
     */
    public String getDirectory()
    {
        return updateSiteBrowseText.getText();
    }

    /**
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent)
    {
    	/*
    	 * Definition of some GridData 
    	 */
        // GridData for Update Site table and Target Directory table
        GridData gridDataForTable = new GridData(GridData.FILL_BOTH);
        gridDataForTable.heightHint = 195;
        
        // GridData for components who need to be filled and grabed horizontally
        GridData horizontalFillAndGrabGridData = new GridData(GridData.FILL_HORIZONTAL);
        
        // GridData for component who need to be filled and grabbed horizontally and vertically + the vertical align starts at the beginning of the area.
        GridData fullFillAndGrabGridData = new GridData(GridData.FILL_BOTH);
        
        // GridData for bottom information button 
        GridData gridDataButtonInfo = new GridData(GridData.FILL_HORIZONTAL);
        gridDataButtonInfo.horizontalSpan = 3;
    	
    	// Main composite
        final Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout(1, false));
        setControl(container);

        // Update composite (main area)
        final Composite updateComposite = new Composite(container, SWT.NONE);
        updateComposite.setLayoutData(fullFillAndGrabGridData);
        updateComposite.setLayout(new GridLayout(3, false));
        
        // tool administration composite (left) 
        final Composite toolAdministrationComposite = new Composite(updateComposite, SWT.NONE);
        toolAdministrationComposite.setLayoutData(fullFillAndGrabGridData);
        toolAdministrationComposite.setLayout(new GridLayout());
        
        // Add / Delete composite (middle)
        final Composite addDeleteComposite = new Composite(updateComposite, SWT.NONE);
        addDeleteComposite.setLayoutData(new GridData());
        addDeleteComposite.setLayout(new GridLayout());   
        
        // update site composite (right)
        final Composite updateSiteComposite = new Composite(updateComposite, SWT.NONE);
        updateSiteComposite.setLayoutData(fullFillAndGrabGridData);
        updateSiteComposite.setLayout(new GridLayout());
        
        // browse tool administration (left)
        final Composite browseToolAdministrationComposite = new Composite(toolAdministrationComposite, SWT.NONE);
        browseToolAdministrationComposite.setLayoutData(horizontalFillAndGrabGridData);
        browseToolAdministrationComposite.setLayout(new GridLayout(2, false));
        
        // browse update site (right)
        final Composite browseUpdateSiteComposite = new Composite(updateSiteComposite, SWT.NONE);
        browseUpdateSiteComposite.setLayoutData(horizontalFillAndGrabGridData);
        browseUpdateSiteComposite.setLayout(new GridLayout(2, false));
            
        // Bottom composite for information button 
        final Composite downComposite = new Composite(container, SWT.NONE);
        downComposite.setLayoutData(gridDataButtonInfo);
        downComposite.setLayout(new GridLayout(1, false));
        
        // field and button for the tool administration (left)
        final Text toolAdministrationText = new Text(browseToolAdministrationComposite, SWT.BORDER);
        toolAdministrationText.setLayoutData(horizontalFillAndGrabGridData);
        toolAdministrationText.setEditable(false);
        toolAdministrationText.setText(wizard.getUpdateSiteToolAdminPath());
        final Button toolAdministrationBrowseButton = new Button(browseToolAdministrationComposite, SWT.NONE);
        toolAdministrationBrowseButton.setLayoutData(new GridData());
        toolAdministrationBrowseButton.setText("Browse...");

        // field and button for the update site (right)
        updateSiteBrowseText = new Text(browseUpdateSiteComposite, SWT.BORDER);
        updateSiteBrowseText.setLayoutData(horizontalFillAndGrabGridData);
        updateSiteBrowseText.setText("");
        updateSiteBrowseButton = new Button(browseUpdateSiteComposite, SWT.NONE);
        updateSiteBrowseButton.setLayoutData(new GridData());
        updateSiteBrowseButton.setText("Browse...");

        // Left table for the tool administration 
        tableTA = new Table(toolAdministrationComposite, SWT.BORDER);
        tableTA.setLayoutData(gridDataForTable);
        tableTA.setLinesVisible(true);
        tableTA.setHeaderVisible(true);
        tableTA.getVerticalBar().setEnabled(true);
        
        // Right table for UpdateSite : Content of repository of 'updateSitePath'.
        tableUS = new Table(updateSiteComposite, SWT.BORDER);
        tableUS.setLayoutData(gridDataForTable);
        tableUS.setLinesVisible(true);
        tableUS.setHeaderVisible(true);
        tableUS.getVerticalBar().setEnabled(true);
        
        // Right table for update site
        final TableColumn tableColumnRepositoryUS = new TableColumn(tableUS, SWT.NONE);
        tableColumnRepositoryUS.setWidth(160);
        tableColumnRepositoryUS.setText("Content");

        // Left table for update site
        final TableColumn tableColumnToolAdministration = new TableColumn(tableTA, SWT.NONE);
        tableColumnToolAdministration.setWidth(160);
        tableColumnToolAdministration.setText("Content");
        
        // Initialization of tool administration and update site area
        tableUS.removeAll();
        tableTA.removeAll();
        String tempPath = wizard.getUpdateSitePath();
        if (!tempPath.endsWith(File.separator))
        {
        	tempPath += File.separator;
        }
        
        String tempToolPath = wizard.getUpdateSiteToolAdminPath();
        if (!tempToolPath.endsWith(File.separator))
        {
        	tempToolPath += File.separator;
        }        
        
        // Initialization of tool administration (left) 
        treatment.setMapContentRepositoryPluginFeatureTA(treatment.initializeMap(tempToolPath + "features" + File.separator));
        treatment.getMapContentRepositoryPluginFeatureTA().putAll(treatment.initializeMapWithJarFile(tempToolPath + "features" + File.separator));
        treatment.displayMap(treatment.getMapContentRepositoryPluginFeatureTA(), tableTA);
        
        // Initialization of update site (right)
        treatment.setMapContentRepositoryPluginFeatureUS(treatment.initializeMap(tempPath + "features" + File.separator));
        treatment.getMapContentRepositoryPluginFeatureUS().putAll(treatment.initializeMapWithJarFile(tempPath + "features" + File.separator));
        treatment.displayMap(treatment.getMapContentRepositoryPluginFeatureUS(), tableUS);

        // Add button
        addButton = new Button(addDeleteComposite, SWT.NONE);
        addButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        addButton.setText("Add");

        // Delete button
        deleteButton = new Button(addDeleteComposite, SWT.NONE);
        deleteButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        deleteButton.setText("Delete");
        
        // Checkbox filter
        final Button checkboxFilter = new Button(downComposite, SWT.CHECK);
        checkboxFilter.setLayoutData(new GridData());
        checkboxFilter.setText("Filter the included Features");
        checkboxFilter.setSelection(false);
        
        // Information button
        final Button informationButton = new Button(downComposite, SWT.NONE);
        informationButton.setText("Feature information");
        informationButton.setToolTipText("Get more information about the selected feature.");

        /*
         * BUTTONS ACTION
         */
        // Checkbox
        checkboxFilter.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (checkboxFilter.getSelection())
                {
                    treatment.filterMap(tableTA);
                }
                else
                {
                    treatment.noFilterMap(tableTA);
                }
            }
        });
        
        
        // Button : Browse of the Tool Administrator
        toolAdministrationBrowseButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                treatment.browseToolAdministratorClick(toolAdministrationText, tableTA);
                if (checkboxFilter.getSelection())
                {
                    treatment.filterMap(tableTA);
                }
                if (!toolAdministrationText.getText().equals("") && !updateSiteBrowseText.getText().equals("")
                        && !wizard.getMapAddPluginFeatureUS().isEmpty())
                {
                    setPageComplete(true);
                }
                else
                {
                    setPageComplete(false);
                }
            }
        });

        // Button : Browse of the Update Site
        updateSiteBrowseButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                switch (wizard.getProtocolConnex())
                {
                    case (UpdateSiteWizard.FTP):
                        treatment.browseUpdateSiteFTP(updateSiteBrowseText, tableUS);
                        break;
                    case (UpdateSiteWizard.SSH):
                        treatment.browseUpdateSiteSSH(updateSiteBrowseText, tableUS);
                        break;
                    default:
                        treatment.browseUpdateSiteJarFile(updateSiteBrowseText, tableUS);
                    	if(updateSiteBrowseText.getText().equals(""))
                    	{
                    		addButton.setEnabled(false);
                    		deleteButton.setEnabled(false);
                    	}
                    	else
                    	{
                    		addButton.setEnabled(true);
                    		deleteButton.setEnabled(true);
                    	}
                        break;
                }

                if (!toolAdministrationText.getText().equals("") && !updateSiteBrowseText.getText().equals("")
                        && !wizard.getMapAddPluginFeatureUS().isEmpty())
                {
                    setPageComplete(true);
                }
                else
                {
                    setPageComplete(false);
                }
            }
        });

        // Button : Add plugins/feature selected in the Update Site
        addButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                TableItem[] itemsSelected = tableTA.getSelection();
                for (int indice = 0; indice < itemsSelected.length; indice++)
                {
                    TableItem itemSelected = itemsSelected[indice];
                    String nameSelected = itemSelected.getText();
                    treatment.addPluginFeature(nameSelected, tableUS);
                }
                if (!toolAdministrationText.getText().equals("") && !updateSiteBrowseText.getText().equals("")
                        && !wizard.getMapAddPluginFeatureUS().isEmpty())
                {
                    setPageComplete(true);
                }
                else
                {
                    setPageComplete(false);
                }
            }
        });

        // Button : Delete plugins/feature selected from the Update Site
        deleteButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                TableItem[] itemsSelected = tableUS.getSelection();
                for (int indice = 0; indice < itemsSelected.length; indice++)
                {
                    treatment.deletePluginFeature(itemsSelected[indice].getText(),tableUS);
                }
                if (!toolAdministrationText.getText().equals("") && !updateSiteBrowseText.getText().equals("")
                        && !wizard.getMapAddPluginFeatureUS().isEmpty())
                {
                    setPageComplete(true);
                }
                else
                {
                    setPageComplete(false);
                }
            }
        });

        // Button : informations for the selected feature
        informationButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                TableItem[] itemsSelected = tableTA.getSelection();
                for (int indice = 0; indice < itemsSelected.length; indice++)
                {
                    TableItem itemSelected;
                    String nameSelected;
                    String path;

                    itemSelected = itemsSelected[indice];
                    nameSelected = itemSelected.getText();
                    // SearchSource
                    path = treatment.searchSource(nameSelected, true);
                    treatment.infosFeature(path);

                }
            }
        });
        
        // Add a listener to enable the add and delete buttons if a repository path has been chosen
        updateSiteBrowseText.addKeyListener(new KeyListener()
        {
			public void keyPressed(KeyEvent e)
			{
				if (wizard.getConnectionPage().getProtocol() == UpdateSiteWizard.FILENOSEC)
		        {
					if(updateSiteBrowseText.getText().equals(""))
					{
						addButton.setEnabled(false);
						deleteButton.setEnabled(false);
					}
					else
					{
						addButton.setEnabled(true);
						deleteButton.setEnabled(true);
					}
		        }
			}

			public void keyReleased(KeyEvent e)
			{
				if (wizard.getConnectionPage().getProtocol() == UpdateSiteWizard.FILENOSEC)
		        {
					if(updateSiteBrowseText.getText().equals(""))
					{
						addButton.setEnabled(false);
						deleteButton.setEnabled(false);
					}
					else
					{
						addButton.setEnabled(true);
						deleteButton.setEnabled(true);
					}
		        }
			}
        	
        });
        setPageComplete(false);
    }

    /**
     * @return boolean
     */
    public boolean finish()
    {
        return isPageComplete();
    }

    /**
     * @see org.eclipse.jface.wizard.WizardPage#canFlipToNextPage()
     */
    public boolean canFlipToNextPage()
    {
    	wizard.setProtocolConnex(wizard.getConnectionPage().getProtocol());
        if (!wizard.isConnected() && wizard.getConnectionPage().getProtocol() != UpdateSiteWizard.FILENOSEC
                && wizard.getConnectionPage().getProtocol() != UpdateSiteWizard.FILESEC)
        {
            wizard.getConnectionPage().updateConnectionInformations();
            switch (wizard.getProtocolConnex())
            {
                case (UpdateSiteWizard.FTP):
                    wizard.createConnectionFTP();
                	updateSiteBrowseText.setText(wizard.getConnectionPage().getUSPath().getText());
            		updateSiteBrowseText.setEnabled(false);
            		updateSiteBrowseButton.setEnabled(false);
                	treatment.browseUpdateSiteFTP(updateSiteBrowseText, tableUS);
                    break;
                case (UpdateSiteWizard.SSH):
                	updateSiteBrowseText.setEnabled(false);
                	updateSiteBrowseButton.setEnabled(false);
                	if (wizard.createConnectionSSH())
                    {	
                    	updateSiteBrowseText.setText(wizard.getConnectionPage().getUSPath().getText());
                    	addButton.setEnabled(true);
                    	deleteButton.setEnabled(true);
                    	treatment.browseUpdateSiteSSH(updateSiteBrowseText, tableUS);
                    }
                    else
                    {
                    	updateSiteBrowseText.setText("");
                    	addButton.setEnabled(false);
                    	deleteButton.setEnabled(false);
                    }
                    break;
            }

            wizard.getConnectionPage().getTreatment().updateConnectionProfiles();
        }
        return finish();
    }
    
    public void update()
    {	
    	wizard.getMapAddPluginFeatureUS().clear();
    	wizard.getMapPluginUS().clear();
    	wizard.getMapFeatureInclude().clear();
    	wizard.getMapRequiredPlugins().clear();
    	tableUS.removeAll();  
		if (wizard.getConnectionPage().getProtocol() == UpdateSiteWizard.FILENOSEC)
		{
    		updateSiteBrowseText.setText("");
    		updateSiteBrowseText.setEnabled(true);
			updateSiteBrowseButton.setEnabled(true);
	    	addButton.setEnabled(false);
	    	deleteButton.setEnabled(false);
		}
		setPageComplete(false);
    }
    
}
