/*********************************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS)
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*********************************************************************************************/ 
package org.topcased.service.updatesite;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ftp.FtpException;
import org.eclipse.ftp.IDirectoryEntry;
import org.eclipse.ftp.internal.FTPClient;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.topcased.bus.core.CorePlugin;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.sshtools.j2ssh.SftpClient;
import com.sshtools.j2ssh.sftp.SftpFile;

/**
 * All treatments necessary to the WizardPage BrowseUpdateSiteWizardPage
 * 
 * @version 1.0.0
 */
public class BrowseUpdateSiteWizardTreatment
{
    // Map of the Source/Destination directories
    private Map mapContentRepositoryPluginFeatureUS; // key=name;
                                                   
    private Map mapContentRepositoryPluginFeatureTA; // key=name;

    // XML tags
    private static final String INCLUDES = "includes";
    
    // XML Attributs
    private static final String ID = "id";

    private static final String VERSION = "version";

    // Reference to the current wizard which use this page
    private UpdateSiteWizard wizard;

    private Map mapFilter;

    private String error;
	
	private Vector IOFTPSSHErrorVector;

    /**
     * Constructor
     * 
     * @param wizard the parent wizard
     */
    public BrowseUpdateSiteWizardTreatment(UpdateSiteWizard wizard)
    {
        this.wizard = wizard;
        mapContentRepositoryPluginFeatureUS = new TreeMap();
        mapContentRepositoryPluginFeatureTA = new TreeMap();
    }

    /**
     * @return Returns the mapContentRepositoryPluginFeatureTA.
     */
    public Map getMapContentRepositoryPluginFeatureTA()
    {
        return mapContentRepositoryPluginFeatureTA;
    }

    /**
     * @param mapContentRepositoryPluginFeatureTA The
     *            mapContentRepositoryPluginFeatureTA to set.
     */
    public void setMapContentRepositoryPluginFeatureTA(Map mapContentRepositoryPluginFeatureTA)
    {
        this.mapContentRepositoryPluginFeatureTA = mapContentRepositoryPluginFeatureTA;
    }

    /**
     * @return Returns the mapContentRepositoryPluginFeatureUS.
     */
    public Map getMapContentRepositoryPluginFeatureUS()
    {
        return mapContentRepositoryPluginFeatureUS;
    }

    /**
     * @param mapContentRepositoryPluginFeatureUS The mapContentRepositoryPluginFeatureUS to set.
     */
    public void setMapContentRepositoryPluginFeatureUS(Map mapContentRepositoryPluginFeatureUS)
    {
        this.mapContentRepositoryPluginFeatureUS = mapContentRepositoryPluginFeatureUS;
    }
    
    /**
     * Initialize a map with the given directory
     * 
     * @param repositoryPath
     * @return Map
     */
    public Map initializeMap(String repositoryPath)
    {
        Map mapReturn = new TreeMap();
        
        // Content of the table = content of the repository "repositoryPath"
        File repository = new File(repositoryPath);
        if (repository != null)
        {
            File[] repositoryContent = repository.listFiles((new FileFilter()
            {
                public boolean accept(File pathname)
                {
                    return pathname.isDirectory();
                }
            }));

            // Add the sub repository in Map of Update Site
            if (repositoryContent != null)
            {
                for (int indice = 0; indice < repositoryContent.length; indice++)
                {
                    File subRepository = repositoryContent[indice];
                    mapReturn.putAll(pathToMap(subRepository));
                }
            }
        }
        return mapReturn;
    }

    /**
     * Display the content Map "map" in the table "table"
     * 
     * @param map : key=name; value=SourceDestination
     * @param table : location where the map is display
     */
    public void displayMap(Map map, Table table)
    {
        Iterator it = map.keySet().iterator();
        while (it.hasNext())
        {
            String key = (String) it.next();
            if (map.get(key) instanceof SourceDestination)
            {
                TableItem itemAdd = new TableItem(table, 0);
                itemAdd.setText(key);
                table.showItem(itemAdd);
                table.redraw();
            }
        }
        table.redraw();
        table.update();
    }

    /**
     * Return the corresponding Map of all 'jar' File from the path repository
     * or file
     * 
     * @param repositoryPath
     * @return Map : key=name, value=SourceDestination
     */
    public Map initializeMapWithJarFile(String repositoryPath)
    {
        Map mapReturn = new TreeMap();
        // Content of the table = content of the repository "repositoryPath"
        File repository = new File(repositoryPath);
        if (repository != null)
        {
            File[] repositoryContent = repository.listFiles((new FileFilter()
            {
                public boolean accept(File pathname)
                {
                    boolean acceptReturn = false;
                    if (pathname.isFile())
                    {
                        String jarFile = pathname.toString();
                        jarFile = jarFile.substring(jarFile.lastIndexOf('.'));
                        if (jarFile.equals(".jar"))
                        {
                            acceptReturn = true;
                        }
                    }
                    return acceptReturn;
                }
            }));

            // Add the sub repository in Map of Update Site
            File subRepository = null;
            if (repositoryContent != null)
            {
                for (int indice = 0; indice < repositoryContent.length; indice++)
                {
                    subRepository = repositoryContent[indice];
                    mapReturn.putAll(pathToMap(subRepository));
                }
            }
        }
        return mapReturn;
    }

    /**
     * Button Action : Browse Directory for the Tool Administrator
     * 
     * @param text : location where the path will be read and write
     * @param table : location where the content of the repository appear
     */
    public void browseToolAdministratorClick(Text text, Table table)
    {
        DirectoryDialog dialogBrowse = new DirectoryDialog(wizard.getShell());
        dialogBrowse.setFilterPath(UpdateSiteWizard.checkPath(wizard.getUpdateSiteToolAdminPath(), true));
        String dialogOpen = dialogBrowse.open();

        // Browse Dialog is opened
        if (dialogOpen != null)
        {
            // Take the path of the repository
            String repositoryPath = UpdateSiteWizard.checkPath(dialogOpen, true);
            text.setText(repositoryPath);
            wizard.setUpdateSiteToolAdminPath(repositoryPath);

            // Initialization
            table.removeAll();
            mapContentRepositoryPluginFeatureTA = initializeMap(UpdateSiteWizard.checkPath(wizard.getUpdateSiteToolAdminPath() + "features", true));
            mapContentRepositoryPluginFeatureTA.putAll(initializeMapWithJarFile(UpdateSiteWizard.checkPath(wizard.getUpdateSiteToolAdminPath() + "features", true)));
            displayMap(mapContentRepositoryPluginFeatureTA, table);
        }
    }

    /**
     * Button Action : Browse Directory for the Update Site (jar File)
     * 
     * @param text : location where the path will be read and write
     * @param table : location where the content of the 'jar' file appear
     */
    public void browseUpdateSiteJarFile(Text text, Table table)
    {
        String dialogOpen;
        DirectoryDialog dialogBrowse = new DirectoryDialog(wizard.getShell());
        dialogBrowse.setFilterPath(wizard.getUpdateSitePath());

        // Browse Dialog is opened
        dialogOpen = dialogBrowse.open();
        if (dialogOpen != null)
        {
            // Take the path of the repository
            String repositoryPath = UpdateSiteWizard.checkPath(dialogOpen, wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC);
            
            File testFileFeat = new File(UpdateSiteWizard.checkPath(repositoryPath + "features", wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC));
            File testFilePlug = new File(UpdateSiteWizard.checkPath(repositoryPath + "plugins", wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC));

            if (!testFileFeat.exists() || !testFilePlug.exists())
            {
                int button;
                MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
                msgBox.setText("Missing directories");
                msgBox.setMessage("The repertories necessary to the installation are missing\nDo you want to create them ?");
                button = msgBox.open();
                if (button == SWT.YES)
                {
                    if (!(testFileFeat.mkdir() && testFilePlug.mkdir()))
                    {
                        msgBox = new MessageBox(wizard.getShell(), SWT.ICON_ERROR | SWT.OK);
                        msgBox.setText("Error");
                        msgBox.setMessage("The directories can't be created\nVerify your permissions");
                        msgBox.open();
                        return;
                    }
                }
                else
                {
                    return;
                }
            }

            text.setText(repositoryPath);
            wizard.setUpdateSitePath(repositoryPath);

            // Add the sub repository in Map of Update Site
            mapContentRepositoryPluginFeatureUS = initializeMap(wizard.getUpdateSitePath() + "features" + File.separator);
            mapContentRepositoryPluginFeatureUS.putAll(initializeMapWithJarFile(wizard.getUpdateSitePath() + "features" + File.separator));
            
            // Test the XML structure and the repertory structure.
            CheckXMLValidity.startTest(wizard, wizard.getUpdateSitePath(), mapContentRepositoryPluginFeatureUS);
            
            table.removeAll();
            displayMap(mapContentRepositoryPluginFeatureUS, table); //, mapDeletePluginFeatureUS);
        }
    }

    /**
     * Button Action : Browse the Update Site from a FTP server
     * 
     * @param text : location where the path will be read and write
     * @param table : location where the content of the repository appear
     */
    public void browseUpdateSiteFTP(Text text, Table table)
    {
		final String path = UpdateSiteWizard.checkPath(text.getText(), wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC);
		String newPath = path;

        IRunnableWithProgress runnableWithProgress = new IRunnableWithProgress()
        {
            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
            {
                try
                {
                    wizard.getClientFTP().changeDirectory(path, monitor);
                    wizard.getClientFTP().createDirectory("features", monitor);
					wizard.getClientFTP().createDirectory("plugins", monitor);
                }
                catch (FtpException e)
                {
                    CorePlugin.log(e);
                    error = e.getMessage();
                }

            }
        };
        ProgressMonitorDialog dialog = new ProgressMonitorDialog(wizard.getShell());
        try
        {
            error = "";
			wizard.getClientFTP().changeDirectory(path, null);
			
            boolean findFeat = false;
			boolean findPlug = false;
			try
            {
				wizard.getClientFTP().changeDirectory("features", null);
				findFeat = true;
				wizard.getClientFTP().changeDirectory("..", null);
			}
            catch (FtpException e)
            {}
			
			try
            {
				wizard.getClientFTP().changeDirectory("plugins", null);
				findPlug = true;
				wizard.getClientFTP().changeDirectory("..", null);
			}
            catch (FtpException e)
            {}
			
			if (!findPlug||!findFeat)
			{
				int button;
                MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
                msgBox.setText("Missing directories");
                msgBox.setMessage("The repertories necessary to the installation are missing\nDo you want to create them ?");
                button = msgBox.open();
                if (button == SWT.YES)
                {
					dialog.run(true, true, runnableWithProgress);
                }
				else
				{
					wizard.setUpdateSitePath("");
		            text.setText(wizard.getUpdateSitePath());
					return;
				}
			}
        }
        catch (InvocationTargetException e)
        {
        	CorePlugin.log(e);
            error = e.getMessage();
        }
        catch (InterruptedException e)
        {
        	CorePlugin.log(e);
            error = e.getMessage();
        }
		catch (FtpException e)
		{
			CorePlugin.log(e);
            error = e.getMessage();
		}
        if (!error.equals(""))
        {
            MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_ERROR | SWT.OK);
            msgBox.setText("Connection error");
            msgBox.setMessage("Error in directory browsing : " + error + "\nSee Console informations");
            msgBox.open();
            wizard.setUpdateSitePath("");
            text.setText(wizard.getUpdateSitePath());
            return;
        }
        else
        {
            final String tempDir = UpdateSiteWizard.checkPath(System.getProperty("java.io.tmpdir"), true);
            
            File testFileFeat = new File(tempDir + "features");
            File testFilePlug = new File(tempDir + "plugins");
            File testFileSite = new File(tempDir + "site.xml");
            testFileSite.delete();
            FilesTreatment.emptyFile(testFileFeat);
            FilesTreatment.emptyFile(testFilePlug);
            testFileFeat.mkdir();
            testFileFeat.deleteOnExit();
            testFilePlug.mkdir();
            testFilePlug.deleteOnExit();

            wizard.setUpdateSitePath(newPath);

			boolean retryTreatment;
			
			do
			{
				IOFTPSSHErrorVector = new Vector();
				retryTreatment = false;
	            // Add the sub repository in Map of Update Site
	            mapContentRepositoryPluginFeatureUS = initializeMapFTP(wizard.getClientFTP());
	            
				// Test the XML structure and the repository structure
	            CheckXMLValidity.startTest(this.wizard, tempDir, mapContentRepositoryPluginFeatureUS);
	            
				if (!IOFTPSSHErrorVector.isEmpty())
		        {
		            int button;
		            Status[] status = new Status[IOFTPSSHErrorVector.size()];
		            for (int i=0; i<IOFTPSSHErrorVector.size(); i++)
		            {
		                status[i] = new Status(IStatus.ERROR, "org.topcased.service.updatesite", 0, (String)IOFTPSSHErrorVector.get(i), null);
		            }
		            ErrorDialog errorBox = new ErrorDialog(wizard.getShell(), "IO Error", "An error occured during the copy", 
		                    new MultiStatus("org.topcased.service.updatesite", 0, status,"See details", null), IStatus.ERROR);
		            errorBox.open();
		            
		            MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_WARNING | SWT.IGNORE | SWT.RETRY | SWT.ABORT);
		            msgBox.setText("IO Error");
		            msgBox.setMessage("If you continue, maybe some files will be not copied.\n\nDo you want to continue ?");
		            button = msgBox.open();
		            switch (button)
		            {
		                case (SWT.ABORT):
                            IRunnableWithProgress runnableWithProgress2 = new IRunnableWithProgress()
                            {
                                public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
                                {
                                    monitor.beginTask("Deleting files...", IProgressMonitor.UNKNOWN);
                                    File suppFile = new File(tempDir + "features");
                                    FilesTreatment.emptyFile(suppFile);
                                    suppFile = new File(tempDir + "plugins");
                                    FilesTreatment.emptyFile(suppFile);
                                    suppFile = new File(tempDir + "site.xml");
                                    FilesTreatment.emptyFile(suppFile);
                                }
                            };
                            ProgressMonitorDialog dialog2 = new ProgressMonitorDialog(wizard.getShell());
                            try
                            {
                                dialog2.run(true, true, runnableWithProgress2);
                            }
                            catch (InvocationTargetException e)
                            {
                            	CorePlugin.log(e);
                            }
                            catch (InterruptedException e)
                            {
                            	CorePlugin.log(e);
                            }
							table.removeAll();
							text.setText("");
							break;
		                case (SWT.RETRY):
							// Retry to add the sub repository in Map of Update Site
				            retryTreatment = true;
		                    break;
		                case (SWT.IGNORE):
							break;
		            }
		        }
			} while (retryTreatment);
			
			text.setText(wizard.getUpdateSitePath());
			table.removeAll();
            displayMap(this.mapContentRepositoryPluginFeatureUS, table);// this.mapDeletePluginFeatureUS);

            wizard.setUpdateSitePath(tempDir); 
        }
    }

    /**
     * The goal of this method is to check the presence of the initial structure, i.e : 
     * - directory 'features' must existed but can be empty
     * - directory 'plugins' must existed but can be empty
     * If nothing exists, the method creates a new update site with the two empty directories.
     * In case where a conflict is detected, the method ask the user the operation to perform and try to solve it.
     * 
     * @param originalPath of the repository
     * @return true if all tests are ok, false if the file site.xml is missing.
     */
    public boolean checkInitialRepository(String originalPath)
    {
		boolean findFeat = false;
		boolean findPlug = false;
		
		try
		{
			wizard.getClientSFTP().cd(originalPath);
			try
	        {
				wizard.getClientSFTP().cd("features");
				findFeat = true;
				wizard.getClientSFTP().cd("..");
	        }
			catch (IOException e)
	        {
	        	CorePlugin.log(this.getClass(), "Directory " + e.getMessage() + ". It will be automatically created.", IStatus.INFO);
	        }
			
	        try
			{
				wizard.getClientSFTP().cd("plugins");
				findPlug = true;
				wizard.getClientSFTP().cd("..");
			}
	        catch (IOException e)
	        {
	        	CorePlugin.log(this.getClass(), "Directory " + e.getMessage() + ". It will be automatically created.", IStatus.INFO);
	        }
			
	        if (!findPlug || !findFeat)
			{
	            MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
	            msgBox.setText("Missing directories");
	            msgBox.setMessage("The repertories necessary to the installation are missing\nDo you want to create them ?");
	            int button = msgBox.open();
	            if (button == SWT.YES)
	            {
					try
			        {
				        // Create features and plugins directories
						wizard.getClientSFTP().cd(originalPath);
			            wizard.getClientSFTP().mkdir("features");
			            wizard.getClientSFTP().mkdir("plugins");
			            return true;
			        }
			        catch (IOException e)
			        {
						msgBox = new MessageBox(wizard.getShell(), SWT.ICON_ERROR | SWT.OK);
			            msgBox.setText("Error");
			            msgBox.setMessage("The directories features and/or plugins can't be created\nCheck your permissions");
			            msgBox.open();
			        }
	            }
			}	
	        return true;
		}
		catch (IOException e)
        {
        	//CorePlugin.log(this.getClass(), "Update Site " + e.getMessage(), IStatus.ERROR);
            MessageBox infoBox = new MessageBox(wizard.getShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
            infoBox.setText("New Update Site");
            infoBox.setMessage("Would you like to create a new Update Site repository in " + originalPath);
            int button = infoBox.open();
            if (button == SWT.YES)
            {
				try
		        {
					wizard.getClientSFTP().mkdirs(originalPath);
					wizard.getClientSFTP().cd(originalPath);
		            wizard.getClientSFTP().mkdir("features");
		            wizard.getClientSFTP().mkdir("plugins");
		            CorePlugin.log(this.getClass(), "New Update Site created in " + originalPath , IStatus.OK);
					return true;
				}
				catch(IOException ioe)
				{
					MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_ERROR | SWT.OK);
		            msgBox.setText("Error");
		            msgBox.setMessage("The directory "+ originalPath +" can't be created\nCheck your permissions");
		            msgBox.open();				
				}
            }
		}
        return false;
    }
    
    /**
     * Button Action : Browse the Update Site from a SSH server
     * 
     * @param text : location where the path will be read and write
     * @param table : location where the content of the repository appear
     */
    public void browseUpdateSiteSSH(Text text, Table table)
    {

    	String path = UpdateSiteWizard.checkPath(text.getText(), wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC);
        // Get files from remote server
        final String tempDir = UpdateSiteWizard.checkPath(System.getProperty("java.io.tmpdir"), true);

        if (!checkInitialRepository(path))
        {
        	// certain components of the browse page are disabled
        	wizard.setUpdateSitePath("");
        	text.setText("");
        	wizard.getSelectionPage().getAddButton().setEnabled(false);
        	wizard.getSelectionPage().getDeleteButton().setEnabled(false);
        }
        else
        {
	        File testFileFeat = new File(tempDir + "features" + File.separator);
	        File testFilePlug = new File(tempDir + "plugins" + File.separator);
	        File testFileSite = new File(tempDir + "site.xml");
	        testFileSite.delete();
	        FilesTreatment.emptyFile(testFileFeat);
	        FilesTreatment.emptyFile(testFilePlug);
	        testFileFeat.mkdir();
	        testFileFeat.deleteOnExit();
	        testFilePlug.mkdir();
	        testFilePlug.deleteOnExit();
	
	        wizard.setUpdateSitePath(path);
			
			// Add the sub repository in Map of Update Site
			IRunnableWithProgress runnableWithProgress = new IRunnableWithProgress()
	        {
	            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
	            {
					monitor.beginTask("Receiving files...", IProgressMonitor.UNKNOWN);
					mapContentRepositoryPluginFeatureUS = initializeMapSSH(wizard.getClientSFTP());
	            }
	        };
			
			boolean retryTreatment;
			
			do
			{
				IOFTPSSHErrorVector = new Vector();
				retryTreatment =false;
		        ProgressMonitorDialog dialog = new ProgressMonitorDialog(wizard.getShell());
		        try
		        {
		            dialog.run(true, true, runnableWithProgress);
		        }
		        catch (InvocationTargetException e)
		        {
		        	CorePlugin.log(e);
		        }
		        catch (InterruptedException e)
		        {
		        	CorePlugin.log(e);
		        }
				
				if (!IOFTPSSHErrorVector.isEmpty())
		        {
		            int button;
		            Status[] status = new Status[IOFTPSSHErrorVector.size()];
		            for (int i=0; i<IOFTPSSHErrorVector.size(); i++)
		            {
		                status[i] = new Status(IStatus.ERROR, "org.topcased.service.updatesite", 0, (String)IOFTPSSHErrorVector.get(i), null);
		            }
		            ErrorDialog errorBox = new ErrorDialog(wizard.getShell(), "IO Error", "An error occured during the copy", 
		                    new MultiStatus("org.topcased.service.updatesite", 0, status,"See details", null), IStatus.ERROR);
		            errorBox.open();
		            
		            MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_WARNING | SWT.IGNORE | SWT.RETRY | SWT.ABORT);
		            msgBox.setText("IO Error");
		            msgBox.setMessage("If you continue, maybe some files will be not copied.\n\nDo you want to continue ?");
		            button = msgBox.open();
		            switch (button)
		            {
		                case (SWT.ABORT):
	                        IRunnableWithProgress runnableWithProgress2 = new IRunnableWithProgress()
	                        {
	                            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
	                            {
	                                monitor.beginTask("Deleting files...", IProgressMonitor.UNKNOWN);
	                                File suppFile = new File(tempDir + "features");
	                                FilesTreatment.emptyFile(suppFile);
	                                suppFile = new File(tempDir + "plugins");
	                                FilesTreatment.emptyFile(suppFile);
	                                suppFile = new File(tempDir + "site.xml");
	                                FilesTreatment.emptyFile(suppFile);
	                            }
	                        };
	                        ProgressMonitorDialog dialog2 = new ProgressMonitorDialog(wizard.getShell());
	                        try
	                        {
	                            dialog2.run(true, true, runnableWithProgress2);
	                        }
	                        catch (InvocationTargetException e)
	                        {
	                        	CorePlugin.log(e);
	                        }
	                        catch (InterruptedException e)
	                        {
	                        	CorePlugin.log(e);
	                        }
	                        table.removeAll();
	                        text.setText("");
							break;
		                case (SWT.RETRY):
							// Retry to add the sub repository in Map of Update Site
							retryTreatment = true;
		                    break;
		                case (SWT.IGNORE):
							break;
		            }
		        }
			} while (retryTreatment);
	        
			// Test the XML structure and the repository structure
	        CheckXMLValidity.startTest(this.wizard, tempDir, this.mapContentRepositoryPluginFeatureUS);
	        
	        text.setText(wizard.getUpdateSitePath());
	        table.removeAll();
	        displayMap(this.mapContentRepositoryPluginFeatureUS, table); //, this.mapDeletePluginFeatureUS);
	
	        wizard.setUpdateSitePath(tempDir);
        }
    }

    /**
     * Button Action : Add the repository seleceted in the tableSource (Tool
     * administrator) to the tableDestination (Update Site)
     * 
     * @param tableSource : Tool Administrator Table
     * @param tableTarget : Update Site Table
     */

    public void addPluginFeature(String nameSelected, Table tableTarget)
    {
    	String key = nameSelected;
        if (mapContentRepositoryPluginFeatureTA.containsKey(key))
        {
        	if (mapContentRepositoryPluginFeatureTA.get(key) instanceof SourceDestination)
            {
               SourceDestination repositorySD = (SourceDestination) this.mapContentRepositoryPluginFeatureTA.get(key);
                        
               if (!mapContentRepositoryPluginFeatureUS.containsKey(key)
               		&& !mapContentRepositoryPluginFeatureUS.containsKey(key + ".jar")
                    && !wizard.getMapAddPluginFeatureUS().containsKey(key))
               {
                   // Add the repository selected in the Map 
            	   SourceDestination addSourceDestination = new SourceDestination(repositorySD.getName(), repositorySD.getSource(), UpdateSiteWizard.checkPath(wizard.getUpdateSitePath() + "features",true));
                   wizard.getMapAddPluginFeatureUS().put(nameSelected, addSourceDestination);
                   mapContentRepositoryPluginFeatureUS.put(nameSelected, addSourceDestination);
                }
            }
        }
        // Display
        tableTarget.removeAll();
        displayMap(mapContentRepositoryPluginFeatureUS, tableTarget);
    }


    /**
     * Button Action : Delete the repository seleceted in the tableSource
     * (UpdateSite)
     * 
     * @param tableSource : Update Site Table
     */
    public void deletePluginFeature(String nameSelected, Table tableUS)
    {
       	if (wizard.getMapAddPluginFeatureUS().containsKey(nameSelected))
    	{
       		tableUS.removeAll();
       		wizard.getMapAddPluginFeatureUS().remove(nameSelected);
       		mapContentRepositoryPluginFeatureUS.remove(nameSelected);
       		displayMap(mapContentRepositoryPluginFeatureUS, tableUS);
    	}
    }

    /**
     * Search the path source of the selected item in different map (updateSite,
     * add, delete)
     * 
     * @param nameSelected
     * @param searchTA
     * @return absolute path
     */
    public String searchSource(String nameSelected, boolean searchTA)
    {
        String path = "";
        // Update Site Map
        if (mapContentRepositoryPluginFeatureUS.containsKey(nameSelected))
        {
            if (mapContentRepositoryPluginFeatureUS.get(nameSelected) instanceof SourceDestination)
            {
                SourceDestination sd = (SourceDestination) mapContentRepositoryPluginFeatureUS.get(nameSelected);
                path = sd.getSource();
            }
        }
        else
        if (searchTA)
        {
            if (mapContentRepositoryPluginFeatureTA.containsKey(nameSelected))
            {
                if (mapContentRepositoryPluginFeatureTA.get(nameSelected) instanceof SourceDestination)
                {
                    SourceDestination sd = (SourceDestination) mapContentRepositoryPluginFeatureTA.get(nameSelected);
                    path = sd.getSource();
                }
            }
        }
        return path;        	
    }

    /**
     * Display informations about feature
     * 
     * @param featurePath
     */
    public void infosFeature(String featurePath)
    {
    	PopupDialog newPopupDialog = new PopupDialog(wizard.getShell(), featurePath);
        newPopupDialog.setBlockOnOpen(true);
        newPopupDialog.open();
    }

    /**
     * Filter the map of the TA path's files Choose only the feature not
     * included in other features
     * 
     * @param table
     */
    public void filterMap(Table table)
    {
        ProgressMonitorDialog dialog;
        TableItem item;
        final TableItem[] items = table.getItems();
        final String name[] = new String[items.length];
        for (int indice = 0; indice < items.length; indice++)
        {
            item = items[indice];
            name[indice] = item.getText();
        }

        IRunnableWithProgress runnableWithProgress = new IRunnableWithProgress()
        {
            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
            {
                monitor.setCanceled(false);

                monitor.beginTask("Filtering of features in progress...", 10);
                String USToolAdminPath = UpdateSiteWizard.checkPath(wizard.getUpdateSiteToolAdminPath() + "features", true);
                mapFilter = initializeMap(USToolAdminPath);
                mapFilter.putAll(initializeMapWithJarFile(USToolAdminPath));
                for (int indice = 0; indice < items.length; indice++)
                {
                    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

                    String featurePath = USToolAdminPath + name[indice];
                    File pathFile;
                    File fileToParse = null;
                    File propertyFile = null;
                    boolean isJarFile = false;
                    
                    if (featurePath.endsWith(".jar"))
                    {
                        isJarFile = true;
                    }
                    pathFile = new File(featurePath);
                    if (isJarFile)
                    {
                        try
                        {
                            fileToParse = FilesTreatment.createTempFileFromJar(pathFile, "feature.xml", FilesTreatment.FEATURE_XML);
                            // Property path
                            propertyFile = FilesTreatment.createTempFileFromJar(pathFile, "feature.properties", FilesTreatment.FEATURE_PROPERTIES);
                        }
                        catch (IOException e1)
                        {
                            CorePlugin.log(this.getClass(), "IOException : " + e1.getMessage(), IStatus.ERROR);
                        }
                    }
                    else
                    {
                        featurePath = UpdateSiteWizard.checkPath(featurePath, true);
                    	fileToParse = new File(featurePath + "feature.xml");
                        // Property path
                        propertyFile = new File(featurePath + "feature.properties");
                    }

                    factory.setIgnoringComments(true);
                    factory.setCoalescing(true);

                    // Parser
                    try
                    {
                        DocumentBuilder parser = factory.newDocumentBuilder();
                        // Document
                        Document document = parser.parse(fileToParse);

                        NodeList featureList = document.getElementsByTagName(INCLUDES);
                        int nbNode = featureList.getLength();
                        for (int i = 0; i < nbNode; i++)
                        {
                            Element node = (Element) featureList.item(i);
                            String id = node.getAttribute(ID);
                            String version = node.getAttribute(VERSION);
                            String featureName = FilesTreatment.analyzeString(id, propertyFile);
                            if (version != null)
                            {
                                featureName = featureName + "_" + FilesTreatment.analyzeString(version, propertyFile);
                            }

                            mapFilter.remove(featureName);
                            mapFilter.remove(featureName + ".jar");

                            monitor.worked(1);
                        }
                    }
                    catch (ParserConfigurationException e)
                    {
                        CorePlugin.log(this.getClass(), "ParserConfiguration error : " + e.getMessage(), IStatus.ERROR);
                    }
                    catch (SAXException e)
                    {
                        CorePlugin.log(this.getClass(), "Error while parsing document : " + e.getMessage(),
                                IStatus.ERROR);
                    }
                    catch (IOException e)
                    {
                        CorePlugin.log(this.getClass(), "Error while opening file : " + e.getMessage()
                                + ". Maybe is not a feature.", IStatus.ERROR);
                    }
                    
                    // Deletes temp files
                    if (isJarFile && fileToParse.exists()) fileToParse.delete();
                    if (isJarFile && propertyFile.exists()) propertyFile.delete();
                }
                monitor.done();
            }
        };
        dialog = new ProgressMonitorDialog(wizard.getShell());
        try
        {
            if (items.length > 0)
            {
                dialog.run(true, true, runnableWithProgress);
                // Initialization
                table.removeAll();
                displayMap(mapFilter, table);
            }
        }
        catch (InvocationTargetException e)
        {
        	CorePlugin.log(e);
        }
        catch (InterruptedException e)
        {
        	CorePlugin.log(e);
        }
    }

    /**
     * Initalize the TA map with all files without filter and display it in the
     * table
     * 
     * @param table
     */
    public void noFilterMap(Table table)
    {
    	String USToolAdminPath = UpdateSiteWizard.checkPath(wizard.getUpdateSiteToolAdminPath() + "features", true);
    	mapFilter = initializeMap(USToolAdminPath);
        mapFilter.putAll(initializeMapWithJarFile(USToolAdminPath));

        // Initialization
        table.removeAll();
        displayMap(mapFilter, table);
    }
    
    /**
     * Initialize a Map from a SSH server
     * 
     * @param clientSFTP the client SFTP of the SSH server
     * @return the Map containing the feature of the Update Site path
     *         "/features"
     */
    private Map initializeMapSSH(SftpClient clientSFTP)
    {
        final Map mapReturn = new TreeMap();
        String tempDir = UpdateSiteWizard.checkPath(System.getProperty("java.io.tmpdir"), true);
        String path = UpdateSiteWizard.checkPath(wizard.getUpdateSitePath() + "features", wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC);

        List files = null;
        try
        {
            files = clientSFTP.ls(path);
        }
        catch (IOException e)
        {

        }

        for (int i = 0; i < files.size(); i++)
        {
            CorePlugin.log(((SftpFile) files.get(i)).getFilename(), IStatus.INFO);
            if (!((SftpFile) files.get(i)).getFilename().equals("..")
                    && !((SftpFile) files.get(i)).getFilename().equals("."))
            {
                String nameSubRepository = ((SftpFile) files.get(i)).getFilename();
                String pathSubRepository = ((SftpFile) files.get(i)).getAbsolutePath();
                String pathDestination = UpdateSiteWizard.checkPath(tempDir + "features", true);

                // Add in the mapReturn
                SourceDestination sourceSubRepository = new SourceDestination(nameSubRepository, pathSubRepository, pathDestination);
                mapReturn.put(nameSubRepository, sourceSubRepository);
            }
        }

        path = UpdateSiteWizard.checkPath(wizard.getUpdateSitePath() + "plugins", wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC);
        files = null;
        try
        {
            files = clientSFTP.ls(path);
        }
        catch (IOException e)
        {

        }

		try
        {
			clientSFTP.get(wizard.getUpdateSitePath() + "site.xml", tempDir + "site.xml");
        }
		catch (IOException e)
        {
        	CorePlugin.log(this.getClass(), "Error while copying site.xml from Update Site: " + e.getMessage(), IStatus.INFO);
        }

        return mapReturn;
    }

    /**
     * Initialize a Map from a FTP server
     * 
     * @param clientFTP the FTP client
     * @return the Map containing the feature of the Update Site path
     *         "/features"
     */
    private Map initializeMapFTP(final FTPClient clientFTP)
    {
		final Map mapReturn = new TreeMap();
        IRunnableWithProgress runnableWithProgress = new IRunnableWithProgress()
        {
            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
            {
                error = "";
                try
                {
                    // Features
                    String tempDir = UpdateSiteWizard.checkPath(System.getProperty("java.io.tmpdir"), true);
                    //String pathDest = UpdateSiteWizard.checkPath(tempDir + "features", true);
                    String path = UpdateSiteWizard.checkPath(wizard.getUpdateSitePath() + "features", wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC);

                    IDirectoryEntry[] files = clientFTP.listFiles(path, false, monitor);
                    if (files.length == 0)
                    {
                        // end system is different from the source system
                        if (File.separator.equals("\\"))
                        {
                            path = path.replace('\\', '/');
                            files = clientFTP.listFiles(path, false, monitor);
                        }
                        else
                        {
                            path = path.replace('/', '\\');
                            files = clientFTP.listFiles(path, false, monitor);
                        }
                    }

                    CorePlugin.log(BrowseUpdateSiteWizardTreatment.this.getClass(),path + " -> " + files.length + " files found", IStatus.INFO);

                    for (int i = 0; i < files.length; i++)
                    {
                        
                        String nameSubRepository = files[i].getName();
                        String pathSubRepository = path + files[i].getName();
                        // Add in the mapReturn
                        SourceDestination sourceSubRepository = new SourceDestination(nameSubRepository, pathSubRepository, path);
                        mapReturn.put(nameSubRepository, sourceSubRepository);
                    }

                    // Plugins
                    path = UpdateSiteWizard.checkPath(wizard.getUpdateSitePath() + "plugins", + wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC);
                    //String pathDest = UpdateSiteWizard.checkPath(tempDir + "plugins", true) ;

                    files = clientFTP.listFiles(path, false, monitor);
                    if (files.length == 0)
                    {
                        // end system is different from the source system
                        if (File.separator.equals("\\"))
                        {
                            files = clientFTP.listFiles(path = path.replace('\\', '/'), false, monitor);
                        }
                        else
                        {
                            files = clientFTP.listFiles(path = path.replace('/', '\\'), false, monitor);
                        }
                    }

                    CorePlugin.log(BrowseUpdateSiteWizardTreatment.this.getClass(),path + " -> " + files.length + " Files received", IStatus.INFO);

                    // Site.xml
                    path = wizard.getUpdateSitePath();

                    files = clientFTP.listFiles(path, false, monitor);
                    if (files.length == 0)
                    {
                        // end system is different from the source system
                        if (File.separator.equals("\\"))
                        {
                            files = clientFTP.listFiles(path = path.replace('\\', '/'), false, monitor);
                        }
                        else
                        {
                            files = clientFTP.listFiles(path = path.replace('/', '\\'), false, monitor);
                        }
                    }

                    for (int i = 0; i < files.length; i++)
                    {
                        if (files[i].getName().equals("site.xml"))
                        {
                            try
							{
                            	FilesTreatment.copyPluginFeatureFromFTP(path, tempDir, files[i], clientFTP, monitor);
							}
							catch (IOException e)
							{
								CorePlugin.log(e);
								IOFTPSSHErrorVector.add(e.getMessage());
							}
                        }
                    }
                }
                catch (FtpException e)
                {
                	CorePlugin.log(e);
                    error = e.getMessage();
					IOFTPSSHErrorVector.add(e.getMessage());
                }
            }
        };
        ProgressMonitorDialog dialog = new ProgressMonitorDialog(wizard.getShell());
        try
        {
            dialog.run(true, true, runnableWithProgress);
        }
        catch (InvocationTargetException e)
        {
        	CorePlugin.log(e);
            error = e.getMessage();
        }
        catch (InterruptedException e)
        {
        	CorePlugin.log(e);
            error = e.getMessage();
        }
        if (!error.equals(""))
        {
            MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_ERROR | SWT.OK);
            msgBox.setText("Connection error");
            msgBox.setMessage("Error in directory browsing : " + error + "\nSee Console informations");
            msgBox.open();
        }

        return mapReturn;
    }

    /**
     * Return the corresponding Map from the path repository or file
     * 
     * @param subRepository : path repository or file
     * @return map (key=name, value=SourceDestination)
     */
    private Map pathToMap(File subRepository)
    {
        Map mapReturn = new TreeMap();
        try
        {
            String nameSubRepository = subRepository.getName();
            String pathSubRepository = subRepository.getCanonicalPath();
            String pathDestination = subRepository.getParent();
            // Add in the mapReturn
            SourceDestination sourceSubRepository = new SourceDestination(nameSubRepository, pathSubRepository, pathDestination);
            mapReturn.put(nameSubRepository, sourceSubRepository);
        }
        catch (IOException e)
        {
            CorePlugin.log(this.getClass(),
                    "Problem to recover the canonic path of the selected repository in the browser.", IStatus.ERROR);
        }
        return mapReturn;
    }
}