/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.updatesite;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

/**
 * This WizardPage allows the user to enter personal information in order to be connected to a remote or local UpdateSite.<br>
 * The personal information is stored inside a hidden file based in the workspace. All kind of information is kept except confidential information such as user password.<b>
 * This enables to be connected faster the next times.
 */
public class ConnectUpdateSiteWizardPage extends WizardPage implements KeyListener
{
	private UpdateSiteWizard refWizard;

	private Text loginTxt;

	private Label loginLab;

	private Label passLab;

	private Text passTxt;

	private Label addrLab;

	private Text addrTxt;

	private Combo protocolCob;

	private Text portTxt;

	private Label portLab;

	private Combo connectCob;

	private Label usPathLabel;

	private Text usPathText;

	private ConnectUpdateSiteWizardTreatment treatment;

	/**
	 * Constructor
	 * 
	 * @param wizard the parent wizard
	 * @param pageName the name of the page
	 */
	public ConnectUpdateSiteWizardPage(UpdateSiteWizard wizard, String pageName)
	{
		super(pageName);
		refWizard = wizard;
		treatment = new ConnectUpdateSiteWizardTreatment(wizard, this);
	}

	public ConnectUpdateSiteWizardTreatment getTreatment()
	{
		return treatment;
	}

	/**
	 * @return Returns the loginTxt.
	 */
	public Text getLoginTxt()
	{
		return loginTxt;
	}

	/**
	 * @return Returns the addrTxt.
	 */
	public Text getAddrTxt()
	{
		return addrTxt;
	}
	
	/**
	 * @return Returns the portTxt.
	 */
	public Text getPortTxt()
	{
		return portTxt;
	}

	/**
	 * @return Returns the protocolCob.
	 */
	public Combo getProtocolCob()
	{
		return protocolCob;
	}

	/**
	 * @return the connection's protocol used
	 */
	public int getProtocol()
	{
		return protocolCob.getSelectionIndex();
	}
	
	/**
	 * @return Returns the connectCob.
	 */
	public Combo getConnectCob()
	{
		return connectCob;
	}

	/**
	 * Getter on the Update Site path
	 * 
	 * @return Returns the US path.
	 */
	public Text getUSPath()
	{
		return usPathText;
	}

	/**
	 * According to the connection kind required by the user, the different fields are enabled or disabled.
	 */
	private void checkConnectionType()
	{
		if (protocolCob.getSelectionIndex() == UpdateSiteWizard.FILENOSEC)
		{
			addrLab.setEnabled(false);
			addrTxt.setEnabled(false);
			portLab.setEnabled(false);
			portTxt.setEnabled(false);
			portTxt.setText("");
			loginLab.setEnabled(false);
			loginTxt.setEnabled(false);
			passLab.setEnabled(false);
			passTxt.setEnabled(false);
			usPathText.setEnabled(false);
			usPathLabel.setEnabled(false);
			setPageComplete(true);
		}
		else
		{
			addrLab.setEnabled(true);
			addrTxt.setEnabled(true);
			portLab.setEnabled(true);
			portTxt.setEnabled(true);
			loginLab.setEnabled(true);
			loginTxt.setEnabled(true);
			passLab.setEnabled(true);
			passTxt.setEnabled(true);
			usPathText.setEnabled(true);
			usPathLabel.setEnabled(true);
			setPageComplete(isPageComplete());
		}
	}

	/**
	 * Update the informations of the connection
	 */
	public void updateConnectionInformations()
	{
		refWizard.setProtocolConnex(protocolCob.getSelectionIndex());
		refWizard.setHost(addrTxt.getText());
		refWizard.setPort(Integer.parseInt(portTxt.getText()));
		refWizard.setUsername(loginTxt.getText());
		refWizard.setPassword(passTxt.getText());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent)
	{
		final Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new GridLayout(1, true));
		setControl(container);

		/*
		 * Main Label
		 */
		final Label label2 = new Label(container, SWT.NONE);
		label2.setText("Complete the following fields :");

		final Composite containerLogPas = new Composite(container, SWT.NONE);
		containerLogPas.setLayout(new GridLayout(2, false));

		/*
		 * Connection profile Label
		 */
		final Label connectLab = new Label(containerLogPas, SWT.NONE);
		GridData gridData1 = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
		connectLab.setLayoutData(gridData1);
		connectLab.setText("Connection profile : ");

		/*
		 * Connection profile Combobox
		 */
		connectCob = new Combo(containerLogPas, SWT.DROP_DOWN | SWT.READ_ONLY | SWT.SIMPLE);
		connectCob.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				if (connectCob.getSelectionIndex() != 0)
				{
					treatment.updateFields(connectCob.getSelectionIndex() - 1);
					checkConnectionType();
				}
			}
		});

		treatment.setProfileConnection();

		/*
		 * Protocol Label
		 */
		final Label protocolLab = new Label(containerLogPas, SWT.NONE);
		GridData gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
		protocolLab.setLayoutData(gridData);
		protocolLab.setText("Protocol : ");

		/*
		 * Protocol Combobox
		 */
		protocolCob = new Combo(containerLogPas, SWT.DROP_DOWN | SWT.READ_ONLY | SWT.SIMPLE);
		protocolCob.add("FTP");
		protocolCob.add("SSH");
		protocolCob.add("File");
		protocolCob.select(0);
		protocolCob.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				refWizard.getConnectionPage().setMessage("Enter the required information to activate the connection.", WizardPage.NONE);
				if (protocolCob.getSelectionIndex() == UpdateSiteWizard.SSH)
				{
					portTxt.setText("22");
				}
				else
				{
					if (protocolCob.getSelectionIndex() == UpdateSiteWizard.FTP)
					{
						portTxt.setText("21");
					}
				}
				checkConnectionType();
			}
		});

		/*
		 * Address label
		 */
		addrLab = new Label(containerLogPas, SWT.NONE);
		gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
		addrLab.setLayoutData(gridData);
		addrLab.setText("Address : ");

		/*
		 * Address Text
		 */
		addrTxt = new Text(containerLogPas, SWT.BORDER);
		gridData = new GridData(SWT.LEFT, SWT.CENTER, true, true);
		gridData.widthHint = 130;
		addrTxt.setLayoutData(gridData);
		addrTxt.addKeyListener(this);

		/*
		 * Port label
		 */
		portLab = new Label(containerLogPas, SWT.NONE);
		gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
		portLab.setLayoutData(gridData);
		portLab.setText("Port : ");

		/*
		 * Port Text
		 */
		portTxt = new Text(containerLogPas, SWT.BORDER);
		gridData = new GridData(SWT.LEFT, SWT.CENTER, true, true);
		gridData.widthHint = 130;
		portTxt.setLayoutData(gridData);
		portTxt.setText("21");
		portTxt.setTextLimit(4); // Limit for Port is set to 4
		portTxt.addKeyListener(new KeyAdapter()
		{
			public void keyReleased(KeyEvent e)
			{
				try
				{
					Integer.parseInt(portTxt.getText());
					refWizard.getConnectionPage().setMessage("Enter the required information to activate the connection.", WizardPage.NONE);
				}
				catch (NumberFormatException eN)
				{
					refWizard.getConnectionPage().setMessage("Invalid Port", WizardPage.ERROR);
				}
				setPageComplete(isPageComplete());
			}
		});

		/*
		 * Login Label
		 */
		loginLab = new Label(containerLogPas, SWT.NONE);
		gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
		loginLab.setLayoutData(gridData);
		loginLab.setText("Login : ");

		/*
		 * Login Text
		 */
		loginTxt = new Text(containerLogPas, SWT.BORDER);
		gridData = new GridData(SWT.LEFT, SWT.CENTER, true, true);
		gridData.widthHint = 130;
		loginTxt.setLayoutData(gridData);
		loginTxt.addKeyListener(this);

		/*
		 * Password Label
		 */
		passLab = new Label(containerLogPas, SWT.NONE);
		gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
		passLab.setLayoutData(gridData);
		passLab.setText("Password : ");

		/*
		 * Password Text (type PASSWORD to have some '*' instead of characters, * nice...)
		 */
		passTxt = new Text(containerLogPas, SWT.BORDER | SWT.PASSWORD);
		gridData = new GridData(SWT.LEFT, SWT.CENTER, true, true);
		gridData.widthHint = 130;
		passTxt.setLayoutData(gridData);
		passTxt.addKeyListener(this);

		usPathLabel = new Label(containerLogPas, SWT.NONE);
		gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
		usPathLabel.setLayoutData(gridData);
		usPathLabel.setText("Path : ");

		usPathText = new Text(containerLogPas, SWT.BORDER);
		gridData = new GridData(SWT.LEFT, SWT.CENTER, true, true);
		gridData.widthHint = 130;
		usPathText.setLayoutData(gridData);
		usPathText.addKeyListener(this);

		setPageComplete(false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.WizardPage#isPageComplete()
	 */
	public boolean isPageComplete()
	{
		if (protocolCob.getSelectionIndex() == UpdateSiteWizard.FILENOSEC)
		{
			return true;
		}
		else
		{
			return !(addrTxt.getText().equals("") || portTxt.getText().equals("") || getMessageType() == WizardPage.ERROR
					|| loginTxt.getText().equals("") || passTxt.getText().equals("") || usPathText.getText().equals(""));
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.WizardPage#canFlipToNextPage()
	 */
	public boolean canFlipToNextPage()
	{
		refWizard.disconnect();
		return isPageComplete();
	}
    
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.WizardPage#getNextPage()
	 */
	public IWizardPage getNextPage()
    {
    	BrowseUpdateSiteWizardPage page = (BrowseUpdateSiteWizardPage) super.getNextPage();
    	if(page != null)
    	{
    		page.update();
    	}
    	return page;
    }

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.KeyAdapter#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent e)
	{
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.events.KeyListener#keyReleased(org.eclipse.swt.events.KeyEvent)
	 */
	public void keyReleased(KeyEvent e)
	{
		setPageComplete(isPageComplete());
	}
}
