/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.updatesite;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.Section;

/**
 * 
 * This WizardPage is used to create 'site.xml' file
 * So the update site features can be associated to a category
 * Some categories can be created
 * The different information about the update site are :
 *  - url
 *  - description
 *  - path associated to an url
 */
public class CreateSiteUpdateSiteWizardPage extends WizardPage
{
    private UpdateSiteWizard wizard;

    private CreateSiteUpdateSiteWizardTreatment treatment;

    private Tree tree;

    private Label featURLInput;

    private Button featPatch;

    private Text featOSInput;

    private Text featWinInput;

    private Text featLangInput;

    private Text featArchiInput;

    private Text catDescInput;

    private Text catLabelInput;

    private Text catNameInput;

    private TreeItem treeItemSelected;

    private Text urlText;

    private Text descriptionText;

    private Table tableArchive;
    
    private Section pathGroupSection;
    
    private Section featureGroupSection;
    
    private Section categorySection;

    private static final int TEXT_LENGTH = 50;
    
    private static final int MAXI_TEXT_LENGTH = 200;

    /**
     * Constructor
     * 
     * @param wizard parent wizard
     * @param pageName the name of the page
     */
    public CreateSiteUpdateSiteWizardPage(UpdateSiteWizard wizard, String pageName)
    {
        super(pageName);
        this.wizard = wizard;
        treatment = new CreateSiteUpdateSiteWizardTreatment(wizard);
    }

    /**
     * @return the treatment of this page
     */
    public CreateSiteUpdateSiteWizardTreatment getTreatment()
    {
        return treatment;
    }

    /**
     * Create the controls of the page
     * 
     * @param parent
     */
    public void createControl(Composite parent)
    {
        final TextTransfer textTransfer;
        
        final Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout(2, true));
        setControl(container);

        tree = new Tree(container, SWT.SINGLE | SWT.BORDER);
        final GridData gridDataTree = new GridData(GridData.FILL_BOTH);
        gridDataTree.heightHint = 300;
        
        tree.setLayoutData(gridDataTree);

        tree.addKeyListener(new KeyListener()
        {
            public void keyPressed(KeyEvent e)
            {
                if (e.keyCode == SWT.DEL)
                {
                    if (tree.getSelectionCount() > 0)
                    {
                        if (tree.getSelection()[0].getImage().equals(TreeUpdateSiteWizardTreatment.picCategory))
                        {
                            treatment.subCategory(tree.getSelection()[0]);
                        }
                    }
                }
            }

            public void keyReleased(KeyEvent e)
            {
            }

        });

        tree.addSelectionListener(new SelectionListener()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (tree.getSelection()[0].getImage().equals(TreeUpdateSiteWizardTreatment.picFeature))
                {
                    // Initialize the fields of the feature's properties
                    featURLInput.setText("URL : " + treatment.getURL(tree.getSelection()[0].getText()));
                    featPatch.setSelection(treatment.getPatch(tree.getSelection()[0].getText()));
                    featOSInput.setText(treatment.getOS(tree.getSelection()[0].getText()));
                    featWinInput.setText(treatment.getWS(tree.getSelection()[0].getText()));
                    featLangInput.setText(treatment.getNL(tree.getSelection()[0].getText()));
                    featArchiInput.setText(treatment.getArch(tree.getSelection()[0].getText()));
                }
                else
                {
                    if (tree.getSelection()[0].getImage().equals(TreeUpdateSiteWizardTreatment.picCategory))
                    {
                        // Initialize the fields of the feature's properties
                        catNameInput.setText(treatment.getName(tree.getSelection()[0].getText()));
                        catLabelInput.setText(treatment.getLabel(tree.getSelection()[0].getText()));
                        catDescInput.setText(treatment.getDesc(tree.getSelection()[0].getText()));
                    }
                }
            }

            public void widgetDefaultSelected(SelectionEvent e)
            {
            }
        });
        
        final GridLayout tabContainerLayout = new GridLayout(1, false);
        tabContainerLayout.marginWidth = 0;
        tabContainerLayout.marginHeight = 0;
        
        final Composite tabContainer = new Composite(container, SWT.NONE);
        tabContainer.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        tabContainer.setLayout(tabContainerLayout);
        displayPathGroup(tabContainer);
        displayCatGroup(tabContainer);
        displayFeatureGroup(tabContainer);

        // Create the drag source on the tree
        DragSource ds = new DragSource(tree, DND.DROP_MOVE);
        ds.setTransfer(new Transfer[] {TextTransfer.getInstance()});
        ds.addDragListener(new DragSourceListener()
        {
            public void dragStart(DragSourceEvent event)
            {
                // Only start the drag if the element is not a category
                if (tree.getSelection()[0].getImage().equals(TreeUpdateSiteWizardTreatment.picCategory))
                {
                    event.doit = false;
                }
            }

            public void dragSetData(DragSourceEvent event)
            {
                // Provide the data of the requested type.
                if (TextTransfer.getInstance().isSupportedType(event.dataType))
                {
                    event.data = tree.getSelection()[0].getText();
                }
            }

            public void dragFinished(DragSourceEvent event)
            {
            }
        });

        // Create the drop target on the text field
        DropTarget dt = new DropTarget(tree, DND.DROP_DEFAULT | DND.DROP_MOVE);
        textTransfer = TextTransfer.getInstance();
        dt.setTransfer(new Transfer[] {textTransfer});
        dt.addDropListener(new DropTargetListener()
        {
            public void dragEnter(DropTargetEvent event)
            {
                if (event.detail == DND.DROP_DEFAULT)
                {
                    if ((event.operations & DND.DROP_MOVE) != 0)
                    {
                        event.detail = DND.DROP_MOVE;
                    }
                    else
                    {
                        event.detail = DND.DROP_NONE;
                    }
                }
                for (int i = 0; i < event.dataTypes.length; i++)
                {
                    if (textTransfer.isSupportedType(event.dataTypes[i]))
                    {
                        event.currentDataType = event.dataTypes[i];
                        // files should only be copied
                        if (event.detail != DND.DROP_MOVE)
                        {
                            event.detail = DND.DROP_NONE;
                        }
                        break;
                    }
                }
            }

            public void dragOver(DropTargetEvent event)
            {
                TextTransfer textTransfer = TextTransfer.getInstance();
                String data = (String) textTransfer.nativeToJava(event.currentDataType);
                if (data != null)
                {
                    // CorePlugin.log(this.getClass(), "Data to be dropped is
                    // (Text)"+data, IStatus.INFO);
                }

                event.feedback = DND.FEEDBACK_SELECT | DND.FEEDBACK_SCROLL;

                if ((event.item != null)
                        && !((TreeItem) event.item).getImage().equals(TreeUpdateSiteWizardTreatment.picCategory))
                {
                    event.detail = DND.DROP_NONE;
                }
                else
                {
                    event.detail = DND.DROP_MOVE;
                }
            }

            public void dragOperationChanged(DropTargetEvent event)
            {
                if (event.detail == DND.DROP_DEFAULT)
                {
                    if ((event.detail = (event.operations & DND.DROP_MOVE)) != 0)
                    {
                        event.detail = DND.DROP_MOVE;
                    }
                    else
                    {
                        event.detail = DND.DROP_NONE;
                    }
                }
                // allow text to be moved but files should only be copied
                if (textTransfer.isSupportedType(event.currentDataType))
                {
                    if (event.detail != DND.DROP_MOVE)
                    {
                        event.detail = DND.DROP_NONE;
                    }
                }
            }

            public void dragLeave(DropTargetEvent event)
            {
            }

            public void dropAccept(DropTargetEvent event)
            {
            }

            public void drop(DropTargetEvent event)
            {
                if (event.item != null)
                {
                    TreeItem treeItemFeature = tree.getSelection()[0];
                    String featureText = treeItemFeature.getText();
                    TreeItem treeItemCategory = ((TreeItem) event.item);
                    if (treeItemCategory != null && (treeItemCategory.getParentItem() != null || treeItemCategory.getParent() != null) )
                    {
                        treeItemFeature.dispose();
                        treeItemFeature = new TreeItem(treeItemCategory, SWT.NONE);
                        treeItemFeature.setText(featureText);
                        treeItemFeature.setImage(TreeUpdateSiteWizardTreatment.picFeature);
                        treatment.setCategoryToFeature(featureText, treeItemCategory.getText());
                    }
                }
                else
                {
                    TreeItem treeItemFeature = tree.getSelection()[0];
                    String featureText = treeItemFeature.getText();
                    treeItemFeature.dispose();
                    treeItemFeature = new TreeItem(tree, SWT.NONE);
                    treeItemFeature.setText(featureText);
                    treeItemFeature.setImage(TreeUpdateSiteWizardTreatment.picFeature);
                    treatment.setCategoryToFeature(featureText, "");
                }
            }
        });

    }
	
    /**
     * Indicates wether the process can finish
     * @return <code>true</code> if the container wizard displays this page, <code>false</code> otherwise.
     */
    public boolean finish()
    {
        return isCurrentPage();
    }
    
    /**
     * @see org.eclipse.jface.wizard.IWizardPage#canFlipToNextPage()
     */
    public boolean canFlipToNextPage()
    {
        treatment.readSiteXMLFile();
        treatment.initializeTree(tree);
        treatment.displayIHMSiteXml(urlText, descriptionText, tableArchive);
        return false;
    }

    /**
     * Display the section of categorie
     * 
     * @param parent
     */
    private void displayCatGroup(Composite parent)
    {
        // Data definition
    	final GridData gridDataDesc = new GridData(SWT.FILL, SWT.CENTER, true, true);
        gridDataDesc.heightHint = 150;
        
        final GridData gridDataButton = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
        gridDataButton.horizontalSpan = 2;
        
        /* ********************************************************************
         * Section for new category
         **********************************************************************/
        categorySection = new Section(parent, Section.TWISTIE | Section.DESCRIPTION);
        categorySection.setText("Category Properties");
        categorySection.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
        categorySection.addExpansionListener(new IExpansionListener() 
        {
        	public void expansionStateChanged(ExpansionEvent e)
        	{
                featureGroupSection.setExpanded(false);
                pathGroupSection.setExpanded(false);
        	}
        	public void expansionStateChanging(ExpansionEvent e) {};
        });
        
        final Composite categoryComposite = new Composite(categorySection, SWT.NONE);
        categoryComposite.setLayout(new GridLayout(2, false));
        categoryComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

        final Label catName = new Label(categoryComposite, SWT.NONE);
        catName.setText("Name : ");
        catName.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, true));
        catNameInput = new Text(categoryComposite, SWT.BORDER);
        catNameInput.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
        catNameInput.addFocusListener(new FocusListener()
        {
            public void focusGained(FocusEvent e)
            {
                if (tree.getSelectionCount() > 0)
                {
                    treeItemSelected = tree.getSelection()[0];
                }
                else
                {
                    treeItemSelected = null;
                }
            }

            public void focusLost(FocusEvent e)
            {
                if (treeItemSelected != null)
                {
                	String oldName = treeItemSelected.getText();
                	String newName = catNameInput.getText();
                	if (!treatment.categoryExists(newName) && !newName.equals(""))
                    {
                    	treatment.setName(oldName, newName, treeItemSelected);
                    }
                    else
                    {
                        if (!oldName.equals(newName))
                        {
                            catNameInput.setText(oldName);
                            MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_ERROR | SWT.OK);
                            msgBox.setText("Invalid category name");
                            msgBox.setMessage("The category must have a unique and not empty name");
                            msgBox.open();
                        }
                    }
                }
            }
        });

        final Label catLabel = new Label(categoryComposite, SWT.NONE);
        catLabel.setText("Label : ");
        catLabel.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, true));
        catLabelInput = new Text(categoryComposite, SWT.BORDER);
        catLabelInput.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
        catLabelInput.addFocusListener(new FocusListener()
        {
            public void focusGained(FocusEvent e)
            {
                if (tree.getSelectionCount() > 0)
                {
                    treeItemSelected = tree.getSelection()[0];
                }
                else
                {
                    treeItemSelected = null;
                }
            }

            public void focusLost(FocusEvent e)
            {
                if (treeItemSelected != null)
                {
					if (!catLabelInput.getText().equals(""))
					{
						treatment.setLabel(treeItemSelected.getText(), catLabelInput.getText());
					}
					else
					{
						catLabelInput.setText("label");
						MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_ERROR | SWT.OK);
                        msgBox.setText("Invalid category label");
                        msgBox.setMessage("The label must have a not empty label");
                        msgBox.open();
                    }
                }
            }
        });

        final Label catDesc = new Label(categoryComposite, SWT.NONE);
        catDesc.setText("Description : ");
        catDesc.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, true));
        catDescInput = new Text(categoryComposite, SWT.BORDER | SWT.MULTI | SWT.WRAP);
        catDescInput.setLayoutData(gridDataDesc);
        catDescInput.addFocusListener(new FocusListener()
        {
            public void focusGained(FocusEvent e)
            {
                if (tree.getSelectionCount() > 0)
                {
                    treeItemSelected = tree.getSelection()[0];
                }
                else
                {
                    treeItemSelected = null;
                }
            }

            public void focusLost(FocusEvent e)
            {
                if (treeItemSelected != null)
                {
                    treatment.setDesc(treeItemSelected.getText(), catDescInput.getText());
                }
            }
        });

        final Button button = new Button(categoryComposite, SWT.NONE);
        button.setLayoutData(gridDataButton);
        button.setText("Create Category");
        button.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (catNameInput.getText().length() != 0 && catLabelInput.getText().length() != 0)
                {
                    boolean success = treatment.addCategory(catNameInput.getText(), catLabelInput.getText(),
                            catDescInput.getText(), tree);
                    if (!success)
                    {
                        MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_ERROR | SWT.OK);
                        msgBox.setText("Invalid category name");
                        msgBox.setMessage("The category " + catNameInput.getText() + " still exists");
                        msgBox.open();
                    }
                }
                else
                {
                    MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_ERROR | SWT.OK);
                    msgBox.setText("Invalid category name");
                    msgBox.setMessage("The category must have a name and a label not empty");
                    msgBox.open();
                }
                catNameInput.setText("");
                catLabelInput.setText("");
                catDescInput.setText("");
            }
        });

        categorySection.setClient(categoryComposite);
    }

    /**
     * Display the section of feature
     * 
     * @param parent
     */
    private void displayFeatureGroup(Composite parent)
    {
        // Layout and Data definition
        final GridData urlLabelData = new GridData(SWT.FILL, SWT.NONE, true, false);
        urlLabelData.horizontalSpan = 2;
        
        final GridData buttonData = new GridData(SWT.LEFT, SWT.NONE, true, false);
        buttonData.horizontalSpan = 2;
        
        /* *********************************************************************
         * Section for editing a feature
         **********************************************************************/

        featureGroupSection = new Section(parent, Section.TWISTIE | Section.DESCRIPTION);
        featureGroupSection.setText("Feature Properties \u0026 Environments");
        featureGroupSection.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
        featureGroupSection.addExpansionListener(new IExpansionListener() 
        {
        	public void expansionStateChanged(ExpansionEvent e)
        	{
        		pathGroupSection.setExpanded(false);
                categorySection.setExpanded(false);
        	}
        	public void expansionStateChanging(ExpansionEvent e) {};
        });
        
        final Composite editingFeatureComposite = new Composite(featureGroupSection, SWT.NONE);
        editingFeatureComposite.setLayout(new GridLayout(2, false));
        editingFeatureComposite.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, true));
        
        featURLInput = new Label(editingFeatureComposite, SWT.NONE);
        featURLInput.setText("URL of the feature");
        featURLInput.setLayoutData(urlLabelData);

        featPatch = new Button(editingFeatureComposite, SWT.CHECK);
        featPatch.setText(" is a patch for another feature");
        featPatch.setLayoutData(buttonData);
        featPatch.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (tree.getSelectionCount() > 0)
                {
                	treatment.setPatch(tree.getSelection()[0].getText(), featPatch.getSelection());
                }
            }
        });

        final Label featOS = new Label(editingFeatureComposite, SWT.NONE);
        featOS.setText("Operating Systems : ");
        
        featOSInput = new Text(editingFeatureComposite, SWT.BORDER);
        featOSInput.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        featOSInput.addFocusListener(new FocusListener()
        {
            public void focusGained(FocusEvent e)
            {
                if (tree.getSelectionCount() > 0)
                {
                    treeItemSelected = tree.getSelection()[0];
                }
                else
                {
                    treeItemSelected = null;
                }
            }

            public void focusLost(FocusEvent e)
            {
                if (treeItemSelected != null)
                {
                    treatment.setOS(treeItemSelected.getText(), featOSInput.getText());
                }
            }
        });

        final Label featWin = new Label(editingFeatureComposite, SWT.NONE);
        featWin.setText("Window Systems : ");
        
        featWinInput = new Text(editingFeatureComposite, SWT.BORDER);
        featWinInput.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        featWinInput.addFocusListener(new FocusListener()
        {
            public void focusGained(FocusEvent e)
            {
                if (tree.getSelectionCount() > 0)
                {
                    treeItemSelected = tree.getSelection()[0];
                }
                else
                {
                    treeItemSelected = null;
                }
            }

            public void focusLost(FocusEvent e)
            {
                if (treeItemSelected != null)
                {
                    treatment.setWin(treeItemSelected.getText(), featWinInput.getText());
                }
            }
        });

        final Label featLang = new Label(editingFeatureComposite, SWT.NONE);
        featLang.setText("Languages : ");
        
        featLangInput = new Text(editingFeatureComposite, SWT.BORDER);
        featLangInput.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        featLangInput.addFocusListener(new FocusListener()
        {
            public void focusGained(FocusEvent e)
            {
                if (tree.getSelectionCount() > 0)
                {
                    treeItemSelected = tree.getSelection()[0];
                }
                else
                {
                    treeItemSelected = null;
                }
            }

            public void focusLost(FocusEvent e)
            {
                if (treeItemSelected != null)
                {
                    treatment.setNl(treeItemSelected.getText(), featLangInput.getText());
                }
            }
        });

        final Label featArchi = new Label(editingFeatureComposite, SWT.NONE);
        featArchi.setText("Architectures : ");
        
        featArchiInput = new Text(editingFeatureComposite, SWT.BORDER);
        featArchiInput.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        featArchiInput.addFocusListener(new FocusListener()
        {
            public void focusGained(FocusEvent e)
            {
                if (tree.getSelectionCount() > 0)
                {
                    treeItemSelected = tree.getSelection()[0];
                }
                else
                {
                    treeItemSelected = null;
                }
            }

            public void focusLost(FocusEvent e)
            {
                if (treeItemSelected != null)
                {
                    treatment.setArch(treeItemSelected.getText(), featArchiInput.getText());
                }
            }
        });

        featureGroupSection.setClient(editingFeatureComposite);
    }

    /**
     * Display the section of description and archive of the site
     * 
     * @param parent
     */
    private void displayPathGroup(Composite parent)
    {
        /* ********************************************************************
         * Section for editing the description and the archive paths
         **********************************************************************/
        pathGroupSection = new Section(parent, Section.TWISTIE | Section.DESCRIPTION);
        pathGroupSection.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
        pathGroupSection.setText("Site Description \u0026 Archive Mapping");
        pathGroupSection.addExpansionListener(new IExpansionListener() 
        {
        	public void expansionStateChanged(ExpansionEvent e)
        	{
                featureGroupSection.setExpanded(false);
                categorySection.setExpanded(false);
        	}
        	public void expansionStateChanging(ExpansionEvent e) {};
        });

        // Layout and Data definition
        final GridData gridDataTwoCells = new GridData(SWT.FILL, SWT.FILL, true, true);
        gridDataTwoCells.horizontalSpan = 2;
        gridDataTwoCells.heightHint = 40;
        
        final GridData buttonsGridData = new GridData(SWT.RIGHT, SWT.NONE, true, false);
        buttonsGridData.horizontalSpan = 2;
        
        final GridData gridDataTwoCells2 = new GridData(SWT.FILL, SWT.NONE, true, false);
        gridDataTwoCells2.horizontalSpan = 2;

        final GridData gridDataTwoCells3 = new GridData(SWT.FILL, SWT.CENTER, true, false);
        gridDataTwoCells3.horizontalSpan = 2;
        
        final GridLayout buttonsLayout = new GridLayout(2, true);
        buttonsLayout.marginHeight = 0;
        buttonsLayout.marginWidth = 0;
        
        final GridData tableGridData = new GridData(SWT.FILL, SWT.NONE, true, false);
        tableGridData.heightHint = 80;
        tableGridData.horizontalSpan = 2;
       
        final GridData separatorGridData = new GridData(SWT.FILL, SWT.CENTER, true, false);
        separatorGridData.heightHint = 22;
        separatorGridData.horizontalSpan = 2;
        
        final GridLayout archiveGridLayout = new GridLayout(2, false);
        archiveGridLayout.marginHeight = 0;
        archiveGridLayout.marginWidth = 0;

        // *****************
        // Site Description
        // *****************
        
        final Composite siteDescriptionComposite = new Composite(pathGroupSection, SWT.NONE);
        siteDescriptionComposite.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
        siteDescriptionComposite.setLayout(new GridLayout(2, false));

        // URL
        final Label urlDescriptionLabel = new Label(siteDescriptionComposite, SWT.NONE);
        urlDescriptionLabel.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false));
        urlDescriptionLabel.setText("URL : ");
        
        urlText = new Text(siteDescriptionComposite, SWT.BORDER);
        urlText.setTextLimit(TEXT_LENGTH);
        urlText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        // Description
        final Label descriptionLabel = new Label(siteDescriptionComposite, SWT.NONE);
        descriptionLabel.setLayoutData(gridDataTwoCells3);
        descriptionLabel.setText("Description : ");
        
        descriptionText = new Text(siteDescriptionComposite, SWT.MULTI | SWT.WRAP | SWT.BORDER);
        descriptionText.setTextLimit(MAXI_TEXT_LENGTH);
        descriptionText.setLayoutData(gridDataTwoCells);

        // Separator bar
        final Label separatorUpdateInfo = new Label(siteDescriptionComposite, SWT.SEPARATOR | SWT.HORIZONTAL);
        separatorUpdateInfo.setLayoutData(separatorGridData);

        // ******************
        // Archive Mapping
        // ******************
        final Composite archiveComposite = new Composite(siteDescriptionComposite, SWT.NONE);
        archiveComposite.setLayoutData(gridDataTwoCells2);
        archiveComposite.setLayout(archiveGridLayout);

        // Archive Mapping : PATH
        final Label pathArchiveLabel = new Label(archiveComposite, SWT.NONE);
        pathArchiveLabel.setText("Path : ");
        pathArchiveLabel.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false));
        final Text pathArchiveText = new Text(archiveComposite, SWT.BORDER);
        pathArchiveText.setTextLimit(TEXT_LENGTH);
        pathArchiveText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        // Archive Mapping : URL
        final Label urlArchiveLabel = new Label(archiveComposite, SWT.NONE);
        urlArchiveLabel.setText("URL : ");
        urlArchiveLabel.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false));
        final Text urlArchiveText = new Text(archiveComposite, SWT.BORDER);
        urlArchiveText.setTextLimit(TEXT_LENGTH);
        urlArchiveText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        final Composite buttonsComposite = new Composite(archiveComposite, SWT.NONE);
        buttonsComposite.setLayout(buttonsLayout);
        buttonsComposite.setLayoutData(buttonsGridData);
        // Add Button
        final Button addButtonArchive = new Button(buttonsComposite, SWT.NONE);
        addButtonArchive.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
        addButtonArchive.setText("Add");
        // Delete Button
        final Button deleteButtonArchive = new Button(buttonsComposite, SWT.NONE);
        deleteButtonArchive.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
        deleteButtonArchive.setText("Delete");
        
        // Table : archive mapping
        tableArchive = new Table(siteDescriptionComposite, SWT.BORDER);
        tableArchive.setLayoutData(tableGridData);
        tableArchive.setLinesVisible(true);
        tableArchive.setHeaderVisible(true);

        final TableColumn labelFeatureColumn = new TableColumn(tableArchive, SWT.NONE);
        labelFeatureColumn.setWidth(100);
        labelFeatureColumn.setText("Path");

        final TableColumn valueFeatureColumn = new TableColumn(tableArchive, SWT.NONE);
        valueFeatureColumn.setWidth(100);
        valueFeatureColumn.setText("Url");

        // ***********
        // Listeners
        // ***********
        // Text fields
        urlText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleUrlDescription(urlText);
            }
        });

        descriptionText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleDescription(descriptionText);
            }
        });

        pathArchiveText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handlePathMapping(pathArchiveText);
            }
        });

        urlArchiveText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleUrlMapping(urlArchiveText);
            }
        });

        // Buttons Actions
        addButtonArchive.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (!urlArchiveText.getText().equals("") && !pathArchiveText.getText().equals(""))
                    treatment.handleAddArchiveMapping(tableArchive);
                urlArchiveText.setText("");
                pathArchiveText.setText("");
            }
        });

        deleteButtonArchive.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                TableItem[] itemsSelected = tableArchive.getSelection();
                for (int indice = 0; indice < itemsSelected.length; indice++)
                {
                    TableItem itemSelected = itemsSelected[indice];
                    // Name Selected
                    String nameSelected = itemSelected.getText();
                    treatment.handleDeleteArchiveMapping(nameSelected);
                    treatment.displayMapArchive(tableArchive);
                }
            }
        });

        pathGroupSection.setClient(siteDescriptionComposite);
    }
}
