/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.updatesite;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.ftp.FtpException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.topcased.bus.core.CorePlugin;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * All treatments necessary to the WizardPage CreateSiteUpdateSiteWizardPage
 */
public class CreateSiteUpdateSiteWizardTreatment
{
	private UpdateSiteWizard wizard;

	private Map mapFeature = new HashMap();

	private Map mapCategory = new HashMap();

	private Map archiveMapping = new HashMap(); // key=path ; value=url

	private String url = "";

	private String description = "";

	private String urlArchiveMapping;

	private String pathArchiveMapping;

	// Tag
	private static final String SITE_TAG = "site";

	private static final String DESCRIPTION_TAG = "description";

	private static final String FEATURE_TAG = "feature";

	private static final String CATEGORY_TAG = "category-def";

	private static final String ARCHIVE_TAG = "archive";

	// Attributes
	private static final String URL_ATTRIBUTE = "url";

	private static final String ID_ATTRIBUTE = "id";

	private static final String VERSION_ATTRIBUTE = "version";

	private static final String PATCH_ATTRIBUTE = "patch";

	private static final String OS_ATTRIBUTE = "os";

	private static final String WS_ATTRIBUTE = "ws";

	private static final String NL_ATTRIBUTE = "nl";

	private static final String ARCH_ATTRIBUTE = "arch";

	private static final String PATH_ATTRIBUTE = "path";

	private static final String CATEGORY_ATTRIBUTE = "category";

	private static final String NAME_ATTRIBUTE = "name";

	private static final String LABEL_ATTRIBUTE = "label";

	/**
	 * Construtor
	 * 
	 * @param wizard the parent wizard
	 */
	public CreateSiteUpdateSiteWizardTreatment(UpdateSiteWizard wiz)
	{
		wizard = wiz;
	}

	/**
	 * Initialize the given tree with the map of the features of the site Add the choosen features to the site
	 * 
	 * @param tree
	 */
	public void initializeTree(Tree tree)
	{
		addFeaturesToMap(wizard.getMapAddPluginFeatureUS());
		updateTree(tree);
	}

	/**
	 * Draw the tree with the features of the site and with the categories
	 * 
	 * @param tree
	 */
	public void updateTree(Tree tree)
	{
		Iterator iterCat = mapCategory.keySet().iterator();
		tree.removeAll();
		while (iterCat.hasNext())
		{
			final String categoryKey = (String) iterCat.next();
			TreeItem treeItem = new TreeItem(tree, SWT.NONE, 0);
			treeItem.setText(categoryKey);
			treeItem.setImage(TreeUpdateSiteWizardTreatment.picCategory);

			Iterator iterFeat = mapFeature.keySet().iterator();
			while (iterFeat.hasNext())
			{
				/** TODO : complete this code in future */
//				SiteFeatureReferenceModel refFeature = (SiteFeatureReferenceModel) mapFeature.get(iterFeat.next());
//				String[] categoriesList = refFeature.getCategoryNames();
//				for (int i = 0 ; i < categoriesList.length ; i++)
//				{
//					if (categoriesList[i].equals(categoryKey)) 
//					{
//						TreeItem treeItemFeat = new TreeItem(treeItem, SWT.NONE);
//						treeItemFeat.setText(refFeature.getFeatureIdentifier());
//						treeItemFeat.setImage(TreeUpdateSiteWizardTreatment.picFeature);
//					}
//				}
				
				String featureKey = (String) iterFeat.next();
				InfoFeatureSite featureInfo = getInfoFeatureSite(featureKey);
				if (featureInfo != null && featureInfo.getCategory().equals(categoryKey))
				{
					TreeItem treeItemFeat = new TreeItem(treeItem, SWT.NONE);
					treeItemFeat.setText(featureKey);
					treeItemFeat.setImage(TreeUpdateSiteWizardTreatment.picFeature);
				}
			}
		}

		Iterator iterFeat = mapFeature.keySet().iterator();
		while (iterFeat.hasNext())
		{
			final String key = (String) iterFeat.next();
			final InfoFeatureSite featureInfo = getInfoFeatureSite(key);
			if (featureInfo != null && "".equals(featureInfo.getCategory()))
			{
				TreeItem treeItem = new TreeItem(tree, SWT.NONE);
				treeItem.setText(key);
				treeItem.setImage(TreeUpdateSiteWizardTreatment.picFeature);
			}
		}
	}

	private void addFeaturesToMap(Map mapFeat)
	{
		Iterator iter = mapFeat.keySet().iterator();
		while (iter.hasNext())
		{
			String key = (String) iter.next();
			if (!mapFeature.containsKey(key))
			{
				String name = key;
				if (!key.endsWith(".jar"))
				{
					name = name.concat(".jar");
				}
				else
				{
					key = key.substring(0, key.lastIndexOf("."));
				}
				String URL = "features/" + name;
				String version = key.substring(key.indexOf("_") + 1);
				String id = key.substring(0, key.indexOf("_"));
				mapFeature.put(key, new InfoFeatureSite(URL, version, id));
			}
		}
	}

	private InfoFeatureSite getInfoFeatureSite(String featureName)
	{
		if (mapFeature.containsKey(featureName) && mapFeature.get(featureName) instanceof InfoFeatureSite)
		{
			InfoFeatureSite featureInfo = (InfoFeatureSite) mapFeature.get(featureName);
			return featureInfo;
		}
		return null;
	}

	private InfoCategorySite getInfoCategorySite(String categoryName)
	{
		if (mapCategory.containsKey(categoryName) && mapCategory.get(categoryName) instanceof InfoCategorySite)
		{
			InfoCategorySite categoryInfo = (InfoCategorySite) mapCategory.get(categoryName);
			return categoryInfo;
		}
		return null;
	}

	/**
	 * @param featureName
	 * @return the URL of the feature
	 */
	public String getURL(String featureName)
	{
		if (mapFeature.containsKey(featureName) && mapFeature.get(featureName) instanceof InfoFeatureSite)
		{
			InfoFeatureSite featureInfo = (InfoFeatureSite) mapFeature.get(featureName);
			String URL = featureInfo.getURL();
			return URL;
		}
		return "";
	}

	/**
	 * Set boolean attribute patch to a feature
	 * 
	 * @param featureName
	 * @param patch
	 */
	public void setPatch(String featureName, boolean patch)
	{
		if (mapFeature.containsKey(featureName))
		{
			if (mapFeature.get(featureName) instanceof InfoFeatureSite)
			{
				InfoFeatureSite featureInfo = (InfoFeatureSite) mapFeature.get(featureName);
				if (patch == true)
				{
					featureInfo.setPatch("true");
				}
				else
				{
					featureInfo.setPatch("false");
				}
			}
		}
	}

	/**
	 * Return boolean attribute patch of a feature
	 * 
	 * @param featureName
	 * @return true if the patch is "true", false otherwise
	 */
	public boolean getPatch(String featureName)
	{
		if (mapFeature.containsKey(featureName) && mapFeature.get(featureName) instanceof InfoFeatureSite)
		{
			InfoFeatureSite featureInfo = (InfoFeatureSite) mapFeature.get(featureName);
			return "true".equals(featureInfo.getPatch());
		}
		return false;
	}

	/**
	 * Set attribute os to a feature
	 * 
	 * @param featureName
	 * @param libOS
	 */
	public void setOS(String featureName, String libOS)
	{
		InfoFeatureSite featureInfo = getInfoFeatureSite(featureName);
		if (featureInfo != null)
		{
			featureInfo.setOs(libOS);
		}
	}

	/**
	 * Return attribute os to a feature
	 * 
	 * @param featureName
	 * @return
	 */
	public String getOS(String featureName)
	{
		InfoFeatureSite featureInfo = getInfoFeatureSite(featureName);
		if (featureInfo != null)
		{
			return featureInfo.getOs();
		}
		return "";
	}

	/**
	 * Set attribute ws to a feature
	 * 
	 * @param featureName
	 * @return
	 */
	public String getWS(String featureName)
	{
		InfoFeatureSite featureInfo = getInfoFeatureSite(featureName);
		if (featureInfo != null)
		{
			return featureInfo.getWs();
		}
		return "";
	}

	/**
	 * Return attribute ws to a feature
	 * 
	 * @param featureName
	 * @return
	 */
	public String getNL(String featureName)
	{
		InfoFeatureSite featureInfo = getInfoFeatureSite(featureName);
		if (featureInfo != null)
		{
			return featureInfo.getNl();
		}
		return "";
	}

	/**
	 * Return attribute arch to a feature
	 * 
	 * @param featureName
	 * @return String
	 */
	public String getArch(String featureName)
	{
		InfoFeatureSite featureInfo = getInfoFeatureSite(featureName);
		if (featureInfo != null)
		{
			return featureInfo.getArch();
		}
		return "";
	}

	/**
	 * Set attribute win to a feature
	 * 
	 * @param featureName
	 * @param win
	 */
	public void setWin(String featureName, String win)
	{
		InfoFeatureSite featureInfo = getInfoFeatureSite(featureName);
		if (featureInfo != null)
		{
			featureInfo.setWs(win);
		}
	}

	/**
	 * Set attribute nl to a feature
	 * 
	 * @param featureName
	 * @param Nl
	 */
	public void setNl(String featureName, String Nl)
	{
		InfoFeatureSite featureInfo = getInfoFeatureSite(featureName);
		if (featureInfo != null)
		{
			featureInfo.setNl(Nl);
		}
	}

	/**
	 * Set attribute arch to a feature
	 * 
	 * @param featureName
	 * @param Arch
	 */
	public void setArch(String featureName, String Arch)
	{
		InfoFeatureSite featureInfo = getInfoFeatureSite(featureName);
		if (featureInfo != null)
		{
			featureInfo.setArch(Arch);
		}
	}

	/**
	 * Return attribute name to a category
	 * 
	 * @param categoryName
	 * @return String
	 */
	public String getName(String categoryName)
	{
		InfoCategorySite categoryInfo = getInfoCategorySite(categoryName);
		if (categoryInfo != null)
		{
			return categoryInfo.getName();
		}
		return "";
	}

	/**
	 * Return attribute label to a category
	 * 
	 * @param categoryName
	 * @return
	 */
	public String getLabel(String categoryName)
	{
		InfoCategorySite categoryInfo = getInfoCategorySite(categoryName);
		if (categoryInfo != null)
		{
			return categoryInfo.getLabel();
		}
		return "";
	}

	/**
	 * Return attribute description to a category
	 * 
	 * @param categoryName
	 * @return
	 */
	public String getDesc(String categoryName)
	{
		InfoCategorySite categoryInfo = getInfoCategorySite(categoryName);
		if (categoryInfo != null)
		{
			return categoryInfo.getDescription();
		}
		return "";
	}

	/**
	 * Set attribute name to a category and update the given tree with this new name
	 * 
	 * @param categoryName
	 * @param name
	 * @param treeItem
	 */
	public void setName(String oldName, String newName, TreeItem treeItem)
	{
		InfoCategorySite categoryInfo = getInfoCategorySite(oldName);
		if (categoryInfo != null)
		{
			categoryInfo.setName(newName);
			mapCategory.remove(oldName);
			mapCategory.put(newName, categoryInfo);
			treeItem.setText(newName);
			updateCategoryFeature(oldName, newName);
		}
	}

	/**
	 * Set attribute label to a category
	 * 
	 * @param categoryName
	 * @param label
	 */
	public void setLabel(String categoryName, String label)
	{
		InfoCategorySite categoryInfo = getInfoCategorySite(categoryName);
		if (categoryInfo != null)
		{
			categoryInfo.setLabel(label);
		}
	}

	/**
	 * Set attribute description to a category
	 * 
	 * @param categoryName
	 * @param desc
	 */
	public void setDesc(String categoryName, String desc)
	{
		InfoCategorySite categoryInfo = getInfoCategorySite(categoryName);
		if (categoryInfo != null)
		{
			categoryInfo.setDescription(desc);
		}
	}

	/**
	 * Add the given category to the map of category and to the tree ONLY if the name of the category don't exist already
	 * 
	 * @param name of the category
	 * @param label of the category
	 * @param tree
	 * @return boolean true if the category is succefully added, false otherwise
	 */
	public boolean addCategory(String name, String label, Tree tree)
	{
		return addCategory(name, label, "", tree);
	}

	/**
	 * Add the given category to the map of category and to the tree ONLY if the name of the category don't exist already
	 * 
	 * @param name of the category
	 * @param label of the category
	 * @param desc description of the category
	 * @param tree
	 * @return boolean true if the category is succefully added, false otherwise
	 */
	public boolean addCategory(String name, String label, String desc, Tree tree)
	{
		if (!mapCategory.containsKey(name))
		{
			TreeItem treeItem;
			InfoCategorySite categoryInfo = new InfoCategorySite(name, label, desc);
			mapCategory.put(name, categoryInfo);

			treeItem = new TreeItem(tree, SWT.NONE, 0);
			treeItem.setText(name);
			treeItem.setImage(TreeUpdateSiteWizardTreatment.picCategory);
			return true;
		}
		return false;
	}

	/**
	 * Dispose the category to the map of category and to the treeItem Take all the features included in the category a the end of the parent of item
	 * 
	 * @param item
	 */
	public void subCategory(TreeItem item)
	{
		String categoryName = item.getText();

		InfoCategorySite categoryInfo = getInfoCategorySite(categoryName);
		if (categoryInfo != null)
		{
			Iterator iter = mapFeature.keySet().iterator();
			while (iter.hasNext())
			{
//				SiteFeatureReferenceModel featureReference = (SiteFeatureReferenceModel)mapFeature.get(iter.next());
//				String[] categoriesList = featureReference.getCategoryNames();
//				for (int i = 0 ; i < categoriesList.length ; i++)
//				{
//				}
				
				String key = (String) iter.next();
				InfoFeatureSite featureInfo = (InfoFeatureSite) mapFeature.get(key);
				if (featureInfo.getCategory().equals(categoryName))
				{
					TreeItem treeItem = new TreeItem(item.getParent(), SWT.NONE);
					treeItem.setText(key);
					treeItem.setImage(TreeUpdateSiteWizardTreatment.picFeature);
				}
			}
			item.dispose();
			mapCategory.remove(categoryName);
		}
	}

	/**
	 * Methods which updates the category name when it changes in the tree
	 * 
	 * @param oldCategoryName The old category name
	 * @param newCategoryName the new category name
	 */
	private void updateCategoryFeature(String oldCategoryName, String newCategoryName)
	{
		Iterator iter = mapFeature.keySet().iterator();
		while (iter.hasNext())
		{
			String key = (String) iter.next();
			InfoFeatureSite featureInfo = (InfoFeatureSite) mapFeature.get(key);

			if (featureInfo.getCategory().equals(oldCategoryName))
			{
				featureInfo.setCategory(newCategoryName);
			}
		}
	}

	/**
	 * @param name
	 * @return true if the category exists, false otherwise
	 */
	public boolean categoryExists(String name)
	{
		return mapCategory.containsKey(name);
	}

	/**
	 * Set the attribute category to a feature
	 * 
	 * @param featureName
	 * @param categoryName
	 */
	public void setCategoryToFeature(String featureName, String categoryName)
	{
		InfoFeatureSite featureInfo = getInfoFeatureSite(featureName);
		if (featureInfo != null)
		{
			featureInfo.setCategory(categoryName);
		}
	}

	/**
	 * Display the IHM of the construction of site.xml file
	 * 
	 * @param url_text
	 * @param description_text
	 * @param tableArchive
	 */
	public void displayIHMSiteXml(Text url_text, Text description_text, Table tableArchive)
	{
		url_text.setText(url);
		description_text.setText(description);
		displayMapArchive(tableArchive);
	}

	/**
	 * Take information about url
	 * 
	 * @param urlText
	 */
	public void handleUrlDescription(Text urlText)
	{
		url = urlText.getText();
	}

	/**
	 * Take information about url description
	 * 
	 * @param descriptionText
	 */
	public void handleDescription(Text descriptionText)
	{
		description = descriptionText.getText();
	}

	/**
	 * Take information about archive mapping url
	 * 
	 * @param urlMapping_text
	 */
	public void handleUrlMapping(Text urlMapping_text)
	{
		urlArchiveMapping = urlMapping_text.getText();
	}

	/**
	 * Take information about archive mapping path
	 * 
	 * @param urlMapping_text
	 */
	public void handlePathMapping(Text urlMapping_text)
	{
		pathArchiveMapping = urlMapping_text.getText();
	}

	/**
	 * Add informations (pathArchiveMapping, urlArchiveMapping) in the Map archiveMapping
	 * 
	 * @param tableArhive
	 */
	public void handleAddArchiveMapping(Table tableArchive)
	{
		archiveMapping.put(pathArchiveMapping, urlArchiveMapping);
		displayMapArchive(tableArchive);
	}

	/**
	 * Delete an archive mapping Delete the selection from the Map archiveMapping (key=path ; value=url)
	 * 
	 * @param selectionName
	 */
	public void handleDeleteArchiveMapping(String selectionName)
	{
		archiveMapping.remove(selectionName);
	}

	/**
	 * Diplay the map archive in the table of archive mapping
	 * 
	 * @param tableArhive
	 */
	public void displayMapArchive(Table tableArchive)
	{
		Iterator iter = archiveMapping.keySet().iterator();
		tableArchive.removeAll();
		while (iter.hasNext())
		{
			String key = (String) iter.next();
			String value = (String) archiveMapping.get(key);
			TableItem itemAdd = new TableItem(tableArchive, 0);
			itemAdd.setText(0, key);
			itemAdd.setText(1, value);
			tableArchive.showItem(itemAdd);
			tableArchive.redraw();
		}
		tableArchive.redraw();
		tableArchive.update();
	}

	/**
	 * Reads and initializeas maps if the file site.xml exists already
	 */
	public void readSiteXMLFile()
	{
		mapFeature.clear();
		mapCategory.clear();
		
		/** TODO : Finish to implement this => handle site.xml file with an Eclipse API */
//		SiteModelFactory siteFactory = new SiteModelFactory();
//		try
//		{
//			SiteModel siteXMLFile = siteFactory.parseSite(new FileInputStream(new File(wizard.getUpdateSitePath() + "site.xml")));
//			
//			// Archives
//			ArchiveReferenceModel[] archives = siteXMLFile.getArchiveReferenceModels();
//			for (int i = 0 ; i < archives.length ; i++)
//			{
//				archiveMapping.put(archives[i].getPath(), archives[i].getURLString());
//			}
//			
//			// Features
//			SiteFeatureReferenceModel[] features = siteXMLFile.getFeatureReferenceModels();
//			for (int i = 0 ; i < features.length ; i++)
//			{
//				mapFeature.put(features[i].getFeatureIdentifier(), features[i]);
//			}
//			
//			// Categories
//			CategoryModel[] categories = siteXMLFile.getCategoryModels();
//			for (int i = 0 ; i < categories.length ; i++)
//			{
//				mapCategory.put(categories[i].getName(), categories[i]);
//			}
//		}
//		catch (FileNotFoundException e1)
//		{
//			e1.printStackTrace();
//		}
//		catch (CoreException e1)
//		{
//			e1.printStackTrace();
//		}
//		catch (InvalidSiteTypeException e1)
//		{
//			e1.printStackTrace();
//		}
//	}
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setIgnoringComments(true);
		factory.setCoalescing(true);

		// Parser
		try
		{
			DocumentBuilder parser = factory.newDocumentBuilder();
			// Document
			File xml = new File(wizard.getUpdateSitePath() + "site.xml");
			if (xml.exists())
			{
				Document document = parser.parse(xml);
				NodeList siteList = document.getElementsByTagName(SITE_TAG);
				int nbNode = siteList.getLength();
				for (int i = 0; i < nbNode; i++)
				{
					Element node = (Element) siteList.item(i);

					// Description
					NodeList descriptionList = node.getElementsByTagName(DESCRIPTION_TAG);
					NodeList archiveList;
					NodeList featureList;
					Element descriptionElement = (Element) descriptionList.item(0);

					if (descriptionElement != null)
					{
						if (descriptionElement.hasChildNodes())
						{
							description = descriptionElement.getFirstChild().getNodeValue().trim();
						}
						url = descriptionElement.getAttribute(URL_ATTRIBUTE);
					}
					else
					{
						url = "";
					}

					// Archive
					archiveList = node.getElementsByTagName(ARCHIVE_TAG);
					int nbArchive = archiveList.getLength();
					for (int j = 0; j < nbArchive; j++)
					{
						Element ArchiveElement = (Element) archiveList.item(j);
						String pathArchive = ArchiveElement.getAttribute(PATH_ATTRIBUTE);
						String urlArchive = ArchiveElement.getAttribute(URL_ATTRIBUTE);
						archiveMapping.put(pathArchive, urlArchive);
					}

					// Feature
					featureList = node.getElementsByTagName(FEATURE_TAG);
					int nbFeature = featureList.getLength();
					for (int j = 0; j < nbFeature; j++)
					{
						NodeList catList;
						Element catElement;
						Element featureElement = (Element) featureList.item(j);
						String name = featureElement.getAttribute(ID_ATTRIBUTE) + "_" + featureElement.getAttribute(VERSION_ATTRIBUTE);
						String url = featureElement.getAttribute(URL_ATTRIBUTE);
						String id = featureElement.getAttribute(ID_ATTRIBUTE);
						String version = featureElement.getAttribute(VERSION_ATTRIBUTE);
						InfoFeatureSite featureInfo = new InfoFeatureSite(url, version, id);

						// Optional attributes
						String patch = featureElement.getAttribute(PATCH_ATTRIBUTE);
						if (!"".equals(patch))
						{
							featureInfo.setPatch(patch);
						}

						String os = featureElement.getAttribute(OS_ATTRIBUTE);
						if (!"".equals(os))
						{
							featureInfo.setOs(os);
						}

						String nl = featureElement.getAttribute(NL_ATTRIBUTE);
						if (!"".equals(nl))
						{
							featureInfo.setNl(nl);
						}

						String ws = featureElement.getAttribute(WS_ATTRIBUTE);
						if (!"".equals(ws))
						{
							featureInfo.setWs(ws);
						}

						String arch = featureElement.getAttribute(ARCH_ATTRIBUTE);
						if (!"".equals(arch))
						{
							featureInfo.setArch(arch);
						}

						// Category
						catList = featureElement.getElementsByTagName(CATEGORY_ATTRIBUTE);
						catElement = (Element) catList.item(0);
						if (catElement != null && !"".equals(catElement.getAttribute(NAME_ATTRIBUTE)))
						{
							featureInfo.setCategory(catElement.getAttribute(NAME_ATTRIBUTE));
						}
						mapFeature.put(name, featureInfo);
					}

					// Category
					NodeList categoryList = node.getElementsByTagName(CATEGORY_TAG);
					int nbCategory = categoryList.getLength();
					for (int j = 0; j < nbCategory; j++)
					{
						InfoCategorySite categoryInfo;
						Element categoryElement = (Element) categoryList.item(j);
						String name = categoryElement.getAttribute(NAME_ATTRIBUTE);
						String label = categoryElement.getAttribute(LABEL_ATTRIBUTE);
						// Description
						NodeList descList = categoryElement.getElementsByTagName(DESCRIPTION_TAG);
						Element descElement = (Element) descList.item(0);
						String desc = "";
						if (descElement != null && descElement.hasChildNodes())
						{
							desc = descElement.getFirstChild().getNodeValue().trim();
						}
						categoryInfo = new InfoCategorySite(name, label, desc);

						mapCategory.put(name, categoryInfo);
					}
				}
			}
		}
		catch (ParserConfigurationException e)
		{
			CorePlugin.log(getClass(), "Parser configuration error : " + e.getMessage(), IStatus.ERROR);
		}
		catch (SAXException e)
		{
			CorePlugin.log(getClass(), "Error while parsing document : " + e.getMessage(), IStatus.ERROR);
		}
		catch (IOException e)
		{
			CorePlugin.log(getClass(), "Error while opening file : " + wizard.getUpdateSitePath() + "site.xml"
					+ ". The file site.xml does not already exist.", IStatus.INFO);
		}
	}

	/**
	 * Creates the file site.xml based on the features' map and categories' map
	 */
	public void createSiteXML()
	{
		/** TODO : improvements of code using Eclipse API to handle file site.xml */
//		SiteModelFactory factorySite = new SiteModelFactory();
//		SiteModel newSiteFile = factorySite.createSiteMapModel();
		
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

		try
		{
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document document = builder.newDocument();
			// Tag creation
			Element root = document.createElement(SITE_TAG);
			document.appendChild(root);

			// Description
			if (!"".equals(url) || !"".equals(description))
			{
				Element descriptionTag = document.createElement(DESCRIPTION_TAG);
				root.appendChild(descriptionTag);
				if (!"".equals(url))
				{
					descriptionTag.setAttribute(URL_ATTRIBUTE, url);
					//newSiteFile.setDescriptionModel();
				}
				descriptionTag.appendChild(document.createTextNode(description));
			}

			// Feature
			Iterator itFeat = mapFeature.keySet().iterator();
			while (itFeat.hasNext())
			{
				String key = (String) itFeat.next();
				InfoFeatureSite featureInfo = getInfoFeatureSite(key);
				if (featureInfo != null)
				{
					Element featureTag = document.createElement(FEATURE_TAG);
					root.appendChild(featureTag);
					featureTag.setAttribute(URL_ATTRIBUTE, featureInfo.getURL());
					featureTag.setAttribute(ID_ATTRIBUTE, featureInfo.getId());
					featureTag.setAttribute(VERSION_ATTRIBUTE, featureInfo.getVersion());

					// OS
					if (!"".equals(featureInfo.getOs()))
					{
						featureTag.setAttribute(OS_ATTRIBUTE, featureInfo.getOs());
					}

					// WS
					if (!"".equals(featureInfo.getWs()))
					{
						featureTag.setAttribute(WS_ATTRIBUTE, featureInfo.getWs());
					}

					// NL
					if (!"".equals(featureInfo.getNl()))
					{
						featureTag.setAttribute(NL_ATTRIBUTE, featureInfo.getNl());
					}

					// Architecture
					if (!"".equals(featureInfo.getArch()))
					{
						featureTag.setAttribute(ARCH_ATTRIBUTE, featureInfo.getArch());
					}

					// Patch
					if (!"false".equals(featureInfo.getPatch()))
					{
						featureTag.setAttribute(PATCH_ATTRIBUTE, featureInfo.getPatch());
					}

					// Category
					if (!"".equals(featureInfo.getCategory()))
					{
						Element featureTagCat = document.createElement(CATEGORY_ATTRIBUTE);
						featureTag.appendChild(featureTagCat);
						featureTagCat.setAttribute(NAME_ATTRIBUTE, featureInfo.getCategory());
					}
				}
			}

			// Category
			Iterator itCat = mapCategory.keySet().iterator();
			while (itCat.hasNext())
			{
				String key = (String) itCat.next();
				InfoCategorySite categoryInfo = getInfoCategorySite(key);
				if (categoryInfo != null)
				{
					Element categoryTag = document.createElement(CATEGORY_TAG);
					root.appendChild(categoryTag);
					categoryTag.setAttribute(NAME_ATTRIBUTE, categoryInfo.getName());
					categoryTag.setAttribute(LABEL_ATTRIBUTE, categoryInfo.getLabel());
					// Description
					if (!categoryInfo.getDescription().equals(""))
					{
						Element categoryTagDesc = document.createElement(DESCRIPTION_TAG);
						categoryTag.appendChild(categoryTagDesc);
						categoryTagDesc.appendChild(document.createTextNode(categoryInfo.getDescription()));
					}
				}
			}

			// Archive
			Iterator iterArch = archiveMapping.keySet().iterator();
			while (iterArch.hasNext())
			{
				String key = (String) iterArch.next();
				Element archiveTag = document.createElement(ARCHIVE_TAG);
				root.appendChild(archiveTag);
				archiveTag.setAttribute(URL_ATTRIBUTE, (String) archiveMapping.get(key));
				archiveTag.setAttribute(PATH_ATTRIBUTE, key);
			}

			// Xml write
			TransformerFactory tFactory = TransformerFactory.newInstance();
			Transformer transformer = tFactory.newTransformer();

			// Indent the XML document
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "3");

			DOMSource sourceDom = new DOMSource(document);
			StreamResult result = new StreamResult(wizard.getUpdateSitePath() + "site.xml");
			transformer.transform(sourceDom, result);
		}
		catch (ParserConfigurationException e)
		{
			CorePlugin.log(getClass(), "Parser configuration exception" + e.getMessage(), IStatus.ERROR);

		}
		catch (TransformerConfigurationException e)
		{
			CorePlugin.log(getClass(), "Transformer configuration exception" + e.getMessage(), IStatus.ERROR);
		}
		catch (TransformerException e)
		{
			CorePlugin.log(getClass(), "Transformer exception" + e.getMessage(), IStatus.ERROR);
			wizard.getIOErrorVector().add(e.getMessage());
		}

		switch (wizard.getProtocolConnex())
		{
		case (UpdateSiteWizard.FTP):
			if (wizard.getClientFTP() != null)
			{
				String pathFile = wizard.getUpdateSitePath() + "site.xml";
				final String path = wizard.getDirectory() + "site.xml";

				try
				{
					// TODO voir s'il faut changer la copie suite aux modifs 
					// faire lors de la consultation des informations
					FilesTreatment.copyFileToFTP(pathFile, path, wizard.getClientFTP());
					CorePlugin.log(getClass(), "File site.xml successfully created on FTP server : " + wizard.getDirectory() + "site.xml",
							IStatus.OK);
				}
				catch (FtpException e)
				{
					CorePlugin.log(getClass(), "FTP Exception : " + e.getMessage(), IStatus.ERROR);
					CorePlugin.log(e);
					wizard.getIOErrorVector().add(e.getMessage());
				}
			}
			break;
		case (UpdateSiteWizard.SSH):
			if (wizard.getClientSSH() != null)
			{
				String pathFile = wizard.getUpdateSitePath() + "site.xml";
				final String path = wizard.getDirectory() + "site.xml";

				try
				{
					wizard.getClientSFTP().put(pathFile, path);
					CorePlugin.log(getClass(), "File site.xml successfully created on SSH server : " + wizard.getDirectory() + "site.xml",
							IStatus.OK);
				}
				catch (IOException e)
				{
					CorePlugin.log(e);
				}
			}
			break;
		default:
			CorePlugin.log(getClass(), "File site.xml successfully created : " + wizard.getUpdateSitePath() + "site.xml", IStatus.OK);
			break;
		}
	}
}
