/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.updatesite;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

/**
 * 
 * This WizardPage displays the tree of the features/plugins
 */
public class TreeUpdateSiteWizardPage extends WizardPage
{
    private TreeUpdateSiteWizardTreatment treatment;

    private Tree tree;

    private Label label;

    private Button checkboxFilter;

    private Label infoPlugFeat;

    private ProgressBar progressBar;

    private Label labelProgBar;

    private UpdateSiteWizard wizard;

    /**
     * Constructor
     * 
     * @param wizard the parent wizard
     * @param pageName the name of the current page
     */
    public TreeUpdateSiteWizardPage(UpdateSiteWizard wizard, String pageName)
    {
        super(pageName);
        this.wizard = wizard;
    }

    /**
     * Create the controls on the page
     * 
     * @param parent
     */
    public void createControl(Composite parent)
    {
        final GridData gridDataTree = new GridData(GridData.FILL_BOTH);
        gridDataTree.heightHint = 250;
        
    	final Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout(1, true));
        container.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        setControl(container);

        treatment = new TreeUpdateSiteWizardTreatment(wizard);

        /*
         * Presentation label
         */
        label = new Label(container, SWT.NONE);
        label.setText("Plugins and Features necessary to the installation :");
        label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        /*
         * Plugins and Features Tree
         */
        tree = new Tree(container, SWT.SINGLE | SWT.BORDER);
        tree.setLayoutData(gridDataTree);
        tree.addSelectionListener(new SelectionListener()
        {
            public void widgetSelected(SelectionEvent e)
            {
                TreeItem treeItem = (TreeItem) e.item;
                SourceDestination data = (SourceDestination) treeItem.getData();
                
                if(data == null)
                {
                    wizard.getRequiredPage().setInfoPlugFeatText("Information about plugins and features...");
                }
                else
                {
                	switch(data.getState())
                	{
			            case (SourceDestination.MISSING):
			            	wizard.getRequiredPage().setInfoPlugFeatText("The source of this element cannot be found");
			                break;
			            case (SourceDestination.DONT_EXIST):
			            	wizard.getRequiredPage().setInfoPlugFeatText("This element will be installed");
			                break;
			            case (SourceDestination.ALREADY_EXIST):
			            	wizard.getRequiredPage().setInfoPlugFeatText("This element is already installed");
			                break;
			        }
                }
            }

            public void widgetDefaultSelected(SelectionEvent e)
            {
                TreeItem treeItem = (TreeItem) e.item;

                treeItem.setExpanded(!treeItem.getExpanded());
            }
        });

        /*
         * Required plugins copy Checkbox
         */
        checkboxFilter = new Button(container, SWT.CHECK);
        checkboxFilter.setText("Copy the required plugins too");
        checkboxFilter.setSelection(false);
        checkboxFilter.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (checkboxFilter.getSelection())
                {
                    enableProgressBar();
                    Thread thread = new Thread()
                    {
                        public void run()
                        {
                            treatment.addRequiredPlugins(tree);
                        }
                    };
                    thread.start();
                }
                else
                {
                    treatment.subRequiredPlugins(tree);
                }
            }
        });

        /*
         * Informations message label
         */
        Group group = new Group(container,SWT.SHADOW_OUT);
        group.setLayout(new GridLayout(1, true));
        group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        group.setText("Information : ");
        
        infoPlugFeat = new Label(group, SWT.NONE);
        infoPlugFeat.setText("Informations about plugins and features...");
        infoPlugFeat.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        /*
         * Label for initializing trees and maps
         */
        labelProgBar = new Label(container, SWT.NONE);
        labelProgBar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        /*
         * Progress Bar initializing trees and maps
         */
        progressBar = new ProgressBar(container, SWT.NONE);
        progressBar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        progressBar.setVisible(false);

        setPageComplete(true);
    }

    /**
     * Set the text in the information bar on the lower part of the page
     * 
     * @param text
     */
    public void setInfoPlugFeatText(String text)
    {
        infoPlugFeat.setText(text);
    }

    /**
     * Return the state of the page, if the button 'Next' can be clicked
     * 
     * @return boolean
     */
    public boolean finish()
    {
        return ( isCurrentPage() || !wizard.getMapPluginUS().isEmpty() );
    }

    /**
     * Redraw the tree, with the features choosen
     */
    public void updateTree()
    {
        treatment.initializeTree(tree);
    }

    /**
     * @param infoPlugFeat The infoPlugFeat text to set.
     */
    public void setTextInfoPlugFeat(String infoPlugFeat)
    {
        this.infoPlugFeat.setText(infoPlugFeat);
    }

    /**
     * @return boolean
     */
    public boolean canFlipToNextPage()
    {
        checkboxFilter.setSelection(false);
        if (isCurrentPage())
        {
            getControl().update();
            updateTree();
        }

        return true;// TODO false;
    }

    /**
     * Disable all the controls of the page, except the progressBar, its
     * associated text
     */
    public void disableAll()
    {
        tree.setEnabled(false);
        label.setEnabled(false);
        checkboxFilter.setEnabled(false);
        infoPlugFeat.setEnabled(false);
    }

    /**
     * Enable all the controls of the page, except the progressBar, its
     * associated text
     */
    public void enableAll()
    {
        tree.setEnabled(true);
        label.setEnabled(true);
        checkboxFilter.setEnabled(true);
        infoPlugFeat.setEnabled(true);
    }

    /**
     * Initialize the size of the progressBar
     * 
     * @param min
     * @param max
     */
    public void initProgressBar(int min, int max)
    {
        progressBar.setMinimum(min);
        progressBar.setMaximum(max);
        progressBar.setSelection(min);
    }

    /**
     * Set the informations just on the top of the progressBar Add 1 at the
     * selection of the progressBar
     * 
     * @param labelInfo
     */
    public void setInfoAboutProgressBar(String labelInfo)
    {
        progressBar.setSelection(progressBar.getSelection() + 1);
        labelProgBar.setText(labelInfo);
    }

    /**
     * Disable the progressBar and its associated text
     */
    public void disableProgressBar()
    {
        progressBar.setVisible(false);
        labelProgBar.setVisible(false);
    }

    /**
     * Enable the progressBar and its associated text
     */
    public void enableProgressBar()
    {
        progressBar.setVisible(true);
        labelProgBar.setVisible(true);
    }
}
