/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.updatesite;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.update.core.IIncludedFeatureReference;
import org.eclipse.update.core.model.FeatureModel;
import org.eclipse.update.core.model.FeatureModelFactory;
import org.osgi.framework.Bundle;
import org.topcased.bus.core.CorePlugin;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * 
 * This class contains the TreeUpdateSiteWizardPage treatments
 */
public class TreeUpdateSiteWizardTreatment
{
	// Map for the Add/Delete repository
	private Map mapAddPluginFeatureUS;

	private Map mapPluginUS;

	private Map mapFeatureInclude;

	private Map mapRequiredPlugins;

	// XML tags
	private static final String PLUGIN = "plugin";

	private static final String INCLUDES = "includes";

	private static final String REQUIRES = "requires";

	private static final String IMPORT = "import";

	// XML Attributs
	private static final String ID = "id";

	private static final String VERSION = "version";

	// Var for images
	public static final String BASE_URL = UpdateSitePlugin.getDefault().getBundle().getLocation();

	public static final String BASE_PATH = File.separator + BASE_URL;// .substring(BASE_URL.indexOf("@") + 1);// getEntry("/");

	public static final String iconPath = "pictures/";

	private UpdateSiteWizard wizard;

	private TreeItem treeItemRequiredPlugin;

	/**
	 * Define an image which represent an element with missing error
	 */
	public static Image picMissingError;

	/**
	 * Define an image which represent an element which is already installed on destination
	 */
	public static Image picAlreadyInstalled;

	/**
	 * Define an image which represent an element which will be installed on destination
	 */
	public static Image picWillBeInstalled;

	/**
	 * Define an image which represent a category of feature
	 */
	public static Image picCategory;

	/**
	 * Define an image which represent a feature
	 */
	public static Image picFeature;

	/**
	 * Contructor : initialize all Maps and all images used in the wizard
	 * 
	 * @param theWizard the current wizard
	 */
	public TreeUpdateSiteWizardTreatment(UpdateSiteWizard theWizard)
	{

		ImageDescriptor imgDesc;
		wizard = theWizard;
		mapAddPluginFeatureUS = wizard.getMapAddPluginFeatureUS();
		mapPluginUS = wizard.getMapPluginUS();
		mapFeatureInclude = wizard.getMapFeatureInclude();
		mapRequiredPlugins = wizard.getMapRequiredPlugins();
		imgDesc = UpdateSitePlugin.getImageDescriptor("icons" + File.separator + "unconf-feature-obj.gif");
		picMissingError = imgDesc.createImage(wizard.getShell().getDisplay());
		imgDesc = UpdateSitePlugin.getImageDescriptor("icons" + File.separator + "not-installed-feature-obj.gif");
		picWillBeInstalled = imgDesc.createImage(wizard.getShell().getDisplay());
		imgDesc = UpdateSitePlugin.getImageDescriptor("icons" + File.separator + "category-obj.gif");
		picCategory = imgDesc.createImage(wizard.getShell().getDisplay());
		imgDesc = UpdateSitePlugin.getImageDescriptor("icons" + File.separator + "feature-obj.gif");
		picFeature = imgDesc.createImage(wizard.getShell().getDisplay());
		imgDesc = UpdateSitePlugin.getImageDescriptor("icons" + File.separator + "feature-obj.gif");
		picAlreadyInstalled = imgDesc.createImage(wizard.getShell().getDisplay());
	}

	/**
	 * Initialize a tree with the map of the Features choosen
	 * 
	 * @param tree Tree to initialize
	 */
	public void initializeTree(Tree tree)
	{
		tree.removeAll();
		mapPluginUS.clear();

		TreeItem treeItem = new TreeItem(tree, SWT.NULL);
		treeItem.setText("Features/plugins necessary");

		wizard.getRequiredPage().disableAll();
		wizard.getRequiredPage().enableProgressBar();
		wizard.getRequiredPage().initProgressBar(0, mapAddPluginFeatureUS.size());
		wizard.getRequiredPage().setInfoAboutProgressBar("Initializing map with choosen features...");

		Iterator it = mapAddPluginFeatureUS.keySet().iterator();
		while (it.hasNext())
		{
			String key = (String) it.next();
			if (mapAddPluginFeatureUS.get(key) instanceof SourceDestination)
			{
				FilesTreatment.autoState((SourceDestination) mapAddPluginFeatureUS.get(key));
				addFeature(treeItem, (SourceDestination) mapAddPluginFeatureUS.get(key), mapFeatureInclude, mapPluginUS);
			}
		}
		wizard.getRequiredPage().enableAll();
		wizard.getRequiredPage().disableProgressBar();
	}

	/**
	 * Add the required plugins to a tree (add the TreeItem "Required plugins") and to the map of required plugins
	 * 
	 * @param tree to initalize
	 */
	public void addRequiredPlugins(final Tree tree)
	{
		mapRequiredPlugins.clear();

		wizard.getRequiredPage().getShell().getDisplay().asyncExec(new Runnable()
		{
			public void run()
			{
				treeItemRequiredPlugin = new TreeItem(tree, SWT.NULL);
				treeItemRequiredPlugin.setText("Required plugins");

				wizard.getRequiredPage().disableAll();
				wizard.getRequiredPage().enableProgressBar();
				wizard.getRequiredPage().initProgressBar(0,
						mapAddPluginFeatureUS.size() + mapFeatureInclude.size() + mapPluginUS.size() + 3);

				wizard.getRequiredPage().setInfoAboutProgressBar("Initializing map with choosen features...");
			}
		});

		readRequiredPluginFromMap(mapAddPluginFeatureUS);

		wizard.getRequiredPage().getShell().getDisplay().asyncExec(new Runnable()
		{
			public void run()
			{
				wizard.getRequiredPage().setInfoAboutProgressBar("Initializing map with included features...");
			}
		});

		readRequiredPluginFromMap(mapFeatureInclude);

		wizard.getRequiredPage().getShell().getDisplay().asyncExec(new Runnable()
		{
			public void run()
			{
				wizard.getRequiredPage().setInfoAboutProgressBar("Initializing map with plugins...");
			}
		});

		readRequiredPluginFromMap(mapPluginUS);

		wizard.getRequiredPage().getShell().getDisplay().asyncExec(new Runnable()
		{
			public void run()
			{
				wizard.getRequiredPage().enableAll();
				wizard.getRequiredPage().disableProgressBar();
			}
		});
	}

	/**
	 * Read all the plugins required for a Map of features, and call addRequiredPlugins for all
	 * 
	 * @param map the map which contains the plugins
	 */
	public void readRequiredPluginFromMap(Map map)
	{
		String fileName = "";

		Iterator it = map.keySet().iterator();
		while (it.hasNext())
		{
			String key = (String) it.next();
			if (map.get(key) instanceof SourceDestination)
			{
				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				factory.setIgnoringComments(true);
				factory.setCoalescing(true);

				try
				{
					DocumentBuilder parser = factory.newDocumentBuilder();

					String featurePath = ((SourceDestination) map.get(key)).getSource();

					final String name = ((SourceDestination) map.get(key)).getName();

					wizard.getRequiredPage().getShell().getDisplay().asyncExec(new Runnable()
					{
						public void run()
						{
							wizard.getRequiredPage().setInfoAboutProgressBar("Initializing map with " + name);
						}
					});

					File fileToParse = null;
					File propertyFile = null;
					boolean isJarFile = false;
					if (featurePath.endsWith(".jar"))
					{
						isJarFile = true;
					}

					String fileToOpen = "";
					String filePropToOpen = "";
					int typeOfFile;
					int typeOfFileProp;

					if (featurePath.indexOf("features") != -1)
					{
						fileToOpen = "feature.xml";
						filePropToOpen = "feature.properties";
						typeOfFile = FilesTreatment.FEATURE_XML;
						typeOfFileProp = FilesTreatment.FEATURE_PROPERTIES;
					}
					else
					{
						fileToOpen = "plugin.xml";
						filePropToOpen = "plugin.properties";
						typeOfFile = FilesTreatment.PLUGIN_XML;
						typeOfFileProp = FilesTreatment.PLUGIN_PROPERTIES;
					}

					// Extract 'feature.xml' from the jar file to a temporary
					// file 'tempFeatureName'
					File pathFile = new File(featurePath);
					if (isJarFile)
					{
						try
						{
							fileToParse = FilesTreatment.createTempFileFromJar(pathFile, "tmp.xml", typeOfFile);
							// Property path
							propertyFile = FilesTreatment.createTempFileFromJar(pathFile, "tmp.properties", typeOfFileProp);
							fileName = featurePath;
						}
						catch (IOException e)
						{
							CorePlugin.log(this.getClass(), "JAR not found in the current environment : " + e.getMessage(),
									IStatus.WARNING, e);
						}
					}
					else
					{
						fileToParse = new File(featurePath + File.separator + fileToOpen);
						// Property path
						propertyFile = new File(featurePath + File.separator + filePropToOpen);

						fileName = featurePath + File.separator + fileToOpen;
					}

					if (fileToParse != null && (isJarFile || fileToParse.exists()))
					{
						// If the file is found, it is parsed.
						// Document
						Document document = parser.parse(fileToParse);
						NodeList requiredList = document.getElementsByTagName(REQUIRES);
						int nbNode = requiredList.getLength();
						for (int i = 0; i < nbNode; i++)
						{
							Element node = (Element) requiredList.item(i);
							NodeList importList = node.getElementsByTagName(IMPORT);
							int nbImport = importList.getLength();
							for (int j = 0; j < nbImport; j++)
							{
								Element importNode = (Element) importList.item(j);

								String plugin = importNode.getAttribute(PLUGIN);
								String version = importNode.getAttribute(VERSION);
								if (plugin.length() > 0)
								{
									// In this case plugins are required
									// but features can be required too.
									// Treat them in the else clause.
									String path2 = FilesTreatment.getFileWithVersionFromNoVersion(wizard.getUpdateSiteToolAdminPath()
											+ "plugins" + File.separator + plugin);
									if (path2 != null)
									{
										String tempPath2 = new String(path2);
										version = "";
										while (tempPath2.indexOf("_") != -1)
										{
											version = tempPath2.substring(tempPath2.lastIndexOf('_')).concat(version);
											tempPath2 = tempPath2.substring(0, tempPath2.lastIndexOf('_'));

										}
									}
									else
									{
										if (FilesTreatment.isFileExistsWithoutVersion(wizard.getUpdateSiteToolAdminPath() + "plugins"
												+ File.separator + plugin))
										{
											version = "_" + FilesTreatment.analyzeString(version, propertyFile);
										}
										else
										{
											version = "";
										}
									}

									String pluginName = FilesTreatment.analyzeString(plugin, propertyFile) + version;

									String pluginPath = UpdateSiteWizard.checkPath(wizard.getUpdateSiteToolAdminPath() + "plugins", true)
											+ pluginName;

									// CorePlugin.log(this.getClass(), "plugin path :
									// "+pluginPath, IStatus.WARNING);

									SourceDestination sd = new SourceDestination(pluginName, pluginPath, UpdateSiteWizard.checkPath(wizard
											.getUpdateSitePath()
											+ "plugins", wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC));
									FilesTreatment.autoState(sd);

									if (!mapRequiredPlugins.containsKey(sd.getName()))
									{
										addRequiredPlugin(treeItemRequiredPlugin, sd, mapRequiredPlugins);
									}
								}

							}
						}
					}
					// Manifest computing
					if (featurePath.indexOf("features") == -1)
					{
						// If it is a plugin, the information can be contained in the manifest file instead of the plugin.xml
						// This variable is used to know the manifest file to read
						File manifestFile = null;
						if (isJarFile)
						{
							try
							{
								manifestFile = FilesTreatment.createTempFileFromJar(pathFile, "manifest.mf", FilesTreatment.MANIFEST);
							}
							catch (IOException e)
							{
								CorePlugin.log(this.getClass(), "JAR not found in the current environment  : " + e.getMessage(),
										IStatus.WARNING);
							}
						}
						else
						{
							manifestFile = new File(featurePath + File.separator + "META-INF" + File.separator + "MANIFEST.MF");
						}

						if (manifestFile != null && (isJarFile || manifestFile.exists()))
						{
							// If the manifest was found, the dependencies are extracted from it
							String[] plugins = UpdateSiteWizard.readManifestFile(manifestFile.getAbsolutePath());
							for (int i = 0; i < plugins.length; i++)
							{
								Bundle pluginEntry = Platform.getBundle(plugins[i]);
								if (pluginEntry != null)
								{
									String version = (String) pluginEntry.getHeaders().get("Bundle-version");
									IPath pluginName = new Path(plugins[i] + "_" + version);
									IPath fullPath = wizard.getPluginUpdateSitePath().append(pluginName);

									if (!fullPath.toFile().isDirectory())
									{
										fullPath = fullPath.addFileExtension("jar");
									}

									SourceDestination sd = new SourceDestination(pluginName.toString(), fullPath.toString(),
											UpdateSiteWizard.checkPath(wizard.getUpdateSitePath() + "plugins",
													wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC));
									FilesTreatment.autoState(sd);

									if (!mapRequiredPlugins.containsKey(sd.getName()))
									{
										addRequiredPlugin(treeItemRequiredPlugin, sd, mapRequiredPlugins);
									}
								}
							}
						}
					}

					// deletes temp files
					if (isJarFile)
					{
						if (propertyFile != null)
						{
							if (propertyFile.exists())
							{
								propertyFile.delete();
							}
						}
						if (fileToParse != null)
						{
							if (fileToParse.exists())
							{
								fileToParse.delete();
							}
						}
					}

				}
				catch (ParserConfigurationException e)
				{
					CorePlugin.log(this.getClass(), "ParserConfiguration error : " + e.getMessage(), IStatus.ERROR);
				}
				catch (SAXException e)
				{
					CorePlugin.log(this.getClass(), "Error while parsing document : " + e.getMessage(), IStatus.ERROR, e);
				}
				catch (IOException e)
				{
					CorePlugin.log(this.getClass(), "Error while opening file : " + e.getMessage(), IStatus.ERROR, e);
				}
				catch (Exception e)
				{
					CorePlugin.log(this.getClass(), "Error : " + e.getMessage() + fileName + " file no found", IStatus.ERROR, e);
				}
			}
		}
	}

	/**
	 * Dispose the TreeItem "Required plugins" to the given tree and clear the map of required plugins
	 * 
	 * @param tree the tree
	 */
	public void subRequiredPlugins(Tree tree)
	{
		treeItemRequiredPlugin.dispose();
		tree.update();
		tree.redraw();
		mapRequiredPlugins.clear();
	}

	/**
	 * Add all the plugins imported and all the features included (recursivly) of the given feature Add the TreeItems to a treeItem
	 * 
	 * @param treeItem
	 * @param featureSD the feature
	 * @param mapFeaturesIncluded the map which contains the features included
	 * @param mapPlugins the map which contains the plugins imported
	 */
	public void addFeature(TreeItem treeItem, SourceDestination featureSD, Map mapFeaturesIncluded, Map mapPlugins)
	{
		wizard.getRequiredPage().setInfoAboutProgressBar("Initializing map with " + featureSD.getName());

		String featureName = featureSD.getName();
		String featurePath = wizard.getUpdateSiteToolAdminPath() + File.separator + "features" + File.separator + featureName;

		/*******************************************************************************************************************************************************
		 * Ajout du TreeItem
		 */
		TreeItem itemFeature = new TreeItem(treeItem, SWT.NULL);
		itemFeature.setText(featureName);
		itemFeature.setData(featureSD);
		switch (featureSD.getState())
		{
		case (SourceDestination.MISSING):
			itemFeature.setImage(picMissingError);
			break;
		case (SourceDestination.DONT_EXIST):
			itemFeature.setImage(picWillBeInstalled);
			break;
		case (SourceDestination.ALREADY_EXIST):
			itemFeature.setImage(picAlreadyInstalled);
			break;
		}

		/*******************************************************************************************************************************************************
		 * Ajout de plugins  mapPlugins
		 */
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setIgnoringComments(true);
		factory.setCoalescing(true);

		File fileToParse = null;
		File propertyFile = null;
		boolean isJarFile = false;
		if (featurePath.endsWith(".jar"))
		{
			isJarFile = true;
		}

		File pathFile = new File(featurePath);
		if (isJarFile)
		{
			try
			{
				fileToParse = FilesTreatment.createTempFileFromJar(pathFile, "feature.xml", FilesTreatment.FEATURE_XML);
				propertyFile = FilesTreatment.createTempFileFromJar(pathFile, "feature.properties", FilesTreatment.FEATURE_PROPERTIES);
			}
			catch (IOException e)
			{
				CorePlugin.log(this.getClass(), "JAR not found in the current environment  : " + e.getMessage(), IStatus.WARNING, e);
			}
		}
		else
		{
			fileToParse = new File(featurePath + File.separator + "feature.xml");
			propertyFile = new File(featurePath + File.separator + "feature.properties");
		}

		// Parser
		try
		{
			DocumentBuilder parser = factory.newDocumentBuilder();
			// Document
			Document document = parser.parse(fileToParse);
			NodeList featureList = document.getElementsByTagName(PLUGIN);
			int nbNode = featureList.getLength();
			for (int i = 0; i < nbNode; i++)
			{
				Element node = (Element) featureList.item(i);
				String id = node.getAttribute(ID);
				String version = node.getAttribute(VERSION);

				String pluginName = FilesTreatment.analyzeString(id, propertyFile) + "_" + version;
				String pluginPath = UpdateSiteWizard.checkPath(wizard.getUpdateSiteToolAdminPath() + "plugins", true) + pluginName;

				if (!FilesTreatment.isFileExists(pluginPath) && !FilesTreatment.isFileExistsWithoutVersion(pluginPath))
				{
					pluginPath = pluginPath + ".jar";
				}
				else
				{
					if (FilesTreatment.isFileExistsWithoutVersion(pluginPath))
					{
						pluginPath = FilesTreatment.getFileWithVersionFromNoVersion(pluginPath);
					}
				}

				SourceDestination sd = new SourceDestination(pluginName, pluginPath, UpdateSiteWizard.checkPath(wizard.getUpdateSitePath()
						+ "plugins", true));
				FilesTreatment.autoState(sd);

				/***********************************************************************************************************************************************
				 * Ajout du TreeItem par plugin
				 */
				TreeItem itemPlugin = new TreeItem(itemFeature, SWT.NULL);
				itemPlugin.setData(sd);
				itemPlugin.setText(pluginName.toString());
				switch (sd.getState())
				{
				case (SourceDestination.MISSING):
					itemPlugin.setImage(picMissingError);
					break;
				case (SourceDestination.DONT_EXIST):
					itemPlugin.setImage(picWillBeInstalled);
					break;
				case (SourceDestination.ALREADY_EXIST):
					itemPlugin.setImage(picAlreadyInstalled);
					break;
				}

				mapPlugins.put(pluginName, sd);
			}
		}
		catch (ParserConfigurationException e)
		{
			CorePlugin.log(this.getClass(), "ParserConfiguration error : " + e.getMessage(), IStatus.ERROR, e);
		}
		catch (SAXException e)
		{
			CorePlugin.log(this.getClass(), "Error while parsing document : " + e.getMessage(), IStatus.ERROR, e);
		}
		catch (IOException e)
		{
			CorePlugin.log(this.getClass(), "Error while opening file : " + e.getMessage() + ". Maybe is not a feature.", IStatus.ERROR, e);
		}

		/**
		 * Process of Included Features
		 */
		try
		{
			FeatureModelFactory featureFactory = new FeatureModelFactory();
			FeatureModel featureModel = featureFactory.parseFeature(new FileInputStream(fileToParse));

			// The included features are read
			IIncludedFeatureReference[] refIncludedFeatures = featureModel.getFeatureIncluded();
			for (int i = 0; i < refIncludedFeatures.length; i++)
			{
				String featureId = refIncludedFeatures[i].getVersionedIdentifier().getIdentifier();
				String featureVersion = refIncludedFeatures[i].getVersionedIdentifier().getVersion().toString();

				featureId = featureId + "_" + featureVersion;

				String newFeaturePath = UpdateSiteWizard.checkPath(wizard.getUpdateSiteToolAdminPath() + "features", true) + featureId;

				SourceDestination sd = new SourceDestination(featureId, newFeaturePath, UpdateSiteWizard.checkPath(wizard
						.getUpdateSitePath()
						+ "features", wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC));

				FilesTreatment.autoState(sd);

				mapFeaturesIncluded.put(featureId, sd);
				addFeature(itemFeature, sd, mapFeaturesIncluded, mapPlugins);
			}
		}
		catch (SAXException e)
		{
			CorePlugin.log(this.getClass(), "Error while parsing document : " + e.getMessage(), IStatus.ERROR, e);
		}
		catch (IOException e)
		{
			CorePlugin.log(this.getClass(), "Error while opening file : " + e.getMessage() + ". Maybe is not a feature.", IStatus.ERROR, e);
		}
		catch (CoreException e)
		{
			CorePlugin.log(this.getClass(), "CoreException reading feature : " + e.getMessage() + ". Maybe is not a feature.",
					IStatus.ERROR, e);
		}

		// deletes temp files
		if (isJarFile && fileToParse.exists())
			fileToParse.delete();
		
		if (isJarFile && propertyFile.exists())
			propertyFile.delete();
	}

	/**
	 * Add the required plugins of a plugins given (recursivity) Add the plugins in a TreeItem
	 * 
	 * @param treeItem
	 * @param pluginSD the plugin
	 * @param mapPluginsRequired the map of the plugins required
	 */
	public void addRequiredPlugin(final TreeItem treeItem, final SourceDestination pluginSD, Map mapPluginsRequired)
	{

		final String name = pluginSD.getName();

		/*******************************************************************************************************************************************************
		 * Ajout du TreeItem pour le plugin
		 */
		if (!mapPluginsRequired.containsKey(name))
		{
			wizard.getRequiredPage().getShell().getDisplay().asyncExec(new Runnable()
			{
				public void run()
				{
					final TreeItem item = new TreeItem(treeItem, SWT.NULL);
					item.setText(name);
					item.setData(pluginSD);
					item.setImage(picWillBeInstalled);
					switch (pluginSD.getState())
					{
					case (SourceDestination.MISSING):
						item.setImage(picMissingError);
						break;
					case (SourceDestination.DONT_EXIST):
						item.setImage(picWillBeInstalled);
						break;
					case (SourceDestination.ALREADY_EXIST):
						item.setImage(picAlreadyInstalled);
						break;
					}
				}
			});
		}
		/*******************************************************************************************************************************************************
		 * Ajout du plugin  la map
		 */
		mapPluginsRequired.put(name, pluginSD);

		/*******************************************************************************************************************************************************
		 * Pareil pour tous les required du plugin courant
		 */
		String path = pluginSD.getSource();

		if (!FilesTreatment.isFileExists(path) && !FilesTreatment.isFileExistsWithoutVersion(path))
		{
			path = path + ".jar";
		}
		else
		{
			if (FilesTreatment.isFileExistsWithoutVersion(path))
			{
				path = FilesTreatment.getFileWithVersionFromNoVersion(path);
			}
		}

		File fileToParse = null;
		File propertyFile = null;
		boolean isJarFile = false;

		try
		{
			if (path.endsWith(".jar"))
			{
				isJarFile = true;
			}
		}
		catch (NullPointerException e)
		{
			CorePlugin.log(this.getClass(), "File does not exist. " + e.getMessage(), IStatus.ERROR, e);
		}

		// Extract 'feature.xml' from the jar file to a temporary file
		// 'tempFeatureName'
		File pathFile = new File(path);
		if (isJarFile)
		{
			try
			{
				fileToParse = FilesTreatment.createTempFileFromJar(pathFile, "plugin.xml", FilesTreatment.PLUGIN_XML);
				propertyFile = FilesTreatment.createTempFileFromJar(pathFile, "plugin.properties", FilesTreatment.PLUGIN_PROPERTIES);
			}
			catch (IOException e)
			{
				CorePlugin.log(this.getClass(), "JAR not found in the current environment : " + e.getMessage(), IStatus.WARNING, e);
			}
		}
		else
		{
			fileToParse = new File(path, "plugin.xml");
			propertyFile = new File(path, "plugin.properties");
		}

		if (fileToParse != null)
		{
			try
			{
				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				factory.setIgnoringComments(true);
				factory.setCoalescing(true);

				DocumentBuilder parser = factory.newDocumentBuilder();

				// Document
				Document document = parser.parse(fileToParse);
				NodeList requiredList = document.getElementsByTagName(REQUIRES);
				int nbNode = requiredList.getLength();
				for (int i = 0; i < nbNode; i++)
				{
					Element node = (Element) requiredList.item(i);
					NodeList importList = node.getElementsByTagName(IMPORT);
					int nbImport = importList.getLength();
					for (int j = 0; j < nbImport; j++)
					{
						Element importNode = (Element) importList.item(j);

						String plugin = importNode.getAttribute(PLUGIN);
						String version = node.getAttribute(VERSION);

						// String path2 = FilesTreatment.getFileWithVersionFromNoVersion(wizard.getUpdateSiteToolAdminPath() + "plugins"
						// + File.separator + plugin);
						// if (path2 != null)
						// {
						// String tempPath2 = new String(path2);
						// version = "";
						// while (tempPath2.indexOf("_") != -1)
						// {
						// version = tempPath2.substring(tempPath2.lastIndexOf('_')).concat(version);
						// tempPath2 = tempPath2.substring(0, tempPath2.lastIndexOf('_'));
						//
						// }
						// }
						// else
						// {
						// if (FilesTreatment.isFileExistsWithoutVersion(wizard.getUpdateSiteToolAdminPath() + "plugins" + File.separator
						// + plugin))
						// {
						// version = "_" + FilesTreatment.analyzeString(version, propertyFile);
						// }
						// else
						// {
						// version = "";
						// }
						// }

						String pluginName = FilesTreatment.analyzeString(plugin, propertyFile) + version;
						String pluginPath = UpdateSiteWizard.checkPath(wizard.getUpdateSiteToolAdminPath() + "plugins", true) + pluginName;

						SourceDestination sd = new SourceDestination(pluginName, pluginPath, UpdateSiteWizard.checkPath(wizard
								.getUpdateSitePath()
								+ "plugins", wizard.getProtocolConnex() == UpdateSiteWizard.FILENOSEC));
						FilesTreatment.autoState(sd);

						if (!mapPluginsRequired.containsKey(sd.getName()))
						{
							addRequiredPlugin(treeItem, sd, mapPluginsRequired);
						}
					}
				}
			}
			catch (ParserConfigurationException e)
			{
				CorePlugin.log(this.getClass(), "ParserConfiguration error : " + e.getMessage(), IStatus.ERROR, e);
			}
			catch (SAXException e)
			{
				CorePlugin.log(this.getClass(), "Error while parsing document : " + e.getMessage(), IStatus.ERROR, e);
			}
			catch (IOException e)
			{
				CorePlugin.log(this.getClass(), "File 'plugin.xml' not found in directory " + path, IStatus.WARNING, e);
			}

			// delete plugin.xml temp files
			if (isJarFile && fileToParse.exists())
			{
				fileToParse.delete();
			}
		}

		// delete property temp files
		if (isJarFile && propertyFile != null && propertyFile.exists())
		{
			propertyFile.delete();
		}
	}
}