/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.updatesite;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;
import java.util.jar.JarEntry;
import java.util.jar.JarOutputStream;
import java.util.jar.Manifest;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.ftp.FtpException;
import org.eclipse.ftp.internal.FTPClient;
import org.eclipse.ftp.internal.FTPProxyLocation;
import org.eclipse.ftp.internal.FTPServerLocation;
import org.eclipse.ftp.internal.IFTPClientListener;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.osgi.util.ManifestElement;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IWorkbench;
import org.topcased.bus.core.CorePlugin;
import org.topcased.facilities.dialogs.ITopcasedWizardConstants;

import com.sshtools.j2ssh.SftpClient;
import com.sshtools.j2ssh.SshClient;
import com.sshtools.j2ssh.authentication.AuthenticationProtocolState;
import com.sshtools.j2ssh.authentication.KBIAuthenticationClient;
import com.sshtools.j2ssh.authentication.PasswordAuthenticationClient;
import com.sshtools.j2ssh.authentication.PublicKeyAuthenticationClient;
import com.sshtools.j2ssh.authentication.SshAuthenticationClient;
import com.sshtools.j2ssh.transport.IgnoreHostKeyVerification;
import com.sshtools.j2ssh.transport.publickey.SshPrivateKey;
import com.sshtools.j2ssh.transport.publickey.SshPrivateKeyFile;

/**
 * 
 * The main update Site wizard. It add all necessary pages It also jars or copy the features/plugins to the destination It can create a FTP or SSH connection
 * and give some information about them. It copy the features/plugins jared to the remote FTP or SSH server
 */
public class UpdateSiteWizard extends Wizard implements ITopcasedWizardConstants
{

	private BrowseUpdateSiteWizardPage selectionPage;

	private TreeUpdateSiteWizardPage requiredPage;

	private ConnectUpdateSiteWizardPage connectionPage;

	private CreateSiteUpdateSiteWizardPage siteXMLPage;

	private String updateSiteToolAdminPath;

	private String updateSitePath = "";

	// Map for the Add/Delete repository
	private Map mapAddPluginFeatureUS = new HashMap(); // key=name; value=SourceDestination

	private Map mapPluginUS = new TreeMap(); // key=path; value=nameFeature

	private Map mapFeatureInclude = new TreeMap();

	private Map mapRequiredPlugins = new TreeMap();

	private static int protocolConnex;

	private String host;

	private int port;

	private String username;

	private String password;

	private FTPClient clientFTP;

	private SshClient clientSSH;

	private SftpClient clientSFTP; // used with clientSSH...

	private String error;

	private Vector IOErrorVector;

	/**
	 * FTP - Identifier of connection host protocol
	 */
	public static final int FTP = 0;

	/**
	 * SSH - Identifier of connection host protocol
	 */
	public static final int SSH = 1;

	/**
	 * FILESEC - Identifier of connection host protocol
	 */
	public static final int FILESEC = 3;

	/**
	 * FILENOSEC - Identifier of connection host protocol
	 */
	public static final int FILENOSEC = 2;

	/**
	 * PRIVATE_KEY_PATH - If a SSH connection is used
	 */
	public static final String PRIVATE_KEY_PATH = "privateKeyPath";

	private IPreferenceStore preferenceStore = UpdateSitePlugin.getDefault().getPreferenceStore();

	/**
	 * Constructor initializing the updateSiteToolAdminPath
	 * 
	 */
	public UpdateSiteWizard()
	{
		setWindowTitle("TOPCASED : Update Site");
		setDefaultPageImageDescriptor(UpdateSitePlugin.getImageDescriptor(TOPCASED_LOGO_PATH));
		try
		{
			URI siteURI = new URI(Platform.getInstallLocation().getURL().toString());
			updateSiteToolAdminPath = new File(siteURI).getAbsolutePath();
		}
		catch (URISyntaxException e)
		{
			updateSiteToolAdminPath = Platform.getInstallLocation().getURL().getPath();
		}
		updateSiteToolAdminPath = checkPath(updateSiteToolAdminPath, true);
	}

	private String directory = "";

	/**
	 * 
	 * @param workbench
	 * @param selection
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection)
	{
		updateSiteToolAdminPath = updateSiteToolAdminPath.replace('\\', (File.separator).charAt(0));
		updateSiteToolAdminPath = updateSiteToolAdminPath.replace('/', (File.separator).charAt(0));

		if (updateSiteToolAdminPath.startsWith("\\"))
		{
			updateSiteToolAdminPath = updateSiteToolAdminPath.substring(1);
		}
	}

	/**
	 * This method is aimed to solve path problems independantly of the system. First, the good file separator is added at the end of the path. Secondly, the
	 * function check that all the file separators contained are correct. If it is not the case, they are replaced by the good one.
	 * 
	 * @param pathToCheck The string to evaluate
	 * @param pathNature The string nature : true for a local path, false for a distant path.
	 * @return The new path corrected.
	 */
	public static String checkPath(String pathToCheck, boolean pathNature)
	{
		if (pathNature)
		{
			// check the path according to the plateform and replace character wether it is necessary
			pathToCheck = pathToCheck.replace('/', File.separatorChar);
			pathToCheck = pathToCheck.replace('\\', File.separatorChar);
			// add file separator to the end of string
			if (!pathToCheck.endsWith(File.separator))
			{
				pathToCheck += File.separator;
			}
		}
		else
		{
			pathToCheck = pathToCheck.replace('\\', '/');
			if (!pathToCheck.endsWith("/"))
			{
				pathToCheck += "/";
			}
		}
		return pathToCheck;
	}

	/**
	 * Function executed when the button 'Cancel' is clicked
	 * 
	 * @return boolean true if the wizard is canceled, false otherwise
	 */
	public boolean performCancel()
	{
		disconnect();
		return super.performCancel();
	}

	/**
	 * Function executed when the button 'Finish' is clicked
	 * 
	 * @return boolean true if the wizard is finished, false otherwise
	 */
	public boolean performFinish()
	{
		IOErrorVector = new Vector();

		directory = checkPath(selectionPage.getDirectory(), getProtocolConnex() == FILENOSEC);

		IRunnableWithProgress runnableWithProgress = new IRunnableWithProgress()
		{
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
			{
				if (monitor.isCanceled())
				{
					monitor.done();
				}

				monitor.beginTask("Installation in progress...", 6);

				monitor.subTask("Copy of features...");
				monitor.worked(1);

				jarMapToDestination(mapAddPluginFeatureUS);

				monitor.subTask("Copy of included features...");
				monitor.worked(1);

				jarMapToDestination(mapFeatureInclude);

				monitor.subTask("Copy of plugins...");
				monitor.worked(1);

				jarMapToDestination(mapPluginUS);

				if (!mapRequiredPlugins.isEmpty())
				{
					monitor.subTask("Copy of required plugins...");
					monitor.worked(1);
					jarMapToDestination(mapRequiredPlugins);
				}

				monitor.subTask("Create file site.xml...");
				monitor.worked(1);

				siteXMLPage.getTreatment().createSiteXML();

				monitor.subTask("Disconnect from the server...");
				monitor.worked(1);

				monitor.done();
			}
		};
		ProgressMonitorDialog dialog = new ProgressMonitorDialog(getShell());
		boolean retryTreatment;

		do
		{
			IOErrorVector = new Vector();
			retryTreatment = false;
			try
			{
				dialog.run(true, true, runnableWithProgress);
			}
			catch (InvocationTargetException e)
			{
				CorePlugin.log(e);
			}
			catch (InterruptedException e)
			{
				CorePlugin.log(e);
			}

			if (!IOErrorVector.isEmpty())
			{
				int button;
				Status[] status = new Status[IOErrorVector.size()];
				for (int i = 0; i < IOErrorVector.size(); i++)
				{
					status[i] = new Status(IStatus.ERROR, "org.topcased.service.updatesite", 0, (String) IOErrorVector.get(i), null);
				}
				ErrorDialog errorBox = new ErrorDialog(getShell(), "IO Error", "An error occured during the copy", new MultiStatus(
						"org.topcased.service.updatesite", 0, status, "See details", null), IStatus.ERROR);
				errorBox.open();

				MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_WARNING | SWT.IGNORE | SWT.RETRY | SWT.ABORT);
				msgBox.setText("IO Error");
				msgBox.setMessage("If you continue, maybe some files will be not copied.\n\nDo you want to continue ?");
				button = msgBox.open();
				switch (button)
				{
				case (SWT.ABORT):
					IRunnableWithProgress runnableWithProgress2 = new IRunnableWithProgress()
					{
						public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
						{
							monitor.beginTask("Deleting files...", IProgressMonitor.UNKNOWN);
							FilesTreatment.emptyFilesFromMap(mapAddPluginFeatureUS);
							FilesTreatment.emptyFilesFromMap(mapFeatureInclude);
							FilesTreatment.emptyFilesFromMap(mapPluginUS);
							FilesTreatment.emptyFilesFromMap(mapRequiredPlugins);
							File suppFile = new File(updateSitePath + "site.xml");
							suppFile.delete();
							File tmpFile = new File(updateSitePath + "site.xml.tmp");
							if (tmpFile.exists())
							{
								tmpFile.renameTo(new File(updateSitePath + "site.xml"));
							}
							String pathFile = updateSitePath + "site.xml";
							String path = directory + "site.xml";
							if (protocolConnex == FTP)
							{
								FilesTreatment.emptyFilesFTPFromMap(clientFTP, mapAddPluginFeatureUS, directory + "features/");
								FilesTreatment.emptyFilesFTPFromMap(clientFTP, mapFeatureInclude, directory + "features/");
								FilesTreatment.emptyFilesFTPFromMap(clientFTP, mapPluginUS, directory + "plugins/");
								FilesTreatment.emptyFilesFTPFromMap(clientFTP, mapRequiredPlugins, directory + "plugins/");
								try
								{
									FilesTreatment.copyFileToFTP(pathFile, path, clientFTP);
								}
								catch (FtpException e)
								{
									CorePlugin.log(e);
								}
							}
							if (protocolConnex == SSH)
							{
								FilesTreatment.emptyFilesSSHFromMap(clientSFTP, mapAddPluginFeatureUS, directory + "features/");
								FilesTreatment.emptyFilesSSHFromMap(clientSFTP, mapFeatureInclude, directory + "features/");
								FilesTreatment.emptyFilesSSHFromMap(clientSFTP, mapPluginUS, directory + "plugins/");
								FilesTreatment.emptyFilesSSHFromMap(clientSFTP, mapRequiredPlugins, directory + "plugins/");
								try
								{
									clientSFTP.put(pathFile, path);
								}
								catch (IOException e)
								{
									CorePlugin.log(e);
								}
							}
						}
					};
					ProgressMonitorDialog dialog2 = new ProgressMonitorDialog(getShell());
					try
					{
						dialog2.run(true, true, runnableWithProgress2);
					}
					catch (InvocationTargetException e)
					{
						CorePlugin.log(e);
					}
					catch (InterruptedException e)
					{
						CorePlugin.log(e);
					}
					return false;
				case (SWT.RETRY):
					// Retry to add the sub repository in Map of Update Site
					retryTreatment = true;
					break;
				case (SWT.IGNORE):
					return true;
				}
			}
		}
		while (retryTreatment);

		// Cleaning here because deleteOnExit seems not to work properly.
		String tempDir = checkPath(System.getProperty("java.io.tmpdir"), true);
		FilesTreatment.emptyFile(new File(tempDir + "features"));
		FilesTreatment.emptyFile(new File(tempDir + "plugins"));
		FilesTreatment.emptyFile(new File(tempDir + "site.xml"));

		// Disconnect the protocol (for FTP and SSH)
		disconnect();
		return true;
	}

	/**
	 * Function returning if the button 'Finish' is enabled or not
	 * 
	 * @return boolean true if the button is enable, false otherwise
	 */
	public boolean canFinish()
	{
		return siteXMLPage.finish();
	}

	/**
	 * Add all the necessary pages of the wizard
	 */
	public void addPages()
	{
		connectionPage = new ConnectUpdateSiteWizardPage(this, "Connexion page");
		connectionPage.setTitle("Information for establishing a distant connection");
		connectionPage.setDescription("Enter the required information to activate the connection");
		addPage(connectionPage);

		selectionPage = new BrowseUpdateSiteWizardPage(this, "Browse Page");
		selectionPage.setTitle("Browse Page");
		selectionPage.setDescription("Choose the features to export to the Update Site");
		addPage(selectionPage);

		requiredPage = new TreeUpdateSiteWizardPage(this, "View page");
		requiredPage.setTitle("Features/Plugins to install");
		requiredPage.setDescription("Preview features and plugins necessary to the current export");
		addPage(requiredPage);

		siteXMLPage = new CreateSiteUpdateSiteWizardPage(this, "Editing site.xml");
		siteXMLPage.setTitle("Create and edit the Update Site XML file");
		siteXMLPage.setDescription("Define and affect various properties to the site.xml file");
		addPage(siteXMLPage);
	}

	/**
	 * Return the path of the source of the features
	 * 
	 * @return String the path
	 */
	public String getUpdateSiteToolAdminPath()
	{
		return updateSiteToolAdminPath;
	}
	
	/**
	 * Set the path of the source of the features
	 * 
	 * @param path the path
	 */
	public void setUpdateSiteToolAdminPath(String path)
	{
		updateSiteToolAdminPath = path;
	}

	/**
	 * @return Returns the updateSitePath.
	 */
	public String getUpdateSitePath()
	{
		return updateSitePath;
	}
	
	/**
	 * Returns the absolute path to the directory 'plugins' of the platform source 
	 * 
	 * @return the absolute path to the directory 'plugins' of the update site
	 */
	public IPath getPluginUpdateSitePath()
	{
		return new Path(new File(updateSiteToolAdminPath, "plugins").getAbsolutePath());
	}
	
	/**
	 * Returns the absolute path to the directory 'features' of tje platform source
	 * 
	 * @return the absolute path to the directory 'features' of the update site
	 */
	public IPath getFeatureUpdateSitePath()
	{
		return new Path(new File(updateSiteToolAdminPath, "features").getAbsolutePath());
	}

	/**
	 * @param updateSitePath The updateSitePath to set.
	 */
	public void setUpdateSitePath(String path)
	{
		updateSitePath = path;
	}

	/**
	 * @return Returns the requiredPage.
	 */
	public TreeUpdateSiteWizardPage getRequiredPage()
	{
		return requiredPage;
	}

	/**
	 * @return Returns the connectionPage.
	 */
	public ConnectUpdateSiteWizardPage getConnectionPage()
	{
		return connectionPage;
	}
	
	/**
	 * @return Returns the selectionPage.
	 */
	public BrowseUpdateSiteWizardPage getSelectionPage()
	{
		return selectionPage;
	}
	
	/**
	 * 
	 * @return Returns the wizard page aimed to build the site.xml file.
	 */
	public CreateSiteUpdateSiteWizardPage getCreateSitePage()
	{
		return siteXMLPage;
	}

	/**
	 * @return Returns the mapAddPluginFeatureUS.
	 */
	public Map getMapAddPluginFeatureUS()
	{
		return mapAddPluginFeatureUS;
	}

	/**
	 * @return Returns the mapFeatureInclude.
	 */
	public Map getMapFeatureInclude()
	{
		return mapFeatureInclude;
	}

	/**
	 * @return Returns the mapPluginUS.
	 */
	public Map getMapPluginUS()
	{
		return mapPluginUS;
	}

	/**
	 * @return Returns the mapRequiredPlugins.
	 */
	public Map getMapRequiredPlugins()
	{
		return mapRequiredPlugins;
	}

	/**
	 * @return Returns the host.
	 */
	public String getHost()
	{
		return host;
	}

	/**
	 * @param host The host to set.
	 */
	public void setHost(String host)
	{
		this.host = host;
	}

	/**
	 * @return Returns the port.
	 */
	public int getPort()
	{
		return port;
	}

	/**
	 * @param port The port to set.
	 */
	public void setPort(int port)
	{
		this.port = port;
	}

	/**
	 * @return Returns the password.
	 */
	public String getPassword()
	{
		return password;
	}

	/**
	 * @param password The password to set.
	 */
	public void setPassword(String password)
	{
		this.password = password;
	}

	/**
	 * @return Returns the protocolConnex.
	 */
	public int getProtocolConnex()
	{
		return protocolConnex;
	}

	/**
	 * @param protocolConnex The protocolConnex to set.
	 */
	public void setProtocolConnex(int protocol)
	{
		protocolConnex = protocol;
	}

	/**
	 * @return Returns the username.
	 */
	public String getUsername()
	{
		return username;
	}

	/**
	 * @param username The username to set.
	 */
	public void setUsername(String user)
	{
		username = user;
	}

	public FTPClient getClientFTP()
	{
		return clientFTP;
	}

	/**
	 * @return Returns the clientSFTP.
	 */
	public SftpClient getClientSFTP()
	{
		return clientSFTP;
	}

	/**
	 * @return Returns the clientSSH.
	 */
	public SshClient getClientSSH()
	{
		return clientSSH;
	}

	public String getDirectory()
	{
		return directory;
	}

	/**
	 * @return Returns the iOErrorVector.
	 */
	public Vector getIOErrorVector()
	{
		return IOErrorVector;
	}

	public void createConnectionFTP()
	{
		/*******************************************************************************************************************************************************
		 * FTP Connexion
		 ******************************************************************************************************************************************************/

		FTPServerLocation ftpServer;
		FTPProxyLocation ftpProxy;

		URL URLServer = null;
		try
		{
			URLServer = new URL("FTP", host, port, "");
		}
		catch (MalformedURLException e1)
		{
			CorePlugin.log(e1);
		}
		ftpServer = new FTPServerLocation(URLServer);// host, port, username, password, false);
		ftpServer.setAuthentication(username, password);

		ftpProxy = null;

		clientFTP = new FTPClient(ftpServer, ftpProxy, new IFTPClientListener()
		{
			public void responseReceived(int responseCode, String responseText)
			{
			}

			public void requestSent(String command, String argument)
			{
			}
		});

		IRunnableWithProgress runnableWithProgress = new IRunnableWithProgress()
		{
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
			{
				error = "";
				try
				{
					clientFTP.open(monitor);
				}
				catch (FtpException e)
				{
					CorePlugin.log(e);
					error = e.getMessage();
				}

			}
		};
		ProgressMonitorDialog dialog = new ProgressMonitorDialog(getShell());
		try
		{
			dialog.run(true, true, runnableWithProgress);
		}
		catch (InvocationTargetException e)
		{
			CorePlugin.log(e);
			error = e.getMessage();
		}
		catch (InterruptedException e)
		{
			CorePlugin.log(e);
			error = e.getMessage();
		}

		if (clientFTP.isOpen())
		{
			clientFTP.setDataTransferMode(FTPClient.PASSIVE_DATA_TRANSFER, true);
			clientFTP.setTimeout(3600);
			MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_INFORMATION | SWT.OK);
			msgBox.setText("Connection information");
			msgBox.setMessage("Connection successfully completed\nHost : " + ftpServer.getHostname() + "\nUsername : "
					+ ftpServer.getUsername());
			msgBox.open();
		}
		else
		{
			if (!error.equals(""))
			{
				MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_ERROR | SWT.OK);
				msgBox.setText("Connection error");
				msgBox.setMessage("Error during the connection : " + error + "\nSee Console information");
				msgBox.open();
			}
		}
	}

	/**
	 * This method authenticates a client according to the choosen method (public key, interactive keyboard, password)
	 * 
	 * @param client represents the current method authentication tried by a client
	 * @return returns true is the user is authenticated, false if the authentication method has failed.
	 */
	public boolean userAuthenticationSHH(SshAuthenticationClient client)
	{
		try
		{
			int result = clientSSH.authenticate(client);

			if (result == AuthenticationProtocolState.COMPLETE)
			{
				MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_INFORMATION | SWT.OK);
				msgBox.setText("SSH Authentication Information");
				msgBox.setMessage("Authentication completed for user " + username);
				msgBox.open();
				clientSFTP = clientSSH.openSftpClient();
				return true;
			}
		}
		catch (IOException e1)
		{
			MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_ERROR | SWT.OK);
			msgBox.setText("SSH Authentication Error");
			msgBox.setMessage("Error during the connection : " + e1.getMessage() + "\nSee Console information");
			msgBox.open();
			CorePlugin.log(e1);
		}
		return false;
	}

	/**
	 * This method establishes a SSH connection in two major steps : - key exchange between the client and the server - authentication of a user
	 * 
	 * @return returns true if the connection is established for a given user, false if the connection failed
	 */
	public boolean createConnectionSSH()
	{
		/*******************************************************************************************************************************************************
		 * SSH Connexion
		 ******************************************************************************************************************************************************/
		boolean authenticated = false;
		clientSSH = new SshClient();

		try
		{
			clientSSH.connect(host, port, new IgnoreHostKeyVerification());
			List authenticationMethods = clientSSH.getAvailableAuthMethods(username);
			String privateKeyPath = preferenceStore.getString(PRIVATE_KEY_PATH);

			// First method : public key
			if (authenticationMethods.contains("publickey") && !privateKeyPath.equals(""))
			{
				CorePlugin.log(this.getClass(), "[SSH] Trying to authenticate " + username + " with public key method", IStatus.INFO);
				PublicKeyAuthenticationClient pk = new PublicKeyAuthenticationClient();
				pk.setUsername(username);
				// Open up the private key file

				try
				{
					SshPrivateKeyFile file = SshPrivateKeyFile.parse(new File(privateKeyPath));
					// Get the key
					SshPrivateKey key = file.toPrivateKey(null);
					pk.setKey(key);
					// launch the authentication
					authenticated = userAuthenticationSHH(pk);
				}
				catch (FileNotFoundException fnfe)
				{
					CorePlugin.log(this.getClass(), "[SSH] Your private key has not been found in " + privateKeyPath, IStatus.ERROR);
					CorePlugin.log(this.getClass(),
							"[SSH] Check or change your private key path in Window > Preferences > Topcased > SSH Connection",
							IStatus.WARNING);
				}
				catch (IOException ioe)
				{
					CorePlugin.log(this.getClass(), "[SSH] IO error while reading your private key", IStatus.ERROR);
				}

				// Logs for console
				if (authenticated)
				{
					CorePlugin.log(this.getClass(), "[SSH] User " + username + " successfully authenticated by public key", IStatus.OK);
				}
				else
				{
					CorePlugin.log(this.getClass(), "[SSH] Authentication by public key has failed for user " + username, IStatus.ERROR);
				}
			}

			// Second method : keyboard interactive authentication (using a Dialog for the password)
			if (authenticationMethods.contains("keyboard-interactive") && !authenticated)
			{
				CorePlugin.log(this.getClass(), "[SSH] Trying to authenticate " + username + " with interactive keyboard method",
						IStatus.INFO);
				KBIAuthenticationClient kbi = new KBIAuthenticationClient();
				kbi.setUsername(username);
				// kbi.setKBIRequestHandler(new UpdateSiteKBIRequestHandlerDialog(getShell())); for a graphical use !!!
				kbi.setKBIRequestHandler(new UpdateSiteKBIRequestHandler(this));
				// Launch the keyboard authentication + logs in console
				if (authenticated = userAuthenticationSHH(kbi))
				{
					CorePlugin.log(this.getClass(), "[SSH] User " + username + " successfully authenticated by interactive keyboard",
							IStatus.OK);
				}
				else
				{
					CorePlugin.log(this.getClass(), "[SSH] Authentication by keyboard interactive has failed for user " + username,
							IStatus.ERROR);
				}
			}

			// Third and last method : password authentication
			if (authenticationMethods.contains("password") && !authenticated)
			{
				CorePlugin.log(this.getClass(), "[SSH] Trying to authenticate " + username + " with password method", IStatus.INFO);
				PasswordAuthenticationClient pwd = new PasswordAuthenticationClient();
				pwd.setUsername(username);
				pwd.setPassword(password);
				// Launch the password authentication + logs in console
				if (authenticated = userAuthenticationSHH(pwd))
				{
					CorePlugin.log(this.getClass(), "[SSH] User " + username + " successfully authenticated by password", IStatus.OK);
				}
				else
				{
					CorePlugin.log(this.getClass(), "[SSH] Authentication by password has failed for user " + username, IStatus.ERROR);
				}
			}

			if (!authenticated)
			{
				// At this level, the 3 authentication methods have failed for this user : impossible tio authenticate this user
				CorePlugin.log(this.getClass(),
						"[SSH] All the available SSH authentication methods have failed : impossible to authenticate user " + username,
						IStatus.ERROR);
				MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_ERROR | SWT.OK);
				msgBox.setText("SSH Authentication Error");
				msgBox.setMessage("Authentification failure for user " + username);
				msgBox.open();
			}
		}
		catch (IOException e)
		{
			MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_ERROR | SWT.OK);
			msgBox.setText("Connection error");
			msgBox.setMessage("Error during the SSH connection : " + e.getMessage() + "\nSee Console for more information");
			msgBox.open();
			CorePlugin.log(this.getClass(), "[SSH] IO error while establishing the SSH connection", IStatus.ERROR);
		}
		return authenticated;
	}

	/**
	 * This method disconnect a client form a remote FTP or SSH server. First, the connection state is checked with the method isConnected().
	 */
	public void disconnect()
	{
		if (isConnected())
		{
			switch (protocolConnex)
			{
			case (FTP):
				try
				{
					clientFTP.close(null);
				}
				catch (FtpException e)
				{
					CorePlugin.log(e);
				}
				break;
			case (SSH):
				clientSSH.disconnect();
				CorePlugin.log(this.getClass(), "[SSH] Remote session closed for user " + username, IStatus.OK);
			default:
				break;
			}
		}
	}

	public boolean isConnected()
	{
		boolean ret;

		switch (protocolConnex)
		{
		case (FTP):
			if (clientFTP != null)
			{
				ret = clientFTP.isOpen();
			}
			else
			{
				ret = false;
			}
			break;
		case (SSH):
			if (clientSSH != null)
			{
				ret = clientSSH.isConnected() && clientSSH.isAuthenticated();
			}
			else
			{
				ret = false;
			}
			break;
		default:
			ret = false;
			break;
		}
		return ret;
	}

	/**
	 * Jar the file contained in the map
	 * 
	 * @param map contains files which will be zipped
	 */
	public void jarMapToDestination(final Map map)
	{
		boolean isJarFile = false;

		Iterator it = map.keySet().iterator();
		while (it.hasNext())
		{
			String key = (String) it.next();
			if (map.get(key) instanceof SourceDestination)
			{
				SourceDestination sd = (SourceDestination) map.get(key);

				if (sd.getState() != SourceDestination.ALREADY_EXIST && sd.getState() != SourceDestination.MISSING)
				{
					isJarFile = false;
					if (sd.getSourceFile().isFile())
					{
						isJarFile = true;
					}

					if (!isJarFile)
					{
						if (sd.getSource() != null && !"".equals(sd.getSource()))
						{
							File pathFile = new File(sd.getSource());
							File[] files = pathFile.listFiles();

							try
							{
								FileOutputStream outputStream = new FileOutputStream(sd.getDestination() + sd.getName() + ".jar");
								JarOutputStream output = new JarOutputStream(outputStream);
								zipFile(files, output, "");
								output.close();
							}
							catch (FileNotFoundException e)
							{
								CorePlugin.log(this.getClass(), "File not found : " + e.getMessage(), IStatus.ERROR);
							}
							catch (IOException e)
							{
								CorePlugin.log(this.getClass(), "IO Error. " + e.getMessage(), IStatus.ERROR);
								IOErrorVector.add(e.getMessage());
							}
							CorePlugin.log(this.getClass(), "Jar built in " + sd.getDestination() + sd.getName() + ".jar", IStatus.OK);
						}
						else
						{
							CorePlugin.log(this.getClass(), "File not found. " + sd.getSource() + " It does not exist.", IStatus.ERROR);
						}
					}
					else
					{
						if (sd.getSource() != null && !sd.getSource().equals(""))
						{
							try
							{
								File pathFile = new File(sd.getSource());
								FileOutputStream outputFile = new FileOutputStream(sd.getDestination() + sd.getName().concat(".jar"));
								FileInputStream inputFile = new FileInputStream(pathFile);
								// Copy
								byte[] buffer = new byte[1024];
								int length;
								while ((length = inputFile.read(buffer)) != -1)
								{
									outputFile.write(buffer, 0, length);
								}
								inputFile.close();
								outputFile.close();
								CorePlugin.log(this.getClass(), "Copy built to : " + sd.getDestination() + sd.getName().concat(".jar"), IStatus.INFO);
							}
							catch (FileNotFoundException e)
							{
								CorePlugin.log(this.getClass(), "Jar file not found : " + e.getMessage(), IStatus.ERROR);
							}
							catch (IOException e)
							{
								CorePlugin.log(this.getClass(), "IO Error. " + e.getMessage(), IStatus.ERROR);
								IOErrorVector.add(e.getMessage());
							}
						}
					}

					switch (protocolConnex)
					{
					case (FTP):
						if (clientFTP != null)
						{
							final String path;
							String pathFile = sd.getDestination() + sd.getName();
							String name = sd.getName();

							if (!name.endsWith(".jar"))
							{
								name = name + ".jar";
								pathFile = pathFile + ".jar";
							}
							else
							{
								pathFile = UpdateSiteWizard.checkPath(pathFile, true);
							}

							if (sd.getDestination().indexOf("plugins") != -1)
							{
								path = checkPath(directory + "plugins", getProtocolConnex() == FILENOSEC) + name;
							}
							else
							{
								path = checkPath(directory + "features", getProtocolConnex() == FILENOSEC) + name;
							}

							try
							{
								FilesTreatment.copyFileToFTP(pathFile, path, clientFTP);
							}
							catch (FtpException e)
							{
								CorePlugin.log(this.getClass(), "FTP Exception : " + e.getMessage(), IStatus.ERROR);
								IOErrorVector.add(e.getMessage());
							}
							CorePlugin.log(this.getClass(), "Copy built to FTP server : " + path, IStatus.INFO);
						}
						break;
					case (SSH):
						if (clientSSH != null)
						{
							final String path;
							String pathFile = sd.getDestination() + sd.getName();
							String name = sd.getName();

							if (!name.endsWith(".jar"))
							{
								name = name + ".jar";
								pathFile = pathFile + ".jar";
							}
							else
							{
								checkPath(pathFile, true);
							}

							if (sd.getDestination().indexOf("plugins") != -1)
							{
								path = checkPath(directory + "plugins", getProtocolConnex() == FILENOSEC);
							}
							else
							{
								path = checkPath(directory + "features", getProtocolConnex() == FILENOSEC);
							}

							try
							{
								clientSFTP.put(pathFile, path);
								CorePlugin.log(this.getClass(), "Copy built to SSH server : " + path + name, IStatus.INFO);
							}
							catch (IOException e)
							{
								CorePlugin.log(e);
								IOErrorVector.add(e.getMessage());
							}
						}
						break;
					default:
						break;
					}
				}
			}
		}
	}

	/**
	 * Zip Files
	 * 
	 * @param files : all files to zip
	 * @param output : destination jar
	 * @param path : path in the jar
	 * @throws IOException
	 */
	public void zipFile(File[] files, JarOutputStream output, String path) throws IOException
	{
		try
		{
			for (int i = 0; i < files.length; i++)
			{
				File file = files[i];
				JarEntry entry = new JarEntry(path + file.getName());

				if (file.isDirectory())
				{
					zipFile(file.listFiles(), output, path + file.getName() + "/");
				}
				else
				{
					FileInputStream input = new FileInputStream(file);
					output.putNextEntry(entry);
					// Copy
					byte[] buffer = new byte[1024];
					int length;
					while ((length = input.read(buffer)) != -1)
					{
						output.write(buffer, 0, length);
					}
					input.close();
				}
			}
		}
		catch (NullPointerException e)
		{
			CorePlugin.log(this.getClass(), "Path error : " + path, IStatus.ERROR, e);
		}
	}

	/**
	 * Reads a manifest file and returns an array containing the dependencies.
	 * 
	 * @param filename the manifest filename
	 * @return an String array containing the dependencies
	 * @throws IOException if an error occur while reading the file
	 */
	public static String[] readManifestFile(String filename)
	{
		Vector result = new Vector();
		try
		{
			Manifest manifest = new Manifest(new FileInputStream(filename));
			String valueAttribute = manifest.getMainAttributes().getValue("Require-Bundle");
			ManifestElement[] dependenciesList = ManifestElement.parseHeader("Require-Bundle", valueAttribute);
			if (dependenciesList != null)
			{
				for (int i = 0; i < dependenciesList.length; i++)
				{
					result.add(dependenciesList[i].getValue());
				}
			}
		}
		catch (Exception e)
		{
			CorePlugin.log(UpdateSiteWizard.class, "Error reading MANIFEST file in JAR : " + filename, IStatus.ERROR, e);
		}

		return (String[]) result.toArray(new String[result.size()]);
	}
}
