package org.topcased.service.updatesite.preferences;

import java.io.File;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.topcased.service.updatesite.UpdateSitePlugin;

public class SSHConnectionPreferencePage extends PreferencePage implements IWorkbenchPreferencePage {

	private Text privateKeyPathText;

	public static final String PRIVATE_KEY_PATH = "privateKeyPath";

	private IPreferenceStore preferenceStore = UpdateSitePlugin.getDefault().getPreferenceStore();

	/**
	 * Constructor 
	 */
	public SSHConnectionPreferencePage() {
		setDescription("SSH connection preference page");
		setPreferenceStore(preferenceStore);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		// Layout for the main composite
		final GridLayout mainLayout = new GridLayout();
		mainLayout.numColumns = 1;
		mainLayout.marginHeight = 0;
		mainLayout.marginWidth = 0;

		// Grid layout for the text
		final GridData textGridData = new GridData(SWT.FILL, SWT.NONE, true, false);
		textGridData.widthHint = 150;

		final Composite mainComposite = new Composite(parent, SWT.NONE);
		mainComposite.setLayout(mainLayout);
		mainComposite.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));

		final Group group = new Group(mainComposite, SWT.SHADOW_OUT);
		group.setText("SSH private key path");
		group.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		group.setLayout(new GridLayout(2, false));

		privateKeyPathText = new Text(group, SWT.BORDER);
		privateKeyPathText.setText(preferenceStore.getString(PRIVATE_KEY_PATH));
		privateKeyPathText.setLayoutData(textGridData);

		Button browseButton = new Button(group, SWT.NONE);
		browseButton.setText("Browse...");
		browseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleBrowseTemplateDirectory(privateKeyPathText);
			}
		});

		return mainComposite;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

	/**
	 * @param text
	 */
	private void handleBrowseTemplateDirectory(Text text) {
		FileDialog dialogBrowse = new FileDialog(getShell());
		String dialogOpen = dialogBrowse.open();
		if (dialogOpen != null) {
			// Takes the path of the repository
			String repositoryPath = dialogBrowse.getFilterPath() + File.separator + dialogBrowse.getFileName();
			text.setText(repositoryPath);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performOk()
	 */
	public boolean performOk() {
		preferenceStore.setValue(PRIVATE_KEY_PATH, privateKeyPathText.getText());
		return super.performOk();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 */
	public void performDefaults() {
		privateKeyPathText.setText(preferenceStore.getString(PRIVATE_KEY_PATH));
		super.performDefaults();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performApply()
	 */
	public void performApply() {
		preferenceStore.setValue(PRIVATE_KEY_PATH, privateKeyPathText.getText());
		super.performApply();
	}
}
