/********************************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
********************************************************************************************/ 
package org.topcased.service.updatesiteproject;

import java.io.File;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

/**
 * 
 * This WizardPage browse the updateSite in order to add features in a target directory
 * It gives the possibility to copy/unjar the selected features.
 */
public class BrowseUpdateSiteProjectPage extends WizardPage
{
    private Table tableUS;

    private Table tableTD;
    
    private Text updateSiteBrowseText;
    
    private Text destinationBrowseText;
    
	private Button updateSiteBrowseButton;
	
	private Button addButton;
	
	private Button deleteButton;
	
    private Button destinationBrowseButton;
    
	private UpdateSiteProjectWizard wizard;

    private BrowseUpdateSiteProjectTreatment treatment;

	 /**
     * Constructor
     * 
     * @param wizard the parent wizard
     * @param pageName the name of the page
     */
    public BrowseUpdateSiteProjectPage(UpdateSiteProjectWizard wizard, String pageName)
    {
        super(pageName);
        this.wizard = wizard;
        treatment = new BrowseUpdateSiteProjectTreatment(wizard);
    }

    /**
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent)
    {
    	/*
    	 * Definition of some GridData 
    	 */
        // GridData for Update Site table and Target Directory table
        GridData gridDataForTable = new GridData(GridData.FILL_BOTH);
        gridDataForTable.heightHint = 195;
        
        // GridData for components who need to be filled and grabed horizontally
        GridData horizontalFillAndGrabGridData = new GridData(GridData.FILL_HORIZONTAL);
        
        // GridData for component who need to be filled and grabbed horizontally and vertically + the vertical align starts at the beginning of the area.
        GridData fullFillAndGrabGridData = new GridData(GridData.FILL_BOTH);
        
        // GridData for bottom information button 
        GridData gridDataButtonInfo = new GridData(GridData.FILL_HORIZONTAL);
        gridDataButtonInfo.horizontalSpan = 3;
    	
    	// Main composite
    	final Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout(1, true));
        setControl(container);
        
        // Update composite (main area)
        final Composite updateComposite = new Composite(container, SWT.NONE);
        updateComposite.setLayoutData(fullFillAndGrabGridData);
        updateComposite.setLayout(new GridLayout(3, false));
        
        final Label updateSiteLabel = new Label(updateComposite, SWT.NONE);
        updateSiteLabel.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        updateSiteLabel.setAlignment(SWT.LEFT);
        updateSiteLabel.setText("Update Site :");

        final Label emptyLabel = new Label(updateComposite, SWT.NONE);
        emptyLabel.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));

        final Label targetDirectoryLabel = new Label(updateComposite, SWT.NONE);
        targetDirectoryLabel.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        targetDirectoryLabel.setAlignment(SWT.LEFT);
        targetDirectoryLabel.setText("Target directory : ");        
                
        // update site composite (left) 
        final Composite updateSiteComposite = new Composite(updateComposite, SWT.NONE);
        updateSiteComposite.setLayoutData(fullFillAndGrabGridData);
        updateSiteComposite.setLayout(new GridLayout());
        
        // Add / Delete composite (middle)
        final Composite addDeleteComposite = new Composite(updateComposite, SWT.NONE);
        addDeleteComposite.setLayoutData(new GridData());
        addDeleteComposite.setLayout(new GridLayout());   
        
        // target directory composite (right)
        final Composite targetDirectoryComposite = new Composite(updateComposite, SWT.NONE);
        targetDirectoryComposite.setLayoutData(fullFillAndGrabGridData);
        targetDirectoryComposite.setLayout(new GridLayout());
        
        // browse UpdateSite (left)
        final Composite browseUpdateSiteComposite = new Composite(updateSiteComposite, SWT.NONE);
        browseUpdateSiteComposite.setLayoutData(horizontalFillAndGrabGridData);
        browseUpdateSiteComposite.setLayout(new GridLayout(2, false));
        
        // browse Target Directory (right)
        final Composite browseTargetDirectoryComposite = new Composite(targetDirectoryComposite, SWT.NONE);
        browseTargetDirectoryComposite.setLayoutData(horizontalFillAndGrabGridData);
        browseTargetDirectoryComposite.setLayout(new GridLayout(2, false));
            
        // Bottom composite for information button 
        final Composite downComposite = new Composite(container, SWT.NONE);
        downComposite.setLayoutData(gridDataButtonInfo);
        downComposite.setLayout(new GridLayout(1, false));

        // field and button for the update site
        updateSiteBrowseText = new Text(browseUpdateSiteComposite, SWT.BORDER);
        updateSiteBrowseText.setLayoutData(horizontalFillAndGrabGridData);
        updateSiteBrowseText.setText(wizard.getProjectPath());
        updateSiteBrowseButton = new Button(browseUpdateSiteComposite, SWT.NONE);
        updateSiteBrowseButton.setLayoutData(new GridData());
        updateSiteBrowseButton.setText("Browse...");
        
        // field and button for the target directory
        destinationBrowseText = new Text(browseTargetDirectoryComposite, SWT.BORDER);
        destinationBrowseText.setLayoutData(horizontalFillAndGrabGridData);
        destinationBrowseText.setText(wizard.getProjectPath());
        destinationBrowseButton = new Button(browseTargetDirectoryComposite, SWT.NONE);
        destinationBrowseButton.setLayoutData(new GridData());
        destinationBrowseButton.setText("Browse...");
        
        // left table for UpdateSite : Content of repository of 'updateSitePath'.
        tableUS = new Table(updateSiteComposite, SWT.BORDER);
        tableUS.setLayoutData(gridDataForTable);
        tableUS.setLinesVisible(true);
        tableUS.setHeaderVisible(true);
        tableUS.getVerticalBar().setEnabled(true);

        // right table for target directory 
        tableTD = new Table(targetDirectoryComposite, SWT.BORDER);
        tableTD.setLayoutData(gridDataForTable);
        tableTD.setLinesVisible(true);
        tableTD.setHeaderVisible(true);
        tableTD.getVerticalBar().setEnabled(true);
        
        // Left table for update site
        final TableColumn tableColumnRepositoryUS = new TableColumn(tableUS, SWT.NONE);
        tableColumnRepositoryUS.setWidth(160);
        tableColumnRepositoryUS.setText("Content");

        // Right table for target directory 
        final TableColumn tableColumnTargetDirectory = new TableColumn(tableTD, SWT.NONE);
        tableColumnTargetDirectory.setWidth(160);
        tableColumnTargetDirectory.setText("Project");

        // Add button
        addButton = new Button(addDeleteComposite, SWT.NONE);
        addButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        addButton.setText("Add");

        // Delete button
        deleteButton = new Button(addDeleteComposite, SWT.NONE);
        deleteButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        deleteButton.setText("Delete");
        
        // Information button
        final Button informationButton = new Button(downComposite, SWT.NONE);
        informationButton.setText("Feature information");
        informationButton.setToolTipText("Get more information about the selected feature.");
        
        // Button : Add feature in the target directory
        addButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                TableItem[] itemsSelected = tableUS.getSelection();
                for (int indice = 0; indice < itemsSelected.length; indice++)
                {
                    TableItem itemSelected = itemsSelected[indice];
                    String nameSelected = itemSelected.getText();
                    treatment.addFeatureOnProject(nameSelected, tableTD);
                }
                
                if (!destinationBrowseText.getText().equals("") && !updateSiteBrowseText.getText().equals("")
                        && !wizard.getMapPA().isEmpty())
                {
                    setPageComplete(true);
                }
                else
                {
                    setPageComplete(false);
                }
            }
        });

        // Button : Delete feature from the project
        deleteButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                TableItem[] itemsSelected = tableTD.getSelection();
                for (int indice = 0; indice < itemsSelected.length; indice++)
                {
                    treatment.deleteFeatureFromProject(itemsSelected[indice].getText(), tableTD);
                }
                
                if (!destinationBrowseText.getText().equals("") && !updateSiteBrowseText.getText().equals("")
                        && !wizard.getMapPA().isEmpty())
                {
                    setPageComplete(true);
                }
                else
                {
                    setPageComplete(false);
                }
            }
        });

        // Button : Browse the target directory
        destinationBrowseButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                treatment.browseProjectFile(destinationBrowseText);
                if (!destinationBrowseText.getText().equals("") && !updateSiteBrowseText.getText().equals("")
                        && !wizard.getMapPA().isEmpty())
                {
                    setPageComplete(true);
                }
                else
                {
                    setPageComplete(false);
                }
            }
        });
        
        // Button : Browse the Update Site directory (JARFile)
        updateSiteBrowseButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                switch (wizard.getProtocolConnexion())
                {
                    case (UpdateSiteProjectWizard.FTP):
                        treatment.browseUpdateSiteFTP(updateSiteBrowseText, tableUS);
                        break;
                    case (UpdateSiteProjectWizard.SSH):
                        treatment.browseUpdateSiteSSH(updateSiteBrowseText, tableUS);
                        break;
                    default:
                        treatment.browseUpdateSiteWithJarFile(updateSiteBrowseText, tableUS);
                        break;
                }
                emptyTargetDirectory();
                addButton.setEnabled(true);
                deleteButton.setEnabled(true);
            }
        });
        
        // Button : get information about the selected feature
        informationButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                TableItem[] itemsSelected = tableUS.getSelection();
                for (int indice = 0; indice < itemsSelected.length; indice++)
                {
                    TableItem itemSelected = itemsSelected[indice];
                    String selectedName = itemSelected.getText();
                    String path = treatment.searchSource(selectedName, true);
      			  	if (wizard.getProtocolConnexion() != UpdateSiteProjectWizard.FILE) 
      			  	{
      			  		// Local copy for SSH and FTP access
      			  		wizard.copyTemporaryFeature(selectedName);
      			  		treatment.getFeatureInformation(path, wizard.getUpdateSitePath() + selectedName);
      			  		if (new File(wizard.getUpdateSitePath() + selectedName).exists())
      			  		{
      			  			new File(wizard.getUpdateSitePath() + selectedName).delete();
      			  		}
      			  	}
      			  	else
      			  	{
      			  		treatment.getFeatureInformation(path, path);
      			  	}
                }
            }
        });
        setPageComplete(false);
    }
	
    /**
     * @return the directory selected for the project location
     */
    public String getDirectory()
    {
        return UpdateSiteProjectWizard.checkPath(updateSiteBrowseText.getText(),wizard.getProtocolConnexion() == UpdateSiteProjectWizard.FILE);
    }

    /**
     * @return boolean, true if the page is completed, false otherwise
     */
    public boolean finish()
    {
        return isPageComplete();
    }

    /**
     * @see org.eclipse.jface.wizard.WizardPage#canFlipToNextPage()
     */
    public boolean canFlipToNextPage()
    {
        wizard.setProtocolConnexion(wizard.getConnectPage().getProtocol());
        if (!wizard.isConnected() && wizard.getConnectPage().getProtocol() != UpdateSiteProjectWizard.FILE)
        {
        	wizard.getConnectPage().updateConnectionInformations();
            switch (wizard.getProtocolConnexion())
            {
                case (UpdateSiteProjectWizard.FTP):
                    wizard.createConnectionFTP();
	                updateSiteBrowseText.setText(UpdateSiteProjectWizard.checkPath(wizard.getConnectPage().getUSPath().getText(),false));;
	                updateSiteBrowseText.setEnabled(false);
	        		updateSiteBrowseButton.setEnabled(false);
	            	treatment.browseUpdateSiteFTP(updateSiteBrowseText, tableUS);
                    break;
                case (UpdateSiteProjectWizard.SSH):

                	updateSiteBrowseText.setEnabled(false);
            		updateSiteBrowseButton.setEnabled(false);
                	if (wizard.createConnectionSSH())
                    {
                    	updateSiteBrowseText.setText(wizard.getConnectPage().getUSPath().getText());
                    	addButton.setEnabled(true);
                    	deleteButton.setEnabled(true);
                    	treatment.browseUpdateSiteSSH(updateSiteBrowseText, tableUS);
                    }
                    else
                    {
                      	destinationBrowseButton.setEnabled(false);
                		destinationBrowseText.setEnabled(false);
                    	updateSiteBrowseText.setText("");
                    	addButton.setEnabled(false);
                    	deleteButton.setEnabled(false);
                    }
                    break;
            }
            emptyTargetDirectory();
            wizard.getConnectPage().getTreatment().updateConnectionProfiles();
            //setPageComplete(false);
        }
        //return isPageComplete();
        return finish();
    }
    
    public void update()
    {
    	if (wizard.getConnectPage().getProtocol() == UpdateSiteProjectWizard.FILE)
    	{
    		emptyTargetDirectory();
    		updateSiteBrowseText.setEnabled(true);
        	updateSiteBrowseButton.setEnabled(true);
    		addButton.setEnabled(false);
			deleteButton.setEnabled(false);
    		updateSiteBrowseText.setText("");
			tableUS.removeAll();
			treatment.getMapUS().clear();
        	setPageComplete(false);
    	}
    }
    
    /**
     * This private method empties the information concerning the target directory : text field, table and map. 
     */
    private void emptyTargetDirectory()
    {
    	tableTD.removeAll();
    	treatment.getMapTD().clear();
    }
}
