/*******************************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS)
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
********************************************************************************************/ 
package org.topcased.service.updatesiteproject;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ftp.FtpException;
import org.eclipse.ftp.IDirectoryEntry;
import org.eclipse.ftp.internal.FTPClient;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.topcased.bus.core.CorePlugin;

import com.sshtools.j2ssh.SftpClient;
import com.sshtools.j2ssh.sftp.SftpFile;

/**
 * This class includes all the treatments necessary to the BrowseUpdateSiteProjectPage
 */
public class BrowseUpdateSiteProjectTreatment
{
    private UpdateSiteProjectWizard wizard;

    private Map mapUS = new TreeMap(); // key=name; value=SourceDestination
    
    private Map mapTD = new TreeMap(); // key=name; value=SourceDestination

    private String error;
	
	private Vector IOFTPSSHErrorVector;

    /**
     * Constructor
     * @param wizard the parent wizard
     */
    public BrowseUpdateSiteProjectTreatment(UpdateSiteProjectWizard wizard)
    {
        this.wizard = wizard;
        mapUS = wizard.getMapUS();
        mapTD = wizard.getMapPA();
    }
    
    public Map getMapUS()
    {
    	return mapUS;
    }
    
    public Map getMapTD()
    {
    	return mapTD;
    }
    
    /**
     * Add the Jar feature in the target directory table 
     * 
     * @param nameSelected
     */
    public void addFeatureOnProject(String nameSelected, Table table)
    {
        String key = nameSelected;
        if (mapUS.containsKey(key))
        {
            if (mapUS.get(key) instanceof SourceDestination)
            {
                SourceDestination repositorySD = (SourceDestination) mapUS.get(key);
                if (!mapTD.containsKey(key))
                {
                    // Add the repository selected in the Map of Project to
                    SourceDestination addSourceDestination = new SourceDestination(repositorySD.getName(),
                            repositorySD.getSource(), wizard.getProjectPath() + "features" + File.separator);
                    mapTD.put(nameSelected, addSourceDestination);
                }
            }
        }
        // Display
        table.removeAll();
        displayMap(mapTD, table);
    }

    /**
     * Delete Jar feature from the table table_PA
     * 
     * @param nameSelected
     */
    public void deleteFeatureFromProject(String nameSelected, Table table_PA)
    {
        mapTD.remove(nameSelected);
        // Display
        table_PA.removeAll();
        displayMap(mapTD, table_PA);
    }

    /**
     * Button Action : Browse Directory for the project
     * 
     * @param text : location where the path will be read and write
     */
    public void browseProjectFile(Text text)
    {
        DirectoryDialog dialogBrowse = new DirectoryDialog(wizard.getShell());
       	dialogBrowse.setFilterPath(wizard.getProjectPath());

        // Browse Dialog is opened
        String dialogOpen = dialogBrowse.open();
        if (dialogOpen != null)
        {
            // Take the path of the repository : the update site
            String repositoryPath = UpdateSiteProjectWizard.checkPath(dialogOpen, true);
            File testFileFeat = new File(UpdateSiteProjectWizard.checkPath(repositoryPath + "features", true));

            if (!testFileFeat.exists())
            {
            	testFileFeat.mkdir();
            }
            text.setText(repositoryPath);
            wizard.setProjectPath(repositoryPath);
        }
    }

    /**
     * Button Action : Browse Directory for the Update Site (jar File)
     * 
     * @param text : location where the path will be read and write
     * @param table : location where the content of the 'jar' file appear
     */
    public void browseUpdateSiteWithJarFile(Text text, Table table)
    {
        DirectoryDialog dialogBrowse = new DirectoryDialog(wizard.getShell());
        dialogBrowse.setFilterPath(wizard.getUpdateSitePath());

        // Browse Dialog is opened
        String dialogOpen = dialogBrowse.open();
        if (dialogOpen != null)
        {
            // Take the path of the repository
            String repositoryPath = UpdateSiteProjectWizard.checkPath(dialogOpen, wizard.getProtocolConnexion() == UpdateSiteProjectWizard.FILE);
            text.setText(repositoryPath);
            wizard.setUpdateSitePath(repositoryPath);

            // Add the sub repository in Map of Update Site
            mapUS = initializeMapWithJarFile(UpdateSiteProjectWizard.checkPath(wizard.getUpdateSitePath() + "features", wizard.getProtocolConnexion() == UpdateSiteProjectWizard.FILE));
            table.removeAll();
            displayMap(mapUS, table);
        }
    }

    /**
     * Return the corresponding Map of all 'jar' File from the path repository
     * or file
     * 
     * @param repositoryPath
     * @return Map : key=name, value=SourceDestination
     */
    public Map initializeMapWithJarFile(String repositoryPath)
    {
        Map mapReturn = new TreeMap();
        // Content of the table = content of the repository "repositoryPath"
        File repository = new File(repositoryPath);
        if (repository != null)
        {
            File[] repositoryContent = repository.listFiles((new FileFilter()
            {
                public boolean accept(File pathname)
                {
                    boolean acceptReturn = false;
                    if (pathname.isFile())
                    {
                        String jarFile = pathname.toString();
                        jarFile = jarFile.substring(jarFile.lastIndexOf('.'));
                        if (jarFile.equals(".jar"))
                        {
                            acceptReturn = true;
                        }
                    }
                    return acceptReturn;
                }
            }));

            // Add the sub repository in Map of Update Site
            File subRepository = null;
            if (repositoryContent != null)
            {
                for (int indice = 0; indice < repositoryContent.length; indice++)
                {
                    subRepository = repositoryContent[indice];
                    mapReturn.putAll(pathToMap(subRepository));

                }
            }
        }
        return mapReturn;
    }

    /**
     * Return the corresponding Map from the path repository or file
     * 
     * @param subRepository : path repository or file
     * @return map (key=name, value=SourceDestination)
     */
    private Map pathToMap(File subRepository)
    {
        Map mapReturn = new TreeMap();
        try
        {
            String nameSubRepository = subRepository.getName();
            String pathSubRepository = subRepository.getCanonicalPath();
            String pathDestination = subRepository.getParent();
            // Add in the mapReturn
            SourceDestination sourceSubRepository = new SourceDestination(nameSubRepository, pathSubRepository, pathDestination);
            mapReturn.put(nameSubRepository, sourceSubRepository);
        }
        catch (IOException e)
        {
            CorePlugin.log(this.getClass(), "Problem to recover the canonic path of the selected repository in the browser.", IStatus.ERROR);
        }
        return mapReturn;
    }

    /**
     * Display the content Map "map" in the table "table"
     * 
     * @param map : key=name; value=SourceDestination
     * @param table : location where the map is display
     */
    public void displayMap(Map map, Table table)
    {
    	Iterator it = map.keySet().iterator();
        while (it.hasNext())
        {
            String key = (String) it.next();
            if (map.get(key) instanceof SourceDestination)
            {
                String name = key;
                TableItem itemAdd = new TableItem(table, 0);
                itemAdd.setText(name);
                table.showItem(itemAdd);
                table.redraw();
            }
        }
        table.redraw();
        table.update();
    }
    
    /**
     * Button Action : Browse the Update Site from a FTP server
     * @param text : location where the path will be read and write
     * @param table : location where the content of the repository appear
     */
    public void browseUpdateSiteFTP(Text text, Table table)
    {
    	File testFileFeat; //, testFilePlug ;
    	final String tempDir = UpdateSiteProjectWizard.checkPath(System.getProperty("java.io.tmpdir"), true);
        final String tempFeatureDir = UpdateSiteProjectWizard.checkPath(tempDir + "features", true);

        testFileFeat = new File(tempFeatureDir);
        FilesTreatment.emptyFile(testFileFeat);
        testFileFeat.mkdir();

        wizard.setUpdateSitePath(wizard.getBrowsePage().getDirectory());

		boolean retryTreatment;
		
		do
		{
			IOFTPSSHErrorVector = new Vector();
			retryTreatment = false;
		
	        // Add the sub repository in Map of Update Site
	        mapUS = initializeMapFTP(wizard.getClientFTP());
	        
	        mapTD.clear();
			
			if (!IOFTPSSHErrorVector.isEmpty())
	        {
	            int button;
	            Status[] status = new Status[IOFTPSSHErrorVector.size()];
	            for (int i=0; i<IOFTPSSHErrorVector.size(); i++)
	            {
	                status[i] = new Status(IStatus.ERROR, "org.topcased.service.updatesiteproject", 0, (String)IOFTPSSHErrorVector.get(i), null);
	            }
	            ErrorDialog errorBox = new ErrorDialog(wizard.getShell(), "IO Error", "An error occured during the copy", 
	                    new MultiStatus("org.topcased.service.updatesiteproject", 0, status,"See details", null), IStatus.ERROR);
	            errorBox.open();
	            
	            MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_WARNING | SWT.IGNORE | SWT.RETRY | SWT.ABORT);
	            msgBox.setText("IO Error");
	            msgBox.setMessage("If you continue, maybe some files will not be copied.\n\nDo you want to continue ?");
	            button = msgBox.open();
	            switch (button)
	            {
	                case (SWT.ABORT):
                        IRunnableWithProgress runnableWithProgress2 = new IRunnableWithProgress()
                        {
                            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
                            {
                                monitor.beginTask("Deleting files...", IProgressMonitor.UNKNOWN);
                                File suppFile = new File(tempFeatureDir);
                                FilesTreatment.emptyFile(suppFile);
                            }
                        };
                        ProgressMonitorDialog dialog2 = new ProgressMonitorDialog(wizard.getShell());
                        try
                        {
                            dialog2.run(true, true, runnableWithProgress2);
                        }
                        catch (InvocationTargetException e)
                        {
                        	CorePlugin.log(e);
                        }
                        catch (InterruptedException e)
                        {
                        	CorePlugin.log(e);
                        }
						break;
	                case (SWT.RETRY):
						// Retry to Add the sub repository in Map of Update Site
			            retryTreatment = true;
	                    break;
	                case (SWT.IGNORE):
						break;
	            }
	        }
		} while (retryTreatment);

        table.removeAll();
        displayMap(mapUS, table);

        wizard.setUpdateSitePath(tempDir); 
    }

    /**
     * Initialize a Map from a FTP server
     * @param clientFTP the FTP client
     * @return the Map containing the feature of the Update Site path "/features"
     */
    public Map initializeMapFTP(final FTPClient clientFTP)
    {
        final Map mapReturn = new TreeMap();
        IRunnableWithProgress runnableWithProgress = new IRunnableWithProgress()
        {
            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
            {
                error = "";
                try
                {
                    // Features
                    String pathDestination = UpdateSiteProjectWizard.checkPath(wizard.getProjectPath() + "features", true);
                    String path = UpdateSiteProjectWizard.checkPath(wizard.getUpdateSitePath() + "features", wizard.getProtocolConnexion() == UpdateSiteProjectWizard.FILE);

                    IDirectoryEntry[] files = clientFTP.listFiles(path, false, monitor);
                    if (files.length == 0)
                    {
                        // end system is different from the source system
                        if (File.separator.equals("\\"))
                        {
                            files = clientFTP.listFiles(path = path.replace('\\', '/'), false, monitor);
                        }
                        else
                        {
                            files = clientFTP.listFiles(path = path.replace('/', '\\'), false, monitor);
                        }
                    }

                    for (int i = 0; i < files.length; i++)
                    {
                        String nameSubRepository = files[i].getName();
                        String pathSubRepository = path + nameSubRepository;
                        // Add in the mapReturn
                        SourceDestination sourceSubRepository = new SourceDestination(nameSubRepository, pathSubRepository, pathDestination);
                        mapReturn.put(nameSubRepository, sourceSubRepository);
                    }
                }
                catch (FtpException e)
                {
                	CorePlugin.log(e);
                    error = e.getMessage();
                }
            }
        };
        ProgressMonitorDialog dialog = new ProgressMonitorDialog(wizard.getShell());
        try
        {
            dialog.run(true, true, runnableWithProgress);
        }
        catch (InvocationTargetException e)
        {
        	CorePlugin.log(e);
            error = e.getMessage();
        }
        catch (InterruptedException e)
        {
        	CorePlugin.log(e);
            error = e.getMessage();
        }
        if (!error.equals(""))
        {
            MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_ERROR | SWT.OK);
            msgBox.setText("Connection error");
            msgBox.setMessage("Error in directory browsing : " + error + "\nSee Console information for more details");
            msgBox.open();
        }

        return mapReturn;
    }
    
    /**
     * Button Action : Browse the Update Site from a SSH server
     * 
     * @param text : location where the path will be read and write
     * @param table : location where the content of the repository appear
     */
    public void browseUpdateSiteSSH(Text text, Table table)
    {
        // Get files from remote server
    	final String tempDir = UpdateSiteProjectWizard.checkPath(System.getProperty("java.io.tmpdir"), true);
        final String tempFeatureDir = UpdateSiteProjectWizard.checkPath(tempDir + "features", true);
        
    	File testFileFeat = new File(tempFeatureDir);
        FilesTreatment.emptyFile(testFileFeat);
        testFileFeat.mkdir();

        wizard.setUpdateSitePath(wizard.getBrowsePage().getDirectory());

        // Add the sub repository in Map of Update Site
		IRunnableWithProgress runnableWithProgress = new IRunnableWithProgress()
        {
            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
            {
				monitor.beginTask("Receiving files...", IProgressMonitor.UNKNOWN);
				mapUS = initializeMapSSH(wizard.getClientSFTP());
            }
        };
        
		boolean retryTreatment;
		do
		{
			IOFTPSSHErrorVector = new Vector();
			retryTreatment =false;
			ProgressMonitorDialog dialog = new ProgressMonitorDialog(wizard.getShell());
	        try
	        {
	            dialog.run(true, true, runnableWithProgress);
	        }
	        catch (InvocationTargetException e)
	        {
	        	CorePlugin.log(e);
	            error = e.getMessage();
	        }
	        catch (InterruptedException e)
	        {
	        	CorePlugin.log(e);
	            error = e.getMessage();
	        }
			
			if (!IOFTPSSHErrorVector.isEmpty())
	        {
	            int button;
	            Status[] status = new Status[IOFTPSSHErrorVector.size()];
	            for (int i=0; i<IOFTPSSHErrorVector.size(); i++)
	            {
	                status[i] = new Status(IStatus.ERROR, "org.topcased.service.updatesiteproject", 0, (String)IOFTPSSHErrorVector.get(i), null);
	            }
	            ErrorDialog errorBox = new ErrorDialog(wizard.getShell(), "IO Error", "An error occured during the copy", 
	                    new MultiStatus("org.topcased.service.updatesiteproject", 0, status,"See details", null), IStatus.ERROR);
	            errorBox.open();
	            
	            MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_WARNING | SWT.IGNORE | SWT.RETRY | SWT.ABORT);
	            msgBox.setText("IO Error");
	            msgBox.setMessage("If you continue, maybe some files will not be copied.\n\nDo you want to continue ?");
	            button = msgBox.open();
	            switch (button)
	            {
	                case (SWT.ABORT):
                        IRunnableWithProgress runnableWithProgress2 = new IRunnableWithProgress()
                        {
                            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
                            {
                                monitor.beginTask("Deleting files...", IProgressMonitor.UNKNOWN);
                                File suppFile = new File(tempFeatureDir);
                                FilesTreatment.emptyFile(suppFile);
                            }
                        };
                        ProgressMonitorDialog dialog2 = new ProgressMonitorDialog(wizard.getShell());
                        try
                        {
                            dialog2.run(true, true, runnableWithProgress2);
                        }
                        catch (InvocationTargetException e)
                        {
                        	CorePlugin.log(e);
                        }
                        catch (InterruptedException e)
                        {
                        	CorePlugin.log(e);
                        }
						break;
	                case (SWT.RETRY):
						// Retry to add the sub repository in Map of Update Site
						retryTreatment = true;
	                    break;
	                case (SWT.IGNORE):
						break;
	            }
	        }
		} 
		while (retryTreatment);
		
        mapTD.clear();

        table.removeAll();
        displayMap(mapUS, table);

        wizard.setUpdateSitePath(tempDir);
    }

    /**
     * Initialize a Map from a SSH server
     * 
     * @param clientSFTP the client SFTP of the SSH server
     * @return the Map containing the feature of the Update Site path
     *         "/features"
     */
    private Map initializeMapSSH(SftpClient clientSFTP)
    {
        final Map mapReturn = new TreeMap();
        String pathDestination = UpdateSiteProjectWizard.checkPath(wizard.getProjectPath() + "features", true);
        String path = UpdateSiteProjectWizard.checkPath(wizard.getUpdateSitePath() + "features", wizard.getProtocolConnexion() == UpdateSiteProjectWizard.FILE);

        List files = null;
        try
        {
            files = clientSFTP.ls(path);
        }
        catch (IOException e)
        {

        	CorePlugin.log(e);
        }

        for (int i = 0; i < files.size(); i++)
        {
            if (!((SftpFile) files.get(i)).getFilename().equals("..")
                    && !((SftpFile) files.get(i)).getFilename().equals("."))
            {
                String nameSubRepository = ((SftpFile) files.get(i)).getFilename();
                String pathSubRepository = UpdateSiteProjectWizard.checkPath(path, true) + ((SftpFile) files.get(i)).getFilename();

                // Add in the mapReturn
                SourceDestination sourceSubRepository = new SourceDestination(nameSubRepository, pathSubRepository, pathDestination);
                mapReturn.put(nameSubRepository, sourceSubRepository);
            }
        }

        return mapReturn;
    }

    /**
     * Search the source path of the selected item in different existing map
     * @param nameSelected
     * @param search
     * @return absolute path
     */
    public String searchSource(String nameSelected, boolean search)
    {
        String path = "";
        // Update Site Map
        if (mapUS.containsKey(nameSelected))
        {
            if (mapUS.get(nameSelected) instanceof SourceDestination)
            {
                SourceDestination sd = (SourceDestination) mapUS.get(nameSelected);
                path = sd.getSource();
            }
        }
        else
        // Target Directory Map
        if (mapTD.containsKey(nameSelected))
        {
            if (mapTD.get(nameSelected) instanceof SourceDestination)
            {
                SourceDestination sd = (SourceDestination) mapTD.get(nameSelected);
                path = sd.getSource();
            }
        }
        return path;
    }

    /**
     * Display information about a given feature
     * 
     * @param realfeaturePath The real path of the feature
     * @param tempFeaturePath The temporary path where the feature can be accessed
     */
    public void getFeatureInformation(String realfeaturePath, String tempFeaturePath)
    {
    	PopupDialog newPopupDialog = new PopupDialog(wizard.getShell(), realfeaturePath, tempFeaturePath);
        newPopupDialog.setBlockOnOpen(true);
        newPopupDialog.open();
    }
}
