/*******************************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS)  
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*********************************************************************************************/ 
package org.topcased.service.updatesiteproject;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

/**
 * 
 *
 * This WizardPage save all informations necessary to a FTP or SSH connection
 */
public class ConnectUpdateSiteProjectPage extends WizardPage
{
    private Composite container;

    private UpdateSiteProjectWizard wizard;

    private Text loginTxt;

    private Label loginLab;

    private Label passLab;

    private Text passTxt;

    private Label addrLab;

    private Text addrTxt;

    private Combo protocolCob;

    private Text portTxt;

    private Label portLab;
    
    private Combo connectCob;
    
    private Label usPathLabel;
    
    private Text usPathText;

    private ConnectUpdateSiteProjectTreatment treatment;    

    /**
     * Constructor
     * @param wizard the parent wizard
     * @param pageName the name of the page
     */
    public ConnectUpdateSiteProjectPage(UpdateSiteProjectWizard wizard, String pageName)
    {
        super(pageName);
        this.wizard = wizard;
        treatment = new ConnectUpdateSiteProjectTreatment(this);
    }

    public ConnectUpdateSiteProjectTreatment getTreatment()
    {
        return treatment;
    }

    /**
     * @return Returns the addrTxt.
     */
    public Text getAddrTxt()
    {
        return addrTxt;
    }

    /**
     * @param addrTxt The addrTxt to set.
     */
    public void setAddrTxt(Text addrTxt)
    {
        this.addrTxt = addrTxt;
    }

    /**
     * @return Returns the connectCob.
     */
    public Combo getConnectCob()
    {
        return connectCob;
    }

    /**
     * @return Returns the loginTxt.
     */
    public Text getLoginTxt()
    {
        return loginTxt;
    }

    /**
     * @return Returns the portTxt.
     */
    public Text getPortTxt()
    {
        return portTxt;
    }

    /**
     * @return Returns the protocolCob.
     */
    public Combo getProtocolCob()
    {
        return protocolCob;
    }
    
    public Text getUSPath()
    {
    	return usPathText;
    }
    
    /**
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent)
    {
        container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout(1, true));
        setControl(container);

        /*
         * Main Label
         */
        final Label label2 = new Label(container, SWT.NONE);
        label2.setText("Complete these following fields :");

        final Composite containerLogPas = new Composite(container, SWT.NONE);
        containerLogPas.setLayout(new GridLayout(2, false));
        
        /*
         * Connection profile Label
         */
        Label connectLab = new Label(containerLogPas, SWT.NONE);
        GridData gridData1 = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
        connectLab.setLayoutData(gridData1);
        connectLab.setText("Connection profile : ");

        /*
         * Connection profile Combobox
         */
        connectCob = new Combo(containerLogPas, SWT.DROP_DOWN | SWT.READ_ONLY | SWT.SIMPLE);
        connectCob.addSelectionListener(new SelectionListener()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (connectCob.getSelectionIndex()!=0)
                {
                    treatment.updateFields(connectCob.getSelectionIndex() - 1);

                    if (protocolCob.getSelectionIndex() == UpdateSiteProjectWizard.FILE)
                    {
                        addrLab.setEnabled(false);
                        addrTxt.setEnabled(false);
                        portLab.setEnabled(false);
                        portTxt.setEnabled(false);
                        loginLab.setEnabled(false);
                        loginTxt.setEnabled(false);
                        passLab.setEnabled(false);
                        passTxt.setEnabled(false);
                        usPathText.setEnabled(false);
                        usPathLabel.setEnabled(false);
                        setPageComplete(true);
                    }
                    else
                    {
                        addrLab.setEnabled(true);
                        addrTxt.setEnabled(true);
                        portLab.setEnabled(true);
                        portTxt.setEnabled(true);
                        loginLab.setEnabled(true);
                        loginTxt.setEnabled(true);
                        passLab.setEnabled(true);
                        passTxt.setEnabled(true);
                        usPathText.setEnabled(true);
                        usPathLabel.setEnabled(true);
                        setPageComplete(finish());
                    }
                }
            }

            public void widgetDefaultSelected(SelectionEvent e)
            {}
        });
        
        treatment.setProfileConnection();

        /*
         * Protocol Label
         */
        Label protocolLab = new Label(containerLogPas, SWT.NONE);
        GridData gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
        protocolLab.setLayoutData(gridData);
        protocolLab.setText("Protocol : ");

        /*
         * Protocol Combobox
         */
        protocolCob = new Combo(containerLogPas, SWT.DROP_DOWN | SWT.READ_ONLY | SWT.SIMPLE);
        protocolCob.add("FTP");
        protocolCob.add("SSH");
        protocolCob.add("File");
        protocolCob.select(0);
        protocolCob.addSelectionListener(new SelectionListener()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (protocolCob.getSelectionIndex() == UpdateSiteProjectWizard.SSH)
                {
                    portTxt.setText("22");
                }
                else {
                	if (protocolCob.getSelectionIndex() == UpdateSiteProjectWizard.FTP) 
                	{
                		portTxt.setText("21");        		
                	}
                	else
                	{
                		wizard.getConnectPage().setMessage("", WizardPage.NONE);
                	}
                }
                if (protocolCob.getSelectionIndex() == UpdateSiteProjectWizard.FILE)
                {
                    addrLab.setEnabled(false);
                    addrTxt.setEnabled(false);
                    portLab.setEnabled(false);
                    portTxt.setEnabled(false);
                    loginLab.setEnabled(false);
                    loginTxt.setEnabled(false);
                    passLab.setEnabled(false);
                    passTxt.setEnabled(false);
                    usPathText.setEnabled(false);
                    usPathLabel.setEnabled(false);
                    setPageComplete(true);
                }
                else
                {
                    addrLab.setEnabled(true);
                    addrTxt.setEnabled(true);
                    portLab.setEnabled(true);
                    portTxt.setEnabled(true);
                    loginLab.setEnabled(true);
                    loginTxt.setEnabled(true);
                    passLab.setEnabled(true);
                    passTxt.setEnabled(true);
                    usPathText.setEnabled(true);
                    usPathLabel.setEnabled(true);
                    setPageComplete(finish());
                }
            }

            public void widgetDefaultSelected(SelectionEvent e)
            {}
        });

        /*
         * Address label
         */
        addrLab = new Label(containerLogPas, SWT.NONE);
        gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
        addrLab.setLayoutData(gridData);
        addrLab.setText("Address : ");

        /*
         * Address Text
         */
        addrTxt = new Text(containerLogPas, SWT.BORDER);
        gridData = new GridData(SWT.LEFT, SWT.CENTER, true, true);
        gridData.widthHint = 130;
        addrTxt.setLayoutData(gridData);
        addrTxt.addKeyListener(new KeyListener()
        {
            public void keyPressed(KeyEvent e)
            {}

            public void keyReleased(KeyEvent e)
            {
                setPageComplete(finish());
            }
        });
        
        /*
         * Port label
         */
        portLab = new Label(containerLogPas, SWT.NONE);
        gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
        portLab.setLayoutData(gridData);
        portLab.setText("Port : ");

        /*
         * Port Text
         */
        portTxt = new Text(containerLogPas, SWT.BORDER);
        gridData = new GridData(SWT.LEFT, SWT.CENTER, true, true);
        gridData.widthHint = 130;
        portTxt.setLayoutData(gridData);
        portTxt.setText("21");
        portTxt.addKeyListener(new KeyListener()
        {
            public void keyPressed(KeyEvent e)
            {
                try
                {
                    Integer.parseInt(portTxt.getText());
                    wizard.getConnectPage().setMessage("", WizardPage.NONE);
                }
                catch (NumberFormatException eN)
                {
                    wizard.getConnectPage().setMessage("Invalid Port", WizardPage.ERROR);
                }
            }

            public void keyReleased(KeyEvent e)
            {
                try
                {
                    Integer.parseInt(portTxt.getText());
                    wizard.getConnectPage().setMessage("", WizardPage.NONE);
                }
                catch (NumberFormatException eN)
                {
                    wizard.getConnectPage().setMessage("Invalid Port", WizardPage.ERROR);
                }
                setPageComplete(finish());
            }
        });

        /*
         * Login Label
         */
        loginLab = new Label(containerLogPas, SWT.NONE);
        gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
        loginLab.setLayoutData(gridData);
        loginLab.setText("Login : ");

        /*
         * Login Text
         */
        loginTxt = new Text(containerLogPas, SWT.BORDER);
        gridData = new GridData(SWT.LEFT, SWT.CENTER, true, true);
        gridData.widthHint = 130;
        loginTxt.setLayoutData(gridData);
        loginTxt.addKeyListener(new KeyListener()
        {
            public void keyPressed(KeyEvent e)
            {}

            public void keyReleased(KeyEvent e)
            {
                setPageComplete(finish());
            }
        });

        /*
         * Password Label
         */
        passLab = new Label(containerLogPas, SWT.NONE);
        gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
        passLab.setLayoutData(gridData);
        passLab.setText("Password : ");

        /*
         * Password Text (type PASSWORD to have some '*' instead of characters,
         * nice...)
         */
        passTxt = new Text(containerLogPas, SWT.BORDER | SWT.PASSWORD);
        gridData = new GridData(SWT.LEFT, SWT.CENTER, true, true);
        gridData.widthHint = 130;
        passTxt.setLayoutData(gridData);
        passTxt.addKeyListener(new KeyListener()
        {
            public void keyPressed(KeyEvent e)
            {}

            public void keyReleased(KeyEvent e)
            {
                setPageComplete(finish());
            }
        });
        
        usPathLabel = new Label(containerLogPas, SWT.NONE);
        gridData = new GridData(SWT.RIGHT, SWT.CENTER, true, true);
        usPathLabel.setLayoutData(gridData);
        usPathLabel.setText("Path : ");

        usPathText = new Text(containerLogPas, SWT.BORDER);
        gridData = new GridData(SWT.LEFT, SWT.CENTER, true, true);
        gridData.widthHint = 130;
        usPathText.setLayoutData(gridData);
        usPathText.addKeyListener(new KeyListener()
        {
            public void keyPressed(KeyEvent e)
            {/* TODO ? */
            }

            public void keyReleased(KeyEvent e)
            {
                setPageComplete(finish());
            }
        });

        setPageComplete(false);
    }

    
    /**
     * @return true if the button 'Next' can be clicked, false otherwise
     */
    public boolean finish()
    {
        if (protocolCob.getSelectionIndex() == UpdateSiteProjectWizard.FILE)
        {
            return true;
        }
        
        return !(addrTxt.getText().equals("") || portTxt.getText().equals("") || getMessageType() == WizardPage.ERROR
                || loginTxt.getText().equals("") || passTxt.getText().equals("")
                || usPathText.getText().equals(""));
    }

    /**
     * Update the informations of the connection
     */
    public void updateConnectionInformations()
    {
        wizard.setProtocolConnexion(this.protocolCob.getSelectionIndex());
        wizard.setHost(addrTxt.getText());
        wizard.setPort(Integer.parseInt(portTxt.getText()));
        wizard.setUsername(this.loginTxt.getText());
        wizard.setPassword(this.passTxt.getText());
    }

    /**
     * @return the connection's protocol used
     */
    public int getProtocol()
    {
        return protocolCob.getSelectionIndex();
    }

    /**
     * @see org.eclipse.jface.wizard.WizardPage#canFlipToNextPage()
     */
    public boolean canFlipToNextPage()
    {
        wizard.disconnect();
        return isPageComplete();
    }
    
    public IWizardPage getNextPage()
    {
    	BrowseUpdateSiteProjectPage page = (BrowseUpdateSiteProjectPage) super.getNextPage();
    	if(page!=null)
    	{
    		page.update();
    	}
    	return page;
    }
}
