/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.updatesiteproject;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.runtime.IStatus;
import org.topcased.bus.core.CorePlugin;
import org.eclipse.core.resources.ResourcesPlugin;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class ConnectUpdateSiteProjectTreatment
{
    public static final String CONNECTION_TAG = "connection";

    public static final String PROFILE_TAG = "profile";

    public static final String CONNECTION_TYPE_ATTRIBUTE = "type";

    public static final String HOST_ATTRIBUTE = "host";

    public static final String PORT_ATTRIBUTE = "port";

    public static final String USERNAME_ATTRIBUTE = "username";
    
    public static final String PATH_ATTRIBUTE = "path"; 

    public static final int NEW_PROFILE = 0;

    ConnectUpdateSiteProjectPage parentPage;

    private Vector tabProtocol;

    private Vector tabAddr;

    private Vector tabPort;

    private Vector tabUser;
    
    private Vector tabPath;

    public ConnectUpdateSiteProjectTreatment(ConnectUpdateSiteProjectPage parentPage)
    {
        this.parentPage = parentPage;
    }

    /**
     * 
     */
    public void setProfileConnection()
    {
        parentPage.getConnectCob().add("New");
        parentPage.getConnectCob().select(0);

        tabProtocol = new Vector();
        tabAddr = new Vector();
        tabPort = new Vector();
        tabUser = new Vector();
        tabPath = new Vector();

        String metadataDir = UpdateSiteProjectWizard.checkPath(ResourcesPlugin.getWorkspace().getRoot().getLocation().toString() + File.separator + ".metadata", true);
        //String metadataDir = UpdateSiteProjectWizard.checkPath(System.getProperty("user.home") + File.separator + ".topcased", true);
        if (! new File(metadataDir).exists())
        {
        	new File(metadataDir).mkdir();
        }
        
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setIgnoringComments(true);
        factory.setCoalescing(true);
        // Parser
        try
        {
            DocumentBuilder parser = factory.newDocumentBuilder();
            // Document
            Document document = parser.parse(new File(metadataDir + "connection_profile.xml"));
            NodeList siteList = document.getElementsByTagName(CONNECTION_TAG);
            int nbNode = siteList.getLength();

            for (int i = 0; i < nbNode; i++)
            {
                Element node = (Element) siteList.item(i);

                // Connections profiles
                NodeList profileList = node.getElementsByTagName(PROFILE_TAG);
                int nbProfile = profileList.getLength();

                for (int j = 0; j < nbProfile; j++)
                {
                    Element ArchiveElement = (Element) profileList.item(j);
                    String connectionType = ArchiveElement.getAttribute(CONNECTION_TYPE_ATTRIBUTE);
                    String host = ArchiveElement.getAttribute(HOST_ATTRIBUTE);
                    String port = ArchiveElement.getAttribute(PORT_ATTRIBUTE);
                    String username = ArchiveElement.getAttribute(USERNAME_ATTRIBUTE);
                    String path = ArchiveElement.getAttribute(PATH_ATTRIBUTE);

                    tabProtocol.add(connectionType);
                    tabAddr.add(host);
                    tabPort.add(port);
                    tabUser.add(username);
                    tabPath.add(path);

                    parentPage.getConnectCob().add(username + "@" + host + ":" + port);
                }
            }
        }
        catch (ParserConfigurationException e)
        {
            CorePlugin.log(this.getClass(), "ParserConfiguration error : " + e.getMessage(), IStatus.ERROR);

        }
        catch (SAXException e)
        {
            CorePlugin.log(this.getClass(), "Error while parsing document : " + e.getMessage(), IStatus.ERROR);
        }
        catch (IOException e)
        {
            CorePlugin.log(this.getClass(), "Error while opening file : " + metadataDir + "connection_profile.xml"
                    + ". The file connection_profile.xml does not exist already.", IStatus.INFO);
        }
    }

    public void updateConnectionProfiles()
    {
        if (parentPage.getConnectCob().getSelectionIndex() == NEW_PROFILE)
        {
            if (parentPage.getConnectCob().getItemCount() - 1 >= 5)
            {
                tabProtocol.removeElementAt(0);
                tabAddr.removeElementAt(0);
                tabPort.removeElementAt(0);
                tabUser.removeElementAt(0);
                tabPath.removeElementAt(0);
            }
        	
            tabProtocol.add(String.valueOf(parentPage.getProtocolCob().getSelectionIndex()));
            tabAddr.add(parentPage.getAddrTxt().getText());
            tabPort.add(parentPage.getPortTxt().getText());
            tabUser.add(parentPage.getLoginTxt().getText());
            tabPath.add(parentPage.getUSPath().getText());

            if (parentPage.getConnectCob().getItemCount() - 1 >= 5)
            {
              	parentPage.getConnectCob().remove(1);
            }
            parentPage.getConnectCob().add(parentPage.getLoginTxt().getText() + "@" + parentPage.getAddrTxt().getText() + ":" + parentPage.getPortTxt().getText());
        }
        else
        {
            tabProtocol.setElementAt(String.valueOf(parentPage.getProtocolCob().getSelectionIndex()), parentPage.getConnectCob().getSelectionIndex() - 1);
            tabAddr.setElementAt(parentPage.getAddrTxt().getText(), parentPage.getConnectCob().getSelectionIndex() - 1);
            tabPort.setElementAt(parentPage.getPortTxt().getText(), parentPage.getConnectCob().getSelectionIndex() - 1);
            tabUser.setElementAt(parentPage.getLoginTxt().getText(), parentPage.getConnectCob().getSelectionIndex() - 1);
            tabPath.setElementAt(parentPage.getUSPath().getText(), parentPage.getConnectCob().getSelectionIndex() - 1);
            parentPage.getConnectCob().setItem(parentPage.getConnectCob().getSelectionIndex(), parentPage.getLoginTxt().getText() + "@" + parentPage.getAddrTxt().getText() + ":"
                    + parentPage.getPortTxt().getText());
        }

        String metadataDir = UpdateSiteProjectWizard.checkPath(ResourcesPlugin.getWorkspace().getRoot().getLocation().toString() + File.separator + ".metadata", true);
        //String metadataDir = UpdateSiteProjectWizard.checkPath(System.getProperty("user.home") + File.separator + ".topcased", true);
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        try
        {
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document document = builder.newDocument();
            // Tag creation
            Element root = document.createElement(CONNECTION_TAG);
            document.appendChild(root);

            // Connection profile
            for (int i = 0; i < parentPage.getConnectCob().getItemCount() - 1; i++)
            {
                Element profileTag = document.createElement(PROFILE_TAG);
                root.appendChild(profileTag);
                profileTag.setAttribute(CONNECTION_TYPE_ATTRIBUTE, (String) tabProtocol.elementAt(i));
                profileTag.setAttribute(HOST_ATTRIBUTE, (String) tabAddr.elementAt(i));
                profileTag.setAttribute(PORT_ATTRIBUTE, (String) tabPort.elementAt(i));
                profileTag.setAttribute(USERNAME_ATTRIBUTE, (String) tabUser.elementAt(i));
                profileTag.setAttribute(PATH_ATTRIBUTE, (String) tabPath.elementAt(i));
            }

            // Xml write
            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer();
            
            // Indent the XML document
            transformer.setOutputProperty(OutputKeys.INDENT, "yes"); 
            transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "3");
            
            DOMSource sourceDom = new DOMSource(document);
            StreamResult result = new StreamResult(metadataDir + "connection_profile.xml"); 
            transformer.transform(sourceDom, result);

        }
        catch (ParserConfigurationException e)
        {
            CorePlugin.log(this.getClass(), "Parse Configuration Exception" + e.getMessage(), IStatus.ERROR);
        }
        catch (TransformerConfigurationException e)
        {
            CorePlugin.log(this.getClass(), "Transformer Configuration Exception" + e.getMessage(), IStatus.ERROR);
        }
        catch (TransformerException e)
        {
            CorePlugin.log(this.getClass(), "Transformer Exception" + e.getMessage(), IStatus.ERROR);
        }

        CorePlugin.log(this.getClass(), "connection_profile.xml succefully updated : " + metadataDir + "connection_profile.xml", IStatus.INFO);
    }

    public void updateFields(int index)
    {
        parentPage.getProtocolCob().select(Integer.parseInt((String)tabProtocol.elementAt(index)));
        parentPage.getAddrTxt().setText((String)tabAddr.elementAt(index));
        parentPage.getPortTxt().setText((String)tabPort.elementAt(index));
        parentPage.getLoginTxt().setText((String)tabUser.elementAt(index));
        parentPage.getUSPath().setText((String)tabPath.elementAt(index));
    }
}
