/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.updatesiteproject;

import java.io.File;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;

/**
 * 
 * @author vcombet
 * 
 * This WizardPage saves all information necessary to create the project feature
 */
public class FeatureUpdateSiteProjectPage extends WizardPage
{
    // Items from the first tab : required feature information
	private Text featureNameText;
    private Text featureVersionText;
	private Text eclipseUpdateJpgBrowseText;
    private Text cplV10HtmlBrowseText;
    private Text licenseHtmlTextBrowse;

    private UpdateSiteProjectWizard wizard;

    private FeatureUpdateSiteProjectTreatment treatment;
    
    // boolean enabling to know if the process can be finished
    private boolean canFinish;

    /**
     * Constructor
     * 
     * @param wizard the parent wizard
     * @param pageName the page name
     */
    public FeatureUpdateSiteProjectPage(UpdateSiteProjectWizard wizard, String pageName)
    {
        super(pageName);
        this.wizard = wizard;
        treatment = new FeatureUpdateSiteProjectTreatment(wizard);
        canFinish = false;
    }

    /**
     * @return boolean, true if the page is completed, false otherwise
     */
    public boolean finish()
    {
    	return canFinish;
    }
    
   
    /**
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent)
    {
    	/*
    	 * Definition of some GridData 
    	 */    	
    	
    	final GridData labelGridData = new GridData();
        labelGridData.widthHint = 130;
        
        final GridData textGridData = new GridData();
        textGridData.widthHint = 160;
        
        final GridData extendedTextGridData = new GridData();
        extendedTextGridData.widthHint = 160;
        extendedTextGridData.horizontalSpan = 3;
        
        final GridData shortTextGridData = new GridData();
        shortTextGridData.widthHint = 50;
        shortTextGridData.horizontalSpan = 3;
        
    	/*final Composite container = new Composite(parent, SWT.NONE);
    	container.setLayout(new GridLayout(1, true));
        setControl(container);*/
                
        final CTabFolder tabFolder = new CTabFolder(parent, SWT.NONE);
        tabFolder.setLayout(new GridLayout(1, true));
        tabFolder.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
        setControl(tabFolder);
        
        CTabItem itemProject = new CTabItem(tabFolder, SWT.NONE);
        itemProject.setText("Required feature information");
                
        ImageDescriptor imgDesc = UpdateSiteProjectPlugin.getImageDescriptor("icons" + File.separator + "project-obj.gif");
        Image img = imgDesc.createImage(wizard.getShell().getDisplay());
        itemProject.setImage(img);

        /***********************************************************************
         * First tab : required feature information
         **********************************************************************/
        // Composite definition
        final Composite requiredComposite = new Composite(tabFolder, SWT.NONE);
        requiredComposite.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER | GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
        requiredComposite.setLayout(new GridLayout(4, false));
        
        // Feature name
        final Label featureNameLabel = new Label(requiredComposite, SWT.NONE);
        featureNameLabel.setText("Feature name : ");
        featureNameText = new Text(requiredComposite, SWT.BORDER);
        featureNameText.setLayoutData(extendedTextGridData);

        // Feature version
        final Label featureVersionLabel = new Label(requiredComposite, SWT.NONE);
        featureVersionLabel.setText("Feature version : ");
        featureVersionText = new Text(requiredComposite, SWT.BORDER);
        featureVersionText.setLayoutData(shortTextGridData);        
        
        // 'eclipse_update_120.jpg'
        final Label eclipseUpdateJpgLabel = new Label(requiredComposite, SWT.NONE);
        eclipseUpdateJpgLabel.setText("Project image : ");
        eclipseUpdateJpgBrowseText = new Text(requiredComposite, SWT.BORDER);
        eclipseUpdateJpgBrowseText.setLayoutData(textGridData);
        eclipseUpdateJpgBrowseText.setText(wizard.getProjectPath());
        
        //Button to browse the project image
        final Button button_browse_EclipseUpdate = new Button(requiredComposite, SWT.NONE);
        button_browse_EclipseUpdate.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        button_browse_EclipseUpdate.setText("Browse...");
        
        //Button to see help about the project image
        final Button button_browse_EclipseUpdate_help = new Button(requiredComposite, SWT.NONE);
        imgDesc = UpdateSiteProjectPlugin.getImageDescriptor("icons" + File.separator + "help.gif");
        button_browse_EclipseUpdate_help.setImage(imgDesc.createImage(wizard.getShell().getDisplay()));
        button_browse_EclipseUpdate_help.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_INFORMATION | SWT.OK);
                msgBox.setText("Help about project image");
                msgBox.setMessage("Image of Eclipse Project\nOfen called 'eclipse_update_120.jpg'");
                msgBox.open();
            }
        });

        // 'cpl-v10.html'
        final Label cplV10htmlLabel = new Label(requiredComposite, SWT.NONE);
        cplV10htmlLabel.setText("Public license : ");
        //cplV10htmlLabel.setLayoutData(labelGridData);
        cplV10HtmlBrowseText = new Text(requiredComposite, SWT.BORDER);
        cplV10HtmlBrowseText.setLayoutData(textGridData);
        cplV10HtmlBrowseText.setText(wizard.getProjectPath());
        final Button button_browse_CplV10Html = new Button(requiredComposite, SWT.NONE);
        button_browse_CplV10Html.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        button_browse_CplV10Html.setText("Browse...");
        final Button button_browse_CplV10Html_help = new Button(requiredComposite, SWT.NONE);
        button_browse_CplV10Html_help.setImage(imgDesc.createImage(wizard.getShell().getDisplay()));
        button_browse_CplV10Html_help.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_INFORMATION | SWT.OK);
                msgBox.setText("Help about Public license");
                msgBox.setMessage("The Public license of Eclipse\nLicense associated with the uses, modifications and redistributions of the project created\nGenerally CPL or EPL License");
                msgBox.open();
            }
        });

        // 'license.html'
        final Label licenseHtmlLabel = new Label(requiredComposite, SWT.NONE);
        licenseHtmlLabel.setText("Software Agreement : ");
        //licenseHtmlLabel.setLayoutData(labelGridData);
        licenseHtmlTextBrowse = new Text(requiredComposite, SWT.BORDER);
        licenseHtmlTextBrowse.setLayoutData(textGridData);
        licenseHtmlTextBrowse.setText(wizard.getProjectPath());
        final Button button_browse_LicenceHtml = new Button(requiredComposite, SWT.NONE);
        button_browse_LicenceHtml.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        button_browse_LicenceHtml.setText("Browse...");
        final Button button_browse_LicenceHtml_help = new Button(requiredComposite, SWT.NONE);
        button_browse_LicenceHtml_help.setImage(imgDesc.createImage(wizard.getShell().getDisplay()));
        button_browse_LicenceHtml_help.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_INFORMATION | SWT.OK);
                msgBox.setText("Help about Software Agreement");
                msgBox.setMessage("License about Software User Agreement\nLicense associated with the uses of the software\nOften a Eclipse Foundation Software User Agreement License");
                msgBox.open();
            }
        });

        // Button : Browse the file 'eclipse_update_120.jpg'
        button_browse_EclipseUpdate.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if(treatment.browse(eclipseUpdateJpgBrowseText, "Project image"))
                {
                	setPageComplete(updateError());	
                }
            }
        });

        // Button : Browse the file 'cpl-v10.html'
        button_browse_CplV10Html.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
            	if(treatment.browse(cplV10HtmlBrowseText, "Public License"))
                {
                	setPageComplete(updateError());
                }
            }
        });

        // Button : Browse the file 'license.html'
        button_browse_LicenceHtml.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if(treatment.browse(licenseHtmlTextBrowse,"Software Agreement"))
                {
                	setPageComplete(updateError());
                }
            }
        });

        itemProject.setControl(requiredComposite);

        /***********************************************************************
         * Second tab : optional feature information
         **********************************************************************/        
        CTabItem itemOptionalFeature = new CTabItem(tabFolder, SWT.NONE);

        itemOptionalFeature.setText("Optional feature information");
        imgDesc = UpdateSiteProjectPlugin.getImageDescriptor("icons" + File.separator + "feature-obj.gif");
        img = imgDesc.createImage(wizard.getShell().getDisplay());
        itemOptionalFeature.setImage(img);

        final Composite optionalComposite = new Composite(tabFolder, SWT.NONE);
        optionalComposite.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        optionalComposite.setLayout(new GridLayout(4,false));

        // Label feature
        final Label featureLabel = new Label(optionalComposite, SWT.NONE);
        featureLabel.setText("Label : ");
        final Text labelText = new Text(optionalComposite, SWT.BORDER);
        labelText.setLayoutData(labelGridData);

        // Provider name feature
        final Label providerNameLabel = new Label(optionalComposite, SWT.NONE);
        providerNameLabel.setText("Provider name : ");
        final Text providerNameText = new Text(optionalComposite, SWT.BORDER);
        providerNameText.setLayoutData(labelGridData);

        // Image feature
        final Label imageLabel = new Label(optionalComposite, SWT.NONE);
        imageLabel.setText("Image : ");
        final Text imageText = new Text(optionalComposite, SWT.BORDER);
        imageText.setLayoutData(labelGridData);

        // Os feature
        final Label osLabel = new Label(optionalComposite, SWT.NONE);
        osLabel.setText("Os : ");
        final Text osText = new Text(optionalComposite, SWT.BORDER);
        osText.setLayoutData(labelGridData);

        // Arch feature
        final Label archLabel = new Label(optionalComposite, SWT.NONE);
        archLabel.setText("Arch : ");
        final Text archText = new Text(optionalComposite, SWT.BORDER);
        archText.setLayoutData(labelGridData);

        // Ws feature
        final Label wsLabel = new Label(optionalComposite, SWT.NONE);
        wsLabel.setText("Ws : ");
        final Text wsText = new Text(optionalComposite, SWT.BORDER);
        wsText.setLayoutData(labelGridData);

        // Nl feature
        final Label nlLabel = new Label(optionalComposite, SWT.NONE);
        nlLabel.setText("Nl : ");
        final Text nlText = new Text(optionalComposite, SWT.BORDER);
        nlText.setLayoutData(labelGridData);

        // Colocation feature
        final Label colocationLabel = new Label(optionalComposite, SWT.NONE);
        colocationLabel.setText("Colocation : ");
        final Text colocationText = new Text(optionalComposite, SWT.BORDER);
        colocationText.setLayoutData(labelGridData);

        // Primary feature
        final Label primaryLabel = new Label(optionalComposite, SWT.NONE);
        primaryLabel.setText("Primary : ");
        final Combo primaryCombo = new Combo(optionalComposite, SWT.BORDER | SWT.READ_ONLY);
        primaryCombo.add("true");
        primaryCombo.add("false");
        primaryCombo.select(1);
        primaryCombo.setLayoutData(labelGridData);

        // Exclusive feature
        final Label exclusiveLabel = new Label(optionalComposite, SWT.NONE);
        exclusiveLabel.setText("Exclusive : ");
        final Combo exclusiveCombo = new Combo(optionalComposite, SWT.BORDER | SWT.READ_ONLY);
        exclusiveCombo.add("true");
        exclusiveCombo.add("false");
        exclusiveCombo.select(1);
        exclusiveCombo.setLayoutData(labelGridData);

        // Plugin feature
        final Label pluginLabel = new Label(optionalComposite, SWT.NONE);
        pluginLabel.setText("Plugin : ");
        final Text pluginText = new Text(optionalComposite, SWT.BORDER);
        pluginText.setLayoutData(labelGridData);

        // Application feature
        final Label applicationLabel = new Label(optionalComposite, SWT.NONE);
        applicationLabel.setText("Application : ");
        final Text applicationText = new Text(optionalComposite, SWT.BORDER);
        applicationText.setLayoutData(labelGridData);

        // Description feature
        final Label desciptionLabel = new Label(optionalComposite, SWT.NONE);
        desciptionLabel.setText("Description : ");
        final Text descriptionText = new Text(optionalComposite, SWT.BORDER);
        descriptionText.setLayoutData(labelGridData);

        // URL Description feature
        final Label urlDescriptionLabel = new Label(optionalComposite, SWT.NONE);
        urlDescriptionLabel.setText("URL description : ");
        final Text urlDescriptionText = new Text(optionalComposite, SWT.BORDER);
        urlDescriptionText.setLayoutData(labelGridData);

        itemOptionalFeature.setControl(optionalComposite);//compItemOptionalFeat);

        tabFolder.setLayout(new FillLayout());
        tabFolder.setLayoutData(new GridData());
        tabFolder.setSelection(0);

        /*
         * TEXTS ACTION
         */
        labelText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleLabelFeatureProject(labelText);
            }
        });
        providerNameText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleProviderNameFeatureProject(providerNameText);
            }
        });
        imageText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleImageFeatureProject(imageText);
            }
        });
        osText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleOsFeatureProject(osText);
            }
        });
        archText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleArchFeatureProject(archText);
            }
        });
        wsText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleWsFeatureProject(wsText);
            }
        });
        nlText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleNlFeatureProject(nlText);
            }
        });
        colocationText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleColocationFeatureProject(colocationText);
            }
        });
        primaryCombo.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handlePrimaryFeatureProject(primaryCombo);
            }
        });
        exclusiveCombo.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleExclusiveFeatureProject(exclusiveCombo);
            }
        });
        pluginText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handlePluginFeatureProject(pluginText);
            }
        });
        applicationText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleApplicationFeatureProject(applicationText);
            }
        });
        descriptionText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleDescriptionFeatureProject(descriptionText);
            }
        });
        urlDescriptionText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleUrlDescriptionFeatureProject(urlDescriptionText);
            }
        });
        featureNameText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleNameFeatureProject(featureNameText);
                setPageComplete(updateError());
            }
        });
        featureVersionText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                treatment.handleVersionFeatureProject(featureVersionText);
                setPageComplete(updateError());
            }
        });        
    }

    /**
     * Updates the error message on the top of the page following the differents
     * missing information which are required (feature name and version, image path, CPL-V10 path and HTML licence path)
     * 
     * @return true if there are no error, false otherwise
     */
    public boolean updateError()
    {
        String error = "";
        boolean alreadyAnError = false;

        if (wizard.getNameFeatureProject().equals(""))
        {
            error += "Feature name";
        	alreadyAnError = true;           
        }
        if (wizard.getVersionFeatureProject().equals(""))
        {
        	if(alreadyAnError)
        	{
        		error += ", ";
        	}
        	error += "Feature version";
        	alreadyAnError = true;
        }
        if (wizard.getEclipseUpdateJpgPath().equals(""))
        {
        	if(alreadyAnError)
        	{
        		error += ", ";
        	}
        	error += "Project image";
        	alreadyAnError = true;
        }
        if (wizard.getCplV10HtmlPath().equals(""))
        {
            if(alreadyAnError)
            {
            	error += ", ";
            }
           	error += "CPL-V10";
        	alreadyAnError = true;
        }
        if (wizard.getLicenceHtmlPath().equals(""))
        {
            if(alreadyAnError)
            {
            	error += ", ";
            }
            error += "HTML License";            
        	alreadyAnError = true;        
        }
      if (!error.equals(""))
        {
            setMessage("Missing " + error + " to continue the generation process.", WizardPage.ERROR);
            return false;
        }
        else
        {
            setMessage("");
            canFinish = true;
            return true;
        }
    }
}
