/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.updatesiteproject;

import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Text;

/**
 * 
 *
 * This class ontanis all FeatureUpdateSiteProjectPage treatments
 */
public class FeatureUpdateSiteProjectTreatment
{
    private UpdateSiteProjectWizard wizard;
    
    // Keep the path directory between each browse dialog opened
    private String savedPathFeatureInfo;

    /**
     * Constructor
     * @param wizard the parent wizard
     */
    public FeatureUpdateSiteProjectTreatment(UpdateSiteProjectWizard wizard)
    {
        this.wizard = wizard;
        savedPathFeatureInfo = "";
    }

    /**
     * Button Action : Browse Directory for one of the three file proposed on the feature information page 
     * 
     * @param text : reference to the text field of the current file
     * @param name : name given to the current file to process 
     * @return true if the file path is validated by the method checkFilePath, false otherwise.
     */
    public boolean browse(Text text, String name)
    {
        FileDialog dialogBrowse = new FileDialog(wizard.getShell());
        dialogBrowse.setFilterPath(savedPathFeatureInfo);

        // Browse Dialog is opened
        String dialogOpen = dialogBrowse.open();
        if (dialogOpen != null)
        {
        	savedPathFeatureInfo = dialogBrowse.getFilterPath();
        	// Checks that the given path is not already used.
        	if(name.equals("Software Agreement") && wizard.checkFilePath(name, dialogOpen))
            {
            	text.setText(dialogOpen);
            	wizard.setLicenceHtmlPath(dialogOpen);
            	return true;
            }
        	
        	if(name.equals("Public License") && wizard.checkFilePath(name, dialogOpen))
            {
            	text.setText(dialogOpen);
            	wizard.setCplV10HtmlPath(dialogOpen);
            	return true;
            }
        	
        	if(name.equals("Project image") && wizard.checkFilePath(name, dialogOpen))
            {
            	text.setText(dialogOpen);
            	wizard.setEclipseUpdateJpgPath(dialogOpen);
            	return true;
            }
        }
        return false;
    }

    /***************************************************************************
     * INFOS FEATURE PROJECT
     **************************************************************************/
    /**
     * Take le value (which is changing) of the id text area
     * 
     * @param idText
     */
    public void handleNameFeatureProject(Text idText)
    {
        wizard.setNameFeatureProject(idText.getText());
    }

    /**
     * Take le value (which is changing) of the label text area
     * 
     * @param idText
     */
    public void handleLabelFeatureProject(Text labelText)
    {
        wizard.setLabelFeatureProject(labelText.getText());
    }

    /**
     * Take le value (which is changing) of the version text area
     * 
     * @param idText
     */
    public void handleVersionFeatureProject(Text versionText)
    {
        wizard.setVersionFeatureProject(versionText.getText());
    }

    /**
     * Take le value (which is changing) of the provider text area
     * 
     * @param idText
     */
    public void handleProviderNameFeatureProject(Text providerText)
    {
        wizard.setProviderNameFeatureProject(providerText.getText());
    }

    /**
     * Take le value (which is changing) of the image text area
     * 
     * @param idText
     */
    public void handleImageFeatureProject(Text imageText)
    {
        wizard.setImageFeatureProject(imageText.getText());
    }

    /**
     * Take le value (which is changing) of the os text area
     * 
     * @param idText
     */
    public void handleOsFeatureProject(Text osText)
    {
        wizard.setOsFeatureProject(osText.getText());
    }

    /**
     * Take le value (which is changing) of the architecture text area
     * 
     * @param idText
     */
    public void handleArchFeatureProject(Text archText)
    {
        wizard.setArchFeatureProject(archText.getText());
    }

    /**
     * Take le value (which is changing) of the ws text area
     * 
     * @param idText
     */
    public void handleWsFeatureProject(Text wsText)
    {
        wizard.setWsFeatureProject(wsText.getText());
    }

    /**
     * Take le value (which is changing) of the nl text area
     * 
     * @param idText
     */
    public void handleNlFeatureProject(Text nlText)
    {
        wizard.setNlFeatureProject(nlText.getText());
    }

    /**
     * Take le value (which is changing) of the colocation text area
     * 
     * @param idText
     */
    public void handleColocationFeatureProject(Text colocationText)
    {
        wizard.setColocationFeatureProject(colocationText.getText());
    }

    /**
     * Take le value (which is changing) of the primary combo area
     * 
     * @param idText
     */
    public void handlePrimaryFeatureProject(Combo primaryCombo)
    {
        wizard.setPrimaryFeatureProject(primaryCombo.getText());
    }
    
    /**
     * Take le value (which is changing) of the exclusive combo area
     * 
     * @param idText
     */
    public void handleExclusiveFeatureProject(Combo exclusiveCombo)
    {
        wizard.setExclusiveFeatureProject(exclusiveCombo.getText());
    }

    /**
     * Take le value (which is changing) of the plugin text area
     * 
     * @param idText
     */
    public void handlePluginFeatureProject(Text pluginText)
    {
        wizard.setPluginFeatureProject(pluginText.getText());
    }

    /**
     * Take le value (which is changing) of the application text area
     * 
     * @param idText
     */
    public void handleApplicationFeatureProject(Text applicationText)
    {
        wizard.setApplicationFeatureProject(applicationText.getText());
    }

    /**
     * Take le value (which is changing) of the description text area
     * 
     * @param idText
     */
    public void handleDescriptionFeatureProject(Text descriptionText)
    {
        wizard.setDescriptionFeatureProject(descriptionText.getText());
    }

    /**
     * Take le value (which is changing) of the url description text area
     * 
     * @param idText
     */
    public void handleUrlDescriptionFeatureProject(Text urlDescriptionText)
    {
        wizard.setUrlDescriptionFeatureProject(urlDescriptionText.getText());
    }
}
