/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.updatesiteproject;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.runtime.IStatus;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.topcased.bus.core.*;

/**
 * All informations which can be find in the file 'feature.xml' 
 * <br>
 */
public class InfosFeature
{
    // Path Feature (feature.xml)
    private String pathFeature = null;
    // Path Property (feature.properties)
    private String pathFeatureProperty= null;
    
    // Feature
    private String name = "";
    private String label = "";
    private String version = "";
    private String providerName = "";
    private String image = "";
    private String os = "";
    private String arch = "";
    private String ws = "";
    private String nl = "";
    private String colocation = "";
    private String primary = "";
    private String exclusive = "";
    private String plugin = "";
    private String application = "";
    
    
    
    // Others tags for the feature
    private String description = "";
    private String descriptionUrl = "";
    
    
    private Map mapRequiresImport = new HashMap();      // key=plugin ; value=InfosImport 
    private Map mapPlugins = new HashMap();             // key=pluginId ; value=InfosPlugin 
    
    // XML Tags
    private static final String FEATURE = "feature";
    private static final String DESCRIPTION = "description";
    private static final String REQUIRE = "requires";
    private static final String IMPORT = "import";
    private static final String IMAGE = "image";
    private static final String COLOCATION = "colocation-affinity";
    private static final String PRIMARY = "primary";
    private static final String EXCLUSIVE = "exclusive";
    private static final String APPLICATION = "application";

    // XML Attributs
    private static final String PLUGIN = "plugin";
    private static final String ID = "id";
    private static final String LABEL = "label";
    private static final String VERSION = "version";
    private static final String PROVIDER = "provider-name";
    private static final String DOWNLOADSIZE = "download-size";
    private static final String INSTALLSIZE = "install-size";
    private static final String FRAGMENT = "fragment";
    private static final String OS = "os";
    private static final String ARCH = "arch";
    private static final String WS = "ws";
    private static final String NL = "nl";
    private static final String UNPACK = "unpack";
    private static final String MATCH = "match";
    private static final String PATCH = "patch";
    private static final String DESCRIPTIONURL = "url";
    
    /**
     * Constructor
     * @param idFeature
     * @param labelFeature
     * @param versionFeature
     * @param providerNameFeature
     * @param imageFeature
     * @param osFeature
     * @param archFeature
     * @param wsFeature
     * @param nlFeature
     * @param colocationFeature
     * @param primaryFeature
     * @param exclusiveFeature
     * @param pluginFeature
     * @param applicationFeature
     */
    public InfosFeature(String nameFeature, String labelFeature, String versionFeature, String providerNameFeature, 
            String imageFeature, String osFeature, String archFeature, String wsFeature, String nlFeature, 
            String colocationFeature, String primaryFeature, String exclusiveFeature, String pluginFeature, 
            String applicationFeature){
        name = nameFeature;
        label = labelFeature;
        version = versionFeature;
        providerName = providerNameFeature;
        image = imageFeature;
        os = osFeature;
        arch = archFeature;
        ws = wsFeature;
        nl = nlFeature;
        colocation = colocationFeature;
        primary = primaryFeature;
        exclusive = exclusiveFeature;
        plugin = pluginFeature;
        application = applicationFeature;
    }
    
    
    /**
     * Constructor
     * @param featurePath : path of the feature.xml
     * @param propertyPath : path of the feature.property
     */
    public InfosFeature(String featurePath, String propertyPath){
        pathFeature = featurePath;
        pathFeatureProperty = propertyPath;
        getInfosFeature(pathFeature, pathFeatureProperty);
    }
    
    /**
     * take informations about the feature featurePath
     * @param featurePath
     * @param propertyPath 
     */
    private void getInfosFeature(String featurePath, String propertyPath){
        DocumentBuilderFactory factory =  DocumentBuilderFactory.newInstance();
        factory.setIgnoringComments(true);
        factory.setCoalescing(true);
        // Parser
        try{
            DocumentBuilder parser =factory.newDocumentBuilder();
            //Document
            Document document = parser.parse(new File(featurePath+File.separator));
            // <feature>
            NodeList featureList = document.getElementsByTagName(FEATURE);
            int nbNode = featureList.getLength();
            for (int i=0; i<nbNode; i++){
                Element node = (Element)featureList.item(i);
                name = node.getAttribute(ID);
                label = node.getAttribute(LABEL);
                version = node.getAttribute(VERSION);
                providerName = node.getAttribute(PROVIDER);
                image = node.getAttribute(IMAGE);
                os = node.getAttribute(OS);
                arch = node.getAttribute(ARCH);
                ws = node.getAttribute(WS);
                nl = node.getAttribute(NL);
                colocation = node.getAttribute(COLOCATION);
                primary = node.getAttribute(PRIMARY);
                exclusive = node.getAttribute(EXCLUSIVE);
                plugin = node.getAttribute(PLUGIN);
                application = node.getAttribute(APPLICATION);
                
                NodeList descriptionList = node.getElementsByTagName(DESCRIPTION);
                int nbDescription = descriptionList.getLength();
                description = null;
                for (int j=0; j<nbDescription; j++){
                    Element descriptionElement = (Element)descriptionList.item(j);
                    if (descriptionElement.hasChildNodes())
                        description = descriptionElement.getFirstChild().getNodeValue().trim();
                }
                
                // TODO : les autres balises? : <copyright> <licence> <url> <update> <discovery>
                
                
                // Property path
                File propertyFile = new File(propertyPath);
                if (propertyFile.exists()){
                    name = analyzeString(name, propertyFile);
                    label = analyzeString(label, propertyFile);
                    version = analyzeString(version, propertyFile);
                    providerName = analyzeString(providerName, propertyFile);
                    image = analyzeString(image, propertyFile);
                    os = analyzeString(os, propertyFile);
                    arch = analyzeString(arch, propertyFile);
                    ws = analyzeString(ws, propertyFile);
                    nl = analyzeString(nl, propertyFile);
                    colocation = analyzeString(colocation, propertyFile);
                    primary = analyzeString(primary, propertyFile);
                    exclusive = analyzeString(exclusive, propertyFile);
                    plugin = analyzeString(plugin, propertyFile);
                    application = analyzeString(application, propertyFile);
                    
                    description = analyzeString(description, propertyFile);
                }
            }
            
            // plugin requires (<requires> <import plugin ... version ...>)
            NodeList requiredList = document.getElementsByTagName(REQUIRE);
            int nbNodeReq = requiredList.getLength();
            for (int i=0; i<nbNodeReq; i++){
                Element node = (Element)requiredList.item(i);
                NodeList importList = node.getElementsByTagName(IMPORT);
                int nbImport = importList.getLength();
                for (int j=0; j<nbImport; j++){
                    Element importNode = (Element)importList.item(j);
                    String pluginRequired = importNode.getAttribute(PLUGIN);
                    String featureRequired = importNode.getAttribute(FEATURE);
                    String versionRequired = importNode.getAttribute(VERSION);
                    String matchRequired = importNode.getAttribute(MATCH);
                    String patchRequired = importNode.getAttribute(PATCH);
                    InfosImport infosImport = new InfosImport(pluginRequired, featureRequired, versionRequired, matchRequired, patchRequired, true);
                    mapRequiresImport.put(infosImport.getPluginImport(), infosImport);
                }
            }
            
            // <plugin id=... version=...> 
            NodeList pluginList = document.getElementsByTagName(PLUGIN);
            int nbNodePlugin = pluginList.getLength();
            for (int i=0; i<nbNodePlugin; i++){
                Element node = (Element)pluginList.item(i);
                String idP = node.getAttribute(ID);
                String versionP = node.getAttribute(VERSION);
                String fragment = node.getAttribute(FRAGMENT);
                String osP= node.getAttribute(OS);
                String archP = node.getAttribute(ARCH);
                String wsP = node.getAttribute(WS);
                String nlP = node.getAttribute(NL);
                String downloadSize = node.getAttribute(DOWNLOADSIZE);
                String installSize = node.getAttribute(INSTALLSIZE);
                String unpack = node.getAttribute(UNPACK);
                InfosPlugin infosPlugin = new InfosPlugin(idP, downloadSize, installSize, fragment, versionP, wsP, osP, archP, nlP, unpack);
                mapPlugins.put(infosPlugin.getIdPlugin(), infosPlugin);
            }
            
        }
        catch(ParserConfigurationException e){
            CorePlugin.log(this.getClass(), "ParserConfiguration error : " + e.getMessage(), IStatus.ERROR);
            
        }
        catch (SAXException e)
        {
            CorePlugin.log(this.getClass(), "Error while parsing document : " + e.getMessage(), IStatus.ERROR);
        }
        catch (IOException e)
        {
            CorePlugin.log(this.getClass(), "Error while opening file : " + e.getMessage() + ". Maybe is not a feature.", IStatus.ERROR);
        }
    }
    
    /**
     * Create  about the feature featurePath
     * @param featurePath
     */
    public void createFeatureProject(String featurePath){
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        try
        {
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document document = builder.newDocument();
            // Tag creation
            Element root = document.createElement(FEATURE);
            document.appendChild(root); 
            root.setAttribute(ID, name);
            root.setAttribute(VERSION, version);
            root.setAttribute(LABEL, label);
            root.setAttribute(PROVIDER, providerName);
            root.setAttribute(IMAGE, image);
            root.setAttribute(OS, os);
            root.setAttribute(ARCH, arch);
            root.setAttribute(WS, ws);
            root.setAttribute(NL, nl);
            root.setAttribute(COLOCATION, colocation);
            root.setAttribute(PRIMARY, primary);
            root.setAttribute(EXCLUSIVE, exclusive);
            root.setAttribute(PLUGIN, plugin);
            root.setAttribute(APPLICATION, application);
            
            // Description
            Element descriptionTag = document.createElement(DESCRIPTION);
            root.appendChild(descriptionTag);
            descriptionTag.setAttribute(DESCRIPTIONURL, descriptionUrl); 
            descriptionTag.appendChild(document.createTextNode(description));
            
            // TODO : les autres balises? : <copyright> <licence> <url> <update> <discovery>
                        
            // TAG <requires> <import ...>
            Iterator it = mapRequiresImport.keySet().iterator();
            if (it.hasNext()){
                Element requiresTag = document.createElement(REQUIRE);
                root.appendChild(requiresTag);
                while(it.hasNext()){
                    String key = (String)it.next();
                    if (mapRequiresImport.get(key) instanceof InfosImport){
                        InfosImport infosImport = (InfosImport)mapRequiresImport.get(key);
                        Element importTag = document.createElement(IMPORT);
                        root.appendChild(importTag);
                        importTag.setAttribute(PLUGIN, infosImport.getPluginImport());
                        importTag.setAttribute(VERSION, infosImport.getVersionImport());
                        importTag.setAttribute(MATCH, infosImport.getMatchImport());
                        importTag.setAttribute(PATCH, infosImport.getPatchImport());
                        importTag.setAttribute(FEATURE, infosImport.getFeatureImport());
                    }
                }
            }
            
            // TAG <plugin id=... version=...> 
            Iterator itPlug = mapPlugins.keySet().iterator();
            if (itPlug.hasNext()){
                while(itPlug.hasNext()){
                    Element pluginTag = document.createElement(PLUGIN);
                    root.appendChild(pluginTag);
                    String key = (String)itPlug.next();
                    if (mapPlugins.get(key) instanceof InfosPlugin){
                        InfosPlugin infosPlugin = (InfosPlugin)mapPlugins.get(key);
                        pluginTag.setAttribute(ID, infosPlugin.getIdPlugin());
                        pluginTag.setAttribute(VERSION, infosPlugin.getVersionPlugin());
                        pluginTag.setAttribute(FRAGMENT, infosPlugin.getFragmentPlugin());
                        pluginTag.setAttribute(OS, infosPlugin.getOsPlugin());
                        pluginTag.setAttribute(ARCH, infosPlugin.getArchPlugin());
                        pluginTag.setAttribute(WS, infosPlugin.getWsPlugin());
                        pluginTag.setAttribute(NL, infosPlugin.getNlPlugin());
                        pluginTag.setAttribute(DOWNLOADSIZE, infosPlugin.getDownloadSizePlugin());
                        pluginTag.setAttribute(INSTALLSIZE, infosPlugin.getInstallSizePlugin());
                        pluginTag.setAttribute(UNPACK, infosPlugin.getUnpackPlugin());
                    }
                }
            }
            
            // Xml write
            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer();
            // Indent the XML document
            transformer.setOutputProperty(OutputKeys.INDENT, "yes"); 
            transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "3"); 
            DOMSource sourceDom = new DOMSource(document);
            StreamResult result = new StreamResult(featurePath+File.separator+"feature.xml");
            transformer.transform(sourceDom, result);
            
        }
        catch (ParserConfigurationException e)
        {
            CorePlugin.log(this.getClass(), "Parse Configuration Exception" + e.getMessage(), IStatus.ERROR);
            
        }
        catch (TransformerConfigurationException e)
        {
            CorePlugin.log(this.getClass(), "Transformer Configuration Exception" + e.getMessage(), IStatus.ERROR);
        }
        catch (TransformerException e)
        {
            CorePlugin.log(this.getClass(), "Transformer Exception" + e.getMessage(), IStatus.ERROR);
        }
        catch (Exception e)
        {
            CorePlugin.log(this.getClass(), "Exception " + e.getMessage(), IStatus.ERROR);
        }

    }
    
    /**
     * Analyze String. 
     * If the String begin with '%', the string returned is the corresponding 
     * value define in pathFileProperty  else it is the String itself
     * @param property
     * @param pathFileProperty
     * @return String 
     * @throws IOException 
     */
    private String analyzeString(String property, File pathFileProperty) throws IOException{
        property = property.trim();
        if (!property.equals("")){
            String result = "";
            if (property.charAt(0)=='%'){
                property = property.substring(1);
                BufferedReader bufferedReader = new BufferedReader(new FileReader(pathFileProperty));
                String line = null;
                while ((line=bufferedReader.readLine())!=null && result.equals("")){
                    line = line.trim();
                    if (line.startsWith(property)){
                        result = line.substring(line.indexOf('=')+1);
                        while (result.endsWith("\\")){
                            result = result.substring(0,result.length()-1);
                            result = result.concat(bufferedReader.readLine());
                        }
                    }
                }
                bufferedReader.close();
                return result;
            }
            else
                return property;
        }
        return property;
    }
    
    /**
     * Get version
     * @return feature version
     */
    public String getVersionFeature(){
        return version;
    }
    
    /**
     * Get id
     * @return feature id
     */
    public String getNameFeature(){
        return name;
    }
    
    /**
     * Get provider
     * @return feature provider
     */
    public String getProviderFeature(){
        return providerName;
    }
    
    /**
     * Get description 
     * @return feature description
     */
    public String getDescriptionFeature(){
        return description;
    }
    
    /**
     * Get label
     * @return feature label
     */
    public String getLabelFeature(){
        return label;
    }
    
    /**
     * Get image
     * @return feature image
     */
    public String getImageFeature(){
        return image;
    }
    
    /**
     * Get os
     * @return feature os
     */
    public String getOsFeature(){
        return os;
    }
    
    /**
     * Get arch
     * @return feature arch
     */
    public String getArchFeature(){
        return arch;
    }
    
    /**
     * Get ws
     * @return feature ws
     */
    public String getWsFeature(){
        return ws;
    }
    
    /**
     * Get nl
     * @return feature nl
     */
    public String getNlFeature(){
        return nl;
    }
    
    /**
     * Get colocation
     * @return feature colocation
     */
    public String getColocationFeature(){
        return colocation;
    }
    
    /**
     * Get primary
     * @return feature primary
     */
    public String getPrimaryFeature(){
        return primary;
    }
    
    /**
     * Get exclusive
     * @return feature exclusive
     */
    public String getExclusiveFeature(){
        return exclusive;
    }
    
    /**
     * Get plugin
     * @return feature plugin
     */
    public String getPluginFeature(){
        return plugin;
    }

    /**
     * Get application
     * @return feature application
     */
    public String getApplicationFeature(){
        return application;
    }
    
    /**
     * Set description 
     * @param descriptionFeature 
     */
    public void setDescriptionFeature(String descriptionFeature){
        description = descriptionFeature;
    }
    
    /**
     * Set url description 
     * @param urlDescriptionFeature 
     */
    public void setUrlDescriptionFeature(String urlDescriptionFeature){
        descriptionUrl = urlDescriptionFeature;
    }
    
    /**
     * Get mapRequiresImport 
     * @return mapRequiresImport (key=plugin ; value=InfosImport)
     */
    public Map getMapRequiresImportFeature(){
        return mapRequiresImport;
    }
    
    /**
     * Get mapPlugins
     * @return mapPlugins (key=pluginId ; value=InfosPlugin)
     */
    public Map getMapPluginsFeature(){
        return mapPlugins;
    }
    
    /**
     * Add requiresImportMap (key=pluginId ; value=InfosPlugin) in mapRequiresImport
     * @param requiresImportMap (key=pluginId ; value=InfosPlugin)
     * @return mapRequiresImport (key=pluginId ; value=InfosPlugin)
     */
    public Map addAllMapRequiresImportFeature(Map requiresImportMap){
        //TODO verif si id existe deja. S'il existe garder la version la + ...
        mapRequiresImport.putAll(requiresImportMap);
        return mapRequiresImport;
    }
    
    /**
     * Add pluginsMap (key=pluginId ; value=InfosPlugin) in mapPlugins
     * @param pluginsMap  (key=pluginId ; value=InfosPlugin)
     * @return mapPlugins (key=pluginId ; value=InfosPlugin)
     */
    public Map addAllMapPluginsFeature(Map pluginsMap){
        //TODO verif si id existe deja. S'il existe garder la version la + ...
        mapPlugins.putAll(pluginsMap);
        return mapPlugins;
    }
}
