/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.updatesiteproject;


/**
 * SourceDestination Class enables to put and take 
 * the path source and path destination of a file or a directory 
 * and the state of the file as missing, still installed, will be installed see the wizard of UpdateSite 
 * <br>
 */
public class SourceDestination
{
    private String nameSD = null;

    private String sourceSD = null;

    private String destinationSD = null;

    private int stateSD = NO_STATE;
    
    /**
     * Define the type of the file as no type defined
     */
    public static final int NO_STATE = 0;
    /**
     * Define the type of the file as missing in the source and in the destination
     */
    public static final int MISSING = 1;
    /**
     * Define the type of the file as still present in the destination
     */
    public static final int STILL_EXIST = 2;
    /**
     * Define the type of the file as no present in the destination
     */
    public static final int DONT_EXIST = 3;
    
    /**
     * Constructor
     * @param name : name of the file or of the directory
     * @param source : source path
     * @param destination : destination path
     */
    public SourceDestination(String name, String source, String destination)
    {
        this.nameSD = new String(name);
        this.sourceSD = new String(source);
        this.destinationSD = new String(destination);
    }
    
    /**
     * Constructor with the state of file
     * @param name
     * @param source
     * @param destination
     * @param state
     */
    public SourceDestination(String name, String source, String destination, int state)
    {
        this(name, source, destination);
        this.stateSD=state;
    }
    
    /**
     * Set Source Destiantion Id
     * @param name : file or repository
     */
    public void setSourceDestinationId(String name)
    {
        this.nameSD = name;
    }
    
    /**
     * Set the source
     * @param source
     */
    public void setSource(String source)
    {
        this.sourceSD = source;
    }
    
    /**
     * Set the destination
     * @param destination
     */
    public void setDestination(String destination)
    {
        this.destinationSD = destination;
    }

    
    /**
     * Returns SourceDestination id 
     * 
     * @return name
     */
    public String getName()
    {
        return this.nameSD;
    }
    
    /**
     * Returns string of the source path 
     * 
     * @return source
     */
    public String getSource()
    {
        return this.sourceSD;
    }
    
    /**
     * Returns string of the destination path
     * 
     * @return destination
     */
    public String getDestination()
    {
        return this.destinationSD;
    }
    
    /**
     * @return Returns the stateSD.
     */
    public int getState() {
        return stateSD;
    }
    
    /**
     * @param stateSD The stateSD to set.
     */
    public void setState(int stateSD) {
        this.stateSD = stateSD;
    }
}
