/******************************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS)
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*********************************************************************************************/ 
package org.topcased.service.updatesiteproject;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;
import java.util.jar.JarOutputStream;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ftp.FtpException;
import org.eclipse.ftp.IDirectoryEntry;
import org.eclipse.ftp.internal.FTPClient;
import org.eclipse.ftp.internal.FTPProxyLocation;
import org.eclipse.ftp.internal.FTPServerLocation;
import org.eclipse.ftp.internal.IFTPClientListener;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IWorkbench;
import org.topcased.bus.core.CorePlugin;
import org.topcased.facilities.dialogs.ITopcasedWizardConstants;

import com.sshtools.j2ssh.SftpClient;
import com.sshtools.j2ssh.SshClient;
import com.sshtools.j2ssh.authentication.AuthenticationProtocolState;
import com.sshtools.j2ssh.authentication.KBIAuthenticationClient;
import com.sshtools.j2ssh.authentication.PasswordAuthenticationClient;
import com.sshtools.j2ssh.authentication.PublicKeyAuthenticationClient;
import com.sshtools.j2ssh.authentication.SshAuthenticationClient;
import com.sshtools.j2ssh.sftp.SftpFile;
import com.sshtools.j2ssh.transport.IgnoreHostKeyVerification;
import com.sshtools.j2ssh.transport.publickey.SshPrivateKey;
import com.sshtools.j2ssh.transport.publickey.SshPrivateKeyFile;

/**
 * 
 * This is the main class of the Wizard This class can search the features
 * included, plugins or plugins required to the features choosen This class can
 * copy/unjar the features/plugins to a destination This class can create a FTP
 * or SSH connections and give some information about them
 */
public class UpdateSiteProjectWizard extends Wizard implements ITopcasedWizardConstants
{
    // Path of the Update Site
    private String updateSitePath = "";

    private String projectPath = "";

    private String eclipseUpdateJpgPath = "";

    private String cplV10HtmlPath = "";

    private String licenceHtmlPath = "";

    private Map mapUS = new TreeMap(); // key=name; value=SourceDestination

    private Map mapPA = new TreeMap(); // key=name; value=SourceDestination

    private Map mapPAFeatureInclude = new HashMap();

    private Map mapFeatures = new HashMap(); // key=path; value=InfosFeature
    
    // The map containing the path of the license, image project and html page
    private Map filePathMap = new HashMap(); 
    
    private InfosFeature infosFeatureProject;

    // Project feature information
    private String nameFeatureProject = "";

    private String labelFeatureProject = "";

    private String versionFeatureProject = "";

    private String providerNameFeatureProject = "";

    private String imageFeatureProject = "";

    private String osFeatureProject = "";

    private String archFeatureProject = "";

    private String wsFeatureProject = "";

    private String nlFeatureProject = "";

    private String colocationFeatureProject = "";

    private String primaryFeatureProject = "";

    private String exclusiveFeatureProject = "";

    private String pluginFeatureProject = "";

    private String applicationFeatureProject = "";

    private String descriptionFeatureProject = "";

    private String urlDescriptionFeatureProject = "";

    /**
     * FTP - Identifier of connection host protocol
     */
    public static final int FTP = 0;

    /**
     * SSH - Identifier of connection host protocol
     */
    public static final int SSH = 1;
    
    /**
     * FILE - Identifier of connection host protocol
     */
    public static final int FILE = 2;
    
	/**
	 * PRIVATE_KEY_PATH - If a SSH connection is used 
	 */
    public static final String PRIVATE_KEY_PATH = "privateKeyPath";
    
    private IPreferenceStore preferenceStore = UpdateSiteProjectPlugin.getDefault().getPreferenceStore();
    
    // Connection attributs
    private String username = "";

    private int protocolConnexion = 0;

    private String password = "";

    private int port = 0;

    private String host = "";
    
    // Page attributs
    private ConnectUpdateSiteProjectPage connectPage;

    private BrowseUpdateSiteProjectPage browsePage;

    private FeatureUpdateSiteProjectPage featurePage;

    private FTPClient clientFTP;

    public String error;

    private SshClient clientSSH;

    private SftpClient clientSFTP;
    
    private Vector IOErrorVector;

    public UpdateSiteProjectWizard()
    {
        setWindowTitle("TOPCASED : Update Site Project");
        
        setDefaultPageImageDescriptor(UpdateSiteProjectPlugin.getImageDescriptor(TOPCASED_LOGO_PATH));
    }
    
    /**
     * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
     *      org.eclipse.jface.viewers.IStructuredSelection)
     */
    public void init(IWorkbench workbench, IStructuredSelection selection)
    {
        if (updateSitePath.startsWith(File.separator))
        {
            updateSitePath = updateSitePath.substring(1);
        }

        updateSitePath = updateSitePath.replace('\\', (File.separator).charAt(0));
        updateSitePath = updateSitePath.replace('/', (File.separator).charAt(0));
    }

    /**
     * This method is aimed to solve path problems independantly of the system.
     * First, the good file separator is added at the end of the path.
     * Secondly, the function check that all the file separator contained are correct. 
     * If it is not the case, they are replaced by the good one.
     * @param pathToCheck The string to evaluate
     * @param pathNature The string nature : true for a local path, false for a distant path.
     * @return The new path corrected.
     */
    public static String checkPath(String pathToCheck, boolean pathNature)
    {
    	if (pathNature)
        {
    		// check the path according to the plateform and replace character wether it is necessary
   			pathToCheck = pathToCheck.replace('/', File.separatorChar);
   			pathToCheck = pathToCheck.replace('\\', File.separatorChar);
        	// add file separator to the end of string 
    		if (!pathToCheck.endsWith(File.separator))
        	{
        		pathToCheck += File.separator;
	        }  
        }
        else
        {
        	pathToCheck = pathToCheck.replace('\\', '/');
        	if (!pathToCheck.endsWith("/"))
        	{
        		pathToCheck += "/";
	        }
        }   
    	return pathToCheck;
    }
    
    /**
     * Function executed when the button 'Cancel' is clicked
     * 
     * @return boolean true if the wizard is canceled, false otherwise
     */
    public boolean performCancel()
    {
    	disconnect();
    	return super.performCancel();
    }
    
    /**
     * @return Returns the iOErrorVector.
     */
    public Vector getIOErrorVector()
    {
        return IOErrorVector;
    }

    /**
     * Add all the necessary pages of the wizard
     */
    public void addPages()
    {
        connectPage = new ConnectUpdateSiteProjectPage(this, "Connection Page");
        connectPage.setTitle("Information for establishing a distant connection");
        connectPage.setDescription("Enter the required information to activate the connection.");
        addPage(connectPage);

        browsePage = new BrowseUpdateSiteProjectPage(this, "Browse Page");
        browsePage.setTitle("Browse Page");
        browsePage.setDescription("Choose the features to install.");
        addPage(browsePage);

        featurePage = new FeatureUpdateSiteProjectPage(this, "Feature Page");
        featurePage.setTitle("Information about the feature");
        featurePage.setDescription("Add required and optional information to your feature description.");
        addPage(featurePage);
    }

    /**
     * @see org.eclipse.jface.wizard.IWizard#performFinish()
     */
    public boolean performFinish()
    {
        IOErrorVector = new Vector();
        
        if (!featurePage.updateError())
        {
            return false;
        }

    	// Download the features 
		String tempDir = UpdateSiteProjectWizard.checkPath(System.getProperty("java.io.tmpdir"), true);
        
        /***************
         * FTP Protocol
         ***************/
        if (connectPage.getProtocol() == FTP)
    	{
			Iterator it = mapPA.keySet().iterator();
			while (it.hasNext())
			{
				String key = (String) it.next();
				if (mapPA.get(key) instanceof SourceDestination)
				{
					SourceDestination sd = (SourceDestination) mapPA.get(key);
					String pathDestTemp = UpdateSiteProjectWizard.checkPath(tempDir + "features", true);
					String path = UpdateSiteProjectWizard.checkPath(browsePage.getDirectory() + "features", getProtocolConnexion() == FILE);

					try
					{
						IDirectoryEntry[] files = clientFTP.listFiles(path, false, null);
						if (files.length == 0)
						{
							// end system is different from the source system
							if (File.separator.equals("\\"))
                            {
                                files = clientFTP.listFiles(path = path.replace('\\', '/'), false, null);
                            }
                            else
                            {
                                files = clientFTP.listFiles(path = path.replace('/', '\\'), false, null);
                            }
						}
                
						for(int i=0; i<files.length; i++)
						{
							if(files[i].getName().equals(key))
							{
								FilesTreatment.copyPluginFeatureFromFTP(path, pathDestTemp, files[i], clientFTP, null);
								sd.setSource(pathDestTemp+key);
							}
						}
					}
					catch (FtpException e)
					{
						CorePlugin.log(e);
					}
					catch (IOException e)
					{
						CorePlugin.log(e);
					}
				}
			}
        }
        
        /**************
         * SSH Protocol 
         **************/
        if (connectPage.getProtocol() == SSH)
        {
			String pathDestTemp = UpdateSiteProjectWizard.checkPath(tempDir + "features", true);
			String path = UpdateSiteProjectWizard.checkPath(browsePage.getDirectory() + "features", getProtocolConnexion() == FILE);
			List files = null;
	       
			try
	        {
	            files = clientSFTP.ls(path);
	        }
	        catch (IOException e)
	        {
	        	CorePlugin.log(e);
	        }
			
			Iterator it = mapPA.keySet().iterator();
			while (it.hasNext())
			{
				String key = (String) it.next();
				if (mapPA.get(key) instanceof SourceDestination)
				{
					SourceDestination sd = (SourceDestination) mapPA.get(key);
					
					try{
            	
						for(int i=0; i<files.size(); i++)
						{
							if(((SftpFile) files.get(i)).getFilename().equals(key))
							{
								FilesTreatment.copyPluginFeatureFromSFTP((SftpFile) files.get(i), UpdateSiteProjectWizard.checkPath(tempDir + "features", true), clientSFTP);
								sd.setSource(pathDestTemp + key);
							}
						}
					}
					catch (IOException e)
					{
						CorePlugin.log(e);
					}
				}
			}
        }
        
        IRunnableWithProgress runnableWithProgress = new IRunnableWithProgress()
        {
            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
            {
                if (monitor.isCanceled())
                {
                    monitor.done();
                }

                monitor.beginTask("Installation in progress...", 10);
                monitor.subTask("Initialization with chosen features ...");
                monitor.worked(1);

                Iterator it = mapPA.keySet().iterator();
                while (it.hasNext())
                {
                    String key = (String) it.next();
                    if (mapPA.get(key) instanceof SourceDestination)
                    {
                        FilesTreatment.autoState((SourceDestination) mapPA.get(key), FilesTreatment.FEATURE_XML);
                    }
                }

                String path = UpdateSiteProjectWizard.checkPath(projectPath + "features", true); 
                monitor.subTask("Unjar the chosen features ...");
                monitor.worked(1);
                installJarProject(path, mapPA);
                monitor.subTask("Unjar the features included...");
                monitor.worked(1);
                installJarProject(path, mapPAFeatureInclude);
                monitor.subTask("Create the project feature...");
                monitor.worked(1);
                createProjectFeature(projectPath);
            }
        };
        boolean retryTreatment;
        
        do
        {
            IOErrorVector = new Vector();
            retryTreatment =false;
            ProgressMonitorDialog dialog = new ProgressMonitorDialog(getShell());
            try
            {
                dialog.run(true, true, runnableWithProgress);
            }
            catch (InvocationTargetException e)
            {
            	
            	CorePlugin.log(e);
                error = e.getMessage();
            }
            catch (InterruptedException e)
            {
            	CorePlugin.log(e);
                error = e.getMessage();
            }
            
            if (!IOErrorVector.isEmpty())
            {
                int button;
                Status[] status = new Status[IOErrorVector.size()];
                for (int i=0; i<IOErrorVector.size(); i++)
                {
                    status[i] = new Status(IStatus.ERROR, "org.topcased.service.updatesiteproject", 0, (String)IOErrorVector.get(i), null);
                }
                ErrorDialog errorBox = new ErrorDialog(getShell(), "IO Error", "An error occured during the copy", 
                        new MultiStatus("org.topcased.service.updatesiteproject", 0, status,"See details", null), IStatus.ERROR);
                errorBox.open();
                
                MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_WARNING | SWT.IGNORE | SWT.RETRY | SWT.ABORT);
                msgBox.setText("IO Error");
                msgBox.setMessage("If you continue, maybe some files will be not copied.\n\nDo you want to continue ?");
                button = msgBox.open();
                switch (button)
                {
                    case (SWT.ABORT):
                        IRunnableWithProgress runnableWithProgress2 = new IRunnableWithProgress()
                        {
                            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
                            {
                                monitor.beginTask("Deleting files...", IProgressMonitor.UNKNOWN);
                                FilesTreatment.emptyFilesFromMap(mapPA);
                                FilesTreatment.emptyFilesFromMap(mapPAFeatureInclude);
                                String tempToDel = UpdateSiteProjectWizard.checkPath(projectPath, true) + nameFeatureProject + "_" + versionFeatureProject;
                                File suppFile = new File(tempToDel + ".jar");
                                suppFile.delete();
                                File tmpFile = new File(tempToDel + ".jar.tmp");
                                if (tmpFile.exists())
                                {
                                    tmpFile.renameTo(new File(tempToDel + ".jar"));
                                }
                            }
                        };
                        ProgressMonitorDialog dialog2 = new ProgressMonitorDialog(getShell());
                        try
                        {
                            dialog2.run(true, true, runnableWithProgress2);
                        }
                        catch (InvocationTargetException e)
                        {
                        	CorePlugin.log(e);
                        }
                        catch (InterruptedException e)
                        {
                        	CorePlugin.log(e);
                        }
                        return false;
                    case (SWT.RETRY):
                        // Retry to add the sub repository in Map of Update Site
                        retryTreatment = true;
                        break;
                    case (SWT.IGNORE):
                        return true;
                }
            }
            else
            {
                File suppFile = new File(UpdateSiteProjectWizard.checkPath(projectPath, true)  + nameFeatureProject + "_" + versionFeatureProject + ".jar.tmp");
                suppFile.delete();
            }
        } 
        while (retryTreatment);

        // Cleaning here because deleteOnExit seems not to work properly.
		//String tempDir = checkPath(System.getProperty("java.io.tmpdir"), true);
        FilesTreatment.emptyFile(new File(tempDir + "features"));
        //FilesTreatment.emptyFile(new File(tempDir + "plugins"));
        //FilesTreatment.emptyFile(new File(tempDir + "site.xml"));
        
        // Disconnect the protocol (for FTP and SSH)
        disconnect();
        return true;
    }

    /**
     * @see org.eclipse.jface.wizard.Wizard#canFinish()
     */
    public boolean canFinish()
    {
    	if (connectPage.finish() && browsePage.finish() && featurePage.finish())
            return true;
        else
        	return false;
    }

    /**
     * @return the connection page
     */
    public ConnectUpdateSiteProjectPage getConnectPage()
    {
        return connectPage;
    }

    /**
     * @return Returns the browsePage.
     */
    public BrowseUpdateSiteProjectPage getBrowsePage()
    {
        return browsePage;
    }

    /**
     * @return Returns the host.
     */
    public String getHost()
    {
        return host;
    }

    /**
     * @param host The host to set.
     */
    public void setHost(String host)
    {
        this.host = host;
    }

    /**
     * @return Returns the port.
     */
    public int getPort()
    {
        return port;
    }

    /**
     * @param port The port to set.
     */
    public void setPort(int port)
    {
        this.port = port;
    }

    /**
     * @return Returns the password.
     */
    public String getPassword()
    {
        return password;
    }

    /**
     * @param password The password to set.
     */
    public void setPassword(String password)
    {
        this.password = password;
    }

    /**
     * @return Returns the protocolConnexion.
     */
    public int getProtocolConnexion()
    {
        return protocolConnexion;
    }

    /**
     * @param protocolConnexion The protocolConnexion to set.
     */
    public void setProtocolConnexion(int protocolConnexion)
    {
        this.protocolConnexion = protocolConnexion;
    }

    /**
     * @return Returns the username.
     */
    public String getUsername()
    {
        return username;
    }

    /**
     * @param username The username to set.
     */
    public void setUsername(String username)
    {
        this.username = username;
    }

    /**
     * @return the update site path
     */
    public String getUpdateSitePath()
    {
        return updateSitePath;
    }

    /**
     * @param path the update site path to set
     */
    public void setUpdateSitePath(String path)
    {
    	updateSitePath = path;
    }

    /**
     * @return the project path
     */
    public String getProjectPath()
    {
        return projectPath;
    }

    /**
     * @param path the project path to set
     */
    public void setProjectPath(String path)
    {
    	projectPath = path;
    }

    /**
     * @return Returns the cplV10HtmlPath.
     */
    public String getCplV10HtmlPath()
    {
        return cplV10HtmlPath;
    }

    /**
     * @param cplV10HtmlPath The cplV10HtmlPath to set.
     */
    public void setCplV10HtmlPath(String cplV10HtmlPath)
    {
        this.cplV10HtmlPath = cplV10HtmlPath;
    }

    /**
     * @return Returns the eclipseUpdateJpgPath.
     */
    public String getEclipseUpdateJpgPath()
    {
        return eclipseUpdateJpgPath;
    }

    /**
     * @param eclipseUpdateJpgPath The eclipseUpdateJpgPath to set.
     */
    public void setEclipseUpdateJpgPath(String eclipseUpdateJpgPath)
    {
        this.eclipseUpdateJpgPath = eclipseUpdateJpgPath;
    }

    /**
     * @return Returns the licenceHtmlPath.
     */
    public String getLicenceHtmlPath()
    {
        return licenceHtmlPath;
    }

    /**
     * @param licenceHtmlPath The licenceHtmlPath to set.
     */
    public void setLicenceHtmlPath(String licenceHtmlPath)
    {
        this.licenceHtmlPath = licenceHtmlPath;
    }

    /**
     * @return Returns the applicationFeatureProject.
     */
    public String getApplicationFeatureProject()
    {
        return applicationFeatureProject;
    }

    /**
     * @param applicationFeatureProject The applicationFeatureProject to set.
     */
    public void setApplicationFeatureProject(String applicationFeatureProject)
    {
        this.applicationFeatureProject = applicationFeatureProject;
    }

    /**
     * @return Returns the archFeatureProject.
     */
    public String getArchFeatureProject()
    {
        return archFeatureProject;
    }

    /**
     * @param archFeatureProject The archFeatureProject to set.
     */
    public void setArchFeatureProject(String archFeatureProject)
    {
        this.archFeatureProject = archFeatureProject;
    }

    /**
     * @return Returns the colocationFeatureProject.
     */
    public String getColocationFeatureProject()
    {
        return colocationFeatureProject;
    }

    /**
     * @param colocationFeatureProject The colocationFeatureProject to set.
     */
    public void setColocationFeatureProject(String colocationFeatureProject)
    {
        this.colocationFeatureProject = colocationFeatureProject;
    }

    /**
     * @return Returns the descriptionFeatureProject.
     */
    public String getDescriptionFeatureProject()
    {
        return descriptionFeatureProject;
    }

    /**
     * @param descriptionFeatureProject The descriptionFeatureProject to set.
     */
    public void setDescriptionFeatureProject(String descriptionFeatureProject)
    {
        this.descriptionFeatureProject = descriptionFeatureProject;
    }

    /**
     * @return Returns the exclusiveFeatureProject.
     */
    public String getExclusiveFeatureProject()
    {
        return exclusiveFeatureProject;
    }

    /**
     * @param exclusiveFeatureProject The exclusiveFeatureProject to set.
     */
    public void setExclusiveFeatureProject(String exclusiveFeatureProject)
    {
        this.exclusiveFeatureProject = exclusiveFeatureProject;
    }

    /**
     * @return Returns the idFeatureProject.
     */
    public String getNameFeatureProject()
    {
        return nameFeatureProject;
    }

    /**
     * @param idFeatureProject The idFeatureProject to set.
     */
    public void setNameFeatureProject(String idFeatureProject)
    {
        this.nameFeatureProject = idFeatureProject;
    }
    
    /**
     * @return Returns the versionFeatureProject.
     */
    public String getVersionFeatureProject()
    {
        return versionFeatureProject;
    }

    /**
     * @param versionFeatureProject The versionFeatureProject to set.
     */
    public void setVersionFeatureProject(String versionFeatureProject)
    {
        this.versionFeatureProject = versionFeatureProject;
    }
    
    /**
     * @return Returns the imageFeatureProject.
     */
    public String getImageFeatureProject()
    {
        return imageFeatureProject;
    }

    /**
     * @param imageFeatureProject The imageFeatureProject to set.
     */
    public void setImageFeatureProject(String imageFeatureProject)
    {
        this.imageFeatureProject = imageFeatureProject;
    }

    /**
     * @return Returns the labelFeatureProject.
     */
    public String getLabelFeatureProject()
    {
        return labelFeatureProject;
    }

    /**
     * @param labelFeatureProject The labelFeatureProject to set.
     */
    public void setLabelFeatureProject(String labelFeatureProject)
    {
        this.labelFeatureProject = labelFeatureProject;
    }

    /**
     * @return Returns the nlFeatureProject.
     */
    public String getNlFeatureProject()
    {
        return nlFeatureProject;
    }

    /**
     * @param nlFeatureProject The nlFeatureProject to set.
     */
    public void setNlFeatureProject(String nlFeatureProject)
    {
        this.nlFeatureProject = nlFeatureProject;
    }

    /**
     * @return Returns the osFeatureProject.
     */
    public String getOsFeatureProject()
    {
        return osFeatureProject;
    }

    /**
     * @param osFeatureProject The osFeatureProject to set.
     */
    public void setOsFeatureProject(String osFeatureProject)
    {
        this.osFeatureProject = osFeatureProject;
    }

    /**
     * @return Returns the pluginFeatureProject.
     */
    public String getPluginFeatureProject()
    {
        return pluginFeatureProject;
    }

    /**
     * @param pluginFeatureProject The pluginFeatureProject to set.
     */
    public void setPluginFeatureProject(String pluginFeatureProject)
    {
        this.pluginFeatureProject = pluginFeatureProject;
    }

    /**
     * @return Returns the primaryFeatureProject.
     */
    public String getPrimaryFeatureProject()
    {
        return primaryFeatureProject;
    }

    /**
     * @param primaryFeatureProject The primaryFeatureProject to set.
     */
    public void setPrimaryFeatureProject(String primaryFeatureProject)
    {
        this.primaryFeatureProject = primaryFeatureProject;
    }

    /**
     * @return Returns the providerNameFeatureProject.
     */
    public String getProviderNameFeatureProject()
    {
        return providerNameFeatureProject;
    }

    /**
     * @param providerNameFeatureProject The providerNameFeatureProject to set.
     */
    public void setProviderNameFeatureProject(String providerNameFeatureProject)
    {
        this.providerNameFeatureProject = providerNameFeatureProject;
    }

    /**
     * @return Returns the urlDescriptionFeatureProject.
     */
    public String getUrlDescriptionFeatureProject()
    {
        return urlDescriptionFeatureProject;
    }

    /**
     * @param urlDescriptionFeatureProject The urlDescriptionFeatureProject to
     *            set.
     */
    public void setUrlDescriptionFeatureProject(String urlDescriptionFeatureProject)
    {
        this.urlDescriptionFeatureProject = urlDescriptionFeatureProject;
    }

    /**
     * @return Returns the wsFeatureProject.
     */
    public String getWsFeatureProject()
    {
        return wsFeatureProject;
    }

    /**
     * @param wsFeatureProject The wsFeatureProject to set.
     */
    public void setWsFeatureProject(String wsFeatureProject)
    {
        this.wsFeatureProject = wsFeatureProject;
    }

    /**
     * @return the feature map of the update site
     */
    public Map getMapUS()
    {
        return mapUS;
    }

    /**
     * @return the feature map of the project location
     */
    public Map getMapPA()
    {
        return mapPA;
    }

    /**
     * @return Returns the mapFeatures.
     */
    public Map getMapFeatures()
    {
        return mapFeatures;
    }

    /**
     * @return Returns the clientFTP.
     */
    public FTPClient getClientFTP()
    {
        return clientFTP;
    }

    /**
     * @param clientFTP The clientFTP to set.
     */
    public void setClientFTP(FTPClient clientFTP)
    {
        this.clientFTP = clientFTP;
    }

    /**
     * @return Returns the clientSFTP.
     */
    public SftpClient getClientSFTP()
    {
        return clientSFTP;
    }

    /**
     * @param clientSFTP The clientSFTP to set.
     */
    public void setClientSFTP(SftpClient clientSFTP)
    {
        this.clientSFTP = clientSFTP;
    }

    /**
     * @return Returns the clientSSH.
     */
    public SshClient getClientSSH()
    {
        return clientSSH;
    }

    /**
     * @param clientSSH The clientSSH to set.
     */
    public void setClientSSH(SshClient clientSSH)
    {
        this.clientSSH = clientSSH;
    }

    /**
     * create a FTP connection to the remote FTP server
     */
    public void createConnectionFTP()
    {
        /***********************************************************************
         * FTP Connexion
         **********************************************************************/

        FTPServerLocation ftpServer;
        FTPProxyLocation ftpProxy;

        URL URLServer = null;
		try
		{
			URLServer = new URL("FTP", host, port, "");
		}
		catch (MalformedURLException e1)
		{
			CorePlugin.log(e1);
		}
        ftpServer = new FTPServerLocation(URLServer);
        ftpServer.setAuthentication(username, password);


        ftpProxy = null;

        clientFTP = new FTPClient(ftpServer, ftpProxy, new IFTPClientListener()
        {
            public void responseReceived(int responseCode, String responseText)
            {
                // Nothing
            }

            public void requestSent(String command, String argument)
            {
                // Nothing
            }
        });

        IRunnableWithProgress runnableWithProgress = new IRunnableWithProgress()
        {
            public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
            {
                error = "";
                try
                {
                    clientFTP.open(monitor);
                    
                }
                catch (FtpException e)
                {
                	CorePlugin.log(e);
                    error = e.getMessage();
                }

            }
        };
        ProgressMonitorDialog dialog = new ProgressMonitorDialog(getShell());
        try
        {
            dialog.run(true, true, runnableWithProgress);
        }
        catch (InvocationTargetException e)
        {
        	CorePlugin.log(e);
            error = e.getMessage();
        }
        catch (InterruptedException e)
        {
        	CorePlugin.log(e);
            error = e.getMessage();
        }

        if (clientFTP.isOpen())
        {
            clientFTP.setDataTransferMode(FTPClient.PASSIVE_DATA_TRANSFER, true);
            clientFTP.setTimeout(3600);
            MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_INFORMATION | SWT.OK);
            msgBox.setText("Connection information");
            msgBox.setMessage("Connection successfully completed\nHost : " + ftpServer.getHostname() + "\nUsername : " + ftpServer.getUsername());
            msgBox.open();
        }
        else
        {
            if (!error.equals(""))
            {
                MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_ERROR | SWT.OK);
                msgBox.setText("Connection error");
                msgBox.setMessage("Error in the connection : " + error + "\nSee Console information");
                msgBox.open();
            }
        }
    }

    /**
     * This method authenticates a client according to the choosen method (public key, interactive keyboard, password)
     * @param client represents the current method authentication tried by a client
     * @return returns true is the user is authenticated, false if the authentication method has failed.
     */
    public boolean userAuthenticationSHH(SshAuthenticationClient client)
    {
        try
        {
    		int result = clientSSH.authenticate(client);
    		
        	if (result == AuthenticationProtocolState.COMPLETE)
            {
                MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_INFORMATION | SWT.OK);
                msgBox.setText("SSH Authentication Information");
                msgBox.setMessage("Authentication completed for user " + username);
                msgBox.open();
                clientSFTP = clientSSH.openSftpClient();
                return true;
            }
        }
        catch (IOException e1)
        {
            MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_ERROR | SWT.OK);
            msgBox.setText("SSH Authentication Error");
            msgBox.setMessage("Error during the connection : " + e1.getMessage() + "\nSee Console information");
            msgBox.open();
            CorePlugin.log(e1);
        }
        return false;
    }
    
    /**
     * This method establishes a SSH connection in two major steps : 
     * - key exchange between the client and the server
     * - authentication of a user using one of the three methods allowed
     * @return returns true if the connection is established for a given user, false if the connection failed
     */
    public boolean createConnectionSSH()
    {
        /***********************************************************************
         * SSH Connexion
         **********************************************************************/
    	boolean authenticated = false;
        clientSSH = new SshClient();

        try
        {
        	clientSSH.connect(host, port, new IgnoreHostKeyVerification());
        	List authenticationMethods = clientSSH.getAvailableAuthMethods(username);
        	String privateKeyPath = preferenceStore.getString(PRIVATE_KEY_PATH);
        	
        	// First method : public key
        	if (authenticationMethods.contains("publickey") && !privateKeyPath.equals(""))
        	{
        		CorePlugin.log(this.getClass(),"[SSH] Trying to authenticate " + username +" with public key method", IStatus.INFO);
        		PublicKeyAuthenticationClient pk = new PublicKeyAuthenticationClient();
	        	pk.setUsername(username);
	            // Open up the private key file
	            
	            try
	            {
	            	SshPrivateKeyFile file = SshPrivateKeyFile.parse(new File(privateKeyPath));
		            // Get the key
		            SshPrivateKey key = file.toPrivateKey(null);
		            pk.setKey(key);
		            // launch the authentication
		            authenticated = userAuthenticationSHH(pk);
	            }
	            catch (FileNotFoundException fnfe)
	            {
	            	CorePlugin.log(this.getClass(),"[SSH] Your private key has not been found in " + privateKeyPath, IStatus.ERROR);
	            	CorePlugin.log(this.getClass(),"[SSH] Check or change your private key path in Window > Preferences > Topcased > SSH Connection", IStatus.WARNING);
	            }
	            catch (IOException ioe)
	            {
	            	CorePlugin.log(this.getClass(),"[SSH] IO error while reading your private key", IStatus.ERROR);
	            }
	            
	            // Logs for console
	            if (authenticated)
	            {
	            	CorePlugin.log(this.getClass(),"[SSH] User " + username +" successfully authenticated by public key", IStatus.OK);
	            }
	            else
	            {
	            	CorePlugin.log(this.getClass(),"[SSH] Authentication by public key has failed for user " + username , IStatus.ERROR);
	            }
        	 }
        	 
        	 // Second method : keyboard interactive authentication (using a Dialog for the password) 
        	 if (authenticationMethods.contains("keyboard-interactive") && !authenticated) 
        	 {
        		 CorePlugin.log(this.getClass(),"[SSH] Trying to authenticate " + username +" with interactive keyboard method", IStatus.INFO); 
        		 KBIAuthenticationClient kbi = new KBIAuthenticationClient();
        	     kbi.setUsername(username);
        	     //kbi.setKBIRequestHandler(new UpdateSiteKBIRequestHandlerDialog(getShell())); for a graphical use !!!
        	     kbi.setKBIRequestHandler(new UpdateSiteProjectKBIRequestHandler(this));
        	     // Launch the keyboard authentication + logs in console 
        	     if (authenticated = userAuthenticationSHH(kbi))
		         {
        	    	 CorePlugin.log(this.getClass(),"[SSH] User " + username +" successfully authenticated by interactive keyboard", IStatus.OK);
		         }
        	     else
        	     {
        	    	 CorePlugin.log(this.getClass(),"[SSH] Authentication by keyboard interactive has failed for user " + username , IStatus.ERROR);
        	     }
        	 }
        	 
        	 // Third and last method : password authentication
        	 if (authenticationMethods.contains("password") && !authenticated)
        	 {
        		 CorePlugin.log(this.getClass(),"[SSH] Trying to authenticate " + username +" with password method", IStatus.INFO);                 
        		 PasswordAuthenticationClient pwd = new PasswordAuthenticationClient();
                 pwd.setUsername(username);
                 pwd.setPassword(password);
        	     // Launch the password authentication + logs in console 
        	     if (authenticated = userAuthenticationSHH(pwd))
		         {
        	    	 CorePlugin.log(this.getClass(),"[SSH] User " + username +" successfully authenticated by password", IStatus.OK);
		         }
        	     else
        	     {
        	    	 CorePlugin.log(this.getClass(),"[SSH] Authentication by password has failed for user " + username , IStatus.ERROR);
        	     }                 
        	 }
        	 
        	 if(!authenticated)
        	 {
                 // At this level, the 3 authentication methods have failed for this user : impossible tio authenticate this user 
        		 CorePlugin.log(this.getClass(),"[SSH] All the available SSH authentication methods have failed : impossible to authenticate user " + username, IStatus.ERROR);
        		 MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_ERROR | SWT.OK);
                 msgBox.setText("SSH Authentication Error");
                 msgBox.setMessage("Authentification failure for user " + username);
                 msgBox.open();
        	 }
        }
        catch (IOException e)
        {
            MessageBox msgBox = new MessageBox(getShell(), SWT.ICON_ERROR | SWT.OK);
            msgBox.setText("Connection error");
            msgBox.setMessage("Error during the SSH connection : " + e.getMessage() + "\nSee Console for more information");
            msgBox.open();
            CorePlugin.log(this.getClass(),"[SSH] IO error while establishing the SSH connection", IStatus.ERROR);
        }
        return authenticated;
    }

    /**
    * This method disconnect a client form a remote FTP or SSH server.
    * First, the connection state is checked with the method isConnected().
    */
    public void disconnect()
    {
        if (isConnected())
        {
        	switch (protocolConnexion)
        	{
            	case (FTP):
                    try
                    {
                        clientFTP.close(null);
                        CorePlugin.log(this.getClass(),"[FTP] Remote session closed for user " + username, IStatus.OK);
                    }
                    catch (FtpException e)
                    {
                    	CorePlugin.log(e);
                    }
	                break;
	            case (SSH):
	                clientSSH.disconnect();
	            	CorePlugin.log(this.getClass(),"[SSH] Remote session closed for user " + username, IStatus.OK);
	            default:
	                break;
        	}
        }
    }

    /**
     * @return true if the wizard is connected to a remote server, false
     *         otherwise
     */
    public boolean isConnected()
    {
        boolean ret;

        switch (protocolConnexion)
        {
            case (FTP):
                if (clientFTP != null)
                {
                    ret = clientFTP.isOpen();
                }
                else
                {
                    ret = false;
                }
                break;
            case (SSH):
                if (clientSSH != null)
                {
                    ret = clientSSH.isConnected() && clientSSH.isAuthenticated();
                }
                else
                {
                    ret = false;
                }
                break;
            default:
                ret = false;
                break;
        }
        return ret;
    }

    /**
     * Install Project : unJar all jarFile(features) selected (mapJar) in the
     * parojectPath
     * 
     * @param pathProject
     * @param mapJar (key=name; value=SourceDestination)
     */
    private void installJarProject(String pathProject, Map mapJar)
    {

        if (!projectPath.equals(""))
        {
        	
            Iterator it = mapJar.keySet().iterator();
            while (it.hasNext())
            {
                String key = (String) it.next();
                if (mapJar.get(key) instanceof SourceDestination)
                {
                    SourceDestination sd = (SourceDestination) mapJar.get(key);
                    String pathJarFeature = sd.getSource();
                    
                    File pathFile = new File(pathJarFeature);
                    try
                    {
                        // UnJar all files
                        JarInputStream input = new JarInputStream(new FileInputStream(pathFile));

                        String jarEntryName = null;
                        JarEntry jarEntry = null;
                        while ((jarEntry = input.getNextJarEntry()) != null)
                        {
                            jarEntryName = jarEntry.getName();

                            if (!jarEntry.isDirectory())
                            {
                                String featureName = sd.getName();
                                featureName = featureName.substring(0, featureName.indexOf(".jar"));
                                // Extract
                                String fileName = jarEntryName.toString(); // ???

                                File fileOutput = new File(pathProject + featureName + File.separator + fileName);
                                // Creation of parent repository if does not exist
                                File repositoryParent = fileOutput.getParentFile();
                                repositoryParent.mkdirs();

                                FileOutputStream output = new FileOutputStream(fileOutput);
                                byte[] buffer = new byte[1024];
                                int length;
                                while ((length = input.read(buffer)) != -1)
                                {
                                    output.write(buffer, 0, length);
                                }
                                output.close();
                            }
                        }
                        input.close();
                        CorePlugin.log(this.getClass(), "UnJar " + sd.getName() + " in " + pathProject, IStatus.INFO);
                    }
                    catch (FileNotFoundException e)
                    {
                        CorePlugin.log(this.getClass(), "File Not Found Exception for " + e.getMessage(), IStatus.ERROR);
                    }
                    catch (IOException e)
                    {
                        CorePlugin.log(this.getClass(), "IO Exception " + pathProject + ". " + e.getMessage(),
                                IStatus.ERROR);

//                      TODO IO exception
                        IOErrorVector.add(e.getMessage());
                    }
                }
            }
        }
        else
            CorePlugin.log(this.getClass(), "Choose a project path thanks to the browse.", IStatus.INFO);

    }

    /**
     * Zip Files
     * 
     * @param files : all files to zip
     * @param output : destination jar
     * @param path : path in the jar
     * @throws IOException
     */
    public void zipFile(File[] files, JarOutputStream output, String path) throws IOException
    {
        try
        {
            for (int i = 0; i < files.length; i++)
            {
                File file = files[i];
                JarEntry entry = new JarEntry(path + file.getName());

                if (file.isDirectory())
                {
                    zipFile(file.listFiles(), output, path + file.getName() + File.separator);
                }
                else
                {
                    FileInputStream input = new FileInputStream(file);
                    output.putNextEntry(entry);
                    // Copy
                    byte[] buffer = new byte[1024];
                    int length;
                    while ((length = input.read(buffer)) != -1)
                    {
                        output.write(buffer, 0, length);
                    }
                    input.close();
                }
            }
        }
        catch (NullPointerException e)
        {
            CorePlugin.log(this.getClass(), "File error : " + path, IStatus.ERROR);
        }
    }

    /**
     * Create a new JarFile in the projectPath with all jarFile(features)
     * selected (mapPA) in the projectPath
     * 
     * @param pathProject
     */
    private void createProjectFeature(String pathProject)
    {
        if (!pathProject.equals(""))
        {
            Iterator it = mapPA.keySet().iterator();
            while (it.hasNext())
            {
                String key = (String) it.next();
                if (mapPA.get(key) instanceof SourceDestination)
                {
                    SourceDestination sd = (SourceDestination) mapPA.get(key);
                    String pathJarFeature = sd.getSource();
                    try
                    {
                        // UnJar pathJarFeature : create temporary file for the
                        // 'feature.xml' and the 'feature.property'
                        File fileToParse = FilesTreatment.createTempFileFromJar(new File(pathJarFeature), "feature.xml", FilesTreatment.FEATURE_XML);
                        File fileToParseProperty = FilesTreatment.createTempFileFromJar(new File(pathJarFeature), "feature.properties", FilesTreatment.FEATURE_PROPERTIES);
                        
                        // test added : fileToParse and fileToParseProperty can be null
                        if (fileToParse != null && fileToParseProperty != null) {
	                        // Take information on features
	                        String pathFeature = fileToParse.getCanonicalPath();
	                        String pathFeatureProperty = fileToParseProperty.getCanonicalPath();
	                        InfosFeature infosFeature = new InfosFeature(pathFeature, pathFeatureProperty);
	
	                        // Put information in mapFeatures
	                        mapFeatures.put(infosFeature.getNameFeature(), infosFeature);
	
	                        // Delete temporary file
	                        if (fileToParse.exists()) fileToParse.delete();
	                        if (fileToParseProperty.exists()) fileToParseProperty.delete();
                        }
                    }
                    catch (FileNotFoundException e)
                    {
                        CorePlugin.log(this.getClass(), "File Not Found Exception. " + e.getMessage(), IStatus.ERROR);
                    }
                    catch (IOException e)
                    {
                        CorePlugin.log(this.getClass(), "IO Exception. " + e.getMessage(), IStatus.ERROR);

                        // TODO IO exception
                        IOErrorVector.add(e.getMessage());
                    }
                }
            }
            it = mapPAFeatureInclude.keySet().iterator();
            while (it.hasNext())
            {
                String key = (String) it.next();
                if (mapPAFeatureInclude.get(key) instanceof SourceDestination)
                {
                    SourceDestination sd = (SourceDestination) mapPAFeatureInclude.get(key);
                    String pathJarFeature = sd.getSource();
                    try
                    {
                        // UnJar pathJarFeature : create temporary file for the
                        // 'feature.xml' and the 'feature.property'
                        File fileToParse = FilesTreatment.createTempFileFromJar(new File(pathJarFeature), "feature.xml", FilesTreatment.FEATURE_XML);
                        File fileToParseProperty = FilesTreatment.createTempFileFromJar(new File(pathJarFeature), "feature.properties", FilesTreatment.FEATURE_PROPERTIES);

                        // Take information on features
                        String pathFeature = fileToParse.getCanonicalPath();
                        String pathFeatureProperty = fileToParseProperty.getCanonicalPath();
                        InfosFeature infosFeature = new InfosFeature(pathFeature, pathFeatureProperty);

                        // Put information in mapFeatures
                        mapFeatures.put(infosFeature.getNameFeature(), infosFeature);

                        // Delete temporary file
                        fileToParse.delete();
                        fileToParseProperty.delete();
                    }
                    catch (FileNotFoundException e)
                    {
                        CorePlugin.log(this.getClass(), "File Not Found Exception. " + e.getMessage(), IStatus.ERROR);
                    }
                    catch (IOException e)
                    {
                        CorePlugin.log(this.getClass(), "IO Exception. " + e.getMessage(), IStatus.ERROR);

//                      TODO IO exception
                        IOErrorVector.add(e.getMessage());
                    }

                }
            }
            createXmlProjectFeature(pathProject);
            CorePlugin.log(this.getClass(), "Project feature created in " + pathProject, IStatus.INFO);
        }

        else
            CorePlugin.log(this.getClass(), "Choose a project path with the \"browse\" button.", IStatus.INFO);
    }

    private void createXmlProjectFeature(String pathProject)
    {
        // Take feature Information from IHM
        infosFeatureProject = new InfosFeature(nameFeatureProject, labelFeatureProject, versionFeatureProject,
                providerNameFeatureProject, imageFeatureProject, osFeatureProject, archFeatureProject,
                wsFeatureProject, nlFeatureProject, colocationFeatureProject, primaryFeatureProject,
                exclusiveFeatureProject, pluginFeatureProject, applicationFeatureProject);

        infosFeatureProject.setDescriptionFeature(descriptionFeatureProject);
        infosFeatureProject.setUrlDescriptionFeature(urlDescriptionFeatureProject);

        Iterator it = mapFeatures.keySet().iterator();
        while (it.hasNext())
        {
            String key = (String) it.next();
            if (mapFeatures.get(key) instanceof InfosFeature)
            {
                InfosFeature infosFeature = (InfosFeature) mapFeatures.get(key);
                Map mapImport = infosFeature.getMapRequiresImportFeature();
                Map mapPlugin = infosFeature.getMapPluginsFeature();

                // InfosImport
                infosFeatureProject.addAllMapRequiresImportFeature(mapImport);

                // InfosPlugin
                infosFeatureProject.addAllMapPluginsFeature(mapPlugin);
            }
        }
        // Create the feature.xml
        infosFeatureProject.createFeatureProject(pathProject);

        // jarFeature
        jarFeatureProject(pathProject);

        // delete feature.xml
        File xmlFeature = new File(pathProject + File.separator + "feature.xml");
        if (xmlFeature.exists())
            xmlFeature.delete();
    }

    private void jarFeatureProject(String pathProject)
    {
        if (!(pathProject.equals("") | eclipseUpdateJpgPath.equals("") | cplV10HtmlPath.equals("")
                | licenceHtmlPath.equals("") | nameFeatureProject.equals("") | versionFeatureProject.equals("")))
        {
            // Jar files in a repository TODO
            File pathFeatureXml = new File(pathProject + File.separator + "feature.xml");
            File[] filesFeatureXml = new File[1];
            filesFeatureXml[0] = pathFeatureXml;

            File pathEclipseUpdateJpg = new File(eclipseUpdateJpgPath);
            File[] filesEclipseUpdateJpg = new File[1];
            filesEclipseUpdateJpg[0] = pathEclipseUpdateJpg;

            File pathLicenceHtml = new File(licenceHtmlPath);
            File[] filesLicenceHtml = new File[1];
            filesLicenceHtml[0] = pathLicenceHtml;

            File pathCplV10HtmlPath = new File(cplV10HtmlPath);
            File[] filesCplV10Html = new File[1];
            filesCplV10Html[0] = pathCplV10HtmlPath;
            try
            {
                File fileJAR = new File(pathProject + nameFeatureProject + "_" + versionFeatureProject + ".jar");
				if (fileJAR.exists())
                {
					fileJAR.renameTo(new File(pathProject + nameFeatureProject + "_" + versionFeatureProject + ".jar.tmp"));
                }
                
                FileOutputStream outputStream = new FileOutputStream(pathProject + nameFeatureProject + "_" + versionFeatureProject + ".jar");
                JarOutputStream output = new JarOutputStream(outputStream);
                zipFile(filesFeatureXml, output, "");
                zipFile(filesEclipseUpdateJpg, output, "");
                zipFile(filesLicenceHtml, output, "");
                zipFile(filesCplV10Html, output, "");// TODO featureName + // File.separator);
                output.close();
                CorePlugin.log(this.getClass(), "Jar successfully built : " + pathProject + nameFeatureProject + "_" + versionFeatureProject + ".jar", IStatus.OK);
            }
            catch (FileNotFoundException e)
            {
                CorePlugin.log(this.getClass(), "File not found : " + e.getMessage(), IStatus.ERROR);
            }
            catch (IOException e)
            {
                CorePlugin.log(this.getClass(), "IO Error. " + e.getMessage(), IStatus.ERROR);
                IOErrorVector.add(e.getMessage());
            }
        }

        else
            CorePlugin.log(
                    this.getClass(),
                    "File path is missing : project or eclipse_update_120.jpg or cpl-v10.html or license.html or feature name missing",
                    IStatus.ERROR);
    }
    
    /**
     * This method concerns and works with the featurePage object.
     * This method checks and detects the cases where a path is already affected for a object which can be the public License, the image project or the HTML page.
     * @return true if the given path is not already contained in the map, otherwise false. 
     */
    public boolean checkFilePath(String object, String path)
    {
    	boolean valueToReturn = true;
    	if(filePathMap.containsValue(path))
    	{
            Iterator it = filePathMap.keySet().iterator();
            while (it.hasNext())
            {
                String key = (String) it.next();
            	if (((String)filePathMap.get(key)).equals(path))
            	{
            		valueToReturn = false;
            		featurePage.setMessage("The path given for the field " + object + " is already affected for the field " + key + ". Please choose a different one.", WizardPage.ERROR);
            	}
            }
    	}
    	else
    	{
    		// We can insert these data in the map
    		filePathMap.put(object, path);
    	}
    	return valueToReturn;
    }
    
   /**
    * This method is applied only in the case of SSH or FTP access.
    * Before displaying feature information, the file is opied in a local temporary directory.
    * @param name The feature name
    * @param pathDestination 
    */
    public void copyTemporaryFeature(String name)
   {
	  String parentPath = connectPage.getUSPath().getText() + "/features/";
	  boolean found = false; 
	  int i=0;
	  try
	   {
		  if (connectPage.getProtocol() == FTP) 
		  {
			   IDirectoryEntry[] files = clientFTP.listFiles(parentPath, false, null);
			   while (i < files.length && !found)
			   {
				   if (files[i].getName().equals(name))
				   {
					   found = true;
					   FilesTreatment.copyPluginFeatureFromFTP(parentPath, updateSitePath, files[i], clientFTP, null);
				   }
				   else
				   {
					   i++;
				   }
			   }
		   }
		   
		   if (connectPage.getProtocol() == SSH)
		   {			   
			   if (connectPage.getProtocol() == SSH)
			   {
				   List files = clientSFTP.ls(parentPath);
				   Iterator itList = files.iterator();
				   SftpFile currentFile;
				   SftpFile saveFile;
				   
				   while (itList.hasNext() && !found)
				   {
					   currentFile = (SftpFile)itList.next();
					   if (currentFile.getFilename().equals(name))
					   {
						   saveFile = currentFile;
						   FilesTreatment.copyPluginFeatureFromSFTP(saveFile ,updateSitePath, clientSFTP);
					   }
				   }
			   }
		   }
	   }
	   catch (FtpException ftpex)
	   {
		   
	   }
	   catch(IOException ioe)
	   {
		   
	   }
   }
} 
