/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.writeemfxmi.service;

import java.io.IOException;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.EObject;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.parameter.ParameterCst;
import org.topcased.service.writeemfxmi.WriteModelManager;

/**
 * Takes a model and saves it in the pathFile
 */
public class WriteEmfXmi implements IService
{

	/**
	 * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{
		// Declaration bloc
		EObject model = null;
		IPath pathFile = null;
		WriteModelManager modelManager = null;

		// Map null
		if (parameters == null)
		{
			CorePlugin.log(getClass(), "Parameters list is null. It must have exactly two parameters", IStatus.ERROR);
			return null;
		}

		// "PathFileString" parameter
		if (parameters.containsKey(ParameterCst.PATH_FILE_STRING))
		{
			if (parameters.get(ParameterCst.PATH_FILE_STRING) instanceof String)
			{
				pathFile = new Path((String) parameters.get(ParameterCst.PATH_FILE_STRING));
			}
			else
			{
				CorePlugin.log(getClass(), "The 'PathFileString' parameter must be a String", IStatus.ERROR);
			}
		}
		else
		{
			CorePlugin.log(getClass(), "The 'PathFileString' parameter is missing", IStatus.ERROR);
		}

		// "Model" parameter
		if (parameters.containsKey(ParameterCst.MODEL))
		{
			if (parameters.get(ParameterCst.MODEL) instanceof EObject)
			{
				model = (EObject) parameters.get(ParameterCst.MODEL);
			}
			else
			{
				CorePlugin.log(getClass(), "The 'Model' parameter must be a EObject", IStatus.ERROR);
			}
		}
		else
		{
			CorePlugin.log(getClass(), "The 'Model' parameter is missing", IStatus.ERROR);
		}

		// Test 'model '
		if (model == null)
		{
			CorePlugin.log(getClass(), "Error : model not define!", IStatus.ERROR);
			return null;
		}

		// Launch Service Write
		modelManager = new WriteModelManager();
		try
		{
			modelManager.save(pathFile, model);
			CorePlugin.log(getClass(), "Write EMF/XMI in " + pathFile, IStatus.INFO);
		}
		catch (IOException e)
		{
			CorePlugin.log(getClass(), "IOException : " + e.getMessage(), IStatus.ERROR);
		}
		return null;
	}
}
