package org.topcased.sysml.tools.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.command.AbstractCommand;
import org.eclipse.uml2.uml.NamedElement;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.tools.operations.DesignElementsCoverOperations;

/**
 * Command that add or remove design elements which satisfy a requirement in the
 * EMF SysML Editor
 */
public class EMFDesignElementsCoverCommand extends AbstractCommand
{
    /** Current requirement */
    private Requirement requirement;

    /** Old values */
    private List oldDatas;

    /** New values */
    private List datas;

    /**
     * Create a command for updating design elements satisfying a requirement
     * 
     * @param req the requirement
     * @param data the new datas
     * @param oldData the old datas
     */
    public EMFDesignElementsCoverCommand(Requirement req, List data, List oldData)
    {
        super("Change Design elements Cover");
        requirement = req;
        datas = data;
        oldDatas = oldData;
    }

    /**
     * Set the values
     */
    protected void setValues()
    {
        // Remove old design elements from covering design elements
        List removed = new ArrayList(oldDatas);
        removed.removeAll(datas);
        for (int i = 0; i < removed.size(); i++)
        {
            DesignElementsCoverOperations.removeCoveringDesignElement(requirement, (NamedElement) removed.get(i));
        }

        // Add new design elements to covering design elements
        List added = new ArrayList(datas);
        added.removeAll(oldDatas);
        for (int i = 0; i < added.size(); i++)
        {
            DesignElementsCoverOperations.addCoveringDesignElement(requirement, (NamedElement) added.get(i));
        }
    }

    /**
     * Switch the old and new values
     */
    protected void switchValues()
    {
        // Reverse old and new value for name
        List temp = datas;
        datas = oldDatas;
        oldDatas = temp;
    }

    /**
     * Set the values
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        setValues();
    }

    /**
     * Set the old values
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        switchValues();
        setValues();
    }

    /**
     * Set the new values
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        switchValues();
        setValues();
    }

    /**
     * always execute
     * @see org.eclipse.emf.common.command.AbstractCommand#canExecute()
     * @return boolean true
     */
    public boolean canExecute()
    {
        return true;
    }
}
