/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
******************************************************************************/
package org.topcased.sysml.tools.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.command.AbstractCommand;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.TestCase;
import org.topcased.sysml.tools.operations.TestCaseCoverOperations;


/**
 * Command that add or remove test cases which verify a requirement in the EMF
 * SysML Editor.
 */
public class EMFTestCaseCoverCommand extends AbstractCommand
{

    /** Current requirement. */
    private Requirement requirement;

    /** Old values. */
    private List oldDatas;

    /** New values. */
    private List datas;

    /**
     * Create a command for updating test cases verifying a requirement.
     * 
     * @param oldData the old datas
     * @param data the new datas
     * @param req the requirement
     */
    public EMFTestCaseCoverCommand(Requirement req, List data, List oldData)
    {
        super("Change Test Case Cover");
        requirement = req;
        datas = data;
        oldDatas = oldData;
    }

    /**
     * Set the values.
     */
    protected void setValues()
    {
        // Remove old test cases from covering test cases
        List removed = new ArrayList(oldDatas);
        removed.removeAll(datas);
        for (int i = 0; i < removed.size(); i++)
        {
            TestCaseCoverOperations.removeCoveringTestCase(requirement, (TestCase) removed.get(i));
        }

        // Add new test cases to covering test cases
        List added = new ArrayList(datas);
        added.removeAll(oldDatas);
        for (int i = 0; i < added.size(); i++)
        {
            TestCaseCoverOperations.addCoveringTestCase(requirement, (TestCase) added.get(i));
        }
    }

    /**
     * Switch the old and new values.
     */
    protected void switchValues()
    {
        // Reverse old and new value for name
        List temp = datas;
        datas = oldDatas;
        oldDatas = temp;
    }

    /**
     * Set the values.
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        setValues();
    }

    /**
     * Set the old values.
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        switchValues();
        setValues();
    }

    /**
     * Set the new values.
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        switchValues();
        setValues();
    }

    /**
     * execute
     * @see org.eclipse.emf.common.command.AbstractCommand#canExecute()
     * @return boolean true
     */
    public boolean canExecute()
    {
        return true;
    }
}
