/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
******************************************************************************/
package org.topcased.sysml.tools.commands;

import java.util.List;

import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.NamedElement;
import org.topcased.modeler.utils.LabelHelper;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.Satisfy;
import org.topcased.sysml.tools.operations.DesignElementsCoverOperations;

/**
 * The Class GEFAddCoverDesignElementsCommand.
 */
public class GEFAddCoverDesignElementsCommand extends Command
{

    /** The edit domain. */
    private EditDomain editDomain;

    /** The requirement. */
    private Requirement requirement;

    /** The design elements. */
    private List < ? > designElements;

    /**
     * The Constructor.
     * 
     * @param designElts the design elts
     * @param domain the domain
     * @param req the req
     */
    public GEFAddCoverDesignElementsCommand(EditDomain domain, Requirement req, List designElts)
    {
        editDomain = domain;
        requirement = req;
        designElements = designElts;
    }

    /** 
     * execute
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        // Add new design elements to covering design elements
        for (int i = 0; i < designElements.size(); i++)
        {
            Satisfy satisfy = DesignElementsCoverOperations.addCoveringDesignElement(requirement,
                    (NamedElement) designElements.get(i));
            // Init the name of the created satisfy link
            LabelHelper labelHelper = new LabelHelper();
            String curName = labelHelper.getName(editDomain, satisfy);
            if (curName == null || curName == "")
            {
                labelHelper.initName(editDomain, satisfy.eContainer(), satisfy);
            }
        }
    }

    /** 
     * undo
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        // Remove design elements that have been added to covering design
        // elements
        for (int i = 0; i < designElements.size(); i++)
        {
            DesignElementsCoverOperations.removeCoveringDesignElement(requirement, (NamedElement) designElements.get(i));
        }
    }
}
