/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
******************************************************************************/
package org.topcased.sysml.tools.commands;

import java.util.List;

import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.utils.LabelHelper;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.TestCase;
import org.topcased.sysml.Verify;
import org.topcased.sysml.tools.operations.TestCaseCoverOperations;

/**
 * The Class GEFAddCoverTestCasesCommand.
 */
public class GEFAddCoverTestCasesCommand extends Command
{

    /** The edit domain. */
    private EditDomain editDomain;

    /** The requirement. */
    private Requirement requirement;

    /** The test cases. */
    private List < ? > testCases;

    /**
     * The Constructor.
     * 
     * @param testCase the test case
     * @param domain the domain
     * @param req the req
     */
    public GEFAddCoverTestCasesCommand(EditDomain domain, Requirement req, List testCase)
    {
        editDomain = domain;
        requirement = req;
        testCases = testCase;
    }

    /**
     * execute
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        // Add new test cases to covering test cases
        for (int i = 0; i < testCases.size(); i++)
        {
            Verify verify = TestCaseCoverOperations.addCoveringTestCase(requirement, (TestCase) testCases.get(i));
            // Init the name of the created verify link
            LabelHelper labelHelper = new LabelHelper();
            String curName = labelHelper.getName(editDomain, verify);
            if (curName == null || curName == "")
            {
                labelHelper.initName(editDomain, verify.eContainer(), verify);
            }
        }
    }

    /** 
     * undo
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        // Remove test cases that have been added to covering test cases
        for (int i = 0; i < testCases.size(); i++)
        {
            TestCaseCoverOperations.removeCoveringTestCase(requirement, (TestCase) testCases.get(i));
        }
    }
}
