/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
******************************************************************************/
package org.topcased.sysml.tools.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.uml2.uml.NamedElement;
import org.topcased.modeler.commands.DeleteModelCommand;
import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.editor.Modeler;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.Satisfy;
import org.topcased.sysml.tools.operations.DesignElementsCoverOperations;

/**
 * Command that add or remove design elements which satisfy a requirement in the
 * GEF SysML Editor.
 */
public class GEFDesignElementsCoverCommand extends CompoundCommand
{

    /** The edit domain. */
    private EditDomain editDomain;

    /** Current requirement. */
    private Requirement requirement;

    /** Old values. */
    private List oldDatas;

    /** New values. */
    private List datas;

    /**
     * Create a command for updating design elements satisfying a requirement.
     * 
     * @param oldData the old datas
     * @param data the new datas
     * @param domain the GEF editing domain
     * @param req the requirement
     */
    public GEFDesignElementsCoverCommand(EditDomain domain, Requirement req, List data, List oldData)
    {
        super("Change Design Elements Cover");
        editDomain = domain;
        requirement = req;
        oldDatas = oldData;
        datas = data;
        initializeDeleteCommands();
        initializeAddCommands();
    }

    /**
     * Initialize delete commands.
     */
    private void initializeDeleteCommands()
    {
        if (editDomain instanceof MixedEditDomain)
        {
            MixedEditDomain domain = (MixedEditDomain) editDomain;
            Diagrams diagrams = ((Modeler) domain.getEditorPart()).getDiagrams();
            if (domain != null && diagrams != null)
            {
                // Remove old design elements from covering design elements
                List removed = new ArrayList(oldDatas);
                removed.removeAll(datas);
                for (int i = 0; i < removed.size(); i++)
                {
                    // Remove satisfy model objects and graph objects associated
                    List linksToRemove = DesignElementsCoverOperations.getSatisfyLinks(requirement,
                            (NamedElement) removed.get(i));
                    for (int j = 0; j < linksToRemove.size(); j++)
                    {
                        add(new DeleteModelCommand(domain, diagrams, (Satisfy) linksToRemove.get(j)));
                    }
                }
            }
        }
    }

    /**
     * Initialize add commands.
     */
    private void initializeAddCommands()
    {
        // Add new design elements to covering design elements
        List added = new ArrayList(datas);
        added.removeAll(oldDatas);
        add(new GEFAddCoverDesignElementsCommand(editDomain, requirement, added));
    }
}
