/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
******************************************************************************/
package org.topcased.sysml.tools.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.CompoundCommand;
import org.topcased.modeler.commands.DeleteModelCommand;
import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.editor.Modeler;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.TestCase;
import org.topcased.sysml.Verify;
import org.topcased.sysml.tools.operations.TestCaseCoverOperations;

/**
 * Command that add or remove test cases which verify a requirement in the GEF
 * SysML Editor.
 */
public class GEFTestCaseCoverCommand extends CompoundCommand
{

    /** The edit domain. */
    private EditDomain editDomain;

    /** Current requirement. */
    private Requirement requirement;

    /** Old values. */
    private List oldDatas;

    /** New values. */
    private List datas;

    /**
     * Create a command for updating test cases verifying a requirement.
     * 
     * @param oldData the old datas
     * @param data the new datas
     * @param domain the GEF editing domain
     * @param req the requirement
     */
    public GEFTestCaseCoverCommand(EditDomain domain, Requirement req, List data, List oldData)
    {
        super("Change Test Case Cover");
        editDomain = domain;
        requirement = req;
        oldDatas = oldData;
        datas = data;
        initializeDeleteCommands();
        initializeAddCommands();
    }

    /**
     * Initialize delete commands.
     */
    private void initializeDeleteCommands()
    {
        if (editDomain instanceof MixedEditDomain)
        {
            MixedEditDomain domain = (MixedEditDomain) editDomain;
            Diagrams diagrams = ((Modeler) domain.getEditorPart()).getDiagrams();
            if (domain != null && diagrams != null)
            {
                // Remove old test cases from covering test cases
                List removed = new ArrayList(oldDatas);
                removed.removeAll(datas);
                for (int i = 0; i < removed.size(); i++)
                {
                    // Remove verify model objects and graph objects associated
                    List linksToRemove = TestCaseCoverOperations.getVerifyLinks(requirement, (TestCase) removed.get(i));
                    for (int j = 0; j < linksToRemove.size(); j++)
                    {
                        add(new DeleteModelCommand(domain, diagrams, (Verify) linksToRemove.get(j)));
                    }
                }
            }
        }
    }

    /**
     * Initialize add commands.
     */
    private void initializeAddCommands()
    {
        // Add new test cases to covering test cases
        List added = new ArrayList(datas);
        added.removeAll(oldDatas);
        add(new GEFAddCoverTestCasesCommand(editDomain, requirement, added));
    }
}
