/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
******************************************************************************/
package org.topcased.sysml.tools.operations;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.UniqueEList;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.PackageSYSML;
import org.topcased.sysml.SysMLFactory;
import org.topcased.sysml.SysMLPackage;
import org.topcased.sysml.TestCase;
import org.topcased.sysml.Verify;
import org.topcased.sysml.tools.SysMLRequirementsTools;

/**
 * The Class TestCaseCoverOperations.
 */
public class TestCaseCoverOperations
{

    /**
     * Return the list of test cases that can verify a requirement A query based
     * on the following OCL expression TestCase.allInstances()
     * 
     * @param requirement the requirement
     * 
     * @return the list of test cases
     */
    public static List getAvailableTestCases(Requirement requirement)
    {
        return new UniqueEList(ItemPropertyDescriptor.getReachableObjectsOfType(requirement,
                SysMLPackage.eINSTANCE.getTestCase()));
    }

    /**
     * Return the list of test cases that verify a requirement A query based on
     * the following OCL expression
     * Verify.allInstances()->select(supplier->includes(self)).client->select(oclIsTypeOf(TestCase))->asSet()
     * 
     * @param requirement the requirement
     * 
     * @return the list of test cases
     */
    public static List < ? > getCoveringTestCases(Requirement requirement)
    {
        return requirement.getVerifiedBy();

        /*
         * Map extents = new HashMap();
         * extents.put(SysMLPackage.eINSTANCE.getVerify(),
         * ItemPropertyDescriptor.getReachableObjectsOfType(requirement,
         * SysMLPackage.eINSTANCE.getVerify()));
         * 
         * Query query = QueryFactory.eINSTANCE.createQuery(
         * "Verify.allInstances()->select(supplier->includes(self)).client->select(oclIsTypeOf(TestCase))->asSet()",SysMLPackage.eINSTANCE.getRequirement());
         * query.setExtentMap(extents); Set result =
         * (Set)query.evaluate(requirement);
         * 
         * return Arrays.asList(result.toArray());
         */
    }

    /**
     * Remove a test case from the test cases verifying a requirement (i.e
     * remove the verify links between the test case and the requirement)
     * 
     * @param requirement the requirement
     * @param testcase the test case to remove from the test cases verifying the
     *            requirement
     */
    public static void removeCoveringTestCase(Requirement requirement, TestCase testcase)
    {
        List verifyLinksToRemove = getVerifyLinks(requirement, testcase);
        for (Iterator it = verifyLinksToRemove.iterator(); it.hasNext();)
        {
            ((Verify) it.next()).destroy();
        }
    }

    /**
     * Add a test case to the test cases verifying a requirement (i.e add a
     * verify link between the test case and the requirement)
     * 
     * @param requirement the requirement
     * @param testcase the test case to add to the test cases verifying the
     *            requirement
     * 
     * @return the created verify link
     */
    public static Verify addCoveringTestCase(Requirement requirement, TestCase testcase)
    {
        Verify verify = SysMLFactory.eINSTANCE.createVerify();
        verify.getClients().add(testcase);
        verify.getSuppliers().add(requirement);
        PackageSYSML rd = SysMLRequirementsTools.getPackageSYSML(requirement);
        if (rd != null)
        {
            rd.getOwnedTraceDependency().add(verify);
        }
        return verify;
    }

    /**
     * Returns all the verify links that match a test case to a requirement.
     * 
     * @param requirement the requirement
     * @param testcase the test case
     * 
     * @return result return a list of result
     */
    public static List < ? > getVerifyLinks(Requirement requirement, TestCase testcase)
    {
        Collection < ? > allVerify = ItemPropertyDescriptor.getReachableObjectsOfType(requirement,
                SysMLPackage.eINSTANCE.getVerify());
        List result = new ArrayList();
        for (Iterator it = allVerify.iterator(); it.hasNext();)
        {
            Verify verifyLink = (Verify) it.next();
            if (verifyLink.getClients().contains(testcase) && verifyLink.getSuppliers().contains(requirement))
                result.add(verifyLink);
        }
        return result;
    }
}
