package org.topcased.toolkit;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin {

	/** The plug-in ID */
	public static final String PLUGIN_ID = "org.topcased.toolkit";

	/** The shared instance */
	private static Activator plugin;
	
	/**
	 * The constructor
	 */
	public Activator() {
		plugin = this;
	}

	/**
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}

	/**
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}

    /**
     * Log a message with given level into the Eclipse log file
     * 
     * @param message the message to log
     * @param level the message priority
     */
    public static void log(String message, int level)
    {
        IStatus status = null;
        status = new Status(level, PLUGIN_ID, IStatus.OK, message, null);
        log(status);
    }

    /**
     * Log an exception into the Eclipse log file
     * 
     * @param e the exception to log
     */
    public static void log(Throwable e)
    {
        log("Error", e);
    }

    /**
     * Log an exception into the Eclipse log file
     * 
     * @param message the message logged *
     * @param e the exception to log
     */
    public static void log(String message, Throwable e)
    {

        Throwable t = e;
        if (e instanceof InvocationTargetException)
        {
            t = ((InvocationTargetException) e).getTargetException();
        }

        IStatus status = null;
        if (t instanceof CoreException)
        {
            status = ((CoreException) t).getStatus();
        }
        else
        {
            status = new Status(IStatus.ERROR, PLUGIN_ID, IStatus.OK, message, e);
        }

        log(status);
    }

    /**
     * Write a debug message if the debug is enabled.
     * 
     * @param message the message to log
     */
    public static void debug(String message)
    {
        if (Platform.inDebugMode())
        {
            System.out.println(message);
        }
    }

    /**
     * Log an IStatus
     * 
     * @param status Status of an operation
     */
    public static void log(IStatus status)
    {
        getDefault().getLog().log(status);
    }

}
