package org.topcased.traceability.analyzers.openOffice;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.Iterator;

import javax.xml.namespace.NamespaceContext;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class OOXPathExtractor {
	private static NamespaceContext namespaceContext = new NamespaceContext() {
		public String getNamespaceURI(String prefix) {
			if ("style".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:style:1.0";
			} else if ("office".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:office:1.0";
			} else if ("text".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:text:1.0";
			} else if ("table".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:table:1.0";
			} else if ("draw".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:drawing:1.0";
			} else if ("fo".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0";
			} else if ("xlink".equals(prefix)) {
				return "http://www.w3.org/1999/xlink";
			} else if ("dc".equals(prefix)) {
				return "http://purl.org/dc/elements/1.1/";
			} else if ("meta".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:meta:1.0";
			} else if ("number".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0";
			} else if ("svg".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0";
			} else if ("chart".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:chart:1.0";
			} else if ("dr3d".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0";
			} else if ("math".equals(prefix)) {
				return "http://www.w3.org/1998/Math/MathML";
			} else if ("form".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:form:1.0";
			} else if ("script".equals(prefix)) {
				return "urn:oasis:names:tc:opendocument:xmlns:script:1.0";
			} else if ("ooo".equals(prefix)) {
				return "http://openoffice.org/2004/office";
			} else if ("ooow".equals(prefix)) {
				return "http://openoffice.org/2004/writer";
			} else if ("oooc".equals(prefix)) {
				return "http://openoffice.org/2004/calc";
			} else if ("dom".equals(prefix)) {
				return "http://www.w3.org/2001/xml-events";
			} else if ("xforms".equals(prefix)) {
				return "http://www.w3.org/2002/xforms";
			} else if ("xsd".equals(prefix)) {
				return "http://www.w3.org/2001/XMLSchema";
			} else if ("xsi".equals(prefix)) {
				return "http://www.w3.org/2001/XMLSchema-instance";
			} else {
				return null;
			}
		}

		public String getPrefix(String namespaceURI) {
			if ("urn:oasis:names:tc:opendocument:xmlns:style:1.0"
					.equals(namespaceURI)) {
				return "style";
			} else if ("urn:oasis:names:tc:opendocument:xmlns:office:1.0"
					.equals(namespaceURI)) {
				return "office";
			} else if ("urn:oasis:names:tc:opendocument:xmlns:text:1.0"
					.equals(namespaceURI)) {
				return "text";
			} else if ("urn:oasis:names:tc:opendocument:xmlns:table:1.0"
					.equals(namespaceURI)) {
				return "table";
			} else if ("urn:oasis:names:tc:opendocument:xmlns:drawing:1.0"
					.equals(namespaceURI)) {
				return "draw";
			} else if ("urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0"
					.equals(namespaceURI)) {
				return "fo";
			} else if ("http://www.w3.org/1999/xlink".equals(namespaceURI)) {
				return "xlink";
			} else if ("http://purl.org/dc/elements/1.1/".equals(namespaceURI)) {
				return "dc";
			} else if ("urn:oasis:names:tc:opendocument:xmlns:meta:1.0"
					.equals(namespaceURI)) {
				return "meta";
			} else if ("urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0"
					.equals(namespaceURI)) {
				return "number";
			} else if ("urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0"
					.equals(namespaceURI)) {
				return "svg";
			} else if ("urn:oasis:names:tc:opendocument:xmlns:chart:1.0"
					.equals(namespaceURI)) {
				return "chart";
			} else if ("urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0"
					.equals(namespaceURI)) {
				return "dr3d";
			} else if ("http://www.w3.org/1998/Math/MathML"
					.equals(namespaceURI)) {
				return "math";
			} else if ("urn:oasis:names:tc:opendocument:xmlns:form:1.0"
					.equals(namespaceURI)) {
				return "form";
			} else if ("urn:oasis:names:tc:opendocument:xmlns:script:1.0"
					.equals(namespaceURI)) {
				return "script";
			} else if ("http://openoffice.org/2004/office".equals(namespaceURI)) {
				return "ooo";
			} else if ("http://openoffice.org/2004/writer".equals(namespaceURI)) {
				return "ooow";
			} else if ("http://openoffice.org/2004/calc".equals(namespaceURI)) {
				return "oooc";
			} else if ("http://www.w3.org/2001/xml-events".equals(namespaceURI)) {
				return "dom";
			} else if ("http://www.w3.org/2002/xforms".equals(namespaceURI)) {
				return "xforms";
			} else if ("http://www.w3.org/2001/XMLSchema".equals(namespaceURI)) {
				return "xsd";
			} else if ("http://www.w3.org/2001/XMLSchema-instance"
					.equals(namespaceURI)) {
				return "xsi";

			} else {
				return null;
			}
		}

		public Iterator<String> getPrefixes(String namespaceURI) {
			return null;
		}
	};

	/** Creates a new instance of ExempleXPath */
	public OOXPathExtractor() {
	}

	private NodeList evaluateExpressionOnFile(String anExpression, File aFile) {
		try {
			InputSource inputSource = new InputSource(
					new FileInputStream(aFile));
			XPathFactory factory = XPathFactory.newInstance();
			XPath environment = factory.newXPath();
			environment.setNamespaceContext(namespaceContext);

			XPathExpression expression = environment.compile(anExpression);
			NodeList nodes = (NodeList) expression.evaluate(inputSource,
					XPathConstants.NODESET);
			return nodes;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (XPathExpressionException e) {
			e.printStackTrace();
		}
		return null;
	}

	HashMap<String, String> extractStylesInFile(File xmlFile) {
		HashMap<String, String> stylesAsso = new HashMap<String, String>();

		NodeList nodes = evaluateExpressionOnFile(
				"/office:document-styles/office:styles/style:style[@style:display-name]",
				xmlFile);

		for (int i = 0; i < nodes.getLength(); i++) {
			Node node = nodes.item(i);
			NamedNodeMap attrs = node.getAttributes();
			String name = attrs.getNamedItem("style:name").getNodeValue();
			Node displayNode = attrs.getNamedItem("style:display-name");
			String displayName = name;
			if (displayNode != null) {
				displayName = displayNode.getNodeValue();
			}
			stylesAsso.put(name, displayName);
		}

		return stylesAsso;
	}

	HashMap<String, String> completeStylesFromContentFile(
			HashMap<String, String> basicStyles, File xmlFile) {
		HashMap<String, String> stylesAsso = basicStyles;
		String expr = "/office:document-content/office:automatic-styles/style:style[@style:parent-style-name]";
		NodeList nodes = evaluateExpressionOnFile(expr, xmlFile);

		for (int i = 0; i < nodes.getLength(); i++) {
			Node node = nodes.item(i);
			NamedNodeMap attrs = node.getAttributes();
			String name = attrs.getNamedItem("style:name").getNodeValue();
			Node parentStyleNode = attrs
					.getNamedItem("style:parent-style-name");
			if (parentStyleNode != null) {
				String parentStyleName = parentStyleNode.getNodeValue();
				String displayName = basicStyles.get(parentStyleName);
				if (displayName != null) {
					stylesAsso.put(name, displayName);
				} else {
					stylesAsso.put(name,parentStyleName);
				}
			}
		}
		return stylesAsso;
	}

	NodeList extractNodesWithStyleName(File contentFile) {		
		String expr = "//office:text/text:p|//office:text/text:h|//office:text/text:p/text:span|//office:text/table:table/table:table-row/table:table-cell/text:p|//office:text/table:table/table:table-row/table:table-cell/text:h|//office:text/table:table/table:table-row/table:table-cell/text:p/text:span";
		return evaluateExpressionOnFile(expr, contentFile);
	}
}
