/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.actions;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.topcased.traceability.hmi.TtmImageRegistry;
import org.topcased.traceability.hmi.TtmPlugin;
import org.topcased.traceability.hmi.contextmenu.ContextAction;
import org.topcased.traceability.hmi.util.ModelHelper;
import org.topcased.traceability.hmi.util.TraceabilityConstants;

import ttm.Element;
import ttm.RuleInstance;

/**
 * Created on 21 fvr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * Action that look for a requirement into the main viewer from the report section selection
 */
public class FindRequirementAction extends ContextAction
{
	private Viewer viewer;
	private Object objectToSearch;
	
	public FindRequirementAction(Viewer targetViewer) {
		super("Search", TtmImageRegistry.getImageDescriptor(TraceabilityConstants.SEARCH_IMAGE_KEY));
		this.viewer = targetViewer;
	}
	
	@Override
	protected void selectionHasChanged() {
		Object selected = getSelection();
		setEnabled(true);
		if (selected instanceof Element) {
			objectToSearch = selected;
		}
		else if (selected instanceof RuleInstance) {
			objectToSearch = ModelHelper.getElementFromRule((RuleInstance)selected);
		}
		else {
			setEnabled(false);
		}
	}
	
	@Override
	public void run() {
		if (viewer != null && objectToSearch != null) {
			viewer.setSelection(new StructuredSelection(objectToSearch));
		}
		else {
			TtmPlugin.log("Unable to find object", IStatus.WARNING);
		}
	}
	
	

	

    
	
    

}
