package org.topcased.traceability.hmi.actions;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.topcased.traceability.hmi.TtmPlugin;
import org.topcased.traceability.hmi.editor.TtmMultipleEditor;
import org.topcased.traceability.hmi.runtime.ResourceLoader;

import ttm.Project;

public abstract class ProjectActionDelegate extends Action {

	/**
	 * Comment for <code>selectedModel</code>
	 */
	private Project selectedProject = null;
	private TtmMultipleEditor activeEditor;

	public ProjectActionDelegate() {
		super();
	}

	public ProjectActionDelegate(String string, ImageDescriptor imageDescriptor) {
		super(string, imageDescriptor);
	}

	public void setActiveEditor(IAction action, IEditorPart targetEditor) {
		if (targetEditor instanceof TtmMultipleEditor) {
			activeEditor = (TtmMultipleEditor)targetEditor;
		}
		else {
			activeEditor = null;
		}
	}

	public void resolveActiveEditor() {
		if (activeEditor == null) {
			try {
				IEditorPart editor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
				if (editor instanceof TtmMultipleEditor) {
					activeEditor = (TtmMultipleEditor)editor;
				}
			}
			catch (Exception e) {
				//Some resources are not available and can involve NullPointerException here.
			}
		}
	}

	public void run() {
	    run(this);
	}
	
	public void run(IAction action)
    {
        Shell activeShell = Display.getDefault().getActiveShell();
    	
        try {
           IRunnableWithProgress op = createAction(); 
           new ProgressMonitorDialog(activeShell).run(true, true, op);
         } catch (InvocationTargetException e) {
            // handle exception
         } catch (InterruptedException e) {
            // handle cancelation
         }
         
    }

	protected abstract IRunnableWithProgress createAction();
	
	private Project getProjectFromEditor() {
		return ((TtmMultipleEditor)activeEditor).getProject();
	}

	public void selectionChanged(IAction action, ISelection selection) {  
		if (activeEditor != null) {
			setSelectedProject(getProjectFromEditor());
		}
		else if (selection != null && !selection.isEmpty() && selection instanceof TreeSelection) {
	        Object strSelection = ((TreeSelection)selection).getFirstElement();
	        if(strSelection != null)
	        {
	            //Package explorer view
	            if(strSelection instanceof IResource)
	            {
	                if (((IResource)strSelection).getFileExtension().equalsIgnoreCase(TtmPlugin.MODEL_EXTENSION))
	                {
	                    setSelectedProject((Project)ResourceLoader.getInstance().load((IResource)strSelection));
	                }
	            }                  
	        }
	
		}
	    action.setEnabled(selectedProject != null);
	
	}

	protected void refreshNavigationView() {
		
		FileEditorInput input = (FileEditorInput) ((TtmMultipleEditor)activeEditor).getEditorInput();
		try {
			input.getFile().getProject().refreshLocal(IResource.DEPTH_INFINITE, new NullProgressMonitor());
		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	public void init(IViewPart view) {
	
	}

	/**
	 * Getter for selected Project
	 * @return Project selectedProject
	 */
	public Project getSelectedProject() {
	    return selectedProject;
	}

	/**
	 * Setter for selectedProject
	 * @param selectedModel
	 */
	public void setSelectedProject(Project selectedModel) {
	    this.selectedProject = selectedModel;
	}



	
	
}