package org.topcased.traceability.hmi.contextmenu;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;

/**
 * creation : 13 june 07
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 *
 * A specific extension of Action for context menu
 */
public abstract class ContextAction extends Action
{
	/** current selection */
	private Object selected;

	/**
	 * Default constructor
	 * @param text
	 * @param img
	 */
	public ContextAction(String text, ImageDescriptor img) {
		super(text, img);
	}

	/**
	 * Internal method called when selection has changed.
	 * @param selected
	 */
	void setSelection(Object selected) {
		if (selected != null && !selected.equals(this.selected)) {
			this.selected = selected;
			selectionHasChanged();
		}
	}
	
	/**
	 * This method is called each time the selection changes. Subclasses may override
	 * it to control action availibilty (IAction{@link #setEnabled(boolean)}). Use {@link #getSelection()} to obtain the 
	 * current selection.
	 */
	protected void selectionHasChanged() {
		
	}

	/**
	 * Get the current selection
	 * @return an object
	 */
	final protected Object getSelection() {
		return selected;
	}
	
	@Override
	public abstract void run();
	
}
