package org.topcased.traceability.hmi.editor;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.topcased.modeler.editor.Modeler;
import org.topcased.traceability.hmi.TtmPlugin;
import org.topcased.traceability.hmi.actions.AnalyseActionDelegate;
import org.topcased.traceability.hmi.ui.form.BoardView;
import org.topcased.traceability.hmi.ui.form.CoverageView;
import org.topcased.traceability.hmi.ui.form.ImpactView;
import org.topcased.traceability.hmi.ui.form.RequirementRefreshableFormPage;
import org.topcased.traceability.hmi.ui.form.RequirementTableView;
import org.topcased.traceability.hmi.util.FileNameUtils;

import ttm.Project;

public class TtmMultipleEditor extends FormEditor implements IEditingDomainProvider,
        ITabbedPropertySheetPageContributor
{

    /**
     * Diagram Editor generated by topcased
     */
    private TtmEditor ttmEditor;

    private boolean openBordView = false;

    private RequirementRefreshableFormPage coverageView;

    private BoardView boardView;

    private ImpactView impactView;

    private RequirementTableView tableView;

    private DocumentChangeChecker changeChecker;

    @Override
    protected void addPages()
    {
        ttmEditor = new TtmEditor()
        {
            @Override
            public void dispose()
            {
                changeChecker.stop();
                super.dispose();
            }
        };

        int index;
        try
        {
            index = addPage(ttmEditor, getEditorInput());
            setPageText(index, "Model View");
            addBoardPage();

            // Check project has ident property set.
            Project currentProject = getProject();
            if (currentProject.getIdent() == null || currentProject.getIdent().equals(""))
            {
                currentProject.setIdent("defaultIdent");
            }

            IFile file = ((FileEditorInput) getEditorInput()).getFile();
            String inputName = file.getFullPath().removeFileExtension().addFileExtension("ttm").lastSegment();
            setPartName(inputName);
        }
        catch (PartInitException e)
        {
            TtmPlugin.log(e);
        }
    }

    /**
     * Create the page used to display the ttmresult content
     * 
     * @throws PartInitException
     */
    private void addBoardPage() throws PartInitException
    {
        boardView = new BoardView(this, "boardViewID", "Board View");
        coverageView = new CoverageView(this, "coverageViewID", "Coverage View");
        impactView = new ImpactView(this, "impactViewID", "Impact View");
        tableView = new RequirementTableView(this, "requirementTableViewID", "Requirement Table View");

        int index = addPage(boardView);
        index = addPage(coverageView);
        index = addPage(impactView);
        index = addPage(tableView);

        if (openBordView)
        {
            setActivePage(index);
        }

        hookPageListeners();

        // hookActions();

    }

    private void hookActions()
    {
        // Add an action to select the requirement in the other editor
        /*
         * ((CoverageView) coverageView).addRequirementAction(new
         * OpenInRequirementViewAction(coverageView, impactView) { protected
         * void open() { setActivePage(3); } });
         * 
         * ((ImpactView) impactView).addRequirementAction(new
         * OpenInRequirementViewAction(impactView, coverageView) { protected
         * void open() { setActivePage(2); } });
         */

    }

    private void hookPageListeners()
    {
        coverageView.setRequirementProvider(impactView);
        impactView.setRequirementProvider(coverageView);

    }

    protected void hookListeners()
    {

    }

    @Override
    public void doSave(IProgressMonitor monitor)
    {
        ttmEditor.doSave(monitor);
        // Reload the analysis. It will be canceled if the
        // documents are not reloaded
        new AnalyseActionDelegate(getProject()).run();

    }

    @Override
    public void doSaveAs()
    {
        ttmEditor.doSaveAs();
    }

    @Override
    public void dispose()
    {
        // getSite().getPage().removeSelectionListener(boardView);
        super.dispose();
    }

    @Override
    public boolean isSaveAsAllowed()
    {
        return ttmEditor.isSaveAsAllowed();
    }

    /**
     * Returns the Editor Id
     * 
     * @return the String that identifies the Editor
     * @generated
     */
    public static String getId()
    {
        return new String("org.topcased.traceability.hmi.editor.TtmMultipleEditor");
    }

    /**
     * @see org.topcased.modeler.editor.Modeler#getAdapter(java.lang.Class)
     * @generated
     */
    public Object getAdapter(Class type)
    {
        if (type.equals(Modeler.class))
        {
            return ttmEditor;
        }
        return ttmEditor.getAdapter(type);
    }

    /**
     * @see org.eclipse.emf.edit.domain.IEditingDomainProvider#getEditingDomain()
     */
    public EditingDomain getEditingDomain()
    {
        return ttmEditor.getEditingDomain();
    }

    public String getContributorId()
    {
        return ttmEditor.getContributorId();
    }

    public TtmEditor getModelEditor()
    {
        return ttmEditor;
    }

    public void showModelPage()
    {
        setActivePage(0);
    }

    public void showAnalysisPage()
    {
        setActivePage(1);
    }

    public Project getProject()
    {
        return (Project) (getModelEditor().getDiagrams().getModel());
    }

    @Override
    /**
     * Assure that input is a ttmdi file
     */
    protected void setInput(IEditorInput input)
    {
        createChangeChecker();

        if (input instanceof IFileEditorInput)
        {
            IFile inputFile = ((IFileEditorInput) input).getFile();

            if (FileNameUtils.isDiagram(inputFile))
            {
                super.setInput(input);
                updateTitle(inputFile.getName());
                startChangeChecker();
            }
            else if (FileNameUtils.isResult(inputFile))
            {
                IProject project = inputFile.getProject();

                IPath ttmdiFilePath = FileNameUtils.resultToDiagram(inputFile);
                try
                {
                    project.refreshLocal(IResource.DEPTH_INFINITE, new NullProgressMonitor());
                }
                catch (CoreException e)
                {
                    TtmPlugin.log(e);
                }
                IResource ttmResFile = project.findMember(ttmdiFilePath);
                if (ttmResFile != null)
                {
                    openBordView = true;
                    super.setInput(new FileEditorInput((IFile) ttmResFile));
                    updateTitle(inputFile.getName());
                    startChangeChecker();
                }
                else
                {
                    //FIXME: Should never occured. Test and fix it.
                    updateTitle("unnamed");
                }
            }
        }
    }

    private void createChangeChecker()
    {
        if (changeChecker == null)
        {
            changeChecker = new DocumentChangeChecker(this);
        }
    }

    /**
     * Start the job testing changes on project files
     * calls at editor initialisation (setInput)
     * and after AnalyseAction. Indeed the job can be stopped by the
     * user. 
     */
    public void startChangeChecker()
    {
        if (changeChecker != null)
        {
            changeChecker.start();
        }
    }

    /**
     * Has the diagram been changed and not saved ?
     * 
     * @return
     */
    protected boolean hasDiagramChanged()
    {
        return ttmEditor.isDirty();
    }

    /**
     * Change the title of the editor, to add the name of the edited file.
     * 
     * @param name
     */
    private void updateTitle(String name)
    {
        /* FIXME : Setting the title like this can cause repetition.
         * We need to find a way to get the initial title of the editor.
         setPartName(getTitle() + " - " + name);
         */
    }

    public void updateViews()
    {
        boardView.refresh();
        coverageView.refresh();
        impactView.refresh();
    }

}
