/*******************************************************************************
 * No CopyrightText Defined in the configurator file.
 ******************************************************************************/
package org.topcased.traceability.hmi.modeler.policies;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy;
import org.topcased.modeler.utils.SourceTargetData;
import org.topcased.modeler.utils.Utils;
import org.topcased.traceability.hmi.modeler.TWSimpleObjectConstants;
import org.topcased.traceability.hmi.modeler.commands.CoversEdgeCreationCommand;

import ttm.CoverPoint;
import ttm.Document;

/**
 * Covers edge creation
 *
 * @generated NOT
 */
public class CoversEdgeCreationEditPolicy extends
		AbstractEdgeCreationEditPolicy {
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#createCommand(org.eclipse.gef.EditDomain, org.topcased.modeler.di.model.GraphEdge, org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected CreateTypedEdgeCommand createCommand(EditDomain domain,
			GraphEdge edge, GraphElement source) {
		return new CoversEdgeCreationCommand(domain, edge, source);
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkEdge(org.topcased.modeler.di.model.GraphEdge)
	 * @generated
	 */
	protected boolean checkEdge(GraphEdge edge) {
		if (edge.getSemanticModel() instanceof SimpleSemanticModelElement) {
			return TWSimpleObjectConstants.SIMPLE_OBJECT_COVERS
					.equals(((SimpleSemanticModelElement) edge
							.getSemanticModel()).getTypeInfo());
		}
		return false;
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkSource(org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected boolean checkSource(GraphElement source) {
		EObject object = Utils.getElement(source);
		if (object instanceof ttm.Document) {
			return true;
		}
		return false;
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkTargetForSource(org.topcased.modeler.di.model.GraphElement, org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected boolean checkTargetForSource(GraphElement source,
			GraphElement target) {
		EObject sourceObject = Utils.getElement(source);
		EObject targetObject = Utils.getElement(target);
		
		boolean res = false;
		
		if (sourceObject instanceof ttm.Document
				&& targetObject instanceof ttm.CoverPoint) {
			if (!sourceObject.equals(targetObject)) {
				
				// Do not allow the same link to be built between 
				// a document and the same cover point.
				ttm.Document sourceDoc = (Document) sourceObject;
				ttm.CoverPoint coverPoint = (CoverPoint) targetObject;
				
				res = !isLinked(sourceDoc, coverPoint) && !isCycle(sourceDoc, coverPoint);
				
			}
		}
		return res;
	}

	/**
	 * Are a coverPoint and a document closing a cycle ? 
	 * A cycle exists whenever a document A covers requirements of another 
	 * document B that covers document A, directly or indirectly.
	 * @param sourceDoc
	 * @param coverPoint
	 * @return
	 */
	private boolean isCycle(Document sourceDoc, CoverPoint coverPoint) {
		
		// Source Doc is the document covering requirements, document A
		Document docA = sourceDoc;		
		// CoverPoint is associated to a document (That is the covered document, B) 
		Document docB = coverPoint.getCoveredDocument();
		
		// Get the list of all documents covered by B
		Set<Document> documentsCoveredByB = getDocumentsCoveredBy(docB);
		// Is A covered by B ? 
		return documentsCoveredByB.contains(docA);
	}

	/**
	 * A set of all documents covered by a document, recursively.
	 * @param docB
	 * @return
	 */
	private Set<Document> getDocumentsCoveredBy(Document doc) {

		return accumulateDocumentsCoveredBy(new HashSet<Document>(), doc.getCoveredDocuments());
				
	}

	/**
	 * Recursively accumulates in a set all documents given as arguments, as
	 * well as all documents covered by those documents.
	 */
	private Set<Document> accumulateDocumentsCoveredBy(
			Set<Document> input, List<Document> coveredDocuments) {

		Set<Document> res = input;
		// Add the covered documents that are not already in the set.
		// This is a set, so there won't be any doublon.
		coveredDocuments.removeAll(input);
		// Add them in the set
		res.addAll(coveredDocuments);
		// Accumulates all document covered by the documents just added
		for (Document document : coveredDocuments) {
			res.addAll(accumulateDocumentsCoveredBy(res, document.getCoveredDocuments()));
		}
		
		// Note : at some point, either the list of covered documents will be empty, 
		// or all documents will be in the set. Therefore this recursion terminates.
				
		return res;
		
	}

	/**
	 * Are a document and a cover point already connected ? 
	 * @param sourceDoc
	 * @param coverPoint
	 * @return
	 */
	private boolean isLinked(Document sourceDoc, CoverPoint coverPoint) {
		return coverPoint.getConnectedDocuments().contains(sourceDoc);
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkCommand(org.eclipse.gef.commands.Command)
	 * @generated
	 */
	protected boolean checkCommand(Command command) {
		return command instanceof CoversEdgeCreationCommand;
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#getSourceTargetData(org.topcased.modeler.di.model.GraphElement, org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected SourceTargetData getSourceTargetData(GraphElement source,
			GraphElement target) {
		EObject sourceObject = Utils.getElement(source);
		EObject targetObject = Utils.getElement(target);

		if (sourceObject instanceof ttm.Document
				&& targetObject instanceof ttm.CoverPoint) {
			return new SourceTargetData(false, false, SourceTargetData.NONE,
					null, null, null, null, null, null, null,
					"connectedDocuments");
		}
		return null;
	}
}