/*******************************************************************************
 * No CopyrightText Defined in the configurator file.
 ******************************************************************************/
package org.topcased.traceability.hmi.preferences;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.topcased.modeler.preferences.ModelerPreferenceConstants;
import org.topcased.traceability.hmi.TtmPlugin;

/**
 * A preference page to configure all preferences of the Ttm editor.
 *
 * @generated NOT
 */
public class TtmPreferencePage extends PreferencePage implements
		IWorkbenchPreferencePage {
	
	/**
	 * @generated
	 */
	private IPreferenceStore ps = TtmPlugin.getDefault().getPreferenceStore();

	/**
	 * @generated
	 */
	private BooleanFieldEditor deleteGraphElements;

	/**
	 * @generated
	 */
	private BooleanFieldEditor deleteModelElements;
	
	
	/**
	 * field used to activate or desactivate the document check job
	 */
	private BooleanFieldEditor activateDocumentCheck;

	
	private IntegerFieldEditor documentCheckInterval;
	
	/**
	 * Creates this preference page contents.
	 *
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 * @generated
	 */
	protected Control createContents(Composite parent) {
		// Create the container composite
		Composite container = new Composite(parent, SWT.NONE);
		GridLayout containerLayout = new GridLayout();
		containerLayout.marginWidth = 0;
		containerLayout.marginHeight = 0;
		container.setLayout(containerLayout);
		container.setLayoutData(new GridData(GridData.FILL_BOTH));

		createHeader(container);

		createDeleteActionsGroup(container);

		createDocumentCheckGroup(container);
		
		
		loadPreferences();

		return container;
	}

	private void createDocumentCheckGroup(Composite parent) {
		Group group = new Group(parent, SWT.SHADOW_ETCHED_OUT);
		group.setLayout(new GridLayout());
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group.setText("Check document changes");

		Composite fieldsContainer = new Composite(group, SWT.NONE);
		fieldsContainer.setLayout(new GridLayout());
		fieldsContainer.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
				
		activateDocumentCheck = new BooleanFieldEditor(
                TtmPlugin.DOCUMENT_CHECK_ACTIVATION_PREFERENCE,
                "Activate the documents check job.",                
                fieldsContainer)
		{
		    @Override
		    public int getNumberOfControls()
		    {		     
		        //use to span the widget over the 2 columns with the 
		        //check on the leftside
		        return 2;
		    }
		};
		activateDocumentCheck.setPreferenceStore(ps);
		activateDocumentCheck.setPropertyChangeListener(new ActivationPropertyChangeListener(fieldsContainer));
		

		documentCheckInterval = new IntegerFieldEditor(TtmPlugin.DOCUMENT_CHECK_INTERVAL_PREFERENCE, 
				"Interval between checks for document changes on disk (seconds)", 
				fieldsContainer); 
		documentCheckInterval.setValidRange(TtmPlugin.DOCUMENT_CHECK_INTERVAL_MIN, TtmPlugin.DOCUMENT_CHECK_INTERVAL_MAX);
		documentCheckInterval.setEmptyStringAllowed(false);
		documentCheckInterval.setValidateStrategy(IntegerFieldEditor.VALIDATE_ON_KEY_STROKE);
		documentCheckInterval.setErrorMessage("Please enter a value between " + TtmPlugin.DOCUMENT_CHECK_INTERVAL_MIN +
				" and " + TtmPlugin.DOCUMENT_CHECK_INTERVAL_MAX + " seconds.");
		documentCheckInterval.setPage(this);
		documentCheckInterval.setPreferenceStore(ps);
		
	}

	/**
	 * Initializes the HMI with preference values.
	 *
	 * @generated
	 */
	private void loadPreferences() {
		deleteGraphElements.load();
		deleteModelElements.load();
		documentCheckInterval.load();
		activateDocumentCheck.load();
	}

	/**
	 * Stores the HMI values into the preference store.
	 *
	 * @generated
	 */
	private void storePreferences() {
		deleteGraphElements.store();
		deleteModelElements.store();
		documentCheckInterval.store();
		activateDocumentCheck.store();
	}

	/**
	 * Initializes the HMI with default preference values.
	 *
	 * @generated
	 */
	private void loadDefaultPreferences() {
		deleteGraphElements.loadDefault();
		deleteModelElements.loadDefault();
		documentCheckInterval.loadDefault();
		activateDocumentCheck.loadDefault();
	}

	/**
	 * Creates this preference page header.
	 *
	 * @param parent the parent composite
	 * @generated
	 */
	private void createHeader(Composite parent) {
		new Label(parent, SWT.WRAP).setText("TRAMway editor preference page");
	}

	/**
	 * Creates the group to configure the delete actions preferences.
	 * 
	 * @param parent the parent composite
	 * @generated
	 */
	private void createDeleteActionsGroup(Composite parent) {
		Group group = new Group(parent, SWT.SHADOW_ETCHED_OUT);
		group.setLayout(new GridLayout());
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		group.setText("Delete actions");

		Composite fieldsContainer = new Composite(group, SWT.NONE);
		fieldsContainer.setLayout(new GridLayout());
		fieldsContainer.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		deleteGraphElements = new BooleanFieldEditor(
				ModelerPreferenceConstants.DELETE_ACTION_CONFIRM,
				"Do not ask for confirmation before deleting graphical elements.",
				fieldsContainer);
		deleteGraphElements.setPreferenceStore(ps);

		deleteModelElements = new BooleanFieldEditor(
				ModelerPreferenceConstants.DELETE_MODEL_ACTION_CONFIRM,
				"Do not ask for confirmation before deleting model elements.",
				fieldsContainer);
		deleteModelElements.setPreferenceStore(ps);
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 * @generated
	 */
	public void init(IWorkbench workbench) {
		// Do nothing
	}

	/**
	 * @see org.eclipse.jface.preference.IPreferencePage#performOk()
	 * @generated
	 */
	public boolean performOk() {
		storePreferences();
		return super.performOk();
	}

	/**
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 * @generated
	 */
	protected void performDefaults() {
		loadDefaultPreferences();
		super.performDefaults();
	}
	
	
	/**
	 * 
	 * Listener use to disable/enable documentCheckInterval on 
	 * activateDocumentCheck changes
	 * <br>
	 * creation : 29 juin 07
	 *
	 * @author <a href="mailto:mickael.gerard@anyware-tech.com">Mickael Gerard</a>
	 */
	class ActivationPropertyChangeListener implements IPropertyChangeListener
	{
	    Composite _parent;
	    
	    public ActivationPropertyChangeListener(Composite parent)
	    {
	        _parent = parent;
	    }
	    
        public void propertyChange(org.eclipse.jface.util.PropertyChangeEvent event)
        {
            documentCheckInterval.setEnabled((event.getNewValue() == Boolean.TRUE), _parent);           
        }
	}
}
