//------------------------------------------------
// $Id: AnalyserIdSection.java,v 1.4 2007-06-28 14:00:53 mickael Exp $
// (c) Anyware Technologies 2007    www.anyware-tech.com
//------------------------------------------------
package org.topcased.traceability.hmi.properties.document;

import java.util.Map;

import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;
import org.topcased.tabbedproperties.sections.AbstractChooserPropertySection;

import tte.extension.DocumentAnalyserExtension;
import tte.extension.DocumentAnalyserManager;
import ttm.TtmPackage;

/**
 * 
 * Section use to display the AnalyserId property of a Document
 * <br>
 * creation : 5 avr. 07
 *
 * @author <a href="mailto:mickael.gerard@anyware-tech.com">Mickael Gerard</a>
 */
public class AnalyserIdSection extends AbstractChooserPropertySection
{

    @Override
    protected EStructuralFeature getFeature()
    {
    	return TtmPackage.eINSTANCE.getDocument_AnalyzerID();
    }

    @Override
    protected String getLabelText()
    {
       return "Document Analyser";
    }

    @Override
    protected Object[] getComboFeatureValues()
    {
        DocumentAnalyserManager manager = DocumentAnalyserManager.getInstance();
        
        Map<String, DocumentAnalyserExtension> analysers = manager.getContributorExtensions();
        if(!analysers.isEmpty())
        {
            return analysers.values().toArray();
        }
        
        return null;
    }

    @Override
    protected Object getFeatureValue()
    {
        String analyserIdentification = getEObject() == null ? null : (String) getEObject().eGet(getFeature());
        if (analyserIdentification == null)
        {
            return "";
        }        
        DocumentAnalyserManager manager = DocumentAnalyserManager.getInstance();        
        return manager.getDocumentAnalyserExtensionByIdentification(analyserIdentification);
    }
    
    @Override
    protected void handleComboModified()
    {               
        DocumentAnalyserExtension analyser = (DocumentAnalyserExtension)getCSingleObjectChooser().getSelection();
        if(analyser != null)
        {
            EditingDomain editingDomain = getEditingDomain();
            CompoundCommand command = new CompoundCommand();            
            
            // The list of available conversion methods depends on the property set  to 'analyser.getId()' by the first an last command.
            // The list is computed after the model changes.
            // Setting the analyser id property twice, and the conversion method to null, makes it possible to do undo properly : 
            // (no analyser, no method) <-> (A1, no method) <-> (A1, M1) <-> (A1, M2) <-> (A2, null) <-> (A2, M2)
            command.append(SetCommand.create(editingDomain, getEObject(), getFeature(), analyser.getId()));
            command.append(SetCommand.create(editingDomain, getEObject(), TtmPackage.eINSTANCE.getDocument_ConversionMethodID(), null));
            command.append(SetCommand.create(editingDomain, getEObject(), TtmPackage.eINSTANCE.getDocument_EditionMethodID(), null));            
            command.append(SetCommand.create(editingDomain, getEObject(), getFeature(), analyser.getId()));
            
            editingDomain.getCommandStack().execute(command);                                               
            
            
        }
    }    
    
    
    @Override
    protected String getItemLabelText(EObject object)
    {
        
        return super.getItemLabelText(object);
    }
     
    
    @Override
    protected ILabelProvider getLabelProvider()
    {
        return new DocumentAnalyserExtensionLabelProvider();
    }
    
    class DocumentAnalyserExtensionLabelProvider implements ILabelProvider
    {

        public Image getImage(Object element)
        {
            // TODO Auto-generated method stub
            return null;
        }

        public String getText(Object element)
        {
            if(element != null && element instanceof DocumentAnalyserExtension)
            {
                return ((DocumentAnalyserExtension)element).getName();
            }
            return null;
        }

        public void addListener(ILabelProviderListener listener)
        {
            // TODO Auto-generated method stub
            
        }

        public void dispose()
        {
            // TODO Auto-generated method stub
            
        }

        public boolean isLabelProperty(Object element, String property)
        {
            // TODO Auto-generated method stub
            return false;
        }

        public void removeListener(ILabelProviderListener listener)
        {
            // TODO Auto-generated method stub
            
        }
        
    }

}
