//------------------------------------------------
// $Id: ConversionMethodIdSection.java,v 1.1 2007-05-22 10:23:58 matg Exp $
// (c) Anyware Technologies 2007    www.anyware-tech.com
//------------------------------------------------
package org.topcased.traceability.hmi.properties.document;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.swt.graphics.Image;
import org.topcased.tabbedproperties.sections.AbstractChooserPropertySection;

import tte.extension.ConversionMethodExtension;
import tte.extension.DocumentAnalyserExtension;
import tte.extension.DocumentAnalyserManager;
import ttm.TtmPackage;

/**
 * 
 * Section use to display the ConversionMethodId property of a Document
 * <br>
 * creation : 5 avr. 07
 *
 * @author <a href="mailto:mickael.gerard@anyware-tech.com">Mickael Gerard</a>
 */
public class ConversionMethodIdSection extends AbstractChooserPropertySection
{

    @Override
    protected EStructuralFeature getFeature()
    {
        return TtmPackage.eINSTANCE.getDocument_ConversionMethodID();
    }

    @Override
    protected String getLabelText()
    {
       return "Conversion Method";
    }

    @Override
    protected Object[] getComboFeatureValues()
    {
        DocumentAnalyserManager manager = DocumentAnalyserManager.getInstance();
        //Retrieve analyser ID
        String analyserId= getEObject() == null ? null : (String) getEObject().eGet(TtmPackage.eINSTANCE.getDocument_AnalyzerID());
        if (analyserId != null)
        {     
            //Retrieve all the ConversionMethods of the selected analyzer ID
            DocumentAnalyserExtension analyser = manager.getDocumentAnalyserExtensionByExtensionId(analyserId);
            if(analyser != null && analyser.getConversionMethods() != null)
            {
                return analyser.getConversionMethods().toArray();
            }           
        }               
        return null;
    }

    @Override
    protected Object getFeatureValue()
    {
        DocumentAnalyserManager manager = DocumentAnalyserManager.getInstance();
        String analyserId= getEObject() == null ? null : (String) getEObject().eGet(TtmPackage.eINSTANCE.getDocument_AnalyzerID());
        if (analyserId != null)
        {     
            String conversionMethodId = getEObject() == null ? null : (String) getEObject().eGet(getFeature());
            if(conversionMethodId != null)
            {
                //Retrieve all the ConversionMethods of the selected analyzer ID
                DocumentAnalyserExtension analyser = manager.getDocumentAnalyserExtensionByExtensionId(analyserId);
                return analyser.getConversionMethod(conversionMethodId);
               
            }
        }    
        return null;
    }
    
    @Override
    protected void handleComboModified()
    {               
        ConversionMethodExtension conversionMethod = (ConversionMethodExtension)getCSingleObjectChooser().getSelection();
        if(conversionMethod != null)
        {
            EditingDomain editingDomain = getEditingDomain();
            editingDomain.getCommandStack().execute(SetCommand.create(editingDomain, getEObject(), getFeature(), conversionMethod.getIdentification()));          
        }
    }    
    
    
    @Override
    protected String getItemLabelText(EObject object)
    {
        
        return super.getItemLabelText(object);
    }
     
    
    @Override
    protected ILabelProvider getLabelProvider()
    {
        return new ConversionMethodExtensionLabelProvider();
    }
    
    class ConversionMethodExtensionLabelProvider implements ILabelProvider
    {

        public Image getImage(Object element)
        {
            // TODO Auto-generated method stub
            return null;
        }

        public String getText(Object element)
        {
            if(element != null && element instanceof ConversionMethodExtension)
            {
                return ((ConversionMethodExtension)element).getDisplayIdentification();
            }
            return null;
        }

        public void addListener(ILabelProviderListener listener)
        {
            // TODO Auto-generated method stub
            
        }

        public void dispose()
        {
            // TODO Auto-generated method stub
            
        }

        public boolean isLabelProperty(Object element, String property)
        {
            // TODO Auto-generated method stub
            return false;
        }

        public void removeListener(ILabelProviderListener listener)
        {
            // TODO Auto-generated method stub
            
        }
        
    }

}
