/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.runtime;


import org.eclipse.core.resources.IResource;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.topcased.traceability.hmi.util.ResourceLoadURIHelper;

import ttm.Project;

/**
 * Created on 21 f�vr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * The ResourceLoader class handles the loading of the project contained in a ttmresult file
 * 
 */
public class ResourceLoader
{

    private static ResourceLoader instance;

    private ResourceLoader()
    {
    }

    /**
     * Return the unique instance of the ResourceLoader class
     * 
     * @return the ResourceLoader instance
     */
    public static ResourceLoader getInstance()
    {
        if (instance == null)
        {
            instance = new ResourceLoader();
        }
        return instance;
    }

    /**
     * Load the project from the given file
     * 
     * @param file input file (*.ttmresults)
     * @return a Project instance
     */
    public Project load(IResource resource)
    {
        ResourceSet resourceSet = new ResourceSetImpl();

        // Register the default resource factory -- only needed for stand-alone!
        resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());
        
        // Get the URI of the model file.
        URI fileURI = ResourceLoadURIHelper.createURI(resource);
        
        Resource loadedResource = resourceSet.getResource(fileURI, true);
                  
        if (loadedResource.getContents().isEmpty())
        {
            // TODO: handle this case
        }
        else
        {
            return (Project) loadedResource.getContents().get(0);
        }

        return null;
    }
  
}
