/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.ui.decorator;

import java.util.List;

import org.eclipse.jface.viewers.IDecoration;
import org.eclipse.jface.viewers.ILightweightLabelDecorator;
import org.eclipse.jface.viewers.LabelProvider;
import org.topcased.traceability.hmi.TtmImageRegistry;
import org.topcased.traceability.hmi.util.RuleHelper;

import ttm.Requirement;
import ttm.Rule;
import ttm.RuleInstance;

/**
 * Created on 21 fvr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * Decorator management.
 * 
 */
public class RequirementDecorator extends LabelProvider implements ILightweightLabelDecorator
{

    /**
     * @see org.eclipse.jface.viewers.ILightweightLabelDecorator#decorate(java.lang.Object,
     *      org.eclipse.jface.viewers.IDecoration)
     */
    public void decorate(Object element, IDecoration decoration)
    {

        Rule rule = null;

        if (element instanceof RuleInstance)
        {
            rule = ((RuleInstance) element).getBaseRule();
        }
        else if (element instanceof Requirement)
        {
            List<RuleInstance> activatedRules = ((Requirement) element).getActivatedRules();

            if (!activatedRules.isEmpty())
            {
                rule = activatedRules.get(0).getBaseRule();
            }
        }

        if (rule != null)
        {
            String key = RuleHelper.getDecoratorKey(rule);
            decoration.addOverlay(TtmImageRegistry.getImageDescriptor(key));
        }

    }
}
