/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.ui.form;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.topcased.traceability.hmi.TtmImageRegistry;
import org.topcased.traceability.hmi.TtmPlugin;
import org.topcased.traceability.hmi.actions.EditDocumentAction;
import org.topcased.traceability.hmi.contextmenu.ContextAction;
import org.topcased.traceability.hmi.contextmenu.ContextMenuManager;
import org.topcased.traceability.hmi.runtime.ResourceLoader;
import org.topcased.traceability.hmi.ui.form.provider.MetricContentProvider;
import org.topcased.traceability.hmi.ui.form.provider.MetricTraceabilityViewLabelProvider;
import org.topcased.traceability.hmi.ui.form.provider.ReportViewContentProvider;
import org.topcased.traceability.hmi.ui.form.provider.ReportViewLabelProvider;
import org.topcased.traceability.hmi.util.Root;
import org.topcased.traceability.hmi.util.TraceabilityConstants;

import ttm.Document;
import ttm.Project;

/**
 * Created on 21 f�vr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * The TRAMWay board view
 */
public class BoardView extends RefreshableFormPage
{
	public BoardView(FormEditor editor, String id, String title) 
	{
		super(editor, id, title);	
        toolkit = editor.getToolkit();
	}

	private FormToolkit toolkit;

    private ScrolledForm form;

    private TreeViewer mainViewer;

    private TreeViewer reportViewer;

    private Project project;

    private IResource currentFile;
    
    private boolean initDone = false;


    /**
     * VIEW_ID
     */
    public static final String VIEW_ID = "org.topcased.traceability.hmi.boardview";
    
    
   @Override
    protected void createFormContent(IManagedForm managedForm)
    {       
        form = managedForm.getForm();
        toolkit = managedForm.getToolkit();
        
        project = null;

        GridLayout layout = new GridLayout();
        form.getBody().setLayout(layout);

        form.setText("TRAMway");

        mainViewer = createMainSection("Metrics", form.getBody());
        
        reportViewer = createReportingSection(form.getBody());
      
        setFormContentCreated(true);
        
        
    }

    private TreeViewer createReportingSection(Composite parent)
    {
        Section section = toolkit.createSection(parent, Section.TWISTIE | Section.TITLE_BAR | Section.EXPANDED);

        section.setLayout(new GridLayout());
        GridData gd1 = new GridData(GridData.FILL_BOTH);
        gd1.horizontalSpan = 2;
        section.setLayoutData(gd1);
        section.setText("Reporting");

        Composite sectionClient = toolkit.createComposite(section, SWT.BORDER);
        sectionClient.setLayout(new GridLayout());
        sectionClient.setLayoutData(new GridData(GridData.FILL_BOTH));

        final TreeViewer viewer = new TreeViewer(sectionClient, SWT.H_SCROLL | SWT.V_SCROLL);
        viewer.setContentProvider(new ReportViewContentProvider());

        viewer.setLabelProvider(new DecoratingLabelProvider(new ReportViewLabelProvider(), PlatformUI.getWorkbench().getDecoratorManager().getLabelDecorator()));
        viewer.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));
        viewer.setInput(project);

        section.setClient(sectionClient);
        
        return viewer;
    }

    private TreeViewer createMainSection(String title, Composite parent)
    {
        Section section = toolkit.createSection(parent, Section.TITLE_BAR);
        section.setLayout(new GridLayout());
        GridData gd1 = new GridData(GridData.FILL_BOTH);
        section.setLayoutData(gd1);

        section.setText(title);

        Composite sectionClient = toolkit.createComposite(section, SWT.BORDER);
        sectionClient.setLayout(new GridLayout());
        sectionClient.setLayoutData(new GridData(GridData.FILL_BOTH));
        TreeViewer viewer = new TreeViewer(sectionClient, SWT.H_SCROLL | SWT.V_SCROLL);

        viewer.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));

        viewer.setContentProvider(new MetricContentProvider());
        viewer.setLabelProvider(new DecoratingLabelProvider(new MetricTraceabilityViewLabelProvider(), PlatformUI.getWorkbench().getDecoratorManager().getLabelDecorator()));
        
        if (project == null)
        {
            viewer.setInput(TraceabilityConstants.NO_PROJECT_LOADED);
        }
        else
        {
        	viewer.setInput(project);
        }
        
        section.setClient(sectionClient);
        
        ContextMenuManager menuMgr = new ContextMenuManager(viewer);
        final ContextAction editDocAction = new EditDocumentAction();
        menuMgr.addAction(editDocAction, Document.class);
        
        return viewer;
    }


    protected void updateBoard()
    {
        reportViewer.setSelection(null);
        
        Thread job = new Thread(new Runnable()
        {
            public void run()
            {
                project = ResourceLoader.getInstance().load(currentFile);                   
                Display.getDefault().asyncExec(new Runnable()
                {
                    public void run()
                    {
                    	mainViewer.setInput(new Root(project));
                        reportViewer.setInput(project);
                        mainViewer.expandToLevel(2);
                        if (project.getRules().isEmpty()) {
                        	form.setImage(TtmImageRegistry.getImage(TraceabilityConstants.RULE_OK_KEY));
                        }
                        else {
                        	form.setImage(TtmImageRegistry.getImage(TraceabilityConstants.RULE_KO_KEY));
                        }
                    }
                });
            }
        });
        job.start();      
        
        
    }
    
    
    @Override
    protected void setInput(IEditorInput input)
    {        
        super.setInput(input);       
    }
    
    @Override
    public void setActive(boolean active)
    {
        super.setActive(active);
        if(active)
        {
            if(!initDone)
            {
                initFormContent();                
            }
        }
    }
    
    /**
     * Initialise the form with the ttmresults link to the ttmdi used as input
     */
    private void initFormContent()
    {        
        IEditorInput input = getEditorInput();  
        if (input instanceof IFileEditorInput)
        {
            IFile inputFile = ((IFileEditorInput) input).getFile();
            IProject project = inputFile.getProject();            
            IPath ttmResultsFilePath = inputFile.getProjectRelativePath();
            ttmResultsFilePath = ttmResultsFilePath.removeFileExtension();
            ttmResultsFilePath = ttmResultsFilePath.addFileExtension("ttmresults");
            try {
				project.refreshLocal(IResource.DEPTH_INFINITE, new NullProgressMonitor());
			} catch (CoreException e) {
				TtmPlugin.log(e);
			}
            IResource ttmResFile = project.findMember(ttmResultsFilePath);            
            if(ttmResFile != null)
            {        
                currentFile = ttmResFile;
                updateBoard();
                //updateBoard(project);
                initDone = true;
            }
            else
            {
                mainViewer.setInput(TraceabilityConstants.NOT_ANALYSED_PROJECT);
            }
        }
    }
        

}
