/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.ui.form;

import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.widgets.Section;
import org.topcased.traceability.hmi.actions.EditDocumentAction;
import org.topcased.traceability.hmi.contextmenu.ContextAction;
import org.topcased.traceability.hmi.contextmenu.ContextMenuManager;
import org.topcased.traceability.hmi.ui.form.provider.CoverByViewLabelProvider;
import org.topcased.traceability.hmi.ui.form.provider.MainViewContentProvider;
import org.topcased.traceability.hmi.ui.form.provider.ReportViewContentProvider;
import org.topcased.traceability.hmi.ui.form.provider.ReportViewLabelProvider;
import org.topcased.traceability.hmi.ui.form.provider.TraceabilityViewLabelProvider;
import org.topcased.traceability.hmi.ui.form.provider.coverage.CoverByViewContentProvider;
import org.topcased.traceability.hmi.ui.form.provider.coverage.CoverViewContentProvider;
import org.topcased.traceability.hmi.util.TraceabilityConstants;

import ttm.Element;

/**
 * Created on 21 f�vr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * The TRAMWay board view
 */
public class CoverageView extends RequirementRefreshableFormPage
{
    /**
     * VIEW_ID
     */
    public static final String VIEW_ID = "org.topcased.traceability.hmi.boardview";

    /**
     * Default constructor
     * @param editor
     * @param id
     * @param title
     */
    public CoverageView(FormEditor editor, String id, String title) 
	{
		super(editor, id, title);	
        toolkit = editor.getToolkit();
	}
    
    
   @Override
    protected void createFormContent(IManagedForm managedForm)
    {       
        form = managedForm.getForm();
        toolkit = managedForm.getToolkit();
        
        project = null;
        //form = toolkit.createScrolledForm(parent);

        GridLayout layout = new GridLayout(3, true);
        form.getBody().setLayout(layout);

        form.setText("TRAMway");
        
        mainViewer = createViewerSection("Selection", form.getBody(), 2);
        initializeMainViewer();

        reportViewer = createReportingSection(form.getBody());

        coverViewer = createViewerSection("Covers", form.getBody(), 1);
        initializeCoverViewer();

        coverByViewer = createViewerSection("Covered By", form.getBody(), 1);
        initializeCoverByViewer();

        createPropertiesSection("Properties", mainViewer);
        createPropertiesSection("Properties", coverViewer);
        createPropertiesSection("Properties", coverByViewer);

        addStandartActions(coverViewer);
        addStandartActions(coverByViewer);
        addStandartActions(reportViewer);
      
        
        setFormContentCreated(true);
    }
    


    private void initializeMainViewer()
    {
        mainViewer.setContentProvider(new MainViewContentProvider());
        mainViewer.setLabelProvider(new DecoratingLabelProvider(new TraceabilityViewLabelProvider(), PlatformUI.getWorkbench().getDecoratorManager().getLabelDecorator()));
        if (project == null)
        {
            mainViewer.setInput(TraceabilityConstants.NO_PROJECT_LOADED);
        }
        else
        {
            mainViewer.setInput(project);
        }
        
        ContextMenuManager menuMgr = new ContextMenuManager(mainViewer);
        final ContextAction editDocAction = new EditDocumentAction();
        menuMgr.addAction(editDocAction, Element.class);
    }

    private void initializeCoverViewer()
    {
        coverViewer.setContentProvider(new CoverViewContentProvider());
        coverViewer.setSorter(new ViewerSorter());
        coverViewer.setLabelProvider(new DecoratingLabelProvider(new TraceabilityViewLabelProvider(), PlatformUI.getWorkbench().getDecoratorManager().getLabelDecorator()));
    }

    private void initializeCoverByViewer()
    {
        coverByViewer.setContentProvider(new CoverByViewContentProvider());
        coverByViewer.setSorter(new ViewerSorter());
        CoverByViewLabelProvider labelProvider = new CoverByViewLabelProvider();
        mainViewer.addSelectionChangedListener(labelProvider);
        coverByViewer.setLabelProvider(new DecoratingLabelProvider(labelProvider, PlatformUI.getWorkbench().getDecoratorManager().getLabelDecorator()));
    }

    private TreeViewer createReportingSection(Composite parent)
    {
        Section section = toolkit.createSection(parent, Section.TWISTIE | Section.TITLE_BAR);

        section.setLayout(new GridLayout());
        GridData gd1 = new GridData(GridData.FILL_HORIZONTAL);
        gd1.horizontalSpan = 2;
        section.setLayoutData(gd1);
        section.setText("Reporting");

        Composite sectionClient = toolkit.createComposite(section, SWT.BORDER);
        sectionClient.setLayout(new GridLayout());
        sectionClient.setLayoutData(new GridData(GridData.FILL_BOTH));

        final TreeViewer viewer = new TreeViewer(sectionClient, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
        viewer.setContentProvider(new ReportViewContentProvider());

        viewer.setLabelProvider(new DecoratingLabelProvider(new ReportViewLabelProvider(), 
        		PlatformUI.getWorkbench().getDecoratorManager().getLabelDecorator()));
        viewer.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));
        viewer.setInput(project);
        
        section.setClient(sectionClient);

        return viewer;
    }

    
	private TreeViewer createViewerSection(String title, Composite parent, int vspan)
    {
        Section section = toolkit.createSection(parent, Section.TITLE_BAR);
        section.setLayout(new GridLayout());
        GridData gd1 = new GridData(GridData.FILL_BOTH);
        gd1.verticalSpan = vspan;
        section.setLayoutData(gd1);

        section.setText(title);

        Composite sectionClient = toolkit.createComposite(section, SWT.BORDER);
        sectionClient.setLayout(new GridLayout());
        sectionClient.setLayoutData(new GridData(GridData.FILL_BOTH));
        TreeViewer viewer = new TreeViewer(sectionClient, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

        viewer.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));

        section.setClient(sectionClient);
        return viewer;
    }

    
	
    
    @Override
    protected void setInput(IEditorInput input)
    {        
        super.setInput(input);       
    }
        
}
