package org.topcased.traceability.hmi.ui.form;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.events.ExpansionAdapter;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.topcased.traceability.hmi.TtmImageRegistry;
import org.topcased.traceability.hmi.TtmPlugin;
import org.topcased.traceability.hmi.actions.EditDocumentAction;
import org.topcased.traceability.hmi.actions.FindRequirementAction;
import org.topcased.traceability.hmi.contextmenu.ContextAction;
import org.topcased.traceability.hmi.contextmenu.ContextMenuManager;
import org.topcased.traceability.hmi.runtime.ResourceLoader;
import org.topcased.traceability.hmi.ui.PropertiesComposite;
import org.topcased.traceability.hmi.util.ModelHelper;
import org.topcased.traceability.hmi.util.Root;
import org.topcased.traceability.hmi.util.TraceabilityConstants;

import ttm.Element;
import ttm.Project;
import ttm.Requirement;
import ttm.RuleInstance;

public abstract class RequirementRefreshableFormPage extends RefreshableFormPage implements IRequirementProvider {

	protected Project project;
	
	protected TreeViewer mainViewer;

	protected TreeViewer reportViewer;
	
	protected IResource currentFile;

	protected FormToolkit toolkit;

	protected ScrolledForm form;

	protected TreeViewer coverViewer;

	protected TreeViewer coverByViewer;

	private List<ContextAction> extraActions = new ArrayList<ContextAction>();
	
	private String nextIdent;

	private IRequirementProvider requirementProvider;
	
	public RequirementRefreshableFormPage(FormEditor arg0, String arg1, String arg2) {
		super(arg0, arg1, arg2);
	}

	public void selectRequirementByIdent(String ident) {
		   
		
		List<Requirement> requirements = ModelHelper.getRequirements(project);
		for (Requirement requirement : requirements) {
			if (requirement.getIdent().equals(ident)) {
				mainViewer.setSelection(new StructuredSelection(requirement));
			}
		}
		
	}


	protected boolean forwardNextChange = true;
	
	public boolean shouldForwardNextChange() {
		return forwardNextChange;
	}
	
	public void setShouldForwardNextChange(boolean forward) {
		forwardNextChange = forward;
	}

	@Override
    public void setActive(boolean active)
    {
        super.setActive(active);
        if(active)
        {
            if(!isFormContentInitialized())
            {
                initFormContent();                
            } 
            
            selectRequirementFromProvider();
       }
    }

	private void selectRequirementFromProvider() {
		
        if (requirementProvider != null ) {
        	Requirement requirement = requirementProvider.getSelectedRequirement();
        	if (requirement != null) {
        		selectRequirementByIdent(requirement.getIdent());
        	}
        	
        }

	}
	
	
	private boolean initDone;

	public boolean isFormContentInitialized() {
		return initDone;
	}
	
	public void setFormContentInitialized(boolean initDone) {
		this.initDone = initDone;
	}
	
	
    protected void updateBoard()
    {
        mainViewer.setSelection(null);
        reportViewer.setSelection(null);
        
        Thread job = new Thread(new Runnable()
        {
            public void run()
            {
                project = (ResourceLoader.getInstance().load(currentFile));
                
                Display.getDefault().asyncExec(new Runnable()
                {
                    public void run()
                    {
                    	
                        mainViewer.setInput(project);
                        reportViewer.setInput(project);
                        
                        if (!isFormContentInitialized()) {
                        	setFormContentInitialized(true);
                        }
                        if (!isFormContentCreated()) {
                        	setFormContentCreated(true);
                        }
                        
                        selectRequirementFromProvider();
                        
                        if (project.getRules().isEmpty()) {
                        	form.setImage(TtmImageRegistry.getImage(TraceabilityConstants.RULE_OK_KEY));
                        }
                        else {
                        	form.setImage(TtmImageRegistry.getImage(TraceabilityConstants.RULE_KO_KEY));
                        }
                    }
                });
            }
        });
        job.start();                
    }
	
	
    protected void initFormContent()
    {        
        IEditorInput input = getEditorInput();
        
        if (input instanceof IFileEditorInput)
        {
            IFile inputFile = ((IFileEditorInput) input).getFile();
            IProject project = inputFile.getProject();            
            IPath ttmResultsFilePath = inputFile.getProjectRelativePath();
            ttmResultsFilePath = ttmResultsFilePath.removeFileExtension();
            ttmResultsFilePath = ttmResultsFilePath.addFileExtension("ttmresults");
            try {
				project.refreshLocal(IResource.DEPTH_INFINITE, new NullProgressMonitor());
			} catch (CoreException e) {
				TtmPlugin.log(e);
			}
            IResource ttmResFile = project.findMember(ttmResultsFilePath);            
            if(ttmResFile != null)
            {        
                currentFile = ttmResFile;
                updateBoard();
                // initDone = true;
                // setFormContentInitialized(true);
            }
            else
            {
                mainViewer.setInput(TraceabilityConstants.NOT_ANALYSED_PROJECT);
            }
        }
    }
    
    public Requirement getSelectedRequirement() {
    	Requirement requirement = null;
    	if(mainViewer != null)
    	{
    	    ISelection selection = mainViewer.getSelection();
        	if (selection != null && selection instanceof IStructuredSelection) {
        		IStructuredSelection structuredSelection = (IStructuredSelection) selection;
        		Object firstElement = structuredSelection.getFirstElement();
        		if (firstElement instanceof Requirement) {
        			requirement = (Requirement) firstElement;
        		}
        		
        	}
    	}
    	return requirement;
    }
    
    
    protected void createPropertiesSection(String title, final TreeViewer viewer)
    {
        Section section = toolkit.createSection(form.getBody(), Section.TITLE_BAR);
        section.setLayout(new GridLayout());
        GridData gd1 = new GridData(GridData.FILL_HORIZONTAL);
        section.setLayoutData(gd1);

        section.addExpansionListener(new ExpansionAdapter()
        {
            public void expansionStateChanged(ExpansionEvent e)
            {
                form.reflow(true);
            }
        });
        section.setText(title);

        Composite sectionClient = toolkit.createComposite(section);
        sectionClient.setLayout(new GridLayout());

        final PropertiesComposite properties = new PropertiesComposite(sectionClient, SWT.NONE);
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);

        gd.heightHint = 150;
        properties.setLayoutData(gd);
        properties.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WHITE));

        viewer.addSelectionChangedListener(new ISelectionChangedListener()
        {
            public void selectionChanged(SelectionChangedEvent event)
            {
                Object selection = ((IStructuredSelection) event.getSelection()).getFirstElement();
                properties.updateContent(selection);
                if (viewer.equals(mainViewer))
                {
                    coverViewer.setSelection(null);
                    coverViewer.setInput(new Root(selection));
                    coverViewer.expandAll();

                    coverByViewer.setSelection(null);
                    coverByViewer.setInput(new Root(selection));
                    coverByViewer.expandAll();
                    
                }
            }
        });

        section.setClient(sectionClient);
    }
	
    protected void addStandartActions(final TreeViewer viewer)
    {
        ContextMenuManager menuMgr = new ContextMenuManager(viewer); 
        
        final ContextAction findReqAction = new FindRequirementAction(mainViewer);
        final ContextAction editDocAction = new EditDocumentAction();
        
        menuMgr.addAction(editDocAction, new Class[]{ Element.class, RuleInstance.class });
        menuMgr.addAction(findReqAction, new Class[]{ Element.class, RuleInstance.class });
        
        
        viewer.addDoubleClickListener(new IDoubleClickListener()
        {
            public void doubleClick(DoubleClickEvent event)
            {
                Object selected = ((IStructuredSelection) viewer.getSelection()).getFirstElement();
                if (selected instanceof RuleInstance)
                {
                    selectRequirement(ModelHelper.getElementFromRule((RuleInstance) selected));
                } else {
                    selectRequirement(selected);
                }
            }
        });
        
        // addExtraActions(menuMgr, viewer);
        
    }

	
	/**
     * Select in the main tree viewer the given object.
     * This can have no effect if the object has no counterpart in the tree.
     * @param object
     */
	public void selectRequirement(Object object) {
		if (object == null) {
			TtmPlugin.log("Unable to find object", IStatus.WARNING);
			return;
		}
		IStructuredSelection selection = new StructuredSelection(object);
		mainViewer.setSelection(selection);
	}	
    

	public IRequirementProvider getRequirementProvider() {
		return requirementProvider;
	}

	public void setRequirementProvider(IRequirementProvider requirementProvider) {
		this.requirementProvider = requirementProvider;
	}
	
    
}