package org.topcased.traceability.hmi.ui.form;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.topcased.traceability.hmi.TtmPlugin;
import org.topcased.traceability.hmi.runtime.ResourceLoader;
import org.topcased.traceability.hmi.ui.form.provider.table.RequirementTableContentProvider;
import org.topcased.traceability.hmi.ui.form.provider.table.RequirementTableViewLabelProvider;
import org.topcased.traceability.hmi.ui.util.TableViewerSorter;
import org.topcased.traceability.hmi.util.TraceabilityConstants;

import ttm.Project;


public class RequirementTableView extends RefreshableFormPage {

	private FormToolkit toolkit;
	private ScrolledForm form;
	private TableViewer tableViewer;
	private TableViewerSorter tableSorter;
	private IResource currentFile;
	private boolean initDone;
	protected Project project;
	
	public RequirementTableView(FormEditor editor, String arg1, String arg2) {
		super(editor, arg1, arg2);
		toolkit = editor.getToolkit();
	}

	@Override
	protected void updateBoard() {
		
		
		
		tableViewer.setSelection(null);
        
        Thread job = new Thread(new Runnable()
        {
            public void run()
            {
                project = ResourceLoader.getInstance().load(currentFile);                   
                Display.getDefault().asyncExec(new Runnable()
                {
                    public void run()
                    {
                    	// Requirements will be read from the project
                        tableViewer.setInput(project);
                    }
                });
            }
        });
        
        job.start();
                  
	}

	@Override
	protected void createFormContent(IManagedForm managedForm) {
		form = managedForm.getForm();
		toolkit = managedForm.getToolkit();
		
		tableSorter = new TableViewerSorter();
		tableViewer = createTableViewer();
	
		form.getBody().setLayout(new GridLayout());
		
		setFormContentCreated(true);
		
	}

	
	private TableViewer createTableViewer() {
		Table table = createTable(form.getBody() , 
				new String[] { "File", "Requirement", "Description"}  
				);
		TableViewer res = new TableViewer(table);
		res.setSorter(tableSorter.getSorter());
		res.setContentProvider(new RequirementTableContentProvider());
		res.setLabelProvider(new RequirementTableViewLabelProvider());
		return res;
	}
	
	
	private Table createTable(Composite composite, String[] columnNames) {
        int style = SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | 
        SWT.FULL_SELECTION | SWT.HIDE_SELECTION;
        
        final Table table = new Table(composite,style);
        // Create each single columns
        List<TableColumn>columns = new ArrayList<TableColumn>();
        
        for (int i = 0 ; i < columnNames.length ; i++) {
           TableColumn column = new TableColumn(table,SWT.LEFT);
           column.setText(columnNames[i]);
           if (i < 2) {
        	   column.setWidth(100);
           }
           else {
        	   column.setWidth(500);
           }
           
          SelectionListener listener = new SelectionAdapter(){
               public void widgetSelected(SelectionEvent e) {
                   TableColumn tc = (TableColumn)e.getSource();
                   final int index = Arrays.asList(table.getColumns()).indexOf(tc);
                   tableViewer.setSorter(tableSorter.getSorter(index));
               }
           };
           
           column.addSelectionListener(listener);
           columns.add(column);
        }
        
        
        
        table.setHeaderVisible(true);
        table.setLinesVisible(true);

        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.grabExcessVerticalSpace = true;
        gridData.horizontalSpan = 3;
        table.setLayoutData(gridData);      
        return table;
    }
	
	@Override
    public void setActive(boolean active)
    {
        super.setActive(active);
        if(active)
        {
            if(!initDone)
            {
                initFormContent();                
            }
        }
    }
	
	/**
     * Initialise the form with the ttmresults link to the ttmdi used as input
     */
    private void initFormContent()
    {        
        IEditorInput input = getEditorInput();  
        if (input instanceof IFileEditorInput)
        {
            IFile inputFile = ((IFileEditorInput) input).getFile();
            IProject project = inputFile.getProject();            
            IPath ttmResultsFilePath = inputFile.getProjectRelativePath();
            ttmResultsFilePath = ttmResultsFilePath.removeFileExtension();
            ttmResultsFilePath = ttmResultsFilePath.addFileExtension("ttmresults");
            try {
				project.refreshLocal(IResource.DEPTH_INFINITE, new NullProgressMonitor());
			} catch (CoreException e) {
				TtmPlugin.log(e);
			}
            IResource ttmResFile = project.findMember(ttmResultsFilePath);            
            if(ttmResFile != null)
            {        
                currentFile = ttmResFile;
                updateBoard();
                initDone = true;
            }
            else
            {
                tableViewer.setInput(TraceabilityConstants.NOT_ANALYSED_PROJECT);
            }
        }
    }
	
}
