package org.topcased.traceability.hmi.ui.form.provider;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.topcased.traceability.hmi.util.MClass;
import org.topcased.traceability.hmi.util.ModelHelper;

import ttm.CoverPoint;
import ttm.Document;
import ttm.Requirement;
import ttm.Section;

public class CoverByViewLabelProvider extends TraceabilityViewLabelProvider implements ILabelProvider, ISelectionChangedListener {

	private Document selectedDocument;

	public String getText(Object element)
    {
        switch (MClass.convert(element))
        {
            case DOCUMENT: {
            	// COVERING ? 
            	Document covering = ((Document) element);
            	String res = covering.getIdent();
            	float ratio = 0;
            	if (selectedDocument !=null) {
            		ratio = ModelHelper.getCoverageResult(selectedDocument, covering);            		
            	}
            	return covering.getIdent() + " (" + Math.round(ratio) + "%)";
            }
            	// return covering.getIdent();
            case SECTION:
                return "Section " + ((Section) element).getIdent();
            case REQUIREMENT:
                return ((Requirement) element).getIdent();
            case INFO:
                return (String) element;
            case COVER_POINT: {
            	CoverPoint cp = (CoverPoint) element;
            	float ratio = ModelHelper.getCoverageResult(cp).floatValue();
            	int nbReqs = ModelHelper.getNbRequirements(cp.getCoveredDocument());
            	return  "Combined Coverage : " + String.valueOf(Math.round(ratio)) + "% of " + nbReqs + " requirements";
            }
        }
        return super.getText(element);

    }

	public void selectionChanged(SelectionChangedEvent event) {
		IStructuredSelection selection = (IStructuredSelection) event.getSelection();
		Object element = selection.getFirstElement();
		if (element != null) {
			switch (MClass.convert(element))
	        {
				case DOCUMENT:{
					selectedDocument = (Document) element;
					break;
		        }
				case SECTION: {
					selectedDocument = ((Section) element).getDocument();
					break;
				}
				case REQUIREMENT: {
					selectedDocument = ((Requirement) element).getDocument();
					break;
				}
	        }
		}
	}
	
	
}
