/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.ui.form.provider;

import java.util.List;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.topcased.traceability.hmi.util.MClass;
import org.topcased.traceability.hmi.util.TraceabilityHelper;

import ttm.Document;
import ttm.HierarchicalElement;
import ttm.Project;
import ttm.Requirement;
import ttm.Section;

/**
 * Created on 21 fvr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * Handle the content of the main treeviewer
 * 
 */
public class MainViewContentProvider implements IStructuredContentProvider, ITreeContentProvider
{

    /**
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement)
    {
        if (inputElement instanceof Project)
        {
            List<Document> documents = ((Project) inputElement).getDocuments();
            return documents.toArray(new Document[documents.size()]);
        }
        else
        {
            return new Object[] {inputElement};
        }
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
     */
    public Object[] getChildren(Object parentElement)
    {

        switch (MClass.convert(parentElement))
        {
            case DOCUMENT:
                return TraceabilityHelper.asArray(((Document) parentElement).getChildren());
            case SECTION:
                return TraceabilityHelper.asArray(((Section) parentElement).getChildren());
            case REQUIREMENT:
            	return TraceabilityHelper.asArray(((Requirement) parentElement).getChildren());
        }
        return new Object[0];
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
     */
    public Object getParent(Object element)
    {
        if (element instanceof HierarchicalElement)
        {
            return ((HierarchicalElement) element).getParent();
        }
        //Probably trying to select and object that not belongs to the tree
        return null;
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
     */
    public boolean hasChildren(Object element)
    {

        switch (MClass.convert(element))
        {
            case DOCUMENT:
                return true;
            case PROJECT:
                return true;
            case SECTION:
                return getChildren(element).length > 0;
            case REQUIREMENT:
                return getChildren(element).length > 0;
            case INFO:
                return false;
        }
        return false;
    }

    /**
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose()
    {
    }

    /**
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object,
     *      java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

}
