/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.ui.form.provider;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.topcased.traceability.hmi.util.CoveringDocument;
import org.topcased.traceability.hmi.util.MClass;
import org.topcased.traceability.hmi.util.Root;
import org.topcased.traceability.hmi.util.TraceabilityHelper;
import org.topcased.traceability.hmi.util.Tree;

import ttm.Document;
import ttm.Project;

/**
 * Created on 21 f�vr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * Handle the content of the CoverBy viewer
 * 
 */
public class MetricContentProvider implements IStructuredContentProvider, ITreeContentProvider
{

    private Tree tree;

    /**
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement)
    {
    	if (inputElement instanceof Root) {
    		return new Object[]{((Root)inputElement).getObject()};
    	}
        return new Object[] {inputElement};
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
     */
    public Object[] getChildren(Object parentElement)
    {
    	switch (MClass.convert(parentElement))
        {
            case PROJECT:
                return TraceabilityHelper.asArray(((Project)parentElement).getDocuments());
            case DOCUMENT: {
            	Document coveredDocument = (Document) parentElement;
            	
            	List<CoveringDocument> children = new ArrayList<CoveringDocument>();
            	for (Document coveringDoc : coveredDocument.getCoveringDocuments()) {
					children.add(new CoveringDocument(coveringDoc, coveredDocument));
				}
            	
            	return TraceabilityHelper.asArray(children);
            }	
        }
        return new Object[0];
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
     */
    public Object getParent(Object element)
    {
    	return null;
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
     */
    public boolean hasChildren(Object element)
    {
    	switch (MClass.convert(element))
        {
        	case COVERING_DOCUMENT:
        		return false;    
        	case DOCUMENT:
            	return true;
            case PROJECT:
                return true;
            
        }
        return false;
    }

    /**
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose()
    {
    }

    /**
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object,
     *      java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

}
