/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.ui.form.provider;

import org.eclipse.swt.graphics.Image;
import org.topcased.traceability.hmi.TtmImageRegistry;
import org.topcased.traceability.hmi.TtmPlugin;
import org.topcased.traceability.hmi.util.CoveringDocument;
import org.topcased.traceability.hmi.util.MClass;
import org.topcased.traceability.hmi.util.ModelHelper;
import org.topcased.traceability.hmi.util.TraceabilityConstants;

import ttm.Document;
import ttm.Project;

/**
 * Created on 21 f�vr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * Label provider for viewer about requirements
 * 
 */
public class MetricTraceabilityViewLabelProvider extends TraceabilityViewLabelProvider
{


    /**
     * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
     */
    public String getText(Object element)
    {
        switch (MClass.convert(element))
        {
        	case PROJECT: {
        		Project project = (Project)element;
        		return project.getIdent() + " (" + project.getDocuments().size() + " documents)";
        	}
            case DOCUMENT: {
            	Document document = (Document)element;
            	float ratio = document.getGlobalCoverageRatio();
            	int nbRequirements = ModelHelper.getNbRequirements(document);
            	if (nbRequirements == 0 ) {
            		return document.getIdent() + " - (No requirements defined)"; 
            	} else {
            		return document.getIdent() + " - " + nbRequirements + " requirements, " + Math.round(ratio) + "% covered";
            	}
            }
            case COVERING_DOCUMENT: {
            	CoveringDocument coveringDocument = (CoveringDocument) element;
            	Document parent = coveringDocument.getParent();
            	float ratio = ModelHelper.getCoverageResult(parent, coveringDocument);
            	return coveringDocument.getIdent() + " - covers " + Math.round(ratio) + "%";
            }
            	
        }
        return super.getText(element);

    }

    /**
     * @see org.eclipse.jface.viewers.LabelProvider#getImage(java.lang.Object)
     */
    public Image getImage(Object element)
    {

        switch (MClass.convert(element))
        {
        	case PROJECT:
        		return TtmImageRegistry.getImage(TraceabilityConstants.REQUIREMENT_IMAGE_KEY);
            case DOCUMENT:
                return getDocumentImage((Document) element);
            case COVERING_DOCUMENT:
            	return getDocumentImage(((CoveringDocument)element).getWrappedDocument());
        }

        return super.getImage(element);
    }

	private Image getDocumentImage(Document doc) {
		return TtmPlugin.getDefault().getWorkbench().getEditorRegistry().getImageDescriptor((doc.getFilename())).createImage();
	}

    
    
}
