/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.ui.form.provider;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.topcased.traceability.hmi.util.ModelHelper;
import org.topcased.traceability.hmi.util.TraceabilityHelper;

import ttm.Element;
import ttm.Project;
import ttm.Rule;
import ttm.RuleInstance;

/**
 * Created on 21 fvr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * Handle the content of the report treeviewer
 * 
 */
public class ReportViewContentProvider implements IStructuredContentProvider, ITreeContentProvider
{

    /**
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement)
    {
        Project project = (Project) inputElement;
        return TraceabilityHelper.asArray(project.getRules());

    }

    /**
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose()
    {
    }

    /**
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object,
     *      java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
     */
    public Object[] getChildren(Object parentElement)
    {
        if (parentElement instanceof Rule)
        {
        	List<RuleInstance> result = new ArrayList<RuleInstance>();
        	List<Element> processed = new ArrayList<Element>();
        	List<RuleInstance> instances = ((Rule) parentElement).getRuleInstances();
        	for (RuleInstance instance: instances) {
        		Element element = ModelHelper.getElementFromRule(instance);
        		if (!processed.contains(element)) {
        			processed.add(element);
        			result.add(instance);
        		}
        	}
            return TraceabilityHelper.asArray(result);
        }
        return new Object[0];
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
     */
    public Object getParent(Object element)
    {
        return null;
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
     */
    public boolean hasChildren(Object element)
    {
        if (element instanceof Rule)
        {
            return getChildren(element).length > 0;
        }
        return false;
    }

}
