/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.ui.form.provider;

import org.eclipse.jface.viewers.IFontProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.topcased.traceability.hmi.TtmImageRegistry;
import org.topcased.traceability.hmi.TtmPlugin;
import org.topcased.traceability.hmi.util.MClass;
import org.topcased.traceability.hmi.util.TraceabilityConstants;

import ttm.Document;
import ttm.Requirement;
import ttm.Section;

/**
 * Created on 21 f�vr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * Label provider for viewer about requirements
 * 
 */
public class TraceabilityViewLabelProvider extends LabelProvider implements IFontProvider
{

    private Font italicFont = null;

    private Font italicAndBoldFont = null;

    /**
     * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
     */
    public String getText(Object element)
    {

        switch (MClass.convert(element))
        {
            case DOCUMENT:
                return ((Document) element).getIdent();
            case SECTION:
                return "Section " + ((Section) element).getIdent();
            case REQUIREMENT:
                return ((Requirement) element).getIdent();
            case INFO:
                return (String) element;
        }
        return super.getText(element);

    }

    /**
     * @see org.eclipse.jface.viewers.LabelProvider#getImage(java.lang.Object)
     */
    public Image getImage(Object element)
    {

        switch (MClass.convert(element))
        {
            case DOCUMENT:
                return TtmPlugin.getDefault().getWorkbench().getEditorRegistry().getImageDescriptor(((Document) element).getFilename()).createImage();
            case SECTION:
                return TtmImageRegistry.getImage(TraceabilityConstants.SECTION_IMAGE_KEY);
            case REQUIREMENT:
                return TtmImageRegistry.getImage(TraceabilityConstants.REQUIREMENT_IMAGE_KEY);
            case COVER_POINT:
                return TtmImageRegistry.getImage(TraceabilityConstants.COVER_POINT_IMAGE_KEY);
        }

        return super.getImage(element);
    }

    /**
     * @see org.eclipse.jface.viewers.IFontProvider#getFont(java.lang.Object)
     */
    public Font getFont(Object element)
    {
        switch (MClass.convert(element))
        {
            case REQUIREMENT:
                if (!((Requirement) element).getActivatedRules().isEmpty())
                {
                    return getItalicAndBoldFont();
                }
                else
                {
                    return null;
                }
            case INFO:
                return getItalicFont();
            default:
                return null;
        }
    }

    private Font getItalicAndBoldFont()
    {
        if (italicAndBoldFont == null)
        {
            italicAndBoldFont = getStyledFont(SWT.BOLD | SWT.ITALIC);
        }
        return italicAndBoldFont;
    }

    private Font getItalicFont()
    {
        if (italicFont == null)
        {
            italicFont = getStyledFont(SWT.ITALIC);
        }
        return italicFont;
    }

    private Font getStyledFont(int style)
    {
        Font originalFont = Display.getDefault().getSystemFont();
        FontData[] fontData = originalFont.getFontData();

        for (int i = 0; i < fontData.length; i++)
        {
            fontData[i].setStyle(fontData[i].getStyle() | style);
        }
        return new Font(Display.getDefault(), fontData);
    }

    /**
     * @see org.eclipse.jface.viewers.LabelProvider#dispose()
     */
    public void dispose()
    {
        if (italicAndBoldFont != null)
        {
            italicAndBoldFont.dispose();
        }
    }

}
