/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.ui.form.provider.coverage;

import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.topcased.traceability.hmi.util.MClass;
import org.topcased.traceability.hmi.util.ModelHelper;
import org.topcased.traceability.hmi.util.Root;
import org.topcased.traceability.hmi.util.Tree;

import ttm.CoverPoint;
import ttm.Document;
import ttm.Requirement;
import ttm.Section;

/**
 * Created on 21 f�vr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * Handle the content of the CoverBy viewer
 * 
 */
public class CoverByViewContentProvider implements IStructuredContentProvider, ITreeContentProvider
{

    private Tree tree;

    /**
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement)
    {

        Object element = ((Root) inputElement).getObject();
        if (element != null)
        {
            tree = new Tree();

            switch (MClass.convert(element))
            {
                case DOCUMENT: {
                    Document document = (Document) element;
                    List coveredDocuments = document.getCoveringDocuments();
                    tree.addRoots(coveredDocuments);
                    rearangeRootOnDocument(document);
                 
                    break;
                }
                case SECTION: {
                    Section section = (Section) element;
                    for (Requirement requirement: ModelHelper.getRequirements(section)) {
                        ModelHelper.updateTreeWithIncomingCoverLinks(tree, requirement);
                    }
                    rearangeRootsOnSection(section);
                    
                    break;
                }
                case REQUIREMENT: {
                    ModelHelper.updateTreeWithIncomingCoverLinks(tree, (Requirement) element);
                    rearrangeRootsOnRequirement((Requirement) element);                    
                    break;
                }
                
            }
            return tree.getRoots();

        }
        return new Object[0];
    }


	private void rearrangeRootsOnRequirement(Requirement element) {
		Document selectedDoc = element.getDocument();
		rearangeRootOnDocument(selectedDoc);
	}


	private void rearangeRootOnDocument(Document selectedDoc) {
		EList coverPoints = selectedDoc.getCoverPoints();
		for (Object object : coverPoints) {
			CoverPoint cp = (CoverPoint) object;
			EList connectedDocuments = cp.getConnectedDocuments();
			tree.putRootAsParent(cp, connectedDocuments);			
		}
	}


	private void rearangeRootsOnSection(Section section) {
		Document selectedDoc = section.getDocument();
		rearangeRootOnDocument(selectedDoc);
	}
	

	/**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
     */
    public Object[] getChildren(Object parentElement)
    {

        switch (MClass.convert(parentElement))
        {
            case DOCUMENT: {
                return tree.getChildren(parentElement);
            }

            case SECTION: {
                return tree.getChildren(parentElement);
            }
            
            case COVER_POINT: {
            	return tree.getChildren(parentElement);
            }
            
           
            
        }
        return null;
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
     */
    public Object getParent(Object element)
    {
        return tree.getParent(element);
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
     */
    public boolean hasChildren(Object element)
    {

        switch (MClass.convert(element))
        {
            case DOCUMENT:
                return true;
            case SECTION:
                return true;
            case REQUIREMENT:
                return true;
            case COVER_POINT:
            	return true;
        }
        return false;
    }

    /**
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose()
    {
    }

    /**
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object,
     *      java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

}
