package org.topcased.traceability.hmi.ui.util;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;


/**
 * Created on 4 sept. 2006
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 *
 * A sorter for the selection view
 *
 */
public class TableViewerSorter {
	
	Map sortDataMap = new HashMap();
	private boolean preserveSortData = false;
	
	/**
	 * An internal class to store sort data 
	 *
	 */
	class SortData {
		Class type;
		boolean reversed = false;
		int index = 0;
		
		SortData(Class type) {
			this.type = type;
		}
	}
	
	
	
	/**
	 * The constructor for the SelectionViewerSorter class
	 */
	public TableViewerSorter() {
	}
	
	
	private SortData getCurrentSortDataForObject(Class clazz, int index) {
		SortData sd = (SortData)sortDataMap.get(clazz);
		if (sd == null) {
			sd = new SortData(clazz);
			sortDataMap.put(clazz, sd);
		}
		else {
			if (index == -1) { //Default
				return sd;
			}
			if (!preserveSortData && sd.index == index) { //Same column, Reverse sorting
				sd.reversed = !sd.reversed;
			}
			else {
				sd.reversed = false; //Other column
				sd.index = index;
			}
		}
		return sd;
	}
	
	/**
	 * Get a default sorter
	 * @return a ViewerSorter instance
	 */
	public ViewerSorter getSorter() {
		return getSorter(-1);
	}
	
	/**
	 * Get a sorter that sort the column matching the given index
	 * @param index
	 * @return a ViewerSorter instance
	 */
	public ViewerSorter getSorter(final int index) {
	
		ViewerSorter sorter = new ViewerSorter() {
		   
		   public int compare(Viewer viewer, Object e1, Object e2) {
			   if (e1 instanceof Integer) {
				  int i1 = ((Integer)e1).intValue();
				  int i2 = ((Integer)e2).intValue();
				  return i1 - i2;
			   }
			   else if (e1 instanceof Date) {
				   long l1 = e1 == null ? 0 : ((Date)e1).getTime();
				   long l2 = e2 == null ? 0 : ((Date)e2).getTime();
				   return (new Long(l1 - l2)).intValue();
			   }
			   else
				  return super.compare(viewer, e1, e2); 
		   }
		   
		   public void sort(final Viewer viewer, Object[] elements) {
			  final ViewerSorter sorter = this;
			  if (elements.length == 0) {
				  return;
			  }
			  
			  //Get sort data for column 'index'
			  final SortData sd = getCurrentSortDataForObject(elements[0].getClass(), index);
			  
			  Arrays.sort(elements, new Comparator() {
		            public int compare(Object a, Object b) {
                        IColumnObjectLabelProvider labelProvider = 
                        	(IColumnObjectLabelProvider) ((TableViewer)viewer).getLabelProvider();
		            	
		            	Object f1 = labelProvider.getColumnObject(a, sd.index);
		            	Object f2 = labelProvider.getColumnObject(b, sd.index);
		            	if (sd.reversed) {
		            		Object temp = f1;
		            		f1 = f2;
		            		f2 = temp;
		            	}
		                return sorter.compare(viewer, f1, f2);
		            }

					
		       });
		}
	   };
	   return sorter;
	}
	
	/**
	 * @param preserveSortData the boolean indicating 
     * whether we want to preserve sort data or not
	 */
	public void setPreserveSortData(boolean preserveSortData) {
		this.preserveSortData = preserveSortData;
	}
	
}
