package org.topcased.traceability.hmi.util;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath;
import org.eclipse.ui.part.FileEditorInput;
import org.topcased.traceability.hmi.TtmPlugin;

public class FileNameUtils {

	/**
	 * Does an inputFile points to a diagram (*.ttmdi) ? 
	 * @param inputFile
	 * @return
	 */
	public static boolean isDiagram(IFile inputFile) {
		return isDiagram(getFileExtension(inputFile)); // .endsWith(TtmPlugin.DIAGRAM_EXTENSION));
	}

	private static boolean isDiagram(String str) {
		return str.endsWith(TtmPlugin.DIAGRAM_EXTENSION);
	}
	
	private static boolean isResult(String str) {
		return str.endsWith(TtmPlugin.RESULT_EXTENSION);
	}
	
	/**
	 * Does an inputFile points to a result file (*.ttmresults) ? 
	 * @param inputFile
	 * @return
	 */
	public static boolean isResult(IFile inputFile) {
		return isResult(getFileExtension(inputFile));
	}

	/**
	 * Extension of an input file
	 * @param inputFile
	 * @return
	 */
	private static String getFileExtension(IFile inputFile) {
		return inputFile.getProjectRelativePath().getFileExtension();
	}

	/**
	 * Given an inputFile pointing to a result file, returns the path
	 * of the corresponding diagram file.
	 * @param inputFile
	 * @return
	 */
	public static IPath resultToDiagram(IFile inputFile) {
		return convertTo(inputFile, TtmPlugin.DIAGRAM_EXTENSION);
	}

	/**
	 * Given an inputFile pointing to a diagram file, returns the path
	 * of the corresponding result file.
	 * @param inputFile
	 * @return
	 */
	public static IPath diagramToResult(IFile inputFile) {
		return convertTo(inputFile, TtmPlugin.RESULT_EXTENSION);
	}
	
	
	/**
	 * Path of an inputFile, with the extension changed
	 * @param inputFile
	 * @param extension the new extension
	 * @return
	 */
	private static IPath convertTo(IFile inputFile, String extension) {
		IPath res = inputFile.getProjectRelativePath();
		return convertPath(res, extension);		
	}

	private static IPath convertPath(IPath path, String extension) {
        IPath res = path.removeFileExtension();
        res = res.addFileExtension(extension);
		return res;
	}

	public static IPath getResultFilePath(FileEditorInput fileInput) {
		IFile file = fileInput.getFile();
		if (isResult(file)) {
			return file.getProjectRelativePath();
		} else if (isDiagram(file)) {
			return diagramToResult(file);
		}
		return null;
	}

	public static IPath getResultFileFullPath(IPath path) {
		if (isResult(path)) {
			return path;
		} else if (isDiagram(path)) {
			return diagramToResult(path);
		}
		return null;
		
	}

	private static IPath diagramToResult(IPath path) {
		return convertPath(path,TtmPlugin.RESULT_EXTENSION );
	}

	private static boolean isDiagram(IPath path) {
		return isDiagram(path.toString());
	}

	private static boolean isResult(IPath path) {
		return isResult(path.toString());
	}

	
	
	
}
