/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.util;

import ttm.CoverPoint;
import ttm.Document;
import ttm.Project;
import ttm.Requirement;
import ttm.Section;

/**
 * Created on 21 f�vr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * Enumeration on model elements
 * 
 */
public enum MClass {

    /**
     * PROJECT OBJECT
     */
    PROJECT(Project.class),
    /**
     * COVERING DOCUMENT OBJECT
     */    
    COVERING_DOCUMENT(CoveringDocument.class),
    /**
     * DOCUMENT OBJECT
     */
    DOCUMENT(Document.class),
    /**
     * SECTION OBJECT
     */
    SECTION(Section.class),
    /**
     * REQUIREMENT OBJECT
     */
    REQUIREMENT(Requirement.class),
    /**
     * COVER POINT OBJECT
     */
    COVER_POINT(CoverPoint.class),    
    /**
     * INFO OBJECT (not model object)
     */
    INFO(String.class);

    
    private Class clazz;

    /**
     * Contructor for enum
     * 
     * @param clazz
     */
    MClass(Class clazz)
    {
        this.clazz = clazz;
    }

    /**
     * This method converts a class instance in the matching enum value
     * 
     * @param object
     * @return the matching enum value
     */
    public static MClass convert(Object object)
    {
    	if (object instanceof CoveringDocument) {
    		return COVERING_DOCUMENT;
    	}
    	
        for (MClass cu : values())
        {
            if (cu.clazz.isInstance(object))
            {
                return cu;
            }
        }
        return null;
    }

}
