/*******************************************************************************
 * No CopyrightText Defined in the configurator file.
 ******************************************************************************/
package org.topcased.traceability.hmi.wizards;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourceAttributes;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.ide.IDE;
import org.topcased.modeler.extensions.Template;
import org.topcased.modeler.extensions.TemplatesManager;
import org.topcased.modeler.tools.DiagramFileInitializer;
import org.topcased.traceability.hmi.TtmImageRegistry;
import org.topcased.traceability.hmi.TtmPlugin;


/**
 * Generated wizard that offers the model creation facilities.
 *
 * @generated NOT
 */
public class NewTtmDiagrams extends Wizard implements INewWizard {
	/**
	 * @generated
	 */
	private IStructuredSelection selection;

	/**
	 * 
	 */
	private TtmDiagramsPage diagPage;

	/**
	 * @generated
	 */
	private IFile createdFile;
	
	private IProject newProject;

	/**
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 * @generated
	 */
	public void init(IWorkbench workbench, IStructuredSelection sel) {
		createdFile = null;
		selection = sel;

		// TODO put the Wizard image
		setDefaultPageImageDescriptor(TtmImageRegistry
				.getImageDescriptor("NEW_PAGE_WZD"));
		setDialogSettings(TtmPlugin.getDefault().getDialogSettings());
		setWindowTitle("Create new traceability project");
	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 * @generated
	 */
	public boolean performFinish() {
		boolean result = true;
		if (diagPage.isPageComplete()) {
			if(diagPage.isNewProject())
			{
				result &= createNewProject(); 
			}
			if (diagPage.isNewModel()) {
				result = result & createModelFile();
				result = result & createDiagramFile();
				if (createdFile != null && result) {
					// Open the newly created model
					try {
						IDE.openEditor(TtmPlugin.getDefault().getWorkbench()
								.getActiveWorkbenchWindow().getActivePage(),
								createdFile, true);
					} catch (PartInitException pie) {
						TtmPlugin.log(pie);
					}
				}
			} else {
				createDiagramFromExistingModel();
			}
		}
		return result;
	}

	/**
	 * @generated
	 */
	private boolean createDiagramFromExistingModel() {
		WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
			/**
			 * @see org.eclipse.ui.actions.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
			 */
			protected void execute(IProgressMonitor monitor)
					throws CoreException, InvocationTargetException,
					InterruptedException {
				DiagramFileInitializer initializer = new DiagramFileInitializer();
				try {
					initializer.createDiagram(diagPage.getRootEObject(),
							diagPage.getDiagramId(), diagPage.isInitialized(),
							monitor);
				} catch (IOException ioe) {
					throw new InvocationTargetException(ioe);
				}
			}
		};

		try {
			getContainer().run(false, true, op);
			return true;
		} catch (InvocationTargetException ite) {
			TtmPlugin.log(ite);
		} catch (InterruptedException ie) {
			// Wizard stopped
		}
		return false;
	}
	
    
    /**
     * Create a new project in or outside the workspace
     * @return true is creation succedded
     */
	private boolean createNewProject()
	{
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		   IWorkspaceRoot root = workspace.getRoot();
		   newProject  = root.getProject(diagPage.getNewProjectName());
           IProjectDescription desc = newProject.getWorkspace().newProjectDescription(diagPage.getNewProjectName());
           if(!diagPage.isDefaultLocation())
           {
               desc.setLocation(new Path(diagPage.getNewProjectLocation()));
           }
		   try
		   {
			   if (!newProject.exists()) 
			   {
				   newProject.create(desc,new NullProgressMonitor());				   
			   }
			   if (!newProject.isOpen())
			   {
				   newProject.open(new NullProgressMonitor());	
				   ResourceAttributes att = new ResourceAttributes();
				   att.setReadOnly(false);	
				   // Attempt to make the file executable under Linux ... 
				   att.setExecutable(true);
				   newProject.setResourceAttributes(att);
			   }
			   return true;
		   }
		   catch (CoreException e)
		   {
			   TtmPlugin.log(e);
			   TtmPlugin.displayDialog(null,
						"An error occured during new directory creation.",
						IStatus.ERROR);
			   newProject = null;
		   }		  
		return false;
	}
	
	/**
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 * @generated
	 */
	public void addPages() {
		diagPage = new TtmDiagramsPage("New traceability project", selection);
		diagPage.setTitle("Create a traceability project");
		diagPage.setDescription("Define the traceability project informations.");
		diagPage.setMessage("Create a new traceability project into an existing eclipse project or a new one");
		addPage(diagPage);
	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#canFinish()
	 * @generated
	 */
	public boolean canFinish() {
		return diagPage.isPageComplete();
	}

	/**
	 * @return true if the model file was successfully created
	 * @generated
	 */
	private boolean createModelFile() {
		try {
			Template template = TemplatesManager.getInstance().find(
					diagPage.getTemplateId()).getTemplateModel();
			if(newProject != null)
			{
				template.setDestination(newProject);
			}
			else
			{
				template.setDestination(diagPage.getSelectedIContainer());
			}
			template.addVariable("name", diagPage.getModelName());

			template.generate(new NullProgressMonitor());
		} catch (CoreException ce) {
			TtmPlugin.log(ce);
			TtmPlugin.displayDialog(null,
					"An error occured during the template generation.",
					IStatus.ERROR);
			return false;
		}
		return true;
	}

	/**
	 * @return true if the diagram file was successfully created
	 * @generated NOT
	 */
	private boolean createDiagramFile() {
		try {
			Template template = TemplatesManager.getInstance().find(
					diagPage.getTemplateId()).getTemplateDI();
			if(newProject != null)
			{
				template.setDestination(newProject);
			}
			else
			{
				template.setDestination(diagPage.getSelectedIContainer());
			}
			template.addVariable("name", diagPage.getModelName());
            // Bug #1395 : Add an additional variable used to encode the model file name
            template.addVariable("escapedName", URI.encodeFragment(diagPage.getModelName(), false));

			createdFile = (IFile) template.generate(new NullProgressMonitor());
		} catch (CoreException ce) {
			TtmPlugin.log(ce);
			TtmPlugin.displayDialog(null,
					"An error occured during the template generation.",
					IStatus.ERROR);
			return false;
		}
		return true;
	}
		
}
