/*******************************************************************************
 * No CopyrightText Defined in the configurator file.
 ******************************************************************************/
package org.topcased.traceability.hmi.wizards;

import java.io.File;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;
import org.topcased.modeler.preferences.ModelerPreferenceConstants;
import org.topcased.traceability.hmi.TtmPlugin;

/**
 * 
 * In order to customise the Page properly we have 
 * deleted the previons 
 * @generated NOT
 */
public class TtmDiagramsPage extends WizardPage {
	
	private static final String BROWSE_TEXT = "...";

    /**
     * Contains the resource that is initially selected when the wizard is
     * opened. In the case of a file with an invalid extension, we store the
     * container of the file. This variable is then used to load the
     * corresponding data in the wizard.
     */
    private IResource selectedResource;

    private boolean allowInit;

    // SWT objects
    private Button existingModelBt;

    private Button newModelBt;

    private Group newModelGrp;

    private Text directoryFd;

    private Button directoryBt;

    private Text nameFd;
    
    private Text modelFd;

    private Button modelBt;    
    
    private EObject existingModelObject;
    
    private Button newProjectBt;
    
    private Button existingProjectBt;
           
    private Group newDirectoryGrp,newProjectGrp;
    
    private Text newProjectNameFd;
    
    private Text newProjectPathFd;
    
    private Button newProjectDefaultLocationBt,newProjectLocationBrowseBut;
    
    private String newProjectDefaultLocation;

    private Button initializeBt;

    private ModifyListener validationListener = new ModifyListener()
    {
        public void modifyText(ModifyEvent e)
        {
            setPageComplete(validatePage());
        }
    };

    /**
     * Constructor
     * 
     * @param pageName
     * @param selection
     */
    public TtmDiagramsPage(String pageName, IStructuredSelection selection)
    {
        this(pageName, selection, false);
    }

    /**
     * Constructor
     * 
     * @param pageName the page title
     * @param selection the selected object
     * @param allowDiagramInitialization
     */
    public TtmDiagramsPage(String pageName, IStructuredSelection selection, boolean allowDiagramInitialization)
    {
        super(pageName);
        setPageComplete(false);

        Object sel = selection.getFirstElement();
        if (sel != null && sel instanceof IResource)
        {
            if (sel instanceof IFile)
            {
                // check if the selected file as the expected extension
                if (((IFile) sel).getFileExtension().equals(getFileExtension()))
                {
                    selectedResource = (IFile) sel;
                }
                else
                {
                    selectedResource = ((IResource) sel).getParent();
                }
            }
            else
            {
                selectedResource = (IResource) sel;
            }
        }
        else
        {
            selectedResource = ResourcesPlugin.getWorkspace().getRoot();
        }

        this.allowInit = allowDiagramInitialization;
    }

    /**
     * Creation of the wizard page
     * 
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent)
    {
        Composite mainComp = new Composite(parent, SWT.NONE);
        mainComp.setLayout(new GridLayout());
        mainComp.setLayoutData(new GridData(GridData.FILL_BOTH));
        mainComp.setFont(parent.getFont());
        
        createModelGroup(mainComp);
        
        createDirectoryGroup(mainComp);

        // Add listeners
        hookListeners();

        // Initialize widgets value
        loadData();

        // Show description on opening
        setErrorMessage(null);
        setMessage(null);
        setControl(mainComp);

        setPageComplete(validatePage());
    }
    
    
    /**
     * Create the Model group allowding to enter the new model name or 
     * to select an existing model file from the repository
     * @param parent
     */
    protected void createModelGroup(Composite parent)
    {
    	newModelGrp = new Group(parent, SWT.NONE);
        newModelGrp.setLayout(new GridLayout(3, false));
        newModelGrp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        newModelGrp.setText("Model section");
        
        
        // Group activated when the user choose to create a new Model file
        newModelBt = new Button(newModelGrp, SWT.RADIO);
        newModelBt.setText("Create a new Model");
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 3;
        newModelBt.setLayoutData(gd);

        // The new name of the model file to create
        Label nameLbl = new Label(newModelGrp, SWT.NONE);
        nameLbl.setText("Model name : ");
        
        nameFd = new Text(newModelGrp, SWT.BORDER);
        GridData layoutNameFd = new GridData(GridData.FILL_HORIZONTAL);
        layoutNameFd.horizontalSpan = 2;
        nameFd.setLayoutData(layoutNameFd);       
    	    	        
        //Group activated when the user choose to create from an existing Model
        existingModelBt = new Button(newModelGrp, SWT.RADIO);
        existingModelBt.setText("Create from an existing Model");
        gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 3;
        existingModelBt.setLayoutData(gd);
        
        // Selection of the existing model file
        Label modelLbl = new Label(newModelGrp, SWT.NONE);
        modelLbl.setText("Model : ");

        modelFd = new Text(newModelGrp, SWT.BORDER);
        modelFd.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        modelFd.setEnabled(false);

        modelBt = new Button(newModelGrp, SWT.PUSH);
        modelBt.setText(BROWSE_TEXT);
        modelBt.setEnabled(false);

        if (allowInit)
        {
            initializeBt = new Button(newModelGrp, SWT.CHECK);
            initializeBt.setText("Initialize the diagram with existing model objects");
            GridData gdInit = new GridData(GridData.FILL_BOTH);
            gdInit.horizontalSpan = 3;
            initializeBt.setLayoutData(gdInit);
        }
    }
    
    /**
     * Create the group defining the directory location
     *  - New one
     *  - Or existing one
     * @param parent
     */
    protected void createDirectoryGroup(Composite parent)
    {
    	newDirectoryGrp = new Group(parent,SWT.NONE);
    	newDirectoryGrp.setLayout(new GridLayout(3, false));
    	GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    	gd.verticalIndent = 10;
    	newDirectoryGrp.setLayoutData(gd);
    	newDirectoryGrp.setText("Directory section");
    	
    	//Group activated when the user choose to create a new repository
    	newProjectBt = new Button(newDirectoryGrp,SWT.RADIO);
    	newProjectBt.setText("Create in a new directory");
    	gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 3;  
        newProjectBt.setLayoutData(gd);
    	    	    	
        createNewProjectGroup(newDirectoryGrp);
    	
    	//Group activated when the user choose to create a new repository
    	existingProjectBt = new Button(newDirectoryGrp,SWT.RADIO);
    	existingProjectBt.setText("Add to an existing directory");
    	 gd = new GridData(GridData.FILL_HORIZONTAL);
         gd.horizontalSpan = 3;           
         gd.verticalIndent = 20;
    	existingProjectBt.setLayoutData(gd);
    	
    	 // The destination directory
        Label directoryLbl = new Label(newDirectoryGrp, SWT.NONE);               
        directoryLbl.setText("Directory : ");

        directoryFd = new Text(newDirectoryGrp, SWT.BORDER);               
        directoryFd.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        directoryFd.setEnabled(false);

        directoryBt = new Button(newDirectoryGrp, SWT.PUSH);
        directoryBt.setText(BROWSE_TEXT);
                
    	newProjectPathFd.setEnabled(false);    	    	    	
    }
   

      
    /**
     * Creates the subgroup containing newProject information :
     * name and directory path
     * @param parent
     */
    private void createNewProjectGroup(Composite parent)
    {
    	newProjectGrp = new Group(parent,SWT.NONE);
    	newProjectGrp.setLayout(new GridLayout(3,false));
    	GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    	gd.horizontalSpan = 3;
    	newProjectGrp.setLayoutData(gd);
    	
    	Label projectLbl = new Label(newProjectGrp,SWT.NONE);    	
    	projectLbl.setText("Directory name :");
    	
    	newProjectNameFd = new Text(newProjectGrp,SWT.BORDER);
    	gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 2;        
    	newProjectNameFd.setLayoutData(gd);
    	
    	newProjectDefaultLocationBt = new Button(newProjectGrp,SWT.CHECK);    	    
    	newProjectDefaultLocationBt.setText("Use default location ");
        newProjectDefaultLocationBt.setSelection(true);
        gd = new GridData(GridData.FILL_HORIZONTAL);
    	gd.horizontalSpan = 3;
    	newProjectDefaultLocationBt.setLayoutData(gd);
    	
    	Label defaultLocationLbl = new Label(newProjectGrp,SWT.NONE);
    	defaultLocationLbl.setText("Location : ");
    	    	    	
    	newProjectPathFd = new Text(newProjectGrp,SWT.BORDER);
    	newProjectPathFd.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	
    	newProjectLocationBrowseBut = new Button(newProjectGrp,SWT.PUSH);
    	newProjectLocationBrowseBut.setText("Browse "+BROWSE_TEXT);
    	    	
    	newProjectLocationBrowseBut.setEnabled(false);
    	computeDefaultProjectLocation();
    	newProjectPathFd.setText(newProjectDefaultLocation);
    }

    /**
     * Initialize widgets value
     */
    private void loadData()
    {
    	if(selectedResource == null)
    	{
    		newProjectBt.setSelection(true);
    	}
              
        if (selectedResource instanceof IFile)
        {
            existingModelBt.setSelection(true); 
            modelBt.setEnabled(true);
            existingProjectBt.setSelection(true);
            modelFd.setText(selectedResource.getFullPath().toString());
            directoryFd.setText("/"+selectedResource.getProject().getName());

            if (loadModelFile())
            {
                existingModelBt.setFocus();
                //disable newProject if a selection is made on an existing model
                newProjectBt.setEnabled(false);  
            }
        }
        else
        {
        	if(selectedResource instanceof IProject)
        	{
        		existingProjectBt.setSelection(true);     
        		handleNewProjectSelection(false);
        		directoryFd.setText("/"+selectedResource.getName());
        	}
        	else
        	{
        		//selectedResource instance of WorkspaceRoot
        		existingProjectBt.setSelection(false); 
        		newProjectBt.setSelection(true);
        		handleNewProjectSelection(true);
        	}
        	
        	directoryFd.setText(selectedResource.getFullPath().toString());
            newModelBt.setSelection(true);                          
            nameFd.setText("DefaultName");
            nameFd.setFocus();
        }

        if (allowInit)
        {
            initializeBt.setSelection(TtmPlugin.getDefault().getPreferenceStore().getBoolean(
                    ModelerPreferenceConstants.P_INITIALIZE_DIAGRAM));
        }
    }

    /**
     * Add the listeners to the Buttons and the Text widgets
     */
    private void hookListeners()
    {
        directoryFd.addModifyListener(validationListener);
        nameFd.addModifyListener(validationListener);
       
        modelFd.addModifyListener(validationListener);
        newProjectNameFd.addModifyListener(validationListener);
        
        
        //Update default location when typing project name
        newProjectNameFd.addModifyListener(new ModifyListener()
        {
        	public void modifyText(ModifyEvent e) 
        	{        
        		if(newProjectDefaultLocationBt.getSelection())
        		{       
        			computeDefaultProjectLocation(); 
        			newProjectPathFd.setText(newProjectDefaultLocation);
        		}        		
        	}
        });
      
        // Button listeners
        directoryBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                handleDirectoryChoose();
            }
        });
        
     // Button listeners
        newProjectLocationBrowseBut.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                handleProjectLocationChoose();
            }
        });

        modelBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                handleModelChoose();
            }
        });

        existingModelBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (existingModelBt.getSelection())
                {
                    nameFd.setEnabled(false);
                    modelBt.setEnabled(true);
                    newProjectBt.setEnabled(false);                    
                }
                setPageComplete(validatePage());
            }
        });

        newModelBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (newModelBt.getSelection())
                {
                    nameFd.setEnabled(true); 
                    modelBt.setEnabled(false);
                    newProjectBt.setEnabled(true);  
                }
                setPageComplete(validatePage());
            }
        });
        
        existingProjectBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
            	handleNewProjectSelection(!existingProjectBt.getSelection());
                setPageComplete(validatePage());
            }
        });

        newProjectBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
            	handleNewProjectSelection(newProjectBt.getSelection());                
                setPageComplete(validatePage());
            }
        });
              
        
        //Listener on newProjectDefaultLocationBt 
        //Enable browze button or set default location to workspace
        newProjectDefaultLocationBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                boolean selection = newProjectDefaultLocationBt.getSelection();                                                   
                newProjectPathFd.setEnabled(!selection);
                if(!selection)
                {
                	newProjectPathFd.setText("");
                }
                else
                {          
                	computeDefaultProjectLocation(); 
                	newProjectPathFd.setText(newProjectDefaultLocation);                
                }
                	
                newProjectLocationBrowseBut.setEnabled(!selection);
                setPageComplete(validatePage());                
            }
        });
       
        if (initializeBt != null)
        {
            initializeBt.addSelectionListener(new SelectionAdapter()
            {
                /**
                 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
                 */
                public void widgetSelected(SelectionEvent e)
                {
                    TtmPlugin.getDefault().getPreferenceStore().setValue(
                            ModelerPreferenceConstants.P_INITIALIZE_DIAGRAM, initializeBt.getSelection());
                }
            });
        }
    }

    /**
     * Handle the directory choice button action
     */
    protected void handleDirectoryChoose()
    {
        ContainerSelectionDialog dialog = new ContainerSelectionDialog(getShell(),
                (IContainer) ResourcesPlugin.getWorkspace().getRoot().findMember(directoryFd.getText()), false,
                "Choose the destination directory :");
        if (dialog.open() == Window.OK)
        {
            Object[] results = dialog.getResult();
            if (results.length == 1 && results[0] instanceof IPath)
            {
                IPath newPath = (IPath) results[0];
                directoryFd.setText(newPath.toString());
            }
        }
    }

    /**
     * Handle the model choice button action
     */
    protected void handleModelChoose()
    {
        // The initial directory is always the Workspace
        String initialDirectory = ResourcesPlugin.getWorkspace().getRoot().getFullPath().toString();

        ResourceSelectionDialog dialog = new ResourceSelectionDialog(getShell(),
                ResourcesPlugin.getWorkspace().getRoot().findMember(initialDirectory), "Choose an existing model file :");
        if (dialog.open() == Window.OK)
        {
            Object[] results = dialog.getResult();
            if (results.length == 1 && results[0] instanceof IFile)
            {
                modelFd.setText(((IFile) results[0]).getFullPath().toString());
                directoryFd.setText("/"+((IFile) results[0]).getProject().getName());
                // update the other widgets
                loadModelFile();
            }
        }
    }
    
    /**
     * Handle the project location choice button action
     */
    protected void handleProjectLocationChoose()
    {
    	 
    	DirectoryDialog dialog = new DirectoryDialog(getShell(),SWT.OPEN);
        String path = dialog.open();
        
    	if (path != null)
        {                        
           newProjectPathFd.setText(path);
        }
         setPageComplete(validatePage());
    }
    
    
    /**
     * Handler called when the selection on newProjectBt changed
     * @param selected
     */
    private void handleNewProjectSelection(boolean selected)
    {
    	newProjectGrp.setEnabled(selected);
    	newProjectNameFd.setEnabled(selected);
    	newProjectDefaultLocationBt.setEnabled(selected);    	
        directoryBt.setEnabled(!selected);
    }
    
    
    /**
     * Initialize the newProjectDefaultLocation to the defaut path :
     * WorkspacePath + / + ProjectName
     */
    private void computeDefaultProjectLocation()
    {
    	
    	newProjectDefaultLocation = ResourcesPlugin.getWorkspace().getRoot().getLocation().toString();
    	if(newProjectNameFd.getText() != null && !"".equals(newProjectNameFd.getText().length()))
    	{
    		newProjectDefaultLocation += "/"+newProjectNameFd.getText();    	
    	}
    }

    /**
     * Load the modelFile selected and update the associated widgets
     * 
     * @return true if the file was loaded successfully, return false otherwise.
     */
    protected boolean loadModelFile()
    {
        URI uri = URI.createPlatformResourceURI(modelFd.getText(),true);

        Resource resource = getResourceSet().getResource(uri, true);
        if(resource != null)
        {
        	existingModelObject = (EObject)resource.getContents().get(0);
        	return true;
        }
        return false;        
    }
        

    /**
     * Return the ResourceSet used to store the model and diagram file. By
     * default create a new ResourceSet.
     * 
     * @return ResourceSet
     */
    protected ResourceSet getResourceSet()
    {
        return new ResourceSetImpl();
    }

    /**
     * Check if the informations contained in the page are OK
     * 
     * @return true is OK
     */
    protected boolean validatePage()
    {
        // erase all previous messages
        setMessage(null);
        setErrorMessage(null);
        boolean validationOk = true;
        
        if(isNewProject())
        {
        	 validationOk = validateNewProject();
        }
        
        //Validate directory selection
        validationOk &= validateDirectory();

        if(!validationOk)
        {
        	//skip model validation on new project error
        	return false;
        }
        if (isNewModel())
        {
            return validateNewModelGroup();
        }
        else
        {
            return validateExistingModel();
        }
    }

    /**
     * Validate the group if the existing model option is chosen
     * 
     * @return <code>true</code> if data is valid
     */
    private boolean validateExistingModel()
    {
        // the user has choosen an existing model
        if (modelFd.getText() == null || "".equals(modelFd.getText()))
        {
            setErrorMessage("You have to choose an existing model file.");
            return false;
        }

        if (!((IFile) ResourcesPlugin.getWorkspace().getRoot().findMember(modelFd.getText())).getFileExtension().equals(
                getFileExtension()))
        {
            setErrorMessage("Invalid model file. The model must be a " + getFileExtension() + " model.");
            return false;
        }

        if (diagramFileExist())
        {
            setMessage("The diagrams file already exists. It will be overwritten !", IMessageProvider.WARNING);
        }      

        return true;
    }

    /**
     * Validate the group if the new model option is chosen
     * 
     * @return <code>true</code> if data is valid
     */
    private boolean validateNewModelGroup()
    {        
        if (getModelName() == null || "".equals(getModelName()))
        {
            setErrorMessage("Model name cannot be empty.");
            return false;
        }
        else
        {
            validateExistingFiles();
        }
        
        return true;
    }
    
    /**
     * Validate the directory selected by the user.
     * - if new Porject -> ok
     * - else test non nullity of container
     * @return
     */
    private boolean validateDirectory()
    {
    	if(existingProjectBt.getSelection())
    	{
	    	IContainer container = getSelectedIContainer();
	        if (container == null || container instanceof IWorkspaceRoot)
	        {
	            setErrorMessage("The destination is not a valid container.");
	            return false;
	        }
    	}
        
        return true;
    }
    
    /**
     * Validate the new Projet data :
     * - name compulsory
     * - path not empty
     * @return
     */
    private boolean validateNewProject()
    {
    	if(getNewProjectName() == null || "".equals(getNewProjectName()))
    	{
    		setErrorMessage("New directory name cannot be empty");
    		return false;
    	}
    	
    	if(newProjectPathFd.getText() == null || "".equals(newProjectPathFd.getText()))
    	{
    		setErrorMessage("New directory path cannot be empty");
    		return false;
    	}
    	
    	//check if the project doesnt exist    	
	    IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		IProject newProject = root.getProject(getNewProjectName());
		boolean projectExists = false;
		if(!newProjectDefaultLocationBt.getSelection())
		{
			IPath path = new Path(getNewProjectLocation());
			projectExists = newProject.exists(path);
		}
		else
		{
			projectExists = newProject.exists();
		}
		if (projectExists) 
		{
			setErrorMessage("A directory with the same name already exists.");
			return false;
		}    	   
    			    	
    	return true;
    }

    /**
     * Check if the created files does not already exist
     */
    private void validateExistingFiles()
    {
        if (modelFileExist())
        {
            if (diagramFileExist())
            {
                setMessage("The model file and the diagram file already exist. They will be overwritten !",
                        IMessageProvider.WARNING);
            }
            else
            {
                setMessage("The model file already exists. It will be overwritten !", IMessageProvider.WARNING);
            }
        }
        else if (diagramFileExist())
        {
            setMessage("The diagram file already exists. It will be overwritten !", IMessageProvider.WARNING);
        }
    }

    private boolean modelFileExist()
    {
        if (isNewModel())
        {
            String path = directoryFd.getText() + File.separator + getModelNameWithoutExtension(nameFd.getText()) + "."
                    + getFileExtension();
            IResource resource = ResourcesPlugin.getWorkspace().getRoot().findMember(path);
            return resource != null;
        }
        else
        {
            IResource resource = ResourcesPlugin.getWorkspace().getRoot().findMember(modelFd.getText());
            return resource != null;
        }
    }

    private boolean diagramFileExist()
    {
        if (isNewModel())
        {
            String path = directoryFd.getText() + File.separator + getModelNameWithoutExtension(nameFd.getText()) + "."
                    + getFileExtension() + "di";
            IResource resource = ResourcesPlugin.getWorkspace().getRoot().findMember(path);
            return resource != null;
        }
        else
        {
            IResource resource = ResourcesPlugin.getWorkspace().getRoot().findMember(modelFd.getText() + "di");
            return resource != null;
        }
    }

    /**
     * Returns if the diagram must be initialized with model objects
     * 
     * @return <code>true</code> if diagram must be initialized
     */
    public boolean isInitialized()
    {
        if (allowInit)
        {
            return TtmPlugin.getDefault().getPreferenceStore().getBoolean(
                    ModelerPreferenceConstants.P_INITIALIZE_DIAGRAM);
        }

        return false;
    }

    /**
     * Returns the selected destination container
     * 
     * @return the selected container
     */
    public IContainer getSelectedIContainer()
    {
        /**if (!isNewModel())
        {
        	return ((IFile) ResourcesPlugin.getWorkspace().getRoot().findMember(modelFd.getText())).getParent();
        }
        else*/
        {            
            return (IContainer) ResourcesPlugin.getWorkspace().getRoot().findMember(directoryFd.getText());
        }
    }

    /**
     * Returns the selected model name
     * 
     * @return the model name. Returns null if no model file name is set.
     */
    public String getModelName()
    {
        if (isNewModel())
        {
            if (nameFd != null && !nameFd.isDisposed())
            {
                return getModelNameWithoutExtension(nameFd.getText());
            }
        }
        else
        {
            if (modelFd != null && !modelFd.isDisposed())
            {
                IResource resource = ResourcesPlugin.getWorkspace().getRoot().findMember(modelFd.getText());
                return getModelNameWithoutExtension(resource.getName());
            }
        }
        return null;
    }
    
    /**
     * Returns the new eclipse project name in which the model will be added  
     * @return the ProjectName Returns null if the project name is not set
     */
    public String getNewProjectName()
    {
    	return newProjectNameFd.getText();
    }

    /**
     * Returns the root EObject of the model file
     * 
     * @return the EObject
     */
    public EObject getRootEObject()
    {
        return existingModelObject;
    }
   

    /**
     * Returns the name of the modelFile without extension
     * 
     * @param fullName the name with (or without) extension
     * @return the name of the modelFile without extension
     */
    protected String getModelNameWithoutExtension(String fullName)
    {
        String[] nameSplitted = fullName.split("." + getFileExtension());
        return nameSplitted[0];
    }

    /**
     * Returns the selected template id
     * 
     * @return the selected template id. Returns null if none found
     */
    public String getTemplateId()
    {
    	return "org.topcased.traceability.hmi.modeler.templates.org.topcased.traceability.hmi.modeler";
    }

    /**
     * Returns the selected diagram id
     * 
     * @return the selected diagram id. Returns null if none found
     */
    public String getDiagramId()
    {      
    	return "org.topcased.traceability.hmi.modeler";
    }

    /**
     * Return the choice of creation made by the user
     * 
     * @return true if the model file is not created
     */
    public boolean isNewModel()
    {
        return newModelBt.getSelection();
    }
    
    /**
     * Return the choice of creation made by the user
     * 
     * @return true if the model file is not created
     */
    public boolean isNewProject()
    {
        return newProjectBt.getSelection();
    }


	/**
	 * @see org.topcased.modeler.wizards.TtmDiagramsPage#getEditorID()
	 * @generated
	 */
	public String getEditorID() {
		return "org.topcased.traceability.hmi.editor.TtmMultipleEditor";
	}

	/**
	 * @see org.topcased.modeler.wizards.TtmDiagramsPage#getFileExtension()
	 * @generated
	 */
	public String getFileExtension() {
		return "ttm";
	}
	
	/**
	 * Return the NewProject Location if the 
	 * @return
	 */
	public String getNewProjectLocation()
	{		
		return newProjectPathFd.getText();
	}
	
	/**
	 * Get the URI for the location field if possible.
	 * 
	 * @return URI or <code>null</code> if it is not valid.
	 */
	public URI getNewProjectLocationURI() 
	{			
		return URI.createFileURI(newProjectPathFd.getText());
	}	
	
	public boolean isDefaultLocation()
	{
		return newProjectDefaultLocationBt.getSelection();
	}
}
