/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies)
 *    - initial API and implementation
 *******************************************************************************/
package org.topcased.validation.ui.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.Action;
import org.topcased.validation.ui.internal.ValidationUIPlugin;
import org.topcased.validation.ui.internal.actions.ValidateJob;

/**
 * Validate a model with the extensible validation process. <br>
 * creation : 11 janv. 2006
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class ValidateAction extends Action
{
    /**
     * Action ID
     */
    public static final String ID = "org.topcased.validation.action.validate";

    private static final String ICON = "icons/validate.gif";

    private IFile file;

    private EObject model;

    /**
     * Constructor
     * 
     * @param f The validated file
     * @param eObject the validated model
     */
    public ValidateAction(IFile f, EObject eObject)
    {
        super();
        init();

        setFile(f);
        setModel(eObject);
    }

    /**
     * Validate a list of EObjects
     * 
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run()
    {
        ValidateJob validation = new ValidateJob(file, model);
        validation.setUser(true);
        validation.schedule();
    }

    /**
     * Determine if the action is valid
     * 
     * @return if the action can be executed
     */
    protected boolean calculateEnabled()
    {
        return file != null && model != null;
    }

    /**
     * Initializes the action
     */
    protected void init()
    {
        setId(ID);
        setText("Model validation");
        setImageDescriptor(ValidationUIPlugin.getImageDescriptor(ICON));
    }

    /**
     * Set the file where the model is stored
     * 
     * @param f the file
     */
    protected void setFile(IFile f)
    {
        file = f;

        refresh();
    }

    /**
     * Set the model that must be validated
     * 
     * @param eObject the model
     */
    protected void setModel(EObject eObject)
    {
        model = eObject;

        refresh();
    }

    /**
     * Refreshes the properties of this action.
     */
    protected void refresh()
    {
        setEnabled(calculateEnabled());
    }
}
