package org.topcased.web.server.SOAP;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;

import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;

/**
 * This class is responsible for sending a SOAP message in a channel.<br>
 * THe SOAP message is encapsulated and send using HTTP protocol.<br/>
 * 
 * @author <a href="mailto:david.brouard@c-s.fr">David BROUARD</a>
 * 
 */
public class SOAPSenderImpl
{
    /**
     * The key for the response channel.
     */
    private SelectionKey _selectionKey;

    /**
     * HTML string used to send the response.
     */
    private static final String HTTP_OK = "HTTP/1.1 200 OK\r\n" + "Content-Length: %d\r\n" + "Content-Type: text/xml\r\n" + "\r\n";

    /**
     * Send the SOAP msg to the channel.
     * 
     * @param msg_ - message to be sent.
     */
    public void sendMessage(SOAPMessage msg_)
    {
        try
        {
            System.out.println(getClass().getName() + " Sending message ");
            msg_.writeTo(System.out);
            ByteArrayOutputStream output = new ByteArrayOutputStream();
            msg_.writeTo(output);
            // get the string Value.
            writeMessage(output.toString());

        }
        catch (SOAPException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    public void setKey(SelectionKey sk_)
    {
        _selectionKey = sk_;
    }

    /**
     * Write the string reprensint the soap message encapsulated in an http response in the response channel.
     * 
     * @param soapMessage_ - string representation of a soap message.
     * @throws IOException
     */
    private void writeMessage(String soapMessage_) throws IOException
    {
        String l_msgTosend = getHttpOkMessage(soapMessage_);
        ByteBuffer buf = ByteBuffer.wrap(l_msgTosend.getBytes());
        // _selectionKey.attach(buf);
        // _selectionKey.interestOps(SelectionKey.OP_WRITE);
        if (((SocketChannel) _selectionKey.channel()).isConnected())
        {
            ((SocketChannel) _selectionKey.channel()).write(buf);
        }
        _selectionKey.cancel();
        // ((SocketChannel)_selectionKey.channel()).close();
    }

    /**
     * Build an http message containing the SOAP message.
     * 
     * @param soapMsg_ - the soap message to encapsulate.
     * @return - String reprensenting an http message with soap message as datas.
     */
    private String getHttpOkMessage(String soapMsg_)
    {
        return String.format(HTTP_OK + soapMsg_, soapMsg_.length());
    }
}
