/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.web.server.SOAP;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.PropertyResourceBundle;

import org.eclipse.core.runtime.IStatus;
import org.topcased.bus.core.CorePlugin;

/**
 * The SOAP Server configuration class.<br>
 * Reads the configuration file contained in the conf directory.
 * 
 * @author <a href="mailto:david.brouard@c-s.fr">David BROUARD</a>
 */
public class SOAPServerConfiguration
{

	/**
	 * Name of the configuration file
	 */
	private final static String m_fileName = "conf/soapServerConf.properties";

	private PropertyResourceBundle m_properties;

	/**
	 * Singleton
	 */
	private static SOAPServerConfiguration m_instance;

	/**
	 * Constructor
	 * 
	 * We use a PropertyResourceBundle because the file can be modified by a person and the file is exterior to the jar
	 */
	private SOAPServerConfiguration()
	{
		try
		{
			m_properties = new PropertyResourceBundle(new FileInputStream(m_fileName));
		}
		catch (FileNotFoundException e)
		{
			CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
		}
		catch (IOException e)
		{
			CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
		}
	}

	/**
	 * Get instance
	 * 
	 * @return
	 */
	public static SOAPServerConfiguration getInstance()
	{
		if (m_instance == null)
		{
			m_instance = new SOAPServerConfiguration();
		}
		return m_instance;
	}

	public String exception_responseHasNullBody()
	{
		return m_properties.getString("exception.responseHasNullBody");
	}

	public String getAllService_SoapService()
	{
		return m_properties.getString("getAllServices_SoapService");
	}
}
