/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.web.server.preferences;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.topcased.web.server.ServerPlugin;

/**
 * Preference page for the soap server.
 */
public class TopcasedServerPreferencePage extends PreferencePage implements IWorkbenchPreferencePage
{

	private Composite parentComposite;

	private Text portToListen;

	private String portString;

	private List<String> urlList;

	private Table urlTable;

	private Text urlText1;

	private Text urlText2;

	private Text urlText3;

	private Text urlText4;

	private int oldSize;

	public static final String PORT_VALUE = "portValue";

	private static final String URL_NUMBER = "urlNumber";

	private static final String URL_VALUE = "urlValue";

	/**
	 * Constructor
	 */
	public TopcasedServerPreferencePage()
	{
		super("Topcased Web Server Preferences");
		setDescription("Topcased Web Server preference page");
		setPreferenceStore(ServerPlugin.getDefault().getPreferenceStore());
		noDefaultAndApplyButton();
	}

	/**
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent)
	{
		parentComposite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		parentComposite.setLayout(layout);
		parentComposite.setFont(parent.getFont());

		// Add the port group
		final Group newPortGroup = new Group(parentComposite, SWT.NONE);
		newPortGroup.setText("Port to listen to :");
		final GridLayout groupPortLayout = new GridLayout();
		groupPortLayout.marginHeight = 0;
		groupPortLayout.marginWidth = 0;
		newPortGroup.setLayout(groupPortLayout);
		newPortGroup.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));

		// Add the port zone
		portToListen = new Text(newPortGroup, SWT.BORDER);
		GridData portGridData = new GridData();
		portGridData.widthHint = 30;
		portToListen.setLayoutData(portGridData);
		portToListen.setTextLimit(4);
		portToListen.setText(portString);
		portToListen.addKeyListener(new KeyListener()
		{
			String temp = "80";

			public void keyPressed(KeyEvent e)
			{
				try
				{
					Integer.parseInt(temp);
					temp = portToListen.getText();

				}
				catch (NumberFormatException n)
				{
					temp = "8080";
				}
			}

			public void keyReleased(KeyEvent e)
			{
				if (Character.isLetter(e.character))
				{
					portToListen.setText(temp);
				}
				else
				{
					temp = portToListen.getText();
				}
			}
		});

		final Group newIPGroup = new Group(parentComposite, SWT.NONE);
		newIPGroup.setText("Allowed IP addresses");
		final GridLayout groupLayout = new GridLayout();
		groupLayout.numColumns = 5;
		groupLayout.marginHeight = 0;
		groupLayout.marginWidth = 0;
		newIPGroup.setLayout(groupLayout);
		newIPGroup.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));

		urlText1 = new Text(newIPGroup, SWT.BORDER);
		final GridData urlTextData1 = new GridData();
		urlTextData1.widthHint = 20;
		urlText1.setLayoutData(urlTextData1);
		urlText1.setTextLimit(3);
		urlText1.addKeyListener(new KeyAdapter()
		{
			String temp = "";

			public void keyReleased(KeyEvent e)
			{
				if (Character.isLetter(e.character))
				{
					urlText1.setText(temp);
				}
				else
				{
					temp = urlText1.getText();
				}
			}
		});

		urlText2 = new Text(newIPGroup, SWT.BORDER);
		GridData urlTextData2 = new GridData();
		urlTextData2.widthHint = 20;
		urlText2.setLayoutData(urlTextData2);
		urlText2.setTextLimit(3);

		urlText3 = new Text(newIPGroup, SWT.BORDER);
		GridData urlTextData3 = new GridData();
		urlTextData3.widthHint = 20;
		urlText3.setLayoutData(urlTextData3);
		urlText3.setTextLimit(3);

		urlText4 = new Text(newIPGroup, SWT.BORDER);
		GridData urlTextData4 = new GridData();
		urlTextData4.widthHint = 20;
		urlText4.setLayoutData(urlTextData4);
		urlText4.setTextLimit(3);

		final Button addButton = new Button(newIPGroup, SWT.PUSH);
		addButton.setText("Add");
		addButton.setSize(100, 25);
		addButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				// TODO check if the string is a valid ip address
				TableItem url = new TableItem(urlTable, SWT.NONE);
				url.setText(urlText1.getText() + "." + urlText2.getText() + "." + urlText3.getText() + "." + urlText4.getText());
			}
		});

		urlTable = new Table(parentComposite, SWT.BORDER | SWT.MULTI);
		urlTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		final TableColumn column = new TableColumn(urlTable, SWT.NONE);
		column.setText("Allowed Ip address");
		column.setWidth(200);

		for(String url : urlList)
		{
			TableItem urlItem = new TableItem(urlTable, SWT.NONE);
			urlItem.setText(url);
		}

		Button deleteButton = new Button(parentComposite, SWT.PUSH);
		deleteButton.setText("delete");
		deleteButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				urlTable.remove(urlTable.getSelectionIndices());
			}
		});

		return parentComposite;
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench)
	{
		IPreferenceStore store = getPreferenceStore();
		portString = new String(String.valueOf(store.getInt(PORT_VALUE)));
		urlList = new ArrayList<String>();
		int urlListSize = store.getInt(URL_NUMBER);
		oldSize = urlListSize;
		for (int i = 0; i < urlListSize; i++)
		{
			urlList.add(store.getString(URL_VALUE + i));
		}
	}

	/**
	 * @see org.eclipse.jface.preference.PreferencePage#performOk()
	 */
	public boolean performOk()
	{
		IPreferenceStore store = getPreferenceStore();
		TableItem[] listItem = urlTable.getItems();
		store.setValue(PORT_VALUE, portToListen.getText());
		store.setValue(URL_NUMBER, listItem.length);
		for (int i = 0; i < listItem.length; i++)
		{
			store.setValue(URL_VALUE + i, listItem[i].getText());
		}
		for (int i = listItem.length; i < oldSize; i++)
		{
			store.setToDefault(URL_VALUE + i);
		}
		return super.performOk();
	}

}
