package tte.analyzers;

import org.eclipse.core.runtime.IConfigurationElement;

import tte.exceptions.AnalyzerMethodException;
import tte.exceptions.EditionException;

/**
 * Class that represents the analyzer edition method instanciated from the parsing
 *  of an extension to DocumentAnalyzer extension point.
 * @author laurentlb
 *
 */
public class ExtendedAnalyzerEditionMethod extends AbstractExtendedClass {

    /**
     * name of xml attribute that represents the name of method
     *  in the extension declaration.
     */
    private static final String ATT_NAME = "name";

    /**
     * name of xml attribute that represents the version of method
     *  in the extension declaration.
     */
    private static final String ATT_VERSION = "version";

   
    /**
     * the method analyzer
     */
    private final IAnalyzer analyzer;
    /**
     * the name of the method.
     */
    private final String name;

    /**
     * the version of the method.
     */
    private final String version;

    /**
     * the configuration element.
     */
    private final IConfigurationElement configElement;


    /**
     * Creates a new instance of analyzer edition method.
     * @param aConfigElement contains the information from extension parsing
     *  to create this method
     */
    public ExtendedAnalyzerEditionMethod(final IConfigurationElement aConfigElement, IAnalyzer parentAnalyzer) {
        configElement = aConfigElement;
        analyzer = parentAnalyzer;
        name = getAttribute(configElement, ATT_NAME, null);
        version = getAttribute(configElement, ATT_VERSION, "0.0.0");
    }

    /**
     * Gets the id of the method, computed from the name and the version
     *  values.
     * @return the computed id of the method (<b>name</b>_<b>version</b>)
     */
    public final String getID() {
        return name + "_" + version;
    }
    
    /**
     * Runs the edition.
    * @param the document that will be edited
     * @param the element that be searched
     * @throws AnalyzerMethodException if an exception occurs
     *  during instanciation of OwningClass
     * @throws EditionException if an occurs during the conversion
     */
    public final void executeEdition(
            final ttm.Document document, final ttm.Element element)
            throws AnalyzerMethodException, EditionException {
            
            analyzer.executeEditionMethodID(
                        getID(),
                        document,
                        element);
    }
    


}
