package tte.analyzers;

import java.io.File;

import tte.exceptions.AnalyzerMethodException;
import tte.exceptions.ConversionException;
import tte.exceptions.EditionException;
import ttm.Document;
import ttm.Element;

/**
 * Interface the must be implemented to develop an owningClass
 *  for a documentAnalyzer extension.
 *
 */
public interface IAnalyzer {

    /**
     * Runs the conversion of the method which id is in parameter.
     * @param id is the method id
     * @param inFile is the source file
     * @param outFile is the file that will contains the result of conversion
     * @param conversionDefaultArguments is a string tha represents common arguments
     *          from the conversion Method declaration
     * @param documentSpecificArguments is the string that represents the arguments
     *          added to the document properties
     *  that will be understood by the conversion method (can be null)
     * @param the document that will be converted
     * @throws AnalyzerMethodException if an exception occurs
     *  during instanciation of OwningClass
     * @throws ConversionException if an occurs during the conversion
     */
    void executeConversionMethodID(String id,
            File inFile,
            File outFile,
            String conversionDefaultArguments,
            Document document) throws AnalyzerMethodException,
                                ConversionException;

    /**
     * Runs the edition method which id is in parameter.
     * @param the id of executed method
     * @param the document that will be edited
     * @param the element that be searched
     * @throws AnalyzerMethodException if an exception occurs
     *  during instanciation of OwningClass
     * @throws EditionException if an occurs during the conversion
     */
    void executeEditionMethodID(String id, Document document, Element element) 
    	throws AnalyzerMethodException, EditionException;
}
