package tte.analyzers.basics;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.jface.util.OpenStrategy;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;

import tte.analyzers.IAnalyzer;
import tte.exceptions.ConversionException;
import tte.exceptions.AnalyzerMethodException;
import tte.exceptions.EditionException;
import ttm.Document;
import ttm.Element;

/**
 * Class taht represents the default analyzer OwningClass used by
 *  the default documentAnalyzer extension.
 *
 */
public class DefaultAnalyzer  implements IAnalyzer {



    /**
     * {@inheritDoc}
     * @see tte.analyzers.IAnalyzer#executeConversionMethodID(java.lang.String,
     *  java.io.File, java.io.File, java.lang.String)
     */
    public final void executeConversionMethodID(final String id,
            final File inFile, final File outFile,
            final String defaultConversionArguments,
            final Document document)
    throws AnalyzerMethodException, ConversionException {
        if (id.equals("defaultConversion_1.0.0")) {
            convertFileIntoIntermediate100(inFile, outFile, defaultConversionArguments);
            return;
        }
        throw new AnalyzerMethodException("Cannot find method which id is '" + id + "'");
    }


    /**
     * conversion method corresponding to id  "<b>defaultConversion_1.0.0</b>".
     * @param inFile is the source file of the conversion
     * @param outFile is the destiantion file of the conversion
     * @param args is string for arguments that may be need
     * @throws ConversionException if exception occurs during conversion
     */
    private void convertFileIntoIntermediate100(final File inFile,
            final File outFile, final String args) throws ConversionException {
        if (!outFile.exists()) {
            try {
                outFile.createNewFile();
            } catch (IOException e) {
                throw new ConversionException(
                        "Cannot create intermediate file \""
                        + outFile
                        + "\"");
            }
        }
        try {
            BufferedReader rbuff = new BufferedReader(new FileReader(inFile));
            String s;
            BufferedWriter wbuff = new BufferedWriter(new FileWriter(outFile));
            while ((s = rbuff.readLine()) != null) {
                wbuff.write(s);
            }
            wbuff.close();
        } catch (IOException e) {
            throw new ConversionException(
                    "Problem during conversion of \""
                    + inFile
                    + "\" to \""
                    + outFile
                    + "\"");
        }
    }


    /**
     * {@inheritDoc}
     * @see tte.analyzers.IAnalyzer#executeConversionMethodID(java.lang.String,
     *  java.io.File, java.io.File, java.lang.String)
     */
	public void executeEditionMethodID(String id, Document document,
			Element element) throws AnalyzerMethodException, EditionException {
        if (id.equals("defaultEdition_1.0.0")) {
            defaultEdition100(document,element);
            return;
        }
        throw new AnalyzerMethodException("Cannot find method which id is '" + id + "'");
	}
	
	private void defaultEdition100(Document document, Element element) throws EditionException {
		String docFilename = document.getFilename();
		URL url = null;
		File sourceFile = new File(docFilename);
		if (!sourceFile.exists()) {
			try {
				url = FileLocator.resolve(new URL(docFilename));

				sourceFile = new File(url.getFile());
			} catch (MalformedURLException e1) {
				e1.printStackTrace();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}
		
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IFile file = (IFile) root.findMember(root.getLocationURI().relativize(
				sourceFile.toURI()).getPath());

		try {
			IDE.openEditor(PlatformUI.getWorkbench().getActiveWorkbenchWindow()
						.getActivePage(), file,
						"org.eclipse.ui.systemExternalEditor", OpenStrategy
								.activateOnOpen());
			} catch (PartInitException e) {
				throw new EditionException(
						"Edition failed.\nNo system editor found.");
			}
		}
}
