//------------------------------------------------
// $Id: DocumentAnalyserExtension.java,v 1.3 2007-05-28 09:57:52 matg Exp $
// (c) Anyware Technologies 2007    www.anyware-tech.com
//------------------------------------------------
package tte.extension;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * Class representing an DocumentAnalyser extension point
 * <br>
 * creation : 5 avr. 07
 *
 * @author <a href="mailto:mickael.gerard@anyware-tech.com">Mickael Gerard</a>
 */
public class DocumentAnalyserExtension
{
    /**
     * ID of the extension point
     */
    private String id;
    
    /**
     * Name of the extension
     */
    private String name;       
    
    /**
     * Class
     */
    private String owningClass;
    
    private Map<String,ConversionMethodExtension> conversionMethods;
    
    private Map<String,EditionMethodExtension> editionMethods;
    
    
    public DocumentAnalyserExtension()
    {
        this("");
    }

    public DocumentAnalyserExtension(String id)
    {
        super();
        this.id = id;        
    }
    
    /**
     * Add a mthod to the set of conversionMethodExtension manage by this document Analyser 
     * @param method
     */
    public void addConversionMethodExtension(ConversionMethodExtension method)
    {
        if(conversionMethods == null)
        {
            conversionMethods = new HashMap<String,ConversionMethodExtension>();
        }
        if(method != null && !conversionMethods.containsKey(method.getIdentification()))
        {
            conversionMethods.put(method.getIdentification(),method);
        }        
    }
    
    /**
     * Remove a ConversionMethodExtension to the list
     * @param method
     */
    public void removeConversionMethodExtension(ConversionMethodExtension method)
    {       
        if(conversionMethods != null && method != null && conversionMethods.containsKey(method.getIdentification()))
        {
            conversionMethods.remove(method.getIdentification());
        }        
    }
    
    /**
     * Add a mthod to the set of conversionMethodExtension manage by this document Analyser 
     * @param method
     */
    public void addEditionMethodExtension(EditionMethodExtension method)
    {
        if(editionMethods == null)
        {
        	editionMethods = new HashMap<String,EditionMethodExtension>();
        }
        if(method != null && !conversionMethods.containsKey(method.getIdentification()))
        {
            editionMethods.put(method.getIdentification(),method);
        }        
    }
    
    /**
     * Remove a ConversionMethodExtension to the list
     * @param method
     */
    public void removeEditionMethodExtension(EditionMethodExtension method)
    {       
        if(editionMethods != null && method != null && editionMethods.containsKey(method.getIdentification()))
        {
        	editionMethods.remove(method.getIdentification());
        }        
    }
    
    /**
     * @return a ConversionMethodExtension corresponding to the given conversionMethodId
     */
    public ConversionMethodExtension getConversionMethod(String conversionMethodId)
    {
        if(conversionMethods == null) {
            return null;
        }
        else
        {
            return conversionMethods.get(conversionMethodId);
        }           
    }
    
    /**
     * @return a Collection of the ConversionMethodExtension link to this document Analyser
     */
    public Collection<ConversionMethodExtension> getConversionMethods()
    {
        if(conversionMethods == null)
        {
            return Collections.emptyList();
        }
        else
        {
            return conversionMethods.values();
        }           
    }
    
    /**
     * @return a Set of all the Id of ConversionMethods of this analyser.
     * The id of a ConversionMethods is given by the getIdentification method of 
     * ConversionMethodExtension class
     */
    public Collection<String> getConversionMethodIdentifications()
    {
        if(conversionMethods == null)
        {
            return Collections.emptyList();
        }
        else
        {
            return conversionMethods.keySet();
        } 
    }

    
    /**
     * @return a EditionMethodExtension corresponding to the given conversionMethodId
     */
    public EditionMethodExtension getEditionMethod(String editionMethodId)
    {
        if(editionMethods == null) {
            return null;
        }
        else
        {
            return editionMethods.get(editionMethodId);
        }           
    }
    
    /**
     * @return a Collection of the EditionMethodExtension link to this document Analyser
     */
    public Collection<EditionMethodExtension> getEditionMethods()
    {
        if(editionMethods == null)
        {
            return Collections.emptyList();
        }
        else
        {
            return editionMethods.values();
        }           
    }
    
    /**
     * @return a Set of all the Id of EditionMethodExtension of this analyser.
     * The id of a EditionMethodExtensions is given by the getIdentification method of 
     * ConversionMethodExtension class
     */
    public Collection<String> getEditionMethodIdentifications()
    {
        if(editionMethods == null)
        {
            return Collections.emptyList();
        }
        else
        {
            return editionMethods.keySet();
        } 
    }
    
    public String getId()
    {
        return id;
    }

    protected void setId(String id)
    {
        this.id = id;
    }

    public String getName()
    {
        return name;
    }

    protected void setName(String name)
    {
        this.name = name;
    }

    protected String getOwningClass()
    {
        return owningClass;
    }

    public void setOwningClass(String owningClass)
    {
        this.owningClass = owningClass;
    }
        
}
