package tte.rules.basics;

import java.util.Iterator;

import tte.rules.IRule;
import ttm.Attribute;
import ttm.AttributeOwner;
import ttm.CoverLink;
import ttm.Element;
import ttm.IdentifiedElement;
import ttm.NavigationLink;
import ttm.Project;

/**
 * Class implemented <b>IRule</b> interface, that is the action class
 * of a Rule for basic RulesSet extension.
 * This Class defines the Rule "Attribute defined several times in an element"
 *
 */
public class DefinedSeveralTimesAttribute implements IRule {

    /**
     * {@inheritDoc}
     * @see tte.rules.IRule#getDisplayMessageForElement(ttm.Element)
     */
    public final String getDisplayMessageForElement(final Element elt) {
        Attribute attr = (Attribute) elt;
        AttributeOwner prnt = attr.getParent();
        String msg = "Attribute "
            + attr.getName()
            + "("
            + attr.getValue()
            + ") is defined several times in ";
        if (prnt instanceof IdentifiedElement) {
            msg += ((IdentifiedElement) prnt).getIdent();
        }
        if (prnt instanceof CoverLink) {
            msg += "cover link "
                + ((CoverLink) prnt).getFrom().getIdent()
                + "->"
                + ((CoverLink) prnt).getTo().getIdent();
        }
        if (prnt instanceof NavigationLink) {
            msg += "cover link "
                + ((NavigationLink) prnt).getFrom().getIdent()
                + "->"
                + ((NavigationLink) prnt).getTo().getIdent();
        }
        return msg;
    }

    /**
     * {@inheritDoc}
     * This method indicates why the Rule would be raised.
     * @see tte.rules.IRule#isRaisedForElement(ttm.Element, ttm.Project)
     */
    public final Boolean isRaisedForElement(final Element elt
            , final Project project) {
        Attribute attr = (Attribute) elt;
        AttributeOwner prnt = attr.getParent();
        int nbAttr = 0;
        for (Iterator<?> it = prnt.getAttributes().iterator(); it.hasNext();) {
            Attribute currentAttr = (Attribute) it.next();
            if (currentAttr.getName()
                    .equals(attr.getName())
                    && currentAttr.getValue().equals(attr.getValue())) {
                nbAttr += 1;
            }
            if (nbAttr > 1) {
                return true;
            }
        }
        return false;
    }

}
