package tte.rules.basics;

import java.util.Iterator;
import java.util.List;

import tte.rules.IRule;
import ttm.Element;
import ttm.Project;
import ttm.Requirement;

/**
 * Class implemented <b>IRule</b> interface, that is the action class
 * of a Rule for basic RulesSet extension.
 * This Class defines the Rule "Requirement defined several times"
 *
 */
public class DefinedSeveralTimesRequirement implements IRule {

    /**
     * {@inheritDoc}
     * @see tte.rules.IRule#getDisplayMessageForElement(ttm.Element)
     */
    public final String getDisplayMessageForElement(final Element elt) {
        Requirement currentReq = (Requirement) elt;
        return "Requirement is defined several times: "
        + currentReq.getIdent()
        + "(Document: "
        + currentReq.getDocument().getIdent()
        + ")";

    }

    /**
     * {@inheritDoc}
     * @see tte.rules.IRule#isRaisedForElement(ttm.Element, ttm.Project)
     */
    public final Boolean isRaisedForElement(final Element elt,
            final Project project) {
        Requirement currentReq = (Requirement) elt;
        int nbReq = 0;
        List < Object > reqs = project.getRequirements();
        if (reqs == null) {
            return false;
        }
        for (Iterator<?> it = reqs.iterator(); it.hasNext();) {
            Requirement req = (Requirement) it.next();
            if (currentReq.getIdent().equals(req.getIdent())) {
                nbReq += 1;
            }
        }
        return nbReq > 1;
    }

}
