package tte.rules.basics;

import tte.rules.IRule;
import ttm.CoverLink;
import ttm.Document;
import ttm.Element;
import ttm.Project;

/**
 * Class implemented <b>IRule</b> interface, that is the action class
 * of a Rule for basic RulesSet extension.
 * This Class defines the Rule "Traceability violation" if an element
 * covers a requirement that is not in a document coveerd by the
 * element's document
 */
public class TraceabilityViolation implements IRule {

    /**
     * {@inheritDoc}
     * @see tte.rules.IRule#getDisplayMessageForElement(ttm.Element)
     */
    public final String getDisplayMessageForElement(final Element elt) {
        CoverLink cvl = (CoverLink) elt;
        return "Traceability violation between "
                    + cvl.getFrom().getIdent()
                    + " and "
                    + cvl.getTo().getIdent();
    }

    /**
     * {@inheritDoc}
     * @see tte.rules.IRule#isRaisedForElement(ttm.Element, ttm.Project)
     */
    public final Boolean isRaisedForElement(final Element elt,
            final Project project) {
        if(project.isAnyDocumentCovered())
               return false;
        CoverLink cvl = (CoverLink) elt;
        if (cvl.isUnresolved()) {
            return false;
        }
        Document fromDoc = cvl.getFrom().getDocument();
        Document toDoc = cvl.getTo().getDocument();
        if (fromDoc.getCoveredDocuments().contains(toDoc)) {
            return false;
        }
        if (fromDoc == toDoc && fromDoc.isCoversItself()) {
        	return false;
        }
        return true;
    }

}
