package tte.rules.basics;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import tte.rules.IRule;
import ttm.CoverLink;
import ttm.CoverPoint;
import ttm.Document;
import ttm.Element;
import ttm.HierarchicalElement;
import ttm.Project;
import ttm.Requirement;

/**
 * Class implemented <b>IRule</b> interface, that is the action class
 * of a Rule for basic RulesSet extension.
 * This Class defines the Rule "Uncovered Requirement" if a requirement
 * is not covered while its document should be covered
 */
public class UncoveredRequirement implements IRule {

    /**
     * {@inheritDoc}
     * @see tte.rules.IRule#getDisplayMessageForElement(ttm.Element)
     */
    public final String getDisplayMessageForElement(final Element elt) {
        return "Uncovered Requirement: "
                    + ((Requirement) elt).getIdent();
    }

    /**
     * {@inheritDoc}
     * @see tte.rules.IRule#isRaisedForElement(ttm.Element, ttm.Project)
     */
    public final Boolean isRaisedForElement(final Element elt,
            final Project project) {
        Requirement req = (Requirement) elt;
        Boolean isCovered = true;
        if (project.isAnyDocumentCovered()){
        	return req.getIncomingCoverLinks().isEmpty();
        }
        for (Iterator<?> it = req.getDocument().getCoverPoints().iterator()
        		; it.hasNext();) {
        	CoverPoint cvPt = (CoverPoint)it.next();
        	isCovered = isCovered && isCovered(req, cvPt);
        }
        return !isCovered;
    }
    
    @SuppressWarnings("unchecked")
	private final Boolean isCovered(Requirement req, CoverPoint cvPt) {
    	List<Document> connectedDocs = cvPt.getConnectedDocuments();
    	List<Document> workingDocs = new ArrayList(connectedDocs);
    	List<CoverLink> incomingCvLinks = req.getIncomingCoverLinks();
    	
    	if(req.getDocument().getCoveringDocuments().isEmpty()) {
    		return false;
    	}
        for (Iterator<?> it = incomingCvLinks.iterator()
        		; it.hasNext();) {
            CoverLink cvLnk = (CoverLink) it.next();
            Document doc = cvLnk.getDocument();
            if (connectedDocs.contains(doc)) {
                if(cvPt.isAndCover()) {
                	workingDocs.remove(doc);
                } else {
                	return true;
                }
            }
    	}
        return workingDocs.isEmpty();
    }

}
