package tte.rules.basics;

import java.util.Iterator;

import tte.rules.IRule;
import ttm.HierarchicalElement;
import ttm.CoverLink;
import ttm.Element;
import ttm.IdentifiedElement;
import ttm.Project;

/**
 * Class implemented <b>IRule</b> interface, that is the action class
 * of a Rule for basic RulesSet extension.
 * This Class defines the Rule "Undefined Requirement" if a cover link
 * destination does not exist in the project
 */
public class UndefinedRequirement implements IRule {

    /**
     * {@inheritDoc}
     * @see tte.rules.IRule#getDisplayMessageForElement(ttm.Element)
     */
    public final String getDisplayMessageForElement(final Element elt) {
        String msg = ((IdentifiedElement) elt).getIdent()
        + " contains reference to undefined requirements: ";
        HierarchicalElement cElt = (HierarchicalElement) elt;
        boolean first = true;
        for (Iterator<?> it = cElt.getCoverLinks().iterator(); it.hasNext();) {
            CoverLink cvl = (CoverLink) it.next();
            if (cvl.isUnresolved()) {
                if (first) {
                    first = false;
                } else {
                    msg += ", ";
                }
                msg += cvl.getReferencedID();
            }
        }
        return msg;
    }

    /**
     * {@inheritDoc}
     * @see tte.rules.IRule#isRaisedForElement(ttm.Element, ttm.Project)
     */
    public final Boolean isRaisedForElement(final Element elt,
            final Project project) {
        HierarchicalElement cElt = (HierarchicalElement) elt;
        for (Iterator<?> it = cElt.getCoverLinks().iterator(); it.hasNext();) {
            CoverLink cvl = (CoverLink) it.next();
            if (cvl.isUnresolved()) {
                return true;
            }
        }
        return false;
    }
}
