package tte.util;

import java.io.*;

public class FileUtils {
	
	/**
	 * Returns <code>true</code> if both input streams byte contents is
	 * identical.
	 * 
	 * @param input1
	 *            first input to contents compare
	 * @param input2
	 *            second input to contents compare
	 * @return <code>true</code> if content is equal
	 */
	public static boolean contentsEqual(File firstFile, File secondFile) throws IOException {

		FileInputStream is1 = null;
		FileInputStream is2 = null;
		
		try {
			is1 = new FileInputStream(firstFile);
			is2 = new FileInputStream(secondFile);
			
			if (is1 == is2) {
				return true;
			}

			if (is1 == null && is2 == null) {
				// no byte contents
				return true;
			}

			if (is1 == null || is2 == null) {
				// only one has contents
				return false;
			}

			int c1 = -1;
			int c2 = -1;
			boolean finished = false;
			while (!finished) {
				c1 = is1.read();
				c2 = is2.read();
				if (c1 != c2) {
					return false;
				}
				if (c1 == -1 && c2 == -1) {
					finished = true;
				}
			}
			return true;
		} 
		finally {
			if (is1 != null) {
				is1.close();
			}
			if (is2 != null) {
				is2.close();
			}
		}
	}

	/**
	 * Copy Files
	 * @param sourceFile
	 * @param targetFile
	 * @throws IOException
	 */
	public static void copyFile(File sourceFile, File targetFile) throws IOException {
		FileInputStream input = null;
		FileOutputStream output = null;
		try {
			input = new FileInputStream(sourceFile);
			output = new FileOutputStream(targetFile);
			
			byte[] buffer = new byte[4096];
			int bytes = 0;
			while ((bytes = input.read(buffer)) > 0 ) {
				output.write(buffer, 0, bytes);
			}
		}
		finally {
			if (input != null) {
				input.close();
			}
			if (output != null) {
				output.close();
			}
		}
	}
	
}
